#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#ifdef _MSC_VER
#pragma optimize("", off)
#elif defined(__clang__)
#pragma clang optimize off
#elif defined(__GNUC__)
#pragma GCC optimize ("O0")
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 5989
#define LARGE_STATE_COUNT 2168
#define SYMBOL_COUNT 347
#define ALIAS_COUNT 4
#define TOKEN_COUNT 157
#define EXTERNAL_TOKEN_COUNT 30
#define FIELD_COUNT 32
#define MAX_ALIAS_SEQUENCE_LENGTH 8
#define PRODUCTION_ID_COUNT 127

enum ts_symbol_identifiers {
  sym_identifier = 1,
  aux_sym_program_token1 = 2,
  sym_uninterpreted = 3,
  anon_sym_BEGIN = 4,
  anon_sym_LBRACE = 5,
  anon_sym_RBRACE = 6,
  anon_sym_END = 7,
  anon_sym_def = 8,
  anon_sym_LPAREN = 9,
  anon_sym_RPAREN = 10,
  anon_sym_DOT = 11,
  anon_sym_COLON_COLON = 12,
  anon_sym_end = 13,
  anon_sym_rescue = 14,
  anon_sym_EQ = 15,
  anon_sym_COMMA = 16,
  anon_sym_PIPE = 17,
  anon_sym_SEMI = 18,
  anon_sym_DOT_DOT_DOT = 19,
  anon_sym_STAR = 20,
  anon_sym_STAR_STAR = 21,
  anon_sym_nil = 22,
  anon_sym_AMP = 23,
  anon_sym_COLON = 24,
  anon_sym_class = 25,
  anon_sym_LT = 26,
  anon_sym_module = 27,
  anon_sym_return = 28,
  anon_sym_yield = 29,
  anon_sym_break = 30,
  anon_sym_next = 31,
  anon_sym_redo = 32,
  anon_sym_retry = 33,
  anon_sym_if = 34,
  anon_sym_unless = 35,
  anon_sym_while = 36,
  anon_sym_until = 37,
  anon_sym_for = 38,
  anon_sym_in = 39,
  anon_sym_do = 40,
  anon_sym_case = 41,
  anon_sym_when = 42,
  anon_sym_EQ_GT = 43,
  anon_sym_LBRACK = 44,
  anon_sym_RBRACK = 45,
  anon_sym_LBRACK2 = 46,
  anon_sym_LPAREN2 = 47,
  anon_sym_DOT_DOT = 48,
  sym_line = 49,
  sym_file = 50,
  sym_encoding = 51,
  anon_sym_CARET = 52,
  anon_sym_elsif = 53,
  anon_sym_else = 54,
  anon_sym_then = 55,
  anon_sym_begin = 56,
  anon_sym_ensure = 57,
  anon_sym_COLON_COLON2 = 58,
  anon_sym_AMP_DOT = 59,
  anon_sym_PLUS_EQ = 60,
  anon_sym_DASH_EQ = 61,
  anon_sym_STAR_EQ = 62,
  anon_sym_STAR_STAR_EQ = 63,
  anon_sym_SLASH_EQ = 64,
  anon_sym_PIPE_PIPE_EQ = 65,
  anon_sym_PIPE_EQ = 66,
  anon_sym_AMP_AMP_EQ = 67,
  anon_sym_AMP_EQ = 68,
  anon_sym_PERCENT_EQ = 69,
  anon_sym_GT_GT_EQ = 70,
  anon_sym_LT_LT_EQ = 71,
  anon_sym_CARET_EQ = 72,
  anon_sym_QMARK = 73,
  anon_sym_COLON2 = 74,
  anon_sym_and = 75,
  anon_sym_or = 76,
  anon_sym_PIPE_PIPE = 77,
  anon_sym_AMP_AMP = 78,
  anon_sym_LT_LT = 79,
  anon_sym_GT_GT = 80,
  anon_sym_LT_EQ = 81,
  anon_sym_GT = 82,
  anon_sym_GT_EQ = 83,
  anon_sym_PLUS = 84,
  anon_sym_SLASH = 85,
  anon_sym_PERCENT = 86,
  anon_sym_EQ_EQ = 87,
  anon_sym_BANG_EQ = 88,
  anon_sym_EQ_EQ_EQ = 89,
  anon_sym_LT_EQ_GT = 90,
  anon_sym_EQ_TILDE = 91,
  anon_sym_BANG_TILDE = 92,
  anon_sym_defined_QMARK = 93,
  anon_sym_not = 94,
  anon_sym_BANG = 95,
  anon_sym_TILDE = 96,
  anon_sym_DASH = 97,
  anon_sym_PLUS_AT = 98,
  anon_sym_DASH_AT = 99,
  anon_sym_TILDE_AT = 100,
  anon_sym_LBRACK_RBRACK = 101,
  anon_sym_LBRACK_RBRACK_EQ = 102,
  anon_sym_BQUOTE = 103,
  anon_sym_EQ2 = 104,
  anon_sym_undef = 105,
  anon_sym_alias = 106,
  sym_comment = 107,
  sym_integer = 108,
  sym_float = 109,
  anon_sym_i = 110,
  anon_sym_ri = 111,
  anon_sym_r = 112,
  sym_super = 113,
  sym_self = 114,
  sym_true = 115,
  sym_false = 116,
  sym_constant = 117,
  aux_sym_constant_suffix_token1 = 118,
  aux_sym_identifier_suffix_token1 = 119,
  sym_instance_variable = 120,
  sym_class_variable = 121,
  sym_global_variable = 122,
  sym_character = 123,
  anon_sym_POUND_LBRACE = 124,
  aux_sym_string_array_token1 = 125,
  sym_escape_sequence = 126,
  anon_sym_DASH_GT = 127,
  sym__line_break = 128,
  sym__no_line_break = 129,
  sym_simple_symbol = 130,
  sym__string_start = 131,
  sym__symbol_start = 132,
  sym__subshell_start = 133,
  sym__regex_start = 134,
  sym__string_array_start = 135,
  sym__symbol_array_start = 136,
  sym__heredoc_body_start = 137,
  sym_string_content = 138,
  sym_heredoc_content = 139,
  sym__string_end = 140,
  sym_heredoc_end = 141,
  sym_heredoc_beginning = 142,
  sym__block_ampersand = 143,
  sym__splat_star = 144,
  sym__unary_minus = 145,
  sym__unary_minus_num = 146,
  sym__binary_minus = 147,
  sym__binary_star = 148,
  sym__singleton_class_left_angle_left_langle = 149,
  sym_hash_key_symbol = 150,
  sym__identifier_suffix = 151,
  sym__constant_suffix = 152,
  sym__hash_splat_star_star = 153,
  sym__binary_star_star = 154,
  sym__element_reference_bracket = 155,
  sym__short_interpolation = 156,
  sym_program = 157,
  sym_block_body = 158,
  sym__statements = 159,
  sym_begin_block = 160,
  sym_end_block = 161,
  sym__statement = 162,
  sym_method = 163,
  sym_singleton_method = 164,
  sym__method_rest = 165,
  sym_rescue_modifier_arg = 166,
  sym_rescue_modifier_expression = 167,
  sym__body_expr = 168,
  sym_parameters = 169,
  sym_bare_parameters = 170,
  sym_block_parameters = 171,
  sym__formal_parameter = 172,
  sym__simple_formal_parameter = 173,
  sym_forward_parameter = 174,
  sym_splat_parameter = 175,
  sym_hash_splat_parameter = 176,
  sym_hash_splat_nil = 177,
  sym_block_parameter = 178,
  sym_keyword_parameter = 179,
  sym_optional_parameter = 180,
  sym_class = 181,
  sym_superclass = 182,
  sym_singleton_class = 183,
  sym_module = 184,
  sym_return_command = 185,
  sym_yield_command = 186,
  sym_break_command = 187,
  sym_next_command = 188,
  sym_return = 189,
  sym_yield = 190,
  sym_break = 191,
  sym_next = 192,
  sym_redo = 193,
  sym_retry = 194,
  sym_if_modifier = 195,
  sym_unless_modifier = 196,
  sym_while_modifier = 197,
  sym_until_modifier = 198,
  sym_rescue_modifier = 199,
  sym_while = 200,
  sym_until = 201,
  sym_for = 202,
  sym_in = 203,
  sym_do = 204,
  sym_case = 205,
  sym_case_match = 206,
  sym_when = 207,
  sym_in_clause = 208,
  sym_pattern = 209,
  sym__guard = 210,
  sym_if_guard = 211,
  sym_unless_guard = 212,
  sym__pattern_top_expr_body = 213,
  sym__array_pattern_n = 214,
  sym__pattern_expr = 215,
  sym_as_pattern = 216,
  sym__pattern_expr_alt = 217,
  sym_alternative_pattern = 218,
  sym__array_pattern_body = 219,
  sym_array_pattern = 220,
  sym__find_pattern_body = 221,
  sym_find_pattern = 222,
  sym__hash_pattern_body = 223,
  sym_keyword_pattern = 224,
  sym__hash_pattern_any_rest = 225,
  sym_hash_pattern = 226,
  sym__pattern_expr_basic = 227,
  sym_parenthesized_pattern = 228,
  sym__pattern_value = 229,
  sym__pattern_range = 230,
  sym__pattern_primitive = 231,
  sym__pattern_lambda = 232,
  sym__pattern_literal = 233,
  sym__keyword_variable = 234,
  sym_variable_reference_pattern = 235,
  sym_expression_reference_pattern = 236,
  sym__pattern_constant = 237,
  sym__pattern_constant_resolution = 238,
  sym_if = 239,
  sym_unless = 240,
  sym_elsif = 241,
  sym_else = 242,
  sym_then = 243,
  sym_begin = 244,
  sym_ensure = 245,
  sym_rescue = 246,
  sym_exceptions = 247,
  sym_exception_variable = 248,
  sym_body_statement = 249,
  sym__body_statement = 250,
  sym__expression = 251,
  sym_match_pattern = 252,
  sym_test_pattern = 253,
  sym__arg = 254,
  sym__unary_minus_pow = 255,
  sym__pow = 256,
  sym__primary = 257,
  sym_parenthesized_statements = 258,
  sym_element_reference = 259,
  sym_scope_resolution = 260,
  sym__call = 261,
  sym_command_call = 262,
  sym_command_call_with_block = 263,
  sym__chained_command_call = 264,
  sym_call = 265,
  sym_command_argument_list = 266,
  sym_argument_list = 267,
  sym__argument_list_with_trailing_comma = 268,
  sym__argument = 269,
  sym_forward_argument = 270,
  sym_splat_argument = 271,
  sym_hash_splat_argument = 272,
  sym_block_argument = 273,
  sym_do_block = 274,
  sym_block = 275,
  sym_assignment = 276,
  sym_command_assignment = 277,
  sym_operator_assignment = 278,
  sym_command_operator_assignment = 279,
  sym_conditional = 280,
  sym_range = 281,
  sym_binary = 282,
  sym_command_binary = 283,
  sym_unary = 284,
  sym_command_unary = 285,
  sym_parenthesized_unary = 286,
  sym_unary_literal = 287,
  sym__literal = 288,
  sym__numeric = 289,
  sym__simple_numeric = 290,
  sym_right_assignment_list = 291,
  sym_left_assignment_list = 292,
  sym__mlhs = 293,
  sym_destructured_left_assignment = 294,
  sym_rest_assignment = 295,
  sym__function_identifier = 296,
  sym__function_identifier_call = 297,
  sym__lhs = 298,
  sym__variable = 299,
  sym_operator = 300,
  sym__method_name = 301,
  sym__nonlocal_variable = 302,
  sym_setter = 303,
  sym_undef = 304,
  sym_alias = 305,
  sym__int_or_float = 306,
  sym_complex = 307,
  sym_rational = 308,
  sym_nil = 309,
  sym_constant_suffix = 310,
  sym_identifier_suffix = 311,
  sym_chained_string = 312,
  sym_interpolation = 313,
  sym_string = 314,
  sym_subshell = 315,
  sym_string_array = 316,
  sym_symbol_array = 317,
  sym_delimited_symbol = 318,
  sym_regex = 319,
  sym_heredoc_body = 320,
  aux_sym__literal_contents = 321,
  sym_array = 322,
  sym_hash = 323,
  sym_pair = 324,
  sym_lambda = 325,
  sym_empty_statement = 326,
  sym__terminator = 327,
  aux_sym__statements_repeat1 = 328,
  aux_sym_parameters_repeat1 = 329,
  aux_sym_block_parameters_repeat1 = 330,
  aux_sym_case_repeat1 = 331,
  aux_sym_case_match_repeat1 = 332,
  aux_sym_when_repeat1 = 333,
  aux_sym__array_pattern_n_repeat1 = 334,
  aux_sym_alternative_pattern_repeat1 = 335,
  aux_sym__hash_pattern_body_repeat1 = 336,
  aux_sym_exceptions_repeat1 = 337,
  aux_sym__body_statement_repeat1 = 338,
  aux_sym_command_argument_list_repeat1 = 339,
  aux_sym__mlhs_repeat1 = 340,
  aux_sym_undef_repeat1 = 341,
  aux_sym_chained_string_repeat1 = 342,
  aux_sym_string_array_repeat1 = 343,
  aux_sym_symbol_array_repeat1 = 344,
  aux_sym_heredoc_body_repeat1 = 345,
  aux_sym_hash_repeat1 = 346,
  alias_sym_bare_string = 347,
  alias_sym_bare_symbol = 348,
  alias_sym_destructured_parameter = 349,
  alias_sym_lambda_parameters = 350,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [aux_sym_program_token1] = "program_token1",
  [sym_uninterpreted] = "uninterpreted",
  [anon_sym_BEGIN] = "BEGIN",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_END] = "END",
  [anon_sym_def] = "def",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_DOT] = ".",
  [anon_sym_COLON_COLON] = "::",
  [anon_sym_end] = "end",
  [anon_sym_rescue] = "rescue",
  [anon_sym_EQ] = "=",
  [anon_sym_COMMA] = ",",
  [anon_sym_PIPE] = "|",
  [anon_sym_SEMI] = ";",
  [anon_sym_DOT_DOT_DOT] = "...",
  [anon_sym_STAR] = "*",
  [anon_sym_STAR_STAR] = "**",
  [anon_sym_nil] = "nil",
  [anon_sym_AMP] = "&",
  [anon_sym_COLON] = ":",
  [anon_sym_class] = "class",
  [anon_sym_LT] = "<",
  [anon_sym_module] = "module",
  [anon_sym_return] = "return",
  [anon_sym_yield] = "yield",
  [anon_sym_break] = "break",
  [anon_sym_next] = "next",
  [anon_sym_redo] = "redo",
  [anon_sym_retry] = "retry",
  [anon_sym_if] = "if",
  [anon_sym_unless] = "unless",
  [anon_sym_while] = "while",
  [anon_sym_until] = "until",
  [anon_sym_for] = "for",
  [anon_sym_in] = "in",
  [anon_sym_do] = "do",
  [anon_sym_case] = "case",
  [anon_sym_when] = "when",
  [anon_sym_EQ_GT] = "=>",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_LBRACK2] = "[",
  [anon_sym_LPAREN2] = "(",
  [anon_sym_DOT_DOT] = "..",
  [sym_line] = "line",
  [sym_file] = "file",
  [sym_encoding] = "encoding",
  [anon_sym_CARET] = "^",
  [anon_sym_elsif] = "elsif",
  [anon_sym_else] = "else",
  [anon_sym_then] = "then",
  [anon_sym_begin] = "begin",
  [anon_sym_ensure] = "ensure",
  [anon_sym_COLON_COLON2] = "::",
  [anon_sym_AMP_DOT] = "&.",
  [anon_sym_PLUS_EQ] = "+=",
  [anon_sym_DASH_EQ] = "-=",
  [anon_sym_STAR_EQ] = "*=",
  [anon_sym_STAR_STAR_EQ] = "**=",
  [anon_sym_SLASH_EQ] = "/=",
  [anon_sym_PIPE_PIPE_EQ] = "||=",
  [anon_sym_PIPE_EQ] = "|=",
  [anon_sym_AMP_AMP_EQ] = "&&=",
  [anon_sym_AMP_EQ] = "&=",
  [anon_sym_PERCENT_EQ] = "%=",
  [anon_sym_GT_GT_EQ] = ">>=",
  [anon_sym_LT_LT_EQ] = "<<=",
  [anon_sym_CARET_EQ] = "^=",
  [anon_sym_QMARK] = "\?",
  [anon_sym_COLON2] = ":",
  [anon_sym_and] = "and",
  [anon_sym_or] = "or",
  [anon_sym_PIPE_PIPE] = "||",
  [anon_sym_AMP_AMP] = "&&",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_GT] = ">",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_PLUS] = "+",
  [anon_sym_SLASH] = "/",
  [anon_sym_PERCENT] = "%",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_EQ_EQ_EQ] = "===",
  [anon_sym_LT_EQ_GT] = "<=>",
  [anon_sym_EQ_TILDE] = "=~",
  [anon_sym_BANG_TILDE] = "!~",
  [anon_sym_defined_QMARK] = "defined\?",
  [anon_sym_not] = "not",
  [anon_sym_BANG] = "!",
  [anon_sym_TILDE] = "~",
  [anon_sym_DASH] = "-",
  [anon_sym_PLUS_AT] = "+@",
  [anon_sym_DASH_AT] = "-@",
  [anon_sym_TILDE_AT] = "~@",
  [anon_sym_LBRACK_RBRACK] = "[]",
  [anon_sym_LBRACK_RBRACK_EQ] = "[]=",
  [anon_sym_BQUOTE] = "`",
  [anon_sym_EQ2] = "=",
  [anon_sym_undef] = "undef",
  [anon_sym_alias] = "alias",
  [sym_comment] = "comment",
  [sym_integer] = "integer",
  [sym_float] = "float",
  [anon_sym_i] = "i",
  [anon_sym_ri] = "ri",
  [anon_sym_r] = "r",
  [sym_super] = "super",
  [sym_self] = "self",
  [sym_true] = "true",
  [sym_false] = "false",
  [sym_constant] = "constant",
  [aux_sym_constant_suffix_token1] = "constant_suffix_token1",
  [aux_sym_identifier_suffix_token1] = "identifier_suffix_token1",
  [sym_instance_variable] = "instance_variable",
  [sym_class_variable] = "class_variable",
  [sym_global_variable] = "global_variable",
  [sym_character] = "character",
  [anon_sym_POUND_LBRACE] = "#{",
  [aux_sym_string_array_token1] = "string_array_token1",
  [sym_escape_sequence] = "escape_sequence",
  [anon_sym_DASH_GT] = "->",
  [sym__line_break] = "_line_break",
  [sym__no_line_break] = "_no_line_break",
  [sym_simple_symbol] = "simple_symbol",
  [sym__string_start] = "\"",
  [sym__symbol_start] = ":\"",
  [sym__subshell_start] = "`",
  [sym__regex_start] = "/",
  [sym__string_array_start] = "%w(",
  [sym__symbol_array_start] = "%i(",
  [sym__heredoc_body_start] = "_heredoc_body_start",
  [sym_string_content] = "string_content",
  [sym_heredoc_content] = "heredoc_content",
  [sym__string_end] = ")",
  [sym_heredoc_end] = "heredoc_end",
  [sym_heredoc_beginning] = "heredoc_beginning",
  [sym__block_ampersand] = "&",
  [sym__splat_star] = "*",
  [sym__unary_minus] = "-",
  [sym__unary_minus_num] = "-",
  [sym__binary_minus] = "-",
  [sym__binary_star] = "*",
  [sym__singleton_class_left_angle_left_langle] = "<<",
  [sym_hash_key_symbol] = "hash_key_symbol",
  [sym__identifier_suffix] = "_identifier_suffix",
  [sym__constant_suffix] = "_constant_suffix",
  [sym__hash_splat_star_star] = "**",
  [sym__binary_star_star] = "**",
  [sym__element_reference_bracket] = "[",
  [sym__short_interpolation] = "_short_interpolation",
  [sym_program] = "program",
  [sym_block_body] = "block_body",
  [sym__statements] = "_statements",
  [sym_begin_block] = "begin_block",
  [sym_end_block] = "end_block",
  [sym__statement] = "_statement",
  [sym_method] = "method",
  [sym_singleton_method] = "singleton_method",
  [sym__method_rest] = "_method_rest",
  [sym_rescue_modifier_arg] = "rescue_modifier",
  [sym_rescue_modifier_expression] = "rescue_modifier",
  [sym__body_expr] = "_body_expr",
  [sym_parameters] = "method_parameters",
  [sym_bare_parameters] = "method_parameters",
  [sym_block_parameters] = "block_parameters",
  [sym__formal_parameter] = "_formal_parameter",
  [sym__simple_formal_parameter] = "_simple_formal_parameter",
  [sym_forward_parameter] = "forward_parameter",
  [sym_splat_parameter] = "splat_parameter",
  [sym_hash_splat_parameter] = "hash_splat_parameter",
  [sym_hash_splat_nil] = "hash_splat_nil",
  [sym_block_parameter] = "block_parameter",
  [sym_keyword_parameter] = "keyword_parameter",
  [sym_optional_parameter] = "optional_parameter",
  [sym_class] = "class",
  [sym_superclass] = "superclass",
  [sym_singleton_class] = "singleton_class",
  [sym_module] = "module",
  [sym_return_command] = "return",
  [sym_yield_command] = "yield",
  [sym_break_command] = "break",
  [sym_next_command] = "next",
  [sym_return] = "return",
  [sym_yield] = "yield",
  [sym_break] = "break",
  [sym_next] = "next",
  [sym_redo] = "redo",
  [sym_retry] = "retry",
  [sym_if_modifier] = "if_modifier",
  [sym_unless_modifier] = "unless_modifier",
  [sym_while_modifier] = "while_modifier",
  [sym_until_modifier] = "until_modifier",
  [sym_rescue_modifier] = "rescue_modifier",
  [sym_while] = "while",
  [sym_until] = "until",
  [sym_for] = "for",
  [sym_in] = "in",
  [sym_do] = "do",
  [sym_case] = "case",
  [sym_case_match] = "case_match",
  [sym_when] = "when",
  [sym_in_clause] = "in_clause",
  [sym_pattern] = "pattern",
  [sym__guard] = "_guard",
  [sym_if_guard] = "if_guard",
  [sym_unless_guard] = "unless_guard",
  [sym__pattern_top_expr_body] = "_pattern_top_expr_body",
  [sym__array_pattern_n] = "_array_pattern_n",
  [sym__pattern_expr] = "_pattern_expr",
  [sym_as_pattern] = "as_pattern",
  [sym__pattern_expr_alt] = "_pattern_expr_alt",
  [sym_alternative_pattern] = "alternative_pattern",
  [sym__array_pattern_body] = "_array_pattern_body",
  [sym_array_pattern] = "array_pattern",
  [sym__find_pattern_body] = "_find_pattern_body",
  [sym_find_pattern] = "find_pattern",
  [sym__hash_pattern_body] = "_hash_pattern_body",
  [sym_keyword_pattern] = "keyword_pattern",
  [sym__hash_pattern_any_rest] = "_hash_pattern_any_rest",
  [sym_hash_pattern] = "hash_pattern",
  [sym__pattern_expr_basic] = "_pattern_expr_basic",
  [sym_parenthesized_pattern] = "parenthesized_pattern",
  [sym__pattern_value] = "_pattern_value",
  [sym__pattern_range] = "range",
  [sym__pattern_primitive] = "_pattern_primitive",
  [sym__pattern_lambda] = "_pattern_lambda",
  [sym__pattern_literal] = "_pattern_literal",
  [sym__keyword_variable] = "_keyword_variable",
  [sym_variable_reference_pattern] = "variable_reference_pattern",
  [sym_expression_reference_pattern] = "expression_reference_pattern",
  [sym__pattern_constant] = "_pattern_constant",
  [sym__pattern_constant_resolution] = "scope_resolution",
  [sym_if] = "if",
  [sym_unless] = "unless",
  [sym_elsif] = "elsif",
  [sym_else] = "else",
  [sym_then] = "then",
  [sym_begin] = "begin",
  [sym_ensure] = "ensure",
  [sym_rescue] = "rescue",
  [sym_exceptions] = "exceptions",
  [sym_exception_variable] = "exception_variable",
  [sym_body_statement] = "body_statement",
  [sym__body_statement] = "_body_statement",
  [sym__expression] = "_expression",
  [sym_match_pattern] = "match_pattern",
  [sym_test_pattern] = "test_pattern",
  [sym__arg] = "_arg",
  [sym__unary_minus_pow] = "unary",
  [sym__pow] = "binary",
  [sym__primary] = "_primary",
  [sym_parenthesized_statements] = "parenthesized_statements",
  [sym_element_reference] = "element_reference",
  [sym_scope_resolution] = "scope_resolution",
  [sym__call] = "_call",
  [sym_command_call] = "call",
  [sym_command_call_with_block] = "call",
  [sym__chained_command_call] = "_chained_command_call",
  [sym_call] = "call",
  [sym_command_argument_list] = "argument_list",
  [sym_argument_list] = "argument_list",
  [sym__argument_list_with_trailing_comma] = "_argument_list_with_trailing_comma",
  [sym__argument] = "_argument",
  [sym_forward_argument] = "forward_argument",
  [sym_splat_argument] = "splat_argument",
  [sym_hash_splat_argument] = "hash_splat_argument",
  [sym_block_argument] = "block_argument",
  [sym_do_block] = "do_block",
  [sym_block] = "block",
  [sym_assignment] = "assignment",
  [sym_command_assignment] = "assignment",
  [sym_operator_assignment] = "operator_assignment",
  [sym_command_operator_assignment] = "operator_assignment",
  [sym_conditional] = "conditional",
  [sym_range] = "range",
  [sym_binary] = "binary",
  [sym_command_binary] = "binary",
  [sym_unary] = "unary",
  [sym_command_unary] = "unary",
  [sym_parenthesized_unary] = "unary",
  [sym_unary_literal] = "unary",
  [sym__literal] = "_literal",
  [sym__numeric] = "_numeric",
  [sym__simple_numeric] = "_simple_numeric",
  [sym_right_assignment_list] = "right_assignment_list",
  [sym_left_assignment_list] = "left_assignment_list",
  [sym__mlhs] = "_mlhs",
  [sym_destructured_left_assignment] = "destructured_left_assignment",
  [sym_rest_assignment] = "rest_assignment",
  [sym__function_identifier] = "_function_identifier",
  [sym__function_identifier_call] = "call",
  [sym__lhs] = "_lhs",
  [sym__variable] = "_variable",
  [sym_operator] = "operator",
  [sym__method_name] = "_method_name",
  [sym__nonlocal_variable] = "_nonlocal_variable",
  [sym_setter] = "setter",
  [sym_undef] = "undef",
  [sym_alias] = "alias",
  [sym__int_or_float] = "_int_or_float",
  [sym_complex] = "complex",
  [sym_rational] = "rational",
  [sym_nil] = "nil",
  [sym_constant_suffix] = "hash_key_symbol",
  [sym_identifier_suffix] = "hash_key_symbol",
  [sym_chained_string] = "chained_string",
  [sym_interpolation] = "interpolation",
  [sym_string] = "string",
  [sym_subshell] = "subshell",
  [sym_string_array] = "string_array",
  [sym_symbol_array] = "symbol_array",
  [sym_delimited_symbol] = "delimited_symbol",
  [sym_regex] = "regex",
  [sym_heredoc_body] = "heredoc_body",
  [aux_sym__literal_contents] = "_literal_contents",
  [sym_array] = "array",
  [sym_hash] = "hash",
  [sym_pair] = "pair",
  [sym_lambda] = "lambda",
  [sym_empty_statement] = "empty_statement",
  [sym__terminator] = "_terminator",
  [aux_sym__statements_repeat1] = "_statements_repeat1",
  [aux_sym_parameters_repeat1] = "parameters_repeat1",
  [aux_sym_block_parameters_repeat1] = "block_parameters_repeat1",
  [aux_sym_case_repeat1] = "case_repeat1",
  [aux_sym_case_match_repeat1] = "case_match_repeat1",
  [aux_sym_when_repeat1] = "when_repeat1",
  [aux_sym__array_pattern_n_repeat1] = "_array_pattern_n_repeat1",
  [aux_sym_alternative_pattern_repeat1] = "alternative_pattern_repeat1",
  [aux_sym__hash_pattern_body_repeat1] = "_hash_pattern_body_repeat1",
  [aux_sym_exceptions_repeat1] = "exceptions_repeat1",
  [aux_sym__body_statement_repeat1] = "_body_statement_repeat1",
  [aux_sym_command_argument_list_repeat1] = "command_argument_list_repeat1",
  [aux_sym__mlhs_repeat1] = "_mlhs_repeat1",
  [aux_sym_undef_repeat1] = "undef_repeat1",
  [aux_sym_chained_string_repeat1] = "chained_string_repeat1",
  [aux_sym_string_array_repeat1] = "string_array_repeat1",
  [aux_sym_symbol_array_repeat1] = "symbol_array_repeat1",
  [aux_sym_heredoc_body_repeat1] = "heredoc_body_repeat1",
  [aux_sym_hash_repeat1] = "hash_repeat1",
  [alias_sym_bare_string] = "bare_string",
  [alias_sym_bare_symbol] = "bare_symbol",
  [alias_sym_destructured_parameter] = "destructured_parameter",
  [alias_sym_lambda_parameters] = "lambda_parameters",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [aux_sym_program_token1] = aux_sym_program_token1,
  [sym_uninterpreted] = sym_uninterpreted,
  [anon_sym_BEGIN] = anon_sym_BEGIN,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_END] = anon_sym_END,
  [anon_sym_def] = anon_sym_def,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_COLON_COLON] = anon_sym_COLON_COLON,
  [anon_sym_end] = anon_sym_end,
  [anon_sym_rescue] = anon_sym_rescue,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_DOT_DOT_DOT] = anon_sym_DOT_DOT_DOT,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_STAR_STAR] = anon_sym_STAR_STAR,
  [anon_sym_nil] = anon_sym_nil,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_class] = anon_sym_class,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_module] = anon_sym_module,
  [anon_sym_return] = anon_sym_return,
  [anon_sym_yield] = anon_sym_yield,
  [anon_sym_break] = anon_sym_break,
  [anon_sym_next] = anon_sym_next,
  [anon_sym_redo] = anon_sym_redo,
  [anon_sym_retry] = anon_sym_retry,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_unless] = anon_sym_unless,
  [anon_sym_while] = anon_sym_while,
  [anon_sym_until] = anon_sym_until,
  [anon_sym_for] = anon_sym_for,
  [anon_sym_in] = anon_sym_in,
  [anon_sym_do] = anon_sym_do,
  [anon_sym_case] = anon_sym_case,
  [anon_sym_when] = anon_sym_when,
  [anon_sym_EQ_GT] = anon_sym_EQ_GT,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_LBRACK2] = anon_sym_LBRACK,
  [anon_sym_LPAREN2] = anon_sym_LPAREN,
  [anon_sym_DOT_DOT] = anon_sym_DOT_DOT,
  [sym_line] = sym_line,
  [sym_file] = sym_file,
  [sym_encoding] = sym_encoding,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_elsif] = anon_sym_elsif,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_then] = anon_sym_then,
  [anon_sym_begin] = anon_sym_begin,
  [anon_sym_ensure] = anon_sym_ensure,
  [anon_sym_COLON_COLON2] = anon_sym_COLON_COLON,
  [anon_sym_AMP_DOT] = anon_sym_AMP_DOT,
  [anon_sym_PLUS_EQ] = anon_sym_PLUS_EQ,
  [anon_sym_DASH_EQ] = anon_sym_DASH_EQ,
  [anon_sym_STAR_EQ] = anon_sym_STAR_EQ,
  [anon_sym_STAR_STAR_EQ] = anon_sym_STAR_STAR_EQ,
  [anon_sym_SLASH_EQ] = anon_sym_SLASH_EQ,
  [anon_sym_PIPE_PIPE_EQ] = anon_sym_PIPE_PIPE_EQ,
  [anon_sym_PIPE_EQ] = anon_sym_PIPE_EQ,
  [anon_sym_AMP_AMP_EQ] = anon_sym_AMP_AMP_EQ,
  [anon_sym_AMP_EQ] = anon_sym_AMP_EQ,
  [anon_sym_PERCENT_EQ] = anon_sym_PERCENT_EQ,
  [anon_sym_GT_GT_EQ] = anon_sym_GT_GT_EQ,
  [anon_sym_LT_LT_EQ] = anon_sym_LT_LT_EQ,
  [anon_sym_CARET_EQ] = anon_sym_CARET_EQ,
  [anon_sym_QMARK] = anon_sym_QMARK,
  [anon_sym_COLON2] = anon_sym_COLON,
  [anon_sym_and] = anon_sym_and,
  [anon_sym_or] = anon_sym_or,
  [anon_sym_PIPE_PIPE] = anon_sym_PIPE_PIPE,
  [anon_sym_AMP_AMP] = anon_sym_AMP_AMP,
  [anon_sym_LT_LT] = anon_sym_LT_LT,
  [anon_sym_GT_GT] = anon_sym_GT_GT,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [anon_sym_BANG_EQ] = anon_sym_BANG_EQ,
  [anon_sym_EQ_EQ_EQ] = anon_sym_EQ_EQ_EQ,
  [anon_sym_LT_EQ_GT] = anon_sym_LT_EQ_GT,
  [anon_sym_EQ_TILDE] = anon_sym_EQ_TILDE,
  [anon_sym_BANG_TILDE] = anon_sym_BANG_TILDE,
  [anon_sym_defined_QMARK] = anon_sym_defined_QMARK,
  [anon_sym_not] = anon_sym_not,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_PLUS_AT] = anon_sym_PLUS_AT,
  [anon_sym_DASH_AT] = anon_sym_DASH_AT,
  [anon_sym_TILDE_AT] = anon_sym_TILDE_AT,
  [anon_sym_LBRACK_RBRACK] = anon_sym_LBRACK_RBRACK,
  [anon_sym_LBRACK_RBRACK_EQ] = anon_sym_LBRACK_RBRACK_EQ,
  [anon_sym_BQUOTE] = anon_sym_BQUOTE,
  [anon_sym_EQ2] = anon_sym_EQ,
  [anon_sym_undef] = anon_sym_undef,
  [anon_sym_alias] = anon_sym_alias,
  [sym_comment] = sym_comment,
  [sym_integer] = sym_integer,
  [sym_float] = sym_float,
  [anon_sym_i] = anon_sym_i,
  [anon_sym_ri] = anon_sym_ri,
  [anon_sym_r] = anon_sym_r,
  [sym_super] = sym_super,
  [sym_self] = sym_self,
  [sym_true] = sym_true,
  [sym_false] = sym_false,
  [sym_constant] = sym_constant,
  [aux_sym_constant_suffix_token1] = aux_sym_constant_suffix_token1,
  [aux_sym_identifier_suffix_token1] = aux_sym_identifier_suffix_token1,
  [sym_instance_variable] = sym_instance_variable,
  [sym_class_variable] = sym_class_variable,
  [sym_global_variable] = sym_global_variable,
  [sym_character] = sym_character,
  [anon_sym_POUND_LBRACE] = anon_sym_POUND_LBRACE,
  [aux_sym_string_array_token1] = aux_sym_string_array_token1,
  [sym_escape_sequence] = sym_escape_sequence,
  [anon_sym_DASH_GT] = anon_sym_DASH_GT,
  [sym__line_break] = sym__line_break,
  [sym__no_line_break] = sym__no_line_break,
  [sym_simple_symbol] = sym_simple_symbol,
  [sym__string_start] = sym__string_start,
  [sym__symbol_start] = sym__symbol_start,
  [sym__subshell_start] = anon_sym_BQUOTE,
  [sym__regex_start] = anon_sym_SLASH,
  [sym__string_array_start] = sym__string_array_start,
  [sym__symbol_array_start] = sym__symbol_array_start,
  [sym__heredoc_body_start] = sym__heredoc_body_start,
  [sym_string_content] = sym_string_content,
  [sym_heredoc_content] = sym_heredoc_content,
  [sym__string_end] = anon_sym_RPAREN,
  [sym_heredoc_end] = sym_heredoc_end,
  [sym_heredoc_beginning] = sym_heredoc_beginning,
  [sym__block_ampersand] = anon_sym_AMP,
  [sym__splat_star] = anon_sym_STAR,
  [sym__unary_minus] = anon_sym_DASH,
  [sym__unary_minus_num] = anon_sym_DASH,
  [sym__binary_minus] = anon_sym_DASH,
  [sym__binary_star] = anon_sym_STAR,
  [sym__singleton_class_left_angle_left_langle] = anon_sym_LT_LT,
  [sym_hash_key_symbol] = sym_hash_key_symbol,
  [sym__identifier_suffix] = sym__identifier_suffix,
  [sym__constant_suffix] = sym__constant_suffix,
  [sym__hash_splat_star_star] = anon_sym_STAR_STAR,
  [sym__binary_star_star] = anon_sym_STAR_STAR,
  [sym__element_reference_bracket] = anon_sym_LBRACK,
  [sym__short_interpolation] = sym__short_interpolation,
  [sym_program] = sym_program,
  [sym_block_body] = sym_block_body,
  [sym__statements] = sym__statements,
  [sym_begin_block] = sym_begin_block,
  [sym_end_block] = sym_end_block,
  [sym__statement] = sym__statement,
  [sym_method] = sym_method,
  [sym_singleton_method] = sym_singleton_method,
  [sym__method_rest] = sym__method_rest,
  [sym_rescue_modifier_arg] = sym_rescue_modifier,
  [sym_rescue_modifier_expression] = sym_rescue_modifier,
  [sym__body_expr] = sym__body_expr,
  [sym_parameters] = sym_parameters,
  [sym_bare_parameters] = sym_parameters,
  [sym_block_parameters] = sym_block_parameters,
  [sym__formal_parameter] = sym__formal_parameter,
  [sym__simple_formal_parameter] = sym__simple_formal_parameter,
  [sym_forward_parameter] = sym_forward_parameter,
  [sym_splat_parameter] = sym_splat_parameter,
  [sym_hash_splat_parameter] = sym_hash_splat_parameter,
  [sym_hash_splat_nil] = sym_hash_splat_nil,
  [sym_block_parameter] = sym_block_parameter,
  [sym_keyword_parameter] = sym_keyword_parameter,
  [sym_optional_parameter] = sym_optional_parameter,
  [sym_class] = sym_class,
  [sym_superclass] = sym_superclass,
  [sym_singleton_class] = sym_singleton_class,
  [sym_module] = sym_module,
  [sym_return_command] = sym_return,
  [sym_yield_command] = sym_yield,
  [sym_break_command] = sym_break,
  [sym_next_command] = sym_next,
  [sym_return] = sym_return,
  [sym_yield] = sym_yield,
  [sym_break] = sym_break,
  [sym_next] = sym_next,
  [sym_redo] = sym_redo,
  [sym_retry] = sym_retry,
  [sym_if_modifier] = sym_if_modifier,
  [sym_unless_modifier] = sym_unless_modifier,
  [sym_while_modifier] = sym_while_modifier,
  [sym_until_modifier] = sym_until_modifier,
  [sym_rescue_modifier] = sym_rescue_modifier,
  [sym_while] = sym_while,
  [sym_until] = sym_until,
  [sym_for] = sym_for,
  [sym_in] = sym_in,
  [sym_do] = sym_do,
  [sym_case] = sym_case,
  [sym_case_match] = sym_case_match,
  [sym_when] = sym_when,
  [sym_in_clause] = sym_in_clause,
  [sym_pattern] = sym_pattern,
  [sym__guard] = sym__guard,
  [sym_if_guard] = sym_if_guard,
  [sym_unless_guard] = sym_unless_guard,
  [sym__pattern_top_expr_body] = sym__pattern_top_expr_body,
  [sym__array_pattern_n] = sym__array_pattern_n,
  [sym__pattern_expr] = sym__pattern_expr,
  [sym_as_pattern] = sym_as_pattern,
  [sym__pattern_expr_alt] = sym__pattern_expr_alt,
  [sym_alternative_pattern] = sym_alternative_pattern,
  [sym__array_pattern_body] = sym__array_pattern_body,
  [sym_array_pattern] = sym_array_pattern,
  [sym__find_pattern_body] = sym__find_pattern_body,
  [sym_find_pattern] = sym_find_pattern,
  [sym__hash_pattern_body] = sym__hash_pattern_body,
  [sym_keyword_pattern] = sym_keyword_pattern,
  [sym__hash_pattern_any_rest] = sym__hash_pattern_any_rest,
  [sym_hash_pattern] = sym_hash_pattern,
  [sym__pattern_expr_basic] = sym__pattern_expr_basic,
  [sym_parenthesized_pattern] = sym_parenthesized_pattern,
  [sym__pattern_value] = sym__pattern_value,
  [sym__pattern_range] = sym_range,
  [sym__pattern_primitive] = sym__pattern_primitive,
  [sym__pattern_lambda] = sym__pattern_lambda,
  [sym__pattern_literal] = sym__pattern_literal,
  [sym__keyword_variable] = sym__keyword_variable,
  [sym_variable_reference_pattern] = sym_variable_reference_pattern,
  [sym_expression_reference_pattern] = sym_expression_reference_pattern,
  [sym__pattern_constant] = sym__pattern_constant,
  [sym__pattern_constant_resolution] = sym_scope_resolution,
  [sym_if] = sym_if,
  [sym_unless] = sym_unless,
  [sym_elsif] = sym_elsif,
  [sym_else] = sym_else,
  [sym_then] = sym_then,
  [sym_begin] = sym_begin,
  [sym_ensure] = sym_ensure,
  [sym_rescue] = sym_rescue,
  [sym_exceptions] = sym_exceptions,
  [sym_exception_variable] = sym_exception_variable,
  [sym_body_statement] = sym_body_statement,
  [sym__body_statement] = sym__body_statement,
  [sym__expression] = sym__expression,
  [sym_match_pattern] = sym_match_pattern,
  [sym_test_pattern] = sym_test_pattern,
  [sym__arg] = sym__arg,
  [sym__unary_minus_pow] = sym_unary,
  [sym__pow] = sym_binary,
  [sym__primary] = sym__primary,
  [sym_parenthesized_statements] = sym_parenthesized_statements,
  [sym_element_reference] = sym_element_reference,
  [sym_scope_resolution] = sym_scope_resolution,
  [sym__call] = sym__call,
  [sym_command_call] = sym_call,
  [sym_command_call_with_block] = sym_call,
  [sym__chained_command_call] = sym__chained_command_call,
  [sym_call] = sym_call,
  [sym_command_argument_list] = sym_argument_list,
  [sym_argument_list] = sym_argument_list,
  [sym__argument_list_with_trailing_comma] = sym__argument_list_with_trailing_comma,
  [sym__argument] = sym__argument,
  [sym_forward_argument] = sym_forward_argument,
  [sym_splat_argument] = sym_splat_argument,
  [sym_hash_splat_argument] = sym_hash_splat_argument,
  [sym_block_argument] = sym_block_argument,
  [sym_do_block] = sym_do_block,
  [sym_block] = sym_block,
  [sym_assignment] = sym_assignment,
  [sym_command_assignment] = sym_assignment,
  [sym_operator_assignment] = sym_operator_assignment,
  [sym_command_operator_assignment] = sym_operator_assignment,
  [sym_conditional] = sym_conditional,
  [sym_range] = sym_range,
  [sym_binary] = sym_binary,
  [sym_command_binary] = sym_binary,
  [sym_unary] = sym_unary,
  [sym_command_unary] = sym_unary,
  [sym_parenthesized_unary] = sym_unary,
  [sym_unary_literal] = sym_unary,
  [sym__literal] = sym__literal,
  [sym__numeric] = sym__numeric,
  [sym__simple_numeric] = sym__simple_numeric,
  [sym_right_assignment_list] = sym_right_assignment_list,
  [sym_left_assignment_list] = sym_left_assignment_list,
  [sym__mlhs] = sym__mlhs,
  [sym_destructured_left_assignment] = sym_destructured_left_assignment,
  [sym_rest_assignment] = sym_rest_assignment,
  [sym__function_identifier] = sym__function_identifier,
  [sym__function_identifier_call] = sym_call,
  [sym__lhs] = sym__lhs,
  [sym__variable] = sym__variable,
  [sym_operator] = sym_operator,
  [sym__method_name] = sym__method_name,
  [sym__nonlocal_variable] = sym__nonlocal_variable,
  [sym_setter] = sym_setter,
  [sym_undef] = sym_undef,
  [sym_alias] = sym_alias,
  [sym__int_or_float] = sym__int_or_float,
  [sym_complex] = sym_complex,
  [sym_rational] = sym_rational,
  [sym_nil] = sym_nil,
  [sym_constant_suffix] = sym_hash_key_symbol,
  [sym_identifier_suffix] = sym_hash_key_symbol,
  [sym_chained_string] = sym_chained_string,
  [sym_interpolation] = sym_interpolation,
  [sym_string] = sym_string,
  [sym_subshell] = sym_subshell,
  [sym_string_array] = sym_string_array,
  [sym_symbol_array] = sym_symbol_array,
  [sym_delimited_symbol] = sym_delimited_symbol,
  [sym_regex] = sym_regex,
  [sym_heredoc_body] = sym_heredoc_body,
  [aux_sym__literal_contents] = aux_sym__literal_contents,
  [sym_array] = sym_array,
  [sym_hash] = sym_hash,
  [sym_pair] = sym_pair,
  [sym_lambda] = sym_lambda,
  [sym_empty_statement] = sym_empty_statement,
  [sym__terminator] = sym__terminator,
  [aux_sym__statements_repeat1] = aux_sym__statements_repeat1,
  [aux_sym_parameters_repeat1] = aux_sym_parameters_repeat1,
  [aux_sym_block_parameters_repeat1] = aux_sym_block_parameters_repeat1,
  [aux_sym_case_repeat1] = aux_sym_case_repeat1,
  [aux_sym_case_match_repeat1] = aux_sym_case_match_repeat1,
  [aux_sym_when_repeat1] = aux_sym_when_repeat1,
  [aux_sym__array_pattern_n_repeat1] = aux_sym__array_pattern_n_repeat1,
  [aux_sym_alternative_pattern_repeat1] = aux_sym_alternative_pattern_repeat1,
  [aux_sym__hash_pattern_body_repeat1] = aux_sym__hash_pattern_body_repeat1,
  [aux_sym_exceptions_repeat1] = aux_sym_exceptions_repeat1,
  [aux_sym__body_statement_repeat1] = aux_sym__body_statement_repeat1,
  [aux_sym_command_argument_list_repeat1] = aux_sym_command_argument_list_repeat1,
  [aux_sym__mlhs_repeat1] = aux_sym__mlhs_repeat1,
  [aux_sym_undef_repeat1] = aux_sym_undef_repeat1,
  [aux_sym_chained_string_repeat1] = aux_sym_chained_string_repeat1,
  [aux_sym_string_array_repeat1] = aux_sym_string_array_repeat1,
  [aux_sym_symbol_array_repeat1] = aux_sym_symbol_array_repeat1,
  [aux_sym_heredoc_body_repeat1] = aux_sym_heredoc_body_repeat1,
  [aux_sym_hash_repeat1] = aux_sym_hash_repeat1,
  [alias_sym_bare_string] = alias_sym_bare_string,
  [alias_sym_bare_symbol] = alias_sym_bare_symbol,
  [alias_sym_destructured_parameter] = alias_sym_destructured_parameter,
  [alias_sym_lambda_parameters] = alias_sym_lambda_parameters,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_program_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_uninterpreted] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_BEGIN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_END] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_def] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_end] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_rescue] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_nil] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_class] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_module] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_yield] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_next] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_redo] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_retry] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_unless] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_while] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_until] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_in] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_do] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_case] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_when] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [sym_line] = {
    .visible = true,
    .named = true,
  },
  [sym_file] = {
    .visible = true,
    .named = true,
  },
  [sym_encoding] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_elsif] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_then] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_begin] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ensure] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_COLON2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_and] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_or] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_defined_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_not] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK_RBRACK_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_undef] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_alias] = {
    .visible = true,
    .named = false,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_integer] = {
    .visible = true,
    .named = true,
  },
  [sym_float] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_i] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_ri] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_r] = {
    .visible = true,
    .named = false,
  },
  [sym_super] = {
    .visible = true,
    .named = true,
  },
  [sym_self] = {
    .visible = true,
    .named = true,
  },
  [sym_true] = {
    .visible = true,
    .named = true,
  },
  [sym_false] = {
    .visible = true,
    .named = true,
  },
  [sym_constant] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_constant_suffix_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_identifier_suffix_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_instance_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_class_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_global_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_character] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_POUND_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_string_array_token1] = {
    .visible = false,
    .named = false,
  },
  [sym_escape_sequence] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DASH_GT] = {
    .visible = true,
    .named = false,
  },
  [sym__line_break] = {
    .visible = false,
    .named = true,
  },
  [sym__no_line_break] = {
    .visible = false,
    .named = true,
  },
  [sym_simple_symbol] = {
    .visible = true,
    .named = true,
  },
  [sym__string_start] = {
    .visible = true,
    .named = false,
  },
  [sym__symbol_start] = {
    .visible = true,
    .named = false,
  },
  [sym__subshell_start] = {
    .visible = true,
    .named = false,
  },
  [sym__regex_start] = {
    .visible = true,
    .named = false,
  },
  [sym__string_array_start] = {
    .visible = true,
    .named = false,
  },
  [sym__symbol_array_start] = {
    .visible = true,
    .named = false,
  },
  [sym__heredoc_body_start] = {
    .visible = false,
    .named = true,
  },
  [sym_string_content] = {
    .visible = true,
    .named = true,
  },
  [sym_heredoc_content] = {
    .visible = true,
    .named = true,
  },
  [sym__string_end] = {
    .visible = true,
    .named = false,
  },
  [sym_heredoc_end] = {
    .visible = true,
    .named = true,
  },
  [sym_heredoc_beginning] = {
    .visible = true,
    .named = true,
  },
  [sym__block_ampersand] = {
    .visible = true,
    .named = false,
  },
  [sym__splat_star] = {
    .visible = true,
    .named = false,
  },
  [sym__unary_minus] = {
    .visible = true,
    .named = false,
  },
  [sym__unary_minus_num] = {
    .visible = true,
    .named = false,
  },
  [sym__binary_minus] = {
    .visible = true,
    .named = false,
  },
  [sym__binary_star] = {
    .visible = true,
    .named = false,
  },
  [sym__singleton_class_left_angle_left_langle] = {
    .visible = true,
    .named = false,
  },
  [sym_hash_key_symbol] = {
    .visible = true,
    .named = true,
  },
  [sym__identifier_suffix] = {
    .visible = false,
    .named = true,
  },
  [sym__constant_suffix] = {
    .visible = false,
    .named = true,
  },
  [sym__hash_splat_star_star] = {
    .visible = true,
    .named = false,
  },
  [sym__binary_star_star] = {
    .visible = true,
    .named = false,
  },
  [sym__element_reference_bracket] = {
    .visible = true,
    .named = false,
  },
  [sym__short_interpolation] = {
    .visible = false,
    .named = true,
  },
  [sym_program] = {
    .visible = true,
    .named = true,
  },
  [sym_block_body] = {
    .visible = true,
    .named = true,
  },
  [sym__statements] = {
    .visible = false,
    .named = true,
  },
  [sym_begin_block] = {
    .visible = true,
    .named = true,
  },
  [sym_end_block] = {
    .visible = true,
    .named = true,
  },
  [sym__statement] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_method] = {
    .visible = true,
    .named = true,
  },
  [sym_singleton_method] = {
    .visible = true,
    .named = true,
  },
  [sym__method_rest] = {
    .visible = false,
    .named = true,
  },
  [sym_rescue_modifier_arg] = {
    .visible = true,
    .named = true,
  },
  [sym_rescue_modifier_expression] = {
    .visible = true,
    .named = true,
  },
  [sym__body_expr] = {
    .visible = false,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_bare_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_block_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym__formal_parameter] = {
    .visible = false,
    .named = true,
  },
  [sym__simple_formal_parameter] = {
    .visible = false,
    .named = true,
  },
  [sym_forward_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_splat_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_hash_splat_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_hash_splat_nil] = {
    .visible = true,
    .named = true,
  },
  [sym_block_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_keyword_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_optional_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_class] = {
    .visible = true,
    .named = true,
  },
  [sym_superclass] = {
    .visible = true,
    .named = true,
  },
  [sym_singleton_class] = {
    .visible = true,
    .named = true,
  },
  [sym_module] = {
    .visible = true,
    .named = true,
  },
  [sym_return_command] = {
    .visible = true,
    .named = true,
  },
  [sym_yield_command] = {
    .visible = true,
    .named = true,
  },
  [sym_break_command] = {
    .visible = true,
    .named = true,
  },
  [sym_next_command] = {
    .visible = true,
    .named = true,
  },
  [sym_return] = {
    .visible = true,
    .named = true,
  },
  [sym_yield] = {
    .visible = true,
    .named = true,
  },
  [sym_break] = {
    .visible = true,
    .named = true,
  },
  [sym_next] = {
    .visible = true,
    .named = true,
  },
  [sym_redo] = {
    .visible = true,
    .named = true,
  },
  [sym_retry] = {
    .visible = true,
    .named = true,
  },
  [sym_if_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_unless_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_while_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_until_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_rescue_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_while] = {
    .visible = true,
    .named = true,
  },
  [sym_until] = {
    .visible = true,
    .named = true,
  },
  [sym_for] = {
    .visible = true,
    .named = true,
  },
  [sym_in] = {
    .visible = true,
    .named = true,
  },
  [sym_do] = {
    .visible = true,
    .named = true,
  },
  [sym_case] = {
    .visible = true,
    .named = true,
  },
  [sym_case_match] = {
    .visible = true,
    .named = true,
  },
  [sym_when] = {
    .visible = true,
    .named = true,
  },
  [sym_in_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__guard] = {
    .visible = false,
    .named = true,
  },
  [sym_if_guard] = {
    .visible = true,
    .named = true,
  },
  [sym_unless_guard] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern_top_expr_body] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__array_pattern_n] = {
    .visible = false,
    .named = true,
  },
  [sym__pattern_expr] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_as_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern_expr_alt] = {
    .visible = false,
    .named = true,
  },
  [sym_alternative_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__array_pattern_body] = {
    .visible = false,
    .named = true,
  },
  [sym_array_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__find_pattern_body] = {
    .visible = false,
    .named = true,
  },
  [sym_find_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__hash_pattern_body] = {
    .visible = false,
    .named = true,
  },
  [sym_keyword_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__hash_pattern_any_rest] = {
    .visible = false,
    .named = true,
  },
  [sym_hash_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern_expr_basic] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_parenthesized_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern_value] = {
    .visible = false,
    .named = true,
  },
  [sym__pattern_range] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern_primitive] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__pattern_lambda] = {
    .visible = false,
    .named = true,
  },
  [sym__pattern_literal] = {
    .visible = false,
    .named = true,
  },
  [sym__keyword_variable] = {
    .visible = false,
    .named = true,
  },
  [sym_variable_reference_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_reference_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__pattern_constant] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__pattern_constant_resolution] = {
    .visible = true,
    .named = true,
  },
  [sym_if] = {
    .visible = true,
    .named = true,
  },
  [sym_unless] = {
    .visible = true,
    .named = true,
  },
  [sym_elsif] = {
    .visible = true,
    .named = true,
  },
  [sym_else] = {
    .visible = true,
    .named = true,
  },
  [sym_then] = {
    .visible = true,
    .named = true,
  },
  [sym_begin] = {
    .visible = true,
    .named = true,
  },
  [sym_ensure] = {
    .visible = true,
    .named = true,
  },
  [sym_rescue] = {
    .visible = true,
    .named = true,
  },
  [sym_exceptions] = {
    .visible = true,
    .named = true,
  },
  [sym_exception_variable] = {
    .visible = true,
    .named = true,
  },
  [sym_body_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__body_statement] = {
    .visible = false,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_match_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_test_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym__arg] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__unary_minus_pow] = {
    .visible = true,
    .named = true,
  },
  [sym__pow] = {
    .visible = true,
    .named = true,
  },
  [sym__primary] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_parenthesized_statements] = {
    .visible = true,
    .named = true,
  },
  [sym_element_reference] = {
    .visible = true,
    .named = true,
  },
  [sym_scope_resolution] = {
    .visible = true,
    .named = true,
  },
  [sym__call] = {
    .visible = false,
    .named = true,
  },
  [sym_command_call] = {
    .visible = true,
    .named = true,
  },
  [sym_command_call_with_block] = {
    .visible = true,
    .named = true,
  },
  [sym__chained_command_call] = {
    .visible = false,
    .named = true,
  },
  [sym_call] = {
    .visible = true,
    .named = true,
  },
  [sym_command_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym__argument_list_with_trailing_comma] = {
    .visible = false,
    .named = true,
  },
  [sym__argument] = {
    .visible = false,
    .named = true,
  },
  [sym_forward_argument] = {
    .visible = true,
    .named = true,
  },
  [sym_splat_argument] = {
    .visible = true,
    .named = true,
  },
  [sym_hash_splat_argument] = {
    .visible = true,
    .named = true,
  },
  [sym_block_argument] = {
    .visible = true,
    .named = true,
  },
  [sym_do_block] = {
    .visible = true,
    .named = true,
  },
  [sym_block] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_command_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_operator_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_command_operator_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_conditional] = {
    .visible = true,
    .named = true,
  },
  [sym_range] = {
    .visible = true,
    .named = true,
  },
  [sym_binary] = {
    .visible = true,
    .named = true,
  },
  [sym_command_binary] = {
    .visible = true,
    .named = true,
  },
  [sym_unary] = {
    .visible = true,
    .named = true,
  },
  [sym_command_unary] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_unary] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_literal] = {
    .visible = true,
    .named = true,
  },
  [sym__literal] = {
    .visible = false,
    .named = true,
  },
  [sym__numeric] = {
    .visible = false,
    .named = true,
  },
  [sym__simple_numeric] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_right_assignment_list] = {
    .visible = true,
    .named = true,
  },
  [sym_left_assignment_list] = {
    .visible = true,
    .named = true,
  },
  [sym__mlhs] = {
    .visible = false,
    .named = true,
  },
  [sym_destructured_left_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym_rest_assignment] = {
    .visible = true,
    .named = true,
  },
  [sym__function_identifier] = {
    .visible = false,
    .named = true,
  },
  [sym__function_identifier_call] = {
    .visible = true,
    .named = true,
  },
  [sym__lhs] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__variable] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_operator] = {
    .visible = true,
    .named = true,
  },
  [sym__method_name] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__nonlocal_variable] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_setter] = {
    .visible = true,
    .named = true,
  },
  [sym_undef] = {
    .visible = true,
    .named = true,
  },
  [sym_alias] = {
    .visible = true,
    .named = true,
  },
  [sym__int_or_float] = {
    .visible = false,
    .named = true,
  },
  [sym_complex] = {
    .visible = true,
    .named = true,
  },
  [sym_rational] = {
    .visible = true,
    .named = true,
  },
  [sym_nil] = {
    .visible = true,
    .named = true,
  },
  [sym_constant_suffix] = {
    .visible = true,
    .named = true,
  },
  [sym_identifier_suffix] = {
    .visible = true,
    .named = true,
  },
  [sym_chained_string] = {
    .visible = true,
    .named = true,
  },
  [sym_interpolation] = {
    .visible = true,
    .named = true,
  },
  [sym_string] = {
    .visible = true,
    .named = true,
  },
  [sym_subshell] = {
    .visible = true,
    .named = true,
  },
  [sym_string_array] = {
    .visible = true,
    .named = true,
  },
  [sym_symbol_array] = {
    .visible = true,
    .named = true,
  },
  [sym_delimited_symbol] = {
    .visible = true,
    .named = true,
  },
  [sym_regex] = {
    .visible = true,
    .named = true,
  },
  [sym_heredoc_body] = {
    .visible = true,
    .named = true,
  },
  [aux_sym__literal_contents] = {
    .visible = false,
    .named = false,
  },
  [sym_array] = {
    .visible = true,
    .named = true,
  },
  [sym_hash] = {
    .visible = true,
    .named = true,
  },
  [sym_pair] = {
    .visible = true,
    .named = true,
  },
  [sym_lambda] = {
    .visible = true,
    .named = true,
  },
  [sym_empty_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__terminator] = {
    .visible = false,
    .named = true,
  },
  [aux_sym__statements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_block_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_case_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_case_match_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_when_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__array_pattern_n_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_alternative_pattern_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__hash_pattern_body_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_exceptions_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__body_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_command_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__mlhs_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_undef_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_chained_string_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_array_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_symbol_array_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_heredoc_body_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_hash_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_bare_string] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_bare_symbol] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_destructured_parameter] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_lambda_parameters] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_alias = 1,
  field_alternative = 2,
  field_alternatives = 3,
  field_arguments = 4,
  field_begin = 5,
  field_block = 6,
  field_body = 7,
  field_class = 8,
  field_clauses = 9,
  field_condition = 10,
  field_consequence = 11,
  field_else = 12,
  field_end = 13,
  field_exceptions = 14,
  field_guard = 15,
  field_handler = 16,
  field_key = 17,
  field_left = 18,
  field_locals = 19,
  field_method = 20,
  field_name = 21,
  field_object = 22,
  field_operand = 23,
  field_operator = 24,
  field_parameters = 25,
  field_pattern = 26,
  field_receiver = 27,
  field_right = 28,
  field_scope = 29,
  field_superclass = 30,
  field_value = 31,
  field_variable = 32,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_alias] = "alias",
  [field_alternative] = "alternative",
  [field_alternatives] = "alternatives",
  [field_arguments] = "arguments",
  [field_begin] = "begin",
  [field_block] = "block",
  [field_body] = "body",
  [field_class] = "class",
  [field_clauses] = "clauses",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_else] = "else",
  [field_end] = "end",
  [field_exceptions] = "exceptions",
  [field_guard] = "guard",
  [field_handler] = "handler",
  [field_key] = "key",
  [field_left] = "left",
  [field_locals] = "locals",
  [field_method] = "method",
  [field_name] = "name",
  [field_object] = "object",
  [field_operand] = "operand",
  [field_operator] = "operator",
  [field_parameters] = "parameters",
  [field_pattern] = "pattern",
  [field_receiver] = "receiver",
  [field_right] = "right",
  [field_scope] = "scope",
  [field_superclass] = "superclass",
  [field_value] = "value",
  [field_variable] = "variable",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [1] = {.index = 0, .length = 2},
  [2] = {.index = 2, .length = 3},
  [4] = {.index = 5, .length = 1},
  [5] = {.index = 6, .length = 1},
  [11] = {.index = 7, .length = 2},
  [12] = {.index = 9, .length = 5},
  [13] = {.index = 14, .length = 3},
  [14] = {.index = 17, .length = 1},
  [15] = {.index = 18, .length = 2},
  [16] = {.index = 20, .length = 1},
  [17] = {.index = 21, .length = 2},
  [18] = {.index = 23, .length = 4},
  [19] = {.index = 27, .length = 4},
  [20] = {.index = 31, .length = 2},
  [21] = {.index = 33, .length = 2},
  [28] = {.index = 35, .length = 1},
  [29] = {.index = 35, .length = 1},
  [30] = {.index = 36, .length = 1},
  [31] = {.index = 37, .length = 2},
  [32] = {.index = 39, .length = 2},
  [33] = {.index = 41, .length = 1},
  [34] = {.index = 42, .length = 1},
  [35] = {.index = 43, .length = 2},
  [37] = {.index = 45, .length = 2},
  [38] = {.index = 47, .length = 2},
  [39] = {.index = 49, .length = 2},
  [40] = {.index = 51, .length = 3},
  [41] = {.index = 54, .length = 3},
  [42] = {.index = 57, .length = 2},
  [46] = {.index = 59, .length = 3},
  [47] = {.index = 62, .length = 2},
  [48] = {.index = 64, .length = 1},
  [49] = {.index = 65, .length = 3},
  [50] = {.index = 68, .length = 3},
  [51] = {.index = 71, .length = 2},
  [52] = {.index = 73, .length = 5},
  [53] = {.index = 78, .length = 2},
  [54] = {.index = 80, .length = 3},
  [57] = {.index = 83, .length = 2},
  [58] = {.index = 83, .length = 2},
  [59] = {.index = 85, .length = 4},
  [60] = {.index = 89, .length = 2},
  [61] = {.index = 91, .length = 3},
  [62] = {.index = 94, .length = 2},
  [63] = {.index = 96, .length = 2},
  [64] = {.index = 98, .length = 1},
  [65] = {.index = 99, .length = 3},
  [66] = {.index = 102, .length = 1},
  [67] = {.index = 103, .length = 2},
  [68] = {.index = 105, .length = 1},
  [69] = {.index = 106, .length = 2},
  [70] = {.index = 108, .length = 2},
  [71] = {.index = 110, .length = 2},
  [72] = {.index = 112, .length = 1},
  [73] = {.index = 113, .length = 2},
  [74] = {.index = 115, .length = 1},
  [75] = {.index = 116, .length = 1},
  [76] = {.index = 117, .length = 2},
  [78] = {.index = 119, .length = 2},
  [79] = {.index = 121, .length = 2},
  [80] = {.index = 123, .length = 4},
  [81] = {.index = 127, .length = 3},
  [82] = {.index = 130, .length = 2},
  [83] = {.index = 132, .length = 2},
  [84] = {.index = 134, .length = 2},
  [85] = {.index = 136, .length = 3},
  [86] = {.index = 139, .length = 2},
  [87] = {.index = 141, .length = 2},
  [88] = {.index = 143, .length = 3},
  [89] = {.index = 146, .length = 2},
  [90] = {.index = 148, .length = 2},
  [91] = {.index = 150, .length = 2},
  [92] = {.index = 152, .length = 3},
  [93] = {.index = 155, .length = 3},
  [94] = {.index = 158, .length = 2},
  [95] = {.index = 45, .length = 2},
  [96] = {.index = 160, .length = 1},
  [97] = {.index = 161, .length = 1},
  [98] = {.index = 162, .length = 2},
  [99] = {.index = 164, .length = 1},
  [100] = {.index = 165, .length = 2},
  [101] = {.index = 167, .length = 1},
  [102] = {.index = 168, .length = 3},
  [103] = {.index = 171, .length = 2},
  [104] = {.index = 173, .length = 4},
  [105] = {.index = 177, .length = 3},
  [106] = {.index = 180, .length = 2},
  [107] = {.index = 182, .length = 3},
  [108] = {.index = 185, .length = 2},
  [109] = {.index = 187, .length = 3},
  [110] = {.index = 190, .length = 3},
  [111] = {.index = 193, .length = 2},
  [112] = {.index = 195, .length = 3},
  [113] = {.index = 198, .length = 1},
  [114] = {.index = 199, .length = 2},
  [115] = {.index = 201, .length = 2},
  [116] = {.index = 203, .length = 3},
  [117] = {.index = 206, .length = 1},
  [118] = {.index = 207, .length = 1},
  [119] = {.index = 208, .length = 2},
  [120] = {.index = 210, .length = 2},
  [121] = {.index = 212, .length = 2},
  [122] = {.index = 214, .length = 2},
  [123] = {.index = 216, .length = 1},
  [124] = {.index = 217, .length = 2},
  [125] = {.index = 219, .length = 1},
  [126] = {.index = 220, .length = 2},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_operand, 0, .inherited = true},
    {field_operator, 0, .inherited = true},
  [2] =
    {field_method, 0, .inherited = true},
    {field_operator, 0, .inherited = true},
    {field_receiver, 0, .inherited = true},
  [5] =
    {field_method, 0},
  [6] =
    {field_method, 0, .inherited = true},
  [7] =
    {field_operand, 1},
    {field_operator, 0},
  [9] =
    {field_left, 1, .inherited = true},
    {field_operand, 1},
    {field_operator, 0},
    {field_operator, 1, .inherited = true},
    {field_right, 1, .inherited = true},
  [14] =
    {field_body, 1, .inherited = true},
    {field_name, 1, .inherited = true},
    {field_parameters, 1, .inherited = true},
  [17] =
    {field_name, 1},
  [18] =
    {field_end, 1},
    {field_operator, 0},
  [20] =
    {field_body, 1},
  [21] =
    {field_begin, 0},
    {field_operator, 1},
  [23] =
    {field_arguments, 1},
    {field_method, 0, .inherited = true},
    {field_operator, 0, .inherited = true},
    {field_receiver, 0, .inherited = true},
  [27] =
    {field_block, 1},
    {field_method, 0, .inherited = true},
    {field_operator, 0, .inherited = true},
    {field_receiver, 0, .inherited = true},
  [31] =
    {field_arguments, 1},
    {field_method, 0},
  [33] =
    {field_block, 1},
    {field_method, 0},
  [35] =
    {field_key, 0},
  [36] =
    {field_name, 0},
  [37] =
    {field_body, 1, .inherited = true},
    {field_name, 0},
  [39] =
    {field_body, 2},
    {field_condition, 1},
  [41] =
    {field_value, 1},
  [42] =
    {field_clauses, 0},
  [43] =
    {field_alias, 2},
    {field_name, 1},
  [45] =
    {field_body, 2},
    {field_parameters, 1},
  [47] =
    {field_body, 0},
    {field_handler, 2},
  [49] =
    {field_body, 0},
    {field_condition, 2},
  [51] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [54] =
    {field_begin, 0},
    {field_end, 2},
    {field_operator, 1},
  [57] =
    {field_pattern, 2},
    {field_value, 0},
  [59] =
    {field_begin, 0, .inherited = true},
    {field_end, 0, .inherited = true},
    {field_operator, 0, .inherited = true},
  [62] =
    {field_name, 0, .inherited = true},
    {field_scope, 0, .inherited = true},
  [64] =
    {field_object, 0},
  [65] =
    {field_method, 2},
    {field_operator, 1},
    {field_receiver, 0},
  [68] =
    {field_arguments, 2},
    {field_operator, 1},
    {field_receiver, 0},
  [71] =
    {field_name, 2},
    {field_scope, 0},
  [73] =
    {field_arguments, 1},
    {field_block, 2},
    {field_method, 0, .inherited = true},
    {field_operator, 0, .inherited = true},
    {field_receiver, 0, .inherited = true},
  [78] =
    {field_left, 0},
    {field_right, 2},
  [80] =
    {field_arguments, 1},
    {field_block, 2},
    {field_method, 0},
  [83] =
    {field_key, 0},
    {field_value, 2},
  [85] =
    {field_body, 3, .inherited = true},
    {field_name, 3, .inherited = true},
    {field_object, 1},
    {field_parameters, 3, .inherited = true},
  [89] =
    {field_name, 0},
    {field_parameters, 1},
  [91] =
    {field_body, 2, .inherited = true},
    {field_name, 0},
    {field_parameters, 1},
  [94] =
    {field_body, 2},
    {field_name, 1},
  [96] =
    {field_condition, 1},
    {field_consequence, 2},
  [98] =
    {field_condition, 1},
  [99] =
    {field_body, 3},
    {field_pattern, 1},
    {field_value, 2},
  [102] =
    {field_value, 2},
  [103] =
    {field_body, 2},
    {field_pattern, 1},
  [105] =
    {field_pattern, 1},
  [106] =
    {field_clauses, 2, .inherited = true},
    {field_value, 1},
  [108] =
    {field_clauses, 0, .inherited = true},
    {field_clauses, 1, .inherited = true},
  [110] =
    {field_body, 2},
    {field_exceptions, 1},
  [112] =
    {field_exceptions, 1},
  [113] =
    {field_body, 2},
    {field_variable, 1},
  [115] =
    {field_variable, 1},
  [116] =
    {field_parameters, 1},
  [117] =
    {field_name, 0},
    {field_value, 2},
  [119] =
    {field_alternatives, 0},
    {field_alternatives, 1, .inherited = true},
  [121] =
    {field_block, 3},
    {field_object, 0},
  [123] =
    {field_arguments, 2},
    {field_block, 3},
    {field_operator, 1},
    {field_receiver, 0},
  [127] =
    {field_body, 2},
    {field_name, 0},
    {field_parameters, 1},
  [130] =
    {field_body, 2},
    {field_name, 0},
  [132] =
    {field_name, 1},
    {field_superclass, 2},
  [134] =
    {field_body, 3},
    {field_name, 1},
  [136] =
    {field_alternative, 3},
    {field_condition, 1},
    {field_consequence, 2},
  [139] =
    {field_alternative, 3},
    {field_condition, 1},
  [141] =
    {field_clauses, 3, .inherited = true},
    {field_value, 2},
  [143] =
    {field_body, 3},
    {field_pattern, 1},
    {field_pattern, 2, .inherited = true},
  [146] =
    {field_pattern, 1},
    {field_pattern, 2, .inherited = true},
  [148] =
    {field_pattern, 0, .inherited = true},
    {field_pattern, 1, .inherited = true},
  [150] =
    {field_clauses, 3, .inherited = true},
    {field_value, 1},
  [152] =
    {field_clauses, 2, .inherited = true},
    {field_else, 3},
    {field_value, 1},
  [155] =
    {field_body, 3},
    {field_exceptions, 1},
    {field_variable, 2},
  [158] =
    {field_exceptions, 1},
    {field_variable, 2},
  [160] =
    {field_parameters, 2},
  [161] =
    {field_body, 2},
  [162] =
    {field_name, 2},
    {field_value, 0},
  [164] =
    {field_alternatives, 1},
  [165] =
    {field_alternatives, 0, .inherited = true},
    {field_alternatives, 1, .inherited = true},
  [167] =
    {field_class, 0},
  [168] =
    {field_alternative, 4},
    {field_condition, 0},
    {field_consequence, 2},
  [171] =
    {field_block, 4},
    {field_object, 0},
  [173] =
    {field_body, 5, .inherited = true},
    {field_name, 5, .inherited = true},
    {field_object, 2},
    {field_parameters, 5, .inherited = true},
  [177] =
    {field_body, 3},
    {field_name, 0},
    {field_parameters, 1},
  [180] =
    {field_body, 4},
    {field_value, 2},
  [182] =
    {field_body, 4},
    {field_name, 1},
    {field_superclass, 2},
  [185] =
    {field_clauses, 4, .inherited = true},
    {field_value, 2},
  [187] =
    {field_clauses, 3, .inherited = true},
    {field_else, 4},
    {field_value, 2},
  [190] =
    {field_body, 3},
    {field_guard, 2},
    {field_pattern, 1},
  [193] =
    {field_guard, 2},
    {field_pattern, 1},
  [195] =
    {field_clauses, 3, .inherited = true},
    {field_else, 4},
    {field_value, 1},
  [198] =
    {field_locals, 2},
  [199] =
    {field_body, 3},
    {field_parameters, 1},
  [201] =
    {field_body, 3},
    {field_parameters, 2},
  [203] =
    {field_clauses, 4, .inherited = true},
    {field_else, 5},
    {field_value, 2},
  [206] =
    {field_locals, 3},
  [207] =
    {field_locals, 1},
  [208] =
    {field_locals, 2},
    {field_locals, 3, .inherited = true},
  [210] =
    {field_locals, 0, .inherited = true},
    {field_locals, 1, .inherited = true},
  [212] =
    {field_body, 4},
    {field_parameters, 2},
  [214] =
    {field_locals, 3},
    {field_locals, 4, .inherited = true},
  [216] =
    {field_locals, 4},
  [217] =
    {field_locals, 4},
    {field_locals, 5, .inherited = true},
  [219] =
    {field_locals, 5},
  [220] =
    {field_locals, 5},
    {field_locals, 6, .inherited = true},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [2] = {
    [0] = sym_call,
  },
  [3] = {
    [0] = sym_call,
  },
  [6] = {
    [0] = sym_constant,
  },
  [7] = {
    [0] = sym_identifier,
  },
  [8] = {
    [1] = sym__string_start,
  },
  [9] = {
    [1] = anon_sym_BQUOTE,
  },
  [10] = {
    [1] = anon_sym_SLASH,
  },
  [22] = {
    [0] = sym_rational,
  },
  [23] = {
    [2] = sym__string_start,
  },
  [24] = {
    [2] = anon_sym_BQUOTE,
  },
  [25] = {
    [2] = anon_sym_SLASH,
  },
  [26] = {
    [1] = alias_sym_bare_string,
  },
  [27] = {
    [1] = alias_sym_bare_symbol,
  },
  [29] = {
    [0] = sym_hash_key_symbol,
  },
  [36] = {
    [0] = alias_sym_destructured_parameter,
  },
  [37] = {
    [1] = alias_sym_lambda_parameters,
  },
  [43] = {
    [0] = sym_array_pattern,
  },
  [44] = {
    [0] = sym_find_pattern,
  },
  [45] = {
    [0] = sym_hash_pattern,
  },
  [55] = {
    [2] = alias_sym_bare_string,
  },
  [56] = {
    [2] = alias_sym_bare_symbol,
  },
  [58] = {
    [0] = sym_hash_key_symbol,
  },
  [77] = {
    [1] = sym_splat_parameter,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym_parameters, 3,
    sym_parameters,
    alias_sym_destructured_parameter,
    alias_sym_lambda_parameters,
  sym_bare_parameters, 2,
    sym_parameters,
    alias_sym_lambda_parameters,
  sym__array_pattern_n, 2,
    sym__array_pattern_n,
    sym_array_pattern,
  sym__find_pattern_body, 2,
    sym__find_pattern_body,
    sym_find_pattern,
  sym__hash_pattern_body, 2,
    sym__hash_pattern_body,
    sym_hash_pattern,
  sym__call, 2,
    sym__call,
    sym_call,
  sym__chained_command_call, 2,
    sym__chained_command_call,
    sym_call,
  sym__int_or_float, 2,
    sym__int_or_float,
    sym_rational,
  sym_constant_suffix, 2,
    sym_hash_key_symbol,
    sym_constant,
  sym_identifier_suffix, 2,
    sym_hash_key_symbol,
    sym_identifier,
  aux_sym__literal_contents, 3,
    aux_sym__literal_contents,
    alias_sym_bare_string,
    alias_sym_bare_symbol,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 2,
  [4] = 4,
  [5] = 2,
  [6] = 4,
  [7] = 4,
  [8] = 2,
  [9] = 4,
  [10] = 2,
  [11] = 4,
  [12] = 2,
  [13] = 4,
  [14] = 4,
  [15] = 2,
  [16] = 4,
  [17] = 2,
  [18] = 2,
  [19] = 4,
  [20] = 4,
  [21] = 4,
  [22] = 4,
  [23] = 2,
  [24] = 4,
  [25] = 2,
  [26] = 2,
  [27] = 4,
  [28] = 2,
  [29] = 4,
  [30] = 4,
  [31] = 2,
  [32] = 2,
  [33] = 2,
  [34] = 2,
  [35] = 4,
  [36] = 2,
  [37] = 4,
  [38] = 4,
  [39] = 2,
  [40] = 2,
  [41] = 4,
  [42] = 42,
  [43] = 42,
  [44] = 42,
  [45] = 45,
  [46] = 46,
  [47] = 47,
  [48] = 48,
  [49] = 47,
  [50] = 48,
  [51] = 46,
  [52] = 45,
  [53] = 42,
  [54] = 42,
  [55] = 42,
  [56] = 42,
  [57] = 42,
  [58] = 48,
  [59] = 47,
  [60] = 42,
  [61] = 42,
  [62] = 42,
  [63] = 46,
  [64] = 45,
  [65] = 42,
  [66] = 42,
  [67] = 42,
  [68] = 48,
  [69] = 47,
  [70] = 42,
  [71] = 42,
  [72] = 46,
  [73] = 48,
  [74] = 45,
  [75] = 45,
  [76] = 42,
  [77] = 46,
  [78] = 42,
  [79] = 47,
  [80] = 48,
  [81] = 45,
  [82] = 47,
  [83] = 46,
  [84] = 45,
  [85] = 48,
  [86] = 46,
  [87] = 47,
  [88] = 46,
  [89] = 45,
  [90] = 48,
  [91] = 47,
  [92] = 46,
  [93] = 45,
  [94] = 42,
  [95] = 48,
  [96] = 47,
  [97] = 48,
  [98] = 48,
  [99] = 46,
  [100] = 45,
  [101] = 46,
  [102] = 45,
  [103] = 47,
  [104] = 42,
  [105] = 47,
  [106] = 45,
  [107] = 48,
  [108] = 46,
  [109] = 47,
  [110] = 47,
  [111] = 46,
  [112] = 45,
  [113] = 48,
  [114] = 114,
  [115] = 114,
  [116] = 114,
  [117] = 114,
  [118] = 118,
  [119] = 118,
  [120] = 118,
  [121] = 118,
  [122] = 118,
  [123] = 123,
  [124] = 123,
  [125] = 123,
  [126] = 126,
  [127] = 127,
  [128] = 126,
  [129] = 127,
  [130] = 126,
  [131] = 127,
  [132] = 132,
  [133] = 132,
  [134] = 132,
  [135] = 135,
  [136] = 135,
  [137] = 135,
  [138] = 138,
  [139] = 135,
  [140] = 135,
  [141] = 138,
  [142] = 138,
  [143] = 135,
  [144] = 138,
  [145] = 135,
  [146] = 135,
  [147] = 138,
  [148] = 138,
  [149] = 135,
  [150] = 135,
  [151] = 135,
  [152] = 135,
  [153] = 135,
  [154] = 135,
  [155] = 138,
  [156] = 135,
  [157] = 135,
  [158] = 138,
  [159] = 135,
  [160] = 135,
  [161] = 138,
  [162] = 135,
  [163] = 135,
  [164] = 138,
  [165] = 135,
  [166] = 135,
  [167] = 135,
  [168] = 135,
  [169] = 169,
  [170] = 169,
  [171] = 171,
  [172] = 169,
  [173] = 173,
  [174] = 171,
  [175] = 169,
  [176] = 173,
  [177] = 173,
  [178] = 171,
  [179] = 169,
  [180] = 173,
  [181] = 171,
  [182] = 173,
  [183] = 171,
  [184] = 169,
  [185] = 173,
  [186] = 171,
  [187] = 169,
  [188] = 173,
  [189] = 171,
  [190] = 169,
  [191] = 173,
  [192] = 171,
  [193] = 173,
  [194] = 171,
  [195] = 171,
  [196] = 169,
  [197] = 173,
  [198] = 169,
  [199] = 199,
  [200] = 200,
  [201] = 199,
  [202] = 202,
  [203] = 199,
  [204] = 200,
  [205] = 205,
  [206] = 206,
  [207] = 207,
  [208] = 208,
  [209] = 209,
  [210] = 210,
  [211] = 211,
  [212] = 212,
  [213] = 205,
  [214] = 206,
  [215] = 207,
  [216] = 208,
  [217] = 209,
  [218] = 210,
  [219] = 206,
  [220] = 207,
  [221] = 202,
  [222] = 206,
  [223] = 207,
  [224] = 206,
  [225] = 207,
  [226] = 206,
  [227] = 207,
  [228] = 206,
  [229] = 208,
  [230] = 207,
  [231] = 209,
  [232] = 206,
  [233] = 210,
  [234] = 207,
  [235] = 206,
  [236] = 207,
  [237] = 207,
  [238] = 211,
  [239] = 212,
  [240] = 205,
  [241] = 211,
  [242] = 206,
  [243] = 212,
  [244] = 244,
  [245] = 200,
  [246] = 244,
  [247] = 244,
  [248] = 248,
  [249] = 249,
  [250] = 249,
  [251] = 249,
  [252] = 252,
  [253] = 249,
  [254] = 249,
  [255] = 249,
  [256] = 249,
  [257] = 257,
  [258] = 248,
  [259] = 249,
  [260] = 260,
  [261] = 200,
  [262] = 262,
  [263] = 202,
  [264] = 264,
  [265] = 264,
  [266] = 266,
  [267] = 264,
  [268] = 268,
  [269] = 266,
  [270] = 202,
  [271] = 200,
  [272] = 266,
  [273] = 273,
  [274] = 202,
  [275] = 275,
  [276] = 276,
  [277] = 276,
  [278] = 200,
  [279] = 200,
  [280] = 280,
  [281] = 281,
  [282] = 282,
  [283] = 276,
  [284] = 284,
  [285] = 202,
  [286] = 200,
  [287] = 284,
  [288] = 202,
  [289] = 289,
  [290] = 284,
  [291] = 284,
  [292] = 284,
  [293] = 284,
  [294] = 284,
  [295] = 284,
  [296] = 200,
  [297] = 297,
  [298] = 298,
  [299] = 299,
  [300] = 284,
  [301] = 301,
  [302] = 302,
  [303] = 284,
  [304] = 200,
  [305] = 200,
  [306] = 202,
  [307] = 202,
  [308] = 202,
  [309] = 200,
  [310] = 200,
  [311] = 202,
  [312] = 200,
  [313] = 313,
  [314] = 200,
  [315] = 313,
  [316] = 248,
  [317] = 313,
  [318] = 313,
  [319] = 313,
  [320] = 313,
  [321] = 313,
  [322] = 202,
  [323] = 313,
  [324] = 313,
  [325] = 200,
  [326] = 200,
  [327] = 313,
  [328] = 202,
  [329] = 329,
  [330] = 330,
  [331] = 331,
  [332] = 330,
  [333] = 330,
  [334] = 331,
  [335] = 330,
  [336] = 330,
  [337] = 330,
  [338] = 248,
  [339] = 330,
  [340] = 330,
  [341] = 330,
  [342] = 330,
  [343] = 202,
  [344] = 330,
  [345] = 330,
  [346] = 252,
  [347] = 200,
  [348] = 348,
  [349] = 349,
  [350] = 350,
  [351] = 329,
  [352] = 329,
  [353] = 353,
  [354] = 348,
  [355] = 257,
  [356] = 248,
  [357] = 348,
  [358] = 330,
  [359] = 200,
  [360] = 331,
  [361] = 248,
  [362] = 202,
  [363] = 248,
  [364] = 202,
  [365] = 202,
  [366] = 200,
  [367] = 200,
  [368] = 200,
  [369] = 248,
  [370] = 248,
  [371] = 200,
  [372] = 200,
  [373] = 202,
  [374] = 202,
  [375] = 200,
  [376] = 200,
  [377] = 202,
  [378] = 248,
  [379] = 202,
  [380] = 380,
  [381] = 248,
  [382] = 200,
  [383] = 380,
  [384] = 200,
  [385] = 202,
  [386] = 380,
  [387] = 200,
  [388] = 200,
  [389] = 200,
  [390] = 248,
  [391] = 202,
  [392] = 200,
  [393] = 200,
  [394] = 200,
  [395] = 200,
  [396] = 200,
  [397] = 202,
  [398] = 248,
  [399] = 202,
  [400] = 200,
  [401] = 200,
  [402] = 402,
  [403] = 403,
  [404] = 404,
  [405] = 200,
  [406] = 406,
  [407] = 404,
  [408] = 202,
  [409] = 403,
  [410] = 402,
  [411] = 411,
  [412] = 402,
  [413] = 404,
  [414] = 406,
  [415] = 403,
  [416] = 200,
  [417] = 406,
  [418] = 200,
  [419] = 202,
  [420] = 200,
  [421] = 200,
  [422] = 422,
  [423] = 422,
  [424] = 424,
  [425] = 422,
  [426] = 202,
  [427] = 422,
  [428] = 428,
  [429] = 424,
  [430] = 424,
  [431] = 422,
  [432] = 432,
  [433] = 428,
  [434] = 200,
  [435] = 435,
  [436] = 422,
  [437] = 200,
  [438] = 428,
  [439] = 439,
  [440] = 422,
  [441] = 441,
  [442] = 200,
  [443] = 443,
  [444] = 443,
  [445] = 443,
  [446] = 443,
  [447] = 443,
  [448] = 443,
  [449] = 443,
  [450] = 443,
  [451] = 443,
  [452] = 443,
  [453] = 443,
  [454] = 443,
  [455] = 443,
  [456] = 443,
  [457] = 457,
  [458] = 457,
  [459] = 457,
  [460] = 457,
  [461] = 457,
  [462] = 457,
  [463] = 457,
  [464] = 457,
  [465] = 457,
  [466] = 457,
  [467] = 457,
  [468] = 457,
  [469] = 457,
  [470] = 457,
  [471] = 457,
  [472] = 457,
  [473] = 457,
  [474] = 457,
  [475] = 457,
  [476] = 457,
  [477] = 457,
  [478] = 457,
  [479] = 457,
  [480] = 457,
  [481] = 457,
  [482] = 482,
  [483] = 483,
  [484] = 484,
  [485] = 482,
  [486] = 484,
  [487] = 484,
  [488] = 484,
  [489] = 484,
  [490] = 484,
  [491] = 483,
  [492] = 484,
  [493] = 484,
  [494] = 484,
  [495] = 202,
  [496] = 496,
  [497] = 484,
  [498] = 498,
  [499] = 484,
  [500] = 484,
  [501] = 484,
  [502] = 484,
  [503] = 484,
  [504] = 484,
  [505] = 484,
  [506] = 484,
  [507] = 484,
  [508] = 484,
  [509] = 484,
  [510] = 484,
  [511] = 484,
  [512] = 512,
  [513] = 484,
  [514] = 484,
  [515] = 498,
  [516] = 496,
  [517] = 202,
  [518] = 512,
  [519] = 519,
  [520] = 520,
  [521] = 521,
  [522] = 522,
  [523] = 523,
  [524] = 524,
  [525] = 525,
  [526] = 526,
  [527] = 527,
  [528] = 528,
  [529] = 525,
  [530] = 526,
  [531] = 527,
  [532] = 528,
  [533] = 533,
  [534] = 533,
  [535] = 526,
  [536] = 527,
  [537] = 528,
  [538] = 533,
  [539] = 533,
  [540] = 540,
  [541] = 525,
  [542] = 542,
  [543] = 520,
  [544] = 526,
  [545] = 527,
  [546] = 526,
  [547] = 525,
  [548] = 526,
  [549] = 527,
  [550] = 528,
  [551] = 533,
  [552] = 527,
  [553] = 528,
  [554] = 533,
  [555] = 528,
  [556] = 533,
  [557] = 557,
  [558] = 525,
  [559] = 524,
  [560] = 521,
  [561] = 525,
  [562] = 525,
  [563] = 563,
  [564] = 522,
  [565] = 523,
  [566] = 524,
  [567] = 526,
  [568] = 527,
  [569] = 525,
  [570] = 528,
  [571] = 533,
  [572] = 526,
  [573] = 527,
  [574] = 528,
  [575] = 533,
  [576] = 525,
  [577] = 542,
  [578] = 520,
  [579] = 521,
  [580] = 522,
  [581] = 523,
  [582] = 524,
  [583] = 524,
  [584] = 525,
  [585] = 526,
  [586] = 525,
  [587] = 527,
  [588] = 525,
  [589] = 526,
  [590] = 527,
  [591] = 528,
  [592] = 533,
  [593] = 528,
  [594] = 525,
  [595] = 524,
  [596] = 533,
  [597] = 527,
  [598] = 528,
  [599] = 533,
  [600] = 525,
  [601] = 526,
  [602] = 526,
  [603] = 527,
  [604] = 525,
  [605] = 528,
  [606] = 533,
  [607] = 527,
  [608] = 528,
  [609] = 533,
  [610] = 525,
  [611] = 525,
  [612] = 525,
  [613] = 525,
  [614] = 526,
  [615] = 526,
  [616] = 527,
  [617] = 528,
  [618] = 533,
  [619] = 527,
  [620] = 557,
  [621] = 528,
  [622] = 533,
  [623] = 526,
  [624] = 527,
  [625] = 526,
  [626] = 527,
  [627] = 528,
  [628] = 533,
  [629] = 528,
  [630] = 533,
  [631] = 525,
  [632] = 526,
  [633] = 527,
  [634] = 528,
  [635] = 533,
  [636] = 526,
  [637] = 527,
  [638] = 528,
  [639] = 533,
  [640] = 526,
  [641] = 527,
  [642] = 528,
  [643] = 525,
  [644] = 533,
  [645] = 525,
  [646] = 542,
  [647] = 526,
  [648] = 527,
  [649] = 528,
  [650] = 525,
  [651] = 533,
  [652] = 526,
  [653] = 527,
  [654] = 526,
  [655] = 527,
  [656] = 528,
  [657] = 533,
  [658] = 526,
  [659] = 527,
  [660] = 528,
  [661] = 533,
  [662] = 526,
  [663] = 527,
  [664] = 528,
  [665] = 533,
  [666] = 525,
  [667] = 525,
  [668] = 528,
  [669] = 526,
  [670] = 670,
  [671] = 670,
  [672] = 670,
  [673] = 670,
  [674] = 670,
  [675] = 483,
  [676] = 482,
  [677] = 677,
  [678] = 678,
  [679] = 202,
  [680] = 496,
  [681] = 482,
  [682] = 498,
  [683] = 483,
  [684] = 512,
  [685] = 496,
  [686] = 512,
  [687] = 202,
  [688] = 483,
  [689] = 482,
  [690] = 498,
  [691] = 482,
  [692] = 483,
  [693] = 482,
  [694] = 483,
  [695] = 498,
  [696] = 482,
  [697] = 697,
  [698] = 697,
  [699] = 202,
  [700] = 512,
  [701] = 202,
  [702] = 496,
  [703] = 697,
  [704] = 697,
  [705] = 697,
  [706] = 483,
  [707] = 498,
  [708] = 498,
  [709] = 512,
  [710] = 697,
  [711] = 697,
  [712] = 482,
  [713] = 496,
  [714] = 202,
  [715] = 697,
  [716] = 483,
  [717] = 512,
  [718] = 697,
  [719] = 496,
  [720] = 697,
  [721] = 697,
  [722] = 482,
  [723] = 697,
  [724] = 697,
  [725] = 697,
  [726] = 483,
  [727] = 202,
  [728] = 498,
  [729] = 498,
  [730] = 512,
  [731] = 202,
  [732] = 496,
  [733] = 483,
  [734] = 512,
  [735] = 512,
  [736] = 202,
  [737] = 496,
  [738] = 496,
  [739] = 498,
  [740] = 482,
  [741] = 741,
  [742] = 742,
  [743] = 512,
  [744] = 202,
  [745] = 741,
  [746] = 746,
  [747] = 202,
  [748] = 746,
  [749] = 483,
  [750] = 482,
  [751] = 742,
  [752] = 742,
  [753] = 498,
  [754] = 741,
  [755] = 496,
  [756] = 746,
  [757] = 757,
  [758] = 757,
  [759] = 757,
  [760] = 757,
  [761] = 757,
  [762] = 757,
  [763] = 757,
  [764] = 757,
  [765] = 757,
  [766] = 757,
  [767] = 757,
  [768] = 757,
  [769] = 757,
  [770] = 757,
  [771] = 757,
  [772] = 757,
  [773] = 757,
  [774] = 757,
  [775] = 757,
  [776] = 757,
  [777] = 757,
  [778] = 757,
  [779] = 757,
  [780] = 757,
  [781] = 757,
  [782] = 757,
  [783] = 757,
  [784] = 757,
  [785] = 757,
  [786] = 757,
  [787] = 757,
  [788] = 757,
  [789] = 789,
  [790] = 757,
  [791] = 757,
  [792] = 757,
  [793] = 757,
  [794] = 757,
  [795] = 498,
  [796] = 757,
  [797] = 757,
  [798] = 757,
  [799] = 757,
  [800] = 757,
  [801] = 757,
  [802] = 757,
  [803] = 757,
  [804] = 757,
  [805] = 805,
  [806] = 805,
  [807] = 807,
  [808] = 512,
  [809] = 202,
  [810] = 810,
  [811] = 811,
  [812] = 811,
  [813] = 811,
  [814] = 811,
  [815] = 811,
  [816] = 811,
  [817] = 811,
  [818] = 811,
  [819] = 811,
  [820] = 805,
  [821] = 811,
  [822] = 811,
  [823] = 811,
  [824] = 811,
  [825] = 811,
  [826] = 811,
  [827] = 811,
  [828] = 805,
  [829] = 811,
  [830] = 811,
  [831] = 811,
  [832] = 811,
  [833] = 811,
  [834] = 811,
  [835] = 811,
  [836] = 811,
  [837] = 811,
  [838] = 811,
  [839] = 811,
  [840] = 811,
  [841] = 811,
  [842] = 842,
  [843] = 843,
  [844] = 843,
  [845] = 842,
  [846] = 843,
  [847] = 842,
  [848] = 843,
  [849] = 843,
  [850] = 843,
  [851] = 843,
  [852] = 843,
  [853] = 842,
  [854] = 843,
  [855] = 842,
  [856] = 843,
  [857] = 843,
  [858] = 842,
  [859] = 843,
  [860] = 843,
  [861] = 843,
  [862] = 843,
  [863] = 842,
  [864] = 842,
  [865] = 843,
  [866] = 843,
  [867] = 843,
  [868] = 842,
  [869] = 843,
  [870] = 843,
  [871] = 843,
  [872] = 842,
  [873] = 843,
  [874] = 842,
  [875] = 843,
  [876] = 842,
  [877] = 843,
  [878] = 843,
  [879] = 842,
  [880] = 843,
  [881] = 843,
  [882] = 842,
  [883] = 842,
  [884] = 843,
  [885] = 843,
  [886] = 842,
  [887] = 843,
  [888] = 843,
  [889] = 843,
  [890] = 843,
  [891] = 843,
  [892] = 843,
  [893] = 843,
  [894] = 843,
  [895] = 843,
  [896] = 843,
  [897] = 843,
  [898] = 842,
  [899] = 842,
  [900] = 843,
  [901] = 842,
  [902] = 843,
  [903] = 842,
  [904] = 843,
  [905] = 843,
  [906] = 843,
  [907] = 842,
  [908] = 842,
  [909] = 842,
  [910] = 842,
  [911] = 843,
  [912] = 912,
  [913] = 912,
  [914] = 914,
  [915] = 915,
  [916] = 916,
  [917] = 917,
  [918] = 918,
  [919] = 919,
  [920] = 920,
  [921] = 921,
  [922] = 922,
  [923] = 923,
  [924] = 924,
  [925] = 925,
  [926] = 926,
  [927] = 927,
  [928] = 928,
  [929] = 929,
  [930] = 930,
  [931] = 931,
  [932] = 932,
  [933] = 933,
  [934] = 934,
  [935] = 916,
  [936] = 917,
  [937] = 918,
  [938] = 919,
  [939] = 933,
  [940] = 940,
  [941] = 915,
  [942] = 920,
  [943] = 921,
  [944] = 922,
  [945] = 923,
  [946] = 930,
  [947] = 932,
  [948] = 934,
  [949] = 924,
  [950] = 925,
  [951] = 914,
  [952] = 915,
  [953] = 953,
  [954] = 926,
  [955] = 927,
  [956] = 928,
  [957] = 916,
  [958] = 917,
  [959] = 918,
  [960] = 919,
  [961] = 929,
  [962] = 920,
  [963] = 921,
  [964] = 922,
  [965] = 923,
  [966] = 916,
  [967] = 924,
  [968] = 925,
  [969] = 953,
  [970] = 926,
  [971] = 927,
  [972] = 928,
  [973] = 929,
  [974] = 931,
  [975] = 917,
  [976] = 931,
  [977] = 933,
  [978] = 933,
  [979] = 930,
  [980] = 932,
  [981] = 934,
  [982] = 916,
  [983] = 917,
  [984] = 918,
  [985] = 919,
  [986] = 920,
  [987] = 921,
  [988] = 922,
  [989] = 923,
  [990] = 924,
  [991] = 925,
  [992] = 953,
  [993] = 926,
  [994] = 927,
  [995] = 928,
  [996] = 929,
  [997] = 918,
  [998] = 919,
  [999] = 931,
  [1000] = 930,
  [1001] = 932,
  [1002] = 934,
  [1003] = 916,
  [1004] = 917,
  [1005] = 918,
  [1006] = 919,
  [1007] = 920,
  [1008] = 921,
  [1009] = 922,
  [1010] = 923,
  [1011] = 924,
  [1012] = 925,
  [1013] = 953,
  [1014] = 926,
  [1015] = 927,
  [1016] = 928,
  [1017] = 929,
  [1018] = 931,
  [1019] = 930,
  [1020] = 932,
  [1021] = 934,
  [1022] = 916,
  [1023] = 917,
  [1024] = 918,
  [1025] = 919,
  [1026] = 920,
  [1027] = 921,
  [1028] = 922,
  [1029] = 923,
  [1030] = 924,
  [1031] = 925,
  [1032] = 926,
  [1033] = 927,
  [1034] = 928,
  [1035] = 929,
  [1036] = 931,
  [1037] = 930,
  [1038] = 934,
  [1039] = 930,
  [1040] = 932,
  [1041] = 934,
  [1042] = 914,
  [1043] = 915,
  [1044] = 916,
  [1045] = 917,
  [1046] = 918,
  [1047] = 919,
  [1048] = 920,
  [1049] = 921,
  [1050] = 922,
  [1051] = 923,
  [1052] = 924,
  [1053] = 925,
  [1054] = 926,
  [1055] = 927,
  [1056] = 928,
  [1057] = 929,
  [1058] = 931,
  [1059] = 933,
  [1060] = 912,
  [1061] = 914,
  [1062] = 930,
  [1063] = 932,
  [1064] = 934,
  [1065] = 914,
  [1066] = 915,
  [1067] = 916,
  [1068] = 917,
  [1069] = 918,
  [1070] = 919,
  [1071] = 920,
  [1072] = 921,
  [1073] = 922,
  [1074] = 923,
  [1075] = 924,
  [1076] = 925,
  [1077] = 953,
  [1078] = 926,
  [1079] = 927,
  [1080] = 928,
  [1081] = 929,
  [1082] = 915,
  [1083] = 931,
  [1084] = 933,
  [1085] = 1085,
  [1086] = 914,
  [1087] = 930,
  [1088] = 934,
  [1089] = 916,
  [1090] = 917,
  [1091] = 918,
  [1092] = 919,
  [1093] = 920,
  [1094] = 921,
  [1095] = 922,
  [1096] = 923,
  [1097] = 924,
  [1098] = 925,
  [1099] = 926,
  [1100] = 927,
  [1101] = 928,
  [1102] = 929,
  [1103] = 915,
  [1104] = 931,
  [1105] = 930,
  [1106] = 934,
  [1107] = 916,
  [1108] = 917,
  [1109] = 918,
  [1110] = 919,
  [1111] = 920,
  [1112] = 921,
  [1113] = 922,
  [1114] = 923,
  [1115] = 924,
  [1116] = 925,
  [1117] = 926,
  [1118] = 927,
  [1119] = 928,
  [1120] = 929,
  [1121] = 931,
  [1122] = 930,
  [1123] = 934,
  [1124] = 916,
  [1125] = 917,
  [1126] = 918,
  [1127] = 919,
  [1128] = 920,
  [1129] = 921,
  [1130] = 922,
  [1131] = 923,
  [1132] = 924,
  [1133] = 925,
  [1134] = 926,
  [1135] = 927,
  [1136] = 928,
  [1137] = 929,
  [1138] = 931,
  [1139] = 916,
  [1140] = 930,
  [1141] = 934,
  [1142] = 917,
  [1143] = 918,
  [1144] = 919,
  [1145] = 914,
  [1146] = 915,
  [1147] = 916,
  [1148] = 917,
  [1149] = 918,
  [1150] = 919,
  [1151] = 916,
  [1152] = 917,
  [1153] = 918,
  [1154] = 920,
  [1155] = 921,
  [1156] = 922,
  [1157] = 923,
  [1158] = 924,
  [1159] = 925,
  [1160] = 926,
  [1161] = 927,
  [1162] = 928,
  [1163] = 929,
  [1164] = 919,
  [1165] = 920,
  [1166] = 931,
  [1167] = 1167,
  [1168] = 933,
  [1169] = 921,
  [1170] = 922,
  [1171] = 920,
  [1172] = 921,
  [1173] = 922,
  [1174] = 920,
  [1175] = 921,
  [1176] = 922,
  [1177] = 930,
  [1178] = 932,
  [1179] = 934,
  [1180] = 923,
  [1181] = 923,
  [1182] = 914,
  [1183] = 915,
  [1184] = 916,
  [1185] = 917,
  [1186] = 918,
  [1187] = 919,
  [1188] = 920,
  [1189] = 921,
  [1190] = 922,
  [1191] = 923,
  [1192] = 924,
  [1193] = 924,
  [1194] = 925,
  [1195] = 926,
  [1196] = 927,
  [1197] = 928,
  [1198] = 929,
  [1199] = 931,
  [1200] = 925,
  [1201] = 926,
  [1202] = 927,
  [1203] = 930,
  [1204] = 934,
  [1205] = 916,
  [1206] = 917,
  [1207] = 918,
  [1208] = 919,
  [1209] = 920,
  [1210] = 921,
  [1211] = 922,
  [1212] = 923,
  [1213] = 924,
  [1214] = 925,
  [1215] = 926,
  [1216] = 927,
  [1217] = 928,
  [1218] = 929,
  [1219] = 928,
  [1220] = 929,
  [1221] = 931,
  [1222] = 930,
  [1223] = 934,
  [1224] = 916,
  [1225] = 917,
  [1226] = 918,
  [1227] = 919,
  [1228] = 920,
  [1229] = 921,
  [1230] = 922,
  [1231] = 923,
  [1232] = 924,
  [1233] = 925,
  [1234] = 926,
  [1235] = 927,
  [1236] = 928,
  [1237] = 929,
  [1238] = 931,
  [1239] = 930,
  [1240] = 934,
  [1241] = 924,
  [1242] = 925,
  [1243] = 914,
  [1244] = 915,
  [1245] = 916,
  [1246] = 917,
  [1247] = 918,
  [1248] = 919,
  [1249] = 926,
  [1250] = 931,
  [1251] = 920,
  [1252] = 921,
  [1253] = 922,
  [1254] = 923,
  [1255] = 924,
  [1256] = 925,
  [1257] = 926,
  [1258] = 927,
  [1259] = 928,
  [1260] = 929,
  [1261] = 927,
  [1262] = 928,
  [1263] = 931,
  [1264] = 929,
  [1265] = 933,
  [1266] = 923,
  [1267] = 931,
  [1268] = 1268,
  [1269] = 940,
  [1270] = 924,
  [1271] = 930,
  [1272] = 932,
  [1273] = 934,
  [1274] = 914,
  [1275] = 915,
  [1276] = 916,
  [1277] = 917,
  [1278] = 918,
  [1279] = 919,
  [1280] = 920,
  [1281] = 921,
  [1282] = 922,
  [1283] = 923,
  [1284] = 924,
  [1285] = 925,
  [1286] = 953,
  [1287] = 926,
  [1288] = 927,
  [1289] = 928,
  [1290] = 929,
  [1291] = 931,
  [1292] = 916,
  [1293] = 930,
  [1294] = 932,
  [1295] = 930,
  [1296] = 934,
  [1297] = 916,
  [1298] = 917,
  [1299] = 918,
  [1300] = 919,
  [1301] = 920,
  [1302] = 921,
  [1303] = 922,
  [1304] = 923,
  [1305] = 924,
  [1306] = 925,
  [1307] = 926,
  [1308] = 927,
  [1309] = 928,
  [1310] = 929,
  [1311] = 940,
  [1312] = 934,
  [1313] = 931,
  [1314] = 930,
  [1315] = 934,
  [1316] = 916,
  [1317] = 917,
  [1318] = 918,
  [1319] = 919,
  [1320] = 920,
  [1321] = 921,
  [1322] = 922,
  [1323] = 923,
  [1324] = 924,
  [1325] = 925,
  [1326] = 926,
  [1327] = 927,
  [1328] = 928,
  [1329] = 929,
  [1330] = 931,
  [1331] = 930,
  [1332] = 934,
  [1333] = 914,
  [1334] = 915,
  [1335] = 916,
  [1336] = 917,
  [1337] = 918,
  [1338] = 919,
  [1339] = 917,
  [1340] = 918,
  [1341] = 920,
  [1342] = 921,
  [1343] = 922,
  [1344] = 923,
  [1345] = 924,
  [1346] = 925,
  [1347] = 926,
  [1348] = 927,
  [1349] = 928,
  [1350] = 929,
  [1351] = 919,
  [1352] = 931,
  [1353] = 930,
  [1354] = 930,
  [1355] = 932,
  [1356] = 934,
  [1357] = 932,
  [1358] = 914,
  [1359] = 915,
  [1360] = 916,
  [1361] = 917,
  [1362] = 918,
  [1363] = 919,
  [1364] = 920,
  [1365] = 921,
  [1366] = 922,
  [1367] = 923,
  [1368] = 924,
  [1369] = 925,
  [1370] = 926,
  [1371] = 927,
  [1372] = 928,
  [1373] = 929,
  [1374] = 931,
  [1375] = 934,
  [1376] = 914,
  [1377] = 915,
  [1378] = 930,
  [1379] = 934,
  [1380] = 916,
  [1381] = 917,
  [1382] = 918,
  [1383] = 919,
  [1384] = 920,
  [1385] = 921,
  [1386] = 922,
  [1387] = 923,
  [1388] = 924,
  [1389] = 925,
  [1390] = 926,
  [1391] = 927,
  [1392] = 928,
  [1393] = 929,
  [1394] = 931,
  [1395] = 930,
  [1396] = 934,
  [1397] = 914,
  [1398] = 915,
  [1399] = 916,
  [1400] = 917,
  [1401] = 918,
  [1402] = 919,
  [1403] = 916,
  [1404] = 920,
  [1405] = 921,
  [1406] = 922,
  [1407] = 923,
  [1408] = 924,
  [1409] = 925,
  [1410] = 926,
  [1411] = 927,
  [1412] = 928,
  [1413] = 929,
  [1414] = 917,
  [1415] = 931,
  [1416] = 918,
  [1417] = 919,
  [1418] = 916,
  [1419] = 917,
  [1420] = 918,
  [1421] = 930,
  [1422] = 934,
  [1423] = 919,
  [1424] = 914,
  [1425] = 915,
  [1426] = 916,
  [1427] = 917,
  [1428] = 918,
  [1429] = 919,
  [1430] = 920,
  [1431] = 921,
  [1432] = 922,
  [1433] = 923,
  [1434] = 924,
  [1435] = 925,
  [1436] = 926,
  [1437] = 927,
  [1438] = 928,
  [1439] = 929,
  [1440] = 931,
  [1441] = 925,
  [1442] = 920,
  [1443] = 921,
  [1444] = 930,
  [1445] = 934,
  [1446] = 916,
  [1447] = 917,
  [1448] = 918,
  [1449] = 919,
  [1450] = 920,
  [1451] = 921,
  [1452] = 922,
  [1453] = 923,
  [1454] = 924,
  [1455] = 925,
  [1456] = 926,
  [1457] = 927,
  [1458] = 928,
  [1459] = 929,
  [1460] = 931,
  [1461] = 930,
  [1462] = 934,
  [1463] = 922,
  [1464] = 914,
  [1465] = 915,
  [1466] = 916,
  [1467] = 917,
  [1468] = 919,
  [1469] = 923,
  [1470] = 1470,
  [1471] = 920,
  [1472] = 921,
  [1473] = 922,
  [1474] = 923,
  [1475] = 924,
  [1476] = 925,
  [1477] = 926,
  [1478] = 927,
  [1479] = 928,
  [1480] = 929,
  [1481] = 931,
  [1482] = 924,
  [1483] = 925,
  [1484] = 920,
  [1485] = 921,
  [1486] = 922,
  [1487] = 923,
  [1488] = 926,
  [1489] = 930,
  [1490] = 934,
  [1491] = 924,
  [1492] = 914,
  [1493] = 915,
  [1494] = 916,
  [1495] = 917,
  [1496] = 918,
  [1497] = 919,
  [1498] = 920,
  [1499] = 921,
  [1500] = 922,
  [1501] = 923,
  [1502] = 924,
  [1503] = 925,
  [1504] = 926,
  [1505] = 927,
  [1506] = 928,
  [1507] = 929,
  [1508] = 931,
  [1509] = 925,
  [1510] = 953,
  [1511] = 930,
  [1512] = 934,
  [1513] = 926,
  [1514] = 914,
  [1515] = 915,
  [1516] = 916,
  [1517] = 917,
  [1518] = 918,
  [1519] = 919,
  [1520] = 927,
  [1521] = 928,
  [1522] = 920,
  [1523] = 921,
  [1524] = 922,
  [1525] = 923,
  [1526] = 924,
  [1527] = 925,
  [1528] = 926,
  [1529] = 927,
  [1530] = 928,
  [1531] = 929,
  [1532] = 931,
  [1533] = 929,
  [1534] = 927,
  [1535] = 928,
  [1536] = 931,
  [1537] = 929,
  [1538] = 926,
  [1539] = 930,
  [1540] = 934,
  [1541] = 931,
  [1542] = 914,
  [1543] = 915,
  [1544] = 916,
  [1545] = 917,
  [1546] = 918,
  [1547] = 919,
  [1548] = 920,
  [1549] = 921,
  [1550] = 922,
  [1551] = 923,
  [1552] = 924,
  [1553] = 925,
  [1554] = 926,
  [1555] = 927,
  [1556] = 928,
  [1557] = 929,
  [1558] = 931,
  [1559] = 933,
  [1560] = 920,
  [1561] = 921,
  [1562] = 930,
  [1563] = 934,
  [1564] = 922,
  [1565] = 914,
  [1566] = 915,
  [1567] = 917,
  [1568] = 918,
  [1569] = 919,
  [1570] = 920,
  [1571] = 921,
  [1572] = 922,
  [1573] = 923,
  [1574] = 924,
  [1575] = 925,
  [1576] = 926,
  [1577] = 927,
  [1578] = 928,
  [1579] = 929,
  [1580] = 931,
  [1581] = 923,
  [1582] = 930,
  [1583] = 930,
  [1584] = 934,
  [1585] = 914,
  [1586] = 915,
  [1587] = 934,
  [1588] = 914,
  [1589] = 915,
  [1590] = 917,
  [1591] = 918,
  [1592] = 919,
  [1593] = 920,
  [1594] = 921,
  [1595] = 922,
  [1596] = 923,
  [1597] = 924,
  [1598] = 925,
  [1599] = 926,
  [1600] = 927,
  [1601] = 928,
  [1602] = 929,
  [1603] = 931,
  [1604] = 930,
  [1605] = 932,
  [1606] = 930,
  [1607] = 934,
  [1608] = 914,
  [1609] = 915,
  [1610] = 934,
  [1611] = 934,
  [1612] = 914,
  [1613] = 915,
  [1614] = 917,
  [1615] = 918,
  [1616] = 919,
  [1617] = 920,
  [1618] = 921,
  [1619] = 922,
  [1620] = 923,
  [1621] = 924,
  [1622] = 925,
  [1623] = 926,
  [1624] = 927,
  [1625] = 928,
  [1626] = 929,
  [1627] = 931,
  [1628] = 927,
  [1629] = 924,
  [1630] = 930,
  [1631] = 934,
  [1632] = 914,
  [1633] = 915,
  [1634] = 916,
  [1635] = 917,
  [1636] = 918,
  [1637] = 914,
  [1638] = 915,
  [1639] = 917,
  [1640] = 918,
  [1641] = 919,
  [1642] = 920,
  [1643] = 921,
  [1644] = 922,
  [1645] = 923,
  [1646] = 924,
  [1647] = 925,
  [1648] = 926,
  [1649] = 927,
  [1650] = 928,
  [1651] = 929,
  [1652] = 931,
  [1653] = 919,
  [1654] = 925,
  [1655] = 930,
  [1656] = 934,
  [1657] = 914,
  [1658] = 915,
  [1659] = 914,
  [1660] = 915,
  [1661] = 914,
  [1662] = 915,
  [1663] = 953,
  [1664] = 914,
  [1665] = 915,
  [1666] = 926,
  [1667] = 920,
  [1668] = 914,
  [1669] = 915,
  [1670] = 921,
  [1671] = 922,
  [1672] = 914,
  [1673] = 915,
  [1674] = 923,
  [1675] = 927,
  [1676] = 914,
  [1677] = 915,
  [1678] = 924,
  [1679] = 925,
  [1680] = 914,
  [1681] = 915,
  [1682] = 953,
  [1683] = 926,
  [1684] = 914,
  [1685] = 915,
  [1686] = 927,
  [1687] = 928,
  [1688] = 914,
  [1689] = 915,
  [1690] = 929,
  [1691] = 928,
  [1692] = 914,
  [1693] = 915,
  [1694] = 929,
  [1695] = 931,
  [1696] = 915,
  [1697] = 928,
  [1698] = 914,
  [1699] = 915,
  [1700] = 933,
  [1701] = 915,
  [1702] = 916,
  [1703] = 914,
  [1704] = 915,
  [1705] = 930,
  [1706] = 932,
  [1707] = 934,
  [1708] = 916,
  [1709] = 916,
  [1710] = 1470,
  [1711] = 917,
  [1712] = 918,
  [1713] = 919,
  [1714] = 1167,
  [1715] = 929,
  [1716] = 920,
  [1717] = 921,
  [1718] = 922,
  [1719] = 923,
  [1720] = 931,
  [1721] = 924,
  [1722] = 925,
  [1723] = 926,
  [1724] = 927,
  [1725] = 928,
  [1726] = 914,
  [1727] = 929,
  [1728] = 931,
  [1729] = 914,
  [1730] = 916,
  [1731] = 916,
  [1732] = 916,
  [1733] = 930,
  [1734] = 1470,
  [1735] = 932,
  [1736] = 934,
  [1737] = 916,
  [1738] = 917,
  [1739] = 918,
  [1740] = 1470,
  [1741] = 1470,
  [1742] = 1470,
  [1743] = 1470,
  [1744] = 1470,
  [1745] = 1470,
  [1746] = 1470,
  [1747] = 1470,
  [1748] = 1470,
  [1749] = 1470,
  [1750] = 1470,
  [1751] = 1470,
  [1752] = 1470,
  [1753] = 1470,
  [1754] = 1470,
  [1755] = 1470,
  [1756] = 1470,
  [1757] = 1470,
  [1758] = 1470,
  [1759] = 1470,
  [1760] = 1470,
  [1761] = 1470,
  [1762] = 1470,
  [1763] = 1470,
  [1764] = 1470,
  [1765] = 1470,
  [1766] = 919,
  [1767] = 1268,
  [1768] = 1470,
  [1769] = 933,
  [1770] = 920,
  [1771] = 921,
  [1772] = 922,
  [1773] = 923,
  [1774] = 924,
  [1775] = 925,
  [1776] = 926,
  [1777] = 927,
  [1778] = 928,
  [1779] = 929,
  [1780] = 931,
  [1781] = 930,
  [1782] = 934,
  [1783] = 933,
  [1784] = 916,
  [1785] = 917,
  [1786] = 918,
  [1787] = 919,
  [1788] = 920,
  [1789] = 921,
  [1790] = 922,
  [1791] = 923,
  [1792] = 924,
  [1793] = 925,
  [1794] = 926,
  [1795] = 927,
  [1796] = 928,
  [1797] = 929,
  [1798] = 931,
  [1799] = 930,
  [1800] = 940,
  [1801] = 931,
  [1802] = 915,
  [1803] = 914,
  [1804] = 934,
  [1805] = 916,
  [1806] = 920,
  [1807] = 921,
  [1808] = 922,
  [1809] = 923,
  [1810] = 924,
  [1811] = 925,
  [1812] = 926,
  [1813] = 927,
  [1814] = 928,
  [1815] = 929,
  [1816] = 931,
  [1817] = 930,
  [1818] = 932,
  [1819] = 917,
  [1820] = 918,
  [1821] = 919,
  [1822] = 916,
  [1823] = 917,
  [1824] = 918,
  [1825] = 919,
  [1826] = 920,
  [1827] = 921,
  [1828] = 922,
  [1829] = 923,
  [1830] = 924,
  [1831] = 925,
  [1832] = 926,
  [1833] = 927,
  [1834] = 928,
  [1835] = 929,
  [1836] = 931,
  [1837] = 930,
  [1838] = 934,
  [1839] = 934,
  [1840] = 914,
  [1841] = 915,
  [1842] = 917,
  [1843] = 918,
  [1844] = 919,
  [1845] = 920,
  [1846] = 921,
  [1847] = 922,
  [1848] = 923,
  [1849] = 924,
  [1850] = 925,
  [1851] = 926,
  [1852] = 927,
  [1853] = 928,
  [1854] = 929,
  [1855] = 931,
  [1856] = 930,
  [1857] = 934,
  [1858] = 914,
  [1859] = 915,
  [1860] = 914,
  [1861] = 915,
  [1862] = 914,
  [1863] = 916,
  [1864] = 1470,
  [1865] = 1470,
  [1866] = 1268,
  [1867] = 1470,
  [1868] = 1470,
  [1869] = 1470,
  [1870] = 1470,
  [1871] = 1470,
  [1872] = 1470,
  [1873] = 1470,
  [1874] = 1470,
  [1875] = 1470,
  [1876] = 1470,
  [1877] = 1470,
  [1878] = 1470,
  [1879] = 1470,
  [1880] = 1470,
  [1881] = 912,
  [1882] = 930,
  [1883] = 932,
  [1884] = 934,
  [1885] = 912,
  [1886] = 912,
  [1887] = 912,
  [1888] = 912,
  [1889] = 912,
  [1890] = 912,
  [1891] = 912,
  [1892] = 912,
  [1893] = 912,
  [1894] = 912,
  [1895] = 912,
  [1896] = 912,
  [1897] = 912,
  [1898] = 912,
  [1899] = 912,
  [1900] = 912,
  [1901] = 912,
  [1902] = 912,
  [1903] = 912,
  [1904] = 912,
  [1905] = 918,
  [1906] = 1906,
  [1907] = 1907,
  [1908] = 1908,
  [1909] = 1909,
  [1910] = 1910,
  [1911] = 1911,
  [1912] = 1907,
  [1913] = 1913,
  [1914] = 1914,
  [1915] = 1915,
  [1916] = 1916,
  [1917] = 1167,
  [1918] = 1167,
  [1919] = 1907,
  [1920] = 1907,
  [1921] = 1167,
  [1922] = 1922,
  [1923] = 1913,
  [1924] = 1909,
  [1925] = 1908,
  [1926] = 1906,
  [1927] = 1915,
  [1928] = 1914,
  [1929] = 1910,
  [1930] = 1911,
  [1931] = 1907,
  [1932] = 1932,
  [1933] = 1167,
  [1934] = 1934,
  [1935] = 1167,
  [1936] = 1907,
  [1937] = 1167,
  [1938] = 1167,
  [1939] = 1167,
  [1940] = 1167,
  [1941] = 1910,
  [1942] = 1167,
  [1943] = 1907,
  [1944] = 1916,
  [1945] = 1945,
  [1946] = 1906,
  [1947] = 1911,
  [1948] = 1907,
  [1949] = 1907,
  [1950] = 1914,
  [1951] = 1167,
  [1952] = 1913,
  [1953] = 1915,
  [1954] = 1909,
  [1955] = 1908,
  [1956] = 1956,
  [1957] = 1934,
  [1958] = 1916,
  [1959] = 1959,
  [1960] = 1960,
  [1961] = 1961,
  [1962] = 1961,
  [1963] = 1961,
  [1964] = 1964,
  [1965] = 1965,
  [1966] = 1915,
  [1967] = 1964,
  [1968] = 1965,
  [1969] = 1906,
  [1970] = 1908,
  [1971] = 1914,
  [1972] = 1972,
  [1973] = 1973,
  [1974] = 1974,
  [1975] = 1974,
  [1976] = 1974,
  [1977] = 1977,
  [1978] = 1977,
  [1979] = 1979,
  [1980] = 1977,
  [1981] = 1965,
  [1982] = 1906,
  [1983] = 1965,
  [1984] = 1964,
  [1985] = 1915,
  [1986] = 1964,
  [1987] = 1908,
  [1988] = 1914,
  [1989] = 1906,
  [1990] = 1964,
  [1991] = 1964,
  [1992] = 1965,
  [1993] = 1965,
  [1994] = 1965,
  [1995] = 1964,
  [1996] = 1965,
  [1997] = 1965,
  [1998] = 1964,
  [1999] = 1964,
  [2000] = 1915,
  [2001] = 1965,
  [2002] = 1964,
  [2003] = 1965,
  [2004] = 1908,
  [2005] = 1964,
  [2006] = 1914,
  [2007] = 1965,
  [2008] = 1965,
  [2009] = 1964,
  [2010] = 1964,
  [2011] = 2011,
  [2012] = 2012,
  [2013] = 2013,
  [2014] = 2014,
  [2015] = 2015,
  [2016] = 2014,
  [2017] = 2015,
  [2018] = 2018,
  [2019] = 2019,
  [2020] = 2020,
  [2021] = 2018,
  [2022] = 2020,
  [2023] = 2018,
  [2024] = 2020,
  [2025] = 2020,
  [2026] = 2018,
  [2027] = 2018,
  [2028] = 2020,
  [2029] = 2011,
  [2030] = 1913,
  [2031] = 1906,
  [2032] = 1915,
  [2033] = 2033,
  [2034] = 1914,
  [2035] = 1908,
  [2036] = 2013,
  [2037] = 2012,
  [2038] = 2011,
  [2039] = 1907,
  [2040] = 2011,
  [2041] = 2041,
  [2042] = 2042,
  [2043] = 2043,
  [2044] = 2044,
  [2045] = 2011,
  [2046] = 2043,
  [2047] = 2013,
  [2048] = 2044,
  [2049] = 2012,
  [2050] = 2050,
  [2051] = 2051,
  [2052] = 2052,
  [2053] = 2053,
  [2054] = 2033,
  [2055] = 2013,
  [2056] = 1915,
  [2057] = 2057,
  [2058] = 2012,
  [2059] = 2059,
  [2060] = 2012,
  [2061] = 2059,
  [2062] = 1913,
  [2063] = 1906,
  [2064] = 2013,
  [2065] = 1908,
  [2066] = 1914,
  [2067] = 2057,
  [2068] = 2068,
  [2069] = 1908,
  [2070] = 1913,
  [2071] = 2071,
  [2072] = 1907,
  [2073] = 1906,
  [2074] = 1915,
  [2075] = 1914,
  [2076] = 2071,
  [2077] = 1934,
  [2078] = 1911,
  [2079] = 1909,
  [2080] = 1910,
  [2081] = 1907,
  [2082] = 2082,
  [2083] = 1915,
  [2084] = 2084,
  [2085] = 2085,
  [2086] = 1908,
  [2087] = 2033,
  [2088] = 2088,
  [2089] = 1906,
  [2090] = 2090,
  [2091] = 2091,
  [2092] = 2092,
  [2093] = 2093,
  [2094] = 2094,
  [2095] = 2095,
  [2096] = 2096,
  [2097] = 2097,
  [2098] = 2098,
  [2099] = 2099,
  [2100] = 2100,
  [2101] = 2101,
  [2102] = 1913,
  [2103] = 2103,
  [2104] = 1914,
  [2105] = 2105,
  [2106] = 2106,
  [2107] = 2107,
  [2108] = 2108,
  [2109] = 2109,
  [2110] = 2110,
  [2111] = 2042,
  [2112] = 2112,
  [2113] = 2113,
  [2114] = 2114,
  [2115] = 2115,
  [2116] = 2116,
  [2117] = 2114,
  [2118] = 2118,
  [2119] = 2119,
  [2120] = 2120,
  [2121] = 2121,
  [2122] = 2122,
  [2123] = 2123,
  [2124] = 2124,
  [2125] = 2125,
  [2126] = 2126,
  [2127] = 2127,
  [2128] = 2128,
  [2129] = 2129,
  [2130] = 2124,
  [2131] = 2114,
  [2132] = 2124,
  [2133] = 2133,
  [2134] = 2133,
  [2135] = 2133,
  [2136] = 2124,
  [2137] = 2133,
  [2138] = 2133,
  [2139] = 2114,
  [2140] = 2114,
  [2141] = 2114,
  [2142] = 2041,
  [2143] = 2050,
  [2144] = 2051,
  [2145] = 2053,
  [2146] = 2068,
  [2147] = 2147,
  [2148] = 2127,
  [2149] = 2123,
  [2150] = 2128,
  [2151] = 2116,
  [2152] = 2147,
  [2153] = 2147,
  [2154] = 2129,
  [2155] = 2147,
  [2156] = 2118,
  [2157] = 2147,
  [2158] = 2122,
  [2159] = 2115,
  [2160] = 2119,
  [2161] = 2161,
  [2162] = 2125,
  [2163] = 2120,
  [2164] = 2161,
  [2165] = 2126,
  [2166] = 2068,
  [2167] = 2167,
  [2168] = 2044,
  [2169] = 2043,
  [2170] = 2110,
  [2171] = 2124,
  [2172] = 2114,
  [2173] = 2114,
  [2174] = 2110,
  [2175] = 2057,
  [2176] = 2059,
  [2177] = 2052,
  [2178] = 1934,
  [2179] = 2114,
  [2180] = 2124,
  [2181] = 1907,
  [2182] = 2124,
  [2183] = 2114,
  [2184] = 2114,
  [2185] = 2185,
  [2186] = 2185,
  [2187] = 2185,
  [2188] = 2057,
  [2189] = 2185,
  [2190] = 1934,
  [2191] = 2059,
  [2192] = 2185,
  [2193] = 2114,
  [2194] = 2124,
  [2195] = 2052,
  [2196] = 2114,
  [2197] = 1906,
  [2198] = 1914,
  [2199] = 2124,
  [2200] = 2200,
  [2201] = 2124,
  [2202] = 2114,
  [2203] = 2043,
  [2204] = 2044,
  [2205] = 2114,
  [2206] = 1909,
  [2207] = 1910,
  [2208] = 1907,
  [2209] = 2200,
  [2210] = 1908,
  [2211] = 1915,
  [2212] = 1911,
  [2213] = 1913,
  [2214] = 2124,
  [2215] = 2114,
  [2216] = 2091,
  [2217] = 2217,
  [2218] = 2095,
  [2219] = 2107,
  [2220] = 2101,
  [2221] = 2108,
  [2222] = 2096,
  [2223] = 2097,
  [2224] = 2098,
  [2225] = 2042,
  [2226] = 2226,
  [2227] = 2113,
  [2228] = 2099,
  [2229] = 2068,
  [2230] = 2088,
  [2231] = 2103,
  [2232] = 2082,
  [2233] = 2068,
  [2234] = 2234,
  [2235] = 2090,
  [2236] = 2085,
  [2237] = 2092,
  [2238] = 2112,
  [2239] = 2093,
  [2240] = 2240,
  [2241] = 2052,
  [2242] = 2094,
  [2243] = 2106,
  [2244] = 2244,
  [2245] = 2100,
  [2246] = 2246,
  [2247] = 2247,
  [2248] = 2248,
  [2249] = 2249,
  [2250] = 2124,
  [2251] = 2107,
  [2252] = 2252,
  [2253] = 1915,
  [2254] = 2254,
  [2255] = 2255,
  [2256] = 2068,
  [2257] = 2257,
  [2258] = 1906,
  [2259] = 2259,
  [2260] = 2260,
  [2261] = 2255,
  [2262] = 2057,
  [2263] = 2263,
  [2264] = 2257,
  [2265] = 2059,
  [2266] = 2263,
  [2267] = 2267,
  [2268] = 2268,
  [2269] = 2114,
  [2270] = 1907,
  [2271] = 2124,
  [2272] = 2114,
  [2273] = 1908,
  [2274] = 2124,
  [2275] = 2114,
  [2276] = 1914,
  [2277] = 1934,
  [2278] = 2114,
  [2279] = 2110,
  [2280] = 2167,
  [2281] = 2281,
  [2282] = 2110,
  [2283] = 2114,
  [2284] = 2114,
  [2285] = 2285,
  [2286] = 2286,
  [2287] = 2287,
  [2288] = 2288,
  [2289] = 2289,
  [2290] = 2290,
  [2291] = 2291,
  [2292] = 2292,
  [2293] = 2293,
  [2294] = 2294,
  [2295] = 2295,
  [2296] = 2296,
  [2297] = 2297,
  [2298] = 2298,
  [2299] = 2299,
  [2300] = 2300,
  [2301] = 2301,
  [2302] = 2302,
  [2303] = 2303,
  [2304] = 2304,
  [2305] = 2305,
  [2306] = 2306,
  [2307] = 2307,
  [2308] = 2308,
  [2309] = 2309,
  [2310] = 2107,
  [2311] = 2311,
  [2312] = 2110,
  [2313] = 2313,
  [2314] = 2314,
  [2315] = 2315,
  [2316] = 2316,
  [2317] = 2317,
  [2318] = 2318,
  [2319] = 2319,
  [2320] = 2320,
  [2321] = 2321,
  [2322] = 2322,
  [2323] = 2323,
  [2324] = 1922,
  [2325] = 2325,
  [2326] = 2326,
  [2327] = 2327,
  [2328] = 2124,
  [2329] = 2329,
  [2330] = 2330,
  [2331] = 2331,
  [2332] = 2332,
  [2333] = 2333,
  [2334] = 2334,
  [2335] = 2335,
  [2336] = 2336,
  [2337] = 2337,
  [2338] = 2338,
  [2339] = 2339,
  [2340] = 2340,
  [2341] = 2341,
  [2342] = 2124,
  [2343] = 2343,
  [2344] = 2344,
  [2345] = 2345,
  [2346] = 2114,
  [2347] = 2347,
  [2348] = 2348,
  [2349] = 2349,
  [2350] = 2350,
  [2351] = 2351,
  [2352] = 2352,
  [2353] = 2353,
  [2354] = 2354,
  [2355] = 2355,
  [2356] = 2356,
  [2357] = 2357,
  [2358] = 2358,
  [2359] = 2359,
  [2360] = 2360,
  [2361] = 2361,
  [2362] = 2362,
  [2363] = 2363,
  [2364] = 2364,
  [2365] = 2365,
  [2366] = 2366,
  [2367] = 2367,
  [2368] = 2368,
  [2369] = 2090,
  [2370] = 2091,
  [2371] = 2092,
  [2372] = 2372,
  [2373] = 2373,
  [2374] = 2093,
  [2375] = 2094,
  [2376] = 2376,
  [2377] = 2377,
  [2378] = 2378,
  [2379] = 2379,
  [2380] = 2380,
  [2381] = 2381,
  [2382] = 2382,
  [2383] = 2383,
  [2384] = 2384,
  [2385] = 2385,
  [2386] = 2386,
  [2387] = 2085,
  [2388] = 2388,
  [2389] = 2389,
  [2390] = 2088,
  [2391] = 2391,
  [2392] = 2301,
  [2393] = 2393,
  [2394] = 2394,
  [2395] = 2395,
  [2396] = 2396,
  [2397] = 2397,
  [2398] = 2398,
  [2399] = 2399,
  [2400] = 2400,
  [2401] = 2401,
  [2402] = 2402,
  [2403] = 2124,
  [2404] = 2404,
  [2405] = 2405,
  [2406] = 2307,
  [2407] = 2301,
  [2408] = 2095,
  [2409] = 2096,
  [2410] = 2254,
  [2411] = 2114,
  [2412] = 2097,
  [2413] = 2301,
  [2414] = 2414,
  [2415] = 2415,
  [2416] = 2416,
  [2417] = 2417,
  [2418] = 2418,
  [2419] = 2301,
  [2420] = 2098,
  [2421] = 2301,
  [2422] = 2422,
  [2423] = 2423,
  [2424] = 2301,
  [2425] = 2425,
  [2426] = 2426,
  [2427] = 2427,
  [2428] = 2301,
  [2429] = 2429,
  [2430] = 2430,
  [2431] = 2431,
  [2432] = 2301,
  [2433] = 2114,
  [2434] = 2301,
  [2435] = 2114,
  [2436] = 2436,
  [2437] = 2437,
  [2438] = 2438,
  [2439] = 2301,
  [2440] = 2113,
  [2441] = 2301,
  [2442] = 2124,
  [2443] = 2099,
  [2444] = 2100,
  [2445] = 2103,
  [2446] = 2082,
  [2447] = 2307,
  [2448] = 2448,
  [2449] = 2449,
  [2450] = 2450,
  [2451] = 2451,
  [2452] = 2452,
  [2453] = 2453,
  [2454] = 2301,
  [2455] = 2455,
  [2456] = 2301,
  [2457] = 2301,
  [2458] = 2301,
  [2459] = 2301,
  [2460] = 2301,
  [2461] = 2301,
  [2462] = 2301,
  [2463] = 2301,
  [2464] = 2301,
  [2465] = 2465,
  [2466] = 2301,
  [2467] = 2467,
  [2468] = 2468,
  [2469] = 2469,
  [2470] = 2301,
  [2471] = 2471,
  [2472] = 2472,
  [2473] = 2473,
  [2474] = 2124,
  [2475] = 2124,
  [2476] = 2124,
  [2477] = 2477,
  [2478] = 2478,
  [2479] = 2114,
  [2480] = 2480,
  [2481] = 1934,
  [2482] = 2482,
  [2483] = 2483,
  [2484] = 2052,
  [2485] = 2485,
  [2486] = 2114,
  [2487] = 2114,
  [2488] = 2114,
  [2489] = 1167,
  [2490] = 1932,
  [2491] = 2491,
  [2492] = 2492,
  [2493] = 2493,
  [2494] = 2494,
  [2495] = 2495,
  [2496] = 2496,
  [2497] = 2497,
  [2498] = 2498,
  [2499] = 2124,
  [2500] = 2500,
  [2501] = 2429,
  [2502] = 2124,
  [2503] = 2114,
  [2504] = 2114,
  [2505] = 2505,
  [2506] = 2429,
  [2507] = 2507,
  [2508] = 2508,
  [2509] = 2509,
  [2510] = 2510,
  [2511] = 2511,
  [2512] = 2512,
  [2513] = 2513,
  [2514] = 2514,
  [2515] = 2515,
  [2516] = 2516,
  [2517] = 2517,
  [2518] = 2518,
  [2519] = 2519,
  [2520] = 2507,
  [2521] = 2521,
  [2522] = 2522,
  [2523] = 2124,
  [2524] = 2124,
  [2525] = 2114,
  [2526] = 2480,
  [2527] = 2477,
  [2528] = 2480,
  [2529] = 2167,
  [2530] = 2114,
  [2531] = 2483,
  [2532] = 2114,
  [2533] = 2477,
  [2534] = 2114,
  [2535] = 2482,
  [2536] = 2114,
  [2537] = 2114,
  [2538] = 2538,
  [2539] = 2539,
  [2540] = 2107,
  [2541] = 2497,
  [2542] = 2538,
  [2543] = 2539,
  [2544] = 2254,
  [2545] = 2538,
  [2546] = 2498,
  [2547] = 2343,
  [2548] = 2538,
  [2549] = 2521,
  [2550] = 2538,
  [2551] = 2539,
  [2552] = 2429,
  [2553] = 2538,
  [2554] = 2539,
  [2555] = 2500,
  [2556] = 2340,
  [2557] = 2539,
  [2558] = 2491,
  [2559] = 2538,
  [2560] = 2539,
  [2561] = 2492,
  [2562] = 2538,
  [2563] = 2539,
  [2564] = 2114,
  [2565] = 2493,
  [2566] = 2114,
  [2567] = 2494,
  [2568] = 2507,
  [2569] = 2389,
  [2570] = 2538,
  [2571] = 2539,
  [2572] = 2114,
  [2573] = 2508,
  [2574] = 2509,
  [2575] = 2510,
  [2576] = 2511,
  [2577] = 2512,
  [2578] = 2513,
  [2579] = 2514,
  [2580] = 2515,
  [2581] = 2516,
  [2582] = 2538,
  [2583] = 2518,
  [2584] = 2519,
  [2585] = 2538,
  [2586] = 2538,
  [2587] = 2539,
  [2588] = 2495,
  [2589] = 2539,
  [2590] = 2539,
  [2591] = 2538,
  [2592] = 2496,
  [2593] = 2539,
  [2594] = 2353,
  [2595] = 2538,
  [2596] = 2539,
  [2597] = 2538,
  [2598] = 2345,
  [2599] = 2538,
  [2600] = 2522,
  [2601] = 2538,
  [2602] = 2539,
  [2603] = 2538,
  [2604] = 2539,
  [2605] = 2539,
  [2606] = 2114,
  [2607] = 2507,
  [2608] = 2539,
  [2609] = 2539,
  [2610] = 2539,
  [2611] = 2538,
  [2612] = 2539,
  [2613] = 2358,
  [2614] = 2359,
  [2615] = 2538,
  [2616] = 2539,
  [2617] = 2538,
  [2618] = 2539,
  [2619] = 2538,
  [2620] = 2539,
  [2621] = 2538,
  [2622] = 2539,
  [2623] = 2538,
  [2624] = 2517,
  [2625] = 2519,
  [2626] = 2497,
  [2627] = 2495,
  [2628] = 2628,
  [2629] = 2493,
  [2630] = 2268,
  [2631] = 2631,
  [2632] = 2631,
  [2633] = 2628,
  [2634] = 2634,
  [2635] = 2634,
  [2636] = 2507,
  [2637] = 2167,
  [2638] = 2496,
  [2639] = 2114,
  [2640] = 2640,
  [2641] = 2508,
  [2642] = 2167,
  [2643] = 2509,
  [2644] = 2498,
  [2645] = 2492,
  [2646] = 2511,
  [2647] = 2114,
  [2648] = 2114,
  [2649] = 2507,
  [2650] = 2512,
  [2651] = 2513,
  [2652] = 2514,
  [2653] = 2515,
  [2654] = 2114,
  [2655] = 2516,
  [2656] = 2517,
  [2657] = 2518,
  [2658] = 2658,
  [2659] = 2483,
  [2660] = 2480,
  [2661] = 2640,
  [2662] = 2482,
  [2663] = 2477,
  [2664] = 2491,
  [2665] = 2494,
  [2666] = 2500,
  [2667] = 2510,
  [2668] = 2521,
  [2669] = 2508,
  [2670] = 2515,
  [2671] = 2500,
  [2672] = 2516,
  [2673] = 2517,
  [2674] = 2522,
  [2675] = 2518,
  [2676] = 2519,
  [2677] = 2510,
  [2678] = 2511,
  [2679] = 2512,
  [2680] = 2513,
  [2681] = 2497,
  [2682] = 2682,
  [2683] = 2683,
  [2684] = 2514,
  [2685] = 2685,
  [2686] = 2491,
  [2687] = 2492,
  [2688] = 2493,
  [2689] = 2689,
  [2690] = 2509,
  [2691] = 2495,
  [2692] = 2254,
  [2693] = 2693,
  [2694] = 2494,
  [2695] = 2695,
  [2696] = 2696,
  [2697] = 2498,
  [2698] = 2477,
  [2699] = 2483,
  [2700] = 2482,
  [2701] = 2480,
  [2702] = 2702,
  [2703] = 2200,
  [2704] = 2704,
  [2705] = 2658,
  [2706] = 2706,
  [2707] = 2254,
  [2708] = 2482,
  [2709] = 2477,
  [2710] = 2710,
  [2711] = 2477,
  [2712] = 2712,
  [2713] = 2480,
  [2714] = 2480,
  [2715] = 2658,
  [2716] = 2483,
  [2717] = 2521,
  [2718] = 2500,
  [2719] = 2478,
  [2720] = 2268,
  [2721] = 2429,
  [2722] = 2507,
  [2723] = 2689,
  [2724] = 2508,
  [2725] = 2497,
  [2726] = 2509,
  [2727] = 2510,
  [2728] = 2511,
  [2729] = 2512,
  [2730] = 2513,
  [2731] = 2514,
  [2732] = 2515,
  [2733] = 2516,
  [2734] = 2517,
  [2735] = 2518,
  [2736] = 2519,
  [2737] = 2682,
  [2738] = 2695,
  [2739] = 2683,
  [2740] = 2508,
  [2741] = 2521,
  [2742] = 2485,
  [2743] = 2491,
  [2744] = 2492,
  [2745] = 2254,
  [2746] = 2509,
  [2747] = 2510,
  [2748] = 2511,
  [2749] = 2512,
  [2750] = 2513,
  [2751] = 2514,
  [2752] = 2515,
  [2753] = 2516,
  [2754] = 2517,
  [2755] = 2518,
  [2756] = 2519,
  [2757] = 2494,
  [2758] = 2500,
  [2759] = 2495,
  [2760] = 2498,
  [2761] = 2507,
  [2762] = 2522,
  [2763] = 2522,
  [2764] = 2226,
  [2765] = 2491,
  [2766] = 2492,
  [2767] = 2493,
  [2768] = 2494,
  [2769] = 2495,
  [2770] = 2496,
  [2771] = 2685,
  [2772] = 2497,
  [2773] = 2696,
  [2774] = 2498,
  [2775] = 2493,
  [2776] = 2518,
  [2777] = 2249,
  [2778] = 2514,
  [2779] = 2685,
  [2780] = 2482,
  [2781] = 2477,
  [2782] = 2511,
  [2783] = 2498,
  [2784] = 2244,
  [2785] = 2515,
  [2786] = 2695,
  [2787] = 2497,
  [2788] = 2516,
  [2789] = 2491,
  [2790] = 2492,
  [2791] = 2508,
  [2792] = 2240,
  [2793] = 2712,
  [2794] = 2517,
  [2795] = 2483,
  [2796] = 2710,
  [2797] = 2217,
  [2798] = 2480,
  [2799] = 2263,
  [2800] = 2513,
  [2801] = 2493,
  [2802] = 2494,
  [2803] = 2477,
  [2804] = 2246,
  [2805] = 2247,
  [2806] = 2495,
  [2807] = 2234,
  [2808] = 2248,
  [2809] = 2267,
  [2810] = 2500,
  [2811] = 2240,
  [2812] = 2689,
  [2813] = 2226,
  [2814] = 2509,
  [2815] = 2512,
  [2816] = 2704,
  [2817] = 2510,
  [2818] = 2257,
  [2819] = 2260,
  [2820] = 2702,
  [2821] = 2706,
  [2822] = 2682,
  [2823] = 2255,
  [2824] = 2683,
  [2825] = 2480,
  [2826] = 2519,
  [2827] = 2492,
  [2828] = 2200,
  [2829] = 2496,
  [2830] = 2497,
  [2831] = 2498,
  [2832] = 2522,
  [2833] = 2281,
  [2834] = 2500,
  [2835] = 2259,
  [2836] = 2477,
  [2837] = 2254,
  [2838] = 2507,
  [2839] = 2508,
  [2840] = 2482,
  [2841] = 2509,
  [2842] = 2510,
  [2843] = 2511,
  [2844] = 2512,
  [2845] = 2513,
  [2846] = 2514,
  [2847] = 1908,
  [2848] = 2516,
  [2849] = 2517,
  [2850] = 2518,
  [2851] = 2480,
  [2852] = 2519,
  [2853] = 2483,
  [2854] = 1914,
  [2855] = 2254,
  [2856] = 2704,
  [2857] = 1915,
  [2858] = 1906,
  [2859] = 2507,
  [2860] = 2429,
  [2861] = 2307,
  [2862] = 2477,
  [2863] = 2307,
  [2864] = 2480,
  [2865] = 2491,
  [2866] = 2252,
  [2867] = 2493,
  [2868] = 2521,
  [2869] = 2494,
  [2870] = 2495,
  [2871] = 2515,
  [2872] = 2240,
  [2873] = 2455,
  [2874] = 2358,
  [2875] = 2268,
  [2876] = 2359,
  [2877] = 2308,
  [2878] = 2356,
  [2879] = 2368,
  [2880] = 2381,
  [2881] = 2469,
  [2882] = 2382,
  [2883] = 2383,
  [2884] = 2404,
  [2885] = 2405,
  [2886] = 2417,
  [2887] = 2418,
  [2888] = 2509,
  [2889] = 2422,
  [2890] = 2289,
  [2891] = 2471,
  [2892] = 2510,
  [2893] = 2511,
  [2894] = 2426,
  [2895] = 2113,
  [2896] = 2099,
  [2897] = 2483,
  [2898] = 2512,
  [2899] = 2513,
  [2900] = 2100,
  [2901] = 2514,
  [2902] = 2515,
  [2903] = 2103,
  [2904] = 2268,
  [2905] = 2516,
  [2906] = 2517,
  [2907] = 2518,
  [2908] = 2082,
  [2909] = 2472,
  [2910] = 2316,
  [2911] = 2318,
  [2912] = 2658,
  [2913] = 2480,
  [2914] = 2496,
  [2915] = 2349,
  [2916] = 2322,
  [2917] = 2323,
  [2918] = 2327,
  [2919] = 2291,
  [2920] = 2292,
  [2921] = 2295,
  [2922] = 2296,
  [2923] = 2297,
  [2924] = 2298,
  [2925] = 2299,
  [2926] = 2300,
  [2927] = 2302,
  [2928] = 2477,
  [2929] = 2304,
  [2930] = 2305,
  [2931] = 2309,
  [2932] = 2311,
  [2933] = 2314,
  [2934] = 2521,
  [2935] = 2522,
  [2936] = 2337,
  [2937] = 2306,
  [2938] = 2397,
  [2939] = 2333,
  [2940] = 2334,
  [2941] = 2325,
  [2942] = 2519,
  [2943] = 2500,
  [2944] = 2326,
  [2945] = 2425,
  [2946] = 2254,
  [2947] = 2329,
  [2948] = 2332,
  [2949] = 2338,
  [2950] = 2339,
  [2951] = 2335,
  [2952] = 2355,
  [2953] = 2362,
  [2954] = 2363,
  [2955] = 2365,
  [2956] = 2366,
  [2957] = 2336,
  [2958] = 2507,
  [2959] = 2330,
  [2960] = 2348,
  [2961] = 2416,
  [2962] = 2303,
  [2963] = 2423,
  [2964] = 2285,
  [2965] = 2341,
  [2966] = 2498,
  [2967] = 2344,
  [2968] = 2429,
  [2969] = 2482,
  [2970] = 2350,
  [2971] = 2351,
  [2972] = 2090,
  [2973] = 2354,
  [2974] = 2091,
  [2975] = 2092,
  [2976] = 2093,
  [2977] = 2094,
  [2978] = 2357,
  [2979] = 2364,
  [2980] = 2367,
  [2981] = 2372,
  [2982] = 2373,
  [2983] = 2287,
  [2984] = 2436,
  [2985] = 2386,
  [2986] = 2388,
  [2987] = 2393,
  [2988] = 2394,
  [2989] = 2395,
  [2990] = 2398,
  [2991] = 2377,
  [2992] = 2399,
  [2993] = 2400,
  [2994] = 2401,
  [2995] = 2402,
  [2996] = 2095,
  [2997] = 2096,
  [2998] = 2097,
  [2999] = 2414,
  [3000] = 2429,
  [3001] = 2415,
  [3002] = 2098,
  [3003] = 2378,
  [3004] = 2226,
  [3005] = 2379,
  [3006] = 2384,
  [3007] = 2294,
  [3008] = 2085,
  [3009] = 2088,
  [3010] = 2317,
  [3011] = 2319,
  [3012] = 2321,
  [3013] = 2360,
  [3014] = 2361,
  [3015] = 2508,
  [3016] = 2507,
  [3017] = 2396,
  [3018] = 2473,
  [3019] = 2497,
  [3020] = 2437,
  [3021] = 2340,
  [3022] = 2438,
  [3023] = 2450,
  [3024] = 2453,
  [3025] = 2389,
  [3026] = 2380,
  [3027] = 2376,
  [3028] = 2491,
  [3029] = 2492,
  [3030] = 2493,
  [3031] = 2491,
  [3032] = 2492,
  [3033] = 2493,
  [3034] = 2494,
  [3035] = 2495,
  [3036] = 2496,
  [3037] = 2497,
  [3038] = 2498,
  [3039] = 2427,
  [3040] = 2500,
  [3041] = 2494,
  [3042] = 2288,
  [3043] = 2343,
  [3044] = 2495,
  [3045] = 2313,
  [3046] = 2331,
  [3047] = 2347,
  [3048] = 2391,
  [3049] = 2448,
  [3050] = 2449,
  [3051] = 2507,
  [3052] = 2508,
  [3053] = 2509,
  [3054] = 2510,
  [3055] = 2511,
  [3056] = 2512,
  [3057] = 2513,
  [3058] = 2514,
  [3059] = 2515,
  [3060] = 2516,
  [3061] = 2517,
  [3062] = 2518,
  [3063] = 2519,
  [3064] = 2345,
  [3065] = 2451,
  [3066] = 2507,
  [3067] = 2452,
  [3068] = 2286,
  [3069] = 2290,
  [3070] = 2293,
  [3071] = 2468,
  [3072] = 2430,
  [3073] = 2431,
  [3074] = 2467,
  [3075] = 2353,
  [3076] = 2385,
  [3077] = 2693,
  [3078] = 2263,
  [3079] = 2352,
  [3080] = 3080,
  [3081] = 3081,
  [3082] = 3081,
  [3083] = 2522,
  [3084] = 2315,
  [3085] = 2268,
  [3086] = 2320,
  [3087] = 3081,
  [3088] = 3081,
  [3089] = 2477,
  [3090] = 2492,
  [3091] = 2507,
  [3092] = 2507,
  [3093] = 2254,
  [3094] = 2493,
  [3095] = 2521,
  [3096] = 2255,
  [3097] = 3080,
  [3098] = 3081,
  [3099] = 2498,
  [3100] = 2315,
  [3101] = 2507,
  [3102] = 2508,
  [3103] = 2500,
  [3104] = 3080,
  [3105] = 2494,
  [3106] = 2320,
  [3107] = 3107,
  [3108] = 2257,
  [3109] = 2495,
  [3110] = 3080,
  [3111] = 2509,
  [3112] = 2510,
  [3113] = 3081,
  [3114] = 2491,
  [3115] = 2492,
  [3116] = 2511,
  [3117] = 2512,
  [3118] = 2513,
  [3119] = 2514,
  [3120] = 2515,
  [3121] = 2516,
  [3122] = 2517,
  [3123] = 3081,
  [3124] = 2519,
  [3125] = 2465,
  [3126] = 2695,
  [3127] = 2352,
  [3128] = 2508,
  [3129] = 3081,
  [3130] = 2689,
  [3131] = 3081,
  [3132] = 3080,
  [3133] = 3080,
  [3134] = 3081,
  [3135] = 3081,
  [3136] = 3080,
  [3137] = 3080,
  [3138] = 3081,
  [3139] = 2254,
  [3140] = 2497,
  [3141] = 2493,
  [3142] = 2494,
  [3143] = 3080,
  [3144] = 3081,
  [3145] = 2682,
  [3146] = 2683,
  [3147] = 2509,
  [3148] = 2510,
  [3149] = 2511,
  [3150] = 2512,
  [3151] = 2513,
  [3152] = 2514,
  [3153] = 2515,
  [3154] = 2516,
  [3155] = 2517,
  [3156] = 2518,
  [3157] = 2519,
  [3158] = 2685,
  [3159] = 2500,
  [3160] = 2497,
  [3161] = 2491,
  [3162] = 2496,
  [3163] = 3080,
  [3164] = 3081,
  [3165] = 2696,
  [3166] = 3081,
  [3167] = 3080,
  [3168] = 3081,
  [3169] = 2495,
  [3170] = 2498,
  [3171] = 2465,
  [3172] = 3081,
  [3173] = 3080,
  [3174] = 2518,
  [3175] = 2482,
  [3176] = 2429,
  [3177] = 2500,
  [3178] = 2483,
  [3179] = 2477,
  [3180] = 2303,
  [3181] = 1910,
  [3182] = 2658,
  [3183] = 2480,
  [3184] = 2254,
  [3185] = 1908,
  [3186] = 1914,
  [3187] = 2333,
  [3188] = 2496,
  [3189] = 1915,
  [3190] = 2480,
  [3191] = 2477,
  [3192] = 2429,
  [3193] = 2480,
  [3194] = 2477,
  [3195] = 2480,
  [3196] = 2482,
  [3197] = 2482,
  [3198] = 2254,
  [3199] = 3199,
  [3200] = 2477,
  [3201] = 2483,
  [3202] = 3202,
  [3203] = 1913,
  [3204] = 2483,
  [3205] = 2480,
  [3206] = 2477,
  [3207] = 2482,
  [3208] = 1906,
  [3209] = 2477,
  [3210] = 2483,
  [3211] = 2658,
  [3212] = 2496,
  [3213] = 2689,
  [3214] = 2320,
  [3215] = 2497,
  [3216] = 2682,
  [3217] = 2683,
  [3218] = 2710,
  [3219] = 2521,
  [3220] = 2494,
  [3221] = 2478,
  [3222] = 2485,
  [3223] = 2491,
  [3224] = 2491,
  [3225] = 2712,
  [3226] = 2492,
  [3227] = 2658,
  [3228] = 2702,
  [3229] = 2706,
  [3230] = 2493,
  [3231] = 2478,
  [3232] = 2485,
  [3233] = 2494,
  [3234] = 2495,
  [3235] = 2492,
  [3236] = 2493,
  [3237] = 2494,
  [3238] = 2495,
  [3239] = 2507,
  [3240] = 2493,
  [3241] = 2495,
  [3242] = 2696,
  [3243] = 2500,
  [3244] = 2508,
  [3245] = 2497,
  [3246] = 2477,
  [3247] = 2480,
  [3248] = 2498,
  [3249] = 2500,
  [3250] = 2509,
  [3251] = 2510,
  [3252] = 2511,
  [3253] = 2512,
  [3254] = 2513,
  [3255] = 2514,
  [3256] = 2515,
  [3257] = 2516,
  [3258] = 2517,
  [3259] = 2518,
  [3260] = 2519,
  [3261] = 2500,
  [3262] = 3262,
  [3263] = 2482,
  [3264] = 2704,
  [3265] = 2521,
  [3266] = 2491,
  [3267] = 2522,
  [3268] = 2483,
  [3269] = 2522,
  [3270] = 2507,
  [3271] = 2480,
  [3272] = 2496,
  [3273] = 2494,
  [3274] = 2695,
  [3275] = 2508,
  [3276] = 2480,
  [3277] = 2482,
  [3278] = 2685,
  [3279] = 2429,
  [3280] = 2500,
  [3281] = 2496,
  [3282] = 3282,
  [3283] = 2508,
  [3284] = 2497,
  [3285] = 2509,
  [3286] = 2510,
  [3287] = 2511,
  [3288] = 2482,
  [3289] = 2513,
  [3290] = 2514,
  [3291] = 2515,
  [3292] = 2516,
  [3293] = 2517,
  [3294] = 2518,
  [3295] = 2519,
  [3296] = 2500,
  [3297] = 2498,
  [3298] = 2491,
  [3299] = 2429,
  [3300] = 2492,
  [3301] = 3262,
  [3302] = 2465,
  [3303] = 3262,
  [3304] = 2507,
  [3305] = 2492,
  [3306] = 2508,
  [3307] = 2497,
  [3308] = 2498,
  [3309] = 2509,
  [3310] = 2510,
  [3311] = 2511,
  [3312] = 2512,
  [3313] = 2513,
  [3314] = 2514,
  [3315] = 2515,
  [3316] = 2516,
  [3317] = 2517,
  [3318] = 2518,
  [3319] = 2519,
  [3320] = 2352,
  [3321] = 2496,
  [3322] = 2483,
  [3323] = 2521,
  [3324] = 2495,
  [3325] = 2498,
  [3326] = 2522,
  [3327] = 2509,
  [3328] = 2507,
  [3329] = 2510,
  [3330] = 2511,
  [3331] = 2512,
  [3332] = 2493,
  [3333] = 2513,
  [3334] = 2507,
  [3335] = 2514,
  [3336] = 2507,
  [3337] = 2483,
  [3338] = 2515,
  [3339] = 2516,
  [3340] = 2517,
  [3341] = 2518,
  [3342] = 2658,
  [3343] = 2307,
  [3344] = 2477,
  [3345] = 2519,
  [3346] = 2521,
  [3347] = 2522,
  [3348] = 2480,
  [3349] = 2477,
  [3350] = 2307,
  [3351] = 2315,
  [3352] = 2477,
  [3353] = 2512,
  [3354] = 2495,
  [3355] = 2493,
  [3356] = 2494,
  [3357] = 2495,
  [3358] = 2498,
  [3359] = 2507,
  [3360] = 2689,
  [3361] = 2508,
  [3362] = 2497,
  [3363] = 2491,
  [3364] = 2492,
  [3365] = 2498,
  [3366] = 2514,
  [3367] = 2480,
  [3368] = 2507,
  [3369] = 3369,
  [3370] = 2509,
  [3371] = 2510,
  [3372] = 2511,
  [3373] = 2512,
  [3374] = 2513,
  [3375] = 2514,
  [3376] = 2515,
  [3377] = 2516,
  [3378] = 2517,
  [3379] = 2518,
  [3380] = 2519,
  [3381] = 2521,
  [3382] = 2353,
  [3383] = 3369,
  [3384] = 2483,
  [3385] = 2345,
  [3386] = 2497,
  [3387] = 2521,
  [3388] = 2695,
  [3389] = 2509,
  [3390] = 2510,
  [3391] = 2511,
  [3392] = 2512,
  [3393] = 2513,
  [3394] = 2514,
  [3395] = 2515,
  [3396] = 2516,
  [3397] = 2517,
  [3398] = 2518,
  [3399] = 2519,
  [3400] = 2682,
  [3401] = 2478,
  [3402] = 2485,
  [3403] = 2482,
  [3404] = 2513,
  [3405] = 2683,
  [3406] = 2496,
  [3407] = 2491,
  [3408] = 2492,
  [3409] = 2522,
  [3410] = 2493,
  [3411] = 2494,
  [3412] = 2507,
  [3413] = 2307,
  [3414] = 2495,
  [3415] = 2507,
  [3416] = 2389,
  [3417] = 2429,
  [3418] = 2493,
  [3419] = 2358,
  [3420] = 2508,
  [3421] = 2521,
  [3422] = 2494,
  [3423] = 2515,
  [3424] = 2516,
  [3425] = 2517,
  [3426] = 2518,
  [3427] = 2480,
  [3428] = 2491,
  [3429] = 2492,
  [3430] = 2493,
  [3431] = 2494,
  [3432] = 2500,
  [3433] = 2495,
  [3434] = 2498,
  [3435] = 2522,
  [3436] = 2497,
  [3437] = 2682,
  [3438] = 2683,
  [3439] = 2500,
  [3440] = 2689,
  [3441] = 2509,
  [3442] = 2510,
  [3443] = 2511,
  [3444] = 2512,
  [3445] = 2513,
  [3446] = 2514,
  [3447] = 2515,
  [3448] = 2516,
  [3449] = 2517,
  [3450] = 2518,
  [3451] = 2519,
  [3452] = 2522,
  [3453] = 2508,
  [3454] = 2497,
  [3455] = 3369,
  [3456] = 2359,
  [3457] = 2696,
  [3458] = 2685,
  [3459] = 2695,
  [3460] = 2508,
  [3461] = 2240,
  [3462] = 3369,
  [3463] = 2340,
  [3464] = 3369,
  [3465] = 2343,
  [3466] = 2685,
  [3467] = 2491,
  [3468] = 2492,
  [3469] = 2509,
  [3470] = 2510,
  [3471] = 2511,
  [3472] = 2519,
  [3473] = 2512,
  [3474] = 2500,
  [3475] = 2498,
  [3476] = 2497,
  [3477] = 2514,
  [3478] = 2493,
  [3479] = 2685,
  [3480] = 2507,
  [3481] = 2494,
  [3482] = 2500,
  [3483] = 2702,
  [3484] = 2710,
  [3485] = 2515,
  [3486] = 2516,
  [3487] = 2495,
  [3488] = 2514,
  [3489] = 2695,
  [3490] = 2521,
  [3491] = 2508,
  [3492] = 2522,
  [3493] = 2517,
  [3494] = 2518,
  [3495] = 2519,
  [3496] = 2485,
  [3497] = 2515,
  [3498] = 2516,
  [3499] = 2712,
  [3500] = 2658,
  [3501] = 2517,
  [3502] = 2477,
  [3503] = 2217,
  [3504] = 2496,
  [3505] = 2509,
  [3506] = 2518,
  [3507] = 1915,
  [3508] = 2519,
  [3509] = 2491,
  [3510] = 2706,
  [3511] = 2508,
  [3512] = 2497,
  [3513] = 2244,
  [3514] = 2480,
  [3515] = 2429,
  [3516] = 2510,
  [3517] = 2495,
  [3518] = 2507,
  [3519] = 2682,
  [3520] = 2689,
  [3521] = 2491,
  [3522] = 1906,
  [3523] = 2498,
  [3524] = 2511,
  [3525] = 2492,
  [3526] = 2477,
  [3527] = 2509,
  [3528] = 2492,
  [3529] = 2477,
  [3530] = 2477,
  [3531] = 2493,
  [3532] = 2477,
  [3533] = 2494,
  [3534] = 2510,
  [3535] = 2511,
  [3536] = 2512,
  [3537] = 2683,
  [3538] = 2500,
  [3539] = 2696,
  [3540] = 2513,
  [3541] = 2478,
  [3542] = 2512,
  [3543] = 2234,
  [3544] = 2513,
  [3545] = 2704,
  [3546] = 2498,
  [3547] = 2495,
  [3548] = 2483,
  [3549] = 2496,
  [3550] = 1964,
  [3551] = 2507,
  [3552] = 2712,
  [3553] = 2710,
  [3554] = 2493,
  [3555] = 2497,
  [3556] = 2702,
  [3557] = 2706,
  [3558] = 2482,
  [3559] = 2480,
  [3560] = 2483,
  [3561] = 2517,
  [3562] = 2491,
  [3563] = 2507,
  [3564] = 2515,
  [3565] = 3565,
  [3566] = 2480,
  [3567] = 2480,
  [3568] = 2685,
  [3569] = 2513,
  [3570] = 2704,
  [3571] = 1965,
  [3572] = 2500,
  [3573] = 2518,
  [3574] = 2480,
  [3575] = 2482,
  [3576] = 2512,
  [3577] = 2704,
  [3578] = 2516,
  [3579] = 2482,
  [3580] = 2500,
  [3581] = 2483,
  [3582] = 2482,
  [3583] = 2494,
  [3584] = 3584,
  [3585] = 2514,
  [3586] = 2498,
  [3587] = 2477,
  [3588] = 2658,
  [3589] = 2492,
  [3590] = 2477,
  [3591] = 2519,
  [3592] = 3592,
  [3593] = 2508,
  [3594] = 2477,
  [3595] = 2480,
  [3596] = 2483,
  [3597] = 2509,
  [3598] = 2510,
  [3599] = 2511,
  [3600] = 2477,
  [3601] = 2658,
  [3602] = 2480,
  [3603] = 2500,
  [3604] = 3604,
  [3605] = 2515,
  [3606] = 2259,
  [3607] = 2482,
  [3608] = 2500,
  [3609] = 3609,
  [3610] = 2491,
  [3611] = 2689,
  [3612] = 2704,
  [3613] = 2682,
  [3614] = 2683,
  [3615] = 2509,
  [3616] = 2510,
  [3617] = 2511,
  [3618] = 2512,
  [3619] = 2519,
  [3620] = 2522,
  [3621] = 2496,
  [3622] = 2513,
  [3623] = 2514,
  [3624] = 2483,
  [3625] = 2658,
  [3626] = 2515,
  [3627] = 2492,
  [3628] = 2483,
  [3629] = 2493,
  [3630] = 2516,
  [3631] = 2517,
  [3632] = 2514,
  [3633] = 2497,
  [3634] = 2518,
  [3635] = 2519,
  [3636] = 2508,
  [3637] = 2521,
  [3638] = 2522,
  [3639] = 2477,
  [3640] = 2495,
  [3641] = 2693,
  [3642] = 2497,
  [3643] = 2685,
  [3644] = 2491,
  [3645] = 2508,
  [3646] = 2508,
  [3647] = 2689,
  [3648] = 2480,
  [3649] = 2496,
  [3650] = 2509,
  [3651] = 2510,
  [3652] = 2511,
  [3653] = 2512,
  [3654] = 2497,
  [3655] = 2513,
  [3656] = 2491,
  [3657] = 2492,
  [3658] = 2493,
  [3659] = 2494,
  [3660] = 2495,
  [3661] = 2507,
  [3662] = 2695,
  [3663] = 2507,
  [3664] = 2514,
  [3665] = 2482,
  [3666] = 2477,
  [3667] = 2498,
  [3668] = 2515,
  [3669] = 2516,
  [3670] = 2517,
  [3671] = 2518,
  [3672] = 2509,
  [3673] = 2658,
  [3674] = 2492,
  [3675] = 2519,
  [3676] = 2510,
  [3677] = 2500,
  [3678] = 2500,
  [3679] = 2695,
  [3680] = 2507,
  [3681] = 2696,
  [3682] = 2483,
  [3683] = 2498,
  [3684] = 2518,
  [3685] = 2521,
  [3686] = 2511,
  [3687] = 2507,
  [3688] = 2508,
  [3689] = 2512,
  [3690] = 2513,
  [3691] = 2514,
  [3692] = 2515,
  [3693] = 2516,
  [3694] = 2517,
  [3695] = 2480,
  [3696] = 2518,
  [3697] = 2491,
  [3698] = 2492,
  [3699] = 2522,
  [3700] = 2696,
  [3701] = 2682,
  [3702] = 2497,
  [3703] = 2683,
  [3704] = 2693,
  [3705] = 2521,
  [3706] = 2477,
  [3707] = 2494,
  [3708] = 2500,
  [3709] = 2493,
  [3710] = 2480,
  [3711] = 2480,
  [3712] = 2494,
  [3713] = 2508,
  [3714] = 2497,
  [3715] = 2498,
  [3716] = 2509,
  [3717] = 2510,
  [3718] = 2511,
  [3719] = 2512,
  [3720] = 2513,
  [3721] = 2514,
  [3722] = 2515,
  [3723] = 2516,
  [3724] = 2517,
  [3725] = 2518,
  [3726] = 2519,
  [3727] = 2519,
  [3728] = 2491,
  [3729] = 2465,
  [3730] = 2492,
  [3731] = 2352,
  [3732] = 2493,
  [3733] = 2494,
  [3734] = 2516,
  [3735] = 2315,
  [3736] = 2320,
  [3737] = 2495,
  [3738] = 2493,
  [3739] = 2509,
  [3740] = 2494,
  [3741] = 2510,
  [3742] = 2685,
  [3743] = 2511,
  [3744] = 2252,
  [3745] = 2500,
  [3746] = 2495,
  [3747] = 2522,
  [3748] = 2477,
  [3749] = 2477,
  [3750] = 2512,
  [3751] = 2496,
  [3752] = 2513,
  [3753] = 2498,
  [3754] = 2495,
  [3755] = 2521,
  [3756] = 2498,
  [3757] = 2477,
  [3758] = 2500,
  [3759] = 2517,
  [3760] = 2482,
  [3761] = 2515,
  [3762] = 2494,
  [3763] = 2500,
  [3764] = 2511,
  [3765] = 2496,
  [3766] = 2517,
  [3767] = 2518,
  [3768] = 2493,
  [3769] = 2480,
  [3770] = 2498,
  [3771] = 2509,
  [3772] = 2510,
  [3773] = 2511,
  [3774] = 2512,
  [3775] = 2512,
  [3776] = 2513,
  [3777] = 2514,
  [3778] = 2515,
  [3779] = 2516,
  [3780] = 2517,
  [3781] = 2512,
  [3782] = 2518,
  [3783] = 2519,
  [3784] = 2480,
  [3785] = 2497,
  [3786] = 2693,
  [3787] = 2513,
  [3788] = 2514,
  [3789] = 2480,
  [3790] = 2515,
  [3791] = 2516,
  [3792] = 2517,
  [3793] = 2483,
  [3794] = 2508,
  [3795] = 2493,
  [3796] = 2518,
  [3797] = 2508,
  [3798] = 2522,
  [3799] = 2495,
  [3800] = 2497,
  [3801] = 2696,
  [3802] = 2658,
  [3803] = 2512,
  [3804] = 2519,
  [3805] = 2495,
  [3806] = 2510,
  [3807] = 2493,
  [3808] = 2521,
  [3809] = 2519,
  [3810] = 2491,
  [3811] = 2513,
  [3812] = 2514,
  [3813] = 2492,
  [3814] = 2500,
  [3815] = 2494,
  [3816] = 2507,
  [3817] = 2482,
  [3818] = 2695,
  [3819] = 2491,
  [3820] = 2492,
  [3821] = 2497,
  [3822] = 2494,
  [3823] = 2521,
  [3824] = 2511,
  [3825] = 2507,
  [3826] = 2512,
  [3827] = 2507,
  [3828] = 2492,
  [3829] = 2509,
  [3830] = 2494,
  [3831] = 2510,
  [3832] = 2521,
  [3833] = 2511,
  [3834] = 2507,
  [3835] = 2495,
  [3836] = 2482,
  [3837] = 2513,
  [3838] = 2514,
  [3839] = 2500,
  [3840] = 2514,
  [3841] = 2516,
  [3842] = 2517,
  [3843] = 2689,
  [3844] = 2497,
  [3845] = 2494,
  [3846] = 2483,
  [3847] = 2518,
  [3848] = 2519,
  [3849] = 2511,
  [3850] = 2515,
  [3851] = 2515,
  [3852] = 2495,
  [3853] = 2682,
  [3854] = 2491,
  [3855] = 2516,
  [3856] = 2491,
  [3857] = 2517,
  [3858] = 2683,
  [3859] = 2518,
  [3860] = 2492,
  [3861] = 2498,
  [3862] = 2498,
  [3863] = 2498,
  [3864] = 2509,
  [3865] = 2522,
  [3866] = 2516,
  [3867] = 2658,
  [3868] = 2508,
  [3869] = 2519,
  [3870] = 2507,
  [3871] = 2509,
  [3872] = 2493,
  [3873] = 2491,
  [3874] = 2508,
  [3875] = 2509,
  [3876] = 2510,
  [3877] = 2497,
  [3878] = 2522,
  [3879] = 2480,
  [3880] = 2493,
  [3881] = 2510,
  [3882] = 2508,
  [3883] = 2495,
  [3884] = 2496,
  [3885] = 2498,
  [3886] = 2685,
  [3887] = 2507,
  [3888] = 2492,
  [3889] = 2513,
  [3890] = 2498,
  [3891] = 2497,
  [3892] = 2658,
  [3893] = 2491,
  [3894] = 2492,
  [3895] = 2493,
  [3896] = 2494,
  [3897] = 2495,
  [3898] = 2500,
  [3899] = 2477,
  [3900] = 2704,
  [3901] = 2712,
  [3902] = 2710,
  [3903] = 2702,
  [3904] = 2706,
  [3905] = 2689,
  [3906] = 2497,
  [3907] = 2682,
  [3908] = 2683,
  [3909] = 2498,
  [3910] = 2658,
  [3911] = 2685,
  [3912] = 2496,
  [3913] = 2495,
  [3914] = 2510,
  [3915] = 2507,
  [3916] = 2519,
  [3917] = 2491,
  [3918] = 2492,
  [3919] = 2493,
  [3920] = 2494,
  [3921] = 2495,
  [3922] = 3922,
  [3923] = 2507,
  [3924] = 2695,
  [3925] = 2508,
  [3926] = 2685,
  [3927] = 2500,
  [3928] = 2511,
  [3929] = 2712,
  [3930] = 2710,
  [3931] = 2702,
  [3932] = 2508,
  [3933] = 2685,
  [3934] = 2480,
  [3935] = 2477,
  [3936] = 2512,
  [3937] = 2704,
  [3938] = 2497,
  [3939] = 2498,
  [3940] = 2507,
  [3941] = 2508,
  [3942] = 2513,
  [3943] = 2509,
  [3944] = 2510,
  [3945] = 2511,
  [3946] = 2512,
  [3947] = 2513,
  [3948] = 2514,
  [3949] = 2515,
  [3950] = 2516,
  [3951] = 2517,
  [3952] = 2518,
  [3953] = 2519,
  [3954] = 2689,
  [3955] = 2695,
  [3956] = 2696,
  [3957] = 2696,
  [3958] = 2682,
  [3959] = 2683,
  [3960] = 2507,
  [3961] = 2514,
  [3962] = 2521,
  [3963] = 2477,
  [3964] = 2522,
  [3965] = 2491,
  [3966] = 2492,
  [3967] = 2521,
  [3968] = 2496,
  [3969] = 2522,
  [3970] = 2477,
  [3971] = 2515,
  [3972] = 2516,
  [3973] = 2517,
  [3974] = 2493,
  [3975] = 2518,
  [3976] = 2494,
  [3977] = 2509,
  [3978] = 2510,
  [3979] = 2511,
  [3980] = 2512,
  [3981] = 2513,
  [3982] = 2500,
  [3983] = 2514,
  [3984] = 2509,
  [3985] = 2515,
  [3986] = 2516,
  [3987] = 2517,
  [3988] = 2518,
  [3989] = 2519,
  [3990] = 2706,
  [3991] = 2495,
  [3992] = 2511,
  [3993] = 2512,
  [3994] = 2513,
  [3995] = 2696,
  [3996] = 2514,
  [3997] = 2515,
  [3998] = 2491,
  [3999] = 2492,
  [4000] = 2522,
  [4001] = 2493,
  [4002] = 2494,
  [4003] = 2516,
  [4004] = 2517,
  [4005] = 2518,
  [4006] = 2695,
  [4007] = 2495,
  [4008] = 2689,
  [4009] = 2521,
  [4010] = 2497,
  [4011] = 2491,
  [4012] = 2480,
  [4013] = 2682,
  [4014] = 2683,
  [4015] = 2696,
  [4016] = 2507,
  [4017] = 2492,
  [4018] = 2482,
  [4019] = 2510,
  [4020] = 2493,
  [4021] = 2494,
  [4022] = 2712,
  [4023] = 2710,
  [4024] = 2702,
  [4025] = 2500,
  [4026] = 2519,
  [4027] = 2706,
  [4028] = 2483,
  [4029] = 2497,
  [4030] = 2682,
  [4031] = 2519,
  [4032] = 2496,
  [4033] = 2480,
  [4034] = 2683,
  [4035] = 2480,
  [4036] = 2689,
  [4037] = 2498,
  [4038] = 2695,
  [4039] = 2508,
  [4040] = 2507,
  [4041] = 2509,
  [4042] = 2508,
  [4043] = 2509,
  [4044] = 2510,
  [4045] = 2511,
  [4046] = 2512,
  [4047] = 2513,
  [4048] = 2514,
  [4049] = 2515,
  [4050] = 2516,
  [4051] = 2517,
  [4052] = 2518,
  [4053] = 2498,
  [4054] = 2704,
  [4055] = 2706,
  [4056] = 2521,
  [4057] = 2508,
  [4058] = 2497,
  [4059] = 2702,
  [4060] = 2706,
  [4061] = 2480,
  [4062] = 2500,
  [4063] = 2685,
  [4064] = 2511,
  [4065] = 2482,
  [4066] = 2498,
  [4067] = 2512,
  [4068] = 2491,
  [4069] = 2492,
  [4070] = 2493,
  [4071] = 2494,
  [4072] = 2513,
  [4073] = 2514,
  [4074] = 1910,
  [4075] = 2509,
  [4076] = 4076,
  [4077] = 2495,
  [4078] = 2515,
  [4079] = 2516,
  [4080] = 2517,
  [4081] = 2518,
  [4082] = 2519,
  [4083] = 2704,
  [4084] = 2500,
  [4085] = 2712,
  [4086] = 2710,
  [4087] = 2510,
  [4088] = 2509,
  [4089] = 2510,
  [4090] = 2511,
  [4091] = 2512,
  [4092] = 2513,
  [4093] = 2514,
  [4094] = 2515,
  [4095] = 2702,
  [4096] = 2517,
  [4097] = 2518,
  [4098] = 2519,
  [4099] = 2491,
  [4100] = 2492,
  [4101] = 2493,
  [4102] = 2494,
  [4103] = 2495,
  [4104] = 2498,
  [4105] = 2508,
  [4106] = 2689,
  [4107] = 2695,
  [4108] = 2702,
  [4109] = 2706,
  [4110] = 2682,
  [4111] = 2683,
  [4112] = 2712,
  [4113] = 2477,
  [4114] = 2498,
  [4115] = 2497,
  [4116] = 2480,
  [4117] = 2477,
  [4118] = 2500,
  [4119] = 2522,
  [4120] = 2477,
  [4121] = 2712,
  [4122] = 2710,
  [4123] = 2483,
  [4124] = 2704,
  [4125] = 2685,
  [4126] = 2477,
  [4127] = 2516,
  [4128] = 2518,
  [4129] = 2516,
  [4130] = 2517,
  [4131] = 2498,
  [4132] = 2518,
  [4133] = 2497,
  [4134] = 2508,
  [4135] = 2303,
  [4136] = 2498,
  [4137] = 2500,
  [4138] = 2491,
  [4139] = 2704,
  [4140] = 2492,
  [4141] = 2508,
  [4142] = 2710,
  [4143] = 2498,
  [4144] = 4144,
  [4145] = 2522,
  [4146] = 2497,
  [4147] = 2689,
  [4148] = 2695,
  [4149] = 2682,
  [4150] = 2683,
  [4151] = 2712,
  [4152] = 2710,
  [4153] = 2519,
  [4154] = 2483,
  [4155] = 2509,
  [4156] = 2510,
  [4157] = 2511,
  [4158] = 2512,
  [4159] = 2513,
  [4160] = 2514,
  [4161] = 2515,
  [4162] = 2516,
  [4163] = 2517,
  [4164] = 2518,
  [4165] = 2519,
  [4166] = 2509,
  [4167] = 2510,
  [4168] = 2511,
  [4169] = 2512,
  [4170] = 2493,
  [4171] = 2513,
  [4172] = 2515,
  [4173] = 2491,
  [4174] = 2492,
  [4175] = 2508,
  [4176] = 2493,
  [4177] = 2494,
  [4178] = 2497,
  [4179] = 2480,
  [4180] = 2495,
  [4181] = 2521,
  [4182] = 2480,
  [4183] = 2702,
  [4184] = 2509,
  [4185] = 2500,
  [4186] = 2706,
  [4187] = 2491,
  [4188] = 2492,
  [4189] = 4189,
  [4190] = 4190,
  [4191] = 2704,
  [4192] = 2510,
  [4193] = 2511,
  [4194] = 2482,
  [4195] = 4195,
  [4196] = 2333,
  [4197] = 2493,
  [4198] = 4189,
  [4199] = 4190,
  [4200] = 2512,
  [4201] = 2513,
  [4202] = 2514,
  [4203] = 2515,
  [4204] = 2516,
  [4205] = 2517,
  [4206] = 2494,
  [4207] = 4144,
  [4208] = 2519,
  [4209] = 4144,
  [4210] = 2522,
  [4211] = 2495,
  [4212] = 2495,
  [4213] = 2494,
  [4214] = 2514,
  [4215] = 2511,
  [4216] = 2497,
  [4217] = 2517,
  [4218] = 2513,
  [4219] = 2500,
  [4220] = 2492,
  [4221] = 2522,
  [4222] = 4189,
  [4223] = 4190,
  [4224] = 2514,
  [4225] = 2491,
  [4226] = 2514,
  [4227] = 2226,
  [4228] = 2515,
  [4229] = 2508,
  [4230] = 2509,
  [4231] = 2492,
  [4232] = 2512,
  [4233] = 2493,
  [4234] = 2518,
  [4235] = 2519,
  [4236] = 2519,
  [4237] = 2498,
  [4238] = 2515,
  [4239] = 2508,
  [4240] = 2495,
  [4241] = 2497,
  [4242] = 2493,
  [4243] = 2494,
  [4244] = 2498,
  [4245] = 2494,
  [4246] = 4189,
  [4247] = 4190,
  [4248] = 2500,
  [4249] = 2521,
  [4250] = 2516,
  [4251] = 2516,
  [4252] = 2517,
  [4253] = 2491,
  [4254] = 2513,
  [4255] = 2509,
  [4256] = 2510,
  [4257] = 2704,
  [4258] = 2511,
  [4259] = 2512,
  [4260] = 2495,
  [4261] = 2518,
  [4262] = 2510,
  [4263] = 2514,
  [4264] = 4264,
  [4265] = 4264,
  [4266] = 4264,
  [4267] = 4264,
  [4268] = 4264,
  [4269] = 4269,
  [4270] = 4264,
  [4271] = 4264,
  [4272] = 4264,
  [4273] = 4264,
  [4274] = 4264,
  [4275] = 4264,
  [4276] = 4264,
  [4277] = 4264,
  [4278] = 4278,
  [4279] = 4264,
  [4280] = 2704,
  [4281] = 4264,
  [4282] = 4264,
  [4283] = 2491,
  [4284] = 2492,
  [4285] = 4264,
  [4286] = 4264,
  [4287] = 4264,
  [4288] = 4264,
  [4289] = 2508,
  [4290] = 2497,
  [4291] = 4264,
  [4292] = 4264,
  [4293] = 4264,
  [4294] = 4264,
  [4295] = 4264,
  [4296] = 4264,
  [4297] = 4264,
  [4298] = 4264,
  [4299] = 4264,
  [4300] = 4264,
  [4301] = 4264,
  [4302] = 4264,
  [4303] = 4264,
  [4304] = 4269,
  [4305] = 4264,
  [4306] = 4264,
  [4307] = 4307,
  [4308] = 4269,
  [4309] = 4264,
  [4310] = 3604,
  [4311] = 4264,
  [4312] = 2509,
  [4313] = 4264,
  [4314] = 2510,
  [4315] = 2511,
  [4316] = 2512,
  [4317] = 2493,
  [4318] = 2513,
  [4319] = 2494,
  [4320] = 2515,
  [4321] = 2516,
  [4322] = 2517,
  [4323] = 2518,
  [4324] = 4264,
  [4325] = 2519,
  [4326] = 2495,
  [4327] = 2498,
  [4328] = 4264,
  [4329] = 4264,
  [4330] = 4264,
  [4331] = 4269,
  [4332] = 3609,
  [4333] = 4269,
  [4334] = 4269,
  [4335] = 4269,
  [4336] = 4269,
  [4337] = 4269,
  [4338] = 4264,
  [4339] = 4269,
  [4340] = 4269,
  [4341] = 4269,
  [4342] = 4269,
  [4343] = 4269,
  [4344] = 4264,
  [4345] = 4264,
  [4346] = 4269,
  [4347] = 4269,
  [4348] = 4269,
  [4349] = 4269,
  [4350] = 4269,
  [4351] = 4269,
  [4352] = 4269,
  [4353] = 4269,
  [4354] = 4269,
  [4355] = 4269,
  [4356] = 4264,
  [4357] = 2113,
  [4358] = 2099,
  [4359] = 2100,
  [4360] = 4360,
  [4361] = 2103,
  [4362] = 2259,
  [4363] = 2082,
  [4364] = 2252,
  [4365] = 2090,
  [4366] = 2091,
  [4367] = 2092,
  [4368] = 4368,
  [4369] = 2093,
  [4370] = 2094,
  [4371] = 2095,
  [4372] = 2096,
  [4373] = 2097,
  [4374] = 2098,
  [4375] = 4375,
  [4376] = 4376,
  [4377] = 2085,
  [4378] = 2088,
  [4379] = 4379,
  [4380] = 2341,
  [4381] = 4381,
  [4382] = 2092,
  [4383] = 2286,
  [4384] = 2334,
  [4385] = 2335,
  [4386] = 2093,
  [4387] = 2293,
  [4388] = 2336,
  [4389] = 2295,
  [4390] = 2296,
  [4391] = 2297,
  [4392] = 2094,
  [4393] = 2298,
  [4394] = 2285,
  [4395] = 2090,
  [4396] = 4396,
  [4397] = 2095,
  [4398] = 2096,
  [4399] = 2097,
  [4400] = 2098,
  [4401] = 2082,
  [4402] = 2385,
  [4403] = 2386,
  [4404] = 2344,
  [4405] = 2091,
  [4406] = 2380,
  [4407] = 2313,
  [4408] = 4408,
  [4409] = 2101,
  [4410] = 4410,
  [4411] = 4411,
  [4412] = 2331,
  [4413] = 2347,
  [4414] = 2349,
  [4415] = 2085,
  [4416] = 2088,
  [4417] = 2322,
  [4418] = 2323,
  [4419] = 2473,
  [4420] = 4420,
  [4421] = 2467,
  [4422] = 2468,
  [4423] = 2469,
  [4424] = 2288,
  [4425] = 2471,
  [4426] = 2472,
  [4427] = 2337,
  [4428] = 2113,
  [4429] = 2099,
  [4430] = 2100,
  [4431] = 2103,
  [4432] = 2327,
  [4433] = 4433,
  [4434] = 4434,
  [4435] = 4435,
  [4436] = 4436,
  [4437] = 4437,
  [4438] = 4438,
  [4439] = 4439,
  [4440] = 1908,
  [4441] = 1914,
  [4442] = 4442,
  [4443] = 3202,
  [4444] = 1915,
  [4445] = 1906,
  [4446] = 4446,
  [4447] = 4447,
  [4448] = 4448,
  [4449] = 4449,
  [4450] = 4450,
  [4451] = 4451,
  [4452] = 4452,
  [4453] = 4453,
  [4454] = 4454,
  [4455] = 3199,
  [4456] = 4456,
  [4457] = 4457,
  [4458] = 3609,
  [4459] = 3604,
  [4460] = 1913,
  [4461] = 4442,
  [4462] = 4462,
  [4463] = 4463,
  [4464] = 4464,
  [4465] = 4465,
  [4466] = 4464,
  [4467] = 4467,
  [4468] = 4468,
  [4469] = 4469,
  [4470] = 4470,
  [4471] = 4471,
  [4472] = 4472,
  [4473] = 4473,
  [4474] = 4474,
  [4475] = 4475,
  [4476] = 4476,
  [4477] = 4477,
  [4478] = 4478,
  [4479] = 4479,
  [4480] = 4480,
  [4481] = 4481,
  [4482] = 4482,
  [4483] = 4483,
  [4484] = 4484,
  [4485] = 4485,
  [4486] = 4486,
  [4487] = 4487,
  [4488] = 4488,
  [4489] = 4489,
  [4490] = 4490,
  [4491] = 4491,
  [4492] = 4492,
  [4493] = 4493,
  [4494] = 4494,
  [4495] = 3609,
  [4496] = 4496,
  [4497] = 4497,
  [4498] = 4498,
  [4499] = 4499,
  [4500] = 4500,
  [4501] = 4501,
  [4502] = 4502,
  [4503] = 4503,
  [4504] = 3609,
  [4505] = 3604,
  [4506] = 4506,
  [4507] = 4507,
  [4508] = 4508,
  [4509] = 4509,
  [4510] = 4510,
  [4511] = 4511,
  [4512] = 4512,
  [4513] = 4513,
  [4514] = 3604,
  [4515] = 4515,
  [4516] = 4516,
  [4517] = 4517,
  [4518] = 4518,
  [4519] = 4519,
  [4520] = 4520,
  [4521] = 4521,
  [4522] = 4522,
  [4523] = 4523,
  [4524] = 4524,
  [4525] = 4525,
  [4526] = 4526,
  [4527] = 4527,
  [4528] = 4528,
  [4529] = 4529,
  [4530] = 4530,
  [4531] = 4531,
  [4532] = 4532,
  [4533] = 4533,
  [4534] = 4534,
  [4535] = 4535,
  [4536] = 4534,
  [4537] = 4534,
  [4538] = 4538,
  [4539] = 4534,
  [4540] = 4534,
  [4541] = 4534,
  [4542] = 4534,
  [4543] = 4534,
  [4544] = 4534,
  [4545] = 4534,
  [4546] = 4534,
  [4547] = 4534,
  [4548] = 4534,
  [4549] = 4534,
  [4550] = 4534,
  [4551] = 4534,
  [4552] = 4534,
  [4553] = 4534,
  [4554] = 4534,
  [4555] = 4534,
  [4556] = 4534,
  [4557] = 4534,
  [4558] = 4534,
  [4559] = 4534,
  [4560] = 4560,
  [4561] = 4533,
  [4562] = 4560,
  [4563] = 4410,
  [4564] = 4560,
  [4565] = 4522,
  [4566] = 4560,
  [4567] = 4567,
  [4568] = 4560,
  [4569] = 4560,
  [4570] = 4560,
  [4571] = 4521,
  [4572] = 4572,
  [4573] = 4573,
  [4574] = 4538,
  [4575] = 4572,
  [4576] = 4576,
  [4577] = 4577,
  [4578] = 4578,
  [4579] = 4579,
  [4580] = 4580,
  [4581] = 4581,
  [4582] = 4582,
  [4583] = 4583,
  [4584] = 4434,
  [4585] = 3107,
  [4586] = 4535,
  [4587] = 2333,
  [4588] = 4588,
  [4589] = 4589,
  [4590] = 4588,
  [4591] = 4591,
  [4592] = 4535,
  [4593] = 4448,
  [4594] = 4594,
  [4595] = 4595,
  [4596] = 4538,
  [4597] = 4597,
  [4598] = 4598,
  [4599] = 2303,
  [4600] = 4591,
  [4601] = 4601,
  [4602] = 4602,
  [4603] = 4603,
  [4604] = 2226,
  [4605] = 4473,
  [4606] = 4470,
  [4607] = 4471,
  [4608] = 4603,
  [4609] = 4602,
  [4610] = 4602,
  [4611] = 4611,
  [4612] = 4475,
  [4613] = 4613,
  [4614] = 4613,
  [4615] = 4603,
  [4616] = 4611,
  [4617] = 4611,
  [4618] = 4474,
  [4619] = 4619,
  [4620] = 4619,
  [4621] = 4410,
  [4622] = 4619,
  [4623] = 4619,
  [4624] = 4517,
  [4625] = 4625,
  [4626] = 4626,
  [4627] = 1910,
  [4628] = 4532,
  [4629] = 2226,
  [4630] = 4528,
  [4631] = 4526,
  [4632] = 4521,
  [4633] = 4533,
  [4634] = 4523,
  [4635] = 4525,
  [4636] = 4531,
  [4637] = 2226,
  [4638] = 4524,
  [4639] = 4520,
  [4640] = 4527,
  [4641] = 4522,
  [4642] = 4530,
  [4643] = 4529,
  [4644] = 4434,
  [4645] = 4535,
  [4646] = 2088,
  [4647] = 2103,
  [4648] = 2082,
  [4649] = 2090,
  [4650] = 2091,
  [4651] = 2043,
  [4652] = 2092,
  [4653] = 4368,
  [4654] = 4379,
  [4655] = 2093,
  [4656] = 4375,
  [4657] = 4376,
  [4658] = 4360,
  [4659] = 4410,
  [4660] = 2252,
  [4661] = 2094,
  [4662] = 2085,
  [4663] = 4538,
  [4664] = 4448,
  [4665] = 2095,
  [4666] = 2259,
  [4667] = 4278,
  [4668] = 2044,
  [4669] = 2096,
  [4670] = 2097,
  [4671] = 2100,
  [4672] = 2113,
  [4673] = 2099,
  [4674] = 2098,
  [4675] = 2098,
  [4676] = 2303,
  [4677] = 2082,
  [4678] = 2333,
  [4679] = 2336,
  [4680] = 4470,
  [4681] = 2088,
  [4682] = 2297,
  [4683] = 2097,
  [4684] = 2467,
  [4685] = 2468,
  [4686] = 4475,
  [4687] = 2093,
  [4688] = 2288,
  [4689] = 2094,
  [4690] = 2473,
  [4691] = 4473,
  [4692] = 4408,
  [4693] = 4517,
  [4694] = 2113,
  [4695] = 2095,
  [4696] = 2096,
  [4697] = 4533,
  [4698] = 2285,
  [4699] = 2380,
  [4700] = 2469,
  [4701] = 2101,
  [4702] = 4396,
  [4703] = 2298,
  [4704] = 2099,
  [4705] = 4381,
  [4706] = 2296,
  [4707] = 2472,
  [4708] = 2043,
  [4709] = 4410,
  [4710] = 2044,
  [4711] = 2295,
  [4712] = 2100,
  [4713] = 2337,
  [4714] = 2286,
  [4715] = 2092,
  [4716] = 2103,
  [4717] = 2327,
  [4718] = 2293,
  [4719] = 2385,
  [4720] = 2386,
  [4721] = 2085,
  [4722] = 2091,
  [4723] = 2313,
  [4724] = 2331,
  [4725] = 2347,
  [4726] = 4521,
  [4727] = 2349,
  [4728] = 4434,
  [4729] = 2341,
  [4730] = 2090,
  [4731] = 4522,
  [4732] = 4474,
  [4733] = 2323,
  [4734] = 2322,
  [4735] = 4411,
  [4736] = 2344,
  [4737] = 2334,
  [4738] = 2335,
  [4739] = 4420,
  [4740] = 2471,
  [4741] = 4434,
  [4742] = 2059,
  [4743] = 4533,
  [4744] = 2057,
  [4745] = 4745,
  [4746] = 4448,
  [4747] = 4535,
  [4748] = 4538,
  [4749] = 4522,
  [4750] = 4448,
  [4751] = 4521,
  [4752] = 4598,
  [4753] = 4753,
  [4754] = 4465,
  [4755] = 4467,
  [4756] = 4474,
  [4757] = 4438,
  [4758] = 4526,
  [4759] = 4528,
  [4760] = 4451,
  [4761] = 4581,
  [4762] = 1915,
  [4763] = 4456,
  [4764] = 1906,
  [4765] = 4475,
  [4766] = 1911,
  [4767] = 4457,
  [4768] = 4523,
  [4769] = 4463,
  [4770] = 4589,
  [4771] = 4535,
  [4772] = 3107,
  [4773] = 1909,
  [4774] = 4753,
  [4775] = 1910,
  [4776] = 4473,
  [4777] = 4538,
  [4778] = 1907,
  [4779] = 4452,
  [4780] = 4470,
  [4781] = 4520,
  [4782] = 4527,
  [4783] = 4449,
  [4784] = 4529,
  [4785] = 4753,
  [4786] = 4468,
  [4787] = 4436,
  [4788] = 4474,
  [4789] = 4447,
  [4790] = 4462,
  [4791] = 4439,
  [4792] = 4597,
  [4793] = 4532,
  [4794] = 4475,
  [4795] = 4454,
  [4796] = 4471,
  [4797] = 4535,
  [4798] = 4538,
  [4799] = 4470,
  [4800] = 4453,
  [4801] = 4601,
  [4802] = 4481,
  [4803] = 4499,
  [4804] = 4435,
  [4805] = 4433,
  [4806] = 4531,
  [4807] = 2057,
  [4808] = 4524,
  [4809] = 4578,
  [4810] = 4469,
  [4811] = 4753,
  [4812] = 4530,
  [4813] = 1913,
  [4814] = 1908,
  [4815] = 1914,
  [4816] = 4450,
  [4817] = 2059,
  [4818] = 4525,
  [4819] = 4446,
  [4820] = 4595,
  [4821] = 4594,
  [4822] = 4753,
  [4823] = 4823,
  [4824] = 1910,
  [4825] = 4576,
  [4826] = 4437,
  [4827] = 4827,
  [4828] = 4828,
  [4829] = 4477,
  [4830] = 4473,
  [4831] = 1910,
  [4832] = 4464,
  [4833] = 1913,
  [4834] = 4834,
  [4835] = 1907,
  [4836] = 2200,
  [4837] = 4589,
  [4838] = 4838,
  [4839] = 4839,
  [4840] = 1913,
  [4841] = 1908,
  [4842] = 1914,
  [4843] = 4521,
  [4844] = 3199,
  [4845] = 4442,
  [4846] = 4503,
  [4847] = 4535,
  [4848] = 1915,
  [4849] = 4533,
  [4850] = 4464,
  [4851] = 1909,
  [4852] = 1911,
  [4853] = 1908,
  [4854] = 3202,
  [4855] = 4538,
  [4856] = 1914,
  [4857] = 4857,
  [4858] = 4478,
  [4859] = 4827,
  [4860] = 4834,
  [4861] = 4625,
  [4862] = 4522,
  [4863] = 4480,
  [4864] = 3107,
  [4865] = 4476,
  [4866] = 4472,
  [4867] = 4839,
  [4868] = 4626,
  [4869] = 4857,
  [4870] = 4857,
  [4871] = 4595,
  [4872] = 4594,
  [4873] = 1915,
  [4874] = 4839,
  [4875] = 1906,
  [4876] = 4834,
  [4877] = 4597,
  [4878] = 4598,
  [4879] = 4827,
  [4880] = 4601,
  [4881] = 4442,
  [4882] = 1906,
  [4883] = 4883,
  [4884] = 4884,
  [4885] = 4885,
  [4886] = 4527,
  [4887] = 4887,
  [4888] = 4884,
  [4889] = 4532,
  [4890] = 4890,
  [4891] = 4521,
  [4892] = 4884,
  [4893] = 4533,
  [4894] = 4479,
  [4895] = 4529,
  [4896] = 4533,
  [4897] = 4521,
  [4898] = 4530,
  [4899] = 4483,
  [4900] = 4884,
  [4901] = 4887,
  [4902] = 4497,
  [4903] = 4883,
  [4904] = 4522,
  [4905] = 4519,
  [4906] = 4906,
  [4907] = 4906,
  [4908] = 4890,
  [4909] = 4887,
  [4910] = 4906,
  [4911] = 4883,
  [4912] = 4526,
  [4913] = 4884,
  [4914] = 4885,
  [4915] = 4906,
  [4916] = 4524,
  [4917] = 4887,
  [4918] = 4535,
  [4919] = 4906,
  [4920] = 4887,
  [4921] = 4885,
  [4922] = 4883,
  [4923] = 4520,
  [4924] = 4538,
  [4925] = 4522,
  [4926] = 4496,
  [4927] = 4489,
  [4928] = 4883,
  [4929] = 4531,
  [4930] = 4531,
  [4931] = 4526,
  [4932] = 4890,
  [4933] = 4487,
  [4934] = 4524,
  [4935] = 4494,
  [4936] = 4523,
  [4937] = 4525,
  [4938] = 4529,
  [4939] = 4939,
  [4940] = 4535,
  [4941] = 4538,
  [4942] = 2108,
  [4943] = 4509,
  [4944] = 4944,
  [4945] = 4500,
  [4946] = 2113,
  [4947] = 4522,
  [4948] = 4515,
  [4949] = 2099,
  [4950] = 4950,
  [4951] = 2100,
  [4952] = 2103,
  [4953] = 2082,
  [4954] = 4954,
  [4955] = 2112,
  [4956] = 4535,
  [4957] = 4538,
  [4958] = 4958,
  [4959] = 2090,
  [4960] = 2091,
  [4961] = 2092,
  [4962] = 2093,
  [4963] = 2094,
  [4964] = 4532,
  [4965] = 2106,
  [4966] = 4511,
  [4967] = 2095,
  [4968] = 2096,
  [4969] = 2097,
  [4970] = 2098,
  [4971] = 4971,
  [4972] = 4523,
  [4973] = 4506,
  [4974] = 4516,
  [4975] = 4530,
  [4976] = 4485,
  [4977] = 4488,
  [4978] = 4521,
  [4979] = 4979,
  [4980] = 4980,
  [4981] = 4981,
  [4982] = 4971,
  [4983] = 4979,
  [4984] = 4979,
  [4985] = 4490,
  [4986] = 4980,
  [4987] = 4980,
  [4988] = 2042,
  [4989] = 4491,
  [4990] = 4482,
  [4991] = 4979,
  [4992] = 4980,
  [4993] = 4981,
  [4994] = 4508,
  [4995] = 4995,
  [4996] = 4522,
  [4997] = 4510,
  [4998] = 4998,
  [4999] = 4971,
  [5000] = 4981,
  [5001] = 5001,
  [5002] = 4512,
  [5003] = 4486,
  [5004] = 4980,
  [5005] = 4492,
  [5006] = 4520,
  [5007] = 4527,
  [5008] = 4981,
  [5009] = 4493,
  [5010] = 4513,
  [5011] = 4518,
  [5012] = 2255,
  [5013] = 2257,
  [5014] = 4533,
  [5015] = 4501,
  [5016] = 5016,
  [5017] = 4484,
  [5018] = 4502,
  [5019] = 4498,
  [5020] = 4507,
  [5021] = 4521,
  [5022] = 5022,
  [5023] = 2085,
  [5024] = 2088,
  [5025] = 4971,
  [5026] = 4981,
  [5027] = 4979,
  [5028] = 4525,
  [5029] = 2263,
  [5030] = 4958,
  [5031] = 2101,
  [5032] = 4958,
  [5033] = 4950,
  [5034] = 4533,
  [5035] = 5035,
  [5036] = 4971,
  [5037] = 5037,
  [5038] = 5038,
  [5039] = 5039,
  [5040] = 2095,
  [5041] = 4522,
  [5042] = 5042,
  [5043] = 5043,
  [5044] = 5039,
  [5045] = 5045,
  [5046] = 5046,
  [5047] = 5047,
  [5048] = 5045,
  [5049] = 5049,
  [5050] = 5050,
  [5051] = 5051,
  [5052] = 2092,
  [5053] = 5037,
  [5054] = 5037,
  [5055] = 5037,
  [5056] = 5037,
  [5057] = 2096,
  [5058] = 2113,
  [5059] = 2097,
  [5060] = 5060,
  [5061] = 5037,
  [5062] = 5062,
  [5063] = 5042,
  [5064] = 5043,
  [5065] = 5039,
  [5066] = 5045,
  [5067] = 2098,
  [5068] = 2093,
  [5069] = 5037,
  [5070] = 5038,
  [5071] = 5071,
  [5072] = 5072,
  [5073] = 5073,
  [5074] = 2091,
  [5075] = 5075,
  [5076] = 2099,
  [5077] = 5042,
  [5078] = 5047,
  [5079] = 5062,
  [5080] = 5039,
  [5081] = 4521,
  [5082] = 2234,
  [5083] = 5037,
  [5084] = 5042,
  [5085] = 5042,
  [5086] = 5043,
  [5087] = 5037,
  [5088] = 5088,
  [5089] = 5038,
  [5090] = 5090,
  [5091] = 5071,
  [5092] = 5072,
  [5093] = 5037,
  [5094] = 5073,
  [5095] = 5095,
  [5096] = 5038,
  [5097] = 5037,
  [5098] = 2226,
  [5099] = 5090,
  [5100] = 2094,
  [5101] = 5071,
  [5102] = 5072,
  [5103] = 5037,
  [5104] = 2101,
  [5105] = 5047,
  [5106] = 5037,
  [5107] = 5062,
  [5108] = 5043,
  [5109] = 5109,
  [5110] = 5039,
  [5111] = 5045,
  [5112] = 5038,
  [5113] = 5047,
  [5114] = 2112,
  [5115] = 5037,
  [5116] = 2100,
  [5117] = 5062,
  [5118] = 5118,
  [5119] = 5050,
  [5120] = 5109,
  [5121] = 5121,
  [5122] = 2217,
  [5123] = 5123,
  [5124] = 2106,
  [5125] = 5037,
  [5126] = 4533,
  [5127] = 5088,
  [5128] = 5047,
  [5129] = 5062,
  [5130] = 2103,
  [5131] = 5037,
  [5132] = 2244,
  [5133] = 5133,
  [5134] = 5037,
  [5135] = 5071,
  [5136] = 5071,
  [5137] = 5037,
  [5138] = 4535,
  [5139] = 4538,
  [5140] = 4521,
  [5141] = 5037,
  [5142] = 5043,
  [5143] = 5037,
  [5144] = 5050,
  [5145] = 5073,
  [5146] = 5037,
  [5147] = 5147,
  [5148] = 5037,
  [5149] = 5088,
  [5150] = 5071,
  [5151] = 5038,
  [5152] = 2042,
  [5153] = 5037,
  [5154] = 5109,
  [5155] = 5121,
  [5156] = 5090,
  [5157] = 5037,
  [5158] = 5037,
  [5159] = 4533,
  [5160] = 5037,
  [5161] = 5071,
  [5162] = 5072,
  [5163] = 5073,
  [5164] = 5088,
  [5165] = 2085,
  [5166] = 5037,
  [5167] = 5073,
  [5168] = 5121,
  [5169] = 5037,
  [5170] = 5050,
  [5171] = 5037,
  [5172] = 5037,
  [5173] = 5037,
  [5174] = 5090,
  [5175] = 2088,
  [5176] = 5037,
  [5177] = 4954,
  [5178] = 5109,
  [5179] = 5043,
  [5180] = 5090,
  [5181] = 5123,
  [5182] = 5072,
  [5183] = 5037,
  [5184] = 4522,
  [5185] = 5043,
  [5186] = 5037,
  [5187] = 5037,
  [5188] = 5037,
  [5189] = 5037,
  [5190] = 5037,
  [5191] = 5037,
  [5192] = 5121,
  [5193] = 5037,
  [5194] = 5037,
  [5195] = 5037,
  [5196] = 5196,
  [5197] = 5037,
  [5198] = 5037,
  [5199] = 5037,
  [5200] = 5037,
  [5201] = 5037,
  [5202] = 5045,
  [5203] = 5050,
  [5204] = 5123,
  [5205] = 5123,
  [5206] = 5109,
  [5207] = 5042,
  [5208] = 5121,
  [5209] = 5088,
  [5210] = 2090,
  [5211] = 2108,
  [5212] = 5123,
  [5213] = 2082,
  [5214] = 4522,
  [5215] = 5215,
  [5216] = 5216,
  [5217] = 5217,
  [5218] = 2246,
  [5219] = 2483,
  [5220] = 5220,
  [5221] = 4535,
  [5222] = 2248,
  [5223] = 5223,
  [5224] = 4538,
  [5225] = 5225,
  [5226] = 2109,
  [5227] = 2249,
  [5228] = 5228,
  [5229] = 5220,
  [5230] = 5230,
  [5231] = 5230,
  [5232] = 5232,
  [5233] = 5232,
  [5234] = 5234,
  [5235] = 5235,
  [5236] = 4535,
  [5237] = 5237,
  [5238] = 5220,
  [5239] = 5239,
  [5240] = 5240,
  [5241] = 5241,
  [5242] = 2105,
  [5243] = 5243,
  [5244] = 4538,
  [5245] = 5245,
  [5246] = 5230,
  [5247] = 5247,
  [5248] = 5220,
  [5249] = 5249,
  [5250] = 5239,
  [5251] = 2455,
  [5252] = 5232,
  [5253] = 4535,
  [5254] = 5254,
  [5255] = 4538,
  [5256] = 2367,
  [5257] = 5240,
  [5258] = 5235,
  [5259] = 1910,
  [5260] = 5220,
  [5261] = 5215,
  [5262] = 5216,
  [5263] = 5235,
  [5264] = 2084,
  [5265] = 5265,
  [5266] = 5266,
  [5267] = 4533,
  [5268] = 5268,
  [5269] = 5269,
  [5270] = 5270,
  [5271] = 5225,
  [5272] = 5228,
  [5273] = 5215,
  [5274] = 5265,
  [5275] = 5266,
  [5276] = 5254,
  [5277] = 5254,
  [5278] = 2247,
  [5279] = 2267,
  [5280] = 5239,
  [5281] = 5240,
  [5282] = 5216,
  [5283] = 5283,
  [5284] = 5228,
  [5285] = 2260,
  [5286] = 5225,
  [5287] = 5265,
  [5288] = 5266,
  [5289] = 4521,
  [5290] = 5290,
  [5291] = 2281,
  [5292] = 2252,
  [5293] = 2259,
  [5294] = 5290,
  [5295] = 5241,
  [5296] = 1910,
  [5297] = 5297,
  [5298] = 5241,
  [5299] = 5299,
  [5300] = 5290,
  [5301] = 5301,
  [5302] = 5237,
  [5303] = 5303,
  [5304] = 5304,
  [5305] = 5297,
  [5306] = 5306,
  [5307] = 5234,
  [5308] = 5299,
  [5309] = 5309,
  [5310] = 5310,
  [5311] = 5311,
  [5312] = 5312,
  [5313] = 1910,
  [5314] = 5314,
  [5315] = 5309,
  [5316] = 5316,
  [5317] = 4482,
  [5318] = 5309,
  [5319] = 5237,
  [5320] = 5320,
  [5321] = 5297,
  [5322] = 5322,
  [5323] = 2389,
  [5324] = 5299,
  [5325] = 5325,
  [5326] = 5326,
  [5327] = 2391,
  [5328] = 2400,
  [5329] = 5329,
  [5330] = 5330,
  [5331] = 2319,
  [5332] = 2299,
  [5333] = 2467,
  [5334] = 2401,
  [5335] = 2402,
  [5336] = 5336,
  [5337] = 2430,
  [5338] = 2431,
  [5339] = 2422,
  [5340] = 4420,
  [5341] = 5341,
  [5342] = 2350,
  [5343] = 5343,
  [5344] = 2351,
  [5345] = 2354,
  [5346] = 2436,
  [5347] = 2321,
  [5348] = 2357,
  [5349] = 2360,
  [5350] = 5350,
  [5351] = 4521,
  [5352] = 2364,
  [5353] = 2325,
  [5354] = 2309,
  [5355] = 2361,
  [5356] = 5356,
  [5357] = 2311,
  [5358] = 5358,
  [5359] = 2326,
  [5360] = 2414,
  [5361] = 5361,
  [5362] = 2385,
  [5363] = 2415,
  [5364] = 2372,
  [5365] = 2286,
  [5366] = 2418,
  [5367] = 2373,
  [5368] = 5368,
  [5369] = 5369,
  [5370] = 2397,
  [5371] = 2377,
  [5372] = 5350,
  [5373] = 2316,
  [5374] = 5374,
  [5375] = 5375,
  [5376] = 2290,
  [5377] = 2313,
  [5378] = 2331,
  [5379] = 2318,
  [5380] = 2378,
  [5381] = 2386,
  [5382] = 5382,
  [5383] = 2425,
  [5384] = 2379,
  [5385] = 5385,
  [5386] = 2384,
  [5387] = 2329,
  [5388] = 5196,
  [5389] = 2388,
  [5390] = 2330,
  [5391] = 2347,
  [5392] = 4535,
  [5393] = 2302,
  [5394] = 2469,
  [5395] = 5341,
  [5396] = 4538,
  [5397] = 2332,
  [5398] = 2356,
  [5399] = 5341,
  [5400] = 2396,
  [5401] = 2393,
  [5402] = 2322,
  [5403] = 5329,
  [5404] = 5404,
  [5405] = 5341,
  [5406] = 2473,
  [5407] = 2394,
  [5408] = 5356,
  [5409] = 2368,
  [5410] = 2381,
  [5411] = 2298,
  [5412] = 2349,
  [5413] = 2382,
  [5414] = 2303,
  [5415] = 2427,
  [5416] = 2323,
  [5417] = 2404,
  [5418] = 2338,
  [5419] = 2380,
  [5420] = 2376,
  [5421] = 2327,
  [5422] = 2291,
  [5423] = 5423,
  [5424] = 2292,
  [5425] = 2306,
  [5426] = 2339,
  [5427] = 2405,
  [5428] = 2448,
  [5429] = 2308,
  [5430] = 2287,
  [5431] = 2449,
  [5432] = 2355,
  [5433] = 5433,
  [5434] = 2468,
  [5435] = 5435,
  [5436] = 2362,
  [5437] = 2041,
  [5438] = 2363,
  [5439] = 2365,
  [5440] = 4535,
  [5441] = 5441,
  [5442] = 5442,
  [5443] = 5356,
  [5444] = 2348,
  [5445] = 2417,
  [5446] = 2395,
  [5447] = 5330,
  [5448] = 5448,
  [5449] = 5329,
  [5450] = 4538,
  [5451] = 2437,
  [5452] = 2289,
  [5453] = 2438,
  [5454] = 5341,
  [5455] = 2366,
  [5456] = 2426,
  [5457] = 2416,
  [5458] = 2295,
  [5459] = 2317,
  [5460] = 2334,
  [5461] = 5309,
  [5462] = 2288,
  [5463] = 2335,
  [5464] = 2336,
  [5465] = 2471,
  [5466] = 2398,
  [5467] = 5467,
  [5468] = 5341,
  [5469] = 5433,
  [5470] = 2423,
  [5471] = 5330,
  [5472] = 2472,
  [5473] = 2304,
  [5474] = 2285,
  [5475] = 2305,
  [5476] = 5476,
  [5477] = 2296,
  [5478] = 2341,
  [5479] = 2293,
  [5480] = 5361,
  [5481] = 2344,
  [5482] = 5482,
  [5483] = 2314,
  [5484] = 5341,
  [5485] = 2300,
  [5486] = 2451,
  [5487] = 2333,
  [5488] = 2294,
  [5489] = 5374,
  [5490] = 2297,
  [5491] = 2337,
  [5492] = 2399,
  [5493] = 2452,
  [5494] = 2383,
  [5495] = 5495,
  [5496] = 5496,
  [5497] = 5497,
  [5498] = 5498,
  [5499] = 5499,
  [5500] = 5498,
  [5501] = 5501,
  [5502] = 5502,
  [5503] = 5503,
  [5504] = 5497,
  [5505] = 5505,
  [5506] = 5503,
  [5507] = 5507,
  [5508] = 5501,
  [5509] = 5509,
  [5510] = 5510,
  [5511] = 5507,
  [5512] = 5512,
  [5513] = 5513,
  [5514] = 5514,
  [5515] = 5515,
  [5516] = 2084,
  [5517] = 5512,
  [5518] = 5518,
  [5519] = 5519,
  [5520] = 5513,
  [5521] = 5498,
  [5522] = 5499,
  [5523] = 5497,
  [5524] = 5512,
  [5525] = 5525,
  [5526] = 5503,
  [5527] = 5514,
  [5528] = 5496,
  [5529] = 5515,
  [5530] = 5530,
  [5531] = 5495,
  [5532] = 5532,
  [5533] = 5270,
  [5534] = 5509,
  [5535] = 5535,
  [5536] = 5519,
  [5537] = 5525,
  [5538] = 5510,
  [5539] = 5539,
  [5540] = 5498,
  [5541] = 5497,
  [5542] = 2105,
  [5543] = 5543,
  [5544] = 5544,
  [5545] = 5268,
  [5546] = 5513,
  [5547] = 5547,
  [5548] = 5535,
  [5549] = 5515,
  [5550] = 5243,
  [5551] = 5498,
  [5552] = 5245,
  [5553] = 2109,
  [5554] = 5518,
  [5555] = 5247,
  [5556] = 5556,
  [5557] = 5518,
  [5558] = 5283,
  [5559] = 5530,
  [5560] = 5518,
  [5561] = 5501,
  [5562] = 5512,
  [5563] = 5513,
  [5564] = 5514,
  [5565] = 5496,
  [5566] = 5566,
  [5567] = 5356,
  [5568] = 5513,
  [5569] = 5569,
  [5570] = 5502,
  [5571] = 5518,
  [5572] = 5512,
  [5573] = 5573,
  [5574] = 5497,
  [5575] = 5499,
  [5576] = 5576,
  [5577] = 5577,
  [5578] = 5577,
  [5579] = 5577,
  [5580] = 5577,
  [5581] = 5581,
  [5582] = 5581,
  [5583] = 5577,
  [5584] = 5577,
  [5585] = 5585,
  [5586] = 5577,
  [5587] = 5581,
  [5588] = 5588,
  [5589] = 5581,
  [5590] = 5581,
  [5591] = 5581,
  [5592] = 5577,
  [5593] = 5593,
  [5594] = 5581,
  [5595] = 5581,
  [5596] = 5577,
  [5597] = 5577,
  [5598] = 5588,
  [5599] = 5581,
  [5600] = 5577,
  [5601] = 5581,
  [5602] = 5581,
  [5603] = 5581,
  [5604] = 5581,
  [5605] = 5588,
  [5606] = 5606,
  [5607] = 5577,
  [5608] = 5608,
  [5609] = 5581,
  [5610] = 5577,
  [5611] = 5577,
  [5612] = 5581,
  [5613] = 5613,
  [5614] = 5577,
  [5615] = 5581,
  [5616] = 5577,
  [5617] = 5581,
  [5618] = 5618,
  [5619] = 5577,
  [5620] = 5577,
  [5621] = 5581,
  [5622] = 5581,
  [5623] = 5577,
  [5624] = 5577,
  [5625] = 5585,
  [5626] = 5577,
  [5627] = 5627,
  [5628] = 1909,
  [5629] = 5581,
  [5630] = 5630,
  [5631] = 5577,
  [5632] = 5581,
  [5633] = 5627,
  [5634] = 5581,
  [5635] = 5577,
  [5636] = 5581,
  [5637] = 5577,
  [5638] = 5638,
  [5639] = 5581,
  [5640] = 5640,
  [5641] = 5641,
  [5642] = 5642,
  [5643] = 5643,
  [5644] = 5644,
  [5645] = 5640,
  [5646] = 5643,
  [5647] = 5647,
  [5648] = 5642,
  [5649] = 5644,
  [5650] = 5650,
  [5651] = 5651,
  [5652] = 5652,
  [5653] = 5640,
  [5654] = 5654,
  [5655] = 5642,
  [5656] = 5656,
  [5657] = 5657,
  [5658] = 5644,
  [5659] = 5644,
  [5660] = 5660,
  [5661] = 5661,
  [5662] = 5662,
  [5663] = 5650,
  [5664] = 5664,
  [5665] = 5665,
  [5666] = 5643,
  [5667] = 5642,
  [5668] = 5644,
  [5669] = 5650,
  [5670] = 5644,
  [5671] = 5671,
  [5672] = 5650,
  [5673] = 5673,
  [5674] = 5674,
  [5675] = 5656,
  [5676] = 5651,
  [5677] = 5677,
  [5678] = 5678,
  [5679] = 5664,
  [5680] = 5644,
  [5681] = 5681,
  [5682] = 5682,
  [5683] = 5683,
  [5684] = 5684,
  [5685] = 5644,
  [5686] = 5652,
  [5687] = 5640,
  [5688] = 5688,
  [5689] = 5654,
  [5690] = 5690,
  [5691] = 5691,
  [5692] = 5651,
  [5693] = 5683,
  [5694] = 5694,
  [5695] = 5647,
  [5696] = 5696,
  [5697] = 5644,
  [5698] = 5651,
  [5699] = 5644,
  [5700] = 5652,
  [5701] = 5701,
  [5702] = 5702,
  [5703] = 5703,
  [5704] = 5704,
  [5705] = 5644,
  [5706] = 5643,
  [5707] = 5651,
  [5708] = 5640,
  [5709] = 5709,
  [5710] = 5710,
  [5711] = 5664,
  [5712] = 5644,
  [5713] = 5713,
  [5714] = 5664,
  [5715] = 5715,
  [5716] = 5651,
  [5717] = 5644,
  [5718] = 5718,
  [5719] = 5719,
  [5720] = 5720,
  [5721] = 5721,
  [5722] = 5651,
  [5723] = 5657,
  [5724] = 5651,
  [5725] = 5725,
  [5726] = 5644,
  [5727] = 5644,
  [5728] = 5728,
  [5729] = 5651,
  [5730] = 5730,
  [5731] = 5644,
  [5732] = 5732,
  [5733] = 5728,
  [5734] = 5650,
  [5735] = 5735,
  [5736] = 5660,
  [5737] = 5651,
  [5738] = 5644,
  [5739] = 5656,
  [5740] = 5656,
  [5741] = 5664,
  [5742] = 5644,
  [5743] = 5743,
  [5744] = 5744,
  [5745] = 5671,
  [5746] = 5651,
  [5747] = 5747,
  [5748] = 5748,
  [5749] = 5749,
  [5750] = 5651,
  [5751] = 5644,
  [5752] = 5671,
  [5753] = 5732,
  [5754] = 5644,
  [5755] = 5665,
  [5756] = 5664,
  [5757] = 5651,
  [5758] = 5758,
  [5759] = 5744,
  [5760] = 5748,
  [5761] = 5761,
  [5762] = 5709,
  [5763] = 5763,
  [5764] = 5657,
  [5765] = 5765,
  [5766] = 5651,
  [5767] = 5644,
  [5768] = 5768,
  [5769] = 5769,
  [5770] = 5747,
  [5771] = 5647,
  [5772] = 5772,
  [5773] = 5773,
  [5774] = 5763,
  [5775] = 5661,
  [5776] = 5696,
  [5777] = 5777,
  [5778] = 5768,
  [5779] = 5662,
  [5780] = 1915,
  [5781] = 5781,
  [5782] = 5744,
  [5783] = 5651,
  [5784] = 5644,
  [5785] = 5720,
  [5786] = 5643,
  [5787] = 5642,
  [5788] = 5688,
  [5789] = 5694,
  [5790] = 5719,
  [5791] = 5671,
  [5792] = 5765,
  [5793] = 5703,
  [5794] = 5644,
  [5795] = 5795,
  [5796] = 5644,
  [5797] = 5704,
  [5798] = 5644,
  [5799] = 5799,
  [5800] = 5678,
  [5801] = 5719,
  [5802] = 5674,
  [5803] = 5719,
  [5804] = 5804,
  [5805] = 5665,
  [5806] = 5683,
  [5807] = 5709,
  [5808] = 5678,
  [5809] = 5674,
  [5810] = 5810,
  [5811] = 5651,
  [5812] = 5678,
  [5813] = 5657,
  [5814] = 5652,
  [5815] = 5641,
  [5816] = 5651,
  [5817] = 5644,
  [5818] = 5769,
  [5819] = 5718,
  [5820] = 5678,
  [5821] = 5677,
  [5822] = 5671,
  [5823] = 5640,
  [5824] = 5824,
  [5825] = 5644,
  [5826] = 5719,
  [5827] = 5827,
  [5828] = 5743,
  [5829] = 5701,
  [5830] = 5728,
  [5831] = 5652,
  [5832] = 5664,
  [5833] = 5656,
  [5834] = 5640,
  [5835] = 5651,
  [5836] = 5744,
  [5837] = 5678,
  [5838] = 5710,
  [5839] = 5773,
  [5840] = 5713,
  [5841] = 5773,
  [5842] = 5656,
  [5843] = 5651,
  [5844] = 5644,
  [5845] = 5644,
  [5846] = 5846,
  [5847] = 5847,
  [5848] = 5683,
  [5849] = 5804,
  [5850] = 5644,
  [5851] = 5644,
  [5852] = 5644,
  [5853] = 5644,
  [5854] = 5810,
  [5855] = 5647,
  [5856] = 5856,
  [5857] = 5761,
  [5858] = 5664,
  [5859] = 5859,
  [5860] = 5678,
  [5861] = 5673,
  [5862] = 5773,
  [5863] = 5644,
  [5864] = 1906,
  [5865] = 5651,
  [5866] = 5748,
  [5867] = 5651,
  [5868] = 5846,
  [5869] = 5735,
  [5870] = 5643,
  [5871] = 5871,
  [5872] = 5642,
  [5873] = 5656,
  [5874] = 5664,
  [5875] = 5875,
  [5876] = 5651,
  [5877] = 5644,
  [5878] = 5651,
  [5879] = 5824,
  [5880] = 5650,
  [5881] = 5656,
  [5882] = 5702,
  [5883] = 5883,
  [5884] = 5769,
  [5885] = 5719,
  [5886] = 5690,
  [5887] = 5702,
  [5888] = 5696,
  [5889] = 5710,
  [5890] = 5644,
  [5891] = 5769,
  [5892] = 5773,
  [5893] = 5893,
  [5894] = 5643,
  [5895] = 5644,
  [5896] = 5664,
  [5897] = 5897,
  [5898] = 5642,
  [5899] = 5651,
  [5900] = 5744,
  [5901] = 5642,
  [5902] = 5652,
  [5903] = 5768,
  [5904] = 5713,
  [5905] = 5644,
  [5906] = 5683,
  [5907] = 5651,
  [5908] = 5664,
  [5909] = 5758,
  [5910] = 5769,
  [5911] = 5719,
  [5912] = 5804,
  [5913] = 5654,
  [5914] = 5720,
  [5915] = 5644,
  [5916] = 5691,
  [5917] = 5781,
  [5918] = 5644,
  [5919] = 5749,
  [5920] = 5650,
  [5921] = 5691,
  [5922] = 5922,
  [5923] = 5643,
  [5924] = 5683,
  [5925] = 5644,
  [5926] = 5827,
  [5927] = 5652,
  [5928] = 5641,
  [5929] = 5640,
  [5930] = 5678,
  [5931] = 5690,
  [5932] = 5749,
  [5933] = 5644,
  [5934] = 5827,
  [5935] = 5683,
  [5936] = 5644,
  [5937] = 5730,
  [5938] = 5847,
  [5939] = 5721,
  [5940] = 5650,
  [5941] = 5721,
  [5942] = 5657,
  [5943] = 5781,
  [5944] = 5944,
  [5945] = 5664,
  [5946] = 5704,
  [5947] = 5642,
  [5948] = 5656,
  [5949] = 5847,
  [5950] = 5950,
  [5951] = 5810,
  [5952] = 5647,
  [5953] = 5652,
  [5954] = 5684,
  [5955] = 5650,
  [5956] = 5922,
  [5957] = 5652,
  [5958] = 5718,
  [5959] = 5640,
  [5960] = 5871,
  [5961] = 5681,
  [5962] = 5652,
  [5963] = 5673,
  [5964] = 5732,
  [5965] = 5846,
  [5966] = 5644,
  [5967] = 5824,
  [5968] = 5644,
  [5969] = 5643,
  [5970] = 5761,
  [5971] = 5640,
  [5972] = 5719,
  [5973] = 5944,
  [5974] = 5719,
  [5975] = 5684,
  [5976] = 5743,
  [5977] = 5735,
  [5978] = 5643,
  [5979] = 5678,
  [5980] = 5719,
  [5981] = 5642,
  [5982] = 5656,
  [5983] = 5944,
  [5984] = 5678,
  [5985] = 5650,
  [5986] = 5644,
  [5987] = 5987,
  [5988] = 5988,
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 293,
        '$', 56,
        '%', 272,
        '&', 215,
        '(', 228,
        ')', 194,
        '*', 211,
        '+', 267,
        ',', 203,
        '-', 284,
        '.', 196,
        '/', 270,
        '0', 296,
        ':', 219,
        ';', 207,
        '<', 222,
        '=', 292,
        '>', 262,
        '?', 249,
        '@', 73,
        'B', 305,
        'E', 308,
        '[', 227,
      );
      if (lookahead == '\\') SKIP(106);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == 'd') ADVANCE(323);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(143);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^') ADVANCE(328);
      END_STATE();
    case 1:
      if (lookahead == '\n') ADVANCE(67);
      if (lookahead == '\r') ADVANCE(1);
      if (lookahead != 0) ADVANCE(1);
      END_STATE();
    case 2:
      if (lookahead == '\n') SKIP(41);
      END_STATE();
    case 3:
      if (lookahead == '\n') SKIP(41);
      if (lookahead == '\r') SKIP(2);
      END_STATE();
    case 4:
      if (lookahead == '\n') SKIP(42);
      END_STATE();
    case 5:
      if (lookahead == '\n') SKIP(42);
      if (lookahead == '\r') SKIP(4);
      END_STATE();
    case 6:
      if (lookahead == '\n') SKIP(44);
      END_STATE();
    case 7:
      if (lookahead == '\n') SKIP(44);
      if (lookahead == '\r') SKIP(6);
      END_STATE();
    case 8:
      if (lookahead == '\n') SKIP(33);
      END_STATE();
    case 9:
      if (lookahead == '\n') SKIP(33);
      if (lookahead == '\r') SKIP(8);
      END_STATE();
    case 10:
      if (lookahead == '\n') SKIP(46);
      END_STATE();
    case 11:
      if (lookahead == '\n') SKIP(46);
      if (lookahead == '\r') SKIP(10);
      END_STATE();
    case 12:
      if (lookahead == '\n') SKIP(37);
      END_STATE();
    case 13:
      if (lookahead == '\n') SKIP(37);
      if (lookahead == '\r') SKIP(12);
      END_STATE();
    case 14:
      if (lookahead == '\n') SKIP(51);
      END_STATE();
    case 15:
      if (lookahead == '\n') SKIP(51);
      if (lookahead == '\r') SKIP(14);
      END_STATE();
    case 16:
      if (lookahead == '\n') SKIP(50);
      END_STATE();
    case 17:
      if (lookahead == '\n') SKIP(50);
      if (lookahead == '\r') SKIP(16);
      END_STATE();
    case 18:
      if (lookahead == '\n') ADVANCE(344);
      if (lookahead == '\r') ADVANCE(343);
      if (lookahead == 'u') ADVANCE(84);
      if (lookahead == 'x') ADVANCE(97);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(347);
      if (lookahead != 0) ADVANCE(343);
      END_STATE();
    case 19:
      if (lookahead == '\n') ADVANCE(345);
      if (lookahead == '\r') ADVANCE(343);
      if (lookahead == 'u') ADVANCE(84);
      if (lookahead == 'x') ADVANCE(97);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(347);
      if (lookahead != 0) ADVANCE(343);
      END_STATE();
    case 20:
      if (lookahead == '\n') SKIP(54);
      END_STATE();
    case 21:
      if (lookahead == '\n') SKIP(54);
      if (lookahead == '\r') SKIP(20);
      END_STATE();
    case 22:
      if (lookahead == '\n') SKIP(39);
      END_STATE();
    case 23:
      if (lookahead == '\n') SKIP(39);
      if (lookahead == '\r') SKIP(22);
      END_STATE();
    case 24:
      if (lookahead == '\n') SKIP(31);
      END_STATE();
    case 25:
      if (lookahead == '\n') SKIP(31);
      if (lookahead == '\r') SKIP(24);
      END_STATE();
    case 26:
      if (lookahead == '\n') SKIP(35);
      END_STATE();
    case 27:
      if (lookahead == '\n') SKIP(35);
      if (lookahead == '\r') SKIP(26);
      END_STATE();
    case 28:
      if (lookahead == '\n') SKIP(52);
      END_STATE();
    case 29:
      if (lookahead == '\n') SKIP(52);
      if (lookahead == '\r') SKIP(28);
      END_STATE();
    case 30:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 272,
        '&', 215,
        '(', 228,
        ')', 194,
        '*', 212,
        '+', 266,
        ',', 203,
        '-', 69,
        '.', 196,
        '/', 270,
        '0', 296,
        ':', 219,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 249,
        '@', 73,
        '[', 227,
      );
      if (lookahead == '\\') SKIP(25);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(31);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 31:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 272,
        '&', 215,
        '(', 193,
        ')', 194,
        '*', 212,
        '+', 266,
        ',', 203,
        '-', 69,
        '.', 196,
        '/', 270,
        '0', 296,
        ':', 63,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(25);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(31);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 32:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 214,
        '(', 193,
        '*', 210,
        '+', 268,
        '-', 284,
        '.', 61,
        '/', 269,
        ':', 218,
        '<', 223,
        '=', 66,
        '>', 263,
        '@', 73,
        '[', 75,
      );
      if (lookahead == '\\') SKIP(9);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(33);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 33:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 214,
        '(', 193,
        '*', 210,
        '+', 268,
        '-', 284,
        '.', 61,
        '/', 269,
        '<', 223,
        '=', 66,
        '>', 263,
        '@', 73,
        '[', 75,
      );
      if (lookahead == '\\') SKIP(9);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(33);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 34:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 214,
        '*', 210,
        '+', 268,
        '-', 284,
        '.', 61,
        '/', 269,
        '<', 223,
        '=', 292,
        '>', 263,
        '@', 73,
        '[', 75,
      );
      if (lookahead == '\\') SKIP(27);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(35);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 35:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 214,
        '*', 210,
        '+', 268,
        '-', 284,
        '.', 61,
        '/', 269,
        '<', 223,
        '=', 66,
        '>', 263,
        '@', 73,
        '[', 75,
      );
      if (lookahead == '\\') SKIP(27);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(35);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 36:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '%', 271,
        '&', 214,
        '(', 228,
        '*', 210,
        '+', 268,
        '-', 284,
        '.', 61,
        '/', 269,
        '<', 223,
        '=', 66,
        '>', 263,
        '[', 75,
      );
      if (lookahead == '\\') SKIP(13);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(37);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 37:
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '%', 271,
        '&', 214,
        '*', 210,
        '+', 268,
        '-', 284,
        '.', 61,
        '/', 269,
        '<', 223,
        '=', 66,
        '>', 263,
        '[', 75,
      );
      if (lookahead == '\\') SKIP(13);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(37);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 38:
      ADVANCE_MAP(
        '!', 280,
        '#', 294,
        '$', 56,
        '&', 58,
        '(', 228,
        '*', 209,
        '+', 265,
        '-', 71,
        '.', 196,
        '0', 296,
        ':', 62,
        ';', 207,
        '<', 221,
        '=', 76,
        '?', 74,
        '@', 73,
        'B', 305,
        'E', 308,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(23);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(39);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 39:
      ADVANCE_MAP(
        '!', 280,
        '#', 294,
        '$', 56,
        '&', 58,
        '(', 193,
        '*', 209,
        '+', 265,
        '-', 71,
        '.', 196,
        '0', 296,
        ':', 63,
        ';', 207,
        '<', 221,
        '=', 76,
        '?', 74,
        '@', 73,
        'B', 305,
        'E', 308,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(23);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(39);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 40:
      ADVANCE_MAP(
        '!', 280,
        '#', 294,
        '$', 56,
        '&', 58,
        '(', 193,
        '*', 209,
        '+', 265,
        '-', 71,
        '.', 196,
        '0', 296,
        ':', 62,
        ';', 207,
        '<', 221,
        '=', 76,
        '?', 74,
        '@', 73,
        'B', 305,
        'E', 308,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(23);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(39);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 41:
      ADVANCE_MAP(
        '!', 280,
        '#', 294,
        '$', 56,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        '-', 71,
        '.', 59,
        '0', 296,
        ':', 63,
        ';', 207,
        '=', 199,
        '?', 74,
        '@', 73,
        'B', 305,
        'E', 308,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(3);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(41);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 42:
      ADVANCE_MAP(
        '!', 280,
        '#', 294,
        '$', 56,
        '(', 193,
        '*', 210,
        '+', 265,
        '-', 71,
        '.', 59,
        '0', 296,
        ':', 63,
        '=', 76,
        '?', 74,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(5);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(42);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 43:
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 272,
        '&', 215,
        '(', 228,
        ')', 194,
        '*', 55,
        '+', 266,
        ',', 203,
        '-', 68,
        '.', 196,
        '/', 270,
        ':', 251,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(7);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(44);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 44:
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 272,
        '&', 215,
        ')', 194,
        '*', 55,
        '+', 266,
        ',', 203,
        '-', 68,
        '.', 196,
        '/', 270,
        ':', 250,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(7);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(44);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 45:
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 271,
        '&', 217,
        '(', 228,
        ')', 194,
        '+', 265,
        ',', 203,
        '.', 196,
        '/', 269,
        ':', 251,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(11);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(46);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 46:
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 271,
        '&', 217,
        ')', 194,
        '+', 265,
        ',', 203,
        '.', 196,
        '/', 269,
        ':', 250,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(11);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(46);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 47:
      if (lookahead == '#') ADVANCE(293);
      if (lookahead == '=') ADVANCE(76);
      if (lookahead == '\\') ADVANCE(18);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(341);
      END_STATE();
    case 48:
      if (lookahead == '#') ADVANCE(293);
      if (lookahead == '=') ADVANCE(76);
      if (lookahead == '\\') ADVANCE(19);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(48);
      END_STATE();
    case 49:
      ADVANCE_MAP(
        '#', 294,
        '&', 214,
        '(', 193,
        ')', 194,
        '*', 210,
        ',', 203,
        '.', 59,
        ':', 218,
        ';', 207,
        '=', 202,
      );
      if (lookahead == '\\') SKIP(17);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(50);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 50:
      ADVANCE_MAP(
        '#', 294,
        '&', 214,
        '(', 193,
        ')', 194,
        '*', 210,
        ',', 203,
        '.', 59,
        ';', 207,
        '=', 202,
      );
      if (lookahead == '\\') SKIP(17);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(50);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 51:
      ADVANCE_MAP(
        '#', 294,
        '&', 214,
        '(', 193,
        ')', 194,
        '*', 210,
        ',', 203,
        '.', 197,
        ':', 63,
        ';', 207,
        '=', 202,
      );
      if (lookahead == '\\') SKIP(15);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(51);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 52:
      ADVANCE_MAP(
        '#', 294,
        '&', 214,
        '(', 193,
        '*', 210,
        '.', 197,
        ':', 63,
        ';', 207,
        '=', 199,
      );
      if (lookahead == '\\') SKIP(29);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(52);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 53:
      ADVANCE_MAP(
        '#', 294,
        '&', 214,
        '(', 193,
        '*', 210,
        '.', 197,
        ':', 63,
        ';', 207,
        '=', 291,
      );
      if (lookahead == '\\') SKIP(29);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(52);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 54:
      if (lookahead == '#') ADVANCE(294);
      if (lookahead == '=') ADVANCE(76);
      if (lookahead == '\\') SKIP(21);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(342);
      END_STATE();
    case 55:
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == '=') ADVANCE(237);
      END_STATE();
    case 56:
      if (lookahead == '-') ADVANCE(101);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(333);
      if (lookahead == '!' ||
          lookahead == '"' ||
          lookahead == '$' ||
          lookahead == '&' ||
          lookahead == '\'' ||
          ('*' <= lookahead && lookahead <= '@') ||
          lookahead == '\\' ||
          lookahead == '`' ||
          lookahead == '~') ADVANCE(332);
      if (('A' <= lookahead && lookahead <= 'Z') ||
          ('_' <= lookahead && lookahead <= 'z')) ADVANCE(334);
      END_STATE();
    case 57:
      if (lookahead == '-') ADVANCE(103);
      END_STATE();
    case 58:
      if (lookahead == '.') ADVANCE(234);
      END_STATE();
    case 59:
      if (lookahead == '.') ADVANCE(230);
      END_STATE();
    case 60:
      if (lookahead == '.') ADVANCE(208);
      END_STATE();
    case 61:
      if (lookahead == '.') ADVANCE(229);
      END_STATE();
    case 62:
      if (lookahead == ':') ADVANCE(233);
      END_STATE();
    case 63:
      if (lookahead == ':') ADVANCE(198);
      END_STATE();
    case 64:
      if (lookahead == '=') ADVANCE(274);
      if (lookahead == '~') ADVANCE(278);
      END_STATE();
    case 65:
      if (lookahead == '=') ADVANCE(273);
      if (lookahead == '>') ADVANCE(224);
      if (lookahead == 'b') ADVANCE(78);
      if (lookahead == '~') ADVANCE(277);
      END_STATE();
    case 66:
      if (lookahead == '=') ADVANCE(273);
      if (lookahead == 'b') ADVANCE(78);
      if (lookahead == '~') ADVANCE(277);
      END_STATE();
    case 67:
      if (lookahead == '=') ADVANCE(79);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ' ||
          lookahead == '*') ADVANCE(67);
      if (lookahead != 0) ADVANCE(67);
      END_STATE();
    case 68:
      if (lookahead == '=') ADVANCE(236);
      END_STATE();
    case 69:
      if (lookahead == '=') ADVANCE(236);
      if (lookahead == '>') ADVANCE(349);
      END_STATE();
    case 70:
      if (lookahead == '=') ADVANCE(238);
      END_STATE();
    case 71:
      if (lookahead == '>') ADVANCE(349);
      END_STATE();
    case 72:
      if (lookahead == '>') ADVANCE(224);
      if (lookahead == 'b') ADVANCE(78);
      END_STATE();
    case 73:
      if (lookahead == '@') ADVANCE(102);
      if (lookahead > '@' &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(330);
      END_STATE();
    case 74:
      if (lookahead == '\\') ADVANCE(339);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != ' ') ADVANCE(335);
      END_STATE();
    case 75:
      if (lookahead == ']') ADVANCE(288);
      END_STATE();
    case 76:
      if (lookahead == 'b') ADVANCE(78);
      END_STATE();
    case 77:
      if (lookahead == 'd') ADVANCE(294);
      if (lookahead != 0) ADVANCE(67);
      END_STATE();
    case 78:
      if (lookahead == 'e') ADVANCE(80);
      END_STATE();
    case 79:
      if (lookahead == 'e') ADVANCE(83);
      if (lookahead != 0) ADVANCE(67);
      END_STATE();
    case 80:
      if (lookahead == 'g') ADVANCE(81);
      END_STATE();
    case 81:
      if (lookahead == 'i') ADVANCE(82);
      END_STATE();
    case 82:
      if (lookahead == 'n') ADVANCE(1);
      END_STATE();
    case 83:
      if (lookahead == 'n') ADVANCE(77);
      if (lookahead != 0) ADVANCE(67);
      END_STATE();
    case 84:
      if (lookahead == '{') ADVANCE(100);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(99);
      END_STATE();
    case 85:
      if (lookahead == '}') ADVANCE(335);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(85);
      END_STATE();
    case 86:
      if (lookahead == '}') ADVANCE(343);
      if (lookahead == ' ' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(86);
      END_STATE();
    case 87:
      if (lookahead == '+' ||
          lookahead == '-') ADVANCE(94);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(303);
      END_STATE();
    case 88:
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(298);
      END_STATE();
    case 89:
      if (lookahead == '8' ||
          lookahead == '9') ADVANCE(297);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(295);
      END_STATE();
    case 90:
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(299);
      END_STATE();
    case 91:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(297);
      END_STATE();
    case 92:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(302);
      END_STATE();
    case 93:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(301);
      END_STATE();
    case 94:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(303);
      END_STATE();
    case 95:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(300);
      END_STATE();
    case 96:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(343);
      END_STATE();
    case 97:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(348);
      END_STATE();
    case 98:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(96);
      END_STATE();
    case 99:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(98);
      END_STATE();
    case 100:
      if (lookahead == ' ' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(86);
      END_STATE();
    case 101:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(332);
      END_STATE();
    case 102:
      if (lookahead > '@' &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(331);
      END_STATE();
    case 103:
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != ' ') ADVANCE(335);
      END_STATE();
    case 104:
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != ' ') ADVANCE(337);
      END_STATE();
    case 105:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(143);
      END_STATE();
    case 106:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(143);
      if (lookahead == '\r') SKIP(105);
      END_STATE();
    case 107:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(151);
      END_STATE();
    case 108:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(151);
      if (lookahead == '\r') SKIP(107);
      END_STATE();
    case 109:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(152);
      END_STATE();
    case 110:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(152);
      if (lookahead == '\r') SKIP(109);
      END_STATE();
    case 111:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(161);
      END_STATE();
    case 112:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(161);
      if (lookahead == '\r') SKIP(111);
      END_STATE();
    case 113:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(160);
      END_STATE();
    case 114:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(160);
      if (lookahead == '\r') SKIP(113);
      END_STATE();
    case 115:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(169);
      END_STATE();
    case 116:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(169);
      if (lookahead == '\r') SKIP(115);
      END_STATE();
    case 117:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(155);
      END_STATE();
    case 118:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(155);
      if (lookahead == '\r') SKIP(117);
      END_STATE();
    case 119:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(156);
      END_STATE();
    case 120:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(156);
      if (lookahead == '\r') SKIP(119);
      END_STATE();
    case 121:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(158);
      END_STATE();
    case 122:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(158);
      if (lookahead == '\r') SKIP(121);
      END_STATE();
    case 123:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(159);
      END_STATE();
    case 124:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(159);
      if (lookahead == '\r') SKIP(123);
      END_STATE();
    case 125:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(167);
      END_STATE();
    case 126:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(167);
      if (lookahead == '\r') SKIP(125);
      END_STATE();
    case 127:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(166);
      END_STATE();
    case 128:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(166);
      if (lookahead == '\r') SKIP(127);
      END_STATE();
    case 129:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(170);
      END_STATE();
    case 130:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(170);
      if (lookahead == '\r') SKIP(129);
      END_STATE();
    case 131:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(163);
      END_STATE();
    case 132:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(163);
      if (lookahead == '\r') SKIP(131);
      END_STATE();
    case 133:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(145);
      END_STATE();
    case 134:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(145);
      if (lookahead == '\r') SKIP(133);
      END_STATE();
    case 135:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(171);
      END_STATE();
    case 136:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(171);
      if (lookahead == '\r') SKIP(135);
      END_STATE();
    case 137:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(148);
      END_STATE();
    case 138:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(148);
      if (lookahead == '\r') SKIP(137);
      END_STATE();
    case 139:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(147);
      END_STATE();
    case 140:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(147);
      if (lookahead == '\r') SKIP(139);
      END_STATE();
    case 141:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(150);
      END_STATE();
    case 142:
      if (eof) ADVANCE(173);
      if (lookahead == '\n') SKIP(150);
      if (lookahead == '\r') SKIP(141);
      END_STATE();
    case 143:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 293,
        '$', 56,
        '%', 272,
        '&', 215,
        '(', 193,
        ')', 194,
        '*', 211,
        '+', 267,
        ',', 203,
        '-', 284,
        '.', 196,
        '/', 270,
        '0', 296,
        ':', 252,
        ';', 207,
        '<', 222,
        '=', 201,
        '>', 262,
        '?', 249,
        '@', 73,
        'B', 305,
        'E', 308,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(106);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '`') ADVANCE(290);
      if (lookahead == 'd') ADVANCE(323);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(283);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(143);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^') ADVANCE(328);
      END_STATE();
    case 144:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 272,
        '&', 215,
        '(', 228,
        ')', 194,
        '*', 212,
        '+', 266,
        ',', 203,
        '-', 69,
        '.', 196,
        '/', 270,
        '0', 296,
        ':', 219,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(134);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(145);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 145:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 272,
        '&', 215,
        '(', 193,
        ')', 194,
        '*', 212,
        '+', 266,
        ',', 203,
        '-', 69,
        '.', 196,
        '/', 270,
        '0', 296,
        ':', 63,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(134);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(145);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 146:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 217,
        '(', 228,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 196,
        '/', 269,
        '0', 296,
        ':', 219,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(138);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(148);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 147:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 217,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 196,
        '/', 269,
        '0', 296,
        ':', 252,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(140);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(147);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 148:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 217,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 196,
        '/', 269,
        '0', 296,
        ':', 63,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(138);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(148);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 149:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 217,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 196,
        '/', 269,
        '0', 296,
        ':', 251,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(140);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(147);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 150:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 281,
        '#', 294,
        '$', 56,
        '%', 271,
        '&', 216,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 59,
        '/', 269,
        '0', 296,
        ':', 252,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 249,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(142);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(150);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 151:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 280,
        '#', 294,
        '$', 56,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 59,
        '0', 296,
        ':', 63,
        ';', 207,
        '=', 72,
        '?', 74,
        '@', 73,
        'B', 305,
        'E', 308,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(108);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(151);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 152:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 280,
        '#', 294,
        '$', 56,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 59,
        '0', 296,
        ':', 63,
        ';', 207,
        '=', 72,
        '?', 74,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(110);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (lookahead == '~') ADVANCE(282);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(152);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`') ADVANCE(321);
      END_STATE();
    case 153:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 272,
        '&', 215,
        '(', 228,
        ')', 194,
        '*', 55,
        '+', 266,
        ',', 203,
        '-', 68,
        '.', 196,
        '/', 270,
        ':', 219,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 248,
        '[', 227,
      );
      if (lookahead == '\\') SKIP(118);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(155);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 154:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 272,
        '&', 215,
        '(', 228,
        ')', 194,
        '*', 55,
        '+', 266,
        ',', 203,
        '-', 68,
        '.', 196,
        '/', 270,
        ':', 219,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(120);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(156);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 155:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 272,
        '&', 215,
        ')', 194,
        '*', 55,
        '+', 266,
        ',', 203,
        '-', 68,
        '.', 196,
        '/', 270,
        ':', 63,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(118);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(155);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 156:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 272,
        '&', 215,
        ')', 194,
        '*', 55,
        '+', 266,
        ',', 203,
        '-', 68,
        '.', 196,
        '/', 270,
        ';', 207,
        '<', 222,
        '=', 200,
        '>', 262,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(120);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(232);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(205);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(156);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 157:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 271,
        '&', 217,
        '(', 228,
        ')', 194,
        '+', 265,
        ',', 203,
        '.', 196,
        '/', 269,
        ':', 219,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(122);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(158);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 158:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 271,
        '&', 217,
        ')', 194,
        '+', 265,
        ',', 203,
        '.', 196,
        '/', 269,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(122);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(158);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 159:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '!', 64,
        '#', 294,
        '%', 271,
        '&', 216,
        ')', 194,
        '+', 265,
        ',', 203,
        '.', 59,
        '/', 269,
        ':', 250,
        ';', 207,
        '<', 223,
        '=', 65,
        '>', 263,
        '?', 248,
      );
      if (lookahead == '\\') SKIP(124);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(206);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(159);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 160:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '$', 56,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 59,
        '0', 296,
        ':', 63,
        ';', 207,
        '=', 72,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(114);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(160);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(311);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 161:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '$', 56,
        '(', 193,
        ')', 194,
        '*', 209,
        '+', 265,
        ',', 203,
        '-', 71,
        '0', 296,
        ':', 63,
        ';', 207,
        '=', 199,
        '?', 74,
        '@', 73,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(112);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == 'd') ADVANCE(316);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(161);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 162:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '&', 214,
        '(', 193,
        ')', 194,
        '*', 210,
        ',', 203,
        '.', 59,
        ':', 218,
        ';', 207,
        '=', 72,
      );
      if (lookahead == '\\') SKIP(132);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(163);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 163:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '&', 214,
        '(', 193,
        ')', 194,
        '*', 210,
        ',', 203,
        '.', 59,
        ';', 207,
        '=', 72,
      );
      if (lookahead == '\\') SKIP(132);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(163);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 164:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '&', 58,
        '(', 228,
        ')', 194,
        ',', 203,
        '.', 195,
        ':', 62,
        ';', 207,
        '=', 72,
      );
      if (lookahead == '\\') SKIP(128);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(166);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 165:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '&', 58,
        '(', 228,
        ')', 194,
        ',', 203,
        '.', 195,
        ':', 62,
        ';', 207,
        '=', 199,
      );
      if (lookahead == '\\') SKIP(126);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(167);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 166:
      if (eof) ADVANCE(173);
      if (lookahead == '#') ADVANCE(294);
      if (lookahead == '&') ADVANCE(58);
      if (lookahead == ')') ADVANCE(194);
      if (lookahead == ',') ADVANCE(203);
      if (lookahead == '.') ADVANCE(195);
      if (lookahead == ';') ADVANCE(207);
      if (lookahead == '=') ADVANCE(72);
      if (lookahead == '\\') SKIP(128);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(166);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 167:
      if (eof) ADVANCE(173);
      if (lookahead == '#') ADVANCE(294);
      if (lookahead == '&') ADVANCE(58);
      if (lookahead == ')') ADVANCE(194);
      if (lookahead == ',') ADVANCE(203);
      if (lookahead == '.') ADVANCE(195);
      if (lookahead == ';') ADVANCE(207);
      if (lookahead == '=') ADVANCE(199);
      if (lookahead == '\\') SKIP(126);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(167);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 168:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '(', 228,
        ')', 194,
        ',', 203,
        '.', 195,
        ':', 220,
        ';', 207,
        '=', 72,
        '[', 227,
      );
      if (lookahead == '\\') SKIP(130);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(170);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 169:
      if (eof) ADVANCE(173);
      ADVANCE_MAP(
        '#', 294,
        '(', 193,
        ')', 194,
        '*', 210,
        '+', 265,
        ',', 203,
        '-', 71,
        '.', 59,
        '0', 296,
        ':', 63,
        ';', 207,
        '=', 76,
        '[', 225,
      );
      if (lookahead == '\\') SKIP(116);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '^') ADVANCE(231);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(169);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(297);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(310);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 170:
      if (eof) ADVANCE(173);
      if (lookahead == '#') ADVANCE(294);
      if (lookahead == ')') ADVANCE(194);
      if (lookahead == ',') ADVANCE(203);
      if (lookahead == '.') ADVANCE(195);
      if (lookahead == ':') ADVANCE(63);
      if (lookahead == ';') ADVANCE(207);
      if (lookahead == '=') ADVANCE(72);
      if (lookahead == '\\') SKIP(130);
      if (lookahead == ']') ADVANCE(226);
      if (lookahead == '{') ADVANCE(190);
      if (lookahead == '|') ADVANCE(204);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(170);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 171:
      if (eof) ADVANCE(173);
      if (lookahead == '#') ADVANCE(294);
      if (lookahead == ')') ADVANCE(194);
      if (lookahead == ',') ADVANCE(203);
      if (lookahead == ';') ADVANCE(207);
      if (lookahead == '=') ADVANCE(76);
      if (lookahead == '\\') SKIP(136);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(171);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 172:
      if (eof) ADVANCE(173);
      if (lookahead == '#') ADVANCE(294);
      if (lookahead == ')') ADVANCE(194);
      if (lookahead == ',') ADVANCE(203);
      if (lookahead == ';') ADVANCE(207);
      if (lookahead == '=') ADVANCE(291);
      if (lookahead == '\\') SKIP(136);
      if (lookahead == '}') ADVANCE(191);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(171);
      if (lookahead > '^' &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 174:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == '\n') ADVANCE(178);
      if (lookahead == '\r') ADVANCE(175);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 175:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == '\n') ADVANCE(178);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 176:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == '\n') ADVANCE(188);
      if (lookahead != 0) ADVANCE(176);
      END_STATE();
    case 177:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == '\n') ADVANCE(179);
      if (lookahead == '\r') ADVANCE(177);
      if (lookahead != 0) ADVANCE(177);
      END_STATE();
    case 178:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == '#') ADVANCE(176);
      if (lookahead == '=') ADVANCE(180);
      if (lookahead == '\\') ADVANCE(174);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(178);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 179:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == '=') ADVANCE(183);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ' ||
          lookahead == '*') ADVANCE(179);
      if (lookahead != 0) ADVANCE(179);
      END_STATE();
    case 180:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'b') ADVANCE(182);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 181:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'd') ADVANCE(176);
      if (lookahead != 0) ADVANCE(179);
      END_STATE();
    case 182:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'e') ADVANCE(184);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 183:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'e') ADVANCE(186);
      if (lookahead != 0) ADVANCE(179);
      END_STATE();
    case 184:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'g') ADVANCE(185);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 185:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'i') ADVANCE(187);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 186:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'n') ADVANCE(181);
      if (lookahead != 0) ADVANCE(179);
      END_STATE();
    case 187:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead == 'n') ADVANCE(177);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 188:
      ACCEPT_TOKEN(sym_uninterpreted);
      if (lookahead != 0) ADVANCE(188);
      END_STATE();
    case 189:
      ACCEPT_TOKEN(anon_sym_BEGIN);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 190:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 191:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 192:
      ACCEPT_TOKEN(anon_sym_END);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 193:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 194:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 195:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 196:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(230);
      END_STATE();
    case 197:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(60);
      END_STATE();
    case 198:
      ACCEPT_TOKEN(anon_sym_COLON_COLON);
      END_STATE();
    case 199:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 200:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(273);
      if (lookahead == '>') ADVANCE(224);
      if (lookahead == '~') ADVANCE(277);
      END_STATE();
    case 201:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(273);
      if (lookahead == '~') ADVANCE(277);
      END_STATE();
    case 202:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '>') ADVANCE(224);
      END_STATE();
    case 203:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 204:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 205:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '=') ADVANCE(241);
      if (lookahead == '|') ADVANCE(254);
      END_STATE();
    case 206:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '|') ADVANCE(253);
      END_STATE();
    case 207:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 208:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_DOT);
      END_STATE();
    case 209:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 210:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '*') ADVANCE(213);
      END_STATE();
    case 211:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '*') ADVANCE(213);
      if (lookahead == '=') ADVANCE(237);
      END_STATE();
    case 212:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '*') ADVANCE(70);
      if (lookahead == '=') ADVANCE(237);
      END_STATE();
    case 213:
      ACCEPT_TOKEN(anon_sym_STAR_STAR);
      END_STATE();
    case 214:
      ACCEPT_TOKEN(anon_sym_AMP);
      END_STATE();
    case 215:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(256);
      if (lookahead == '.') ADVANCE(234);
      if (lookahead == '=') ADVANCE(243);
      END_STATE();
    case 216:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(255);
      END_STATE();
    case 217:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(255);
      if (lookahead == '.') ADVANCE(234);
      END_STATE();
    case 218:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 219:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == ':') ADVANCE(233);
      END_STATE();
    case 220:
      ACCEPT_TOKEN(anon_sym_COLON);
      if (lookahead == ':') ADVANCE(198);
      END_STATE();
    case 221:
      ACCEPT_TOKEN(anon_sym_LT);
      END_STATE();
    case 222:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(258);
      if (lookahead == '=') ADVANCE(261);
      END_STATE();
    case 223:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(257);
      if (lookahead == '=') ADVANCE(261);
      END_STATE();
    case 224:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      END_STATE();
    case 225:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 226:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 227:
      ACCEPT_TOKEN(anon_sym_LBRACK2);
      END_STATE();
    case 228:
      ACCEPT_TOKEN(anon_sym_LPAREN2);
      END_STATE();
    case 229:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      END_STATE();
    case 230:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      if (lookahead == '.') ADVANCE(208);
      END_STATE();
    case 231:
      ACCEPT_TOKEN(anon_sym_CARET);
      END_STATE();
    case 232:
      ACCEPT_TOKEN(anon_sym_CARET);
      if (lookahead == '=') ADVANCE(247);
      END_STATE();
    case 233:
      ACCEPT_TOKEN(anon_sym_COLON_COLON2);
      END_STATE();
    case 234:
      ACCEPT_TOKEN(anon_sym_AMP_DOT);
      END_STATE();
    case 235:
      ACCEPT_TOKEN(anon_sym_PLUS_EQ);
      END_STATE();
    case 236:
      ACCEPT_TOKEN(anon_sym_DASH_EQ);
      END_STATE();
    case 237:
      ACCEPT_TOKEN(anon_sym_STAR_EQ);
      END_STATE();
    case 238:
      ACCEPT_TOKEN(anon_sym_STAR_STAR_EQ);
      END_STATE();
    case 239:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 240:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE_EQ);
      END_STATE();
    case 241:
      ACCEPT_TOKEN(anon_sym_PIPE_EQ);
      END_STATE();
    case 242:
      ACCEPT_TOKEN(anon_sym_AMP_AMP_EQ);
      END_STATE();
    case 243:
      ACCEPT_TOKEN(anon_sym_AMP_EQ);
      END_STATE();
    case 244:
      ACCEPT_TOKEN(anon_sym_PERCENT_EQ);
      END_STATE();
    case 245:
      ACCEPT_TOKEN(anon_sym_GT_GT_EQ);
      END_STATE();
    case 246:
      ACCEPT_TOKEN(anon_sym_LT_LT_EQ);
      END_STATE();
    case 247:
      ACCEPT_TOKEN(anon_sym_CARET_EQ);
      END_STATE();
    case 248:
      ACCEPT_TOKEN(anon_sym_QMARK);
      END_STATE();
    case 249:
      ACCEPT_TOKEN(anon_sym_QMARK);
      if (lookahead == '\\') ADVANCE(339);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != ' ') ADVANCE(335);
      END_STATE();
    case 250:
      ACCEPT_TOKEN(anon_sym_COLON2);
      END_STATE();
    case 251:
      ACCEPT_TOKEN(anon_sym_COLON2);
      if (lookahead == ':') ADVANCE(233);
      END_STATE();
    case 252:
      ACCEPT_TOKEN(anon_sym_COLON2);
      if (lookahead == ':') ADVANCE(198);
      END_STATE();
    case 253:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 254:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      if (lookahead == '=') ADVANCE(240);
      END_STATE();
    case 255:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 256:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      if (lookahead == '=') ADVANCE(242);
      END_STATE();
    case 257:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      END_STATE();
    case 258:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      if (lookahead == '=') ADVANCE(246);
      END_STATE();
    case 259:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      END_STATE();
    case 260:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '=') ADVANCE(245);
      END_STATE();
    case 261:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      if (lookahead == '>') ADVANCE(276);
      END_STATE();
    case 262:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(264);
      if (lookahead == '>') ADVANCE(260);
      END_STATE();
    case 263:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(264);
      if (lookahead == '>') ADVANCE(259);
      END_STATE();
    case 264:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 265:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 266:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '=') ADVANCE(235);
      END_STATE();
    case 267:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '=') ADVANCE(235);
      if (lookahead == '@') ADVANCE(285);
      END_STATE();
    case 268:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '@') ADVANCE(285);
      END_STATE();
    case 269:
      ACCEPT_TOKEN(anon_sym_SLASH);
      END_STATE();
    case 270:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '=') ADVANCE(239);
      END_STATE();
    case 271:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 272:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      if (lookahead == '=') ADVANCE(244);
      END_STATE();
    case 273:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      if (lookahead == '=') ADVANCE(275);
      END_STATE();
    case 274:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 275:
      ACCEPT_TOKEN(anon_sym_EQ_EQ_EQ);
      END_STATE();
    case 276:
      ACCEPT_TOKEN(anon_sym_LT_EQ_GT);
      END_STATE();
    case 277:
      ACCEPT_TOKEN(anon_sym_EQ_TILDE);
      END_STATE();
    case 278:
      ACCEPT_TOKEN(anon_sym_BANG_TILDE);
      END_STATE();
    case 279:
      ACCEPT_TOKEN(anon_sym_defined_QMARK);
      END_STATE();
    case 280:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 281:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=') ADVANCE(274);
      if (lookahead == '~') ADVANCE(278);
      END_STATE();
    case 282:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 283:
      ACCEPT_TOKEN(anon_sym_TILDE);
      if (lookahead == '@') ADVANCE(287);
      END_STATE();
    case 284:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '@') ADVANCE(286);
      END_STATE();
    case 285:
      ACCEPT_TOKEN(anon_sym_PLUS_AT);
      END_STATE();
    case 286:
      ACCEPT_TOKEN(anon_sym_DASH_AT);
      END_STATE();
    case 287:
      ACCEPT_TOKEN(anon_sym_TILDE_AT);
      END_STATE();
    case 288:
      ACCEPT_TOKEN(anon_sym_LBRACK_RBRACK);
      if (lookahead == '=') ADVANCE(289);
      END_STATE();
    case 289:
      ACCEPT_TOKEN(anon_sym_LBRACK_RBRACK_EQ);
      END_STATE();
    case 290:
      ACCEPT_TOKEN(anon_sym_BQUOTE);
      END_STATE();
    case 291:
      ACCEPT_TOKEN(anon_sym_EQ2);
      END_STATE();
    case 292:
      ACCEPT_TOKEN(anon_sym_EQ2);
      if (lookahead == '=') ADVANCE(273);
      if (lookahead == '~') ADVANCE(277);
      END_STATE();
    case 293:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '{') ADVANCE(340);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(294);
      END_STATE();
    case 294:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(294);
      END_STATE();
    case 295:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '.') ADVANCE(92);
      if (lookahead == '_') ADVANCE(89);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(87);
      if (lookahead == '8' ||
          lookahead == '9') ADVANCE(297);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(295);
      END_STATE();
    case 296:
      ACCEPT_TOKEN(sym_integer);
      ADVANCE_MAP(
        '.', 92,
        '_', 91,
        'B', 88,
        'b', 88,
        'D', 93,
        'd', 93,
        'E', 87,
        'e', 87,
        'O', 90,
        'o', 90,
        'X', 95,
        'x', 95,
        '8', 297,
        '9', 297,
      );
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(295);
      END_STATE();
    case 297:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '.') ADVANCE(92);
      if (lookahead == '_') ADVANCE(91);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(87);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(297);
      END_STATE();
    case 298:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '_') ADVANCE(88);
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(298);
      END_STATE();
    case 299:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '_') ADVANCE(90);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(299);
      END_STATE();
    case 300:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '_') ADVANCE(95);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(300);
      END_STATE();
    case 301:
      ACCEPT_TOKEN(sym_integer);
      if (lookahead == '_') ADVANCE(93);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(301);
      END_STATE();
    case 302:
      ACCEPT_TOKEN(sym_float);
      if (lookahead == '_') ADVANCE(92);
      if (lookahead == 'E' ||
          lookahead == 'e') ADVANCE(87);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(302);
      END_STATE();
    case 303:
      ACCEPT_TOKEN(sym_float);
      if (lookahead == '_') ADVANCE(94);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(303);
      END_STATE();
    case 304:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead == 'D') ADVANCE(192);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 305:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead == 'E') ADVANCE(306);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 306:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead == 'G') ADVANCE(307);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 307:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead == 'I') ADVANCE(309);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 308:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead == 'N') ADVANCE(304);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 309:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead == 'N') ADVANCE(189);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 310:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead == '?') ADVANCE(312);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(310);
      END_STATE();
    case 311:
      ACCEPT_TOKEN(sym_constant);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(311);
      END_STATE();
    case 312:
      ACCEPT_TOKEN(aux_sym_constant_suffix_token1);
      END_STATE();
    case 313:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(279);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 314:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(279);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 315:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(329);
      if (lookahead == 'd') ADVANCE(314);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 316:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(329);
      if (lookahead == 'e') ADVANCE(318);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 317:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(329);
      if (lookahead == 'e') ADVANCE(315);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 318:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(329);
      if (lookahead == 'f') ADVANCE(319);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 319:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(329);
      if (lookahead == 'i') ADVANCE(320);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 320:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(329);
      if (lookahead == 'n') ADVANCE(317);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 321:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '?') ADVANCE(329);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(321);
      END_STATE();
    case 322:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'd') ADVANCE(313);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 323:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'e') ADVANCE(325);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 324:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'e') ADVANCE(322);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 325:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'f') ADVANCE(326);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 326:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'i') ADVANCE(327);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 327:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'n') ADVANCE(324);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 328:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(328);
      END_STATE();
    case 329:
      ACCEPT_TOKEN(aux_sym_identifier_suffix_token1);
      END_STATE();
    case 330:
      ACCEPT_TOKEN(sym_instance_variable);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(330);
      END_STATE();
    case 331:
      ACCEPT_TOKEN(sym_class_variable);
      if (lookahead > '/' &&
          (lookahead < ':' || '@' < lookahead) &&
          (lookahead < '[' || '^' < lookahead) &&
          lookahead != '`' &&
          (lookahead < '{' || '~' < lookahead)) ADVANCE(331);
      END_STATE();
    case 332:
      ACCEPT_TOKEN(sym_global_variable);
      END_STATE();
    case 333:
      ACCEPT_TOKEN(sym_global_variable);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(333);
      END_STATE();
    case 334:
      ACCEPT_TOKEN(sym_global_variable);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(334);
      END_STATE();
    case 335:
      ACCEPT_TOKEN(sym_character);
      END_STATE();
    case 336:
      ACCEPT_TOKEN(sym_character);
      if (lookahead == '-') ADVANCE(104);
      if (lookahead == '{') ADVANCE(85);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(338);
      END_STATE();
    case 337:
      ACCEPT_TOKEN(sym_character);
      if (lookahead == 'C' ||
          lookahead == 'M') ADVANCE(57);
      END_STATE();
    case 338:
      ACCEPT_TOKEN(sym_character);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(338);
      END_STATE();
    case 339:
      ACCEPT_TOKEN(sym_character);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != ' ') ADVANCE(336);
      END_STATE();
    case 340:
      ACCEPT_TOKEN(anon_sym_POUND_LBRACE);
      END_STATE();
    case 341:
      ACCEPT_TOKEN(aux_sym_string_array_token1);
      if (lookahead == '\\') ADVANCE(18);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(341);
      END_STATE();
    case 342:
      ACCEPT_TOKEN(aux_sym_string_array_token1);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(342);
      END_STATE();
    case 343:
      ACCEPT_TOKEN(sym_escape_sequence);
      END_STATE();
    case 344:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (lookahead == '\\') ADVANCE(18);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(341);
      END_STATE();
    case 345:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (lookahead == '\\') ADVANCE(19);
      END_STATE();
    case 346:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(343);
      END_STATE();
    case 347:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (('0' <= lookahead && lookahead <= '7')) ADVANCE(346);
      END_STATE();
    case 348:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(343);
      END_STATE();
    case 349:
      ACCEPT_TOKEN(anon_sym_DASH_GT);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (lookahead == '\\') SKIP(1);
      if (lookahead == '_') ADVANCE(2);
      if (lookahead == 'a') ADVANCE(3);
      if (lookahead == 'b') ADVANCE(4);
      if (lookahead == 'c') ADVANCE(5);
      if (lookahead == 'd') ADVANCE(6);
      if (lookahead == 'e') ADVANCE(7);
      if (lookahead == 'f') ADVANCE(8);
      if (lookahead == 'i') ADVANCE(9);
      if (lookahead == 'm') ADVANCE(10);
      if (lookahead == 'n') ADVANCE(11);
      if (lookahead == 'o') ADVANCE(12);
      if (lookahead == 'r') ADVANCE(13);
      if (lookahead == 's') ADVANCE(14);
      if (lookahead == 't') ADVANCE(15);
      if (lookahead == 'u') ADVANCE(16);
      if (lookahead == 'w') ADVANCE(17);
      if (lookahead == 'y') ADVANCE(18);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(19);
      END_STATE();
    case 1:
      if (lookahead == '\n') SKIP(19);
      if (lookahead == '\r') SKIP(20);
      END_STATE();
    case 2:
      if (lookahead == '_') ADVANCE(21);
      END_STATE();
    case 3:
      if (lookahead == 'l') ADVANCE(22);
      if (lookahead == 'n') ADVANCE(23);
      END_STATE();
    case 4:
      if (lookahead == 'e') ADVANCE(24);
      if (lookahead == 'r') ADVANCE(25);
      END_STATE();
    case 5:
      if (lookahead == 'a') ADVANCE(26);
      if (lookahead == 'l') ADVANCE(27);
      END_STATE();
    case 6:
      if (lookahead == 'e') ADVANCE(28);
      if (lookahead == 'o') ADVANCE(29);
      END_STATE();
    case 7:
      if (lookahead == 'l') ADVANCE(30);
      if (lookahead == 'n') ADVANCE(31);
      END_STATE();
    case 8:
      if (lookahead == 'a') ADVANCE(32);
      if (lookahead == 'o') ADVANCE(33);
      END_STATE();
    case 9:
      ACCEPT_TOKEN(anon_sym_i);
      if (lookahead == 'f') ADVANCE(34);
      if (lookahead == 'n') ADVANCE(35);
      END_STATE();
    case 10:
      if (lookahead == 'o') ADVANCE(36);
      END_STATE();
    case 11:
      if (lookahead == 'e') ADVANCE(37);
      if (lookahead == 'i') ADVANCE(38);
      if (lookahead == 'o') ADVANCE(39);
      END_STATE();
    case 12:
      if (lookahead == 'r') ADVANCE(40);
      END_STATE();
    case 13:
      ACCEPT_TOKEN(anon_sym_r);
      if (lookahead == 'e') ADVANCE(41);
      if (lookahead == 'i') ADVANCE(42);
      END_STATE();
    case 14:
      if (lookahead == 'e') ADVANCE(43);
      if (lookahead == 'u') ADVANCE(44);
      END_STATE();
    case 15:
      if (lookahead == 'h') ADVANCE(45);
      if (lookahead == 'r') ADVANCE(46);
      END_STATE();
    case 16:
      if (lookahead == 'n') ADVANCE(47);
      END_STATE();
    case 17:
      if (lookahead == 'h') ADVANCE(48);
      END_STATE();
    case 18:
      if (lookahead == 'i') ADVANCE(49);
      END_STATE();
    case 19:
      if (lookahead == '\\') SKIP(1);
      if (lookahead == '_') ADVANCE(2);
      if (lookahead == 'a') ADVANCE(3);
      if (lookahead == 'b') ADVANCE(4);
      if (lookahead == 'c') ADVANCE(5);
      if (lookahead == 'd') ADVANCE(6);
      if (lookahead == 'e') ADVANCE(7);
      if (lookahead == 'f') ADVANCE(8);
      if (lookahead == 'i') ADVANCE(50);
      if (lookahead == 'm') ADVANCE(10);
      if (lookahead == 'n') ADVANCE(11);
      if (lookahead == 'o') ADVANCE(12);
      if (lookahead == 'r') ADVANCE(51);
      if (lookahead == 's') ADVANCE(14);
      if (lookahead == 't') ADVANCE(15);
      if (lookahead == 'u') ADVANCE(16);
      if (lookahead == 'w') ADVANCE(17);
      if (lookahead == 'y') ADVANCE(18);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(19);
      END_STATE();
    case 20:
      if (lookahead == '\n') SKIP(19);
      END_STATE();
    case 21:
      if (lookahead == 'E') ADVANCE(52);
      if (lookahead == 'F') ADVANCE(53);
      if (lookahead == 'L') ADVANCE(54);
      END_STATE();
    case 22:
      if (lookahead == 'i') ADVANCE(55);
      END_STATE();
    case 23:
      if (lookahead == 'd') ADVANCE(56);
      END_STATE();
    case 24:
      if (lookahead == 'g') ADVANCE(57);
      END_STATE();
    case 25:
      if (lookahead == 'e') ADVANCE(58);
      END_STATE();
    case 26:
      if (lookahead == 's') ADVANCE(59);
      END_STATE();
    case 27:
      if (lookahead == 'a') ADVANCE(60);
      END_STATE();
    case 28:
      if (lookahead == 'f') ADVANCE(61);
      END_STATE();
    case 29:
      ACCEPT_TOKEN(anon_sym_do);
      END_STATE();
    case 30:
      if (lookahead == 's') ADVANCE(62);
      END_STATE();
    case 31:
      if (lookahead == 'd') ADVANCE(63);
      if (lookahead == 's') ADVANCE(64);
      END_STATE();
    case 32:
      if (lookahead == 'l') ADVANCE(65);
      END_STATE();
    case 33:
      if (lookahead == 'r') ADVANCE(66);
      END_STATE();
    case 34:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 35:
      ACCEPT_TOKEN(anon_sym_in);
      END_STATE();
    case 36:
      if (lookahead == 'd') ADVANCE(67);
      END_STATE();
    case 37:
      if (lookahead == 'x') ADVANCE(68);
      END_STATE();
    case 38:
      if (lookahead == 'l') ADVANCE(69);
      END_STATE();
    case 39:
      if (lookahead == 't') ADVANCE(70);
      END_STATE();
    case 40:
      ACCEPT_TOKEN(anon_sym_or);
      END_STATE();
    case 41:
      if (lookahead == 'd') ADVANCE(71);
      if (lookahead == 's') ADVANCE(72);
      if (lookahead == 't') ADVANCE(73);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_ri);
      END_STATE();
    case 43:
      if (lookahead == 'l') ADVANCE(74);
      END_STATE();
    case 44:
      if (lookahead == 'p') ADVANCE(75);
      END_STATE();
    case 45:
      if (lookahead == 'e') ADVANCE(76);
      END_STATE();
    case 46:
      if (lookahead == 'u') ADVANCE(77);
      END_STATE();
    case 47:
      if (lookahead == 'd') ADVANCE(78);
      if (lookahead == 'l') ADVANCE(79);
      if (lookahead == 't') ADVANCE(80);
      END_STATE();
    case 48:
      if (lookahead == 'e') ADVANCE(81);
      if (lookahead == 'i') ADVANCE(82);
      END_STATE();
    case 49:
      if (lookahead == 'e') ADVANCE(83);
      END_STATE();
    case 50:
      if (lookahead == 'f') ADVANCE(34);
      if (lookahead == 'n') ADVANCE(35);
      END_STATE();
    case 51:
      if (lookahead == 'e') ADVANCE(41);
      END_STATE();
    case 52:
      if (lookahead == 'N') ADVANCE(84);
      END_STATE();
    case 53:
      if (lookahead == 'I') ADVANCE(85);
      END_STATE();
    case 54:
      if (lookahead == 'I') ADVANCE(86);
      END_STATE();
    case 55:
      if (lookahead == 'a') ADVANCE(87);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_and);
      END_STATE();
    case 57:
      if (lookahead == 'i') ADVANCE(88);
      END_STATE();
    case 58:
      if (lookahead == 'a') ADVANCE(89);
      END_STATE();
    case 59:
      if (lookahead == 'e') ADVANCE(90);
      END_STATE();
    case 60:
      if (lookahead == 's') ADVANCE(91);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_def);
      END_STATE();
    case 62:
      if (lookahead == 'e') ADVANCE(92);
      if (lookahead == 'i') ADVANCE(93);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_end);
      END_STATE();
    case 64:
      if (lookahead == 'u') ADVANCE(94);
      END_STATE();
    case 65:
      if (lookahead == 's') ADVANCE(95);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_for);
      END_STATE();
    case 67:
      if (lookahead == 'u') ADVANCE(96);
      END_STATE();
    case 68:
      if (lookahead == 't') ADVANCE(97);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_nil);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_not);
      END_STATE();
    case 71:
      if (lookahead == 'o') ADVANCE(98);
      END_STATE();
    case 72:
      if (lookahead == 'c') ADVANCE(99);
      END_STATE();
    case 73:
      if (lookahead == 'r') ADVANCE(100);
      if (lookahead == 'u') ADVANCE(101);
      END_STATE();
    case 74:
      if (lookahead == 'f') ADVANCE(102);
      END_STATE();
    case 75:
      if (lookahead == 'e') ADVANCE(103);
      END_STATE();
    case 76:
      if (lookahead == 'n') ADVANCE(104);
      END_STATE();
    case 77:
      if (lookahead == 'e') ADVANCE(105);
      END_STATE();
    case 78:
      if (lookahead == 'e') ADVANCE(106);
      END_STATE();
    case 79:
      if (lookahead == 'e') ADVANCE(107);
      END_STATE();
    case 80:
      if (lookahead == 'i') ADVANCE(108);
      END_STATE();
    case 81:
      if (lookahead == 'n') ADVANCE(109);
      END_STATE();
    case 82:
      if (lookahead == 'l') ADVANCE(110);
      END_STATE();
    case 83:
      if (lookahead == 'l') ADVANCE(111);
      END_STATE();
    case 84:
      if (lookahead == 'C') ADVANCE(112);
      if (lookahead == 'D') ADVANCE(113);
      END_STATE();
    case 85:
      if (lookahead == 'L') ADVANCE(114);
      END_STATE();
    case 86:
      if (lookahead == 'N') ADVANCE(115);
      END_STATE();
    case 87:
      if (lookahead == 's') ADVANCE(116);
      END_STATE();
    case 88:
      if (lookahead == 'n') ADVANCE(117);
      END_STATE();
    case 89:
      if (lookahead == 'k') ADVANCE(118);
      END_STATE();
    case 90:
      ACCEPT_TOKEN(anon_sym_case);
      END_STATE();
    case 91:
      if (lookahead == 's') ADVANCE(119);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 93:
      if (lookahead == 'f') ADVANCE(120);
      END_STATE();
    case 94:
      if (lookahead == 'r') ADVANCE(121);
      END_STATE();
    case 95:
      if (lookahead == 'e') ADVANCE(122);
      END_STATE();
    case 96:
      if (lookahead == 'l') ADVANCE(123);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(anon_sym_next);
      END_STATE();
    case 98:
      ACCEPT_TOKEN(anon_sym_redo);
      END_STATE();
    case 99:
      if (lookahead == 'u') ADVANCE(124);
      END_STATE();
    case 100:
      if (lookahead == 'y') ADVANCE(125);
      END_STATE();
    case 101:
      if (lookahead == 'r') ADVANCE(126);
      END_STATE();
    case 102:
      ACCEPT_TOKEN(sym_self);
      END_STATE();
    case 103:
      if (lookahead == 'r') ADVANCE(127);
      END_STATE();
    case 104:
      ACCEPT_TOKEN(anon_sym_then);
      END_STATE();
    case 105:
      ACCEPT_TOKEN(sym_true);
      END_STATE();
    case 106:
      if (lookahead == 'f') ADVANCE(128);
      END_STATE();
    case 107:
      if (lookahead == 's') ADVANCE(129);
      END_STATE();
    case 108:
      if (lookahead == 'l') ADVANCE(130);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(anon_sym_when);
      END_STATE();
    case 110:
      if (lookahead == 'e') ADVANCE(131);
      END_STATE();
    case 111:
      if (lookahead == 'd') ADVANCE(132);
      END_STATE();
    case 112:
      if (lookahead == 'O') ADVANCE(133);
      END_STATE();
    case 113:
      if (lookahead == '_') ADVANCE(134);
      END_STATE();
    case 114:
      if (lookahead == 'E') ADVANCE(135);
      END_STATE();
    case 115:
      if (lookahead == 'E') ADVANCE(136);
      END_STATE();
    case 116:
      ACCEPT_TOKEN(anon_sym_alias);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(anon_sym_begin);
      END_STATE();
    case 118:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_class);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_elsif);
      END_STATE();
    case 121:
      if (lookahead == 'e') ADVANCE(137);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(sym_false);
      END_STATE();
    case 123:
      if (lookahead == 'e') ADVANCE(138);
      END_STATE();
    case 124:
      if (lookahead == 'e') ADVANCE(139);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_retry);
      END_STATE();
    case 126:
      if (lookahead == 'n') ADVANCE(140);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(sym_super);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_undef);
      END_STATE();
    case 129:
      if (lookahead == 's') ADVANCE(141);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_until);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_while);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(anon_sym_yield);
      END_STATE();
    case 133:
      if (lookahead == 'D') ADVANCE(142);
      END_STATE();
    case 134:
      if (lookahead == '_') ADVANCE(143);
      END_STATE();
    case 135:
      if (lookahead == '_') ADVANCE(144);
      END_STATE();
    case 136:
      if (lookahead == '_') ADVANCE(145);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(anon_sym_ensure);
      END_STATE();
    case 138:
      ACCEPT_TOKEN(anon_sym_module);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(anon_sym_rescue);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(anon_sym_unless);
      END_STATE();
    case 142:
      if (lookahead == 'I') ADVANCE(146);
      END_STATE();
    case 143:
      ACCEPT_TOKEN(aux_sym_program_token1);
      END_STATE();
    case 144:
      if (lookahead == '_') ADVANCE(147);
      END_STATE();
    case 145:
      if (lookahead == '_') ADVANCE(148);
      END_STATE();
    case 146:
      if (lookahead == 'N') ADVANCE(149);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(sym_file);
      END_STATE();
    case 148:
      ACCEPT_TOKEN(sym_line);
      END_STATE();
    case 149:
      if (lookahead == 'G') ADVANCE(150);
      END_STATE();
    case 150:
      if (lookahead == '_') ADVANCE(151);
      END_STATE();
    case 151:
      if (lookahead == '_') ADVANCE(152);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(sym_encoding);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 151, .external_lex_state = 2},
  [2] = {.lex_state = 144, .external_lex_state = 3},
  [3] = {.lex_state = 144, .external_lex_state = 3},
  [4] = {.lex_state = 144, .external_lex_state = 3},
  [5] = {.lex_state = 144, .external_lex_state = 3},
  [6] = {.lex_state = 144, .external_lex_state = 3},
  [7] = {.lex_state = 144, .external_lex_state = 3},
  [8] = {.lex_state = 144, .external_lex_state = 3},
  [9] = {.lex_state = 144, .external_lex_state = 3},
  [10] = {.lex_state = 144, .external_lex_state = 3},
  [11] = {.lex_state = 144, .external_lex_state = 3},
  [12] = {.lex_state = 144, .external_lex_state = 3},
  [13] = {.lex_state = 144, .external_lex_state = 3},
  [14] = {.lex_state = 144, .external_lex_state = 3},
  [15] = {.lex_state = 144, .external_lex_state = 3},
  [16] = {.lex_state = 144, .external_lex_state = 3},
  [17] = {.lex_state = 144, .external_lex_state = 3},
  [18] = {.lex_state = 144, .external_lex_state = 3},
  [19] = {.lex_state = 144, .external_lex_state = 3},
  [20] = {.lex_state = 144, .external_lex_state = 3},
  [21] = {.lex_state = 144, .external_lex_state = 3},
  [22] = {.lex_state = 144, .external_lex_state = 3},
  [23] = {.lex_state = 144, .external_lex_state = 3},
  [24] = {.lex_state = 144, .external_lex_state = 4},
  [25] = {.lex_state = 144, .external_lex_state = 4},
  [26] = {.lex_state = 144, .external_lex_state = 3},
  [27] = {.lex_state = 144, .external_lex_state = 4},
  [28] = {.lex_state = 144, .external_lex_state = 3},
  [29] = {.lex_state = 144, .external_lex_state = 3},
  [30] = {.lex_state = 144, .external_lex_state = 3},
  [31] = {.lex_state = 144, .external_lex_state = 3},
  [32] = {.lex_state = 144, .external_lex_state = 3},
  [33] = {.lex_state = 144, .external_lex_state = 4},
  [34] = {.lex_state = 144, .external_lex_state = 4},
  [35] = {.lex_state = 144, .external_lex_state = 4},
  [36] = {.lex_state = 144, .external_lex_state = 4},
  [37] = {.lex_state = 144, .external_lex_state = 4},
  [38] = {.lex_state = 144, .external_lex_state = 4},
  [39] = {.lex_state = 144, .external_lex_state = 4},
  [40] = {.lex_state = 144, .external_lex_state = 4},
  [41] = {.lex_state = 144, .external_lex_state = 4},
  [42] = {.lex_state = 146, .external_lex_state = 3},
  [43] = {.lex_state = 146, .external_lex_state = 3},
  [44] = {.lex_state = 146, .external_lex_state = 3},
  [45] = {.lex_state = 146, .external_lex_state = 3},
  [46] = {.lex_state = 146, .external_lex_state = 3},
  [47] = {.lex_state = 146, .external_lex_state = 3},
  [48] = {.lex_state = 146, .external_lex_state = 3},
  [49] = {.lex_state = 146, .external_lex_state = 3},
  [50] = {.lex_state = 146, .external_lex_state = 3},
  [51] = {.lex_state = 146, .external_lex_state = 3},
  [52] = {.lex_state = 146, .external_lex_state = 3},
  [53] = {.lex_state = 146, .external_lex_state = 3},
  [54] = {.lex_state = 146, .external_lex_state = 3},
  [55] = {.lex_state = 146, .external_lex_state = 3},
  [56] = {.lex_state = 146, .external_lex_state = 3},
  [57] = {.lex_state = 146, .external_lex_state = 3},
  [58] = {.lex_state = 146, .external_lex_state = 3},
  [59] = {.lex_state = 146, .external_lex_state = 3},
  [60] = {.lex_state = 146, .external_lex_state = 3},
  [61] = {.lex_state = 146, .external_lex_state = 3},
  [62] = {.lex_state = 146, .external_lex_state = 3},
  [63] = {.lex_state = 146, .external_lex_state = 3},
  [64] = {.lex_state = 146, .external_lex_state = 3},
  [65] = {.lex_state = 146, .external_lex_state = 4},
  [66] = {.lex_state = 146, .external_lex_state = 4},
  [67] = {.lex_state = 146, .external_lex_state = 3},
  [68] = {.lex_state = 146, .external_lex_state = 3},
  [69] = {.lex_state = 146, .external_lex_state = 3},
  [70] = {.lex_state = 146, .external_lex_state = 4},
  [71] = {.lex_state = 146, .external_lex_state = 3},
  [72] = {.lex_state = 146, .external_lex_state = 3},
  [73] = {.lex_state = 146, .external_lex_state = 3},
  [74] = {.lex_state = 146, .external_lex_state = 3},
  [75] = {.lex_state = 146, .external_lex_state = 3},
  [76] = {.lex_state = 146, .external_lex_state = 4},
  [77] = {.lex_state = 146, .external_lex_state = 3},
  [78] = {.lex_state = 146, .external_lex_state = 3},
  [79] = {.lex_state = 146, .external_lex_state = 3},
  [80] = {.lex_state = 146, .external_lex_state = 3},
  [81] = {.lex_state = 146, .external_lex_state = 3},
  [82] = {.lex_state = 146, .external_lex_state = 3},
  [83] = {.lex_state = 146, .external_lex_state = 3},
  [84] = {.lex_state = 146, .external_lex_state = 3},
  [85] = {.lex_state = 146, .external_lex_state = 3},
  [86] = {.lex_state = 146, .external_lex_state = 3},
  [87] = {.lex_state = 146, .external_lex_state = 3},
  [88] = {.lex_state = 146, .external_lex_state = 3},
  [89] = {.lex_state = 146, .external_lex_state = 3},
  [90] = {.lex_state = 146, .external_lex_state = 3},
  [91] = {.lex_state = 146, .external_lex_state = 4},
  [92] = {.lex_state = 146, .external_lex_state = 4},
  [93] = {.lex_state = 146, .external_lex_state = 4},
  [94] = {.lex_state = 146, .external_lex_state = 4},
  [95] = {.lex_state = 146, .external_lex_state = 4},
  [96] = {.lex_state = 146, .external_lex_state = 3},
  [97] = {.lex_state = 146, .external_lex_state = 4},
  [98] = {.lex_state = 146, .external_lex_state = 3},
  [99] = {.lex_state = 146, .external_lex_state = 4},
  [100] = {.lex_state = 146, .external_lex_state = 4},
  [101] = {.lex_state = 146, .external_lex_state = 3},
  [102] = {.lex_state = 146, .external_lex_state = 3},
  [103] = {.lex_state = 146, .external_lex_state = 4},
  [104] = {.lex_state = 146, .external_lex_state = 4},
  [105] = {.lex_state = 146, .external_lex_state = 3},
  [106] = {.lex_state = 146, .external_lex_state = 4},
  [107] = {.lex_state = 146, .external_lex_state = 4},
  [108] = {.lex_state = 146, .external_lex_state = 4},
  [109] = {.lex_state = 146, .external_lex_state = 4},
  [110] = {.lex_state = 146, .external_lex_state = 4},
  [111] = {.lex_state = 146, .external_lex_state = 4},
  [112] = {.lex_state = 146, .external_lex_state = 4},
  [113] = {.lex_state = 146, .external_lex_state = 4},
  [114] = {.lex_state = 151, .external_lex_state = 2},
  [115] = {.lex_state = 151, .external_lex_state = 2},
  [116] = {.lex_state = 151, .external_lex_state = 2},
  [117] = {.lex_state = 151, .external_lex_state = 2},
  [118] = {.lex_state = 152, .external_lex_state = 5},
  [119] = {.lex_state = 152, .external_lex_state = 5},
  [120] = {.lex_state = 152, .external_lex_state = 5},
  [121] = {.lex_state = 152, .external_lex_state = 5},
  [122] = {.lex_state = 152, .external_lex_state = 5},
  [123] = {.lex_state = 38, .external_lex_state = 6},
  [124] = {.lex_state = 38, .external_lex_state = 6},
  [125] = {.lex_state = 38, .external_lex_state = 6},
  [126] = {.lex_state = 40, .external_lex_state = 6},
  [127] = {.lex_state = 38, .external_lex_state = 6},
  [128] = {.lex_state = 40, .external_lex_state = 6},
  [129] = {.lex_state = 38, .external_lex_state = 6},
  [130] = {.lex_state = 40, .external_lex_state = 6},
  [131] = {.lex_state = 38, .external_lex_state = 6},
  [132] = {.lex_state = 40, .external_lex_state = 6},
  [133] = {.lex_state = 40, .external_lex_state = 6},
  [134] = {.lex_state = 40, .external_lex_state = 6},
  [135] = {.lex_state = 41, .external_lex_state = 7},
  [136] = {.lex_state = 41, .external_lex_state = 7},
  [137] = {.lex_state = 41, .external_lex_state = 7},
  [138] = {.lex_state = 151, .external_lex_state = 7},
  [139] = {.lex_state = 41, .external_lex_state = 7},
  [140] = {.lex_state = 41, .external_lex_state = 7},
  [141] = {.lex_state = 151, .external_lex_state = 7},
  [142] = {.lex_state = 151, .external_lex_state = 7},
  [143] = {.lex_state = 41, .external_lex_state = 7},
  [144] = {.lex_state = 151, .external_lex_state = 7},
  [145] = {.lex_state = 41, .external_lex_state = 7},
  [146] = {.lex_state = 41, .external_lex_state = 7},
  [147] = {.lex_state = 151, .external_lex_state = 7},
  [148] = {.lex_state = 151, .external_lex_state = 7},
  [149] = {.lex_state = 41, .external_lex_state = 7},
  [150] = {.lex_state = 41, .external_lex_state = 7},
  [151] = {.lex_state = 41, .external_lex_state = 7},
  [152] = {.lex_state = 41, .external_lex_state = 7},
  [153] = {.lex_state = 41, .external_lex_state = 7},
  [154] = {.lex_state = 41, .external_lex_state = 7},
  [155] = {.lex_state = 151, .external_lex_state = 7},
  [156] = {.lex_state = 41, .external_lex_state = 7},
  [157] = {.lex_state = 41, .external_lex_state = 7},
  [158] = {.lex_state = 151, .external_lex_state = 7},
  [159] = {.lex_state = 41, .external_lex_state = 7},
  [160] = {.lex_state = 41, .external_lex_state = 7},
  [161] = {.lex_state = 151, .external_lex_state = 7},
  [162] = {.lex_state = 41, .external_lex_state = 7},
  [163] = {.lex_state = 41, .external_lex_state = 7},
  [164] = {.lex_state = 151, .external_lex_state = 7},
  [165] = {.lex_state = 41, .external_lex_state = 7},
  [166] = {.lex_state = 41, .external_lex_state = 7},
  [167] = {.lex_state = 41, .external_lex_state = 7},
  [168] = {.lex_state = 41, .external_lex_state = 7},
  [169] = {.lex_state = 151, .external_lex_state = 2},
  [170] = {.lex_state = 151, .external_lex_state = 2},
  [171] = {.lex_state = 151, .external_lex_state = 7},
  [172] = {.lex_state = 151, .external_lex_state = 2},
  [173] = {.lex_state = 151, .external_lex_state = 7},
  [174] = {.lex_state = 151, .external_lex_state = 7},
  [175] = {.lex_state = 151, .external_lex_state = 2},
  [176] = {.lex_state = 151, .external_lex_state = 7},
  [177] = {.lex_state = 151, .external_lex_state = 7},
  [178] = {.lex_state = 151, .external_lex_state = 7},
  [179] = {.lex_state = 151, .external_lex_state = 2},
  [180] = {.lex_state = 151, .external_lex_state = 7},
  [181] = {.lex_state = 151, .external_lex_state = 7},
  [182] = {.lex_state = 151, .external_lex_state = 7},
  [183] = {.lex_state = 151, .external_lex_state = 7},
  [184] = {.lex_state = 151, .external_lex_state = 2},
  [185] = {.lex_state = 151, .external_lex_state = 7},
  [186] = {.lex_state = 151, .external_lex_state = 7},
  [187] = {.lex_state = 151, .external_lex_state = 2},
  [188] = {.lex_state = 151, .external_lex_state = 7},
  [189] = {.lex_state = 151, .external_lex_state = 7},
  [190] = {.lex_state = 151, .external_lex_state = 2},
  [191] = {.lex_state = 151, .external_lex_state = 7},
  [192] = {.lex_state = 151, .external_lex_state = 7},
  [193] = {.lex_state = 151, .external_lex_state = 7},
  [194] = {.lex_state = 151, .external_lex_state = 7},
  [195] = {.lex_state = 151, .external_lex_state = 7},
  [196] = {.lex_state = 151, .external_lex_state = 2},
  [197] = {.lex_state = 151, .external_lex_state = 7},
  [198] = {.lex_state = 151, .external_lex_state = 2},
  [199] = {.lex_state = 151, .external_lex_state = 7},
  [200] = {.lex_state = 149, .external_lex_state = 8},
  [201] = {.lex_state = 151, .external_lex_state = 7},
  [202] = {.lex_state = 149, .external_lex_state = 8},
  [203] = {.lex_state = 151, .external_lex_state = 7},
  [204] = {.lex_state = 149, .external_lex_state = 8},
  [205] = {.lex_state = 151, .external_lex_state = 2},
  [206] = {.lex_state = 151, .external_lex_state = 2},
  [207] = {.lex_state = 151, .external_lex_state = 2},
  [208] = {.lex_state = 151, .external_lex_state = 2},
  [209] = {.lex_state = 151, .external_lex_state = 2},
  [210] = {.lex_state = 151, .external_lex_state = 2},
  [211] = {.lex_state = 151, .external_lex_state = 2},
  [212] = {.lex_state = 151, .external_lex_state = 2},
  [213] = {.lex_state = 151, .external_lex_state = 2},
  [214] = {.lex_state = 151, .external_lex_state = 2},
  [215] = {.lex_state = 151, .external_lex_state = 2},
  [216] = {.lex_state = 151, .external_lex_state = 2},
  [217] = {.lex_state = 151, .external_lex_state = 2},
  [218] = {.lex_state = 151, .external_lex_state = 2},
  [219] = {.lex_state = 151, .external_lex_state = 2},
  [220] = {.lex_state = 151, .external_lex_state = 2},
  [221] = {.lex_state = 149, .external_lex_state = 8},
  [222] = {.lex_state = 151, .external_lex_state = 2},
  [223] = {.lex_state = 151, .external_lex_state = 2},
  [224] = {.lex_state = 151, .external_lex_state = 2},
  [225] = {.lex_state = 151, .external_lex_state = 2},
  [226] = {.lex_state = 151, .external_lex_state = 2},
  [227] = {.lex_state = 151, .external_lex_state = 2},
  [228] = {.lex_state = 151, .external_lex_state = 2},
  [229] = {.lex_state = 151, .external_lex_state = 2},
  [230] = {.lex_state = 151, .external_lex_state = 2},
  [231] = {.lex_state = 151, .external_lex_state = 2},
  [232] = {.lex_state = 151, .external_lex_state = 2},
  [233] = {.lex_state = 151, .external_lex_state = 2},
  [234] = {.lex_state = 151, .external_lex_state = 2},
  [235] = {.lex_state = 151, .external_lex_state = 2},
  [236] = {.lex_state = 151, .external_lex_state = 2},
  [237] = {.lex_state = 151, .external_lex_state = 2},
  [238] = {.lex_state = 151, .external_lex_state = 2},
  [239] = {.lex_state = 151, .external_lex_state = 2},
  [240] = {.lex_state = 151, .external_lex_state = 2},
  [241] = {.lex_state = 151, .external_lex_state = 2},
  [242] = {.lex_state = 151, .external_lex_state = 2},
  [243] = {.lex_state = 151, .external_lex_state = 2},
  [244] = {.lex_state = 151, .external_lex_state = 2},
  [245] = {.lex_state = 149, .external_lex_state = 8},
  [246] = {.lex_state = 151, .external_lex_state = 2},
  [247] = {.lex_state = 151, .external_lex_state = 2},
  [248] = {.lex_state = 152, .external_lex_state = 9},
  [249] = {.lex_state = 151, .external_lex_state = 10},
  [250] = {.lex_state = 151, .external_lex_state = 10},
  [251] = {.lex_state = 151, .external_lex_state = 10},
  [252] = {.lex_state = 151, .external_lex_state = 2},
  [253] = {.lex_state = 151, .external_lex_state = 10},
  [254] = {.lex_state = 151, .external_lex_state = 10},
  [255] = {.lex_state = 151, .external_lex_state = 10},
  [256] = {.lex_state = 151, .external_lex_state = 10},
  [257] = {.lex_state = 151, .external_lex_state = 2},
  [258] = {.lex_state = 152, .external_lex_state = 9},
  [259] = {.lex_state = 151, .external_lex_state = 10},
  [260] = {.lex_state = 151, .external_lex_state = 2},
  [261] = {.lex_state = 150, .external_lex_state = 11},
  [262] = {.lex_state = 151, .external_lex_state = 2},
  [263] = {.lex_state = 150, .external_lex_state = 11},
  [264] = {.lex_state = 151, .external_lex_state = 2},
  [265] = {.lex_state = 151, .external_lex_state = 2},
  [266] = {.lex_state = 151, .external_lex_state = 2},
  [267] = {.lex_state = 151, .external_lex_state = 2},
  [268] = {.lex_state = 151, .external_lex_state = 7},
  [269] = {.lex_state = 151, .external_lex_state = 2},
  [270] = {.lex_state = 150, .external_lex_state = 11},
  [271] = {.lex_state = 150, .external_lex_state = 11},
  [272] = {.lex_state = 151, .external_lex_state = 2},
  [273] = {.lex_state = 151, .external_lex_state = 2},
  [274] = {.lex_state = 149, .external_lex_state = 8},
  [275] = {.lex_state = 151, .external_lex_state = 2},
  [276] = {.lex_state = 151, .external_lex_state = 7},
  [277] = {.lex_state = 151, .external_lex_state = 7},
  [278] = {.lex_state = 149, .external_lex_state = 8},
  [279] = {.lex_state = 150, .external_lex_state = 11},
  [280] = {.lex_state = 151, .external_lex_state = 2},
  [281] = {.lex_state = 151, .external_lex_state = 2},
  [282] = {.lex_state = 151, .external_lex_state = 2},
  [283] = {.lex_state = 151, .external_lex_state = 7},
  [284] = {.lex_state = 151, .external_lex_state = 2},
  [285] = {.lex_state = 149, .external_lex_state = 8},
  [286] = {.lex_state = 149, .external_lex_state = 8},
  [287] = {.lex_state = 151, .external_lex_state = 2},
  [288] = {.lex_state = 149, .external_lex_state = 12},
  [289] = {.lex_state = 151, .external_lex_state = 2},
  [290] = {.lex_state = 151, .external_lex_state = 2},
  [291] = {.lex_state = 151, .external_lex_state = 2},
  [292] = {.lex_state = 151, .external_lex_state = 2},
  [293] = {.lex_state = 151, .external_lex_state = 2},
  [294] = {.lex_state = 151, .external_lex_state = 2},
  [295] = {.lex_state = 151, .external_lex_state = 2},
  [296] = {.lex_state = 149, .external_lex_state = 12},
  [297] = {.lex_state = 151, .external_lex_state = 2},
  [298] = {.lex_state = 151, .external_lex_state = 2},
  [299] = {.lex_state = 151, .external_lex_state = 2},
  [300] = {.lex_state = 151, .external_lex_state = 2},
  [301] = {.lex_state = 151, .external_lex_state = 2},
  [302] = {.lex_state = 151, .external_lex_state = 2},
  [303] = {.lex_state = 151, .external_lex_state = 2},
  [304] = {.lex_state = 149, .external_lex_state = 8},
  [305] = {.lex_state = 149, .external_lex_state = 12},
  [306] = {.lex_state = 149, .external_lex_state = 12},
  [307] = {.lex_state = 149, .external_lex_state = 8},
  [308] = {.lex_state = 149, .external_lex_state = 12},
  [309] = {.lex_state = 149, .external_lex_state = 12},
  [310] = {.lex_state = 149, .external_lex_state = 8},
  [311] = {.lex_state = 149, .external_lex_state = 8},
  [312] = {.lex_state = 149, .external_lex_state = 8},
  [313] = {.lex_state = 151, .external_lex_state = 2},
  [314] = {.lex_state = 149, .external_lex_state = 8},
  [315] = {.lex_state = 151, .external_lex_state = 2},
  [316] = {.lex_state = 152, .external_lex_state = 9},
  [317] = {.lex_state = 151, .external_lex_state = 2},
  [318] = {.lex_state = 151, .external_lex_state = 2},
  [319] = {.lex_state = 151, .external_lex_state = 2},
  [320] = {.lex_state = 151, .external_lex_state = 2},
  [321] = {.lex_state = 151, .external_lex_state = 2},
  [322] = {.lex_state = 149, .external_lex_state = 8},
  [323] = {.lex_state = 151, .external_lex_state = 2},
  [324] = {.lex_state = 151, .external_lex_state = 2},
  [325] = {.lex_state = 149, .external_lex_state = 8},
  [326] = {.lex_state = 149, .external_lex_state = 12},
  [327] = {.lex_state = 151, .external_lex_state = 2},
  [328] = {.lex_state = 149, .external_lex_state = 12},
  [329] = {.lex_state = 151, .external_lex_state = 2},
  [330] = {.lex_state = 42, .external_lex_state = 2},
  [331] = {.lex_state = 151, .external_lex_state = 2},
  [332] = {.lex_state = 42, .external_lex_state = 2},
  [333] = {.lex_state = 42, .external_lex_state = 2},
  [334] = {.lex_state = 151, .external_lex_state = 2},
  [335] = {.lex_state = 42, .external_lex_state = 2},
  [336] = {.lex_state = 42, .external_lex_state = 2},
  [337] = {.lex_state = 42, .external_lex_state = 2},
  [338] = {.lex_state = 152, .external_lex_state = 9},
  [339] = {.lex_state = 42, .external_lex_state = 2},
  [340] = {.lex_state = 42, .external_lex_state = 2},
  [341] = {.lex_state = 42, .external_lex_state = 2},
  [342] = {.lex_state = 42, .external_lex_state = 2},
  [343] = {.lex_state = 150, .external_lex_state = 11},
  [344] = {.lex_state = 42, .external_lex_state = 2},
  [345] = {.lex_state = 42, .external_lex_state = 2},
  [346] = {.lex_state = 151, .external_lex_state = 2},
  [347] = {.lex_state = 149, .external_lex_state = 8},
  [348] = {.lex_state = 151, .external_lex_state = 2},
  [349] = {.lex_state = 151, .external_lex_state = 2},
  [350] = {.lex_state = 151, .external_lex_state = 2},
  [351] = {.lex_state = 151, .external_lex_state = 2},
  [352] = {.lex_state = 151, .external_lex_state = 2},
  [353] = {.lex_state = 151, .external_lex_state = 2},
  [354] = {.lex_state = 151, .external_lex_state = 2},
  [355] = {.lex_state = 151, .external_lex_state = 2},
  [356] = {.lex_state = 152, .external_lex_state = 9},
  [357] = {.lex_state = 151, .external_lex_state = 2},
  [358] = {.lex_state = 42, .external_lex_state = 2},
  [359] = {.lex_state = 150, .external_lex_state = 11},
  [360] = {.lex_state = 151, .external_lex_state = 2},
  [361] = {.lex_state = 152, .external_lex_state = 9},
  [362] = {.lex_state = 150, .external_lex_state = 11},
  [363] = {.lex_state = 152, .external_lex_state = 5},
  [364] = {.lex_state = 149, .external_lex_state = 8},
  [365] = {.lex_state = 150, .external_lex_state = 13},
  [366] = {.lex_state = 150, .external_lex_state = 13},
  [367] = {.lex_state = 149, .external_lex_state = 12},
  [368] = {.lex_state = 150, .external_lex_state = 11},
  [369] = {.lex_state = 152, .external_lex_state = 9},
  [370] = {.lex_state = 152, .external_lex_state = 9},
  [371] = {.lex_state = 149, .external_lex_state = 8},
  [372] = {.lex_state = 149, .external_lex_state = 12},
  [373] = {.lex_state = 150, .external_lex_state = 13},
  [374] = {.lex_state = 149, .external_lex_state = 12},
  [375] = {.lex_state = 149, .external_lex_state = 12},
  [376] = {.lex_state = 150, .external_lex_state = 13},
  [377] = {.lex_state = 150, .external_lex_state = 11},
  [378] = {.lex_state = 152, .external_lex_state = 5},
  [379] = {.lex_state = 150, .external_lex_state = 13},
  [380] = {.lex_state = 151, .external_lex_state = 2},
  [381] = {.lex_state = 152, .external_lex_state = 9},
  [382] = {.lex_state = 149, .external_lex_state = 8},
  [383] = {.lex_state = 151, .external_lex_state = 2},
  [384] = {.lex_state = 150, .external_lex_state = 13},
  [385] = {.lex_state = 150, .external_lex_state = 11},
  [386] = {.lex_state = 151, .external_lex_state = 2},
  [387] = {.lex_state = 150, .external_lex_state = 11},
  [388] = {.lex_state = 150, .external_lex_state = 11},
  [389] = {.lex_state = 150, .external_lex_state = 11},
  [390] = {.lex_state = 152, .external_lex_state = 5},
  [391] = {.lex_state = 150, .external_lex_state = 11},
  [392] = {.lex_state = 150, .external_lex_state = 11},
  [393] = {.lex_state = 150, .external_lex_state = 13},
  [394] = {.lex_state = 149, .external_lex_state = 12},
  [395] = {.lex_state = 149, .external_lex_state = 12},
  [396] = {.lex_state = 150, .external_lex_state = 11},
  [397] = {.lex_state = 150, .external_lex_state = 13},
  [398] = {.lex_state = 152, .external_lex_state = 5},
  [399] = {.lex_state = 149, .external_lex_state = 12},
  [400] = {.lex_state = 149, .external_lex_state = 12},
  [401] = {.lex_state = 150, .external_lex_state = 11},
  [402] = {.lex_state = 151, .external_lex_state = 2},
  [403] = {.lex_state = 151, .external_lex_state = 2},
  [404] = {.lex_state = 151, .external_lex_state = 2},
  [405] = {.lex_state = 150, .external_lex_state = 13},
  [406] = {.lex_state = 151, .external_lex_state = 2},
  [407] = {.lex_state = 151, .external_lex_state = 2},
  [408] = {.lex_state = 150, .external_lex_state = 11},
  [409] = {.lex_state = 151, .external_lex_state = 2},
  [410] = {.lex_state = 151, .external_lex_state = 2},
  [411] = {.lex_state = 151, .external_lex_state = 2},
  [412] = {.lex_state = 151, .external_lex_state = 2},
  [413] = {.lex_state = 151, .external_lex_state = 2},
  [414] = {.lex_state = 151, .external_lex_state = 2},
  [415] = {.lex_state = 151, .external_lex_state = 2},
  [416] = {.lex_state = 150, .external_lex_state = 11},
  [417] = {.lex_state = 151, .external_lex_state = 2},
  [418] = {.lex_state = 150, .external_lex_state = 11},
  [419] = {.lex_state = 150, .external_lex_state = 13},
  [420] = {.lex_state = 150, .external_lex_state = 13},
  [421] = {.lex_state = 150, .external_lex_state = 13},
  [422] = {.lex_state = 152, .external_lex_state = 5},
  [423] = {.lex_state = 152, .external_lex_state = 5},
  [424] = {.lex_state = 152, .external_lex_state = 5},
  [425] = {.lex_state = 152, .external_lex_state = 5},
  [426] = {.lex_state = 150, .external_lex_state = 13},
  [427] = {.lex_state = 152, .external_lex_state = 5},
  [428] = {.lex_state = 152, .external_lex_state = 5},
  [429] = {.lex_state = 152, .external_lex_state = 5},
  [430] = {.lex_state = 152, .external_lex_state = 5},
  [431] = {.lex_state = 152, .external_lex_state = 5},
  [432] = {.lex_state = 152, .external_lex_state = 5},
  [433] = {.lex_state = 152, .external_lex_state = 5},
  [434] = {.lex_state = 150, .external_lex_state = 13},
  [435] = {.lex_state = 152, .external_lex_state = 5},
  [436] = {.lex_state = 152, .external_lex_state = 5},
  [437] = {.lex_state = 150, .external_lex_state = 13},
  [438] = {.lex_state = 152, .external_lex_state = 5},
  [439] = {.lex_state = 152, .external_lex_state = 14},
  [440] = {.lex_state = 152, .external_lex_state = 5},
  [441] = {.lex_state = 152, .external_lex_state = 2},
  [442] = {.lex_state = 150, .external_lex_state = 13},
  [443] = {.lex_state = 152, .external_lex_state = 5},
  [444] = {.lex_state = 152, .external_lex_state = 5},
  [445] = {.lex_state = 152, .external_lex_state = 5},
  [446] = {.lex_state = 152, .external_lex_state = 5},
  [447] = {.lex_state = 152, .external_lex_state = 5},
  [448] = {.lex_state = 152, .external_lex_state = 5},
  [449] = {.lex_state = 152, .external_lex_state = 5},
  [450] = {.lex_state = 152, .external_lex_state = 5},
  [451] = {.lex_state = 152, .external_lex_state = 5},
  [452] = {.lex_state = 152, .external_lex_state = 5},
  [453] = {.lex_state = 152, .external_lex_state = 5},
  [454] = {.lex_state = 152, .external_lex_state = 5},
  [455] = {.lex_state = 152, .external_lex_state = 5},
  [456] = {.lex_state = 152, .external_lex_state = 5},
  [457] = {.lex_state = 152, .external_lex_state = 15},
  [458] = {.lex_state = 152, .external_lex_state = 15},
  [459] = {.lex_state = 152, .external_lex_state = 15},
  [460] = {.lex_state = 152, .external_lex_state = 15},
  [461] = {.lex_state = 152, .external_lex_state = 15},
  [462] = {.lex_state = 152, .external_lex_state = 15},
  [463] = {.lex_state = 152, .external_lex_state = 15},
  [464] = {.lex_state = 152, .external_lex_state = 15},
  [465] = {.lex_state = 152, .external_lex_state = 15},
  [466] = {.lex_state = 152, .external_lex_state = 15},
  [467] = {.lex_state = 152, .external_lex_state = 15},
  [468] = {.lex_state = 152, .external_lex_state = 15},
  [469] = {.lex_state = 152, .external_lex_state = 15},
  [470] = {.lex_state = 152, .external_lex_state = 15},
  [471] = {.lex_state = 152, .external_lex_state = 15},
  [472] = {.lex_state = 152, .external_lex_state = 15},
  [473] = {.lex_state = 152, .external_lex_state = 15},
  [474] = {.lex_state = 152, .external_lex_state = 15},
  [475] = {.lex_state = 152, .external_lex_state = 15},
  [476] = {.lex_state = 152, .external_lex_state = 15},
  [477] = {.lex_state = 152, .external_lex_state = 15},
  [478] = {.lex_state = 152, .external_lex_state = 15},
  [479] = {.lex_state = 152, .external_lex_state = 15},
  [480] = {.lex_state = 152, .external_lex_state = 15},
  [481] = {.lex_state = 152, .external_lex_state = 15},
  [482] = {.lex_state = 152, .external_lex_state = 16},
  [483] = {.lex_state = 152, .external_lex_state = 16},
  [484] = {.lex_state = 152, .external_lex_state = 2},
  [485] = {.lex_state = 152, .external_lex_state = 16},
  [486] = {.lex_state = 152, .external_lex_state = 2},
  [487] = {.lex_state = 152, .external_lex_state = 2},
  [488] = {.lex_state = 152, .external_lex_state = 2},
  [489] = {.lex_state = 152, .external_lex_state = 2},
  [490] = {.lex_state = 152, .external_lex_state = 2},
  [491] = {.lex_state = 152, .external_lex_state = 16},
  [492] = {.lex_state = 152, .external_lex_state = 2},
  [493] = {.lex_state = 152, .external_lex_state = 2},
  [494] = {.lex_state = 152, .external_lex_state = 2},
  [495] = {.lex_state = 152, .external_lex_state = 7},
  [496] = {.lex_state = 152, .external_lex_state = 7},
  [497] = {.lex_state = 152, .external_lex_state = 2},
  [498] = {.lex_state = 152, .external_lex_state = 7},
  [499] = {.lex_state = 152, .external_lex_state = 2},
  [500] = {.lex_state = 152, .external_lex_state = 2},
  [501] = {.lex_state = 152, .external_lex_state = 2},
  [502] = {.lex_state = 152, .external_lex_state = 2},
  [503] = {.lex_state = 152, .external_lex_state = 2},
  [504] = {.lex_state = 152, .external_lex_state = 2},
  [505] = {.lex_state = 152, .external_lex_state = 2},
  [506] = {.lex_state = 152, .external_lex_state = 2},
  [507] = {.lex_state = 152, .external_lex_state = 2},
  [508] = {.lex_state = 152, .external_lex_state = 2},
  [509] = {.lex_state = 152, .external_lex_state = 2},
  [510] = {.lex_state = 152, .external_lex_state = 2},
  [511] = {.lex_state = 152, .external_lex_state = 2},
  [512] = {.lex_state = 152, .external_lex_state = 7},
  [513] = {.lex_state = 152, .external_lex_state = 2},
  [514] = {.lex_state = 152, .external_lex_state = 2},
  [515] = {.lex_state = 152, .external_lex_state = 7},
  [516] = {.lex_state = 152, .external_lex_state = 7},
  [517] = {.lex_state = 152, .external_lex_state = 7},
  [518] = {.lex_state = 152, .external_lex_state = 7},
  [519] = {.lex_state = 152, .external_lex_state = 2},
  [520] = {.lex_state = 152, .external_lex_state = 2},
  [521] = {.lex_state = 152, .external_lex_state = 2},
  [522] = {.lex_state = 152, .external_lex_state = 2},
  [523] = {.lex_state = 152, .external_lex_state = 2},
  [524] = {.lex_state = 152, .external_lex_state = 2},
  [525] = {.lex_state = 152, .external_lex_state = 2},
  [526] = {.lex_state = 152, .external_lex_state = 2},
  [527] = {.lex_state = 152, .external_lex_state = 2},
  [528] = {.lex_state = 152, .external_lex_state = 2},
  [529] = {.lex_state = 152, .external_lex_state = 2},
  [530] = {.lex_state = 152, .external_lex_state = 2},
  [531] = {.lex_state = 152, .external_lex_state = 2},
  [532] = {.lex_state = 152, .external_lex_state = 2},
  [533] = {.lex_state = 152, .external_lex_state = 2},
  [534] = {.lex_state = 152, .external_lex_state = 2},
  [535] = {.lex_state = 152, .external_lex_state = 2},
  [536] = {.lex_state = 152, .external_lex_state = 2},
  [537] = {.lex_state = 152, .external_lex_state = 2},
  [538] = {.lex_state = 152, .external_lex_state = 2},
  [539] = {.lex_state = 152, .external_lex_state = 2},
  [540] = {.lex_state = 152, .external_lex_state = 2},
  [541] = {.lex_state = 152, .external_lex_state = 2},
  [542] = {.lex_state = 152, .external_lex_state = 2},
  [543] = {.lex_state = 152, .external_lex_state = 2},
  [544] = {.lex_state = 152, .external_lex_state = 2},
  [545] = {.lex_state = 152, .external_lex_state = 2},
  [546] = {.lex_state = 152, .external_lex_state = 2},
  [547] = {.lex_state = 152, .external_lex_state = 2},
  [548] = {.lex_state = 152, .external_lex_state = 2},
  [549] = {.lex_state = 152, .external_lex_state = 2},
  [550] = {.lex_state = 152, .external_lex_state = 2},
  [551] = {.lex_state = 152, .external_lex_state = 2},
  [552] = {.lex_state = 152, .external_lex_state = 2},
  [553] = {.lex_state = 152, .external_lex_state = 2},
  [554] = {.lex_state = 152, .external_lex_state = 2},
  [555] = {.lex_state = 152, .external_lex_state = 2},
  [556] = {.lex_state = 152, .external_lex_state = 2},
  [557] = {.lex_state = 152, .external_lex_state = 2},
  [558] = {.lex_state = 152, .external_lex_state = 2},
  [559] = {.lex_state = 152, .external_lex_state = 2},
  [560] = {.lex_state = 152, .external_lex_state = 2},
  [561] = {.lex_state = 152, .external_lex_state = 2},
  [562] = {.lex_state = 152, .external_lex_state = 2},
  [563] = {.lex_state = 152, .external_lex_state = 2},
  [564] = {.lex_state = 152, .external_lex_state = 2},
  [565] = {.lex_state = 152, .external_lex_state = 2},
  [566] = {.lex_state = 152, .external_lex_state = 2},
  [567] = {.lex_state = 152, .external_lex_state = 2},
  [568] = {.lex_state = 152, .external_lex_state = 2},
  [569] = {.lex_state = 152, .external_lex_state = 2},
  [570] = {.lex_state = 152, .external_lex_state = 2},
  [571] = {.lex_state = 152, .external_lex_state = 2},
  [572] = {.lex_state = 152, .external_lex_state = 2},
  [573] = {.lex_state = 152, .external_lex_state = 2},
  [574] = {.lex_state = 152, .external_lex_state = 2},
  [575] = {.lex_state = 152, .external_lex_state = 2},
  [576] = {.lex_state = 152, .external_lex_state = 2},
  [577] = {.lex_state = 152, .external_lex_state = 2},
  [578] = {.lex_state = 152, .external_lex_state = 2},
  [579] = {.lex_state = 152, .external_lex_state = 2},
  [580] = {.lex_state = 152, .external_lex_state = 2},
  [581] = {.lex_state = 152, .external_lex_state = 2},
  [582] = {.lex_state = 152, .external_lex_state = 2},
  [583] = {.lex_state = 152, .external_lex_state = 2},
  [584] = {.lex_state = 152, .external_lex_state = 2},
  [585] = {.lex_state = 152, .external_lex_state = 2},
  [586] = {.lex_state = 152, .external_lex_state = 2},
  [587] = {.lex_state = 152, .external_lex_state = 2},
  [588] = {.lex_state = 152, .external_lex_state = 2},
  [589] = {.lex_state = 152, .external_lex_state = 2},
  [590] = {.lex_state = 152, .external_lex_state = 2},
  [591] = {.lex_state = 152, .external_lex_state = 2},
  [592] = {.lex_state = 152, .external_lex_state = 2},
  [593] = {.lex_state = 152, .external_lex_state = 2},
  [594] = {.lex_state = 152, .external_lex_state = 2},
  [595] = {.lex_state = 152, .external_lex_state = 2},
  [596] = {.lex_state = 152, .external_lex_state = 2},
  [597] = {.lex_state = 152, .external_lex_state = 2},
  [598] = {.lex_state = 152, .external_lex_state = 2},
  [599] = {.lex_state = 152, .external_lex_state = 2},
  [600] = {.lex_state = 152, .external_lex_state = 2},
  [601] = {.lex_state = 152, .external_lex_state = 2},
  [602] = {.lex_state = 152, .external_lex_state = 2},
  [603] = {.lex_state = 152, .external_lex_state = 2},
  [604] = {.lex_state = 152, .external_lex_state = 2},
  [605] = {.lex_state = 152, .external_lex_state = 2},
  [606] = {.lex_state = 152, .external_lex_state = 2},
  [607] = {.lex_state = 152, .external_lex_state = 2},
  [608] = {.lex_state = 152, .external_lex_state = 2},
  [609] = {.lex_state = 152, .external_lex_state = 2},
  [610] = {.lex_state = 152, .external_lex_state = 2},
  [611] = {.lex_state = 152, .external_lex_state = 2},
  [612] = {.lex_state = 152, .external_lex_state = 2},
  [613] = {.lex_state = 152, .external_lex_state = 2},
  [614] = {.lex_state = 152, .external_lex_state = 2},
  [615] = {.lex_state = 152, .external_lex_state = 2},
  [616] = {.lex_state = 152, .external_lex_state = 2},
  [617] = {.lex_state = 152, .external_lex_state = 2},
  [618] = {.lex_state = 152, .external_lex_state = 2},
  [619] = {.lex_state = 152, .external_lex_state = 2},
  [620] = {.lex_state = 152, .external_lex_state = 2},
  [621] = {.lex_state = 152, .external_lex_state = 2},
  [622] = {.lex_state = 152, .external_lex_state = 2},
  [623] = {.lex_state = 152, .external_lex_state = 2},
  [624] = {.lex_state = 152, .external_lex_state = 2},
  [625] = {.lex_state = 152, .external_lex_state = 2},
  [626] = {.lex_state = 152, .external_lex_state = 2},
  [627] = {.lex_state = 152, .external_lex_state = 2},
  [628] = {.lex_state = 152, .external_lex_state = 2},
  [629] = {.lex_state = 152, .external_lex_state = 2},
  [630] = {.lex_state = 152, .external_lex_state = 2},
  [631] = {.lex_state = 152, .external_lex_state = 2},
  [632] = {.lex_state = 152, .external_lex_state = 2},
  [633] = {.lex_state = 152, .external_lex_state = 2},
  [634] = {.lex_state = 152, .external_lex_state = 2},
  [635] = {.lex_state = 152, .external_lex_state = 2},
  [636] = {.lex_state = 152, .external_lex_state = 2},
  [637] = {.lex_state = 152, .external_lex_state = 2},
  [638] = {.lex_state = 152, .external_lex_state = 2},
  [639] = {.lex_state = 152, .external_lex_state = 2},
  [640] = {.lex_state = 152, .external_lex_state = 2},
  [641] = {.lex_state = 152, .external_lex_state = 2},
  [642] = {.lex_state = 152, .external_lex_state = 2},
  [643] = {.lex_state = 152, .external_lex_state = 2},
  [644] = {.lex_state = 152, .external_lex_state = 2},
  [645] = {.lex_state = 152, .external_lex_state = 2},
  [646] = {.lex_state = 152, .external_lex_state = 2},
  [647] = {.lex_state = 152, .external_lex_state = 2},
  [648] = {.lex_state = 152, .external_lex_state = 2},
  [649] = {.lex_state = 152, .external_lex_state = 2},
  [650] = {.lex_state = 152, .external_lex_state = 2},
  [651] = {.lex_state = 152, .external_lex_state = 2},
  [652] = {.lex_state = 152, .external_lex_state = 2},
  [653] = {.lex_state = 152, .external_lex_state = 2},
  [654] = {.lex_state = 152, .external_lex_state = 2},
  [655] = {.lex_state = 152, .external_lex_state = 2},
  [656] = {.lex_state = 152, .external_lex_state = 2},
  [657] = {.lex_state = 152, .external_lex_state = 2},
  [658] = {.lex_state = 152, .external_lex_state = 2},
  [659] = {.lex_state = 152, .external_lex_state = 2},
  [660] = {.lex_state = 152, .external_lex_state = 2},
  [661] = {.lex_state = 152, .external_lex_state = 2},
  [662] = {.lex_state = 152, .external_lex_state = 2},
  [663] = {.lex_state = 152, .external_lex_state = 2},
  [664] = {.lex_state = 152, .external_lex_state = 2},
  [665] = {.lex_state = 152, .external_lex_state = 2},
  [666] = {.lex_state = 152, .external_lex_state = 2},
  [667] = {.lex_state = 152, .external_lex_state = 2},
  [668] = {.lex_state = 152, .external_lex_state = 2},
  [669] = {.lex_state = 152, .external_lex_state = 2},
  [670] = {.lex_state = 42, .external_lex_state = 10},
  [671] = {.lex_state = 42, .external_lex_state = 10},
  [672] = {.lex_state = 42, .external_lex_state = 10},
  [673] = {.lex_state = 42, .external_lex_state = 10},
  [674] = {.lex_state = 42, .external_lex_state = 10},
  [675] = {.lex_state = 152, .external_lex_state = 16},
  [676] = {.lex_state = 152, .external_lex_state = 16},
  [677] = {.lex_state = 152, .external_lex_state = 17},
  [678] = {.lex_state = 152, .external_lex_state = 2},
  [679] = {.lex_state = 152, .external_lex_state = 7},
  [680] = {.lex_state = 152, .external_lex_state = 7},
  [681] = {.lex_state = 152, .external_lex_state = 16},
  [682] = {.lex_state = 152, .external_lex_state = 7},
  [683] = {.lex_state = 152, .external_lex_state = 16},
  [684] = {.lex_state = 152, .external_lex_state = 7},
  [685] = {.lex_state = 152, .external_lex_state = 7},
  [686] = {.lex_state = 152, .external_lex_state = 7},
  [687] = {.lex_state = 152, .external_lex_state = 7},
  [688] = {.lex_state = 152, .external_lex_state = 14},
  [689] = {.lex_state = 152, .external_lex_state = 14},
  [690] = {.lex_state = 152, .external_lex_state = 7},
  [691] = {.lex_state = 152, .external_lex_state = 16},
  [692] = {.lex_state = 152, .external_lex_state = 16},
  [693] = {.lex_state = 152, .external_lex_state = 16},
  [694] = {.lex_state = 152, .external_lex_state = 16},
  [695] = {.lex_state = 152, .external_lex_state = 7},
  [696] = {.lex_state = 152, .external_lex_state = 16},
  [697] = {.lex_state = 152, .external_lex_state = 2},
  [698] = {.lex_state = 152, .external_lex_state = 2},
  [699] = {.lex_state = 152, .external_lex_state = 7},
  [700] = {.lex_state = 152, .external_lex_state = 7},
  [701] = {.lex_state = 152, .external_lex_state = 7},
  [702] = {.lex_state = 152, .external_lex_state = 7},
  [703] = {.lex_state = 152, .external_lex_state = 2},
  [704] = {.lex_state = 152, .external_lex_state = 2},
  [705] = {.lex_state = 152, .external_lex_state = 2},
  [706] = {.lex_state = 152, .external_lex_state = 14},
  [707] = {.lex_state = 152, .external_lex_state = 2},
  [708] = {.lex_state = 152, .external_lex_state = 7},
  [709] = {.lex_state = 152, .external_lex_state = 7},
  [710] = {.lex_state = 152, .external_lex_state = 2},
  [711] = {.lex_state = 152, .external_lex_state = 2},
  [712] = {.lex_state = 152, .external_lex_state = 14},
  [713] = {.lex_state = 152, .external_lex_state = 7},
  [714] = {.lex_state = 152, .external_lex_state = 2},
  [715] = {.lex_state = 152, .external_lex_state = 2},
  [716] = {.lex_state = 152, .external_lex_state = 14},
  [717] = {.lex_state = 152, .external_lex_state = 2},
  [718] = {.lex_state = 152, .external_lex_state = 2},
  [719] = {.lex_state = 152, .external_lex_state = 2},
  [720] = {.lex_state = 152, .external_lex_state = 2},
  [721] = {.lex_state = 152, .external_lex_state = 2},
  [722] = {.lex_state = 152, .external_lex_state = 14},
  [723] = {.lex_state = 152, .external_lex_state = 2},
  [724] = {.lex_state = 152, .external_lex_state = 2},
  [725] = {.lex_state = 152, .external_lex_state = 2},
  [726] = {.lex_state = 152, .external_lex_state = 16},
  [727] = {.lex_state = 152, .external_lex_state = 2},
  [728] = {.lex_state = 152, .external_lex_state = 2},
  [729] = {.lex_state = 152, .external_lex_state = 2},
  [730] = {.lex_state = 152, .external_lex_state = 2},
  [731] = {.lex_state = 152, .external_lex_state = 2},
  [732] = {.lex_state = 152, .external_lex_state = 7},
  [733] = {.lex_state = 152, .external_lex_state = 14},
  [734] = {.lex_state = 152, .external_lex_state = 7},
  [735] = {.lex_state = 152, .external_lex_state = 2},
  [736] = {.lex_state = 152, .external_lex_state = 7},
  [737] = {.lex_state = 152, .external_lex_state = 2},
  [738] = {.lex_state = 152, .external_lex_state = 2},
  [739] = {.lex_state = 152, .external_lex_state = 7},
  [740] = {.lex_state = 152, .external_lex_state = 14},
  [741] = {.lex_state = 152, .external_lex_state = 10},
  [742] = {.lex_state = 152, .external_lex_state = 10},
  [743] = {.lex_state = 152, .external_lex_state = 2},
  [744] = {.lex_state = 152, .external_lex_state = 2},
  [745] = {.lex_state = 152, .external_lex_state = 10},
  [746] = {.lex_state = 152, .external_lex_state = 10},
  [747] = {.lex_state = 152, .external_lex_state = 7},
  [748] = {.lex_state = 152, .external_lex_state = 10},
  [749] = {.lex_state = 152, .external_lex_state = 14},
  [750] = {.lex_state = 152, .external_lex_state = 14},
  [751] = {.lex_state = 152, .external_lex_state = 10},
  [752] = {.lex_state = 152, .external_lex_state = 10},
  [753] = {.lex_state = 152, .external_lex_state = 2},
  [754] = {.lex_state = 152, .external_lex_state = 10},
  [755] = {.lex_state = 152, .external_lex_state = 2},
  [756] = {.lex_state = 152, .external_lex_state = 10},
  [757] = {.lex_state = 152, .external_lex_state = 15},
  [758] = {.lex_state = 152, .external_lex_state = 15},
  [759] = {.lex_state = 152, .external_lex_state = 15},
  [760] = {.lex_state = 152, .external_lex_state = 15},
  [761] = {.lex_state = 152, .external_lex_state = 15},
  [762] = {.lex_state = 152, .external_lex_state = 15},
  [763] = {.lex_state = 152, .external_lex_state = 15},
  [764] = {.lex_state = 152, .external_lex_state = 15},
  [765] = {.lex_state = 152, .external_lex_state = 15},
  [766] = {.lex_state = 152, .external_lex_state = 15},
  [767] = {.lex_state = 152, .external_lex_state = 15},
  [768] = {.lex_state = 152, .external_lex_state = 15},
  [769] = {.lex_state = 152, .external_lex_state = 15},
  [770] = {.lex_state = 152, .external_lex_state = 15},
  [771] = {.lex_state = 152, .external_lex_state = 15},
  [772] = {.lex_state = 152, .external_lex_state = 15},
  [773] = {.lex_state = 152, .external_lex_state = 15},
  [774] = {.lex_state = 152, .external_lex_state = 15},
  [775] = {.lex_state = 152, .external_lex_state = 15},
  [776] = {.lex_state = 152, .external_lex_state = 15},
  [777] = {.lex_state = 152, .external_lex_state = 15},
  [778] = {.lex_state = 152, .external_lex_state = 15},
  [779] = {.lex_state = 152, .external_lex_state = 15},
  [780] = {.lex_state = 152, .external_lex_state = 15},
  [781] = {.lex_state = 152, .external_lex_state = 15},
  [782] = {.lex_state = 152, .external_lex_state = 15},
  [783] = {.lex_state = 152, .external_lex_state = 15},
  [784] = {.lex_state = 152, .external_lex_state = 15},
  [785] = {.lex_state = 152, .external_lex_state = 15},
  [786] = {.lex_state = 152, .external_lex_state = 15},
  [787] = {.lex_state = 152, .external_lex_state = 15},
  [788] = {.lex_state = 152, .external_lex_state = 15},
  [789] = {.lex_state = 152, .external_lex_state = 10},
  [790] = {.lex_state = 152, .external_lex_state = 15},
  [791] = {.lex_state = 152, .external_lex_state = 15},
  [792] = {.lex_state = 152, .external_lex_state = 15},
  [793] = {.lex_state = 152, .external_lex_state = 15},
  [794] = {.lex_state = 152, .external_lex_state = 15},
  [795] = {.lex_state = 152, .external_lex_state = 2},
  [796] = {.lex_state = 152, .external_lex_state = 15},
  [797] = {.lex_state = 152, .external_lex_state = 15},
  [798] = {.lex_state = 152, .external_lex_state = 15},
  [799] = {.lex_state = 152, .external_lex_state = 15},
  [800] = {.lex_state = 152, .external_lex_state = 15},
  [801] = {.lex_state = 152, .external_lex_state = 15},
  [802] = {.lex_state = 152, .external_lex_state = 15},
  [803] = {.lex_state = 152, .external_lex_state = 15},
  [804] = {.lex_state = 152, .external_lex_state = 15},
  [805] = {.lex_state = 152, .external_lex_state = 2},
  [806] = {.lex_state = 152, .external_lex_state = 7},
  [807] = {.lex_state = 152, .external_lex_state = 15},
  [808] = {.lex_state = 152, .external_lex_state = 2},
  [809] = {.lex_state = 152, .external_lex_state = 2},
  [810] = {.lex_state = 152, .external_lex_state = 15},
  [811] = {.lex_state = 152, .external_lex_state = 15},
  [812] = {.lex_state = 152, .external_lex_state = 15},
  [813] = {.lex_state = 152, .external_lex_state = 15},
  [814] = {.lex_state = 152, .external_lex_state = 15},
  [815] = {.lex_state = 152, .external_lex_state = 15},
  [816] = {.lex_state = 152, .external_lex_state = 15},
  [817] = {.lex_state = 152, .external_lex_state = 15},
  [818] = {.lex_state = 152, .external_lex_state = 15},
  [819] = {.lex_state = 152, .external_lex_state = 15},
  [820] = {.lex_state = 152, .external_lex_state = 2},
  [821] = {.lex_state = 152, .external_lex_state = 15},
  [822] = {.lex_state = 152, .external_lex_state = 15},
  [823] = {.lex_state = 152, .external_lex_state = 15},
  [824] = {.lex_state = 152, .external_lex_state = 15},
  [825] = {.lex_state = 152, .external_lex_state = 15},
  [826] = {.lex_state = 152, .external_lex_state = 15},
  [827] = {.lex_state = 152, .external_lex_state = 15},
  [828] = {.lex_state = 152, .external_lex_state = 2},
  [829] = {.lex_state = 152, .external_lex_state = 15},
  [830] = {.lex_state = 152, .external_lex_state = 15},
  [831] = {.lex_state = 152, .external_lex_state = 15},
  [832] = {.lex_state = 152, .external_lex_state = 15},
  [833] = {.lex_state = 152, .external_lex_state = 15},
  [834] = {.lex_state = 152, .external_lex_state = 15},
  [835] = {.lex_state = 152, .external_lex_state = 15},
  [836] = {.lex_state = 152, .external_lex_state = 15},
  [837] = {.lex_state = 152, .external_lex_state = 15},
  [838] = {.lex_state = 152, .external_lex_state = 15},
  [839] = {.lex_state = 152, .external_lex_state = 15},
  [840] = {.lex_state = 152, .external_lex_state = 15},
  [841] = {.lex_state = 152, .external_lex_state = 15},
  [842] = {.lex_state = 152, .external_lex_state = 2},
  [843] = {.lex_state = 152, .external_lex_state = 2},
  [844] = {.lex_state = 152, .external_lex_state = 2},
  [845] = {.lex_state = 152, .external_lex_state = 2},
  [846] = {.lex_state = 152, .external_lex_state = 2},
  [847] = {.lex_state = 152, .external_lex_state = 2},
  [848] = {.lex_state = 152, .external_lex_state = 2},
  [849] = {.lex_state = 152, .external_lex_state = 2},
  [850] = {.lex_state = 152, .external_lex_state = 2},
  [851] = {.lex_state = 152, .external_lex_state = 2},
  [852] = {.lex_state = 152, .external_lex_state = 2},
  [853] = {.lex_state = 152, .external_lex_state = 2},
  [854] = {.lex_state = 152, .external_lex_state = 2},
  [855] = {.lex_state = 152, .external_lex_state = 2},
  [856] = {.lex_state = 152, .external_lex_state = 2},
  [857] = {.lex_state = 152, .external_lex_state = 2},
  [858] = {.lex_state = 152, .external_lex_state = 2},
  [859] = {.lex_state = 152, .external_lex_state = 2},
  [860] = {.lex_state = 152, .external_lex_state = 2},
  [861] = {.lex_state = 152, .external_lex_state = 2},
  [862] = {.lex_state = 152, .external_lex_state = 2},
  [863] = {.lex_state = 152, .external_lex_state = 2},
  [864] = {.lex_state = 152, .external_lex_state = 2},
  [865] = {.lex_state = 152, .external_lex_state = 2},
  [866] = {.lex_state = 152, .external_lex_state = 2},
  [867] = {.lex_state = 152, .external_lex_state = 2},
  [868] = {.lex_state = 152, .external_lex_state = 2},
  [869] = {.lex_state = 152, .external_lex_state = 2},
  [870] = {.lex_state = 152, .external_lex_state = 2},
  [871] = {.lex_state = 152, .external_lex_state = 2},
  [872] = {.lex_state = 152, .external_lex_state = 2},
  [873] = {.lex_state = 152, .external_lex_state = 2},
  [874] = {.lex_state = 152, .external_lex_state = 2},
  [875] = {.lex_state = 152, .external_lex_state = 2},
  [876] = {.lex_state = 152, .external_lex_state = 2},
  [877] = {.lex_state = 152, .external_lex_state = 2},
  [878] = {.lex_state = 152, .external_lex_state = 2},
  [879] = {.lex_state = 152, .external_lex_state = 2},
  [880] = {.lex_state = 152, .external_lex_state = 2},
  [881] = {.lex_state = 152, .external_lex_state = 2},
  [882] = {.lex_state = 152, .external_lex_state = 2},
  [883] = {.lex_state = 152, .external_lex_state = 2},
  [884] = {.lex_state = 152, .external_lex_state = 2},
  [885] = {.lex_state = 152, .external_lex_state = 2},
  [886] = {.lex_state = 152, .external_lex_state = 2},
  [887] = {.lex_state = 152, .external_lex_state = 2},
  [888] = {.lex_state = 152, .external_lex_state = 2},
  [889] = {.lex_state = 152, .external_lex_state = 2},
  [890] = {.lex_state = 152, .external_lex_state = 2},
  [891] = {.lex_state = 152, .external_lex_state = 2},
  [892] = {.lex_state = 152, .external_lex_state = 2},
  [893] = {.lex_state = 152, .external_lex_state = 2},
  [894] = {.lex_state = 152, .external_lex_state = 2},
  [895] = {.lex_state = 152, .external_lex_state = 2},
  [896] = {.lex_state = 152, .external_lex_state = 2},
  [897] = {.lex_state = 152, .external_lex_state = 2},
  [898] = {.lex_state = 152, .external_lex_state = 2},
  [899] = {.lex_state = 152, .external_lex_state = 2},
  [900] = {.lex_state = 152, .external_lex_state = 2},
  [901] = {.lex_state = 152, .external_lex_state = 2},
  [902] = {.lex_state = 152, .external_lex_state = 2},
  [903] = {.lex_state = 152, .external_lex_state = 2},
  [904] = {.lex_state = 152, .external_lex_state = 2},
  [905] = {.lex_state = 152, .external_lex_state = 2},
  [906] = {.lex_state = 152, .external_lex_state = 2},
  [907] = {.lex_state = 152, .external_lex_state = 2},
  [908] = {.lex_state = 152, .external_lex_state = 2},
  [909] = {.lex_state = 152, .external_lex_state = 2},
  [910] = {.lex_state = 152, .external_lex_state = 2},
  [911] = {.lex_state = 152, .external_lex_state = 2},
  [912] = {.lex_state = 152, .external_lex_state = 2},
  [913] = {.lex_state = 152, .external_lex_state = 2},
  [914] = {.lex_state = 152, .external_lex_state = 2},
  [915] = {.lex_state = 152, .external_lex_state = 2},
  [916] = {.lex_state = 152, .external_lex_state = 2},
  [917] = {.lex_state = 152, .external_lex_state = 2},
  [918] = {.lex_state = 152, .external_lex_state = 2},
  [919] = {.lex_state = 152, .external_lex_state = 2},
  [920] = {.lex_state = 152, .external_lex_state = 2},
  [921] = {.lex_state = 152, .external_lex_state = 2},
  [922] = {.lex_state = 152, .external_lex_state = 2},
  [923] = {.lex_state = 152, .external_lex_state = 2},
  [924] = {.lex_state = 152, .external_lex_state = 2},
  [925] = {.lex_state = 152, .external_lex_state = 2},
  [926] = {.lex_state = 152, .external_lex_state = 2},
  [927] = {.lex_state = 152, .external_lex_state = 2},
  [928] = {.lex_state = 152, .external_lex_state = 2},
  [929] = {.lex_state = 152, .external_lex_state = 2},
  [930] = {.lex_state = 152, .external_lex_state = 2},
  [931] = {.lex_state = 152, .external_lex_state = 2},
  [932] = {.lex_state = 152, .external_lex_state = 2},
  [933] = {.lex_state = 152, .external_lex_state = 2},
  [934] = {.lex_state = 152, .external_lex_state = 2},
  [935] = {.lex_state = 152, .external_lex_state = 2},
  [936] = {.lex_state = 152, .external_lex_state = 2},
  [937] = {.lex_state = 152, .external_lex_state = 2},
  [938] = {.lex_state = 152, .external_lex_state = 2},
  [939] = {.lex_state = 152, .external_lex_state = 2},
  [940] = {.lex_state = 152, .external_lex_state = 2},
  [941] = {.lex_state = 152, .external_lex_state = 2},
  [942] = {.lex_state = 152, .external_lex_state = 2},
  [943] = {.lex_state = 152, .external_lex_state = 2},
  [944] = {.lex_state = 152, .external_lex_state = 2},
  [945] = {.lex_state = 152, .external_lex_state = 2},
  [946] = {.lex_state = 152, .external_lex_state = 2},
  [947] = {.lex_state = 152, .external_lex_state = 2},
  [948] = {.lex_state = 152, .external_lex_state = 2},
  [949] = {.lex_state = 152, .external_lex_state = 2},
  [950] = {.lex_state = 152, .external_lex_state = 2},
  [951] = {.lex_state = 152, .external_lex_state = 2},
  [952] = {.lex_state = 152, .external_lex_state = 2},
  [953] = {.lex_state = 152, .external_lex_state = 2},
  [954] = {.lex_state = 152, .external_lex_state = 2},
  [955] = {.lex_state = 152, .external_lex_state = 2},
  [956] = {.lex_state = 152, .external_lex_state = 2},
  [957] = {.lex_state = 152, .external_lex_state = 2},
  [958] = {.lex_state = 152, .external_lex_state = 2},
  [959] = {.lex_state = 152, .external_lex_state = 2},
  [960] = {.lex_state = 152, .external_lex_state = 2},
  [961] = {.lex_state = 152, .external_lex_state = 2},
  [962] = {.lex_state = 152, .external_lex_state = 2},
  [963] = {.lex_state = 152, .external_lex_state = 2},
  [964] = {.lex_state = 152, .external_lex_state = 2},
  [965] = {.lex_state = 152, .external_lex_state = 2},
  [966] = {.lex_state = 152, .external_lex_state = 2},
  [967] = {.lex_state = 152, .external_lex_state = 2},
  [968] = {.lex_state = 152, .external_lex_state = 2},
  [969] = {.lex_state = 152, .external_lex_state = 2},
  [970] = {.lex_state = 152, .external_lex_state = 2},
  [971] = {.lex_state = 152, .external_lex_state = 2},
  [972] = {.lex_state = 152, .external_lex_state = 2},
  [973] = {.lex_state = 152, .external_lex_state = 2},
  [974] = {.lex_state = 152, .external_lex_state = 2},
  [975] = {.lex_state = 152, .external_lex_state = 2},
  [976] = {.lex_state = 152, .external_lex_state = 2},
  [977] = {.lex_state = 152, .external_lex_state = 2},
  [978] = {.lex_state = 152, .external_lex_state = 2},
  [979] = {.lex_state = 152, .external_lex_state = 2},
  [980] = {.lex_state = 152, .external_lex_state = 2},
  [981] = {.lex_state = 152, .external_lex_state = 2},
  [982] = {.lex_state = 152, .external_lex_state = 2},
  [983] = {.lex_state = 152, .external_lex_state = 2},
  [984] = {.lex_state = 152, .external_lex_state = 2},
  [985] = {.lex_state = 152, .external_lex_state = 2},
  [986] = {.lex_state = 152, .external_lex_state = 2},
  [987] = {.lex_state = 152, .external_lex_state = 2},
  [988] = {.lex_state = 152, .external_lex_state = 2},
  [989] = {.lex_state = 152, .external_lex_state = 2},
  [990] = {.lex_state = 152, .external_lex_state = 2},
  [991] = {.lex_state = 152, .external_lex_state = 2},
  [992] = {.lex_state = 152, .external_lex_state = 2},
  [993] = {.lex_state = 152, .external_lex_state = 2},
  [994] = {.lex_state = 152, .external_lex_state = 2},
  [995] = {.lex_state = 152, .external_lex_state = 2},
  [996] = {.lex_state = 152, .external_lex_state = 2},
  [997] = {.lex_state = 152, .external_lex_state = 2},
  [998] = {.lex_state = 152, .external_lex_state = 2},
  [999] = {.lex_state = 152, .external_lex_state = 2},
  [1000] = {.lex_state = 152, .external_lex_state = 2},
  [1001] = {.lex_state = 152, .external_lex_state = 2},
  [1002] = {.lex_state = 152, .external_lex_state = 2},
  [1003] = {.lex_state = 152, .external_lex_state = 2},
  [1004] = {.lex_state = 152, .external_lex_state = 2},
  [1005] = {.lex_state = 152, .external_lex_state = 2},
  [1006] = {.lex_state = 152, .external_lex_state = 2},
  [1007] = {.lex_state = 152, .external_lex_state = 2},
  [1008] = {.lex_state = 152, .external_lex_state = 2},
  [1009] = {.lex_state = 152, .external_lex_state = 2},
  [1010] = {.lex_state = 152, .external_lex_state = 2},
  [1011] = {.lex_state = 152, .external_lex_state = 2},
  [1012] = {.lex_state = 152, .external_lex_state = 2},
  [1013] = {.lex_state = 152, .external_lex_state = 2},
  [1014] = {.lex_state = 152, .external_lex_state = 2},
  [1015] = {.lex_state = 152, .external_lex_state = 2},
  [1016] = {.lex_state = 152, .external_lex_state = 2},
  [1017] = {.lex_state = 152, .external_lex_state = 2},
  [1018] = {.lex_state = 152, .external_lex_state = 2},
  [1019] = {.lex_state = 152, .external_lex_state = 2},
  [1020] = {.lex_state = 152, .external_lex_state = 2},
  [1021] = {.lex_state = 152, .external_lex_state = 2},
  [1022] = {.lex_state = 152, .external_lex_state = 2},
  [1023] = {.lex_state = 152, .external_lex_state = 2},
  [1024] = {.lex_state = 152, .external_lex_state = 2},
  [1025] = {.lex_state = 152, .external_lex_state = 2},
  [1026] = {.lex_state = 152, .external_lex_state = 2},
  [1027] = {.lex_state = 152, .external_lex_state = 2},
  [1028] = {.lex_state = 152, .external_lex_state = 2},
  [1029] = {.lex_state = 152, .external_lex_state = 2},
  [1030] = {.lex_state = 152, .external_lex_state = 2},
  [1031] = {.lex_state = 152, .external_lex_state = 2},
  [1032] = {.lex_state = 152, .external_lex_state = 2},
  [1033] = {.lex_state = 152, .external_lex_state = 2},
  [1034] = {.lex_state = 152, .external_lex_state = 2},
  [1035] = {.lex_state = 152, .external_lex_state = 2},
  [1036] = {.lex_state = 152, .external_lex_state = 2},
  [1037] = {.lex_state = 152, .external_lex_state = 2},
  [1038] = {.lex_state = 152, .external_lex_state = 2},
  [1039] = {.lex_state = 152, .external_lex_state = 2},
  [1040] = {.lex_state = 152, .external_lex_state = 2},
  [1041] = {.lex_state = 152, .external_lex_state = 2},
  [1042] = {.lex_state = 152, .external_lex_state = 2},
  [1043] = {.lex_state = 152, .external_lex_state = 2},
  [1044] = {.lex_state = 152, .external_lex_state = 2},
  [1045] = {.lex_state = 152, .external_lex_state = 2},
  [1046] = {.lex_state = 152, .external_lex_state = 2},
  [1047] = {.lex_state = 152, .external_lex_state = 2},
  [1048] = {.lex_state = 152, .external_lex_state = 2},
  [1049] = {.lex_state = 152, .external_lex_state = 2},
  [1050] = {.lex_state = 152, .external_lex_state = 2},
  [1051] = {.lex_state = 152, .external_lex_state = 2},
  [1052] = {.lex_state = 152, .external_lex_state = 2},
  [1053] = {.lex_state = 152, .external_lex_state = 2},
  [1054] = {.lex_state = 152, .external_lex_state = 2},
  [1055] = {.lex_state = 152, .external_lex_state = 2},
  [1056] = {.lex_state = 152, .external_lex_state = 2},
  [1057] = {.lex_state = 152, .external_lex_state = 2},
  [1058] = {.lex_state = 152, .external_lex_state = 2},
  [1059] = {.lex_state = 152, .external_lex_state = 2},
  [1060] = {.lex_state = 152, .external_lex_state = 2},
  [1061] = {.lex_state = 152, .external_lex_state = 2},
  [1062] = {.lex_state = 152, .external_lex_state = 2},
  [1063] = {.lex_state = 152, .external_lex_state = 2},
  [1064] = {.lex_state = 152, .external_lex_state = 2},
  [1065] = {.lex_state = 152, .external_lex_state = 2},
  [1066] = {.lex_state = 152, .external_lex_state = 2},
  [1067] = {.lex_state = 152, .external_lex_state = 2},
  [1068] = {.lex_state = 152, .external_lex_state = 2},
  [1069] = {.lex_state = 152, .external_lex_state = 2},
  [1070] = {.lex_state = 152, .external_lex_state = 2},
  [1071] = {.lex_state = 152, .external_lex_state = 2},
  [1072] = {.lex_state = 152, .external_lex_state = 2},
  [1073] = {.lex_state = 152, .external_lex_state = 2},
  [1074] = {.lex_state = 152, .external_lex_state = 2},
  [1075] = {.lex_state = 152, .external_lex_state = 2},
  [1076] = {.lex_state = 152, .external_lex_state = 2},
  [1077] = {.lex_state = 152, .external_lex_state = 2},
  [1078] = {.lex_state = 152, .external_lex_state = 2},
  [1079] = {.lex_state = 152, .external_lex_state = 2},
  [1080] = {.lex_state = 152, .external_lex_state = 2},
  [1081] = {.lex_state = 152, .external_lex_state = 2},
  [1082] = {.lex_state = 152, .external_lex_state = 2},
  [1083] = {.lex_state = 152, .external_lex_state = 2},
  [1084] = {.lex_state = 152, .external_lex_state = 2},
  [1085] = {.lex_state = 152, .external_lex_state = 2},
  [1086] = {.lex_state = 152, .external_lex_state = 2},
  [1087] = {.lex_state = 152, .external_lex_state = 2},
  [1088] = {.lex_state = 152, .external_lex_state = 2},
  [1089] = {.lex_state = 152, .external_lex_state = 2},
  [1090] = {.lex_state = 152, .external_lex_state = 2},
  [1091] = {.lex_state = 152, .external_lex_state = 2},
  [1092] = {.lex_state = 152, .external_lex_state = 2},
  [1093] = {.lex_state = 152, .external_lex_state = 2},
  [1094] = {.lex_state = 152, .external_lex_state = 2},
  [1095] = {.lex_state = 152, .external_lex_state = 2},
  [1096] = {.lex_state = 152, .external_lex_state = 2},
  [1097] = {.lex_state = 152, .external_lex_state = 2},
  [1098] = {.lex_state = 152, .external_lex_state = 2},
  [1099] = {.lex_state = 152, .external_lex_state = 2},
  [1100] = {.lex_state = 152, .external_lex_state = 2},
  [1101] = {.lex_state = 152, .external_lex_state = 2},
  [1102] = {.lex_state = 152, .external_lex_state = 2},
  [1103] = {.lex_state = 152, .external_lex_state = 2},
  [1104] = {.lex_state = 152, .external_lex_state = 2},
  [1105] = {.lex_state = 152, .external_lex_state = 2},
  [1106] = {.lex_state = 152, .external_lex_state = 2},
  [1107] = {.lex_state = 152, .external_lex_state = 2},
  [1108] = {.lex_state = 152, .external_lex_state = 2},
  [1109] = {.lex_state = 152, .external_lex_state = 2},
  [1110] = {.lex_state = 152, .external_lex_state = 2},
  [1111] = {.lex_state = 152, .external_lex_state = 2},
  [1112] = {.lex_state = 152, .external_lex_state = 2},
  [1113] = {.lex_state = 152, .external_lex_state = 2},
  [1114] = {.lex_state = 152, .external_lex_state = 2},
  [1115] = {.lex_state = 152, .external_lex_state = 2},
  [1116] = {.lex_state = 152, .external_lex_state = 2},
  [1117] = {.lex_state = 152, .external_lex_state = 2},
  [1118] = {.lex_state = 152, .external_lex_state = 2},
  [1119] = {.lex_state = 152, .external_lex_state = 2},
  [1120] = {.lex_state = 152, .external_lex_state = 2},
  [1121] = {.lex_state = 152, .external_lex_state = 2},
  [1122] = {.lex_state = 152, .external_lex_state = 2},
  [1123] = {.lex_state = 152, .external_lex_state = 2},
  [1124] = {.lex_state = 152, .external_lex_state = 2},
  [1125] = {.lex_state = 152, .external_lex_state = 2},
  [1126] = {.lex_state = 152, .external_lex_state = 2},
  [1127] = {.lex_state = 152, .external_lex_state = 2},
  [1128] = {.lex_state = 152, .external_lex_state = 2},
  [1129] = {.lex_state = 152, .external_lex_state = 2},
  [1130] = {.lex_state = 152, .external_lex_state = 2},
  [1131] = {.lex_state = 152, .external_lex_state = 2},
  [1132] = {.lex_state = 152, .external_lex_state = 2},
  [1133] = {.lex_state = 152, .external_lex_state = 2},
  [1134] = {.lex_state = 152, .external_lex_state = 2},
  [1135] = {.lex_state = 152, .external_lex_state = 2},
  [1136] = {.lex_state = 152, .external_lex_state = 2},
  [1137] = {.lex_state = 152, .external_lex_state = 2},
  [1138] = {.lex_state = 152, .external_lex_state = 2},
  [1139] = {.lex_state = 152, .external_lex_state = 2},
  [1140] = {.lex_state = 152, .external_lex_state = 2},
  [1141] = {.lex_state = 152, .external_lex_state = 2},
  [1142] = {.lex_state = 152, .external_lex_state = 2},
  [1143] = {.lex_state = 152, .external_lex_state = 2},
  [1144] = {.lex_state = 152, .external_lex_state = 2},
  [1145] = {.lex_state = 152, .external_lex_state = 2},
  [1146] = {.lex_state = 152, .external_lex_state = 2},
  [1147] = {.lex_state = 152, .external_lex_state = 2},
  [1148] = {.lex_state = 152, .external_lex_state = 2},
  [1149] = {.lex_state = 152, .external_lex_state = 2},
  [1150] = {.lex_state = 152, .external_lex_state = 2},
  [1151] = {.lex_state = 152, .external_lex_state = 2},
  [1152] = {.lex_state = 152, .external_lex_state = 2},
  [1153] = {.lex_state = 152, .external_lex_state = 2},
  [1154] = {.lex_state = 152, .external_lex_state = 2},
  [1155] = {.lex_state = 152, .external_lex_state = 2},
  [1156] = {.lex_state = 152, .external_lex_state = 2},
  [1157] = {.lex_state = 152, .external_lex_state = 2},
  [1158] = {.lex_state = 152, .external_lex_state = 2},
  [1159] = {.lex_state = 152, .external_lex_state = 2},
  [1160] = {.lex_state = 152, .external_lex_state = 2},
  [1161] = {.lex_state = 152, .external_lex_state = 2},
  [1162] = {.lex_state = 152, .external_lex_state = 2},
  [1163] = {.lex_state = 152, .external_lex_state = 2},
  [1164] = {.lex_state = 152, .external_lex_state = 2},
  [1165] = {.lex_state = 152, .external_lex_state = 2},
  [1166] = {.lex_state = 152, .external_lex_state = 2},
  [1167] = {.lex_state = 144, .external_lex_state = 3},
  [1168] = {.lex_state = 152, .external_lex_state = 2},
  [1169] = {.lex_state = 152, .external_lex_state = 2},
  [1170] = {.lex_state = 152, .external_lex_state = 2},
  [1171] = {.lex_state = 152, .external_lex_state = 2},
  [1172] = {.lex_state = 152, .external_lex_state = 2},
  [1173] = {.lex_state = 152, .external_lex_state = 2},
  [1174] = {.lex_state = 152, .external_lex_state = 2},
  [1175] = {.lex_state = 152, .external_lex_state = 2},
  [1176] = {.lex_state = 152, .external_lex_state = 2},
  [1177] = {.lex_state = 152, .external_lex_state = 2},
  [1178] = {.lex_state = 152, .external_lex_state = 2},
  [1179] = {.lex_state = 152, .external_lex_state = 2},
  [1180] = {.lex_state = 152, .external_lex_state = 2},
  [1181] = {.lex_state = 152, .external_lex_state = 2},
  [1182] = {.lex_state = 152, .external_lex_state = 2},
  [1183] = {.lex_state = 152, .external_lex_state = 2},
  [1184] = {.lex_state = 152, .external_lex_state = 2},
  [1185] = {.lex_state = 152, .external_lex_state = 2},
  [1186] = {.lex_state = 152, .external_lex_state = 2},
  [1187] = {.lex_state = 152, .external_lex_state = 2},
  [1188] = {.lex_state = 152, .external_lex_state = 2},
  [1189] = {.lex_state = 152, .external_lex_state = 2},
  [1190] = {.lex_state = 152, .external_lex_state = 2},
  [1191] = {.lex_state = 152, .external_lex_state = 2},
  [1192] = {.lex_state = 152, .external_lex_state = 2},
  [1193] = {.lex_state = 152, .external_lex_state = 2},
  [1194] = {.lex_state = 152, .external_lex_state = 2},
  [1195] = {.lex_state = 152, .external_lex_state = 2},
  [1196] = {.lex_state = 152, .external_lex_state = 2},
  [1197] = {.lex_state = 152, .external_lex_state = 2},
  [1198] = {.lex_state = 152, .external_lex_state = 2},
  [1199] = {.lex_state = 152, .external_lex_state = 2},
  [1200] = {.lex_state = 152, .external_lex_state = 2},
  [1201] = {.lex_state = 152, .external_lex_state = 2},
  [1202] = {.lex_state = 152, .external_lex_state = 2},
  [1203] = {.lex_state = 152, .external_lex_state = 2},
  [1204] = {.lex_state = 152, .external_lex_state = 2},
  [1205] = {.lex_state = 152, .external_lex_state = 2},
  [1206] = {.lex_state = 152, .external_lex_state = 2},
  [1207] = {.lex_state = 152, .external_lex_state = 2},
  [1208] = {.lex_state = 152, .external_lex_state = 2},
  [1209] = {.lex_state = 152, .external_lex_state = 2},
  [1210] = {.lex_state = 152, .external_lex_state = 2},
  [1211] = {.lex_state = 152, .external_lex_state = 2},
  [1212] = {.lex_state = 152, .external_lex_state = 2},
  [1213] = {.lex_state = 152, .external_lex_state = 2},
  [1214] = {.lex_state = 152, .external_lex_state = 2},
  [1215] = {.lex_state = 152, .external_lex_state = 2},
  [1216] = {.lex_state = 152, .external_lex_state = 2},
  [1217] = {.lex_state = 152, .external_lex_state = 2},
  [1218] = {.lex_state = 152, .external_lex_state = 2},
  [1219] = {.lex_state = 152, .external_lex_state = 2},
  [1220] = {.lex_state = 152, .external_lex_state = 2},
  [1221] = {.lex_state = 152, .external_lex_state = 2},
  [1222] = {.lex_state = 152, .external_lex_state = 2},
  [1223] = {.lex_state = 152, .external_lex_state = 2},
  [1224] = {.lex_state = 152, .external_lex_state = 2},
  [1225] = {.lex_state = 152, .external_lex_state = 2},
  [1226] = {.lex_state = 152, .external_lex_state = 2},
  [1227] = {.lex_state = 152, .external_lex_state = 2},
  [1228] = {.lex_state = 152, .external_lex_state = 2},
  [1229] = {.lex_state = 152, .external_lex_state = 2},
  [1230] = {.lex_state = 152, .external_lex_state = 2},
  [1231] = {.lex_state = 152, .external_lex_state = 2},
  [1232] = {.lex_state = 152, .external_lex_state = 2},
  [1233] = {.lex_state = 152, .external_lex_state = 2},
  [1234] = {.lex_state = 152, .external_lex_state = 2},
  [1235] = {.lex_state = 152, .external_lex_state = 2},
  [1236] = {.lex_state = 152, .external_lex_state = 2},
  [1237] = {.lex_state = 152, .external_lex_state = 2},
  [1238] = {.lex_state = 152, .external_lex_state = 2},
  [1239] = {.lex_state = 152, .external_lex_state = 2},
  [1240] = {.lex_state = 152, .external_lex_state = 2},
  [1241] = {.lex_state = 152, .external_lex_state = 2},
  [1242] = {.lex_state = 152, .external_lex_state = 2},
  [1243] = {.lex_state = 152, .external_lex_state = 2},
  [1244] = {.lex_state = 152, .external_lex_state = 2},
  [1245] = {.lex_state = 152, .external_lex_state = 2},
  [1246] = {.lex_state = 152, .external_lex_state = 2},
  [1247] = {.lex_state = 152, .external_lex_state = 2},
  [1248] = {.lex_state = 152, .external_lex_state = 2},
  [1249] = {.lex_state = 152, .external_lex_state = 2},
  [1250] = {.lex_state = 152, .external_lex_state = 2},
  [1251] = {.lex_state = 152, .external_lex_state = 2},
  [1252] = {.lex_state = 152, .external_lex_state = 2},
  [1253] = {.lex_state = 152, .external_lex_state = 2},
  [1254] = {.lex_state = 152, .external_lex_state = 2},
  [1255] = {.lex_state = 152, .external_lex_state = 2},
  [1256] = {.lex_state = 152, .external_lex_state = 2},
  [1257] = {.lex_state = 152, .external_lex_state = 2},
  [1258] = {.lex_state = 152, .external_lex_state = 2},
  [1259] = {.lex_state = 152, .external_lex_state = 2},
  [1260] = {.lex_state = 152, .external_lex_state = 2},
  [1261] = {.lex_state = 152, .external_lex_state = 2},
  [1262] = {.lex_state = 152, .external_lex_state = 2},
  [1263] = {.lex_state = 152, .external_lex_state = 2},
  [1264] = {.lex_state = 152, .external_lex_state = 2},
  [1265] = {.lex_state = 152, .external_lex_state = 2},
  [1266] = {.lex_state = 152, .external_lex_state = 2},
  [1267] = {.lex_state = 152, .external_lex_state = 2},
  [1268] = {.lex_state = 152, .external_lex_state = 2},
  [1269] = {.lex_state = 152, .external_lex_state = 2},
  [1270] = {.lex_state = 152, .external_lex_state = 2},
  [1271] = {.lex_state = 152, .external_lex_state = 2},
  [1272] = {.lex_state = 152, .external_lex_state = 2},
  [1273] = {.lex_state = 152, .external_lex_state = 2},
  [1274] = {.lex_state = 152, .external_lex_state = 2},
  [1275] = {.lex_state = 152, .external_lex_state = 2},
  [1276] = {.lex_state = 152, .external_lex_state = 2},
  [1277] = {.lex_state = 152, .external_lex_state = 2},
  [1278] = {.lex_state = 152, .external_lex_state = 2},
  [1279] = {.lex_state = 152, .external_lex_state = 2},
  [1280] = {.lex_state = 152, .external_lex_state = 2},
  [1281] = {.lex_state = 152, .external_lex_state = 2},
  [1282] = {.lex_state = 152, .external_lex_state = 2},
  [1283] = {.lex_state = 152, .external_lex_state = 2},
  [1284] = {.lex_state = 152, .external_lex_state = 2},
  [1285] = {.lex_state = 152, .external_lex_state = 2},
  [1286] = {.lex_state = 152, .external_lex_state = 2},
  [1287] = {.lex_state = 152, .external_lex_state = 2},
  [1288] = {.lex_state = 152, .external_lex_state = 2},
  [1289] = {.lex_state = 152, .external_lex_state = 2},
  [1290] = {.lex_state = 152, .external_lex_state = 2},
  [1291] = {.lex_state = 152, .external_lex_state = 2},
  [1292] = {.lex_state = 152, .external_lex_state = 2},
  [1293] = {.lex_state = 152, .external_lex_state = 2},
  [1294] = {.lex_state = 152, .external_lex_state = 2},
  [1295] = {.lex_state = 152, .external_lex_state = 2},
  [1296] = {.lex_state = 152, .external_lex_state = 2},
  [1297] = {.lex_state = 152, .external_lex_state = 2},
  [1298] = {.lex_state = 152, .external_lex_state = 2},
  [1299] = {.lex_state = 152, .external_lex_state = 2},
  [1300] = {.lex_state = 152, .external_lex_state = 2},
  [1301] = {.lex_state = 152, .external_lex_state = 2},
  [1302] = {.lex_state = 152, .external_lex_state = 2},
  [1303] = {.lex_state = 152, .external_lex_state = 2},
  [1304] = {.lex_state = 152, .external_lex_state = 2},
  [1305] = {.lex_state = 152, .external_lex_state = 2},
  [1306] = {.lex_state = 152, .external_lex_state = 2},
  [1307] = {.lex_state = 152, .external_lex_state = 2},
  [1308] = {.lex_state = 152, .external_lex_state = 2},
  [1309] = {.lex_state = 152, .external_lex_state = 2},
  [1310] = {.lex_state = 152, .external_lex_state = 2},
  [1311] = {.lex_state = 152, .external_lex_state = 2},
  [1312] = {.lex_state = 152, .external_lex_state = 2},
  [1313] = {.lex_state = 152, .external_lex_state = 2},
  [1314] = {.lex_state = 152, .external_lex_state = 2},
  [1315] = {.lex_state = 152, .external_lex_state = 2},
  [1316] = {.lex_state = 152, .external_lex_state = 2},
  [1317] = {.lex_state = 152, .external_lex_state = 2},
  [1318] = {.lex_state = 152, .external_lex_state = 2},
  [1319] = {.lex_state = 152, .external_lex_state = 2},
  [1320] = {.lex_state = 152, .external_lex_state = 2},
  [1321] = {.lex_state = 152, .external_lex_state = 2},
  [1322] = {.lex_state = 152, .external_lex_state = 2},
  [1323] = {.lex_state = 152, .external_lex_state = 2},
  [1324] = {.lex_state = 152, .external_lex_state = 2},
  [1325] = {.lex_state = 152, .external_lex_state = 2},
  [1326] = {.lex_state = 152, .external_lex_state = 2},
  [1327] = {.lex_state = 152, .external_lex_state = 2},
  [1328] = {.lex_state = 152, .external_lex_state = 2},
  [1329] = {.lex_state = 152, .external_lex_state = 2},
  [1330] = {.lex_state = 152, .external_lex_state = 2},
  [1331] = {.lex_state = 152, .external_lex_state = 2},
  [1332] = {.lex_state = 152, .external_lex_state = 2},
  [1333] = {.lex_state = 152, .external_lex_state = 2},
  [1334] = {.lex_state = 152, .external_lex_state = 2},
  [1335] = {.lex_state = 152, .external_lex_state = 2},
  [1336] = {.lex_state = 152, .external_lex_state = 2},
  [1337] = {.lex_state = 152, .external_lex_state = 2},
  [1338] = {.lex_state = 152, .external_lex_state = 2},
  [1339] = {.lex_state = 152, .external_lex_state = 2},
  [1340] = {.lex_state = 152, .external_lex_state = 2},
  [1341] = {.lex_state = 152, .external_lex_state = 2},
  [1342] = {.lex_state = 152, .external_lex_state = 2},
  [1343] = {.lex_state = 152, .external_lex_state = 2},
  [1344] = {.lex_state = 152, .external_lex_state = 2},
  [1345] = {.lex_state = 152, .external_lex_state = 2},
  [1346] = {.lex_state = 152, .external_lex_state = 2},
  [1347] = {.lex_state = 152, .external_lex_state = 2},
  [1348] = {.lex_state = 152, .external_lex_state = 2},
  [1349] = {.lex_state = 152, .external_lex_state = 2},
  [1350] = {.lex_state = 152, .external_lex_state = 2},
  [1351] = {.lex_state = 152, .external_lex_state = 2},
  [1352] = {.lex_state = 152, .external_lex_state = 2},
  [1353] = {.lex_state = 152, .external_lex_state = 2},
  [1354] = {.lex_state = 152, .external_lex_state = 2},
  [1355] = {.lex_state = 152, .external_lex_state = 2},
  [1356] = {.lex_state = 152, .external_lex_state = 2},
  [1357] = {.lex_state = 152, .external_lex_state = 2},
  [1358] = {.lex_state = 152, .external_lex_state = 2},
  [1359] = {.lex_state = 152, .external_lex_state = 2},
  [1360] = {.lex_state = 152, .external_lex_state = 2},
  [1361] = {.lex_state = 152, .external_lex_state = 2},
  [1362] = {.lex_state = 152, .external_lex_state = 2},
  [1363] = {.lex_state = 152, .external_lex_state = 2},
  [1364] = {.lex_state = 152, .external_lex_state = 2},
  [1365] = {.lex_state = 152, .external_lex_state = 2},
  [1366] = {.lex_state = 152, .external_lex_state = 2},
  [1367] = {.lex_state = 152, .external_lex_state = 2},
  [1368] = {.lex_state = 152, .external_lex_state = 2},
  [1369] = {.lex_state = 152, .external_lex_state = 2},
  [1370] = {.lex_state = 152, .external_lex_state = 2},
  [1371] = {.lex_state = 152, .external_lex_state = 2},
  [1372] = {.lex_state = 152, .external_lex_state = 2},
  [1373] = {.lex_state = 152, .external_lex_state = 2},
  [1374] = {.lex_state = 152, .external_lex_state = 2},
  [1375] = {.lex_state = 152, .external_lex_state = 2},
  [1376] = {.lex_state = 152, .external_lex_state = 2},
  [1377] = {.lex_state = 152, .external_lex_state = 2},
  [1378] = {.lex_state = 152, .external_lex_state = 2},
  [1379] = {.lex_state = 152, .external_lex_state = 2},
  [1380] = {.lex_state = 152, .external_lex_state = 2},
  [1381] = {.lex_state = 152, .external_lex_state = 2},
  [1382] = {.lex_state = 152, .external_lex_state = 2},
  [1383] = {.lex_state = 152, .external_lex_state = 2},
  [1384] = {.lex_state = 152, .external_lex_state = 2},
  [1385] = {.lex_state = 152, .external_lex_state = 2},
  [1386] = {.lex_state = 152, .external_lex_state = 2},
  [1387] = {.lex_state = 152, .external_lex_state = 2},
  [1388] = {.lex_state = 152, .external_lex_state = 2},
  [1389] = {.lex_state = 152, .external_lex_state = 2},
  [1390] = {.lex_state = 152, .external_lex_state = 2},
  [1391] = {.lex_state = 152, .external_lex_state = 2},
  [1392] = {.lex_state = 152, .external_lex_state = 2},
  [1393] = {.lex_state = 152, .external_lex_state = 2},
  [1394] = {.lex_state = 152, .external_lex_state = 2},
  [1395] = {.lex_state = 152, .external_lex_state = 2},
  [1396] = {.lex_state = 152, .external_lex_state = 2},
  [1397] = {.lex_state = 152, .external_lex_state = 2},
  [1398] = {.lex_state = 152, .external_lex_state = 2},
  [1399] = {.lex_state = 152, .external_lex_state = 2},
  [1400] = {.lex_state = 152, .external_lex_state = 2},
  [1401] = {.lex_state = 152, .external_lex_state = 2},
  [1402] = {.lex_state = 152, .external_lex_state = 2},
  [1403] = {.lex_state = 152, .external_lex_state = 2},
  [1404] = {.lex_state = 152, .external_lex_state = 2},
  [1405] = {.lex_state = 152, .external_lex_state = 2},
  [1406] = {.lex_state = 152, .external_lex_state = 2},
  [1407] = {.lex_state = 152, .external_lex_state = 2},
  [1408] = {.lex_state = 152, .external_lex_state = 2},
  [1409] = {.lex_state = 152, .external_lex_state = 2},
  [1410] = {.lex_state = 152, .external_lex_state = 2},
  [1411] = {.lex_state = 152, .external_lex_state = 2},
  [1412] = {.lex_state = 152, .external_lex_state = 2},
  [1413] = {.lex_state = 152, .external_lex_state = 2},
  [1414] = {.lex_state = 152, .external_lex_state = 2},
  [1415] = {.lex_state = 152, .external_lex_state = 2},
  [1416] = {.lex_state = 152, .external_lex_state = 2},
  [1417] = {.lex_state = 152, .external_lex_state = 2},
  [1418] = {.lex_state = 152, .external_lex_state = 2},
  [1419] = {.lex_state = 152, .external_lex_state = 2},
  [1420] = {.lex_state = 152, .external_lex_state = 2},
  [1421] = {.lex_state = 152, .external_lex_state = 2},
  [1422] = {.lex_state = 152, .external_lex_state = 2},
  [1423] = {.lex_state = 152, .external_lex_state = 2},
  [1424] = {.lex_state = 152, .external_lex_state = 2},
  [1425] = {.lex_state = 152, .external_lex_state = 2},
  [1426] = {.lex_state = 152, .external_lex_state = 2},
  [1427] = {.lex_state = 152, .external_lex_state = 2},
  [1428] = {.lex_state = 152, .external_lex_state = 2},
  [1429] = {.lex_state = 152, .external_lex_state = 2},
  [1430] = {.lex_state = 152, .external_lex_state = 2},
  [1431] = {.lex_state = 152, .external_lex_state = 2},
  [1432] = {.lex_state = 152, .external_lex_state = 2},
  [1433] = {.lex_state = 152, .external_lex_state = 2},
  [1434] = {.lex_state = 152, .external_lex_state = 2},
  [1435] = {.lex_state = 152, .external_lex_state = 2},
  [1436] = {.lex_state = 152, .external_lex_state = 2},
  [1437] = {.lex_state = 152, .external_lex_state = 2},
  [1438] = {.lex_state = 152, .external_lex_state = 2},
  [1439] = {.lex_state = 152, .external_lex_state = 2},
  [1440] = {.lex_state = 152, .external_lex_state = 2},
  [1441] = {.lex_state = 152, .external_lex_state = 2},
  [1442] = {.lex_state = 152, .external_lex_state = 2},
  [1443] = {.lex_state = 152, .external_lex_state = 2},
  [1444] = {.lex_state = 152, .external_lex_state = 2},
  [1445] = {.lex_state = 152, .external_lex_state = 2},
  [1446] = {.lex_state = 152, .external_lex_state = 2},
  [1447] = {.lex_state = 152, .external_lex_state = 2},
  [1448] = {.lex_state = 152, .external_lex_state = 2},
  [1449] = {.lex_state = 152, .external_lex_state = 2},
  [1450] = {.lex_state = 152, .external_lex_state = 2},
  [1451] = {.lex_state = 152, .external_lex_state = 2},
  [1452] = {.lex_state = 152, .external_lex_state = 2},
  [1453] = {.lex_state = 152, .external_lex_state = 2},
  [1454] = {.lex_state = 152, .external_lex_state = 2},
  [1455] = {.lex_state = 152, .external_lex_state = 2},
  [1456] = {.lex_state = 152, .external_lex_state = 2},
  [1457] = {.lex_state = 152, .external_lex_state = 2},
  [1458] = {.lex_state = 152, .external_lex_state = 2},
  [1459] = {.lex_state = 152, .external_lex_state = 2},
  [1460] = {.lex_state = 152, .external_lex_state = 2},
  [1461] = {.lex_state = 152, .external_lex_state = 2},
  [1462] = {.lex_state = 152, .external_lex_state = 2},
  [1463] = {.lex_state = 152, .external_lex_state = 2},
  [1464] = {.lex_state = 152, .external_lex_state = 2},
  [1465] = {.lex_state = 152, .external_lex_state = 2},
  [1466] = {.lex_state = 152, .external_lex_state = 2},
  [1467] = {.lex_state = 152, .external_lex_state = 2},
  [1468] = {.lex_state = 152, .external_lex_state = 2},
  [1469] = {.lex_state = 152, .external_lex_state = 2},
  [1470] = {.lex_state = 152, .external_lex_state = 2},
  [1471] = {.lex_state = 152, .external_lex_state = 2},
  [1472] = {.lex_state = 152, .external_lex_state = 2},
  [1473] = {.lex_state = 152, .external_lex_state = 2},
  [1474] = {.lex_state = 152, .external_lex_state = 2},
  [1475] = {.lex_state = 152, .external_lex_state = 2},
  [1476] = {.lex_state = 152, .external_lex_state = 2},
  [1477] = {.lex_state = 152, .external_lex_state = 2},
  [1478] = {.lex_state = 152, .external_lex_state = 2},
  [1479] = {.lex_state = 152, .external_lex_state = 2},
  [1480] = {.lex_state = 152, .external_lex_state = 2},
  [1481] = {.lex_state = 152, .external_lex_state = 2},
  [1482] = {.lex_state = 152, .external_lex_state = 2},
  [1483] = {.lex_state = 152, .external_lex_state = 2},
  [1484] = {.lex_state = 152, .external_lex_state = 2},
  [1485] = {.lex_state = 152, .external_lex_state = 2},
  [1486] = {.lex_state = 152, .external_lex_state = 2},
  [1487] = {.lex_state = 152, .external_lex_state = 2},
  [1488] = {.lex_state = 152, .external_lex_state = 2},
  [1489] = {.lex_state = 152, .external_lex_state = 2},
  [1490] = {.lex_state = 152, .external_lex_state = 2},
  [1491] = {.lex_state = 152, .external_lex_state = 2},
  [1492] = {.lex_state = 152, .external_lex_state = 2},
  [1493] = {.lex_state = 152, .external_lex_state = 2},
  [1494] = {.lex_state = 152, .external_lex_state = 2},
  [1495] = {.lex_state = 152, .external_lex_state = 2},
  [1496] = {.lex_state = 152, .external_lex_state = 2},
  [1497] = {.lex_state = 152, .external_lex_state = 2},
  [1498] = {.lex_state = 152, .external_lex_state = 2},
  [1499] = {.lex_state = 152, .external_lex_state = 2},
  [1500] = {.lex_state = 152, .external_lex_state = 2},
  [1501] = {.lex_state = 152, .external_lex_state = 2},
  [1502] = {.lex_state = 152, .external_lex_state = 2},
  [1503] = {.lex_state = 152, .external_lex_state = 2},
  [1504] = {.lex_state = 152, .external_lex_state = 2},
  [1505] = {.lex_state = 152, .external_lex_state = 2},
  [1506] = {.lex_state = 152, .external_lex_state = 2},
  [1507] = {.lex_state = 152, .external_lex_state = 2},
  [1508] = {.lex_state = 152, .external_lex_state = 2},
  [1509] = {.lex_state = 152, .external_lex_state = 2},
  [1510] = {.lex_state = 152, .external_lex_state = 2},
  [1511] = {.lex_state = 152, .external_lex_state = 2},
  [1512] = {.lex_state = 152, .external_lex_state = 2},
  [1513] = {.lex_state = 152, .external_lex_state = 2},
  [1514] = {.lex_state = 152, .external_lex_state = 2},
  [1515] = {.lex_state = 152, .external_lex_state = 2},
  [1516] = {.lex_state = 152, .external_lex_state = 2},
  [1517] = {.lex_state = 152, .external_lex_state = 2},
  [1518] = {.lex_state = 152, .external_lex_state = 2},
  [1519] = {.lex_state = 152, .external_lex_state = 2},
  [1520] = {.lex_state = 152, .external_lex_state = 2},
  [1521] = {.lex_state = 152, .external_lex_state = 2},
  [1522] = {.lex_state = 152, .external_lex_state = 2},
  [1523] = {.lex_state = 152, .external_lex_state = 2},
  [1524] = {.lex_state = 152, .external_lex_state = 2},
  [1525] = {.lex_state = 152, .external_lex_state = 2},
  [1526] = {.lex_state = 152, .external_lex_state = 2},
  [1527] = {.lex_state = 152, .external_lex_state = 2},
  [1528] = {.lex_state = 152, .external_lex_state = 2},
  [1529] = {.lex_state = 152, .external_lex_state = 2},
  [1530] = {.lex_state = 152, .external_lex_state = 2},
  [1531] = {.lex_state = 152, .external_lex_state = 2},
  [1532] = {.lex_state = 152, .external_lex_state = 2},
  [1533] = {.lex_state = 152, .external_lex_state = 2},
  [1534] = {.lex_state = 152, .external_lex_state = 2},
  [1535] = {.lex_state = 152, .external_lex_state = 2},
  [1536] = {.lex_state = 152, .external_lex_state = 2},
  [1537] = {.lex_state = 152, .external_lex_state = 2},
  [1538] = {.lex_state = 152, .external_lex_state = 2},
  [1539] = {.lex_state = 152, .external_lex_state = 2},
  [1540] = {.lex_state = 152, .external_lex_state = 2},
  [1541] = {.lex_state = 152, .external_lex_state = 2},
  [1542] = {.lex_state = 152, .external_lex_state = 2},
  [1543] = {.lex_state = 152, .external_lex_state = 2},
  [1544] = {.lex_state = 152, .external_lex_state = 2},
  [1545] = {.lex_state = 152, .external_lex_state = 2},
  [1546] = {.lex_state = 152, .external_lex_state = 2},
  [1547] = {.lex_state = 152, .external_lex_state = 2},
  [1548] = {.lex_state = 152, .external_lex_state = 2},
  [1549] = {.lex_state = 152, .external_lex_state = 2},
  [1550] = {.lex_state = 152, .external_lex_state = 2},
  [1551] = {.lex_state = 152, .external_lex_state = 2},
  [1552] = {.lex_state = 152, .external_lex_state = 2},
  [1553] = {.lex_state = 152, .external_lex_state = 2},
  [1554] = {.lex_state = 152, .external_lex_state = 2},
  [1555] = {.lex_state = 152, .external_lex_state = 2},
  [1556] = {.lex_state = 152, .external_lex_state = 2},
  [1557] = {.lex_state = 152, .external_lex_state = 2},
  [1558] = {.lex_state = 152, .external_lex_state = 2},
  [1559] = {.lex_state = 152, .external_lex_state = 2},
  [1560] = {.lex_state = 152, .external_lex_state = 2},
  [1561] = {.lex_state = 152, .external_lex_state = 2},
  [1562] = {.lex_state = 152, .external_lex_state = 2},
  [1563] = {.lex_state = 152, .external_lex_state = 2},
  [1564] = {.lex_state = 152, .external_lex_state = 2},
  [1565] = {.lex_state = 152, .external_lex_state = 2},
  [1566] = {.lex_state = 152, .external_lex_state = 2},
  [1567] = {.lex_state = 152, .external_lex_state = 2},
  [1568] = {.lex_state = 152, .external_lex_state = 2},
  [1569] = {.lex_state = 152, .external_lex_state = 2},
  [1570] = {.lex_state = 152, .external_lex_state = 2},
  [1571] = {.lex_state = 152, .external_lex_state = 2},
  [1572] = {.lex_state = 152, .external_lex_state = 2},
  [1573] = {.lex_state = 152, .external_lex_state = 2},
  [1574] = {.lex_state = 152, .external_lex_state = 2},
  [1575] = {.lex_state = 152, .external_lex_state = 2},
  [1576] = {.lex_state = 152, .external_lex_state = 2},
  [1577] = {.lex_state = 152, .external_lex_state = 2},
  [1578] = {.lex_state = 152, .external_lex_state = 2},
  [1579] = {.lex_state = 152, .external_lex_state = 2},
  [1580] = {.lex_state = 152, .external_lex_state = 2},
  [1581] = {.lex_state = 152, .external_lex_state = 2},
  [1582] = {.lex_state = 152, .external_lex_state = 2},
  [1583] = {.lex_state = 152, .external_lex_state = 2},
  [1584] = {.lex_state = 152, .external_lex_state = 2},
  [1585] = {.lex_state = 152, .external_lex_state = 2},
  [1586] = {.lex_state = 152, .external_lex_state = 2},
  [1587] = {.lex_state = 152, .external_lex_state = 2},
  [1588] = {.lex_state = 152, .external_lex_state = 2},
  [1589] = {.lex_state = 152, .external_lex_state = 2},
  [1590] = {.lex_state = 152, .external_lex_state = 2},
  [1591] = {.lex_state = 152, .external_lex_state = 2},
  [1592] = {.lex_state = 152, .external_lex_state = 2},
  [1593] = {.lex_state = 152, .external_lex_state = 2},
  [1594] = {.lex_state = 152, .external_lex_state = 2},
  [1595] = {.lex_state = 152, .external_lex_state = 2},
  [1596] = {.lex_state = 152, .external_lex_state = 2},
  [1597] = {.lex_state = 152, .external_lex_state = 2},
  [1598] = {.lex_state = 152, .external_lex_state = 2},
  [1599] = {.lex_state = 152, .external_lex_state = 2},
  [1600] = {.lex_state = 152, .external_lex_state = 2},
  [1601] = {.lex_state = 152, .external_lex_state = 2},
  [1602] = {.lex_state = 152, .external_lex_state = 2},
  [1603] = {.lex_state = 152, .external_lex_state = 2},
  [1604] = {.lex_state = 152, .external_lex_state = 2},
  [1605] = {.lex_state = 152, .external_lex_state = 2},
  [1606] = {.lex_state = 152, .external_lex_state = 2},
  [1607] = {.lex_state = 152, .external_lex_state = 2},
  [1608] = {.lex_state = 152, .external_lex_state = 2},
  [1609] = {.lex_state = 152, .external_lex_state = 2},
  [1610] = {.lex_state = 152, .external_lex_state = 2},
  [1611] = {.lex_state = 152, .external_lex_state = 2},
  [1612] = {.lex_state = 152, .external_lex_state = 2},
  [1613] = {.lex_state = 152, .external_lex_state = 2},
  [1614] = {.lex_state = 152, .external_lex_state = 2},
  [1615] = {.lex_state = 152, .external_lex_state = 2},
  [1616] = {.lex_state = 152, .external_lex_state = 2},
  [1617] = {.lex_state = 152, .external_lex_state = 2},
  [1618] = {.lex_state = 152, .external_lex_state = 2},
  [1619] = {.lex_state = 152, .external_lex_state = 2},
  [1620] = {.lex_state = 152, .external_lex_state = 2},
  [1621] = {.lex_state = 152, .external_lex_state = 2},
  [1622] = {.lex_state = 152, .external_lex_state = 2},
  [1623] = {.lex_state = 152, .external_lex_state = 2},
  [1624] = {.lex_state = 152, .external_lex_state = 2},
  [1625] = {.lex_state = 152, .external_lex_state = 2},
  [1626] = {.lex_state = 152, .external_lex_state = 2},
  [1627] = {.lex_state = 152, .external_lex_state = 2},
  [1628] = {.lex_state = 152, .external_lex_state = 2},
  [1629] = {.lex_state = 152, .external_lex_state = 2},
  [1630] = {.lex_state = 152, .external_lex_state = 2},
  [1631] = {.lex_state = 152, .external_lex_state = 2},
  [1632] = {.lex_state = 152, .external_lex_state = 2},
  [1633] = {.lex_state = 152, .external_lex_state = 2},
  [1634] = {.lex_state = 152, .external_lex_state = 2},
  [1635] = {.lex_state = 152, .external_lex_state = 2},
  [1636] = {.lex_state = 152, .external_lex_state = 2},
  [1637] = {.lex_state = 152, .external_lex_state = 2},
  [1638] = {.lex_state = 152, .external_lex_state = 2},
  [1639] = {.lex_state = 152, .external_lex_state = 2},
  [1640] = {.lex_state = 152, .external_lex_state = 2},
  [1641] = {.lex_state = 152, .external_lex_state = 2},
  [1642] = {.lex_state = 152, .external_lex_state = 2},
  [1643] = {.lex_state = 152, .external_lex_state = 2},
  [1644] = {.lex_state = 152, .external_lex_state = 2},
  [1645] = {.lex_state = 152, .external_lex_state = 2},
  [1646] = {.lex_state = 152, .external_lex_state = 2},
  [1647] = {.lex_state = 152, .external_lex_state = 2},
  [1648] = {.lex_state = 152, .external_lex_state = 2},
  [1649] = {.lex_state = 152, .external_lex_state = 2},
  [1650] = {.lex_state = 152, .external_lex_state = 2},
  [1651] = {.lex_state = 152, .external_lex_state = 2},
  [1652] = {.lex_state = 152, .external_lex_state = 2},
  [1653] = {.lex_state = 152, .external_lex_state = 2},
  [1654] = {.lex_state = 152, .external_lex_state = 2},
  [1655] = {.lex_state = 152, .external_lex_state = 2},
  [1656] = {.lex_state = 152, .external_lex_state = 2},
  [1657] = {.lex_state = 152, .external_lex_state = 2},
  [1658] = {.lex_state = 152, .external_lex_state = 2},
  [1659] = {.lex_state = 152, .external_lex_state = 2},
  [1660] = {.lex_state = 152, .external_lex_state = 2},
  [1661] = {.lex_state = 152, .external_lex_state = 2},
  [1662] = {.lex_state = 152, .external_lex_state = 2},
  [1663] = {.lex_state = 152, .external_lex_state = 2},
  [1664] = {.lex_state = 152, .external_lex_state = 2},
  [1665] = {.lex_state = 152, .external_lex_state = 2},
  [1666] = {.lex_state = 152, .external_lex_state = 2},
  [1667] = {.lex_state = 152, .external_lex_state = 2},
  [1668] = {.lex_state = 152, .external_lex_state = 2},
  [1669] = {.lex_state = 152, .external_lex_state = 2},
  [1670] = {.lex_state = 152, .external_lex_state = 2},
  [1671] = {.lex_state = 152, .external_lex_state = 2},
  [1672] = {.lex_state = 152, .external_lex_state = 2},
  [1673] = {.lex_state = 152, .external_lex_state = 2},
  [1674] = {.lex_state = 152, .external_lex_state = 2},
  [1675] = {.lex_state = 152, .external_lex_state = 2},
  [1676] = {.lex_state = 152, .external_lex_state = 2},
  [1677] = {.lex_state = 152, .external_lex_state = 2},
  [1678] = {.lex_state = 152, .external_lex_state = 2},
  [1679] = {.lex_state = 152, .external_lex_state = 2},
  [1680] = {.lex_state = 152, .external_lex_state = 2},
  [1681] = {.lex_state = 152, .external_lex_state = 2},
  [1682] = {.lex_state = 152, .external_lex_state = 2},
  [1683] = {.lex_state = 152, .external_lex_state = 2},
  [1684] = {.lex_state = 152, .external_lex_state = 2},
  [1685] = {.lex_state = 152, .external_lex_state = 2},
  [1686] = {.lex_state = 152, .external_lex_state = 2},
  [1687] = {.lex_state = 152, .external_lex_state = 2},
  [1688] = {.lex_state = 152, .external_lex_state = 2},
  [1689] = {.lex_state = 152, .external_lex_state = 2},
  [1690] = {.lex_state = 152, .external_lex_state = 2},
  [1691] = {.lex_state = 152, .external_lex_state = 2},
  [1692] = {.lex_state = 152, .external_lex_state = 2},
  [1693] = {.lex_state = 152, .external_lex_state = 2},
  [1694] = {.lex_state = 152, .external_lex_state = 2},
  [1695] = {.lex_state = 152, .external_lex_state = 2},
  [1696] = {.lex_state = 152, .external_lex_state = 2},
  [1697] = {.lex_state = 152, .external_lex_state = 2},
  [1698] = {.lex_state = 152, .external_lex_state = 2},
  [1699] = {.lex_state = 152, .external_lex_state = 2},
  [1700] = {.lex_state = 152, .external_lex_state = 2},
  [1701] = {.lex_state = 152, .external_lex_state = 2},
  [1702] = {.lex_state = 152, .external_lex_state = 2},
  [1703] = {.lex_state = 152, .external_lex_state = 2},
  [1704] = {.lex_state = 152, .external_lex_state = 2},
  [1705] = {.lex_state = 152, .external_lex_state = 2},
  [1706] = {.lex_state = 152, .external_lex_state = 2},
  [1707] = {.lex_state = 152, .external_lex_state = 2},
  [1708] = {.lex_state = 152, .external_lex_state = 2},
  [1709] = {.lex_state = 152, .external_lex_state = 2},
  [1710] = {.lex_state = 152, .external_lex_state = 2},
  [1711] = {.lex_state = 152, .external_lex_state = 2},
  [1712] = {.lex_state = 152, .external_lex_state = 2},
  [1713] = {.lex_state = 152, .external_lex_state = 2},
  [1714] = {.lex_state = 144, .external_lex_state = 3},
  [1715] = {.lex_state = 152, .external_lex_state = 2},
  [1716] = {.lex_state = 152, .external_lex_state = 2},
  [1717] = {.lex_state = 152, .external_lex_state = 2},
  [1718] = {.lex_state = 152, .external_lex_state = 2},
  [1719] = {.lex_state = 152, .external_lex_state = 2},
  [1720] = {.lex_state = 152, .external_lex_state = 2},
  [1721] = {.lex_state = 152, .external_lex_state = 2},
  [1722] = {.lex_state = 152, .external_lex_state = 2},
  [1723] = {.lex_state = 152, .external_lex_state = 2},
  [1724] = {.lex_state = 152, .external_lex_state = 2},
  [1725] = {.lex_state = 152, .external_lex_state = 2},
  [1726] = {.lex_state = 152, .external_lex_state = 2},
  [1727] = {.lex_state = 152, .external_lex_state = 2},
  [1728] = {.lex_state = 152, .external_lex_state = 2},
  [1729] = {.lex_state = 152, .external_lex_state = 2},
  [1730] = {.lex_state = 152, .external_lex_state = 2},
  [1731] = {.lex_state = 152, .external_lex_state = 2},
  [1732] = {.lex_state = 152, .external_lex_state = 2},
  [1733] = {.lex_state = 152, .external_lex_state = 2},
  [1734] = {.lex_state = 152, .external_lex_state = 2},
  [1735] = {.lex_state = 152, .external_lex_state = 2},
  [1736] = {.lex_state = 152, .external_lex_state = 2},
  [1737] = {.lex_state = 152, .external_lex_state = 2},
  [1738] = {.lex_state = 152, .external_lex_state = 2},
  [1739] = {.lex_state = 152, .external_lex_state = 2},
  [1740] = {.lex_state = 152, .external_lex_state = 2},
  [1741] = {.lex_state = 152, .external_lex_state = 2},
  [1742] = {.lex_state = 152, .external_lex_state = 2},
  [1743] = {.lex_state = 152, .external_lex_state = 2},
  [1744] = {.lex_state = 152, .external_lex_state = 2},
  [1745] = {.lex_state = 152, .external_lex_state = 2},
  [1746] = {.lex_state = 152, .external_lex_state = 2},
  [1747] = {.lex_state = 152, .external_lex_state = 2},
  [1748] = {.lex_state = 152, .external_lex_state = 2},
  [1749] = {.lex_state = 152, .external_lex_state = 2},
  [1750] = {.lex_state = 152, .external_lex_state = 2},
  [1751] = {.lex_state = 152, .external_lex_state = 2},
  [1752] = {.lex_state = 152, .external_lex_state = 2},
  [1753] = {.lex_state = 152, .external_lex_state = 2},
  [1754] = {.lex_state = 152, .external_lex_state = 2},
  [1755] = {.lex_state = 152, .external_lex_state = 2},
  [1756] = {.lex_state = 152, .external_lex_state = 2},
  [1757] = {.lex_state = 152, .external_lex_state = 2},
  [1758] = {.lex_state = 152, .external_lex_state = 2},
  [1759] = {.lex_state = 152, .external_lex_state = 2},
  [1760] = {.lex_state = 152, .external_lex_state = 2},
  [1761] = {.lex_state = 152, .external_lex_state = 2},
  [1762] = {.lex_state = 152, .external_lex_state = 2},
  [1763] = {.lex_state = 152, .external_lex_state = 2},
  [1764] = {.lex_state = 152, .external_lex_state = 2},
  [1765] = {.lex_state = 152, .external_lex_state = 2},
  [1766] = {.lex_state = 152, .external_lex_state = 2},
  [1767] = {.lex_state = 152, .external_lex_state = 2},
  [1768] = {.lex_state = 152, .external_lex_state = 2},
  [1769] = {.lex_state = 152, .external_lex_state = 2},
  [1770] = {.lex_state = 152, .external_lex_state = 2},
  [1771] = {.lex_state = 152, .external_lex_state = 2},
  [1772] = {.lex_state = 152, .external_lex_state = 2},
  [1773] = {.lex_state = 152, .external_lex_state = 2},
  [1774] = {.lex_state = 152, .external_lex_state = 2},
  [1775] = {.lex_state = 152, .external_lex_state = 2},
  [1776] = {.lex_state = 152, .external_lex_state = 2},
  [1777] = {.lex_state = 152, .external_lex_state = 2},
  [1778] = {.lex_state = 152, .external_lex_state = 2},
  [1779] = {.lex_state = 152, .external_lex_state = 2},
  [1780] = {.lex_state = 152, .external_lex_state = 2},
  [1781] = {.lex_state = 152, .external_lex_state = 2},
  [1782] = {.lex_state = 152, .external_lex_state = 2},
  [1783] = {.lex_state = 152, .external_lex_state = 2},
  [1784] = {.lex_state = 152, .external_lex_state = 2},
  [1785] = {.lex_state = 152, .external_lex_state = 2},
  [1786] = {.lex_state = 152, .external_lex_state = 2},
  [1787] = {.lex_state = 152, .external_lex_state = 2},
  [1788] = {.lex_state = 152, .external_lex_state = 2},
  [1789] = {.lex_state = 152, .external_lex_state = 2},
  [1790] = {.lex_state = 152, .external_lex_state = 2},
  [1791] = {.lex_state = 152, .external_lex_state = 2},
  [1792] = {.lex_state = 152, .external_lex_state = 2},
  [1793] = {.lex_state = 152, .external_lex_state = 2},
  [1794] = {.lex_state = 152, .external_lex_state = 2},
  [1795] = {.lex_state = 152, .external_lex_state = 2},
  [1796] = {.lex_state = 152, .external_lex_state = 2},
  [1797] = {.lex_state = 152, .external_lex_state = 2},
  [1798] = {.lex_state = 152, .external_lex_state = 2},
  [1799] = {.lex_state = 152, .external_lex_state = 2},
  [1800] = {.lex_state = 152, .external_lex_state = 2},
  [1801] = {.lex_state = 152, .external_lex_state = 2},
  [1802] = {.lex_state = 152, .external_lex_state = 2},
  [1803] = {.lex_state = 152, .external_lex_state = 2},
  [1804] = {.lex_state = 152, .external_lex_state = 2},
  [1805] = {.lex_state = 152, .external_lex_state = 2},
  [1806] = {.lex_state = 152, .external_lex_state = 2},
  [1807] = {.lex_state = 152, .external_lex_state = 2},
  [1808] = {.lex_state = 152, .external_lex_state = 2},
  [1809] = {.lex_state = 152, .external_lex_state = 2},
  [1810] = {.lex_state = 152, .external_lex_state = 2},
  [1811] = {.lex_state = 152, .external_lex_state = 2},
  [1812] = {.lex_state = 152, .external_lex_state = 2},
  [1813] = {.lex_state = 152, .external_lex_state = 2},
  [1814] = {.lex_state = 152, .external_lex_state = 2},
  [1815] = {.lex_state = 152, .external_lex_state = 2},
  [1816] = {.lex_state = 152, .external_lex_state = 2},
  [1817] = {.lex_state = 152, .external_lex_state = 2},
  [1818] = {.lex_state = 152, .external_lex_state = 2},
  [1819] = {.lex_state = 152, .external_lex_state = 2},
  [1820] = {.lex_state = 152, .external_lex_state = 2},
  [1821] = {.lex_state = 152, .external_lex_state = 2},
  [1822] = {.lex_state = 152, .external_lex_state = 2},
  [1823] = {.lex_state = 152, .external_lex_state = 2},
  [1824] = {.lex_state = 152, .external_lex_state = 2},
  [1825] = {.lex_state = 152, .external_lex_state = 2},
  [1826] = {.lex_state = 152, .external_lex_state = 2},
  [1827] = {.lex_state = 152, .external_lex_state = 2},
  [1828] = {.lex_state = 152, .external_lex_state = 2},
  [1829] = {.lex_state = 152, .external_lex_state = 2},
  [1830] = {.lex_state = 152, .external_lex_state = 2},
  [1831] = {.lex_state = 152, .external_lex_state = 2},
  [1832] = {.lex_state = 152, .external_lex_state = 2},
  [1833] = {.lex_state = 152, .external_lex_state = 2},
  [1834] = {.lex_state = 152, .external_lex_state = 2},
  [1835] = {.lex_state = 152, .external_lex_state = 2},
  [1836] = {.lex_state = 152, .external_lex_state = 2},
  [1837] = {.lex_state = 152, .external_lex_state = 2},
  [1838] = {.lex_state = 152, .external_lex_state = 2},
  [1839] = {.lex_state = 152, .external_lex_state = 2},
  [1840] = {.lex_state = 152, .external_lex_state = 2},
  [1841] = {.lex_state = 152, .external_lex_state = 2},
  [1842] = {.lex_state = 152, .external_lex_state = 2},
  [1843] = {.lex_state = 152, .external_lex_state = 2},
  [1844] = {.lex_state = 152, .external_lex_state = 2},
  [1845] = {.lex_state = 152, .external_lex_state = 2},
  [1846] = {.lex_state = 152, .external_lex_state = 2},
  [1847] = {.lex_state = 152, .external_lex_state = 2},
  [1848] = {.lex_state = 152, .external_lex_state = 2},
  [1849] = {.lex_state = 152, .external_lex_state = 2},
  [1850] = {.lex_state = 152, .external_lex_state = 2},
  [1851] = {.lex_state = 152, .external_lex_state = 2},
  [1852] = {.lex_state = 152, .external_lex_state = 2},
  [1853] = {.lex_state = 152, .external_lex_state = 2},
  [1854] = {.lex_state = 152, .external_lex_state = 2},
  [1855] = {.lex_state = 152, .external_lex_state = 2},
  [1856] = {.lex_state = 152, .external_lex_state = 2},
  [1857] = {.lex_state = 152, .external_lex_state = 2},
  [1858] = {.lex_state = 152, .external_lex_state = 2},
  [1859] = {.lex_state = 152, .external_lex_state = 2},
  [1860] = {.lex_state = 152, .external_lex_state = 2},
  [1861] = {.lex_state = 152, .external_lex_state = 2},
  [1862] = {.lex_state = 152, .external_lex_state = 2},
  [1863] = {.lex_state = 152, .external_lex_state = 2},
  [1864] = {.lex_state = 152, .external_lex_state = 2},
  [1865] = {.lex_state = 152, .external_lex_state = 2},
  [1866] = {.lex_state = 152, .external_lex_state = 2},
  [1867] = {.lex_state = 152, .external_lex_state = 2},
  [1868] = {.lex_state = 152, .external_lex_state = 2},
  [1869] = {.lex_state = 152, .external_lex_state = 2},
  [1870] = {.lex_state = 152, .external_lex_state = 2},
  [1871] = {.lex_state = 152, .external_lex_state = 2},
  [1872] = {.lex_state = 152, .external_lex_state = 2},
  [1873] = {.lex_state = 152, .external_lex_state = 2},
  [1874] = {.lex_state = 152, .external_lex_state = 2},
  [1875] = {.lex_state = 152, .external_lex_state = 2},
  [1876] = {.lex_state = 152, .external_lex_state = 2},
  [1877] = {.lex_state = 152, .external_lex_state = 2},
  [1878] = {.lex_state = 152, .external_lex_state = 2},
  [1879] = {.lex_state = 152, .external_lex_state = 2},
  [1880] = {.lex_state = 152, .external_lex_state = 2},
  [1881] = {.lex_state = 152, .external_lex_state = 2},
  [1882] = {.lex_state = 152, .external_lex_state = 2},
  [1883] = {.lex_state = 152, .external_lex_state = 2},
  [1884] = {.lex_state = 152, .external_lex_state = 2},
  [1885] = {.lex_state = 152, .external_lex_state = 2},
  [1886] = {.lex_state = 152, .external_lex_state = 2},
  [1887] = {.lex_state = 152, .external_lex_state = 2},
  [1888] = {.lex_state = 152, .external_lex_state = 2},
  [1889] = {.lex_state = 152, .external_lex_state = 2},
  [1890] = {.lex_state = 152, .external_lex_state = 2},
  [1891] = {.lex_state = 152, .external_lex_state = 2},
  [1892] = {.lex_state = 152, .external_lex_state = 2},
  [1893] = {.lex_state = 152, .external_lex_state = 2},
  [1894] = {.lex_state = 152, .external_lex_state = 2},
  [1895] = {.lex_state = 152, .external_lex_state = 2},
  [1896] = {.lex_state = 152, .external_lex_state = 2},
  [1897] = {.lex_state = 152, .external_lex_state = 2},
  [1898] = {.lex_state = 152, .external_lex_state = 2},
  [1899] = {.lex_state = 152, .external_lex_state = 2},
  [1900] = {.lex_state = 152, .external_lex_state = 2},
  [1901] = {.lex_state = 152, .external_lex_state = 2},
  [1902] = {.lex_state = 152, .external_lex_state = 2},
  [1903] = {.lex_state = 152, .external_lex_state = 2},
  [1904] = {.lex_state = 152, .external_lex_state = 2},
  [1905] = {.lex_state = 152, .external_lex_state = 2},
  [1906] = {.lex_state = 144, .external_lex_state = 3},
  [1907] = {.lex_state = 144, .external_lex_state = 3},
  [1908] = {.lex_state = 144, .external_lex_state = 3},
  [1909] = {.lex_state = 144, .external_lex_state = 3},
  [1910] = {.lex_state = 144, .external_lex_state = 3},
  [1911] = {.lex_state = 144, .external_lex_state = 3},
  [1912] = {.lex_state = 144, .external_lex_state = 3},
  [1913] = {.lex_state = 144, .external_lex_state = 3},
  [1914] = {.lex_state = 144, .external_lex_state = 3},
  [1915] = {.lex_state = 144, .external_lex_state = 3},
  [1916] = {.lex_state = 161, .external_lex_state = 18},
  [1917] = {.lex_state = 144, .external_lex_state = 3},
  [1918] = {.lex_state = 144, .external_lex_state = 3},
  [1919] = {.lex_state = 144, .external_lex_state = 3},
  [1920] = {.lex_state = 144, .external_lex_state = 3},
  [1921] = {.lex_state = 144, .external_lex_state = 3},
  [1922] = {.lex_state = 30, .external_lex_state = 3},
  [1923] = {.lex_state = 144, .external_lex_state = 3},
  [1924] = {.lex_state = 144, .external_lex_state = 3},
  [1925] = {.lex_state = 144, .external_lex_state = 3},
  [1926] = {.lex_state = 144, .external_lex_state = 3},
  [1927] = {.lex_state = 144, .external_lex_state = 3},
  [1928] = {.lex_state = 144, .external_lex_state = 3},
  [1929] = {.lex_state = 144, .external_lex_state = 3},
  [1930] = {.lex_state = 144, .external_lex_state = 3},
  [1931] = {.lex_state = 144, .external_lex_state = 3},
  [1932] = {.lex_state = 144, .external_lex_state = 3},
  [1933] = {.lex_state = 144, .external_lex_state = 4},
  [1934] = {.lex_state = 144, .external_lex_state = 3},
  [1935] = {.lex_state = 144, .external_lex_state = 4},
  [1936] = {.lex_state = 144, .external_lex_state = 3},
  [1937] = {.lex_state = 144, .external_lex_state = 3},
  [1938] = {.lex_state = 144, .external_lex_state = 3},
  [1939] = {.lex_state = 144, .external_lex_state = 3},
  [1940] = {.lex_state = 144, .external_lex_state = 4},
  [1941] = {.lex_state = 144, .external_lex_state = 4},
  [1942] = {.lex_state = 144, .external_lex_state = 4},
  [1943] = {.lex_state = 144, .external_lex_state = 4},
  [1944] = {.lex_state = 161, .external_lex_state = 18},
  [1945] = {.lex_state = 30, .external_lex_state = 4},
  [1946] = {.lex_state = 144, .external_lex_state = 4},
  [1947] = {.lex_state = 144, .external_lex_state = 4},
  [1948] = {.lex_state = 144, .external_lex_state = 3},
  [1949] = {.lex_state = 144, .external_lex_state = 4},
  [1950] = {.lex_state = 144, .external_lex_state = 4},
  [1951] = {.lex_state = 144, .external_lex_state = 4},
  [1952] = {.lex_state = 144, .external_lex_state = 4},
  [1953] = {.lex_state = 144, .external_lex_state = 4},
  [1954] = {.lex_state = 144, .external_lex_state = 4},
  [1955] = {.lex_state = 144, .external_lex_state = 4},
  [1956] = {.lex_state = 144, .external_lex_state = 4},
  [1957] = {.lex_state = 144, .external_lex_state = 4},
  [1958] = {.lex_state = 161, .external_lex_state = 19},
  [1959] = {.lex_state = 161, .external_lex_state = 19},
  [1960] = {.lex_state = 161, .external_lex_state = 19},
  [1961] = {.lex_state = 152, .external_lex_state = 19},
  [1962] = {.lex_state = 152, .external_lex_state = 19},
  [1963] = {.lex_state = 152, .external_lex_state = 19},
  [1964] = {.lex_state = 146, .external_lex_state = 3},
  [1965] = {.lex_state = 146, .external_lex_state = 3},
  [1966] = {.lex_state = 146, .external_lex_state = 3},
  [1967] = {.lex_state = 146, .external_lex_state = 3},
  [1968] = {.lex_state = 146, .external_lex_state = 3},
  [1969] = {.lex_state = 146, .external_lex_state = 3},
  [1970] = {.lex_state = 146, .external_lex_state = 3},
  [1971] = {.lex_state = 146, .external_lex_state = 3},
  [1972] = {.lex_state = 161, .external_lex_state = 19},
  [1973] = {.lex_state = 152, .external_lex_state = 19},
  [1974] = {.lex_state = 152, .external_lex_state = 20},
  [1975] = {.lex_state = 152, .external_lex_state = 20},
  [1976] = {.lex_state = 152, .external_lex_state = 20},
  [1977] = {.lex_state = 152, .external_lex_state = 19},
  [1978] = {.lex_state = 152, .external_lex_state = 19},
  [1979] = {.lex_state = 152, .external_lex_state = 19},
  [1980] = {.lex_state = 152, .external_lex_state = 19},
  [1981] = {.lex_state = 146, .external_lex_state = 3},
  [1982] = {.lex_state = 146, .external_lex_state = 3},
  [1983] = {.lex_state = 146, .external_lex_state = 3},
  [1984] = {.lex_state = 146, .external_lex_state = 3},
  [1985] = {.lex_state = 146, .external_lex_state = 3},
  [1986] = {.lex_state = 146, .external_lex_state = 3},
  [1987] = {.lex_state = 146, .external_lex_state = 3},
  [1988] = {.lex_state = 146, .external_lex_state = 3},
  [1989] = {.lex_state = 146, .external_lex_state = 4},
  [1990] = {.lex_state = 146, .external_lex_state = 3},
  [1991] = {.lex_state = 146, .external_lex_state = 4},
  [1992] = {.lex_state = 146, .external_lex_state = 3},
  [1993] = {.lex_state = 146, .external_lex_state = 3},
  [1994] = {.lex_state = 146, .external_lex_state = 4},
  [1995] = {.lex_state = 146, .external_lex_state = 3},
  [1996] = {.lex_state = 146, .external_lex_state = 3},
  [1997] = {.lex_state = 146, .external_lex_state = 3},
  [1998] = {.lex_state = 146, .external_lex_state = 3},
  [1999] = {.lex_state = 146, .external_lex_state = 3},
  [2000] = {.lex_state = 146, .external_lex_state = 4},
  [2001] = {.lex_state = 146, .external_lex_state = 4},
  [2002] = {.lex_state = 146, .external_lex_state = 4},
  [2003] = {.lex_state = 146, .external_lex_state = 4},
  [2004] = {.lex_state = 146, .external_lex_state = 4},
  [2005] = {.lex_state = 146, .external_lex_state = 4},
  [2006] = {.lex_state = 146, .external_lex_state = 4},
  [2007] = {.lex_state = 146, .external_lex_state = 4},
  [2008] = {.lex_state = 146, .external_lex_state = 4},
  [2009] = {.lex_state = 146, .external_lex_state = 4},
  [2010] = {.lex_state = 146, .external_lex_state = 4},
  [2011] = {.lex_state = 160, .external_lex_state = 21},
  [2012] = {.lex_state = 160, .external_lex_state = 21},
  [2013] = {.lex_state = 160, .external_lex_state = 21},
  [2014] = {.lex_state = 169, .external_lex_state = 19},
  [2015] = {.lex_state = 169, .external_lex_state = 19},
  [2016] = {.lex_state = 169, .external_lex_state = 19},
  [2017] = {.lex_state = 169, .external_lex_state = 19},
  [2018] = {.lex_state = 169, .external_lex_state = 19},
  [2019] = {.lex_state = 169, .external_lex_state = 19},
  [2020] = {.lex_state = 169, .external_lex_state = 19},
  [2021] = {.lex_state = 169, .external_lex_state = 19},
  [2022] = {.lex_state = 169, .external_lex_state = 19},
  [2023] = {.lex_state = 169, .external_lex_state = 19},
  [2024] = {.lex_state = 169, .external_lex_state = 19},
  [2025] = {.lex_state = 169, .external_lex_state = 19},
  [2026] = {.lex_state = 169, .external_lex_state = 19},
  [2027] = {.lex_state = 169, .external_lex_state = 19},
  [2028] = {.lex_state = 169, .external_lex_state = 19},
  [2029] = {.lex_state = 160, .external_lex_state = 21},
  [2030] = {.lex_state = 152, .external_lex_state = 9},
  [2031] = {.lex_state = 152, .external_lex_state = 9},
  [2032] = {.lex_state = 152, .external_lex_state = 9},
  [2033] = {.lex_state = 152, .external_lex_state = 9},
  [2034] = {.lex_state = 152, .external_lex_state = 9},
  [2035] = {.lex_state = 152, .external_lex_state = 9},
  [2036] = {.lex_state = 160, .external_lex_state = 21},
  [2037] = {.lex_state = 160, .external_lex_state = 21},
  [2038] = {.lex_state = 160, .external_lex_state = 21},
  [2039] = {.lex_state = 38, .external_lex_state = 6},
  [2040] = {.lex_state = 160, .external_lex_state = 22},
  [2041] = {.lex_state = 151, .external_lex_state = 2},
  [2042] = {.lex_state = 40, .external_lex_state = 6},
  [2043] = {.lex_state = 153, .external_lex_state = 23},
  [2044] = {.lex_state = 153, .external_lex_state = 23},
  [2045] = {.lex_state = 160, .external_lex_state = 22},
  [2046] = {.lex_state = 153, .external_lex_state = 23},
  [2047] = {.lex_state = 160, .external_lex_state = 21},
  [2048] = {.lex_state = 153, .external_lex_state = 23},
  [2049] = {.lex_state = 160, .external_lex_state = 21},
  [2050] = {.lex_state = 151, .external_lex_state = 2},
  [2051] = {.lex_state = 151, .external_lex_state = 2},
  [2052] = {.lex_state = 153, .external_lex_state = 23},
  [2053] = {.lex_state = 151, .external_lex_state = 2},
  [2054] = {.lex_state = 152, .external_lex_state = 9},
  [2055] = {.lex_state = 160, .external_lex_state = 22},
  [2056] = {.lex_state = 152, .external_lex_state = 9},
  [2057] = {.lex_state = 153, .external_lex_state = 23},
  [2058] = {.lex_state = 160, .external_lex_state = 22},
  [2059] = {.lex_state = 153, .external_lex_state = 23},
  [2060] = {.lex_state = 160, .external_lex_state = 22},
  [2061] = {.lex_state = 153, .external_lex_state = 23},
  [2062] = {.lex_state = 152, .external_lex_state = 9},
  [2063] = {.lex_state = 152, .external_lex_state = 9},
  [2064] = {.lex_state = 160, .external_lex_state = 22},
  [2065] = {.lex_state = 152, .external_lex_state = 9},
  [2066] = {.lex_state = 152, .external_lex_state = 9},
  [2067] = {.lex_state = 153, .external_lex_state = 23},
  [2068] = {.lex_state = 153, .external_lex_state = 23},
  [2069] = {.lex_state = 153, .external_lex_state = 23},
  [2070] = {.lex_state = 153, .external_lex_state = 23},
  [2071] = {.lex_state = 160, .external_lex_state = 22},
  [2072] = {.lex_state = 153, .external_lex_state = 23},
  [2073] = {.lex_state = 153, .external_lex_state = 23},
  [2074] = {.lex_state = 153, .external_lex_state = 23},
  [2075] = {.lex_state = 153, .external_lex_state = 23},
  [2076] = {.lex_state = 160, .external_lex_state = 22},
  [2077] = {.lex_state = 153, .external_lex_state = 23},
  [2078] = {.lex_state = 153, .external_lex_state = 23},
  [2079] = {.lex_state = 153, .external_lex_state = 23},
  [2080] = {.lex_state = 153, .external_lex_state = 23},
  [2081] = {.lex_state = 153, .external_lex_state = 23},
  [2082] = {.lex_state = 153, .external_lex_state = 23},
  [2083] = {.lex_state = 152, .external_lex_state = 5},
  [2084] = {.lex_state = 41, .external_lex_state = 7},
  [2085] = {.lex_state = 153, .external_lex_state = 23},
  [2086] = {.lex_state = 152, .external_lex_state = 5},
  [2087] = {.lex_state = 152, .external_lex_state = 5},
  [2088] = {.lex_state = 153, .external_lex_state = 23},
  [2089] = {.lex_state = 152, .external_lex_state = 5},
  [2090] = {.lex_state = 153, .external_lex_state = 23},
  [2091] = {.lex_state = 153, .external_lex_state = 23},
  [2092] = {.lex_state = 153, .external_lex_state = 23},
  [2093] = {.lex_state = 153, .external_lex_state = 23},
  [2094] = {.lex_state = 153, .external_lex_state = 23},
  [2095] = {.lex_state = 153, .external_lex_state = 23},
  [2096] = {.lex_state = 153, .external_lex_state = 23},
  [2097] = {.lex_state = 153, .external_lex_state = 23},
  [2098] = {.lex_state = 153, .external_lex_state = 23},
  [2099] = {.lex_state = 153, .external_lex_state = 23},
  [2100] = {.lex_state = 153, .external_lex_state = 23},
  [2101] = {.lex_state = 153, .external_lex_state = 23},
  [2102] = {.lex_state = 152, .external_lex_state = 5},
  [2103] = {.lex_state = 153, .external_lex_state = 23},
  [2104] = {.lex_state = 152, .external_lex_state = 5},
  [2105] = {.lex_state = 41, .external_lex_state = 7},
  [2106] = {.lex_state = 153, .external_lex_state = 23},
  [2107] = {.lex_state = 153, .external_lex_state = 23},
  [2108] = {.lex_state = 153, .external_lex_state = 23},
  [2109] = {.lex_state = 41, .external_lex_state = 7},
  [2110] = {.lex_state = 154, .external_lex_state = 23},
  [2111] = {.lex_state = 153, .external_lex_state = 23},
  [2112] = {.lex_state = 153, .external_lex_state = 23},
  [2113] = {.lex_state = 153, .external_lex_state = 23},
  [2114] = {.lex_state = 153, .external_lex_state = 23},
  [2115] = {.lex_state = 151, .external_lex_state = 7},
  [2116] = {.lex_state = 151, .external_lex_state = 7},
  [2117] = {.lex_state = 153, .external_lex_state = 23},
  [2118] = {.lex_state = 151, .external_lex_state = 7},
  [2119] = {.lex_state = 151, .external_lex_state = 7},
  [2120] = {.lex_state = 151, .external_lex_state = 7},
  [2121] = {.lex_state = 151, .external_lex_state = 2},
  [2122] = {.lex_state = 151, .external_lex_state = 7},
  [2123] = {.lex_state = 151, .external_lex_state = 7},
  [2124] = {.lex_state = 153, .external_lex_state = 23},
  [2125] = {.lex_state = 151, .external_lex_state = 7},
  [2126] = {.lex_state = 151, .external_lex_state = 7},
  [2127] = {.lex_state = 151, .external_lex_state = 7},
  [2128] = {.lex_state = 151, .external_lex_state = 7},
  [2129] = {.lex_state = 151, .external_lex_state = 7},
  [2130] = {.lex_state = 153, .external_lex_state = 23},
  [2131] = {.lex_state = 153, .external_lex_state = 23},
  [2132] = {.lex_state = 153, .external_lex_state = 23},
  [2133] = {.lex_state = 160, .external_lex_state = 22},
  [2134] = {.lex_state = 160, .external_lex_state = 22},
  [2135] = {.lex_state = 160, .external_lex_state = 22},
  [2136] = {.lex_state = 153, .external_lex_state = 23},
  [2137] = {.lex_state = 160, .external_lex_state = 22},
  [2138] = {.lex_state = 160, .external_lex_state = 22},
  [2139] = {.lex_state = 153, .external_lex_state = 23},
  [2140] = {.lex_state = 153, .external_lex_state = 23},
  [2141] = {.lex_state = 153, .external_lex_state = 23},
  [2142] = {.lex_state = 151, .external_lex_state = 2},
  [2143] = {.lex_state = 151, .external_lex_state = 2},
  [2144] = {.lex_state = 151, .external_lex_state = 2},
  [2145] = {.lex_state = 151, .external_lex_state = 2},
  [2146] = {.lex_state = 153, .external_lex_state = 23},
  [2147] = {.lex_state = 160, .external_lex_state = 22},
  [2148] = {.lex_state = 151, .external_lex_state = 2},
  [2149] = {.lex_state = 151, .external_lex_state = 2},
  [2150] = {.lex_state = 151, .external_lex_state = 2},
  [2151] = {.lex_state = 151, .external_lex_state = 2},
  [2152] = {.lex_state = 160, .external_lex_state = 22},
  [2153] = {.lex_state = 160, .external_lex_state = 22},
  [2154] = {.lex_state = 151, .external_lex_state = 2},
  [2155] = {.lex_state = 160, .external_lex_state = 22},
  [2156] = {.lex_state = 151, .external_lex_state = 2},
  [2157] = {.lex_state = 160, .external_lex_state = 22},
  [2158] = {.lex_state = 151, .external_lex_state = 2},
  [2159] = {.lex_state = 151, .external_lex_state = 2},
  [2160] = {.lex_state = 151, .external_lex_state = 2},
  [2161] = {.lex_state = 160, .external_lex_state = 22},
  [2162] = {.lex_state = 151, .external_lex_state = 2},
  [2163] = {.lex_state = 151, .external_lex_state = 2},
  [2164] = {.lex_state = 160, .external_lex_state = 22},
  [2165] = {.lex_state = 151, .external_lex_state = 2},
  [2166] = {.lex_state = 153, .external_lex_state = 23},
  [2167] = {.lex_state = 160, .external_lex_state = 21},
  [2168] = {.lex_state = 43, .external_lex_state = 24},
  [2169] = {.lex_state = 43, .external_lex_state = 24},
  [2170] = {.lex_state = 154, .external_lex_state = 23},
  [2171] = {.lex_state = 153, .external_lex_state = 23},
  [2172] = {.lex_state = 153, .external_lex_state = 23},
  [2173] = {.lex_state = 153, .external_lex_state = 23},
  [2174] = {.lex_state = 154, .external_lex_state = 23},
  [2175] = {.lex_state = 153, .external_lex_state = 23},
  [2176] = {.lex_state = 153, .external_lex_state = 23},
  [2177] = {.lex_state = 153, .external_lex_state = 23},
  [2178] = {.lex_state = 153, .external_lex_state = 23},
  [2179] = {.lex_state = 153, .external_lex_state = 23},
  [2180] = {.lex_state = 153, .external_lex_state = 23},
  [2181] = {.lex_state = 153, .external_lex_state = 23},
  [2182] = {.lex_state = 153, .external_lex_state = 23},
  [2183] = {.lex_state = 153, .external_lex_state = 23},
  [2184] = {.lex_state = 153, .external_lex_state = 23},
  [2185] = {.lex_state = 160, .external_lex_state = 22},
  [2186] = {.lex_state = 160, .external_lex_state = 22},
  [2187] = {.lex_state = 160, .external_lex_state = 22},
  [2188] = {.lex_state = 43, .external_lex_state = 24},
  [2189] = {.lex_state = 160, .external_lex_state = 22},
  [2190] = {.lex_state = 153, .external_lex_state = 23},
  [2191] = {.lex_state = 43, .external_lex_state = 24},
  [2192] = {.lex_state = 160, .external_lex_state = 22},
  [2193] = {.lex_state = 153, .external_lex_state = 23},
  [2194] = {.lex_state = 153, .external_lex_state = 23},
  [2195] = {.lex_state = 153, .external_lex_state = 23},
  [2196] = {.lex_state = 153, .external_lex_state = 23},
  [2197] = {.lex_state = 43, .external_lex_state = 24},
  [2198] = {.lex_state = 43, .external_lex_state = 24},
  [2199] = {.lex_state = 153, .external_lex_state = 23},
  [2200] = {.lex_state = 157, .external_lex_state = 23},
  [2201] = {.lex_state = 153, .external_lex_state = 23},
  [2202] = {.lex_state = 153, .external_lex_state = 23},
  [2203] = {.lex_state = 153, .external_lex_state = 24},
  [2204] = {.lex_state = 153, .external_lex_state = 24},
  [2205] = {.lex_state = 153, .external_lex_state = 23},
  [2206] = {.lex_state = 43, .external_lex_state = 24},
  [2207] = {.lex_state = 43, .external_lex_state = 24},
  [2208] = {.lex_state = 43, .external_lex_state = 24},
  [2209] = {.lex_state = 157, .external_lex_state = 23},
  [2210] = {.lex_state = 43, .external_lex_state = 24},
  [2211] = {.lex_state = 43, .external_lex_state = 24},
  [2212] = {.lex_state = 43, .external_lex_state = 24},
  [2213] = {.lex_state = 43, .external_lex_state = 24},
  [2214] = {.lex_state = 153, .external_lex_state = 23},
  [2215] = {.lex_state = 153, .external_lex_state = 23},
  [2216] = {.lex_state = 43, .external_lex_state = 24},
  [2217] = {.lex_state = 157, .external_lex_state = 25},
  [2218] = {.lex_state = 43, .external_lex_state = 24},
  [2219] = {.lex_state = 153, .external_lex_state = 23},
  [2220] = {.lex_state = 43, .external_lex_state = 24},
  [2221] = {.lex_state = 43, .external_lex_state = 24},
  [2222] = {.lex_state = 43, .external_lex_state = 24},
  [2223] = {.lex_state = 43, .external_lex_state = 24},
  [2224] = {.lex_state = 43, .external_lex_state = 24},
  [2225] = {.lex_state = 43, .external_lex_state = 24},
  [2226] = {.lex_state = 157, .external_lex_state = 23},
  [2227] = {.lex_state = 43, .external_lex_state = 24},
  [2228] = {.lex_state = 43, .external_lex_state = 24},
  [2229] = {.lex_state = 153, .external_lex_state = 23},
  [2230] = {.lex_state = 43, .external_lex_state = 24},
  [2231] = {.lex_state = 43, .external_lex_state = 24},
  [2232] = {.lex_state = 43, .external_lex_state = 24},
  [2233] = {.lex_state = 153, .external_lex_state = 24},
  [2234] = {.lex_state = 157, .external_lex_state = 25},
  [2235] = {.lex_state = 43, .external_lex_state = 24},
  [2236] = {.lex_state = 43, .external_lex_state = 24},
  [2237] = {.lex_state = 43, .external_lex_state = 24},
  [2238] = {.lex_state = 43, .external_lex_state = 24},
  [2239] = {.lex_state = 43, .external_lex_state = 24},
  [2240] = {.lex_state = 157, .external_lex_state = 25},
  [2241] = {.lex_state = 153, .external_lex_state = 24},
  [2242] = {.lex_state = 43, .external_lex_state = 24},
  [2243] = {.lex_state = 43, .external_lex_state = 24},
  [2244] = {.lex_state = 157, .external_lex_state = 25},
  [2245] = {.lex_state = 43, .external_lex_state = 24},
  [2246] = {.lex_state = 157, .external_lex_state = 23},
  [2247] = {.lex_state = 157, .external_lex_state = 23},
  [2248] = {.lex_state = 157, .external_lex_state = 23},
  [2249] = {.lex_state = 157, .external_lex_state = 23},
  [2250] = {.lex_state = 153, .external_lex_state = 23},
  [2251] = {.lex_state = 153, .external_lex_state = 23},
  [2252] = {.lex_state = 157, .external_lex_state = 25},
  [2253] = {.lex_state = 157, .external_lex_state = 23},
  [2254] = {.lex_state = 157, .external_lex_state = 25},
  [2255] = {.lex_state = 157, .external_lex_state = 23},
  [2256] = {.lex_state = 153, .external_lex_state = 24},
  [2257] = {.lex_state = 157, .external_lex_state = 23},
  [2258] = {.lex_state = 157, .external_lex_state = 23},
  [2259] = {.lex_state = 157, .external_lex_state = 25},
  [2260] = {.lex_state = 157, .external_lex_state = 23},
  [2261] = {.lex_state = 157, .external_lex_state = 23},
  [2262] = {.lex_state = 153, .external_lex_state = 24},
  [2263] = {.lex_state = 157, .external_lex_state = 23},
  [2264] = {.lex_state = 157, .external_lex_state = 23},
  [2265] = {.lex_state = 153, .external_lex_state = 24},
  [2266] = {.lex_state = 157, .external_lex_state = 23},
  [2267] = {.lex_state = 157, .external_lex_state = 23},
  [2268] = {.lex_state = 157, .external_lex_state = 25},
  [2269] = {.lex_state = 153, .external_lex_state = 23},
  [2270] = {.lex_state = 153, .external_lex_state = 24},
  [2271] = {.lex_state = 153, .external_lex_state = 23},
  [2272] = {.lex_state = 153, .external_lex_state = 23},
  [2273] = {.lex_state = 157, .external_lex_state = 23},
  [2274] = {.lex_state = 153, .external_lex_state = 24},
  [2275] = {.lex_state = 153, .external_lex_state = 24},
  [2276] = {.lex_state = 157, .external_lex_state = 23},
  [2277] = {.lex_state = 153, .external_lex_state = 24},
  [2278] = {.lex_state = 153, .external_lex_state = 24},
  [2279] = {.lex_state = 154, .external_lex_state = 23},
  [2280] = {.lex_state = 160, .external_lex_state = 21},
  [2281] = {.lex_state = 157, .external_lex_state = 25},
  [2282] = {.lex_state = 154, .external_lex_state = 24},
  [2283] = {.lex_state = 153, .external_lex_state = 24},
  [2284] = {.lex_state = 153, .external_lex_state = 24},
  [2285] = {.lex_state = 157, .external_lex_state = 23},
  [2286] = {.lex_state = 157, .external_lex_state = 23},
  [2287] = {.lex_state = 157, .external_lex_state = 23},
  [2288] = {.lex_state = 157, .external_lex_state = 23},
  [2289] = {.lex_state = 157, .external_lex_state = 23},
  [2290] = {.lex_state = 157, .external_lex_state = 23},
  [2291] = {.lex_state = 157, .external_lex_state = 23},
  [2292] = {.lex_state = 157, .external_lex_state = 23},
  [2293] = {.lex_state = 157, .external_lex_state = 23},
  [2294] = {.lex_state = 157, .external_lex_state = 23},
  [2295] = {.lex_state = 157, .external_lex_state = 23},
  [2296] = {.lex_state = 157, .external_lex_state = 23},
  [2297] = {.lex_state = 157, .external_lex_state = 23},
  [2298] = {.lex_state = 157, .external_lex_state = 23},
  [2299] = {.lex_state = 157, .external_lex_state = 23},
  [2300] = {.lex_state = 157, .external_lex_state = 23},
  [2301] = {.lex_state = 32, .external_lex_state = 26},
  [2302] = {.lex_state = 157, .external_lex_state = 23},
  [2303] = {.lex_state = 157, .external_lex_state = 23},
  [2304] = {.lex_state = 157, .external_lex_state = 23},
  [2305] = {.lex_state = 157, .external_lex_state = 23},
  [2306] = {.lex_state = 157, .external_lex_state = 23},
  [2307] = {.lex_state = 157, .external_lex_state = 23},
  [2308] = {.lex_state = 157, .external_lex_state = 23},
  [2309] = {.lex_state = 157, .external_lex_state = 23},
  [2310] = {.lex_state = 153, .external_lex_state = 24},
  [2311] = {.lex_state = 157, .external_lex_state = 23},
  [2312] = {.lex_state = 154, .external_lex_state = 24},
  [2313] = {.lex_state = 157, .external_lex_state = 23},
  [2314] = {.lex_state = 157, .external_lex_state = 23},
  [2315] = {.lex_state = 157, .external_lex_state = 23},
  [2316] = {.lex_state = 157, .external_lex_state = 23},
  [2317] = {.lex_state = 157, .external_lex_state = 23},
  [2318] = {.lex_state = 157, .external_lex_state = 23},
  [2319] = {.lex_state = 157, .external_lex_state = 23},
  [2320] = {.lex_state = 157, .external_lex_state = 23},
  [2321] = {.lex_state = 157, .external_lex_state = 23},
  [2322] = {.lex_state = 157, .external_lex_state = 23},
  [2323] = {.lex_state = 157, .external_lex_state = 23},
  [2324] = {.lex_state = 153, .external_lex_state = 24},
  [2325] = {.lex_state = 157, .external_lex_state = 23},
  [2326] = {.lex_state = 157, .external_lex_state = 23},
  [2327] = {.lex_state = 157, .external_lex_state = 23},
  [2328] = {.lex_state = 153, .external_lex_state = 23},
  [2329] = {.lex_state = 157, .external_lex_state = 23},
  [2330] = {.lex_state = 157, .external_lex_state = 23},
  [2331] = {.lex_state = 157, .external_lex_state = 23},
  [2332] = {.lex_state = 157, .external_lex_state = 23},
  [2333] = {.lex_state = 157, .external_lex_state = 23},
  [2334] = {.lex_state = 157, .external_lex_state = 23},
  [2335] = {.lex_state = 157, .external_lex_state = 23},
  [2336] = {.lex_state = 157, .external_lex_state = 23},
  [2337] = {.lex_state = 157, .external_lex_state = 23},
  [2338] = {.lex_state = 157, .external_lex_state = 23},
  [2339] = {.lex_state = 157, .external_lex_state = 23},
  [2340] = {.lex_state = 157, .external_lex_state = 23},
  [2341] = {.lex_state = 157, .external_lex_state = 23},
  [2342] = {.lex_state = 153, .external_lex_state = 23},
  [2343] = {.lex_state = 157, .external_lex_state = 23},
  [2344] = {.lex_state = 157, .external_lex_state = 23},
  [2345] = {.lex_state = 157, .external_lex_state = 23},
  [2346] = {.lex_state = 153, .external_lex_state = 24},
  [2347] = {.lex_state = 157, .external_lex_state = 23},
  [2348] = {.lex_state = 157, .external_lex_state = 23},
  [2349] = {.lex_state = 157, .external_lex_state = 23},
  [2350] = {.lex_state = 157, .external_lex_state = 23},
  [2351] = {.lex_state = 157, .external_lex_state = 23},
  [2352] = {.lex_state = 157, .external_lex_state = 23},
  [2353] = {.lex_state = 157, .external_lex_state = 23},
  [2354] = {.lex_state = 157, .external_lex_state = 23},
  [2355] = {.lex_state = 157, .external_lex_state = 23},
  [2356] = {.lex_state = 157, .external_lex_state = 23},
  [2357] = {.lex_state = 157, .external_lex_state = 23},
  [2358] = {.lex_state = 157, .external_lex_state = 23},
  [2359] = {.lex_state = 157, .external_lex_state = 23},
  [2360] = {.lex_state = 157, .external_lex_state = 23},
  [2361] = {.lex_state = 157, .external_lex_state = 23},
  [2362] = {.lex_state = 157, .external_lex_state = 23},
  [2363] = {.lex_state = 157, .external_lex_state = 23},
  [2364] = {.lex_state = 157, .external_lex_state = 23},
  [2365] = {.lex_state = 157, .external_lex_state = 23},
  [2366] = {.lex_state = 157, .external_lex_state = 23},
  [2367] = {.lex_state = 157, .external_lex_state = 23},
  [2368] = {.lex_state = 157, .external_lex_state = 23},
  [2369] = {.lex_state = 157, .external_lex_state = 23},
  [2370] = {.lex_state = 157, .external_lex_state = 23},
  [2371] = {.lex_state = 157, .external_lex_state = 23},
  [2372] = {.lex_state = 157, .external_lex_state = 23},
  [2373] = {.lex_state = 157, .external_lex_state = 23},
  [2374] = {.lex_state = 157, .external_lex_state = 23},
  [2375] = {.lex_state = 157, .external_lex_state = 23},
  [2376] = {.lex_state = 157, .external_lex_state = 23},
  [2377] = {.lex_state = 157, .external_lex_state = 23},
  [2378] = {.lex_state = 157, .external_lex_state = 23},
  [2379] = {.lex_state = 157, .external_lex_state = 23},
  [2380] = {.lex_state = 157, .external_lex_state = 23},
  [2381] = {.lex_state = 157, .external_lex_state = 23},
  [2382] = {.lex_state = 157, .external_lex_state = 23},
  [2383] = {.lex_state = 157, .external_lex_state = 23},
  [2384] = {.lex_state = 157, .external_lex_state = 23},
  [2385] = {.lex_state = 157, .external_lex_state = 23},
  [2386] = {.lex_state = 157, .external_lex_state = 23},
  [2387] = {.lex_state = 157, .external_lex_state = 23},
  [2388] = {.lex_state = 157, .external_lex_state = 23},
  [2389] = {.lex_state = 157, .external_lex_state = 23},
  [2390] = {.lex_state = 157, .external_lex_state = 23},
  [2391] = {.lex_state = 157, .external_lex_state = 23},
  [2392] = {.lex_state = 32, .external_lex_state = 26},
  [2393] = {.lex_state = 157, .external_lex_state = 23},
  [2394] = {.lex_state = 157, .external_lex_state = 23},
  [2395] = {.lex_state = 157, .external_lex_state = 23},
  [2396] = {.lex_state = 157, .external_lex_state = 23},
  [2397] = {.lex_state = 157, .external_lex_state = 23},
  [2398] = {.lex_state = 157, .external_lex_state = 23},
  [2399] = {.lex_state = 157, .external_lex_state = 23},
  [2400] = {.lex_state = 157, .external_lex_state = 23},
  [2401] = {.lex_state = 157, .external_lex_state = 23},
  [2402] = {.lex_state = 157, .external_lex_state = 23},
  [2403] = {.lex_state = 153, .external_lex_state = 24},
  [2404] = {.lex_state = 157, .external_lex_state = 23},
  [2405] = {.lex_state = 157, .external_lex_state = 23},
  [2406] = {.lex_state = 157, .external_lex_state = 23},
  [2407] = {.lex_state = 32, .external_lex_state = 26},
  [2408] = {.lex_state = 157, .external_lex_state = 23},
  [2409] = {.lex_state = 157, .external_lex_state = 23},
  [2410] = {.lex_state = 157, .external_lex_state = 25},
  [2411] = {.lex_state = 153, .external_lex_state = 24},
  [2412] = {.lex_state = 157, .external_lex_state = 23},
  [2413] = {.lex_state = 32, .external_lex_state = 26},
  [2414] = {.lex_state = 157, .external_lex_state = 23},
  [2415] = {.lex_state = 157, .external_lex_state = 23},
  [2416] = {.lex_state = 157, .external_lex_state = 23},
  [2417] = {.lex_state = 157, .external_lex_state = 23},
  [2418] = {.lex_state = 157, .external_lex_state = 23},
  [2419] = {.lex_state = 32, .external_lex_state = 26},
  [2420] = {.lex_state = 157, .external_lex_state = 23},
  [2421] = {.lex_state = 32, .external_lex_state = 26},
  [2422] = {.lex_state = 157, .external_lex_state = 23},
  [2423] = {.lex_state = 157, .external_lex_state = 23},
  [2424] = {.lex_state = 32, .external_lex_state = 26},
  [2425] = {.lex_state = 157, .external_lex_state = 23},
  [2426] = {.lex_state = 157, .external_lex_state = 23},
  [2427] = {.lex_state = 157, .external_lex_state = 23},
  [2428] = {.lex_state = 32, .external_lex_state = 26},
  [2429] = {.lex_state = 157, .external_lex_state = 23},
  [2430] = {.lex_state = 157, .external_lex_state = 23},
  [2431] = {.lex_state = 157, .external_lex_state = 23},
  [2432] = {.lex_state = 32, .external_lex_state = 26},
  [2433] = {.lex_state = 153, .external_lex_state = 24},
  [2434] = {.lex_state = 32, .external_lex_state = 26},
  [2435] = {.lex_state = 153, .external_lex_state = 24},
  [2436] = {.lex_state = 157, .external_lex_state = 23},
  [2437] = {.lex_state = 157, .external_lex_state = 23},
  [2438] = {.lex_state = 157, .external_lex_state = 23},
  [2439] = {.lex_state = 32, .external_lex_state = 26},
  [2440] = {.lex_state = 157, .external_lex_state = 23},
  [2441] = {.lex_state = 32, .external_lex_state = 26},
  [2442] = {.lex_state = 153, .external_lex_state = 23},
  [2443] = {.lex_state = 157, .external_lex_state = 23},
  [2444] = {.lex_state = 157, .external_lex_state = 23},
  [2445] = {.lex_state = 157, .external_lex_state = 23},
  [2446] = {.lex_state = 157, .external_lex_state = 23},
  [2447] = {.lex_state = 157, .external_lex_state = 23},
  [2448] = {.lex_state = 157, .external_lex_state = 23},
  [2449] = {.lex_state = 157, .external_lex_state = 23},
  [2450] = {.lex_state = 157, .external_lex_state = 23},
  [2451] = {.lex_state = 157, .external_lex_state = 23},
  [2452] = {.lex_state = 157, .external_lex_state = 23},
  [2453] = {.lex_state = 157, .external_lex_state = 23},
  [2454] = {.lex_state = 32, .external_lex_state = 26},
  [2455] = {.lex_state = 157, .external_lex_state = 23},
  [2456] = {.lex_state = 32, .external_lex_state = 26},
  [2457] = {.lex_state = 32, .external_lex_state = 26},
  [2458] = {.lex_state = 32, .external_lex_state = 26},
  [2459] = {.lex_state = 32, .external_lex_state = 26},
  [2460] = {.lex_state = 32, .external_lex_state = 26},
  [2461] = {.lex_state = 32, .external_lex_state = 26},
  [2462] = {.lex_state = 32, .external_lex_state = 26},
  [2463] = {.lex_state = 32, .external_lex_state = 26},
  [2464] = {.lex_state = 32, .external_lex_state = 26},
  [2465] = {.lex_state = 157, .external_lex_state = 23},
  [2466] = {.lex_state = 32, .external_lex_state = 26},
  [2467] = {.lex_state = 157, .external_lex_state = 23},
  [2468] = {.lex_state = 157, .external_lex_state = 23},
  [2469] = {.lex_state = 157, .external_lex_state = 23},
  [2470] = {.lex_state = 32, .external_lex_state = 26},
  [2471] = {.lex_state = 157, .external_lex_state = 23},
  [2472] = {.lex_state = 157, .external_lex_state = 23},
  [2473] = {.lex_state = 157, .external_lex_state = 23},
  [2474] = {.lex_state = 153, .external_lex_state = 24},
  [2475] = {.lex_state = 153, .external_lex_state = 23},
  [2476] = {.lex_state = 153, .external_lex_state = 23},
  [2477] = {.lex_state = 157, .external_lex_state = 23},
  [2478] = {.lex_state = 157, .external_lex_state = 23},
  [2479] = {.lex_state = 153, .external_lex_state = 23},
  [2480] = {.lex_state = 157, .external_lex_state = 23},
  [2481] = {.lex_state = 153, .external_lex_state = 24},
  [2482] = {.lex_state = 157, .external_lex_state = 23},
  [2483] = {.lex_state = 157, .external_lex_state = 23},
  [2484] = {.lex_state = 153, .external_lex_state = 24},
  [2485] = {.lex_state = 157, .external_lex_state = 23},
  [2486] = {.lex_state = 153, .external_lex_state = 23},
  [2487] = {.lex_state = 153, .external_lex_state = 23},
  [2488] = {.lex_state = 153, .external_lex_state = 23},
  [2489] = {.lex_state = 154, .external_lex_state = 24},
  [2490] = {.lex_state = 153, .external_lex_state = 24},
  [2491] = {.lex_state = 157, .external_lex_state = 23},
  [2492] = {.lex_state = 157, .external_lex_state = 23},
  [2493] = {.lex_state = 157, .external_lex_state = 23},
  [2494] = {.lex_state = 157, .external_lex_state = 23},
  [2495] = {.lex_state = 157, .external_lex_state = 23},
  [2496] = {.lex_state = 157, .external_lex_state = 23},
  [2497] = {.lex_state = 157, .external_lex_state = 23},
  [2498] = {.lex_state = 157, .external_lex_state = 23},
  [2499] = {.lex_state = 153, .external_lex_state = 24},
  [2500] = {.lex_state = 157, .external_lex_state = 23},
  [2501] = {.lex_state = 157, .external_lex_state = 23},
  [2502] = {.lex_state = 153, .external_lex_state = 24},
  [2503] = {.lex_state = 153, .external_lex_state = 24},
  [2504] = {.lex_state = 153, .external_lex_state = 24},
  [2505] = {.lex_state = 154, .external_lex_state = 24},
  [2506] = {.lex_state = 157, .external_lex_state = 23},
  [2507] = {.lex_state = 157, .external_lex_state = 23},
  [2508] = {.lex_state = 157, .external_lex_state = 23},
  [2509] = {.lex_state = 157, .external_lex_state = 23},
  [2510] = {.lex_state = 157, .external_lex_state = 23},
  [2511] = {.lex_state = 157, .external_lex_state = 23},
  [2512] = {.lex_state = 157, .external_lex_state = 23},
  [2513] = {.lex_state = 157, .external_lex_state = 23},
  [2514] = {.lex_state = 157, .external_lex_state = 23},
  [2515] = {.lex_state = 157, .external_lex_state = 23},
  [2516] = {.lex_state = 157, .external_lex_state = 23},
  [2517] = {.lex_state = 157, .external_lex_state = 23},
  [2518] = {.lex_state = 157, .external_lex_state = 23},
  [2519] = {.lex_state = 157, .external_lex_state = 23},
  [2520] = {.lex_state = 157, .external_lex_state = 23},
  [2521] = {.lex_state = 157, .external_lex_state = 23},
  [2522] = {.lex_state = 157, .external_lex_state = 23},
  [2523] = {.lex_state = 153, .external_lex_state = 24},
  [2524] = {.lex_state = 153, .external_lex_state = 24},
  [2525] = {.lex_state = 153, .external_lex_state = 24},
  [2526] = {.lex_state = 157, .external_lex_state = 23},
  [2527] = {.lex_state = 157, .external_lex_state = 23},
  [2528] = {.lex_state = 157, .external_lex_state = 23},
  [2529] = {.lex_state = 160, .external_lex_state = 21},
  [2530] = {.lex_state = 153, .external_lex_state = 23},
  [2531] = {.lex_state = 157, .external_lex_state = 23},
  [2532] = {.lex_state = 153, .external_lex_state = 24},
  [2533] = {.lex_state = 157, .external_lex_state = 23},
  [2534] = {.lex_state = 153, .external_lex_state = 23},
  [2535] = {.lex_state = 157, .external_lex_state = 23},
  [2536] = {.lex_state = 153, .external_lex_state = 23},
  [2537] = {.lex_state = 153, .external_lex_state = 23},
  [2538] = {.lex_state = 32, .external_lex_state = 26},
  [2539] = {.lex_state = 32, .external_lex_state = 26},
  [2540] = {.lex_state = 153, .external_lex_state = 24},
  [2541] = {.lex_state = 157, .external_lex_state = 23},
  [2542] = {.lex_state = 32, .external_lex_state = 26},
  [2543] = {.lex_state = 32, .external_lex_state = 26},
  [2544] = {.lex_state = 157, .external_lex_state = 25},
  [2545] = {.lex_state = 32, .external_lex_state = 26},
  [2546] = {.lex_state = 157, .external_lex_state = 23},
  [2547] = {.lex_state = 159, .external_lex_state = 27},
  [2548] = {.lex_state = 32, .external_lex_state = 26},
  [2549] = {.lex_state = 157, .external_lex_state = 23},
  [2550] = {.lex_state = 32, .external_lex_state = 26},
  [2551] = {.lex_state = 32, .external_lex_state = 26},
  [2552] = {.lex_state = 157, .external_lex_state = 23},
  [2553] = {.lex_state = 32, .external_lex_state = 26},
  [2554] = {.lex_state = 32, .external_lex_state = 26},
  [2555] = {.lex_state = 157, .external_lex_state = 23},
  [2556] = {.lex_state = 159, .external_lex_state = 27},
  [2557] = {.lex_state = 32, .external_lex_state = 26},
  [2558] = {.lex_state = 157, .external_lex_state = 23},
  [2559] = {.lex_state = 32, .external_lex_state = 26},
  [2560] = {.lex_state = 32, .external_lex_state = 26},
  [2561] = {.lex_state = 157, .external_lex_state = 23},
  [2562] = {.lex_state = 32, .external_lex_state = 26},
  [2563] = {.lex_state = 32, .external_lex_state = 26},
  [2564] = {.lex_state = 43, .external_lex_state = 24},
  [2565] = {.lex_state = 157, .external_lex_state = 23},
  [2566] = {.lex_state = 43, .external_lex_state = 24},
  [2567] = {.lex_state = 157, .external_lex_state = 23},
  [2568] = {.lex_state = 157, .external_lex_state = 23},
  [2569] = {.lex_state = 159, .external_lex_state = 27},
  [2570] = {.lex_state = 32, .external_lex_state = 26},
  [2571] = {.lex_state = 32, .external_lex_state = 26},
  [2572] = {.lex_state = 43, .external_lex_state = 24},
  [2573] = {.lex_state = 157, .external_lex_state = 23},
  [2574] = {.lex_state = 157, .external_lex_state = 23},
  [2575] = {.lex_state = 157, .external_lex_state = 23},
  [2576] = {.lex_state = 157, .external_lex_state = 23},
  [2577] = {.lex_state = 157, .external_lex_state = 23},
  [2578] = {.lex_state = 157, .external_lex_state = 23},
  [2579] = {.lex_state = 157, .external_lex_state = 23},
  [2580] = {.lex_state = 157, .external_lex_state = 23},
  [2581] = {.lex_state = 157, .external_lex_state = 23},
  [2582] = {.lex_state = 32, .external_lex_state = 26},
  [2583] = {.lex_state = 157, .external_lex_state = 23},
  [2584] = {.lex_state = 157, .external_lex_state = 23},
  [2585] = {.lex_state = 32, .external_lex_state = 26},
  [2586] = {.lex_state = 32, .external_lex_state = 26},
  [2587] = {.lex_state = 32, .external_lex_state = 26},
  [2588] = {.lex_state = 157, .external_lex_state = 23},
  [2589] = {.lex_state = 32, .external_lex_state = 26},
  [2590] = {.lex_state = 32, .external_lex_state = 26},
  [2591] = {.lex_state = 32, .external_lex_state = 26},
  [2592] = {.lex_state = 157, .external_lex_state = 23},
  [2593] = {.lex_state = 32, .external_lex_state = 26},
  [2594] = {.lex_state = 159, .external_lex_state = 27},
  [2595] = {.lex_state = 32, .external_lex_state = 26},
  [2596] = {.lex_state = 32, .external_lex_state = 26},
  [2597] = {.lex_state = 32, .external_lex_state = 26},
  [2598] = {.lex_state = 159, .external_lex_state = 27},
  [2599] = {.lex_state = 32, .external_lex_state = 26},
  [2600] = {.lex_state = 157, .external_lex_state = 23},
  [2601] = {.lex_state = 32, .external_lex_state = 26},
  [2602] = {.lex_state = 32, .external_lex_state = 26},
  [2603] = {.lex_state = 32, .external_lex_state = 26},
  [2604] = {.lex_state = 32, .external_lex_state = 26},
  [2605] = {.lex_state = 32, .external_lex_state = 26},
  [2606] = {.lex_state = 43, .external_lex_state = 24},
  [2607] = {.lex_state = 157, .external_lex_state = 23},
  [2608] = {.lex_state = 32, .external_lex_state = 26},
  [2609] = {.lex_state = 32, .external_lex_state = 26},
  [2610] = {.lex_state = 32, .external_lex_state = 26},
  [2611] = {.lex_state = 32, .external_lex_state = 26},
  [2612] = {.lex_state = 32, .external_lex_state = 26},
  [2613] = {.lex_state = 159, .external_lex_state = 27},
  [2614] = {.lex_state = 159, .external_lex_state = 27},
  [2615] = {.lex_state = 32, .external_lex_state = 26},
  [2616] = {.lex_state = 32, .external_lex_state = 26},
  [2617] = {.lex_state = 32, .external_lex_state = 26},
  [2618] = {.lex_state = 32, .external_lex_state = 26},
  [2619] = {.lex_state = 32, .external_lex_state = 26},
  [2620] = {.lex_state = 32, .external_lex_state = 26},
  [2621] = {.lex_state = 32, .external_lex_state = 26},
  [2622] = {.lex_state = 32, .external_lex_state = 26},
  [2623] = {.lex_state = 32, .external_lex_state = 26},
  [2624] = {.lex_state = 157, .external_lex_state = 23},
  [2625] = {.lex_state = 157, .external_lex_state = 23},
  [2626] = {.lex_state = 157, .external_lex_state = 23},
  [2627] = {.lex_state = 157, .external_lex_state = 23},
  [2628] = {.lex_state = 32, .external_lex_state = 26},
  [2629] = {.lex_state = 157, .external_lex_state = 23},
  [2630] = {.lex_state = 157, .external_lex_state = 25},
  [2631] = {.lex_state = 32, .external_lex_state = 26},
  [2632] = {.lex_state = 32, .external_lex_state = 26},
  [2633] = {.lex_state = 32, .external_lex_state = 26},
  [2634] = {.lex_state = 32, .external_lex_state = 26},
  [2635] = {.lex_state = 32, .external_lex_state = 26},
  [2636] = {.lex_state = 157, .external_lex_state = 23},
  [2637] = {.lex_state = 160, .external_lex_state = 22},
  [2638] = {.lex_state = 157, .external_lex_state = 23},
  [2639] = {.lex_state = 43, .external_lex_state = 24},
  [2640] = {.lex_state = 32, .external_lex_state = 26},
  [2641] = {.lex_state = 157, .external_lex_state = 23},
  [2642] = {.lex_state = 160, .external_lex_state = 22},
  [2643] = {.lex_state = 157, .external_lex_state = 23},
  [2644] = {.lex_state = 157, .external_lex_state = 23},
  [2645] = {.lex_state = 157, .external_lex_state = 23},
  [2646] = {.lex_state = 157, .external_lex_state = 23},
  [2647] = {.lex_state = 43, .external_lex_state = 24},
  [2648] = {.lex_state = 43, .external_lex_state = 24},
  [2649] = {.lex_state = 157, .external_lex_state = 23},
  [2650] = {.lex_state = 157, .external_lex_state = 23},
  [2651] = {.lex_state = 157, .external_lex_state = 23},
  [2652] = {.lex_state = 157, .external_lex_state = 23},
  [2653] = {.lex_state = 157, .external_lex_state = 23},
  [2654] = {.lex_state = 43, .external_lex_state = 24},
  [2655] = {.lex_state = 157, .external_lex_state = 23},
  [2656] = {.lex_state = 157, .external_lex_state = 23},
  [2657] = {.lex_state = 157, .external_lex_state = 23},
  [2658] = {.lex_state = 159, .external_lex_state = 27},
  [2659] = {.lex_state = 159, .external_lex_state = 27},
  [2660] = {.lex_state = 159, .external_lex_state = 27},
  [2661] = {.lex_state = 32, .external_lex_state = 26},
  [2662] = {.lex_state = 159, .external_lex_state = 27},
  [2663] = {.lex_state = 159, .external_lex_state = 27},
  [2664] = {.lex_state = 157, .external_lex_state = 23},
  [2665] = {.lex_state = 157, .external_lex_state = 23},
  [2666] = {.lex_state = 157, .external_lex_state = 23},
  [2667] = {.lex_state = 157, .external_lex_state = 23},
  [2668] = {.lex_state = 159, .external_lex_state = 27},
  [2669] = {.lex_state = 159, .external_lex_state = 27},
  [2670] = {.lex_state = 159, .external_lex_state = 27},
  [2671] = {.lex_state = 159, .external_lex_state = 27},
  [2672] = {.lex_state = 159, .external_lex_state = 27},
  [2673] = {.lex_state = 159, .external_lex_state = 27},
  [2674] = {.lex_state = 159, .external_lex_state = 27},
  [2675] = {.lex_state = 159, .external_lex_state = 27},
  [2676] = {.lex_state = 159, .external_lex_state = 27},
  [2677] = {.lex_state = 159, .external_lex_state = 27},
  [2678] = {.lex_state = 159, .external_lex_state = 27},
  [2679] = {.lex_state = 159, .external_lex_state = 27},
  [2680] = {.lex_state = 159, .external_lex_state = 27},
  [2681] = {.lex_state = 159, .external_lex_state = 27},
  [2682] = {.lex_state = 159, .external_lex_state = 27},
  [2683] = {.lex_state = 159, .external_lex_state = 27},
  [2684] = {.lex_state = 159, .external_lex_state = 27},
  [2685] = {.lex_state = 159, .external_lex_state = 27},
  [2686] = {.lex_state = 159, .external_lex_state = 27},
  [2687] = {.lex_state = 159, .external_lex_state = 27},
  [2688] = {.lex_state = 159, .external_lex_state = 27},
  [2689] = {.lex_state = 159, .external_lex_state = 27},
  [2690] = {.lex_state = 159, .external_lex_state = 27},
  [2691] = {.lex_state = 159, .external_lex_state = 27},
  [2692] = {.lex_state = 157, .external_lex_state = 25},
  [2693] = {.lex_state = 159, .external_lex_state = 27},
  [2694] = {.lex_state = 159, .external_lex_state = 27},
  [2695] = {.lex_state = 159, .external_lex_state = 27},
  [2696] = {.lex_state = 159, .external_lex_state = 27},
  [2697] = {.lex_state = 159, .external_lex_state = 27},
  [2698] = {.lex_state = 159, .external_lex_state = 27},
  [2699] = {.lex_state = 157, .external_lex_state = 23},
  [2700] = {.lex_state = 159, .external_lex_state = 27},
  [2701] = {.lex_state = 157, .external_lex_state = 23},
  [2702] = {.lex_state = 159, .external_lex_state = 27},
  [2703] = {.lex_state = 45, .external_lex_state = 24},
  [2704] = {.lex_state = 159, .external_lex_state = 27},
  [2705] = {.lex_state = 159, .external_lex_state = 27},
  [2706] = {.lex_state = 159, .external_lex_state = 27},
  [2707] = {.lex_state = 157, .external_lex_state = 25},
  [2708] = {.lex_state = 157, .external_lex_state = 23},
  [2709] = {.lex_state = 157, .external_lex_state = 23},
  [2710] = {.lex_state = 159, .external_lex_state = 27},
  [2711] = {.lex_state = 159, .external_lex_state = 27},
  [2712] = {.lex_state = 159, .external_lex_state = 27},
  [2713] = {.lex_state = 159, .external_lex_state = 27},
  [2714] = {.lex_state = 159, .external_lex_state = 27},
  [2715] = {.lex_state = 159, .external_lex_state = 27},
  [2716] = {.lex_state = 159, .external_lex_state = 27},
  [2717] = {.lex_state = 157, .external_lex_state = 23},
  [2718] = {.lex_state = 157, .external_lex_state = 23},
  [2719] = {.lex_state = 157, .external_lex_state = 23},
  [2720] = {.lex_state = 157, .external_lex_state = 25},
  [2721] = {.lex_state = 157, .external_lex_state = 23},
  [2722] = {.lex_state = 157, .external_lex_state = 23},
  [2723] = {.lex_state = 159, .external_lex_state = 27},
  [2724] = {.lex_state = 157, .external_lex_state = 23},
  [2725] = {.lex_state = 159, .external_lex_state = 27},
  [2726] = {.lex_state = 157, .external_lex_state = 23},
  [2727] = {.lex_state = 157, .external_lex_state = 23},
  [2728] = {.lex_state = 157, .external_lex_state = 23},
  [2729] = {.lex_state = 157, .external_lex_state = 23},
  [2730] = {.lex_state = 157, .external_lex_state = 23},
  [2731] = {.lex_state = 157, .external_lex_state = 23},
  [2732] = {.lex_state = 157, .external_lex_state = 23},
  [2733] = {.lex_state = 157, .external_lex_state = 23},
  [2734] = {.lex_state = 157, .external_lex_state = 23},
  [2735] = {.lex_state = 157, .external_lex_state = 23},
  [2736] = {.lex_state = 157, .external_lex_state = 23},
  [2737] = {.lex_state = 159, .external_lex_state = 27},
  [2738] = {.lex_state = 159, .external_lex_state = 27},
  [2739] = {.lex_state = 159, .external_lex_state = 27},
  [2740] = {.lex_state = 159, .external_lex_state = 27},
  [2741] = {.lex_state = 159, .external_lex_state = 27},
  [2742] = {.lex_state = 157, .external_lex_state = 23},
  [2743] = {.lex_state = 159, .external_lex_state = 27},
  [2744] = {.lex_state = 159, .external_lex_state = 27},
  [2745] = {.lex_state = 157, .external_lex_state = 25},
  [2746] = {.lex_state = 159, .external_lex_state = 27},
  [2747] = {.lex_state = 159, .external_lex_state = 27},
  [2748] = {.lex_state = 159, .external_lex_state = 27},
  [2749] = {.lex_state = 159, .external_lex_state = 27},
  [2750] = {.lex_state = 159, .external_lex_state = 27},
  [2751] = {.lex_state = 159, .external_lex_state = 27},
  [2752] = {.lex_state = 159, .external_lex_state = 27},
  [2753] = {.lex_state = 159, .external_lex_state = 27},
  [2754] = {.lex_state = 159, .external_lex_state = 27},
  [2755] = {.lex_state = 159, .external_lex_state = 27},
  [2756] = {.lex_state = 159, .external_lex_state = 27},
  [2757] = {.lex_state = 159, .external_lex_state = 27},
  [2758] = {.lex_state = 159, .external_lex_state = 27},
  [2759] = {.lex_state = 159, .external_lex_state = 27},
  [2760] = {.lex_state = 159, .external_lex_state = 27},
  [2761] = {.lex_state = 157, .external_lex_state = 23},
  [2762] = {.lex_state = 159, .external_lex_state = 27},
  [2763] = {.lex_state = 157, .external_lex_state = 23},
  [2764] = {.lex_state = 45, .external_lex_state = 24},
  [2765] = {.lex_state = 157, .external_lex_state = 23},
  [2766] = {.lex_state = 157, .external_lex_state = 23},
  [2767] = {.lex_state = 157, .external_lex_state = 23},
  [2768] = {.lex_state = 157, .external_lex_state = 23},
  [2769] = {.lex_state = 157, .external_lex_state = 23},
  [2770] = {.lex_state = 157, .external_lex_state = 23},
  [2771] = {.lex_state = 159, .external_lex_state = 27},
  [2772] = {.lex_state = 157, .external_lex_state = 23},
  [2773] = {.lex_state = 159, .external_lex_state = 27},
  [2774] = {.lex_state = 157, .external_lex_state = 23},
  [2775] = {.lex_state = 159, .external_lex_state = 27},
  [2776] = {.lex_state = 159, .external_lex_state = 27},
  [2777] = {.lex_state = 45, .external_lex_state = 24},
  [2778] = {.lex_state = 159, .external_lex_state = 27},
  [2779] = {.lex_state = 159, .external_lex_state = 27},
  [2780] = {.lex_state = 157, .external_lex_state = 23},
  [2781] = {.lex_state = 157, .external_lex_state = 23},
  [2782] = {.lex_state = 159, .external_lex_state = 27},
  [2783] = {.lex_state = 159, .external_lex_state = 27},
  [2784] = {.lex_state = 157, .external_lex_state = 28},
  [2785] = {.lex_state = 159, .external_lex_state = 27},
  [2786] = {.lex_state = 159, .external_lex_state = 27},
  [2787] = {.lex_state = 159, .external_lex_state = 27},
  [2788] = {.lex_state = 159, .external_lex_state = 27},
  [2789] = {.lex_state = 159, .external_lex_state = 27},
  [2790] = {.lex_state = 159, .external_lex_state = 27},
  [2791] = {.lex_state = 159, .external_lex_state = 27},
  [2792] = {.lex_state = 157, .external_lex_state = 25},
  [2793] = {.lex_state = 159, .external_lex_state = 27},
  [2794] = {.lex_state = 159, .external_lex_state = 27},
  [2795] = {.lex_state = 157, .external_lex_state = 23},
  [2796] = {.lex_state = 159, .external_lex_state = 27},
  [2797] = {.lex_state = 157, .external_lex_state = 28},
  [2798] = {.lex_state = 157, .external_lex_state = 23},
  [2799] = {.lex_state = 45, .external_lex_state = 24},
  [2800] = {.lex_state = 159, .external_lex_state = 27},
  [2801] = {.lex_state = 159, .external_lex_state = 27},
  [2802] = {.lex_state = 159, .external_lex_state = 27},
  [2803] = {.lex_state = 157, .external_lex_state = 23},
  [2804] = {.lex_state = 45, .external_lex_state = 24},
  [2805] = {.lex_state = 45, .external_lex_state = 24},
  [2806] = {.lex_state = 159, .external_lex_state = 27},
  [2807] = {.lex_state = 157, .external_lex_state = 28},
  [2808] = {.lex_state = 45, .external_lex_state = 24},
  [2809] = {.lex_state = 45, .external_lex_state = 24},
  [2810] = {.lex_state = 159, .external_lex_state = 27},
  [2811] = {.lex_state = 157, .external_lex_state = 25},
  [2812] = {.lex_state = 159, .external_lex_state = 27},
  [2813] = {.lex_state = 45, .external_lex_state = 24},
  [2814] = {.lex_state = 159, .external_lex_state = 27},
  [2815] = {.lex_state = 159, .external_lex_state = 27},
  [2816] = {.lex_state = 159, .external_lex_state = 27},
  [2817] = {.lex_state = 159, .external_lex_state = 27},
  [2818] = {.lex_state = 45, .external_lex_state = 24},
  [2819] = {.lex_state = 45, .external_lex_state = 24},
  [2820] = {.lex_state = 159, .external_lex_state = 27},
  [2821] = {.lex_state = 159, .external_lex_state = 27},
  [2822] = {.lex_state = 159, .external_lex_state = 27},
  [2823] = {.lex_state = 45, .external_lex_state = 24},
  [2824] = {.lex_state = 159, .external_lex_state = 27},
  [2825] = {.lex_state = 157, .external_lex_state = 23},
  [2826] = {.lex_state = 159, .external_lex_state = 27},
  [2827] = {.lex_state = 157, .external_lex_state = 23},
  [2828] = {.lex_state = 157, .external_lex_state = 24},
  [2829] = {.lex_state = 157, .external_lex_state = 23},
  [2830] = {.lex_state = 157, .external_lex_state = 23},
  [2831] = {.lex_state = 157, .external_lex_state = 23},
  [2832] = {.lex_state = 157, .external_lex_state = 23},
  [2833] = {.lex_state = 45, .external_lex_state = 28},
  [2834] = {.lex_state = 157, .external_lex_state = 23},
  [2835] = {.lex_state = 157, .external_lex_state = 28},
  [2836] = {.lex_state = 157, .external_lex_state = 23},
  [2837] = {.lex_state = 157, .external_lex_state = 25},
  [2838] = {.lex_state = 157, .external_lex_state = 23},
  [2839] = {.lex_state = 157, .external_lex_state = 23},
  [2840] = {.lex_state = 157, .external_lex_state = 23},
  [2841] = {.lex_state = 157, .external_lex_state = 23},
  [2842] = {.lex_state = 157, .external_lex_state = 23},
  [2843] = {.lex_state = 157, .external_lex_state = 23},
  [2844] = {.lex_state = 157, .external_lex_state = 23},
  [2845] = {.lex_state = 157, .external_lex_state = 23},
  [2846] = {.lex_state = 157, .external_lex_state = 23},
  [2847] = {.lex_state = 45, .external_lex_state = 24},
  [2848] = {.lex_state = 157, .external_lex_state = 23},
  [2849] = {.lex_state = 157, .external_lex_state = 23},
  [2850] = {.lex_state = 157, .external_lex_state = 23},
  [2851] = {.lex_state = 157, .external_lex_state = 23},
  [2852] = {.lex_state = 157, .external_lex_state = 23},
  [2853] = {.lex_state = 157, .external_lex_state = 23},
  [2854] = {.lex_state = 45, .external_lex_state = 24},
  [2855] = {.lex_state = 157, .external_lex_state = 28},
  [2856] = {.lex_state = 159, .external_lex_state = 27},
  [2857] = {.lex_state = 157, .external_lex_state = 24},
  [2858] = {.lex_state = 157, .external_lex_state = 24},
  [2859] = {.lex_state = 157, .external_lex_state = 23},
  [2860] = {.lex_state = 157, .external_lex_state = 23},
  [2861] = {.lex_state = 157, .external_lex_state = 23},
  [2862] = {.lex_state = 157, .external_lex_state = 23},
  [2863] = {.lex_state = 157, .external_lex_state = 23},
  [2864] = {.lex_state = 157, .external_lex_state = 23},
  [2865] = {.lex_state = 157, .external_lex_state = 23},
  [2866] = {.lex_state = 157, .external_lex_state = 28},
  [2867] = {.lex_state = 157, .external_lex_state = 23},
  [2868] = {.lex_state = 157, .external_lex_state = 23},
  [2869] = {.lex_state = 157, .external_lex_state = 23},
  [2870] = {.lex_state = 157, .external_lex_state = 23},
  [2871] = {.lex_state = 157, .external_lex_state = 23},
  [2872] = {.lex_state = 157, .external_lex_state = 28},
  [2873] = {.lex_state = 45, .external_lex_state = 24},
  [2874] = {.lex_state = 45, .external_lex_state = 24},
  [2875] = {.lex_state = 157, .external_lex_state = 25},
  [2876] = {.lex_state = 45, .external_lex_state = 24},
  [2877] = {.lex_state = 45, .external_lex_state = 24},
  [2878] = {.lex_state = 45, .external_lex_state = 24},
  [2879] = {.lex_state = 45, .external_lex_state = 24},
  [2880] = {.lex_state = 45, .external_lex_state = 24},
  [2881] = {.lex_state = 45, .external_lex_state = 24},
  [2882] = {.lex_state = 45, .external_lex_state = 24},
  [2883] = {.lex_state = 45, .external_lex_state = 24},
  [2884] = {.lex_state = 45, .external_lex_state = 24},
  [2885] = {.lex_state = 45, .external_lex_state = 24},
  [2886] = {.lex_state = 45, .external_lex_state = 24},
  [2887] = {.lex_state = 45, .external_lex_state = 24},
  [2888] = {.lex_state = 157, .external_lex_state = 23},
  [2889] = {.lex_state = 45, .external_lex_state = 24},
  [2890] = {.lex_state = 45, .external_lex_state = 24},
  [2891] = {.lex_state = 45, .external_lex_state = 24},
  [2892] = {.lex_state = 157, .external_lex_state = 23},
  [2893] = {.lex_state = 157, .external_lex_state = 23},
  [2894] = {.lex_state = 45, .external_lex_state = 24},
  [2895] = {.lex_state = 45, .external_lex_state = 24},
  [2896] = {.lex_state = 45, .external_lex_state = 24},
  [2897] = {.lex_state = 159, .external_lex_state = 27},
  [2898] = {.lex_state = 157, .external_lex_state = 23},
  [2899] = {.lex_state = 157, .external_lex_state = 23},
  [2900] = {.lex_state = 45, .external_lex_state = 24},
  [2901] = {.lex_state = 157, .external_lex_state = 23},
  [2902] = {.lex_state = 157, .external_lex_state = 23},
  [2903] = {.lex_state = 45, .external_lex_state = 24},
  [2904] = {.lex_state = 157, .external_lex_state = 28},
  [2905] = {.lex_state = 157, .external_lex_state = 23},
  [2906] = {.lex_state = 157, .external_lex_state = 23},
  [2907] = {.lex_state = 157, .external_lex_state = 23},
  [2908] = {.lex_state = 45, .external_lex_state = 24},
  [2909] = {.lex_state = 45, .external_lex_state = 24},
  [2910] = {.lex_state = 45, .external_lex_state = 24},
  [2911] = {.lex_state = 45, .external_lex_state = 24},
  [2912] = {.lex_state = 159, .external_lex_state = 27},
  [2913] = {.lex_state = 159, .external_lex_state = 27},
  [2914] = {.lex_state = 157, .external_lex_state = 23},
  [2915] = {.lex_state = 45, .external_lex_state = 24},
  [2916] = {.lex_state = 45, .external_lex_state = 24},
  [2917] = {.lex_state = 45, .external_lex_state = 24},
  [2918] = {.lex_state = 45, .external_lex_state = 24},
  [2919] = {.lex_state = 45, .external_lex_state = 24},
  [2920] = {.lex_state = 45, .external_lex_state = 24},
  [2921] = {.lex_state = 45, .external_lex_state = 24},
  [2922] = {.lex_state = 45, .external_lex_state = 24},
  [2923] = {.lex_state = 45, .external_lex_state = 24},
  [2924] = {.lex_state = 45, .external_lex_state = 24},
  [2925] = {.lex_state = 45, .external_lex_state = 24},
  [2926] = {.lex_state = 45, .external_lex_state = 24},
  [2927] = {.lex_state = 45, .external_lex_state = 24},
  [2928] = {.lex_state = 159, .external_lex_state = 27},
  [2929] = {.lex_state = 45, .external_lex_state = 24},
  [2930] = {.lex_state = 45, .external_lex_state = 24},
  [2931] = {.lex_state = 45, .external_lex_state = 24},
  [2932] = {.lex_state = 45, .external_lex_state = 24},
  [2933] = {.lex_state = 45, .external_lex_state = 24},
  [2934] = {.lex_state = 157, .external_lex_state = 23},
  [2935] = {.lex_state = 157, .external_lex_state = 23},
  [2936] = {.lex_state = 45, .external_lex_state = 24},
  [2937] = {.lex_state = 45, .external_lex_state = 24},
  [2938] = {.lex_state = 45, .external_lex_state = 24},
  [2939] = {.lex_state = 45, .external_lex_state = 24},
  [2940] = {.lex_state = 45, .external_lex_state = 24},
  [2941] = {.lex_state = 45, .external_lex_state = 24},
  [2942] = {.lex_state = 157, .external_lex_state = 23},
  [2943] = {.lex_state = 157, .external_lex_state = 23},
  [2944] = {.lex_state = 45, .external_lex_state = 24},
  [2945] = {.lex_state = 45, .external_lex_state = 24},
  [2946] = {.lex_state = 157, .external_lex_state = 28},
  [2947] = {.lex_state = 45, .external_lex_state = 24},
  [2948] = {.lex_state = 45, .external_lex_state = 24},
  [2949] = {.lex_state = 45, .external_lex_state = 24},
  [2950] = {.lex_state = 45, .external_lex_state = 24},
  [2951] = {.lex_state = 45, .external_lex_state = 24},
  [2952] = {.lex_state = 45, .external_lex_state = 24},
  [2953] = {.lex_state = 45, .external_lex_state = 24},
  [2954] = {.lex_state = 45, .external_lex_state = 24},
  [2955] = {.lex_state = 45, .external_lex_state = 24},
  [2956] = {.lex_state = 45, .external_lex_state = 24},
  [2957] = {.lex_state = 45, .external_lex_state = 24},
  [2958] = {.lex_state = 157, .external_lex_state = 23},
  [2959] = {.lex_state = 45, .external_lex_state = 24},
  [2960] = {.lex_state = 45, .external_lex_state = 24},
  [2961] = {.lex_state = 45, .external_lex_state = 24},
  [2962] = {.lex_state = 45, .external_lex_state = 24},
  [2963] = {.lex_state = 45, .external_lex_state = 24},
  [2964] = {.lex_state = 45, .external_lex_state = 24},
  [2965] = {.lex_state = 45, .external_lex_state = 24},
  [2966] = {.lex_state = 157, .external_lex_state = 23},
  [2967] = {.lex_state = 45, .external_lex_state = 24},
  [2968] = {.lex_state = 157, .external_lex_state = 23},
  [2969] = {.lex_state = 159, .external_lex_state = 27},
  [2970] = {.lex_state = 45, .external_lex_state = 24},
  [2971] = {.lex_state = 45, .external_lex_state = 24},
  [2972] = {.lex_state = 45, .external_lex_state = 24},
  [2973] = {.lex_state = 45, .external_lex_state = 24},
  [2974] = {.lex_state = 45, .external_lex_state = 24},
  [2975] = {.lex_state = 45, .external_lex_state = 24},
  [2976] = {.lex_state = 45, .external_lex_state = 24},
  [2977] = {.lex_state = 45, .external_lex_state = 24},
  [2978] = {.lex_state = 45, .external_lex_state = 24},
  [2979] = {.lex_state = 45, .external_lex_state = 24},
  [2980] = {.lex_state = 45, .external_lex_state = 24},
  [2981] = {.lex_state = 45, .external_lex_state = 24},
  [2982] = {.lex_state = 45, .external_lex_state = 24},
  [2983] = {.lex_state = 45, .external_lex_state = 24},
  [2984] = {.lex_state = 45, .external_lex_state = 24},
  [2985] = {.lex_state = 45, .external_lex_state = 24},
  [2986] = {.lex_state = 45, .external_lex_state = 24},
  [2987] = {.lex_state = 45, .external_lex_state = 24},
  [2988] = {.lex_state = 45, .external_lex_state = 24},
  [2989] = {.lex_state = 45, .external_lex_state = 24},
  [2990] = {.lex_state = 45, .external_lex_state = 24},
  [2991] = {.lex_state = 45, .external_lex_state = 24},
  [2992] = {.lex_state = 45, .external_lex_state = 24},
  [2993] = {.lex_state = 45, .external_lex_state = 24},
  [2994] = {.lex_state = 45, .external_lex_state = 24},
  [2995] = {.lex_state = 45, .external_lex_state = 24},
  [2996] = {.lex_state = 45, .external_lex_state = 24},
  [2997] = {.lex_state = 45, .external_lex_state = 24},
  [2998] = {.lex_state = 45, .external_lex_state = 24},
  [2999] = {.lex_state = 45, .external_lex_state = 24},
  [3000] = {.lex_state = 157, .external_lex_state = 23},
  [3001] = {.lex_state = 45, .external_lex_state = 24},
  [3002] = {.lex_state = 45, .external_lex_state = 24},
  [3003] = {.lex_state = 45, .external_lex_state = 24},
  [3004] = {.lex_state = 157, .external_lex_state = 23},
  [3005] = {.lex_state = 45, .external_lex_state = 24},
  [3006] = {.lex_state = 45, .external_lex_state = 24},
  [3007] = {.lex_state = 45, .external_lex_state = 24},
  [3008] = {.lex_state = 45, .external_lex_state = 24},
  [3009] = {.lex_state = 45, .external_lex_state = 24},
  [3010] = {.lex_state = 45, .external_lex_state = 24},
  [3011] = {.lex_state = 45, .external_lex_state = 24},
  [3012] = {.lex_state = 45, .external_lex_state = 24},
  [3013] = {.lex_state = 45, .external_lex_state = 24},
  [3014] = {.lex_state = 45, .external_lex_state = 24},
  [3015] = {.lex_state = 157, .external_lex_state = 23},
  [3016] = {.lex_state = 157, .external_lex_state = 23},
  [3017] = {.lex_state = 45, .external_lex_state = 24},
  [3018] = {.lex_state = 45, .external_lex_state = 24},
  [3019] = {.lex_state = 157, .external_lex_state = 23},
  [3020] = {.lex_state = 45, .external_lex_state = 24},
  [3021] = {.lex_state = 45, .external_lex_state = 24},
  [3022] = {.lex_state = 45, .external_lex_state = 24},
  [3023] = {.lex_state = 45, .external_lex_state = 24},
  [3024] = {.lex_state = 45, .external_lex_state = 24},
  [3025] = {.lex_state = 45, .external_lex_state = 24},
  [3026] = {.lex_state = 45, .external_lex_state = 24},
  [3027] = {.lex_state = 45, .external_lex_state = 24},
  [3028] = {.lex_state = 157, .external_lex_state = 23},
  [3029] = {.lex_state = 157, .external_lex_state = 23},
  [3030] = {.lex_state = 157, .external_lex_state = 23},
  [3031] = {.lex_state = 157, .external_lex_state = 23},
  [3032] = {.lex_state = 157, .external_lex_state = 23},
  [3033] = {.lex_state = 157, .external_lex_state = 23},
  [3034] = {.lex_state = 157, .external_lex_state = 23},
  [3035] = {.lex_state = 157, .external_lex_state = 23},
  [3036] = {.lex_state = 157, .external_lex_state = 23},
  [3037] = {.lex_state = 157, .external_lex_state = 23},
  [3038] = {.lex_state = 157, .external_lex_state = 23},
  [3039] = {.lex_state = 45, .external_lex_state = 24},
  [3040] = {.lex_state = 157, .external_lex_state = 23},
  [3041] = {.lex_state = 157, .external_lex_state = 23},
  [3042] = {.lex_state = 45, .external_lex_state = 24},
  [3043] = {.lex_state = 45, .external_lex_state = 24},
  [3044] = {.lex_state = 157, .external_lex_state = 23},
  [3045] = {.lex_state = 45, .external_lex_state = 24},
  [3046] = {.lex_state = 45, .external_lex_state = 24},
  [3047] = {.lex_state = 45, .external_lex_state = 24},
  [3048] = {.lex_state = 45, .external_lex_state = 24},
  [3049] = {.lex_state = 45, .external_lex_state = 24},
  [3050] = {.lex_state = 45, .external_lex_state = 24},
  [3051] = {.lex_state = 157, .external_lex_state = 23},
  [3052] = {.lex_state = 157, .external_lex_state = 23},
  [3053] = {.lex_state = 157, .external_lex_state = 23},
  [3054] = {.lex_state = 157, .external_lex_state = 23},
  [3055] = {.lex_state = 157, .external_lex_state = 23},
  [3056] = {.lex_state = 157, .external_lex_state = 23},
  [3057] = {.lex_state = 157, .external_lex_state = 23},
  [3058] = {.lex_state = 157, .external_lex_state = 23},
  [3059] = {.lex_state = 157, .external_lex_state = 23},
  [3060] = {.lex_state = 157, .external_lex_state = 23},
  [3061] = {.lex_state = 157, .external_lex_state = 23},
  [3062] = {.lex_state = 157, .external_lex_state = 23},
  [3063] = {.lex_state = 157, .external_lex_state = 23},
  [3064] = {.lex_state = 45, .external_lex_state = 24},
  [3065] = {.lex_state = 45, .external_lex_state = 24},
  [3066] = {.lex_state = 157, .external_lex_state = 23},
  [3067] = {.lex_state = 45, .external_lex_state = 24},
  [3068] = {.lex_state = 45, .external_lex_state = 24},
  [3069] = {.lex_state = 45, .external_lex_state = 24},
  [3070] = {.lex_state = 45, .external_lex_state = 24},
  [3071] = {.lex_state = 45, .external_lex_state = 24},
  [3072] = {.lex_state = 45, .external_lex_state = 24},
  [3073] = {.lex_state = 45, .external_lex_state = 24},
  [3074] = {.lex_state = 45, .external_lex_state = 24},
  [3075] = {.lex_state = 45, .external_lex_state = 24},
  [3076] = {.lex_state = 45, .external_lex_state = 24},
  [3077] = {.lex_state = 159, .external_lex_state = 27},
  [3078] = {.lex_state = 157, .external_lex_state = 24},
  [3079] = {.lex_state = 157, .external_lex_state = 23},
  [3080] = {.lex_state = 36, .external_lex_state = 29},
  [3081] = {.lex_state = 36, .external_lex_state = 29},
  [3082] = {.lex_state = 36, .external_lex_state = 29},
  [3083] = {.lex_state = 159, .external_lex_state = 27},
  [3084] = {.lex_state = 157, .external_lex_state = 23},
  [3085] = {.lex_state = 157, .external_lex_state = 28},
  [3086] = {.lex_state = 157, .external_lex_state = 23},
  [3087] = {.lex_state = 36, .external_lex_state = 29},
  [3088] = {.lex_state = 36, .external_lex_state = 29},
  [3089] = {.lex_state = 157, .external_lex_state = 23},
  [3090] = {.lex_state = 157, .external_lex_state = 23},
  [3091] = {.lex_state = 157, .external_lex_state = 23},
  [3092] = {.lex_state = 157, .external_lex_state = 23},
  [3093] = {.lex_state = 157, .external_lex_state = 28},
  [3094] = {.lex_state = 157, .external_lex_state = 23},
  [3095] = {.lex_state = 159, .external_lex_state = 27},
  [3096] = {.lex_state = 157, .external_lex_state = 24},
  [3097] = {.lex_state = 36, .external_lex_state = 29},
  [3098] = {.lex_state = 36, .external_lex_state = 29},
  [3099] = {.lex_state = 157, .external_lex_state = 23},
  [3100] = {.lex_state = 157, .external_lex_state = 23},
  [3101] = {.lex_state = 157, .external_lex_state = 23},
  [3102] = {.lex_state = 157, .external_lex_state = 23},
  [3103] = {.lex_state = 157, .external_lex_state = 23},
  [3104] = {.lex_state = 36, .external_lex_state = 29},
  [3105] = {.lex_state = 157, .external_lex_state = 23},
  [3106] = {.lex_state = 157, .external_lex_state = 23},
  [3107] = {.lex_state = 34, .external_lex_state = 26},
  [3108] = {.lex_state = 157, .external_lex_state = 24},
  [3109] = {.lex_state = 157, .external_lex_state = 23},
  [3110] = {.lex_state = 36, .external_lex_state = 29},
  [3111] = {.lex_state = 157, .external_lex_state = 23},
  [3112] = {.lex_state = 157, .external_lex_state = 23},
  [3113] = {.lex_state = 36, .external_lex_state = 29},
  [3114] = {.lex_state = 159, .external_lex_state = 27},
  [3115] = {.lex_state = 159, .external_lex_state = 27},
  [3116] = {.lex_state = 157, .external_lex_state = 23},
  [3117] = {.lex_state = 157, .external_lex_state = 23},
  [3118] = {.lex_state = 157, .external_lex_state = 23},
  [3119] = {.lex_state = 157, .external_lex_state = 23},
  [3120] = {.lex_state = 157, .external_lex_state = 23},
  [3121] = {.lex_state = 157, .external_lex_state = 23},
  [3122] = {.lex_state = 157, .external_lex_state = 23},
  [3123] = {.lex_state = 36, .external_lex_state = 29},
  [3124] = {.lex_state = 157, .external_lex_state = 23},
  [3125] = {.lex_state = 157, .external_lex_state = 23},
  [3126] = {.lex_state = 159, .external_lex_state = 27},
  [3127] = {.lex_state = 157, .external_lex_state = 23},
  [3128] = {.lex_state = 159, .external_lex_state = 27},
  [3129] = {.lex_state = 36, .external_lex_state = 29},
  [3130] = {.lex_state = 159, .external_lex_state = 27},
  [3131] = {.lex_state = 36, .external_lex_state = 29},
  [3132] = {.lex_state = 36, .external_lex_state = 29},
  [3133] = {.lex_state = 36, .external_lex_state = 29},
  [3134] = {.lex_state = 36, .external_lex_state = 29},
  [3135] = {.lex_state = 36, .external_lex_state = 29},
  [3136] = {.lex_state = 36, .external_lex_state = 29},
  [3137] = {.lex_state = 36, .external_lex_state = 29},
  [3138] = {.lex_state = 36, .external_lex_state = 29},
  [3139] = {.lex_state = 157, .external_lex_state = 25},
  [3140] = {.lex_state = 159, .external_lex_state = 27},
  [3141] = {.lex_state = 159, .external_lex_state = 27},
  [3142] = {.lex_state = 159, .external_lex_state = 27},
  [3143] = {.lex_state = 36, .external_lex_state = 29},
  [3144] = {.lex_state = 36, .external_lex_state = 29},
  [3145] = {.lex_state = 159, .external_lex_state = 27},
  [3146] = {.lex_state = 159, .external_lex_state = 27},
  [3147] = {.lex_state = 159, .external_lex_state = 27},
  [3148] = {.lex_state = 159, .external_lex_state = 27},
  [3149] = {.lex_state = 159, .external_lex_state = 27},
  [3150] = {.lex_state = 159, .external_lex_state = 27},
  [3151] = {.lex_state = 159, .external_lex_state = 27},
  [3152] = {.lex_state = 159, .external_lex_state = 27},
  [3153] = {.lex_state = 159, .external_lex_state = 27},
  [3154] = {.lex_state = 159, .external_lex_state = 27},
  [3155] = {.lex_state = 159, .external_lex_state = 27},
  [3156] = {.lex_state = 159, .external_lex_state = 27},
  [3157] = {.lex_state = 159, .external_lex_state = 27},
  [3158] = {.lex_state = 159, .external_lex_state = 27},
  [3159] = {.lex_state = 159, .external_lex_state = 27},
  [3160] = {.lex_state = 157, .external_lex_state = 23},
  [3161] = {.lex_state = 157, .external_lex_state = 23},
  [3162] = {.lex_state = 157, .external_lex_state = 23},
  [3163] = {.lex_state = 36, .external_lex_state = 29},
  [3164] = {.lex_state = 36, .external_lex_state = 29},
  [3165] = {.lex_state = 159, .external_lex_state = 27},
  [3166] = {.lex_state = 36, .external_lex_state = 29},
  [3167] = {.lex_state = 36, .external_lex_state = 29},
  [3168] = {.lex_state = 36, .external_lex_state = 29},
  [3169] = {.lex_state = 159, .external_lex_state = 27},
  [3170] = {.lex_state = 159, .external_lex_state = 27},
  [3171] = {.lex_state = 157, .external_lex_state = 23},
  [3172] = {.lex_state = 36, .external_lex_state = 29},
  [3173] = {.lex_state = 36, .external_lex_state = 29},
  [3174] = {.lex_state = 157, .external_lex_state = 23},
  [3175] = {.lex_state = 157, .external_lex_state = 24},
  [3176] = {.lex_state = 157, .external_lex_state = 23},
  [3177] = {.lex_state = 157, .external_lex_state = 23},
  [3178] = {.lex_state = 157, .external_lex_state = 24},
  [3179] = {.lex_state = 159, .external_lex_state = 27},
  [3180] = {.lex_state = 32, .external_lex_state = 26},
  [3181] = {.lex_state = 32, .external_lex_state = 26},
  [3182] = {.lex_state = 159, .external_lex_state = 27},
  [3183] = {.lex_state = 159, .external_lex_state = 27},
  [3184] = {.lex_state = 157, .external_lex_state = 28},
  [3185] = {.lex_state = 32, .external_lex_state = 26},
  [3186] = {.lex_state = 32, .external_lex_state = 26},
  [3187] = {.lex_state = 32, .external_lex_state = 26},
  [3188] = {.lex_state = 157, .external_lex_state = 23},
  [3189] = {.lex_state = 32, .external_lex_state = 26},
  [3190] = {.lex_state = 157, .external_lex_state = 23},
  [3191] = {.lex_state = 157, .external_lex_state = 24},
  [3192] = {.lex_state = 45, .external_lex_state = 24},
  [3193] = {.lex_state = 159, .external_lex_state = 27},
  [3194] = {.lex_state = 157, .external_lex_state = 24},
  [3195] = {.lex_state = 157, .external_lex_state = 24},
  [3196] = {.lex_state = 157, .external_lex_state = 23},
  [3197] = {.lex_state = 157, .external_lex_state = 24},
  [3198] = {.lex_state = 157, .external_lex_state = 28},
  [3199] = {.lex_state = 32, .external_lex_state = 26},
  [3200] = {.lex_state = 157, .external_lex_state = 24},
  [3201] = {.lex_state = 157, .external_lex_state = 23},
  [3202] = {.lex_state = 32, .external_lex_state = 26},
  [3203] = {.lex_state = 32, .external_lex_state = 26},
  [3204] = {.lex_state = 157, .external_lex_state = 24},
  [3205] = {.lex_state = 157, .external_lex_state = 24},
  [3206] = {.lex_state = 157, .external_lex_state = 24},
  [3207] = {.lex_state = 159, .external_lex_state = 27},
  [3208] = {.lex_state = 32, .external_lex_state = 26},
  [3209] = {.lex_state = 159, .external_lex_state = 27},
  [3210] = {.lex_state = 159, .external_lex_state = 27},
  [3211] = {.lex_state = 159, .external_lex_state = 27},
  [3212] = {.lex_state = 157, .external_lex_state = 24},
  [3213] = {.lex_state = 159, .external_lex_state = 27},
  [3214] = {.lex_state = 157, .external_lex_state = 24},
  [3215] = {.lex_state = 159, .external_lex_state = 27},
  [3216] = {.lex_state = 159, .external_lex_state = 27},
  [3217] = {.lex_state = 159, .external_lex_state = 27},
  [3218] = {.lex_state = 159, .external_lex_state = 27},
  [3219] = {.lex_state = 159, .external_lex_state = 27},
  [3220] = {.lex_state = 157, .external_lex_state = 24},
  [3221] = {.lex_state = 157, .external_lex_state = 23},
  [3222] = {.lex_state = 157, .external_lex_state = 23},
  [3223] = {.lex_state = 157, .external_lex_state = 24},
  [3224] = {.lex_state = 157, .external_lex_state = 23},
  [3225] = {.lex_state = 159, .external_lex_state = 27},
  [3226] = {.lex_state = 157, .external_lex_state = 23},
  [3227] = {.lex_state = 159, .external_lex_state = 27},
  [3228] = {.lex_state = 159, .external_lex_state = 27},
  [3229] = {.lex_state = 159, .external_lex_state = 27},
  [3230] = {.lex_state = 157, .external_lex_state = 23},
  [3231] = {.lex_state = 157, .external_lex_state = 24},
  [3232] = {.lex_state = 157, .external_lex_state = 24},
  [3233] = {.lex_state = 157, .external_lex_state = 23},
  [3234] = {.lex_state = 157, .external_lex_state = 23},
  [3235] = {.lex_state = 157, .external_lex_state = 24},
  [3236] = {.lex_state = 157, .external_lex_state = 24},
  [3237] = {.lex_state = 157, .external_lex_state = 24},
  [3238] = {.lex_state = 157, .external_lex_state = 24},
  [3239] = {.lex_state = 157, .external_lex_state = 23},
  [3240] = {.lex_state = 159, .external_lex_state = 27},
  [3241] = {.lex_state = 157, .external_lex_state = 24},
  [3242] = {.lex_state = 159, .external_lex_state = 27},
  [3243] = {.lex_state = 157, .external_lex_state = 24},
  [3244] = {.lex_state = 157, .external_lex_state = 24},
  [3245] = {.lex_state = 157, .external_lex_state = 24},
  [3246] = {.lex_state = 159, .external_lex_state = 27},
  [3247] = {.lex_state = 157, .external_lex_state = 24},
  [3248] = {.lex_state = 157, .external_lex_state = 24},
  [3249] = {.lex_state = 157, .external_lex_state = 24},
  [3250] = {.lex_state = 157, .external_lex_state = 24},
  [3251] = {.lex_state = 157, .external_lex_state = 24},
  [3252] = {.lex_state = 157, .external_lex_state = 24},
  [3253] = {.lex_state = 157, .external_lex_state = 24},
  [3254] = {.lex_state = 157, .external_lex_state = 24},
  [3255] = {.lex_state = 157, .external_lex_state = 24},
  [3256] = {.lex_state = 157, .external_lex_state = 24},
  [3257] = {.lex_state = 157, .external_lex_state = 24},
  [3258] = {.lex_state = 157, .external_lex_state = 24},
  [3259] = {.lex_state = 157, .external_lex_state = 24},
  [3260] = {.lex_state = 157, .external_lex_state = 24},
  [3261] = {.lex_state = 157, .external_lex_state = 24},
  [3262] = {.lex_state = 32, .external_lex_state = 29},
  [3263] = {.lex_state = 157, .external_lex_state = 24},
  [3264] = {.lex_state = 159, .external_lex_state = 27},
  [3265] = {.lex_state = 157, .external_lex_state = 24},
  [3266] = {.lex_state = 159, .external_lex_state = 27},
  [3267] = {.lex_state = 157, .external_lex_state = 24},
  [3268] = {.lex_state = 157, .external_lex_state = 24},
  [3269] = {.lex_state = 157, .external_lex_state = 24},
  [3270] = {.lex_state = 157, .external_lex_state = 24},
  [3271] = {.lex_state = 157, .external_lex_state = 24},
  [3272] = {.lex_state = 157, .external_lex_state = 24},
  [3273] = {.lex_state = 159, .external_lex_state = 27},
  [3274] = {.lex_state = 159, .external_lex_state = 27},
  [3275] = {.lex_state = 159, .external_lex_state = 27},
  [3276] = {.lex_state = 159, .external_lex_state = 27},
  [3277] = {.lex_state = 159, .external_lex_state = 27},
  [3278] = {.lex_state = 159, .external_lex_state = 27},
  [3279] = {.lex_state = 157, .external_lex_state = 24},
  [3280] = {.lex_state = 159, .external_lex_state = 27},
  [3281] = {.lex_state = 157, .external_lex_state = 24},
  [3282] = {.lex_state = 157, .external_lex_state = 28},
  [3283] = {.lex_state = 157, .external_lex_state = 24},
  [3284] = {.lex_state = 157, .external_lex_state = 24},
  [3285] = {.lex_state = 157, .external_lex_state = 24},
  [3286] = {.lex_state = 157, .external_lex_state = 24},
  [3287] = {.lex_state = 157, .external_lex_state = 24},
  [3288] = {.lex_state = 157, .external_lex_state = 24},
  [3289] = {.lex_state = 157, .external_lex_state = 24},
  [3290] = {.lex_state = 157, .external_lex_state = 24},
  [3291] = {.lex_state = 157, .external_lex_state = 24},
  [3292] = {.lex_state = 157, .external_lex_state = 24},
  [3293] = {.lex_state = 157, .external_lex_state = 24},
  [3294] = {.lex_state = 157, .external_lex_state = 24},
  [3295] = {.lex_state = 157, .external_lex_state = 24},
  [3296] = {.lex_state = 157, .external_lex_state = 24},
  [3297] = {.lex_state = 157, .external_lex_state = 24},
  [3298] = {.lex_state = 157, .external_lex_state = 24},
  [3299] = {.lex_state = 157, .external_lex_state = 24},
  [3300] = {.lex_state = 157, .external_lex_state = 24},
  [3301] = {.lex_state = 32, .external_lex_state = 29},
  [3302] = {.lex_state = 157, .external_lex_state = 24},
  [3303] = {.lex_state = 32, .external_lex_state = 29},
  [3304] = {.lex_state = 157, .external_lex_state = 24},
  [3305] = {.lex_state = 159, .external_lex_state = 27},
  [3306] = {.lex_state = 157, .external_lex_state = 23},
  [3307] = {.lex_state = 157, .external_lex_state = 23},
  [3308] = {.lex_state = 157, .external_lex_state = 23},
  [3309] = {.lex_state = 157, .external_lex_state = 23},
  [3310] = {.lex_state = 157, .external_lex_state = 23},
  [3311] = {.lex_state = 157, .external_lex_state = 23},
  [3312] = {.lex_state = 157, .external_lex_state = 23},
  [3313] = {.lex_state = 157, .external_lex_state = 23},
  [3314] = {.lex_state = 157, .external_lex_state = 23},
  [3315] = {.lex_state = 157, .external_lex_state = 23},
  [3316] = {.lex_state = 157, .external_lex_state = 23},
  [3317] = {.lex_state = 157, .external_lex_state = 23},
  [3318] = {.lex_state = 157, .external_lex_state = 23},
  [3319] = {.lex_state = 157, .external_lex_state = 23},
  [3320] = {.lex_state = 157, .external_lex_state = 24},
  [3321] = {.lex_state = 157, .external_lex_state = 24},
  [3322] = {.lex_state = 159, .external_lex_state = 27},
  [3323] = {.lex_state = 157, .external_lex_state = 24},
  [3324] = {.lex_state = 159, .external_lex_state = 27},
  [3325] = {.lex_state = 159, .external_lex_state = 27},
  [3326] = {.lex_state = 159, .external_lex_state = 27},
  [3327] = {.lex_state = 159, .external_lex_state = 27},
  [3328] = {.lex_state = 157, .external_lex_state = 24},
  [3329] = {.lex_state = 159, .external_lex_state = 27},
  [3330] = {.lex_state = 159, .external_lex_state = 27},
  [3331] = {.lex_state = 159, .external_lex_state = 27},
  [3332] = {.lex_state = 157, .external_lex_state = 24},
  [3333] = {.lex_state = 159, .external_lex_state = 27},
  [3334] = {.lex_state = 157, .external_lex_state = 24},
  [3335] = {.lex_state = 159, .external_lex_state = 27},
  [3336] = {.lex_state = 157, .external_lex_state = 23},
  [3337] = {.lex_state = 157, .external_lex_state = 24},
  [3338] = {.lex_state = 159, .external_lex_state = 27},
  [3339] = {.lex_state = 159, .external_lex_state = 27},
  [3340] = {.lex_state = 159, .external_lex_state = 27},
  [3341] = {.lex_state = 159, .external_lex_state = 27},
  [3342] = {.lex_state = 159, .external_lex_state = 27},
  [3343] = {.lex_state = 157, .external_lex_state = 24},
  [3344] = {.lex_state = 159, .external_lex_state = 27},
  [3345] = {.lex_state = 159, .external_lex_state = 27},
  [3346] = {.lex_state = 157, .external_lex_state = 23},
  [3347] = {.lex_state = 157, .external_lex_state = 23},
  [3348] = {.lex_state = 159, .external_lex_state = 27},
  [3349] = {.lex_state = 157, .external_lex_state = 23},
  [3350] = {.lex_state = 157, .external_lex_state = 24},
  [3351] = {.lex_state = 157, .external_lex_state = 24},
  [3352] = {.lex_state = 157, .external_lex_state = 23},
  [3353] = {.lex_state = 157, .external_lex_state = 24},
  [3354] = {.lex_state = 159, .external_lex_state = 27},
  [3355] = {.lex_state = 157, .external_lex_state = 24},
  [3356] = {.lex_state = 157, .external_lex_state = 24},
  [3357] = {.lex_state = 157, .external_lex_state = 24},
  [3358] = {.lex_state = 157, .external_lex_state = 24},
  [3359] = {.lex_state = 157, .external_lex_state = 24},
  [3360] = {.lex_state = 159, .external_lex_state = 27},
  [3361] = {.lex_state = 157, .external_lex_state = 24},
  [3362] = {.lex_state = 157, .external_lex_state = 24},
  [3363] = {.lex_state = 159, .external_lex_state = 27},
  [3364] = {.lex_state = 159, .external_lex_state = 27},
  [3365] = {.lex_state = 157, .external_lex_state = 24},
  [3366] = {.lex_state = 159, .external_lex_state = 27},
  [3367] = {.lex_state = 157, .external_lex_state = 23},
  [3368] = {.lex_state = 157, .external_lex_state = 24},
  [3369] = {.lex_state = 160, .external_lex_state = 22},
  [3370] = {.lex_state = 157, .external_lex_state = 24},
  [3371] = {.lex_state = 157, .external_lex_state = 24},
  [3372] = {.lex_state = 157, .external_lex_state = 24},
  [3373] = {.lex_state = 157, .external_lex_state = 24},
  [3374] = {.lex_state = 157, .external_lex_state = 24},
  [3375] = {.lex_state = 157, .external_lex_state = 24},
  [3376] = {.lex_state = 157, .external_lex_state = 24},
  [3377] = {.lex_state = 157, .external_lex_state = 24},
  [3378] = {.lex_state = 157, .external_lex_state = 24},
  [3379] = {.lex_state = 157, .external_lex_state = 24},
  [3380] = {.lex_state = 157, .external_lex_state = 24},
  [3381] = {.lex_state = 157, .external_lex_state = 24},
  [3382] = {.lex_state = 159, .external_lex_state = 30},
  [3383] = {.lex_state = 160, .external_lex_state = 22},
  [3384] = {.lex_state = 157, .external_lex_state = 23},
  [3385] = {.lex_state = 159, .external_lex_state = 30},
  [3386] = {.lex_state = 159, .external_lex_state = 27},
  [3387] = {.lex_state = 159, .external_lex_state = 27},
  [3388] = {.lex_state = 159, .external_lex_state = 27},
  [3389] = {.lex_state = 157, .external_lex_state = 24},
  [3390] = {.lex_state = 157, .external_lex_state = 24},
  [3391] = {.lex_state = 157, .external_lex_state = 24},
  [3392] = {.lex_state = 157, .external_lex_state = 24},
  [3393] = {.lex_state = 157, .external_lex_state = 24},
  [3394] = {.lex_state = 157, .external_lex_state = 24},
  [3395] = {.lex_state = 157, .external_lex_state = 24},
  [3396] = {.lex_state = 157, .external_lex_state = 24},
  [3397] = {.lex_state = 157, .external_lex_state = 24},
  [3398] = {.lex_state = 157, .external_lex_state = 24},
  [3399] = {.lex_state = 157, .external_lex_state = 24},
  [3400] = {.lex_state = 159, .external_lex_state = 27},
  [3401] = {.lex_state = 157, .external_lex_state = 24},
  [3402] = {.lex_state = 157, .external_lex_state = 24},
  [3403] = {.lex_state = 157, .external_lex_state = 23},
  [3404] = {.lex_state = 159, .external_lex_state = 27},
  [3405] = {.lex_state = 159, .external_lex_state = 27},
  [3406] = {.lex_state = 157, .external_lex_state = 23},
  [3407] = {.lex_state = 157, .external_lex_state = 24},
  [3408] = {.lex_state = 157, .external_lex_state = 24},
  [3409] = {.lex_state = 157, .external_lex_state = 24},
  [3410] = {.lex_state = 157, .external_lex_state = 24},
  [3411] = {.lex_state = 157, .external_lex_state = 24},
  [3412] = {.lex_state = 157, .external_lex_state = 24},
  [3413] = {.lex_state = 157, .external_lex_state = 24},
  [3414] = {.lex_state = 157, .external_lex_state = 24},
  [3415] = {.lex_state = 157, .external_lex_state = 24},
  [3416] = {.lex_state = 159, .external_lex_state = 30},
  [3417] = {.lex_state = 157, .external_lex_state = 23},
  [3418] = {.lex_state = 159, .external_lex_state = 27},
  [3419] = {.lex_state = 159, .external_lex_state = 30},
  [3420] = {.lex_state = 159, .external_lex_state = 27},
  [3421] = {.lex_state = 157, .external_lex_state = 24},
  [3422] = {.lex_state = 159, .external_lex_state = 27},
  [3423] = {.lex_state = 159, .external_lex_state = 27},
  [3424] = {.lex_state = 159, .external_lex_state = 27},
  [3425] = {.lex_state = 159, .external_lex_state = 27},
  [3426] = {.lex_state = 159, .external_lex_state = 27},
  [3427] = {.lex_state = 157, .external_lex_state = 23},
  [3428] = {.lex_state = 159, .external_lex_state = 27},
  [3429] = {.lex_state = 159, .external_lex_state = 27},
  [3430] = {.lex_state = 159, .external_lex_state = 27},
  [3431] = {.lex_state = 159, .external_lex_state = 27},
  [3432] = {.lex_state = 159, .external_lex_state = 27},
  [3433] = {.lex_state = 159, .external_lex_state = 27},
  [3434] = {.lex_state = 159, .external_lex_state = 27},
  [3435] = {.lex_state = 157, .external_lex_state = 24},
  [3436] = {.lex_state = 159, .external_lex_state = 27},
  [3437] = {.lex_state = 159, .external_lex_state = 27},
  [3438] = {.lex_state = 159, .external_lex_state = 27},
  [3439] = {.lex_state = 157, .external_lex_state = 23},
  [3440] = {.lex_state = 159, .external_lex_state = 27},
  [3441] = {.lex_state = 159, .external_lex_state = 27},
  [3442] = {.lex_state = 159, .external_lex_state = 27},
  [3443] = {.lex_state = 159, .external_lex_state = 27},
  [3444] = {.lex_state = 159, .external_lex_state = 27},
  [3445] = {.lex_state = 159, .external_lex_state = 27},
  [3446] = {.lex_state = 159, .external_lex_state = 27},
  [3447] = {.lex_state = 159, .external_lex_state = 27},
  [3448] = {.lex_state = 159, .external_lex_state = 27},
  [3449] = {.lex_state = 159, .external_lex_state = 27},
  [3450] = {.lex_state = 159, .external_lex_state = 27},
  [3451] = {.lex_state = 159, .external_lex_state = 27},
  [3452] = {.lex_state = 159, .external_lex_state = 27},
  [3453] = {.lex_state = 157, .external_lex_state = 24},
  [3454] = {.lex_state = 157, .external_lex_state = 24},
  [3455] = {.lex_state = 160, .external_lex_state = 22},
  [3456] = {.lex_state = 159, .external_lex_state = 30},
  [3457] = {.lex_state = 159, .external_lex_state = 27},
  [3458] = {.lex_state = 159, .external_lex_state = 27},
  [3459] = {.lex_state = 159, .external_lex_state = 27},
  [3460] = {.lex_state = 159, .external_lex_state = 27},
  [3461] = {.lex_state = 157, .external_lex_state = 28},
  [3462] = {.lex_state = 160, .external_lex_state = 22},
  [3463] = {.lex_state = 159, .external_lex_state = 30},
  [3464] = {.lex_state = 160, .external_lex_state = 22},
  [3465] = {.lex_state = 159, .external_lex_state = 30},
  [3466] = {.lex_state = 159, .external_lex_state = 27},
  [3467] = {.lex_state = 157, .external_lex_state = 24},
  [3468] = {.lex_state = 157, .external_lex_state = 24},
  [3469] = {.lex_state = 159, .external_lex_state = 27},
  [3470] = {.lex_state = 159, .external_lex_state = 27},
  [3471] = {.lex_state = 159, .external_lex_state = 27},
  [3472] = {.lex_state = 159, .external_lex_state = 27},
  [3473] = {.lex_state = 159, .external_lex_state = 27},
  [3474] = {.lex_state = 159, .external_lex_state = 27},
  [3475] = {.lex_state = 159, .external_lex_state = 27},
  [3476] = {.lex_state = 159, .external_lex_state = 27},
  [3477] = {.lex_state = 157, .external_lex_state = 23},
  [3478] = {.lex_state = 159, .external_lex_state = 27},
  [3479] = {.lex_state = 159, .external_lex_state = 27},
  [3480] = {.lex_state = 157, .external_lex_state = 23},
  [3481] = {.lex_state = 159, .external_lex_state = 27},
  [3482] = {.lex_state = 159, .external_lex_state = 27},
  [3483] = {.lex_state = 159, .external_lex_state = 27},
  [3484] = {.lex_state = 159, .external_lex_state = 27},
  [3485] = {.lex_state = 157, .external_lex_state = 23},
  [3486] = {.lex_state = 157, .external_lex_state = 23},
  [3487] = {.lex_state = 159, .external_lex_state = 27},
  [3488] = {.lex_state = 159, .external_lex_state = 27},
  [3489] = {.lex_state = 159, .external_lex_state = 27},
  [3490] = {.lex_state = 157, .external_lex_state = 23},
  [3491] = {.lex_state = 159, .external_lex_state = 27},
  [3492] = {.lex_state = 157, .external_lex_state = 23},
  [3493] = {.lex_state = 157, .external_lex_state = 23},
  [3494] = {.lex_state = 157, .external_lex_state = 23},
  [3495] = {.lex_state = 157, .external_lex_state = 23},
  [3496] = {.lex_state = 157, .external_lex_state = 24},
  [3497] = {.lex_state = 159, .external_lex_state = 27},
  [3498] = {.lex_state = 159, .external_lex_state = 27},
  [3499] = {.lex_state = 159, .external_lex_state = 27},
  [3500] = {.lex_state = 159, .external_lex_state = 27},
  [3501] = {.lex_state = 159, .external_lex_state = 27},
  [3502] = {.lex_state = 157, .external_lex_state = 24},
  [3503] = {.lex_state = 45, .external_lex_state = 28},
  [3504] = {.lex_state = 157, .external_lex_state = 23},
  [3505] = {.lex_state = 159, .external_lex_state = 27},
  [3506] = {.lex_state = 159, .external_lex_state = 27},
  [3507] = {.lex_state = 45, .external_lex_state = 24},
  [3508] = {.lex_state = 159, .external_lex_state = 27},
  [3509] = {.lex_state = 157, .external_lex_state = 23},
  [3510] = {.lex_state = 159, .external_lex_state = 27},
  [3511] = {.lex_state = 157, .external_lex_state = 23},
  [3512] = {.lex_state = 157, .external_lex_state = 23},
  [3513] = {.lex_state = 45, .external_lex_state = 28},
  [3514] = {.lex_state = 157, .external_lex_state = 24},
  [3515] = {.lex_state = 157, .external_lex_state = 24},
  [3516] = {.lex_state = 159, .external_lex_state = 27},
  [3517] = {.lex_state = 157, .external_lex_state = 23},
  [3518] = {.lex_state = 157, .external_lex_state = 23},
  [3519] = {.lex_state = 159, .external_lex_state = 27},
  [3520] = {.lex_state = 159, .external_lex_state = 27},
  [3521] = {.lex_state = 159, .external_lex_state = 27},
  [3522] = {.lex_state = 45, .external_lex_state = 24},
  [3523] = {.lex_state = 159, .external_lex_state = 27},
  [3524] = {.lex_state = 159, .external_lex_state = 27},
  [3525] = {.lex_state = 159, .external_lex_state = 27},
  [3526] = {.lex_state = 157, .external_lex_state = 23},
  [3527] = {.lex_state = 157, .external_lex_state = 23},
  [3528] = {.lex_state = 157, .external_lex_state = 23},
  [3529] = {.lex_state = 159, .external_lex_state = 27},
  [3530] = {.lex_state = 157, .external_lex_state = 23},
  [3531] = {.lex_state = 157, .external_lex_state = 23},
  [3532] = {.lex_state = 157, .external_lex_state = 24},
  [3533] = {.lex_state = 157, .external_lex_state = 23},
  [3534] = {.lex_state = 157, .external_lex_state = 23},
  [3535] = {.lex_state = 157, .external_lex_state = 23},
  [3536] = {.lex_state = 159, .external_lex_state = 27},
  [3537] = {.lex_state = 159, .external_lex_state = 27},
  [3538] = {.lex_state = 157, .external_lex_state = 23},
  [3539] = {.lex_state = 159, .external_lex_state = 27},
  [3540] = {.lex_state = 159, .external_lex_state = 27},
  [3541] = {.lex_state = 157, .external_lex_state = 24},
  [3542] = {.lex_state = 157, .external_lex_state = 23},
  [3543] = {.lex_state = 45, .external_lex_state = 28},
  [3544] = {.lex_state = 157, .external_lex_state = 23},
  [3545] = {.lex_state = 159, .external_lex_state = 27},
  [3546] = {.lex_state = 157, .external_lex_state = 23},
  [3547] = {.lex_state = 157, .external_lex_state = 23},
  [3548] = {.lex_state = 159, .external_lex_state = 30},
  [3549] = {.lex_state = 157, .external_lex_state = 23},
  [3550] = {.lex_state = 157, .external_lex_state = 24},
  [3551] = {.lex_state = 157, .external_lex_state = 23},
  [3552] = {.lex_state = 159, .external_lex_state = 27},
  [3553] = {.lex_state = 159, .external_lex_state = 27},
  [3554] = {.lex_state = 157, .external_lex_state = 23},
  [3555] = {.lex_state = 157, .external_lex_state = 23},
  [3556] = {.lex_state = 159, .external_lex_state = 27},
  [3557] = {.lex_state = 159, .external_lex_state = 27},
  [3558] = {.lex_state = 159, .external_lex_state = 27},
  [3559] = {.lex_state = 157, .external_lex_state = 23},
  [3560] = {.lex_state = 159, .external_lex_state = 27},
  [3561] = {.lex_state = 157, .external_lex_state = 23},
  [3562] = {.lex_state = 157, .external_lex_state = 23},
  [3563] = {.lex_state = 157, .external_lex_state = 23},
  [3564] = {.lex_state = 157, .external_lex_state = 23},
  [3565] = {.lex_state = 157, .external_lex_state = 28},
  [3566] = {.lex_state = 159, .external_lex_state = 27},
  [3567] = {.lex_state = 159, .external_lex_state = 30},
  [3568] = {.lex_state = 159, .external_lex_state = 27},
  [3569] = {.lex_state = 157, .external_lex_state = 23},
  [3570] = {.lex_state = 159, .external_lex_state = 27},
  [3571] = {.lex_state = 157, .external_lex_state = 24},
  [3572] = {.lex_state = 157, .external_lex_state = 23},
  [3573] = {.lex_state = 157, .external_lex_state = 23},
  [3574] = {.lex_state = 159, .external_lex_state = 30},
  [3575] = {.lex_state = 157, .external_lex_state = 23},
  [3576] = {.lex_state = 157, .external_lex_state = 23},
  [3577] = {.lex_state = 159, .external_lex_state = 27},
  [3578] = {.lex_state = 157, .external_lex_state = 23},
  [3579] = {.lex_state = 159, .external_lex_state = 30},
  [3580] = {.lex_state = 159, .external_lex_state = 27},
  [3581] = {.lex_state = 157, .external_lex_state = 23},
  [3582] = {.lex_state = 159, .external_lex_state = 30},
  [3583] = {.lex_state = 157, .external_lex_state = 23},
  [3584] = {.lex_state = 157, .external_lex_state = 24},
  [3585] = {.lex_state = 157, .external_lex_state = 23},
  [3586] = {.lex_state = 157, .external_lex_state = 23},
  [3587] = {.lex_state = 159, .external_lex_state = 30},
  [3588] = {.lex_state = 159, .external_lex_state = 30},
  [3589] = {.lex_state = 157, .external_lex_state = 23},
  [3590] = {.lex_state = 159, .external_lex_state = 30},
  [3591] = {.lex_state = 157, .external_lex_state = 23},
  [3592] = {.lex_state = 157, .external_lex_state = 24},
  [3593] = {.lex_state = 157, .external_lex_state = 23},
  [3594] = {.lex_state = 159, .external_lex_state = 30},
  [3595] = {.lex_state = 157, .external_lex_state = 24},
  [3596] = {.lex_state = 159, .external_lex_state = 30},
  [3597] = {.lex_state = 157, .external_lex_state = 23},
  [3598] = {.lex_state = 157, .external_lex_state = 23},
  [3599] = {.lex_state = 157, .external_lex_state = 23},
  [3600] = {.lex_state = 159, .external_lex_state = 30},
  [3601] = {.lex_state = 159, .external_lex_state = 30},
  [3602] = {.lex_state = 157, .external_lex_state = 23},
  [3603] = {.lex_state = 159, .external_lex_state = 30},
  [3604] = {.lex_state = 169, .external_lex_state = 31},
  [3605] = {.lex_state = 159, .external_lex_state = 30},
  [3606] = {.lex_state = 45, .external_lex_state = 28},
  [3607] = {.lex_state = 159, .external_lex_state = 30},
  [3608] = {.lex_state = 159, .external_lex_state = 30},
  [3609] = {.lex_state = 169, .external_lex_state = 31},
  [3610] = {.lex_state = 159, .external_lex_state = 30},
  [3611] = {.lex_state = 159, .external_lex_state = 27},
  [3612] = {.lex_state = 159, .external_lex_state = 27},
  [3613] = {.lex_state = 159, .external_lex_state = 27},
  [3614] = {.lex_state = 159, .external_lex_state = 27},
  [3615] = {.lex_state = 159, .external_lex_state = 27},
  [3616] = {.lex_state = 159, .external_lex_state = 27},
  [3617] = {.lex_state = 159, .external_lex_state = 27},
  [3618] = {.lex_state = 159, .external_lex_state = 27},
  [3619] = {.lex_state = 159, .external_lex_state = 30},
  [3620] = {.lex_state = 159, .external_lex_state = 27},
  [3621] = {.lex_state = 157, .external_lex_state = 23},
  [3622] = {.lex_state = 159, .external_lex_state = 27},
  [3623] = {.lex_state = 159, .external_lex_state = 27},
  [3624] = {.lex_state = 159, .external_lex_state = 30},
  [3625] = {.lex_state = 159, .external_lex_state = 27},
  [3626] = {.lex_state = 159, .external_lex_state = 27},
  [3627] = {.lex_state = 159, .external_lex_state = 30},
  [3628] = {.lex_state = 159, .external_lex_state = 30},
  [3629] = {.lex_state = 159, .external_lex_state = 30},
  [3630] = {.lex_state = 159, .external_lex_state = 27},
  [3631] = {.lex_state = 159, .external_lex_state = 27},
  [3632] = {.lex_state = 159, .external_lex_state = 30},
  [3633] = {.lex_state = 159, .external_lex_state = 27},
  [3634] = {.lex_state = 159, .external_lex_state = 27},
  [3635] = {.lex_state = 159, .external_lex_state = 27},
  [3636] = {.lex_state = 159, .external_lex_state = 30},
  [3637] = {.lex_state = 157, .external_lex_state = 23},
  [3638] = {.lex_state = 157, .external_lex_state = 23},
  [3639] = {.lex_state = 45, .external_lex_state = 24},
  [3640] = {.lex_state = 159, .external_lex_state = 30},
  [3641] = {.lex_state = 159, .external_lex_state = 30},
  [3642] = {.lex_state = 159, .external_lex_state = 30},
  [3643] = {.lex_state = 159, .external_lex_state = 30},
  [3644] = {.lex_state = 159, .external_lex_state = 30},
  [3645] = {.lex_state = 157, .external_lex_state = 24},
  [3646] = {.lex_state = 159, .external_lex_state = 27},
  [3647] = {.lex_state = 159, .external_lex_state = 30},
  [3648] = {.lex_state = 45, .external_lex_state = 24},
  [3649] = {.lex_state = 157, .external_lex_state = 24},
  [3650] = {.lex_state = 157, .external_lex_state = 24},
  [3651] = {.lex_state = 157, .external_lex_state = 24},
  [3652] = {.lex_state = 157, .external_lex_state = 24},
  [3653] = {.lex_state = 157, .external_lex_state = 24},
  [3654] = {.lex_state = 157, .external_lex_state = 24},
  [3655] = {.lex_state = 157, .external_lex_state = 24},
  [3656] = {.lex_state = 157, .external_lex_state = 23},
  [3657] = {.lex_state = 157, .external_lex_state = 23},
  [3658] = {.lex_state = 157, .external_lex_state = 23},
  [3659] = {.lex_state = 157, .external_lex_state = 23},
  [3660] = {.lex_state = 157, .external_lex_state = 23},
  [3661] = {.lex_state = 157, .external_lex_state = 23},
  [3662] = {.lex_state = 159, .external_lex_state = 30},
  [3663] = {.lex_state = 157, .external_lex_state = 24},
  [3664] = {.lex_state = 157, .external_lex_state = 24},
  [3665] = {.lex_state = 159, .external_lex_state = 30},
  [3666] = {.lex_state = 159, .external_lex_state = 27},
  [3667] = {.lex_state = 159, .external_lex_state = 30},
  [3668] = {.lex_state = 157, .external_lex_state = 24},
  [3669] = {.lex_state = 157, .external_lex_state = 24},
  [3670] = {.lex_state = 157, .external_lex_state = 24},
  [3671] = {.lex_state = 157, .external_lex_state = 24},
  [3672] = {.lex_state = 159, .external_lex_state = 30},
  [3673] = {.lex_state = 159, .external_lex_state = 27},
  [3674] = {.lex_state = 159, .external_lex_state = 30},
  [3675] = {.lex_state = 157, .external_lex_state = 24},
  [3676] = {.lex_state = 159, .external_lex_state = 30},
  [3677] = {.lex_state = 157, .external_lex_state = 24},
  [3678] = {.lex_state = 157, .external_lex_state = 24},
  [3679] = {.lex_state = 159, .external_lex_state = 27},
  [3680] = {.lex_state = 157, .external_lex_state = 24},
  [3681] = {.lex_state = 159, .external_lex_state = 27},
  [3682] = {.lex_state = 45, .external_lex_state = 24},
  [3683] = {.lex_state = 157, .external_lex_state = 24},
  [3684] = {.lex_state = 159, .external_lex_state = 30},
  [3685] = {.lex_state = 159, .external_lex_state = 30},
  [3686] = {.lex_state = 159, .external_lex_state = 30},
  [3687] = {.lex_state = 157, .external_lex_state = 23},
  [3688] = {.lex_state = 159, .external_lex_state = 30},
  [3689] = {.lex_state = 159, .external_lex_state = 30},
  [3690] = {.lex_state = 159, .external_lex_state = 30},
  [3691] = {.lex_state = 159, .external_lex_state = 30},
  [3692] = {.lex_state = 159, .external_lex_state = 30},
  [3693] = {.lex_state = 159, .external_lex_state = 30},
  [3694] = {.lex_state = 159, .external_lex_state = 30},
  [3695] = {.lex_state = 45, .external_lex_state = 24},
  [3696] = {.lex_state = 159, .external_lex_state = 30},
  [3697] = {.lex_state = 157, .external_lex_state = 24},
  [3698] = {.lex_state = 157, .external_lex_state = 24},
  [3699] = {.lex_state = 159, .external_lex_state = 30},
  [3700] = {.lex_state = 159, .external_lex_state = 30},
  [3701] = {.lex_state = 159, .external_lex_state = 30},
  [3702] = {.lex_state = 159, .external_lex_state = 30},
  [3703] = {.lex_state = 159, .external_lex_state = 30},
  [3704] = {.lex_state = 159, .external_lex_state = 27},
  [3705] = {.lex_state = 159, .external_lex_state = 27},
  [3706] = {.lex_state = 45, .external_lex_state = 24},
  [3707] = {.lex_state = 159, .external_lex_state = 30},
  [3708] = {.lex_state = 159, .external_lex_state = 30},
  [3709] = {.lex_state = 159, .external_lex_state = 30},
  [3710] = {.lex_state = 159, .external_lex_state = 30},
  [3711] = {.lex_state = 159, .external_lex_state = 30},
  [3712] = {.lex_state = 159, .external_lex_state = 30},
  [3713] = {.lex_state = 157, .external_lex_state = 23},
  [3714] = {.lex_state = 157, .external_lex_state = 23},
  [3715] = {.lex_state = 157, .external_lex_state = 23},
  [3716] = {.lex_state = 157, .external_lex_state = 23},
  [3717] = {.lex_state = 157, .external_lex_state = 23},
  [3718] = {.lex_state = 157, .external_lex_state = 23},
  [3719] = {.lex_state = 157, .external_lex_state = 23},
  [3720] = {.lex_state = 157, .external_lex_state = 23},
  [3721] = {.lex_state = 157, .external_lex_state = 23},
  [3722] = {.lex_state = 157, .external_lex_state = 23},
  [3723] = {.lex_state = 157, .external_lex_state = 23},
  [3724] = {.lex_state = 157, .external_lex_state = 23},
  [3725] = {.lex_state = 157, .external_lex_state = 23},
  [3726] = {.lex_state = 157, .external_lex_state = 23},
  [3727] = {.lex_state = 159, .external_lex_state = 30},
  [3728] = {.lex_state = 159, .external_lex_state = 27},
  [3729] = {.lex_state = 157, .external_lex_state = 24},
  [3730] = {.lex_state = 159, .external_lex_state = 27},
  [3731] = {.lex_state = 157, .external_lex_state = 24},
  [3732] = {.lex_state = 159, .external_lex_state = 27},
  [3733] = {.lex_state = 159, .external_lex_state = 27},
  [3734] = {.lex_state = 159, .external_lex_state = 30},
  [3735] = {.lex_state = 157, .external_lex_state = 24},
  [3736] = {.lex_state = 157, .external_lex_state = 24},
  [3737] = {.lex_state = 159, .external_lex_state = 27},
  [3738] = {.lex_state = 157, .external_lex_state = 24},
  [3739] = {.lex_state = 159, .external_lex_state = 30},
  [3740] = {.lex_state = 157, .external_lex_state = 24},
  [3741] = {.lex_state = 159, .external_lex_state = 30},
  [3742] = {.lex_state = 159, .external_lex_state = 30},
  [3743] = {.lex_state = 159, .external_lex_state = 30},
  [3744] = {.lex_state = 45, .external_lex_state = 28},
  [3745] = {.lex_state = 159, .external_lex_state = 30},
  [3746] = {.lex_state = 159, .external_lex_state = 30},
  [3747] = {.lex_state = 159, .external_lex_state = 30},
  [3748] = {.lex_state = 159, .external_lex_state = 27},
  [3749] = {.lex_state = 45, .external_lex_state = 24},
  [3750] = {.lex_state = 159, .external_lex_state = 30},
  [3751] = {.lex_state = 157, .external_lex_state = 24},
  [3752] = {.lex_state = 159, .external_lex_state = 30},
  [3753] = {.lex_state = 159, .external_lex_state = 27},
  [3754] = {.lex_state = 157, .external_lex_state = 24},
  [3755] = {.lex_state = 159, .external_lex_state = 30},
  [3756] = {.lex_state = 159, .external_lex_state = 30},
  [3757] = {.lex_state = 45, .external_lex_state = 24},
  [3758] = {.lex_state = 157, .external_lex_state = 23},
  [3759] = {.lex_state = 159, .external_lex_state = 30},
  [3760] = {.lex_state = 45, .external_lex_state = 24},
  [3761] = {.lex_state = 159, .external_lex_state = 30},
  [3762] = {.lex_state = 157, .external_lex_state = 24},
  [3763] = {.lex_state = 159, .external_lex_state = 27},
  [3764] = {.lex_state = 159, .external_lex_state = 30},
  [3765] = {.lex_state = 45, .external_lex_state = 24},
  [3766] = {.lex_state = 157, .external_lex_state = 24},
  [3767] = {.lex_state = 157, .external_lex_state = 24},
  [3768] = {.lex_state = 157, .external_lex_state = 23},
  [3769] = {.lex_state = 45, .external_lex_state = 24},
  [3770] = {.lex_state = 157, .external_lex_state = 23},
  [3771] = {.lex_state = 157, .external_lex_state = 23},
  [3772] = {.lex_state = 157, .external_lex_state = 23},
  [3773] = {.lex_state = 157, .external_lex_state = 23},
  [3774] = {.lex_state = 157, .external_lex_state = 23},
  [3775] = {.lex_state = 45, .external_lex_state = 24},
  [3776] = {.lex_state = 157, .external_lex_state = 23},
  [3777] = {.lex_state = 157, .external_lex_state = 23},
  [3778] = {.lex_state = 157, .external_lex_state = 23},
  [3779] = {.lex_state = 157, .external_lex_state = 23},
  [3780] = {.lex_state = 157, .external_lex_state = 23},
  [3781] = {.lex_state = 159, .external_lex_state = 30},
  [3782] = {.lex_state = 157, .external_lex_state = 23},
  [3783] = {.lex_state = 157, .external_lex_state = 23},
  [3784] = {.lex_state = 159, .external_lex_state = 27},
  [3785] = {.lex_state = 157, .external_lex_state = 23},
  [3786] = {.lex_state = 159, .external_lex_state = 30},
  [3787] = {.lex_state = 45, .external_lex_state = 24},
  [3788] = {.lex_state = 45, .external_lex_state = 24},
  [3789] = {.lex_state = 159, .external_lex_state = 27},
  [3790] = {.lex_state = 45, .external_lex_state = 24},
  [3791] = {.lex_state = 45, .external_lex_state = 24},
  [3792] = {.lex_state = 45, .external_lex_state = 24},
  [3793] = {.lex_state = 45, .external_lex_state = 24},
  [3794] = {.lex_state = 45, .external_lex_state = 24},
  [3795] = {.lex_state = 159, .external_lex_state = 30},
  [3796] = {.lex_state = 45, .external_lex_state = 24},
  [3797] = {.lex_state = 159, .external_lex_state = 30},
  [3798] = {.lex_state = 159, .external_lex_state = 30},
  [3799] = {.lex_state = 157, .external_lex_state = 24},
  [3800] = {.lex_state = 159, .external_lex_state = 30},
  [3801] = {.lex_state = 159, .external_lex_state = 30},
  [3802] = {.lex_state = 159, .external_lex_state = 30},
  [3803] = {.lex_state = 159, .external_lex_state = 30},
  [3804] = {.lex_state = 45, .external_lex_state = 24},
  [3805] = {.lex_state = 157, .external_lex_state = 23},
  [3806] = {.lex_state = 157, .external_lex_state = 24},
  [3807] = {.lex_state = 159, .external_lex_state = 30},
  [3808] = {.lex_state = 45, .external_lex_state = 24},
  [3809] = {.lex_state = 157, .external_lex_state = 24},
  [3810] = {.lex_state = 159, .external_lex_state = 30},
  [3811] = {.lex_state = 159, .external_lex_state = 30},
  [3812] = {.lex_state = 159, .external_lex_state = 30},
  [3813] = {.lex_state = 159, .external_lex_state = 30},
  [3814] = {.lex_state = 45, .external_lex_state = 24},
  [3815] = {.lex_state = 45, .external_lex_state = 24},
  [3816] = {.lex_state = 157, .external_lex_state = 23},
  [3817] = {.lex_state = 45, .external_lex_state = 24},
  [3818] = {.lex_state = 159, .external_lex_state = 30},
  [3819] = {.lex_state = 157, .external_lex_state = 24},
  [3820] = {.lex_state = 157, .external_lex_state = 24},
  [3821] = {.lex_state = 159, .external_lex_state = 30},
  [3822] = {.lex_state = 157, .external_lex_state = 23},
  [3823] = {.lex_state = 159, .external_lex_state = 30},
  [3824] = {.lex_state = 157, .external_lex_state = 24},
  [3825] = {.lex_state = 157, .external_lex_state = 24},
  [3826] = {.lex_state = 157, .external_lex_state = 24},
  [3827] = {.lex_state = 45, .external_lex_state = 24},
  [3828] = {.lex_state = 45, .external_lex_state = 24},
  [3829] = {.lex_state = 159, .external_lex_state = 30},
  [3830] = {.lex_state = 159, .external_lex_state = 30},
  [3831] = {.lex_state = 159, .external_lex_state = 30},
  [3832] = {.lex_state = 159, .external_lex_state = 30},
  [3833] = {.lex_state = 45, .external_lex_state = 24},
  [3834] = {.lex_state = 45, .external_lex_state = 24},
  [3835] = {.lex_state = 45, .external_lex_state = 24},
  [3836] = {.lex_state = 159, .external_lex_state = 27},
  [3837] = {.lex_state = 157, .external_lex_state = 24},
  [3838] = {.lex_state = 159, .external_lex_state = 30},
  [3839] = {.lex_state = 45, .external_lex_state = 24},
  [3840] = {.lex_state = 157, .external_lex_state = 24},
  [3841] = {.lex_state = 159, .external_lex_state = 30},
  [3842] = {.lex_state = 159, .external_lex_state = 30},
  [3843] = {.lex_state = 159, .external_lex_state = 30},
  [3844] = {.lex_state = 45, .external_lex_state = 24},
  [3845] = {.lex_state = 159, .external_lex_state = 30},
  [3846] = {.lex_state = 159, .external_lex_state = 27},
  [3847] = {.lex_state = 159, .external_lex_state = 30},
  [3848] = {.lex_state = 159, .external_lex_state = 30},
  [3849] = {.lex_state = 159, .external_lex_state = 30},
  [3850] = {.lex_state = 159, .external_lex_state = 30},
  [3851] = {.lex_state = 157, .external_lex_state = 24},
  [3852] = {.lex_state = 159, .external_lex_state = 30},
  [3853] = {.lex_state = 159, .external_lex_state = 30},
  [3854] = {.lex_state = 45, .external_lex_state = 24},
  [3855] = {.lex_state = 159, .external_lex_state = 30},
  [3856] = {.lex_state = 159, .external_lex_state = 30},
  [3857] = {.lex_state = 159, .external_lex_state = 30},
  [3858] = {.lex_state = 159, .external_lex_state = 30},
  [3859] = {.lex_state = 159, .external_lex_state = 30},
  [3860] = {.lex_state = 159, .external_lex_state = 30},
  [3861] = {.lex_state = 159, .external_lex_state = 30},
  [3862] = {.lex_state = 157, .external_lex_state = 24},
  [3863] = {.lex_state = 45, .external_lex_state = 24},
  [3864] = {.lex_state = 159, .external_lex_state = 30},
  [3865] = {.lex_state = 45, .external_lex_state = 24},
  [3866] = {.lex_state = 157, .external_lex_state = 24},
  [3867] = {.lex_state = 159, .external_lex_state = 30},
  [3868] = {.lex_state = 157, .external_lex_state = 24},
  [3869] = {.lex_state = 159, .external_lex_state = 30},
  [3870] = {.lex_state = 157, .external_lex_state = 23},
  [3871] = {.lex_state = 157, .external_lex_state = 24},
  [3872] = {.lex_state = 45, .external_lex_state = 24},
  [3873] = {.lex_state = 157, .external_lex_state = 23},
  [3874] = {.lex_state = 159, .external_lex_state = 30},
  [3875] = {.lex_state = 45, .external_lex_state = 24},
  [3876] = {.lex_state = 45, .external_lex_state = 24},
  [3877] = {.lex_state = 157, .external_lex_state = 24},
  [3878] = {.lex_state = 159, .external_lex_state = 30},
  [3879] = {.lex_state = 45, .external_lex_state = 24},
  [3880] = {.lex_state = 157, .external_lex_state = 24},
  [3881] = {.lex_state = 159, .external_lex_state = 30},
  [3882] = {.lex_state = 157, .external_lex_state = 23},
  [3883] = {.lex_state = 159, .external_lex_state = 30},
  [3884] = {.lex_state = 45, .external_lex_state = 24},
  [3885] = {.lex_state = 159, .external_lex_state = 30},
  [3886] = {.lex_state = 159, .external_lex_state = 27},
  [3887] = {.lex_state = 157, .external_lex_state = 24},
  [3888] = {.lex_state = 157, .external_lex_state = 23},
  [3889] = {.lex_state = 159, .external_lex_state = 30},
  [3890] = {.lex_state = 45, .external_lex_state = 24},
  [3891] = {.lex_state = 45, .external_lex_state = 24},
  [3892] = {.lex_state = 159, .external_lex_state = 30},
  [3893] = {.lex_state = 159, .external_lex_state = 27},
  [3894] = {.lex_state = 159, .external_lex_state = 27},
  [3895] = {.lex_state = 159, .external_lex_state = 27},
  [3896] = {.lex_state = 159, .external_lex_state = 27},
  [3897] = {.lex_state = 159, .external_lex_state = 27},
  [3898] = {.lex_state = 45, .external_lex_state = 24},
  [3899] = {.lex_state = 159, .external_lex_state = 27},
  [3900] = {.lex_state = 159, .external_lex_state = 30},
  [3901] = {.lex_state = 159, .external_lex_state = 30},
  [3902] = {.lex_state = 159, .external_lex_state = 30},
  [3903] = {.lex_state = 159, .external_lex_state = 30},
  [3904] = {.lex_state = 159, .external_lex_state = 30},
  [3905] = {.lex_state = 159, .external_lex_state = 27},
  [3906] = {.lex_state = 159, .external_lex_state = 27},
  [3907] = {.lex_state = 159, .external_lex_state = 27},
  [3908] = {.lex_state = 159, .external_lex_state = 27},
  [3909] = {.lex_state = 159, .external_lex_state = 27},
  [3910] = {.lex_state = 159, .external_lex_state = 30},
  [3911] = {.lex_state = 159, .external_lex_state = 30},
  [3912] = {.lex_state = 45, .external_lex_state = 24},
  [3913] = {.lex_state = 45, .external_lex_state = 24},
  [3914] = {.lex_state = 45, .external_lex_state = 24},
  [3915] = {.lex_state = 45, .external_lex_state = 24},
  [3916] = {.lex_state = 45, .external_lex_state = 24},
  [3917] = {.lex_state = 45, .external_lex_state = 24},
  [3918] = {.lex_state = 45, .external_lex_state = 24},
  [3919] = {.lex_state = 45, .external_lex_state = 24},
  [3920] = {.lex_state = 45, .external_lex_state = 24},
  [3921] = {.lex_state = 45, .external_lex_state = 24},
  [3922] = {.lex_state = 159, .external_lex_state = 27},
  [3923] = {.lex_state = 45, .external_lex_state = 24},
  [3924] = {.lex_state = 159, .external_lex_state = 27},
  [3925] = {.lex_state = 159, .external_lex_state = 27},
  [3926] = {.lex_state = 159, .external_lex_state = 27},
  [3927] = {.lex_state = 159, .external_lex_state = 27},
  [3928] = {.lex_state = 45, .external_lex_state = 24},
  [3929] = {.lex_state = 159, .external_lex_state = 27},
  [3930] = {.lex_state = 159, .external_lex_state = 27},
  [3931] = {.lex_state = 159, .external_lex_state = 27},
  [3932] = {.lex_state = 45, .external_lex_state = 24},
  [3933] = {.lex_state = 159, .external_lex_state = 30},
  [3934] = {.lex_state = 159, .external_lex_state = 30},
  [3935] = {.lex_state = 159, .external_lex_state = 30},
  [3936] = {.lex_state = 45, .external_lex_state = 24},
  [3937] = {.lex_state = 159, .external_lex_state = 27},
  [3938] = {.lex_state = 45, .external_lex_state = 24},
  [3939] = {.lex_state = 45, .external_lex_state = 24},
  [3940] = {.lex_state = 45, .external_lex_state = 24},
  [3941] = {.lex_state = 45, .external_lex_state = 24},
  [3942] = {.lex_state = 45, .external_lex_state = 24},
  [3943] = {.lex_state = 45, .external_lex_state = 24},
  [3944] = {.lex_state = 45, .external_lex_state = 24},
  [3945] = {.lex_state = 45, .external_lex_state = 24},
  [3946] = {.lex_state = 45, .external_lex_state = 24},
  [3947] = {.lex_state = 45, .external_lex_state = 24},
  [3948] = {.lex_state = 45, .external_lex_state = 24},
  [3949] = {.lex_state = 45, .external_lex_state = 24},
  [3950] = {.lex_state = 45, .external_lex_state = 24},
  [3951] = {.lex_state = 45, .external_lex_state = 24},
  [3952] = {.lex_state = 45, .external_lex_state = 24},
  [3953] = {.lex_state = 45, .external_lex_state = 24},
  [3954] = {.lex_state = 159, .external_lex_state = 30},
  [3955] = {.lex_state = 159, .external_lex_state = 30},
  [3956] = {.lex_state = 159, .external_lex_state = 27},
  [3957] = {.lex_state = 159, .external_lex_state = 30},
  [3958] = {.lex_state = 159, .external_lex_state = 30},
  [3959] = {.lex_state = 159, .external_lex_state = 30},
  [3960] = {.lex_state = 45, .external_lex_state = 24},
  [3961] = {.lex_state = 45, .external_lex_state = 24},
  [3962] = {.lex_state = 159, .external_lex_state = 27},
  [3963] = {.lex_state = 159, .external_lex_state = 30},
  [3964] = {.lex_state = 159, .external_lex_state = 27},
  [3965] = {.lex_state = 45, .external_lex_state = 24},
  [3966] = {.lex_state = 45, .external_lex_state = 24},
  [3967] = {.lex_state = 45, .external_lex_state = 24},
  [3968] = {.lex_state = 45, .external_lex_state = 24},
  [3969] = {.lex_state = 45, .external_lex_state = 24},
  [3970] = {.lex_state = 159, .external_lex_state = 27},
  [3971] = {.lex_state = 45, .external_lex_state = 24},
  [3972] = {.lex_state = 45, .external_lex_state = 24},
  [3973] = {.lex_state = 45, .external_lex_state = 24},
  [3974] = {.lex_state = 45, .external_lex_state = 24},
  [3975] = {.lex_state = 45, .external_lex_state = 24},
  [3976] = {.lex_state = 45, .external_lex_state = 24},
  [3977] = {.lex_state = 159, .external_lex_state = 27},
  [3978] = {.lex_state = 159, .external_lex_state = 27},
  [3979] = {.lex_state = 159, .external_lex_state = 27},
  [3980] = {.lex_state = 159, .external_lex_state = 27},
  [3981] = {.lex_state = 159, .external_lex_state = 27},
  [3982] = {.lex_state = 45, .external_lex_state = 24},
  [3983] = {.lex_state = 159, .external_lex_state = 27},
  [3984] = {.lex_state = 45, .external_lex_state = 24},
  [3985] = {.lex_state = 159, .external_lex_state = 27},
  [3986] = {.lex_state = 159, .external_lex_state = 27},
  [3987] = {.lex_state = 159, .external_lex_state = 27},
  [3988] = {.lex_state = 159, .external_lex_state = 27},
  [3989] = {.lex_state = 159, .external_lex_state = 27},
  [3990] = {.lex_state = 159, .external_lex_state = 27},
  [3991] = {.lex_state = 45, .external_lex_state = 24},
  [3992] = {.lex_state = 159, .external_lex_state = 27},
  [3993] = {.lex_state = 159, .external_lex_state = 27},
  [3994] = {.lex_state = 159, .external_lex_state = 27},
  [3995] = {.lex_state = 159, .external_lex_state = 30},
  [3996] = {.lex_state = 159, .external_lex_state = 27},
  [3997] = {.lex_state = 159, .external_lex_state = 27},
  [3998] = {.lex_state = 159, .external_lex_state = 27},
  [3999] = {.lex_state = 159, .external_lex_state = 27},
  [4000] = {.lex_state = 159, .external_lex_state = 27},
  [4001] = {.lex_state = 159, .external_lex_state = 27},
  [4002] = {.lex_state = 159, .external_lex_state = 27},
  [4003] = {.lex_state = 159, .external_lex_state = 27},
  [4004] = {.lex_state = 159, .external_lex_state = 27},
  [4005] = {.lex_state = 159, .external_lex_state = 27},
  [4006] = {.lex_state = 159, .external_lex_state = 30},
  [4007] = {.lex_state = 159, .external_lex_state = 27},
  [4008] = {.lex_state = 159, .external_lex_state = 27},
  [4009] = {.lex_state = 159, .external_lex_state = 27},
  [4010] = {.lex_state = 45, .external_lex_state = 24},
  [4011] = {.lex_state = 45, .external_lex_state = 24},
  [4012] = {.lex_state = 159, .external_lex_state = 30},
  [4013] = {.lex_state = 159, .external_lex_state = 30},
  [4014] = {.lex_state = 159, .external_lex_state = 30},
  [4015] = {.lex_state = 159, .external_lex_state = 30},
  [4016] = {.lex_state = 45, .external_lex_state = 24},
  [4017] = {.lex_state = 45, .external_lex_state = 24},
  [4018] = {.lex_state = 159, .external_lex_state = 27},
  [4019] = {.lex_state = 159, .external_lex_state = 27},
  [4020] = {.lex_state = 45, .external_lex_state = 24},
  [4021] = {.lex_state = 45, .external_lex_state = 24},
  [4022] = {.lex_state = 159, .external_lex_state = 30},
  [4023] = {.lex_state = 159, .external_lex_state = 30},
  [4024] = {.lex_state = 159, .external_lex_state = 30},
  [4025] = {.lex_state = 159, .external_lex_state = 27},
  [4026] = {.lex_state = 45, .external_lex_state = 24},
  [4027] = {.lex_state = 159, .external_lex_state = 30},
  [4028] = {.lex_state = 159, .external_lex_state = 27},
  [4029] = {.lex_state = 159, .external_lex_state = 27},
  [4030] = {.lex_state = 159, .external_lex_state = 27},
  [4031] = {.lex_state = 159, .external_lex_state = 27},
  [4032] = {.lex_state = 157, .external_lex_state = 24},
  [4033] = {.lex_state = 159, .external_lex_state = 27},
  [4034] = {.lex_state = 159, .external_lex_state = 27},
  [4035] = {.lex_state = 159, .external_lex_state = 27},
  [4036] = {.lex_state = 159, .external_lex_state = 30},
  [4037] = {.lex_state = 45, .external_lex_state = 24},
  [4038] = {.lex_state = 159, .external_lex_state = 27},
  [4039] = {.lex_state = 159, .external_lex_state = 27},
  [4040] = {.lex_state = 45, .external_lex_state = 24},
  [4041] = {.lex_state = 159, .external_lex_state = 27},
  [4042] = {.lex_state = 45, .external_lex_state = 24},
  [4043] = {.lex_state = 45, .external_lex_state = 24},
  [4044] = {.lex_state = 45, .external_lex_state = 24},
  [4045] = {.lex_state = 45, .external_lex_state = 24},
  [4046] = {.lex_state = 45, .external_lex_state = 24},
  [4047] = {.lex_state = 45, .external_lex_state = 24},
  [4048] = {.lex_state = 45, .external_lex_state = 24},
  [4049] = {.lex_state = 45, .external_lex_state = 24},
  [4050] = {.lex_state = 45, .external_lex_state = 24},
  [4051] = {.lex_state = 45, .external_lex_state = 24},
  [4052] = {.lex_state = 45, .external_lex_state = 24},
  [4053] = {.lex_state = 159, .external_lex_state = 27},
  [4054] = {.lex_state = 159, .external_lex_state = 30},
  [4055] = {.lex_state = 159, .external_lex_state = 30},
  [4056] = {.lex_state = 159, .external_lex_state = 27},
  [4057] = {.lex_state = 159, .external_lex_state = 30},
  [4058] = {.lex_state = 159, .external_lex_state = 30},
  [4059] = {.lex_state = 159, .external_lex_state = 27},
  [4060] = {.lex_state = 159, .external_lex_state = 27},
  [4061] = {.lex_state = 159, .external_lex_state = 30},
  [4062] = {.lex_state = 159, .external_lex_state = 27},
  [4063] = {.lex_state = 159, .external_lex_state = 30},
  [4064] = {.lex_state = 159, .external_lex_state = 30},
  [4065] = {.lex_state = 159, .external_lex_state = 30},
  [4066] = {.lex_state = 159, .external_lex_state = 30},
  [4067] = {.lex_state = 159, .external_lex_state = 30},
  [4068] = {.lex_state = 159, .external_lex_state = 30},
  [4069] = {.lex_state = 159, .external_lex_state = 30},
  [4070] = {.lex_state = 159, .external_lex_state = 30},
  [4071] = {.lex_state = 159, .external_lex_state = 30},
  [4072] = {.lex_state = 159, .external_lex_state = 30},
  [4073] = {.lex_state = 159, .external_lex_state = 30},
  [4074] = {.lex_state = 51, .external_lex_state = 32},
  [4075] = {.lex_state = 159, .external_lex_state = 30},
  [4076] = {.lex_state = 159, .external_lex_state = 27},
  [4077] = {.lex_state = 159, .external_lex_state = 30},
  [4078] = {.lex_state = 159, .external_lex_state = 30},
  [4079] = {.lex_state = 159, .external_lex_state = 30},
  [4080] = {.lex_state = 159, .external_lex_state = 30},
  [4081] = {.lex_state = 159, .external_lex_state = 30},
  [4082] = {.lex_state = 159, .external_lex_state = 30},
  [4083] = {.lex_state = 159, .external_lex_state = 30},
  [4084] = {.lex_state = 159, .external_lex_state = 30},
  [4085] = {.lex_state = 159, .external_lex_state = 30},
  [4086] = {.lex_state = 159, .external_lex_state = 30},
  [4087] = {.lex_state = 159, .external_lex_state = 30},
  [4088] = {.lex_state = 159, .external_lex_state = 27},
  [4089] = {.lex_state = 159, .external_lex_state = 27},
  [4090] = {.lex_state = 159, .external_lex_state = 27},
  [4091] = {.lex_state = 159, .external_lex_state = 27},
  [4092] = {.lex_state = 159, .external_lex_state = 27},
  [4093] = {.lex_state = 159, .external_lex_state = 27},
  [4094] = {.lex_state = 159, .external_lex_state = 27},
  [4095] = {.lex_state = 159, .external_lex_state = 30},
  [4096] = {.lex_state = 159, .external_lex_state = 27},
  [4097] = {.lex_state = 159, .external_lex_state = 27},
  [4098] = {.lex_state = 159, .external_lex_state = 27},
  [4099] = {.lex_state = 159, .external_lex_state = 27},
  [4100] = {.lex_state = 159, .external_lex_state = 27},
  [4101] = {.lex_state = 159, .external_lex_state = 27},
  [4102] = {.lex_state = 159, .external_lex_state = 27},
  [4103] = {.lex_state = 159, .external_lex_state = 27},
  [4104] = {.lex_state = 159, .external_lex_state = 27},
  [4105] = {.lex_state = 159, .external_lex_state = 27},
  [4106] = {.lex_state = 159, .external_lex_state = 30},
  [4107] = {.lex_state = 159, .external_lex_state = 30},
  [4108] = {.lex_state = 159, .external_lex_state = 30},
  [4109] = {.lex_state = 159, .external_lex_state = 30},
  [4110] = {.lex_state = 159, .external_lex_state = 30},
  [4111] = {.lex_state = 159, .external_lex_state = 30},
  [4112] = {.lex_state = 159, .external_lex_state = 30},
  [4113] = {.lex_state = 159, .external_lex_state = 30},
  [4114] = {.lex_state = 157, .external_lex_state = 24},
  [4115] = {.lex_state = 159, .external_lex_state = 27},
  [4116] = {.lex_state = 159, .external_lex_state = 30},
  [4117] = {.lex_state = 159, .external_lex_state = 30},
  [4118] = {.lex_state = 159, .external_lex_state = 30},
  [4119] = {.lex_state = 159, .external_lex_state = 27},
  [4120] = {.lex_state = 159, .external_lex_state = 30},
  [4121] = {.lex_state = 159, .external_lex_state = 27},
  [4122] = {.lex_state = 159, .external_lex_state = 27},
  [4123] = {.lex_state = 159, .external_lex_state = 30},
  [4124] = {.lex_state = 159, .external_lex_state = 27},
  [4125] = {.lex_state = 159, .external_lex_state = 30},
  [4126] = {.lex_state = 159, .external_lex_state = 30},
  [4127] = {.lex_state = 159, .external_lex_state = 27},
  [4128] = {.lex_state = 159, .external_lex_state = 30},
  [4129] = {.lex_state = 159, .external_lex_state = 30},
  [4130] = {.lex_state = 159, .external_lex_state = 30},
  [4131] = {.lex_state = 159, .external_lex_state = 27},
  [4132] = {.lex_state = 159, .external_lex_state = 30},
  [4133] = {.lex_state = 159, .external_lex_state = 27},
  [4134] = {.lex_state = 159, .external_lex_state = 30},
  [4135] = {.lex_state = 49, .external_lex_state = 32},
  [4136] = {.lex_state = 159, .external_lex_state = 30},
  [4137] = {.lex_state = 159, .external_lex_state = 30},
  [4138] = {.lex_state = 159, .external_lex_state = 30},
  [4139] = {.lex_state = 159, .external_lex_state = 30},
  [4140] = {.lex_state = 159, .external_lex_state = 30},
  [4141] = {.lex_state = 159, .external_lex_state = 27},
  [4142] = {.lex_state = 159, .external_lex_state = 30},
  [4143] = {.lex_state = 159, .external_lex_state = 30},
  [4144] = {.lex_state = 159, .external_lex_state = 27},
  [4145] = {.lex_state = 159, .external_lex_state = 30},
  [4146] = {.lex_state = 159, .external_lex_state = 30},
  [4147] = {.lex_state = 159, .external_lex_state = 30},
  [4148] = {.lex_state = 159, .external_lex_state = 30},
  [4149] = {.lex_state = 159, .external_lex_state = 30},
  [4150] = {.lex_state = 159, .external_lex_state = 30},
  [4151] = {.lex_state = 159, .external_lex_state = 30},
  [4152] = {.lex_state = 159, .external_lex_state = 30},
  [4153] = {.lex_state = 159, .external_lex_state = 30},
  [4154] = {.lex_state = 159, .external_lex_state = 30},
  [4155] = {.lex_state = 159, .external_lex_state = 27},
  [4156] = {.lex_state = 159, .external_lex_state = 27},
  [4157] = {.lex_state = 159, .external_lex_state = 27},
  [4158] = {.lex_state = 159, .external_lex_state = 27},
  [4159] = {.lex_state = 159, .external_lex_state = 27},
  [4160] = {.lex_state = 159, .external_lex_state = 27},
  [4161] = {.lex_state = 159, .external_lex_state = 27},
  [4162] = {.lex_state = 159, .external_lex_state = 27},
  [4163] = {.lex_state = 159, .external_lex_state = 27},
  [4164] = {.lex_state = 159, .external_lex_state = 27},
  [4165] = {.lex_state = 159, .external_lex_state = 27},
  [4166] = {.lex_state = 159, .external_lex_state = 30},
  [4167] = {.lex_state = 159, .external_lex_state = 30},
  [4168] = {.lex_state = 159, .external_lex_state = 30},
  [4169] = {.lex_state = 159, .external_lex_state = 30},
  [4170] = {.lex_state = 159, .external_lex_state = 30},
  [4171] = {.lex_state = 159, .external_lex_state = 30},
  [4172] = {.lex_state = 159, .external_lex_state = 30},
  [4173] = {.lex_state = 159, .external_lex_state = 27},
  [4174] = {.lex_state = 159, .external_lex_state = 27},
  [4175] = {.lex_state = 159, .external_lex_state = 30},
  [4176] = {.lex_state = 159, .external_lex_state = 27},
  [4177] = {.lex_state = 159, .external_lex_state = 27},
  [4178] = {.lex_state = 159, .external_lex_state = 30},
  [4179] = {.lex_state = 159, .external_lex_state = 30},
  [4180] = {.lex_state = 159, .external_lex_state = 27},
  [4181] = {.lex_state = 159, .external_lex_state = 30},
  [4182] = {.lex_state = 159, .external_lex_state = 30},
  [4183] = {.lex_state = 159, .external_lex_state = 30},
  [4184] = {.lex_state = 159, .external_lex_state = 30},
  [4185] = {.lex_state = 159, .external_lex_state = 30},
  [4186] = {.lex_state = 159, .external_lex_state = 30},
  [4187] = {.lex_state = 159, .external_lex_state = 30},
  [4188] = {.lex_state = 159, .external_lex_state = 30},
  [4189] = {.lex_state = 159, .external_lex_state = 27},
  [4190] = {.lex_state = 159, .external_lex_state = 27},
  [4191] = {.lex_state = 159, .external_lex_state = 27},
  [4192] = {.lex_state = 159, .external_lex_state = 30},
  [4193] = {.lex_state = 159, .external_lex_state = 30},
  [4194] = {.lex_state = 159, .external_lex_state = 30},
  [4195] = {.lex_state = 159, .external_lex_state = 27},
  [4196] = {.lex_state = 49, .external_lex_state = 32},
  [4197] = {.lex_state = 159, .external_lex_state = 30},
  [4198] = {.lex_state = 159, .external_lex_state = 30},
  [4199] = {.lex_state = 159, .external_lex_state = 30},
  [4200] = {.lex_state = 159, .external_lex_state = 30},
  [4201] = {.lex_state = 159, .external_lex_state = 30},
  [4202] = {.lex_state = 159, .external_lex_state = 30},
  [4203] = {.lex_state = 159, .external_lex_state = 30},
  [4204] = {.lex_state = 159, .external_lex_state = 30},
  [4205] = {.lex_state = 159, .external_lex_state = 30},
  [4206] = {.lex_state = 159, .external_lex_state = 30},
  [4207] = {.lex_state = 159, .external_lex_state = 27},
  [4208] = {.lex_state = 159, .external_lex_state = 30},
  [4209] = {.lex_state = 159, .external_lex_state = 27},
  [4210] = {.lex_state = 159, .external_lex_state = 30},
  [4211] = {.lex_state = 159, .external_lex_state = 30},
  [4212] = {.lex_state = 159, .external_lex_state = 30},
  [4213] = {.lex_state = 159, .external_lex_state = 30},
  [4214] = {.lex_state = 159, .external_lex_state = 30},
  [4215] = {.lex_state = 159, .external_lex_state = 30},
  [4216] = {.lex_state = 159, .external_lex_state = 30},
  [4217] = {.lex_state = 159, .external_lex_state = 30},
  [4218] = {.lex_state = 159, .external_lex_state = 30},
  [4219] = {.lex_state = 159, .external_lex_state = 30},
  [4220] = {.lex_state = 159, .external_lex_state = 30},
  [4221] = {.lex_state = 159, .external_lex_state = 30},
  [4222] = {.lex_state = 159, .external_lex_state = 30},
  [4223] = {.lex_state = 159, .external_lex_state = 30},
  [4224] = {.lex_state = 159, .external_lex_state = 30},
  [4225] = {.lex_state = 159, .external_lex_state = 30},
  [4226] = {.lex_state = 159, .external_lex_state = 30},
  [4227] = {.lex_state = 160, .external_lex_state = 32},
  [4228] = {.lex_state = 159, .external_lex_state = 30},
  [4229] = {.lex_state = 159, .external_lex_state = 30},
  [4230] = {.lex_state = 159, .external_lex_state = 30},
  [4231] = {.lex_state = 159, .external_lex_state = 30},
  [4232] = {.lex_state = 159, .external_lex_state = 30},
  [4233] = {.lex_state = 159, .external_lex_state = 30},
  [4234] = {.lex_state = 159, .external_lex_state = 30},
  [4235] = {.lex_state = 159, .external_lex_state = 30},
  [4236] = {.lex_state = 159, .external_lex_state = 30},
  [4237] = {.lex_state = 159, .external_lex_state = 30},
  [4238] = {.lex_state = 159, .external_lex_state = 30},
  [4239] = {.lex_state = 159, .external_lex_state = 30},
  [4240] = {.lex_state = 159, .external_lex_state = 30},
  [4241] = {.lex_state = 159, .external_lex_state = 30},
  [4242] = {.lex_state = 159, .external_lex_state = 30},
  [4243] = {.lex_state = 159, .external_lex_state = 30},
  [4244] = {.lex_state = 159, .external_lex_state = 30},
  [4245] = {.lex_state = 159, .external_lex_state = 30},
  [4246] = {.lex_state = 159, .external_lex_state = 30},
  [4247] = {.lex_state = 159, .external_lex_state = 30},
  [4248] = {.lex_state = 159, .external_lex_state = 30},
  [4249] = {.lex_state = 159, .external_lex_state = 30},
  [4250] = {.lex_state = 159, .external_lex_state = 30},
  [4251] = {.lex_state = 159, .external_lex_state = 30},
  [4252] = {.lex_state = 159, .external_lex_state = 30},
  [4253] = {.lex_state = 159, .external_lex_state = 30},
  [4254] = {.lex_state = 159, .external_lex_state = 30},
  [4255] = {.lex_state = 159, .external_lex_state = 30},
  [4256] = {.lex_state = 159, .external_lex_state = 30},
  [4257] = {.lex_state = 159, .external_lex_state = 30},
  [4258] = {.lex_state = 159, .external_lex_state = 30},
  [4259] = {.lex_state = 159, .external_lex_state = 30},
  [4260] = {.lex_state = 159, .external_lex_state = 30},
  [4261] = {.lex_state = 159, .external_lex_state = 30},
  [4262] = {.lex_state = 159, .external_lex_state = 30},
  [4263] = {.lex_state = 159, .external_lex_state = 30},
  [4264] = {.lex_state = 159, .external_lex_state = 30},
  [4265] = {.lex_state = 159, .external_lex_state = 30},
  [4266] = {.lex_state = 159, .external_lex_state = 30},
  [4267] = {.lex_state = 159, .external_lex_state = 30},
  [4268] = {.lex_state = 159, .external_lex_state = 30},
  [4269] = {.lex_state = 159, .external_lex_state = 30},
  [4270] = {.lex_state = 159, .external_lex_state = 30},
  [4271] = {.lex_state = 159, .external_lex_state = 30},
  [4272] = {.lex_state = 159, .external_lex_state = 30},
  [4273] = {.lex_state = 159, .external_lex_state = 30},
  [4274] = {.lex_state = 159, .external_lex_state = 30},
  [4275] = {.lex_state = 159, .external_lex_state = 30},
  [4276] = {.lex_state = 159, .external_lex_state = 30},
  [4277] = {.lex_state = 159, .external_lex_state = 30},
  [4278] = {.lex_state = 165, .external_lex_state = 33},
  [4279] = {.lex_state = 159, .external_lex_state = 30},
  [4280] = {.lex_state = 159, .external_lex_state = 30},
  [4281] = {.lex_state = 159, .external_lex_state = 30},
  [4282] = {.lex_state = 159, .external_lex_state = 30},
  [4283] = {.lex_state = 159, .external_lex_state = 30},
  [4284] = {.lex_state = 159, .external_lex_state = 30},
  [4285] = {.lex_state = 159, .external_lex_state = 30},
  [4286] = {.lex_state = 159, .external_lex_state = 30},
  [4287] = {.lex_state = 159, .external_lex_state = 30},
  [4288] = {.lex_state = 159, .external_lex_state = 30},
  [4289] = {.lex_state = 159, .external_lex_state = 30},
  [4290] = {.lex_state = 159, .external_lex_state = 30},
  [4291] = {.lex_state = 159, .external_lex_state = 30},
  [4292] = {.lex_state = 159, .external_lex_state = 30},
  [4293] = {.lex_state = 159, .external_lex_state = 30},
  [4294] = {.lex_state = 159, .external_lex_state = 30},
  [4295] = {.lex_state = 159, .external_lex_state = 30},
  [4296] = {.lex_state = 159, .external_lex_state = 30},
  [4297] = {.lex_state = 159, .external_lex_state = 30},
  [4298] = {.lex_state = 159, .external_lex_state = 30},
  [4299] = {.lex_state = 159, .external_lex_state = 30},
  [4300] = {.lex_state = 159, .external_lex_state = 30},
  [4301] = {.lex_state = 159, .external_lex_state = 30},
  [4302] = {.lex_state = 159, .external_lex_state = 30},
  [4303] = {.lex_state = 159, .external_lex_state = 30},
  [4304] = {.lex_state = 159, .external_lex_state = 30},
  [4305] = {.lex_state = 159, .external_lex_state = 30},
  [4306] = {.lex_state = 159, .external_lex_state = 30},
  [4307] = {.lex_state = 159, .external_lex_state = 30},
  [4308] = {.lex_state = 159, .external_lex_state = 30},
  [4309] = {.lex_state = 159, .external_lex_state = 30},
  [4310] = {.lex_state = 169, .external_lex_state = 31},
  [4311] = {.lex_state = 159, .external_lex_state = 30},
  [4312] = {.lex_state = 159, .external_lex_state = 30},
  [4313] = {.lex_state = 159, .external_lex_state = 30},
  [4314] = {.lex_state = 159, .external_lex_state = 30},
  [4315] = {.lex_state = 159, .external_lex_state = 30},
  [4316] = {.lex_state = 159, .external_lex_state = 30},
  [4317] = {.lex_state = 159, .external_lex_state = 30},
  [4318] = {.lex_state = 159, .external_lex_state = 30},
  [4319] = {.lex_state = 159, .external_lex_state = 30},
  [4320] = {.lex_state = 159, .external_lex_state = 30},
  [4321] = {.lex_state = 159, .external_lex_state = 30},
  [4322] = {.lex_state = 159, .external_lex_state = 30},
  [4323] = {.lex_state = 159, .external_lex_state = 30},
  [4324] = {.lex_state = 159, .external_lex_state = 30},
  [4325] = {.lex_state = 159, .external_lex_state = 30},
  [4326] = {.lex_state = 159, .external_lex_state = 30},
  [4327] = {.lex_state = 159, .external_lex_state = 30},
  [4328] = {.lex_state = 159, .external_lex_state = 30},
  [4329] = {.lex_state = 159, .external_lex_state = 30},
  [4330] = {.lex_state = 159, .external_lex_state = 30},
  [4331] = {.lex_state = 159, .external_lex_state = 30},
  [4332] = {.lex_state = 169, .external_lex_state = 31},
  [4333] = {.lex_state = 159, .external_lex_state = 30},
  [4334] = {.lex_state = 159, .external_lex_state = 30},
  [4335] = {.lex_state = 159, .external_lex_state = 30},
  [4336] = {.lex_state = 159, .external_lex_state = 30},
  [4337] = {.lex_state = 159, .external_lex_state = 30},
  [4338] = {.lex_state = 159, .external_lex_state = 30},
  [4339] = {.lex_state = 159, .external_lex_state = 30},
  [4340] = {.lex_state = 159, .external_lex_state = 30},
  [4341] = {.lex_state = 159, .external_lex_state = 30},
  [4342] = {.lex_state = 159, .external_lex_state = 30},
  [4343] = {.lex_state = 159, .external_lex_state = 30},
  [4344] = {.lex_state = 159, .external_lex_state = 30},
  [4345] = {.lex_state = 159, .external_lex_state = 30},
  [4346] = {.lex_state = 159, .external_lex_state = 30},
  [4347] = {.lex_state = 159, .external_lex_state = 30},
  [4348] = {.lex_state = 159, .external_lex_state = 30},
  [4349] = {.lex_state = 159, .external_lex_state = 30},
  [4350] = {.lex_state = 159, .external_lex_state = 30},
  [4351] = {.lex_state = 159, .external_lex_state = 30},
  [4352] = {.lex_state = 159, .external_lex_state = 30},
  [4353] = {.lex_state = 159, .external_lex_state = 30},
  [4354] = {.lex_state = 159, .external_lex_state = 30},
  [4355] = {.lex_state = 159, .external_lex_state = 30},
  [4356] = {.lex_state = 159, .external_lex_state = 30},
  [4357] = {.lex_state = 164, .external_lex_state = 32},
  [4358] = {.lex_state = 164, .external_lex_state = 32},
  [4359] = {.lex_state = 164, .external_lex_state = 32},
  [4360] = {.lex_state = 168, .external_lex_state = 32},
  [4361] = {.lex_state = 164, .external_lex_state = 32},
  [4362] = {.lex_state = 162, .external_lex_state = 32},
  [4363] = {.lex_state = 164, .external_lex_state = 32},
  [4364] = {.lex_state = 162, .external_lex_state = 32},
  [4365] = {.lex_state = 164, .external_lex_state = 32},
  [4366] = {.lex_state = 164, .external_lex_state = 32},
  [4367] = {.lex_state = 164, .external_lex_state = 32},
  [4368] = {.lex_state = 168, .external_lex_state = 32},
  [4369] = {.lex_state = 164, .external_lex_state = 32},
  [4370] = {.lex_state = 164, .external_lex_state = 32},
  [4371] = {.lex_state = 164, .external_lex_state = 32},
  [4372] = {.lex_state = 164, .external_lex_state = 32},
  [4373] = {.lex_state = 164, .external_lex_state = 32},
  [4374] = {.lex_state = 164, .external_lex_state = 32},
  [4375] = {.lex_state = 168, .external_lex_state = 32},
  [4376] = {.lex_state = 168, .external_lex_state = 32},
  [4377] = {.lex_state = 164, .external_lex_state = 32},
  [4378] = {.lex_state = 164, .external_lex_state = 32},
  [4379] = {.lex_state = 168, .external_lex_state = 32},
  [4380] = {.lex_state = 160, .external_lex_state = 32},
  [4381] = {.lex_state = 160, .external_lex_state = 32},
  [4382] = {.lex_state = 160, .external_lex_state = 32},
  [4383] = {.lex_state = 160, .external_lex_state = 32},
  [4384] = {.lex_state = 160, .external_lex_state = 32},
  [4385] = {.lex_state = 160, .external_lex_state = 32},
  [4386] = {.lex_state = 160, .external_lex_state = 32},
  [4387] = {.lex_state = 160, .external_lex_state = 32},
  [4388] = {.lex_state = 160, .external_lex_state = 32},
  [4389] = {.lex_state = 160, .external_lex_state = 32},
  [4390] = {.lex_state = 160, .external_lex_state = 32},
  [4391] = {.lex_state = 160, .external_lex_state = 32},
  [4392] = {.lex_state = 160, .external_lex_state = 32},
  [4393] = {.lex_state = 160, .external_lex_state = 32},
  [4394] = {.lex_state = 160, .external_lex_state = 32},
  [4395] = {.lex_state = 160, .external_lex_state = 32},
  [4396] = {.lex_state = 160, .external_lex_state = 32},
  [4397] = {.lex_state = 160, .external_lex_state = 32},
  [4398] = {.lex_state = 160, .external_lex_state = 32},
  [4399] = {.lex_state = 160, .external_lex_state = 32},
  [4400] = {.lex_state = 160, .external_lex_state = 32},
  [4401] = {.lex_state = 160, .external_lex_state = 32},
  [4402] = {.lex_state = 160, .external_lex_state = 32},
  [4403] = {.lex_state = 160, .external_lex_state = 32},
  [4404] = {.lex_state = 160, .external_lex_state = 32},
  [4405] = {.lex_state = 160, .external_lex_state = 32},
  [4406] = {.lex_state = 160, .external_lex_state = 32},
  [4407] = {.lex_state = 160, .external_lex_state = 32},
  [4408] = {.lex_state = 160, .external_lex_state = 32},
  [4409] = {.lex_state = 160, .external_lex_state = 32},
  [4410] = {.lex_state = 168, .external_lex_state = 32},
  [4411] = {.lex_state = 160, .external_lex_state = 32},
  [4412] = {.lex_state = 160, .external_lex_state = 32},
  [4413] = {.lex_state = 160, .external_lex_state = 32},
  [4414] = {.lex_state = 160, .external_lex_state = 32},
  [4415] = {.lex_state = 160, .external_lex_state = 32},
  [4416] = {.lex_state = 160, .external_lex_state = 32},
  [4417] = {.lex_state = 160, .external_lex_state = 32},
  [4418] = {.lex_state = 160, .external_lex_state = 32},
  [4419] = {.lex_state = 160, .external_lex_state = 32},
  [4420] = {.lex_state = 160, .external_lex_state = 32},
  [4421] = {.lex_state = 160, .external_lex_state = 32},
  [4422] = {.lex_state = 160, .external_lex_state = 32},
  [4423] = {.lex_state = 160, .external_lex_state = 32},
  [4424] = {.lex_state = 160, .external_lex_state = 32},
  [4425] = {.lex_state = 160, .external_lex_state = 32},
  [4426] = {.lex_state = 160, .external_lex_state = 32},
  [4427] = {.lex_state = 160, .external_lex_state = 32},
  [4428] = {.lex_state = 160, .external_lex_state = 32},
  [4429] = {.lex_state = 160, .external_lex_state = 32},
  [4430] = {.lex_state = 160, .external_lex_state = 32},
  [4431] = {.lex_state = 160, .external_lex_state = 32},
  [4432] = {.lex_state = 160, .external_lex_state = 32},
  [4433] = {.lex_state = 164, .external_lex_state = 32},
  [4434] = {.lex_state = 162, .external_lex_state = 32},
  [4435] = {.lex_state = 164, .external_lex_state = 32},
  [4436] = {.lex_state = 164, .external_lex_state = 32},
  [4437] = {.lex_state = 164, .external_lex_state = 32},
  [4438] = {.lex_state = 160, .external_lex_state = 32},
  [4439] = {.lex_state = 160, .external_lex_state = 32},
  [4440] = {.lex_state = 51, .external_lex_state = 32},
  [4441] = {.lex_state = 51, .external_lex_state = 32},
  [4442] = {.lex_state = 160, .external_lex_state = 32},
  [4443] = {.lex_state = 51, .external_lex_state = 32},
  [4444] = {.lex_state = 51, .external_lex_state = 32},
  [4445] = {.lex_state = 51, .external_lex_state = 32},
  [4446] = {.lex_state = 160, .external_lex_state = 32},
  [4447] = {.lex_state = 160, .external_lex_state = 32},
  [4448] = {.lex_state = 160, .external_lex_state = 32},
  [4449] = {.lex_state = 160, .external_lex_state = 32},
  [4450] = {.lex_state = 160, .external_lex_state = 32},
  [4451] = {.lex_state = 160, .external_lex_state = 32},
  [4452] = {.lex_state = 160, .external_lex_state = 32},
  [4453] = {.lex_state = 160, .external_lex_state = 32},
  [4454] = {.lex_state = 160, .external_lex_state = 32},
  [4455] = {.lex_state = 51, .external_lex_state = 32},
  [4456] = {.lex_state = 160, .external_lex_state = 32},
  [4457] = {.lex_state = 160, .external_lex_state = 32},
  [4458] = {.lex_state = 169, .external_lex_state = 31},
  [4459] = {.lex_state = 169, .external_lex_state = 31},
  [4460] = {.lex_state = 51, .external_lex_state = 32},
  [4461] = {.lex_state = 160, .external_lex_state = 32},
  [4462] = {.lex_state = 160, .external_lex_state = 32},
  [4463] = {.lex_state = 160, .external_lex_state = 32},
  [4464] = {.lex_state = 160, .external_lex_state = 32},
  [4465] = {.lex_state = 160, .external_lex_state = 32},
  [4466] = {.lex_state = 160, .external_lex_state = 32},
  [4467] = {.lex_state = 160, .external_lex_state = 32},
  [4468] = {.lex_state = 160, .external_lex_state = 32},
  [4469] = {.lex_state = 160, .external_lex_state = 32},
  [4470] = {.lex_state = 160, .external_lex_state = 32},
  [4471] = {.lex_state = 164, .external_lex_state = 32},
  [4472] = {.lex_state = 160, .external_lex_state = 32},
  [4473] = {.lex_state = 162, .external_lex_state = 32},
  [4474] = {.lex_state = 160, .external_lex_state = 32},
  [4475] = {.lex_state = 160, .external_lex_state = 32},
  [4476] = {.lex_state = 160, .external_lex_state = 32},
  [4477] = {.lex_state = 160, .external_lex_state = 32},
  [4478] = {.lex_state = 160, .external_lex_state = 32},
  [4479] = {.lex_state = 160, .external_lex_state = 32},
  [4480] = {.lex_state = 160, .external_lex_state = 32},
  [4481] = {.lex_state = 160, .external_lex_state = 32},
  [4482] = {.lex_state = 160, .external_lex_state = 32},
  [4483] = {.lex_state = 160, .external_lex_state = 32},
  [4484] = {.lex_state = 160, .external_lex_state = 32},
  [4485] = {.lex_state = 160, .external_lex_state = 32},
  [4486] = {.lex_state = 160, .external_lex_state = 32},
  [4487] = {.lex_state = 160, .external_lex_state = 32},
  [4488] = {.lex_state = 160, .external_lex_state = 32},
  [4489] = {.lex_state = 160, .external_lex_state = 32},
  [4490] = {.lex_state = 160, .external_lex_state = 32},
  [4491] = {.lex_state = 160, .external_lex_state = 32},
  [4492] = {.lex_state = 160, .external_lex_state = 32},
  [4493] = {.lex_state = 160, .external_lex_state = 32},
  [4494] = {.lex_state = 160, .external_lex_state = 32},
  [4495] = {.lex_state = 169, .external_lex_state = 34},
  [4496] = {.lex_state = 160, .external_lex_state = 32},
  [4497] = {.lex_state = 160, .external_lex_state = 32},
  [4498] = {.lex_state = 160, .external_lex_state = 32},
  [4499] = {.lex_state = 160, .external_lex_state = 32},
  [4500] = {.lex_state = 160, .external_lex_state = 32},
  [4501] = {.lex_state = 160, .external_lex_state = 32},
  [4502] = {.lex_state = 160, .external_lex_state = 32},
  [4503] = {.lex_state = 160, .external_lex_state = 32},
  [4504] = {.lex_state = 169, .external_lex_state = 34},
  [4505] = {.lex_state = 169, .external_lex_state = 34},
  [4506] = {.lex_state = 160, .external_lex_state = 32},
  [4507] = {.lex_state = 160, .external_lex_state = 32},
  [4508] = {.lex_state = 160, .external_lex_state = 32},
  [4509] = {.lex_state = 160, .external_lex_state = 32},
  [4510] = {.lex_state = 160, .external_lex_state = 32},
  [4511] = {.lex_state = 160, .external_lex_state = 32},
  [4512] = {.lex_state = 160, .external_lex_state = 32},
  [4513] = {.lex_state = 160, .external_lex_state = 32},
  [4514] = {.lex_state = 169, .external_lex_state = 34},
  [4515] = {.lex_state = 160, .external_lex_state = 32},
  [4516] = {.lex_state = 160, .external_lex_state = 32},
  [4517] = {.lex_state = 160, .external_lex_state = 32},
  [4518] = {.lex_state = 160, .external_lex_state = 32},
  [4519] = {.lex_state = 160, .external_lex_state = 32},
  [4520] = {.lex_state = 160, .external_lex_state = 32},
  [4521] = {.lex_state = 160, .external_lex_state = 32},
  [4522] = {.lex_state = 160, .external_lex_state = 32},
  [4523] = {.lex_state = 160, .external_lex_state = 32},
  [4524] = {.lex_state = 160, .external_lex_state = 32},
  [4525] = {.lex_state = 160, .external_lex_state = 32},
  [4526] = {.lex_state = 160, .external_lex_state = 32},
  [4527] = {.lex_state = 160, .external_lex_state = 32},
  [4528] = {.lex_state = 160, .external_lex_state = 32},
  [4529] = {.lex_state = 160, .external_lex_state = 32},
  [4530] = {.lex_state = 160, .external_lex_state = 32},
  [4531] = {.lex_state = 160, .external_lex_state = 32},
  [4532] = {.lex_state = 160, .external_lex_state = 32},
  [4533] = {.lex_state = 160, .external_lex_state = 32},
  [4534] = {.lex_state = 51, .external_lex_state = 32},
  [4535] = {.lex_state = 160, .external_lex_state = 32},
  [4536] = {.lex_state = 51, .external_lex_state = 32},
  [4537] = {.lex_state = 51, .external_lex_state = 32},
  [4538] = {.lex_state = 160, .external_lex_state = 32},
  [4539] = {.lex_state = 51, .external_lex_state = 32},
  [4540] = {.lex_state = 51, .external_lex_state = 32},
  [4541] = {.lex_state = 51, .external_lex_state = 32},
  [4542] = {.lex_state = 51, .external_lex_state = 32},
  [4543] = {.lex_state = 51, .external_lex_state = 32},
  [4544] = {.lex_state = 51, .external_lex_state = 32},
  [4545] = {.lex_state = 51, .external_lex_state = 32},
  [4546] = {.lex_state = 51, .external_lex_state = 32},
  [4547] = {.lex_state = 51, .external_lex_state = 32},
  [4548] = {.lex_state = 51, .external_lex_state = 32},
  [4549] = {.lex_state = 51, .external_lex_state = 32},
  [4550] = {.lex_state = 51, .external_lex_state = 32},
  [4551] = {.lex_state = 51, .external_lex_state = 32},
  [4552] = {.lex_state = 51, .external_lex_state = 32},
  [4553] = {.lex_state = 51, .external_lex_state = 32},
  [4554] = {.lex_state = 51, .external_lex_state = 32},
  [4555] = {.lex_state = 51, .external_lex_state = 32},
  [4556] = {.lex_state = 51, .external_lex_state = 32},
  [4557] = {.lex_state = 51, .external_lex_state = 32},
  [4558] = {.lex_state = 51, .external_lex_state = 32},
  [4559] = {.lex_state = 51, .external_lex_state = 32},
  [4560] = {.lex_state = 162, .external_lex_state = 35},
  [4561] = {.lex_state = 160, .external_lex_state = 32},
  [4562] = {.lex_state = 162, .external_lex_state = 35},
  [4563] = {.lex_state = 168, .external_lex_state = 32},
  [4564] = {.lex_state = 162, .external_lex_state = 35},
  [4565] = {.lex_state = 160, .external_lex_state = 32},
  [4566] = {.lex_state = 162, .external_lex_state = 35},
  [4567] = {.lex_state = 160, .external_lex_state = 32},
  [4568] = {.lex_state = 162, .external_lex_state = 35},
  [4569] = {.lex_state = 162, .external_lex_state = 35},
  [4570] = {.lex_state = 162, .external_lex_state = 35},
  [4571] = {.lex_state = 160, .external_lex_state = 32},
  [4572] = {.lex_state = 162, .external_lex_state = 35},
  [4573] = {.lex_state = 160, .external_lex_state = 32},
  [4574] = {.lex_state = 160, .external_lex_state = 32},
  [4575] = {.lex_state = 162, .external_lex_state = 35},
  [4576] = {.lex_state = 160, .external_lex_state = 32},
  [4577] = {.lex_state = 160, .external_lex_state = 32},
  [4578] = {.lex_state = 160, .external_lex_state = 32},
  [4579] = {.lex_state = 160, .external_lex_state = 32},
  [4580] = {.lex_state = 160, .external_lex_state = 32},
  [4581] = {.lex_state = 160, .external_lex_state = 32},
  [4582] = {.lex_state = 160, .external_lex_state = 32},
  [4583] = {.lex_state = 160, .external_lex_state = 32},
  [4584] = {.lex_state = 162, .external_lex_state = 32},
  [4585] = {.lex_state = 172, .external_lex_state = 32},
  [4586] = {.lex_state = 160, .external_lex_state = 32},
  [4587] = {.lex_state = 160, .external_lex_state = 32},
  [4588] = {.lex_state = 162, .external_lex_state = 35},
  [4589] = {.lex_state = 160, .external_lex_state = 32},
  [4590] = {.lex_state = 162, .external_lex_state = 35},
  [4591] = {.lex_state = 162, .external_lex_state = 35},
  [4592] = {.lex_state = 160, .external_lex_state = 32},
  [4593] = {.lex_state = 160, .external_lex_state = 32},
  [4594] = {.lex_state = 160, .external_lex_state = 32},
  [4595] = {.lex_state = 160, .external_lex_state = 32},
  [4596] = {.lex_state = 160, .external_lex_state = 32},
  [4597] = {.lex_state = 160, .external_lex_state = 32},
  [4598] = {.lex_state = 160, .external_lex_state = 32},
  [4599] = {.lex_state = 160, .external_lex_state = 32},
  [4600] = {.lex_state = 162, .external_lex_state = 35},
  [4601] = {.lex_state = 160, .external_lex_state = 32},
  [4602] = {.lex_state = 160, .external_lex_state = 32},
  [4603] = {.lex_state = 160, .external_lex_state = 32},
  [4604] = {.lex_state = 160, .external_lex_state = 32},
  [4605] = {.lex_state = 162, .external_lex_state = 32},
  [4606] = {.lex_state = 160, .external_lex_state = 32},
  [4607] = {.lex_state = 164, .external_lex_state = 32},
  [4608] = {.lex_state = 160, .external_lex_state = 32},
  [4609] = {.lex_state = 160, .external_lex_state = 32},
  [4610] = {.lex_state = 160, .external_lex_state = 32},
  [4611] = {.lex_state = 162, .external_lex_state = 35},
  [4612] = {.lex_state = 160, .external_lex_state = 32},
  [4613] = {.lex_state = 169, .external_lex_state = 34},
  [4614] = {.lex_state = 169, .external_lex_state = 34},
  [4615] = {.lex_state = 160, .external_lex_state = 32},
  [4616] = {.lex_state = 162, .external_lex_state = 35},
  [4617] = {.lex_state = 162, .external_lex_state = 35},
  [4618] = {.lex_state = 160, .external_lex_state = 32},
  [4619] = {.lex_state = 162, .external_lex_state = 35},
  [4620] = {.lex_state = 162, .external_lex_state = 35},
  [4621] = {.lex_state = 168, .external_lex_state = 32},
  [4622] = {.lex_state = 162, .external_lex_state = 35},
  [4623] = {.lex_state = 162, .external_lex_state = 35},
  [4624] = {.lex_state = 160, .external_lex_state = 32},
  [4625] = {.lex_state = 160, .external_lex_state = 32},
  [4626] = {.lex_state = 160, .external_lex_state = 32},
  [4627] = {.lex_state = 160, .external_lex_state = 32},
  [4628] = {.lex_state = 160, .external_lex_state = 32},
  [4629] = {.lex_state = 160, .external_lex_state = 35},
  [4630] = {.lex_state = 160, .external_lex_state = 32},
  [4631] = {.lex_state = 160, .external_lex_state = 32},
  [4632] = {.lex_state = 160, .external_lex_state = 32},
  [4633] = {.lex_state = 160, .external_lex_state = 32},
  [4634] = {.lex_state = 160, .external_lex_state = 32},
  [4635] = {.lex_state = 160, .external_lex_state = 32},
  [4636] = {.lex_state = 160, .external_lex_state = 32},
  [4637] = {.lex_state = 160, .external_lex_state = 35},
  [4638] = {.lex_state = 160, .external_lex_state = 32},
  [4639] = {.lex_state = 160, .external_lex_state = 32},
  [4640] = {.lex_state = 160, .external_lex_state = 32},
  [4641] = {.lex_state = 160, .external_lex_state = 32},
  [4642] = {.lex_state = 160, .external_lex_state = 32},
  [4643] = {.lex_state = 160, .external_lex_state = 32},
  [4644] = {.lex_state = 162, .external_lex_state = 32},
  [4645] = {.lex_state = 160, .external_lex_state = 32},
  [4646] = {.lex_state = 164, .external_lex_state = 35},
  [4647] = {.lex_state = 164, .external_lex_state = 35},
  [4648] = {.lex_state = 164, .external_lex_state = 35},
  [4649] = {.lex_state = 164, .external_lex_state = 35},
  [4650] = {.lex_state = 164, .external_lex_state = 35},
  [4651] = {.lex_state = 165, .external_lex_state = 36},
  [4652] = {.lex_state = 164, .external_lex_state = 35},
  [4653] = {.lex_state = 168, .external_lex_state = 35},
  [4654] = {.lex_state = 168, .external_lex_state = 35},
  [4655] = {.lex_state = 164, .external_lex_state = 35},
  [4656] = {.lex_state = 168, .external_lex_state = 35},
  [4657] = {.lex_state = 168, .external_lex_state = 35},
  [4658] = {.lex_state = 168, .external_lex_state = 35},
  [4659] = {.lex_state = 168, .external_lex_state = 35},
  [4660] = {.lex_state = 162, .external_lex_state = 35},
  [4661] = {.lex_state = 164, .external_lex_state = 35},
  [4662] = {.lex_state = 164, .external_lex_state = 35},
  [4663] = {.lex_state = 160, .external_lex_state = 32},
  [4664] = {.lex_state = 160, .external_lex_state = 32},
  [4665] = {.lex_state = 164, .external_lex_state = 35},
  [4666] = {.lex_state = 162, .external_lex_state = 35},
  [4667] = {.lex_state = 165, .external_lex_state = 36},
  [4668] = {.lex_state = 165, .external_lex_state = 36},
  [4669] = {.lex_state = 164, .external_lex_state = 35},
  [4670] = {.lex_state = 164, .external_lex_state = 35},
  [4671] = {.lex_state = 164, .external_lex_state = 35},
  [4672] = {.lex_state = 164, .external_lex_state = 35},
  [4673] = {.lex_state = 164, .external_lex_state = 35},
  [4674] = {.lex_state = 164, .external_lex_state = 35},
  [4675] = {.lex_state = 160, .external_lex_state = 35},
  [4676] = {.lex_state = 160, .external_lex_state = 35},
  [4677] = {.lex_state = 160, .external_lex_state = 35},
  [4678] = {.lex_state = 160, .external_lex_state = 35},
  [4679] = {.lex_state = 160, .external_lex_state = 35},
  [4680] = {.lex_state = 160, .external_lex_state = 32},
  [4681] = {.lex_state = 160, .external_lex_state = 35},
  [4682] = {.lex_state = 160, .external_lex_state = 35},
  [4683] = {.lex_state = 160, .external_lex_state = 35},
  [4684] = {.lex_state = 160, .external_lex_state = 35},
  [4685] = {.lex_state = 160, .external_lex_state = 35},
  [4686] = {.lex_state = 160, .external_lex_state = 32},
  [4687] = {.lex_state = 160, .external_lex_state = 35},
  [4688] = {.lex_state = 160, .external_lex_state = 35},
  [4689] = {.lex_state = 160, .external_lex_state = 35},
  [4690] = {.lex_state = 160, .external_lex_state = 35},
  [4691] = {.lex_state = 162, .external_lex_state = 32},
  [4692] = {.lex_state = 160, .external_lex_state = 35},
  [4693] = {.lex_state = 160, .external_lex_state = 35},
  [4694] = {.lex_state = 160, .external_lex_state = 35},
  [4695] = {.lex_state = 160, .external_lex_state = 35},
  [4696] = {.lex_state = 160, .external_lex_state = 35},
  [4697] = {.lex_state = 160, .external_lex_state = 32},
  [4698] = {.lex_state = 160, .external_lex_state = 35},
  [4699] = {.lex_state = 160, .external_lex_state = 35},
  [4700] = {.lex_state = 160, .external_lex_state = 35},
  [4701] = {.lex_state = 160, .external_lex_state = 35},
  [4702] = {.lex_state = 160, .external_lex_state = 35},
  [4703] = {.lex_state = 160, .external_lex_state = 35},
  [4704] = {.lex_state = 160, .external_lex_state = 35},
  [4705] = {.lex_state = 160, .external_lex_state = 35},
  [4706] = {.lex_state = 160, .external_lex_state = 35},
  [4707] = {.lex_state = 160, .external_lex_state = 35},
  [4708] = {.lex_state = 164, .external_lex_state = 33},
  [4709] = {.lex_state = 168, .external_lex_state = 35},
  [4710] = {.lex_state = 164, .external_lex_state = 33},
  [4711] = {.lex_state = 160, .external_lex_state = 35},
  [4712] = {.lex_state = 160, .external_lex_state = 35},
  [4713] = {.lex_state = 160, .external_lex_state = 35},
  [4714] = {.lex_state = 160, .external_lex_state = 35},
  [4715] = {.lex_state = 160, .external_lex_state = 35},
  [4716] = {.lex_state = 160, .external_lex_state = 35},
  [4717] = {.lex_state = 160, .external_lex_state = 35},
  [4718] = {.lex_state = 160, .external_lex_state = 35},
  [4719] = {.lex_state = 160, .external_lex_state = 35},
  [4720] = {.lex_state = 160, .external_lex_state = 35},
  [4721] = {.lex_state = 160, .external_lex_state = 35},
  [4722] = {.lex_state = 160, .external_lex_state = 35},
  [4723] = {.lex_state = 160, .external_lex_state = 35},
  [4724] = {.lex_state = 160, .external_lex_state = 35},
  [4725] = {.lex_state = 160, .external_lex_state = 35},
  [4726] = {.lex_state = 160, .external_lex_state = 32},
  [4727] = {.lex_state = 160, .external_lex_state = 35},
  [4728] = {.lex_state = 162, .external_lex_state = 35},
  [4729] = {.lex_state = 160, .external_lex_state = 35},
  [4730] = {.lex_state = 160, .external_lex_state = 35},
  [4731] = {.lex_state = 160, .external_lex_state = 32},
  [4732] = {.lex_state = 160, .external_lex_state = 32},
  [4733] = {.lex_state = 160, .external_lex_state = 35},
  [4734] = {.lex_state = 160, .external_lex_state = 35},
  [4735] = {.lex_state = 160, .external_lex_state = 35},
  [4736] = {.lex_state = 160, .external_lex_state = 35},
  [4737] = {.lex_state = 160, .external_lex_state = 35},
  [4738] = {.lex_state = 160, .external_lex_state = 35},
  [4739] = {.lex_state = 160, .external_lex_state = 35},
  [4740] = {.lex_state = 160, .external_lex_state = 35},
  [4741] = {.lex_state = 162, .external_lex_state = 35},
  [4742] = {.lex_state = 165, .external_lex_state = 36},
  [4743] = {.lex_state = 160, .external_lex_state = 32},
  [4744] = {.lex_state = 165, .external_lex_state = 36},
  [4745] = {.lex_state = 53, .external_lex_state = 32},
  [4746] = {.lex_state = 160, .external_lex_state = 35},
  [4747] = {.lex_state = 160, .external_lex_state = 32},
  [4748] = {.lex_state = 160, .external_lex_state = 32},
  [4749] = {.lex_state = 160, .external_lex_state = 32},
  [4750] = {.lex_state = 160, .external_lex_state = 35},
  [4751] = {.lex_state = 160, .external_lex_state = 32},
  [4752] = {.lex_state = 160, .external_lex_state = 32},
  [4753] = {.lex_state = 169, .external_lex_state = 34},
  [4754] = {.lex_state = 160, .external_lex_state = 35},
  [4755] = {.lex_state = 160, .external_lex_state = 35},
  [4756] = {.lex_state = 160, .external_lex_state = 35},
  [4757] = {.lex_state = 160, .external_lex_state = 35},
  [4758] = {.lex_state = 160, .external_lex_state = 32},
  [4759] = {.lex_state = 160, .external_lex_state = 35},
  [4760] = {.lex_state = 160, .external_lex_state = 35},
  [4761] = {.lex_state = 160, .external_lex_state = 32},
  [4762] = {.lex_state = 165, .external_lex_state = 36},
  [4763] = {.lex_state = 160, .external_lex_state = 35},
  [4764] = {.lex_state = 165, .external_lex_state = 36},
  [4765] = {.lex_state = 160, .external_lex_state = 35},
  [4766] = {.lex_state = 165, .external_lex_state = 36},
  [4767] = {.lex_state = 160, .external_lex_state = 35},
  [4768] = {.lex_state = 160, .external_lex_state = 32},
  [4769] = {.lex_state = 160, .external_lex_state = 35},
  [4770] = {.lex_state = 160, .external_lex_state = 32},
  [4771] = {.lex_state = 160, .external_lex_state = 32},
  [4772] = {.lex_state = 172, .external_lex_state = 32},
  [4773] = {.lex_state = 165, .external_lex_state = 36},
  [4774] = {.lex_state = 169, .external_lex_state = 34},
  [4775] = {.lex_state = 165, .external_lex_state = 36},
  [4776] = {.lex_state = 162, .external_lex_state = 35},
  [4777] = {.lex_state = 160, .external_lex_state = 32},
  [4778] = {.lex_state = 165, .external_lex_state = 36},
  [4779] = {.lex_state = 160, .external_lex_state = 35},
  [4780] = {.lex_state = 160, .external_lex_state = 35},
  [4781] = {.lex_state = 160, .external_lex_state = 32},
  [4782] = {.lex_state = 160, .external_lex_state = 32},
  [4783] = {.lex_state = 160, .external_lex_state = 35},
  [4784] = {.lex_state = 160, .external_lex_state = 32},
  [4785] = {.lex_state = 169, .external_lex_state = 34},
  [4786] = {.lex_state = 160, .external_lex_state = 35},
  [4787] = {.lex_state = 164, .external_lex_state = 35},
  [4788] = {.lex_state = 160, .external_lex_state = 35},
  [4789] = {.lex_state = 160, .external_lex_state = 35},
  [4790] = {.lex_state = 160, .external_lex_state = 35},
  [4791] = {.lex_state = 160, .external_lex_state = 35},
  [4792] = {.lex_state = 160, .external_lex_state = 32},
  [4793] = {.lex_state = 160, .external_lex_state = 32},
  [4794] = {.lex_state = 160, .external_lex_state = 35},
  [4795] = {.lex_state = 160, .external_lex_state = 35},
  [4796] = {.lex_state = 164, .external_lex_state = 35},
  [4797] = {.lex_state = 160, .external_lex_state = 32},
  [4798] = {.lex_state = 160, .external_lex_state = 32},
  [4799] = {.lex_state = 160, .external_lex_state = 35},
  [4800] = {.lex_state = 160, .external_lex_state = 35},
  [4801] = {.lex_state = 160, .external_lex_state = 32},
  [4802] = {.lex_state = 160, .external_lex_state = 35},
  [4803] = {.lex_state = 160, .external_lex_state = 35},
  [4804] = {.lex_state = 164, .external_lex_state = 35},
  [4805] = {.lex_state = 164, .external_lex_state = 35},
  [4806] = {.lex_state = 160, .external_lex_state = 32},
  [4807] = {.lex_state = 164, .external_lex_state = 33},
  [4808] = {.lex_state = 160, .external_lex_state = 32},
  [4809] = {.lex_state = 160, .external_lex_state = 32},
  [4810] = {.lex_state = 160, .external_lex_state = 35},
  [4811] = {.lex_state = 169, .external_lex_state = 34},
  [4812] = {.lex_state = 160, .external_lex_state = 32},
  [4813] = {.lex_state = 165, .external_lex_state = 36},
  [4814] = {.lex_state = 165, .external_lex_state = 36},
  [4815] = {.lex_state = 165, .external_lex_state = 36},
  [4816] = {.lex_state = 160, .external_lex_state = 35},
  [4817] = {.lex_state = 164, .external_lex_state = 33},
  [4818] = {.lex_state = 160, .external_lex_state = 32},
  [4819] = {.lex_state = 160, .external_lex_state = 35},
  [4820] = {.lex_state = 160, .external_lex_state = 32},
  [4821] = {.lex_state = 160, .external_lex_state = 32},
  [4822] = {.lex_state = 169, .external_lex_state = 34},
  [4823] = {.lex_state = 51, .external_lex_state = 32},
  [4824] = {.lex_state = 160, .external_lex_state = 35},
  [4825] = {.lex_state = 160, .external_lex_state = 32},
  [4826] = {.lex_state = 164, .external_lex_state = 35},
  [4827] = {.lex_state = 160, .external_lex_state = 32},
  [4828] = {.lex_state = 160, .external_lex_state = 32},
  [4829] = {.lex_state = 160, .external_lex_state = 35},
  [4830] = {.lex_state = 162, .external_lex_state = 35},
  [4831] = {.lex_state = 164, .external_lex_state = 33},
  [4832] = {.lex_state = 160, .external_lex_state = 35},
  [4833] = {.lex_state = 164, .external_lex_state = 33},
  [4834] = {.lex_state = 160, .external_lex_state = 32},
  [4835] = {.lex_state = 164, .external_lex_state = 33},
  [4836] = {.lex_state = 164, .external_lex_state = 36},
  [4837] = {.lex_state = 160, .external_lex_state = 32},
  [4838] = {.lex_state = 160, .external_lex_state = 32},
  [4839] = {.lex_state = 160, .external_lex_state = 32},
  [4840] = {.lex_state = 160, .external_lex_state = 32},
  [4841] = {.lex_state = 164, .external_lex_state = 33},
  [4842] = {.lex_state = 164, .external_lex_state = 33},
  [4843] = {.lex_state = 160, .external_lex_state = 32},
  [4844] = {.lex_state = 160, .external_lex_state = 32},
  [4845] = {.lex_state = 160, .external_lex_state = 35},
  [4846] = {.lex_state = 160, .external_lex_state = 35},
  [4847] = {.lex_state = 160, .external_lex_state = 32},
  [4848] = {.lex_state = 164, .external_lex_state = 33},
  [4849] = {.lex_state = 160, .external_lex_state = 32},
  [4850] = {.lex_state = 160, .external_lex_state = 35},
  [4851] = {.lex_state = 164, .external_lex_state = 33},
  [4852] = {.lex_state = 164, .external_lex_state = 33},
  [4853] = {.lex_state = 160, .external_lex_state = 32},
  [4854] = {.lex_state = 160, .external_lex_state = 32},
  [4855] = {.lex_state = 160, .external_lex_state = 32},
  [4856] = {.lex_state = 160, .external_lex_state = 32},
  [4857] = {.lex_state = 160, .external_lex_state = 32},
  [4858] = {.lex_state = 160, .external_lex_state = 35},
  [4859] = {.lex_state = 160, .external_lex_state = 32},
  [4860] = {.lex_state = 160, .external_lex_state = 32},
  [4861] = {.lex_state = 160, .external_lex_state = 32},
  [4862] = {.lex_state = 160, .external_lex_state = 32},
  [4863] = {.lex_state = 160, .external_lex_state = 35},
  [4864] = {.lex_state = 53, .external_lex_state = 32},
  [4865] = {.lex_state = 160, .external_lex_state = 35},
  [4866] = {.lex_state = 160, .external_lex_state = 35},
  [4867] = {.lex_state = 160, .external_lex_state = 32},
  [4868] = {.lex_state = 160, .external_lex_state = 32},
  [4869] = {.lex_state = 160, .external_lex_state = 32},
  [4870] = {.lex_state = 160, .external_lex_state = 32},
  [4871] = {.lex_state = 160, .external_lex_state = 32},
  [4872] = {.lex_state = 160, .external_lex_state = 32},
  [4873] = {.lex_state = 160, .external_lex_state = 32},
  [4874] = {.lex_state = 160, .external_lex_state = 32},
  [4875] = {.lex_state = 160, .external_lex_state = 32},
  [4876] = {.lex_state = 160, .external_lex_state = 32},
  [4877] = {.lex_state = 160, .external_lex_state = 32},
  [4878] = {.lex_state = 160, .external_lex_state = 32},
  [4879] = {.lex_state = 160, .external_lex_state = 32},
  [4880] = {.lex_state = 160, .external_lex_state = 32},
  [4881] = {.lex_state = 160, .external_lex_state = 35},
  [4882] = {.lex_state = 164, .external_lex_state = 33},
  [4883] = {.lex_state = 47, .external_lex_state = 37},
  [4884] = {.lex_state = 47, .external_lex_state = 37},
  [4885] = {.lex_state = 160, .external_lex_state = 35},
  [4886] = {.lex_state = 160, .external_lex_state = 35},
  [4887] = {.lex_state = 47, .external_lex_state = 37},
  [4888] = {.lex_state = 47, .external_lex_state = 37},
  [4889] = {.lex_state = 160, .external_lex_state = 35},
  [4890] = {.lex_state = 160, .external_lex_state = 35},
  [4891] = {.lex_state = 160, .external_lex_state = 32},
  [4892] = {.lex_state = 47, .external_lex_state = 37},
  [4893] = {.lex_state = 160, .external_lex_state = 35},
  [4894] = {.lex_state = 160, .external_lex_state = 35},
  [4895] = {.lex_state = 160, .external_lex_state = 35},
  [4896] = {.lex_state = 160, .external_lex_state = 32},
  [4897] = {.lex_state = 160, .external_lex_state = 35},
  [4898] = {.lex_state = 160, .external_lex_state = 35},
  [4899] = {.lex_state = 160, .external_lex_state = 35},
  [4900] = {.lex_state = 47, .external_lex_state = 37},
  [4901] = {.lex_state = 47, .external_lex_state = 37},
  [4902] = {.lex_state = 160, .external_lex_state = 35},
  [4903] = {.lex_state = 47, .external_lex_state = 37},
  [4904] = {.lex_state = 160, .external_lex_state = 35},
  [4905] = {.lex_state = 160, .external_lex_state = 35},
  [4906] = {.lex_state = 47, .external_lex_state = 37},
  [4907] = {.lex_state = 47, .external_lex_state = 37},
  [4908] = {.lex_state = 160, .external_lex_state = 35},
  [4909] = {.lex_state = 47, .external_lex_state = 37},
  [4910] = {.lex_state = 47, .external_lex_state = 37},
  [4911] = {.lex_state = 47, .external_lex_state = 37},
  [4912] = {.lex_state = 160, .external_lex_state = 35},
  [4913] = {.lex_state = 47, .external_lex_state = 37},
  [4914] = {.lex_state = 160, .external_lex_state = 35},
  [4915] = {.lex_state = 47, .external_lex_state = 37},
  [4916] = {.lex_state = 160, .external_lex_state = 35},
  [4917] = {.lex_state = 47, .external_lex_state = 37},
  [4918] = {.lex_state = 160, .external_lex_state = 32},
  [4919] = {.lex_state = 47, .external_lex_state = 37},
  [4920] = {.lex_state = 47, .external_lex_state = 37},
  [4921] = {.lex_state = 160, .external_lex_state = 35},
  [4922] = {.lex_state = 47, .external_lex_state = 37},
  [4923] = {.lex_state = 160, .external_lex_state = 35},
  [4924] = {.lex_state = 160, .external_lex_state = 32},
  [4925] = {.lex_state = 160, .external_lex_state = 32},
  [4926] = {.lex_state = 160, .external_lex_state = 35},
  [4927] = {.lex_state = 160, .external_lex_state = 35},
  [4928] = {.lex_state = 47, .external_lex_state = 37},
  [4929] = {.lex_state = 160, .external_lex_state = 35},
  [4930] = {.lex_state = 160, .external_lex_state = 35},
  [4931] = {.lex_state = 160, .external_lex_state = 35},
  [4932] = {.lex_state = 160, .external_lex_state = 35},
  [4933] = {.lex_state = 160, .external_lex_state = 35},
  [4934] = {.lex_state = 160, .external_lex_state = 35},
  [4935] = {.lex_state = 160, .external_lex_state = 35},
  [4936] = {.lex_state = 160, .external_lex_state = 35},
  [4937] = {.lex_state = 160, .external_lex_state = 35},
  [4938] = {.lex_state = 160, .external_lex_state = 35},
  [4939] = {.lex_state = 47, .external_lex_state = 37},
  [4940] = {.lex_state = 160, .external_lex_state = 35},
  [4941] = {.lex_state = 160, .external_lex_state = 35},
  [4942] = {.lex_state = 165, .external_lex_state = 36},
  [4943] = {.lex_state = 160, .external_lex_state = 35},
  [4944] = {.lex_state = 160, .external_lex_state = 35},
  [4945] = {.lex_state = 160, .external_lex_state = 35},
  [4946] = {.lex_state = 165, .external_lex_state = 36},
  [4947] = {.lex_state = 160, .external_lex_state = 32},
  [4948] = {.lex_state = 160, .external_lex_state = 35},
  [4949] = {.lex_state = 165, .external_lex_state = 36},
  [4950] = {.lex_state = 160, .external_lex_state = 32},
  [4951] = {.lex_state = 165, .external_lex_state = 36},
  [4952] = {.lex_state = 165, .external_lex_state = 36},
  [4953] = {.lex_state = 165, .external_lex_state = 36},
  [4954] = {.lex_state = 47, .external_lex_state = 37},
  [4955] = {.lex_state = 165, .external_lex_state = 36},
  [4956] = {.lex_state = 160, .external_lex_state = 35},
  [4957] = {.lex_state = 160, .external_lex_state = 35},
  [4958] = {.lex_state = 164, .external_lex_state = 36},
  [4959] = {.lex_state = 165, .external_lex_state = 36},
  [4960] = {.lex_state = 165, .external_lex_state = 36},
  [4961] = {.lex_state = 165, .external_lex_state = 36},
  [4962] = {.lex_state = 165, .external_lex_state = 36},
  [4963] = {.lex_state = 165, .external_lex_state = 36},
  [4964] = {.lex_state = 160, .external_lex_state = 35},
  [4965] = {.lex_state = 165, .external_lex_state = 36},
  [4966] = {.lex_state = 160, .external_lex_state = 35},
  [4967] = {.lex_state = 165, .external_lex_state = 36},
  [4968] = {.lex_state = 165, .external_lex_state = 36},
  [4969] = {.lex_state = 165, .external_lex_state = 36},
  [4970] = {.lex_state = 165, .external_lex_state = 36},
  [4971] = {.lex_state = 47, .external_lex_state = 37},
  [4972] = {.lex_state = 160, .external_lex_state = 35},
  [4973] = {.lex_state = 160, .external_lex_state = 35},
  [4974] = {.lex_state = 160, .external_lex_state = 35},
  [4975] = {.lex_state = 160, .external_lex_state = 35},
  [4976] = {.lex_state = 160, .external_lex_state = 35},
  [4977] = {.lex_state = 160, .external_lex_state = 35},
  [4978] = {.lex_state = 160, .external_lex_state = 32},
  [4979] = {.lex_state = 47, .external_lex_state = 37},
  [4980] = {.lex_state = 47, .external_lex_state = 37},
  [4981] = {.lex_state = 47, .external_lex_state = 37},
  [4982] = {.lex_state = 47, .external_lex_state = 37},
  [4983] = {.lex_state = 47, .external_lex_state = 37},
  [4984] = {.lex_state = 47, .external_lex_state = 37},
  [4985] = {.lex_state = 160, .external_lex_state = 35},
  [4986] = {.lex_state = 47, .external_lex_state = 37},
  [4987] = {.lex_state = 47, .external_lex_state = 37},
  [4988] = {.lex_state = 165, .external_lex_state = 36},
  [4989] = {.lex_state = 160, .external_lex_state = 35},
  [4990] = {.lex_state = 160, .external_lex_state = 35},
  [4991] = {.lex_state = 47, .external_lex_state = 37},
  [4992] = {.lex_state = 47, .external_lex_state = 37},
  [4993] = {.lex_state = 47, .external_lex_state = 37},
  [4994] = {.lex_state = 160, .external_lex_state = 35},
  [4995] = {.lex_state = 160, .external_lex_state = 35},
  [4996] = {.lex_state = 160, .external_lex_state = 35},
  [4997] = {.lex_state = 160, .external_lex_state = 35},
  [4998] = {.lex_state = 160, .external_lex_state = 35},
  [4999] = {.lex_state = 47, .external_lex_state = 37},
  [5000] = {.lex_state = 47, .external_lex_state = 37},
  [5001] = {.lex_state = 160, .external_lex_state = 35},
  [5002] = {.lex_state = 160, .external_lex_state = 35},
  [5003] = {.lex_state = 160, .external_lex_state = 35},
  [5004] = {.lex_state = 47, .external_lex_state = 37},
  [5005] = {.lex_state = 160, .external_lex_state = 35},
  [5006] = {.lex_state = 160, .external_lex_state = 35},
  [5007] = {.lex_state = 160, .external_lex_state = 35},
  [5008] = {.lex_state = 47, .external_lex_state = 37},
  [5009] = {.lex_state = 160, .external_lex_state = 35},
  [5010] = {.lex_state = 160, .external_lex_state = 35},
  [5011] = {.lex_state = 160, .external_lex_state = 35},
  [5012] = {.lex_state = 164, .external_lex_state = 36},
  [5013] = {.lex_state = 164, .external_lex_state = 36},
  [5014] = {.lex_state = 160, .external_lex_state = 35},
  [5015] = {.lex_state = 160, .external_lex_state = 35},
  [5016] = {.lex_state = 165, .external_lex_state = 36},
  [5017] = {.lex_state = 160, .external_lex_state = 35},
  [5018] = {.lex_state = 160, .external_lex_state = 35},
  [5019] = {.lex_state = 160, .external_lex_state = 35},
  [5020] = {.lex_state = 160, .external_lex_state = 35},
  [5021] = {.lex_state = 160, .external_lex_state = 35},
  [5022] = {.lex_state = 165, .external_lex_state = 36},
  [5023] = {.lex_state = 165, .external_lex_state = 36},
  [5024] = {.lex_state = 165, .external_lex_state = 36},
  [5025] = {.lex_state = 47, .external_lex_state = 37},
  [5026] = {.lex_state = 47, .external_lex_state = 37},
  [5027] = {.lex_state = 47, .external_lex_state = 37},
  [5028] = {.lex_state = 160, .external_lex_state = 35},
  [5029] = {.lex_state = 164, .external_lex_state = 36},
  [5030] = {.lex_state = 164, .external_lex_state = 36},
  [5031] = {.lex_state = 165, .external_lex_state = 36},
  [5032] = {.lex_state = 164, .external_lex_state = 36},
  [5033] = {.lex_state = 160, .external_lex_state = 32},
  [5034] = {.lex_state = 160, .external_lex_state = 32},
  [5035] = {.lex_state = 47, .external_lex_state = 37},
  [5036] = {.lex_state = 47, .external_lex_state = 37},
  [5037] = {.lex_state = 151, .external_lex_state = 35},
  [5038] = {.lex_state = 48, .external_lex_state = 37},
  [5039] = {.lex_state = 48, .external_lex_state = 37},
  [5040] = {.lex_state = 164, .external_lex_state = 33},
  [5041] = {.lex_state = 160, .external_lex_state = 35},
  [5042] = {.lex_state = 48, .external_lex_state = 37},
  [5043] = {.lex_state = 48, .external_lex_state = 37},
  [5044] = {.lex_state = 48, .external_lex_state = 37},
  [5045] = {.lex_state = 48, .external_lex_state = 37},
  [5046] = {.lex_state = 160, .external_lex_state = 35},
  [5047] = {.lex_state = 48, .external_lex_state = 37},
  [5048] = {.lex_state = 48, .external_lex_state = 37},
  [5049] = {.lex_state = 48, .external_lex_state = 38},
  [5050] = {.lex_state = 48, .external_lex_state = 37},
  [5051] = {.lex_state = 160, .external_lex_state = 32},
  [5052] = {.lex_state = 164, .external_lex_state = 33},
  [5053] = {.lex_state = 151, .external_lex_state = 35},
  [5054] = {.lex_state = 151, .external_lex_state = 35},
  [5055] = {.lex_state = 151, .external_lex_state = 35},
  [5056] = {.lex_state = 151, .external_lex_state = 35},
  [5057] = {.lex_state = 164, .external_lex_state = 33},
  [5058] = {.lex_state = 164, .external_lex_state = 33},
  [5059] = {.lex_state = 164, .external_lex_state = 33},
  [5060] = {.lex_state = 160, .external_lex_state = 32},
  [5061] = {.lex_state = 151, .external_lex_state = 35},
  [5062] = {.lex_state = 48, .external_lex_state = 37},
  [5063] = {.lex_state = 48, .external_lex_state = 37},
  [5064] = {.lex_state = 48, .external_lex_state = 37},
  [5065] = {.lex_state = 48, .external_lex_state = 37},
  [5066] = {.lex_state = 48, .external_lex_state = 37},
  [5067] = {.lex_state = 164, .external_lex_state = 33},
  [5068] = {.lex_state = 164, .external_lex_state = 33},
  [5069] = {.lex_state = 151, .external_lex_state = 35},
  [5070] = {.lex_state = 48, .external_lex_state = 37},
  [5071] = {.lex_state = 48, .external_lex_state = 37},
  [5072] = {.lex_state = 48, .external_lex_state = 37},
  [5073] = {.lex_state = 48, .external_lex_state = 37},
  [5074] = {.lex_state = 164, .external_lex_state = 33},
  [5075] = {.lex_state = 48, .external_lex_state = 38},
  [5076] = {.lex_state = 164, .external_lex_state = 33},
  [5077] = {.lex_state = 48, .external_lex_state = 37},
  [5078] = {.lex_state = 48, .external_lex_state = 37},
  [5079] = {.lex_state = 48, .external_lex_state = 37},
  [5080] = {.lex_state = 48, .external_lex_state = 37},
  [5081] = {.lex_state = 160, .external_lex_state = 35},
  [5082] = {.lex_state = 164, .external_lex_state = 39},
  [5083] = {.lex_state = 151, .external_lex_state = 35},
  [5084] = {.lex_state = 48, .external_lex_state = 37},
  [5085] = {.lex_state = 48, .external_lex_state = 37},
  [5086] = {.lex_state = 48, .external_lex_state = 37},
  [5087] = {.lex_state = 151, .external_lex_state = 35},
  [5088] = {.lex_state = 48, .external_lex_state = 37},
  [5089] = {.lex_state = 48, .external_lex_state = 37},
  [5090] = {.lex_state = 48, .external_lex_state = 37},
  [5091] = {.lex_state = 48, .external_lex_state = 37},
  [5092] = {.lex_state = 48, .external_lex_state = 37},
  [5093] = {.lex_state = 151, .external_lex_state = 35},
  [5094] = {.lex_state = 48, .external_lex_state = 37},
  [5095] = {.lex_state = 160, .external_lex_state = 35},
  [5096] = {.lex_state = 48, .external_lex_state = 37},
  [5097] = {.lex_state = 151, .external_lex_state = 35},
  [5098] = {.lex_state = 164, .external_lex_state = 36},
  [5099] = {.lex_state = 48, .external_lex_state = 37},
  [5100] = {.lex_state = 164, .external_lex_state = 33},
  [5101] = {.lex_state = 48, .external_lex_state = 37},
  [5102] = {.lex_state = 48, .external_lex_state = 37},
  [5103] = {.lex_state = 151, .external_lex_state = 35},
  [5104] = {.lex_state = 164, .external_lex_state = 33},
  [5105] = {.lex_state = 48, .external_lex_state = 37},
  [5106] = {.lex_state = 151, .external_lex_state = 35},
  [5107] = {.lex_state = 48, .external_lex_state = 37},
  [5108] = {.lex_state = 48, .external_lex_state = 37},
  [5109] = {.lex_state = 48, .external_lex_state = 37},
  [5110] = {.lex_state = 48, .external_lex_state = 37},
  [5111] = {.lex_state = 48, .external_lex_state = 37},
  [5112] = {.lex_state = 48, .external_lex_state = 37},
  [5113] = {.lex_state = 48, .external_lex_state = 37},
  [5114] = {.lex_state = 164, .external_lex_state = 33},
  [5115] = {.lex_state = 151, .external_lex_state = 35},
  [5116] = {.lex_state = 164, .external_lex_state = 33},
  [5117] = {.lex_state = 48, .external_lex_state = 37},
  [5118] = {.lex_state = 164, .external_lex_state = 33},
  [5119] = {.lex_state = 48, .external_lex_state = 37},
  [5120] = {.lex_state = 48, .external_lex_state = 37},
  [5121] = {.lex_state = 48, .external_lex_state = 37},
  [5122] = {.lex_state = 164, .external_lex_state = 39},
  [5123] = {.lex_state = 48, .external_lex_state = 37},
  [5124] = {.lex_state = 164, .external_lex_state = 33},
  [5125] = {.lex_state = 151, .external_lex_state = 35},
  [5126] = {.lex_state = 160, .external_lex_state = 35},
  [5127] = {.lex_state = 48, .external_lex_state = 37},
  [5128] = {.lex_state = 48, .external_lex_state = 37},
  [5129] = {.lex_state = 48, .external_lex_state = 37},
  [5130] = {.lex_state = 164, .external_lex_state = 33},
  [5131] = {.lex_state = 151, .external_lex_state = 35},
  [5132] = {.lex_state = 164, .external_lex_state = 39},
  [5133] = {.lex_state = 160, .external_lex_state = 32},
  [5134] = {.lex_state = 151, .external_lex_state = 35},
  [5135] = {.lex_state = 48, .external_lex_state = 37},
  [5136] = {.lex_state = 48, .external_lex_state = 37},
  [5137] = {.lex_state = 151, .external_lex_state = 35},
  [5138] = {.lex_state = 160, .external_lex_state = 32},
  [5139] = {.lex_state = 160, .external_lex_state = 32},
  [5140] = {.lex_state = 160, .external_lex_state = 32},
  [5141] = {.lex_state = 151, .external_lex_state = 35},
  [5142] = {.lex_state = 48, .external_lex_state = 37},
  [5143] = {.lex_state = 151, .external_lex_state = 35},
  [5144] = {.lex_state = 48, .external_lex_state = 37},
  [5145] = {.lex_state = 48, .external_lex_state = 37},
  [5146] = {.lex_state = 151, .external_lex_state = 35},
  [5147] = {.lex_state = 48, .external_lex_state = 38},
  [5148] = {.lex_state = 151, .external_lex_state = 35},
  [5149] = {.lex_state = 48, .external_lex_state = 37},
  [5150] = {.lex_state = 48, .external_lex_state = 37},
  [5151] = {.lex_state = 48, .external_lex_state = 37},
  [5152] = {.lex_state = 164, .external_lex_state = 33},
  [5153] = {.lex_state = 151, .external_lex_state = 35},
  [5154] = {.lex_state = 48, .external_lex_state = 37},
  [5155] = {.lex_state = 48, .external_lex_state = 37},
  [5156] = {.lex_state = 48, .external_lex_state = 37},
  [5157] = {.lex_state = 151, .external_lex_state = 35},
  [5158] = {.lex_state = 151, .external_lex_state = 35},
  [5159] = {.lex_state = 160, .external_lex_state = 32},
  [5160] = {.lex_state = 151, .external_lex_state = 35},
  [5161] = {.lex_state = 48, .external_lex_state = 37},
  [5162] = {.lex_state = 48, .external_lex_state = 37},
  [5163] = {.lex_state = 48, .external_lex_state = 37},
  [5164] = {.lex_state = 48, .external_lex_state = 37},
  [5165] = {.lex_state = 164, .external_lex_state = 33},
  [5166] = {.lex_state = 151, .external_lex_state = 35},
  [5167] = {.lex_state = 48, .external_lex_state = 37},
  [5168] = {.lex_state = 48, .external_lex_state = 37},
  [5169] = {.lex_state = 151, .external_lex_state = 35},
  [5170] = {.lex_state = 48, .external_lex_state = 37},
  [5171] = {.lex_state = 151, .external_lex_state = 35},
  [5172] = {.lex_state = 151, .external_lex_state = 35},
  [5173] = {.lex_state = 151, .external_lex_state = 35},
  [5174] = {.lex_state = 48, .external_lex_state = 37},
  [5175] = {.lex_state = 164, .external_lex_state = 33},
  [5176] = {.lex_state = 151, .external_lex_state = 35},
  [5177] = {.lex_state = 48, .external_lex_state = 37},
  [5178] = {.lex_state = 48, .external_lex_state = 37},
  [5179] = {.lex_state = 48, .external_lex_state = 37},
  [5180] = {.lex_state = 48, .external_lex_state = 37},
  [5181] = {.lex_state = 48, .external_lex_state = 37},
  [5182] = {.lex_state = 48, .external_lex_state = 37},
  [5183] = {.lex_state = 151, .external_lex_state = 35},
  [5184] = {.lex_state = 160, .external_lex_state = 32},
  [5185] = {.lex_state = 48, .external_lex_state = 37},
  [5186] = {.lex_state = 151, .external_lex_state = 35},
  [5187] = {.lex_state = 151, .external_lex_state = 35},
  [5188] = {.lex_state = 151, .external_lex_state = 35},
  [5189] = {.lex_state = 151, .external_lex_state = 35},
  [5190] = {.lex_state = 151, .external_lex_state = 35},
  [5191] = {.lex_state = 151, .external_lex_state = 35},
  [5192] = {.lex_state = 48, .external_lex_state = 37},
  [5193] = {.lex_state = 151, .external_lex_state = 35},
  [5194] = {.lex_state = 151, .external_lex_state = 35},
  [5195] = {.lex_state = 151, .external_lex_state = 35},
  [5196] = {.lex_state = 160, .external_lex_state = 35},
  [5197] = {.lex_state = 151, .external_lex_state = 35},
  [5198] = {.lex_state = 151, .external_lex_state = 35},
  [5199] = {.lex_state = 151, .external_lex_state = 35},
  [5200] = {.lex_state = 151, .external_lex_state = 35},
  [5201] = {.lex_state = 151, .external_lex_state = 35},
  [5202] = {.lex_state = 48, .external_lex_state = 37},
  [5203] = {.lex_state = 48, .external_lex_state = 37},
  [5204] = {.lex_state = 48, .external_lex_state = 37},
  [5205] = {.lex_state = 48, .external_lex_state = 37},
  [5206] = {.lex_state = 48, .external_lex_state = 37},
  [5207] = {.lex_state = 48, .external_lex_state = 37},
  [5208] = {.lex_state = 48, .external_lex_state = 37},
  [5209] = {.lex_state = 48, .external_lex_state = 37},
  [5210] = {.lex_state = 164, .external_lex_state = 33},
  [5211] = {.lex_state = 164, .external_lex_state = 33},
  [5212] = {.lex_state = 48, .external_lex_state = 37},
  [5213] = {.lex_state = 164, .external_lex_state = 33},
  [5214] = {.lex_state = 160, .external_lex_state = 35},
  [5215] = {.lex_state = 160, .external_lex_state = 35},
  [5216] = {.lex_state = 160, .external_lex_state = 35},
  [5217] = {.lex_state = 160, .external_lex_state = 32},
  [5218] = {.lex_state = 164, .external_lex_state = 36},
  [5219] = {.lex_state = 160, .external_lex_state = 32},
  [5220] = {.lex_state = 151, .external_lex_state = 35},
  [5221] = {.lex_state = 160, .external_lex_state = 35},
  [5222] = {.lex_state = 164, .external_lex_state = 36},
  [5223] = {.lex_state = 48, .external_lex_state = 40},
  [5224] = {.lex_state = 160, .external_lex_state = 35},
  [5225] = {.lex_state = 160, .external_lex_state = 35},
  [5226] = {.lex_state = 160, .external_lex_state = 35},
  [5227] = {.lex_state = 164, .external_lex_state = 36},
  [5228] = {.lex_state = 160, .external_lex_state = 35},
  [5229] = {.lex_state = 151, .external_lex_state = 35},
  [5230] = {.lex_state = 160, .external_lex_state = 35},
  [5231] = {.lex_state = 160, .external_lex_state = 35},
  [5232] = {.lex_state = 160, .external_lex_state = 35},
  [5233] = {.lex_state = 160, .external_lex_state = 35},
  [5234] = {.lex_state = 47, .external_lex_state = 37},
  [5235] = {.lex_state = 160, .external_lex_state = 35},
  [5236] = {.lex_state = 160, .external_lex_state = 32},
  [5237] = {.lex_state = 47, .external_lex_state = 37},
  [5238] = {.lex_state = 151, .external_lex_state = 35},
  [5239] = {.lex_state = 160, .external_lex_state = 35},
  [5240] = {.lex_state = 160, .external_lex_state = 35},
  [5241] = {.lex_state = 47, .external_lex_state = 37},
  [5242] = {.lex_state = 160, .external_lex_state = 35},
  [5243] = {.lex_state = 160, .external_lex_state = 35},
  [5244] = {.lex_state = 160, .external_lex_state = 32},
  [5245] = {.lex_state = 160, .external_lex_state = 35},
  [5246] = {.lex_state = 160, .external_lex_state = 35},
  [5247] = {.lex_state = 160, .external_lex_state = 35},
  [5248] = {.lex_state = 151, .external_lex_state = 35},
  [5249] = {.lex_state = 160, .external_lex_state = 32},
  [5250] = {.lex_state = 160, .external_lex_state = 35},
  [5251] = {.lex_state = 164, .external_lex_state = 36},
  [5252] = {.lex_state = 160, .external_lex_state = 35},
  [5253] = {.lex_state = 160, .external_lex_state = 35},
  [5254] = {.lex_state = 160, .external_lex_state = 35},
  [5255] = {.lex_state = 160, .external_lex_state = 35},
  [5256] = {.lex_state = 164, .external_lex_state = 36},
  [5257] = {.lex_state = 160, .external_lex_state = 35},
  [5258] = {.lex_state = 160, .external_lex_state = 35},
  [5259] = {.lex_state = 47, .external_lex_state = 37},
  [5260] = {.lex_state = 151, .external_lex_state = 35},
  [5261] = {.lex_state = 160, .external_lex_state = 35},
  [5262] = {.lex_state = 160, .external_lex_state = 35},
  [5263] = {.lex_state = 160, .external_lex_state = 35},
  [5264] = {.lex_state = 160, .external_lex_state = 35},
  [5265] = {.lex_state = 160, .external_lex_state = 35},
  [5266] = {.lex_state = 160, .external_lex_state = 35},
  [5267] = {.lex_state = 160, .external_lex_state = 35},
  [5268] = {.lex_state = 160, .external_lex_state = 35},
  [5269] = {.lex_state = 48, .external_lex_state = 40},
  [5270] = {.lex_state = 160, .external_lex_state = 35},
  [5271] = {.lex_state = 160, .external_lex_state = 35},
  [5272] = {.lex_state = 160, .external_lex_state = 35},
  [5273] = {.lex_state = 160, .external_lex_state = 35},
  [5274] = {.lex_state = 160, .external_lex_state = 35},
  [5275] = {.lex_state = 160, .external_lex_state = 35},
  [5276] = {.lex_state = 160, .external_lex_state = 35},
  [5277] = {.lex_state = 160, .external_lex_state = 35},
  [5278] = {.lex_state = 164, .external_lex_state = 36},
  [5279] = {.lex_state = 164, .external_lex_state = 36},
  [5280] = {.lex_state = 160, .external_lex_state = 35},
  [5281] = {.lex_state = 160, .external_lex_state = 35},
  [5282] = {.lex_state = 160, .external_lex_state = 35},
  [5283] = {.lex_state = 160, .external_lex_state = 35},
  [5284] = {.lex_state = 160, .external_lex_state = 35},
  [5285] = {.lex_state = 164, .external_lex_state = 36},
  [5286] = {.lex_state = 160, .external_lex_state = 35},
  [5287] = {.lex_state = 160, .external_lex_state = 35},
  [5288] = {.lex_state = 160, .external_lex_state = 35},
  [5289] = {.lex_state = 160, .external_lex_state = 35},
  [5290] = {.lex_state = 160, .external_lex_state = 35},
  [5291] = {.lex_state = 164, .external_lex_state = 39},
  [5292] = {.lex_state = 164, .external_lex_state = 39},
  [5293] = {.lex_state = 164, .external_lex_state = 39},
  [5294] = {.lex_state = 160, .external_lex_state = 35},
  [5295] = {.lex_state = 48, .external_lex_state = 38},
  [5296] = {.lex_state = 48, .external_lex_state = 37},
  [5297] = {.lex_state = 160, .external_lex_state = 35},
  [5298] = {.lex_state = 48, .external_lex_state = 37},
  [5299] = {.lex_state = 160, .external_lex_state = 32},
  [5300] = {.lex_state = 160, .external_lex_state = 35},
  [5301] = {.lex_state = 51, .external_lex_state = 35},
  [5302] = {.lex_state = 48, .external_lex_state = 37},
  [5303] = {.lex_state = 48, .external_lex_state = 38},
  [5304] = {.lex_state = 51, .external_lex_state = 35},
  [5305] = {.lex_state = 160, .external_lex_state = 35},
  [5306] = {.lex_state = 51, .external_lex_state = 35},
  [5307] = {.lex_state = 48, .external_lex_state = 37},
  [5308] = {.lex_state = 160, .external_lex_state = 32},
  [5309] = {.lex_state = 49, .external_lex_state = 32},
  [5310] = {.lex_state = 160, .external_lex_state = 35},
  [5311] = {.lex_state = 160, .external_lex_state = 32},
  [5312] = {.lex_state = 160, .external_lex_state = 32},
  [5313] = {.lex_state = 48, .external_lex_state = 38},
  [5314] = {.lex_state = 160, .external_lex_state = 32},
  [5315] = {.lex_state = 49, .external_lex_state = 35},
  [5316] = {.lex_state = 160, .external_lex_state = 35},
  [5317] = {.lex_state = 160, .external_lex_state = 35},
  [5318] = {.lex_state = 49, .external_lex_state = 35},
  [5319] = {.lex_state = 48, .external_lex_state = 38},
  [5320] = {.lex_state = 160, .external_lex_state = 32},
  [5321] = {.lex_state = 160, .external_lex_state = 35},
  [5322] = {.lex_state = 160, .external_lex_state = 32},
  [5323] = {.lex_state = 160, .external_lex_state = 32},
  [5324] = {.lex_state = 160, .external_lex_state = 32},
  [5325] = {.lex_state = 160, .external_lex_state = 32},
  [5326] = {.lex_state = 160, .external_lex_state = 35},
  [5327] = {.lex_state = 164, .external_lex_state = 36},
  [5328] = {.lex_state = 164, .external_lex_state = 36},
  [5329] = {.lex_state = 151, .external_lex_state = 35},
  [5330] = {.lex_state = 164, .external_lex_state = 36},
  [5331] = {.lex_state = 164, .external_lex_state = 36},
  [5332] = {.lex_state = 164, .external_lex_state = 36},
  [5333] = {.lex_state = 164, .external_lex_state = 36},
  [5334] = {.lex_state = 164, .external_lex_state = 36},
  [5335] = {.lex_state = 164, .external_lex_state = 36},
  [5336] = {.lex_state = 51, .external_lex_state = 35},
  [5337] = {.lex_state = 164, .external_lex_state = 36},
  [5338] = {.lex_state = 164, .external_lex_state = 36},
  [5339] = {.lex_state = 164, .external_lex_state = 36},
  [5340] = {.lex_state = 164, .external_lex_state = 36},
  [5341] = {.lex_state = 160, .external_lex_state = 35},
  [5342] = {.lex_state = 164, .external_lex_state = 36},
  [5343] = {.lex_state = 160, .external_lex_state = 35},
  [5344] = {.lex_state = 164, .external_lex_state = 36},
  [5345] = {.lex_state = 164, .external_lex_state = 36},
  [5346] = {.lex_state = 164, .external_lex_state = 36},
  [5347] = {.lex_state = 164, .external_lex_state = 36},
  [5348] = {.lex_state = 164, .external_lex_state = 36},
  [5349] = {.lex_state = 164, .external_lex_state = 36},
  [5350] = {.lex_state = 151, .external_lex_state = 35},
  [5351] = {.lex_state = 151, .external_lex_state = 35},
  [5352] = {.lex_state = 164, .external_lex_state = 36},
  [5353] = {.lex_state = 164, .external_lex_state = 36},
  [5354] = {.lex_state = 164, .external_lex_state = 36},
  [5355] = {.lex_state = 164, .external_lex_state = 36},
  [5356] = {.lex_state = 151, .external_lex_state = 35},
  [5357] = {.lex_state = 164, .external_lex_state = 36},
  [5358] = {.lex_state = 164, .external_lex_state = 36},
  [5359] = {.lex_state = 164, .external_lex_state = 36},
  [5360] = {.lex_state = 164, .external_lex_state = 36},
  [5361] = {.lex_state = 160, .external_lex_state = 35},
  [5362] = {.lex_state = 164, .external_lex_state = 36},
  [5363] = {.lex_state = 164, .external_lex_state = 36},
  [5364] = {.lex_state = 164, .external_lex_state = 36},
  [5365] = {.lex_state = 164, .external_lex_state = 36},
  [5366] = {.lex_state = 164, .external_lex_state = 36},
  [5367] = {.lex_state = 164, .external_lex_state = 36},
  [5368] = {.lex_state = 160, .external_lex_state = 35},
  [5369] = {.lex_state = 160, .external_lex_state = 32},
  [5370] = {.lex_state = 164, .external_lex_state = 36},
  [5371] = {.lex_state = 164, .external_lex_state = 36},
  [5372] = {.lex_state = 151, .external_lex_state = 35},
  [5373] = {.lex_state = 164, .external_lex_state = 36},
  [5374] = {.lex_state = 151, .external_lex_state = 35},
  [5375] = {.lex_state = 160, .external_lex_state = 32},
  [5376] = {.lex_state = 164, .external_lex_state = 36},
  [5377] = {.lex_state = 164, .external_lex_state = 36},
  [5378] = {.lex_state = 164, .external_lex_state = 36},
  [5379] = {.lex_state = 164, .external_lex_state = 36},
  [5380] = {.lex_state = 164, .external_lex_state = 36},
  [5381] = {.lex_state = 164, .external_lex_state = 36},
  [5382] = {.lex_state = 51, .external_lex_state = 35},
  [5383] = {.lex_state = 164, .external_lex_state = 36},
  [5384] = {.lex_state = 164, .external_lex_state = 36},
  [5385] = {.lex_state = 160, .external_lex_state = 35},
  [5386] = {.lex_state = 164, .external_lex_state = 36},
  [5387] = {.lex_state = 164, .external_lex_state = 36},
  [5388] = {.lex_state = 160, .external_lex_state = 32},
  [5389] = {.lex_state = 164, .external_lex_state = 36},
  [5390] = {.lex_state = 164, .external_lex_state = 36},
  [5391] = {.lex_state = 164, .external_lex_state = 36},
  [5392] = {.lex_state = 160, .external_lex_state = 35},
  [5393] = {.lex_state = 164, .external_lex_state = 36},
  [5394] = {.lex_state = 164, .external_lex_state = 36},
  [5395] = {.lex_state = 160, .external_lex_state = 35},
  [5396] = {.lex_state = 160, .external_lex_state = 35},
  [5397] = {.lex_state = 164, .external_lex_state = 36},
  [5398] = {.lex_state = 164, .external_lex_state = 36},
  [5399] = {.lex_state = 160, .external_lex_state = 35},
  [5400] = {.lex_state = 164, .external_lex_state = 36},
  [5401] = {.lex_state = 164, .external_lex_state = 36},
  [5402] = {.lex_state = 164, .external_lex_state = 36},
  [5403] = {.lex_state = 151, .external_lex_state = 35},
  [5404] = {.lex_state = 160, .external_lex_state = 35},
  [5405] = {.lex_state = 160, .external_lex_state = 35},
  [5406] = {.lex_state = 164, .external_lex_state = 36},
  [5407] = {.lex_state = 164, .external_lex_state = 36},
  [5408] = {.lex_state = 151, .external_lex_state = 32},
  [5409] = {.lex_state = 164, .external_lex_state = 36},
  [5410] = {.lex_state = 164, .external_lex_state = 36},
  [5411] = {.lex_state = 164, .external_lex_state = 36},
  [5412] = {.lex_state = 164, .external_lex_state = 36},
  [5413] = {.lex_state = 164, .external_lex_state = 36},
  [5414] = {.lex_state = 164, .external_lex_state = 36},
  [5415] = {.lex_state = 164, .external_lex_state = 36},
  [5416] = {.lex_state = 164, .external_lex_state = 36},
  [5417] = {.lex_state = 164, .external_lex_state = 36},
  [5418] = {.lex_state = 164, .external_lex_state = 36},
  [5419] = {.lex_state = 164, .external_lex_state = 36},
  [5420] = {.lex_state = 164, .external_lex_state = 36},
  [5421] = {.lex_state = 164, .external_lex_state = 36},
  [5422] = {.lex_state = 164, .external_lex_state = 36},
  [5423] = {.lex_state = 151, .external_lex_state = 35},
  [5424] = {.lex_state = 164, .external_lex_state = 36},
  [5425] = {.lex_state = 164, .external_lex_state = 36},
  [5426] = {.lex_state = 164, .external_lex_state = 36},
  [5427] = {.lex_state = 164, .external_lex_state = 36},
  [5428] = {.lex_state = 164, .external_lex_state = 36},
  [5429] = {.lex_state = 164, .external_lex_state = 36},
  [5430] = {.lex_state = 164, .external_lex_state = 36},
  [5431] = {.lex_state = 164, .external_lex_state = 36},
  [5432] = {.lex_state = 164, .external_lex_state = 36},
  [5433] = {.lex_state = 151, .external_lex_state = 32},
  [5434] = {.lex_state = 164, .external_lex_state = 36},
  [5435] = {.lex_state = 160, .external_lex_state = 35},
  [5436] = {.lex_state = 164, .external_lex_state = 36},
  [5437] = {.lex_state = 160, .external_lex_state = 35},
  [5438] = {.lex_state = 164, .external_lex_state = 36},
  [5439] = {.lex_state = 164, .external_lex_state = 36},
  [5440] = {.lex_state = 160, .external_lex_state = 35},
  [5441] = {.lex_state = 160, .external_lex_state = 32},
  [5442] = {.lex_state = 151, .external_lex_state = 35},
  [5443] = {.lex_state = 160, .external_lex_state = 35},
  [5444] = {.lex_state = 164, .external_lex_state = 36},
  [5445] = {.lex_state = 164, .external_lex_state = 36},
  [5446] = {.lex_state = 164, .external_lex_state = 36},
  [5447] = {.lex_state = 164, .external_lex_state = 36},
  [5448] = {.lex_state = 160, .external_lex_state = 35},
  [5449] = {.lex_state = 151, .external_lex_state = 35},
  [5450] = {.lex_state = 160, .external_lex_state = 35},
  [5451] = {.lex_state = 164, .external_lex_state = 36},
  [5452] = {.lex_state = 164, .external_lex_state = 36},
  [5453] = {.lex_state = 164, .external_lex_state = 36},
  [5454] = {.lex_state = 160, .external_lex_state = 35},
  [5455] = {.lex_state = 164, .external_lex_state = 36},
  [5456] = {.lex_state = 164, .external_lex_state = 36},
  [5457] = {.lex_state = 164, .external_lex_state = 36},
  [5458] = {.lex_state = 164, .external_lex_state = 36},
  [5459] = {.lex_state = 164, .external_lex_state = 36},
  [5460] = {.lex_state = 164, .external_lex_state = 36},
  [5461] = {.lex_state = 49, .external_lex_state = 35},
  [5462] = {.lex_state = 164, .external_lex_state = 36},
  [5463] = {.lex_state = 164, .external_lex_state = 36},
  [5464] = {.lex_state = 164, .external_lex_state = 36},
  [5465] = {.lex_state = 164, .external_lex_state = 36},
  [5466] = {.lex_state = 164, .external_lex_state = 36},
  [5467] = {.lex_state = 151, .external_lex_state = 35},
  [5468] = {.lex_state = 160, .external_lex_state = 35},
  [5469] = {.lex_state = 160, .external_lex_state = 35},
  [5470] = {.lex_state = 164, .external_lex_state = 36},
  [5471] = {.lex_state = 164, .external_lex_state = 36},
  [5472] = {.lex_state = 164, .external_lex_state = 36},
  [5473] = {.lex_state = 164, .external_lex_state = 36},
  [5474] = {.lex_state = 164, .external_lex_state = 36},
  [5475] = {.lex_state = 164, .external_lex_state = 36},
  [5476] = {.lex_state = 160, .external_lex_state = 35},
  [5477] = {.lex_state = 164, .external_lex_state = 36},
  [5478] = {.lex_state = 164, .external_lex_state = 36},
  [5479] = {.lex_state = 164, .external_lex_state = 36},
  [5480] = {.lex_state = 151, .external_lex_state = 32},
  [5481] = {.lex_state = 164, .external_lex_state = 36},
  [5482] = {.lex_state = 160, .external_lex_state = 35},
  [5483] = {.lex_state = 164, .external_lex_state = 36},
  [5484] = {.lex_state = 160, .external_lex_state = 35},
  [5485] = {.lex_state = 164, .external_lex_state = 36},
  [5486] = {.lex_state = 164, .external_lex_state = 36},
  [5487] = {.lex_state = 164, .external_lex_state = 36},
  [5488] = {.lex_state = 164, .external_lex_state = 36},
  [5489] = {.lex_state = 151, .external_lex_state = 35},
  [5490] = {.lex_state = 164, .external_lex_state = 36},
  [5491] = {.lex_state = 164, .external_lex_state = 36},
  [5492] = {.lex_state = 164, .external_lex_state = 36},
  [5493] = {.lex_state = 164, .external_lex_state = 36},
  [5494] = {.lex_state = 164, .external_lex_state = 36},
  [5495] = {.lex_state = 151, .external_lex_state = 35},
  [5496] = {.lex_state = 151, .external_lex_state = 32},
  [5497] = {.lex_state = 54, .external_lex_state = 41},
  [5498] = {.lex_state = 54, .external_lex_state = 41},
  [5499] = {.lex_state = 151, .external_lex_state = 35},
  [5500] = {.lex_state = 54, .external_lex_state = 41},
  [5501] = {.lex_state = 151, .external_lex_state = 35},
  [5502] = {.lex_state = 151, .external_lex_state = 35},
  [5503] = {.lex_state = 151, .external_lex_state = 35},
  [5504] = {.lex_state = 54, .external_lex_state = 41},
  [5505] = {.lex_state = 160, .external_lex_state = 35},
  [5506] = {.lex_state = 151, .external_lex_state = 35},
  [5507] = {.lex_state = 160, .external_lex_state = 35},
  [5508] = {.lex_state = 151, .external_lex_state = 35},
  [5509] = {.lex_state = 151, .external_lex_state = 35},
  [5510] = {.lex_state = 151, .external_lex_state = 35},
  [5511] = {.lex_state = 160, .external_lex_state = 35},
  [5512] = {.lex_state = 54, .external_lex_state = 41},
  [5513] = {.lex_state = 54, .external_lex_state = 41},
  [5514] = {.lex_state = 151, .external_lex_state = 32},
  [5515] = {.lex_state = 151, .external_lex_state = 35},
  [5516] = {.lex_state = 151, .external_lex_state = 32},
  [5517] = {.lex_state = 54, .external_lex_state = 41},
  [5518] = {.lex_state = 160, .external_lex_state = 35},
  [5519] = {.lex_state = 151, .external_lex_state = 35},
  [5520] = {.lex_state = 54, .external_lex_state = 41},
  [5521] = {.lex_state = 54, .external_lex_state = 41},
  [5522] = {.lex_state = 151, .external_lex_state = 35},
  [5523] = {.lex_state = 54, .external_lex_state = 41},
  [5524] = {.lex_state = 54, .external_lex_state = 41},
  [5525] = {.lex_state = 151, .external_lex_state = 35},
  [5526] = {.lex_state = 151, .external_lex_state = 35},
  [5527] = {.lex_state = 151, .external_lex_state = 32},
  [5528] = {.lex_state = 151, .external_lex_state = 32},
  [5529] = {.lex_state = 151, .external_lex_state = 35},
  [5530] = {.lex_state = 151, .external_lex_state = 35},
  [5531] = {.lex_state = 151, .external_lex_state = 35},
  [5532] = {.lex_state = 160, .external_lex_state = 35},
  [5533] = {.lex_state = 151, .external_lex_state = 32},
  [5534] = {.lex_state = 151, .external_lex_state = 35},
  [5535] = {.lex_state = 151, .external_lex_state = 35},
  [5536] = {.lex_state = 151, .external_lex_state = 35},
  [5537] = {.lex_state = 151, .external_lex_state = 35},
  [5538] = {.lex_state = 151, .external_lex_state = 35},
  [5539] = {.lex_state = 160, .external_lex_state = 35},
  [5540] = {.lex_state = 54, .external_lex_state = 41},
  [5541] = {.lex_state = 54, .external_lex_state = 41},
  [5542] = {.lex_state = 151, .external_lex_state = 32},
  [5543] = {.lex_state = 160, .external_lex_state = 32},
  [5544] = {.lex_state = 160, .external_lex_state = 35},
  [5545] = {.lex_state = 151, .external_lex_state = 32},
  [5546] = {.lex_state = 54, .external_lex_state = 41},
  [5547] = {.lex_state = 151, .external_lex_state = 35},
  [5548] = {.lex_state = 151, .external_lex_state = 35},
  [5549] = {.lex_state = 151, .external_lex_state = 35},
  [5550] = {.lex_state = 151, .external_lex_state = 32},
  [5551] = {.lex_state = 54, .external_lex_state = 41},
  [5552] = {.lex_state = 151, .external_lex_state = 32},
  [5553] = {.lex_state = 151, .external_lex_state = 32},
  [5554] = {.lex_state = 160, .external_lex_state = 35},
  [5555] = {.lex_state = 151, .external_lex_state = 32},
  [5556] = {.lex_state = 160, .external_lex_state = 35},
  [5557] = {.lex_state = 160, .external_lex_state = 35},
  [5558] = {.lex_state = 151, .external_lex_state = 32},
  [5559] = {.lex_state = 151, .external_lex_state = 35},
  [5560] = {.lex_state = 160, .external_lex_state = 35},
  [5561] = {.lex_state = 151, .external_lex_state = 35},
  [5562] = {.lex_state = 54, .external_lex_state = 41},
  [5563] = {.lex_state = 54, .external_lex_state = 41},
  [5564] = {.lex_state = 151, .external_lex_state = 32},
  [5565] = {.lex_state = 151, .external_lex_state = 32},
  [5566] = {.lex_state = 151, .external_lex_state = 35},
  [5567] = {.lex_state = 151, .external_lex_state = 35},
  [5568] = {.lex_state = 54, .external_lex_state = 41},
  [5569] = {.lex_state = 54, .external_lex_state = 41},
  [5570] = {.lex_state = 151, .external_lex_state = 35},
  [5571] = {.lex_state = 160, .external_lex_state = 35},
  [5572] = {.lex_state = 54, .external_lex_state = 41},
  [5573] = {.lex_state = 54, .external_lex_state = 41},
  [5574] = {.lex_state = 54, .external_lex_state = 41},
  [5575] = {.lex_state = 151, .external_lex_state = 35},
  [5576] = {.lex_state = 160, .external_lex_state = 35},
  [5577] = {.lex_state = 168, .external_lex_state = 35},
  [5578] = {.lex_state = 168, .external_lex_state = 35},
  [5579] = {.lex_state = 168, .external_lex_state = 35},
  [5580] = {.lex_state = 168, .external_lex_state = 35},
  [5581] = {.lex_state = 168, .external_lex_state = 35},
  [5582] = {.lex_state = 168, .external_lex_state = 35},
  [5583] = {.lex_state = 168, .external_lex_state = 35},
  [5584] = {.lex_state = 168, .external_lex_state = 35},
  [5585] = {.lex_state = 151, .external_lex_state = 35},
  [5586] = {.lex_state = 168, .external_lex_state = 35},
  [5587] = {.lex_state = 168, .external_lex_state = 35},
  [5588] = {.lex_state = 160, .external_lex_state = 35},
  [5589] = {.lex_state = 168, .external_lex_state = 35},
  [5590] = {.lex_state = 168, .external_lex_state = 35},
  [5591] = {.lex_state = 168, .external_lex_state = 35},
  [5592] = {.lex_state = 168, .external_lex_state = 35},
  [5593] = {.lex_state = 160, .external_lex_state = 35},
  [5594] = {.lex_state = 168, .external_lex_state = 35},
  [5595] = {.lex_state = 168, .external_lex_state = 35},
  [5596] = {.lex_state = 168, .external_lex_state = 35},
  [5597] = {.lex_state = 168, .external_lex_state = 35},
  [5598] = {.lex_state = 160, .external_lex_state = 35},
  [5599] = {.lex_state = 168, .external_lex_state = 35},
  [5600] = {.lex_state = 168, .external_lex_state = 35},
  [5601] = {.lex_state = 168, .external_lex_state = 35},
  [5602] = {.lex_state = 168, .external_lex_state = 35},
  [5603] = {.lex_state = 168, .external_lex_state = 35},
  [5604] = {.lex_state = 168, .external_lex_state = 35},
  [5605] = {.lex_state = 160, .external_lex_state = 35},
  [5606] = {.lex_state = 41, .external_lex_state = 35},
  [5607] = {.lex_state = 168, .external_lex_state = 35},
  [5608] = {.lex_state = 151, .external_lex_state = 35},
  [5609] = {.lex_state = 168, .external_lex_state = 35},
  [5610] = {.lex_state = 168, .external_lex_state = 35},
  [5611] = {.lex_state = 168, .external_lex_state = 35},
  [5612] = {.lex_state = 168, .external_lex_state = 35},
  [5613] = {.lex_state = 151, .external_lex_state = 35},
  [5614] = {.lex_state = 168, .external_lex_state = 35},
  [5615] = {.lex_state = 168, .external_lex_state = 35},
  [5616] = {.lex_state = 168, .external_lex_state = 35},
  [5617] = {.lex_state = 168, .external_lex_state = 35},
  [5618] = {.lex_state = 151, .external_lex_state = 35},
  [5619] = {.lex_state = 168, .external_lex_state = 35},
  [5620] = {.lex_state = 168, .external_lex_state = 35},
  [5621] = {.lex_state = 168, .external_lex_state = 35},
  [5622] = {.lex_state = 168, .external_lex_state = 35},
  [5623] = {.lex_state = 168, .external_lex_state = 35},
  [5624] = {.lex_state = 168, .external_lex_state = 35},
  [5625] = {.lex_state = 151, .external_lex_state = 35},
  [5626] = {.lex_state = 168, .external_lex_state = 35},
  [5627] = {.lex_state = 151, .external_lex_state = 35},
  [5628] = {.lex_state = 168, .external_lex_state = 35},
  [5629] = {.lex_state = 168, .external_lex_state = 35},
  [5630] = {.lex_state = 51, .external_lex_state = 35},
  [5631] = {.lex_state = 168, .external_lex_state = 35},
  [5632] = {.lex_state = 168, .external_lex_state = 35},
  [5633] = {.lex_state = 151, .external_lex_state = 35},
  [5634] = {.lex_state = 168, .external_lex_state = 35},
  [5635] = {.lex_state = 168, .external_lex_state = 35},
  [5636] = {.lex_state = 168, .external_lex_state = 35},
  [5637] = {.lex_state = 168, .external_lex_state = 35},
  [5638] = {.lex_state = 151, .external_lex_state = 35},
  [5639] = {.lex_state = 168, .external_lex_state = 35},
  [5640] = {.lex_state = 160, .external_lex_state = 35},
  [5641] = {.lex_state = 151, .external_lex_state = 35},
  [5642] = {.lex_state = 160, .external_lex_state = 35},
  [5643] = {.lex_state = 160, .external_lex_state = 35},
  [5644] = {.lex_state = 41, .external_lex_state = 35},
  [5645] = {.lex_state = 160, .external_lex_state = 35},
  [5646] = {.lex_state = 160, .external_lex_state = 35},
  [5647] = {.lex_state = 160, .external_lex_state = 35},
  [5648] = {.lex_state = 160, .external_lex_state = 35},
  [5649] = {.lex_state = 41, .external_lex_state = 35},
  [5650] = {.lex_state = 160, .external_lex_state = 35},
  [5651] = {.lex_state = 41, .external_lex_state = 35},
  [5652] = {.lex_state = 151, .external_lex_state = 35},
  [5653] = {.lex_state = 160, .external_lex_state = 35},
  [5654] = {.lex_state = 160, .external_lex_state = 35},
  [5655] = {.lex_state = 160, .external_lex_state = 35},
  [5656] = {.lex_state = 160, .external_lex_state = 35},
  [5657] = {.lex_state = 32, .external_lex_state = 35},
  [5658] = {.lex_state = 41, .external_lex_state = 35},
  [5659] = {.lex_state = 41, .external_lex_state = 35},
  [5660] = {.lex_state = 151, .external_lex_state = 35},
  [5661] = {.lex_state = 151, .external_lex_state = 35},
  [5662] = {.lex_state = 151, .external_lex_state = 35},
  [5663] = {.lex_state = 160, .external_lex_state = 35},
  [5664] = {.lex_state = 32, .external_lex_state = 35},
  [5665] = {.lex_state = 160, .external_lex_state = 35},
  [5666] = {.lex_state = 160, .external_lex_state = 35},
  [5667] = {.lex_state = 160, .external_lex_state = 35},
  [5668] = {.lex_state = 41, .external_lex_state = 35},
  [5669] = {.lex_state = 160, .external_lex_state = 35},
  [5670] = {.lex_state = 41, .external_lex_state = 35},
  [5671] = {.lex_state = 151, .external_lex_state = 41},
  [5672] = {.lex_state = 160, .external_lex_state = 35},
  [5673] = {.lex_state = 160, .external_lex_state = 35},
  [5674] = {.lex_state = 160, .external_lex_state = 35},
  [5675] = {.lex_state = 160, .external_lex_state = 35},
  [5676] = {.lex_state = 41, .external_lex_state = 35},
  [5677] = {.lex_state = 160, .external_lex_state = 35},
  [5678] = {.lex_state = 160, .external_lex_state = 35},
  [5679] = {.lex_state = 32, .external_lex_state = 35},
  [5680] = {.lex_state = 41, .external_lex_state = 35},
  [5681] = {.lex_state = 160, .external_lex_state = 35},
  [5682] = {.lex_state = 151, .external_lex_state = 35},
  [5683] = {.lex_state = 151, .external_lex_state = 35},
  [5684] = {.lex_state = 160, .external_lex_state = 35},
  [5685] = {.lex_state = 41, .external_lex_state = 35},
  [5686] = {.lex_state = 151, .external_lex_state = 35},
  [5687] = {.lex_state = 160, .external_lex_state = 35},
  [5688] = {.lex_state = 151, .external_lex_state = 35},
  [5689] = {.lex_state = 160, .external_lex_state = 35},
  [5690] = {.lex_state = 160, .external_lex_state = 35},
  [5691] = {.lex_state = 160, .external_lex_state = 35},
  [5692] = {.lex_state = 41, .external_lex_state = 35},
  [5693] = {.lex_state = 151, .external_lex_state = 35},
  [5694] = {.lex_state = 151, .external_lex_state = 35},
  [5695] = {.lex_state = 160, .external_lex_state = 35},
  [5696] = {.lex_state = 160, .external_lex_state = 35},
  [5697] = {.lex_state = 41, .external_lex_state = 35},
  [5698] = {.lex_state = 41, .external_lex_state = 35},
  [5699] = {.lex_state = 41, .external_lex_state = 35},
  [5700] = {.lex_state = 151, .external_lex_state = 35},
  [5701] = {.lex_state = 151, .external_lex_state = 35},
  [5702] = {.lex_state = 160, .external_lex_state = 35},
  [5703] = {.lex_state = 151, .external_lex_state = 35},
  [5704] = {.lex_state = 160, .external_lex_state = 35},
  [5705] = {.lex_state = 41, .external_lex_state = 35},
  [5706] = {.lex_state = 160, .external_lex_state = 35},
  [5707] = {.lex_state = 41, .external_lex_state = 35},
  [5708] = {.lex_state = 160, .external_lex_state = 35},
  [5709] = {.lex_state = 151, .external_lex_state = 35},
  [5710] = {.lex_state = 160, .external_lex_state = 35},
  [5711] = {.lex_state = 32, .external_lex_state = 35},
  [5712] = {.lex_state = 41, .external_lex_state = 35},
  [5713] = {.lex_state = 160, .external_lex_state = 35},
  [5714] = {.lex_state = 32, .external_lex_state = 35},
  [5715] = {.lex_state = 151, .external_lex_state = 35},
  [5716] = {.lex_state = 41, .external_lex_state = 35},
  [5717] = {.lex_state = 41, .external_lex_state = 35},
  [5718] = {.lex_state = 160, .external_lex_state = 35},
  [5719] = {.lex_state = 151, .external_lex_state = 35},
  [5720] = {.lex_state = 160, .external_lex_state = 35},
  [5721] = {.lex_state = 160, .external_lex_state = 35},
  [5722] = {.lex_state = 41, .external_lex_state = 35},
  [5723] = {.lex_state = 32, .external_lex_state = 35},
  [5724] = {.lex_state = 41, .external_lex_state = 35},
  [5725] = {.lex_state = 151, .external_lex_state = 35},
  [5726] = {.lex_state = 41, .external_lex_state = 35},
  [5727] = {.lex_state = 41, .external_lex_state = 35},
  [5728] = {.lex_state = 160, .external_lex_state = 35},
  [5729] = {.lex_state = 41, .external_lex_state = 35},
  [5730] = {.lex_state = 160, .external_lex_state = 35},
  [5731] = {.lex_state = 41, .external_lex_state = 35},
  [5732] = {.lex_state = 160, .external_lex_state = 35},
  [5733] = {.lex_state = 160, .external_lex_state = 35},
  [5734] = {.lex_state = 160, .external_lex_state = 35},
  [5735] = {.lex_state = 160, .external_lex_state = 35},
  [5736] = {.lex_state = 151, .external_lex_state = 35},
  [5737] = {.lex_state = 41, .external_lex_state = 35},
  [5738] = {.lex_state = 41, .external_lex_state = 35},
  [5739] = {.lex_state = 160, .external_lex_state = 35},
  [5740] = {.lex_state = 160, .external_lex_state = 35},
  [5741] = {.lex_state = 32, .external_lex_state = 35},
  [5742] = {.lex_state = 41, .external_lex_state = 35},
  [5743] = {.lex_state = 160, .external_lex_state = 35},
  [5744] = {.lex_state = 160, .external_lex_state = 35},
  [5745] = {.lex_state = 151, .external_lex_state = 41},
  [5746] = {.lex_state = 41, .external_lex_state = 35},
  [5747] = {.lex_state = 160, .external_lex_state = 35},
  [5748] = {.lex_state = 151, .external_lex_state = 35},
  [5749] = {.lex_state = 160, .external_lex_state = 35},
  [5750] = {.lex_state = 41, .external_lex_state = 35},
  [5751] = {.lex_state = 41, .external_lex_state = 35},
  [5752] = {.lex_state = 151, .external_lex_state = 41},
  [5753] = {.lex_state = 160, .external_lex_state = 35},
  [5754] = {.lex_state = 41, .external_lex_state = 35},
  [5755] = {.lex_state = 160, .external_lex_state = 35},
  [5756] = {.lex_state = 32, .external_lex_state = 35},
  [5757] = {.lex_state = 41, .external_lex_state = 35},
  [5758] = {.lex_state = 151, .external_lex_state = 35},
  [5759] = {.lex_state = 160, .external_lex_state = 35},
  [5760] = {.lex_state = 151, .external_lex_state = 35},
  [5761] = {.lex_state = 160, .external_lex_state = 35},
  [5762] = {.lex_state = 151, .external_lex_state = 35},
  [5763] = {.lex_state = 151, .external_lex_state = 35},
  [5764] = {.lex_state = 32, .external_lex_state = 35},
  [5765] = {.lex_state = 160, .external_lex_state = 35},
  [5766] = {.lex_state = 41, .external_lex_state = 35},
  [5767] = {.lex_state = 41, .external_lex_state = 35},
  [5768] = {.lex_state = 160, .external_lex_state = 35},
  [5769] = {.lex_state = 151, .external_lex_state = 41},
  [5770] = {.lex_state = 160, .external_lex_state = 35},
  [5771] = {.lex_state = 160, .external_lex_state = 35},
  [5772] = {.lex_state = 151, .external_lex_state = 35},
  [5773] = {.lex_state = 32, .external_lex_state = 35},
  [5774] = {.lex_state = 151, .external_lex_state = 35},
  [5775] = {.lex_state = 151, .external_lex_state = 35},
  [5776] = {.lex_state = 160, .external_lex_state = 35},
  [5777] = {.lex_state = 160, .external_lex_state = 35},
  [5778] = {.lex_state = 160, .external_lex_state = 35},
  [5779] = {.lex_state = 151, .external_lex_state = 35},
  [5780] = {.lex_state = 32, .external_lex_state = 35},
  [5781] = {.lex_state = 160, .external_lex_state = 35},
  [5782] = {.lex_state = 160, .external_lex_state = 35},
  [5783] = {.lex_state = 41, .external_lex_state = 35},
  [5784] = {.lex_state = 41, .external_lex_state = 35},
  [5785] = {.lex_state = 160, .external_lex_state = 35},
  [5786] = {.lex_state = 160, .external_lex_state = 35},
  [5787] = {.lex_state = 160, .external_lex_state = 35},
  [5788] = {.lex_state = 151, .external_lex_state = 35},
  [5789] = {.lex_state = 151, .external_lex_state = 35},
  [5790] = {.lex_state = 151, .external_lex_state = 35},
  [5791] = {.lex_state = 151, .external_lex_state = 41},
  [5792] = {.lex_state = 160, .external_lex_state = 35},
  [5793] = {.lex_state = 151, .external_lex_state = 35},
  [5794] = {.lex_state = 41, .external_lex_state = 35},
  [5795] = {.lex_state = 160, .external_lex_state = 35},
  [5796] = {.lex_state = 41, .external_lex_state = 35},
  [5797] = {.lex_state = 160, .external_lex_state = 35},
  [5798] = {.lex_state = 41, .external_lex_state = 35},
  [5799] = {.lex_state = 160, .external_lex_state = 35},
  [5800] = {.lex_state = 160, .external_lex_state = 35},
  [5801] = {.lex_state = 151, .external_lex_state = 35},
  [5802] = {.lex_state = 160, .external_lex_state = 35},
  [5803] = {.lex_state = 151, .external_lex_state = 35},
  [5804] = {.lex_state = 160, .external_lex_state = 35},
  [5805] = {.lex_state = 160, .external_lex_state = 35},
  [5806] = {.lex_state = 151, .external_lex_state = 35},
  [5807] = {.lex_state = 151, .external_lex_state = 35},
  [5808] = {.lex_state = 160, .external_lex_state = 35},
  [5809] = {.lex_state = 160, .external_lex_state = 35},
  [5810] = {.lex_state = 160, .external_lex_state = 35},
  [5811] = {.lex_state = 41, .external_lex_state = 35},
  [5812] = {.lex_state = 160, .external_lex_state = 35},
  [5813] = {.lex_state = 32, .external_lex_state = 35},
  [5814] = {.lex_state = 151, .external_lex_state = 35},
  [5815] = {.lex_state = 151, .external_lex_state = 35},
  [5816] = {.lex_state = 41, .external_lex_state = 35},
  [5817] = {.lex_state = 41, .external_lex_state = 35},
  [5818] = {.lex_state = 151, .external_lex_state = 41},
  [5819] = {.lex_state = 160, .external_lex_state = 35},
  [5820] = {.lex_state = 160, .external_lex_state = 35},
  [5821] = {.lex_state = 160, .external_lex_state = 35},
  [5822] = {.lex_state = 151, .external_lex_state = 41},
  [5823] = {.lex_state = 160, .external_lex_state = 35},
  [5824] = {.lex_state = 160, .external_lex_state = 35},
  [5825] = {.lex_state = 41, .external_lex_state = 35},
  [5826] = {.lex_state = 151, .external_lex_state = 35},
  [5827] = {.lex_state = 151, .external_lex_state = 35},
  [5828] = {.lex_state = 160, .external_lex_state = 35},
  [5829] = {.lex_state = 151, .external_lex_state = 35},
  [5830] = {.lex_state = 160, .external_lex_state = 35},
  [5831] = {.lex_state = 151, .external_lex_state = 35},
  [5832] = {.lex_state = 32, .external_lex_state = 35},
  [5833] = {.lex_state = 160, .external_lex_state = 35},
  [5834] = {.lex_state = 160, .external_lex_state = 35},
  [5835] = {.lex_state = 41, .external_lex_state = 35},
  [5836] = {.lex_state = 160, .external_lex_state = 35},
  [5837] = {.lex_state = 160, .external_lex_state = 35},
  [5838] = {.lex_state = 160, .external_lex_state = 35},
  [5839] = {.lex_state = 32, .external_lex_state = 35},
  [5840] = {.lex_state = 160, .external_lex_state = 35},
  [5841] = {.lex_state = 32, .external_lex_state = 35},
  [5842] = {.lex_state = 160, .external_lex_state = 35},
  [5843] = {.lex_state = 41, .external_lex_state = 35},
  [5844] = {.lex_state = 41, .external_lex_state = 35},
  [5845] = {.lex_state = 41, .external_lex_state = 35},
  [5846] = {.lex_state = 160, .external_lex_state = 35},
  [5847] = {.lex_state = 160, .external_lex_state = 35},
  [5848] = {.lex_state = 151, .external_lex_state = 35},
  [5849] = {.lex_state = 160, .external_lex_state = 35},
  [5850] = {.lex_state = 41, .external_lex_state = 35},
  [5851] = {.lex_state = 41, .external_lex_state = 35},
  [5852] = {.lex_state = 41, .external_lex_state = 35},
  [5853] = {.lex_state = 41, .external_lex_state = 35},
  [5854] = {.lex_state = 160, .external_lex_state = 35},
  [5855] = {.lex_state = 160, .external_lex_state = 35},
  [5856] = {.lex_state = 151, .external_lex_state = 35},
  [5857] = {.lex_state = 160, .external_lex_state = 35},
  [5858] = {.lex_state = 32, .external_lex_state = 35},
  [5859] = {.lex_state = 160, .external_lex_state = 35},
  [5860] = {.lex_state = 160, .external_lex_state = 35},
  [5861] = {.lex_state = 160, .external_lex_state = 35},
  [5862] = {.lex_state = 32, .external_lex_state = 35},
  [5863] = {.lex_state = 41, .external_lex_state = 35},
  [5864] = {.lex_state = 32, .external_lex_state = 35},
  [5865] = {.lex_state = 41, .external_lex_state = 35},
  [5866] = {.lex_state = 151, .external_lex_state = 35},
  [5867] = {.lex_state = 41, .external_lex_state = 35},
  [5868] = {.lex_state = 160, .external_lex_state = 35},
  [5869] = {.lex_state = 160, .external_lex_state = 35},
  [5870] = {.lex_state = 160, .external_lex_state = 35},
  [5871] = {.lex_state = 160, .external_lex_state = 35},
  [5872] = {.lex_state = 160, .external_lex_state = 35},
  [5873] = {.lex_state = 160, .external_lex_state = 35},
  [5874] = {.lex_state = 32, .external_lex_state = 35},
  [5875] = {.lex_state = 178, .external_lex_state = 35},
  [5876] = {.lex_state = 41, .external_lex_state = 35},
  [5877] = {.lex_state = 41, .external_lex_state = 35},
  [5878] = {.lex_state = 41, .external_lex_state = 35},
  [5879] = {.lex_state = 160, .external_lex_state = 35},
  [5880] = {.lex_state = 160, .external_lex_state = 35},
  [5881] = {.lex_state = 160, .external_lex_state = 35},
  [5882] = {.lex_state = 160, .external_lex_state = 35},
  [5883] = {.lex_state = 151, .external_lex_state = 35},
  [5884] = {.lex_state = 151, .external_lex_state = 41},
  [5885] = {.lex_state = 151, .external_lex_state = 35},
  [5886] = {.lex_state = 160, .external_lex_state = 35},
  [5887] = {.lex_state = 160, .external_lex_state = 35},
  [5888] = {.lex_state = 160, .external_lex_state = 35},
  [5889] = {.lex_state = 160, .external_lex_state = 35},
  [5890] = {.lex_state = 41, .external_lex_state = 35},
  [5891] = {.lex_state = 151, .external_lex_state = 41},
  [5892] = {.lex_state = 32, .external_lex_state = 35},
  [5893] = {.lex_state = 151, .external_lex_state = 35},
  [5894] = {.lex_state = 160, .external_lex_state = 35},
  [5895] = {.lex_state = 41, .external_lex_state = 35},
  [5896] = {.lex_state = 32, .external_lex_state = 35},
  [5897] = {.lex_state = 151, .external_lex_state = 35},
  [5898] = {.lex_state = 160, .external_lex_state = 35},
  [5899] = {.lex_state = 41, .external_lex_state = 35},
  [5900] = {.lex_state = 160, .external_lex_state = 35},
  [5901] = {.lex_state = 160, .external_lex_state = 35},
  [5902] = {.lex_state = 151, .external_lex_state = 35},
  [5903] = {.lex_state = 160, .external_lex_state = 35},
  [5904] = {.lex_state = 160, .external_lex_state = 35},
  [5905] = {.lex_state = 41, .external_lex_state = 35},
  [5906] = {.lex_state = 151, .external_lex_state = 35},
  [5907] = {.lex_state = 41, .external_lex_state = 35},
  [5908] = {.lex_state = 32, .external_lex_state = 35},
  [5909] = {.lex_state = 151, .external_lex_state = 35},
  [5910] = {.lex_state = 151, .external_lex_state = 41},
  [5911] = {.lex_state = 151, .external_lex_state = 35},
  [5912] = {.lex_state = 160, .external_lex_state = 35},
  [5913] = {.lex_state = 160, .external_lex_state = 35},
  [5914] = {.lex_state = 160, .external_lex_state = 35},
  [5915] = {.lex_state = 41, .external_lex_state = 35},
  [5916] = {.lex_state = 160, .external_lex_state = 35},
  [5917] = {.lex_state = 160, .external_lex_state = 35},
  [5918] = {.lex_state = 41, .external_lex_state = 35},
  [5919] = {.lex_state = 160, .external_lex_state = 35},
  [5920] = {.lex_state = 160, .external_lex_state = 35},
  [5921] = {.lex_state = 160, .external_lex_state = 35},
  [5922] = {.lex_state = 160, .external_lex_state = 35},
  [5923] = {.lex_state = 160, .external_lex_state = 35},
  [5924] = {.lex_state = 151, .external_lex_state = 35},
  [5925] = {.lex_state = 41, .external_lex_state = 35},
  [5926] = {.lex_state = 151, .external_lex_state = 35},
  [5927] = {.lex_state = 151, .external_lex_state = 35},
  [5928] = {.lex_state = 151, .external_lex_state = 35},
  [5929] = {.lex_state = 160, .external_lex_state = 35},
  [5930] = {.lex_state = 160, .external_lex_state = 35},
  [5931] = {.lex_state = 160, .external_lex_state = 35},
  [5932] = {.lex_state = 160, .external_lex_state = 35},
  [5933] = {.lex_state = 41, .external_lex_state = 35},
  [5934] = {.lex_state = 151, .external_lex_state = 35},
  [5935] = {.lex_state = 151, .external_lex_state = 35},
  [5936] = {.lex_state = 41, .external_lex_state = 35},
  [5937] = {.lex_state = 160, .external_lex_state = 35},
  [5938] = {.lex_state = 160, .external_lex_state = 35},
  [5939] = {.lex_state = 160, .external_lex_state = 35},
  [5940] = {.lex_state = 160, .external_lex_state = 35},
  [5941] = {.lex_state = 160, .external_lex_state = 35},
  [5942] = {.lex_state = 32, .external_lex_state = 35},
  [5943] = {.lex_state = 160, .external_lex_state = 35},
  [5944] = {.lex_state = 160, .external_lex_state = 35},
  [5945] = {.lex_state = 32, .external_lex_state = 35},
  [5946] = {.lex_state = 160, .external_lex_state = 35},
  [5947] = {.lex_state = 160, .external_lex_state = 35},
  [5948] = {.lex_state = 160, .external_lex_state = 35},
  [5949] = {.lex_state = 160, .external_lex_state = 35},
  [5950] = {.lex_state = 178, .external_lex_state = 35},
  [5951] = {.lex_state = 160, .external_lex_state = 35},
  [5952] = {.lex_state = 160, .external_lex_state = 35},
  [5953] = {.lex_state = 151, .external_lex_state = 35},
  [5954] = {.lex_state = 160, .external_lex_state = 35},
  [5955] = {.lex_state = 160, .external_lex_state = 35},
  [5956] = {.lex_state = 160, .external_lex_state = 35},
  [5957] = {.lex_state = 151, .external_lex_state = 35},
  [5958] = {.lex_state = 160, .external_lex_state = 35},
  [5959] = {.lex_state = 160, .external_lex_state = 35},
  [5960] = {.lex_state = 160, .external_lex_state = 35},
  [5961] = {.lex_state = 160, .external_lex_state = 35},
  [5962] = {.lex_state = 151, .external_lex_state = 35},
  [5963] = {.lex_state = 160, .external_lex_state = 35},
  [5964] = {.lex_state = 160, .external_lex_state = 35},
  [5965] = {.lex_state = 160, .external_lex_state = 35},
  [5966] = {.lex_state = 41, .external_lex_state = 35},
  [5967] = {.lex_state = 160, .external_lex_state = 35},
  [5968] = {.lex_state = 41, .external_lex_state = 35},
  [5969] = {.lex_state = 160, .external_lex_state = 35},
  [5970] = {.lex_state = 160, .external_lex_state = 35},
  [5971] = {.lex_state = 160, .external_lex_state = 35},
  [5972] = {.lex_state = 151, .external_lex_state = 35},
  [5973] = {.lex_state = 160, .external_lex_state = 35},
  [5974] = {.lex_state = 151, .external_lex_state = 35},
  [5975] = {.lex_state = 160, .external_lex_state = 35},
  [5976] = {.lex_state = 160, .external_lex_state = 35},
  [5977] = {.lex_state = 160, .external_lex_state = 35},
  [5978] = {.lex_state = 160, .external_lex_state = 35},
  [5979] = {.lex_state = 160, .external_lex_state = 35},
  [5980] = {.lex_state = 151, .external_lex_state = 35},
  [5981] = {.lex_state = 160, .external_lex_state = 35},
  [5982] = {.lex_state = 160, .external_lex_state = 35},
  [5983] = {.lex_state = 160, .external_lex_state = 35},
  [5984] = {.lex_state = 160, .external_lex_state = 35},
  [5985] = {.lex_state = 160, .external_lex_state = 35},
  [5986] = {.lex_state = 41, .external_lex_state = 35},
  [5987] = {(TSStateId)(-1)},
  [5988] = {(TSStateId)(-1)},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [sym_heredoc_body] = STATE(0),
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [aux_sym_program_token1] = ACTIONS(1),
    [anon_sym_BEGIN] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_END] = ACTIONS(1),
    [anon_sym_def] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_COLON_COLON] = ACTIONS(1),
    [anon_sym_end] = ACTIONS(1),
    [anon_sym_rescue] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_STAR_STAR] = ACTIONS(1),
    [anon_sym_nil] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_class] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_module] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_yield] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_next] = ACTIONS(1),
    [anon_sym_redo] = ACTIONS(1),
    [anon_sym_retry] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_unless] = ACTIONS(1),
    [anon_sym_while] = ACTIONS(1),
    [anon_sym_until] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_in] = ACTIONS(1),
    [anon_sym_do] = ACTIONS(1),
    [anon_sym_case] = ACTIONS(1),
    [anon_sym_when] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_LBRACK2] = ACTIONS(1),
    [anon_sym_LPAREN2] = ACTIONS(1),
    [anon_sym_DOT_DOT] = ACTIONS(1),
    [sym_line] = ACTIONS(1),
    [sym_file] = ACTIONS(1),
    [sym_encoding] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_elsif] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_then] = ACTIONS(1),
    [anon_sym_begin] = ACTIONS(1),
    [anon_sym_ensure] = ACTIONS(1),
    [anon_sym_COLON_COLON2] = ACTIONS(1),
    [anon_sym_AMP_DOT] = ACTIONS(1),
    [anon_sym_PLUS_EQ] = ACTIONS(1),
    [anon_sym_STAR_EQ] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(1),
    [anon_sym_PIPE_EQ] = ACTIONS(1),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(1),
    [anon_sym_AMP_EQ] = ACTIONS(1),
    [anon_sym_PERCENT_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_LT_LT_EQ] = ACTIONS(1),
    [anon_sym_CARET_EQ] = ACTIONS(1),
    [anon_sym_QMARK] = ACTIONS(1),
    [anon_sym_COLON2] = ACTIONS(1),
    [anon_sym_and] = ACTIONS(1),
    [anon_sym_or] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1),
    [anon_sym_LT_EQ_GT] = ACTIONS(1),
    [anon_sym_EQ_TILDE] = ACTIONS(1),
    [anon_sym_BANG_TILDE] = ACTIONS(1),
    [anon_sym_defined_QMARK] = ACTIONS(1),
    [anon_sym_not] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_PLUS_AT] = ACTIONS(1),
    [anon_sym_DASH_AT] = ACTIONS(1),
    [anon_sym_TILDE_AT] = ACTIONS(1),
    [anon_sym_BQUOTE] = ACTIONS(1),
    [anon_sym_EQ2] = ACTIONS(1),
    [anon_sym_undef] = ACTIONS(1),
    [anon_sym_alias] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(1),
    [sym_float] = ACTIONS(1),
    [anon_sym_i] = ACTIONS(1),
    [anon_sym_ri] = ACTIONS(1),
    [anon_sym_r] = ACTIONS(1),
    [sym_super] = ACTIONS(1),
    [sym_self] = ACTIONS(1),
    [sym_true] = ACTIONS(1),
    [sym_false] = ACTIONS(1),
    [sym_constant] = ACTIONS(1),
    [aux_sym_constant_suffix_token1] = ACTIONS(1),
    [sym_instance_variable] = ACTIONS(1),
    [sym_class_variable] = ACTIONS(1),
    [sym_global_variable] = ACTIONS(1),
    [sym_character] = ACTIONS(1),
    [anon_sym_POUND_LBRACE] = ACTIONS(1),
    [sym__line_break] = ACTIONS(1),
    [sym__no_line_break] = ACTIONS(1),
    [sym_simple_symbol] = ACTIONS(1),
    [sym__string_start] = ACTIONS(1),
    [sym__symbol_start] = ACTIONS(1),
    [sym__subshell_start] = ACTIONS(1),
    [sym__regex_start] = ACTIONS(1),
    [sym__string_array_start] = ACTIONS(1),
    [sym__symbol_array_start] = ACTIONS(1),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_string_content] = ACTIONS(1),
    [sym_heredoc_content] = ACTIONS(1),
    [sym__string_end] = ACTIONS(1),
    [sym_heredoc_end] = ACTIONS(1),
    [sym_heredoc_beginning] = ACTIONS(1),
    [sym__block_ampersand] = ACTIONS(1),
    [sym__splat_star] = ACTIONS(1),
    [sym__unary_minus] = ACTIONS(1),
    [sym__unary_minus_num] = ACTIONS(1),
    [sym__binary_minus] = ACTIONS(1),
    [sym__binary_star] = ACTIONS(1),
    [sym__singleton_class_left_angle_left_langle] = ACTIONS(1),
    [sym_hash_key_symbol] = ACTIONS(1),
    [sym__identifier_suffix] = ACTIONS(1),
    [sym__constant_suffix] = ACTIONS(1),
    [sym__hash_splat_star_star] = ACTIONS(1),
    [sym__binary_star_star] = ACTIONS(1),
    [sym__element_reference_bracket] = ACTIONS(1),
    [sym__short_interpolation] = ACTIONS(1),
  },
  [1] = {
    [sym_program] = STATE(5883),
    [sym__statements] = STATE(5593),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4861),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4770),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2144),
    [aux_sym__statements_repeat1] = STATE(346),
    [ts_builtin_sym_end] = ACTIONS(7),
    [sym_identifier] = ACTIONS(9),
    [aux_sym_program_token1] = ACTIONS(11),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(25),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [anon_sym_undef] = ACTIONS(75),
    [anon_sym_alias] = ACTIONS(77),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [2] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(2),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_RBRACE] = ACTIONS(121),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(127),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(127),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_elsif] = ACTIONS(127),
    [anon_sym_else] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(127),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [3] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(3),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_RBRACE] = ACTIONS(121),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(127),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(127),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_elsif] = ACTIONS(127),
    [anon_sym_else] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(127),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [4] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(4),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_RBRACE] = ACTIONS(177),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(179),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(179),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_elsif] = ACTIONS(179),
    [anon_sym_else] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(179),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [5] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(5),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_RBRACE] = ACTIONS(121),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(127),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(127),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_elsif] = ACTIONS(127),
    [anon_sym_else] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(127),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [6] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(6),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_RBRACE] = ACTIONS(177),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(179),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(179),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_elsif] = ACTIONS(179),
    [anon_sym_else] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(179),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [7] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4461),
    [sym_argument_list] = STATE(2261),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(7),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_RBRACE] = ACTIONS(177),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(179),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(179),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_elsif] = ACTIONS(179),
    [anon_sym_else] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(179),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [8] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(8),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(121),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(127),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [9] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4461),
    [sym_argument_list] = STATE(2261),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(9),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(177),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(179),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [10] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(10),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(121),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(127),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [11] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(11),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(177),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(179),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [12] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(12),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(121),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(127),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [13] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(13),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(177),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(179),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [14] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(14),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [15] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(15),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [16] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4461),
    [sym_argument_list] = STATE(2261),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(16),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [17] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(17),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [18] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(18),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [19] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(19),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [20] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4461),
    [sym_argument_list] = STATE(2261),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(20),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [21] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4461),
    [sym_argument_list] = STATE(2261),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(21),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(267),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [22] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(22),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [23] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(23),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [24] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4881),
    [sym_argument_list] = STATE(2823),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2937),
    [sym_block] = STATE(2877),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(24),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(271),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(177),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [25] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(25),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(271),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(121),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [26] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(26),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [27] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4845),
    [sym_argument_list] = STATE(3096),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2937),
    [sym_block] = STATE(2877),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(27),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(369),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(177),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [28] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(28),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(267),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [29] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4461),
    [sym_argument_list] = STATE(2261),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(29),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(179),
    [anon_sym_unless] = ACTIONS(179),
    [anon_sym_while] = ACTIONS(179),
    [anon_sym_until] = ACTIONS(179),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [30] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4442),
    [sym_argument_list] = STATE(2255),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(30),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_then] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(177),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [31] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(31),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_then] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [32] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(32),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(127),
    [anon_sym_unless] = ACTIONS(127),
    [anon_sym_while] = ACTIONS(127),
    [anon_sym_until] = ACTIONS(127),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(121),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [33] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4850),
    [sym_argument_list] = STATE(3108),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(33),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(369),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(121),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [34] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(34),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(271),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(121),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [35] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4845),
    [sym_argument_list] = STATE(3096),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2937),
    [sym_block] = STATE(2877),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(35),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(369),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(179),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(177),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [36] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4850),
    [sym_argument_list] = STATE(3108),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(36),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(369),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(127),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(121),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [37] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4881),
    [sym_argument_list] = STATE(2823),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2937),
    [sym_block] = STATE(2877),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(37),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(271),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(177),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [38] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4881),
    [sym_argument_list] = STATE(2823),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2937),
    [sym_block] = STATE(2877),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(38),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(179),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [39] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(39),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(127),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [40] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(40),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(127),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(127),
    [anon_sym_and] = ACTIONS(127),
    [anon_sym_or] = ACTIONS(127),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [41] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4881),
    [sym_argument_list] = STATE(2823),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2937),
    [sym_block] = STATE(2877),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(41),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(179),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(179),
    [anon_sym_and] = ACTIONS(179),
    [anon_sym_or] = ACTIONS(179),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [42] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(42),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_RBRACE] = ACTIONS(409),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(411),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(411),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_elsif] = ACTIONS(411),
    [anon_sym_else] = ACTIONS(411),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(411),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [43] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(43),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_RBRACE] = ACTIONS(409),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(411),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(411),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_elsif] = ACTIONS(411),
    [anon_sym_else] = ACTIONS(411),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(411),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [44] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(44),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_RBRACE] = ACTIONS(409),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(411),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(411),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_elsif] = ACTIONS(411),
    [anon_sym_else] = ACTIONS(411),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(411),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [45] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(45),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(413),
    [anon_sym_RBRACE] = ACTIONS(413),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(413),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(415),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(415),
    [anon_sym_unless] = ACTIONS(415),
    [anon_sym_while] = ACTIONS(415),
    [anon_sym_until] = ACTIONS(415),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_do] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(415),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_elsif] = ACTIONS(415),
    [anon_sym_else] = ACTIONS(415),
    [anon_sym_then] = ACTIONS(415),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(415),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [46] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(46),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(417),
    [anon_sym_RBRACE] = ACTIONS(417),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(417),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(419),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(419),
    [anon_sym_unless] = ACTIONS(419),
    [anon_sym_while] = ACTIONS(419),
    [anon_sym_until] = ACTIONS(419),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_do] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(419),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_elsif] = ACTIONS(419),
    [anon_sym_else] = ACTIONS(419),
    [anon_sym_then] = ACTIONS(419),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(419),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [47] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(47),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(421),
    [anon_sym_RBRACE] = ACTIONS(421),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(421),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(423),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(423),
    [anon_sym_unless] = ACTIONS(423),
    [anon_sym_while] = ACTIONS(423),
    [anon_sym_until] = ACTIONS(423),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_do] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(423),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_elsif] = ACTIONS(423),
    [anon_sym_else] = ACTIONS(423),
    [anon_sym_then] = ACTIONS(423),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(423),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [48] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(48),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(425),
    [anon_sym_RBRACE] = ACTIONS(425),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(425),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(427),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(427),
    [anon_sym_unless] = ACTIONS(427),
    [anon_sym_while] = ACTIONS(427),
    [anon_sym_until] = ACTIONS(427),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_do] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(427),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_elsif] = ACTIONS(427),
    [anon_sym_else] = ACTIONS(427),
    [anon_sym_then] = ACTIONS(427),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(427),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [49] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4565),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(49),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(421),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(421),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(423),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(423),
    [anon_sym_unless] = ACTIONS(423),
    [anon_sym_while] = ACTIONS(423),
    [anon_sym_until] = ACTIONS(423),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(423),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_elsif] = ACTIONS(423),
    [anon_sym_else] = ACTIONS(423),
    [anon_sym_then] = ACTIONS(423),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(423),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(445),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [50] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4565),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(50),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(425),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(425),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(427),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(427),
    [anon_sym_unless] = ACTIONS(427),
    [anon_sym_while] = ACTIONS(427),
    [anon_sym_until] = ACTIONS(427),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(427),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_elsif] = ACTIONS(427),
    [anon_sym_else] = ACTIONS(427),
    [anon_sym_then] = ACTIONS(427),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(427),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(445),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [51] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4565),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(51),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(417),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(417),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(419),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(419),
    [anon_sym_unless] = ACTIONS(419),
    [anon_sym_while] = ACTIONS(419),
    [anon_sym_until] = ACTIONS(419),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(419),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_elsif] = ACTIONS(419),
    [anon_sym_else] = ACTIONS(419),
    [anon_sym_then] = ACTIONS(419),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(419),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(445),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [52] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4565),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(52),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(413),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_RPAREN] = ACTIONS(413),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(415),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(415),
    [anon_sym_unless] = ACTIONS(415),
    [anon_sym_while] = ACTIONS(415),
    [anon_sym_until] = ACTIONS(415),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(415),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_elsif] = ACTIONS(415),
    [anon_sym_else] = ACTIONS(415),
    [anon_sym_then] = ACTIONS(415),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(415),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(445),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [53] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(53),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(409),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(411),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [54] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(54),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(409),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(411),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [55] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(55),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [56] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(56),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [57] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(57),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(409),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(411),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [58] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(58),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(425),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(427),
    [anon_sym_LBRACE] = ACTIONS(425),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(427),
    [anon_sym_unless] = ACTIONS(427),
    [anon_sym_while] = ACTIONS(427),
    [anon_sym_until] = ACTIONS(427),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_do] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [59] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(59),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(421),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(423),
    [anon_sym_LBRACE] = ACTIONS(421),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(423),
    [anon_sym_unless] = ACTIONS(423),
    [anon_sym_while] = ACTIONS(423),
    [anon_sym_until] = ACTIONS(423),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_do] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [60] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(60),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(119),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [61] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(61),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(181),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [62] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(62),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [63] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(63),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(417),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(419),
    [anon_sym_LBRACE] = ACTIONS(417),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(419),
    [anon_sym_unless] = ACTIONS(419),
    [anon_sym_while] = ACTIONS(419),
    [anon_sym_until] = ACTIONS(419),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_do] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [64] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(64),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(413),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(415),
    [anon_sym_LBRACE] = ACTIONS(413),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(415),
    [anon_sym_unless] = ACTIONS(415),
    [anon_sym_while] = ACTIONS(415),
    [anon_sym_until] = ACTIONS(415),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_do] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(199),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [65] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(65),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(271),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(409),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [66] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4850),
    [sym_argument_list] = STATE(3108),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(66),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(369),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(409),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [67] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(67),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(267),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [68] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3242),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4731),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1981),
    [sym_identifier_suffix] = STATE(1986),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2692),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(68),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(425),
    [sym_identifier] = ACTIONS(461),
    [aux_sym_program_token1] = ACTIONS(427),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(427),
    [anon_sym_unless] = ACTIONS(427),
    [anon_sym_while] = ACTIONS(427),
    [anon_sym_until] = ACTIONS(427),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(469),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(461),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(473),
    [sym__splat_star] = ACTIONS(475),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(481),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(483),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [69] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(69),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(421),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_do] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_then] = ACTIONS(423),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [70] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(70),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(271),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(409),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [71] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4464),
    [sym_argument_list] = STATE(2264),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(71),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(411),
    [anon_sym_unless] = ACTIONS(411),
    [anon_sym_while] = ACTIONS(411),
    [anon_sym_until] = ACTIONS(411),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(147),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [72] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(72),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(417),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_do] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_then] = ACTIONS(419),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [73] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(73),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(425),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_do] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_then] = ACTIONS(427),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [74] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(74),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(413),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_do] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_then] = ACTIONS(415),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(227),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [75] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3242),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4731),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1981),
    [sym_identifier_suffix] = STATE(1986),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2692),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(75),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(413),
    [sym_identifier] = ACTIONS(461),
    [aux_sym_program_token1] = ACTIONS(415),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(415),
    [anon_sym_unless] = ACTIONS(415),
    [anon_sym_while] = ACTIONS(415),
    [anon_sym_until] = ACTIONS(415),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(469),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(461),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(473),
    [sym__splat_star] = ACTIONS(475),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(481),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(483),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [76] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4850),
    [sym_argument_list] = STATE(3108),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(76),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(369),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(409),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(411),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(409),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [77] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3242),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4731),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1981),
    [sym_identifier_suffix] = STATE(1986),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2692),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(77),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(417),
    [sym_identifier] = ACTIONS(461),
    [aux_sym_program_token1] = ACTIONS(419),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(419),
    [anon_sym_unless] = ACTIONS(419),
    [anon_sym_while] = ACTIONS(419),
    [anon_sym_until] = ACTIONS(419),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(469),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(461),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(473),
    [sym__splat_star] = ACTIONS(475),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(481),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(483),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [78] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4466),
    [sym_argument_list] = STATE(2257),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(78),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(173),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_SEMI] = ACTIONS(409),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(175),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_then] = ACTIONS(411),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(409),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [79] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3242),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4731),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1981),
    [sym_identifier_suffix] = STATE(1986),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2692),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(79),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(421),
    [sym_identifier] = ACTIONS(461),
    [aux_sym_program_token1] = ACTIONS(423),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(423),
    [anon_sym_unless] = ACTIONS(423),
    [anon_sym_while] = ACTIONS(423),
    [anon_sym_until] = ACTIONS(423),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(469),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(461),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(473),
    [sym__splat_star] = ACTIONS(475),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(481),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(483),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [80] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4947),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(80),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_then] = ACTIONS(427),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(497),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [81] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4947),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(81),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_then] = ACTIONS(415),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(497),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [82] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3457),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4749),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1997),
    [sym_identifier_suffix] = STATE(1999),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2707),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(82),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(503),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(423),
    [anon_sym_unless] = ACTIONS(423),
    [anon_sym_while] = ACTIONS(423),
    [anon_sym_until] = ACTIONS(423),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_do] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(519),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(503),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(523),
    [sym__splat_star] = ACTIONS(525),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(531),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(533),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [83] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3457),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4749),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1997),
    [sym_identifier_suffix] = STATE(1999),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2707),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(83),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(503),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(419),
    [anon_sym_unless] = ACTIONS(419),
    [anon_sym_while] = ACTIONS(419),
    [anon_sym_until] = ACTIONS(419),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_do] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(519),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(503),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(523),
    [sym__splat_star] = ACTIONS(525),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(531),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(533),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [84] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3457),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4749),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1997),
    [sym_identifier_suffix] = STATE(1999),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2707),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(84),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(503),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(415),
    [anon_sym_unless] = ACTIONS(415),
    [anon_sym_while] = ACTIONS(415),
    [anon_sym_until] = ACTIONS(415),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_do] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(519),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(503),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(523),
    [sym__splat_star] = ACTIONS(525),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(531),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(533),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [85] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3457),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4749),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1997),
    [sym_identifier_suffix] = STATE(1999),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2707),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(85),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(503),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(427),
    [anon_sym_unless] = ACTIONS(427),
    [anon_sym_while] = ACTIONS(427),
    [anon_sym_until] = ACTIONS(427),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_do] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(519),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(503),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(523),
    [sym__splat_star] = ACTIONS(525),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(531),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(533),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [86] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4947),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(86),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_then] = ACTIONS(419),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(497),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [87] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(87),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(421),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_do] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_then] = ACTIONS(423),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [88] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(88),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(417),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_do] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_then] = ACTIONS(419),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [89] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(89),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(413),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_do] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_then] = ACTIONS(415),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [90] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(90),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(425),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_do] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_then] = ACTIONS(427),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(249),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [91] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4989),
    [sym_argument_list] = STATE(3007),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(91),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(421),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(421),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_do] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(421),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [92] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5003),
    [sym_argument_list] = STATE(3010),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(92),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(417),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(417),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_do] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(417),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [93] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5005),
    [sym_argument_list] = STATE(3011),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(93),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(413),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(413),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_do] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(413),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [94] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(94),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(411),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [95] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5010),
    [sym_argument_list] = STATE(3012),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(95),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(425),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(425),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_do] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(425),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(321),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [96] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(4947),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(96),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_then] = ACTIONS(423),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(497),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [97] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3957),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5010),
    [sym_argument_list] = STATE(3012),
    [sym__argument] = STATE(5041),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(1994),
    [sym_identifier_suffix] = STATE(1991),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3093),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(97),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(535),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(425),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(427),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(425),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(553),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(535),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(557),
    [sym__splat_star] = ACTIONS(559),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(565),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(567),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [98] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3956),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4513),
    [sym_argument_list] = STATE(2321),
    [sym__argument] = STATE(5184),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1996),
    [sym_identifier_suffix] = STATE(1998),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(3139),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(98),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(569),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_SEMI] = ACTIONS(425),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_then] = ACTIONS(427),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(585),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(569),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(425),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(589),
    [sym__splat_star] = ACTIONS(591),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(597),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(599),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [99] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3957),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5003),
    [sym_argument_list] = STATE(3010),
    [sym__argument] = STATE(5041),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(1994),
    [sym_identifier_suffix] = STATE(1991),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3093),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(99),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(535),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(417),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(419),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(417),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(553),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(535),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(557),
    [sym__splat_star] = ACTIONS(559),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(565),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(567),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [100] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3957),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5005),
    [sym_argument_list] = STATE(3011),
    [sym__argument] = STATE(5041),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(1994),
    [sym_identifier_suffix] = STATE(1991),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3093),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(100),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(535),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(413),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(415),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(413),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(553),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(535),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(557),
    [sym__splat_star] = ACTIONS(559),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(565),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(567),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [101] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3956),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4486),
    [sym_argument_list] = STATE(2317),
    [sym__argument] = STATE(5184),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1996),
    [sym_identifier_suffix] = STATE(1998),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(3139),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(101),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(569),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_SEMI] = ACTIONS(417),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_then] = ACTIONS(419),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(585),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(569),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(417),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(589),
    [sym__splat_star] = ACTIONS(591),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(597),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(599),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [102] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3956),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4492),
    [sym_argument_list] = STATE(2319),
    [sym__argument] = STATE(5184),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1996),
    [sym_identifier_suffix] = STATE(1998),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(3139),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(102),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(569),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_SEMI] = ACTIONS(413),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_then] = ACTIONS(415),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(585),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(569),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(413),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(589),
    [sym__splat_star] = ACTIONS(591),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(597),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(599),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [103] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3957),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4989),
    [sym_argument_list] = STATE(3007),
    [sym__argument] = STATE(5041),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(1994),
    [sym_identifier_suffix] = STATE(1991),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3093),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(103),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(535),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(421),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(423),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(421),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(553),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(535),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(557),
    [sym__splat_star] = ACTIONS(559),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(565),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(567),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [104] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4832),
    [sym_argument_list] = STATE(2818),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_do_block] = STATE(2910),
    [sym_block] = STATE(2911),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(104),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(407),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(409),
    [anon_sym_DOT] = ACTIONS(411),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_PIPE] = ACTIONS(411),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(409),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(411),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(411),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(411),
    [anon_sym_do] = ACTIONS(307),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(409),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(411),
    [anon_sym_CARET] = ACTIONS(409),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(409),
    [anon_sym_AMP_DOT] = ACTIONS(409),
    [anon_sym_QMARK] = ACTIONS(411),
    [anon_sym_and] = ACTIONS(411),
    [anon_sym_or] = ACTIONS(411),
    [anon_sym_PIPE_PIPE] = ACTIONS(409),
    [anon_sym_AMP_AMP] = ACTIONS(409),
    [anon_sym_LT_LT] = ACTIONS(409),
    [anon_sym_GT_GT] = ACTIONS(409),
    [anon_sym_LT_EQ] = ACTIONS(411),
    [anon_sym_GT] = ACTIONS(411),
    [anon_sym_GT_EQ] = ACTIONS(409),
    [anon_sym_PLUS] = ACTIONS(409),
    [anon_sym_SLASH] = ACTIONS(409),
    [anon_sym_PERCENT] = ACTIONS(409),
    [anon_sym_EQ_EQ] = ACTIONS(411),
    [anon_sym_BANG_EQ] = ACTIONS(409),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(409),
    [anon_sym_LT_EQ_GT] = ACTIONS(409),
    [anon_sym_EQ_TILDE] = ACTIONS(409),
    [anon_sym_BANG_TILDE] = ACTIONS(409),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(409),
    [sym__binary_star] = ACTIONS(409),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(409),
    [sym__element_reference_bracket] = ACTIONS(409),
  },
  [105] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3956),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4491),
    [sym_argument_list] = STATE(2294),
    [sym__argument] = STATE(5184),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1996),
    [sym_identifier_suffix] = STATE(1998),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(3139),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(105),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(569),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_SEMI] = ACTIONS(421),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_then] = ACTIONS(423),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(585),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(569),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(421),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(589),
    [sym__splat_star] = ACTIONS(591),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(597),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(599),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [106] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5005),
    [sym_argument_list] = STATE(3011),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(106),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(413),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(413),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_do] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(413),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [107] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5010),
    [sym_argument_list] = STATE(3012),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(107),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(425),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(425),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_do] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(425),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [108] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5003),
    [sym_argument_list] = STATE(3010),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(108),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(417),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(417),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_do] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(417),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [109] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4989),
    [sym_argument_list] = STATE(3007),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(109),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(421),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(421),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_do] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(421),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(389),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [110] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4015),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4989),
    [sym_argument_list] = STATE(3007),
    [sym__argument] = STATE(5214),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2007),
    [sym_identifier_suffix] = STATE(2010),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3198),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(110),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(601),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(421),
    [anon_sym_DOT] = ACTIONS(423),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(421),
    [anon_sym_PIPE] = ACTIONS(423),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(421),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(423),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(423),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(423),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(421),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(421),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(423),
    [anon_sym_CARET] = ACTIONS(421),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(421),
    [anon_sym_AMP_DOT] = ACTIONS(421),
    [anon_sym_QMARK] = ACTIONS(423),
    [anon_sym_and] = ACTIONS(423),
    [anon_sym_or] = ACTIONS(423),
    [anon_sym_PIPE_PIPE] = ACTIONS(421),
    [anon_sym_AMP_AMP] = ACTIONS(421),
    [anon_sym_LT_LT] = ACTIONS(421),
    [anon_sym_GT_GT] = ACTIONS(421),
    [anon_sym_LT_EQ] = ACTIONS(423),
    [anon_sym_GT] = ACTIONS(423),
    [anon_sym_GT_EQ] = ACTIONS(421),
    [anon_sym_PLUS] = ACTIONS(421),
    [anon_sym_SLASH] = ACTIONS(421),
    [anon_sym_PERCENT] = ACTIONS(421),
    [anon_sym_EQ_EQ] = ACTIONS(423),
    [anon_sym_BANG_EQ] = ACTIONS(421),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(421),
    [anon_sym_LT_EQ_GT] = ACTIONS(421),
    [anon_sym_EQ_TILDE] = ACTIONS(421),
    [anon_sym_BANG_TILDE] = ACTIONS(421),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(617),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(601),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(621),
    [sym__splat_star] = ACTIONS(623),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(421),
    [sym__binary_star] = ACTIONS(421),
    [sym_hash_key_symbol] = ACTIONS(629),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(631),
    [sym__binary_star_star] = ACTIONS(421),
    [sym__element_reference_bracket] = ACTIONS(421),
  },
  [111] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4015),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5003),
    [sym_argument_list] = STATE(3010),
    [sym__argument] = STATE(5214),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2007),
    [sym_identifier_suffix] = STATE(2010),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3198),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(111),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(601),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(417),
    [anon_sym_DOT] = ACTIONS(419),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(417),
    [anon_sym_PIPE] = ACTIONS(419),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(417),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(419),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(419),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(419),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(417),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(417),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(419),
    [anon_sym_CARET] = ACTIONS(417),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(417),
    [anon_sym_AMP_DOT] = ACTIONS(417),
    [anon_sym_QMARK] = ACTIONS(419),
    [anon_sym_and] = ACTIONS(419),
    [anon_sym_or] = ACTIONS(419),
    [anon_sym_PIPE_PIPE] = ACTIONS(417),
    [anon_sym_AMP_AMP] = ACTIONS(417),
    [anon_sym_LT_LT] = ACTIONS(417),
    [anon_sym_GT_GT] = ACTIONS(417),
    [anon_sym_LT_EQ] = ACTIONS(419),
    [anon_sym_GT] = ACTIONS(419),
    [anon_sym_GT_EQ] = ACTIONS(417),
    [anon_sym_PLUS] = ACTIONS(417),
    [anon_sym_SLASH] = ACTIONS(417),
    [anon_sym_PERCENT] = ACTIONS(417),
    [anon_sym_EQ_EQ] = ACTIONS(419),
    [anon_sym_BANG_EQ] = ACTIONS(417),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(417),
    [anon_sym_LT_EQ_GT] = ACTIONS(417),
    [anon_sym_EQ_TILDE] = ACTIONS(417),
    [anon_sym_BANG_TILDE] = ACTIONS(417),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(617),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(601),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(621),
    [sym__splat_star] = ACTIONS(623),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(417),
    [sym__binary_star] = ACTIONS(417),
    [sym_hash_key_symbol] = ACTIONS(629),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(631),
    [sym__binary_star_star] = ACTIONS(417),
    [sym__element_reference_bracket] = ACTIONS(417),
  },
  [112] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4015),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5005),
    [sym_argument_list] = STATE(3011),
    [sym__argument] = STATE(5214),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2007),
    [sym_identifier_suffix] = STATE(2010),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3198),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(112),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(601),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(413),
    [anon_sym_DOT] = ACTIONS(415),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(413),
    [anon_sym_PIPE] = ACTIONS(415),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(413),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(415),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(415),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(415),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(413),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(413),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(415),
    [anon_sym_CARET] = ACTIONS(413),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(413),
    [anon_sym_AMP_DOT] = ACTIONS(413),
    [anon_sym_QMARK] = ACTIONS(415),
    [anon_sym_and] = ACTIONS(415),
    [anon_sym_or] = ACTIONS(415),
    [anon_sym_PIPE_PIPE] = ACTIONS(413),
    [anon_sym_AMP_AMP] = ACTIONS(413),
    [anon_sym_LT_LT] = ACTIONS(413),
    [anon_sym_GT_GT] = ACTIONS(413),
    [anon_sym_LT_EQ] = ACTIONS(415),
    [anon_sym_GT] = ACTIONS(415),
    [anon_sym_GT_EQ] = ACTIONS(413),
    [anon_sym_PLUS] = ACTIONS(413),
    [anon_sym_SLASH] = ACTIONS(413),
    [anon_sym_PERCENT] = ACTIONS(413),
    [anon_sym_EQ_EQ] = ACTIONS(415),
    [anon_sym_BANG_EQ] = ACTIONS(413),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(413),
    [anon_sym_LT_EQ_GT] = ACTIONS(413),
    [anon_sym_EQ_TILDE] = ACTIONS(413),
    [anon_sym_BANG_TILDE] = ACTIONS(413),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(617),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(601),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(621),
    [sym__splat_star] = ACTIONS(623),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(413),
    [sym__binary_star] = ACTIONS(413),
    [sym_hash_key_symbol] = ACTIONS(629),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(631),
    [sym__binary_star_star] = ACTIONS(413),
    [sym__element_reference_bracket] = ACTIONS(413),
  },
  [113] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4015),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(5010),
    [sym_argument_list] = STATE(3012),
    [sym__argument] = STATE(5214),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2007),
    [sym_identifier_suffix] = STATE(2010),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3198),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(113),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(601),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(275),
    [anon_sym_RPAREN] = ACTIONS(425),
    [anon_sym_DOT] = ACTIONS(427),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(425),
    [anon_sym_PIPE] = ACTIONS(427),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(425),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(427),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(427),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(425),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(425),
    [anon_sym_LPAREN2] = ACTIONS(313),
    [anon_sym_DOT_DOT] = ACTIONS(427),
    [anon_sym_CARET] = ACTIONS(425),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(425),
    [anon_sym_AMP_DOT] = ACTIONS(425),
    [anon_sym_QMARK] = ACTIONS(427),
    [anon_sym_and] = ACTIONS(427),
    [anon_sym_or] = ACTIONS(427),
    [anon_sym_PIPE_PIPE] = ACTIONS(425),
    [anon_sym_AMP_AMP] = ACTIONS(425),
    [anon_sym_LT_LT] = ACTIONS(425),
    [anon_sym_GT_GT] = ACTIONS(425),
    [anon_sym_LT_EQ] = ACTIONS(427),
    [anon_sym_GT] = ACTIONS(427),
    [anon_sym_GT_EQ] = ACTIONS(425),
    [anon_sym_PLUS] = ACTIONS(425),
    [anon_sym_SLASH] = ACTIONS(425),
    [anon_sym_PERCENT] = ACTIONS(425),
    [anon_sym_EQ_EQ] = ACTIONS(427),
    [anon_sym_BANG_EQ] = ACTIONS(425),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(425),
    [anon_sym_LT_EQ_GT] = ACTIONS(425),
    [anon_sym_EQ_TILDE] = ACTIONS(425),
    [anon_sym_BANG_TILDE] = ACTIONS(425),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(617),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(601),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(621),
    [sym__splat_star] = ACTIONS(623),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(425),
    [sym__binary_star] = ACTIONS(425),
    [sym_hash_key_symbol] = ACTIONS(629),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(631),
    [sym__binary_star_star] = ACTIONS(425),
    [sym__element_reference_bracket] = ACTIONS(425),
  },
  [114] = {
    [sym__statements] = STATE(5866),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_expr] = STATE(5585),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(3171),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5606),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2219),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2811),
    [sym_subshell] = STATE(3171),
    [sym_string_array] = STATE(3171),
    [sym_symbol_array] = STATE(3171),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(3171),
    [sym_heredoc_body] = STATE(114),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(3079),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(633),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(635),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(639),
    [anon_sym_RPAREN] = ACTIONS(641),
    [anon_sym_COLON_COLON] = ACTIONS(643),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(647),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(649),
    [anon_sym_DOT_DOT] = ACTIONS(651),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(669),
    [sym_true] = ACTIONS(671),
    [sym_false] = ACTIONS(671),
    [sym_constant] = ACTIONS(673),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(675),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [115] = {
    [sym__statements] = STATE(5866),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_expr] = STATE(5625),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(3171),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5606),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2219),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2811),
    [sym_subshell] = STATE(3171),
    [sym_string_array] = STATE(3171),
    [sym_symbol_array] = STATE(3171),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(3171),
    [sym_heredoc_body] = STATE(115),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(3079),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(633),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(635),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(639),
    [anon_sym_RPAREN] = ACTIONS(641),
    [anon_sym_COLON_COLON] = ACTIONS(643),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(647),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(649),
    [anon_sym_DOT_DOT] = ACTIONS(651),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(669),
    [sym_true] = ACTIONS(671),
    [sym_false] = ACTIONS(671),
    [sym_constant] = ACTIONS(673),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(675),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [116] = {
    [sym__statements] = STATE(5760),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_expr] = STATE(5585),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(3171),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5606),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2219),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2811),
    [sym_subshell] = STATE(3171),
    [sym_string_array] = STATE(3171),
    [sym_symbol_array] = STATE(3171),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(3171),
    [sym_heredoc_body] = STATE(116),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(3079),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(633),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(635),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(639),
    [anon_sym_RPAREN] = ACTIONS(683),
    [anon_sym_COLON_COLON] = ACTIONS(643),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(647),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(649),
    [anon_sym_DOT_DOT] = ACTIONS(651),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(669),
    [sym_true] = ACTIONS(671),
    [sym_false] = ACTIONS(671),
    [sym_constant] = ACTIONS(673),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(675),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [117] = {
    [sym__statements] = STATE(5760),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_expr] = STATE(5625),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(3171),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5606),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2219),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2811),
    [sym_subshell] = STATE(3171),
    [sym_string_array] = STATE(3171),
    [sym_symbol_array] = STATE(3171),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(3171),
    [sym_heredoc_body] = STATE(117),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(3079),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(633),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(635),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(639),
    [anon_sym_RPAREN] = ACTIONS(683),
    [anon_sym_COLON_COLON] = ACTIONS(643),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(647),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(649),
    [anon_sym_DOT_DOT] = ACTIONS(651),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(669),
    [sym_true] = ACTIONS(671),
    [sym_false] = ACTIONS(671),
    [sym_constant] = ACTIONS(673),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(675),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [118] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5758),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5763),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5926),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3729),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2540),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3282),
    [sym_subshell] = STATE(3729),
    [sym_string_array] = STATE(3729),
    [sym_symbol_array] = STATE(3729),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3729),
    [sym_heredoc_body] = STATE(118),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3731),
    [sym_identifier] = ACTIONS(685),
    [anon_sym_LBRACE] = ACTIONS(687),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(691),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(693),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(697),
    [anon_sym_RBRACK] = ACTIONS(699),
    [anon_sym_DOT_DOT] = ACTIONS(701),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(705),
    [sym_true] = ACTIONS(707),
    [sym_false] = ACTIONS(707),
    [sym_constant] = ACTIONS(709),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(711),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [119] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5909),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5774),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5926),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3729),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2540),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3282),
    [sym_subshell] = STATE(3729),
    [sym_string_array] = STATE(3729),
    [sym_symbol_array] = STATE(3729),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3729),
    [sym_heredoc_body] = STATE(119),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3731),
    [sym_identifier] = ACTIONS(685),
    [anon_sym_LBRACE] = ACTIONS(687),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(691),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(693),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(697),
    [anon_sym_RBRACK] = ACTIONS(723),
    [anon_sym_DOT_DOT] = ACTIONS(701),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(705),
    [sym_true] = ACTIONS(707),
    [sym_false] = ACTIONS(707),
    [sym_constant] = ACTIONS(709),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(711),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [120] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5758),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5763),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5827),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3729),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2540),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3282),
    [sym_subshell] = STATE(3729),
    [sym_string_array] = STATE(3729),
    [sym_symbol_array] = STATE(3729),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3729),
    [sym_heredoc_body] = STATE(120),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3731),
    [sym_identifier] = ACTIONS(685),
    [anon_sym_LBRACE] = ACTIONS(687),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(691),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(693),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(697),
    [anon_sym_RBRACK] = ACTIONS(725),
    [anon_sym_DOT_DOT] = ACTIONS(701),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(705),
    [sym_true] = ACTIONS(707),
    [sym_false] = ACTIONS(707),
    [sym_constant] = ACTIONS(709),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(711),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [121] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5909),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5774),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5827),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3729),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2540),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3282),
    [sym_subshell] = STATE(3729),
    [sym_string_array] = STATE(3729),
    [sym_symbol_array] = STATE(3729),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3729),
    [sym_heredoc_body] = STATE(121),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3731),
    [sym_identifier] = ACTIONS(685),
    [anon_sym_LBRACE] = ACTIONS(687),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(691),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(693),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(697),
    [anon_sym_RBRACK] = ACTIONS(727),
    [anon_sym_DOT_DOT] = ACTIONS(701),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(705),
    [sym_true] = ACTIONS(707),
    [sym_false] = ACTIONS(707),
    [sym_constant] = ACTIONS(709),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(711),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [122] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5758),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5763),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5926),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3729),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2540),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3282),
    [sym_subshell] = STATE(3729),
    [sym_string_array] = STATE(3729),
    [sym_symbol_array] = STATE(3729),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3729),
    [sym_heredoc_body] = STATE(122),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3731),
    [sym_identifier] = ACTIONS(685),
    [anon_sym_LBRACE] = ACTIONS(687),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(691),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(693),
    [anon_sym_STAR] = ACTIONS(695),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(697),
    [anon_sym_RBRACK] = ACTIONS(729),
    [anon_sym_DOT_DOT] = ACTIONS(701),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(705),
    [sym_true] = ACTIONS(707),
    [sym_false] = ACTIONS(707),
    [sym_constant] = ACTIONS(709),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(711),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [123] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_superclass] = STATE(5528),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5912),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(123),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(209),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(733),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(741),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [124] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_superclass] = STATE(5565),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5849),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(124),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(231),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(753),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(741),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [125] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_superclass] = STATE(5496),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5804),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(125),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(217),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(755),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(741),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [126] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_superclass] = STATE(5496),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5804),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(126),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(217),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(755),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(741),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [127] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5781),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(127),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(218),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(757),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [128] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_superclass] = STATE(5565),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5849),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(128),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(231),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(753),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(741),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [129] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5943),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(129),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(233),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(759),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [130] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_superclass] = STATE(5528),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5912),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(130),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(209),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(733),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(741),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [131] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5917),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(131),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(210),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(125),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(761),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [132] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5943),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(132),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(233),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(759),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [133] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5781),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(133),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(218),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(757),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [134] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5917),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(134),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(210),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(761),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [135] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(135),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(765),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [136] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(136),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(767),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [137] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(137),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(771),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [138] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(189),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5860),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(138),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(190),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(773),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [139] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(139),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(777),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [140] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(140),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(779),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [141] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(192),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5930),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(141),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(169),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(781),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [142] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(171),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5984),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(142),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(172),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(783),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [143] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(143),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(785),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [144] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(195),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5808),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(144),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(196),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(787),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [145] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(145),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(789),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [146] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(146),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(791),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [147] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(194),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5678),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(147),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(170),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(793),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [148] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(181),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5812),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(148),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(198),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(795),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [149] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(149),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(797),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [150] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(150),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(799),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [151] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(151),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(801),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [152] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(152),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(803),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [153] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(5457),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5776),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(153),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(211),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(805),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(807),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [154] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(154),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(809),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [155] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(183),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5820),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(155),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(184),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(811),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [156] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(156),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(813),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [157] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(157),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(815),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [158] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(174),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5979),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(158),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(175),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(817),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [159] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(159),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(819),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [160] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(160),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(821),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [161] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(186),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5837),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(161),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(187),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(823),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [162] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(162),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(825),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [163] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(163),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(827),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [164] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(178),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5800),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(164),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(179),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(829),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [165] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(165),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(831),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [166] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(166),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(833),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [167] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2416),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5888),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(167),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(238),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(763),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(835),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [168] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym__body_expr] = STATE(2961),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5696),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(168),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(241),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(769),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_EQ] = ACTIONS(837),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [169] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(193),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5982),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(169),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(839),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [170] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(177),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5833),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(170),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(841),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [171] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5971),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(171),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(206),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(843),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [172] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(173),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5675),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(172),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(845),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [173] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5787),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(173),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(207),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(847),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [174] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5708),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(174),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(214),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(849),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [175] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(176),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5740),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(175),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(851),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [176] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5898),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(176),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(215),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(853),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [177] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5947),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(177),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(237),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(855),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [178] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5653),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(178),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(219),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(857),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [179] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(180),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5656),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(179),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(859),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [180] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5667),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(180),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(220),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(861),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [181] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5687),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(181),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(222),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(863),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [182] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5901),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(182),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(223),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(865),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [183] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5640),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(183),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(224),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(867),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [184] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(185),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5881),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(184),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(869),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [185] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5642),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(185),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(225),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(871),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [186] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5929),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(186),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(226),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(873),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [187] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(188),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5948),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(187),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(875),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [188] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5981),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(188),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(227),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(877),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [189] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5645),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(189),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(228),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(879),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [190] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(191),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5873),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(190),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(881),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [191] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5655),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(191),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(230),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(883),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [192] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5959),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(192),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(232),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(885),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [193] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5648),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(193),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(234),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(887),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [194] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5823),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(194),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(242),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(889),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [195] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5834),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(195),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(235),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(891),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [196] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(197),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5842),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(196),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(893),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [197] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5872),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(197),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(236),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(895),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [198] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(182),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5739),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(198),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(897),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_PIPE] = ACTIONS(775),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [199] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym__body_statement] = STATE(5721),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(199),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(247),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(899),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [200] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2493),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(200),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(905),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(905),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_elsif] = ACTIONS(905),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(905),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(925),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [201] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym__body_statement] = STATE(5941),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(201),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(244),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(941),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [202] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2522),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(202),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(945),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(945),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_elsif] = ACTIONS(945),
    [anon_sym_else] = ACTIONS(945),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(945),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(925),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [203] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym__body_statement] = STATE(5939),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(203),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(246),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(947),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [204] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2565),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(204),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(905),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(905),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_elsif] = ACTIONS(905),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(905),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [205] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5965),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(205),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(967),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [206] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5786),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(206),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(969),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [207] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5734),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(207),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(971),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [208] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5914),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(208),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(973),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [209] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5976),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(209),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(975),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [210] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5665),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(210),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(977),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [211] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5854),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(211),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(979),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [212] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5861),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(212),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(981),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [213] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5868),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(213),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(983),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [214] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5706),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(214),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(985),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [215] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5920),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(215),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(987),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [216] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5785),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(216),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(989),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [217] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5828),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(217),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(991),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [218] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5805),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(218),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(993),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [219] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5666),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(219),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(995),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [220] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5669),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(220),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(997),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [221] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2600),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(221),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(945),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(945),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_elsif] = ACTIONS(945),
    [anon_sym_else] = ACTIONS(945),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(945),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(959),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [222] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5894),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(222),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(999),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [223] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5940),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(223),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1001),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [224] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5969),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(224),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1003),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [225] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5663),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(225),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1005),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [226] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5978),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(226),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1007),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [227] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5985),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(227),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1009),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [228] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5643),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(228),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1011),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [229] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5720),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(229),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1013),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [230] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5672),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(230),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1015),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [231] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5743),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(231),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1017),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [232] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5646),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(232),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1019),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [233] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5755),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(233),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1021),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [234] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5650),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(234),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1023),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [235] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5870),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(235),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1025),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [236] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5880),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(236),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1027),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [237] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5955),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(237),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1029),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [238] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5810),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(238),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1031),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [239] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5673),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(239),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1033),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [240] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5846),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(240),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1035),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [241] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5951),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(241),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1037),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [242] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5923),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(242),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1039),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [243] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym_body_statement] = STATE(5963),
    [sym__body_statement] = STATE(5777),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(243),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1041),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [244] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym__body_statement] = STATE(5973),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(244),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1043),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [245] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2629),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(245),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_end] = ACTIONS(905),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(905),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_elsif] = ACTIONS(905),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(905),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1055),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [246] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym__body_statement] = STATE(5944),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(246),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1063),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [247] = {
    [sym__statements] = STATE(4944),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5404),
    [sym_begin] = STATE(2376),
    [sym_ensure] = STATE(5404),
    [sym_rescue] = STATE(5404),
    [sym__body_statement] = STATE(5983),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(247),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [aux_sym__body_statement_repeat1] = STATE(5001),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1065),
    [anon_sym_rescue] = ACTIONS(735),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(749),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [248] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4522),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(248),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(1067),
    [anon_sym_RBRACE] = ACTIONS(1067),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(1067),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1069),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1071),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(1069),
    [anon_sym_unless] = ACTIONS(1069),
    [anon_sym_while] = ACTIONS(1069),
    [anon_sym_until] = ACTIONS(1069),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(1069),
    [anon_sym_do] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1069),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_elsif] = ACTIONS(1069),
    [anon_sym_else] = ACTIONS(1069),
    [anon_sym_then] = ACTIONS(1069),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1069),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
  },
  [249] = {
    [sym_block_body] = STATE(5885),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(318),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(249),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1081),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [250] = {
    [sym_block_body] = STATE(5885),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(318),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(250),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1085),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [251] = {
    [sym_block_body] = STATE(5980),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(323),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5506),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(251),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5506),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1087),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [252] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4626),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(252),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(257),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1089),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(1089),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1091),
    [anon_sym_rescue] = ACTIONS(1091),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(1091),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1091),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1091),
    [anon_sym_else] = ACTIONS(1091),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1091),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [253] = {
    [sym_block_body] = STATE(5980),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(323),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5506),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(253),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5506),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1093),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [254] = {
    [sym_block_body] = STATE(5885),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(318),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(254),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1095),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [255] = {
    [sym_block_body] = STATE(5885),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(318),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(255),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1097),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [256] = {
    [sym_block_body] = STATE(5980),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(323),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5506),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(256),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5506),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1099),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [257] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4950),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(257),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(257),
    [sym_identifier] = ACTIONS(1101),
    [anon_sym_BEGIN] = ACTIONS(1104),
    [anon_sym_LBRACE] = ACTIONS(1107),
    [anon_sym_RBRACE] = ACTIONS(1110),
    [anon_sym_END] = ACTIONS(1112),
    [anon_sym_def] = ACTIONS(1115),
    [anon_sym_LPAREN] = ACTIONS(1118),
    [anon_sym_RPAREN] = ACTIONS(1110),
    [anon_sym_COLON_COLON] = ACTIONS(1121),
    [anon_sym_end] = ACTIONS(1124),
    [anon_sym_rescue] = ACTIONS(1124),
    [anon_sym_SEMI] = ACTIONS(1126),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1129),
    [anon_sym_STAR] = ACTIONS(1132),
    [anon_sym_nil] = ACTIONS(1135),
    [anon_sym_class] = ACTIONS(1138),
    [anon_sym_module] = ACTIONS(1141),
    [anon_sym_return] = ACTIONS(1144),
    [anon_sym_yield] = ACTIONS(1147),
    [anon_sym_break] = ACTIONS(1150),
    [anon_sym_next] = ACTIONS(1153),
    [anon_sym_redo] = ACTIONS(1156),
    [anon_sym_retry] = ACTIONS(1159),
    [anon_sym_if] = ACTIONS(1162),
    [anon_sym_unless] = ACTIONS(1165),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_until] = ACTIONS(1171),
    [anon_sym_for] = ACTIONS(1174),
    [anon_sym_in] = ACTIONS(1124),
    [anon_sym_case] = ACTIONS(1177),
    [anon_sym_when] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1180),
    [anon_sym_DOT_DOT] = ACTIONS(1183),
    [anon_sym_elsif] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_begin] = ACTIONS(1186),
    [anon_sym_ensure] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1189),
    [anon_sym_defined_QMARK] = ACTIONS(1192),
    [anon_sym_not] = ACTIONS(1195),
    [anon_sym_BANG] = ACTIONS(1198),
    [anon_sym_TILDE] = ACTIONS(1198),
    [anon_sym_undef] = ACTIONS(1201),
    [anon_sym_alias] = ACTIONS(1204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(1207),
    [sym_float] = ACTIONS(1207),
    [sym_super] = ACTIONS(1101),
    [sym_self] = ACTIONS(1101),
    [sym_true] = ACTIONS(1210),
    [sym_false] = ACTIONS(1210),
    [sym_constant] = ACTIONS(1101),
    [aux_sym_constant_suffix_token1] = ACTIONS(1213),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1216),
    [sym_instance_variable] = ACTIONS(1219),
    [sym_class_variable] = ACTIONS(1219),
    [sym_global_variable] = ACTIONS(1219),
    [sym_character] = ACTIONS(1222),
    [anon_sym_DASH_GT] = ACTIONS(1225),
    [sym_simple_symbol] = ACTIONS(1228),
    [sym__string_start] = ACTIONS(1231),
    [sym__symbol_start] = ACTIONS(1234),
    [sym__subshell_start] = ACTIONS(1237),
    [sym__regex_start] = ACTIONS(1240),
    [sym__string_array_start] = ACTIONS(1243),
    [sym__symbol_array_start] = ACTIONS(1246),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1222),
    [sym__unary_minus] = ACTIONS(1249),
    [sym__unary_minus_num] = ACTIONS(1252),
    [sym__binary_minus] = ACTIONS(1255),
    [sym__identifier_suffix] = ACTIONS(1258),
    [sym__constant_suffix] = ACTIONS(1213),
  },
  [258] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4565),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(258),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1067),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(1067),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1069),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1261),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(1069),
    [anon_sym_unless] = ACTIONS(1069),
    [anon_sym_while] = ACTIONS(1069),
    [anon_sym_until] = ACTIONS(1069),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1069),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_elsif] = ACTIONS(1069),
    [anon_sym_else] = ACTIONS(1069),
    [anon_sym_then] = ACTIONS(1069),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1069),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
  },
  [259] = {
    [sym_block_body] = STATE(5885),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(318),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3539),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1992),
    [sym_identifier_suffix] = STATE(1995),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2745),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(259),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(1079),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1269),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(1079),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [260] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(260),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1271),
    [anon_sym_rescue] = ACTIONS(1271),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(1271),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1271),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1271),
    [anon_sym_else] = ACTIONS(1271),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1271),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [261] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2688),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(261),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(905),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(905),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_elsif] = ACTIONS(905),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(905),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1281),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [262] = {
    [sym__statements] = STATE(5046),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(262),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1285),
    [anon_sym_rescue] = ACTIONS(1285),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(1285),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1285),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1285),
    [anon_sym_else] = ACTIONS(1285),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1285),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [263] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2674),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(263),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(945),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(945),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_elsif] = ACTIONS(945),
    [anon_sym_else] = ACTIONS(945),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(945),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1281),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [264] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_elsif] = STATE(5768),
    [sym_else] = STATE(5768),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(264),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1287),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1289),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [265] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_elsif] = STATE(5903),
    [sym_else] = STATE(5903),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(265),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1293),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1289),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [266] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_elsif] = STATE(5949),
    [sym_else] = STATE(5949),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(266),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1295),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1289),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [267] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_elsif] = STATE(5778),
    [sym_else] = STATE(5778),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(267),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1297),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1289),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [268] = {
    [sym__statements] = STATE(5448),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(268),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [sym__terminator] = STATE(289),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1299),
    [anon_sym_rescue] = ACTIONS(1299),
    [anon_sym_SEMI] = ACTIONS(737),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1299),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1299),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [269] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_elsif] = STATE(5938),
    [sym_else] = STATE(5938),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(269),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1301),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1289),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [270] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2762),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(270),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(945),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(945),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_elsif] = ACTIONS(945),
    [anon_sym_else] = ACTIONS(945),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(945),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1311),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [271] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2775),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(271),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(905),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(905),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_elsif] = ACTIONS(905),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(905),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1311),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [272] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_elsif] = STATE(5847),
    [sym_else] = STATE(5847),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(272),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1315),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1289),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [273] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_elsif] = STATE(5799),
    [sym_else] = STATE(5799),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(273),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1317),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_elsif] = ACTIONS(1289),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [274] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2763),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(274),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(943),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(945),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1329),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [275] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(275),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1337),
    [anon_sym_rescue] = ACTIONS(1337),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1337),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1337),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [276] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4609),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_when] = STATE(5505),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5921),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(276),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym__terminator] = STATE(5239),
    [aux_sym_case_repeat1] = STATE(5240),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1339),
    [anon_sym_SEMI] = ACTIONS(1341),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1343),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1345),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [277] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4602),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_when] = STATE(5505),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5916),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(277),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym__terminator] = STATE(5280),
    [aux_sym_case_repeat1] = STATE(5281),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1347),
    [anon_sym_SEMI] = ACTIONS(1341),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1343),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1349),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [278] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2767),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(278),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(903),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1329),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [279] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2801),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(279),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(905),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(905),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_elsif] = ACTIONS(905),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(905),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1357),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [280] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(280),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1361),
    [anon_sym_rescue] = ACTIONS(1361),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1361),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1361),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [281] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(281),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1363),
    [anon_sym_rescue] = ACTIONS(1363),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1363),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1363),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [282] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(282),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1365),
    [anon_sym_rescue] = ACTIONS(1365),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1365),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1365),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [283] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4610),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_when] = STATE(5505),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_else] = STATE(5691),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(283),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym__terminator] = STATE(5250),
    [aux_sym_case_repeat1] = STATE(5257),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1367),
    [anon_sym_SEMI] = ACTIONS(1341),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1343),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(747),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1369),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [284] = {
    [sym_block_body] = STATE(5790),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(315),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(284),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1371),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [285] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2832),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(285),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(943),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(945),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1383),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [286] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2867),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(286),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(903),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1383),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [287] = {
    [sym_block_body] = STATE(5972),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(317),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(287),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1391),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [288] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3269),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(288),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [289] = {
    [sym__statements] = STATE(5368),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(289),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1431),
    [anon_sym_rescue] = ACTIONS(1431),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1431),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1431),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [290] = {
    [sym_block_body] = STATE(5980),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(323),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(290),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1433),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [291] = {
    [sym_block_body] = STATE(5911),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(319),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(291),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1435),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [292] = {
    [sym_block_body] = STATE(5803),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(327),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(292),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1437),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [293] = {
    [sym_block_body] = STATE(5801),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(321),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(293),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1439),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [294] = {
    [sym_block_body] = STATE(5719),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(313),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(294),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1441),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [295] = {
    [sym_block_body] = STATE(5826),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(324),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(295),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1443),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [296] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3332),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(296),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [297] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(297),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1445),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(1445),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1445),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [298] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(298),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1447),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(1447),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1447),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [299] = {
    [sym__statements] = STATE(5385),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(299),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1449),
    [anon_sym_rescue] = ACTIONS(1449),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1449),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(1449),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [300] = {
    [sym_block_body] = STATE(5974),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(320),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(300),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1451),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [301] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(301),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1453),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1453),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1453),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [302] = {
    [sym__statements] = STATE(5095),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(302),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1455),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(1455),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(1455),
    [anon_sym_then] = ACTIONS(1291),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [303] = {
    [sym_block_body] = STATE(5885),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_block_parameters] = STATE(318),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(303),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1457),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(1083),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [304] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3230),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(304),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1469),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [305] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3410),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(305),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1487),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [306] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3267),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(306),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1505),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [307] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2935),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(307),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1523),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [308] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3409),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(308),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1487),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [309] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3236),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(309),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1505),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [310] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3030),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(310),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(903),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1541),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [311] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3347),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(311),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1469),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [312] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3033),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(312),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1523),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [313] = {
    [sym_block_body] = STATE(5652),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(313),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1549),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [314] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3531),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(314),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1561),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [315] = {
    [sym_block_body] = STATE(5962),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(315),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1569),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [316] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4641),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(316),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(1067),
    [sym_identifier] = ACTIONS(183),
    [aux_sym_program_token1] = ACTIONS(1069),
    [anon_sym_LBRACE] = ACTIONS(1067),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1571),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(1069),
    [anon_sym_unless] = ACTIONS(1069),
    [anon_sym_while] = ACTIONS(1069),
    [anon_sym_until] = ACTIONS(1069),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
  },
  [317] = {
    [sym_block_body] = STATE(5953),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(317),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1579),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [318] = {
    [sym_block_body] = STATE(5814),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(318),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1581),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [319] = {
    [sym_block_body] = STATE(5957),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(319),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1583),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [320] = {
    [sym_block_body] = STATE(5700),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(320),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1585),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [321] = {
    [sym_block_body] = STATE(5831),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(321),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1587),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [322] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3492),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(322),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1561),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [323] = {
    [sym_block_body] = STATE(5902),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(323),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1589),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [324] = {
    [sym_block_body] = STATE(5927),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(324),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1591),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [325] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3094),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(325),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1605),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [326] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3355),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(326),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1623),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [327] = {
    [sym_block_body] = STATE(5686),
    [sym__statements] = STATE(5893),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(327),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1631),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [328] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3435),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(328),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1623),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [329] = {
    [sym__statements] = STATE(5760),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5606),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(329),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(683),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [330] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_splat_parameter] = STATE(4523),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4530),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4531),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2820),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2478),
    [sym_identifier_suffix] = STATE(2485),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2268),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(330),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(1633),
    [anon_sym_LBRACE] = ACTIONS(1635),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(1637),
    [anon_sym_COLON_COLON] = ACTIONS(1639),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1641),
    [anon_sym_STAR] = ACTIONS(1643),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(1647),
    [anon_sym_DOT_DOT] = ACTIONS(1649),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(1659),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [331] = {
    [sym__statements] = STATE(5684),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(331),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1663),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [332] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_top_expr_body] = STATE(4985),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4975),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4024),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(3401),
    [sym_identifier_suffix] = STATE(3402),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3085),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(332),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LBRACE] = ACTIONS(1667),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(1669),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1671),
    [anon_sym_STAR] = ACTIONS(1673),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1677),
    [anon_sym_DOT_DOT] = ACTIONS(1679),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1693),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [333] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_splat_parameter] = STATE(4634),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4642),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4636),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3228),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2719),
    [sym_identifier_suffix] = STATE(2742),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2630),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(333),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(1699),
    [anon_sym_LBRACE] = ACTIONS(1635),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(1637),
    [anon_sym_COLON_COLON] = ACTIONS(1639),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1701),
    [anon_sym_STAR] = ACTIONS(1703),
    [anon_sym_STAR_STAR] = ACTIONS(1705),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(1647),
    [anon_sym_DOT_DOT] = ACTIONS(1707),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(1719),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [334] = {
    [sym__statements] = STATE(5975),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(334),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1721),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [335] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_top_expr_body] = STATE(4985),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4975),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4095),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(3401),
    [sym_identifier_suffix] = STATE(3402),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3085),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(335),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LBRACE] = ACTIONS(1667),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(1669),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1723),
    [anon_sym_STAR] = ACTIONS(1673),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1677),
    [anon_sym_DOT_DOT] = ACTIONS(1725),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1693),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [336] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_splat_parameter] = STATE(4523),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4530),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4531),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2702),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2478),
    [sym_identifier_suffix] = STATE(2485),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2268),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(336),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(1633),
    [anon_sym_LBRACE] = ACTIONS(1635),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(1637),
    [anon_sym_COLON_COLON] = ACTIONS(1639),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1735),
    [anon_sym_STAR] = ACTIONS(1643),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(1647),
    [anon_sym_DOT_DOT] = ACTIONS(1737),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(1659),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [337] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_splat_parameter] = STATE(4634),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4642),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4636),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3483),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2719),
    [sym_identifier_suffix] = STATE(2742),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2630),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(337),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(1699),
    [anon_sym_LBRACE] = ACTIONS(1635),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(1637),
    [anon_sym_COLON_COLON] = ACTIONS(1639),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1739),
    [anon_sym_STAR] = ACTIONS(1703),
    [anon_sym_STAR_STAR] = ACTIONS(1705),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(1647),
    [anon_sym_DOT_DOT] = ACTIONS(1741),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(1719),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [338] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4862),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(338),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(1067),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1071),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_then] = ACTIONS(1069),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
  },
  [339] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4523),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4530),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4531),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4095),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(3125),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2251),
    [sym_constant_suffix] = STATE(3541),
    [sym_identifier_suffix] = STATE(3496),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2720),
    [sym_subshell] = STATE(3125),
    [sym_string_array] = STATE(3125),
    [sym_symbol_array] = STATE(3125),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(3125),
    [sym_heredoc_body] = STATE(339),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3127),
    [sym_identifier] = ACTIONS(1755),
    [anon_sym_LBRACE] = ACTIONS(1757),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1759),
    [anon_sym_COLON_COLON] = ACTIONS(1761),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1763),
    [anon_sym_STAR] = ACTIONS(1643),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1765),
    [anon_sym_DOT_DOT] = ACTIONS(1767),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1771),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_constant] = ACTIONS(1775),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1777),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(1779),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [340] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_top_expr_body] = STATE(4985),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4975),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4183),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(3401),
    [sym_identifier_suffix] = STATE(3402),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3085),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(340),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LBRACE] = ACTIONS(1667),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(1669),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1723),
    [anon_sym_STAR] = ACTIONS(1673),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1677),
    [anon_sym_DOT_DOT] = ACTIONS(1725),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1693),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [341] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4936),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_top_expr_body] = STATE(4985),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4898),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4929),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4108),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(3231),
    [sym_identifier_suffix] = STATE(3232),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2904),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(341),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1781),
    [anon_sym_LBRACE] = ACTIONS(1667),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(1669),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1783),
    [anon_sym_STAR] = ACTIONS(1673),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1677),
    [anon_sym_DOT_DOT] = ACTIONS(1785),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1795),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [342] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_splat_parameter] = STATE(4768),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4812),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4806),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3931),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(3221),
    [sym_identifier_suffix] = STATE(3222),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2875),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(342),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(1799),
    [anon_sym_LBRACE] = ACTIONS(1635),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(1637),
    [anon_sym_COLON_COLON] = ACTIONS(1639),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1801),
    [anon_sym_STAR] = ACTIONS(1643),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(1647),
    [anon_sym_DOT_DOT] = ACTIONS(1803),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(1813),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [343] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3083),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(343),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(943),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(945),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1819),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [344] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_splat_parameter] = STATE(4936),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_top_expr_body] = STATE(4985),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4898),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4929),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3903),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(3231),
    [sym_identifier_suffix] = STATE(3232),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2904),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(344),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1781),
    [anon_sym_LBRACE] = ACTIONS(1667),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(1669),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1821),
    [anon_sym_STAR] = ACTIONS(1673),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1677),
    [anon_sym_DOT_DOT] = ACTIONS(1823),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1795),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [345] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_splat_parameter] = STATE(4768),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4812),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4806),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4059),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(3221),
    [sym_identifier_suffix] = STATE(3222),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2875),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(345),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(1799),
    [anon_sym_LBRACE] = ACTIONS(1635),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(1637),
    [anon_sym_COLON_COLON] = ACTIONS(1639),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1835),
    [anon_sym_STAR] = ACTIONS(1643),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(1647),
    [anon_sym_DOT_DOT] = ACTIONS(1837),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(1813),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [346] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4868),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4770),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(346),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2144),
    [aux_sym__statements_repeat1] = STATE(355),
    [ts_builtin_sym_end] = ACTIONS(1089),
    [sym_identifier] = ACTIONS(9),
    [aux_sym_program_token1] = ACTIONS(1091),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(25),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [anon_sym_undef] = ACTIONS(75),
    [anon_sym_alias] = ACTIONS(77),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [347] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3554),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(347),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [348] = {
    [sym__statements] = STATE(5709),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(348),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1869),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [349] = {
    [sym__statements] = STATE(5772),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(349),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1871),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [350] = {
    [sym__statements] = STATE(5856),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(350),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1873),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [351] = {
    [sym__statements] = STATE(5866),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5606),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(351),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(641),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [352] = {
    [sym__statements] = STATE(5748),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5606),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(352),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(1875),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [353] = {
    [sym__statements] = STATE(5748),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(353),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(1875),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [354] = {
    [sym__statements] = STATE(5762),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(354),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1877),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [355] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(5033),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(355),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2144),
    [aux_sym__statements_repeat1] = STATE(355),
    [ts_builtin_sym_end] = ACTIONS(1110),
    [sym_identifier] = ACTIONS(1101),
    [aux_sym_program_token1] = ACTIONS(1124),
    [anon_sym_BEGIN] = ACTIONS(1104),
    [anon_sym_LBRACE] = ACTIONS(1107),
    [anon_sym_END] = ACTIONS(1112),
    [anon_sym_def] = ACTIONS(1115),
    [anon_sym_LPAREN] = ACTIONS(1118),
    [anon_sym_COLON_COLON] = ACTIONS(1121),
    [anon_sym_SEMI] = ACTIONS(1879),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1129),
    [anon_sym_STAR] = ACTIONS(1132),
    [anon_sym_nil] = ACTIONS(1135),
    [anon_sym_class] = ACTIONS(1138),
    [anon_sym_module] = ACTIONS(1141),
    [anon_sym_return] = ACTIONS(1144),
    [anon_sym_yield] = ACTIONS(1147),
    [anon_sym_break] = ACTIONS(1150),
    [anon_sym_next] = ACTIONS(1153),
    [anon_sym_redo] = ACTIONS(1156),
    [anon_sym_retry] = ACTIONS(1159),
    [anon_sym_if] = ACTIONS(1162),
    [anon_sym_unless] = ACTIONS(1165),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_until] = ACTIONS(1171),
    [anon_sym_for] = ACTIONS(1174),
    [anon_sym_case] = ACTIONS(1177),
    [anon_sym_LBRACK] = ACTIONS(1180),
    [anon_sym_DOT_DOT] = ACTIONS(1183),
    [anon_sym_begin] = ACTIONS(1186),
    [anon_sym_PLUS] = ACTIONS(1189),
    [anon_sym_defined_QMARK] = ACTIONS(1192),
    [anon_sym_not] = ACTIONS(1195),
    [anon_sym_BANG] = ACTIONS(1198),
    [anon_sym_TILDE] = ACTIONS(1198),
    [anon_sym_undef] = ACTIONS(1201),
    [anon_sym_alias] = ACTIONS(1204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(1207),
    [sym_float] = ACTIONS(1207),
    [sym_super] = ACTIONS(1101),
    [sym_self] = ACTIONS(1101),
    [sym_true] = ACTIONS(1210),
    [sym_false] = ACTIONS(1210),
    [sym_constant] = ACTIONS(1101),
    [aux_sym_constant_suffix_token1] = ACTIONS(1213),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1216),
    [sym_instance_variable] = ACTIONS(1219),
    [sym_class_variable] = ACTIONS(1219),
    [sym_global_variable] = ACTIONS(1219),
    [sym_character] = ACTIONS(1222),
    [anon_sym_DASH_GT] = ACTIONS(1225),
    [sym_simple_symbol] = ACTIONS(1228),
    [sym__string_start] = ACTIONS(1231),
    [sym__symbol_start] = ACTIONS(1234),
    [sym__subshell_start] = ACTIONS(1237),
    [sym__regex_start] = ACTIONS(1240),
    [sym__string_array_start] = ACTIONS(1243),
    [sym__symbol_array_start] = ACTIONS(1246),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1222),
    [sym__unary_minus] = ACTIONS(1249),
    [sym__unary_minus_num] = ACTIONS(1252),
    [sym__binary_minus] = ACTIONS(1255),
    [sym__identifier_suffix] = ACTIONS(1258),
    [sym__constant_suffix] = ACTIONS(1213),
  },
  [356] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3242),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4731),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1981),
    [sym_identifier_suffix] = STATE(1986),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2692),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(356),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(1067),
    [sym_identifier] = ACTIONS(461),
    [aux_sym_program_token1] = ACTIONS(1069),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1882),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(1069),
    [anon_sym_unless] = ACTIONS(1069),
    [anon_sym_while] = ACTIONS(1069),
    [anon_sym_until] = ACTIONS(1069),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(461),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(473),
    [sym__splat_star] = ACTIONS(475),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym_hash_key_symbol] = ACTIONS(481),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(483),
  },
  [357] = {
    [sym__statements] = STATE(5807),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(357),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(1888),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [358] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_splat_parameter] = STATE(4523),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4530),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4531),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3556),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2478),
    [sym_identifier_suffix] = STATE(2485),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2268),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(358),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(1633),
    [anon_sym_LBRACE] = ACTIONS(1635),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(1637),
    [anon_sym_COLON_COLON] = ACTIONS(1639),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1890),
    [anon_sym_STAR] = ACTIONS(1643),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(1647),
    [anon_sym_DOT_DOT] = ACTIONS(1892),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(1659),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [359] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3141),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(359),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(903),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1819),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [360] = {
    [sym__statements] = STATE(5954),
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4625),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(360),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_empty_statement] = STATE(2051),
    [aux_sym__statements_repeat1] = STATE(252),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(1904),
    [anon_sym_SEMI] = ACTIONS(645),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [361] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3457),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4749),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1997),
    [sym_identifier_suffix] = STATE(1999),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2707),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(361),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(503),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1906),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(1069),
    [anon_sym_unless] = ACTIONS(1069),
    [anon_sym_while] = ACTIONS(1069),
    [anon_sym_until] = ACTIONS(1069),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(503),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(523),
    [sym__splat_star] = ACTIONS(525),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym_hash_key_symbol] = ACTIONS(531),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(533),
  },
  [362] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3326),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(362),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(943),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(945),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1914),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [363] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4945),
    [sym__argument] = STATE(4904),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(363),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(1067),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(1067),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1918),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(1067),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
  },
  [364] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3638),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(364),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1936),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [365] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3699),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(365),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1948),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [366] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3629),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(366),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1948),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [367] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3738),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(367),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1962),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [368] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3240),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(368),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(903),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1914),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [369] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4947),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(369),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1261),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_then] = ACTIONS(1069),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
  },
  [370] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(4925),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(370),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(1067),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1972),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_then] = ACTIONS(1069),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(1976),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
  },
  [371] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3658),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(371),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1936),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [372] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3872),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(372),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1990),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [373] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3747),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(373),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(2008),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [374] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3865),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(374),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_COLON2] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1990),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [375] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3880),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(375),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2022),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [376] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3795),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(376),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(903),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(2034),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [377] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3452),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(377),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(2038),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [378] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4945),
    [sym__argument] = STATE(4996),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(378),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(1067),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(1067),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2040),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(1069),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(1067),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(2044),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(391),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
  },
  [379] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3798),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(379),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(2034),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [380] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4608),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(380),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2046),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2046),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(2046),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [381] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3956),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_command_argument_list] = STATE(4500),
    [sym__argument] = STATE(5184),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1996),
    [sym_identifier_suffix] = STATE(1998),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(3139),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(381),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(569),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_SEMI] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2048),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(1069),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(2052),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(587),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(569),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(1067),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(589),
    [sym__splat_star] = ACTIONS(591),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym_hash_key_symbol] = ACTIONS(597),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(599),
  },
  [382] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3768),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(382),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(129),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2066),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [383] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4603),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(383),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2046),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2046),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(2046),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [384] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3709),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(384),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(2008),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [385] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3620),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(385),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(2076),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [386] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4615),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(386),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2046),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2046),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_else] = ACTIONS(2046),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [387] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3732),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(387),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(2076),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [388] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3430),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(388),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(2038),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [389] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3418),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(389),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(903),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2084),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [390] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3957),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4945),
    [sym__argument] = STATE(5041),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(1994),
    [sym_identifier_suffix] = STATE(1991),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3093),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(390),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(535),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(1067),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(1069),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2088),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(1067),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(535),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(557),
    [sym__splat_star] = ACTIONS(559),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym_hash_key_symbol] = ACTIONS(565),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(567),
  },
  [391] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3964),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(391),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(2094),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [392] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3478),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(392),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2106),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [393] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3807),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(393),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(903),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(2116),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [394] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3974),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(394),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2130),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [395] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3919),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(395),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2150),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [396] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3895),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(396),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(2094),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [397] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3878),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(397),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(2116),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [398] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4015),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_command_argument_list] = STATE(4945),
    [sym__argument] = STATE(5214),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2007),
    [sym_identifier_suffix] = STATE(2010),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3198),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(398),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(601),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(1067),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(1067),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(1067),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(1069),
    [anon_sym_or] = ACTIONS(1069),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(601),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(621),
    [sym__splat_star] = ACTIONS(623),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(629),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(631),
  },
  [399] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3969),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(399),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_DOT] = ACTIONS(945),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(943),
    [anon_sym_AMP_DOT] = ACTIONS(943),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_COLON2] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2150),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(943),
    [sym__element_reference_bracket] = ACTIONS(943),
  },
  [400] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4020),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(400),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_DOT] = ACTIONS(905),
    [anon_sym_COLON_COLON] = ACTIONS(277),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_COLON_COLON2] = ACTIONS(903),
    [anon_sym_AMP_DOT] = ACTIONS(903),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2172),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
    [sym__element_reference_bracket] = ACTIONS(903),
  },
  [401] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4001),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(401),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_then] = ACTIONS(905),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2190),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [402] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4869),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4837),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(402),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [403] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4876),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(403),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [404] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4827),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4837),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(404),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [405] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4070),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(405),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2218),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [406] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4839),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(406),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [407] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4879),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4837),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(407),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [408] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4119),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(408),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2236),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [409] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4860),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(409),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [410] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4870),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4837),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(410),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [411] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4838),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(411),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [412] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4857),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4837),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(412),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [413] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4859),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4837),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(413),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [414] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4867),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(414),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [415] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4834),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(415),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [416] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4101),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(416),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2236),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [417] = {
    [sym_begin_block] = STATE(4579),
    [sym_end_block] = STATE(4579),
    [sym__statement] = STATE(4874),
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_if_modifier] = STATE(4579),
    [sym_unless_modifier] = STATE(4579),
    [sym_while_modifier] = STATE(4579),
    [sym_until_modifier] = STATE(4579),
    [sym_rescue_modifier] = STATE(4579),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4589),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym_undef] = STATE(4579),
    [sym_alias] = STATE(4579),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(417),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_BEGIN] = ACTIONS(13),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_END] = ACTIONS(17),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [anon_sym_undef] = ACTIONS(665),
    [anon_sym_alias] = ACTIONS(667),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [418] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4176),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(418),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_SEMI] = ACTIONS(903),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2254),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(903),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [419] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4210),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(419),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_COLON2] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2272),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [420] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4197),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(420),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2272),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [421] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4170),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(421),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(903),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_in] = ACTIONS(905),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_EQ_GT] = ACTIONS(903),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2290),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [422] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5693),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(422),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2300),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [423] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5906),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(423),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2302),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [424] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5827),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(424),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2304),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [425] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5924),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(425),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2306),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [426] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4221),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(426),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_PIPE] = ACTIONS(945),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(945),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(945),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_CARET] = ACTIONS(943),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(945),
    [anon_sym_COLON2] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PIPE_PIPE] = ACTIONS(943),
    [anon_sym_AMP_AMP] = ACTIONS(943),
    [anon_sym_LT_LT] = ACTIONS(943),
    [anon_sym_GT_GT] = ACTIONS(943),
    [anon_sym_LT_EQ] = ACTIONS(945),
    [anon_sym_GT] = ACTIONS(945),
    [anon_sym_GT_EQ] = ACTIONS(943),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_SLASH] = ACTIONS(943),
    [anon_sym_PERCENT] = ACTIONS(943),
    [anon_sym_EQ_EQ] = ACTIONS(945),
    [anon_sym_BANG_EQ] = ACTIONS(943),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(943),
    [anon_sym_LT_EQ_GT] = ACTIONS(943),
    [anon_sym_EQ_TILDE] = ACTIONS(943),
    [anon_sym_BANG_TILDE] = ACTIONS(943),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2318),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__binary_star] = ACTIONS(943),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(943),
  },
  [427] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5935),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(427),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2326),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [428] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5815),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(428),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2328),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [429] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5934),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(429),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2330),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [430] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5926),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(430),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2332),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [431] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5683),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(431),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2334),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [432] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument] = STATE(5018),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(432),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2336),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2336),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [433] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5928),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(433),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2338),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [434] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4242),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(434),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(903),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2318),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [435] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument] = STATE(5018),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(435),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2340),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2340),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [436] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5806),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(436),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2342),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [437] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4233),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(437),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(905),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2354),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [438] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5641),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(438),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2362),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [439] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_expr] = STATE(4858),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4055),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2872),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(439),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(2364),
    [anon_sym_LBRACE] = ACTIONS(1667),
    [anon_sym_RBRACE] = ACTIONS(2366),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(1669),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1723),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1677),
    [anon_sym_DOT_DOT] = ACTIONS(1725),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(2368),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2370),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [440] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument_list_with_trailing_comma] = STATE(5848),
    [sym__argument] = STATE(5442),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(440),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2372),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [441] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_expr] = STATE(4829),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4095),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2872),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(441),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(2364),
    [anon_sym_LBRACE] = ACTIONS(1667),
    [anon_sym_RBRACE] = ACTIONS(2374),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(689),
    [anon_sym_COLON_COLON] = ACTIONS(1669),
    [anon_sym_COMMA] = ACTIONS(2374),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1723),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(1677),
    [anon_sym_DOT_DOT] = ACTIONS(1725),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(2368),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [442] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4317),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(442),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_PIPE] = ACTIONS(905),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_AMP] = ACTIONS(905),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_LT] = ACTIONS(905),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_CARET] = ACTIONS(903),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_QMARK] = ACTIONS(905),
    [anon_sym_COLON2] = ACTIONS(905),
    [anon_sym_and] = ACTIONS(905),
    [anon_sym_or] = ACTIONS(905),
    [anon_sym_PIPE_PIPE] = ACTIONS(903),
    [anon_sym_AMP_AMP] = ACTIONS(903),
    [anon_sym_LT_LT] = ACTIONS(903),
    [anon_sym_GT_GT] = ACTIONS(903),
    [anon_sym_LT_EQ] = ACTIONS(905),
    [anon_sym_GT] = ACTIONS(905),
    [anon_sym_GT_EQ] = ACTIONS(903),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_SLASH] = ACTIONS(903),
    [anon_sym_PERCENT] = ACTIONS(903),
    [anon_sym_EQ_EQ] = ACTIONS(905),
    [anon_sym_BANG_EQ] = ACTIONS(903),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(903),
    [anon_sym_LT_EQ_GT] = ACTIONS(903),
    [anon_sym_EQ_TILDE] = ACTIONS(903),
    [anon_sym_BANG_TILDE] = ACTIONS(903),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2386),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__binary_star] = ACTIONS(903),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
    [sym__binary_star_star] = ACTIONS(903),
  },
  [443] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(443),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1071),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
  },
  [444] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1983),
    [sym_identifier_suffix] = STATE(1984),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2544),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(444),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(183),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1571),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(183),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(203),
    [sym__splat_star] = ACTIONS(205),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym_hash_key_symbol] = ACTIONS(211),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(213),
  },
  [445] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3242),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1981),
    [sym_identifier_suffix] = STATE(1986),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2692),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(445),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(461),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1882),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(461),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(473),
    [sym__splat_star] = ACTIONS(475),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym_hash_key_symbol] = ACTIONS(481),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
    [sym__hash_splat_star_star] = ACTIONS(483),
  },
  [446] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4015),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument] = STATE(5018),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2007),
    [sym_identifier_suffix] = STATE(2010),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3198),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(446),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(601),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(601),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(621),
    [sym__splat_star] = ACTIONS(623),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(629),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(631),
  },
  [447] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3957),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym__argument] = STATE(5018),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(1994),
    [sym_identifier_suffix] = STATE(1991),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3093),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(447),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(535),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2088),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(535),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(557),
    [sym__splat_star] = ACTIONS(559),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym_hash_key_symbol] = ACTIONS(565),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(567),
  },
  [448] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1993),
    [sym_identifier_suffix] = STATE(1990),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2837),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(448),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(233),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1972),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1976),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(233),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(255),
    [sym__splat_star] = ACTIONS(257),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym_hash_key_symbol] = ACTIONS(263),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(265),
  },
  [449] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3457),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1997),
    [sym_identifier_suffix] = STATE(1999),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2707),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(449),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(503),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1906),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(503),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(523),
    [sym__splat_star] = ACTIONS(525),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym_hash_key_symbol] = ACTIONS(531),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(533),
  },
  [450] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(450),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1261),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
  },
  [451] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2696),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1968),
    [sym_identifier_suffix] = STATE(1964),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2254),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(451),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(117),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1071),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(117),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(159),
    [sym__splat_star] = ACTIONS(161),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym_hash_key_symbol] = ACTIONS(167),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(171),
  },
  [452] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3700),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym__argument] = STATE(5018),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2001),
    [sym_identifier_suffix] = STATE(2002),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2855),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(452),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(269),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1918),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(269),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(355),
    [sym__splat_star] = ACTIONS(357),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym_hash_key_symbol] = ACTIONS(363),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(367),
  },
  [453] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4990),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3801),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym__argument] = STATE(5018),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2008),
    [sym_identifier_suffix] = STATE(2009),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2946),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(453),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(373),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2040),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2044),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(391),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(373),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(395),
    [sym__splat_star] = ACTIONS(397),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym_hash_key_symbol] = ACTIONS(403),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(405),
  },
  [454] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3956),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1996),
    [sym_identifier_suffix] = STATE(1998),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(3139),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(454),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(569),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2048),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2052),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(587),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(569),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(589),
    [sym__splat_star] = ACTIONS(591),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym_hash_key_symbol] = ACTIONS(597),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(599),
  },
  [455] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5317),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3995),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym__argument] = STATE(5018),
    [sym_forward_argument] = STATE(5002),
    [sym_splat_argument] = STATE(5002),
    [sym_hash_splat_argument] = STATE(5002),
    [sym_block_argument] = STATE(5002),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2003),
    [sym_identifier_suffix] = STATE(2005),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(455),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5002),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2298),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2158),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2298),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__block_ampersand] = ACTIONS(713),
    [sym__splat_star] = ACTIONS(715),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [456] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4482),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2773),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym__argument] = STATE(4502),
    [sym_forward_argument] = STATE(4512),
    [sym_splat_argument] = STATE(4512),
    [sym_hash_splat_argument] = STATE(4512),
    [sym_block_argument] = STATE(4512),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1965),
    [sym_identifier_suffix] = STATE(1967),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2410),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(456),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_pair] = STATE(4512),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(429),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1261),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(429),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__block_ampersand] = ACTIONS(449),
    [sym__splat_star] = ACTIONS(451),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym_hash_key_symbol] = ACTIONS(457),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
    [sym__hash_splat_star_star] = ACTIONS(459),
  },
  [457] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_rescue_modifier_expression] = STATE(4994),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5221),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3802),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3710),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(457),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2394),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [458] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_rescue_modifier_expression] = STATE(4994),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5392),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3867),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3574),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(458),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2404),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [459] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4535),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2658),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2660),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(459),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2408),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [460] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4592),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2715),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2714),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(460),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [461] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_rescue_modifier_expression] = STATE(4994),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5253),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3867),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3574),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(461),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2404),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [462] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4592),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2715),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2714),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(462),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [463] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4747),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3211),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3193),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(463),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2432),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [464] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4586),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2705),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2713),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(464),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [465] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_rescue_modifier_expression] = STATE(4994),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5440),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3910),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(4012),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(465),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2394),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [466] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(5236),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3673),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3789),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(466),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2434),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [467] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_rescue_modifier_expression] = STATE(4994),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5392),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3892),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3934),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(467),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2404),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [468] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4797),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3342),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3348),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(468),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2436),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [469] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4847),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3227),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3276),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(469),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2436),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [470] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(5138),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3625),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3784),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(470),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2434),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [471] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4771),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3211),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3193),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(471),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2432),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [472] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4918),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3500),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3566),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(472),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2458),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [473] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4535),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2658),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2660),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(473),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2408),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [474] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4645),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2912),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2913),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(474),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2460),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [475] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_rescue_modifier_expression] = STATE(4994),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4956),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3588),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3567),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(475),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2462),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [476] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4592),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2705),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2713),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(476),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [477] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_rescue_modifier_expression] = STATE(4994),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4940),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3601),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3711),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(477),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2464),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [478] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4847),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3342),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3348),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(478),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2436),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [479] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4592),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2705),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2713),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(479),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [480] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4586),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2705),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2713),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(480),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [481] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_rescue_modifier_expression] = STATE(4508),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4771),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3182),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3183),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(481),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2432),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [482] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2706),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(482),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2366),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2466),
    [anon_sym_rescue] = ACTIONS(2466),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2466),
    [anon_sym_do] = ACTIONS(2466),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2466),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_elsif] = ACTIONS(2466),
    [anon_sym_else] = ACTIONS(2466),
    [anon_sym_then] = ACTIONS(2466),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2466),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2366),
    [sym__no_line_break] = ACTIONS(2468),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [483] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2702),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(483),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2470),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2472),
    [anon_sym_rescue] = ACTIONS(2472),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2472),
    [anon_sym_do] = ACTIONS(2472),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2472),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_elsif] = ACTIONS(2472),
    [anon_sym_else] = ACTIONS(2472),
    [anon_sym_then] = ACTIONS(2472),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2472),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2470),
    [sym__no_line_break] = ACTIONS(2474),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [484] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4855),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3458),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(484),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [485] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2821),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(485),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2366),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2466),
    [anon_sym_rescue] = ACTIONS(2466),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2466),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2466),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_elsif] = ACTIONS(2466),
    [anon_sym_else] = ACTIONS(2466),
    [anon_sym_then] = ACTIONS(2466),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2466),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2366),
    [sym__no_line_break] = ACTIONS(2468),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [486] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(5244),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3926),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(19),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5729),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(62),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2328),
    [sym__variable] = STATE(17),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(486),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2476),
    [anon_sym_defined_QMARK] = ACTIONS(2478),
    [anon_sym_not] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_TILDE] = ACTIONS(2482),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2484),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [487] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4777),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3278),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(487),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [488] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4798),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3458),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(488),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [489] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(5139),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3886),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(489),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [490] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_rescue_modifier_expression] = STATE(4997),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5450),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4125),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(38),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5783),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(94),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2502),
    [sym__variable] = STATE(39),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(490),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2486),
    [anon_sym_defined_QMARK] = ACTIONS(2488),
    [anon_sym_not] = ACTIONS(2490),
    [anon_sym_BANG] = ACTIONS(2492),
    [anon_sym_TILDE] = ACTIONS(2492),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2494),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [491] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2820),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(491),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2470),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2472),
    [anon_sym_rescue] = ACTIONS(2472),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2472),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2472),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_elsif] = ACTIONS(2472),
    [anon_sym_else] = ACTIONS(2472),
    [anon_sym_then] = ACTIONS(2472),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2472),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2470),
    [sym__no_line_break] = ACTIONS(2474),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [492] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4924),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3568),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(492),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [493] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4596),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2779),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(493),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [494] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4596),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2779),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(19),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5729),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(62),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2328),
    [sym__variable] = STATE(17),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(494),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2476),
    [anon_sym_defined_QMARK] = ACTIONS(2478),
    [anon_sym_not] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_TILDE] = ACTIONS(2482),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2484),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [495] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2674),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(495),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(945),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(945),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_elsif] = ACTIONS(945),
    [anon_sym_else] = ACTIONS(945),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [496] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(496),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2496),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2496),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2498),
    [anon_sym_rescue] = ACTIONS(2498),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2498),
    [anon_sym_do] = ACTIONS(2498),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2498),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_elsif] = ACTIONS(2498),
    [anon_sym_else] = ACTIONS(2498),
    [anon_sym_then] = ACTIONS(2498),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2498),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2496),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [497] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4574),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2771),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(497),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [498] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2712),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(498),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2500),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2500),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2502),
    [anon_sym_rescue] = ACTIONS(2502),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2502),
    [anon_sym_do] = ACTIONS(2502),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2502),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_elsif] = ACTIONS(2502),
    [anon_sym_else] = ACTIONS(2502),
    [anon_sym_then] = ACTIONS(2502),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2502),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2500),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [499] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_rescue_modifier_expression] = STATE(4997),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4957),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3742),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(499),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [500] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4663),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3158),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(500),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [501] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_rescue_modifier_expression] = STATE(4997),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5255),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3933),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(501),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [502] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4596),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2771),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(502),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [503] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4748),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3278),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(503),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [504] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_rescue_modifier_expression] = STATE(4997),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5396),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4063),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(38),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5783),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(94),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2502),
    [sym__variable] = STATE(39),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(504),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2486),
    [anon_sym_defined_QMARK] = ACTIONS(2488),
    [anon_sym_not] = ACTIONS(2490),
    [anon_sym_BANG] = ACTIONS(2492),
    [anon_sym_TILDE] = ACTIONS(2492),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2494),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [505] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_rescue_modifier_expression] = STATE(4997),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5224),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3911),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(505),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [506] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_rescue_modifier_expression] = STATE(4997),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4941),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3643),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(506),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [507] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4538),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2685),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(507),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [508] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4574),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2771),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(508),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [509] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4596),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2771),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(509),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [510] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4855),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3479),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(510),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [511] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4538),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2685),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(511),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [512] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2710),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(512),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2504),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2504),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2506),
    [anon_sym_rescue] = ACTIONS(2506),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2506),
    [anon_sym_do] = ACTIONS(2506),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2506),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_elsif] = ACTIONS(2506),
    [anon_sym_else] = ACTIONS(2506),
    [anon_sym_then] = ACTIONS(2506),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2506),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2504),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [513] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_rescue_modifier_expression] = STATE(4510),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4777),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3466),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(513),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [514] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_rescue_modifier_expression] = STATE(4997),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5396),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3933),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(514),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [515] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2793),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(515),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2500),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2500),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2502),
    [anon_sym_rescue] = ACTIONS(2502),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2502),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2502),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_elsif] = ACTIONS(2502),
    [anon_sym_else] = ACTIONS(2502),
    [anon_sym_then] = ACTIONS(2502),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2502),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2500),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [516] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2725),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(516),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2496),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2496),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2498),
    [anon_sym_rescue] = ACTIONS(2498),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2498),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2498),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_elsif] = ACTIONS(2498),
    [anon_sym_else] = ACTIONS(2498),
    [anon_sym_then] = ACTIONS(2498),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2498),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2496),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [517] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2762),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(517),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(943),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(945),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(945),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_elsif] = ACTIONS(945),
    [anon_sym_else] = ACTIONS(945),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(945),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [518] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2796),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(518),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_RBRACE] = ACTIONS(2504),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_RPAREN] = ACTIONS(2504),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_end] = ACTIONS(2506),
    [anon_sym_rescue] = ACTIONS(2506),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_in] = ACTIONS(2506),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_when] = ACTIONS(2506),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_elsif] = ACTIONS(2506),
    [anon_sym_else] = ACTIONS(2506),
    [anon_sym_then] = ACTIONS(2506),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_ensure] = ACTIONS(2506),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2504),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [519] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(5311),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4008),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(519),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [520] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4594),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(520),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [521] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4597),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(521),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [522] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4598),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(522),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [523] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4601),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(523),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [524] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4511),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(524),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [525] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2786),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(525),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [526] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3274),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(526),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [527] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3216),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(527),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [528] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3217),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(528),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [529] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3955),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(529),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [530] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(530),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [531] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2737),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(531),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [532] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2739),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(532),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [533] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(533),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [534] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3077),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(534),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [535] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3818),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(535),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2044),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(391),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [536] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5017),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3853),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(536),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2044),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(391),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [537] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5019),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3858),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(537),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2044),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(391),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [538] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5020),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3786),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(538),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2044),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(391),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [539] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3077),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(539),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [540] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(5369),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4008),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(540),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [541] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3489),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(541),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [542] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4820),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(542),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [543] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4821),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(543),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [544] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4006),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(544),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [545] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5017),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4013),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(545),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [546] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4038),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(546),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [547] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3662),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(547),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [548] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3459),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(548),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [549] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3437),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(549),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [550] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3438),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(550),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [551] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(551),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [552] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4030),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(552),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [553] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4034),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(553),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [554] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3704),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(554),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [555] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5019),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4014),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(555),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [556] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5020),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3786),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(556),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [557] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5511),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4147),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(557),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [558] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3126),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(558),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [559] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4511),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4008),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(559),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [560] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4792),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(560),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [561] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3388),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(561),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [562] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3818),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(35),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(378),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5746),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(76),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2403),
    [sym__variable] = STATE(36),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(562),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(377),
    [anon_sym_yield] = ACTIONS(379),
    [anon_sym_break] = ACTIONS(381),
    [anon_sym_next] = ACTIONS(383),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2044),
    [anon_sym_defined_QMARK] = ACTIONS(385),
    [anon_sym_not] = ACTIONS(387),
    [anon_sym_BANG] = ACTIONS(391),
    [anon_sym_TILDE] = ACTIONS(391),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(399),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [563] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(5312),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4008),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(563),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [564] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4752),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(564),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [565] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4801),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(565),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [566] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4966),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4036),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(566),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [567] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3489),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(567),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [568] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3519),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(568),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [569] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4148),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(569),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [570] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3537),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(570),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [571] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(571),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [572] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3955),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(572),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [573] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5017),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3958),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(573),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [574] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5019),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3959),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(574),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [575] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5020),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3641),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5722),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2474),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(575),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2090),
    [anon_sym_defined_QMARK] = ACTIONS(549),
    [anon_sym_not] = ACTIONS(551),
    [anon_sym_BANG] = ACTIONS(555),
    [anon_sym_TILDE] = ACTIONS(555),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(561),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [576] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5876),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2136),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(576),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1265),
    [anon_sym_defined_QMARK] = ACTIONS(441),
    [anon_sym_not] = ACTIONS(443),
    [anon_sym_BANG] = ACTIONS(447),
    [anon_sym_TILDE] = ACTIONS(447),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(453),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [577] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4871),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(577),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [578] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4872),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(578),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [579] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4877),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(579),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [580] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4878),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(580),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [581] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4880),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(581),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [582] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4511),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3520),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(29),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5867),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(71),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2201),
    [sym__variable] = STATE(32),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(582),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2198),
    [anon_sym_defined_QMARK] = ACTIONS(2200),
    [anon_sym_not] = ACTIONS(2202),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2206),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [583] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4511),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3360),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(583),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [584] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4107),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(38),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5783),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(94),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2502),
    [sym__variable] = STATE(39),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(584),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2486),
    [anon_sym_defined_QMARK] = ACTIONS(2488),
    [anon_sym_not] = ACTIONS(2490),
    [anon_sym_BANG] = ACTIONS(2492),
    [anon_sym_TILDE] = ACTIONS(2492),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2494),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [585] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3388),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(585),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [586] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2695),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(586),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [587] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3400),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(587),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [588] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4006),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(37),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5843),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(70),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2499),
    [sym__variable] = STATE(34),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(588),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(703),
    [anon_sym_defined_QMARK] = ACTIONS(613),
    [anon_sym_not] = ACTIONS(615),
    [anon_sym_BANG] = ACTIONS(619),
    [anon_sym_TILDE] = ACTIONS(619),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(625),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [589] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4148),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(589),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [590] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5017),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4149),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(590),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [591] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5019),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4150),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(591),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [592] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5020),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3786),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(592),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [593] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3405),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(11),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5899),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(57),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2182),
    [sym__variable] = STATE(8),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(593),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(67),
    [anon_sym_defined_QMARK] = ACTIONS(69),
    [anon_sym_not] = ACTIONS(71),
    [anon_sym_BANG] = ACTIONS(73),
    [anon_sym_TILDE] = ACTIONS(73),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(109),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [594] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3459),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5692),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2199),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(594),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1910),
    [anon_sym_defined_QMARK] = ACTIONS(515),
    [anon_sym_not] = ACTIONS(517),
    [anon_sym_BANG] = ACTIONS(521),
    [anon_sym_TILDE] = ACTIONS(521),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(527),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [595] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(4966),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4147),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(595),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [596] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(596),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [597] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3613),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(597),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1976),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [598] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3614),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(598),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1976),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [599] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3704),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(599),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1976),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [600] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(4038),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(22),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5724),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(78),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2476),
    [sym__variable] = STATE(23),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(600),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2508),
    [anon_sym_defined_QMARK] = ACTIONS(2510),
    [anon_sym_not] = ACTIONS(2512),
    [anon_sym_BANG] = ACTIONS(2514),
    [anon_sym_TILDE] = ACTIONS(2514),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2516),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [601] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2786),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(601),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [602] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3126),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(602),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [603] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3145),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(603),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [604] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3924),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(604),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2052),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(587),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [605] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3146),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(605),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [606] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3077),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2721),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(9),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(316),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5865),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(53),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2171),
    [sym__variable] = STATE(10),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(606),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(187),
    [anon_sym_yield] = ACTIONS(189),
    [anon_sym_break] = ACTIONS(191),
    [anon_sym_next] = ACTIONS(193),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1575),
    [anon_sym_defined_QMARK] = ACTIONS(195),
    [anon_sym_not] = ACTIONS(197),
    [anon_sym_BANG] = ACTIONS(201),
    [anon_sym_TILDE] = ACTIONS(201),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(207),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [607] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2822),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(607),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [608] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2824),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(608),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [609] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(609),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [610] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3679),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(610),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1976),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [611] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(611),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [612] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2786),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(19),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5729),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(62),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2328),
    [sym__variable] = STATE(17),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(612),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2476),
    [anon_sym_defined_QMARK] = ACTIONS(2478),
    [anon_sym_not] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_TILDE] = ACTIONS(2482),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2484),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [613] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2695),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(613),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [614] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3662),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(614),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [615] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3924),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(615),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2052),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(587),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [616] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3907),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(616),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2052),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(587),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [617] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3908),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(617),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2052),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(587),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [618] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3704),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(30),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(381),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5816),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(61),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2475),
    [sym__variable] = STATE(31),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(618),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(573),
    [anon_sym_yield] = ACTIONS(575),
    [anon_sym_break] = ACTIONS(577),
    [anon_sym_next] = ACTIONS(579),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2052),
    [anon_sym_defined_QMARK] = ACTIONS(581),
    [anon_sym_not] = ACTIONS(583),
    [anon_sym_BANG] = ACTIONS(587),
    [anon_sym_TILDE] = ACTIONS(587),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(593),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [619] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5017),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3701),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(619),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [620] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5507),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4147),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(41),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(398),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5698),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(104),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2524),
    [sym__variable] = STATE(40),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(620),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_yield] = ACTIONS(607),
    [anon_sym_break] = ACTIONS(609),
    [anon_sym_next] = ACTIONS(611),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2518),
    [anon_sym_defined_QMARK] = ACTIONS(2520),
    [anon_sym_not] = ACTIONS(2522),
    [anon_sym_BANG] = ACTIONS(2524),
    [anon_sym_TILDE] = ACTIONS(2524),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2526),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [621] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5019),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3703),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(621),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [622] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5020),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3641),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3279),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(27),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(363),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5651),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(66),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2274),
    [sym__variable] = STATE(33),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(622),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(285),
    [anon_sym_yield] = ACTIONS(287),
    [anon_sym_break] = ACTIONS(289),
    [anon_sym_next] = ACTIONS(291),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1922),
    [anon_sym_defined_QMARK] = ACTIONS(317),
    [anon_sym_not] = ACTIONS(319),
    [anon_sym_BANG] = ACTIONS(323),
    [anon_sym_TILDE] = ACTIONS(323),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(359),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [623] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4107),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(38),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5783),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(94),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2502),
    [sym__variable] = STATE(39),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(623),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2486),
    [anon_sym_defined_QMARK] = ACTIONS(2488),
    [anon_sym_not] = ACTIONS(2490),
    [anon_sym_BANG] = ACTIONS(2492),
    [anon_sym_TILDE] = ACTIONS(2492),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2494),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [624] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5017),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4110),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(38),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5783),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(94),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2502),
    [sym__variable] = STATE(39),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(624),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2486),
    [anon_sym_defined_QMARK] = ACTIONS(2488),
    [anon_sym_not] = ACTIONS(2490),
    [anon_sym_BANG] = ACTIONS(2492),
    [anon_sym_TILDE] = ACTIONS(2492),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2494),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [625] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3955),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(625),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [626] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5017),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3958),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(626),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [627] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5019),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3959),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(627),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [628] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5020),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3641),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(628),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [629] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5019),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(4111),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(38),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5783),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(94),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2502),
    [sym__variable] = STATE(39),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(629),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2486),
    [anon_sym_defined_QMARK] = ACTIONS(2488),
    [anon_sym_not] = ACTIONS(2490),
    [anon_sym_BANG] = ACTIONS(2492),
    [anon_sym_TILDE] = ACTIONS(2492),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2494),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [630] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5020),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3641),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(38),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5783),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(94),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2502),
    [sym__variable] = STATE(39),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(630),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2486),
    [anon_sym_defined_QMARK] = ACTIONS(2488),
    [anon_sym_not] = ACTIONS(2490),
    [anon_sym_BANG] = ACTIONS(2492),
    [anon_sym_TILDE] = ACTIONS(2492),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2494),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [631] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return_command] = STATE(4943),
    [sym_yield_command] = STATE(4943),
    [sym_break_command] = STATE(4943),
    [sym_next_command] = STATE(4943),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__expression] = STATE(5015),
    [sym_match_pattern] = STATE(4943),
    [sym_test_pattern] = STATE(4943),
    [sym__arg] = STATE(3955),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3515),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(24),
    [sym_command_call] = STATE(4948),
    [sym_command_call_with_block] = STATE(4796),
    [sym__chained_command_call] = STATE(390),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_command_assignment] = STATE(4943),
    [sym_operator_assignment] = STATE(3465),
    [sym_command_operator_assignment] = STATE(4943),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_command_binary] = STATE(4943),
    [sym_unary] = STATE(3465),
    [sym_command_unary] = STATE(4943),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5766),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(65),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2523),
    [sym__variable] = STATE(25),
    [sym__nonlocal_variable] = STATE(1954),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2004),
    [sym_identifier_suffix] = STATE(2006),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(631),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(327),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(541),
    [anon_sym_yield] = ACTIONS(543),
    [anon_sym_break] = ACTIONS(545),
    [anon_sym_next] = ACTIONS(547),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2396),
    [anon_sym_defined_QMARK] = ACTIONS(2398),
    [anon_sym_not] = ACTIONS(2400),
    [anon_sym_BANG] = ACTIONS(2402),
    [anon_sym_TILDE] = ACTIONS(2402),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(327),
    [sym_self] = ACTIONS(327),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(327),
    [aux_sym_constant_suffix_token1] = ACTIONS(331),
    [aux_sym_identifier_suffix_token1] = ACTIONS(333),
    [sym_instance_variable] = ACTIONS(335),
    [sym_class_variable] = ACTIONS(335),
    [sym_global_variable] = ACTIONS(335),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2406),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(365),
    [sym__constant_suffix] = ACTIONS(331),
  },
  [632] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2786),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(19),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5729),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(62),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2328),
    [sym__variable] = STATE(17),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(632),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2476),
    [anon_sym_defined_QMARK] = ACTIONS(2478),
    [anon_sym_not] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_TILDE] = ACTIONS(2482),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2484),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [633] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2822),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(19),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5729),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(62),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2328),
    [sym__variable] = STATE(17),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(633),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2476),
    [anon_sym_defined_QMARK] = ACTIONS(2478),
    [anon_sym_not] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_TILDE] = ACTIONS(2482),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2484),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [634] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2824),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(19),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5729),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(62),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2328),
    [sym__variable] = STATE(17),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(634),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2476),
    [anon_sym_defined_QMARK] = ACTIONS(2478),
    [anon_sym_not] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_TILDE] = ACTIONS(2482),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2484),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [635] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(19),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5729),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(62),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2328),
    [sym__variable] = STATE(17),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(635),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2476),
    [anon_sym_defined_QMARK] = ACTIONS(2478),
    [anon_sym_not] = ACTIONS(2480),
    [anon_sym_BANG] = ACTIONS(2482),
    [anon_sym_TILDE] = ACTIONS(2482),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2484),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [636] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3274),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(636),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [637] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3216),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(637),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [638] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3217),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(638),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [639] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3077),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(639),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [640] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(640),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [641] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2737),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(641),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [642] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2739),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(642),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [643] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3274),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5878),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2180),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(643),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2448),
    [anon_sym_defined_QMARK] = ACTIONS(2450),
    [anon_sym_not] = ACTIONS(2452),
    [anon_sym_BANG] = ACTIONS(2454),
    [anon_sym_TILDE] = ACTIONS(2454),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2456),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [644] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5716),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2342),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(644),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2410),
    [anon_sym_defined_QMARK] = ACTIONS(2412),
    [anon_sym_not] = ACTIONS(2414),
    [anon_sym_BANG] = ACTIONS(2416),
    [anon_sym_TILDE] = ACTIONS(2416),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2420),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [645] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3274),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2860),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(13),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4607),
    [sym__chained_command_call] = STATE(356),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5835),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(54),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2194),
    [sym__variable] = STATE(12),
    [sym__nonlocal_variable] = STATE(1924),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1987),
    [sym_identifier_suffix] = STATE(1988),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(645),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(37),
    [anon_sym_yield] = ACTIONS(39),
    [anon_sym_break] = ACTIONS(41),
    [anon_sym_next] = ACTIONS(43),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1886),
    [anon_sym_defined_QMARK] = ACTIONS(465),
    [anon_sym_not] = ACTIONS(467),
    [anon_sym_BANG] = ACTIONS(471),
    [anon_sym_TILDE] = ACTIONS(471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(9),
    [sym_self] = ACTIONS(9),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(9),
    [aux_sym_constant_suffix_token1] = ACTIONS(85),
    [aux_sym_identifier_suffix_token1] = ACTIONS(87),
    [sym_instance_variable] = ACTIONS(89),
    [sym_class_variable] = ACTIONS(89),
    [sym_global_variable] = ACTIONS(89),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(477),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(115),
    [sym__constant_suffix] = ACTIONS(85),
  },
  [646] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4595),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2812),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(4),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5811),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(44),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2132),
    [sym__variable] = STATE(3),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(646),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_defined_QMARK] = ACTIONS(659),
    [anon_sym_not] = ACTIONS(661),
    [anon_sym_BANG] = ACTIONS(663),
    [anon_sym_TILDE] = ACTIONS(663),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(677),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [647] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2695),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(647),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [648] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2682),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(648),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [649] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2683),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(649),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [650] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(650),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [651] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2506),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(7),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(248),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5737),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(43),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2124),
    [sym__variable] = STATE(2),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(651),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(133),
    [anon_sym_yield] = ACTIONS(135),
    [anon_sym_break] = ACTIONS(137),
    [anon_sym_next] = ACTIONS(139),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1075),
    [anon_sym_defined_QMARK] = ACTIONS(143),
    [anon_sym_not] = ACTIONS(145),
    [anon_sym_BANG] = ACTIONS(149),
    [anon_sym_TILDE] = ACTIONS(149),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(163),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [652] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2695),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(652),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [653] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2682),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(653),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [654] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(654),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [655] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2737),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(655),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [656] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2739),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(656),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [657] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3417),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(14),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(369),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5757),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(55),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2442),
    [sym__variable] = STATE(15),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(657),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(485),
    [anon_sym_yield] = ACTIONS(487),
    [anon_sym_break] = ACTIONS(489),
    [anon_sym_next] = ACTIONS(491),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1970),
    [anon_sym_defined_QMARK] = ACTIONS(493),
    [anon_sym_not] = ACTIONS(495),
    [anon_sym_BANG] = ACTIONS(499),
    [anon_sym_TILDE] = ACTIONS(499),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(501),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [658] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(658),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [659] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2737),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(659),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [660] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2739),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(660),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [661] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(661),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [662] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3459),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(662),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [663] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4484),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3437),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(663),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [664] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3438),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(664),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [665] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4507),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2693),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(665),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [666] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2738),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2552),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(6),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(258),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5676),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(42),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2130),
    [sym__variable] = STATE(5),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(666),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(433),
    [anon_sym_yield] = ACTIONS(435),
    [anon_sym_break] = ACTIONS(437),
    [anon_sym_next] = ACTIONS(439),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2422),
    [anon_sym_defined_QMARK] = ACTIONS(2424),
    [anon_sym_not] = ACTIONS(2426),
    [anon_sym_BANG] = ACTIONS(2428),
    [anon_sym_TILDE] = ACTIONS(2428),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2430),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [667] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3459),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2968),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(21),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(361),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5707),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(67),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2214),
    [sym__variable] = STATE(28),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(667),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(507),
    [anon_sym_yield] = ACTIONS(509),
    [anon_sym_break] = ACTIONS(511),
    [anon_sym_next] = ACTIONS(513),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2438),
    [anon_sym_defined_QMARK] = ACTIONS(2440),
    [anon_sym_not] = ACTIONS(2442),
    [anon_sym_BANG] = ACTIONS(2444),
    [anon_sym_TILDE] = ACTIONS(2444),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2446),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [668] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4498),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(2683),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(16),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(338),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5750),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(56),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2250),
    [sym__variable] = STATE(18),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(668),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(215),
    [anon_sym_yield] = ACTIONS(217),
    [anon_sym_break] = ACTIONS(219),
    [anon_sym_next] = ACTIONS(221),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1753),
    [anon_sym_defined_QMARK] = ACTIONS(223),
    [anon_sym_not] = ACTIONS(225),
    [anon_sym_BANG] = ACTIONS(229),
    [anon_sym_TILDE] = ACTIONS(229),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(231),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [669] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return_command] = STATE(4509),
    [sym_yield_command] = STATE(4509),
    [sym_break_command] = STATE(4509),
    [sym_next_command] = STATE(4509),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__expression] = STATE(4501),
    [sym_match_pattern] = STATE(4509),
    [sym_test_pattern] = STATE(4509),
    [sym__arg] = STATE(3679),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3176),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(20),
    [sym_command_call] = STATE(4515),
    [sym_command_call_with_block] = STATE(4471),
    [sym__chained_command_call] = STATE(370),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_command_assignment] = STATE(4509),
    [sym_operator_assignment] = STATE(2547),
    [sym_command_operator_assignment] = STATE(4509),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_command_binary] = STATE(4509),
    [sym_unary] = STATE(2547),
    [sym_command_unary] = STATE(4509),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5907),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(60),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2271),
    [sym__variable] = STATE(26),
    [sym__nonlocal_variable] = STATE(1909),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(1970),
    [sym_identifier_suffix] = STATE(1971),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(669),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(151),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(239),
    [anon_sym_break] = ACTIONS(241),
    [anon_sym_next] = ACTIONS(243),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1976),
    [anon_sym_defined_QMARK] = ACTIONS(245),
    [anon_sym_not] = ACTIONS(247),
    [anon_sym_BANG] = ACTIONS(251),
    [anon_sym_TILDE] = ACTIONS(251),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(151),
    [sym_self] = ACTIONS(151),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(151),
    [aux_sym_constant_suffix_token1] = ACTIONS(153),
    [aux_sym_identifier_suffix_token1] = ACTIONS(155),
    [sym_instance_variable] = ACTIONS(157),
    [sym_class_variable] = ACTIONS(157),
    [sym_global_variable] = ACTIONS(157),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(259),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(169),
    [sym__constant_suffix] = ACTIONS(153),
  },
  [670] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__hash_pattern_body] = STATE(5701),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5506),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3584),
    [sym_identifier_suffix] = STATE(3592),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3565),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(670),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5506),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2528),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2530),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2528),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [671] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__hash_pattern_body] = STATE(5829),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3584),
    [sym_identifier_suffix] = STATE(3592),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3565),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(671),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2528),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2532),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2528),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [672] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__hash_pattern_body] = STATE(5829),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5506),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3584),
    [sym_identifier_suffix] = STATE(3592),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3565),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(672),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5506),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2528),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2534),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2528),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [673] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__hash_pattern_body] = STATE(5701),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3584),
    [sym_identifier_suffix] = STATE(3592),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3565),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(673),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2528),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2536),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2528),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [674] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__hash_pattern_body] = STATE(5701),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5506),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3584),
    [sym_identifier_suffix] = STATE(3592),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3565),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(674),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5506),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2528),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2538),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(131),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2528),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [675] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3228),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(675),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2470),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2472),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2472),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2472),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2470),
    [sym__no_line_break] = ACTIONS(2474),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [676] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3229),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(676),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2366),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2466),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2466),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2466),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2366),
    [sym__no_line_break] = ACTIONS(2468),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [677] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_then] = STATE(5476),
    [sym_begin] = STATE(2376),
    [sym_exceptions] = STATE(5060),
    [sym_exception_variable] = STATE(5320),
    [sym__arg] = STATE(3922),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(5217),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(677),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym__terminator] = STATE(282),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_SEMI] = ACTIONS(751),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(2540),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(2542),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(751),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2544),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [678] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_primitive] = STATE(4789),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3821),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3729),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2540),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3461),
    [sym_subshell] = STATE(3729),
    [sym_string_array] = STATE(3729),
    [sym_symbol_array] = STATE(3729),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3729),
    [sym_heredoc_body] = STATE(678),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3731),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2496),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(2546),
    [sym_true] = ACTIONS(707),
    [sym_false] = ACTIONS(707),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(711),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [679] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3083),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(679),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(943),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(945),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [680] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3140),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(680),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2496),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2498),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2498),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2498),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2496),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [681] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3510),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(681),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2366),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2466),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2466),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2366),
    [sym__no_line_break] = ACTIONS(2468),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [682] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3225),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(682),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2500),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2502),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2502),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2502),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2500),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [683] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3483),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(683),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2470),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2472),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2472),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2470),
    [sym__no_line_break] = ACTIONS(2474),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [684] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3218),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(684),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2504),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2506),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2506),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2506),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2504),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [685] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3215),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(685),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2496),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2498),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2498),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2496),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [686] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3484),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(686),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2504),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2506),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2506),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2504),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [687] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3326),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(687),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(943),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(945),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [688] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3903),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(688),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2472),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2472),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2470),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2548),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [689] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3904),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(689),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2466),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2466),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2366),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2370),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [690] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3499),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(690),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [ts_builtin_sym_end] = ACTIONS(2500),
    [sym_identifier] = ACTIONS(901),
    [aux_sym_program_token1] = ACTIONS(2502),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2502),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2500),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [691] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3557),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(691),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2466),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2466),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2366),
    [sym__no_line_break] = ACTIONS(2468),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [692] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3556),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(692),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2472),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2472),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2470),
    [sym__no_line_break] = ACTIONS(2474),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [693] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3990),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(693),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2466),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_then] = ACTIONS(2466),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2366),
    [sym__no_line_break] = ACTIONS(2468),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [694] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3931),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(694),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2472),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_then] = ACTIONS(2472),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2470),
    [sym__no_line_break] = ACTIONS(2474),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [695] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3552),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(695),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2502),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2502),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2500),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [696] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4060),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(696),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_SEMI] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(2466),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2366),
    [sym__no_line_break] = ACTIONS(2468),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [697] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_primitive] = STATE(4789),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3642),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2872),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(697),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [698] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3906),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2240),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(698),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [699] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3620),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(699),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [700] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3553),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(700),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2506),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2506),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2504),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [701] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3452),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(701),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [702] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3436),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(702),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_rescue] = ACTIONS(2498),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2498),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2496),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [703] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3140),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2240),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(703),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [704] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_primitive] = STATE(4789),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3821),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2872),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(704),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [705] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_primitive] = STATE(4789),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3800),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2872),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(705),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [706] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4108),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(706),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2472),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2470),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2548),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [707] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3901),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(707),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2500),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2502),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2502),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2500),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [708] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3929),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(708),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2502),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_then] = ACTIONS(2502),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2500),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [709] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3930),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(709),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2506),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_then] = ACTIONS(2506),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2504),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [710] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_primitive] = STATE(4789),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3821),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3729),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2540),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3461),
    [sym_subshell] = STATE(3729),
    [sym_string_array] = STATE(3729),
    [sym_symbol_array] = STATE(3729),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3729),
    [sym_heredoc_body] = STATE(710),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3731),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(2546),
    [sym_true] = ACTIONS(707),
    [sym_false] = ACTIONS(707),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(711),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [711] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3436),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2240),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(711),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [712] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4109),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(712),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2466),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2366),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2370),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [713] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3633),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(713),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_do] = ACTIONS(2498),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_then] = ACTIONS(2498),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2496),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [714] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3747),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(714),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [715] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_primitive] = STATE(4789),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3702),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3302),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2310),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2872),
    [sym_subshell] = STATE(3302),
    [sym_string_array] = STATE(3302),
    [sym_symbol_array] = STATE(3302),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3302),
    [sym_heredoc_body] = STATE(715),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3320),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1689),
    [sym_true] = ACTIONS(1691),
    [sym_false] = ACTIONS(1691),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1695),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [716] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4024),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(716),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2472),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2470),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2548),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [717] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3902),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(717),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2504),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2506),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2506),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2504),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [718] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2240),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(718),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [719] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3702),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(719),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2496),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2498),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2498),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2496),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [720] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3215),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2240),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(720),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [721] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2725),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2240),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(721),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [722] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4027),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(722),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2466),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2366),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2370),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [723] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4789),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2787),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(3171),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2219),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2811),
    [sym_subshell] = STATE(3171),
    [sym_string_array] = STATE(3171),
    [sym_symbol_array] = STATE(3171),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(3171),
    [sym_heredoc_body] = STATE(723),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(3079),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(2550),
    [sym_true] = ACTIONS(671),
    [sym_false] = ACTIONS(671),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(675),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [724] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3633),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2465),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2107),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2240),
    [sym_subshell] = STATE(2465),
    [sym_string_array] = STATE(2465),
    [sym_symbol_array] = STATE(2465),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2465),
    [sym_heredoc_body] = STATE(724),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2352),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(1655),
    [sym_true] = ACTIONS(1657),
    [sym_false] = ACTIONS(1657),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1661),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [725] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym__pattern_primitive] = STATE(4447),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3821),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(3125),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2251),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2792),
    [sym_subshell] = STATE(3125),
    [sym_string_array] = STATE(3125),
    [sym_symbol_array] = STATE(3125),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(3125),
    [sym_heredoc_body] = STATE(725),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3127),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1769),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1771),
    [sym_true] = ACTIONS(1773),
    [sym_false] = ACTIONS(1773),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1777),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(1779),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [726] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4059),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(726),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_SEMI] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(2472),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2470),
    [sym__no_line_break] = ACTIONS(2474),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [727] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3699),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(727),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(945),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [728] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4022),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(728),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2500),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2502),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2500),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [729] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4085),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(729),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2500),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2502),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2500),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [730] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4023),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(730),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2504),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2506),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2504),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [731] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3878),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(731),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [732] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3906),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(732),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_SEMI] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(2498),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2496),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [733] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4183),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(733),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2470),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2472),
    [anon_sym_or] = ACTIONS(2472),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2548),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [734] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4122),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(734),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_SEMI] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(2506),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2504),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [735] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4086),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(735),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2504),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2506),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2504),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [736] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3964),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(736),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [737] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3800),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(737),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2496),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2498),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2496),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [738] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3642),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(738),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2496),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_rescue] = ACTIONS(2498),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2496),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [739] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4121),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(739),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_SEMI] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(2502),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2500),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [740] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4186),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(740),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2366),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2466),
    [anon_sym_or] = ACTIONS(2466),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2370),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [741] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5526),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(741),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5526),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2554),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [742] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5618),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(742),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5618),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2556),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [743] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4152),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(743),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2504),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2504),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2506),
    [anon_sym_or] = ACTIONS(2506),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [744] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3798),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(744),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(945),
    [anon_sym_or] = ACTIONS(945),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [745] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5506),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(745),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5506),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2558),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [746] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5618),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(746),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5618),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2560),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [747] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4000),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(747),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_SEMI] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_EQ_GT] = ACTIONS(943),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_then] = ACTIONS(945),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(943),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [748] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5618),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(748),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5618),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2562),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [749] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4095),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(749),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2470),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2470),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2470),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2548),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [750] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4055),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(750),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2366),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2366),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2366),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2366),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym__no_line_break] = ACTIONS(2370),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [751] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5618),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(751),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5618),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2564),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [752] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5618),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(752),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5618),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2566),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [753] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4151),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(753),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2500),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2500),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2502),
    [anon_sym_or] = ACTIONS(2502),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [754] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5503),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(754),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5503),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2568),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [755] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3821),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(755),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2496),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2496),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2496),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_and] = ACTIONS(2498),
    [anon_sym_or] = ACTIONS(2498),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [756] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5618),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(756),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5618),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2570),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [757] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3191),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3195),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(757),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2572),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [758] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3209),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3183),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(758),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2432),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [759] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3963),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(4012),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(759),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2394),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [760] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3899),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(4035),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(760),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2574),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [761] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3590),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3567),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(761),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2462),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [762] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2527),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2528),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(762),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2576),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [763] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3194),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3205),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(763),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2578),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [764] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2781),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2825),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(764),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2580),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [765] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3706),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3695),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(765),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2582),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [766] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3666),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3789),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(766),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2434),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [767] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2836),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2851),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(767),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2584),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [768] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4126),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(4179),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(768),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2586),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [769] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2709),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2701),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(769),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2588),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [770] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3532),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3595),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(770),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2590),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [771] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2533),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2526),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(771),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2576),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [772] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3206),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3247),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(772),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2590),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [773] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2477),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2480),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(773),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2592),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [774] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3200),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3271),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(774),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2594),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [775] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3529),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3566),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(775),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2458),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [776] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3749),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3879),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(776),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2596),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [777] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3349),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3427),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(777),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2598),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [778] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3757),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3648),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(778),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2582),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [779] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3352),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3367),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(779),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2598),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [780] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3639),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3769),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(780),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2596),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [781] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3526),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3559),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(781),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2600),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [782] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3089),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3190),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(782),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2602),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [783] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3748),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3784),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(783),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2434),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [784] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3600),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3574),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(784),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2404),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [785] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3587),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3711),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(785),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2464),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [786] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3935),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3934),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(786),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2404),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [787] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2663),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2660),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(787),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2408),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [788] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2928),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2913),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(788),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2460),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [789] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4307),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_hash_splat_argument] = STATE(5618),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(3571),
    [sym_identifier_suffix] = STATE(3550),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3184),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(789),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_pair] = STATE(5618),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(2552),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(2552),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym_hash_key_symbol] = ACTIONS(719),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
    [sym__hash_splat_star_star] = ACTIONS(721),
  },
  [790] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2698),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2714),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(790),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [791] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2874),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3502),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3514),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(2874),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(791),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2572),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [792] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3179),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3193),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(792),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2432),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [793] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3246),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3276),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(793),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2436),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [794] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2803),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2798),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(794),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2580),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [795] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4112),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(795),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_RBRACE] = ACTIONS(2500),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2500),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2500),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2500),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [796] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4113),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(4182),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(796),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2586),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [797] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3344),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3348),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(797),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2436),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [798] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2862),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2864),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(798),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2584),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [799] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3970),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(4033),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(799),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2574),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [800] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2358),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3530),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(3602),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2358),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(800),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2600),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [801] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4120),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(4061),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(801),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2604),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [802] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4117),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(4116),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(802),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2604),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [803] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2613),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2711),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2713),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_right_assignment_list] = STATE(2613),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(803),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [804] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3419),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3594),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3710),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_right_assignment_list] = STATE(3419),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(804),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2394),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [805] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4223),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(805),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2606),
    [anon_sym_PIPE] = ACTIONS(2606),
    [anon_sym_SEMI] = ACTIONS(2606),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [806] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4190),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(806),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_COMMA] = ACTIONS(2606),
    [anon_sym_SEMI] = ACTIONS(2606),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym__line_break] = ACTIONS(2606),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [807] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_pattern] = STATE(5375),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4076),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(5441),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(807),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2544),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [808] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4142),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(808),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2504),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2504),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(2504),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [809] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4145),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(809),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(943),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(943),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_RBRACK] = ACTIONS(943),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [810] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_pattern] = STATE(5051),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4076),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(5441),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(810),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2544),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [811] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3219),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(811),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2432),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [812] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2741),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(812),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2418),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [813] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3832),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3416),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(813),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2394),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [814] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2868),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(814),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2580),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [815] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3705),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(815),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2458),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [816] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3095),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(816),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2460),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [817] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3755),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3416),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(817),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2462),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [818] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2668),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(818),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2408),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [819] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4249),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3416),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(819),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2586),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [820] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4199),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(820),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2606),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_do] = ACTIONS(2608),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [821] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3808),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3025),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(821),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2582),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [822] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4009),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(5323),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(822),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2544),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [823] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3323),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3025),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(823),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2572),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [824] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3490),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(824),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2598),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [825] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2549),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(825),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2576),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [826] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3346),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(826),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2602),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [827] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2521),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(827),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2592),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [828] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4247),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(828),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_RPAREN] = ACTIONS(2606),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_COMMA] = ACTIONS(2606),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [829] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3387),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(829),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2436),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [830] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3685),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3416),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(830),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2404),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [831] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3421),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3025),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(831),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2594),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [832] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4056),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(832),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2574),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [833] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3381),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3025),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(833),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2590),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [834] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3823),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3416),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(834),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2464),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [835] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3967),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3025),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(835),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2596),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [836] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3265),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3025),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(836),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2578),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [837] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2717),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(837),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2588),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [838] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2934),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(838),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2584),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [839] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3962),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2569),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(839),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2434),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [840] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4181),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_splat_argument] = STATE(3416),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(840),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__splat_star] = ACTIONS(2604),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [841] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3637),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_splat_argument] = STATE(2389),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(841),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__splat_star] = ACTIONS(2600),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [842] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3884),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(842),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [843] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3439),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(843),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [844] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2718),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(844),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [845] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3162),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(845),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [846] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3159),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(846),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [847] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3406),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(847),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [848] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2671),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(848),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [849] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3758),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(849),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [850] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3249),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(850),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [851] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3177),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(851),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [852] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3927),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(852),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [853] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3968),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(853),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [854] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4219),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(854),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [855] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3912),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(855),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [856] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2834),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(856),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [857] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3261),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(857),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [858] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3272),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(858),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [859] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2810),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(859),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [860] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2555),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(860),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [861] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4062),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(861),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [862] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3763),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(862),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [863] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2638),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(863),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [864] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2496),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(864),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [865] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3243),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(865),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [866] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4137),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(866),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [867] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4118),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(867),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [868] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3751),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(868),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [869] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2500),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(869),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [870] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2666),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(870),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [871] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3603),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(871),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [872] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3281),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(872),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [873] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3898),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(873),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [874] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3765),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(874),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [875] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2943),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(875),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [876] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3188),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(876),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [877] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3482),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(877),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [878] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3608),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(878),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [879] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3036),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(879),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [880] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3280),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(880),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [881] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3677),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(881),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [882] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3549),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(882),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [883] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3321),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(883),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [884] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3103),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(884),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [885] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3432),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(885),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [886] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2592),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(886),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [887] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3814),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(887),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [888] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3040),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(888),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [889] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3982),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(889),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [890] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4025),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(890),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [891] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3538),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(891),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [892] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2359),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3572),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(892),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [893] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3678),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(893),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [894] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3745),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(894),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [895] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4248),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(895),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [896] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3708),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(896),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [897] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4185),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(897),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [898] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3621),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(898),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [899] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2770),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(899),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [900] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3474),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(900),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [901] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3212),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(901),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [902] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3839),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(902),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [903] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2829),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(903),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [904] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2876),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3296),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(904),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [905] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2758),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(905),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [906] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(3456),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4084),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(906),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [907] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(5390),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4032),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(907),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [908] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2914),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(908),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [909] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_rescue_modifier_arg] = STATE(2959),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3649),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(909),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [910] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2330),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3504),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(910),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [911] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_rescue_modifier_arg] = STATE(2614),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3580),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(911),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [912] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4348),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(912),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [913] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4269),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(913),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [914] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3491),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(914),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [915] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3476),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(915),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [916] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3491),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(916),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [917] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3266),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(917),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [918] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3305),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(918),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [919] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(919),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [920] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3327),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(920),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [921] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3329),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(921),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [922] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3330),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(922),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [923] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3331),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(923),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [924] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3333),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(924),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [925] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3335),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(925),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [926] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3338),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(926),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [927] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3339),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(927),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [928] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3340),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(928),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [929] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3341),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(929),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [930] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3354),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(930),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [931] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3345),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(931),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [932] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3570),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(932),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [933] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4024),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(933),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [934] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3475),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(934),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [935] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4039),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(935),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [936] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3998),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(936),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [937] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3999),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(937),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [938] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(938),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [939] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4095),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(939),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [940] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4222),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(940),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [941] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3386),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(941),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [942] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4041),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(942),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [943] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4019),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(943),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [944] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3992),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(944),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [945] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3993),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(945),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [946] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3324),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(946),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [947] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3545),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(947),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [948] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2783),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(948),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [949] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3994),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(949),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [950] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3996),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(950),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [951] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3797),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(951),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [952] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3800),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(952),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [953] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4002),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(953),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [954] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3997),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(954),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [955] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4003),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(955),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [956] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4004),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(956),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [957] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4289),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(957),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [958] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4283),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(958),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [959] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4284),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(959),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [960] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(960),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [961] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4005),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(961),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [962] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3864),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(962),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [963] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3881),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(963),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [964] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3764),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(964),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [965] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3803),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(965),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [966] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3420),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(966),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [967] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3811),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(967),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [968] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3812),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(968),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [969] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3830),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(969),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [970] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3761),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(970),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [971] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3841),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(971),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [972] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3842),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(972),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [973] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3847),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(973),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [974] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4031),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(974),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [975] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3363),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(975),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [976] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3848),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(976),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [977] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2820),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(977),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [978] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4059),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(978),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [979] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3883),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(979),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [980] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4280),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(980),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [981] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3862),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(981),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [982] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4141),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(982),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [983] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4173),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(983),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [984] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4174),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(984),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [985] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(985),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [986] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4155),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(986),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [987] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4156),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(987),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [988] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4157),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(988),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [989] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4158),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(989),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [990] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4159),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(990),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [991] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4160),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(991),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [992] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4177),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(992),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [993] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4161),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(993),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [994] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4162),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(994),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [995] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4163),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(995),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [996] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4164),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(996),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [997] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3364),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(997),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [998] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(998),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [999] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4165),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(999),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1000] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4180),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1000),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1001] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3937),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1001),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1002] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3586),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1002),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1003] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3645),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1003),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1004] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3697),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1004),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1005] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3698),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1005),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1006] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1006),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1007] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3650),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1007),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1008] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3651),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1008),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1009] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3652),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1009),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1010] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3653),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1010),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1011] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3655),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1011),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1012] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3664),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1012),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1013] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3740),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1013),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1014] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3668),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1014),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1015] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3669),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1015),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1016] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3670),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1016),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1017] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3671),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1017),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1018] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3675),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1018),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1019] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3754),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1019),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1020] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4257),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1020),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1021] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3683),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1021),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1022] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3932),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1022),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1023] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3965),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1023),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1024] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3966),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1024),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1025] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1025),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1026] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3984),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1026),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1027] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3914),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1027),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1028] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3928),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1028),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1029] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3936),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1029),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1030] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3942),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1030),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1031] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3961),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1031),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1032] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3971),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1032),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1033] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3972),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1033),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1034] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3973),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1034),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1035] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3975),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1035),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1036] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3916),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1036),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1037] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3913),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1037),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1038] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3890),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1038),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1039] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4007),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1039),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1040] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4191),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1040),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1041] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4053),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1041),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1042] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3128),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1042),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1043] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3140),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1043),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1044] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3128),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1044),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1045] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3521),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1045),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1046] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3525),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1046),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1047] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1047),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1048] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3505),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1048),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1049] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3516),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1049),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1050] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3524),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1050),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1051] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3536),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1051),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1052] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3540),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1052),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1053] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3488),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1053),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1054] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3497),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1054),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1055] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3498),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1055),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1056] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3501),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1056),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1057] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3506),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1057),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1058] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3508),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1058),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1059] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4108),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1059),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1060] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4308),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1060),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1061] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4134),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1061),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1062] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3487),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1062),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1063] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3612),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1063),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1064] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3325),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1064),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1065] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4289),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1065),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1066] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4290),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1066),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1067] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3797),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1067),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1068] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3856),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1068),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1069] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3860),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1069),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1070] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1070),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1071] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4312),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1071),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1072] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4314),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1072),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1073] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4315),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1073),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1074] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4316),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1074),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1075] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4318),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1075),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1076] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4263),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1076),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1077] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4319),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1077),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1078] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4320),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1078),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1079] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4321),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1079),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1080] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4322),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1080),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1081] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4323),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1081),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1082] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4146),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1082),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1083] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4325),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1083),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1084] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2702),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1084),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1085] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4195),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1085),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1086] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3275),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1086),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1087] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4326),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1087),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1088] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4114),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1088),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1089] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3646),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1089),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1090] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3728),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1090),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1091] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3730),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1091),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1092] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1092),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1093] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3615),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1093),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1094] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3616),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1094),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1095] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3617),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1095),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1096] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3618),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1096),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1097] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3622),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1097),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1098] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3623),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1098),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1099] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3626),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1099),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1100] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3630),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1100),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1101] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3631),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1101),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1102] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3634),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1102),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1103] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3215),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1103),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1104] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3635),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1104),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1105] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3737),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1105),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1106] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4131),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1106),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1107] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3244),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1107),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1108] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3298),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1108),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1109] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3300),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1109),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1110] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1110),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1111] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3250),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1111),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1112] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3251),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1112),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1113] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3252),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1113),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1114] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3253),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1114),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1115] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3254),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1115),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1116] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3255),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1116),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1117] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3256),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1117),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1118] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3257),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1118),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1119] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3258),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1119),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1120] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3259),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1120),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1121] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3260),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1121),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1122] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3241),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1122),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1123] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3297),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1123),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1124] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3794),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1124),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1125] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3854),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1125),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1126] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3828),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1126),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1127] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1127),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1128] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3875),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1128),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1129] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3876),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1129),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1130] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3833),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1130),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1131] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3775),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1131),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1132] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3787),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1132),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1133] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3788),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1133),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1134] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3790),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1134),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1135] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3791),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1135),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1136] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3792),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1136),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1137] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3796),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1137),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1138] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3804),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1138),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1139] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3636),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1139),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1140] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3835),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1140),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1141] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3863),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1141),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1142] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3610),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1142),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1143] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3627),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1143),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1144] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1144),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1145] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4039),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1145),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1146] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2725),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1146),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1147] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3015),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1147),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1148] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3114),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1148),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1149] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3115),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1149),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1150] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1150),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1151] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3275),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1151),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1152] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2789),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1152),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1153] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2790),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1153),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1154] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3147),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1154),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1155] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3148),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1155),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1156] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3149),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1156),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1157] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3150),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1157),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1158] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3151),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1158),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1159] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3152),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1159),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1160] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3153),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1160),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1161] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3154),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1161),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1162] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3155),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1162),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1163] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3156),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1163),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1164] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1164),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1165] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3469),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1165),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1166] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3157),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1166),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1167] = {
    [sym_heredoc_body] = STATE(1167),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_end] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2610),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_when] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_elsif] = ACTIONS(731),
    [anon_sym_else] = ACTIONS(731),
    [anon_sym_then] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_ensure] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1168] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3903),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1168),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1169] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3470),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1169),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1170] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3471),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1170),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1171] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3672),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1171),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1172] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3676),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1172),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1173] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3686),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1173),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1174] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2814),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1174),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1175] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2817),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1175),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1176] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2782),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1176),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1177] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3169),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1177),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1178] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3264),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1178),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1179] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3523),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5712),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2202),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1179),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2096),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2098),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2100),
    [anon_sym_defined_QMARK] = ACTIONS(2102),
    [anon_sym_not] = ACTIONS(2104),
    [anon_sym_BANG] = ACTIONS(2108),
    [anon_sym_TILDE] = ACTIONS(2108),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2110),
    [sym__unary_minus_num] = ACTIONS(2112),
    [sym__binary_minus] = ACTIONS(2110),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1180] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2815),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1180),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1181] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3689),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1181),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1182] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3636),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1182),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1183] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3642),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1183),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1184] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4229),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1184),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1185] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4253),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1185),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1186] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4220),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1186),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1187] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1187),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1188] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4255),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1188),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1189] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4256),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1189),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1190] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4258),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1190),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1191] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4259),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1191),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1192] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2800),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1192),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1193] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4254),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1193),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1194] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4226),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1194),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1195] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4228),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1195),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1196] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4250),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1196),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1197] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4217),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1197),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1198] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4234),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1198),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1199] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4236),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1199),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1200] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2778),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1200),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1201] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2785),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1201),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1202] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2788),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1202),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1203] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4240),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1203),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1204] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3358),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1204),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1205] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3593),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1205),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1206] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3562),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1206),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1207] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3589),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1207),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1208] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1208),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1209] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3597),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1209),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1210] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3598),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1210),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1211] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3599),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1211),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1212] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3576),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1212),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1213] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3569),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1213),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1214] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3585),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1214),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1215] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3564),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1215),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1216] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3578),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1216),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1217] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3561),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1217),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1218] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3573),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1218),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1219] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2794),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1219),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1220] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2776),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1220),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1221] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3591),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1221),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1222] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3547),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1222),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1223] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3546),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1223),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1224] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3688),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1224),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1225] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3644),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1225),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1226] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3674),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1226),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1227] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1227),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1228] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3739),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1228),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1229] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3741),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1229),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1230] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3743),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1230),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1231] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3750),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1231),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1232] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3752),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1232),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1233] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3632),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1233),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1234] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3605),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1234),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1235] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3734),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1235),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1236] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3759),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1236),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1237] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3684),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1237),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1238] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3619),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1238),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1239] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3746),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1239),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1240] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3756),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1240),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1241] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3690),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1241),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1242] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3691),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1242),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1243] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3925),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1243),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1244] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2681),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1244),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1245] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2740),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1245),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1246] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3028),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1246),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1247] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3029),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1247),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1248] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1248),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1249] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3692),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1249),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1250] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2826),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1250),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1251] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2746),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1251),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1252] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2747),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1252),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1253] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2748),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1253),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1254] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2749),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1254),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1255] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2750),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1255),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1256] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2751),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1256),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1257] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2752),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1257),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1258] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2753),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1258),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1259] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2754),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1259),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1260] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2755),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1260),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1261] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3693),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1261),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1262] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3694),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1262),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1263] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2756),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1263),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1264] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3696),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1264),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1265] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4183),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1265),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1266] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3473),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1266),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1267] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3727),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1267),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1268] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4209),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1268),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1269] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4198),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1269),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1270] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3404),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1270),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1271] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2759),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1271),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1272] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2816),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1272),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1273] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3170),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1273),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1274] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4057),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1274),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1275] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4058),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1275),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1276] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3868),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1276),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1277] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3819),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1277),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1278] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3820),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1278),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1279] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1279),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1280] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3871),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1280),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1281] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3806),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1281),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1282] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3824),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1282),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1283] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3826),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1283),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1284] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3837),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1284),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1285] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3840),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1285),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1286] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3762),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1286),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1287] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3851),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1287),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1288] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3866),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1288),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1289] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3766),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1289),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1290] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3767),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1290),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1291] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3809),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1291),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1292] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4134),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1292),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1293] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3640),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1293),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1294] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4083),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1294),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1295] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3799),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1295),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1296] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3885),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1296),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1297] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3511),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1297),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1298] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3509),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1298),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1299] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3528),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1299),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1300] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1300),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1301] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3527),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1301),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1302] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3534),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1302),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1303] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3535),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1303),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1304] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3542),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1304),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1305] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3544),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1305),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1306] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3477),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1306),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1307] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3485),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1307),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1308] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3486),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1308),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1309] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3493),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1309),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1310] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3494),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1310),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1311] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4246),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1311),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1312] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3667),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5670),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2284),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1312),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1944),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1946),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1787),
    [anon_sym_defined_QMARK] = ACTIONS(1789),
    [anon_sym_not] = ACTIONS(1791),
    [anon_sym_BANG] = ACTIONS(1793),
    [anon_sym_TILDE] = ACTIONS(1793),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1797),
    [sym__unary_minus_num] = ACTIONS(563),
    [sym__binary_minus] = ACTIONS(1797),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1313] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3495),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1313),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1314] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3517),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1314),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1315] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3753),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1315),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1316] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3283),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1316),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1317] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3223),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1317),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1318] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3235),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1318),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1319] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1319),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1320] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3285),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1320),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1321] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3286),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1321),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1322] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3287),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1322),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1323] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3353),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1323),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1324] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3289),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1324),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1325] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3290),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1325),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1326] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3291),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1326),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1327] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3292),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1327),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1328] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3293),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1328),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1329] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3294),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1329),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1330] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3295),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1330),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1331] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3238),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1331),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1332] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3248),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1332),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1333] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3015),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1333),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1334] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4029),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1334),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1335] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2669),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1335),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1336] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2743),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1336),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1337] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2744),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1337),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1338] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1338),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1339] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4187),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1339),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1340] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4188),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1340),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1341] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2690),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1341),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1342] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2677),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1342),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1343] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2678),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1343),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1344] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2679),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1344),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1345] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2680),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1345),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1346] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2684),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1346),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1347] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2670),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1347),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1348] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2672),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1348),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1349] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2673),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1349),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1350] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2675),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1350),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1351] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1351),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1352] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2676),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1352),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1353] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2806),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1353),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1354] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2691),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1354),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1355] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2704),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1355),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1356] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2644),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1356),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1357] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2856),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1357),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1358] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3645),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1358),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1359] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3654),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5936),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2504),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1359),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1952),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1954),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1956),
    [anon_sym_defined_QMARK] = ACTIONS(1958),
    [anon_sym_not] = ACTIONS(1960),
    [anon_sym_BANG] = ACTIONS(1964),
    [anon_sym_TILDE] = ACTIONS(1964),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1966),
    [sym__unary_minus_num] = ACTIONS(1968),
    [sym__binary_minus] = ACTIONS(1966),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1360] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3453),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1360),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1361] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3407),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1361),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1362] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3408),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1362),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1363] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1363),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1364] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3389),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1364),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1365] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3390),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1365),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1366] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3391),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1366),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1367] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3392),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1367),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1368] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3393),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1368),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1369] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3394),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1369),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1370] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3395),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1370),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1371] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3396),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1371),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1372] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3397),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1372),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1373] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3398),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1373),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1374] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3399),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1374),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1375] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2966),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1375),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1376] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3874),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1376),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1377] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3821),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1377),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1378] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3414),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1378),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1379] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4327),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1379),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1380] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3882),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1380),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1381] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3873),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1381),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1382] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3888),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1382),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1383] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1383),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1384] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3771),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1384),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1385] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3772),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1385),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1386] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3773),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1386),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1387] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3774),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1387),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1388] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3776),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1388),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1389] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3777),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1389),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1390] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3778),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1390),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1391] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3779),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1391),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1392] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3780),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1392),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1393] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3782),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1393),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1394] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3783),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1394),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1395] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3805),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1395),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1396] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3770),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1396),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1397] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4141),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1397),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1398] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3906),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1398),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1399] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2839),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1399),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1400] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2686),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1400),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1401] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2687),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1401),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1402] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1402),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1403] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4175),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1403),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1404] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2888),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1404),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1405] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2892),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1405),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1406] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2893),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1406),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1407] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2898),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1407),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1408] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2899),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1408),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1409] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2901),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1409),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1410] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2902),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1410),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1411] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2905),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1411),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1412] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2906),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1412),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1413] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2907),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1413),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1414] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4138),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1414),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1415] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2942),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1415),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1416] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4140),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1416),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1417] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1417),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1418] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3874),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1418),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1419] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3810),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1419),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1420] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3813),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1420),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1421] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3044),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1421),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1422] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2760),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1422),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1423] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1423),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1424] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3244),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1424),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1425] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3245),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5817),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2275),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1425),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(539),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1397),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1407),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1409),
    [anon_sym_defined_QMARK] = ACTIONS(1411),
    [anon_sym_not] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1417),
    [anon_sym_TILDE] = ACTIONS(1417),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1425),
    [sym__unary_minus_num] = ACTIONS(1427),
    [sym__binary_minus] = ACTIONS(1425),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1426] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3361),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1426),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1427] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3467),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1427),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1428] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3468),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1428),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1429] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1429),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1430] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3370),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1430),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1431] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3371),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1431),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1432] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3372),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1432),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1433] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3373),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1433),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1434] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3374),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1434),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1435] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3375),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1435),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1436] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3376),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1436),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1437] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3377),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1437),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1438] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3378),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1438),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1439] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3379),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1439),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1440] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3380),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1440),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1441] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3366),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1441),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1442] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4184),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1442),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1443] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4192),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1443),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1444] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3357),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1444),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1445] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3365),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1445),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1446] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3306),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1446),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1447] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3224),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1447),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1448] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3226),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1448),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1449] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1449),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1450] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3309),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1450),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1451] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3310),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1451),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1452] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3311),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1452),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1453] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3312),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1453),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1454] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3313),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1454),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1455] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3314),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1455),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1456] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3315),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1456),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1457] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3316),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1457),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1458] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3317),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1458),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1459] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3318),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1459),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1460] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3319),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1460),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1461] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3234),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1461),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1462] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3308),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1462),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1463] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4193),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1463),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1464] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2641),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1464),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1465] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3019),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5925),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2193),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1465),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1531),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1533),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1535),
    [anon_sym_defined_QMARK] = ACTIONS(1537),
    [anon_sym_not] = ACTIONS(1539),
    [anon_sym_BANG] = ACTIONS(1543),
    [anon_sym_TILDE] = ACTIONS(1543),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1545),
    [sym__unary_minus_num] = ACTIONS(1547),
    [sym__binary_minus] = ACTIONS(1545),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1466] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3102),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1466),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1467] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2664),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1467),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1468] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1468),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1469] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4200),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1469),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1470] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4305),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1470),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1471] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2643),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1471),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1472] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2667),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1472),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1473] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2646),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1473),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1474] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2650),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1474),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1475] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2651),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1475),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1476] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2652),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1476),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1477] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2653),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1477),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1478] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2655),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1478),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1479] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2656),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1479),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1480] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2657),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1480),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1481] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2625),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1481),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1482] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4201),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1482),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1483] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4202),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1483),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1484] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3829),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1484),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1485] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3831),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1485),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1486] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3849),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1486),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1487] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3781),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1487),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1488] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4203),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1488),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1489] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2627),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1489),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1490] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2697),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1490),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1491] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3889),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1491),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1492] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3688),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1492),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1493] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3702),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1493),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1494] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4042),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1494),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1495] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4011),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1495),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1496] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4017),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1496),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1497] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1497),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1498] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4043),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1498),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1499] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4044),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1499),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1500] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4045),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1500),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1501] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4046),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1501),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1502] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4047),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1502),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1503] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4048),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1503),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1504] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4049),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1504),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1505] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4050),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1505),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1506] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4051),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1506),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1507] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4052),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1507),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1508] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4026),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1508),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1509] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3838),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1509),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1510] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3845),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1510),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1511] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3991),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1511),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1512] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4037),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1512),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1513] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3850),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1513),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1514] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3646),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1514),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1515] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4133),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5794),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2534),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1515),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2244),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2246),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2248),
    [anon_sym_defined_QMARK] = ACTIONS(2250),
    [anon_sym_not] = ACTIONS(2252),
    [anon_sym_BANG] = ACTIONS(2256),
    [anon_sym_TILDE] = ACTIONS(2256),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2258),
    [sym__unary_minus_num] = ACTIONS(2260),
    [sym__binary_minus] = ACTIONS(2258),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1516] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2724),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1516),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1517] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2865),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1517),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1518] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2827),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1518),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1519] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1519),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1520] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3855),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1520),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1521] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3857),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1521),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1522] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2841),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1522),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1523] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2842),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1523),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1524] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2843),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1524),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1525] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2844),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1525),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1526] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2845),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1526),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1527] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2846),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1527),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1528] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2871),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1528),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1529] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2848),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1529),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1530] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2849),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1530),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1531] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2850),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1531),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1532] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2852),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1532),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1533] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3859),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1533),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1534] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4204),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1534),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1535] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4205),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1535),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1536] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3869),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1536),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1537] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4128),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1537),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1538] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3423),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1538),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1539] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2870),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1539),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1540] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2831),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1540),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1541] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4208),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1541),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1542] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4175),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1542),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1543] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4178),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1543),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1544] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3941),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1544),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1545] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3917),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1545),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1546] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3918),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1546),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1547] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3075),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1547),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1548] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3943),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1548),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1549] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3944),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1549),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1550] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3945),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1550),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1551] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3946),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1551),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1552] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3947),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1552),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1553] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3948),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1553),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1554] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3949),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1554),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1555] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3950),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1555),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1556] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3951),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1556),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1557] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3952),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1557),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1558] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3953),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1558),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1559] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3228),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5863),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2172),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1559),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1817),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1573),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1711),
    [anon_sym_defined_QMARK] = ACTIONS(1713),
    [anon_sym_not] = ACTIONS(1715),
    [anon_sym_BANG] = ACTIONS(1717),
    [anon_sym_TILDE] = ACTIONS(1717),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1577),
    [sym__unary_minus_num] = ACTIONS(209),
    [sym__binary_minus] = ACTIONS(1577),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1560] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4166),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1560),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1561] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4167),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1561),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1562] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3921),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1562),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1563] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4244),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1563),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1564] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4168),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1564),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1565] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2839),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1565),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1566] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2626),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1566),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1567] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3161),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1567),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1568] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3090),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1568),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1569] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1569),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1570] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3111),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1570),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1571] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3112),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1571),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1572] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3116),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1572),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1573] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3117),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1573),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1574] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3118),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1574),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1575] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3119),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1575),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1576] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3120),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1576),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1577] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3121),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1577),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1578] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3122),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1578),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1579] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3174),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1579),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1580] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3124),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1580),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1581] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4169),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1581),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1582] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4211),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1582),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1583] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3109),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1583),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1584] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3099),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1584),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1585] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4239),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1585),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1586] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4241),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1586),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1587] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3939),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1587),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1588] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3102),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1588),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1589] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3633),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1589),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1590] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2765),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1590),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1591] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2766),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1591),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1592] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1592),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1593] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2726),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1593),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1594] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2727),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1594),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1595] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2728),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1595),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1596] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2729),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1596),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1597] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2730),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1597),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1598] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2731),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1598),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1599] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2732),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1599),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1600] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2733),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1600),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1601] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2734),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1601),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1602] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2735),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1602),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1603] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2736),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1603),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1604] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3852),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1604),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1605] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4054),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5754),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2411),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1605),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2114),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2042),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1681),
    [anon_sym_defined_QMARK] = ACTIONS(1683),
    [anon_sym_not] = ACTIONS(1685),
    [anon_sym_BANG] = ACTIONS(1687),
    [anon_sym_TILDE] = ACTIONS(1687),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1697),
    [sym__unary_minus_num] = ACTIONS(401),
    [sym__binary_minus] = ACTIONS(1697),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1606] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2769),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1606),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1607] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2774),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1607),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1608] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3283),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1608),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1609] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3284),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5915),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2283),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1609),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1495),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1497),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1499),
    [anon_sym_defined_QMARK] = ACTIONS(1501),
    [anon_sym_not] = ACTIONS(1503),
    [anon_sym_BANG] = ACTIONS(1507),
    [anon_sym_TILDE] = ACTIONS(1507),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1509),
    [sym__unary_minus_num] = ACTIONS(1511),
    [sym__binary_minus] = ACTIONS(1509),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1610] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3861),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1610),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1611] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4136),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5738),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2566),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1611),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2262),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2264),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2266),
    [anon_sym_defined_QMARK] = ACTIONS(2268),
    [anon_sym_not] = ACTIONS(2270),
    [anon_sym_BANG] = ACTIONS(2274),
    [anon_sym_TILDE] = ACTIONS(2274),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2276),
    [sym__unary_minus_num] = ACTIONS(2278),
    [sym__binary_minus] = ACTIONS(2276),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1612] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2724),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1612),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1613] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2830),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5933),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2179),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1613),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1373),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1375),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1377),
    [anon_sym_defined_QMARK] = ACTIONS(1379),
    [anon_sym_not] = ACTIONS(1381),
    [anon_sym_BANG] = ACTIONS(1385),
    [anon_sym_TILDE] = ACTIONS(1385),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1387),
    [sym__unary_minus_num] = ACTIONS(1389),
    [sym__binary_minus] = ACTIONS(1387),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1614] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2558),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1614),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1615] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2561),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1615),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1616] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1616),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1617] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2574),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1617),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1618] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2575),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1618),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1619] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2576),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1619),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1620] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2577),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1620),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1621] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2578),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1621),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1622] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2579),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1622),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1623] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2580),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1623),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1624] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2581),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1624),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1625] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2624),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1625),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1626] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2583),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1626),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1627] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2584),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1627),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1628] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3424),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1628),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1629] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4171),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1629),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1630] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2588),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1630),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1631] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2546),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1631),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1632] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3932),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1632),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1633] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3891),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5659),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2572),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1633),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2120),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2122),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2124),
    [anon_sym_defined_QMARK] = ACTIONS(2126),
    [anon_sym_not] = ACTIONS(2128),
    [anon_sym_BANG] = ACTIONS(2132),
    [anon_sym_TILDE] = ACTIONS(2132),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2134),
    [sym__unary_minus_num] = ACTIONS(2136),
    [sym__binary_minus] = ACTIONS(2134),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1634] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3925),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1634),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1635] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3893),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1635),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1636] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3894),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1636),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1637] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2573),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1637),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1638] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3160),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5968),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2196),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1638),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1593),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1595),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1597),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1599),
    [anon_sym_defined_QMARK] = ACTIONS(1601),
    [anon_sym_not] = ACTIONS(1603),
    [anon_sym_BANG] = ACTIONS(1607),
    [anon_sym_TILDE] = ACTIONS(1607),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1609),
    [sym__unary_minus_num] = ACTIONS(1611),
    [sym__binary_minus] = ACTIONS(1609),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1639] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2491),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1639),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1640] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2492),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1640),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1641] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1641),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1642] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2509),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1642),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1643] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2510),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1643),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1644] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2511),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1644),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1645] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2512),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1645),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1646] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2513),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1646),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1647] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2514),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1647),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1648] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2515),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1648),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1649] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2516),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1649),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1650] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2517),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1650),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1651] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2518),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1651),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1652] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2519),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1652),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1653] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1653),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1654] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4214),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1654),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1655] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2495),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1655),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1656] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2498),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1656),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1657] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4229),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1657),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1658] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4216),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5699),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2648),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1658),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2308),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2310),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2312),
    [anon_sym_defined_QMARK] = ACTIONS(2314),
    [anon_sym_not] = ACTIONS(2316),
    [anon_sym_BANG] = ACTIONS(2320),
    [anon_sym_TILDE] = ACTIONS(2320),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2322),
    [sym__unary_minus_num] = ACTIONS(2324),
    [sym__binary_minus] = ACTIONS(2322),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1659] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2508),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1659),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1660] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2772),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5644),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2173),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1660),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(185),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1319),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1321),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1323),
    [anon_sym_defined_QMARK] = ACTIONS(1325),
    [anon_sym_not] = ACTIONS(1327),
    [anon_sym_BANG] = ACTIONS(1331),
    [anon_sym_TILDE] = ACTIONS(1331),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1333),
    [sym__unary_minus_num] = ACTIONS(1335),
    [sym__binary_minus] = ACTIONS(1333),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1661] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3868),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1661),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1662] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3877),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5717),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2525),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1662),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2012),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2014),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2016),
    [anon_sym_defined_QMARK] = ACTIONS(2018),
    [anon_sym_not] = ACTIONS(2020),
    [anon_sym_BANG] = ACTIONS(2024),
    [anon_sym_TILDE] = ACTIONS(2024),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2026),
    [sym__unary_minus_num] = ACTIONS(2028),
    [sym__binary_minus] = ACTIONS(2026),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1663] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4213),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1663),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1664] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3593),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1664),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1665] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2541),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1665),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1666] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4172),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1666),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1667] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3977),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1667),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1668] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3453),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1668),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1669] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3454),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5731),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2433),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1669),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1477),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1479),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1481),
    [anon_sym_defined_QMARK] = ACTIONS(1483),
    [anon_sym_not] = ACTIONS(1485),
    [anon_sym_BANG] = ACTIONS(1489),
    [anon_sym_TILDE] = ACTIONS(1489),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1491),
    [sym__unary_minus_num] = ACTIONS(1493),
    [sym__binary_minus] = ACTIONS(1491),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1670] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3978),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1670),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1671] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3979),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1671),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1672] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3511),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1672),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1673] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2497),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1673),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1674] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3980),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1674),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1675] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4129),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1675),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1676] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3361),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1676),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1677] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3362),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5751),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2435),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1677),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(375),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1613),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1615),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1617),
    [anon_sym_defined_QMARK] = ACTIONS(1619),
    [anon_sym_not] = ACTIONS(1621),
    [anon_sym_BANG] = ACTIONS(1625),
    [anon_sym_TILDE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1627),
    [sym__unary_minus_num] = ACTIONS(1629),
    [sym__binary_minus] = ACTIONS(1627),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1678] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3981),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1678),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1679] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3983),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1679),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1680] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3882),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1680),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1681] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3555),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5680),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2479),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1681),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1851),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1853),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1855),
    [anon_sym_defined_QMARK] = ACTIONS(1857),
    [anon_sym_not] = ACTIONS(1859),
    [anon_sym_BANG] = ACTIONS(1863),
    [anon_sym_TILDE] = ACTIONS(1863),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1865),
    [sym__unary_minus_num] = ACTIONS(1867),
    [sym__binary_minus] = ACTIONS(1865),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1682] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3896),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1682),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1683] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3985),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1683),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1684] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4042),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1684),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1685] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4010),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5767),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2647),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1685),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2162),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2164),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2166),
    [anon_sym_defined_QMARK] = ACTIONS(2168),
    [anon_sym_not] = ACTIONS(2170),
    [anon_sym_BANG] = ACTIONS(2174),
    [anon_sym_TILDE] = ACTIONS(2174),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2176),
    [sym__unary_minus_num] = ACTIONS(2178),
    [sym__binary_minus] = ACTIONS(2176),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1686] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3986),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1686),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1687] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3987),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1687),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1688] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3306),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1688),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1689] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3512),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5685),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2488),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1689),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1551),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1553),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1555),
    [anon_sym_defined_QMARK] = ACTIONS(1557),
    [anon_sym_not] = ACTIONS(1559),
    [anon_sym_BANG] = ACTIONS(1563),
    [anon_sym_TILDE] = ACTIONS(1563),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1565),
    [sym__unary_minus_num] = ACTIONS(1567),
    [sym__binary_minus] = ACTIONS(1565),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1690] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3988),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1690),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1691] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4130),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1691),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1692] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3794),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1692),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1693] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3844),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5784),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2606),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1693),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1978),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1980),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1982),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1984),
    [anon_sym_defined_QMARK] = ACTIONS(1986),
    [anon_sym_not] = ACTIONS(1988),
    [anon_sym_BANG] = ACTIONS(1992),
    [anon_sym_TILDE] = ACTIONS(1992),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2000),
    [sym__unary_minus_num] = ACTIONS(2002),
    [sym__binary_minus] = ACTIONS(2000),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1694] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4132),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1694),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1695] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3989),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1695),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1696] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3785),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5697),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2530),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1696),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(2054),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2056),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2058),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2060),
    [anon_sym_defined_QMARK] = ACTIONS(2062),
    [anon_sym_not] = ACTIONS(2064),
    [anon_sym_BANG] = ACTIONS(2068),
    [anon_sym_TILDE] = ACTIONS(2068),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2070),
    [sym__unary_minus_num] = ACTIONS(2072),
    [sym__binary_minus] = ACTIONS(2070),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1697] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3425),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1697),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1698] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3941),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1698),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1699] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3938),
    [sym__unary_minus_pow] = STATE(3021),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3043),
    [sym_operator_assignment] = STATE(3043),
    [sym_conditional] = STATE(3043),
    [sym_range] = STATE(3043),
    [sym_binary] = STATE(3043),
    [sym_unary] = STATE(3043),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5798),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2654),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1699),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2138),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2142),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2144),
    [anon_sym_defined_QMARK] = ACTIONS(2146),
    [anon_sym_not] = ACTIONS(2148),
    [anon_sym_BANG] = ACTIONS(2152),
    [anon_sym_TILDE] = ACTIONS(2152),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2154),
    [sym__unary_minus_num] = ACTIONS(2156),
    [sym__binary_minus] = ACTIONS(2154),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1700] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3931),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5668),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2272),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1700),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2074),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1974),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1805),
    [anon_sym_defined_QMARK] = ACTIONS(1807),
    [anon_sym_not] = ACTIONS(1809),
    [anon_sym_BANG] = ACTIONS(1811),
    [anon_sym_TILDE] = ACTIONS(1811),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1815),
    [sym__unary_minus_num] = ACTIONS(261),
    [sym__binary_minus] = ACTIONS(1815),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1701] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3307),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5705),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2269),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1701),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1459),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1461),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1463),
    [anon_sym_defined_QMARK] = ACTIONS(1465),
    [anon_sym_not] = ACTIONS(1467),
    [anon_sym_BANG] = ACTIONS(1471),
    [anon_sym_TILDE] = ACTIONS(1471),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1473),
    [sym__unary_minus_num] = ACTIONS(1475),
    [sym__binary_minus] = ACTIONS(1473),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1702] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3874),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1702),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1703] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2791),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1703),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1704] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2787),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1704),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1705] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3897),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1705),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1706] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4124),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1706),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1707] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3909),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1707),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1708] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2791),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5850),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2139),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1708),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(739),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(745),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1351),
    [anon_sym_defined_QMARK] = ACTIONS(1353),
    [anon_sym_not] = ACTIONS(1355),
    [anon_sym_BANG] = ACTIONS(1359),
    [anon_sym_TILDE] = ACTIONS(1359),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(681),
    [sym__unary_minus_num] = ACTIONS(679),
    [sym__binary_minus] = ACTIONS(681),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1709] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4057),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1709),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1710] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4281),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1710),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1711] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4068),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1711),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1712] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4069),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1712),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1713] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1713),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1714] = {
    [sym_heredoc_body] = STATE(1714),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_end] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2612),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_when] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_elsif] = ACTIONS(731),
    [anon_sym_else] = ACTIONS(731),
    [anon_sym_then] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_ensure] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1715] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3426),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1715),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1716] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4075),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1716),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1717] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4087),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1717),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1718] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4064),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1718),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1719] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4067),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1719),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1720] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4153),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1720),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1721] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4072),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1721),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1722] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4073),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1722),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1723] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4078),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1723),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1724] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4079),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1724),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1725] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4080),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1725),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1726] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2740),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5966),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2140),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1726),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1303),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1263),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1305),
    [anon_sym_defined_QMARK] = ACTIONS(1307),
    [anon_sym_not] = ACTIONS(1309),
    [anon_sym_BANG] = ACTIONS(1313),
    [anon_sym_TILDE] = ACTIONS(1313),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1267),
    [sym__unary_minus_num] = ACTIONS(455),
    [sym__binary_minus] = ACTIONS(1267),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1727] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4081),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1727),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1728] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4082),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1728),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1729] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2669),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5986),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2114),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1729),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1273),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1073),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1275),
    [anon_sym_defined_QMARK] = ACTIONS(1277),
    [anon_sym_not] = ACTIONS(1279),
    [anon_sym_BANG] = ACTIONS(1283),
    [anon_sym_TILDE] = ACTIONS(1283),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1077),
    [sym__unary_minus_num] = ACTIONS(165),
    [sym__binary_minus] = ACTIONS(1077),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1730] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2641),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1730),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1731] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2573),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5649),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2131),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1731),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(431),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(949),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(951),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(953),
    [anon_sym_defined_QMARK] = ACTIONS(955),
    [anon_sym_not] = ACTIONS(957),
    [anon_sym_BANG] = ACTIONS(961),
    [anon_sym_TILDE] = ACTIONS(961),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(963),
    [sym__unary_minus_num] = ACTIONS(965),
    [sym__binary_minus] = ACTIONS(963),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1732] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2508),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2429),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5658),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2117),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1732),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(123),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(907),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(917),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(919),
    [anon_sym_defined_QMARK] = ACTIONS(921),
    [anon_sym_not] = ACTIONS(923),
    [anon_sym_BANG] = ACTIONS(927),
    [anon_sym_TILDE] = ACTIONS(927),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(935),
    [sym__unary_minus_num] = ACTIONS(937),
    [sym__binary_minus] = ACTIONS(935),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1733] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4077),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1733),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1734] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4279),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1734),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1735] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3900),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3299),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2203),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5825),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2828),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2278),
    [sym__variable] = STATE(2204),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1735),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(273),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2006),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(1920),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1825),
    [anon_sym_defined_QMARK] = ACTIONS(1827),
    [anon_sym_not] = ACTIONS(1829),
    [anon_sym_BANG] = ACTIONS(1831),
    [anon_sym_TILDE] = ACTIONS(1831),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(1833),
    [sym__unary_minus_num] = ACTIONS(361),
    [sym__binary_minus] = ACTIONS(1833),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1736] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4066),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5742),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2503),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1736),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(1950),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2210),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2212),
    [anon_sym_defined_QMARK] = ACTIONS(2214),
    [anon_sym_not] = ACTIONS(2216),
    [anon_sym_BANG] = ACTIONS(2220),
    [anon_sym_TILDE] = ACTIONS(2220),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2222),
    [sym__unary_minus_num] = ACTIONS(2224),
    [sym__binary_minus] = ACTIONS(2222),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1737] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4239),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3023),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1737),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1738] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4225),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1738),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1739] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4231),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3024),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1739),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1740] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4306),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1740),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1741] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4282),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1741),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1742] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4293),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1742),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1743] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4285),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1743),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1744] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4288),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1744),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1745] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4292),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1745),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1746] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4297),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1746),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1747] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4309),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1747),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1748] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4311),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1748),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1749] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4328),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1749),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1750] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4338),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1750),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1751] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4344),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1751),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1752] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4345),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1752),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1753] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4264),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1753),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1754] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4265),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1754),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1755] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4266),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1755),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1756] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4267),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1756),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1757] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4268),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1757),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1758] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4356),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1758),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1759] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4270),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1759),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1760] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4271),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1760),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1761] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4272),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1761),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1762] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4273),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1762),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1763] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4274),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1763),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1764] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4275),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1764),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1765] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4276),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1765),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1766] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(3382),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1766),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1767] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4207),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1767),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1768] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4277),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1768),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1769] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3483),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5844),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2183),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1769),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(463),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1912),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1884),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1743),
    [anon_sym_defined_QMARK] = ACTIONS(1745),
    [anon_sym_not] = ACTIONS(1747),
    [anon_sym_BANG] = ACTIONS(1749),
    [anon_sym_TILDE] = ACTIONS(1749),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1751),
    [sym__unary_minus_num] = ACTIONS(479),
    [sym__binary_minus] = ACTIONS(1751),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1770] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4230),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1770),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1771] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4262),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1771),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1772] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4215),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1772),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1773] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4232),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1773),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1774] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4218),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1774),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1775] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4224),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1775),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1776] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4238),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1776),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1777] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4251),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1777),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1778] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4252),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1778),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1779] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4261),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1779),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1780] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4235),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1780),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1781] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4260),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1781),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1782] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4237),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5890),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2564),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1782),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2118),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2344),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2346),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2348),
    [anon_sym_defined_QMARK] = ACTIONS(2350),
    [anon_sym_not] = ACTIONS(2352),
    [anon_sym_BANG] = ACTIONS(2356),
    [anon_sym_TILDE] = ACTIONS(2356),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(325),
    [sym_float] = ACTIONS(325),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2358),
    [sym__unary_minus_num] = ACTIONS(2360),
    [sym__binary_minus] = ACTIONS(2358),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1783] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3556),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1783),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1784] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4105),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1784),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1785] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4099),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1785),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1786] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4100),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1786),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1787] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1787),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1788] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4088),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1788),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1789] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4089),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1789),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1790] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4090),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1790),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1791] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4091),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1791),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1792] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4092),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1792),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1793] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4093),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1793),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1794] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4094),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1794),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1795] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4127),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1795),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1796] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4096),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1796),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1797] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4097),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1797),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1798] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4098),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1798),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1799] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4103),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1799),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1800] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4189),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5796),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2487),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1800),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(571),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2092),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2050),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1839),
    [anon_sym_defined_QMARK] = ACTIONS(1841),
    [anon_sym_not] = ACTIONS(1843),
    [anon_sym_BANG] = ACTIONS(1845),
    [anon_sym_TILDE] = ACTIONS(1845),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1847),
    [sym__unary_minus_num] = ACTIONS(595),
    [sym__binary_minus] = ACTIONS(1847),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1801] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3472),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1801),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1802] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3436),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1802),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1803] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3420),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5727),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2184),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1803),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(19),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(27),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(63),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2078),
    [anon_sym_defined_QMARK] = ACTIONS(2080),
    [anon_sym_not] = ACTIONS(2082),
    [anon_sym_BANG] = ACTIONS(2086),
    [anon_sym_TILDE] = ACTIONS(2086),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(113),
    [sym__unary_minus_num] = ACTIONS(111),
    [sym__binary_minus] = ACTIONS(113),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1804] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4104),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1804),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1805] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3460),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1805),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1806] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3441),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1806),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1807] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3442),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1807),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1808] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3443),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1808),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1809] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3444),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1809),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1810] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3445),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1810),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1811] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3446),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1811),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1812] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3447),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1812),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1813] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3448),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1813),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1814] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3449),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1814),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1815] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3450),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1815),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1816] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3451),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1816),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1817] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3433),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1817),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1818] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3577),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1818),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1819] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3428),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1819),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1820] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3429),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1820),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1821] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2594),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1821),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1822] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3713),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1822),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1823] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3656),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1823),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1824] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3657),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1824),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1825] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1825),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1826] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3716),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1826),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1827] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3717),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1827),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1828] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3718),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1828),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1829] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3719),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1829),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1830] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3720),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1830),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1831] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3721),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1831),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1832] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3722),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1832),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1833] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3723),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1833),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1834] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3724),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1834),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1835] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3725),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1835),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1836] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3726),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1836),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1837] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3660),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1837),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1838] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3715),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1838),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1839] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3434),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1839),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1840] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4105),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1840),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1841] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4115),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5852),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2536),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1841),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2226),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2228),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2230),
    [anon_sym_defined_QMARK] = ACTIONS(2232),
    [anon_sym_not] = ACTIONS(2234),
    [anon_sym_BANG] = ACTIONS(2238),
    [anon_sym_TILDE] = ACTIONS(2238),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2240),
    [sym__unary_minus_num] = ACTIONS(2242),
    [sym__binary_minus] = ACTIONS(2240),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1842] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3031),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1842),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1843] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3032),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1843),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1844] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2353),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1844),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1845] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3053),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1845),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1846] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3054),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1846),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1847] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3055),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1847),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1848] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3056),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1848),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1849] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3057),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1849),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1850] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3058),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1850),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1851] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3059),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1851),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1852] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3060),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1852),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1853] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3061),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1853),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1854] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3062),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1854),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1855] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3063),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1855),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1856] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3035),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1856),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1857] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3038),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1857),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1858] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3052),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1858),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1859] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3037),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1859),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1860] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3713),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1860),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1861] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3714),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5853),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2537),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1861),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1924),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1926),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1928),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1930),
    [anon_sym_defined_QMARK] = ACTIONS(1932),
    [anon_sym_not] = ACTIONS(1934),
    [anon_sym_BANG] = ACTIONS(1938),
    [anon_sym_TILDE] = ACTIONS(1938),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1940),
    [sym__unary_minus_num] = ACTIONS(1942),
    [sym__binary_minus] = ACTIONS(1940),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1862] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3460),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5845),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2215),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1862),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2036),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1908),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1894),
    [anon_sym_defined_QMARK] = ACTIONS(1896),
    [anon_sym_not] = ACTIONS(1898),
    [anon_sym_BANG] = ACTIONS(1900),
    [anon_sym_TILDE] = ACTIONS(1900),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1902),
    [sym__unary_minus_num] = ACTIONS(529),
    [sym__binary_minus] = ACTIONS(1902),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1863] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(3052),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(3000),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2043),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2450),
    [sym_left_assignment_list] = STATE(5851),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2200),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2205),
    [sym__variable] = STATE(2048),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1863),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(505),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1513),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1515),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1517),
    [anon_sym_defined_QMARK] = ACTIONS(1519),
    [anon_sym_not] = ACTIONS(1521),
    [anon_sym_BANG] = ACTIONS(1525),
    [anon_sym_TILDE] = ACTIONS(1525),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1527),
    [sym__unary_minus_num] = ACTIONS(1529),
    [sym__binary_minus] = ACTIONS(1527),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1864] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4313),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1864),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1865] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4324),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1865),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1866] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(4144),
    [sym__unary_minus_pow] = STATE(2556),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2376),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2547),
    [sym_operator_assignment] = STATE(2547),
    [sym_conditional] = STATE(2547),
    [sym_range] = STATE(2547),
    [sym_binary] = STATE(2547),
    [sym_unary] = STATE(2547),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5918),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2486),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1866),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(1849),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(2182),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(2184),
    [anon_sym_defined_QMARK] = ACTIONS(2186),
    [anon_sym_not] = ACTIONS(2188),
    [anon_sym_BANG] = ACTIONS(2192),
    [anon_sym_TILDE] = ACTIONS(2192),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(253),
    [sym_float] = ACTIONS(253),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(2194),
    [sym__unary_minus_num] = ACTIONS(2196),
    [sym__binary_minus] = ACTIONS(2194),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1867] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4286),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1867),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1868] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4287),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1868),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1869] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4291),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1869),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1870] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4294),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1870),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1871] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4295),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1871),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1872] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4329),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1872),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1873] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4296),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1873),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1874] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4298),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1874),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1875] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4299),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1875),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1876] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4300),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1876),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1877] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4301),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1877),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1878] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4302),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1878),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1879] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4303),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1879),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1880] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4330),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5895),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2639),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(3543),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1880),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2376),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2378),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2380),
    [anon_sym_defined_QMARK] = ACTIONS(2382),
    [anon_sym_not] = ACTIONS(2384),
    [anon_sym_BANG] = ACTIONS(2388),
    [anon_sym_TILDE] = ACTIONS(2388),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1994),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1996),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(1998),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2390),
    [sym__unary_minus_num] = ACTIONS(2392),
    [sym__binary_minus] = ACTIONS(2390),
    [sym__identifier_suffix] = ACTIONS(2004),
    [sym__constant_suffix] = ACTIONS(1994),
  },
  [1881] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4304),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1881),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1882] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4212),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1882),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1883] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4139),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5726),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2346),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1883),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(603),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2030),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2032),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(1727),
    [anon_sym_defined_QMARK] = ACTIONS(1729),
    [anon_sym_not] = ACTIONS(1731),
    [anon_sym_BANG] = ACTIONS(1733),
    [anon_sym_TILDE] = ACTIONS(1733),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(717),
    [sym__unary_minus_num] = ACTIONS(627),
    [sym__binary_minus] = ACTIONS(717),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1884] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4143),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1884),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1885] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4331),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1885),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1886] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4333),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1886),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1887] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4334),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1887),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1888] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4335),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1888),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1889] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4336),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1889),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1890] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4337),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1890),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1891] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4339),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1891),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1892] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4340),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1892),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1893] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4341),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1893),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1894] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4342),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1894),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1895] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4343),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1895),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1896] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4346),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1896),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1897] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4347),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1897),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1898] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4349),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1898),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1899] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4350),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1899),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1900] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4351),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1900),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1901] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4352),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1901),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1902] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4353),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1902),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1903] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4354),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1903),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1904] = {
    [sym_method] = STATE(3027),
    [sym_singleton_method] = STATE(3027),
    [sym_class] = STATE(3027),
    [sym_singleton_class] = STATE(3027),
    [sym_module] = STATE(3027),
    [sym_return] = STATE(3027),
    [sym_yield] = STATE(3027),
    [sym_break] = STATE(3027),
    [sym_next] = STATE(3027),
    [sym_redo] = STATE(3027),
    [sym_retry] = STATE(3027),
    [sym_while] = STATE(3027),
    [sym_until] = STATE(3027),
    [sym_for] = STATE(3027),
    [sym_case] = STATE(3027),
    [sym_case_match] = STATE(3027),
    [sym_if] = STATE(3027),
    [sym_unless] = STATE(3027),
    [sym_begin] = STATE(3027),
    [sym__arg] = STATE(4355),
    [sym__unary_minus_pow] = STATE(3463),
    [sym__primary] = STATE(3192),
    [sym_parenthesized_statements] = STATE(3027),
    [sym_element_reference] = STATE(2221),
    [sym_scope_resolution] = STATE(2221),
    [sym__call] = STATE(2169),
    [sym_call] = STATE(3027),
    [sym_assignment] = STATE(3465),
    [sym_operator_assignment] = STATE(3465),
    [sym_conditional] = STATE(3465),
    [sym_range] = STATE(3465),
    [sym_binary] = STATE(3465),
    [sym_unary] = STATE(3465),
    [sym_parenthesized_unary] = STATE(3027),
    [sym_unary_literal] = STATE(3068),
    [sym__literal] = STATE(3027),
    [sym__numeric] = STATE(3026),
    [sym__simple_numeric] = STATE(3068),
    [sym_left_assignment_list] = STATE(5905),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2703),
    [sym__function_identifier_call] = STATE(3069),
    [sym__lhs] = STATE(2532),
    [sym__variable] = STATE(2168),
    [sym__nonlocal_variable] = STATE(2206),
    [sym__int_or_float] = STATE(5560),
    [sym_complex] = STATE(3070),
    [sym_rational] = STATE(3070),
    [sym_nil] = STATE(2221),
    [sym_constant_suffix] = STATE(2847),
    [sym_identifier_suffix] = STATE(2854),
    [sym_chained_string] = STATE(3027),
    [sym_string] = STATE(2807),
    [sym_subshell] = STATE(3027),
    [sym_string_array] = STATE(3027),
    [sym_symbol_array] = STATE(3027),
    [sym_delimited_symbol] = STATE(3026),
    [sym_regex] = STATE(3027),
    [sym_heredoc_body] = STATE(1904),
    [sym_array] = STATE(3027),
    [sym_hash] = STATE(3027),
    [sym_lambda] = STATE(3027),
    [sym_identifier] = ACTIONS(1393),
    [anon_sym_LBRACE] = ACTIONS(537),
    [anon_sym_def] = ACTIONS(2010),
    [anon_sym_LPAREN] = ACTIONS(1395),
    [anon_sym_COLON_COLON] = ACTIONS(1916),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2280),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(279),
    [anon_sym_class] = ACTIONS(281),
    [anon_sym_module] = ACTIONS(283),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1401),
    [anon_sym_break] = ACTIONS(1403),
    [anon_sym_next] = ACTIONS(1405),
    [anon_sym_redo] = ACTIONS(293),
    [anon_sym_retry] = ACTIONS(295),
    [anon_sym_if] = ACTIONS(297),
    [anon_sym_unless] = ACTIONS(299),
    [anon_sym_while] = ACTIONS(301),
    [anon_sym_until] = ACTIONS(303),
    [anon_sym_for] = ACTIONS(305),
    [anon_sym_case] = ACTIONS(309),
    [anon_sym_LBRACK] = ACTIONS(311),
    [anon_sym_DOT_DOT] = ACTIONS(2282),
    [anon_sym_begin] = ACTIONS(315),
    [anon_sym_PLUS] = ACTIONS(2284),
    [anon_sym_defined_QMARK] = ACTIONS(2286),
    [anon_sym_not] = ACTIONS(2288),
    [anon_sym_BANG] = ACTIONS(2292),
    [anon_sym_TILDE] = ACTIONS(2292),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(393),
    [sym_float] = ACTIONS(393),
    [sym_super] = ACTIONS(1393),
    [sym_self] = ACTIONS(1393),
    [sym_true] = ACTIONS(329),
    [sym_false] = ACTIONS(329),
    [sym_constant] = ACTIONS(1393),
    [aux_sym_constant_suffix_token1] = ACTIONS(1419),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1421),
    [sym_instance_variable] = ACTIONS(1423),
    [sym_class_variable] = ACTIONS(1423),
    [sym_global_variable] = ACTIONS(1423),
    [sym_character] = ACTIONS(337),
    [anon_sym_DASH_GT] = ACTIONS(339),
    [sym_simple_symbol] = ACTIONS(341),
    [sym__string_start] = ACTIONS(343),
    [sym__symbol_start] = ACTIONS(345),
    [sym__subshell_start] = ACTIONS(347),
    [sym__regex_start] = ACTIONS(349),
    [sym__string_array_start] = ACTIONS(351),
    [sym__symbol_array_start] = ACTIONS(353),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(337),
    [sym__unary_minus] = ACTIONS(2294),
    [sym__unary_minus_num] = ACTIONS(2296),
    [sym__binary_minus] = ACTIONS(2294),
    [sym__identifier_suffix] = ACTIONS(1429),
    [sym__constant_suffix] = ACTIONS(1419),
  },
  [1905] = {
    [sym_method] = STATE(2376),
    [sym_singleton_method] = STATE(2376),
    [sym_class] = STATE(2376),
    [sym_singleton_class] = STATE(2376),
    [sym_module] = STATE(2376),
    [sym_return] = STATE(2376),
    [sym_yield] = STATE(2376),
    [sym_break] = STATE(2376),
    [sym_next] = STATE(2376),
    [sym_redo] = STATE(2376),
    [sym_retry] = STATE(2376),
    [sym_while] = STATE(2376),
    [sym_until] = STATE(2376),
    [sym_for] = STATE(2376),
    [sym_case] = STATE(2376),
    [sym_case_match] = STATE(2376),
    [sym_if] = STATE(2376),
    [sym_unless] = STATE(2376),
    [sym_begin] = STATE(2376),
    [sym__arg] = STATE(2645),
    [sym__unary_minus_pow] = STATE(2340),
    [sym__primary] = STATE(2501),
    [sym_parenthesized_statements] = STATE(2453),
    [sym_element_reference] = STATE(2108),
    [sym_scope_resolution] = STATE(2108),
    [sym__call] = STATE(2046),
    [sym_call] = STATE(2376),
    [sym_assignment] = STATE(2343),
    [sym_operator_assignment] = STATE(2343),
    [sym_conditional] = STATE(2343),
    [sym_range] = STATE(2343),
    [sym_binary] = STATE(2343),
    [sym_unary] = STATE(2343),
    [sym_parenthesized_unary] = STATE(2376),
    [sym_unary_literal] = STATE(2286),
    [sym__literal] = STATE(2376),
    [sym__numeric] = STATE(2380),
    [sym__simple_numeric] = STATE(2286),
    [sym_left_assignment_list] = STATE(5877),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(2209),
    [sym__function_identifier_call] = STATE(2290),
    [sym__lhs] = STATE(2141),
    [sym__variable] = STATE(2044),
    [sym__nonlocal_variable] = STATE(2079),
    [sym__int_or_float] = STATE(5554),
    [sym_complex] = STATE(2293),
    [sym_rational] = STATE(2293),
    [sym_nil] = STATE(2108),
    [sym_constant_suffix] = STATE(2273),
    [sym_identifier_suffix] = STATE(2276),
    [sym_chained_string] = STATE(2376),
    [sym_string] = STATE(2234),
    [sym_subshell] = STATE(2376),
    [sym_string_array] = STATE(2376),
    [sym_symbol_array] = STATE(2376),
    [sym_delimited_symbol] = STATE(2380),
    [sym_regex] = STATE(2376),
    [sym_heredoc_body] = STATE(1905),
    [sym_array] = STATE(2376),
    [sym_hash] = STATE(2376),
    [sym_lambda] = STATE(2376),
    [sym_identifier] = ACTIONS(901),
    [anon_sym_LBRACE] = ACTIONS(15),
    [anon_sym_def] = ACTIONS(637),
    [anon_sym_LPAREN] = ACTIONS(21),
    [anon_sym_COLON_COLON] = ACTIONS(23),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1045),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(31),
    [anon_sym_class] = ACTIONS(33),
    [anon_sym_module] = ACTIONS(35),
    [anon_sym_return] = ACTIONS(909),
    [anon_sym_yield] = ACTIONS(911),
    [anon_sym_break] = ACTIONS(913),
    [anon_sym_next] = ACTIONS(915),
    [anon_sym_redo] = ACTIONS(45),
    [anon_sym_retry] = ACTIONS(47),
    [anon_sym_if] = ACTIONS(49),
    [anon_sym_unless] = ACTIONS(51),
    [anon_sym_while] = ACTIONS(53),
    [anon_sym_until] = ACTIONS(55),
    [anon_sym_for] = ACTIONS(57),
    [anon_sym_case] = ACTIONS(59),
    [anon_sym_LBRACK] = ACTIONS(61),
    [anon_sym_DOT_DOT] = ACTIONS(1047),
    [anon_sym_begin] = ACTIONS(65),
    [anon_sym_PLUS] = ACTIONS(1049),
    [anon_sym_defined_QMARK] = ACTIONS(1051),
    [anon_sym_not] = ACTIONS(1053),
    [anon_sym_BANG] = ACTIONS(1057),
    [anon_sym_TILDE] = ACTIONS(1057),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(81),
    [sym_float] = ACTIONS(81),
    [sym_super] = ACTIONS(901),
    [sym_self] = ACTIONS(901),
    [sym_true] = ACTIONS(83),
    [sym_false] = ACTIONS(83),
    [sym_constant] = ACTIONS(901),
    [aux_sym_constant_suffix_token1] = ACTIONS(929),
    [aux_sym_identifier_suffix_token1] = ACTIONS(931),
    [sym_instance_variable] = ACTIONS(933),
    [sym_class_variable] = ACTIONS(933),
    [sym_global_variable] = ACTIONS(933),
    [sym_character] = ACTIONS(91),
    [anon_sym_DASH_GT] = ACTIONS(93),
    [sym_simple_symbol] = ACTIONS(95),
    [sym__string_start] = ACTIONS(97),
    [sym__symbol_start] = ACTIONS(99),
    [sym__subshell_start] = ACTIONS(101),
    [sym__regex_start] = ACTIONS(103),
    [sym__string_array_start] = ACTIONS(105),
    [sym__symbol_array_start] = ACTIONS(107),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(91),
    [sym__unary_minus] = ACTIONS(1059),
    [sym__unary_minus_num] = ACTIONS(1061),
    [sym__binary_minus] = ACTIONS(1059),
    [sym__identifier_suffix] = ACTIONS(939),
    [sym__constant_suffix] = ACTIONS(929),
  },
  [1906] = {
    [sym_heredoc_body] = STATE(1906),
    [sym_identifier] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_RBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2614),
    [anon_sym_RPAREN] = ACTIONS(2616),
    [anon_sym_DOT] = ACTIONS(2614),
    [anon_sym_COLON_COLON] = ACTIONS(2614),
    [anon_sym_end] = ACTIONS(2614),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_EQ] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_SEMI] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2614),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_in] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_when] = ACTIONS(2614),
    [anon_sym_EQ_GT] = ACTIONS(2616),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_LPAREN2] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_CARET] = ACTIONS(2614),
    [anon_sym_elsif] = ACTIONS(2614),
    [anon_sym_else] = ACTIONS(2614),
    [anon_sym_then] = ACTIONS(2614),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_ensure] = ACTIONS(2614),
    [anon_sym_COLON_COLON2] = ACTIONS(2616),
    [anon_sym_AMP_DOT] = ACTIONS(2616),
    [anon_sym_PLUS_EQ] = ACTIONS(2616),
    [anon_sym_DASH_EQ] = ACTIONS(2616),
    [anon_sym_STAR_EQ] = ACTIONS(2616),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2616),
    [anon_sym_SLASH_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2616),
    [anon_sym_AMP_EQ] = ACTIONS(2616),
    [anon_sym_PERCENT_EQ] = ACTIONS(2616),
    [anon_sym_GT_GT_EQ] = ACTIONS(2616),
    [anon_sym_LT_LT_EQ] = ACTIONS(2616),
    [anon_sym_CARET_EQ] = ACTIONS(2616),
    [anon_sym_QMARK] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PIPE_PIPE] = ACTIONS(2614),
    [anon_sym_AMP_AMP] = ACTIONS(2614),
    [anon_sym_LT_LT] = ACTIONS(2614),
    [anon_sym_GT_GT] = ACTIONS(2614),
    [anon_sym_LT_EQ] = ACTIONS(2614),
    [anon_sym_GT] = ACTIONS(2614),
    [anon_sym_GT_EQ] = ACTIONS(2616),
    [anon_sym_PLUS] = ACTIONS(2614),
    [anon_sym_SLASH] = ACTIONS(2614),
    [anon_sym_PERCENT] = ACTIONS(2614),
    [anon_sym_EQ_EQ] = ACTIONS(2614),
    [anon_sym_BANG_EQ] = ACTIONS(2616),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2616),
    [anon_sym_LT_EQ_GT] = ACTIONS(2616),
    [anon_sym_EQ_TILDE] = ACTIONS(2616),
    [anon_sym_BANG_TILDE] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2614),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym__line_break] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym__binary_star] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
    [sym__binary_star_star] = ACTIONS(2616),
    [sym__element_reference_bracket] = ACTIONS(2616),
  },
  [1907] = {
    [sym_heredoc_body] = STATE(1907),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2620),
    [anon_sym_RBRACE] = ACTIONS(2620),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_RPAREN] = ACTIONS(2620),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_end] = ACTIONS(2622),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2622),
    [anon_sym_unless] = ACTIONS(2622),
    [anon_sym_while] = ACTIONS(2622),
    [anon_sym_until] = ACTIONS(2622),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2622),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_when] = ACTIONS(2622),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_elsif] = ACTIONS(2622),
    [anon_sym_else] = ACTIONS(2622),
    [anon_sym_then] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_ensure] = ACTIONS(2622),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1908] = {
    [sym_heredoc_body] = STATE(1908),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_RBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_end] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_EQ] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2626),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_when] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2626),
    [anon_sym_elsif] = ACTIONS(2626),
    [anon_sym_else] = ACTIONS(2626),
    [anon_sym_then] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_ensure] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_PLUS_EQ] = ACTIONS(2628),
    [anon_sym_DASH_EQ] = ACTIONS(2628),
    [anon_sym_STAR_EQ] = ACTIONS(2628),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2628),
    [anon_sym_SLASH_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2628),
    [anon_sym_AMP_EQ] = ACTIONS(2628),
    [anon_sym_PERCENT_EQ] = ACTIONS(2628),
    [anon_sym_GT_GT_EQ] = ACTIONS(2628),
    [anon_sym_LT_LT_EQ] = ACTIONS(2628),
    [anon_sym_CARET_EQ] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2626),
    [anon_sym_AMP_AMP] = ACTIONS(2626),
    [anon_sym_LT_LT] = ACTIONS(2626),
    [anon_sym_GT_GT] = ACTIONS(2626),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2626),
    [anon_sym_SLASH] = ACTIONS(2626),
    [anon_sym_PERCENT] = ACTIONS(2626),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1909] = {
    [sym_heredoc_body] = STATE(1909),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_end] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_when] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_elsif] = ACTIONS(731),
    [anon_sym_else] = ACTIONS(731),
    [anon_sym_then] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_ensure] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1910] = {
    [sym_heredoc_body] = STATE(1910),
    [sym_identifier] = ACTIONS(2630),
    [anon_sym_LBRACE] = ACTIONS(2632),
    [anon_sym_RBRACE] = ACTIONS(2632),
    [anon_sym_def] = ACTIONS(2630),
    [anon_sym_LPAREN] = ACTIONS(2630),
    [anon_sym_RPAREN] = ACTIONS(2632),
    [anon_sym_DOT] = ACTIONS(2630),
    [anon_sym_COLON_COLON] = ACTIONS(2630),
    [anon_sym_end] = ACTIONS(2630),
    [anon_sym_rescue] = ACTIONS(2630),
    [anon_sym_EQ] = ACTIONS(2630),
    [anon_sym_COMMA] = ACTIONS(2632),
    [anon_sym_PIPE] = ACTIONS(2630),
    [anon_sym_SEMI] = ACTIONS(2632),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2632),
    [anon_sym_STAR] = ACTIONS(2630),
    [anon_sym_nil] = ACTIONS(2630),
    [anon_sym_AMP] = ACTIONS(2630),
    [anon_sym_class] = ACTIONS(2630),
    [anon_sym_LT] = ACTIONS(2630),
    [anon_sym_module] = ACTIONS(2630),
    [anon_sym_return] = ACTIONS(2630),
    [anon_sym_yield] = ACTIONS(2630),
    [anon_sym_break] = ACTIONS(2630),
    [anon_sym_next] = ACTIONS(2630),
    [anon_sym_redo] = ACTIONS(2630),
    [anon_sym_retry] = ACTIONS(2630),
    [anon_sym_if] = ACTIONS(2630),
    [anon_sym_unless] = ACTIONS(2630),
    [anon_sym_while] = ACTIONS(2630),
    [anon_sym_until] = ACTIONS(2630),
    [anon_sym_for] = ACTIONS(2630),
    [anon_sym_in] = ACTIONS(2630),
    [anon_sym_do] = ACTIONS(2630),
    [anon_sym_case] = ACTIONS(2630),
    [anon_sym_when] = ACTIONS(2630),
    [anon_sym_EQ_GT] = ACTIONS(2632),
    [anon_sym_LBRACK] = ACTIONS(2632),
    [anon_sym_LPAREN2] = ACTIONS(2632),
    [anon_sym_DOT_DOT] = ACTIONS(2630),
    [anon_sym_CARET] = ACTIONS(2630),
    [anon_sym_elsif] = ACTIONS(2630),
    [anon_sym_else] = ACTIONS(2630),
    [anon_sym_then] = ACTIONS(2630),
    [anon_sym_begin] = ACTIONS(2630),
    [anon_sym_ensure] = ACTIONS(2630),
    [anon_sym_COLON_COLON2] = ACTIONS(2632),
    [anon_sym_AMP_DOT] = ACTIONS(2632),
    [anon_sym_PLUS_EQ] = ACTIONS(2632),
    [anon_sym_DASH_EQ] = ACTIONS(2632),
    [anon_sym_STAR_EQ] = ACTIONS(2632),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2632),
    [anon_sym_SLASH_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2632),
    [anon_sym_AMP_EQ] = ACTIONS(2632),
    [anon_sym_PERCENT_EQ] = ACTIONS(2632),
    [anon_sym_GT_GT_EQ] = ACTIONS(2632),
    [anon_sym_LT_LT_EQ] = ACTIONS(2632),
    [anon_sym_CARET_EQ] = ACTIONS(2632),
    [anon_sym_QMARK] = ACTIONS(2630),
    [anon_sym_and] = ACTIONS(2630),
    [anon_sym_or] = ACTIONS(2630),
    [anon_sym_PIPE_PIPE] = ACTIONS(2630),
    [anon_sym_AMP_AMP] = ACTIONS(2630),
    [anon_sym_LT_LT] = ACTIONS(2630),
    [anon_sym_GT_GT] = ACTIONS(2630),
    [anon_sym_LT_EQ] = ACTIONS(2630),
    [anon_sym_GT] = ACTIONS(2630),
    [anon_sym_GT_EQ] = ACTIONS(2632),
    [anon_sym_PLUS] = ACTIONS(2630),
    [anon_sym_SLASH] = ACTIONS(2630),
    [anon_sym_PERCENT] = ACTIONS(2630),
    [anon_sym_EQ_EQ] = ACTIONS(2630),
    [anon_sym_BANG_EQ] = ACTIONS(2632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2632),
    [anon_sym_LT_EQ_GT] = ACTIONS(2632),
    [anon_sym_EQ_TILDE] = ACTIONS(2632),
    [anon_sym_BANG_TILDE] = ACTIONS(2632),
    [anon_sym_defined_QMARK] = ACTIONS(2632),
    [anon_sym_not] = ACTIONS(2630),
    [anon_sym_BANG] = ACTIONS(2630),
    [anon_sym_TILDE] = ACTIONS(2632),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2630),
    [sym_float] = ACTIONS(2630),
    [sym_super] = ACTIONS(2630),
    [sym_self] = ACTIONS(2630),
    [sym_true] = ACTIONS(2630),
    [sym_false] = ACTIONS(2630),
    [sym_constant] = ACTIONS(2630),
    [aux_sym_constant_suffix_token1] = ACTIONS(2632),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2630),
    [sym_instance_variable] = ACTIONS(2632),
    [sym_class_variable] = ACTIONS(2632),
    [sym_global_variable] = ACTIONS(2632),
    [sym_character] = ACTIONS(2632),
    [anon_sym_DASH_GT] = ACTIONS(2632),
    [sym__line_break] = ACTIONS(2632),
    [sym_simple_symbol] = ACTIONS(2632),
    [sym__string_start] = ACTIONS(2632),
    [sym__symbol_start] = ACTIONS(2632),
    [sym__subshell_start] = ACTIONS(2632),
    [sym__regex_start] = ACTIONS(2632),
    [sym__string_array_start] = ACTIONS(2632),
    [sym__symbol_array_start] = ACTIONS(2632),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2632),
    [sym__block_ampersand] = ACTIONS(2632),
    [sym__splat_star] = ACTIONS(2632),
    [sym__unary_minus] = ACTIONS(2632),
    [sym__unary_minus_num] = ACTIONS(2632),
    [sym__binary_minus] = ACTIONS(2632),
    [sym__binary_star] = ACTIONS(2632),
    [sym_hash_key_symbol] = ACTIONS(2632),
    [sym__identifier_suffix] = ACTIONS(2632),
    [sym__constant_suffix] = ACTIONS(2632),
    [sym__hash_splat_star_star] = ACTIONS(2632),
    [sym__binary_star_star] = ACTIONS(2632),
    [sym__element_reference_bracket] = ACTIONS(2632),
  },
  [1911] = {
    [sym_heredoc_body] = STATE(1911),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_RBRACE] = ACTIONS(2624),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_RPAREN] = ACTIONS(2624),
    [anon_sym_DOT] = ACTIONS(2618),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_end] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2618),
    [anon_sym_EQ] = ACTIONS(2618),
    [anon_sym_COMMA] = ACTIONS(2624),
    [anon_sym_PIPE] = ACTIONS(2618),
    [anon_sym_SEMI] = ACTIONS(2624),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2624),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2618),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2618),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2618),
    [anon_sym_unless] = ACTIONS(2618),
    [anon_sym_while] = ACTIONS(2618),
    [anon_sym_until] = ACTIONS(2618),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2618),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_when] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2624),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2618),
    [anon_sym_CARET] = ACTIONS(2618),
    [anon_sym_elsif] = ACTIONS(2618),
    [anon_sym_else] = ACTIONS(2618),
    [anon_sym_then] = ACTIONS(2618),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_ensure] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2624),
    [anon_sym_AMP_DOT] = ACTIONS(2624),
    [anon_sym_PLUS_EQ] = ACTIONS(2624),
    [anon_sym_DASH_EQ] = ACTIONS(2624),
    [anon_sym_STAR_EQ] = ACTIONS(2624),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2624),
    [anon_sym_SLASH_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2624),
    [anon_sym_AMP_EQ] = ACTIONS(2624),
    [anon_sym_PERCENT_EQ] = ACTIONS(2624),
    [anon_sym_GT_GT_EQ] = ACTIONS(2624),
    [anon_sym_LT_LT_EQ] = ACTIONS(2624),
    [anon_sym_CARET_EQ] = ACTIONS(2624),
    [anon_sym_QMARK] = ACTIONS(2618),
    [anon_sym_and] = ACTIONS(2618),
    [anon_sym_or] = ACTIONS(2618),
    [anon_sym_PIPE_PIPE] = ACTIONS(2618),
    [anon_sym_AMP_AMP] = ACTIONS(2618),
    [anon_sym_LT_LT] = ACTIONS(2618),
    [anon_sym_GT_GT] = ACTIONS(2618),
    [anon_sym_LT_EQ] = ACTIONS(2618),
    [anon_sym_GT] = ACTIONS(2618),
    [anon_sym_GT_EQ] = ACTIONS(2624),
    [anon_sym_PLUS] = ACTIONS(2618),
    [anon_sym_SLASH] = ACTIONS(2618),
    [anon_sym_PERCENT] = ACTIONS(2618),
    [anon_sym_EQ_EQ] = ACTIONS(2618),
    [anon_sym_BANG_EQ] = ACTIONS(2624),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2624),
    [anon_sym_LT_EQ_GT] = ACTIONS(2624),
    [anon_sym_EQ_TILDE] = ACTIONS(2624),
    [anon_sym_BANG_TILDE] = ACTIONS(2624),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2624),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2624),
    [sym__binary_star] = ACTIONS(2624),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2624),
    [sym__element_reference_bracket] = ACTIONS(2624),
  },
  [1912] = {
    [sym_heredoc_body] = STATE(1912),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_RBRACE] = ACTIONS(2620),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_RPAREN] = ACTIONS(2620),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_end] = ACTIONS(2622),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2622),
    [anon_sym_unless] = ACTIONS(2622),
    [anon_sym_while] = ACTIONS(2622),
    [anon_sym_until] = ACTIONS(2622),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_when] = ACTIONS(2622),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_elsif] = ACTIONS(2622),
    [anon_sym_else] = ACTIONS(2622),
    [anon_sym_then] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_ensure] = ACTIONS(2622),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1913] = {
    [sym_heredoc_body] = STATE(1913),
    [sym_identifier] = ACTIONS(2634),
    [anon_sym_LBRACE] = ACTIONS(2636),
    [anon_sym_RBRACE] = ACTIONS(2636),
    [anon_sym_def] = ACTIONS(2634),
    [anon_sym_LPAREN] = ACTIONS(2634),
    [anon_sym_RPAREN] = ACTIONS(2636),
    [anon_sym_DOT] = ACTIONS(2634),
    [anon_sym_COLON_COLON] = ACTIONS(2634),
    [anon_sym_end] = ACTIONS(2634),
    [anon_sym_rescue] = ACTIONS(2634),
    [anon_sym_EQ] = ACTIONS(2634),
    [anon_sym_COMMA] = ACTIONS(2636),
    [anon_sym_PIPE] = ACTIONS(2634),
    [anon_sym_SEMI] = ACTIONS(2636),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2636),
    [anon_sym_STAR] = ACTIONS(2634),
    [anon_sym_nil] = ACTIONS(2634),
    [anon_sym_AMP] = ACTIONS(2634),
    [anon_sym_class] = ACTIONS(2634),
    [anon_sym_LT] = ACTIONS(2634),
    [anon_sym_module] = ACTIONS(2634),
    [anon_sym_return] = ACTIONS(2634),
    [anon_sym_yield] = ACTIONS(2634),
    [anon_sym_break] = ACTIONS(2634),
    [anon_sym_next] = ACTIONS(2634),
    [anon_sym_redo] = ACTIONS(2634),
    [anon_sym_retry] = ACTIONS(2634),
    [anon_sym_if] = ACTIONS(2634),
    [anon_sym_unless] = ACTIONS(2634),
    [anon_sym_while] = ACTIONS(2634),
    [anon_sym_until] = ACTIONS(2634),
    [anon_sym_for] = ACTIONS(2634),
    [anon_sym_in] = ACTIONS(2634),
    [anon_sym_do] = ACTIONS(2634),
    [anon_sym_case] = ACTIONS(2634),
    [anon_sym_when] = ACTIONS(2634),
    [anon_sym_EQ_GT] = ACTIONS(2636),
    [anon_sym_LBRACK] = ACTIONS(2636),
    [anon_sym_LPAREN2] = ACTIONS(2636),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_CARET] = ACTIONS(2634),
    [anon_sym_elsif] = ACTIONS(2634),
    [anon_sym_else] = ACTIONS(2634),
    [anon_sym_then] = ACTIONS(2634),
    [anon_sym_begin] = ACTIONS(2634),
    [anon_sym_ensure] = ACTIONS(2634),
    [anon_sym_COLON_COLON2] = ACTIONS(2636),
    [anon_sym_AMP_DOT] = ACTIONS(2636),
    [anon_sym_PLUS_EQ] = ACTIONS(2636),
    [anon_sym_DASH_EQ] = ACTIONS(2636),
    [anon_sym_STAR_EQ] = ACTIONS(2636),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2636),
    [anon_sym_SLASH_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2636),
    [anon_sym_AMP_EQ] = ACTIONS(2636),
    [anon_sym_PERCENT_EQ] = ACTIONS(2636),
    [anon_sym_GT_GT_EQ] = ACTIONS(2636),
    [anon_sym_LT_LT_EQ] = ACTIONS(2636),
    [anon_sym_CARET_EQ] = ACTIONS(2636),
    [anon_sym_QMARK] = ACTIONS(2634),
    [anon_sym_and] = ACTIONS(2634),
    [anon_sym_or] = ACTIONS(2634),
    [anon_sym_PIPE_PIPE] = ACTIONS(2634),
    [anon_sym_AMP_AMP] = ACTIONS(2634),
    [anon_sym_LT_LT] = ACTIONS(2634),
    [anon_sym_GT_GT] = ACTIONS(2634),
    [anon_sym_LT_EQ] = ACTIONS(2634),
    [anon_sym_GT] = ACTIONS(2634),
    [anon_sym_GT_EQ] = ACTIONS(2636),
    [anon_sym_PLUS] = ACTIONS(2634),
    [anon_sym_SLASH] = ACTIONS(2634),
    [anon_sym_PERCENT] = ACTIONS(2634),
    [anon_sym_EQ_EQ] = ACTIONS(2634),
    [anon_sym_BANG_EQ] = ACTIONS(2636),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2636),
    [anon_sym_LT_EQ_GT] = ACTIONS(2636),
    [anon_sym_EQ_TILDE] = ACTIONS(2636),
    [anon_sym_BANG_TILDE] = ACTIONS(2636),
    [anon_sym_defined_QMARK] = ACTIONS(2636),
    [anon_sym_not] = ACTIONS(2634),
    [anon_sym_BANG] = ACTIONS(2634),
    [anon_sym_TILDE] = ACTIONS(2636),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2634),
    [sym_float] = ACTIONS(2634),
    [sym_super] = ACTIONS(2634),
    [sym_self] = ACTIONS(2634),
    [sym_true] = ACTIONS(2634),
    [sym_false] = ACTIONS(2634),
    [sym_constant] = ACTIONS(2634),
    [aux_sym_constant_suffix_token1] = ACTIONS(2636),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2634),
    [sym_instance_variable] = ACTIONS(2636),
    [sym_class_variable] = ACTIONS(2636),
    [sym_global_variable] = ACTIONS(2636),
    [sym_character] = ACTIONS(2636),
    [anon_sym_DASH_GT] = ACTIONS(2636),
    [sym__line_break] = ACTIONS(2636),
    [sym_simple_symbol] = ACTIONS(2636),
    [sym__string_start] = ACTIONS(2636),
    [sym__symbol_start] = ACTIONS(2636),
    [sym__subshell_start] = ACTIONS(2636),
    [sym__regex_start] = ACTIONS(2636),
    [sym__string_array_start] = ACTIONS(2636),
    [sym__symbol_array_start] = ACTIONS(2636),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2636),
    [sym__block_ampersand] = ACTIONS(2636),
    [sym__splat_star] = ACTIONS(2636),
    [sym__unary_minus] = ACTIONS(2636),
    [sym__unary_minus_num] = ACTIONS(2636),
    [sym__binary_minus] = ACTIONS(2636),
    [sym__binary_star] = ACTIONS(2636),
    [sym_hash_key_symbol] = ACTIONS(2636),
    [sym__identifier_suffix] = ACTIONS(2636),
    [sym__constant_suffix] = ACTIONS(2636),
    [sym__hash_splat_star_star] = ACTIONS(2636),
    [sym__binary_star_star] = ACTIONS(2636),
    [sym__element_reference_bracket] = ACTIONS(2636),
  },
  [1914] = {
    [sym_heredoc_body] = STATE(1914),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_RBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_end] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_EQ] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2638),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_when] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2638),
    [anon_sym_elsif] = ACTIONS(2638),
    [anon_sym_else] = ACTIONS(2638),
    [anon_sym_then] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_ensure] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_PLUS_EQ] = ACTIONS(2640),
    [anon_sym_DASH_EQ] = ACTIONS(2640),
    [anon_sym_STAR_EQ] = ACTIONS(2640),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2640),
    [anon_sym_SLASH_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2640),
    [anon_sym_AMP_EQ] = ACTIONS(2640),
    [anon_sym_PERCENT_EQ] = ACTIONS(2640),
    [anon_sym_GT_GT_EQ] = ACTIONS(2640),
    [anon_sym_LT_LT_EQ] = ACTIONS(2640),
    [anon_sym_CARET_EQ] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2638),
    [anon_sym_AMP_AMP] = ACTIONS(2638),
    [anon_sym_LT_LT] = ACTIONS(2638),
    [anon_sym_GT_GT] = ACTIONS(2638),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2638),
    [anon_sym_SLASH] = ACTIONS(2638),
    [anon_sym_PERCENT] = ACTIONS(2638),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1915] = {
    [sym_heredoc_body] = STATE(1915),
    [sym_identifier] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_RBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2642),
    [anon_sym_RPAREN] = ACTIONS(2644),
    [anon_sym_DOT] = ACTIONS(2642),
    [anon_sym_COLON_COLON] = ACTIONS(2642),
    [anon_sym_end] = ACTIONS(2642),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_EQ] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_SEMI] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2642),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_in] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_when] = ACTIONS(2642),
    [anon_sym_EQ_GT] = ACTIONS(2644),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_LPAREN2] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_CARET] = ACTIONS(2642),
    [anon_sym_elsif] = ACTIONS(2642),
    [anon_sym_else] = ACTIONS(2642),
    [anon_sym_then] = ACTIONS(2642),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_ensure] = ACTIONS(2642),
    [anon_sym_COLON_COLON2] = ACTIONS(2644),
    [anon_sym_AMP_DOT] = ACTIONS(2644),
    [anon_sym_PLUS_EQ] = ACTIONS(2644),
    [anon_sym_DASH_EQ] = ACTIONS(2644),
    [anon_sym_STAR_EQ] = ACTIONS(2644),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2644),
    [anon_sym_SLASH_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2644),
    [anon_sym_AMP_EQ] = ACTIONS(2644),
    [anon_sym_PERCENT_EQ] = ACTIONS(2644),
    [anon_sym_GT_GT_EQ] = ACTIONS(2644),
    [anon_sym_LT_LT_EQ] = ACTIONS(2644),
    [anon_sym_CARET_EQ] = ACTIONS(2644),
    [anon_sym_QMARK] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PIPE_PIPE] = ACTIONS(2642),
    [anon_sym_AMP_AMP] = ACTIONS(2642),
    [anon_sym_LT_LT] = ACTIONS(2642),
    [anon_sym_GT_GT] = ACTIONS(2642),
    [anon_sym_LT_EQ] = ACTIONS(2642),
    [anon_sym_GT] = ACTIONS(2642),
    [anon_sym_GT_EQ] = ACTIONS(2644),
    [anon_sym_PLUS] = ACTIONS(2642),
    [anon_sym_SLASH] = ACTIONS(2642),
    [anon_sym_PERCENT] = ACTIONS(2642),
    [anon_sym_EQ_EQ] = ACTIONS(2642),
    [anon_sym_BANG_EQ] = ACTIONS(2644),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2644),
    [anon_sym_LT_EQ_GT] = ACTIONS(2644),
    [anon_sym_EQ_TILDE] = ACTIONS(2644),
    [anon_sym_BANG_TILDE] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2642),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym__line_break] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym__binary_star] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
    [sym__binary_star_star] = ACTIONS(2644),
    [sym__element_reference_bracket] = ACTIONS(2644),
  },
  [1916] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5016),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1916),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2646),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_RBRACE] = ACTIONS(2650),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_RPAREN] = ACTIONS(2650),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_end] = ACTIONS(2658),
    [anon_sym_rescue] = ACTIONS(2658),
    [anon_sym_EQ] = ACTIONS(2660),
    [anon_sym_COMMA] = ACTIONS(2660),
    [anon_sym_SEMI] = ACTIONS(2650),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_in] = ACTIONS(2658),
    [anon_sym_do] = ACTIONS(2658),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_when] = ACTIONS(2658),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_elsif] = ACTIONS(2658),
    [anon_sym_else] = ACTIONS(2658),
    [anon_sym_then] = ACTIONS(2658),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_ensure] = ACTIONS(2658),
    [anon_sym_and] = ACTIONS(2658),
    [anon_sym_or] = ACTIONS(2658),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym__line_break] = ACTIONS(2650),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1917] = {
    [sym_heredoc_body] = STATE(1917),
    [ts_builtin_sym_end] = ACTIONS(743),
    [sym_identifier] = ACTIONS(731),
    [aux_sym_program_token1] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2734),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1918] = {
    [sym_heredoc_body] = STATE(1918),
    [ts_builtin_sym_end] = ACTIONS(743),
    [sym_identifier] = ACTIONS(731),
    [aux_sym_program_token1] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2736),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1919] = {
    [sym_heredoc_body] = STATE(1919),
    [ts_builtin_sym_end] = ACTIONS(2620),
    [sym_identifier] = ACTIONS(2618),
    [aux_sym_program_token1] = ACTIONS(2622),
    [anon_sym_LBRACE] = ACTIONS(2620),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2622),
    [anon_sym_unless] = ACTIONS(2622),
    [anon_sym_while] = ACTIONS(2622),
    [anon_sym_until] = ACTIONS(2622),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2622),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1920] = {
    [sym_heredoc_body] = STATE(1920),
    [ts_builtin_sym_end] = ACTIONS(2620),
    [sym_identifier] = ACTIONS(2618),
    [aux_sym_program_token1] = ACTIONS(2622),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2622),
    [anon_sym_unless] = ACTIONS(2622),
    [anon_sym_while] = ACTIONS(2622),
    [anon_sym_until] = ACTIONS(2622),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1921] = {
    [sym_heredoc_body] = STATE(1921),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1922] = {
    [sym_heredoc_body] = STATE(1922),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(731),
    [anon_sym_LBRACK2] = ACTIONS(2740),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1923] = {
    [sym_heredoc_body] = STATE(1923),
    [ts_builtin_sym_end] = ACTIONS(2636),
    [sym_identifier] = ACTIONS(2634),
    [aux_sym_program_token1] = ACTIONS(2634),
    [anon_sym_LBRACE] = ACTIONS(2636),
    [anon_sym_def] = ACTIONS(2634),
    [anon_sym_LPAREN] = ACTIONS(2634),
    [anon_sym_DOT] = ACTIONS(2634),
    [anon_sym_COLON_COLON] = ACTIONS(2634),
    [anon_sym_rescue] = ACTIONS(2634),
    [anon_sym_EQ] = ACTIONS(2634),
    [anon_sym_COMMA] = ACTIONS(2636),
    [anon_sym_PIPE] = ACTIONS(2634),
    [anon_sym_SEMI] = ACTIONS(2636),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2636),
    [anon_sym_STAR] = ACTIONS(2634),
    [anon_sym_nil] = ACTIONS(2634),
    [anon_sym_AMP] = ACTIONS(2634),
    [anon_sym_class] = ACTIONS(2634),
    [anon_sym_LT] = ACTIONS(2634),
    [anon_sym_module] = ACTIONS(2634),
    [anon_sym_return] = ACTIONS(2634),
    [anon_sym_yield] = ACTIONS(2634),
    [anon_sym_break] = ACTIONS(2634),
    [anon_sym_next] = ACTIONS(2634),
    [anon_sym_redo] = ACTIONS(2634),
    [anon_sym_retry] = ACTIONS(2634),
    [anon_sym_if] = ACTIONS(2634),
    [anon_sym_unless] = ACTIONS(2634),
    [anon_sym_while] = ACTIONS(2634),
    [anon_sym_until] = ACTIONS(2634),
    [anon_sym_for] = ACTIONS(2634),
    [anon_sym_in] = ACTIONS(2634),
    [anon_sym_do] = ACTIONS(2634),
    [anon_sym_case] = ACTIONS(2634),
    [anon_sym_EQ_GT] = ACTIONS(2636),
    [anon_sym_LBRACK] = ACTIONS(2636),
    [anon_sym_LPAREN2] = ACTIONS(2636),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_CARET] = ACTIONS(2634),
    [anon_sym_begin] = ACTIONS(2634),
    [anon_sym_COLON_COLON2] = ACTIONS(2636),
    [anon_sym_AMP_DOT] = ACTIONS(2636),
    [anon_sym_PLUS_EQ] = ACTIONS(2636),
    [anon_sym_DASH_EQ] = ACTIONS(2636),
    [anon_sym_STAR_EQ] = ACTIONS(2636),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2636),
    [anon_sym_SLASH_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2636),
    [anon_sym_AMP_EQ] = ACTIONS(2636),
    [anon_sym_PERCENT_EQ] = ACTIONS(2636),
    [anon_sym_GT_GT_EQ] = ACTIONS(2636),
    [anon_sym_LT_LT_EQ] = ACTIONS(2636),
    [anon_sym_CARET_EQ] = ACTIONS(2636),
    [anon_sym_QMARK] = ACTIONS(2634),
    [anon_sym_and] = ACTIONS(2634),
    [anon_sym_or] = ACTIONS(2634),
    [anon_sym_PIPE_PIPE] = ACTIONS(2634),
    [anon_sym_AMP_AMP] = ACTIONS(2634),
    [anon_sym_LT_LT] = ACTIONS(2634),
    [anon_sym_GT_GT] = ACTIONS(2634),
    [anon_sym_LT_EQ] = ACTIONS(2634),
    [anon_sym_GT] = ACTIONS(2634),
    [anon_sym_GT_EQ] = ACTIONS(2636),
    [anon_sym_PLUS] = ACTIONS(2634),
    [anon_sym_SLASH] = ACTIONS(2634),
    [anon_sym_PERCENT] = ACTIONS(2634),
    [anon_sym_EQ_EQ] = ACTIONS(2634),
    [anon_sym_BANG_EQ] = ACTIONS(2636),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2636),
    [anon_sym_LT_EQ_GT] = ACTIONS(2636),
    [anon_sym_EQ_TILDE] = ACTIONS(2636),
    [anon_sym_BANG_TILDE] = ACTIONS(2636),
    [anon_sym_defined_QMARK] = ACTIONS(2636),
    [anon_sym_not] = ACTIONS(2634),
    [anon_sym_BANG] = ACTIONS(2634),
    [anon_sym_TILDE] = ACTIONS(2636),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2634),
    [sym_float] = ACTIONS(2634),
    [sym_super] = ACTIONS(2634),
    [sym_self] = ACTIONS(2634),
    [sym_true] = ACTIONS(2634),
    [sym_false] = ACTIONS(2634),
    [sym_constant] = ACTIONS(2634),
    [aux_sym_constant_suffix_token1] = ACTIONS(2636),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2634),
    [sym_instance_variable] = ACTIONS(2636),
    [sym_class_variable] = ACTIONS(2636),
    [sym_global_variable] = ACTIONS(2636),
    [sym_character] = ACTIONS(2636),
    [anon_sym_DASH_GT] = ACTIONS(2636),
    [sym__line_break] = ACTIONS(2636),
    [sym_simple_symbol] = ACTIONS(2636),
    [sym__string_start] = ACTIONS(2636),
    [sym__symbol_start] = ACTIONS(2636),
    [sym__subshell_start] = ACTIONS(2636),
    [sym__regex_start] = ACTIONS(2636),
    [sym__string_array_start] = ACTIONS(2636),
    [sym__symbol_array_start] = ACTIONS(2636),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2636),
    [sym__block_ampersand] = ACTIONS(2636),
    [sym__splat_star] = ACTIONS(2636),
    [sym__unary_minus] = ACTIONS(2636),
    [sym__unary_minus_num] = ACTIONS(2636),
    [sym__binary_minus] = ACTIONS(2636),
    [sym__binary_star] = ACTIONS(2636),
    [sym_hash_key_symbol] = ACTIONS(2636),
    [sym__identifier_suffix] = ACTIONS(2636),
    [sym__constant_suffix] = ACTIONS(2636),
    [sym__hash_splat_star_star] = ACTIONS(2636),
    [sym__binary_star_star] = ACTIONS(2636),
    [sym__element_reference_bracket] = ACTIONS(2636),
  },
  [1924] = {
    [sym_heredoc_body] = STATE(1924),
    [ts_builtin_sym_end] = ACTIONS(743),
    [sym_identifier] = ACTIONS(731),
    [aux_sym_program_token1] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1925] = {
    [sym_heredoc_body] = STATE(1925),
    [ts_builtin_sym_end] = ACTIONS(2628),
    [sym_identifier] = ACTIONS(2626),
    [aux_sym_program_token1] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_EQ] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2626),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_PLUS_EQ] = ACTIONS(2628),
    [anon_sym_DASH_EQ] = ACTIONS(2628),
    [anon_sym_STAR_EQ] = ACTIONS(2628),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2628),
    [anon_sym_SLASH_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2628),
    [anon_sym_AMP_EQ] = ACTIONS(2628),
    [anon_sym_PERCENT_EQ] = ACTIONS(2628),
    [anon_sym_GT_GT_EQ] = ACTIONS(2628),
    [anon_sym_LT_LT_EQ] = ACTIONS(2628),
    [anon_sym_CARET_EQ] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2626),
    [anon_sym_AMP_AMP] = ACTIONS(2626),
    [anon_sym_LT_LT] = ACTIONS(2626),
    [anon_sym_GT_GT] = ACTIONS(2626),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2626),
    [anon_sym_SLASH] = ACTIONS(2626),
    [anon_sym_PERCENT] = ACTIONS(2626),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1926] = {
    [sym_heredoc_body] = STATE(1926),
    [ts_builtin_sym_end] = ACTIONS(2616),
    [sym_identifier] = ACTIONS(2614),
    [aux_sym_program_token1] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2614),
    [anon_sym_DOT] = ACTIONS(2614),
    [anon_sym_COLON_COLON] = ACTIONS(2614),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_EQ] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_SEMI] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2614),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_in] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_EQ_GT] = ACTIONS(2616),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_LPAREN2] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_CARET] = ACTIONS(2614),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_COLON_COLON2] = ACTIONS(2616),
    [anon_sym_AMP_DOT] = ACTIONS(2616),
    [anon_sym_PLUS_EQ] = ACTIONS(2616),
    [anon_sym_DASH_EQ] = ACTIONS(2616),
    [anon_sym_STAR_EQ] = ACTIONS(2616),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2616),
    [anon_sym_SLASH_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2616),
    [anon_sym_AMP_EQ] = ACTIONS(2616),
    [anon_sym_PERCENT_EQ] = ACTIONS(2616),
    [anon_sym_GT_GT_EQ] = ACTIONS(2616),
    [anon_sym_LT_LT_EQ] = ACTIONS(2616),
    [anon_sym_CARET_EQ] = ACTIONS(2616),
    [anon_sym_QMARK] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PIPE_PIPE] = ACTIONS(2614),
    [anon_sym_AMP_AMP] = ACTIONS(2614),
    [anon_sym_LT_LT] = ACTIONS(2614),
    [anon_sym_GT_GT] = ACTIONS(2614),
    [anon_sym_LT_EQ] = ACTIONS(2614),
    [anon_sym_GT] = ACTIONS(2614),
    [anon_sym_GT_EQ] = ACTIONS(2616),
    [anon_sym_PLUS] = ACTIONS(2614),
    [anon_sym_SLASH] = ACTIONS(2614),
    [anon_sym_PERCENT] = ACTIONS(2614),
    [anon_sym_EQ_EQ] = ACTIONS(2614),
    [anon_sym_BANG_EQ] = ACTIONS(2616),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2616),
    [anon_sym_LT_EQ_GT] = ACTIONS(2616),
    [anon_sym_EQ_TILDE] = ACTIONS(2616),
    [anon_sym_BANG_TILDE] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2614),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym__line_break] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym__binary_star] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
    [sym__binary_star_star] = ACTIONS(2616),
    [sym__element_reference_bracket] = ACTIONS(2616),
  },
  [1927] = {
    [sym_heredoc_body] = STATE(1927),
    [ts_builtin_sym_end] = ACTIONS(2644),
    [sym_identifier] = ACTIONS(2642),
    [aux_sym_program_token1] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2642),
    [anon_sym_DOT] = ACTIONS(2642),
    [anon_sym_COLON_COLON] = ACTIONS(2642),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_EQ] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_SEMI] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2642),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_in] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_EQ_GT] = ACTIONS(2644),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_LPAREN2] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_CARET] = ACTIONS(2642),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_COLON_COLON2] = ACTIONS(2644),
    [anon_sym_AMP_DOT] = ACTIONS(2644),
    [anon_sym_PLUS_EQ] = ACTIONS(2644),
    [anon_sym_DASH_EQ] = ACTIONS(2644),
    [anon_sym_STAR_EQ] = ACTIONS(2644),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2644),
    [anon_sym_SLASH_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2644),
    [anon_sym_AMP_EQ] = ACTIONS(2644),
    [anon_sym_PERCENT_EQ] = ACTIONS(2644),
    [anon_sym_GT_GT_EQ] = ACTIONS(2644),
    [anon_sym_LT_LT_EQ] = ACTIONS(2644),
    [anon_sym_CARET_EQ] = ACTIONS(2644),
    [anon_sym_QMARK] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PIPE_PIPE] = ACTIONS(2642),
    [anon_sym_AMP_AMP] = ACTIONS(2642),
    [anon_sym_LT_LT] = ACTIONS(2642),
    [anon_sym_GT_GT] = ACTIONS(2642),
    [anon_sym_LT_EQ] = ACTIONS(2642),
    [anon_sym_GT] = ACTIONS(2642),
    [anon_sym_GT_EQ] = ACTIONS(2644),
    [anon_sym_PLUS] = ACTIONS(2642),
    [anon_sym_SLASH] = ACTIONS(2642),
    [anon_sym_PERCENT] = ACTIONS(2642),
    [anon_sym_EQ_EQ] = ACTIONS(2642),
    [anon_sym_BANG_EQ] = ACTIONS(2644),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2644),
    [anon_sym_LT_EQ_GT] = ACTIONS(2644),
    [anon_sym_EQ_TILDE] = ACTIONS(2644),
    [anon_sym_BANG_TILDE] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2642),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym__line_break] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym__binary_star] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
    [sym__binary_star_star] = ACTIONS(2644),
    [sym__element_reference_bracket] = ACTIONS(2644),
  },
  [1928] = {
    [sym_heredoc_body] = STATE(1928),
    [ts_builtin_sym_end] = ACTIONS(2640),
    [sym_identifier] = ACTIONS(2638),
    [aux_sym_program_token1] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_EQ] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2638),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_PLUS_EQ] = ACTIONS(2640),
    [anon_sym_DASH_EQ] = ACTIONS(2640),
    [anon_sym_STAR_EQ] = ACTIONS(2640),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2640),
    [anon_sym_SLASH_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2640),
    [anon_sym_AMP_EQ] = ACTIONS(2640),
    [anon_sym_PERCENT_EQ] = ACTIONS(2640),
    [anon_sym_GT_GT_EQ] = ACTIONS(2640),
    [anon_sym_LT_LT_EQ] = ACTIONS(2640),
    [anon_sym_CARET_EQ] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2638),
    [anon_sym_AMP_AMP] = ACTIONS(2638),
    [anon_sym_LT_LT] = ACTIONS(2638),
    [anon_sym_GT_GT] = ACTIONS(2638),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2638),
    [anon_sym_SLASH] = ACTIONS(2638),
    [anon_sym_PERCENT] = ACTIONS(2638),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1929] = {
    [sym_heredoc_body] = STATE(1929),
    [ts_builtin_sym_end] = ACTIONS(2632),
    [sym_identifier] = ACTIONS(2630),
    [aux_sym_program_token1] = ACTIONS(2630),
    [anon_sym_LBRACE] = ACTIONS(2632),
    [anon_sym_def] = ACTIONS(2630),
    [anon_sym_LPAREN] = ACTIONS(2630),
    [anon_sym_DOT] = ACTIONS(2630),
    [anon_sym_COLON_COLON] = ACTIONS(2630),
    [anon_sym_rescue] = ACTIONS(2630),
    [anon_sym_EQ] = ACTIONS(2630),
    [anon_sym_COMMA] = ACTIONS(2632),
    [anon_sym_PIPE] = ACTIONS(2630),
    [anon_sym_SEMI] = ACTIONS(2632),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2632),
    [anon_sym_STAR] = ACTIONS(2630),
    [anon_sym_nil] = ACTIONS(2630),
    [anon_sym_AMP] = ACTIONS(2630),
    [anon_sym_class] = ACTIONS(2630),
    [anon_sym_LT] = ACTIONS(2630),
    [anon_sym_module] = ACTIONS(2630),
    [anon_sym_return] = ACTIONS(2630),
    [anon_sym_yield] = ACTIONS(2630),
    [anon_sym_break] = ACTIONS(2630),
    [anon_sym_next] = ACTIONS(2630),
    [anon_sym_redo] = ACTIONS(2630),
    [anon_sym_retry] = ACTIONS(2630),
    [anon_sym_if] = ACTIONS(2630),
    [anon_sym_unless] = ACTIONS(2630),
    [anon_sym_while] = ACTIONS(2630),
    [anon_sym_until] = ACTIONS(2630),
    [anon_sym_for] = ACTIONS(2630),
    [anon_sym_in] = ACTIONS(2630),
    [anon_sym_do] = ACTIONS(2630),
    [anon_sym_case] = ACTIONS(2630),
    [anon_sym_EQ_GT] = ACTIONS(2632),
    [anon_sym_LBRACK] = ACTIONS(2632),
    [anon_sym_LPAREN2] = ACTIONS(2632),
    [anon_sym_DOT_DOT] = ACTIONS(2630),
    [anon_sym_CARET] = ACTIONS(2630),
    [anon_sym_begin] = ACTIONS(2630),
    [anon_sym_COLON_COLON2] = ACTIONS(2632),
    [anon_sym_AMP_DOT] = ACTIONS(2632),
    [anon_sym_PLUS_EQ] = ACTIONS(2632),
    [anon_sym_DASH_EQ] = ACTIONS(2632),
    [anon_sym_STAR_EQ] = ACTIONS(2632),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2632),
    [anon_sym_SLASH_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2632),
    [anon_sym_AMP_EQ] = ACTIONS(2632),
    [anon_sym_PERCENT_EQ] = ACTIONS(2632),
    [anon_sym_GT_GT_EQ] = ACTIONS(2632),
    [anon_sym_LT_LT_EQ] = ACTIONS(2632),
    [anon_sym_CARET_EQ] = ACTIONS(2632),
    [anon_sym_QMARK] = ACTIONS(2630),
    [anon_sym_and] = ACTIONS(2630),
    [anon_sym_or] = ACTIONS(2630),
    [anon_sym_PIPE_PIPE] = ACTIONS(2630),
    [anon_sym_AMP_AMP] = ACTIONS(2630),
    [anon_sym_LT_LT] = ACTIONS(2630),
    [anon_sym_GT_GT] = ACTIONS(2630),
    [anon_sym_LT_EQ] = ACTIONS(2630),
    [anon_sym_GT] = ACTIONS(2630),
    [anon_sym_GT_EQ] = ACTIONS(2632),
    [anon_sym_PLUS] = ACTIONS(2630),
    [anon_sym_SLASH] = ACTIONS(2630),
    [anon_sym_PERCENT] = ACTIONS(2630),
    [anon_sym_EQ_EQ] = ACTIONS(2630),
    [anon_sym_BANG_EQ] = ACTIONS(2632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2632),
    [anon_sym_LT_EQ_GT] = ACTIONS(2632),
    [anon_sym_EQ_TILDE] = ACTIONS(2632),
    [anon_sym_BANG_TILDE] = ACTIONS(2632),
    [anon_sym_defined_QMARK] = ACTIONS(2632),
    [anon_sym_not] = ACTIONS(2630),
    [anon_sym_BANG] = ACTIONS(2630),
    [anon_sym_TILDE] = ACTIONS(2632),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2630),
    [sym_float] = ACTIONS(2630),
    [sym_super] = ACTIONS(2630),
    [sym_self] = ACTIONS(2630),
    [sym_true] = ACTIONS(2630),
    [sym_false] = ACTIONS(2630),
    [sym_constant] = ACTIONS(2630),
    [aux_sym_constant_suffix_token1] = ACTIONS(2632),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2630),
    [sym_instance_variable] = ACTIONS(2632),
    [sym_class_variable] = ACTIONS(2632),
    [sym_global_variable] = ACTIONS(2632),
    [sym_character] = ACTIONS(2632),
    [anon_sym_DASH_GT] = ACTIONS(2632),
    [sym__line_break] = ACTIONS(2632),
    [sym_simple_symbol] = ACTIONS(2632),
    [sym__string_start] = ACTIONS(2632),
    [sym__symbol_start] = ACTIONS(2632),
    [sym__subshell_start] = ACTIONS(2632),
    [sym__regex_start] = ACTIONS(2632),
    [sym__string_array_start] = ACTIONS(2632),
    [sym__symbol_array_start] = ACTIONS(2632),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2632),
    [sym__block_ampersand] = ACTIONS(2632),
    [sym__splat_star] = ACTIONS(2632),
    [sym__unary_minus] = ACTIONS(2632),
    [sym__unary_minus_num] = ACTIONS(2632),
    [sym__binary_minus] = ACTIONS(2632),
    [sym__binary_star] = ACTIONS(2632),
    [sym_hash_key_symbol] = ACTIONS(2632),
    [sym__identifier_suffix] = ACTIONS(2632),
    [sym__constant_suffix] = ACTIONS(2632),
    [sym__hash_splat_star_star] = ACTIONS(2632),
    [sym__binary_star_star] = ACTIONS(2632),
    [sym__element_reference_bracket] = ACTIONS(2632),
  },
  [1930] = {
    [sym_heredoc_body] = STATE(1930),
    [ts_builtin_sym_end] = ACTIONS(2624),
    [sym_identifier] = ACTIONS(2618),
    [aux_sym_program_token1] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_DOT] = ACTIONS(2618),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2618),
    [anon_sym_EQ] = ACTIONS(2618),
    [anon_sym_COMMA] = ACTIONS(2624),
    [anon_sym_PIPE] = ACTIONS(2618),
    [anon_sym_SEMI] = ACTIONS(2624),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2624),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2618),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2618),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2618),
    [anon_sym_unless] = ACTIONS(2618),
    [anon_sym_while] = ACTIONS(2618),
    [anon_sym_until] = ACTIONS(2618),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2618),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2624),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2618),
    [anon_sym_CARET] = ACTIONS(2618),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2624),
    [anon_sym_AMP_DOT] = ACTIONS(2624),
    [anon_sym_PLUS_EQ] = ACTIONS(2624),
    [anon_sym_DASH_EQ] = ACTIONS(2624),
    [anon_sym_STAR_EQ] = ACTIONS(2624),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2624),
    [anon_sym_SLASH_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2624),
    [anon_sym_AMP_EQ] = ACTIONS(2624),
    [anon_sym_PERCENT_EQ] = ACTIONS(2624),
    [anon_sym_GT_GT_EQ] = ACTIONS(2624),
    [anon_sym_LT_LT_EQ] = ACTIONS(2624),
    [anon_sym_CARET_EQ] = ACTIONS(2624),
    [anon_sym_QMARK] = ACTIONS(2618),
    [anon_sym_and] = ACTIONS(2618),
    [anon_sym_or] = ACTIONS(2618),
    [anon_sym_PIPE_PIPE] = ACTIONS(2618),
    [anon_sym_AMP_AMP] = ACTIONS(2618),
    [anon_sym_LT_LT] = ACTIONS(2618),
    [anon_sym_GT_GT] = ACTIONS(2618),
    [anon_sym_LT_EQ] = ACTIONS(2618),
    [anon_sym_GT] = ACTIONS(2618),
    [anon_sym_GT_EQ] = ACTIONS(2624),
    [anon_sym_PLUS] = ACTIONS(2618),
    [anon_sym_SLASH] = ACTIONS(2618),
    [anon_sym_PERCENT] = ACTIONS(2618),
    [anon_sym_EQ_EQ] = ACTIONS(2618),
    [anon_sym_BANG_EQ] = ACTIONS(2624),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2624),
    [anon_sym_LT_EQ_GT] = ACTIONS(2624),
    [anon_sym_EQ_TILDE] = ACTIONS(2624),
    [anon_sym_BANG_TILDE] = ACTIONS(2624),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2624),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2624),
    [sym__binary_star] = ACTIONS(2624),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2624),
    [sym__element_reference_bracket] = ACTIONS(2624),
  },
  [1931] = {
    [sym_heredoc_body] = STATE(1931),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2620),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2618),
    [anon_sym_unless] = ACTIONS(2618),
    [anon_sym_while] = ACTIONS(2618),
    [anon_sym_until] = ACTIONS(2618),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2622),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_then] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1932] = {
    [sym_heredoc_body] = STATE(1932),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1933] = {
    [sym_heredoc_body] = STATE(1933),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2742),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1934] = {
    [sym_heredoc_body] = STATE(1934),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1935] = {
    [sym_heredoc_body] = STATE(1935),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2744),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1936] = {
    [sym_heredoc_body] = STATE(1936),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2618),
    [anon_sym_unless] = ACTIONS(2618),
    [anon_sym_while] = ACTIONS(2618),
    [anon_sym_until] = ACTIONS(2618),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_then] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1937] = {
    [sym_heredoc_body] = STATE(1937),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2746),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_then] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1938] = {
    [sym_heredoc_body] = STATE(1938),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2748),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_then] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1939] = {
    [sym_heredoc_body] = STATE(1939),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2750),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym__line_break] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1940] = {
    [sym_heredoc_body] = STATE(1940),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2752),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1941] = {
    [sym_heredoc_body] = STATE(1941),
    [sym_identifier] = ACTIONS(2630),
    [anon_sym_LBRACE] = ACTIONS(2632),
    [anon_sym_def] = ACTIONS(2630),
    [anon_sym_LPAREN] = ACTIONS(2630),
    [anon_sym_RPAREN] = ACTIONS(2632),
    [anon_sym_DOT] = ACTIONS(2630),
    [anon_sym_COLON_COLON] = ACTIONS(2630),
    [anon_sym_rescue] = ACTIONS(2630),
    [anon_sym_EQ] = ACTIONS(2630),
    [anon_sym_COMMA] = ACTIONS(2632),
    [anon_sym_PIPE] = ACTIONS(2630),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2632),
    [anon_sym_STAR] = ACTIONS(2630),
    [anon_sym_nil] = ACTIONS(2630),
    [anon_sym_AMP] = ACTIONS(2630),
    [anon_sym_class] = ACTIONS(2630),
    [anon_sym_LT] = ACTIONS(2630),
    [anon_sym_module] = ACTIONS(2630),
    [anon_sym_return] = ACTIONS(2630),
    [anon_sym_yield] = ACTIONS(2630),
    [anon_sym_break] = ACTIONS(2630),
    [anon_sym_next] = ACTIONS(2630),
    [anon_sym_redo] = ACTIONS(2630),
    [anon_sym_retry] = ACTIONS(2630),
    [anon_sym_if] = ACTIONS(2630),
    [anon_sym_unless] = ACTIONS(2630),
    [anon_sym_while] = ACTIONS(2630),
    [anon_sym_until] = ACTIONS(2630),
    [anon_sym_for] = ACTIONS(2630),
    [anon_sym_in] = ACTIONS(2630),
    [anon_sym_do] = ACTIONS(2630),
    [anon_sym_case] = ACTIONS(2630),
    [anon_sym_EQ_GT] = ACTIONS(2632),
    [anon_sym_LBRACK] = ACTIONS(2632),
    [anon_sym_RBRACK] = ACTIONS(2632),
    [anon_sym_LPAREN2] = ACTIONS(2632),
    [anon_sym_DOT_DOT] = ACTIONS(2630),
    [anon_sym_CARET] = ACTIONS(2630),
    [anon_sym_begin] = ACTIONS(2630),
    [anon_sym_COLON_COLON2] = ACTIONS(2632),
    [anon_sym_AMP_DOT] = ACTIONS(2632),
    [anon_sym_PLUS_EQ] = ACTIONS(2632),
    [anon_sym_DASH_EQ] = ACTIONS(2632),
    [anon_sym_STAR_EQ] = ACTIONS(2632),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2632),
    [anon_sym_SLASH_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2632),
    [anon_sym_AMP_EQ] = ACTIONS(2632),
    [anon_sym_PERCENT_EQ] = ACTIONS(2632),
    [anon_sym_GT_GT_EQ] = ACTIONS(2632),
    [anon_sym_LT_LT_EQ] = ACTIONS(2632),
    [anon_sym_CARET_EQ] = ACTIONS(2632),
    [anon_sym_QMARK] = ACTIONS(2630),
    [anon_sym_and] = ACTIONS(2630),
    [anon_sym_or] = ACTIONS(2630),
    [anon_sym_PIPE_PIPE] = ACTIONS(2630),
    [anon_sym_AMP_AMP] = ACTIONS(2630),
    [anon_sym_LT_LT] = ACTIONS(2630),
    [anon_sym_GT_GT] = ACTIONS(2630),
    [anon_sym_LT_EQ] = ACTIONS(2630),
    [anon_sym_GT] = ACTIONS(2630),
    [anon_sym_GT_EQ] = ACTIONS(2632),
    [anon_sym_PLUS] = ACTIONS(2630),
    [anon_sym_SLASH] = ACTIONS(2630),
    [anon_sym_PERCENT] = ACTIONS(2630),
    [anon_sym_EQ_EQ] = ACTIONS(2630),
    [anon_sym_BANG_EQ] = ACTIONS(2632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2632),
    [anon_sym_LT_EQ_GT] = ACTIONS(2632),
    [anon_sym_EQ_TILDE] = ACTIONS(2632),
    [anon_sym_BANG_TILDE] = ACTIONS(2632),
    [anon_sym_defined_QMARK] = ACTIONS(2632),
    [anon_sym_not] = ACTIONS(2630),
    [anon_sym_BANG] = ACTIONS(2630),
    [anon_sym_TILDE] = ACTIONS(2632),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2630),
    [sym_float] = ACTIONS(2630),
    [sym_super] = ACTIONS(2630),
    [sym_self] = ACTIONS(2630),
    [sym_true] = ACTIONS(2630),
    [sym_false] = ACTIONS(2630),
    [sym_constant] = ACTIONS(2630),
    [aux_sym_constant_suffix_token1] = ACTIONS(2632),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2630),
    [sym_instance_variable] = ACTIONS(2632),
    [sym_class_variable] = ACTIONS(2632),
    [sym_global_variable] = ACTIONS(2632),
    [sym_character] = ACTIONS(2632),
    [anon_sym_DASH_GT] = ACTIONS(2632),
    [sym_simple_symbol] = ACTIONS(2632),
    [sym__string_start] = ACTIONS(2632),
    [sym__symbol_start] = ACTIONS(2632),
    [sym__subshell_start] = ACTIONS(2632),
    [sym__regex_start] = ACTIONS(2632),
    [sym__string_array_start] = ACTIONS(2632),
    [sym__symbol_array_start] = ACTIONS(2632),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2632),
    [sym__block_ampersand] = ACTIONS(2632),
    [sym__splat_star] = ACTIONS(2632),
    [sym__unary_minus] = ACTIONS(2632),
    [sym__unary_minus_num] = ACTIONS(2632),
    [sym__binary_minus] = ACTIONS(2632),
    [sym__binary_star] = ACTIONS(2632),
    [sym_hash_key_symbol] = ACTIONS(2632),
    [sym__identifier_suffix] = ACTIONS(2632),
    [sym__constant_suffix] = ACTIONS(2632),
    [sym__hash_splat_star_star] = ACTIONS(2632),
    [sym__binary_star_star] = ACTIONS(2632),
    [sym__element_reference_bracket] = ACTIONS(2632),
  },
  [1942] = {
    [sym_heredoc_body] = STATE(1942),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2754),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1943] = {
    [sym_heredoc_body] = STATE(1943),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_RPAREN] = ACTIONS(2620),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2618),
    [anon_sym_unless] = ACTIONS(2618),
    [anon_sym_while] = ACTIONS(2618),
    [anon_sym_until] = ACTIONS(2618),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_RBRACK] = ACTIONS(2620),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1944] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5016),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1944),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [ts_builtin_sym_end] = ACTIONS(2650),
    [sym_identifier] = ACTIONS(2646),
    [aux_sym_program_token1] = ACTIONS(2658),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_rescue] = ACTIONS(2658),
    [anon_sym_EQ] = ACTIONS(2660),
    [anon_sym_COMMA] = ACTIONS(2660),
    [anon_sym_SEMI] = ACTIONS(2650),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_do] = ACTIONS(2658),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_and] = ACTIONS(2658),
    [anon_sym_or] = ACTIONS(2658),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym__line_break] = ACTIONS(2650),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1945] = {
    [sym_heredoc_body] = STATE(1945),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(731),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LBRACK2] = ACTIONS(2740),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1946] = {
    [sym_heredoc_body] = STATE(1946),
    [sym_identifier] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2614),
    [anon_sym_RPAREN] = ACTIONS(2616),
    [anon_sym_DOT] = ACTIONS(2614),
    [anon_sym_COLON_COLON] = ACTIONS(2614),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_EQ] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2614),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_in] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_EQ_GT] = ACTIONS(2616),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_RBRACK] = ACTIONS(2616),
    [anon_sym_LPAREN2] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_CARET] = ACTIONS(2614),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_COLON_COLON2] = ACTIONS(2616),
    [anon_sym_AMP_DOT] = ACTIONS(2616),
    [anon_sym_PLUS_EQ] = ACTIONS(2616),
    [anon_sym_DASH_EQ] = ACTIONS(2616),
    [anon_sym_STAR_EQ] = ACTIONS(2616),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2616),
    [anon_sym_SLASH_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2616),
    [anon_sym_AMP_EQ] = ACTIONS(2616),
    [anon_sym_PERCENT_EQ] = ACTIONS(2616),
    [anon_sym_GT_GT_EQ] = ACTIONS(2616),
    [anon_sym_LT_LT_EQ] = ACTIONS(2616),
    [anon_sym_CARET_EQ] = ACTIONS(2616),
    [anon_sym_QMARK] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PIPE_PIPE] = ACTIONS(2614),
    [anon_sym_AMP_AMP] = ACTIONS(2614),
    [anon_sym_LT_LT] = ACTIONS(2614),
    [anon_sym_GT_GT] = ACTIONS(2614),
    [anon_sym_LT_EQ] = ACTIONS(2614),
    [anon_sym_GT] = ACTIONS(2614),
    [anon_sym_GT_EQ] = ACTIONS(2616),
    [anon_sym_PLUS] = ACTIONS(2614),
    [anon_sym_SLASH] = ACTIONS(2614),
    [anon_sym_PERCENT] = ACTIONS(2614),
    [anon_sym_EQ_EQ] = ACTIONS(2614),
    [anon_sym_BANG_EQ] = ACTIONS(2616),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2616),
    [anon_sym_LT_EQ_GT] = ACTIONS(2616),
    [anon_sym_EQ_TILDE] = ACTIONS(2616),
    [anon_sym_BANG_TILDE] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2614),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym__binary_star] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
    [sym__binary_star_star] = ACTIONS(2616),
    [sym__element_reference_bracket] = ACTIONS(2616),
  },
  [1947] = {
    [sym_heredoc_body] = STATE(1947),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_RPAREN] = ACTIONS(2624),
    [anon_sym_DOT] = ACTIONS(2618),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2618),
    [anon_sym_EQ] = ACTIONS(2618),
    [anon_sym_COMMA] = ACTIONS(2624),
    [anon_sym_PIPE] = ACTIONS(2618),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2624),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2618),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2618),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2618),
    [anon_sym_unless] = ACTIONS(2618),
    [anon_sym_while] = ACTIONS(2618),
    [anon_sym_until] = ACTIONS(2618),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2618),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2624),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_RBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2618),
    [anon_sym_CARET] = ACTIONS(2618),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2624),
    [anon_sym_AMP_DOT] = ACTIONS(2624),
    [anon_sym_PLUS_EQ] = ACTIONS(2624),
    [anon_sym_DASH_EQ] = ACTIONS(2624),
    [anon_sym_STAR_EQ] = ACTIONS(2624),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2624),
    [anon_sym_SLASH_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2624),
    [anon_sym_AMP_EQ] = ACTIONS(2624),
    [anon_sym_PERCENT_EQ] = ACTIONS(2624),
    [anon_sym_GT_GT_EQ] = ACTIONS(2624),
    [anon_sym_LT_LT_EQ] = ACTIONS(2624),
    [anon_sym_CARET_EQ] = ACTIONS(2624),
    [anon_sym_QMARK] = ACTIONS(2618),
    [anon_sym_and] = ACTIONS(2618),
    [anon_sym_or] = ACTIONS(2618),
    [anon_sym_PIPE_PIPE] = ACTIONS(2618),
    [anon_sym_AMP_AMP] = ACTIONS(2618),
    [anon_sym_LT_LT] = ACTIONS(2618),
    [anon_sym_GT_GT] = ACTIONS(2618),
    [anon_sym_LT_EQ] = ACTIONS(2618),
    [anon_sym_GT] = ACTIONS(2618),
    [anon_sym_GT_EQ] = ACTIONS(2624),
    [anon_sym_PLUS] = ACTIONS(2618),
    [anon_sym_SLASH] = ACTIONS(2618),
    [anon_sym_PERCENT] = ACTIONS(2618),
    [anon_sym_EQ_EQ] = ACTIONS(2618),
    [anon_sym_BANG_EQ] = ACTIONS(2624),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2624),
    [anon_sym_LT_EQ_GT] = ACTIONS(2624),
    [anon_sym_EQ_TILDE] = ACTIONS(2624),
    [anon_sym_BANG_TILDE] = ACTIONS(2624),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2624),
    [sym__binary_star] = ACTIONS(2624),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2624),
    [sym__element_reference_bracket] = ACTIONS(2624),
  },
  [1948] = {
    [sym_heredoc_body] = STATE(1948),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2622),
    [anon_sym_unless] = ACTIONS(2622),
    [anon_sym_while] = ACTIONS(2622),
    [anon_sym_until] = ACTIONS(2622),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2622),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1949] = {
    [sym_heredoc_body] = STATE(1949),
    [sym_identifier] = ACTIONS(2618),
    [anon_sym_LBRACE] = ACTIONS(2620),
    [anon_sym_def] = ACTIONS(2618),
    [anon_sym_LPAREN] = ACTIONS(2618),
    [anon_sym_RPAREN] = ACTIONS(2620),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2618),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2618),
    [anon_sym_nil] = ACTIONS(2618),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_module] = ACTIONS(2618),
    [anon_sym_return] = ACTIONS(2618),
    [anon_sym_yield] = ACTIONS(2618),
    [anon_sym_break] = ACTIONS(2618),
    [anon_sym_next] = ACTIONS(2618),
    [anon_sym_redo] = ACTIONS(2618),
    [anon_sym_retry] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2618),
    [anon_sym_unless] = ACTIONS(2618),
    [anon_sym_while] = ACTIONS(2618),
    [anon_sym_until] = ACTIONS(2618),
    [anon_sym_for] = ACTIONS(2618),
    [anon_sym_in] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2622),
    [anon_sym_case] = ACTIONS(2618),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LBRACK] = ACTIONS(2624),
    [anon_sym_RBRACK] = ACTIONS(2620),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2618),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2622),
    [anon_sym_and] = ACTIONS(2622),
    [anon_sym_or] = ACTIONS(2622),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2624),
    [anon_sym_not] = ACTIONS(2618),
    [anon_sym_BANG] = ACTIONS(2618),
    [anon_sym_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2618),
    [sym_float] = ACTIONS(2618),
    [sym_super] = ACTIONS(2618),
    [sym_self] = ACTIONS(2618),
    [sym_true] = ACTIONS(2618),
    [sym_false] = ACTIONS(2618),
    [sym_constant] = ACTIONS(2618),
    [aux_sym_constant_suffix_token1] = ACTIONS(2624),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2618),
    [sym_instance_variable] = ACTIONS(2624),
    [sym_class_variable] = ACTIONS(2624),
    [sym_global_variable] = ACTIONS(2624),
    [sym_character] = ACTIONS(2624),
    [anon_sym_DASH_GT] = ACTIONS(2624),
    [sym_simple_symbol] = ACTIONS(2624),
    [sym__string_start] = ACTIONS(2624),
    [sym__symbol_start] = ACTIONS(2624),
    [sym__subshell_start] = ACTIONS(2624),
    [sym__regex_start] = ACTIONS(2624),
    [sym__string_array_start] = ACTIONS(2624),
    [sym__symbol_array_start] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2624),
    [sym__block_ampersand] = ACTIONS(2624),
    [sym__splat_star] = ACTIONS(2624),
    [sym__unary_minus] = ACTIONS(2624),
    [sym__unary_minus_num] = ACTIONS(2624),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym_hash_key_symbol] = ACTIONS(2624),
    [sym__identifier_suffix] = ACTIONS(2624),
    [sym__constant_suffix] = ACTIONS(2624),
    [sym__hash_splat_star_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [1950] = {
    [sym_heredoc_body] = STATE(1950),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_EQ] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2638),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_PLUS_EQ] = ACTIONS(2640),
    [anon_sym_DASH_EQ] = ACTIONS(2640),
    [anon_sym_STAR_EQ] = ACTIONS(2640),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2640),
    [anon_sym_SLASH_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2640),
    [anon_sym_AMP_EQ] = ACTIONS(2640),
    [anon_sym_PERCENT_EQ] = ACTIONS(2640),
    [anon_sym_GT_GT_EQ] = ACTIONS(2640),
    [anon_sym_LT_LT_EQ] = ACTIONS(2640),
    [anon_sym_CARET_EQ] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2638),
    [anon_sym_AMP_AMP] = ACTIONS(2638),
    [anon_sym_LT_LT] = ACTIONS(2638),
    [anon_sym_GT_GT] = ACTIONS(2638),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2638),
    [anon_sym_SLASH] = ACTIONS(2638),
    [anon_sym_PERCENT] = ACTIONS(2638),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1951] = {
    [sym_heredoc_body] = STATE(1951),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1952] = {
    [sym_heredoc_body] = STATE(1952),
    [sym_identifier] = ACTIONS(2634),
    [anon_sym_LBRACE] = ACTIONS(2636),
    [anon_sym_def] = ACTIONS(2634),
    [anon_sym_LPAREN] = ACTIONS(2634),
    [anon_sym_RPAREN] = ACTIONS(2636),
    [anon_sym_DOT] = ACTIONS(2634),
    [anon_sym_COLON_COLON] = ACTIONS(2634),
    [anon_sym_rescue] = ACTIONS(2634),
    [anon_sym_EQ] = ACTIONS(2634),
    [anon_sym_COMMA] = ACTIONS(2636),
    [anon_sym_PIPE] = ACTIONS(2634),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2636),
    [anon_sym_STAR] = ACTIONS(2634),
    [anon_sym_nil] = ACTIONS(2634),
    [anon_sym_AMP] = ACTIONS(2634),
    [anon_sym_class] = ACTIONS(2634),
    [anon_sym_LT] = ACTIONS(2634),
    [anon_sym_module] = ACTIONS(2634),
    [anon_sym_return] = ACTIONS(2634),
    [anon_sym_yield] = ACTIONS(2634),
    [anon_sym_break] = ACTIONS(2634),
    [anon_sym_next] = ACTIONS(2634),
    [anon_sym_redo] = ACTIONS(2634),
    [anon_sym_retry] = ACTIONS(2634),
    [anon_sym_if] = ACTIONS(2634),
    [anon_sym_unless] = ACTIONS(2634),
    [anon_sym_while] = ACTIONS(2634),
    [anon_sym_until] = ACTIONS(2634),
    [anon_sym_for] = ACTIONS(2634),
    [anon_sym_in] = ACTIONS(2634),
    [anon_sym_do] = ACTIONS(2634),
    [anon_sym_case] = ACTIONS(2634),
    [anon_sym_EQ_GT] = ACTIONS(2636),
    [anon_sym_LBRACK] = ACTIONS(2636),
    [anon_sym_RBRACK] = ACTIONS(2636),
    [anon_sym_LPAREN2] = ACTIONS(2636),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_CARET] = ACTIONS(2634),
    [anon_sym_begin] = ACTIONS(2634),
    [anon_sym_COLON_COLON2] = ACTIONS(2636),
    [anon_sym_AMP_DOT] = ACTIONS(2636),
    [anon_sym_PLUS_EQ] = ACTIONS(2636),
    [anon_sym_DASH_EQ] = ACTIONS(2636),
    [anon_sym_STAR_EQ] = ACTIONS(2636),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2636),
    [anon_sym_SLASH_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2636),
    [anon_sym_AMP_EQ] = ACTIONS(2636),
    [anon_sym_PERCENT_EQ] = ACTIONS(2636),
    [anon_sym_GT_GT_EQ] = ACTIONS(2636),
    [anon_sym_LT_LT_EQ] = ACTIONS(2636),
    [anon_sym_CARET_EQ] = ACTIONS(2636),
    [anon_sym_QMARK] = ACTIONS(2634),
    [anon_sym_and] = ACTIONS(2634),
    [anon_sym_or] = ACTIONS(2634),
    [anon_sym_PIPE_PIPE] = ACTIONS(2634),
    [anon_sym_AMP_AMP] = ACTIONS(2634),
    [anon_sym_LT_LT] = ACTIONS(2634),
    [anon_sym_GT_GT] = ACTIONS(2634),
    [anon_sym_LT_EQ] = ACTIONS(2634),
    [anon_sym_GT] = ACTIONS(2634),
    [anon_sym_GT_EQ] = ACTIONS(2636),
    [anon_sym_PLUS] = ACTIONS(2634),
    [anon_sym_SLASH] = ACTIONS(2634),
    [anon_sym_PERCENT] = ACTIONS(2634),
    [anon_sym_EQ_EQ] = ACTIONS(2634),
    [anon_sym_BANG_EQ] = ACTIONS(2636),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2636),
    [anon_sym_LT_EQ_GT] = ACTIONS(2636),
    [anon_sym_EQ_TILDE] = ACTIONS(2636),
    [anon_sym_BANG_TILDE] = ACTIONS(2636),
    [anon_sym_defined_QMARK] = ACTIONS(2636),
    [anon_sym_not] = ACTIONS(2634),
    [anon_sym_BANG] = ACTIONS(2634),
    [anon_sym_TILDE] = ACTIONS(2636),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2634),
    [sym_float] = ACTIONS(2634),
    [sym_super] = ACTIONS(2634),
    [sym_self] = ACTIONS(2634),
    [sym_true] = ACTIONS(2634),
    [sym_false] = ACTIONS(2634),
    [sym_constant] = ACTIONS(2634),
    [aux_sym_constant_suffix_token1] = ACTIONS(2636),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2634),
    [sym_instance_variable] = ACTIONS(2636),
    [sym_class_variable] = ACTIONS(2636),
    [sym_global_variable] = ACTIONS(2636),
    [sym_character] = ACTIONS(2636),
    [anon_sym_DASH_GT] = ACTIONS(2636),
    [sym_simple_symbol] = ACTIONS(2636),
    [sym__string_start] = ACTIONS(2636),
    [sym__symbol_start] = ACTIONS(2636),
    [sym__subshell_start] = ACTIONS(2636),
    [sym__regex_start] = ACTIONS(2636),
    [sym__string_array_start] = ACTIONS(2636),
    [sym__symbol_array_start] = ACTIONS(2636),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2636),
    [sym__block_ampersand] = ACTIONS(2636),
    [sym__splat_star] = ACTIONS(2636),
    [sym__unary_minus] = ACTIONS(2636),
    [sym__unary_minus_num] = ACTIONS(2636),
    [sym__binary_minus] = ACTIONS(2636),
    [sym__binary_star] = ACTIONS(2636),
    [sym_hash_key_symbol] = ACTIONS(2636),
    [sym__identifier_suffix] = ACTIONS(2636),
    [sym__constant_suffix] = ACTIONS(2636),
    [sym__hash_splat_star_star] = ACTIONS(2636),
    [sym__binary_star_star] = ACTIONS(2636),
    [sym__element_reference_bracket] = ACTIONS(2636),
  },
  [1953] = {
    [sym_heredoc_body] = STATE(1953),
    [sym_identifier] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2642),
    [anon_sym_RPAREN] = ACTIONS(2644),
    [anon_sym_DOT] = ACTIONS(2642),
    [anon_sym_COLON_COLON] = ACTIONS(2642),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_EQ] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2642),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_in] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_EQ_GT] = ACTIONS(2644),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_RBRACK] = ACTIONS(2644),
    [anon_sym_LPAREN2] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_CARET] = ACTIONS(2642),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_COLON_COLON2] = ACTIONS(2644),
    [anon_sym_AMP_DOT] = ACTIONS(2644),
    [anon_sym_PLUS_EQ] = ACTIONS(2644),
    [anon_sym_DASH_EQ] = ACTIONS(2644),
    [anon_sym_STAR_EQ] = ACTIONS(2644),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2644),
    [anon_sym_SLASH_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2644),
    [anon_sym_AMP_EQ] = ACTIONS(2644),
    [anon_sym_PERCENT_EQ] = ACTIONS(2644),
    [anon_sym_GT_GT_EQ] = ACTIONS(2644),
    [anon_sym_LT_LT_EQ] = ACTIONS(2644),
    [anon_sym_CARET_EQ] = ACTIONS(2644),
    [anon_sym_QMARK] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PIPE_PIPE] = ACTIONS(2642),
    [anon_sym_AMP_AMP] = ACTIONS(2642),
    [anon_sym_LT_LT] = ACTIONS(2642),
    [anon_sym_GT_GT] = ACTIONS(2642),
    [anon_sym_LT_EQ] = ACTIONS(2642),
    [anon_sym_GT] = ACTIONS(2642),
    [anon_sym_GT_EQ] = ACTIONS(2644),
    [anon_sym_PLUS] = ACTIONS(2642),
    [anon_sym_SLASH] = ACTIONS(2642),
    [anon_sym_PERCENT] = ACTIONS(2642),
    [anon_sym_EQ_EQ] = ACTIONS(2642),
    [anon_sym_BANG_EQ] = ACTIONS(2644),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2644),
    [anon_sym_LT_EQ_GT] = ACTIONS(2644),
    [anon_sym_EQ_TILDE] = ACTIONS(2644),
    [anon_sym_BANG_TILDE] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2642),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym__binary_star] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
    [sym__binary_star_star] = ACTIONS(2644),
    [sym__element_reference_bracket] = ACTIONS(2644),
  },
  [1954] = {
    [sym_heredoc_body] = STATE(1954),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_rescue] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1955] = {
    [sym_heredoc_body] = STATE(1955),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_EQ] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2626),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_PLUS_EQ] = ACTIONS(2628),
    [anon_sym_DASH_EQ] = ACTIONS(2628),
    [anon_sym_STAR_EQ] = ACTIONS(2628),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2628),
    [anon_sym_SLASH_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2628),
    [anon_sym_AMP_EQ] = ACTIONS(2628),
    [anon_sym_PERCENT_EQ] = ACTIONS(2628),
    [anon_sym_GT_GT_EQ] = ACTIONS(2628),
    [anon_sym_LT_LT_EQ] = ACTIONS(2628),
    [anon_sym_CARET_EQ] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2626),
    [anon_sym_AMP_AMP] = ACTIONS(2626),
    [anon_sym_LT_LT] = ACTIONS(2626),
    [anon_sym_GT_GT] = ACTIONS(2626),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2626),
    [anon_sym_SLASH] = ACTIONS(2626),
    [anon_sym_PERCENT] = ACTIONS(2626),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1956] = {
    [sym_heredoc_body] = STATE(1956),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1957] = {
    [sym_heredoc_body] = STATE(1957),
    [sym_identifier] = ACTIONS(731),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_def] = ACTIONS(731),
    [anon_sym_LPAREN] = ACTIONS(731),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(731),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_STAR] = ACTIONS(731),
    [anon_sym_nil] = ACTIONS(731),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_class] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_module] = ACTIONS(731),
    [anon_sym_return] = ACTIONS(731),
    [anon_sym_yield] = ACTIONS(731),
    [anon_sym_break] = ACTIONS(731),
    [anon_sym_next] = ACTIONS(731),
    [anon_sym_redo] = ACTIONS(731),
    [anon_sym_retry] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(731),
    [anon_sym_unless] = ACTIONS(731),
    [anon_sym_while] = ACTIONS(731),
    [anon_sym_until] = ACTIONS(731),
    [anon_sym_for] = ACTIONS(731),
    [anon_sym_in] = ACTIONS(731),
    [anon_sym_do] = ACTIONS(731),
    [anon_sym_case] = ACTIONS(731),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LBRACK] = ACTIONS(743),
    [anon_sym_RBRACK] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_begin] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(731),
    [anon_sym_and] = ACTIONS(731),
    [anon_sym_or] = ACTIONS(731),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [anon_sym_defined_QMARK] = ACTIONS(743),
    [anon_sym_not] = ACTIONS(731),
    [anon_sym_BANG] = ACTIONS(731),
    [anon_sym_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(731),
    [sym_float] = ACTIONS(731),
    [sym_super] = ACTIONS(731),
    [sym_self] = ACTIONS(731),
    [sym_true] = ACTIONS(731),
    [sym_false] = ACTIONS(731),
    [sym_constant] = ACTIONS(731),
    [aux_sym_constant_suffix_token1] = ACTIONS(743),
    [aux_sym_identifier_suffix_token1] = ACTIONS(731),
    [sym_instance_variable] = ACTIONS(743),
    [sym_class_variable] = ACTIONS(743),
    [sym_global_variable] = ACTIONS(743),
    [sym_character] = ACTIONS(743),
    [anon_sym_DASH_GT] = ACTIONS(743),
    [sym_simple_symbol] = ACTIONS(743),
    [sym__string_start] = ACTIONS(743),
    [sym__symbol_start] = ACTIONS(743),
    [sym__subshell_start] = ACTIONS(743),
    [sym__regex_start] = ACTIONS(743),
    [sym__string_array_start] = ACTIONS(743),
    [sym__symbol_array_start] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(743),
    [sym__block_ampersand] = ACTIONS(743),
    [sym__splat_star] = ACTIONS(743),
    [sym__unary_minus] = ACTIONS(743),
    [sym__unary_minus_num] = ACTIONS(743),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym_hash_key_symbol] = ACTIONS(743),
    [sym__identifier_suffix] = ACTIONS(743),
    [sym__constant_suffix] = ACTIONS(743),
    [sym__hash_splat_star_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [1958] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5016),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1958),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2756),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_RPAREN] = ACTIONS(2650),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_rescue] = ACTIONS(2658),
    [anon_sym_EQ] = ACTIONS(2660),
    [anon_sym_COMMA] = ACTIONS(2660),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_do] = ACTIONS(2658),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_RBRACK] = ACTIONS(2650),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_and] = ACTIONS(2658),
    [anon_sym_or] = ACTIONS(2658),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1959] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym_destructured_left_assignment] = STATE(5382),
    [sym_rest_assignment] = STATE(5382),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5022),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1959),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2758),
    [anon_sym_RPAREN] = ACTIONS(2760),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_EQ] = ACTIONS(2760),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_in] = ACTIONS(2762),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1960] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym_destructured_left_assignment] = STATE(5382),
    [sym_rest_assignment] = STATE(5382),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5022),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1960),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2758),
    [anon_sym_RPAREN] = ACTIONS(2764),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_EQ] = ACTIONS(2764),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_in] = ACTIONS(2766),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1961] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym_left_assignment_list] = STATE(5598),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5032),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1961),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2758),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1962] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym_left_assignment_list] = STATE(5605),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5030),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1962),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2758),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1963] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym_left_assignment_list] = STATE(5588),
    [sym__mlhs] = STATE(5630),
    [sym_destructured_left_assignment] = STATE(5306),
    [sym_rest_assignment] = STATE(5306),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(4958),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1963),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2758),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1964] = {
    [sym_heredoc_body] = STATE(1964),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_RBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_end] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2612),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_when] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_elsif] = ACTIONS(2638),
    [anon_sym_else] = ACTIONS(2638),
    [anon_sym_then] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_ensure] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1965] = {
    [sym_heredoc_body] = STATE(1965),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_RBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_end] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2610),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_when] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_elsif] = ACTIONS(2626),
    [anon_sym_else] = ACTIONS(2626),
    [anon_sym_then] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_ensure] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1966] = {
    [sym_heredoc_body] = STATE(1966),
    [sym_identifier] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_RBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2642),
    [anon_sym_RPAREN] = ACTIONS(2644),
    [anon_sym_DOT] = ACTIONS(2642),
    [anon_sym_COLON_COLON] = ACTIONS(2642),
    [anon_sym_end] = ACTIONS(2642),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_SEMI] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2644),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_COLON] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_in] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_when] = ACTIONS(2642),
    [anon_sym_EQ_GT] = ACTIONS(2644),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_LPAREN2] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_CARET] = ACTIONS(2644),
    [anon_sym_elsif] = ACTIONS(2642),
    [anon_sym_else] = ACTIONS(2642),
    [anon_sym_then] = ACTIONS(2642),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_ensure] = ACTIONS(2642),
    [anon_sym_COLON_COLON2] = ACTIONS(2644),
    [anon_sym_AMP_DOT] = ACTIONS(2644),
    [anon_sym_QMARK] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PIPE_PIPE] = ACTIONS(2644),
    [anon_sym_AMP_AMP] = ACTIONS(2644),
    [anon_sym_LT_LT] = ACTIONS(2644),
    [anon_sym_GT_GT] = ACTIONS(2644),
    [anon_sym_LT_EQ] = ACTIONS(2642),
    [anon_sym_GT] = ACTIONS(2642),
    [anon_sym_GT_EQ] = ACTIONS(2644),
    [anon_sym_PLUS] = ACTIONS(2644),
    [anon_sym_SLASH] = ACTIONS(2644),
    [anon_sym_PERCENT] = ACTIONS(2644),
    [anon_sym_EQ_EQ] = ACTIONS(2642),
    [anon_sym_BANG_EQ] = ACTIONS(2644),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2644),
    [anon_sym_LT_EQ_GT] = ACTIONS(2644),
    [anon_sym_EQ_TILDE] = ACTIONS(2644),
    [anon_sym_BANG_TILDE] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2642),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym__line_break] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym__binary_star] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
    [sym__binary_star_star] = ACTIONS(2644),
    [sym__element_reference_bracket] = ACTIONS(2644),
  },
  [1967] = {
    [sym_heredoc_body] = STATE(1967),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_RBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_end] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2610),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_when] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_elsif] = ACTIONS(2638),
    [anon_sym_else] = ACTIONS(2638),
    [anon_sym_then] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_ensure] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1968] = {
    [sym_heredoc_body] = STATE(1968),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_RBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_end] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2612),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_when] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_elsif] = ACTIONS(2626),
    [anon_sym_else] = ACTIONS(2626),
    [anon_sym_then] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_ensure] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1969] = {
    [sym_heredoc_body] = STATE(1969),
    [sym_identifier] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_RBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2614),
    [anon_sym_RPAREN] = ACTIONS(2616),
    [anon_sym_DOT] = ACTIONS(2614),
    [anon_sym_COLON_COLON] = ACTIONS(2614),
    [anon_sym_end] = ACTIONS(2614),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_SEMI] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2616),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_COLON] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_in] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_when] = ACTIONS(2614),
    [anon_sym_EQ_GT] = ACTIONS(2616),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_LPAREN2] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_CARET] = ACTIONS(2616),
    [anon_sym_elsif] = ACTIONS(2614),
    [anon_sym_else] = ACTIONS(2614),
    [anon_sym_then] = ACTIONS(2614),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_ensure] = ACTIONS(2614),
    [anon_sym_COLON_COLON2] = ACTIONS(2616),
    [anon_sym_AMP_DOT] = ACTIONS(2616),
    [anon_sym_QMARK] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PIPE_PIPE] = ACTIONS(2616),
    [anon_sym_AMP_AMP] = ACTIONS(2616),
    [anon_sym_LT_LT] = ACTIONS(2616),
    [anon_sym_GT_GT] = ACTIONS(2616),
    [anon_sym_LT_EQ] = ACTIONS(2614),
    [anon_sym_GT] = ACTIONS(2614),
    [anon_sym_GT_EQ] = ACTIONS(2616),
    [anon_sym_PLUS] = ACTIONS(2616),
    [anon_sym_SLASH] = ACTIONS(2616),
    [anon_sym_PERCENT] = ACTIONS(2616),
    [anon_sym_EQ_EQ] = ACTIONS(2614),
    [anon_sym_BANG_EQ] = ACTIONS(2616),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2616),
    [anon_sym_LT_EQ_GT] = ACTIONS(2616),
    [anon_sym_EQ_TILDE] = ACTIONS(2616),
    [anon_sym_BANG_TILDE] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2614),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym__line_break] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym__binary_star] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
    [sym__binary_star_star] = ACTIONS(2616),
    [sym__element_reference_bracket] = ACTIONS(2616),
  },
  [1970] = {
    [sym_heredoc_body] = STATE(1970),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_RBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_end] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_when] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_elsif] = ACTIONS(2626),
    [anon_sym_else] = ACTIONS(2626),
    [anon_sym_then] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_ensure] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1971] = {
    [sym_heredoc_body] = STATE(1971),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_RBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_end] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_when] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_elsif] = ACTIONS(2638),
    [anon_sym_else] = ACTIONS(2638),
    [anon_sym_then] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_ensure] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1972] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5016),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1972),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_RPAREN] = ACTIONS(2660),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_EQ] = ACTIONS(2660),
    [anon_sym_COMMA] = ACTIONS(2660),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_in] = ACTIONS(2768),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1973] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5471),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(4942),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym_destructured_left_assignment] = STATE(5382),
    [sym_rest_assignment] = STATE(5382),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5022),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1973),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2758),
    [anon_sym_COLON_COLON] = ACTIONS(2656),
    [anon_sym_STAR] = ACTIONS(29),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2704),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1974] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5447),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(128),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5420),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1974),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2770),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2772),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__singleton_class_left_angle_left_langle] = ACTIONS(2774),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1975] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5447),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(126),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5420),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1975),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2770),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2776),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__singleton_class_left_angle_left_langle] = ACTIONS(2778),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1976] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5447),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(130),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5420),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1976),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2770),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2780),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__singleton_class_left_angle_left_langle] = ACTIONS(2782),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1977] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5447),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(134),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5420),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1977),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2770),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2784),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1978] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5447),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(133),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5420),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1978),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2770),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2786),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1979] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5330),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(5211),
    [sym_scope_resolution] = STATE(5211),
    [sym__call] = STATE(4708),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5118),
    [sym__variable] = STATE(4710),
    [sym__nonlocal_variable] = STATE(4851),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(5211),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1979),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2788),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2790),
    [anon_sym_nil] = ACTIONS(2792),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2788),
    [sym_self] = ACTIONS(2788),
    [sym_true] = ACTIONS(2794),
    [sym_false] = ACTIONS(2794),
    [sym_constant] = ACTIONS(2788),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2796),
    [sym_class_variable] = ACTIONS(2796),
    [sym_global_variable] = ACTIONS(2796),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1980] = {
    [sym_method] = STATE(5420),
    [sym_singleton_method] = STATE(5420),
    [sym_class] = STATE(5420),
    [sym_singleton_class] = STATE(5420),
    [sym_module] = STATE(5420),
    [sym_return] = STATE(5420),
    [sym_yield] = STATE(5420),
    [sym_break] = STATE(5420),
    [sym_next] = STATE(5420),
    [sym_redo] = STATE(5420),
    [sym_retry] = STATE(5420),
    [sym_while] = STATE(5420),
    [sym_until] = STATE(5420),
    [sym_for] = STATE(5420),
    [sym_case] = STATE(5420),
    [sym_case_match] = STATE(5420),
    [sym_if] = STATE(5420),
    [sym_unless] = STATE(5420),
    [sym_begin] = STATE(5420),
    [sym__primary] = STATE(5447),
    [sym_parenthesized_statements] = STATE(5420),
    [sym_element_reference] = STATE(4942),
    [sym_scope_resolution] = STATE(132),
    [sym__call] = STATE(4651),
    [sym_call] = STATE(5420),
    [sym_parenthesized_unary] = STATE(5420),
    [sym_unary_literal] = STATE(5365),
    [sym__literal] = STATE(5420),
    [sym__numeric] = STATE(5419),
    [sym__simple_numeric] = STATE(5365),
    [sym__function_identifier] = STATE(4836),
    [sym__function_identifier_call] = STATE(5376),
    [sym__lhs] = STATE(5420),
    [sym__variable] = STATE(4668),
    [sym__nonlocal_variable] = STATE(4773),
    [sym__int_or_float] = STATE(5571),
    [sym_complex] = STATE(5479),
    [sym_rational] = STATE(5479),
    [sym_nil] = STATE(4942),
    [sym_constant_suffix] = STATE(4814),
    [sym_identifier_suffix] = STATE(4815),
    [sym_chained_string] = STATE(5420),
    [sym_string] = STATE(5082),
    [sym_subshell] = STATE(5420),
    [sym_string_array] = STATE(5420),
    [sym_symbol_array] = STATE(5420),
    [sym_delimited_symbol] = STATE(5419),
    [sym_regex] = STATE(5420),
    [sym_heredoc_body] = STATE(1980),
    [sym_array] = STATE(5420),
    [sym_hash] = STATE(5420),
    [sym_lambda] = STATE(5420),
    [sym_identifier] = ACTIONS(2704),
    [anon_sym_LBRACE] = ACTIONS(2648),
    [anon_sym_def] = ACTIONS(2652),
    [anon_sym_LPAREN] = ACTIONS(2654),
    [anon_sym_COLON_COLON] = ACTIONS(2770),
    [anon_sym_nil] = ACTIONS(2662),
    [anon_sym_class] = ACTIONS(2664),
    [anon_sym_module] = ACTIONS(2666),
    [anon_sym_return] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(2670),
    [anon_sym_break] = ACTIONS(2672),
    [anon_sym_next] = ACTIONS(2674),
    [anon_sym_redo] = ACTIONS(2676),
    [anon_sym_retry] = ACTIONS(2678),
    [anon_sym_if] = ACTIONS(2680),
    [anon_sym_unless] = ACTIONS(2682),
    [anon_sym_while] = ACTIONS(2684),
    [anon_sym_until] = ACTIONS(2686),
    [anon_sym_for] = ACTIONS(2688),
    [anon_sym_case] = ACTIONS(2690),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_begin] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2696),
    [anon_sym_defined_QMARK] = ACTIONS(2698),
    [anon_sym_not] = ACTIONS(2700),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2702),
    [sym_float] = ACTIONS(2702),
    [sym_super] = ACTIONS(2704),
    [sym_self] = ACTIONS(2704),
    [sym_true] = ACTIONS(2706),
    [sym_false] = ACTIONS(2706),
    [sym_constant] = ACTIONS(2798),
    [aux_sym_constant_suffix_token1] = ACTIONS(2708),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2710),
    [sym_instance_variable] = ACTIONS(2712),
    [sym_class_variable] = ACTIONS(2712),
    [sym_global_variable] = ACTIONS(2712),
    [sym_character] = ACTIONS(2714),
    [anon_sym_DASH_GT] = ACTIONS(2716),
    [sym_simple_symbol] = ACTIONS(2718),
    [sym__string_start] = ACTIONS(2720),
    [sym__symbol_start] = ACTIONS(2722),
    [sym__subshell_start] = ACTIONS(2724),
    [sym__regex_start] = ACTIONS(2726),
    [sym__string_array_start] = ACTIONS(2728),
    [sym__symbol_array_start] = ACTIONS(2730),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2714),
    [sym__unary_minus_num] = ACTIONS(2696),
    [sym__identifier_suffix] = ACTIONS(2732),
    [sym__constant_suffix] = ACTIONS(2708),
  },
  [1981] = {
    [sym_heredoc_body] = STATE(1981),
    [ts_builtin_sym_end] = ACTIONS(2628),
    [sym_identifier] = ACTIONS(2626),
    [aux_sym_program_token1] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2734),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1982] = {
    [sym_heredoc_body] = STATE(1982),
    [ts_builtin_sym_end] = ACTIONS(2616),
    [sym_identifier] = ACTIONS(2614),
    [aux_sym_program_token1] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2614),
    [anon_sym_DOT] = ACTIONS(2614),
    [anon_sym_COLON_COLON] = ACTIONS(2614),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_SEMI] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2616),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_COLON] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_in] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_EQ_GT] = ACTIONS(2616),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_LPAREN2] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_CARET] = ACTIONS(2616),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_COLON_COLON2] = ACTIONS(2616),
    [anon_sym_AMP_DOT] = ACTIONS(2616),
    [anon_sym_QMARK] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PIPE_PIPE] = ACTIONS(2616),
    [anon_sym_AMP_AMP] = ACTIONS(2616),
    [anon_sym_LT_LT] = ACTIONS(2616),
    [anon_sym_GT_GT] = ACTIONS(2616),
    [anon_sym_LT_EQ] = ACTIONS(2614),
    [anon_sym_GT] = ACTIONS(2614),
    [anon_sym_GT_EQ] = ACTIONS(2616),
    [anon_sym_PLUS] = ACTIONS(2616),
    [anon_sym_SLASH] = ACTIONS(2616),
    [anon_sym_PERCENT] = ACTIONS(2616),
    [anon_sym_EQ_EQ] = ACTIONS(2614),
    [anon_sym_BANG_EQ] = ACTIONS(2616),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2616),
    [anon_sym_LT_EQ_GT] = ACTIONS(2616),
    [anon_sym_EQ_TILDE] = ACTIONS(2616),
    [anon_sym_BANG_TILDE] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2614),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym__line_break] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym__binary_star] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
    [sym__binary_star_star] = ACTIONS(2616),
    [sym__element_reference_bracket] = ACTIONS(2616),
  },
  [1983] = {
    [sym_heredoc_body] = STATE(1983),
    [ts_builtin_sym_end] = ACTIONS(2628),
    [sym_identifier] = ACTIONS(2626),
    [aux_sym_program_token1] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2736),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1984] = {
    [sym_heredoc_body] = STATE(1984),
    [ts_builtin_sym_end] = ACTIONS(2640),
    [sym_identifier] = ACTIONS(2638),
    [aux_sym_program_token1] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2736),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1985] = {
    [sym_heredoc_body] = STATE(1985),
    [ts_builtin_sym_end] = ACTIONS(2644),
    [sym_identifier] = ACTIONS(2642),
    [aux_sym_program_token1] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2642),
    [anon_sym_DOT] = ACTIONS(2642),
    [anon_sym_COLON_COLON] = ACTIONS(2642),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_SEMI] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2644),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_COLON] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_in] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_EQ_GT] = ACTIONS(2644),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_LPAREN2] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_CARET] = ACTIONS(2644),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_COLON_COLON2] = ACTIONS(2644),
    [anon_sym_AMP_DOT] = ACTIONS(2644),
    [anon_sym_QMARK] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PIPE_PIPE] = ACTIONS(2644),
    [anon_sym_AMP_AMP] = ACTIONS(2644),
    [anon_sym_LT_LT] = ACTIONS(2644),
    [anon_sym_GT_GT] = ACTIONS(2644),
    [anon_sym_LT_EQ] = ACTIONS(2642),
    [anon_sym_GT] = ACTIONS(2642),
    [anon_sym_GT_EQ] = ACTIONS(2644),
    [anon_sym_PLUS] = ACTIONS(2644),
    [anon_sym_SLASH] = ACTIONS(2644),
    [anon_sym_PERCENT] = ACTIONS(2644),
    [anon_sym_EQ_EQ] = ACTIONS(2642),
    [anon_sym_BANG_EQ] = ACTIONS(2644),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2644),
    [anon_sym_LT_EQ_GT] = ACTIONS(2644),
    [anon_sym_EQ_TILDE] = ACTIONS(2644),
    [anon_sym_BANG_TILDE] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2642),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym__line_break] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym__binary_star] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
    [sym__binary_star_star] = ACTIONS(2644),
    [sym__element_reference_bracket] = ACTIONS(2644),
  },
  [1986] = {
    [sym_heredoc_body] = STATE(1986),
    [ts_builtin_sym_end] = ACTIONS(2640),
    [sym_identifier] = ACTIONS(2638),
    [aux_sym_program_token1] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2734),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1987] = {
    [sym_heredoc_body] = STATE(1987),
    [ts_builtin_sym_end] = ACTIONS(2628),
    [sym_identifier] = ACTIONS(2626),
    [aux_sym_program_token1] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1988] = {
    [sym_heredoc_body] = STATE(1988),
    [ts_builtin_sym_end] = ACTIONS(2640),
    [sym_identifier] = ACTIONS(2638),
    [aux_sym_program_token1] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1989] = {
    [sym_heredoc_body] = STATE(1989),
    [sym_identifier] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2614),
    [anon_sym_RPAREN] = ACTIONS(2616),
    [anon_sym_DOT] = ACTIONS(2614),
    [anon_sym_COLON_COLON] = ACTIONS(2614),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2616),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_COLON] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_in] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_EQ_GT] = ACTIONS(2616),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_RBRACK] = ACTIONS(2616),
    [anon_sym_LPAREN2] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_CARET] = ACTIONS(2616),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_COLON_COLON2] = ACTIONS(2616),
    [anon_sym_AMP_DOT] = ACTIONS(2616),
    [anon_sym_QMARK] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PIPE_PIPE] = ACTIONS(2616),
    [anon_sym_AMP_AMP] = ACTIONS(2616),
    [anon_sym_LT_LT] = ACTIONS(2616),
    [anon_sym_GT_GT] = ACTIONS(2616),
    [anon_sym_LT_EQ] = ACTIONS(2614),
    [anon_sym_GT] = ACTIONS(2614),
    [anon_sym_GT_EQ] = ACTIONS(2616),
    [anon_sym_PLUS] = ACTIONS(2616),
    [anon_sym_SLASH] = ACTIONS(2616),
    [anon_sym_PERCENT] = ACTIONS(2616),
    [anon_sym_EQ_EQ] = ACTIONS(2614),
    [anon_sym_BANG_EQ] = ACTIONS(2616),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2616),
    [anon_sym_LT_EQ_GT] = ACTIONS(2616),
    [anon_sym_EQ_TILDE] = ACTIONS(2616),
    [anon_sym_BANG_TILDE] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2614),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym__binary_star] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
    [sym__binary_star_star] = ACTIONS(2616),
    [sym__element_reference_bracket] = ACTIONS(2616),
  },
  [1990] = {
    [sym_heredoc_body] = STATE(1990),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2748),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_then] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1991] = {
    [sym_heredoc_body] = STATE(1991),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2742),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1992] = {
    [sym_heredoc_body] = STATE(1992),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_RBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1993] = {
    [sym_heredoc_body] = STATE(1993),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2748),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_then] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1994] = {
    [sym_heredoc_body] = STATE(1994),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2742),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1995] = {
    [sym_heredoc_body] = STATE(1995),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_RBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1996] = {
    [sym_heredoc_body] = STATE(1996),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2746),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_then] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1997] = {
    [sym_heredoc_body] = STATE(1997),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2750),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [1998] = {
    [sym_heredoc_body] = STATE(1998),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2746),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_then] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [1999] = {
    [sym_heredoc_body] = STATE(1999),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2750),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [2000] = {
    [sym_heredoc_body] = STATE(2000),
    [sym_identifier] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2642),
    [anon_sym_RPAREN] = ACTIONS(2644),
    [anon_sym_DOT] = ACTIONS(2642),
    [anon_sym_COLON_COLON] = ACTIONS(2642),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2644),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_COLON] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_in] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_EQ_GT] = ACTIONS(2644),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_RBRACK] = ACTIONS(2644),
    [anon_sym_LPAREN2] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_CARET] = ACTIONS(2644),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_COLON_COLON2] = ACTIONS(2644),
    [anon_sym_AMP_DOT] = ACTIONS(2644),
    [anon_sym_QMARK] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PIPE_PIPE] = ACTIONS(2644),
    [anon_sym_AMP_AMP] = ACTIONS(2644),
    [anon_sym_LT_LT] = ACTIONS(2644),
    [anon_sym_GT_GT] = ACTIONS(2644),
    [anon_sym_LT_EQ] = ACTIONS(2642),
    [anon_sym_GT] = ACTIONS(2642),
    [anon_sym_GT_EQ] = ACTIONS(2644),
    [anon_sym_PLUS] = ACTIONS(2644),
    [anon_sym_SLASH] = ACTIONS(2644),
    [anon_sym_PERCENT] = ACTIONS(2644),
    [anon_sym_EQ_EQ] = ACTIONS(2642),
    [anon_sym_BANG_EQ] = ACTIONS(2644),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2644),
    [anon_sym_LT_EQ_GT] = ACTIONS(2644),
    [anon_sym_EQ_TILDE] = ACTIONS(2644),
    [anon_sym_BANG_TILDE] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2642),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym__binary_star] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
    [sym__binary_star_star] = ACTIONS(2644),
    [sym__element_reference_bracket] = ACTIONS(2644),
  },
  [2001] = {
    [sym_heredoc_body] = STATE(2001),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2744),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [2002] = {
    [sym_heredoc_body] = STATE(2002),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2744),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [2003] = {
    [sym_heredoc_body] = STATE(2003),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [2004] = {
    [sym_heredoc_body] = STATE(2004),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [2005] = {
    [sym_heredoc_body] = STATE(2005),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2738),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [2006] = {
    [sym_heredoc_body] = STATE(2006),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [2007] = {
    [sym_heredoc_body] = STATE(2007),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2752),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [2008] = {
    [sym_heredoc_body] = STATE(2008),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_COLON_COLON] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_COLON] = ACTIONS(2754),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2628),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PIPE_PIPE] = ACTIONS(2628),
    [anon_sym_AMP_AMP] = ACTIONS(2628),
    [anon_sym_LT_LT] = ACTIONS(2628),
    [anon_sym_GT_GT] = ACTIONS(2628),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_SLASH] = ACTIONS(2628),
    [anon_sym_PERCENT] = ACTIONS(2628),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [2009] = {
    [sym_heredoc_body] = STATE(2009),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2754),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [2010] = {
    [sym_heredoc_body] = STATE(2010),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2638),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_COLON_COLON] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_COLON] = ACTIONS(2752),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2640),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PIPE_PIPE] = ACTIONS(2640),
    [anon_sym_AMP_AMP] = ACTIONS(2640),
    [anon_sym_LT_LT] = ACTIONS(2640),
    [anon_sym_GT_GT] = ACTIONS(2640),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_SLASH] = ACTIONS(2640),
    [anon_sym_PERCENT] = ACTIONS(2640),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2638),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [2011] = {
    [sym_splat_parameter] = STATE(4527),
    [sym__array_pattern_n] = STATE(4493),
    [sym__pattern_expr] = STATE(4532),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2011),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_RBRACE] = ACTIONS(2804),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_RPAREN] = ACTIONS(2804),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_end] = ACTIONS(2810),
    [anon_sym_rescue] = ACTIONS(2810),
    [anon_sym_COMMA] = ACTIONS(2804),
    [anon_sym_SEMI] = ACTIONS(2804),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2812),
    [anon_sym_STAR] = ACTIONS(2814),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2810),
    [anon_sym_unless] = ACTIONS(2810),
    [anon_sym_while] = ACTIONS(2810),
    [anon_sym_until] = ACTIONS(2810),
    [anon_sym_in] = ACTIONS(2810),
    [anon_sym_do] = ACTIONS(2810),
    [anon_sym_when] = ACTIONS(2810),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2820),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_elsif] = ACTIONS(2810),
    [anon_sym_else] = ACTIONS(2810),
    [anon_sym_then] = ACTIONS(2810),
    [anon_sym_ensure] = ACTIONS(2810),
    [anon_sym_and] = ACTIONS(2810),
    [anon_sym_or] = ACTIONS(2810),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2804),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2012] = {
    [sym__pattern_expr] = STATE(4478),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2012),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_RBRACE] = ACTIONS(2846),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_RPAREN] = ACTIONS(2846),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_end] = ACTIONS(2848),
    [anon_sym_rescue] = ACTIONS(2848),
    [anon_sym_COMMA] = ACTIONS(2846),
    [anon_sym_SEMI] = ACTIONS(2846),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2812),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2848),
    [anon_sym_unless] = ACTIONS(2848),
    [anon_sym_while] = ACTIONS(2848),
    [anon_sym_until] = ACTIONS(2848),
    [anon_sym_in] = ACTIONS(2848),
    [anon_sym_do] = ACTIONS(2848),
    [anon_sym_when] = ACTIONS(2848),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2820),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_elsif] = ACTIONS(2848),
    [anon_sym_else] = ACTIONS(2848),
    [anon_sym_then] = ACTIONS(2848),
    [anon_sym_ensure] = ACTIONS(2848),
    [anon_sym_and] = ACTIONS(2848),
    [anon_sym_or] = ACTIONS(2848),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2846),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2013] = {
    [sym__pattern_expr] = STATE(4477),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2013),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_RBRACE] = ACTIONS(2374),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_RPAREN] = ACTIONS(2374),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_end] = ACTIONS(2850),
    [anon_sym_rescue] = ACTIONS(2850),
    [anon_sym_COMMA] = ACTIONS(2374),
    [anon_sym_SEMI] = ACTIONS(2374),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2812),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2850),
    [anon_sym_unless] = ACTIONS(2850),
    [anon_sym_while] = ACTIONS(2850),
    [anon_sym_until] = ACTIONS(2850),
    [anon_sym_in] = ACTIONS(2850),
    [anon_sym_do] = ACTIONS(2850),
    [anon_sym_when] = ACTIONS(2850),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2820),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_elsif] = ACTIONS(2850),
    [anon_sym_else] = ACTIONS(2850),
    [anon_sym_then] = ACTIONS(2850),
    [anon_sym_ensure] = ACTIONS(2850),
    [anon_sym_and] = ACTIONS(2850),
    [anon_sym_or] = ACTIONS(2850),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2374),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2014] = {
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5788),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5789),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(5793),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5892),
    [sym_identifier_suffix] = STATE(5892),
    [sym_string] = STATE(4741),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2014),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2852),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2858),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2876),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2015] = {
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5736),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5775),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(5779),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5892),
    [sym_identifier_suffix] = STATE(5892),
    [sym_string] = STATE(4741),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2015),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2852),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2858),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2876),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2016] = {
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5688),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5694),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(5703),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5892),
    [sym_identifier_suffix] = STATE(5892),
    [sym_string] = STATE(4741),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2016),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2852),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2900),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2876),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2017] = {
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5660),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5661),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(5662),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5892),
    [sym_identifier_suffix] = STATE(5892),
    [sym_string] = STATE(4741),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2017),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2852),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2900),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2876),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2018] = {
    [sym_splat_parameter] = STATE(4936),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__pattern_top_expr_body] = STATE(4973),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4898),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4929),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5839),
    [sym_identifier_suffix] = STATE(5839),
    [sym_string] = STATE(4728),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2018),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2902),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2904),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2908),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2910),
    [sym_float] = ACTIONS(2910),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2912),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2908),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2019] = {
    [sym_splat_parameter] = STATE(4768),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym__pattern_top_expr_body] = STATE(4828),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4812),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4806),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_constant_suffix] = STATE(5773),
    [sym_identifier_suffix] = STATE(5773),
    [sym_string] = STATE(4644),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2019),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2914),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_STAR] = ACTIONS(2918),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2926),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2020] = {
    [sym_splat_parameter] = STATE(4936),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__pattern_top_expr_body] = STATE(4985),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4898),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4929),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5839),
    [sym_identifier_suffix] = STATE(5839),
    [sym_string] = STATE(4728),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2020),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2902),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2904),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2908),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2910),
    [sym_float] = ACTIONS(2910),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2912),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2908),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2021] = {
    [sym_splat_parameter] = STATE(4523),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym__pattern_top_expr_body] = STATE(4506),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4530),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4531),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_constant_suffix] = STATE(5841),
    [sym_identifier_suffix] = STATE(5841),
    [sym_string] = STATE(4434),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2021),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2928),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2812),
    [anon_sym_STAR] = ACTIONS(2918),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2820),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2930),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2022] = {
    [sym_splat_parameter] = STATE(4523),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4530),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4531),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_constant_suffix] = STATE(5841),
    [sym_identifier_suffix] = STATE(5841),
    [sym_string] = STATE(4434),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2022),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2928),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2812),
    [anon_sym_STAR] = ACTIONS(2918),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2820),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2930),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2023] = {
    [sym_splat_parameter] = STATE(4768),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym__pattern_top_expr_body] = STATE(4506),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4812),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4806),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_constant_suffix] = STATE(5773),
    [sym_identifier_suffix] = STATE(5773),
    [sym_string] = STATE(4644),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2023),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2914),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_STAR] = ACTIONS(2918),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2926),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2024] = {
    [sym_splat_parameter] = STATE(4768),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4812),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4806),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_constant_suffix] = STATE(5773),
    [sym_identifier_suffix] = STATE(5773),
    [sym_string] = STATE(4644),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2024),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2914),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_STAR] = ACTIONS(2918),
    [anon_sym_STAR_STAR] = ACTIONS(1645),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2926),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2025] = {
    [sym_splat_parameter] = STATE(4634),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym__pattern_top_expr_body] = STATE(4490),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4642),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4636),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_constant_suffix] = STATE(5862),
    [sym_identifier_suffix] = STATE(5862),
    [sym_string] = STATE(4584),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2025),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2932),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2934),
    [anon_sym_STAR] = ACTIONS(2936),
    [anon_sym_STAR_STAR] = ACTIONS(1705),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2940),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2942),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2026] = {
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__pattern_top_expr_body] = STATE(4973),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4975),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5892),
    [sym_identifier_suffix] = STATE(5892),
    [sym_string] = STATE(4741),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2026),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2852),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2876),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2027] = {
    [sym_splat_parameter] = STATE(4634),
    [sym_hash_splat_parameter] = STATE(4483),
    [sym_hash_splat_nil] = STATE(4483),
    [sym__pattern_top_expr_body] = STATE(4506),
    [sym__array_pattern_n] = STATE(4516),
    [sym__pattern_expr] = STATE(4642),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym__find_pattern_body] = STATE(4485),
    [sym_find_pattern] = STATE(4446),
    [sym__hash_pattern_body] = STATE(4488),
    [sym_keyword_pattern] = STATE(4636),
    [sym__hash_pattern_any_rest] = STATE(4489),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_constant_suffix] = STATE(5862),
    [sym_identifier_suffix] = STATE(5862),
    [sym_string] = STATE(4584),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2027),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2932),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2934),
    [anon_sym_STAR] = ACTIONS(2936),
    [anon_sym_STAR_STAR] = ACTIONS(1705),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2940),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2942),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2028] = {
    [sym_splat_parameter] = STATE(4972),
    [sym_hash_splat_parameter] = STATE(4899),
    [sym_hash_splat_nil] = STATE(4899),
    [sym__pattern_top_expr_body] = STATE(4985),
    [sym__array_pattern_n] = STATE(4974),
    [sym__pattern_expr] = STATE(4975),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(4976),
    [sym_find_pattern] = STATE(4819),
    [sym__hash_pattern_body] = STATE(4977),
    [sym_keyword_pattern] = STATE(4930),
    [sym__hash_pattern_any_rest] = STATE(4927),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_constant_suffix] = STATE(5892),
    [sym_identifier_suffix] = STATE(5892),
    [sym_string] = STATE(4741),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2028),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2852),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2864),
    [anon_sym_STAR_STAR] = ACTIONS(1675),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2876),
    [aux_sym_constant_suffix_token1] = ACTIONS(2878),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2880),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
    [sym__identifier_suffix] = ACTIONS(2880),
    [sym__constant_suffix] = ACTIONS(2878),
  },
  [2029] = {
    [sym_splat_parameter] = STATE(4640),
    [sym__array_pattern_n] = STATE(4493),
    [sym__pattern_expr] = STATE(4628),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2029),
    [sym_lambda] = STATE(4411),
    [ts_builtin_sym_end] = ACTIONS(2804),
    [sym_identifier] = ACTIONS(2800),
    [aux_sym_program_token1] = ACTIONS(2810),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_rescue] = ACTIONS(2810),
    [anon_sym_COMMA] = ACTIONS(2804),
    [anon_sym_SEMI] = ACTIONS(2804),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2934),
    [anon_sym_STAR] = ACTIONS(2944),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2810),
    [anon_sym_unless] = ACTIONS(2810),
    [anon_sym_while] = ACTIONS(2810),
    [anon_sym_until] = ACTIONS(2810),
    [anon_sym_do] = ACTIONS(2810),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_and] = ACTIONS(2810),
    [anon_sym_or] = ACTIONS(2810),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2804),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2942),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2030] = {
    [sym_heredoc_body] = STATE(2030),
    [sym_identifier] = ACTIONS(2634),
    [anon_sym_LBRACE] = ACTIONS(2636),
    [anon_sym_RBRACE] = ACTIONS(2636),
    [anon_sym_def] = ACTIONS(2634),
    [anon_sym_LPAREN] = ACTIONS(2636),
    [anon_sym_RPAREN] = ACTIONS(2636),
    [anon_sym_COLON_COLON] = ACTIONS(2636),
    [anon_sym_end] = ACTIONS(2634),
    [anon_sym_rescue] = ACTIONS(2634),
    [anon_sym_COMMA] = ACTIONS(2636),
    [anon_sym_SEMI] = ACTIONS(2636),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2636),
    [anon_sym_STAR] = ACTIONS(2636),
    [anon_sym_nil] = ACTIONS(2634),
    [anon_sym_class] = ACTIONS(2634),
    [anon_sym_module] = ACTIONS(2634),
    [anon_sym_return] = ACTIONS(2634),
    [anon_sym_yield] = ACTIONS(2634),
    [anon_sym_break] = ACTIONS(2634),
    [anon_sym_next] = ACTIONS(2634),
    [anon_sym_redo] = ACTIONS(2634),
    [anon_sym_retry] = ACTIONS(2634),
    [anon_sym_if] = ACTIONS(2634),
    [anon_sym_unless] = ACTIONS(2634),
    [anon_sym_while] = ACTIONS(2634),
    [anon_sym_until] = ACTIONS(2634),
    [anon_sym_for] = ACTIONS(2634),
    [anon_sym_in] = ACTIONS(2634),
    [anon_sym_do] = ACTIONS(2634),
    [anon_sym_case] = ACTIONS(2634),
    [anon_sym_when] = ACTIONS(2634),
    [anon_sym_LBRACK] = ACTIONS(2636),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_elsif] = ACTIONS(2634),
    [anon_sym_else] = ACTIONS(2634),
    [anon_sym_then] = ACTIONS(2634),
    [anon_sym_begin] = ACTIONS(2634),
    [anon_sym_ensure] = ACTIONS(2634),
    [anon_sym_and] = ACTIONS(2634),
    [anon_sym_or] = ACTIONS(2634),
    [anon_sym_PLUS] = ACTIONS(2636),
    [anon_sym_defined_QMARK] = ACTIONS(2636),
    [anon_sym_not] = ACTIONS(2634),
    [anon_sym_BANG] = ACTIONS(2636),
    [anon_sym_TILDE] = ACTIONS(2636),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2634),
    [sym_float] = ACTIONS(2634),
    [sym_super] = ACTIONS(2634),
    [sym_self] = ACTIONS(2634),
    [sym_true] = ACTIONS(2634),
    [sym_false] = ACTIONS(2634),
    [sym_constant] = ACTIONS(2634),
    [aux_sym_constant_suffix_token1] = ACTIONS(2636),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2634),
    [sym_instance_variable] = ACTIONS(2636),
    [sym_class_variable] = ACTIONS(2636),
    [sym_global_variable] = ACTIONS(2636),
    [sym_character] = ACTIONS(2636),
    [anon_sym_DASH_GT] = ACTIONS(2636),
    [sym__line_break] = ACTIONS(2636),
    [sym_simple_symbol] = ACTIONS(2636),
    [sym__string_start] = ACTIONS(2636),
    [sym__symbol_start] = ACTIONS(2636),
    [sym__subshell_start] = ACTIONS(2636),
    [sym__regex_start] = ACTIONS(2636),
    [sym__string_array_start] = ACTIONS(2636),
    [sym__symbol_array_start] = ACTIONS(2636),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2636),
    [sym__block_ampersand] = ACTIONS(2636),
    [sym__splat_star] = ACTIONS(2636),
    [sym__unary_minus] = ACTIONS(2636),
    [sym__unary_minus_num] = ACTIONS(2636),
    [sym__binary_minus] = ACTIONS(2636),
    [sym_hash_key_symbol] = ACTIONS(2636),
    [sym__identifier_suffix] = ACTIONS(2636),
    [sym__constant_suffix] = ACTIONS(2636),
    [sym__hash_splat_star_star] = ACTIONS(2636),
  },
  [2031] = {
    [sym_heredoc_body] = STATE(2031),
    [sym_identifier] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_RBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2616),
    [anon_sym_RPAREN] = ACTIONS(2616),
    [anon_sym_COLON_COLON] = ACTIONS(2616),
    [anon_sym_end] = ACTIONS(2614),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_SEMI] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2616),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_in] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_when] = ACTIONS(2614),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_elsif] = ACTIONS(2614),
    [anon_sym_else] = ACTIONS(2614),
    [anon_sym_then] = ACTIONS(2614),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_ensure] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PLUS] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2616),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym__line_break] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
  },
  [2032] = {
    [sym_heredoc_body] = STATE(2032),
    [sym_identifier] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_RBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2644),
    [anon_sym_RPAREN] = ACTIONS(2644),
    [anon_sym_COLON_COLON] = ACTIONS(2644),
    [anon_sym_end] = ACTIONS(2642),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_SEMI] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2644),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_in] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_when] = ACTIONS(2642),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_elsif] = ACTIONS(2642),
    [anon_sym_else] = ACTIONS(2642),
    [anon_sym_then] = ACTIONS(2642),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_ensure] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PLUS] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2644),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym__line_break] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
  },
  [2033] = {
    [sym_heredoc_body] = STATE(2033),
    [sym_identifier] = ACTIONS(2946),
    [anon_sym_LBRACE] = ACTIONS(2948),
    [anon_sym_RBRACE] = ACTIONS(2948),
    [anon_sym_def] = ACTIONS(2946),
    [anon_sym_LPAREN] = ACTIONS(2948),
    [anon_sym_RPAREN] = ACTIONS(2948),
    [anon_sym_COLON_COLON] = ACTIONS(2948),
    [anon_sym_end] = ACTIONS(2946),
    [anon_sym_rescue] = ACTIONS(2946),
    [anon_sym_COMMA] = ACTIONS(2948),
    [anon_sym_SEMI] = ACTIONS(2948),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2948),
    [anon_sym_STAR] = ACTIONS(2948),
    [anon_sym_nil] = ACTIONS(2946),
    [anon_sym_class] = ACTIONS(2946),
    [anon_sym_module] = ACTIONS(2946),
    [anon_sym_return] = ACTIONS(2946),
    [anon_sym_yield] = ACTIONS(2946),
    [anon_sym_break] = ACTIONS(2946),
    [anon_sym_next] = ACTIONS(2946),
    [anon_sym_redo] = ACTIONS(2946),
    [anon_sym_retry] = ACTIONS(2946),
    [anon_sym_if] = ACTIONS(2946),
    [anon_sym_unless] = ACTIONS(2946),
    [anon_sym_while] = ACTIONS(2946),
    [anon_sym_until] = ACTIONS(2946),
    [anon_sym_for] = ACTIONS(2946),
    [anon_sym_in] = ACTIONS(2946),
    [anon_sym_do] = ACTIONS(2946),
    [anon_sym_case] = ACTIONS(2946),
    [anon_sym_when] = ACTIONS(2946),
    [anon_sym_LBRACK] = ACTIONS(2948),
    [anon_sym_DOT_DOT] = ACTIONS(2946),
    [anon_sym_elsif] = ACTIONS(2946),
    [anon_sym_else] = ACTIONS(2946),
    [anon_sym_then] = ACTIONS(2946),
    [anon_sym_begin] = ACTIONS(2946),
    [anon_sym_ensure] = ACTIONS(2946),
    [anon_sym_and] = ACTIONS(2946),
    [anon_sym_or] = ACTIONS(2946),
    [anon_sym_PLUS] = ACTIONS(2948),
    [anon_sym_defined_QMARK] = ACTIONS(2948),
    [anon_sym_not] = ACTIONS(2946),
    [anon_sym_BANG] = ACTIONS(2948),
    [anon_sym_TILDE] = ACTIONS(2948),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2946),
    [sym_float] = ACTIONS(2946),
    [sym_super] = ACTIONS(2946),
    [sym_self] = ACTIONS(2946),
    [sym_true] = ACTIONS(2946),
    [sym_false] = ACTIONS(2946),
    [sym_constant] = ACTIONS(2946),
    [aux_sym_constant_suffix_token1] = ACTIONS(2948),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2946),
    [sym_instance_variable] = ACTIONS(2948),
    [sym_class_variable] = ACTIONS(2948),
    [sym_global_variable] = ACTIONS(2948),
    [sym_character] = ACTIONS(2948),
    [anon_sym_DASH_GT] = ACTIONS(2948),
    [sym__line_break] = ACTIONS(2948),
    [sym_simple_symbol] = ACTIONS(2948),
    [sym__string_start] = ACTIONS(2948),
    [sym__symbol_start] = ACTIONS(2948),
    [sym__subshell_start] = ACTIONS(2948),
    [sym__regex_start] = ACTIONS(2948),
    [sym__string_array_start] = ACTIONS(2948),
    [sym__symbol_array_start] = ACTIONS(2948),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2948),
    [sym__block_ampersand] = ACTIONS(2948),
    [sym__splat_star] = ACTIONS(2948),
    [sym__unary_minus] = ACTIONS(2948),
    [sym__unary_minus_num] = ACTIONS(2948),
    [sym__binary_minus] = ACTIONS(2948),
    [sym_hash_key_symbol] = ACTIONS(2948),
    [sym__identifier_suffix] = ACTIONS(2948),
    [sym__constant_suffix] = ACTIONS(2948),
    [sym__hash_splat_star_star] = ACTIONS(2948),
  },
  [2034] = {
    [sym_heredoc_body] = STATE(2034),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_RBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2640),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_COLON_COLON] = ACTIONS(2640),
    [anon_sym_end] = ACTIONS(2638),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_in] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_when] = ACTIONS(2638),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_elsif] = ACTIONS(2638),
    [anon_sym_else] = ACTIONS(2638),
    [anon_sym_then] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_ensure] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2640),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
  },
  [2035] = {
    [sym_heredoc_body] = STATE(2035),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_RBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2628),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_COLON_COLON] = ACTIONS(2628),
    [anon_sym_end] = ACTIONS(2626),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_in] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_when] = ACTIONS(2626),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_elsif] = ACTIONS(2626),
    [anon_sym_else] = ACTIONS(2626),
    [anon_sym_then] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_ensure] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2628),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
  },
  [2036] = {
    [sym__pattern_expr] = STATE(4477),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2036),
    [sym_lambda] = STATE(4411),
    [ts_builtin_sym_end] = ACTIONS(2374),
    [sym_identifier] = ACTIONS(2800),
    [aux_sym_program_token1] = ACTIONS(2850),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_rescue] = ACTIONS(2850),
    [anon_sym_COMMA] = ACTIONS(2374),
    [anon_sym_SEMI] = ACTIONS(2374),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2934),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2850),
    [anon_sym_unless] = ACTIONS(2850),
    [anon_sym_while] = ACTIONS(2850),
    [anon_sym_until] = ACTIONS(2850),
    [anon_sym_do] = ACTIONS(2850),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_and] = ACTIONS(2850),
    [anon_sym_or] = ACTIONS(2850),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2374),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2942),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2037] = {
    [sym__pattern_expr] = STATE(4478),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2037),
    [sym_lambda] = STATE(4411),
    [ts_builtin_sym_end] = ACTIONS(2846),
    [sym_identifier] = ACTIONS(2800),
    [aux_sym_program_token1] = ACTIONS(2848),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_rescue] = ACTIONS(2848),
    [anon_sym_COMMA] = ACTIONS(2846),
    [anon_sym_SEMI] = ACTIONS(2846),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2934),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2848),
    [anon_sym_unless] = ACTIONS(2848),
    [anon_sym_while] = ACTIONS(2848),
    [anon_sym_until] = ACTIONS(2848),
    [anon_sym_do] = ACTIONS(2848),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_and] = ACTIONS(2848),
    [anon_sym_or] = ACTIONS(2848),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2846),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2942),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2038] = {
    [sym_splat_parameter] = STATE(4782),
    [sym__array_pattern_n] = STATE(4493),
    [sym__pattern_expr] = STATE(4793),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2038),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_COMMA] = ACTIONS(2804),
    [anon_sym_SEMI] = ACTIONS(2804),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_STAR] = ACTIONS(2814),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2810),
    [anon_sym_unless] = ACTIONS(2810),
    [anon_sym_do] = ACTIONS(2810),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_then] = ACTIONS(2810),
    [anon_sym_and] = ACTIONS(2810),
    [anon_sym_or] = ACTIONS(2810),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2804),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
  },
  [2039] = {
    [sym_heredoc_body] = STATE(2039),
    [sym_identifier] = ACTIONS(2622),
    [anon_sym_BEGIN] = ACTIONS(2622),
    [anon_sym_LBRACE] = ACTIONS(2620),
    [anon_sym_END] = ACTIONS(2622),
    [anon_sym_def] = ACTIONS(2622),
    [anon_sym_LPAREN] = ACTIONS(2622),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_COLON_COLON] = ACTIONS(2622),
    [anon_sym_end] = ACTIONS(2622),
    [anon_sym_rescue] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_STAR] = ACTIONS(2620),
    [anon_sym_nil] = ACTIONS(2622),
    [anon_sym_class] = ACTIONS(2622),
    [anon_sym_LT] = ACTIONS(2620),
    [anon_sym_module] = ACTIONS(2622),
    [anon_sym_return] = ACTIONS(2622),
    [anon_sym_yield] = ACTIONS(2622),
    [anon_sym_break] = ACTIONS(2622),
    [anon_sym_next] = ACTIONS(2622),
    [anon_sym_redo] = ACTIONS(2622),
    [anon_sym_retry] = ACTIONS(2622),
    [anon_sym_if] = ACTIONS(2622),
    [anon_sym_unless] = ACTIONS(2622),
    [anon_sym_while] = ACTIONS(2622),
    [anon_sym_until] = ACTIONS(2622),
    [anon_sym_for] = ACTIONS(2622),
    [anon_sym_do] = ACTIONS(2618),
    [anon_sym_case] = ACTIONS(2622),
    [anon_sym_LBRACK] = ACTIONS(2620),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_else] = ACTIONS(2622),
    [anon_sym_begin] = ACTIONS(2622),
    [anon_sym_ensure] = ACTIONS(2622),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2620),
    [anon_sym_defined_QMARK] = ACTIONS(2620),
    [anon_sym_not] = ACTIONS(2622),
    [anon_sym_BANG] = ACTIONS(2620),
    [anon_sym_TILDE] = ACTIONS(2620),
    [anon_sym_undef] = ACTIONS(2622),
    [anon_sym_alias] = ACTIONS(2622),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2622),
    [sym_float] = ACTIONS(2622),
    [sym_super] = ACTIONS(2622),
    [sym_self] = ACTIONS(2622),
    [sym_true] = ACTIONS(2622),
    [sym_false] = ACTIONS(2622),
    [sym_constant] = ACTIONS(2622),
    [aux_sym_constant_suffix_token1] = ACTIONS(2620),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2622),
    [sym_instance_variable] = ACTIONS(2620),
    [sym_class_variable] = ACTIONS(2620),
    [sym_global_variable] = ACTIONS(2620),
    [sym_character] = ACTIONS(2620),
    [anon_sym_DASH_GT] = ACTIONS(2620),
    [sym__line_break] = ACTIONS(2620),
    [sym_simple_symbol] = ACTIONS(2620),
    [sym__string_start] = ACTIONS(2620),
    [sym__symbol_start] = ACTIONS(2620),
    [sym__subshell_start] = ACTIONS(2620),
    [sym__regex_start] = ACTIONS(2620),
    [sym__string_array_start] = ACTIONS(2620),
    [sym__symbol_array_start] = ACTIONS(2620),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2620),
    [sym__unary_minus] = ACTIONS(2620),
    [sym__unary_minus_num] = ACTIONS(2620),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__identifier_suffix] = ACTIONS(2620),
    [sym__constant_suffix] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [2040] = {
    [sym_splat_parameter] = STATE(4886),
    [sym__array_pattern_n] = STATE(5009),
    [sym__pattern_expr] = STATE(4889),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2040),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2804),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_rescue] = ACTIONS(2810),
    [anon_sym_COMMA] = ACTIONS(2804),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2904),
    [anon_sym_STAR] = ACTIONS(2952),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_do] = ACTIONS(2810),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2804),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_and] = ACTIONS(2810),
    [anon_sym_or] = ACTIONS(2810),
    [anon_sym_PLUS] = ACTIONS(2908),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2910),
    [sym_float] = ACTIONS(2910),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2908),
  },
  [2041] = {
    [sym_heredoc_body] = STATE(2041),
    [sym_identifier] = ACTIONS(2046),
    [anon_sym_BEGIN] = ACTIONS(2046),
    [anon_sym_LBRACE] = ACTIONS(2956),
    [anon_sym_RBRACE] = ACTIONS(2956),
    [anon_sym_END] = ACTIONS(2046),
    [anon_sym_def] = ACTIONS(2046),
    [anon_sym_LPAREN] = ACTIONS(2956),
    [anon_sym_RPAREN] = ACTIONS(2956),
    [anon_sym_COLON_COLON] = ACTIONS(2956),
    [anon_sym_end] = ACTIONS(2046),
    [anon_sym_rescue] = ACTIONS(2046),
    [anon_sym_PIPE] = ACTIONS(2956),
    [anon_sym_SEMI] = ACTIONS(2956),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2956),
    [anon_sym_STAR] = ACTIONS(2956),
    [anon_sym_nil] = ACTIONS(2046),
    [anon_sym_class] = ACTIONS(2046),
    [anon_sym_module] = ACTIONS(2046),
    [anon_sym_return] = ACTIONS(2046),
    [anon_sym_yield] = ACTIONS(2046),
    [anon_sym_break] = ACTIONS(2046),
    [anon_sym_next] = ACTIONS(2046),
    [anon_sym_redo] = ACTIONS(2046),
    [anon_sym_retry] = ACTIONS(2046),
    [anon_sym_if] = ACTIONS(2046),
    [anon_sym_unless] = ACTIONS(2046),
    [anon_sym_while] = ACTIONS(2046),
    [anon_sym_until] = ACTIONS(2046),
    [anon_sym_for] = ACTIONS(2046),
    [anon_sym_in] = ACTIONS(2046),
    [anon_sym_case] = ACTIONS(2046),
    [anon_sym_when] = ACTIONS(2046),
    [anon_sym_LBRACK] = ACTIONS(2956),
    [anon_sym_DOT_DOT] = ACTIONS(2046),
    [anon_sym_elsif] = ACTIONS(2046),
    [anon_sym_else] = ACTIONS(2046),
    [anon_sym_then] = ACTIONS(2046),
    [anon_sym_begin] = ACTIONS(2046),
    [anon_sym_ensure] = ACTIONS(2046),
    [anon_sym_PLUS] = ACTIONS(2956),
    [anon_sym_defined_QMARK] = ACTIONS(2956),
    [anon_sym_not] = ACTIONS(2046),
    [anon_sym_BANG] = ACTIONS(2956),
    [anon_sym_TILDE] = ACTIONS(2956),
    [anon_sym_undef] = ACTIONS(2046),
    [anon_sym_alias] = ACTIONS(2046),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2046),
    [sym_float] = ACTIONS(2046),
    [sym_super] = ACTIONS(2046),
    [sym_self] = ACTIONS(2046),
    [sym_true] = ACTIONS(2046),
    [sym_false] = ACTIONS(2046),
    [sym_constant] = ACTIONS(2046),
    [aux_sym_constant_suffix_token1] = ACTIONS(2956),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2046),
    [sym_instance_variable] = ACTIONS(2956),
    [sym_class_variable] = ACTIONS(2956),
    [sym_global_variable] = ACTIONS(2956),
    [sym_character] = ACTIONS(2956),
    [anon_sym_DASH_GT] = ACTIONS(2956),
    [sym_simple_symbol] = ACTIONS(2956),
    [sym__string_start] = ACTIONS(2956),
    [sym__symbol_start] = ACTIONS(2956),
    [sym__subshell_start] = ACTIONS(2956),
    [sym__regex_start] = ACTIONS(2956),
    [sym__string_array_start] = ACTIONS(2956),
    [sym__symbol_array_start] = ACTIONS(2956),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2956),
    [sym__unary_minus] = ACTIONS(2956),
    [sym__unary_minus_num] = ACTIONS(2956),
    [sym__binary_minus] = ACTIONS(2956),
    [sym__identifier_suffix] = ACTIONS(2956),
    [sym__constant_suffix] = ACTIONS(2956),
  },
  [2042] = {
    [sym_heredoc_body] = STATE(2042),
    [sym_identifier] = ACTIONS(2958),
    [anon_sym_BEGIN] = ACTIONS(2958),
    [anon_sym_LBRACE] = ACTIONS(2960),
    [anon_sym_END] = ACTIONS(2958),
    [anon_sym_def] = ACTIONS(2958),
    [anon_sym_LPAREN] = ACTIONS(2960),
    [anon_sym_DOT] = ACTIONS(2958),
    [anon_sym_COLON_COLON] = ACTIONS(2958),
    [anon_sym_end] = ACTIONS(2958),
    [anon_sym_rescue] = ACTIONS(2958),
    [anon_sym_SEMI] = ACTIONS(2960),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2960),
    [anon_sym_STAR] = ACTIONS(2960),
    [anon_sym_nil] = ACTIONS(2958),
    [anon_sym_class] = ACTIONS(2958),
    [anon_sym_LT] = ACTIONS(2960),
    [anon_sym_module] = ACTIONS(2958),
    [anon_sym_return] = ACTIONS(2958),
    [anon_sym_yield] = ACTIONS(2958),
    [anon_sym_break] = ACTIONS(2958),
    [anon_sym_next] = ACTIONS(2958),
    [anon_sym_redo] = ACTIONS(2958),
    [anon_sym_retry] = ACTIONS(2958),
    [anon_sym_if] = ACTIONS(2958),
    [anon_sym_unless] = ACTIONS(2958),
    [anon_sym_while] = ACTIONS(2958),
    [anon_sym_until] = ACTIONS(2958),
    [anon_sym_for] = ACTIONS(2958),
    [anon_sym_case] = ACTIONS(2958),
    [anon_sym_LBRACK] = ACTIONS(2960),
    [anon_sym_DOT_DOT] = ACTIONS(2958),
    [anon_sym_else] = ACTIONS(2958),
    [anon_sym_begin] = ACTIONS(2958),
    [anon_sym_ensure] = ACTIONS(2958),
    [anon_sym_COLON_COLON2] = ACTIONS(2960),
    [anon_sym_AMP_DOT] = ACTIONS(2960),
    [anon_sym_PLUS] = ACTIONS(2960),
    [anon_sym_defined_QMARK] = ACTIONS(2960),
    [anon_sym_not] = ACTIONS(2958),
    [anon_sym_BANG] = ACTIONS(2960),
    [anon_sym_TILDE] = ACTIONS(2960),
    [anon_sym_undef] = ACTIONS(2958),
    [anon_sym_alias] = ACTIONS(2958),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2958),
    [sym_float] = ACTIONS(2958),
    [sym_super] = ACTIONS(2958),
    [sym_self] = ACTIONS(2958),
    [sym_true] = ACTIONS(2958),
    [sym_false] = ACTIONS(2958),
    [sym_constant] = ACTIONS(2958),
    [aux_sym_constant_suffix_token1] = ACTIONS(2960),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2958),
    [sym_instance_variable] = ACTIONS(2960),
    [sym_class_variable] = ACTIONS(2960),
    [sym_global_variable] = ACTIONS(2960),
    [sym_character] = ACTIONS(2960),
    [anon_sym_DASH_GT] = ACTIONS(2960),
    [sym__line_break] = ACTIONS(2960),
    [sym_simple_symbol] = ACTIONS(2960),
    [sym__string_start] = ACTIONS(2960),
    [sym__symbol_start] = ACTIONS(2960),
    [sym__subshell_start] = ACTIONS(2960),
    [sym__regex_start] = ACTIONS(2960),
    [sym__string_array_start] = ACTIONS(2960),
    [sym__symbol_array_start] = ACTIONS(2960),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2960),
    [sym__unary_minus] = ACTIONS(2960),
    [sym__unary_minus_num] = ACTIONS(2960),
    [sym__binary_minus] = ACTIONS(2960),
    [sym__identifier_suffix] = ACTIONS(2960),
    [sym__constant_suffix] = ACTIONS(2960),
    [sym__element_reference_bracket] = ACTIONS(2960),
  },
  [2043] = {
    [sym_argument_list] = STATE(2261),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_heredoc_body] = STATE(2043),
    [ts_builtin_sym_end] = ACTIONS(177),
    [aux_sym_program_token1] = ACTIONS(177),
    [anon_sym_LBRACE] = ACTIONS(2962),
    [anon_sym_RBRACE] = ACTIONS(177),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_end] = ACTIONS(177),
    [anon_sym_rescue] = ACTIONS(177),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_if] = ACTIONS(177),
    [anon_sym_unless] = ACTIONS(177),
    [anon_sym_while] = ACTIONS(177),
    [anon_sym_until] = ACTIONS(177),
    [anon_sym_in] = ACTIONS(177),
    [anon_sym_do] = ACTIONS(177),
    [anon_sym_when] = ACTIONS(177),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_elsif] = ACTIONS(177),
    [anon_sym_else] = ACTIONS(177),
    [anon_sym_then] = ACTIONS(177),
    [anon_sym_ensure] = ACTIONS(177),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(177),
    [anon_sym_and] = ACTIONS(177),
    [anon_sym_or] = ACTIONS(177),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(177),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [2044] = {
    [sym_argument_list] = STATE(2257),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_heredoc_body] = STATE(2044),
    [ts_builtin_sym_end] = ACTIONS(121),
    [aux_sym_program_token1] = ACTIONS(121),
    [anon_sym_LBRACE] = ACTIONS(2962),
    [anon_sym_RBRACE] = ACTIONS(121),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_end] = ACTIONS(121),
    [anon_sym_rescue] = ACTIONS(121),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_if] = ACTIONS(121),
    [anon_sym_unless] = ACTIONS(121),
    [anon_sym_while] = ACTIONS(121),
    [anon_sym_until] = ACTIONS(121),
    [anon_sym_in] = ACTIONS(121),
    [anon_sym_do] = ACTIONS(2964),
    [anon_sym_when] = ACTIONS(121),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_elsif] = ACTIONS(121),
    [anon_sym_else] = ACTIONS(121),
    [anon_sym_then] = ACTIONS(121),
    [anon_sym_ensure] = ACTIONS(121),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(121),
    [anon_sym_and] = ACTIONS(121),
    [anon_sym_or] = ACTIONS(121),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(121),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [2045] = {
    [sym_splat_parameter] = STATE(5007),
    [sym__array_pattern_n] = STATE(5009),
    [sym__pattern_expr] = STATE(4964),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2045),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2804),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_COMMA] = ACTIONS(2804),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2952),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_do] = ACTIONS(2810),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2804),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_and] = ACTIONS(2810),
    [anon_sym_or] = ACTIONS(2810),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2046] = {
    [sym_argument_list] = STATE(2255),
    [sym_do_block] = STATE(2306),
    [sym_block] = STATE(2308),
    [sym_heredoc_body] = STATE(2046),
    [ts_builtin_sym_end] = ACTIONS(177),
    [aux_sym_program_token1] = ACTIONS(177),
    [anon_sym_LBRACE] = ACTIONS(2962),
    [anon_sym_RBRACE] = ACTIONS(177),
    [anon_sym_RPAREN] = ACTIONS(177),
    [anon_sym_DOT] = ACTIONS(179),
    [anon_sym_end] = ACTIONS(177),
    [anon_sym_rescue] = ACTIONS(177),
    [anon_sym_EQ] = ACTIONS(179),
    [anon_sym_COMMA] = ACTIONS(177),
    [anon_sym_PIPE] = ACTIONS(179),
    [anon_sym_SEMI] = ACTIONS(177),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(177),
    [anon_sym_AMP] = ACTIONS(179),
    [anon_sym_LT] = ACTIONS(179),
    [anon_sym_if] = ACTIONS(177),
    [anon_sym_unless] = ACTIONS(177),
    [anon_sym_while] = ACTIONS(177),
    [anon_sym_until] = ACTIONS(177),
    [anon_sym_in] = ACTIONS(177),
    [anon_sym_do] = ACTIONS(2964),
    [anon_sym_when] = ACTIONS(177),
    [anon_sym_EQ_GT] = ACTIONS(177),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(179),
    [anon_sym_CARET] = ACTIONS(179),
    [anon_sym_elsif] = ACTIONS(177),
    [anon_sym_else] = ACTIONS(177),
    [anon_sym_then] = ACTIONS(177),
    [anon_sym_ensure] = ACTIONS(177),
    [anon_sym_COLON_COLON2] = ACTIONS(177),
    [anon_sym_AMP_DOT] = ACTIONS(177),
    [anon_sym_PLUS_EQ] = ACTIONS(177),
    [anon_sym_DASH_EQ] = ACTIONS(177),
    [anon_sym_STAR_EQ] = ACTIONS(177),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(177),
    [anon_sym_SLASH_EQ] = ACTIONS(177),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(177),
    [anon_sym_PIPE_EQ] = ACTIONS(177),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(177),
    [anon_sym_AMP_EQ] = ACTIONS(177),
    [anon_sym_PERCENT_EQ] = ACTIONS(177),
    [anon_sym_GT_GT_EQ] = ACTIONS(177),
    [anon_sym_LT_LT_EQ] = ACTIONS(177),
    [anon_sym_CARET_EQ] = ACTIONS(177),
    [anon_sym_QMARK] = ACTIONS(177),
    [anon_sym_and] = ACTIONS(177),
    [anon_sym_or] = ACTIONS(177),
    [anon_sym_PIPE_PIPE] = ACTIONS(179),
    [anon_sym_AMP_AMP] = ACTIONS(179),
    [anon_sym_LT_LT] = ACTIONS(179),
    [anon_sym_GT_GT] = ACTIONS(179),
    [anon_sym_LT_EQ] = ACTIONS(179),
    [anon_sym_GT] = ACTIONS(179),
    [anon_sym_GT_EQ] = ACTIONS(177),
    [anon_sym_PLUS] = ACTIONS(179),
    [anon_sym_SLASH] = ACTIONS(179),
    [anon_sym_PERCENT] = ACTIONS(179),
    [anon_sym_EQ_EQ] = ACTIONS(179),
    [anon_sym_BANG_EQ] = ACTIONS(177),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(177),
    [anon_sym_LT_EQ_GT] = ACTIONS(177),
    [anon_sym_EQ_TILDE] = ACTIONS(177),
    [anon_sym_BANG_TILDE] = ACTIONS(177),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(177),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(177),
    [sym__binary_star] = ACTIONS(177),
    [sym__binary_star_star] = ACTIONS(177),
    [sym__element_reference_bracket] = ACTIONS(177),
  },
  [2047] = {
    [sym__pattern_expr] = STATE(4477),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2047),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_COMMA] = ACTIONS(2374),
    [anon_sym_SEMI] = ACTIONS(2374),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2850),
    [anon_sym_unless] = ACTIONS(2850),
    [anon_sym_do] = ACTIONS(2850),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_then] = ACTIONS(2850),
    [anon_sym_and] = ACTIONS(2850),
    [anon_sym_or] = ACTIONS(2850),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2374),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
  },
  [2048] = {
    [sym_argument_list] = STATE(2264),
    [sym_do_block] = STATE(2316),
    [sym_block] = STATE(2318),
    [sym_heredoc_body] = STATE(2048),
    [ts_builtin_sym_end] = ACTIONS(121),
    [aux_sym_program_token1] = ACTIONS(121),
    [anon_sym_LBRACE] = ACTIONS(2962),
    [anon_sym_RBRACE] = ACTIONS(121),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_end] = ACTIONS(121),
    [anon_sym_rescue] = ACTIONS(121),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_if] = ACTIONS(121),
    [anon_sym_unless] = ACTIONS(121),
    [anon_sym_while] = ACTIONS(121),
    [anon_sym_until] = ACTIONS(121),
    [anon_sym_in] = ACTIONS(121),
    [anon_sym_do] = ACTIONS(121),
    [anon_sym_when] = ACTIONS(121),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_LPAREN2] = ACTIONS(141),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_elsif] = ACTIONS(121),
    [anon_sym_else] = ACTIONS(121),
    [anon_sym_then] = ACTIONS(121),
    [anon_sym_ensure] = ACTIONS(121),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(121),
    [anon_sym_and] = ACTIONS(121),
    [anon_sym_or] = ACTIONS(121),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(121),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [2049] = {
    [sym__pattern_expr] = STATE(4478),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2049),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_COMMA] = ACTIONS(2846),
    [anon_sym_SEMI] = ACTIONS(2846),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_if] = ACTIONS(2848),
    [anon_sym_unless] = ACTIONS(2848),
    [anon_sym_do] = ACTIONS(2848),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_then] = ACTIONS(2848),
    [anon_sym_and] = ACTIONS(2848),
    [anon_sym_or] = ACTIONS(2848),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym__line_break] = ACTIONS(2846),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
  },
  [2050] = {
    [sym_heredoc_body] = STATE(2050),
    [sym_identifier] = ACTIONS(2966),
    [anon_sym_BEGIN] = ACTIONS(2966),
    [anon_sym_LBRACE] = ACTIONS(2968),
    [anon_sym_RBRACE] = ACTIONS(2968),
    [anon_sym_END] = ACTIONS(2966),
    [anon_sym_def] = ACTIONS(2966),
    [anon_sym_LPAREN] = ACTIONS(2968),
    [anon_sym_RPAREN] = ACTIONS(2968),
    [anon_sym_COLON_COLON] = ACTIONS(2968),
    [anon_sym_end] = ACTIONS(2966),
    [anon_sym_rescue] = ACTIONS(2966),
    [anon_sym_SEMI] = ACTIONS(2968),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2968),
    [anon_sym_STAR] = ACTIONS(2968),
    [anon_sym_nil] = ACTIONS(2966),
    [anon_sym_class] = ACTIONS(2966),
    [anon_sym_module] = ACTIONS(2966),
    [anon_sym_return] = ACTIONS(2966),
    [anon_sym_yield] = ACTIONS(2966),
    [anon_sym_break] = ACTIONS(2966),
    [anon_sym_next] = ACTIONS(2966),
    [anon_sym_redo] = ACTIONS(2966),
    [anon_sym_retry] = ACTIONS(2966),
    [anon_sym_if] = ACTIONS(2966),
    [anon_sym_unless] = ACTIONS(2966),
    [anon_sym_while] = ACTIONS(2966),
    [anon_sym_until] = ACTIONS(2966),
    [anon_sym_for] = ACTIONS(2966),
    [anon_sym_in] = ACTIONS(2966),
    [anon_sym_case] = ACTIONS(2966),
    [anon_sym_when] = ACTIONS(2966),
    [anon_sym_LBRACK] = ACTIONS(2968),
    [anon_sym_DOT_DOT] = ACTIONS(2966),
    [anon_sym_elsif] = ACTIONS(2966),
    [anon_sym_else] = ACTIONS(2966),
    [anon_sym_begin] = ACTIONS(2966),
    [anon_sym_ensure] = ACTIONS(2966),
    [anon_sym_PLUS] = ACTIONS(2968),
    [anon_sym_defined_QMARK] = ACTIONS(2968),
    [anon_sym_not] = ACTIONS(2966),
    [anon_sym_BANG] = ACTIONS(2968),
    [anon_sym_TILDE] = ACTIONS(2968),
    [anon_sym_undef] = ACTIONS(2966),
    [anon_sym_alias] = ACTIONS(2966),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2966),
    [sym_float] = ACTIONS(2966),
    [sym_super] = ACTIONS(2966),
    [sym_self] = ACTIONS(2966),
    [sym_true] = ACTIONS(2966),
    [sym_false] = ACTIONS(2966),
    [sym_constant] = ACTIONS(2966),
    [aux_sym_constant_suffix_token1] = ACTIONS(2968),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2966),
    [sym_instance_variable] = ACTIONS(2968),
    [sym_class_variable] = ACTIONS(2968),
    [sym_global_variable] = ACTIONS(2968),
    [sym_character] = ACTIONS(2968),
    [anon_sym_DASH_GT] = ACTIONS(2968),
    [sym_simple_symbol] = ACTIONS(2968),
    [sym__string_start] = ACTIONS(2968),
    [sym__symbol_start] = ACTIONS(2968),
    [sym__subshell_start] = ACTIONS(2968),
    [sym__regex_start] = ACTIONS(2968),
    [sym__string_array_start] = ACTIONS(2968),
    [sym__symbol_array_start] = ACTIONS(2968),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2968),
    [sym__unary_minus] = ACTIONS(2968),
    [sym__unary_minus_num] = ACTIONS(2968),
    [sym__binary_minus] = ACTIONS(2968),
    [sym__identifier_suffix] = ACTIONS(2968),
    [sym__constant_suffix] = ACTIONS(2968),
  },
  [2051] = {
    [sym_heredoc_body] = STATE(2051),
    [sym_identifier] = ACTIONS(2970),
    [anon_sym_BEGIN] = ACTIONS(2970),
    [anon_sym_LBRACE] = ACTIONS(2972),
    [anon_sym_RBRACE] = ACTIONS(2972),
    [anon_sym_END] = ACTIONS(2970),
    [anon_sym_def] = ACTIONS(2970),
    [anon_sym_LPAREN] = ACTIONS(2972),
    [anon_sym_RPAREN] = ACTIONS(2972),
    [anon_sym_COLON_COLON] = ACTIONS(2972),
    [anon_sym_end] = ACTIONS(2970),
    [anon_sym_rescue] = ACTIONS(2970),
    [anon_sym_SEMI] = ACTIONS(2972),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2972),
    [anon_sym_STAR] = ACTIONS(2972),
    [anon_sym_nil] = ACTIONS(2970),
    [anon_sym_class] = ACTIONS(2970),
    [anon_sym_module] = ACTIONS(2970),
    [anon_sym_return] = ACTIONS(2970),
    [anon_sym_yield] = ACTIONS(2970),
    [anon_sym_break] = ACTIONS(2970),
    [anon_sym_next] = ACTIONS(2970),
    [anon_sym_redo] = ACTIONS(2970),
    [anon_sym_retry] = ACTIONS(2970),
    [anon_sym_if] = ACTIONS(2970),
    [anon_sym_unless] = ACTIONS(2970),
    [anon_sym_while] = ACTIONS(2970),
    [anon_sym_until] = ACTIONS(2970),
    [anon_sym_for] = ACTIONS(2970),
    [anon_sym_in] = ACTIONS(2970),
    [anon_sym_case] = ACTIONS(2970),
    [anon_sym_when] = ACTIONS(2970),
    [anon_sym_LBRACK] = ACTIONS(2972),
    [anon_sym_DOT_DOT] = ACTIONS(2970),
    [anon_sym_elsif] = ACTIONS(2970),
    [anon_sym_else] = ACTIONS(2970),
    [anon_sym_begin] = ACTIONS(2970),
    [anon_sym_ensure] = ACTIONS(2970),
    [anon_sym_PLUS] = ACTIONS(2972),
    [anon_sym_defined_QMARK] = ACTIONS(2972),
    [anon_sym_not] = ACTIONS(2970),
    [anon_sym_BANG] = ACTIONS(2972),
    [anon_sym_TILDE] = ACTIONS(2972),
    [anon_sym_undef] = ACTIONS(2970),
    [anon_sym_alias] = ACTIONS(2970),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2970),
    [sym_float] = ACTIONS(2970),
    [sym_super] = ACTIONS(2970),
    [sym_self] = ACTIONS(2970),
    [sym_true] = ACTIONS(2970),
    [sym_false] = ACTIONS(2970),
    [sym_constant] = ACTIONS(2970),
    [aux_sym_constant_suffix_token1] = ACTIONS(2972),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2970),
    [sym_instance_variable] = ACTIONS(2972),
    [sym_class_variable] = ACTIONS(2972),
    [sym_global_variable] = ACTIONS(2972),
    [sym_character] = ACTIONS(2972),
    [anon_sym_DASH_GT] = ACTIONS(2972),
    [sym_simple_symbol] = ACTIONS(2972),
    [sym__string_start] = ACTIONS(2972),
    [sym__symbol_start] = ACTIONS(2972),
    [sym__subshell_start] = ACTIONS(2972),
    [sym__regex_start] = ACTIONS(2972),
    [sym__string_array_start] = ACTIONS(2972),
    [sym__symbol_array_start] = ACTIONS(2972),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2972),
    [sym__unary_minus] = ACTIONS(2972),
    [sym__unary_minus_num] = ACTIONS(2972),
    [sym__binary_minus] = ACTIONS(2972),
    [sym__identifier_suffix] = ACTIONS(2972),
    [sym__constant_suffix] = ACTIONS(2972),
  },
  [2052] = {
    [sym_heredoc_body] = STATE(2052),
    [ts_builtin_sym_end] = ACTIONS(2960),
    [aux_sym_program_token1] = ACTIONS(2960),
    [anon_sym_LBRACE] = ACTIONS(2960),
    [anon_sym_RBRACE] = ACTIONS(2960),
    [anon_sym_RPAREN] = ACTIONS(2960),
    [anon_sym_DOT] = ACTIONS(2958),
    [anon_sym_COLON_COLON] = ACTIONS(2974),
    [anon_sym_end] = ACTIONS(2960),
    [anon_sym_rescue] = ACTIONS(2960),
    [anon_sym_EQ] = ACTIONS(2958),
    [anon_sym_COMMA] = ACTIONS(2960),
    [anon_sym_PIPE] = ACTIONS(2958),
    [anon_sym_SEMI] = ACTIONS(2960),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2960),
    [anon_sym_AMP] = ACTIONS(2958),
    [anon_sym_LT] = ACTIONS(2958),
    [anon_sym_if] = ACTIONS(2960),
    [anon_sym_unless] = ACTIONS(2960),
    [anon_sym_while] = ACTIONS(2960),
    [anon_sym_until] = ACTIONS(2960),
    [anon_sym_in] = ACTIONS(2960),
    [anon_sym_do] = ACTIONS(2960),
    [anon_sym_when] = ACTIONS(2960),
    [anon_sym_EQ_GT] = ACTIONS(2976),
    [anon_sym_LBRACK2] = ACTIONS(2976),
    [anon_sym_LPAREN2] = ACTIONS(2976),
    [anon_sym_DOT_DOT] = ACTIONS(2958),
    [anon_sym_CARET] = ACTIONS(2958),
    [anon_sym_elsif] = ACTIONS(2960),
    [anon_sym_else] = ACTIONS(2960),
    [anon_sym_then] = ACTIONS(2960),
    [anon_sym_ensure] = ACTIONS(2960),
    [anon_sym_COLON_COLON2] = ACTIONS(2960),
    [anon_sym_AMP_DOT] = ACTIONS(2960),
    [anon_sym_PLUS_EQ] = ACTIONS(2960),
    [anon_sym_DASH_EQ] = ACTIONS(2960),
    [anon_sym_STAR_EQ] = ACTIONS(2960),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2960),
    [anon_sym_SLASH_EQ] = ACTIONS(2960),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2960),
    [anon_sym_PIPE_EQ] = ACTIONS(2960),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2960),
    [anon_sym_AMP_EQ] = ACTIONS(2960),
    [anon_sym_PERCENT_EQ] = ACTIONS(2960),
    [anon_sym_GT_GT_EQ] = ACTIONS(2960),
    [anon_sym_LT_LT_EQ] = ACTIONS(2960),
    [anon_sym_CARET_EQ] = ACTIONS(2960),
    [anon_sym_QMARK] = ACTIONS(2960),
    [anon_sym_and] = ACTIONS(2960),
    [anon_sym_or] = ACTIONS(2960),
    [anon_sym_PIPE_PIPE] = ACTIONS(2958),
    [anon_sym_AMP_AMP] = ACTIONS(2958),
    [anon_sym_LT_LT] = ACTIONS(2958),
    [anon_sym_GT_GT] = ACTIONS(2958),
    [anon_sym_LT_EQ] = ACTIONS(2958),
    [anon_sym_GT] = ACTIONS(2958),
    [anon_sym_GT_EQ] = ACTIONS(2960),
    [anon_sym_PLUS] = ACTIONS(2958),
    [anon_sym_SLASH] = ACTIONS(2958),
    [anon_sym_PERCENT] = ACTIONS(2958),
    [anon_sym_EQ_EQ] = ACTIONS(2958),
    [anon_sym_BANG_EQ] = ACTIONS(2960),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2960),
    [anon_sym_LT_EQ_GT] = ACTIONS(2960),
    [anon_sym_EQ_TILDE] = ACTIONS(2960),
    [anon_sym_BANG_TILDE] = ACTIONS(2960),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2960),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2960),
    [sym__binary_star] = ACTIONS(2960),
    [sym__binary_star_star] = ACTIONS(2960),
    [sym__element_reference_bracket] = ACTIONS(2960),
  },
  [2053] = {
    [sym_heredoc_body] = STATE(2053),
    [sym_identifier] = ACTIONS(1124),
    [anon_sym_BEGIN] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1110),
    [anon_sym_RBRACE] = ACTIONS(1110),
    [anon_sym_END] = ACTIONS(1124),
    [anon_sym_def] = ACTIONS(1124),
    [anon_sym_LPAREN] = ACTIONS(1110),
    [anon_sym_RPAREN] = ACTIONS(1110),
    [anon_sym_COLON_COLON] = ACTIONS(1110),
    [anon_sym_end] = ACTIONS(1124),
    [anon_sym_rescue] = ACTIONS(1124),
    [anon_sym_SEMI] = ACTIONS(1110),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1110),
    [anon_sym_STAR] = ACTIONS(1110),
    [anon_sym_nil] = ACTIONS(1124),
    [anon_sym_class] = ACTIONS(1124),
    [anon_sym_module] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_yield] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_next] = ACTIONS(1124),
    [anon_sym_redo] = ACTIONS(1124),
    [anon_sym_retry] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_unless] = ACTIONS(1124),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_until] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_in] = ACTIONS(1124),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_when] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1110),
    [anon_sym_DOT_DOT] = ACTIONS(1124),
    [anon_sym_elsif] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_begin] = ACTIONS(1124),
    [anon_sym_ensure] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1110),
    [anon_sym_defined_QMARK] = ACTIONS(1110),
    [anon_sym_not] = ACTIONS(1124),
    [anon_sym_BANG] = ACTIONS(1110),
    [anon_sym_TILDE] = ACTIONS(1110),
    [anon_sym_undef] = ACTIONS(1124),
    [anon_sym_alias] = ACTIONS(1124),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(1124),
    [sym_float] = ACTIONS(1124),
    [sym_super] = ACTIONS(1124),
    [sym_self] = ACTIONS(1124),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [sym_constant] = ACTIONS(1124),
    [aux_sym_constant_suffix_token1] = ACTIONS(1110),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1124),
    [sym_instance_variable] = ACTIONS(1110),
    [sym_class_variable] = ACTIONS(1110),
    [sym_global_variable] = ACTIONS(1110),
    [sym_character] = ACTIONS(1110),
    [anon_sym_DASH_GT] = ACTIONS(1110),
    [sym_simple_symbol] = ACTIONS(1110),
    [sym__string_start] = ACTIONS(1110),
    [sym__symbol_start] = ACTIONS(1110),
    [sym__subshell_start] = ACTIONS(1110),
    [sym__regex_start] = ACTIONS(1110),
    [sym__string_array_start] = ACTIONS(1110),
    [sym__symbol_array_start] = ACTIONS(1110),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1110),
    [sym__unary_minus] = ACTIONS(1110),
    [sym__unary_minus_num] = ACTIONS(1110),
    [sym__binary_minus] = ACTIONS(1110),
    [sym__identifier_suffix] = ACTIONS(1110),
    [sym__constant_suffix] = ACTIONS(1110),
  },
  [2054] = {
    [sym_heredoc_body] = STATE(2054),
    [ts_builtin_sym_end] = ACTIONS(2948),
    [sym_identifier] = ACTIONS(2946),
    [aux_sym_program_token1] = ACTIONS(2946),
    [anon_sym_LBRACE] = ACTIONS(2948),
    [anon_sym_def] = ACTIONS(2946),
    [anon_sym_LPAREN] = ACTIONS(2948),
    [anon_sym_COLON_COLON] = ACTIONS(2948),
    [anon_sym_rescue] = ACTIONS(2946),
    [anon_sym_COMMA] = ACTIONS(2948),
    [anon_sym_SEMI] = ACTIONS(2948),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2948),
    [anon_sym_STAR] = ACTIONS(2948),
    [anon_sym_nil] = ACTIONS(2946),
    [anon_sym_class] = ACTIONS(2946),
    [anon_sym_module] = ACTIONS(2946),
    [anon_sym_return] = ACTIONS(2946),
    [anon_sym_yield] = ACTIONS(2946),
    [anon_sym_break] = ACTIONS(2946),
    [anon_sym_next] = ACTIONS(2946),
    [anon_sym_redo] = ACTIONS(2946),
    [anon_sym_retry] = ACTIONS(2946),
    [anon_sym_if] = ACTIONS(2946),
    [anon_sym_unless] = ACTIONS(2946),
    [anon_sym_while] = ACTIONS(2946),
    [anon_sym_until] = ACTIONS(2946),
    [anon_sym_for] = ACTIONS(2946),
    [anon_sym_do] = ACTIONS(2946),
    [anon_sym_case] = ACTIONS(2946),
    [anon_sym_LBRACK] = ACTIONS(2948),
    [anon_sym_DOT_DOT] = ACTIONS(2946),
    [anon_sym_begin] = ACTIONS(2946),
    [anon_sym_and] = ACTIONS(2946),
    [anon_sym_or] = ACTIONS(2946),
    [anon_sym_PLUS] = ACTIONS(2948),
    [anon_sym_defined_QMARK] = ACTIONS(2948),
    [anon_sym_not] = ACTIONS(2946),
    [anon_sym_BANG] = ACTIONS(2948),
    [anon_sym_TILDE] = ACTIONS(2948),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2946),
    [sym_float] = ACTIONS(2946),
    [sym_super] = ACTIONS(2946),
    [sym_self] = ACTIONS(2946),
    [sym_true] = ACTIONS(2946),
    [sym_false] = ACTIONS(2946),
    [sym_constant] = ACTIONS(2946),
    [aux_sym_constant_suffix_token1] = ACTIONS(2948),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2946),
    [sym_instance_variable] = ACTIONS(2948),
    [sym_class_variable] = ACTIONS(2948),
    [sym_global_variable] = ACTIONS(2948),
    [sym_character] = ACTIONS(2948),
    [anon_sym_DASH_GT] = ACTIONS(2948),
    [sym__line_break] = ACTIONS(2948),
    [sym_simple_symbol] = ACTIONS(2948),
    [sym__string_start] = ACTIONS(2948),
    [sym__symbol_start] = ACTIONS(2948),
    [sym__subshell_start] = ACTIONS(2948),
    [sym__regex_start] = ACTIONS(2948),
    [sym__string_array_start] = ACTIONS(2948),
    [sym__symbol_array_start] = ACTIONS(2948),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2948),
    [sym__block_ampersand] = ACTIONS(2948),
    [sym__splat_star] = ACTIONS(2948),
    [sym__unary_minus] = ACTIONS(2948),
    [sym__unary_minus_num] = ACTIONS(2948),
    [sym__binary_minus] = ACTIONS(2948),
    [sym_hash_key_symbol] = ACTIONS(2948),
    [sym__identifier_suffix] = ACTIONS(2948),
    [sym__constant_suffix] = ACTIONS(2948),
    [sym__hash_splat_star_star] = ACTIONS(2948),
  },
  [2055] = {
    [sym__pattern_expr] = STATE(4829),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2055),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2374),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_rescue] = ACTIONS(2850),
    [anon_sym_COMMA] = ACTIONS(2374),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2904),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_do] = ACTIONS(2850),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2374),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_and] = ACTIONS(2850),
    [anon_sym_or] = ACTIONS(2850),
    [anon_sym_PLUS] = ACTIONS(2908),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2910),
    [sym_float] = ACTIONS(2910),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2908),
  },
  [2056] = {
    [sym_heredoc_body] = STATE(2056),
    [ts_builtin_sym_end] = ACTIONS(2644),
    [sym_identifier] = ACTIONS(2642),
    [aux_sym_program_token1] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2644),
    [anon_sym_COLON_COLON] = ACTIONS(2644),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_SEMI] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2644),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PLUS] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2644),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym__line_break] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
  },
  [2057] = {
    [sym_do_block] = STATE(2112),
    [sym_block] = STATE(2112),
    [sym_heredoc_body] = STATE(2057),
    [ts_builtin_sym_end] = ACTIONS(2978),
    [aux_sym_program_token1] = ACTIONS(2978),
    [anon_sym_LBRACE] = ACTIONS(2980),
    [anon_sym_RBRACE] = ACTIONS(2978),
    [anon_sym_RPAREN] = ACTIONS(2978),
    [anon_sym_DOT] = ACTIONS(2982),
    [anon_sym_end] = ACTIONS(2978),
    [anon_sym_rescue] = ACTIONS(2978),
    [anon_sym_EQ] = ACTIONS(2982),
    [anon_sym_COMMA] = ACTIONS(2978),
    [anon_sym_PIPE] = ACTIONS(2982),
    [anon_sym_SEMI] = ACTIONS(2978),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2978),
    [anon_sym_AMP] = ACTIONS(2982),
    [anon_sym_LT] = ACTIONS(2982),
    [anon_sym_if] = ACTIONS(2978),
    [anon_sym_unless] = ACTIONS(2978),
    [anon_sym_while] = ACTIONS(2978),
    [anon_sym_until] = ACTIONS(2978),
    [anon_sym_in] = ACTIONS(2978),
    [anon_sym_do] = ACTIONS(2984),
    [anon_sym_when] = ACTIONS(2978),
    [anon_sym_EQ_GT] = ACTIONS(2978),
    [anon_sym_DOT_DOT] = ACTIONS(2982),
    [anon_sym_CARET] = ACTIONS(2982),
    [anon_sym_elsif] = ACTIONS(2978),
    [anon_sym_else] = ACTIONS(2978),
    [anon_sym_then] = ACTIONS(2978),
    [anon_sym_ensure] = ACTIONS(2978),
    [anon_sym_COLON_COLON2] = ACTIONS(2978),
    [anon_sym_AMP_DOT] = ACTIONS(2978),
    [anon_sym_PLUS_EQ] = ACTIONS(2978),
    [anon_sym_DASH_EQ] = ACTIONS(2978),
    [anon_sym_STAR_EQ] = ACTIONS(2978),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2978),
    [anon_sym_SLASH_EQ] = ACTIONS(2978),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2978),
    [anon_sym_PIPE_EQ] = ACTIONS(2978),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2978),
    [anon_sym_AMP_EQ] = ACTIONS(2978),
    [anon_sym_PERCENT_EQ] = ACTIONS(2978),
    [anon_sym_GT_GT_EQ] = ACTIONS(2978),
    [anon_sym_LT_LT_EQ] = ACTIONS(2978),
    [anon_sym_CARET_EQ] = ACTIONS(2978),
    [anon_sym_QMARK] = ACTIONS(2978),
    [anon_sym_and] = ACTIONS(2978),
    [anon_sym_or] = ACTIONS(2978),
    [anon_sym_PIPE_PIPE] = ACTIONS(2982),
    [anon_sym_AMP_AMP] = ACTIONS(2982),
    [anon_sym_LT_LT] = ACTIONS(2982),
    [anon_sym_GT_GT] = ACTIONS(2982),
    [anon_sym_LT_EQ] = ACTIONS(2982),
    [anon_sym_GT] = ACTIONS(2982),
    [anon_sym_GT_EQ] = ACTIONS(2978),
    [anon_sym_PLUS] = ACTIONS(2982),
    [anon_sym_SLASH] = ACTIONS(2982),
    [anon_sym_PERCENT] = ACTIONS(2982),
    [anon_sym_EQ_EQ] = ACTIONS(2982),
    [anon_sym_BANG_EQ] = ACTIONS(2978),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2978),
    [anon_sym_LT_EQ_GT] = ACTIONS(2978),
    [anon_sym_EQ_TILDE] = ACTIONS(2978),
    [anon_sym_BANG_TILDE] = ACTIONS(2978),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2978),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2978),
    [sym__binary_star] = ACTIONS(2978),
    [sym__binary_star_star] = ACTIONS(2978),
    [sym__element_reference_bracket] = ACTIONS(2978),
  },
  [2058] = {
    [sym__pattern_expr] = STATE(4858),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2058),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2846),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_rescue] = ACTIONS(2848),
    [anon_sym_COMMA] = ACTIONS(2846),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2904),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_do] = ACTIONS(2848),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2846),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_and] = ACTIONS(2848),
    [anon_sym_or] = ACTIONS(2848),
    [anon_sym_PLUS] = ACTIONS(2908),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2910),
    [sym_float] = ACTIONS(2910),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2908),
  },
  [2059] = {
    [sym_do_block] = STATE(2106),
    [sym_block] = STATE(2106),
    [sym_heredoc_body] = STATE(2059),
    [ts_builtin_sym_end] = ACTIONS(2986),
    [aux_sym_program_token1] = ACTIONS(2986),
    [anon_sym_LBRACE] = ACTIONS(2986),
    [anon_sym_RBRACE] = ACTIONS(2986),
    [anon_sym_RPAREN] = ACTIONS(2986),
    [anon_sym_DOT] = ACTIONS(2988),
    [anon_sym_end] = ACTIONS(2986),
    [anon_sym_rescue] = ACTIONS(2986),
    [anon_sym_EQ] = ACTIONS(2988),
    [anon_sym_COMMA] = ACTIONS(2986),
    [anon_sym_PIPE] = ACTIONS(2988),
    [anon_sym_SEMI] = ACTIONS(2986),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2986),
    [anon_sym_AMP] = ACTIONS(2988),
    [anon_sym_LT] = ACTIONS(2988),
    [anon_sym_if] = ACTIONS(2986),
    [anon_sym_unless] = ACTIONS(2986),
    [anon_sym_while] = ACTIONS(2986),
    [anon_sym_until] = ACTIONS(2986),
    [anon_sym_in] = ACTIONS(2986),
    [anon_sym_do] = ACTIONS(2986),
    [anon_sym_when] = ACTIONS(2986),
    [anon_sym_EQ_GT] = ACTIONS(2986),
    [anon_sym_DOT_DOT] = ACTIONS(2988),
    [anon_sym_CARET] = ACTIONS(2988),
    [anon_sym_elsif] = ACTIONS(2986),
    [anon_sym_else] = ACTIONS(2986),
    [anon_sym_then] = ACTIONS(2986),
    [anon_sym_ensure] = ACTIONS(2986),
    [anon_sym_COLON_COLON2] = ACTIONS(2986),
    [anon_sym_AMP_DOT] = ACTIONS(2986),
    [anon_sym_PLUS_EQ] = ACTIONS(2986),
    [anon_sym_DASH_EQ] = ACTIONS(2986),
    [anon_sym_STAR_EQ] = ACTIONS(2986),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2986),
    [anon_sym_SLASH_EQ] = ACTIONS(2986),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2986),
    [anon_sym_PIPE_EQ] = ACTIONS(2986),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2986),
    [anon_sym_AMP_EQ] = ACTIONS(2986),
    [anon_sym_PERCENT_EQ] = ACTIONS(2986),
    [anon_sym_GT_GT_EQ] = ACTIONS(2986),
    [anon_sym_LT_LT_EQ] = ACTIONS(2986),
    [anon_sym_CARET_EQ] = ACTIONS(2986),
    [anon_sym_QMARK] = ACTIONS(2986),
    [anon_sym_and] = ACTIONS(2986),
    [anon_sym_or] = ACTIONS(2986),
    [anon_sym_PIPE_PIPE] = ACTIONS(2988),
    [anon_sym_AMP_AMP] = ACTIONS(2988),
    [anon_sym_LT_LT] = ACTIONS(2988),
    [anon_sym_GT_GT] = ACTIONS(2988),
    [anon_sym_LT_EQ] = ACTIONS(2988),
    [anon_sym_GT] = ACTIONS(2988),
    [anon_sym_GT_EQ] = ACTIONS(2986),
    [anon_sym_PLUS] = ACTIONS(2988),
    [anon_sym_SLASH] = ACTIONS(2988),
    [anon_sym_PERCENT] = ACTIONS(2988),
    [anon_sym_EQ_EQ] = ACTIONS(2988),
    [anon_sym_BANG_EQ] = ACTIONS(2986),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2986),
    [anon_sym_LT_EQ_GT] = ACTIONS(2986),
    [anon_sym_EQ_TILDE] = ACTIONS(2986),
    [anon_sym_BANG_TILDE] = ACTIONS(2986),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2986),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2986),
    [sym__binary_star] = ACTIONS(2986),
    [sym__binary_star_star] = ACTIONS(2986),
    [sym__element_reference_bracket] = ACTIONS(2986),
  },
  [2060] = {
    [sym__pattern_expr] = STATE(4858),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2060),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_RBRACE] = ACTIONS(2846),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2846),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_COMMA] = ACTIONS(2846),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_do] = ACTIONS(2848),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2846),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_and] = ACTIONS(2848),
    [anon_sym_or] = ACTIONS(2848),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2061] = {
    [sym_do_block] = STATE(2106),
    [sym_block] = STATE(2106),
    [sym_heredoc_body] = STATE(2061),
    [ts_builtin_sym_end] = ACTIONS(2986),
    [aux_sym_program_token1] = ACTIONS(2986),
    [anon_sym_LBRACE] = ACTIONS(2980),
    [anon_sym_RBRACE] = ACTIONS(2986),
    [anon_sym_RPAREN] = ACTIONS(2986),
    [anon_sym_DOT] = ACTIONS(2988),
    [anon_sym_end] = ACTIONS(2986),
    [anon_sym_rescue] = ACTIONS(2986),
    [anon_sym_EQ] = ACTIONS(2988),
    [anon_sym_COMMA] = ACTIONS(2986),
    [anon_sym_PIPE] = ACTIONS(2988),
    [anon_sym_SEMI] = ACTIONS(2986),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2986),
    [anon_sym_AMP] = ACTIONS(2988),
    [anon_sym_LT] = ACTIONS(2988),
    [anon_sym_if] = ACTIONS(2986),
    [anon_sym_unless] = ACTIONS(2986),
    [anon_sym_while] = ACTIONS(2986),
    [anon_sym_until] = ACTIONS(2986),
    [anon_sym_in] = ACTIONS(2986),
    [anon_sym_do] = ACTIONS(2984),
    [anon_sym_when] = ACTIONS(2986),
    [anon_sym_EQ_GT] = ACTIONS(2986),
    [anon_sym_DOT_DOT] = ACTIONS(2988),
    [anon_sym_CARET] = ACTIONS(2988),
    [anon_sym_elsif] = ACTIONS(2986),
    [anon_sym_else] = ACTIONS(2986),
    [anon_sym_then] = ACTIONS(2986),
    [anon_sym_ensure] = ACTIONS(2986),
    [anon_sym_COLON_COLON2] = ACTIONS(2986),
    [anon_sym_AMP_DOT] = ACTIONS(2986),
    [anon_sym_PLUS_EQ] = ACTIONS(2986),
    [anon_sym_DASH_EQ] = ACTIONS(2986),
    [anon_sym_STAR_EQ] = ACTIONS(2986),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2986),
    [anon_sym_SLASH_EQ] = ACTIONS(2986),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2986),
    [anon_sym_PIPE_EQ] = ACTIONS(2986),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2986),
    [anon_sym_AMP_EQ] = ACTIONS(2986),
    [anon_sym_PERCENT_EQ] = ACTIONS(2986),
    [anon_sym_GT_GT_EQ] = ACTIONS(2986),
    [anon_sym_LT_LT_EQ] = ACTIONS(2986),
    [anon_sym_CARET_EQ] = ACTIONS(2986),
    [anon_sym_QMARK] = ACTIONS(2986),
    [anon_sym_and] = ACTIONS(2986),
    [anon_sym_or] = ACTIONS(2986),
    [anon_sym_PIPE_PIPE] = ACTIONS(2988),
    [anon_sym_AMP_AMP] = ACTIONS(2988),
    [anon_sym_LT_LT] = ACTIONS(2988),
    [anon_sym_GT_GT] = ACTIONS(2988),
    [anon_sym_LT_EQ] = ACTIONS(2988),
    [anon_sym_GT] = ACTIONS(2988),
    [anon_sym_GT_EQ] = ACTIONS(2986),
    [anon_sym_PLUS] = ACTIONS(2988),
    [anon_sym_SLASH] = ACTIONS(2988),
    [anon_sym_PERCENT] = ACTIONS(2988),
    [anon_sym_EQ_EQ] = ACTIONS(2988),
    [anon_sym_BANG_EQ] = ACTIONS(2986),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2986),
    [anon_sym_LT_EQ_GT] = ACTIONS(2986),
    [anon_sym_EQ_TILDE] = ACTIONS(2986),
    [anon_sym_BANG_TILDE] = ACTIONS(2986),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2986),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2986),
    [sym__binary_star] = ACTIONS(2986),
    [sym__binary_star_star] = ACTIONS(2986),
    [sym__element_reference_bracket] = ACTIONS(2986),
  },
  [2062] = {
    [sym_heredoc_body] = STATE(2062),
    [ts_builtin_sym_end] = ACTIONS(2636),
    [sym_identifier] = ACTIONS(2634),
    [aux_sym_program_token1] = ACTIONS(2634),
    [anon_sym_LBRACE] = ACTIONS(2636),
    [anon_sym_def] = ACTIONS(2634),
    [anon_sym_LPAREN] = ACTIONS(2636),
    [anon_sym_COLON_COLON] = ACTIONS(2636),
    [anon_sym_rescue] = ACTIONS(2634),
    [anon_sym_COMMA] = ACTIONS(2636),
    [anon_sym_SEMI] = ACTIONS(2636),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2636),
    [anon_sym_STAR] = ACTIONS(2636),
    [anon_sym_nil] = ACTIONS(2634),
    [anon_sym_class] = ACTIONS(2634),
    [anon_sym_module] = ACTIONS(2634),
    [anon_sym_return] = ACTIONS(2634),
    [anon_sym_yield] = ACTIONS(2634),
    [anon_sym_break] = ACTIONS(2634),
    [anon_sym_next] = ACTIONS(2634),
    [anon_sym_redo] = ACTIONS(2634),
    [anon_sym_retry] = ACTIONS(2634),
    [anon_sym_if] = ACTIONS(2634),
    [anon_sym_unless] = ACTIONS(2634),
    [anon_sym_while] = ACTIONS(2634),
    [anon_sym_until] = ACTIONS(2634),
    [anon_sym_for] = ACTIONS(2634),
    [anon_sym_do] = ACTIONS(2634),
    [anon_sym_case] = ACTIONS(2634),
    [anon_sym_LBRACK] = ACTIONS(2636),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_begin] = ACTIONS(2634),
    [anon_sym_and] = ACTIONS(2634),
    [anon_sym_or] = ACTIONS(2634),
    [anon_sym_PLUS] = ACTIONS(2636),
    [anon_sym_defined_QMARK] = ACTIONS(2636),
    [anon_sym_not] = ACTIONS(2634),
    [anon_sym_BANG] = ACTIONS(2636),
    [anon_sym_TILDE] = ACTIONS(2636),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2634),
    [sym_float] = ACTIONS(2634),
    [sym_super] = ACTIONS(2634),
    [sym_self] = ACTIONS(2634),
    [sym_true] = ACTIONS(2634),
    [sym_false] = ACTIONS(2634),
    [sym_constant] = ACTIONS(2634),
    [aux_sym_constant_suffix_token1] = ACTIONS(2636),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2634),
    [sym_instance_variable] = ACTIONS(2636),
    [sym_class_variable] = ACTIONS(2636),
    [sym_global_variable] = ACTIONS(2636),
    [sym_character] = ACTIONS(2636),
    [anon_sym_DASH_GT] = ACTIONS(2636),
    [sym__line_break] = ACTIONS(2636),
    [sym_simple_symbol] = ACTIONS(2636),
    [sym__string_start] = ACTIONS(2636),
    [sym__symbol_start] = ACTIONS(2636),
    [sym__subshell_start] = ACTIONS(2636),
    [sym__regex_start] = ACTIONS(2636),
    [sym__string_array_start] = ACTIONS(2636),
    [sym__symbol_array_start] = ACTIONS(2636),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2636),
    [sym__block_ampersand] = ACTIONS(2636),
    [sym__splat_star] = ACTIONS(2636),
    [sym__unary_minus] = ACTIONS(2636),
    [sym__unary_minus_num] = ACTIONS(2636),
    [sym__binary_minus] = ACTIONS(2636),
    [sym_hash_key_symbol] = ACTIONS(2636),
    [sym__identifier_suffix] = ACTIONS(2636),
    [sym__constant_suffix] = ACTIONS(2636),
    [sym__hash_splat_star_star] = ACTIONS(2636),
  },
  [2063] = {
    [sym_heredoc_body] = STATE(2063),
    [ts_builtin_sym_end] = ACTIONS(2616),
    [sym_identifier] = ACTIONS(2614),
    [aux_sym_program_token1] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2616),
    [anon_sym_COLON_COLON] = ACTIONS(2616),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_SEMI] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2616),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PLUS] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2616),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym__line_break] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
  },
  [2064] = {
    [sym__pattern_expr] = STATE(4829),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2064),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_RBRACE] = ACTIONS(2374),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_RPAREN] = ACTIONS(2374),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_COMMA] = ACTIONS(2374),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_do] = ACTIONS(2850),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2374),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_and] = ACTIONS(2850),
    [anon_sym_or] = ACTIONS(2850),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2065] = {
    [sym_heredoc_body] = STATE(2065),
    [ts_builtin_sym_end] = ACTIONS(2628),
    [sym_identifier] = ACTIONS(2626),
    [aux_sym_program_token1] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2628),
    [anon_sym_COLON_COLON] = ACTIONS(2628),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2628),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym__line_break] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
  },
  [2066] = {
    [sym_heredoc_body] = STATE(2066),
    [ts_builtin_sym_end] = ACTIONS(2640),
    [sym_identifier] = ACTIONS(2638),
    [aux_sym_program_token1] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2640),
    [anon_sym_COLON_COLON] = ACTIONS(2640),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2640),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym__line_break] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
  },
  [2067] = {
    [sym_do_block] = STATE(2112),
    [sym_block] = STATE(2112),
    [sym_heredoc_body] = STATE(2067),
    [ts_builtin_sym_end] = ACTIONS(2978),
    [aux_sym_program_token1] = ACTIONS(2978),
    [anon_sym_LBRACE] = ACTIONS(2978),
    [anon_sym_RBRACE] = ACTIONS(2978),
    [anon_sym_RPAREN] = ACTIONS(2978),
    [anon_sym_DOT] = ACTIONS(2982),
    [anon_sym_end] = ACTIONS(2978),
    [anon_sym_rescue] = ACTIONS(2978),
    [anon_sym_EQ] = ACTIONS(2982),
    [anon_sym_COMMA] = ACTIONS(2978),
    [anon_sym_PIPE] = ACTIONS(2982),
    [anon_sym_SEMI] = ACTIONS(2978),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2978),
    [anon_sym_AMP] = ACTIONS(2982),
    [anon_sym_LT] = ACTIONS(2982),
    [anon_sym_if] = ACTIONS(2978),
    [anon_sym_unless] = ACTIONS(2978),
    [anon_sym_while] = ACTIONS(2978),
    [anon_sym_until] = ACTIONS(2978),
    [anon_sym_in] = ACTIONS(2978),
    [anon_sym_do] = ACTIONS(2978),
    [anon_sym_when] = ACTIONS(2978),
    [anon_sym_EQ_GT] = ACTIONS(2978),
    [anon_sym_DOT_DOT] = ACTIONS(2982),
    [anon_sym_CARET] = ACTIONS(2982),
    [anon_sym_elsif] = ACTIONS(2978),
    [anon_sym_else] = ACTIONS(2978),
    [anon_sym_then] = ACTIONS(2978),
    [anon_sym_ensure] = ACTIONS(2978),
    [anon_sym_COLON_COLON2] = ACTIONS(2978),
    [anon_sym_AMP_DOT] = ACTIONS(2978),
    [anon_sym_PLUS_EQ] = ACTIONS(2978),
    [anon_sym_DASH_EQ] = ACTIONS(2978),
    [anon_sym_STAR_EQ] = ACTIONS(2978),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2978),
    [anon_sym_SLASH_EQ] = ACTIONS(2978),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2978),
    [anon_sym_PIPE_EQ] = ACTIONS(2978),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2978),
    [anon_sym_AMP_EQ] = ACTIONS(2978),
    [anon_sym_PERCENT_EQ] = ACTIONS(2978),
    [anon_sym_GT_GT_EQ] = ACTIONS(2978),
    [anon_sym_LT_LT_EQ] = ACTIONS(2978),
    [anon_sym_CARET_EQ] = ACTIONS(2978),
    [anon_sym_QMARK] = ACTIONS(2978),
    [anon_sym_and] = ACTIONS(2978),
    [anon_sym_or] = ACTIONS(2978),
    [anon_sym_PIPE_PIPE] = ACTIONS(2982),
    [anon_sym_AMP_AMP] = ACTIONS(2982),
    [anon_sym_LT_LT] = ACTIONS(2982),
    [anon_sym_GT_GT] = ACTIONS(2982),
    [anon_sym_LT_EQ] = ACTIONS(2982),
    [anon_sym_GT] = ACTIONS(2982),
    [anon_sym_GT_EQ] = ACTIONS(2978),
    [anon_sym_PLUS] = ACTIONS(2982),
    [anon_sym_SLASH] = ACTIONS(2982),
    [anon_sym_PERCENT] = ACTIONS(2982),
    [anon_sym_EQ_EQ] = ACTIONS(2982),
    [anon_sym_BANG_EQ] = ACTIONS(2978),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2978),
    [anon_sym_LT_EQ_GT] = ACTIONS(2978),
    [anon_sym_EQ_TILDE] = ACTIONS(2978),
    [anon_sym_BANG_TILDE] = ACTIONS(2978),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2978),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2978),
    [sym__binary_star] = ACTIONS(2978),
    [sym__binary_star_star] = ACTIONS(2978),
    [sym__element_reference_bracket] = ACTIONS(2978),
  },
  [2068] = {
    [sym_heredoc_body] = STATE(2068),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(2990),
    [anon_sym_end] = ACTIONS(743),
    [anon_sym_rescue] = ACTIONS(743),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2992),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(743),
    [anon_sym_unless] = ACTIONS(743),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_until] = ACTIONS(743),
    [anon_sym_in] = ACTIONS(743),
    [anon_sym_do] = ACTIONS(743),
    [anon_sym_when] = ACTIONS(743),
    [anon_sym_EQ_GT] = ACTIONS(2740),
    [anon_sym_LBRACK2] = ACTIONS(2740),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_elsif] = ACTIONS(743),
    [anon_sym_else] = ACTIONS(743),
    [anon_sym_then] = ACTIONS(743),
    [anon_sym_ensure] = ACTIONS(743),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(743),
    [anon_sym_and] = ACTIONS(743),
    [anon_sym_or] = ACTIONS(743),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [2069] = {
    [sym_heredoc_body] = STATE(2069),
    [ts_builtin_sym_end] = ACTIONS(2628),
    [aux_sym_program_token1] = ACTIONS(2628),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_RBRACE] = ACTIONS(2628),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_DOT] = ACTIONS(2626),
    [anon_sym_end] = ACTIONS(2628),
    [anon_sym_rescue] = ACTIONS(2628),
    [anon_sym_EQ] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_PIPE] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_AMP] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2628),
    [anon_sym_unless] = ACTIONS(2628),
    [anon_sym_while] = ACTIONS(2628),
    [anon_sym_until] = ACTIONS(2628),
    [anon_sym_in] = ACTIONS(2628),
    [anon_sym_do] = ACTIONS(2628),
    [anon_sym_when] = ACTIONS(2628),
    [anon_sym_EQ_GT] = ACTIONS(2628),
    [anon_sym_LPAREN2] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_CARET] = ACTIONS(2626),
    [anon_sym_elsif] = ACTIONS(2628),
    [anon_sym_else] = ACTIONS(2628),
    [anon_sym_then] = ACTIONS(2628),
    [anon_sym_ensure] = ACTIONS(2628),
    [anon_sym_COLON_COLON2] = ACTIONS(2628),
    [anon_sym_AMP_DOT] = ACTIONS(2628),
    [anon_sym_PLUS_EQ] = ACTIONS(2628),
    [anon_sym_DASH_EQ] = ACTIONS(2628),
    [anon_sym_STAR_EQ] = ACTIONS(2628),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2628),
    [anon_sym_SLASH_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_PIPE_EQ] = ACTIONS(2628),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2628),
    [anon_sym_AMP_EQ] = ACTIONS(2628),
    [anon_sym_PERCENT_EQ] = ACTIONS(2628),
    [anon_sym_GT_GT_EQ] = ACTIONS(2628),
    [anon_sym_LT_LT_EQ] = ACTIONS(2628),
    [anon_sym_CARET_EQ] = ACTIONS(2628),
    [anon_sym_QMARK] = ACTIONS(2628),
    [anon_sym_and] = ACTIONS(2628),
    [anon_sym_or] = ACTIONS(2628),
    [anon_sym_PIPE_PIPE] = ACTIONS(2626),
    [anon_sym_AMP_AMP] = ACTIONS(2626),
    [anon_sym_LT_LT] = ACTIONS(2626),
    [anon_sym_GT_GT] = ACTIONS(2626),
    [anon_sym_LT_EQ] = ACTIONS(2626),
    [anon_sym_GT] = ACTIONS(2626),
    [anon_sym_GT_EQ] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2626),
    [anon_sym_SLASH] = ACTIONS(2626),
    [anon_sym_PERCENT] = ACTIONS(2626),
    [anon_sym_EQ_EQ] = ACTIONS(2626),
    [anon_sym_BANG_EQ] = ACTIONS(2628),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2628),
    [anon_sym_LT_EQ_GT] = ACTIONS(2628),
    [anon_sym_EQ_TILDE] = ACTIONS(2628),
    [anon_sym_BANG_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2628),
    [sym__binary_star] = ACTIONS(2628),
    [sym__binary_star_star] = ACTIONS(2628),
    [sym__element_reference_bracket] = ACTIONS(2628),
  },
  [2070] = {
    [sym_heredoc_body] = STATE(2070),
    [ts_builtin_sym_end] = ACTIONS(2636),
    [aux_sym_program_token1] = ACTIONS(2636),
    [anon_sym_LBRACE] = ACTIONS(2636),
    [anon_sym_RBRACE] = ACTIONS(2636),
    [anon_sym_RPAREN] = ACTIONS(2636),
    [anon_sym_DOT] = ACTIONS(2634),
    [anon_sym_end] = ACTIONS(2636),
    [anon_sym_rescue] = ACTIONS(2636),
    [anon_sym_EQ] = ACTIONS(2634),
    [anon_sym_COMMA] = ACTIONS(2636),
    [anon_sym_PIPE] = ACTIONS(2634),
    [anon_sym_SEMI] = ACTIONS(2636),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2636),
    [anon_sym_AMP] = ACTIONS(2634),
    [anon_sym_LT] = ACTIONS(2634),
    [anon_sym_if] = ACTIONS(2636),
    [anon_sym_unless] = ACTIONS(2636),
    [anon_sym_while] = ACTIONS(2636),
    [anon_sym_until] = ACTIONS(2636),
    [anon_sym_in] = ACTIONS(2636),
    [anon_sym_do] = ACTIONS(2636),
    [anon_sym_when] = ACTIONS(2636),
    [anon_sym_EQ_GT] = ACTIONS(2636),
    [anon_sym_LPAREN2] = ACTIONS(2636),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_CARET] = ACTIONS(2634),
    [anon_sym_elsif] = ACTIONS(2636),
    [anon_sym_else] = ACTIONS(2636),
    [anon_sym_then] = ACTIONS(2636),
    [anon_sym_ensure] = ACTIONS(2636),
    [anon_sym_COLON_COLON2] = ACTIONS(2636),
    [anon_sym_AMP_DOT] = ACTIONS(2636),
    [anon_sym_PLUS_EQ] = ACTIONS(2636),
    [anon_sym_DASH_EQ] = ACTIONS(2636),
    [anon_sym_STAR_EQ] = ACTIONS(2636),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2636),
    [anon_sym_SLASH_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_PIPE_EQ] = ACTIONS(2636),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2636),
    [anon_sym_AMP_EQ] = ACTIONS(2636),
    [anon_sym_PERCENT_EQ] = ACTIONS(2636),
    [anon_sym_GT_GT_EQ] = ACTIONS(2636),
    [anon_sym_LT_LT_EQ] = ACTIONS(2636),
    [anon_sym_CARET_EQ] = ACTIONS(2636),
    [anon_sym_QMARK] = ACTIONS(2636),
    [anon_sym_and] = ACTIONS(2636),
    [anon_sym_or] = ACTIONS(2636),
    [anon_sym_PIPE_PIPE] = ACTIONS(2634),
    [anon_sym_AMP_AMP] = ACTIONS(2634),
    [anon_sym_LT_LT] = ACTIONS(2634),
    [anon_sym_GT_GT] = ACTIONS(2634),
    [anon_sym_LT_EQ] = ACTIONS(2634),
    [anon_sym_GT] = ACTIONS(2634),
    [anon_sym_GT_EQ] = ACTIONS(2636),
    [anon_sym_PLUS] = ACTIONS(2634),
    [anon_sym_SLASH] = ACTIONS(2634),
    [anon_sym_PERCENT] = ACTIONS(2634),
    [anon_sym_EQ_EQ] = ACTIONS(2634),
    [anon_sym_BANG_EQ] = ACTIONS(2636),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2636),
    [anon_sym_LT_EQ_GT] = ACTIONS(2636),
    [anon_sym_EQ_TILDE] = ACTIONS(2636),
    [anon_sym_BANG_TILDE] = ACTIONS(2636),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2636),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2636),
    [sym__binary_star] = ACTIONS(2636),
    [sym__binary_star_star] = ACTIONS(2636),
    [sym__element_reference_bracket] = ACTIONS(2636),
  },
  [2071] = {
    [sym_splat_parameter] = STATE(4972),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5909),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5774),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2071),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2952),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2994),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2072] = {
    [sym_heredoc_body] = STATE(2072),
    [ts_builtin_sym_end] = ACTIONS(2620),
    [aux_sym_program_token1] = ACTIONS(2620),
    [anon_sym_LBRACE] = ACTIONS(2620),
    [anon_sym_RBRACE] = ACTIONS(2620),
    [anon_sym_RPAREN] = ACTIONS(2620),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_end] = ACTIONS(2620),
    [anon_sym_rescue] = ACTIONS(2620),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_if] = ACTIONS(2620),
    [anon_sym_unless] = ACTIONS(2620),
    [anon_sym_while] = ACTIONS(2620),
    [anon_sym_until] = ACTIONS(2620),
    [anon_sym_in] = ACTIONS(2620),
    [anon_sym_do] = ACTIONS(2620),
    [anon_sym_when] = ACTIONS(2620),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_elsif] = ACTIONS(2620),
    [anon_sym_else] = ACTIONS(2620),
    [anon_sym_then] = ACTIONS(2620),
    [anon_sym_ensure] = ACTIONS(2620),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2620),
    [anon_sym_and] = ACTIONS(2620),
    [anon_sym_or] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2620),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [2073] = {
    [sym_heredoc_body] = STATE(2073),
    [ts_builtin_sym_end] = ACTIONS(2616),
    [aux_sym_program_token1] = ACTIONS(2616),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_RBRACE] = ACTIONS(2616),
    [anon_sym_RPAREN] = ACTIONS(2616),
    [anon_sym_DOT] = ACTIONS(2614),
    [anon_sym_end] = ACTIONS(2616),
    [anon_sym_rescue] = ACTIONS(2616),
    [anon_sym_EQ] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_PIPE] = ACTIONS(2614),
    [anon_sym_SEMI] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_AMP] = ACTIONS(2614),
    [anon_sym_LT] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2616),
    [anon_sym_unless] = ACTIONS(2616),
    [anon_sym_while] = ACTIONS(2616),
    [anon_sym_until] = ACTIONS(2616),
    [anon_sym_in] = ACTIONS(2616),
    [anon_sym_do] = ACTIONS(2616),
    [anon_sym_when] = ACTIONS(2616),
    [anon_sym_EQ_GT] = ACTIONS(2616),
    [anon_sym_LPAREN2] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_CARET] = ACTIONS(2614),
    [anon_sym_elsif] = ACTIONS(2616),
    [anon_sym_else] = ACTIONS(2616),
    [anon_sym_then] = ACTIONS(2616),
    [anon_sym_ensure] = ACTIONS(2616),
    [anon_sym_COLON_COLON2] = ACTIONS(2616),
    [anon_sym_AMP_DOT] = ACTIONS(2616),
    [anon_sym_PLUS_EQ] = ACTIONS(2616),
    [anon_sym_DASH_EQ] = ACTIONS(2616),
    [anon_sym_STAR_EQ] = ACTIONS(2616),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2616),
    [anon_sym_SLASH_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_PIPE_EQ] = ACTIONS(2616),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2616),
    [anon_sym_AMP_EQ] = ACTIONS(2616),
    [anon_sym_PERCENT_EQ] = ACTIONS(2616),
    [anon_sym_GT_GT_EQ] = ACTIONS(2616),
    [anon_sym_LT_LT_EQ] = ACTIONS(2616),
    [anon_sym_CARET_EQ] = ACTIONS(2616),
    [anon_sym_QMARK] = ACTIONS(2616),
    [anon_sym_and] = ACTIONS(2616),
    [anon_sym_or] = ACTIONS(2616),
    [anon_sym_PIPE_PIPE] = ACTIONS(2614),
    [anon_sym_AMP_AMP] = ACTIONS(2614),
    [anon_sym_LT_LT] = ACTIONS(2614),
    [anon_sym_GT_GT] = ACTIONS(2614),
    [anon_sym_LT_EQ] = ACTIONS(2614),
    [anon_sym_GT] = ACTIONS(2614),
    [anon_sym_GT_EQ] = ACTIONS(2616),
    [anon_sym_PLUS] = ACTIONS(2614),
    [anon_sym_SLASH] = ACTIONS(2614),
    [anon_sym_PERCENT] = ACTIONS(2614),
    [anon_sym_EQ_EQ] = ACTIONS(2614),
    [anon_sym_BANG_EQ] = ACTIONS(2616),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2616),
    [anon_sym_LT_EQ_GT] = ACTIONS(2616),
    [anon_sym_EQ_TILDE] = ACTIONS(2616),
    [anon_sym_BANG_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2616),
    [sym__binary_star] = ACTIONS(2616),
    [sym__binary_star_star] = ACTIONS(2616),
    [sym__element_reference_bracket] = ACTIONS(2616),
  },
  [2074] = {
    [sym_heredoc_body] = STATE(2074),
    [ts_builtin_sym_end] = ACTIONS(2644),
    [aux_sym_program_token1] = ACTIONS(2644),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_RBRACE] = ACTIONS(2644),
    [anon_sym_RPAREN] = ACTIONS(2644),
    [anon_sym_DOT] = ACTIONS(2642),
    [anon_sym_end] = ACTIONS(2644),
    [anon_sym_rescue] = ACTIONS(2644),
    [anon_sym_EQ] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_PIPE] = ACTIONS(2642),
    [anon_sym_SEMI] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_AMP] = ACTIONS(2642),
    [anon_sym_LT] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2644),
    [anon_sym_unless] = ACTIONS(2644),
    [anon_sym_while] = ACTIONS(2644),
    [anon_sym_until] = ACTIONS(2644),
    [anon_sym_in] = ACTIONS(2644),
    [anon_sym_do] = ACTIONS(2644),
    [anon_sym_when] = ACTIONS(2644),
    [anon_sym_EQ_GT] = ACTIONS(2644),
    [anon_sym_LPAREN2] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_CARET] = ACTIONS(2642),
    [anon_sym_elsif] = ACTIONS(2644),
    [anon_sym_else] = ACTIONS(2644),
    [anon_sym_then] = ACTIONS(2644),
    [anon_sym_ensure] = ACTIONS(2644),
    [anon_sym_COLON_COLON2] = ACTIONS(2644),
    [anon_sym_AMP_DOT] = ACTIONS(2644),
    [anon_sym_PLUS_EQ] = ACTIONS(2644),
    [anon_sym_DASH_EQ] = ACTIONS(2644),
    [anon_sym_STAR_EQ] = ACTIONS(2644),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2644),
    [anon_sym_SLASH_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_PIPE_EQ] = ACTIONS(2644),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2644),
    [anon_sym_AMP_EQ] = ACTIONS(2644),
    [anon_sym_PERCENT_EQ] = ACTIONS(2644),
    [anon_sym_GT_GT_EQ] = ACTIONS(2644),
    [anon_sym_LT_LT_EQ] = ACTIONS(2644),
    [anon_sym_CARET_EQ] = ACTIONS(2644),
    [anon_sym_QMARK] = ACTIONS(2644),
    [anon_sym_and] = ACTIONS(2644),
    [anon_sym_or] = ACTIONS(2644),
    [anon_sym_PIPE_PIPE] = ACTIONS(2642),
    [anon_sym_AMP_AMP] = ACTIONS(2642),
    [anon_sym_LT_LT] = ACTIONS(2642),
    [anon_sym_GT_GT] = ACTIONS(2642),
    [anon_sym_LT_EQ] = ACTIONS(2642),
    [anon_sym_GT] = ACTIONS(2642),
    [anon_sym_GT_EQ] = ACTIONS(2644),
    [anon_sym_PLUS] = ACTIONS(2642),
    [anon_sym_SLASH] = ACTIONS(2642),
    [anon_sym_PERCENT] = ACTIONS(2642),
    [anon_sym_EQ_EQ] = ACTIONS(2642),
    [anon_sym_BANG_EQ] = ACTIONS(2644),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2644),
    [anon_sym_LT_EQ_GT] = ACTIONS(2644),
    [anon_sym_EQ_TILDE] = ACTIONS(2644),
    [anon_sym_BANG_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2644),
    [sym__binary_star] = ACTIONS(2644),
    [sym__binary_star_star] = ACTIONS(2644),
    [sym__element_reference_bracket] = ACTIONS(2644),
  },
  [2075] = {
    [sym_heredoc_body] = STATE(2075),
    [ts_builtin_sym_end] = ACTIONS(2640),
    [aux_sym_program_token1] = ACTIONS(2640),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_RBRACE] = ACTIONS(2640),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_DOT] = ACTIONS(2638),
    [anon_sym_end] = ACTIONS(2640),
    [anon_sym_rescue] = ACTIONS(2640),
    [anon_sym_EQ] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_PIPE] = ACTIONS(2638),
    [anon_sym_SEMI] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_AMP] = ACTIONS(2638),
    [anon_sym_LT] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2640),
    [anon_sym_unless] = ACTIONS(2640),
    [anon_sym_while] = ACTIONS(2640),
    [anon_sym_until] = ACTIONS(2640),
    [anon_sym_in] = ACTIONS(2640),
    [anon_sym_do] = ACTIONS(2640),
    [anon_sym_when] = ACTIONS(2640),
    [anon_sym_EQ_GT] = ACTIONS(2640),
    [anon_sym_LPAREN2] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_CARET] = ACTIONS(2638),
    [anon_sym_elsif] = ACTIONS(2640),
    [anon_sym_else] = ACTIONS(2640),
    [anon_sym_then] = ACTIONS(2640),
    [anon_sym_ensure] = ACTIONS(2640),
    [anon_sym_COLON_COLON2] = ACTIONS(2640),
    [anon_sym_AMP_DOT] = ACTIONS(2640),
    [anon_sym_PLUS_EQ] = ACTIONS(2640),
    [anon_sym_DASH_EQ] = ACTIONS(2640),
    [anon_sym_STAR_EQ] = ACTIONS(2640),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2640),
    [anon_sym_SLASH_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_PIPE_EQ] = ACTIONS(2640),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2640),
    [anon_sym_AMP_EQ] = ACTIONS(2640),
    [anon_sym_PERCENT_EQ] = ACTIONS(2640),
    [anon_sym_GT_GT_EQ] = ACTIONS(2640),
    [anon_sym_LT_LT_EQ] = ACTIONS(2640),
    [anon_sym_CARET_EQ] = ACTIONS(2640),
    [anon_sym_QMARK] = ACTIONS(2640),
    [anon_sym_and] = ACTIONS(2640),
    [anon_sym_or] = ACTIONS(2640),
    [anon_sym_PIPE_PIPE] = ACTIONS(2638),
    [anon_sym_AMP_AMP] = ACTIONS(2638),
    [anon_sym_LT_LT] = ACTIONS(2638),
    [anon_sym_GT_GT] = ACTIONS(2638),
    [anon_sym_LT_EQ] = ACTIONS(2638),
    [anon_sym_GT] = ACTIONS(2638),
    [anon_sym_GT_EQ] = ACTIONS(2640),
    [anon_sym_PLUS] = ACTIONS(2638),
    [anon_sym_SLASH] = ACTIONS(2638),
    [anon_sym_PERCENT] = ACTIONS(2638),
    [anon_sym_EQ_EQ] = ACTIONS(2638),
    [anon_sym_BANG_EQ] = ACTIONS(2640),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2640),
    [anon_sym_LT_EQ_GT] = ACTIONS(2640),
    [anon_sym_EQ_TILDE] = ACTIONS(2640),
    [anon_sym_BANG_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2640),
    [sym__binary_star] = ACTIONS(2640),
    [sym__binary_star_star] = ACTIONS(2640),
    [sym__element_reference_bracket] = ACTIONS(2640),
  },
  [2076] = {
    [sym_splat_parameter] = STATE(4972),
    [sym__array_pattern_n] = STATE(5638),
    [sym__pattern_expr] = STATE(5423),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym__array_pattern_body] = STATE(5758),
    [sym_array_pattern] = STATE(4819),
    [sym__find_pattern_body] = STATE(5763),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2076),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2952),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_RBRACK] = ACTIONS(2996),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2077] = {
    [sym_heredoc_body] = STATE(2077),
    [ts_builtin_sym_end] = ACTIONS(743),
    [aux_sym_program_token1] = ACTIONS(743),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_end] = ACTIONS(743),
    [anon_sym_rescue] = ACTIONS(743),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(743),
    [anon_sym_unless] = ACTIONS(743),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_until] = ACTIONS(743),
    [anon_sym_in] = ACTIONS(743),
    [anon_sym_do] = ACTIONS(743),
    [anon_sym_when] = ACTIONS(743),
    [anon_sym_EQ_GT] = ACTIONS(2998),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_elsif] = ACTIONS(743),
    [anon_sym_else] = ACTIONS(743),
    [anon_sym_then] = ACTIONS(743),
    [anon_sym_ensure] = ACTIONS(743),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(743),
    [anon_sym_and] = ACTIONS(743),
    [anon_sym_or] = ACTIONS(743),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [2078] = {
    [sym_heredoc_body] = STATE(2078),
    [ts_builtin_sym_end] = ACTIONS(2624),
    [aux_sym_program_token1] = ACTIONS(2624),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_RBRACE] = ACTIONS(2624),
    [anon_sym_RPAREN] = ACTIONS(2624),
    [anon_sym_DOT] = ACTIONS(2618),
    [anon_sym_end] = ACTIONS(2624),
    [anon_sym_rescue] = ACTIONS(2624),
    [anon_sym_EQ] = ACTIONS(2618),
    [anon_sym_COMMA] = ACTIONS(2624),
    [anon_sym_PIPE] = ACTIONS(2618),
    [anon_sym_SEMI] = ACTIONS(2624),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2624),
    [anon_sym_AMP] = ACTIONS(2618),
    [anon_sym_LT] = ACTIONS(2618),
    [anon_sym_if] = ACTIONS(2624),
    [anon_sym_unless] = ACTIONS(2624),
    [anon_sym_while] = ACTIONS(2624),
    [anon_sym_until] = ACTIONS(2624),
    [anon_sym_in] = ACTIONS(2624),
    [anon_sym_do] = ACTIONS(2624),
    [anon_sym_when] = ACTIONS(2624),
    [anon_sym_EQ_GT] = ACTIONS(2624),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2618),
    [anon_sym_CARET] = ACTIONS(2618),
    [anon_sym_elsif] = ACTIONS(2624),
    [anon_sym_else] = ACTIONS(2624),
    [anon_sym_then] = ACTIONS(2624),
    [anon_sym_ensure] = ACTIONS(2624),
    [anon_sym_COLON_COLON2] = ACTIONS(2624),
    [anon_sym_AMP_DOT] = ACTIONS(2624),
    [anon_sym_PLUS_EQ] = ACTIONS(2624),
    [anon_sym_DASH_EQ] = ACTIONS(2624),
    [anon_sym_STAR_EQ] = ACTIONS(2624),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2624),
    [anon_sym_SLASH_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_PIPE_EQ] = ACTIONS(2624),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2624),
    [anon_sym_AMP_EQ] = ACTIONS(2624),
    [anon_sym_PERCENT_EQ] = ACTIONS(2624),
    [anon_sym_GT_GT_EQ] = ACTIONS(2624),
    [anon_sym_LT_LT_EQ] = ACTIONS(2624),
    [anon_sym_CARET_EQ] = ACTIONS(2624),
    [anon_sym_QMARK] = ACTIONS(2624),
    [anon_sym_and] = ACTIONS(2624),
    [anon_sym_or] = ACTIONS(2624),
    [anon_sym_PIPE_PIPE] = ACTIONS(2618),
    [anon_sym_AMP_AMP] = ACTIONS(2618),
    [anon_sym_LT_LT] = ACTIONS(2618),
    [anon_sym_GT_GT] = ACTIONS(2618),
    [anon_sym_LT_EQ] = ACTIONS(2618),
    [anon_sym_GT] = ACTIONS(2618),
    [anon_sym_GT_EQ] = ACTIONS(2624),
    [anon_sym_PLUS] = ACTIONS(2618),
    [anon_sym_SLASH] = ACTIONS(2618),
    [anon_sym_PERCENT] = ACTIONS(2618),
    [anon_sym_EQ_EQ] = ACTIONS(2618),
    [anon_sym_BANG_EQ] = ACTIONS(2624),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2624),
    [anon_sym_LT_EQ_GT] = ACTIONS(2624),
    [anon_sym_EQ_TILDE] = ACTIONS(2624),
    [anon_sym_BANG_TILDE] = ACTIONS(2624),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2624),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2624),
    [sym__binary_star] = ACTIONS(2624),
    [sym__binary_star_star] = ACTIONS(2624),
    [sym__element_reference_bracket] = ACTIONS(2624),
  },
  [2079] = {
    [sym_heredoc_body] = STATE(2079),
    [ts_builtin_sym_end] = ACTIONS(743),
    [aux_sym_program_token1] = ACTIONS(743),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_end] = ACTIONS(743),
    [anon_sym_rescue] = ACTIONS(743),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(743),
    [anon_sym_unless] = ACTIONS(743),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_until] = ACTIONS(743),
    [anon_sym_in] = ACTIONS(743),
    [anon_sym_do] = ACTIONS(743),
    [anon_sym_when] = ACTIONS(743),
    [anon_sym_EQ_GT] = ACTIONS(743),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_elsif] = ACTIONS(743),
    [anon_sym_else] = ACTIONS(743),
    [anon_sym_then] = ACTIONS(743),
    [anon_sym_ensure] = ACTIONS(743),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(743),
    [anon_sym_and] = ACTIONS(743),
    [anon_sym_or] = ACTIONS(743),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [2080] = {
    [sym_heredoc_body] = STATE(2080),
    [ts_builtin_sym_end] = ACTIONS(2632),
    [aux_sym_program_token1] = ACTIONS(2632),
    [anon_sym_LBRACE] = ACTIONS(2632),
    [anon_sym_RBRACE] = ACTIONS(2632),
    [anon_sym_RPAREN] = ACTIONS(2632),
    [anon_sym_DOT] = ACTIONS(2630),
    [anon_sym_end] = ACTIONS(2632),
    [anon_sym_rescue] = ACTIONS(2632),
    [anon_sym_EQ] = ACTIONS(2630),
    [anon_sym_COMMA] = ACTIONS(2632),
    [anon_sym_PIPE] = ACTIONS(2630),
    [anon_sym_SEMI] = ACTIONS(2632),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2632),
    [anon_sym_AMP] = ACTIONS(2630),
    [anon_sym_LT] = ACTIONS(2630),
    [anon_sym_if] = ACTIONS(2632),
    [anon_sym_unless] = ACTIONS(2632),
    [anon_sym_while] = ACTIONS(2632),
    [anon_sym_until] = ACTIONS(2632),
    [anon_sym_in] = ACTIONS(2632),
    [anon_sym_do] = ACTIONS(2632),
    [anon_sym_when] = ACTIONS(2632),
    [anon_sym_EQ_GT] = ACTIONS(2632),
    [anon_sym_LPAREN2] = ACTIONS(2632),
    [anon_sym_DOT_DOT] = ACTIONS(2630),
    [anon_sym_CARET] = ACTIONS(2630),
    [anon_sym_elsif] = ACTIONS(2632),
    [anon_sym_else] = ACTIONS(2632),
    [anon_sym_then] = ACTIONS(2632),
    [anon_sym_ensure] = ACTIONS(2632),
    [anon_sym_COLON_COLON2] = ACTIONS(2632),
    [anon_sym_AMP_DOT] = ACTIONS(2632),
    [anon_sym_PLUS_EQ] = ACTIONS(2632),
    [anon_sym_DASH_EQ] = ACTIONS(2632),
    [anon_sym_STAR_EQ] = ACTIONS(2632),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2632),
    [anon_sym_SLASH_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_PIPE_EQ] = ACTIONS(2632),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2632),
    [anon_sym_AMP_EQ] = ACTIONS(2632),
    [anon_sym_PERCENT_EQ] = ACTIONS(2632),
    [anon_sym_GT_GT_EQ] = ACTIONS(2632),
    [anon_sym_LT_LT_EQ] = ACTIONS(2632),
    [anon_sym_CARET_EQ] = ACTIONS(2632),
    [anon_sym_QMARK] = ACTIONS(2632),
    [anon_sym_and] = ACTIONS(2632),
    [anon_sym_or] = ACTIONS(2632),
    [anon_sym_PIPE_PIPE] = ACTIONS(2630),
    [anon_sym_AMP_AMP] = ACTIONS(2630),
    [anon_sym_LT_LT] = ACTIONS(2630),
    [anon_sym_GT_GT] = ACTIONS(2630),
    [anon_sym_LT_EQ] = ACTIONS(2630),
    [anon_sym_GT] = ACTIONS(2630),
    [anon_sym_GT_EQ] = ACTIONS(2632),
    [anon_sym_PLUS] = ACTIONS(2630),
    [anon_sym_SLASH] = ACTIONS(2630),
    [anon_sym_PERCENT] = ACTIONS(2630),
    [anon_sym_EQ_EQ] = ACTIONS(2630),
    [anon_sym_BANG_EQ] = ACTIONS(2632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2632),
    [anon_sym_LT_EQ_GT] = ACTIONS(2632),
    [anon_sym_EQ_TILDE] = ACTIONS(2632),
    [anon_sym_BANG_TILDE] = ACTIONS(2632),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2632),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2632),
    [sym__binary_star] = ACTIONS(2632),
    [sym__binary_star_star] = ACTIONS(2632),
    [sym__element_reference_bracket] = ACTIONS(2632),
  },
  [2081] = {
    [sym_heredoc_body] = STATE(2081),
    [ts_builtin_sym_end] = ACTIONS(2620),
    [aux_sym_program_token1] = ACTIONS(2620),
    [anon_sym_LBRACE] = ACTIONS(2624),
    [anon_sym_RBRACE] = ACTIONS(2620),
    [anon_sym_RPAREN] = ACTIONS(2620),
    [anon_sym_DOT] = ACTIONS(2622),
    [anon_sym_end] = ACTIONS(2620),
    [anon_sym_rescue] = ACTIONS(2620),
    [anon_sym_EQ] = ACTIONS(2622),
    [anon_sym_COMMA] = ACTIONS(2620),
    [anon_sym_PIPE] = ACTIONS(2622),
    [anon_sym_SEMI] = ACTIONS(2620),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2620),
    [anon_sym_AMP] = ACTIONS(2622),
    [anon_sym_LT] = ACTIONS(2622),
    [anon_sym_if] = ACTIONS(2620),
    [anon_sym_unless] = ACTIONS(2620),
    [anon_sym_while] = ACTIONS(2620),
    [anon_sym_until] = ACTIONS(2620),
    [anon_sym_in] = ACTIONS(2620),
    [anon_sym_do] = ACTIONS(2624),
    [anon_sym_when] = ACTIONS(2620),
    [anon_sym_EQ_GT] = ACTIONS(2620),
    [anon_sym_LPAREN2] = ACTIONS(2624),
    [anon_sym_DOT_DOT] = ACTIONS(2622),
    [anon_sym_CARET] = ACTIONS(2622),
    [anon_sym_elsif] = ACTIONS(2620),
    [anon_sym_else] = ACTIONS(2620),
    [anon_sym_then] = ACTIONS(2620),
    [anon_sym_ensure] = ACTIONS(2620),
    [anon_sym_COLON_COLON2] = ACTIONS(2620),
    [anon_sym_AMP_DOT] = ACTIONS(2620),
    [anon_sym_PLUS_EQ] = ACTIONS(2620),
    [anon_sym_DASH_EQ] = ACTIONS(2620),
    [anon_sym_STAR_EQ] = ACTIONS(2620),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2620),
    [anon_sym_SLASH_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_PIPE_EQ] = ACTIONS(2620),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2620),
    [anon_sym_AMP_EQ] = ACTIONS(2620),
    [anon_sym_PERCENT_EQ] = ACTIONS(2620),
    [anon_sym_GT_GT_EQ] = ACTIONS(2620),
    [anon_sym_LT_LT_EQ] = ACTIONS(2620),
    [anon_sym_CARET_EQ] = ACTIONS(2620),
    [anon_sym_QMARK] = ACTIONS(2620),
    [anon_sym_and] = ACTIONS(2620),
    [anon_sym_or] = ACTIONS(2620),
    [anon_sym_PIPE_PIPE] = ACTIONS(2622),
    [anon_sym_AMP_AMP] = ACTIONS(2622),
    [anon_sym_LT_LT] = ACTIONS(2622),
    [anon_sym_GT_GT] = ACTIONS(2622),
    [anon_sym_LT_EQ] = ACTIONS(2622),
    [anon_sym_GT] = ACTIONS(2622),
    [anon_sym_GT_EQ] = ACTIONS(2620),
    [anon_sym_PLUS] = ACTIONS(2622),
    [anon_sym_SLASH] = ACTIONS(2622),
    [anon_sym_PERCENT] = ACTIONS(2622),
    [anon_sym_EQ_EQ] = ACTIONS(2622),
    [anon_sym_BANG_EQ] = ACTIONS(2620),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2620),
    [anon_sym_LT_EQ_GT] = ACTIONS(2620),
    [anon_sym_EQ_TILDE] = ACTIONS(2620),
    [anon_sym_BANG_TILDE] = ACTIONS(2620),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2620),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2620),
    [sym__binary_star] = ACTIONS(2620),
    [sym__binary_star_star] = ACTIONS(2620),
    [sym__element_reference_bracket] = ACTIONS(2620),
  },
  [2082] = {
    [sym_heredoc_body] = STATE(2082),
    [ts_builtin_sym_end] = ACTIONS(3000),
    [aux_sym_program_token1] = ACTIONS(3000),
    [anon_sym_LBRACE] = ACTIONS(3000),
    [anon_sym_RBRACE] = ACTIONS(3000),
    [anon_sym_RPAREN] = ACTIONS(3000),
    [anon_sym_DOT] = ACTIONS(3002),
    [anon_sym_end] = ACTIONS(3000),
    [anon_sym_rescue] = ACTIONS(3000),
    [anon_sym_EQ] = ACTIONS(3002),
    [anon_sym_COMMA] = ACTIONS(3000),
    [anon_sym_PIPE] = ACTIONS(3002),
    [anon_sym_SEMI] = ACTIONS(3000),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3000),
    [anon_sym_AMP] = ACTIONS(3002),
    [anon_sym_LT] = ACTIONS(3002),
    [anon_sym_if] = ACTIONS(3000),
    [anon_sym_unless] = ACTIONS(3000),
    [anon_sym_while] = ACTIONS(3000),
    [anon_sym_until] = ACTIONS(3000),
    [anon_sym_in] = ACTIONS(3000),
    [anon_sym_do] = ACTIONS(3000),
    [anon_sym_when] = ACTIONS(3000),
    [anon_sym_EQ_GT] = ACTIONS(3000),
    [anon_sym_DOT_DOT] = ACTIONS(3002),
    [anon_sym_CARET] = ACTIONS(3002),
    [anon_sym_elsif] = ACTIONS(3000),
    [anon_sym_else] = ACTIONS(3000),
    [anon_sym_then] = ACTIONS(3000),
    [anon_sym_ensure] = ACTIONS(3000),
    [anon_sym_COLON_COLON2] = ACTIONS(3000),
    [anon_sym_AMP_DOT] = ACTIONS(3000),
    [anon_sym_PLUS_EQ] = ACTIONS(3000),
    [anon_sym_DASH_EQ] = ACTIONS(3000),
    [anon_sym_STAR_EQ] = ACTIONS(3000),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3000),
    [anon_sym_SLASH_EQ] = ACTIONS(3000),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3000),
    [anon_sym_PIPE_EQ] = ACTIONS(3000),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3000),
    [anon_sym_AMP_EQ] = ACTIONS(3000),
    [anon_sym_PERCENT_EQ] = ACTIONS(3000),
    [anon_sym_GT_GT_EQ] = ACTIONS(3000),
    [anon_sym_LT_LT_EQ] = ACTIONS(3000),
    [anon_sym_CARET_EQ] = ACTIONS(3000),
    [anon_sym_QMARK] = ACTIONS(3000),
    [anon_sym_and] = ACTIONS(3000),
    [anon_sym_or] = ACTIONS(3000),
    [anon_sym_PIPE_PIPE] = ACTIONS(3002),
    [anon_sym_AMP_AMP] = ACTIONS(3002),
    [anon_sym_LT_LT] = ACTIONS(3002),
    [anon_sym_GT_GT] = ACTIONS(3002),
    [anon_sym_LT_EQ] = ACTIONS(3002),
    [anon_sym_GT] = ACTIONS(3002),
    [anon_sym_GT_EQ] = ACTIONS(3000),
    [anon_sym_PLUS] = ACTIONS(3002),
    [anon_sym_SLASH] = ACTIONS(3002),
    [anon_sym_PERCENT] = ACTIONS(3002),
    [anon_sym_EQ_EQ] = ACTIONS(3002),
    [anon_sym_BANG_EQ] = ACTIONS(3000),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3000),
    [anon_sym_LT_EQ_GT] = ACTIONS(3000),
    [anon_sym_EQ_TILDE] = ACTIONS(3000),
    [anon_sym_BANG_TILDE] = ACTIONS(3000),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3000),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3000),
    [sym__binary_star] = ACTIONS(3000),
    [sym__binary_star_star] = ACTIONS(3000),
    [sym__element_reference_bracket] = ACTIONS(3000),
  },
  [2083] = {
    [sym_heredoc_body] = STATE(2083),
    [sym_identifier] = ACTIONS(2642),
    [anon_sym_LBRACE] = ACTIONS(2644),
    [anon_sym_def] = ACTIONS(2642),
    [anon_sym_LPAREN] = ACTIONS(2644),
    [anon_sym_RPAREN] = ACTIONS(2644),
    [anon_sym_COLON_COLON] = ACTIONS(2644),
    [anon_sym_rescue] = ACTIONS(2642),
    [anon_sym_COMMA] = ACTIONS(2644),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2644),
    [anon_sym_STAR] = ACTIONS(2644),
    [anon_sym_nil] = ACTIONS(2642),
    [anon_sym_class] = ACTIONS(2642),
    [anon_sym_module] = ACTIONS(2642),
    [anon_sym_return] = ACTIONS(2642),
    [anon_sym_yield] = ACTIONS(2642),
    [anon_sym_break] = ACTIONS(2642),
    [anon_sym_next] = ACTIONS(2642),
    [anon_sym_redo] = ACTIONS(2642),
    [anon_sym_retry] = ACTIONS(2642),
    [anon_sym_if] = ACTIONS(2642),
    [anon_sym_unless] = ACTIONS(2642),
    [anon_sym_while] = ACTIONS(2642),
    [anon_sym_until] = ACTIONS(2642),
    [anon_sym_for] = ACTIONS(2642),
    [anon_sym_do] = ACTIONS(2642),
    [anon_sym_case] = ACTIONS(2642),
    [anon_sym_LBRACK] = ACTIONS(2644),
    [anon_sym_RBRACK] = ACTIONS(2644),
    [anon_sym_DOT_DOT] = ACTIONS(2642),
    [anon_sym_begin] = ACTIONS(2642),
    [anon_sym_and] = ACTIONS(2642),
    [anon_sym_or] = ACTIONS(2642),
    [anon_sym_PLUS] = ACTIONS(2644),
    [anon_sym_defined_QMARK] = ACTIONS(2644),
    [anon_sym_not] = ACTIONS(2642),
    [anon_sym_BANG] = ACTIONS(2644),
    [anon_sym_TILDE] = ACTIONS(2644),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2642),
    [sym_float] = ACTIONS(2642),
    [sym_super] = ACTIONS(2642),
    [sym_self] = ACTIONS(2642),
    [sym_true] = ACTIONS(2642),
    [sym_false] = ACTIONS(2642),
    [sym_constant] = ACTIONS(2642),
    [aux_sym_constant_suffix_token1] = ACTIONS(2644),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2642),
    [sym_instance_variable] = ACTIONS(2644),
    [sym_class_variable] = ACTIONS(2644),
    [sym_global_variable] = ACTIONS(2644),
    [sym_character] = ACTIONS(2644),
    [anon_sym_DASH_GT] = ACTIONS(2644),
    [sym_simple_symbol] = ACTIONS(2644),
    [sym__string_start] = ACTIONS(2644),
    [sym__symbol_start] = ACTIONS(2644),
    [sym__subshell_start] = ACTIONS(2644),
    [sym__regex_start] = ACTIONS(2644),
    [sym__string_array_start] = ACTIONS(2644),
    [sym__symbol_array_start] = ACTIONS(2644),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2644),
    [sym__block_ampersand] = ACTIONS(2644),
    [sym__splat_star] = ACTIONS(2644),
    [sym__unary_minus] = ACTIONS(2644),
    [sym__unary_minus_num] = ACTIONS(2644),
    [sym__binary_minus] = ACTIONS(2644),
    [sym_hash_key_symbol] = ACTIONS(2644),
    [sym__identifier_suffix] = ACTIONS(2644),
    [sym__constant_suffix] = ACTIONS(2644),
    [sym__hash_splat_star_star] = ACTIONS(2644),
  },
  [2084] = {
    [sym_heredoc_body] = STATE(2084),
    [sym_identifier] = ACTIONS(3004),
    [anon_sym_BEGIN] = ACTIONS(3004),
    [anon_sym_LBRACE] = ACTIONS(3006),
    [anon_sym_END] = ACTIONS(3004),
    [anon_sym_def] = ACTIONS(3004),
    [anon_sym_LPAREN] = ACTIONS(3006),
    [anon_sym_COLON_COLON] = ACTIONS(3006),
    [anon_sym_end] = ACTIONS(3004),
    [anon_sym_rescue] = ACTIONS(3004),
    [anon_sym_EQ] = ACTIONS(3006),
    [anon_sym_SEMI] = ACTIONS(3006),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3006),
    [anon_sym_STAR] = ACTIONS(3006),
    [anon_sym_nil] = ACTIONS(3004),
    [anon_sym_class] = ACTIONS(3004),
    [anon_sym_module] = ACTIONS(3004),
    [anon_sym_return] = ACTIONS(3004),
    [anon_sym_yield] = ACTIONS(3004),
    [anon_sym_break] = ACTIONS(3004),
    [anon_sym_next] = ACTIONS(3004),
    [anon_sym_redo] = ACTIONS(3004),
    [anon_sym_retry] = ACTIONS(3004),
    [anon_sym_if] = ACTIONS(3004),
    [anon_sym_unless] = ACTIONS(3004),
    [anon_sym_while] = ACTIONS(3004),
    [anon_sym_until] = ACTIONS(3004),
    [anon_sym_for] = ACTIONS(3004),
    [anon_sym_case] = ACTIONS(3004),
    [anon_sym_LBRACK] = ACTIONS(3006),
    [anon_sym_DOT_DOT] = ACTIONS(3004),
    [anon_sym_else] = ACTIONS(3004),
    [anon_sym_begin] = ACTIONS(3004),
    [anon_sym_ensure] = ACTIONS(3004),
    [anon_sym_PLUS] = ACTIONS(3006),
    [anon_sym_defined_QMARK] = ACTIONS(3006),
    [anon_sym_not] = ACTIONS(3004),
    [anon_sym_BANG] = ACTIONS(3006),
    [anon_sym_TILDE] = ACTIONS(3006),
    [anon_sym_undef] = ACTIONS(3004),
    [anon_sym_alias] = ACTIONS(3004),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(3004),
    [sym_float] = ACTIONS(3004),
    [sym_super] = ACTIONS(3004),
    [sym_self] = ACTIONS(3004),
    [sym_true] = ACTIONS(3004),
    [sym_false] = ACTIONS(3004),
    [sym_constant] = ACTIONS(3004),
    [aux_sym_constant_suffix_token1] = ACTIONS(3006),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3004),
    [sym_instance_variable] = ACTIONS(3006),
    [sym_class_variable] = ACTIONS(3006),
    [sym_global_variable] = ACTIONS(3006),
    [sym_character] = ACTIONS(3006),
    [anon_sym_DASH_GT] = ACTIONS(3006),
    [sym__line_break] = ACTIONS(3006),
    [sym_simple_symbol] = ACTIONS(3006),
    [sym__string_start] = ACTIONS(3006),
    [sym__symbol_start] = ACTIONS(3006),
    [sym__subshell_start] = ACTIONS(3006),
    [sym__regex_start] = ACTIONS(3006),
    [sym__string_array_start] = ACTIONS(3006),
    [sym__symbol_array_start] = ACTIONS(3006),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3006),
    [sym__unary_minus] = ACTIONS(3006),
    [sym__unary_minus_num] = ACTIONS(3006),
    [sym__binary_minus] = ACTIONS(3006),
    [sym__identifier_suffix] = ACTIONS(3006),
    [sym__constant_suffix] = ACTIONS(3006),
  },
  [2085] = {
    [sym_heredoc_body] = STATE(2085),
    [ts_builtin_sym_end] = ACTIONS(3008),
    [aux_sym_program_token1] = ACTIONS(3008),
    [anon_sym_LBRACE] = ACTIONS(3008),
    [anon_sym_RBRACE] = ACTIONS(3008),
    [anon_sym_RPAREN] = ACTIONS(3008),
    [anon_sym_DOT] = ACTIONS(3010),
    [anon_sym_end] = ACTIONS(3008),
    [anon_sym_rescue] = ACTIONS(3008),
    [anon_sym_EQ] = ACTIONS(3010),
    [anon_sym_COMMA] = ACTIONS(3008),
    [anon_sym_PIPE] = ACTIONS(3010),
    [anon_sym_SEMI] = ACTIONS(3008),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3008),
    [anon_sym_AMP] = ACTIONS(3010),
    [anon_sym_LT] = ACTIONS(3010),
    [anon_sym_if] = ACTIONS(3008),
    [anon_sym_unless] = ACTIONS(3008),
    [anon_sym_while] = ACTIONS(3008),
    [anon_sym_until] = ACTIONS(3008),
    [anon_sym_in] = ACTIONS(3008),
    [anon_sym_do] = ACTIONS(3008),
    [anon_sym_when] = ACTIONS(3008),
    [anon_sym_EQ_GT] = ACTIONS(3008),
    [anon_sym_DOT_DOT] = ACTIONS(3010),
    [anon_sym_CARET] = ACTIONS(3010),
    [anon_sym_elsif] = ACTIONS(3008),
    [anon_sym_else] = ACTIONS(3008),
    [anon_sym_then] = ACTIONS(3008),
    [anon_sym_ensure] = ACTIONS(3008),
    [anon_sym_COLON_COLON2] = ACTIONS(3008),
    [anon_sym_AMP_DOT] = ACTIONS(3008),
    [anon_sym_PLUS_EQ] = ACTIONS(3008),
    [anon_sym_DASH_EQ] = ACTIONS(3008),
    [anon_sym_STAR_EQ] = ACTIONS(3008),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3008),
    [anon_sym_SLASH_EQ] = ACTIONS(3008),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3008),
    [anon_sym_PIPE_EQ] = ACTIONS(3008),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3008),
    [anon_sym_AMP_EQ] = ACTIONS(3008),
    [anon_sym_PERCENT_EQ] = ACTIONS(3008),
    [anon_sym_GT_GT_EQ] = ACTIONS(3008),
    [anon_sym_LT_LT_EQ] = ACTIONS(3008),
    [anon_sym_CARET_EQ] = ACTIONS(3008),
    [anon_sym_QMARK] = ACTIONS(3008),
    [anon_sym_and] = ACTIONS(3008),
    [anon_sym_or] = ACTIONS(3008),
    [anon_sym_PIPE_PIPE] = ACTIONS(3010),
    [anon_sym_AMP_AMP] = ACTIONS(3010),
    [anon_sym_LT_LT] = ACTIONS(3010),
    [anon_sym_GT_GT] = ACTIONS(3010),
    [anon_sym_LT_EQ] = ACTIONS(3010),
    [anon_sym_GT] = ACTIONS(3010),
    [anon_sym_GT_EQ] = ACTIONS(3008),
    [anon_sym_PLUS] = ACTIONS(3010),
    [anon_sym_SLASH] = ACTIONS(3010),
    [anon_sym_PERCENT] = ACTIONS(3010),
    [anon_sym_EQ_EQ] = ACTIONS(3010),
    [anon_sym_BANG_EQ] = ACTIONS(3008),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3008),
    [anon_sym_LT_EQ_GT] = ACTIONS(3008),
    [anon_sym_EQ_TILDE] = ACTIONS(3008),
    [anon_sym_BANG_TILDE] = ACTIONS(3008),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3008),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3008),
    [sym__binary_star] = ACTIONS(3008),
    [sym__binary_star_star] = ACTIONS(3008),
    [sym__element_reference_bracket] = ACTIONS(3008),
  },
  [2086] = {
    [sym_heredoc_body] = STATE(2086),
    [sym_identifier] = ACTIONS(2626),
    [anon_sym_LBRACE] = ACTIONS(2628),
    [anon_sym_def] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2628),
    [anon_sym_RPAREN] = ACTIONS(2628),
    [anon_sym_COLON_COLON] = ACTIONS(2628),
    [anon_sym_rescue] = ACTIONS(2626),
    [anon_sym_COMMA] = ACTIONS(2628),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2628),
    [anon_sym_STAR] = ACTIONS(2628),
    [anon_sym_nil] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2626),
    [anon_sym_module] = ACTIONS(2626),
    [anon_sym_return] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2626),
    [anon_sym_break] = ACTIONS(2626),
    [anon_sym_next] = ACTIONS(2626),
    [anon_sym_redo] = ACTIONS(2626),
    [anon_sym_retry] = ACTIONS(2626),
    [anon_sym_if] = ACTIONS(2626),
    [anon_sym_unless] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [anon_sym_until] = ACTIONS(2626),
    [anon_sym_for] = ACTIONS(2626),
    [anon_sym_do] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2626),
    [anon_sym_LBRACK] = ACTIONS(2628),
    [anon_sym_RBRACK] = ACTIONS(2628),
    [anon_sym_DOT_DOT] = ACTIONS(2626),
    [anon_sym_begin] = ACTIONS(2626),
    [anon_sym_and] = ACTIONS(2626),
    [anon_sym_or] = ACTIONS(2626),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_defined_QMARK] = ACTIONS(2628),
    [anon_sym_not] = ACTIONS(2626),
    [anon_sym_BANG] = ACTIONS(2628),
    [anon_sym_TILDE] = ACTIONS(2628),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2626),
    [sym_float] = ACTIONS(2626),
    [sym_super] = ACTIONS(2626),
    [sym_self] = ACTIONS(2626),
    [sym_true] = ACTIONS(2626),
    [sym_false] = ACTIONS(2626),
    [sym_constant] = ACTIONS(2626),
    [aux_sym_constant_suffix_token1] = ACTIONS(2628),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2626),
    [sym_instance_variable] = ACTIONS(2628),
    [sym_class_variable] = ACTIONS(2628),
    [sym_global_variable] = ACTIONS(2628),
    [sym_character] = ACTIONS(2628),
    [anon_sym_DASH_GT] = ACTIONS(2628),
    [sym_simple_symbol] = ACTIONS(2628),
    [sym__string_start] = ACTIONS(2628),
    [sym__symbol_start] = ACTIONS(2628),
    [sym__subshell_start] = ACTIONS(2628),
    [sym__regex_start] = ACTIONS(2628),
    [sym__string_array_start] = ACTIONS(2628),
    [sym__symbol_array_start] = ACTIONS(2628),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2628),
    [sym__block_ampersand] = ACTIONS(2628),
    [sym__splat_star] = ACTIONS(2628),
    [sym__unary_minus] = ACTIONS(2628),
    [sym__unary_minus_num] = ACTIONS(2628),
    [sym__binary_minus] = ACTIONS(2628),
    [sym_hash_key_symbol] = ACTIONS(2628),
    [sym__identifier_suffix] = ACTIONS(2628),
    [sym__constant_suffix] = ACTIONS(2628),
    [sym__hash_splat_star_star] = ACTIONS(2628),
  },
  [2087] = {
    [sym_heredoc_body] = STATE(2087),
    [sym_identifier] = ACTIONS(2946),
    [anon_sym_LBRACE] = ACTIONS(2948),
    [anon_sym_def] = ACTIONS(2946),
    [anon_sym_LPAREN] = ACTIONS(2948),
    [anon_sym_RPAREN] = ACTIONS(2948),
    [anon_sym_COLON_COLON] = ACTIONS(2948),
    [anon_sym_rescue] = ACTIONS(2946),
    [anon_sym_COMMA] = ACTIONS(2948),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2948),
    [anon_sym_STAR] = ACTIONS(2948),
    [anon_sym_nil] = ACTIONS(2946),
    [anon_sym_class] = ACTIONS(2946),
    [anon_sym_module] = ACTIONS(2946),
    [anon_sym_return] = ACTIONS(2946),
    [anon_sym_yield] = ACTIONS(2946),
    [anon_sym_break] = ACTIONS(2946),
    [anon_sym_next] = ACTIONS(2946),
    [anon_sym_redo] = ACTIONS(2946),
    [anon_sym_retry] = ACTIONS(2946),
    [anon_sym_if] = ACTIONS(2946),
    [anon_sym_unless] = ACTIONS(2946),
    [anon_sym_while] = ACTIONS(2946),
    [anon_sym_until] = ACTIONS(2946),
    [anon_sym_for] = ACTIONS(2946),
    [anon_sym_do] = ACTIONS(2946),
    [anon_sym_case] = ACTIONS(2946),
    [anon_sym_LBRACK] = ACTIONS(2948),
    [anon_sym_RBRACK] = ACTIONS(2948),
    [anon_sym_DOT_DOT] = ACTIONS(2946),
    [anon_sym_begin] = ACTIONS(2946),
    [anon_sym_and] = ACTIONS(2946),
    [anon_sym_or] = ACTIONS(2946),
    [anon_sym_PLUS] = ACTIONS(2948),
    [anon_sym_defined_QMARK] = ACTIONS(2948),
    [anon_sym_not] = ACTIONS(2946),
    [anon_sym_BANG] = ACTIONS(2948),
    [anon_sym_TILDE] = ACTIONS(2948),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2946),
    [sym_float] = ACTIONS(2946),
    [sym_super] = ACTIONS(2946),
    [sym_self] = ACTIONS(2946),
    [sym_true] = ACTIONS(2946),
    [sym_false] = ACTIONS(2946),
    [sym_constant] = ACTIONS(2946),
    [aux_sym_constant_suffix_token1] = ACTIONS(2948),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2946),
    [sym_instance_variable] = ACTIONS(2948),
    [sym_class_variable] = ACTIONS(2948),
    [sym_global_variable] = ACTIONS(2948),
    [sym_character] = ACTIONS(2948),
    [anon_sym_DASH_GT] = ACTIONS(2948),
    [sym_simple_symbol] = ACTIONS(2948),
    [sym__string_start] = ACTIONS(2948),
    [sym__symbol_start] = ACTIONS(2948),
    [sym__subshell_start] = ACTIONS(2948),
    [sym__regex_start] = ACTIONS(2948),
    [sym__string_array_start] = ACTIONS(2948),
    [sym__symbol_array_start] = ACTIONS(2948),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2948),
    [sym__block_ampersand] = ACTIONS(2948),
    [sym__splat_star] = ACTIONS(2948),
    [sym__unary_minus] = ACTIONS(2948),
    [sym__unary_minus_num] = ACTIONS(2948),
    [sym__binary_minus] = ACTIONS(2948),
    [sym_hash_key_symbol] = ACTIONS(2948),
    [sym__identifier_suffix] = ACTIONS(2948),
    [sym__constant_suffix] = ACTIONS(2948),
    [sym__hash_splat_star_star] = ACTIONS(2948),
  },
  [2088] = {
    [sym_heredoc_body] = STATE(2088),
    [ts_builtin_sym_end] = ACTIONS(3012),
    [aux_sym_program_token1] = ACTIONS(3012),
    [anon_sym_LBRACE] = ACTIONS(3012),
    [anon_sym_RBRACE] = ACTIONS(3012),
    [anon_sym_RPAREN] = ACTIONS(3012),
    [anon_sym_DOT] = ACTIONS(3014),
    [anon_sym_end] = ACTIONS(3012),
    [anon_sym_rescue] = ACTIONS(3012),
    [anon_sym_EQ] = ACTIONS(3014),
    [anon_sym_COMMA] = ACTIONS(3012),
    [anon_sym_PIPE] = ACTIONS(3014),
    [anon_sym_SEMI] = ACTIONS(3012),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3012),
    [anon_sym_AMP] = ACTIONS(3014),
    [anon_sym_LT] = ACTIONS(3014),
    [anon_sym_if] = ACTIONS(3012),
    [anon_sym_unless] = ACTIONS(3012),
    [anon_sym_while] = ACTIONS(3012),
    [anon_sym_until] = ACTIONS(3012),
    [anon_sym_in] = ACTIONS(3012),
    [anon_sym_do] = ACTIONS(3012),
    [anon_sym_when] = ACTIONS(3012),
    [anon_sym_EQ_GT] = ACTIONS(3012),
    [anon_sym_DOT_DOT] = ACTIONS(3014),
    [anon_sym_CARET] = ACTIONS(3014),
    [anon_sym_elsif] = ACTIONS(3012),
    [anon_sym_else] = ACTIONS(3012),
    [anon_sym_then] = ACTIONS(3012),
    [anon_sym_ensure] = ACTIONS(3012),
    [anon_sym_COLON_COLON2] = ACTIONS(3012),
    [anon_sym_AMP_DOT] = ACTIONS(3012),
    [anon_sym_PLUS_EQ] = ACTIONS(3012),
    [anon_sym_DASH_EQ] = ACTIONS(3012),
    [anon_sym_STAR_EQ] = ACTIONS(3012),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3012),
    [anon_sym_SLASH_EQ] = ACTIONS(3012),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3012),
    [anon_sym_PIPE_EQ] = ACTIONS(3012),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3012),
    [anon_sym_AMP_EQ] = ACTIONS(3012),
    [anon_sym_PERCENT_EQ] = ACTIONS(3012),
    [anon_sym_GT_GT_EQ] = ACTIONS(3012),
    [anon_sym_LT_LT_EQ] = ACTIONS(3012),
    [anon_sym_CARET_EQ] = ACTIONS(3012),
    [anon_sym_QMARK] = ACTIONS(3012),
    [anon_sym_and] = ACTIONS(3012),
    [anon_sym_or] = ACTIONS(3012),
    [anon_sym_PIPE_PIPE] = ACTIONS(3014),
    [anon_sym_AMP_AMP] = ACTIONS(3014),
    [anon_sym_LT_LT] = ACTIONS(3014),
    [anon_sym_GT_GT] = ACTIONS(3014),
    [anon_sym_LT_EQ] = ACTIONS(3014),
    [anon_sym_GT] = ACTIONS(3014),
    [anon_sym_GT_EQ] = ACTIONS(3012),
    [anon_sym_PLUS] = ACTIONS(3014),
    [anon_sym_SLASH] = ACTIONS(3014),
    [anon_sym_PERCENT] = ACTIONS(3014),
    [anon_sym_EQ_EQ] = ACTIONS(3014),
    [anon_sym_BANG_EQ] = ACTIONS(3012),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3012),
    [anon_sym_LT_EQ_GT] = ACTIONS(3012),
    [anon_sym_EQ_TILDE] = ACTIONS(3012),
    [anon_sym_BANG_TILDE] = ACTIONS(3012),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3012),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3012),
    [sym__binary_star] = ACTIONS(3012),
    [sym__binary_star_star] = ACTIONS(3012),
    [sym__element_reference_bracket] = ACTIONS(3012),
  },
  [2089] = {
    [sym_heredoc_body] = STATE(2089),
    [sym_identifier] = ACTIONS(2614),
    [anon_sym_LBRACE] = ACTIONS(2616),
    [anon_sym_def] = ACTIONS(2614),
    [anon_sym_LPAREN] = ACTIONS(2616),
    [anon_sym_RPAREN] = ACTIONS(2616),
    [anon_sym_COLON_COLON] = ACTIONS(2616),
    [anon_sym_rescue] = ACTIONS(2614),
    [anon_sym_COMMA] = ACTIONS(2616),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2616),
    [anon_sym_STAR] = ACTIONS(2616),
    [anon_sym_nil] = ACTIONS(2614),
    [anon_sym_class] = ACTIONS(2614),
    [anon_sym_module] = ACTIONS(2614),
    [anon_sym_return] = ACTIONS(2614),
    [anon_sym_yield] = ACTIONS(2614),
    [anon_sym_break] = ACTIONS(2614),
    [anon_sym_next] = ACTIONS(2614),
    [anon_sym_redo] = ACTIONS(2614),
    [anon_sym_retry] = ACTIONS(2614),
    [anon_sym_if] = ACTIONS(2614),
    [anon_sym_unless] = ACTIONS(2614),
    [anon_sym_while] = ACTIONS(2614),
    [anon_sym_until] = ACTIONS(2614),
    [anon_sym_for] = ACTIONS(2614),
    [anon_sym_do] = ACTIONS(2614),
    [anon_sym_case] = ACTIONS(2614),
    [anon_sym_LBRACK] = ACTIONS(2616),
    [anon_sym_RBRACK] = ACTIONS(2616),
    [anon_sym_DOT_DOT] = ACTIONS(2614),
    [anon_sym_begin] = ACTIONS(2614),
    [anon_sym_and] = ACTIONS(2614),
    [anon_sym_or] = ACTIONS(2614),
    [anon_sym_PLUS] = ACTIONS(2616),
    [anon_sym_defined_QMARK] = ACTIONS(2616),
    [anon_sym_not] = ACTIONS(2614),
    [anon_sym_BANG] = ACTIONS(2616),
    [anon_sym_TILDE] = ACTIONS(2616),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2614),
    [sym_float] = ACTIONS(2614),
    [sym_super] = ACTIONS(2614),
    [sym_self] = ACTIONS(2614),
    [sym_true] = ACTIONS(2614),
    [sym_false] = ACTIONS(2614),
    [sym_constant] = ACTIONS(2614),
    [aux_sym_constant_suffix_token1] = ACTIONS(2616),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2614),
    [sym_instance_variable] = ACTIONS(2616),
    [sym_class_variable] = ACTIONS(2616),
    [sym_global_variable] = ACTIONS(2616),
    [sym_character] = ACTIONS(2616),
    [anon_sym_DASH_GT] = ACTIONS(2616),
    [sym_simple_symbol] = ACTIONS(2616),
    [sym__string_start] = ACTIONS(2616),
    [sym__symbol_start] = ACTIONS(2616),
    [sym__subshell_start] = ACTIONS(2616),
    [sym__regex_start] = ACTIONS(2616),
    [sym__string_array_start] = ACTIONS(2616),
    [sym__symbol_array_start] = ACTIONS(2616),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2616),
    [sym__block_ampersand] = ACTIONS(2616),
    [sym__splat_star] = ACTIONS(2616),
    [sym__unary_minus] = ACTIONS(2616),
    [sym__unary_minus_num] = ACTIONS(2616),
    [sym__binary_minus] = ACTIONS(2616),
    [sym_hash_key_symbol] = ACTIONS(2616),
    [sym__identifier_suffix] = ACTIONS(2616),
    [sym__constant_suffix] = ACTIONS(2616),
    [sym__hash_splat_star_star] = ACTIONS(2616),
  },
  [2090] = {
    [sym_heredoc_body] = STATE(2090),
    [ts_builtin_sym_end] = ACTIONS(3016),
    [aux_sym_program_token1] = ACTIONS(3016),
    [anon_sym_LBRACE] = ACTIONS(3016),
    [anon_sym_RBRACE] = ACTIONS(3016),
    [anon_sym_RPAREN] = ACTIONS(3016),
    [anon_sym_DOT] = ACTIONS(3018),
    [anon_sym_end] = ACTIONS(3016),
    [anon_sym_rescue] = ACTIONS(3016),
    [anon_sym_EQ] = ACTIONS(3018),
    [anon_sym_COMMA] = ACTIONS(3016),
    [anon_sym_PIPE] = ACTIONS(3018),
    [anon_sym_SEMI] = ACTIONS(3016),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3016),
    [anon_sym_AMP] = ACTIONS(3018),
    [anon_sym_LT] = ACTIONS(3018),
    [anon_sym_if] = ACTIONS(3016),
    [anon_sym_unless] = ACTIONS(3016),
    [anon_sym_while] = ACTIONS(3016),
    [anon_sym_until] = ACTIONS(3016),
    [anon_sym_in] = ACTIONS(3016),
    [anon_sym_do] = ACTIONS(3016),
    [anon_sym_when] = ACTIONS(3016),
    [anon_sym_EQ_GT] = ACTIONS(3016),
    [anon_sym_DOT_DOT] = ACTIONS(3018),
    [anon_sym_CARET] = ACTIONS(3018),
    [anon_sym_elsif] = ACTIONS(3016),
    [anon_sym_else] = ACTIONS(3016),
    [anon_sym_then] = ACTIONS(3016),
    [anon_sym_ensure] = ACTIONS(3016),
    [anon_sym_COLON_COLON2] = ACTIONS(3016),
    [anon_sym_AMP_DOT] = ACTIONS(3016),
    [anon_sym_PLUS_EQ] = ACTIONS(3016),
    [anon_sym_DASH_EQ] = ACTIONS(3016),
    [anon_sym_STAR_EQ] = ACTIONS(3016),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3016),
    [anon_sym_SLASH_EQ] = ACTIONS(3016),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3016),
    [anon_sym_PIPE_EQ] = ACTIONS(3016),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3016),
    [anon_sym_AMP_EQ] = ACTIONS(3016),
    [anon_sym_PERCENT_EQ] = ACTIONS(3016),
    [anon_sym_GT_GT_EQ] = ACTIONS(3016),
    [anon_sym_LT_LT_EQ] = ACTIONS(3016),
    [anon_sym_CARET_EQ] = ACTIONS(3016),
    [anon_sym_QMARK] = ACTIONS(3016),
    [anon_sym_and] = ACTIONS(3016),
    [anon_sym_or] = ACTIONS(3016),
    [anon_sym_PIPE_PIPE] = ACTIONS(3018),
    [anon_sym_AMP_AMP] = ACTIONS(3018),
    [anon_sym_LT_LT] = ACTIONS(3018),
    [anon_sym_GT_GT] = ACTIONS(3018),
    [anon_sym_LT_EQ] = ACTIONS(3018),
    [anon_sym_GT] = ACTIONS(3018),
    [anon_sym_GT_EQ] = ACTIONS(3016),
    [anon_sym_PLUS] = ACTIONS(3018),
    [anon_sym_SLASH] = ACTIONS(3018),
    [anon_sym_PERCENT] = ACTIONS(3018),
    [anon_sym_EQ_EQ] = ACTIONS(3018),
    [anon_sym_BANG_EQ] = ACTIONS(3016),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3016),
    [anon_sym_LT_EQ_GT] = ACTIONS(3016),
    [anon_sym_EQ_TILDE] = ACTIONS(3016),
    [anon_sym_BANG_TILDE] = ACTIONS(3016),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3016),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3016),
    [sym__binary_star] = ACTIONS(3016),
    [sym__binary_star_star] = ACTIONS(3016),
    [sym__element_reference_bracket] = ACTIONS(3016),
  },
  [2091] = {
    [sym_heredoc_body] = STATE(2091),
    [ts_builtin_sym_end] = ACTIONS(3020),
    [aux_sym_program_token1] = ACTIONS(3020),
    [anon_sym_LBRACE] = ACTIONS(3020),
    [anon_sym_RBRACE] = ACTIONS(3020),
    [anon_sym_RPAREN] = ACTIONS(3020),
    [anon_sym_DOT] = ACTIONS(3022),
    [anon_sym_end] = ACTIONS(3020),
    [anon_sym_rescue] = ACTIONS(3020),
    [anon_sym_EQ] = ACTIONS(3022),
    [anon_sym_COMMA] = ACTIONS(3020),
    [anon_sym_PIPE] = ACTIONS(3022),
    [anon_sym_SEMI] = ACTIONS(3020),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3020),
    [anon_sym_AMP] = ACTIONS(3022),
    [anon_sym_LT] = ACTIONS(3022),
    [anon_sym_if] = ACTIONS(3020),
    [anon_sym_unless] = ACTIONS(3020),
    [anon_sym_while] = ACTIONS(3020),
    [anon_sym_until] = ACTIONS(3020),
    [anon_sym_in] = ACTIONS(3020),
    [anon_sym_do] = ACTIONS(3020),
    [anon_sym_when] = ACTIONS(3020),
    [anon_sym_EQ_GT] = ACTIONS(3020),
    [anon_sym_DOT_DOT] = ACTIONS(3022),
    [anon_sym_CARET] = ACTIONS(3022),
    [anon_sym_elsif] = ACTIONS(3020),
    [anon_sym_else] = ACTIONS(3020),
    [anon_sym_then] = ACTIONS(3020),
    [anon_sym_ensure] = ACTIONS(3020),
    [anon_sym_COLON_COLON2] = ACTIONS(3020),
    [anon_sym_AMP_DOT] = ACTIONS(3020),
    [anon_sym_PLUS_EQ] = ACTIONS(3020),
    [anon_sym_DASH_EQ] = ACTIONS(3020),
    [anon_sym_STAR_EQ] = ACTIONS(3020),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3020),
    [anon_sym_SLASH_EQ] = ACTIONS(3020),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3020),
    [anon_sym_PIPE_EQ] = ACTIONS(3020),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3020),
    [anon_sym_AMP_EQ] = ACTIONS(3020),
    [anon_sym_PERCENT_EQ] = ACTIONS(3020),
    [anon_sym_GT_GT_EQ] = ACTIONS(3020),
    [anon_sym_LT_LT_EQ] = ACTIONS(3020),
    [anon_sym_CARET_EQ] = ACTIONS(3020),
    [anon_sym_QMARK] = ACTIONS(3020),
    [anon_sym_and] = ACTIONS(3020),
    [anon_sym_or] = ACTIONS(3020),
    [anon_sym_PIPE_PIPE] = ACTIONS(3022),
    [anon_sym_AMP_AMP] = ACTIONS(3022),
    [anon_sym_LT_LT] = ACTIONS(3022),
    [anon_sym_GT_GT] = ACTIONS(3022),
    [anon_sym_LT_EQ] = ACTIONS(3022),
    [anon_sym_GT] = ACTIONS(3022),
    [anon_sym_GT_EQ] = ACTIONS(3020),
    [anon_sym_PLUS] = ACTIONS(3022),
    [anon_sym_SLASH] = ACTIONS(3022),
    [anon_sym_PERCENT] = ACTIONS(3022),
    [anon_sym_EQ_EQ] = ACTIONS(3022),
    [anon_sym_BANG_EQ] = ACTIONS(3020),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3020),
    [anon_sym_LT_EQ_GT] = ACTIONS(3020),
    [anon_sym_EQ_TILDE] = ACTIONS(3020),
    [anon_sym_BANG_TILDE] = ACTIONS(3020),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3020),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3020),
    [sym__binary_star] = ACTIONS(3020),
    [sym__binary_star_star] = ACTIONS(3020),
    [sym__element_reference_bracket] = ACTIONS(3020),
  },
  [2092] = {
    [sym_heredoc_body] = STATE(2092),
    [ts_builtin_sym_end] = ACTIONS(3024),
    [aux_sym_program_token1] = ACTIONS(3024),
    [anon_sym_LBRACE] = ACTIONS(3024),
    [anon_sym_RBRACE] = ACTIONS(3024),
    [anon_sym_RPAREN] = ACTIONS(3024),
    [anon_sym_DOT] = ACTIONS(3026),
    [anon_sym_end] = ACTIONS(3024),
    [anon_sym_rescue] = ACTIONS(3024),
    [anon_sym_EQ] = ACTIONS(3026),
    [anon_sym_COMMA] = ACTIONS(3024),
    [anon_sym_PIPE] = ACTIONS(3026),
    [anon_sym_SEMI] = ACTIONS(3024),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3024),
    [anon_sym_AMP] = ACTIONS(3026),
    [anon_sym_LT] = ACTIONS(3026),
    [anon_sym_if] = ACTIONS(3024),
    [anon_sym_unless] = ACTIONS(3024),
    [anon_sym_while] = ACTIONS(3024),
    [anon_sym_until] = ACTIONS(3024),
    [anon_sym_in] = ACTIONS(3024),
    [anon_sym_do] = ACTIONS(3024),
    [anon_sym_when] = ACTIONS(3024),
    [anon_sym_EQ_GT] = ACTIONS(3024),
    [anon_sym_DOT_DOT] = ACTIONS(3026),
    [anon_sym_CARET] = ACTIONS(3026),
    [anon_sym_elsif] = ACTIONS(3024),
    [anon_sym_else] = ACTIONS(3024),
    [anon_sym_then] = ACTIONS(3024),
    [anon_sym_ensure] = ACTIONS(3024),
    [anon_sym_COLON_COLON2] = ACTIONS(3024),
    [anon_sym_AMP_DOT] = ACTIONS(3024),
    [anon_sym_PLUS_EQ] = ACTIONS(3024),
    [anon_sym_DASH_EQ] = ACTIONS(3024),
    [anon_sym_STAR_EQ] = ACTIONS(3024),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3024),
    [anon_sym_SLASH_EQ] = ACTIONS(3024),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3024),
    [anon_sym_PIPE_EQ] = ACTIONS(3024),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3024),
    [anon_sym_AMP_EQ] = ACTIONS(3024),
    [anon_sym_PERCENT_EQ] = ACTIONS(3024),
    [anon_sym_GT_GT_EQ] = ACTIONS(3024),
    [anon_sym_LT_LT_EQ] = ACTIONS(3024),
    [anon_sym_CARET_EQ] = ACTIONS(3024),
    [anon_sym_QMARK] = ACTIONS(3024),
    [anon_sym_and] = ACTIONS(3024),
    [anon_sym_or] = ACTIONS(3024),
    [anon_sym_PIPE_PIPE] = ACTIONS(3026),
    [anon_sym_AMP_AMP] = ACTIONS(3026),
    [anon_sym_LT_LT] = ACTIONS(3026),
    [anon_sym_GT_GT] = ACTIONS(3026),
    [anon_sym_LT_EQ] = ACTIONS(3026),
    [anon_sym_GT] = ACTIONS(3026),
    [anon_sym_GT_EQ] = ACTIONS(3024),
    [anon_sym_PLUS] = ACTIONS(3026),
    [anon_sym_SLASH] = ACTIONS(3026),
    [anon_sym_PERCENT] = ACTIONS(3026),
    [anon_sym_EQ_EQ] = ACTIONS(3026),
    [anon_sym_BANG_EQ] = ACTIONS(3024),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3024),
    [anon_sym_LT_EQ_GT] = ACTIONS(3024),
    [anon_sym_EQ_TILDE] = ACTIONS(3024),
    [anon_sym_BANG_TILDE] = ACTIONS(3024),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3024),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3024),
    [sym__binary_star] = ACTIONS(3024),
    [sym__binary_star_star] = ACTIONS(3024),
    [sym__element_reference_bracket] = ACTIONS(3024),
  },
  [2093] = {
    [sym_heredoc_body] = STATE(2093),
    [ts_builtin_sym_end] = ACTIONS(3028),
    [aux_sym_program_token1] = ACTIONS(3028),
    [anon_sym_LBRACE] = ACTIONS(3028),
    [anon_sym_RBRACE] = ACTIONS(3028),
    [anon_sym_RPAREN] = ACTIONS(3028),
    [anon_sym_DOT] = ACTIONS(3030),
    [anon_sym_end] = ACTIONS(3028),
    [anon_sym_rescue] = ACTIONS(3028),
    [anon_sym_EQ] = ACTIONS(3030),
    [anon_sym_COMMA] = ACTIONS(3028),
    [anon_sym_PIPE] = ACTIONS(3030),
    [anon_sym_SEMI] = ACTIONS(3028),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3028),
    [anon_sym_AMP] = ACTIONS(3030),
    [anon_sym_LT] = ACTIONS(3030),
    [anon_sym_if] = ACTIONS(3028),
    [anon_sym_unless] = ACTIONS(3028),
    [anon_sym_while] = ACTIONS(3028),
    [anon_sym_until] = ACTIONS(3028),
    [anon_sym_in] = ACTIONS(3028),
    [anon_sym_do] = ACTIONS(3028),
    [anon_sym_when] = ACTIONS(3028),
    [anon_sym_EQ_GT] = ACTIONS(3028),
    [anon_sym_DOT_DOT] = ACTIONS(3030),
    [anon_sym_CARET] = ACTIONS(3030),
    [anon_sym_elsif] = ACTIONS(3028),
    [anon_sym_else] = ACTIONS(3028),
    [anon_sym_then] = ACTIONS(3028),
    [anon_sym_ensure] = ACTIONS(3028),
    [anon_sym_COLON_COLON2] = ACTIONS(3028),
    [anon_sym_AMP_DOT] = ACTIONS(3028),
    [anon_sym_PLUS_EQ] = ACTIONS(3028),
    [anon_sym_DASH_EQ] = ACTIONS(3028),
    [anon_sym_STAR_EQ] = ACTIONS(3028),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3028),
    [anon_sym_SLASH_EQ] = ACTIONS(3028),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3028),
    [anon_sym_PIPE_EQ] = ACTIONS(3028),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3028),
    [anon_sym_AMP_EQ] = ACTIONS(3028),
    [anon_sym_PERCENT_EQ] = ACTIONS(3028),
    [anon_sym_GT_GT_EQ] = ACTIONS(3028),
    [anon_sym_LT_LT_EQ] = ACTIONS(3028),
    [anon_sym_CARET_EQ] = ACTIONS(3028),
    [anon_sym_QMARK] = ACTIONS(3028),
    [anon_sym_and] = ACTIONS(3028),
    [anon_sym_or] = ACTIONS(3028),
    [anon_sym_PIPE_PIPE] = ACTIONS(3030),
    [anon_sym_AMP_AMP] = ACTIONS(3030),
    [anon_sym_LT_LT] = ACTIONS(3030),
    [anon_sym_GT_GT] = ACTIONS(3030),
    [anon_sym_LT_EQ] = ACTIONS(3030),
    [anon_sym_GT] = ACTIONS(3030),
    [anon_sym_GT_EQ] = ACTIONS(3028),
    [anon_sym_PLUS] = ACTIONS(3030),
    [anon_sym_SLASH] = ACTIONS(3030),
    [anon_sym_PERCENT] = ACTIONS(3030),
    [anon_sym_EQ_EQ] = ACTIONS(3030),
    [anon_sym_BANG_EQ] = ACTIONS(3028),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3028),
    [anon_sym_LT_EQ_GT] = ACTIONS(3028),
    [anon_sym_EQ_TILDE] = ACTIONS(3028),
    [anon_sym_BANG_TILDE] = ACTIONS(3028),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3028),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3028),
    [sym__binary_star] = ACTIONS(3028),
    [sym__binary_star_star] = ACTIONS(3028),
    [sym__element_reference_bracket] = ACTIONS(3028),
  },
  [2094] = {
    [sym_heredoc_body] = STATE(2094),
    [ts_builtin_sym_end] = ACTIONS(3032),
    [aux_sym_program_token1] = ACTIONS(3032),
    [anon_sym_LBRACE] = ACTIONS(3032),
    [anon_sym_RBRACE] = ACTIONS(3032),
    [anon_sym_RPAREN] = ACTIONS(3032),
    [anon_sym_DOT] = ACTIONS(3034),
    [anon_sym_end] = ACTIONS(3032),
    [anon_sym_rescue] = ACTIONS(3032),
    [anon_sym_EQ] = ACTIONS(3034),
    [anon_sym_COMMA] = ACTIONS(3032),
    [anon_sym_PIPE] = ACTIONS(3034),
    [anon_sym_SEMI] = ACTIONS(3032),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3032),
    [anon_sym_AMP] = ACTIONS(3034),
    [anon_sym_LT] = ACTIONS(3034),
    [anon_sym_if] = ACTIONS(3032),
    [anon_sym_unless] = ACTIONS(3032),
    [anon_sym_while] = ACTIONS(3032),
    [anon_sym_until] = ACTIONS(3032),
    [anon_sym_in] = ACTIONS(3032),
    [anon_sym_do] = ACTIONS(3032),
    [anon_sym_when] = ACTIONS(3032),
    [anon_sym_EQ_GT] = ACTIONS(3032),
    [anon_sym_DOT_DOT] = ACTIONS(3034),
    [anon_sym_CARET] = ACTIONS(3034),
    [anon_sym_elsif] = ACTIONS(3032),
    [anon_sym_else] = ACTIONS(3032),
    [anon_sym_then] = ACTIONS(3032),
    [anon_sym_ensure] = ACTIONS(3032),
    [anon_sym_COLON_COLON2] = ACTIONS(3032),
    [anon_sym_AMP_DOT] = ACTIONS(3032),
    [anon_sym_PLUS_EQ] = ACTIONS(3032),
    [anon_sym_DASH_EQ] = ACTIONS(3032),
    [anon_sym_STAR_EQ] = ACTIONS(3032),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3032),
    [anon_sym_SLASH_EQ] = ACTIONS(3032),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3032),
    [anon_sym_PIPE_EQ] = ACTIONS(3032),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3032),
    [anon_sym_AMP_EQ] = ACTIONS(3032),
    [anon_sym_PERCENT_EQ] = ACTIONS(3032),
    [anon_sym_GT_GT_EQ] = ACTIONS(3032),
    [anon_sym_LT_LT_EQ] = ACTIONS(3032),
    [anon_sym_CARET_EQ] = ACTIONS(3032),
    [anon_sym_QMARK] = ACTIONS(3032),
    [anon_sym_and] = ACTIONS(3032),
    [anon_sym_or] = ACTIONS(3032),
    [anon_sym_PIPE_PIPE] = ACTIONS(3034),
    [anon_sym_AMP_AMP] = ACTIONS(3034),
    [anon_sym_LT_LT] = ACTIONS(3034),
    [anon_sym_GT_GT] = ACTIONS(3034),
    [anon_sym_LT_EQ] = ACTIONS(3034),
    [anon_sym_GT] = ACTIONS(3034),
    [anon_sym_GT_EQ] = ACTIONS(3032),
    [anon_sym_PLUS] = ACTIONS(3034),
    [anon_sym_SLASH] = ACTIONS(3034),
    [anon_sym_PERCENT] = ACTIONS(3034),
    [anon_sym_EQ_EQ] = ACTIONS(3034),
    [anon_sym_BANG_EQ] = ACTIONS(3032),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3032),
    [anon_sym_LT_EQ_GT] = ACTIONS(3032),
    [anon_sym_EQ_TILDE] = ACTIONS(3032),
    [anon_sym_BANG_TILDE] = ACTIONS(3032),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3032),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3032),
    [sym__binary_star] = ACTIONS(3032),
    [sym__binary_star_star] = ACTIONS(3032),
    [sym__element_reference_bracket] = ACTIONS(3032),
  },
  [2095] = {
    [sym_heredoc_body] = STATE(2095),
    [ts_builtin_sym_end] = ACTIONS(3036),
    [aux_sym_program_token1] = ACTIONS(3036),
    [anon_sym_LBRACE] = ACTIONS(3036),
    [anon_sym_RBRACE] = ACTIONS(3036),
    [anon_sym_RPAREN] = ACTIONS(3036),
    [anon_sym_DOT] = ACTIONS(3038),
    [anon_sym_end] = ACTIONS(3036),
    [anon_sym_rescue] = ACTIONS(3036),
    [anon_sym_EQ] = ACTIONS(3038),
    [anon_sym_COMMA] = ACTIONS(3036),
    [anon_sym_PIPE] = ACTIONS(3038),
    [anon_sym_SEMI] = ACTIONS(3036),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3036),
    [anon_sym_AMP] = ACTIONS(3038),
    [anon_sym_LT] = ACTIONS(3038),
    [anon_sym_if] = ACTIONS(3036),
    [anon_sym_unless] = ACTIONS(3036),
    [anon_sym_while] = ACTIONS(3036),
    [anon_sym_until] = ACTIONS(3036),
    [anon_sym_in] = ACTIONS(3036),
    [anon_sym_do] = ACTIONS(3036),
    [anon_sym_when] = ACTIONS(3036),
    [anon_sym_EQ_GT] = ACTIONS(3036),
    [anon_sym_DOT_DOT] = ACTIONS(3038),
    [anon_sym_CARET] = ACTIONS(3038),
    [anon_sym_elsif] = ACTIONS(3036),
    [anon_sym_else] = ACTIONS(3036),
    [anon_sym_then] = ACTIONS(3036),
    [anon_sym_ensure] = ACTIONS(3036),
    [anon_sym_COLON_COLON2] = ACTIONS(3036),
    [anon_sym_AMP_DOT] = ACTIONS(3036),
    [anon_sym_PLUS_EQ] = ACTIONS(3036),
    [anon_sym_DASH_EQ] = ACTIONS(3036),
    [anon_sym_STAR_EQ] = ACTIONS(3036),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3036),
    [anon_sym_SLASH_EQ] = ACTIONS(3036),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3036),
    [anon_sym_PIPE_EQ] = ACTIONS(3036),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3036),
    [anon_sym_AMP_EQ] = ACTIONS(3036),
    [anon_sym_PERCENT_EQ] = ACTIONS(3036),
    [anon_sym_GT_GT_EQ] = ACTIONS(3036),
    [anon_sym_LT_LT_EQ] = ACTIONS(3036),
    [anon_sym_CARET_EQ] = ACTIONS(3036),
    [anon_sym_QMARK] = ACTIONS(3036),
    [anon_sym_and] = ACTIONS(3036),
    [anon_sym_or] = ACTIONS(3036),
    [anon_sym_PIPE_PIPE] = ACTIONS(3038),
    [anon_sym_AMP_AMP] = ACTIONS(3038),
    [anon_sym_LT_LT] = ACTIONS(3038),
    [anon_sym_GT_GT] = ACTIONS(3038),
    [anon_sym_LT_EQ] = ACTIONS(3038),
    [anon_sym_GT] = ACTIONS(3038),
    [anon_sym_GT_EQ] = ACTIONS(3036),
    [anon_sym_PLUS] = ACTIONS(3038),
    [anon_sym_SLASH] = ACTIONS(3038),
    [anon_sym_PERCENT] = ACTIONS(3038),
    [anon_sym_EQ_EQ] = ACTIONS(3038),
    [anon_sym_BANG_EQ] = ACTIONS(3036),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3036),
    [anon_sym_LT_EQ_GT] = ACTIONS(3036),
    [anon_sym_EQ_TILDE] = ACTIONS(3036),
    [anon_sym_BANG_TILDE] = ACTIONS(3036),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3036),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3036),
    [sym__binary_star] = ACTIONS(3036),
    [sym__binary_star_star] = ACTIONS(3036),
    [sym__element_reference_bracket] = ACTIONS(3036),
  },
  [2096] = {
    [sym_heredoc_body] = STATE(2096),
    [ts_builtin_sym_end] = ACTIONS(3040),
    [aux_sym_program_token1] = ACTIONS(3040),
    [anon_sym_LBRACE] = ACTIONS(3040),
    [anon_sym_RBRACE] = ACTIONS(3040),
    [anon_sym_RPAREN] = ACTIONS(3040),
    [anon_sym_DOT] = ACTIONS(3042),
    [anon_sym_end] = ACTIONS(3040),
    [anon_sym_rescue] = ACTIONS(3040),
    [anon_sym_EQ] = ACTIONS(3042),
    [anon_sym_COMMA] = ACTIONS(3040),
    [anon_sym_PIPE] = ACTIONS(3042),
    [anon_sym_SEMI] = ACTIONS(3040),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3040),
    [anon_sym_AMP] = ACTIONS(3042),
    [anon_sym_LT] = ACTIONS(3042),
    [anon_sym_if] = ACTIONS(3040),
    [anon_sym_unless] = ACTIONS(3040),
    [anon_sym_while] = ACTIONS(3040),
    [anon_sym_until] = ACTIONS(3040),
    [anon_sym_in] = ACTIONS(3040),
    [anon_sym_do] = ACTIONS(3040),
    [anon_sym_when] = ACTIONS(3040),
    [anon_sym_EQ_GT] = ACTIONS(3040),
    [anon_sym_DOT_DOT] = ACTIONS(3042),
    [anon_sym_CARET] = ACTIONS(3042),
    [anon_sym_elsif] = ACTIONS(3040),
    [anon_sym_else] = ACTIONS(3040),
    [anon_sym_then] = ACTIONS(3040),
    [anon_sym_ensure] = ACTIONS(3040),
    [anon_sym_COLON_COLON2] = ACTIONS(3040),
    [anon_sym_AMP_DOT] = ACTIONS(3040),
    [anon_sym_PLUS_EQ] = ACTIONS(3040),
    [anon_sym_DASH_EQ] = ACTIONS(3040),
    [anon_sym_STAR_EQ] = ACTIONS(3040),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3040),
    [anon_sym_SLASH_EQ] = ACTIONS(3040),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3040),
    [anon_sym_PIPE_EQ] = ACTIONS(3040),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3040),
    [anon_sym_AMP_EQ] = ACTIONS(3040),
    [anon_sym_PERCENT_EQ] = ACTIONS(3040),
    [anon_sym_GT_GT_EQ] = ACTIONS(3040),
    [anon_sym_LT_LT_EQ] = ACTIONS(3040),
    [anon_sym_CARET_EQ] = ACTIONS(3040),
    [anon_sym_QMARK] = ACTIONS(3040),
    [anon_sym_and] = ACTIONS(3040),
    [anon_sym_or] = ACTIONS(3040),
    [anon_sym_PIPE_PIPE] = ACTIONS(3042),
    [anon_sym_AMP_AMP] = ACTIONS(3042),
    [anon_sym_LT_LT] = ACTIONS(3042),
    [anon_sym_GT_GT] = ACTIONS(3042),
    [anon_sym_LT_EQ] = ACTIONS(3042),
    [anon_sym_GT] = ACTIONS(3042),
    [anon_sym_GT_EQ] = ACTIONS(3040),
    [anon_sym_PLUS] = ACTIONS(3042),
    [anon_sym_SLASH] = ACTIONS(3042),
    [anon_sym_PERCENT] = ACTIONS(3042),
    [anon_sym_EQ_EQ] = ACTIONS(3042),
    [anon_sym_BANG_EQ] = ACTIONS(3040),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3040),
    [anon_sym_LT_EQ_GT] = ACTIONS(3040),
    [anon_sym_EQ_TILDE] = ACTIONS(3040),
    [anon_sym_BANG_TILDE] = ACTIONS(3040),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3040),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3040),
    [sym__binary_star] = ACTIONS(3040),
    [sym__binary_star_star] = ACTIONS(3040),
    [sym__element_reference_bracket] = ACTIONS(3040),
  },
  [2097] = {
    [sym_heredoc_body] = STATE(2097),
    [ts_builtin_sym_end] = ACTIONS(3044),
    [aux_sym_program_token1] = ACTIONS(3044),
    [anon_sym_LBRACE] = ACTIONS(3044),
    [anon_sym_RBRACE] = ACTIONS(3044),
    [anon_sym_RPAREN] = ACTIONS(3044),
    [anon_sym_DOT] = ACTIONS(3046),
    [anon_sym_end] = ACTIONS(3044),
    [anon_sym_rescue] = ACTIONS(3044),
    [anon_sym_EQ] = ACTIONS(3046),
    [anon_sym_COMMA] = ACTIONS(3044),
    [anon_sym_PIPE] = ACTIONS(3046),
    [anon_sym_SEMI] = ACTIONS(3044),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3044),
    [anon_sym_AMP] = ACTIONS(3046),
    [anon_sym_LT] = ACTIONS(3046),
    [anon_sym_if] = ACTIONS(3044),
    [anon_sym_unless] = ACTIONS(3044),
    [anon_sym_while] = ACTIONS(3044),
    [anon_sym_until] = ACTIONS(3044),
    [anon_sym_in] = ACTIONS(3044),
    [anon_sym_do] = ACTIONS(3044),
    [anon_sym_when] = ACTIONS(3044),
    [anon_sym_EQ_GT] = ACTIONS(3044),
    [anon_sym_DOT_DOT] = ACTIONS(3046),
    [anon_sym_CARET] = ACTIONS(3046),
    [anon_sym_elsif] = ACTIONS(3044),
    [anon_sym_else] = ACTIONS(3044),
    [anon_sym_then] = ACTIONS(3044),
    [anon_sym_ensure] = ACTIONS(3044),
    [anon_sym_COLON_COLON2] = ACTIONS(3044),
    [anon_sym_AMP_DOT] = ACTIONS(3044),
    [anon_sym_PLUS_EQ] = ACTIONS(3044),
    [anon_sym_DASH_EQ] = ACTIONS(3044),
    [anon_sym_STAR_EQ] = ACTIONS(3044),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3044),
    [anon_sym_SLASH_EQ] = ACTIONS(3044),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3044),
    [anon_sym_PIPE_EQ] = ACTIONS(3044),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3044),
    [anon_sym_AMP_EQ] = ACTIONS(3044),
    [anon_sym_PERCENT_EQ] = ACTIONS(3044),
    [anon_sym_GT_GT_EQ] = ACTIONS(3044),
    [anon_sym_LT_LT_EQ] = ACTIONS(3044),
    [anon_sym_CARET_EQ] = ACTIONS(3044),
    [anon_sym_QMARK] = ACTIONS(3044),
    [anon_sym_and] = ACTIONS(3044),
    [anon_sym_or] = ACTIONS(3044),
    [anon_sym_PIPE_PIPE] = ACTIONS(3046),
    [anon_sym_AMP_AMP] = ACTIONS(3046),
    [anon_sym_LT_LT] = ACTIONS(3046),
    [anon_sym_GT_GT] = ACTIONS(3046),
    [anon_sym_LT_EQ] = ACTIONS(3046),
    [anon_sym_GT] = ACTIONS(3046),
    [anon_sym_GT_EQ] = ACTIONS(3044),
    [anon_sym_PLUS] = ACTIONS(3046),
    [anon_sym_SLASH] = ACTIONS(3046),
    [anon_sym_PERCENT] = ACTIONS(3046),
    [anon_sym_EQ_EQ] = ACTIONS(3046),
    [anon_sym_BANG_EQ] = ACTIONS(3044),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3044),
    [anon_sym_LT_EQ_GT] = ACTIONS(3044),
    [anon_sym_EQ_TILDE] = ACTIONS(3044),
    [anon_sym_BANG_TILDE] = ACTIONS(3044),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3044),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3044),
    [sym__binary_star] = ACTIONS(3044),
    [sym__binary_star_star] = ACTIONS(3044),
    [sym__element_reference_bracket] = ACTIONS(3044),
  },
  [2098] = {
    [sym_heredoc_body] = STATE(2098),
    [ts_builtin_sym_end] = ACTIONS(3048),
    [aux_sym_program_token1] = ACTIONS(3048),
    [anon_sym_LBRACE] = ACTIONS(3048),
    [anon_sym_RBRACE] = ACTIONS(3048),
    [anon_sym_RPAREN] = ACTIONS(3048),
    [anon_sym_DOT] = ACTIONS(3050),
    [anon_sym_end] = ACTIONS(3048),
    [anon_sym_rescue] = ACTIONS(3048),
    [anon_sym_EQ] = ACTIONS(3050),
    [anon_sym_COMMA] = ACTIONS(3048),
    [anon_sym_PIPE] = ACTIONS(3050),
    [anon_sym_SEMI] = ACTIONS(3048),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3048),
    [anon_sym_AMP] = ACTIONS(3050),
    [anon_sym_LT] = ACTIONS(3050),
    [anon_sym_if] = ACTIONS(3048),
    [anon_sym_unless] = ACTIONS(3048),
    [anon_sym_while] = ACTIONS(3048),
    [anon_sym_until] = ACTIONS(3048),
    [anon_sym_in] = ACTIONS(3048),
    [anon_sym_do] = ACTIONS(3048),
    [anon_sym_when] = ACTIONS(3048),
    [anon_sym_EQ_GT] = ACTIONS(3048),
    [anon_sym_DOT_DOT] = ACTIONS(3050),
    [anon_sym_CARET] = ACTIONS(3050),
    [anon_sym_elsif] = ACTIONS(3048),
    [anon_sym_else] = ACTIONS(3048),
    [anon_sym_then] = ACTIONS(3048),
    [anon_sym_ensure] = ACTIONS(3048),
    [anon_sym_COLON_COLON2] = ACTIONS(3048),
    [anon_sym_AMP_DOT] = ACTIONS(3048),
    [anon_sym_PLUS_EQ] = ACTIONS(3048),
    [anon_sym_DASH_EQ] = ACTIONS(3048),
    [anon_sym_STAR_EQ] = ACTIONS(3048),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3048),
    [anon_sym_SLASH_EQ] = ACTIONS(3048),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3048),
    [anon_sym_PIPE_EQ] = ACTIONS(3048),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3048),
    [anon_sym_AMP_EQ] = ACTIONS(3048),
    [anon_sym_PERCENT_EQ] = ACTIONS(3048),
    [anon_sym_GT_GT_EQ] = ACTIONS(3048),
    [anon_sym_LT_LT_EQ] = ACTIONS(3048),
    [anon_sym_CARET_EQ] = ACTIONS(3048),
    [anon_sym_QMARK] = ACTIONS(3048),
    [anon_sym_and] = ACTIONS(3048),
    [anon_sym_or] = ACTIONS(3048),
    [anon_sym_PIPE_PIPE] = ACTIONS(3050),
    [anon_sym_AMP_AMP] = ACTIONS(3050),
    [anon_sym_LT_LT] = ACTIONS(3050),
    [anon_sym_GT_GT] = ACTIONS(3050),
    [anon_sym_LT_EQ] = ACTIONS(3050),
    [anon_sym_GT] = ACTIONS(3050),
    [anon_sym_GT_EQ] = ACTIONS(3048),
    [anon_sym_PLUS] = ACTIONS(3050),
    [anon_sym_SLASH] = ACTIONS(3050),
    [anon_sym_PERCENT] = ACTIONS(3050),
    [anon_sym_EQ_EQ] = ACTIONS(3050),
    [anon_sym_BANG_EQ] = ACTIONS(3048),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3048),
    [anon_sym_LT_EQ_GT] = ACTIONS(3048),
    [anon_sym_EQ_TILDE] = ACTIONS(3048),
    [anon_sym_BANG_TILDE] = ACTIONS(3048),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3048),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3048),
    [sym__binary_star] = ACTIONS(3048),
    [sym__binary_star_star] = ACTIONS(3048),
    [sym__element_reference_bracket] = ACTIONS(3048),
  },
  [2099] = {
    [sym_heredoc_body] = STATE(2099),
    [ts_builtin_sym_end] = ACTIONS(3052),
    [aux_sym_program_token1] = ACTIONS(3052),
    [anon_sym_LBRACE] = ACTIONS(3052),
    [anon_sym_RBRACE] = ACTIONS(3052),
    [anon_sym_RPAREN] = ACTIONS(3052),
    [anon_sym_DOT] = ACTIONS(3054),
    [anon_sym_end] = ACTIONS(3052),
    [anon_sym_rescue] = ACTIONS(3052),
    [anon_sym_EQ] = ACTIONS(3054),
    [anon_sym_COMMA] = ACTIONS(3052),
    [anon_sym_PIPE] = ACTIONS(3054),
    [anon_sym_SEMI] = ACTIONS(3052),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3052),
    [anon_sym_AMP] = ACTIONS(3054),
    [anon_sym_LT] = ACTIONS(3054),
    [anon_sym_if] = ACTIONS(3052),
    [anon_sym_unless] = ACTIONS(3052),
    [anon_sym_while] = ACTIONS(3052),
    [anon_sym_until] = ACTIONS(3052),
    [anon_sym_in] = ACTIONS(3052),
    [anon_sym_do] = ACTIONS(3052),
    [anon_sym_when] = ACTIONS(3052),
    [anon_sym_EQ_GT] = ACTIONS(3052),
    [anon_sym_DOT_DOT] = ACTIONS(3054),
    [anon_sym_CARET] = ACTIONS(3054),
    [anon_sym_elsif] = ACTIONS(3052),
    [anon_sym_else] = ACTIONS(3052),
    [anon_sym_then] = ACTIONS(3052),
    [anon_sym_ensure] = ACTIONS(3052),
    [anon_sym_COLON_COLON2] = ACTIONS(3052),
    [anon_sym_AMP_DOT] = ACTIONS(3052),
    [anon_sym_PLUS_EQ] = ACTIONS(3052),
    [anon_sym_DASH_EQ] = ACTIONS(3052),
    [anon_sym_STAR_EQ] = ACTIONS(3052),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3052),
    [anon_sym_SLASH_EQ] = ACTIONS(3052),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3052),
    [anon_sym_PIPE_EQ] = ACTIONS(3052),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3052),
    [anon_sym_AMP_EQ] = ACTIONS(3052),
    [anon_sym_PERCENT_EQ] = ACTIONS(3052),
    [anon_sym_GT_GT_EQ] = ACTIONS(3052),
    [anon_sym_LT_LT_EQ] = ACTIONS(3052),
    [anon_sym_CARET_EQ] = ACTIONS(3052),
    [anon_sym_QMARK] = ACTIONS(3052),
    [anon_sym_and] = ACTIONS(3052),
    [anon_sym_or] = ACTIONS(3052),
    [anon_sym_PIPE_PIPE] = ACTIONS(3054),
    [anon_sym_AMP_AMP] = ACTIONS(3054),
    [anon_sym_LT_LT] = ACTIONS(3054),
    [anon_sym_GT_GT] = ACTIONS(3054),
    [anon_sym_LT_EQ] = ACTIONS(3054),
    [anon_sym_GT] = ACTIONS(3054),
    [anon_sym_GT_EQ] = ACTIONS(3052),
    [anon_sym_PLUS] = ACTIONS(3054),
    [anon_sym_SLASH] = ACTIONS(3054),
    [anon_sym_PERCENT] = ACTIONS(3054),
    [anon_sym_EQ_EQ] = ACTIONS(3054),
    [anon_sym_BANG_EQ] = ACTIONS(3052),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3052),
    [anon_sym_LT_EQ_GT] = ACTIONS(3052),
    [anon_sym_EQ_TILDE] = ACTIONS(3052),
    [anon_sym_BANG_TILDE] = ACTIONS(3052),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3052),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3052),
    [sym__binary_star] = ACTIONS(3052),
    [sym__binary_star_star] = ACTIONS(3052),
    [sym__element_reference_bracket] = ACTIONS(3052),
  },
  [2100] = {
    [sym_heredoc_body] = STATE(2100),
    [ts_builtin_sym_end] = ACTIONS(3056),
    [aux_sym_program_token1] = ACTIONS(3056),
    [anon_sym_LBRACE] = ACTIONS(3056),
    [anon_sym_RBRACE] = ACTIONS(3056),
    [anon_sym_RPAREN] = ACTIONS(3056),
    [anon_sym_DOT] = ACTIONS(3058),
    [anon_sym_end] = ACTIONS(3056),
    [anon_sym_rescue] = ACTIONS(3056),
    [anon_sym_EQ] = ACTIONS(3058),
    [anon_sym_COMMA] = ACTIONS(3056),
    [anon_sym_PIPE] = ACTIONS(3058),
    [anon_sym_SEMI] = ACTIONS(3056),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3056),
    [anon_sym_AMP] = ACTIONS(3058),
    [anon_sym_LT] = ACTIONS(3058),
    [anon_sym_if] = ACTIONS(3056),
    [anon_sym_unless] = ACTIONS(3056),
    [anon_sym_while] = ACTIONS(3056),
    [anon_sym_until] = ACTIONS(3056),
    [anon_sym_in] = ACTIONS(3056),
    [anon_sym_do] = ACTIONS(3056),
    [anon_sym_when] = ACTIONS(3056),
    [anon_sym_EQ_GT] = ACTIONS(3056),
    [anon_sym_DOT_DOT] = ACTIONS(3058),
    [anon_sym_CARET] = ACTIONS(3058),
    [anon_sym_elsif] = ACTIONS(3056),
    [anon_sym_else] = ACTIONS(3056),
    [anon_sym_then] = ACTIONS(3056),
    [anon_sym_ensure] = ACTIONS(3056),
    [anon_sym_COLON_COLON2] = ACTIONS(3056),
    [anon_sym_AMP_DOT] = ACTIONS(3056),
    [anon_sym_PLUS_EQ] = ACTIONS(3056),
    [anon_sym_DASH_EQ] = ACTIONS(3056),
    [anon_sym_STAR_EQ] = ACTIONS(3056),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3056),
    [anon_sym_SLASH_EQ] = ACTIONS(3056),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3056),
    [anon_sym_PIPE_EQ] = ACTIONS(3056),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3056),
    [anon_sym_AMP_EQ] = ACTIONS(3056),
    [anon_sym_PERCENT_EQ] = ACTIONS(3056),
    [anon_sym_GT_GT_EQ] = ACTIONS(3056),
    [anon_sym_LT_LT_EQ] = ACTIONS(3056),
    [anon_sym_CARET_EQ] = ACTIONS(3056),
    [anon_sym_QMARK] = ACTIONS(3056),
    [anon_sym_and] = ACTIONS(3056),
    [anon_sym_or] = ACTIONS(3056),
    [anon_sym_PIPE_PIPE] = ACTIONS(3058),
    [anon_sym_AMP_AMP] = ACTIONS(3058),
    [anon_sym_LT_LT] = ACTIONS(3058),
    [anon_sym_GT_GT] = ACTIONS(3058),
    [anon_sym_LT_EQ] = ACTIONS(3058),
    [anon_sym_GT] = ACTIONS(3058),
    [anon_sym_GT_EQ] = ACTIONS(3056),
    [anon_sym_PLUS] = ACTIONS(3058),
    [anon_sym_SLASH] = ACTIONS(3058),
    [anon_sym_PERCENT] = ACTIONS(3058),
    [anon_sym_EQ_EQ] = ACTIONS(3058),
    [anon_sym_BANG_EQ] = ACTIONS(3056),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3056),
    [anon_sym_LT_EQ_GT] = ACTIONS(3056),
    [anon_sym_EQ_TILDE] = ACTIONS(3056),
    [anon_sym_BANG_TILDE] = ACTIONS(3056),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3056),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3056),
    [sym__binary_star] = ACTIONS(3056),
    [sym__binary_star_star] = ACTIONS(3056),
    [sym__element_reference_bracket] = ACTIONS(3056),
  },
  [2101] = {
    [sym_heredoc_body] = STATE(2101),
    [ts_builtin_sym_end] = ACTIONS(3060),
    [aux_sym_program_token1] = ACTIONS(3060),
    [anon_sym_LBRACE] = ACTIONS(3060),
    [anon_sym_RBRACE] = ACTIONS(3060),
    [anon_sym_RPAREN] = ACTIONS(3060),
    [anon_sym_DOT] = ACTIONS(3062),
    [anon_sym_end] = ACTIONS(3060),
    [anon_sym_rescue] = ACTIONS(3060),
    [anon_sym_EQ] = ACTIONS(3062),
    [anon_sym_COMMA] = ACTIONS(3060),
    [anon_sym_PIPE] = ACTIONS(3062),
    [anon_sym_SEMI] = ACTIONS(3060),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3060),
    [anon_sym_AMP] = ACTIONS(3062),
    [anon_sym_LT] = ACTIONS(3062),
    [anon_sym_if] = ACTIONS(3060),
    [anon_sym_unless] = ACTIONS(3060),
    [anon_sym_while] = ACTIONS(3060),
    [anon_sym_until] = ACTIONS(3060),
    [anon_sym_in] = ACTIONS(3060),
    [anon_sym_do] = ACTIONS(3060),
    [anon_sym_when] = ACTIONS(3060),
    [anon_sym_EQ_GT] = ACTIONS(3060),
    [anon_sym_DOT_DOT] = ACTIONS(3062),
    [anon_sym_CARET] = ACTIONS(3062),
    [anon_sym_elsif] = ACTIONS(3060),
    [anon_sym_else] = ACTIONS(3060),
    [anon_sym_then] = ACTIONS(3060),
    [anon_sym_ensure] = ACTIONS(3060),
    [anon_sym_COLON_COLON2] = ACTIONS(3060),
    [anon_sym_AMP_DOT] = ACTIONS(3060),
    [anon_sym_PLUS_EQ] = ACTIONS(3060),
    [anon_sym_DASH_EQ] = ACTIONS(3060),
    [anon_sym_STAR_EQ] = ACTIONS(3060),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3060),
    [anon_sym_SLASH_EQ] = ACTIONS(3060),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3060),
    [anon_sym_PIPE_EQ] = ACTIONS(3060),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3060),
    [anon_sym_AMP_EQ] = ACTIONS(3060),
    [anon_sym_PERCENT_EQ] = ACTIONS(3060),
    [anon_sym_GT_GT_EQ] = ACTIONS(3060),
    [anon_sym_LT_LT_EQ] = ACTIONS(3060),
    [anon_sym_CARET_EQ] = ACTIONS(3060),
    [anon_sym_QMARK] = ACTIONS(3060),
    [anon_sym_and] = ACTIONS(3060),
    [anon_sym_or] = ACTIONS(3060),
    [anon_sym_PIPE_PIPE] = ACTIONS(3062),
    [anon_sym_AMP_AMP] = ACTIONS(3062),
    [anon_sym_LT_LT] = ACTIONS(3062),
    [anon_sym_GT_GT] = ACTIONS(3062),
    [anon_sym_LT_EQ] = ACTIONS(3062),
    [anon_sym_GT] = ACTIONS(3062),
    [anon_sym_GT_EQ] = ACTIONS(3060),
    [anon_sym_PLUS] = ACTIONS(3062),
    [anon_sym_SLASH] = ACTIONS(3062),
    [anon_sym_PERCENT] = ACTIONS(3062),
    [anon_sym_EQ_EQ] = ACTIONS(3062),
    [anon_sym_BANG_EQ] = ACTIONS(3060),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3060),
    [anon_sym_LT_EQ_GT] = ACTIONS(3060),
    [anon_sym_EQ_TILDE] = ACTIONS(3060),
    [anon_sym_BANG_TILDE] = ACTIONS(3060),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3060),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3060),
    [sym__binary_star] = ACTIONS(3060),
    [sym__binary_star_star] = ACTIONS(3060),
    [sym__element_reference_bracket] = ACTIONS(3060),
  },
  [2102] = {
    [sym_heredoc_body] = STATE(2102),
    [sym_identifier] = ACTIONS(2634),
    [anon_sym_LBRACE] = ACTIONS(2636),
    [anon_sym_def] = ACTIONS(2634),
    [anon_sym_LPAREN] = ACTIONS(2636),
    [anon_sym_RPAREN] = ACTIONS(2636),
    [anon_sym_COLON_COLON] = ACTIONS(2636),
    [anon_sym_rescue] = ACTIONS(2634),
    [anon_sym_COMMA] = ACTIONS(2636),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2636),
    [anon_sym_STAR] = ACTIONS(2636),
    [anon_sym_nil] = ACTIONS(2634),
    [anon_sym_class] = ACTIONS(2634),
    [anon_sym_module] = ACTIONS(2634),
    [anon_sym_return] = ACTIONS(2634),
    [anon_sym_yield] = ACTIONS(2634),
    [anon_sym_break] = ACTIONS(2634),
    [anon_sym_next] = ACTIONS(2634),
    [anon_sym_redo] = ACTIONS(2634),
    [anon_sym_retry] = ACTIONS(2634),
    [anon_sym_if] = ACTIONS(2634),
    [anon_sym_unless] = ACTIONS(2634),
    [anon_sym_while] = ACTIONS(2634),
    [anon_sym_until] = ACTIONS(2634),
    [anon_sym_for] = ACTIONS(2634),
    [anon_sym_do] = ACTIONS(2634),
    [anon_sym_case] = ACTIONS(2634),
    [anon_sym_LBRACK] = ACTIONS(2636),
    [anon_sym_RBRACK] = ACTIONS(2636),
    [anon_sym_DOT_DOT] = ACTIONS(2634),
    [anon_sym_begin] = ACTIONS(2634),
    [anon_sym_and] = ACTIONS(2634),
    [anon_sym_or] = ACTIONS(2634),
    [anon_sym_PLUS] = ACTIONS(2636),
    [anon_sym_defined_QMARK] = ACTIONS(2636),
    [anon_sym_not] = ACTIONS(2634),
    [anon_sym_BANG] = ACTIONS(2636),
    [anon_sym_TILDE] = ACTIONS(2636),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2634),
    [sym_float] = ACTIONS(2634),
    [sym_super] = ACTIONS(2634),
    [sym_self] = ACTIONS(2634),
    [sym_true] = ACTIONS(2634),
    [sym_false] = ACTIONS(2634),
    [sym_constant] = ACTIONS(2634),
    [aux_sym_constant_suffix_token1] = ACTIONS(2636),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2634),
    [sym_instance_variable] = ACTIONS(2636),
    [sym_class_variable] = ACTIONS(2636),
    [sym_global_variable] = ACTIONS(2636),
    [sym_character] = ACTIONS(2636),
    [anon_sym_DASH_GT] = ACTIONS(2636),
    [sym_simple_symbol] = ACTIONS(2636),
    [sym__string_start] = ACTIONS(2636),
    [sym__symbol_start] = ACTIONS(2636),
    [sym__subshell_start] = ACTIONS(2636),
    [sym__regex_start] = ACTIONS(2636),
    [sym__string_array_start] = ACTIONS(2636),
    [sym__symbol_array_start] = ACTIONS(2636),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2636),
    [sym__block_ampersand] = ACTIONS(2636),
    [sym__splat_star] = ACTIONS(2636),
    [sym__unary_minus] = ACTIONS(2636),
    [sym__unary_minus_num] = ACTIONS(2636),
    [sym__binary_minus] = ACTIONS(2636),
    [sym_hash_key_symbol] = ACTIONS(2636),
    [sym__identifier_suffix] = ACTIONS(2636),
    [sym__constant_suffix] = ACTIONS(2636),
    [sym__hash_splat_star_star] = ACTIONS(2636),
  },
  [2103] = {
    [sym_heredoc_body] = STATE(2103),
    [ts_builtin_sym_end] = ACTIONS(3064),
    [aux_sym_program_token1] = ACTIONS(3064),
    [anon_sym_LBRACE] = ACTIONS(3064),
    [anon_sym_RBRACE] = ACTIONS(3064),
    [anon_sym_RPAREN] = ACTIONS(3064),
    [anon_sym_DOT] = ACTIONS(3066),
    [anon_sym_end] = ACTIONS(3064),
    [anon_sym_rescue] = ACTIONS(3064),
    [anon_sym_EQ] = ACTIONS(3066),
    [anon_sym_COMMA] = ACTIONS(3064),
    [anon_sym_PIPE] = ACTIONS(3066),
    [anon_sym_SEMI] = ACTIONS(3064),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3064),
    [anon_sym_AMP] = ACTIONS(3066),
    [anon_sym_LT] = ACTIONS(3066),
    [anon_sym_if] = ACTIONS(3064),
    [anon_sym_unless] = ACTIONS(3064),
    [anon_sym_while] = ACTIONS(3064),
    [anon_sym_until] = ACTIONS(3064),
    [anon_sym_in] = ACTIONS(3064),
    [anon_sym_do] = ACTIONS(3064),
    [anon_sym_when] = ACTIONS(3064),
    [anon_sym_EQ_GT] = ACTIONS(3064),
    [anon_sym_DOT_DOT] = ACTIONS(3066),
    [anon_sym_CARET] = ACTIONS(3066),
    [anon_sym_elsif] = ACTIONS(3064),
    [anon_sym_else] = ACTIONS(3064),
    [anon_sym_then] = ACTIONS(3064),
    [anon_sym_ensure] = ACTIONS(3064),
    [anon_sym_COLON_COLON2] = ACTIONS(3064),
    [anon_sym_AMP_DOT] = ACTIONS(3064),
    [anon_sym_PLUS_EQ] = ACTIONS(3064),
    [anon_sym_DASH_EQ] = ACTIONS(3064),
    [anon_sym_STAR_EQ] = ACTIONS(3064),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3064),
    [anon_sym_SLASH_EQ] = ACTIONS(3064),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3064),
    [anon_sym_PIPE_EQ] = ACTIONS(3064),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3064),
    [anon_sym_AMP_EQ] = ACTIONS(3064),
    [anon_sym_PERCENT_EQ] = ACTIONS(3064),
    [anon_sym_GT_GT_EQ] = ACTIONS(3064),
    [anon_sym_LT_LT_EQ] = ACTIONS(3064),
    [anon_sym_CARET_EQ] = ACTIONS(3064),
    [anon_sym_QMARK] = ACTIONS(3064),
    [anon_sym_and] = ACTIONS(3064),
    [anon_sym_or] = ACTIONS(3064),
    [anon_sym_PIPE_PIPE] = ACTIONS(3066),
    [anon_sym_AMP_AMP] = ACTIONS(3066),
    [anon_sym_LT_LT] = ACTIONS(3066),
    [anon_sym_GT_GT] = ACTIONS(3066),
    [anon_sym_LT_EQ] = ACTIONS(3066),
    [anon_sym_GT] = ACTIONS(3066),
    [anon_sym_GT_EQ] = ACTIONS(3064),
    [anon_sym_PLUS] = ACTIONS(3066),
    [anon_sym_SLASH] = ACTIONS(3066),
    [anon_sym_PERCENT] = ACTIONS(3066),
    [anon_sym_EQ_EQ] = ACTIONS(3066),
    [anon_sym_BANG_EQ] = ACTIONS(3064),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3064),
    [anon_sym_LT_EQ_GT] = ACTIONS(3064),
    [anon_sym_EQ_TILDE] = ACTIONS(3064),
    [anon_sym_BANG_TILDE] = ACTIONS(3064),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3064),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3064),
    [sym__binary_star] = ACTIONS(3064),
    [sym__binary_star_star] = ACTIONS(3064),
    [sym__element_reference_bracket] = ACTIONS(3064),
  },
  [2104] = {
    [sym_heredoc_body] = STATE(2104),
    [sym_identifier] = ACTIONS(2638),
    [anon_sym_LBRACE] = ACTIONS(2640),
    [anon_sym_def] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(2640),
    [anon_sym_RPAREN] = ACTIONS(2640),
    [anon_sym_COLON_COLON] = ACTIONS(2640),
    [anon_sym_rescue] = ACTIONS(2638),
    [anon_sym_COMMA] = ACTIONS(2640),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2640),
    [anon_sym_STAR] = ACTIONS(2640),
    [anon_sym_nil] = ACTIONS(2638),
    [anon_sym_class] = ACTIONS(2638),
    [anon_sym_module] = ACTIONS(2638),
    [anon_sym_return] = ACTIONS(2638),
    [anon_sym_yield] = ACTIONS(2638),
    [anon_sym_break] = ACTIONS(2638),
    [anon_sym_next] = ACTIONS(2638),
    [anon_sym_redo] = ACTIONS(2638),
    [anon_sym_retry] = ACTIONS(2638),
    [anon_sym_if] = ACTIONS(2638),
    [anon_sym_unless] = ACTIONS(2638),
    [anon_sym_while] = ACTIONS(2638),
    [anon_sym_until] = ACTIONS(2638),
    [anon_sym_for] = ACTIONS(2638),
    [anon_sym_do] = ACTIONS(2638),
    [anon_sym_case] = ACTIONS(2638),
    [anon_sym_LBRACK] = ACTIONS(2640),
    [anon_sym_RBRACK] = ACTIONS(2640),
    [anon_sym_DOT_DOT] = ACTIONS(2638),
    [anon_sym_begin] = ACTIONS(2638),
    [anon_sym_and] = ACTIONS(2638),
    [anon_sym_or] = ACTIONS(2638),
    [anon_sym_PLUS] = ACTIONS(2640),
    [anon_sym_defined_QMARK] = ACTIONS(2640),
    [anon_sym_not] = ACTIONS(2638),
    [anon_sym_BANG] = ACTIONS(2640),
    [anon_sym_TILDE] = ACTIONS(2640),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2638),
    [sym_float] = ACTIONS(2638),
    [sym_super] = ACTIONS(2638),
    [sym_self] = ACTIONS(2638),
    [sym_true] = ACTIONS(2638),
    [sym_false] = ACTIONS(2638),
    [sym_constant] = ACTIONS(2638),
    [aux_sym_constant_suffix_token1] = ACTIONS(2640),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2638),
    [sym_instance_variable] = ACTIONS(2640),
    [sym_class_variable] = ACTIONS(2640),
    [sym_global_variable] = ACTIONS(2640),
    [sym_character] = ACTIONS(2640),
    [anon_sym_DASH_GT] = ACTIONS(2640),
    [sym_simple_symbol] = ACTIONS(2640),
    [sym__string_start] = ACTIONS(2640),
    [sym__symbol_start] = ACTIONS(2640),
    [sym__subshell_start] = ACTIONS(2640),
    [sym__regex_start] = ACTIONS(2640),
    [sym__string_array_start] = ACTIONS(2640),
    [sym__symbol_array_start] = ACTIONS(2640),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2640),
    [sym__block_ampersand] = ACTIONS(2640),
    [sym__splat_star] = ACTIONS(2640),
    [sym__unary_minus] = ACTIONS(2640),
    [sym__unary_minus_num] = ACTIONS(2640),
    [sym__binary_minus] = ACTIONS(2640),
    [sym_hash_key_symbol] = ACTIONS(2640),
    [sym__identifier_suffix] = ACTIONS(2640),
    [sym__constant_suffix] = ACTIONS(2640),
    [sym__hash_splat_star_star] = ACTIONS(2640),
  },
  [2105] = {
    [sym_heredoc_body] = STATE(2105),
    [sym_identifier] = ACTIONS(3068),
    [anon_sym_BEGIN] = ACTIONS(3068),
    [anon_sym_LBRACE] = ACTIONS(3070),
    [anon_sym_END] = ACTIONS(3068),
    [anon_sym_def] = ACTIONS(3068),
    [anon_sym_LPAREN] = ACTIONS(3070),
    [anon_sym_COLON_COLON] = ACTIONS(3070),
    [anon_sym_end] = ACTIONS(3068),
    [anon_sym_rescue] = ACTIONS(3068),
    [anon_sym_EQ] = ACTIONS(3070),
    [anon_sym_SEMI] = ACTIONS(3070),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3070),
    [anon_sym_STAR] = ACTIONS(3070),
    [anon_sym_nil] = ACTIONS(3068),
    [anon_sym_class] = ACTIONS(3068),
    [anon_sym_module] = ACTIONS(3068),
    [anon_sym_return] = ACTIONS(3068),
    [anon_sym_yield] = ACTIONS(3068),
    [anon_sym_break] = ACTIONS(3068),
    [anon_sym_next] = ACTIONS(3068),
    [anon_sym_redo] = ACTIONS(3068),
    [anon_sym_retry] = ACTIONS(3068),
    [anon_sym_if] = ACTIONS(3068),
    [anon_sym_unless] = ACTIONS(3068),
    [anon_sym_while] = ACTIONS(3068),
    [anon_sym_until] = ACTIONS(3068),
    [anon_sym_for] = ACTIONS(3068),
    [anon_sym_case] = ACTIONS(3068),
    [anon_sym_LBRACK] = ACTIONS(3070),
    [anon_sym_DOT_DOT] = ACTIONS(3068),
    [anon_sym_else] = ACTIONS(3068),
    [anon_sym_begin] = ACTIONS(3068),
    [anon_sym_ensure] = ACTIONS(3068),
    [anon_sym_PLUS] = ACTIONS(3070),
    [anon_sym_defined_QMARK] = ACTIONS(3070),
    [anon_sym_not] = ACTIONS(3068),
    [anon_sym_BANG] = ACTIONS(3070),
    [anon_sym_TILDE] = ACTIONS(3070),
    [anon_sym_undef] = ACTIONS(3068),
    [anon_sym_alias] = ACTIONS(3068),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(3068),
    [sym_float] = ACTIONS(3068),
    [sym_super] = ACTIONS(3068),
    [sym_self] = ACTIONS(3068),
    [sym_true] = ACTIONS(3068),
    [sym_false] = ACTIONS(3068),
    [sym_constant] = ACTIONS(3068),
    [aux_sym_constant_suffix_token1] = ACTIONS(3070),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3068),
    [sym_instance_variable] = ACTIONS(3070),
    [sym_class_variable] = ACTIONS(3070),
    [sym_global_variable] = ACTIONS(3070),
    [sym_character] = ACTIONS(3070),
    [anon_sym_DASH_GT] = ACTIONS(3070),
    [sym__line_break] = ACTIONS(3070),
    [sym_simple_symbol] = ACTIONS(3070),
    [sym__string_start] = ACTIONS(3070),
    [sym__symbol_start] = ACTIONS(3070),
    [sym__subshell_start] = ACTIONS(3070),
    [sym__regex_start] = ACTIONS(3070),
    [sym__string_array_start] = ACTIONS(3070),
    [sym__symbol_array_start] = ACTIONS(3070),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3070),
    [sym__unary_minus] = ACTIONS(3070),
    [sym__unary_minus_num] = ACTIONS(3070),
    [sym__binary_minus] = ACTIONS(3070),
    [sym__identifier_suffix] = ACTIONS(3070),
    [sym__constant_suffix] = ACTIONS(3070),
  },
  [2106] = {
    [sym_heredoc_body] = STATE(2106),
    [ts_builtin_sym_end] = ACTIONS(3072),
    [aux_sym_program_token1] = ACTIONS(3072),
    [anon_sym_LBRACE] = ACTIONS(3072),
    [anon_sym_RBRACE] = ACTIONS(3072),
    [anon_sym_RPAREN] = ACTIONS(3072),
    [anon_sym_DOT] = ACTIONS(3074),
    [anon_sym_end] = ACTIONS(3072),
    [anon_sym_rescue] = ACTIONS(3072),
    [anon_sym_EQ] = ACTIONS(3074),
    [anon_sym_COMMA] = ACTIONS(3072),
    [anon_sym_PIPE] = ACTIONS(3074),
    [anon_sym_SEMI] = ACTIONS(3072),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3072),
    [anon_sym_AMP] = ACTIONS(3074),
    [anon_sym_LT] = ACTIONS(3074),
    [anon_sym_if] = ACTIONS(3072),
    [anon_sym_unless] = ACTIONS(3072),
    [anon_sym_while] = ACTIONS(3072),
    [anon_sym_until] = ACTIONS(3072),
    [anon_sym_in] = ACTIONS(3072),
    [anon_sym_do] = ACTIONS(3072),
    [anon_sym_when] = ACTIONS(3072),
    [anon_sym_EQ_GT] = ACTIONS(3072),
    [anon_sym_DOT_DOT] = ACTIONS(3074),
    [anon_sym_CARET] = ACTIONS(3074),
    [anon_sym_elsif] = ACTIONS(3072),
    [anon_sym_else] = ACTIONS(3072),
    [anon_sym_then] = ACTIONS(3072),
    [anon_sym_ensure] = ACTIONS(3072),
    [anon_sym_COLON_COLON2] = ACTIONS(3072),
    [anon_sym_AMP_DOT] = ACTIONS(3072),
    [anon_sym_PLUS_EQ] = ACTIONS(3072),
    [anon_sym_DASH_EQ] = ACTIONS(3072),
    [anon_sym_STAR_EQ] = ACTIONS(3072),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3072),
    [anon_sym_SLASH_EQ] = ACTIONS(3072),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3072),
    [anon_sym_PIPE_EQ] = ACTIONS(3072),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3072),
    [anon_sym_AMP_EQ] = ACTIONS(3072),
    [anon_sym_PERCENT_EQ] = ACTIONS(3072),
    [anon_sym_GT_GT_EQ] = ACTIONS(3072),
    [anon_sym_LT_LT_EQ] = ACTIONS(3072),
    [anon_sym_CARET_EQ] = ACTIONS(3072),
    [anon_sym_QMARK] = ACTIONS(3072),
    [anon_sym_and] = ACTIONS(3072),
    [anon_sym_or] = ACTIONS(3072),
    [anon_sym_PIPE_PIPE] = ACTIONS(3074),
    [anon_sym_AMP_AMP] = ACTIONS(3074),
    [anon_sym_LT_LT] = ACTIONS(3074),
    [anon_sym_GT_GT] = ACTIONS(3074),
    [anon_sym_LT_EQ] = ACTIONS(3074),
    [anon_sym_GT] = ACTIONS(3074),
    [anon_sym_GT_EQ] = ACTIONS(3072),
    [anon_sym_PLUS] = ACTIONS(3074),
    [anon_sym_SLASH] = ACTIONS(3074),
    [anon_sym_PERCENT] = ACTIONS(3074),
    [anon_sym_EQ_EQ] = ACTIONS(3074),
    [anon_sym_BANG_EQ] = ACTIONS(3072),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3072),
    [anon_sym_LT_EQ_GT] = ACTIONS(3072),
    [anon_sym_EQ_TILDE] = ACTIONS(3072),
    [anon_sym_BANG_TILDE] = ACTIONS(3072),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3072),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3072),
    [sym__binary_star] = ACTIONS(3072),
    [sym__binary_star_star] = ACTIONS(3072),
    [sym__element_reference_bracket] = ACTIONS(3072),
  },
  [2107] = {
    [sym_heredoc_body] = STATE(2107),
    [ts_builtin_sym_end] = ACTIONS(121),
    [aux_sym_program_token1] = ACTIONS(121),
    [anon_sym_LBRACE] = ACTIONS(121),
    [anon_sym_RBRACE] = ACTIONS(121),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_end] = ACTIONS(121),
    [anon_sym_rescue] = ACTIONS(121),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_if] = ACTIONS(121),
    [anon_sym_unless] = ACTIONS(121),
    [anon_sym_while] = ACTIONS(121),
    [anon_sym_until] = ACTIONS(121),
    [anon_sym_in] = ACTIONS(121),
    [anon_sym_do] = ACTIONS(121),
    [anon_sym_when] = ACTIONS(121),
    [anon_sym_EQ_GT] = ACTIONS(2998),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_elsif] = ACTIONS(121),
    [anon_sym_else] = ACTIONS(121),
    [anon_sym_then] = ACTIONS(121),
    [anon_sym_ensure] = ACTIONS(121),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(121),
    [anon_sym_and] = ACTIONS(121),
    [anon_sym_or] = ACTIONS(121),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(121),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [2108] = {
    [sym_heredoc_body] = STATE(2108),
    [ts_builtin_sym_end] = ACTIONS(121),
    [aux_sym_program_token1] = ACTIONS(121),
    [anon_sym_LBRACE] = ACTIONS(121),
    [anon_sym_RBRACE] = ACTIONS(121),
    [anon_sym_RPAREN] = ACTIONS(121),
    [anon_sym_DOT] = ACTIONS(127),
    [anon_sym_end] = ACTIONS(121),
    [anon_sym_rescue] = ACTIONS(121),
    [anon_sym_EQ] = ACTIONS(127),
    [anon_sym_COMMA] = ACTIONS(121),
    [anon_sym_PIPE] = ACTIONS(127),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(121),
    [anon_sym_AMP] = ACTIONS(127),
    [anon_sym_LT] = ACTIONS(127),
    [anon_sym_if] = ACTIONS(121),
    [anon_sym_unless] = ACTIONS(121),
    [anon_sym_while] = ACTIONS(121),
    [anon_sym_until] = ACTIONS(121),
    [anon_sym_in] = ACTIONS(121),
    [anon_sym_do] = ACTIONS(121),
    [anon_sym_when] = ACTIONS(121),
    [anon_sym_EQ_GT] = ACTIONS(121),
    [anon_sym_DOT_DOT] = ACTIONS(127),
    [anon_sym_CARET] = ACTIONS(127),
    [anon_sym_elsif] = ACTIONS(121),
    [anon_sym_else] = ACTIONS(121),
    [anon_sym_then] = ACTIONS(121),
    [anon_sym_ensure] = ACTIONS(121),
    [anon_sym_COLON_COLON2] = ACTIONS(121),
    [anon_sym_AMP_DOT] = ACTIONS(121),
    [anon_sym_PLUS_EQ] = ACTIONS(121),
    [anon_sym_DASH_EQ] = ACTIONS(121),
    [anon_sym_STAR_EQ] = ACTIONS(121),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(121),
    [anon_sym_SLASH_EQ] = ACTIONS(121),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(121),
    [anon_sym_PIPE_EQ] = ACTIONS(121),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(121),
    [anon_sym_AMP_EQ] = ACTIONS(121),
    [anon_sym_PERCENT_EQ] = ACTIONS(121),
    [anon_sym_GT_GT_EQ] = ACTIONS(121),
    [anon_sym_LT_LT_EQ] = ACTIONS(121),
    [anon_sym_CARET_EQ] = ACTIONS(121),
    [anon_sym_QMARK] = ACTIONS(121),
    [anon_sym_and] = ACTIONS(121),
    [anon_sym_or] = ACTIONS(121),
    [anon_sym_PIPE_PIPE] = ACTIONS(127),
    [anon_sym_AMP_AMP] = ACTIONS(127),
    [anon_sym_LT_LT] = ACTIONS(127),
    [anon_sym_GT_GT] = ACTIONS(127),
    [anon_sym_LT_EQ] = ACTIONS(127),
    [anon_sym_GT] = ACTIONS(127),
    [anon_sym_GT_EQ] = ACTIONS(121),
    [anon_sym_PLUS] = ACTIONS(127),
    [anon_sym_SLASH] = ACTIONS(127),
    [anon_sym_PERCENT] = ACTIONS(127),
    [anon_sym_EQ_EQ] = ACTIONS(127),
    [anon_sym_BANG_EQ] = ACTIONS(121),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(121),
    [anon_sym_LT_EQ_GT] = ACTIONS(121),
    [anon_sym_EQ_TILDE] = ACTIONS(121),
    [anon_sym_BANG_TILDE] = ACTIONS(121),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(121),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(121),
    [sym__binary_star] = ACTIONS(121),
    [sym__binary_star_star] = ACTIONS(121),
    [sym__element_reference_bracket] = ACTIONS(121),
  },
  [2109] = {
    [sym_heredoc_body] = STATE(2109),
    [sym_identifier] = ACTIONS(3076),
    [anon_sym_BEGIN] = ACTIONS(3076),
    [anon_sym_LBRACE] = ACTIONS(3078),
    [anon_sym_END] = ACTIONS(3076),
    [anon_sym_def] = ACTIONS(3076),
    [anon_sym_LPAREN] = ACTIONS(3078),
    [anon_sym_COLON_COLON] = ACTIONS(3078),
    [anon_sym_end] = ACTIONS(3076),
    [anon_sym_rescue] = ACTIONS(3076),
    [anon_sym_EQ] = ACTIONS(3078),
    [anon_sym_SEMI] = ACTIONS(3078),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3078),
    [anon_sym_STAR] = ACTIONS(3078),
    [anon_sym_nil] = ACTIONS(3076),
    [anon_sym_class] = ACTIONS(3076),
    [anon_sym_module] = ACTIONS(3076),
    [anon_sym_return] = ACTIONS(3076),
    [anon_sym_yield] = ACTIONS(3076),
    [anon_sym_break] = ACTIONS(3076),
    [anon_sym_next] = ACTIONS(3076),
    [anon_sym_redo] = ACTIONS(3076),
    [anon_sym_retry] = ACTIONS(3076),
    [anon_sym_if] = ACTIONS(3076),
    [anon_sym_unless] = ACTIONS(3076),
    [anon_sym_while] = ACTIONS(3076),
    [anon_sym_until] = ACTIONS(3076),
    [anon_sym_for] = ACTIONS(3076),
    [anon_sym_case] = ACTIONS(3076),
    [anon_sym_LBRACK] = ACTIONS(3078),
    [anon_sym_DOT_DOT] = ACTIONS(3076),
    [anon_sym_else] = ACTIONS(3076),
    [anon_sym_begin] = ACTIONS(3076),
    [anon_sym_ensure] = ACTIONS(3076),
    [anon_sym_PLUS] = ACTIONS(3078),
    [anon_sym_defined_QMARK] = ACTIONS(3078),
    [anon_sym_not] = ACTIONS(3076),
    [anon_sym_BANG] = ACTIONS(3078),
    [anon_sym_TILDE] = ACTIONS(3078),
    [anon_sym_undef] = ACTIONS(3076),
    [anon_sym_alias] = ACTIONS(3076),
    [sym_comment] = ACTIONS(3),
    [sym_integer] = ACTIONS(3076),
    [sym_float] = ACTIONS(3076),
    [sym_super] = ACTIONS(3076),
    [sym_self] = ACTIONS(3076),
    [sym_true] = ACTIONS(3076),
    [sym_false] = ACTIONS(3076),
    [sym_constant] = ACTIONS(3076),
    [aux_sym_constant_suffix_token1] = ACTIONS(3078),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3076),
    [sym_instance_variable] = ACTIONS(3078),
    [sym_class_variable] = ACTIONS(3078),
    [sym_global_variable] = ACTIONS(3078),
    [sym_character] = ACTIONS(3078),
    [anon_sym_DASH_GT] = ACTIONS(3078),
    [sym__line_break] = ACTIONS(3078),
    [sym_simple_symbol] = ACTIONS(3078),
    [sym__string_start] = ACTIONS(3078),
    [sym__symbol_start] = ACTIONS(3078),
    [sym__subshell_start] = ACTIONS(3078),
    [sym__regex_start] = ACTIONS(3078),
    [sym__string_array_start] = ACTIONS(3078),
    [sym__symbol_array_start] = ACTIONS(3078),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3078),
    [sym__unary_minus] = ACTIONS(3078),
    [sym__unary_minus_num] = ACTIONS(3078),
    [sym__binary_minus] = ACTIONS(3078),
    [sym__identifier_suffix] = ACTIONS(3078),
    [sym__constant_suffix] = ACTIONS(3078),
  },
  [2110] = {
    [sym_heredoc_body] = STATE(2110),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_RPAREN] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_end] = ACTIONS(743),
    [anon_sym_rescue] = ACTIONS(743),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2992),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(743),
    [anon_sym_unless] = ACTIONS(743),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_until] = ACTIONS(743),
    [anon_sym_in] = ACTIONS(743),
    [anon_sym_do] = ACTIONS(743),
    [anon_sym_when] = ACTIONS(743),
    [anon_sym_EQ_GT] = ACTIONS(3080),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_elsif] = ACTIONS(743),
    [anon_sym_else] = ACTIONS(743),
    [anon_sym_then] = ACTIONS(743),
    [anon_sym_ensure] = ACTIONS(743),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(743),
    [anon_sym_and] = ACTIONS(743),
    [anon_sym_or] = ACTIONS(743),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [2111] = {
    [sym_heredoc_body] = STATE(2111),
    [ts_builtin_sym_end] = ACTIONS(2960),
    [aux_sym_program_token1] = ACTIONS(2960),
    [anon_sym_LBRACE] = ACTIONS(2960),
    [anon_sym_RBRACE] = ACTIONS(2960),
    [anon_sym_RPAREN] = ACTIONS(2960),
    [anon_sym_DOT] = ACTIONS(2958),
    [anon_sym_end] = ACTIONS(2960),
    [anon_sym_rescue] = ACTIONS(2960),
    [anon_sym_EQ] = ACTIONS(2958),
    [anon_sym_COMMA] = ACTIONS(2960),
    [anon_sym_PIPE] = ACTIONS(2958),
    [anon_sym_SEMI] = ACTIONS(2960),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2960),
    [anon_sym_AMP] = ACTIONS(2958),
    [anon_sym_LT] = ACTIONS(2958),
    [anon_sym_if] = ACTIONS(2960),
    [anon_sym_unless] = ACTIONS(2960),
    [anon_sym_while] = ACTIONS(2960),
    [anon_sym_until] = ACTIONS(2960),
    [anon_sym_in] = ACTIONS(2960),
    [anon_sym_do] = ACTIONS(2960),
    [anon_sym_when] = ACTIONS(2960),
    [anon_sym_EQ_GT] = ACTIONS(2960),
    [anon_sym_DOT_DOT] = ACTIONS(2958),
    [anon_sym_CARET] = ACTIONS(2958),
    [anon_sym_elsif] = ACTIONS(2960),
    [anon_sym_else] = ACTIONS(2960),
    [anon_sym_then] = ACTIONS(2960),
    [anon_sym_ensure] = ACTIONS(2960),
    [anon_sym_COLON_COLON2] = ACTIONS(2960),
    [anon_sym_AMP_DOT] = ACTIONS(2960),
    [anon_sym_PLUS_EQ] = ACTIONS(2960),
    [anon_sym_DASH_EQ] = ACTIONS(2960),
    [anon_sym_STAR_EQ] = ACTIONS(2960),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(2960),
    [anon_sym_SLASH_EQ] = ACTIONS(2960),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(2960),
    [anon_sym_PIPE_EQ] = ACTIONS(2960),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(2960),
    [anon_sym_AMP_EQ] = ACTIONS(2960),
    [anon_sym_PERCENT_EQ] = ACTIONS(2960),
    [anon_sym_GT_GT_EQ] = ACTIONS(2960),
    [anon_sym_LT_LT_EQ] = ACTIONS(2960),
    [anon_sym_CARET_EQ] = ACTIONS(2960),
    [anon_sym_QMARK] = ACTIONS(2960),
    [anon_sym_and] = ACTIONS(2960),
    [anon_sym_or] = ACTIONS(2960),
    [anon_sym_PIPE_PIPE] = ACTIONS(2958),
    [anon_sym_AMP_AMP] = ACTIONS(2958),
    [anon_sym_LT_LT] = ACTIONS(2958),
    [anon_sym_GT_GT] = ACTIONS(2958),
    [anon_sym_LT_EQ] = ACTIONS(2958),
    [anon_sym_GT] = ACTIONS(2958),
    [anon_sym_GT_EQ] = ACTIONS(2960),
    [anon_sym_PLUS] = ACTIONS(2958),
    [anon_sym_SLASH] = ACTIONS(2958),
    [anon_sym_PERCENT] = ACTIONS(2958),
    [anon_sym_EQ_EQ] = ACTIONS(2958),
    [anon_sym_BANG_EQ] = ACTIONS(2960),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2960),
    [anon_sym_LT_EQ_GT] = ACTIONS(2960),
    [anon_sym_EQ_TILDE] = ACTIONS(2960),
    [anon_sym_BANG_TILDE] = ACTIONS(2960),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2960),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(2960),
    [sym__binary_star] = ACTIONS(2960),
    [sym__binary_star_star] = ACTIONS(2960),
    [sym__element_reference_bracket] = ACTIONS(2960),
  },
  [2112] = {
    [sym_heredoc_body] = STATE(2112),
    [ts_builtin_sym_end] = ACTIONS(3082),
    [aux_sym_program_token1] = ACTIONS(3082),
    [anon_sym_LBRACE] = ACTIONS(3082),
    [anon_sym_RBRACE] = ACTIONS(3082),
    [anon_sym_RPAREN] = ACTIONS(3082),
    [anon_sym_DOT] = ACTIONS(3084),
    [anon_sym_end] = ACTIONS(3082),
    [anon_sym_rescue] = ACTIONS(3082),
    [anon_sym_EQ] = ACTIONS(3084),
    [anon_sym_COMMA] = ACTIONS(3082),
    [anon_sym_PIPE] = ACTIONS(3084),
    [anon_sym_SEMI] = ACTIONS(3082),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3082),
    [anon_sym_AMP] = ACTIONS(3084),
    [anon_sym_LT] = ACTIONS(3084),
    [anon_sym_if] = ACTIONS(3082),
    [anon_sym_unless] = ACTIONS(3082),
    [anon_sym_while] = ACTIONS(3082),
    [anon_sym_until] = ACTIONS(3082),
    [anon_sym_in] = ACTIONS(3082),
    [anon_sym_do] = ACTIONS(3082),
    [anon_sym_when] = ACTIONS(3082),
    [anon_sym_EQ_GT] = ACTIONS(3082),
    [anon_sym_DOT_DOT] = ACTIONS(3084),
    [anon_sym_CARET] = ACTIONS(3084),
    [anon_sym_elsif] = ACTIONS(3082),
    [anon_sym_else] = ACTIONS(3082),
    [anon_sym_then] = ACTIONS(3082),
    [anon_sym_ensure] = ACTIONS(3082),
    [anon_sym_COLON_COLON2] = ACTIONS(3082),
    [anon_sym_AMP_DOT] = ACTIONS(3082),
    [anon_sym_PLUS_EQ] = ACTIONS(3082),
    [anon_sym_DASH_EQ] = ACTIONS(3082),
    [anon_sym_STAR_EQ] = ACTIONS(3082),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3082),
    [anon_sym_SLASH_EQ] = ACTIONS(3082),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3082),
    [anon_sym_PIPE_EQ] = ACTIONS(3082),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3082),
    [anon_sym_AMP_EQ] = ACTIONS(3082),
    [anon_sym_PERCENT_EQ] = ACTIONS(3082),
    [anon_sym_GT_GT_EQ] = ACTIONS(3082),
    [anon_sym_LT_LT_EQ] = ACTIONS(3082),
    [anon_sym_CARET_EQ] = ACTIONS(3082),
    [anon_sym_QMARK] = ACTIONS(3082),
    [anon_sym_and] = ACTIONS(3082),
    [anon_sym_or] = ACTIONS(3082),
    [anon_sym_PIPE_PIPE] = ACTIONS(3084),
    [anon_sym_AMP_AMP] = ACTIONS(3084),
    [anon_sym_LT_LT] = ACTIONS(3084),
    [anon_sym_GT_GT] = ACTIONS(3084),
    [anon_sym_LT_EQ] = ACTIONS(3084),
    [anon_sym_GT] = ACTIONS(3084),
    [anon_sym_GT_EQ] = ACTIONS(3082),
    [anon_sym_PLUS] = ACTIONS(3084),
    [anon_sym_SLASH] = ACTIONS(3084),
    [anon_sym_PERCENT] = ACTIONS(3084),
    [anon_sym_EQ_EQ] = ACTIONS(3084),
    [anon_sym_BANG_EQ] = ACTIONS(3082),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3082),
    [anon_sym_LT_EQ_GT] = ACTIONS(3082),
    [anon_sym_EQ_TILDE] = ACTIONS(3082),
    [anon_sym_BANG_TILDE] = ACTIONS(3082),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3082),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3082),
    [sym__binary_star] = ACTIONS(3082),
    [sym__binary_star_star] = ACTIONS(3082),
    [sym__element_reference_bracket] = ACTIONS(3082),
  },
  [2113] = {
    [sym_heredoc_body] = STATE(2113),
    [ts_builtin_sym_end] = ACTIONS(3086),
    [aux_sym_program_token1] = ACTIONS(3086),
    [anon_sym_LBRACE] = ACTIONS(3086),
    [anon_sym_RBRACE] = ACTIONS(3086),
    [anon_sym_RPAREN] = ACTIONS(3086),
    [anon_sym_DOT] = ACTIONS(3088),
    [anon_sym_end] = ACTIONS(3086),
    [anon_sym_rescue] = ACTIONS(3086),
    [anon_sym_EQ] = ACTIONS(3088),
    [anon_sym_COMMA] = ACTIONS(3086),
    [anon_sym_PIPE] = ACTIONS(3088),
    [anon_sym_SEMI] = ACTIONS(3086),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3086),
    [anon_sym_AMP] = ACTIONS(3088),
    [anon_sym_LT] = ACTIONS(3088),
    [anon_sym_if] = ACTIONS(3086),
    [anon_sym_unless] = ACTIONS(3086),
    [anon_sym_while] = ACTIONS(3086),
    [anon_sym_until] = ACTIONS(3086),
    [anon_sym_in] = ACTIONS(3086),
    [anon_sym_do] = ACTIONS(3086),
    [anon_sym_when] = ACTIONS(3086),
    [anon_sym_EQ_GT] = ACTIONS(3086),
    [anon_sym_DOT_DOT] = ACTIONS(3088),
    [anon_sym_CARET] = ACTIONS(3088),
    [anon_sym_elsif] = ACTIONS(3086),
    [anon_sym_else] = ACTIONS(3086),
    [anon_sym_then] = ACTIONS(3086),
    [anon_sym_ensure] = ACTIONS(3086),
    [anon_sym_COLON_COLON2] = ACTIONS(3086),
    [anon_sym_AMP_DOT] = ACTIONS(3086),
    [anon_sym_PLUS_EQ] = ACTIONS(3086),
    [anon_sym_DASH_EQ] = ACTIONS(3086),
    [anon_sym_STAR_EQ] = ACTIONS(3086),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3086),
    [anon_sym_SLASH_EQ] = ACTIONS(3086),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3086),
    [anon_sym_PIPE_EQ] = ACTIONS(3086),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3086),
    [anon_sym_AMP_EQ] = ACTIONS(3086),
    [anon_sym_PERCENT_EQ] = ACTIONS(3086),
    [anon_sym_GT_GT_EQ] = ACTIONS(3086),
    [anon_sym_LT_LT_EQ] = ACTIONS(3086),
    [anon_sym_CARET_EQ] = ACTIONS(3086),
    [anon_sym_QMARK] = ACTIONS(3086),
    [anon_sym_and] = ACTIONS(3086),
    [anon_sym_or] = ACTIONS(3086),
    [anon_sym_PIPE_PIPE] = ACTIONS(3088),
    [anon_sym_AMP_AMP] = ACTIONS(3088),
    [anon_sym_LT_LT] = ACTIONS(3088),
    [anon_sym_GT_GT] = ACTIONS(3088),
    [anon_sym_LT_EQ] = ACTIONS(3088),
    [anon_sym_GT] = ACTIONS(3088),
    [anon_sym_GT_EQ] = ACTIONS(3086),
    [anon_sym_PLUS] = ACTIONS(3088),
    [anon_sym_SLASH] = ACTIONS(3088),
    [anon_sym_PERCENT] = ACTIONS(3088),
    [anon_sym_EQ_EQ] = ACTIONS(3088),
    [anon_sym_BANG_EQ] = ACTIONS(3086),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3086),
    [anon_sym_LT_EQ_GT] = ACTIONS(3086),
    [anon_sym_EQ_TILDE] = ACTIONS(3086),
    [anon_sym_BANG_TILDE] = ACTIONS(3086),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3086),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3086),
    [sym__binary_star] = ACTIONS(3086),
    [sym__binary_star_star] = ACTIONS(3086),
    [sym__element_reference_bracket] = ACTIONS(3086),
  },
  [2114] = {
    [sym_heredoc_body] = STATE(2114),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_LBRACE] = ACTIONS(3090),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3094),
    [anon_sym_COMMA] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_do] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3096),
    [anon_sym_DASH_EQ] = ACTIONS(3096),
    [anon_sym_STAR_EQ] = ACTIONS(3096),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3096),
    [anon_sym_SLASH_EQ] = ACTIONS(3096),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3096),
    [anon_sym_PIPE_EQ] = ACTIONS(3096),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3096),
    [anon_sym_AMP_EQ] = ACTIONS(3096),
    [anon_sym_PERCENT_EQ] = ACTIONS(3096),
    [anon_sym_GT_GT_EQ] = ACTIONS(3096),
    [anon_sym_LT_LT_EQ] = ACTIONS(3096),
    [anon_sym_CARET_EQ] = ACTIONS(3096),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2115] = {
    [sym_heredoc_body] = STATE(2115),
    [sym_identifier] = ACTIONS(3098),
    [anon_sym_BEGIN] = ACTIONS(3098),
    [anon_sym_LBRACE] = ACTIONS(3100),
    [anon_sym_END] = ACTIONS(3098),
    [anon_sym_def] = ACTIONS(3098),
    [anon_sym_LPAREN] = ACTIONS(3100),
    [anon_sym_COLON_COLON] = ACTIONS(3100),
    [anon_sym_end] = ACTIONS(3098),
    [anon_sym_rescue] = ACTIONS(3098),
    [anon_sym_SEMI] = ACTIONS(3100),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3100),
    [anon_sym_STAR] = ACTIONS(3100),
    [anon_sym_nil] = ACTIONS(3098),
    [anon_sym_class] = ACTIONS(3098),
    [anon_sym_module] = ACTIONS(3098),
    [anon_sym_return] = ACTIONS(3098),
    [anon_sym_yield] = ACTIONS(3098),
    [anon_sym_break] = ACTIONS(3098),
    [anon_sym_next] = ACTIONS(3098),
    [anon_sym_redo] = ACTIONS(3098),
    [anon_sym_retry] = ACTIONS(3098),
    [anon_sym_if] = ACTIONS(3098),
    [anon_sym_unless] = ACTIONS(3098),
    [anon_sym_while] = ACTIONS(3098),
    [anon_sym_until] = ACTIONS(3098),
    [anon_sym_for] = ACTIONS(3098),
    [anon_sym_case] = ACTIONS(3098),
    [anon_sym_LBRACK] = ACTIONS(3100),
    [anon_sym_DOT_DOT] = ACTIONS(3098),
    [anon_sym_else] = ACTIONS(3098),
    [anon_sym_begin] = ACTIONS(3098),
    [anon_sym_ensure] = ACTIONS(3098),
    [anon_sym_PLUS] = ACTIONS(3100),
    [anon_sym_defined_QMARK] = ACTIONS(3100),
    [anon_sym_not] = ACTIONS(3098),
    [anon_sym_BANG] = ACTIONS(3100),
    [anon_sym_TILDE] = ACTIONS(3100),
    [anon_sym_undef] = ACTIONS(3098),
    [anon_sym_alias] = ACTIONS(3098),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3098),
    [sym_float] = ACTIONS(3098),
    [sym_super] = ACTIONS(3098),
    [sym_self] = ACTIONS(3098),
    [sym_true] = ACTIONS(3098),
    [sym_false] = ACTIONS(3098),
    [sym_constant] = ACTIONS(3098),
    [aux_sym_constant_suffix_token1] = ACTIONS(3100),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3098),
    [sym_instance_variable] = ACTIONS(3100),
    [sym_class_variable] = ACTIONS(3100),
    [sym_global_variable] = ACTIONS(3100),
    [sym_character] = ACTIONS(3100),
    [anon_sym_DASH_GT] = ACTIONS(3100),
    [sym__line_break] = ACTIONS(3100),
    [sym_simple_symbol] = ACTIONS(3100),
    [sym__string_start] = ACTIONS(3100),
    [sym__symbol_start] = ACTIONS(3100),
    [sym__subshell_start] = ACTIONS(3100),
    [sym__regex_start] = ACTIONS(3100),
    [sym__string_array_start] = ACTIONS(3100),
    [sym__symbol_array_start] = ACTIONS(3100),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3100),
    [sym__unary_minus] = ACTIONS(3100),
    [sym__unary_minus_num] = ACTIONS(3100),
    [sym__binary_minus] = ACTIONS(3100),
    [sym__identifier_suffix] = ACTIONS(3100),
    [sym__constant_suffix] = ACTIONS(3100),
  },
  [2116] = {
    [sym_heredoc_body] = STATE(2116),
    [sym_identifier] = ACTIONS(3102),
    [anon_sym_BEGIN] = ACTIONS(3102),
    [anon_sym_LBRACE] = ACTIONS(3104),
    [anon_sym_END] = ACTIONS(3102),
    [anon_sym_def] = ACTIONS(3102),
    [anon_sym_LPAREN] = ACTIONS(3104),
    [anon_sym_COLON_COLON] = ACTIONS(3104),
    [anon_sym_end] = ACTIONS(3102),
    [anon_sym_rescue] = ACTIONS(3102),
    [anon_sym_SEMI] = ACTIONS(3104),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3104),
    [anon_sym_STAR] = ACTIONS(3104),
    [anon_sym_nil] = ACTIONS(3102),
    [anon_sym_class] = ACTIONS(3102),
    [anon_sym_module] = ACTIONS(3102),
    [anon_sym_return] = ACTIONS(3102),
    [anon_sym_yield] = ACTIONS(3102),
    [anon_sym_break] = ACTIONS(3102),
    [anon_sym_next] = ACTIONS(3102),
    [anon_sym_redo] = ACTIONS(3102),
    [anon_sym_retry] = ACTIONS(3102),
    [anon_sym_if] = ACTIONS(3102),
    [anon_sym_unless] = ACTIONS(3102),
    [anon_sym_while] = ACTIONS(3102),
    [anon_sym_until] = ACTIONS(3102),
    [anon_sym_for] = ACTIONS(3102),
    [anon_sym_case] = ACTIONS(3102),
    [anon_sym_LBRACK] = ACTIONS(3104),
    [anon_sym_DOT_DOT] = ACTIONS(3102),
    [anon_sym_else] = ACTIONS(3102),
    [anon_sym_begin] = ACTIONS(3102),
    [anon_sym_ensure] = ACTIONS(3102),
    [anon_sym_PLUS] = ACTIONS(3104),
    [anon_sym_defined_QMARK] = ACTIONS(3104),
    [anon_sym_not] = ACTIONS(3102),
    [anon_sym_BANG] = ACTIONS(3104),
    [anon_sym_TILDE] = ACTIONS(3104),
    [anon_sym_undef] = ACTIONS(3102),
    [anon_sym_alias] = ACTIONS(3102),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3102),
    [sym_float] = ACTIONS(3102),
    [sym_super] = ACTIONS(3102),
    [sym_self] = ACTIONS(3102),
    [sym_true] = ACTIONS(3102),
    [sym_false] = ACTIONS(3102),
    [sym_constant] = ACTIONS(3102),
    [aux_sym_constant_suffix_token1] = ACTIONS(3104),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3102),
    [sym_instance_variable] = ACTIONS(3104),
    [sym_class_variable] = ACTIONS(3104),
    [sym_global_variable] = ACTIONS(3104),
    [sym_character] = ACTIONS(3104),
    [anon_sym_DASH_GT] = ACTIONS(3104),
    [sym__line_break] = ACTIONS(3104),
    [sym_simple_symbol] = ACTIONS(3104),
    [sym__string_start] = ACTIONS(3104),
    [sym__symbol_start] = ACTIONS(3104),
    [sym__subshell_start] = ACTIONS(3104),
    [sym__regex_start] = ACTIONS(3104),
    [sym__string_array_start] = ACTIONS(3104),
    [sym__symbol_array_start] = ACTIONS(3104),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3104),
    [sym__unary_minus] = ACTIONS(3104),
    [sym__unary_minus_num] = ACTIONS(3104),
    [sym__binary_minus] = ACTIONS(3104),
    [sym__identifier_suffix] = ACTIONS(3104),
    [sym__constant_suffix] = ACTIONS(3104),
  },
  [2117] = {
    [sym_heredoc_body] = STATE(2117),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_LBRACE] = ACTIONS(3090),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3106),
    [anon_sym_COMMA] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_do] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3108),
    [anon_sym_DASH_EQ] = ACTIONS(3108),
    [anon_sym_STAR_EQ] = ACTIONS(3108),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3108),
    [anon_sym_SLASH_EQ] = ACTIONS(3108),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3108),
    [anon_sym_PIPE_EQ] = ACTIONS(3108),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3108),
    [anon_sym_AMP_EQ] = ACTIONS(3108),
    [anon_sym_PERCENT_EQ] = ACTIONS(3108),
    [anon_sym_GT_GT_EQ] = ACTIONS(3108),
    [anon_sym_LT_LT_EQ] = ACTIONS(3108),
    [anon_sym_CARET_EQ] = ACTIONS(3108),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2118] = {
    [sym_heredoc_body] = STATE(2118),
    [sym_identifier] = ACTIONS(3110),
    [anon_sym_BEGIN] = ACTIONS(3110),
    [anon_sym_LBRACE] = ACTIONS(3112),
    [anon_sym_END] = ACTIONS(3110),
    [anon_sym_def] = ACTIONS(3110),
    [anon_sym_LPAREN] = ACTIONS(3112),
    [anon_sym_COLON_COLON] = ACTIONS(3112),
    [anon_sym_end] = ACTIONS(3110),
    [anon_sym_rescue] = ACTIONS(3110),
    [anon_sym_SEMI] = ACTIONS(3112),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3112),
    [anon_sym_STAR] = ACTIONS(3112),
    [anon_sym_nil] = ACTIONS(3110),
    [anon_sym_class] = ACTIONS(3110),
    [anon_sym_module] = ACTIONS(3110),
    [anon_sym_return] = ACTIONS(3110),
    [anon_sym_yield] = ACTIONS(3110),
    [anon_sym_break] = ACTIONS(3110),
    [anon_sym_next] = ACTIONS(3110),
    [anon_sym_redo] = ACTIONS(3110),
    [anon_sym_retry] = ACTIONS(3110),
    [anon_sym_if] = ACTIONS(3110),
    [anon_sym_unless] = ACTIONS(3110),
    [anon_sym_while] = ACTIONS(3110),
    [anon_sym_until] = ACTIONS(3110),
    [anon_sym_for] = ACTIONS(3110),
    [anon_sym_case] = ACTIONS(3110),
    [anon_sym_LBRACK] = ACTIONS(3112),
    [anon_sym_DOT_DOT] = ACTIONS(3110),
    [anon_sym_else] = ACTIONS(3110),
    [anon_sym_begin] = ACTIONS(3110),
    [anon_sym_ensure] = ACTIONS(3110),
    [anon_sym_PLUS] = ACTIONS(3112),
    [anon_sym_defined_QMARK] = ACTIONS(3112),
    [anon_sym_not] = ACTIONS(3110),
    [anon_sym_BANG] = ACTIONS(3112),
    [anon_sym_TILDE] = ACTIONS(3112),
    [anon_sym_undef] = ACTIONS(3110),
    [anon_sym_alias] = ACTIONS(3110),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3110),
    [sym_float] = ACTIONS(3110),
    [sym_super] = ACTIONS(3110),
    [sym_self] = ACTIONS(3110),
    [sym_true] = ACTIONS(3110),
    [sym_false] = ACTIONS(3110),
    [sym_constant] = ACTIONS(3110),
    [aux_sym_constant_suffix_token1] = ACTIONS(3112),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3110),
    [sym_instance_variable] = ACTIONS(3112),
    [sym_class_variable] = ACTIONS(3112),
    [sym_global_variable] = ACTIONS(3112),
    [sym_character] = ACTIONS(3112),
    [anon_sym_DASH_GT] = ACTIONS(3112),
    [sym__line_break] = ACTIONS(3112),
    [sym_simple_symbol] = ACTIONS(3112),
    [sym__string_start] = ACTIONS(3112),
    [sym__symbol_start] = ACTIONS(3112),
    [sym__subshell_start] = ACTIONS(3112),
    [sym__regex_start] = ACTIONS(3112),
    [sym__string_array_start] = ACTIONS(3112),
    [sym__symbol_array_start] = ACTIONS(3112),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3112),
    [sym__unary_minus] = ACTIONS(3112),
    [sym__unary_minus_num] = ACTIONS(3112),
    [sym__binary_minus] = ACTIONS(3112),
    [sym__identifier_suffix] = ACTIONS(3112),
    [sym__constant_suffix] = ACTIONS(3112),
  },
  [2119] = {
    [sym_heredoc_body] = STATE(2119),
    [sym_identifier] = ACTIONS(3114),
    [anon_sym_BEGIN] = ACTIONS(3114),
    [anon_sym_LBRACE] = ACTIONS(3116),
    [anon_sym_END] = ACTIONS(3114),
    [anon_sym_def] = ACTIONS(3114),
    [anon_sym_LPAREN] = ACTIONS(3116),
    [anon_sym_COLON_COLON] = ACTIONS(3116),
    [anon_sym_end] = ACTIONS(3114),
    [anon_sym_rescue] = ACTIONS(3114),
    [anon_sym_SEMI] = ACTIONS(3116),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3116),
    [anon_sym_STAR] = ACTIONS(3116),
    [anon_sym_nil] = ACTIONS(3114),
    [anon_sym_class] = ACTIONS(3114),
    [anon_sym_module] = ACTIONS(3114),
    [anon_sym_return] = ACTIONS(3114),
    [anon_sym_yield] = ACTIONS(3114),
    [anon_sym_break] = ACTIONS(3114),
    [anon_sym_next] = ACTIONS(3114),
    [anon_sym_redo] = ACTIONS(3114),
    [anon_sym_retry] = ACTIONS(3114),
    [anon_sym_if] = ACTIONS(3114),
    [anon_sym_unless] = ACTIONS(3114),
    [anon_sym_while] = ACTIONS(3114),
    [anon_sym_until] = ACTIONS(3114),
    [anon_sym_for] = ACTIONS(3114),
    [anon_sym_case] = ACTIONS(3114),
    [anon_sym_LBRACK] = ACTIONS(3116),
    [anon_sym_DOT_DOT] = ACTIONS(3114),
    [anon_sym_else] = ACTIONS(3114),
    [anon_sym_begin] = ACTIONS(3114),
    [anon_sym_ensure] = ACTIONS(3114),
    [anon_sym_PLUS] = ACTIONS(3116),
    [anon_sym_defined_QMARK] = ACTIONS(3116),
    [anon_sym_not] = ACTIONS(3114),
    [anon_sym_BANG] = ACTIONS(3116),
    [anon_sym_TILDE] = ACTIONS(3116),
    [anon_sym_undef] = ACTIONS(3114),
    [anon_sym_alias] = ACTIONS(3114),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3114),
    [sym_float] = ACTIONS(3114),
    [sym_super] = ACTIONS(3114),
    [sym_self] = ACTIONS(3114),
    [sym_true] = ACTIONS(3114),
    [sym_false] = ACTIONS(3114),
    [sym_constant] = ACTIONS(3114),
    [aux_sym_constant_suffix_token1] = ACTIONS(3116),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3114),
    [sym_instance_variable] = ACTIONS(3116),
    [sym_class_variable] = ACTIONS(3116),
    [sym_global_variable] = ACTIONS(3116),
    [sym_character] = ACTIONS(3116),
    [anon_sym_DASH_GT] = ACTIONS(3116),
    [sym__line_break] = ACTIONS(3116),
    [sym_simple_symbol] = ACTIONS(3116),
    [sym__string_start] = ACTIONS(3116),
    [sym__symbol_start] = ACTIONS(3116),
    [sym__subshell_start] = ACTIONS(3116),
    [sym__regex_start] = ACTIONS(3116),
    [sym__string_array_start] = ACTIONS(3116),
    [sym__symbol_array_start] = ACTIONS(3116),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3116),
    [sym__unary_minus] = ACTIONS(3116),
    [sym__unary_minus_num] = ACTIONS(3116),
    [sym__binary_minus] = ACTIONS(3116),
    [sym__identifier_suffix] = ACTIONS(3116),
    [sym__constant_suffix] = ACTIONS(3116),
  },
  [2120] = {
    [sym_heredoc_body] = STATE(2120),
    [sym_identifier] = ACTIONS(3118),
    [anon_sym_BEGIN] = ACTIONS(3118),
    [anon_sym_LBRACE] = ACTIONS(3120),
    [anon_sym_END] = ACTIONS(3118),
    [anon_sym_def] = ACTIONS(3118),
    [anon_sym_LPAREN] = ACTIONS(3120),
    [anon_sym_COLON_COLON] = ACTIONS(3120),
    [anon_sym_end] = ACTIONS(3118),
    [anon_sym_rescue] = ACTIONS(3118),
    [anon_sym_SEMI] = ACTIONS(3120),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3120),
    [anon_sym_STAR] = ACTIONS(3120),
    [anon_sym_nil] = ACTIONS(3118),
    [anon_sym_class] = ACTIONS(3118),
    [anon_sym_module] = ACTIONS(3118),
    [anon_sym_return] = ACTIONS(3118),
    [anon_sym_yield] = ACTIONS(3118),
    [anon_sym_break] = ACTIONS(3118),
    [anon_sym_next] = ACTIONS(3118),
    [anon_sym_redo] = ACTIONS(3118),
    [anon_sym_retry] = ACTIONS(3118),
    [anon_sym_if] = ACTIONS(3118),
    [anon_sym_unless] = ACTIONS(3118),
    [anon_sym_while] = ACTIONS(3118),
    [anon_sym_until] = ACTIONS(3118),
    [anon_sym_for] = ACTIONS(3118),
    [anon_sym_case] = ACTIONS(3118),
    [anon_sym_LBRACK] = ACTIONS(3120),
    [anon_sym_DOT_DOT] = ACTIONS(3118),
    [anon_sym_else] = ACTIONS(3118),
    [anon_sym_begin] = ACTIONS(3118),
    [anon_sym_ensure] = ACTIONS(3118),
    [anon_sym_PLUS] = ACTIONS(3120),
    [anon_sym_defined_QMARK] = ACTIONS(3120),
    [anon_sym_not] = ACTIONS(3118),
    [anon_sym_BANG] = ACTIONS(3120),
    [anon_sym_TILDE] = ACTIONS(3120),
    [anon_sym_undef] = ACTIONS(3118),
    [anon_sym_alias] = ACTIONS(3118),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3118),
    [sym_float] = ACTIONS(3118),
    [sym_super] = ACTIONS(3118),
    [sym_self] = ACTIONS(3118),
    [sym_true] = ACTIONS(3118),
    [sym_false] = ACTIONS(3118),
    [sym_constant] = ACTIONS(3118),
    [aux_sym_constant_suffix_token1] = ACTIONS(3120),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3118),
    [sym_instance_variable] = ACTIONS(3120),
    [sym_class_variable] = ACTIONS(3120),
    [sym_global_variable] = ACTIONS(3120),
    [sym_character] = ACTIONS(3120),
    [anon_sym_DASH_GT] = ACTIONS(3120),
    [sym__line_break] = ACTIONS(3120),
    [sym_simple_symbol] = ACTIONS(3120),
    [sym__string_start] = ACTIONS(3120),
    [sym__symbol_start] = ACTIONS(3120),
    [sym__subshell_start] = ACTIONS(3120),
    [sym__regex_start] = ACTIONS(3120),
    [sym__string_array_start] = ACTIONS(3120),
    [sym__symbol_array_start] = ACTIONS(3120),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3120),
    [sym__unary_minus] = ACTIONS(3120),
    [sym__unary_minus_num] = ACTIONS(3120),
    [sym__binary_minus] = ACTIONS(3120),
    [sym__identifier_suffix] = ACTIONS(3120),
    [sym__constant_suffix] = ACTIONS(3120),
  },
  [2121] = {
    [sym_heredoc_body] = STATE(2121),
    [sym_identifier] = ACTIONS(2046),
    [anon_sym_BEGIN] = ACTIONS(2046),
    [anon_sym_LBRACE] = ACTIONS(2956),
    [anon_sym_END] = ACTIONS(2046),
    [anon_sym_def] = ACTIONS(2046),
    [anon_sym_LPAREN] = ACTIONS(2956),
    [anon_sym_COLON_COLON] = ACTIONS(2956),
    [anon_sym_end] = ACTIONS(2046),
    [anon_sym_rescue] = ACTIONS(2046),
    [anon_sym_PIPE] = ACTIONS(2956),
    [anon_sym_SEMI] = ACTIONS(2956),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2956),
    [anon_sym_STAR] = ACTIONS(2956),
    [anon_sym_nil] = ACTIONS(2046),
    [anon_sym_class] = ACTIONS(2046),
    [anon_sym_module] = ACTIONS(2046),
    [anon_sym_return] = ACTIONS(2046),
    [anon_sym_yield] = ACTIONS(2046),
    [anon_sym_break] = ACTIONS(2046),
    [anon_sym_next] = ACTIONS(2046),
    [anon_sym_redo] = ACTIONS(2046),
    [anon_sym_retry] = ACTIONS(2046),
    [anon_sym_if] = ACTIONS(2046),
    [anon_sym_unless] = ACTIONS(2046),
    [anon_sym_while] = ACTIONS(2046),
    [anon_sym_until] = ACTIONS(2046),
    [anon_sym_for] = ACTIONS(2046),
    [anon_sym_case] = ACTIONS(2046),
    [anon_sym_LBRACK] = ACTIONS(2956),
    [anon_sym_DOT_DOT] = ACTIONS(2046),
    [anon_sym_else] = ACTIONS(2046),
    [anon_sym_begin] = ACTIONS(2046),
    [anon_sym_ensure] = ACTIONS(2046),
    [anon_sym_PLUS] = ACTIONS(2956),
    [anon_sym_defined_QMARK] = ACTIONS(2956),
    [anon_sym_not] = ACTIONS(2046),
    [anon_sym_BANG] = ACTIONS(2956),
    [anon_sym_TILDE] = ACTIONS(2956),
    [anon_sym_undef] = ACTIONS(2046),
    [anon_sym_alias] = ACTIONS(2046),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2046),
    [sym_float] = ACTIONS(2046),
    [sym_super] = ACTIONS(2046),
    [sym_self] = ACTIONS(2046),
    [sym_true] = ACTIONS(2046),
    [sym_false] = ACTIONS(2046),
    [sym_constant] = ACTIONS(2046),
    [aux_sym_constant_suffix_token1] = ACTIONS(2956),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2046),
    [sym_instance_variable] = ACTIONS(2956),
    [sym_class_variable] = ACTIONS(2956),
    [sym_global_variable] = ACTIONS(2956),
    [sym_character] = ACTIONS(2956),
    [anon_sym_DASH_GT] = ACTIONS(2956),
    [sym_simple_symbol] = ACTIONS(2956),
    [sym__string_start] = ACTIONS(2956),
    [sym__symbol_start] = ACTIONS(2956),
    [sym__subshell_start] = ACTIONS(2956),
    [sym__regex_start] = ACTIONS(2956),
    [sym__string_array_start] = ACTIONS(2956),
    [sym__symbol_array_start] = ACTIONS(2956),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2956),
    [sym__unary_minus] = ACTIONS(2956),
    [sym__unary_minus_num] = ACTIONS(2956),
    [sym__binary_minus] = ACTIONS(2956),
    [sym__identifier_suffix] = ACTIONS(2956),
    [sym__constant_suffix] = ACTIONS(2956),
  },
  [2122] = {
    [sym_heredoc_body] = STATE(2122),
    [sym_identifier] = ACTIONS(3122),
    [anon_sym_BEGIN] = ACTIONS(3122),
    [anon_sym_LBRACE] = ACTIONS(3124),
    [anon_sym_END] = ACTIONS(3122),
    [anon_sym_def] = ACTIONS(3122),
    [anon_sym_LPAREN] = ACTIONS(3124),
    [anon_sym_COLON_COLON] = ACTIONS(3124),
    [anon_sym_end] = ACTIONS(3122),
    [anon_sym_rescue] = ACTIONS(3122),
    [anon_sym_SEMI] = ACTIONS(3124),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3124),
    [anon_sym_STAR] = ACTIONS(3124),
    [anon_sym_nil] = ACTIONS(3122),
    [anon_sym_class] = ACTIONS(3122),
    [anon_sym_module] = ACTIONS(3122),
    [anon_sym_return] = ACTIONS(3122),
    [anon_sym_yield] = ACTIONS(3122),
    [anon_sym_break] = ACTIONS(3122),
    [anon_sym_next] = ACTIONS(3122),
    [anon_sym_redo] = ACTIONS(3122),
    [anon_sym_retry] = ACTIONS(3122),
    [anon_sym_if] = ACTIONS(3122),
    [anon_sym_unless] = ACTIONS(3122),
    [anon_sym_while] = ACTIONS(3122),
    [anon_sym_until] = ACTIONS(3122),
    [anon_sym_for] = ACTIONS(3122),
    [anon_sym_case] = ACTIONS(3122),
    [anon_sym_LBRACK] = ACTIONS(3124),
    [anon_sym_DOT_DOT] = ACTIONS(3122),
    [anon_sym_else] = ACTIONS(3122),
    [anon_sym_begin] = ACTIONS(3122),
    [anon_sym_ensure] = ACTIONS(3122),
    [anon_sym_PLUS] = ACTIONS(3124),
    [anon_sym_defined_QMARK] = ACTIONS(3124),
    [anon_sym_not] = ACTIONS(3122),
    [anon_sym_BANG] = ACTIONS(3124),
    [anon_sym_TILDE] = ACTIONS(3124),
    [anon_sym_undef] = ACTIONS(3122),
    [anon_sym_alias] = ACTIONS(3122),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3122),
    [sym_float] = ACTIONS(3122),
    [sym_super] = ACTIONS(3122),
    [sym_self] = ACTIONS(3122),
    [sym_true] = ACTIONS(3122),
    [sym_false] = ACTIONS(3122),
    [sym_constant] = ACTIONS(3122),
    [aux_sym_constant_suffix_token1] = ACTIONS(3124),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3122),
    [sym_instance_variable] = ACTIONS(3124),
    [sym_class_variable] = ACTIONS(3124),
    [sym_global_variable] = ACTIONS(3124),
    [sym_character] = ACTIONS(3124),
    [anon_sym_DASH_GT] = ACTIONS(3124),
    [sym__line_break] = ACTIONS(3124),
    [sym_simple_symbol] = ACTIONS(3124),
    [sym__string_start] = ACTIONS(3124),
    [sym__symbol_start] = ACTIONS(3124),
    [sym__subshell_start] = ACTIONS(3124),
    [sym__regex_start] = ACTIONS(3124),
    [sym__string_array_start] = ACTIONS(3124),
    [sym__symbol_array_start] = ACTIONS(3124),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3124),
    [sym__unary_minus] = ACTIONS(3124),
    [sym__unary_minus_num] = ACTIONS(3124),
    [sym__binary_minus] = ACTIONS(3124),
    [sym__identifier_suffix] = ACTIONS(3124),
    [sym__constant_suffix] = ACTIONS(3124),
  },
  [2123] = {
    [sym_heredoc_body] = STATE(2123),
    [sym_identifier] = ACTIONS(3126),
    [anon_sym_BEGIN] = ACTIONS(3126),
    [anon_sym_LBRACE] = ACTIONS(3128),
    [anon_sym_END] = ACTIONS(3126),
    [anon_sym_def] = ACTIONS(3126),
    [anon_sym_LPAREN] = ACTIONS(3128),
    [anon_sym_COLON_COLON] = ACTIONS(3128),
    [anon_sym_end] = ACTIONS(3126),
    [anon_sym_rescue] = ACTIONS(3126),
    [anon_sym_SEMI] = ACTIONS(3128),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3128),
    [anon_sym_STAR] = ACTIONS(3128),
    [anon_sym_nil] = ACTIONS(3126),
    [anon_sym_class] = ACTIONS(3126),
    [anon_sym_module] = ACTIONS(3126),
    [anon_sym_return] = ACTIONS(3126),
    [anon_sym_yield] = ACTIONS(3126),
    [anon_sym_break] = ACTIONS(3126),
    [anon_sym_next] = ACTIONS(3126),
    [anon_sym_redo] = ACTIONS(3126),
    [anon_sym_retry] = ACTIONS(3126),
    [anon_sym_if] = ACTIONS(3126),
    [anon_sym_unless] = ACTIONS(3126),
    [anon_sym_while] = ACTIONS(3126),
    [anon_sym_until] = ACTIONS(3126),
    [anon_sym_for] = ACTIONS(3126),
    [anon_sym_case] = ACTIONS(3126),
    [anon_sym_LBRACK] = ACTIONS(3128),
    [anon_sym_DOT_DOT] = ACTIONS(3126),
    [anon_sym_else] = ACTIONS(3126),
    [anon_sym_begin] = ACTIONS(3126),
    [anon_sym_ensure] = ACTIONS(3126),
    [anon_sym_PLUS] = ACTIONS(3128),
    [anon_sym_defined_QMARK] = ACTIONS(3128),
    [anon_sym_not] = ACTIONS(3126),
    [anon_sym_BANG] = ACTIONS(3128),
    [anon_sym_TILDE] = ACTIONS(3128),
    [anon_sym_undef] = ACTIONS(3126),
    [anon_sym_alias] = ACTIONS(3126),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3126),
    [sym_float] = ACTIONS(3126),
    [sym_super] = ACTIONS(3126),
    [sym_self] = ACTIONS(3126),
    [sym_true] = ACTIONS(3126),
    [sym_false] = ACTIONS(3126),
    [sym_constant] = ACTIONS(3126),
    [aux_sym_constant_suffix_token1] = ACTIONS(3128),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3126),
    [sym_instance_variable] = ACTIONS(3128),
    [sym_class_variable] = ACTIONS(3128),
    [sym_global_variable] = ACTIONS(3128),
    [sym_character] = ACTIONS(3128),
    [anon_sym_DASH_GT] = ACTIONS(3128),
    [sym__line_break] = ACTIONS(3128),
    [sym_simple_symbol] = ACTIONS(3128),
    [sym__string_start] = ACTIONS(3128),
    [sym__symbol_start] = ACTIONS(3128),
    [sym__subshell_start] = ACTIONS(3128),
    [sym__regex_start] = ACTIONS(3128),
    [sym__string_array_start] = ACTIONS(3128),
    [sym__symbol_array_start] = ACTIONS(3128),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3128),
    [sym__unary_minus] = ACTIONS(3128),
    [sym__unary_minus_num] = ACTIONS(3128),
    [sym__binary_minus] = ACTIONS(3128),
    [sym__identifier_suffix] = ACTIONS(3128),
    [sym__constant_suffix] = ACTIONS(3128),
  },
  [2124] = {
    [sym_heredoc_body] = STATE(2124),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_LBRACE] = ACTIONS(3090),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3130),
    [anon_sym_COMMA] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_do] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3132),
    [anon_sym_DASH_EQ] = ACTIONS(3132),
    [anon_sym_STAR_EQ] = ACTIONS(3132),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3132),
    [anon_sym_SLASH_EQ] = ACTIONS(3132),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3132),
    [anon_sym_PIPE_EQ] = ACTIONS(3132),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3132),
    [anon_sym_AMP_EQ] = ACTIONS(3132),
    [anon_sym_PERCENT_EQ] = ACTIONS(3132),
    [anon_sym_GT_GT_EQ] = ACTIONS(3132),
    [anon_sym_LT_LT_EQ] = ACTIONS(3132),
    [anon_sym_CARET_EQ] = ACTIONS(3132),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2125] = {
    [sym_heredoc_body] = STATE(2125),
    [sym_identifier] = ACTIONS(3134),
    [anon_sym_BEGIN] = ACTIONS(3134),
    [anon_sym_LBRACE] = ACTIONS(3136),
    [anon_sym_END] = ACTIONS(3134),
    [anon_sym_def] = ACTIONS(3134),
    [anon_sym_LPAREN] = ACTIONS(3136),
    [anon_sym_COLON_COLON] = ACTIONS(3136),
    [anon_sym_end] = ACTIONS(3134),
    [anon_sym_rescue] = ACTIONS(3134),
    [anon_sym_SEMI] = ACTIONS(3136),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3136),
    [anon_sym_STAR] = ACTIONS(3136),
    [anon_sym_nil] = ACTIONS(3134),
    [anon_sym_class] = ACTIONS(3134),
    [anon_sym_module] = ACTIONS(3134),
    [anon_sym_return] = ACTIONS(3134),
    [anon_sym_yield] = ACTIONS(3134),
    [anon_sym_break] = ACTIONS(3134),
    [anon_sym_next] = ACTIONS(3134),
    [anon_sym_redo] = ACTIONS(3134),
    [anon_sym_retry] = ACTIONS(3134),
    [anon_sym_if] = ACTIONS(3134),
    [anon_sym_unless] = ACTIONS(3134),
    [anon_sym_while] = ACTIONS(3134),
    [anon_sym_until] = ACTIONS(3134),
    [anon_sym_for] = ACTIONS(3134),
    [anon_sym_case] = ACTIONS(3134),
    [anon_sym_LBRACK] = ACTIONS(3136),
    [anon_sym_DOT_DOT] = ACTIONS(3134),
    [anon_sym_else] = ACTIONS(3134),
    [anon_sym_begin] = ACTIONS(3134),
    [anon_sym_ensure] = ACTIONS(3134),
    [anon_sym_PLUS] = ACTIONS(3136),
    [anon_sym_defined_QMARK] = ACTIONS(3136),
    [anon_sym_not] = ACTIONS(3134),
    [anon_sym_BANG] = ACTIONS(3136),
    [anon_sym_TILDE] = ACTIONS(3136),
    [anon_sym_undef] = ACTIONS(3134),
    [anon_sym_alias] = ACTIONS(3134),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3134),
    [sym_float] = ACTIONS(3134),
    [sym_super] = ACTIONS(3134),
    [sym_self] = ACTIONS(3134),
    [sym_true] = ACTIONS(3134),
    [sym_false] = ACTIONS(3134),
    [sym_constant] = ACTIONS(3134),
    [aux_sym_constant_suffix_token1] = ACTIONS(3136),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3134),
    [sym_instance_variable] = ACTIONS(3136),
    [sym_class_variable] = ACTIONS(3136),
    [sym_global_variable] = ACTIONS(3136),
    [sym_character] = ACTIONS(3136),
    [anon_sym_DASH_GT] = ACTIONS(3136),
    [sym__line_break] = ACTIONS(3136),
    [sym_simple_symbol] = ACTIONS(3136),
    [sym__string_start] = ACTIONS(3136),
    [sym__symbol_start] = ACTIONS(3136),
    [sym__subshell_start] = ACTIONS(3136),
    [sym__regex_start] = ACTIONS(3136),
    [sym__string_array_start] = ACTIONS(3136),
    [sym__symbol_array_start] = ACTIONS(3136),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3136),
    [sym__unary_minus] = ACTIONS(3136),
    [sym__unary_minus_num] = ACTIONS(3136),
    [sym__binary_minus] = ACTIONS(3136),
    [sym__identifier_suffix] = ACTIONS(3136),
    [sym__constant_suffix] = ACTIONS(3136),
  },
  [2126] = {
    [sym_heredoc_body] = STATE(2126),
    [sym_identifier] = ACTIONS(3138),
    [anon_sym_BEGIN] = ACTIONS(3138),
    [anon_sym_LBRACE] = ACTIONS(3140),
    [anon_sym_END] = ACTIONS(3138),
    [anon_sym_def] = ACTIONS(3138),
    [anon_sym_LPAREN] = ACTIONS(3140),
    [anon_sym_COLON_COLON] = ACTIONS(3140),
    [anon_sym_end] = ACTIONS(3138),
    [anon_sym_rescue] = ACTIONS(3138),
    [anon_sym_SEMI] = ACTIONS(3140),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3140),
    [anon_sym_STAR] = ACTIONS(3140),
    [anon_sym_nil] = ACTIONS(3138),
    [anon_sym_class] = ACTIONS(3138),
    [anon_sym_module] = ACTIONS(3138),
    [anon_sym_return] = ACTIONS(3138),
    [anon_sym_yield] = ACTIONS(3138),
    [anon_sym_break] = ACTIONS(3138),
    [anon_sym_next] = ACTIONS(3138),
    [anon_sym_redo] = ACTIONS(3138),
    [anon_sym_retry] = ACTIONS(3138),
    [anon_sym_if] = ACTIONS(3138),
    [anon_sym_unless] = ACTIONS(3138),
    [anon_sym_while] = ACTIONS(3138),
    [anon_sym_until] = ACTIONS(3138),
    [anon_sym_for] = ACTIONS(3138),
    [anon_sym_case] = ACTIONS(3138),
    [anon_sym_LBRACK] = ACTIONS(3140),
    [anon_sym_DOT_DOT] = ACTIONS(3138),
    [anon_sym_else] = ACTIONS(3138),
    [anon_sym_begin] = ACTIONS(3138),
    [anon_sym_ensure] = ACTIONS(3138),
    [anon_sym_PLUS] = ACTIONS(3140),
    [anon_sym_defined_QMARK] = ACTIONS(3140),
    [anon_sym_not] = ACTIONS(3138),
    [anon_sym_BANG] = ACTIONS(3140),
    [anon_sym_TILDE] = ACTIONS(3140),
    [anon_sym_undef] = ACTIONS(3138),
    [anon_sym_alias] = ACTIONS(3138),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3138),
    [sym_float] = ACTIONS(3138),
    [sym_super] = ACTIONS(3138),
    [sym_self] = ACTIONS(3138),
    [sym_true] = ACTIONS(3138),
    [sym_false] = ACTIONS(3138),
    [sym_constant] = ACTIONS(3138),
    [aux_sym_constant_suffix_token1] = ACTIONS(3140),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3138),
    [sym_instance_variable] = ACTIONS(3140),
    [sym_class_variable] = ACTIONS(3140),
    [sym_global_variable] = ACTIONS(3140),
    [sym_character] = ACTIONS(3140),
    [anon_sym_DASH_GT] = ACTIONS(3140),
    [sym__line_break] = ACTIONS(3140),
    [sym_simple_symbol] = ACTIONS(3140),
    [sym__string_start] = ACTIONS(3140),
    [sym__symbol_start] = ACTIONS(3140),
    [sym__subshell_start] = ACTIONS(3140),
    [sym__regex_start] = ACTIONS(3140),
    [sym__string_array_start] = ACTIONS(3140),
    [sym__symbol_array_start] = ACTIONS(3140),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3140),
    [sym__unary_minus] = ACTIONS(3140),
    [sym__unary_minus_num] = ACTIONS(3140),
    [sym__binary_minus] = ACTIONS(3140),
    [sym__identifier_suffix] = ACTIONS(3140),
    [sym__constant_suffix] = ACTIONS(3140),
  },
  [2127] = {
    [sym_heredoc_body] = STATE(2127),
    [sym_identifier] = ACTIONS(3142),
    [anon_sym_BEGIN] = ACTIONS(3142),
    [anon_sym_LBRACE] = ACTIONS(3144),
    [anon_sym_END] = ACTIONS(3142),
    [anon_sym_def] = ACTIONS(3142),
    [anon_sym_LPAREN] = ACTIONS(3144),
    [anon_sym_COLON_COLON] = ACTIONS(3144),
    [anon_sym_end] = ACTIONS(3142),
    [anon_sym_rescue] = ACTIONS(3142),
    [anon_sym_SEMI] = ACTIONS(3144),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3144),
    [anon_sym_STAR] = ACTIONS(3144),
    [anon_sym_nil] = ACTIONS(3142),
    [anon_sym_class] = ACTIONS(3142),
    [anon_sym_module] = ACTIONS(3142),
    [anon_sym_return] = ACTIONS(3142),
    [anon_sym_yield] = ACTIONS(3142),
    [anon_sym_break] = ACTIONS(3142),
    [anon_sym_next] = ACTIONS(3142),
    [anon_sym_redo] = ACTIONS(3142),
    [anon_sym_retry] = ACTIONS(3142),
    [anon_sym_if] = ACTIONS(3142),
    [anon_sym_unless] = ACTIONS(3142),
    [anon_sym_while] = ACTIONS(3142),
    [anon_sym_until] = ACTIONS(3142),
    [anon_sym_for] = ACTIONS(3142),
    [anon_sym_case] = ACTIONS(3142),
    [anon_sym_LBRACK] = ACTIONS(3144),
    [anon_sym_DOT_DOT] = ACTIONS(3142),
    [anon_sym_else] = ACTIONS(3142),
    [anon_sym_begin] = ACTIONS(3142),
    [anon_sym_ensure] = ACTIONS(3142),
    [anon_sym_PLUS] = ACTIONS(3144),
    [anon_sym_defined_QMARK] = ACTIONS(3144),
    [anon_sym_not] = ACTIONS(3142),
    [anon_sym_BANG] = ACTIONS(3144),
    [anon_sym_TILDE] = ACTIONS(3144),
    [anon_sym_undef] = ACTIONS(3142),
    [anon_sym_alias] = ACTIONS(3142),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3142),
    [sym_float] = ACTIONS(3142),
    [sym_super] = ACTIONS(3142),
    [sym_self] = ACTIONS(3142),
    [sym_true] = ACTIONS(3142),
    [sym_false] = ACTIONS(3142),
    [sym_constant] = ACTIONS(3142),
    [aux_sym_constant_suffix_token1] = ACTIONS(3144),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3142),
    [sym_instance_variable] = ACTIONS(3144),
    [sym_class_variable] = ACTIONS(3144),
    [sym_global_variable] = ACTIONS(3144),
    [sym_character] = ACTIONS(3144),
    [anon_sym_DASH_GT] = ACTIONS(3144),
    [sym__line_break] = ACTIONS(3144),
    [sym_simple_symbol] = ACTIONS(3144),
    [sym__string_start] = ACTIONS(3144),
    [sym__symbol_start] = ACTIONS(3144),
    [sym__subshell_start] = ACTIONS(3144),
    [sym__regex_start] = ACTIONS(3144),
    [sym__string_array_start] = ACTIONS(3144),
    [sym__symbol_array_start] = ACTIONS(3144),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3144),
    [sym__unary_minus] = ACTIONS(3144),
    [sym__unary_minus_num] = ACTIONS(3144),
    [sym__binary_minus] = ACTIONS(3144),
    [sym__identifier_suffix] = ACTIONS(3144),
    [sym__constant_suffix] = ACTIONS(3144),
  },
  [2128] = {
    [sym_heredoc_body] = STATE(2128),
    [sym_identifier] = ACTIONS(3146),
    [anon_sym_BEGIN] = ACTIONS(3146),
    [anon_sym_LBRACE] = ACTIONS(3148),
    [anon_sym_END] = ACTIONS(3146),
    [anon_sym_def] = ACTIONS(3146),
    [anon_sym_LPAREN] = ACTIONS(3148),
    [anon_sym_COLON_COLON] = ACTIONS(3148),
    [anon_sym_end] = ACTIONS(3146),
    [anon_sym_rescue] = ACTIONS(3146),
    [anon_sym_SEMI] = ACTIONS(3148),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3148),
    [anon_sym_STAR] = ACTIONS(3148),
    [anon_sym_nil] = ACTIONS(3146),
    [anon_sym_class] = ACTIONS(3146),
    [anon_sym_module] = ACTIONS(3146),
    [anon_sym_return] = ACTIONS(3146),
    [anon_sym_yield] = ACTIONS(3146),
    [anon_sym_break] = ACTIONS(3146),
    [anon_sym_next] = ACTIONS(3146),
    [anon_sym_redo] = ACTIONS(3146),
    [anon_sym_retry] = ACTIONS(3146),
    [anon_sym_if] = ACTIONS(3146),
    [anon_sym_unless] = ACTIONS(3146),
    [anon_sym_while] = ACTIONS(3146),
    [anon_sym_until] = ACTIONS(3146),
    [anon_sym_for] = ACTIONS(3146),
    [anon_sym_case] = ACTIONS(3146),
    [anon_sym_LBRACK] = ACTIONS(3148),
    [anon_sym_DOT_DOT] = ACTIONS(3146),
    [anon_sym_else] = ACTIONS(3146),
    [anon_sym_begin] = ACTIONS(3146),
    [anon_sym_ensure] = ACTIONS(3146),
    [anon_sym_PLUS] = ACTIONS(3148),
    [anon_sym_defined_QMARK] = ACTIONS(3148),
    [anon_sym_not] = ACTIONS(3146),
    [anon_sym_BANG] = ACTIONS(3148),
    [anon_sym_TILDE] = ACTIONS(3148),
    [anon_sym_undef] = ACTIONS(3146),
    [anon_sym_alias] = ACTIONS(3146),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3146),
    [sym_float] = ACTIONS(3146),
    [sym_super] = ACTIONS(3146),
    [sym_self] = ACTIONS(3146),
    [sym_true] = ACTIONS(3146),
    [sym_false] = ACTIONS(3146),
    [sym_constant] = ACTIONS(3146),
    [aux_sym_constant_suffix_token1] = ACTIONS(3148),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3146),
    [sym_instance_variable] = ACTIONS(3148),
    [sym_class_variable] = ACTIONS(3148),
    [sym_global_variable] = ACTIONS(3148),
    [sym_character] = ACTIONS(3148),
    [anon_sym_DASH_GT] = ACTIONS(3148),
    [sym__line_break] = ACTIONS(3148),
    [sym_simple_symbol] = ACTIONS(3148),
    [sym__string_start] = ACTIONS(3148),
    [sym__symbol_start] = ACTIONS(3148),
    [sym__subshell_start] = ACTIONS(3148),
    [sym__regex_start] = ACTIONS(3148),
    [sym__string_array_start] = ACTIONS(3148),
    [sym__symbol_array_start] = ACTIONS(3148),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3148),
    [sym__unary_minus] = ACTIONS(3148),
    [sym__unary_minus_num] = ACTIONS(3148),
    [sym__binary_minus] = ACTIONS(3148),
    [sym__identifier_suffix] = ACTIONS(3148),
    [sym__constant_suffix] = ACTIONS(3148),
  },
  [2129] = {
    [sym_heredoc_body] = STATE(2129),
    [sym_identifier] = ACTIONS(3150),
    [anon_sym_BEGIN] = ACTIONS(3150),
    [anon_sym_LBRACE] = ACTIONS(3152),
    [anon_sym_END] = ACTIONS(3150),
    [anon_sym_def] = ACTIONS(3150),
    [anon_sym_LPAREN] = ACTIONS(3152),
    [anon_sym_COLON_COLON] = ACTIONS(3152),
    [anon_sym_end] = ACTIONS(3150),
    [anon_sym_rescue] = ACTIONS(3150),
    [anon_sym_SEMI] = ACTIONS(3152),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3152),
    [anon_sym_STAR] = ACTIONS(3152),
    [anon_sym_nil] = ACTIONS(3150),
    [anon_sym_class] = ACTIONS(3150),
    [anon_sym_module] = ACTIONS(3150),
    [anon_sym_return] = ACTIONS(3150),
    [anon_sym_yield] = ACTIONS(3150),
    [anon_sym_break] = ACTIONS(3150),
    [anon_sym_next] = ACTIONS(3150),
    [anon_sym_redo] = ACTIONS(3150),
    [anon_sym_retry] = ACTIONS(3150),
    [anon_sym_if] = ACTIONS(3150),
    [anon_sym_unless] = ACTIONS(3150),
    [anon_sym_while] = ACTIONS(3150),
    [anon_sym_until] = ACTIONS(3150),
    [anon_sym_for] = ACTIONS(3150),
    [anon_sym_case] = ACTIONS(3150),
    [anon_sym_LBRACK] = ACTIONS(3152),
    [anon_sym_DOT_DOT] = ACTIONS(3150),
    [anon_sym_else] = ACTIONS(3150),
    [anon_sym_begin] = ACTIONS(3150),
    [anon_sym_ensure] = ACTIONS(3150),
    [anon_sym_PLUS] = ACTIONS(3152),
    [anon_sym_defined_QMARK] = ACTIONS(3152),
    [anon_sym_not] = ACTIONS(3150),
    [anon_sym_BANG] = ACTIONS(3152),
    [anon_sym_TILDE] = ACTIONS(3152),
    [anon_sym_undef] = ACTIONS(3150),
    [anon_sym_alias] = ACTIONS(3150),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3150),
    [sym_float] = ACTIONS(3150),
    [sym_super] = ACTIONS(3150),
    [sym_self] = ACTIONS(3150),
    [sym_true] = ACTIONS(3150),
    [sym_false] = ACTIONS(3150),
    [sym_constant] = ACTIONS(3150),
    [aux_sym_constant_suffix_token1] = ACTIONS(3152),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3150),
    [sym_instance_variable] = ACTIONS(3152),
    [sym_class_variable] = ACTIONS(3152),
    [sym_global_variable] = ACTIONS(3152),
    [sym_character] = ACTIONS(3152),
    [anon_sym_DASH_GT] = ACTIONS(3152),
    [sym__line_break] = ACTIONS(3152),
    [sym_simple_symbol] = ACTIONS(3152),
    [sym__string_start] = ACTIONS(3152),
    [sym__symbol_start] = ACTIONS(3152),
    [sym__subshell_start] = ACTIONS(3152),
    [sym__regex_start] = ACTIONS(3152),
    [sym__string_array_start] = ACTIONS(3152),
    [sym__symbol_array_start] = ACTIONS(3152),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3152),
    [sym__unary_minus] = ACTIONS(3152),
    [sym__unary_minus_num] = ACTIONS(3152),
    [sym__binary_minus] = ACTIONS(3152),
    [sym__identifier_suffix] = ACTIONS(3152),
    [sym__constant_suffix] = ACTIONS(3152),
  },
  [2130] = {
    [sym_heredoc_body] = STATE(2130),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3154),
    [anon_sym_COMMA] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3156),
    [anon_sym_DASH_EQ] = ACTIONS(3156),
    [anon_sym_STAR_EQ] = ACTIONS(3156),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3156),
    [anon_sym_SLASH_EQ] = ACTIONS(3156),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3156),
    [anon_sym_PIPE_EQ] = ACTIONS(3156),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3156),
    [anon_sym_AMP_EQ] = ACTIONS(3156),
    [anon_sym_PERCENT_EQ] = ACTIONS(3156),
    [anon_sym_GT_GT_EQ] = ACTIONS(3156),
    [anon_sym_LT_LT_EQ] = ACTIONS(3156),
    [anon_sym_CARET_EQ] = ACTIONS(3156),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2131] = {
    [sym_heredoc_body] = STATE(2131),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3158),
    [anon_sym_COMMA] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3160),
    [anon_sym_DASH_EQ] = ACTIONS(3160),
    [anon_sym_STAR_EQ] = ACTIONS(3160),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3160),
    [anon_sym_SLASH_EQ] = ACTIONS(3160),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3160),
    [anon_sym_PIPE_EQ] = ACTIONS(3160),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3160),
    [anon_sym_AMP_EQ] = ACTIONS(3160),
    [anon_sym_PERCENT_EQ] = ACTIONS(3160),
    [anon_sym_GT_GT_EQ] = ACTIONS(3160),
    [anon_sym_LT_LT_EQ] = ACTIONS(3160),
    [anon_sym_CARET_EQ] = ACTIONS(3160),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2132] = {
    [sym_heredoc_body] = STATE(2132),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3162),
    [anon_sym_COMMA] = ACTIONS(3164),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3166),
    [anon_sym_DASH_EQ] = ACTIONS(3166),
    [anon_sym_STAR_EQ] = ACTIONS(3166),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3166),
    [anon_sym_SLASH_EQ] = ACTIONS(3166),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3166),
    [anon_sym_PIPE_EQ] = ACTIONS(3166),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3166),
    [anon_sym_AMP_EQ] = ACTIONS(3166),
    [anon_sym_PERCENT_EQ] = ACTIONS(3166),
    [anon_sym_GT_GT_EQ] = ACTIONS(3166),
    [anon_sym_LT_LT_EQ] = ACTIONS(3166),
    [anon_sym_CARET_EQ] = ACTIONS(3166),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2133] = {
    [sym_splat_parameter] = STATE(4518),
    [sym__pattern_expr] = STATE(4479),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2133),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2812),
    [anon_sym_STAR] = ACTIONS(2814),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2820),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2134] = {
    [sym_splat_parameter] = STATE(4518),
    [sym__pattern_expr] = STATE(4479),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2134),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2934),
    [anon_sym_STAR] = ACTIONS(2944),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2942),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2135] = {
    [sym_splat_parameter] = STATE(5011),
    [sym__pattern_expr] = STATE(4894),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2135),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2904),
    [anon_sym_STAR] = ACTIONS(2952),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2908),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2910),
    [sym_float] = ACTIONS(2910),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2908),
  },
  [2136] = {
    [sym_heredoc_body] = STATE(2136),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3168),
    [anon_sym_COMMA] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3170),
    [anon_sym_DASH_EQ] = ACTIONS(3170),
    [anon_sym_STAR_EQ] = ACTIONS(3170),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3170),
    [anon_sym_SLASH_EQ] = ACTIONS(3170),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3170),
    [anon_sym_PIPE_EQ] = ACTIONS(3170),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3170),
    [anon_sym_AMP_EQ] = ACTIONS(3170),
    [anon_sym_PERCENT_EQ] = ACTIONS(3170),
    [anon_sym_GT_GT_EQ] = ACTIONS(3170),
    [anon_sym_LT_LT_EQ] = ACTIONS(3170),
    [anon_sym_CARET_EQ] = ACTIONS(3170),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2137] = {
    [sym_splat_parameter] = STATE(4518),
    [sym__pattern_expr] = STATE(4479),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2137),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_STAR] = ACTIONS(2814),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
  },
  [2138] = {
    [sym_splat_parameter] = STATE(5011),
    [sym__pattern_expr] = STATE(4894),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2138),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_STAR] = ACTIONS(2952),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2139] = {
    [sym_heredoc_body] = STATE(2139),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3172),
    [anon_sym_COMMA] = ACTIONS(3164),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3174),
    [anon_sym_DASH_EQ] = ACTIONS(3174),
    [anon_sym_STAR_EQ] = ACTIONS(3174),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3174),
    [anon_sym_SLASH_EQ] = ACTIONS(3174),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3174),
    [anon_sym_PIPE_EQ] = ACTIONS(3174),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3174),
    [anon_sym_AMP_EQ] = ACTIONS(3174),
    [anon_sym_PERCENT_EQ] = ACTIONS(3174),
    [anon_sym_GT_GT_EQ] = ACTIONS(3174),
    [anon_sym_LT_LT_EQ] = ACTIONS(3174),
    [anon_sym_CARET_EQ] = ACTIONS(3174),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2140] = {
    [sym_heredoc_body] = STATE(2140),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3176),
    [anon_sym_COMMA] = ACTIONS(3090),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3178),
    [anon_sym_DASH_EQ] = ACTIONS(3178),
    [anon_sym_STAR_EQ] = ACTIONS(3178),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3178),
    [anon_sym_SLASH_EQ] = ACTIONS(3178),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3178),
    [anon_sym_PIPE_EQ] = ACTIONS(3178),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3178),
    [anon_sym_AMP_EQ] = ACTIONS(3178),
    [anon_sym_PERCENT_EQ] = ACTIONS(3178),
    [anon_sym_GT_GT_EQ] = ACTIONS(3178),
    [anon_sym_LT_LT_EQ] = ACTIONS(3178),
    [anon_sym_CARET_EQ] = ACTIONS(3178),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2141] = {
    [sym_heredoc_body] = STATE(2141),
    [aux_sym__mlhs_repeat1] = STATE(5304),
    [anon_sym_RBRACE] = ACTIONS(3090),
    [anon_sym_RPAREN] = ACTIONS(3090),
    [anon_sym_DOT] = ACTIONS(3092),
    [anon_sym_end] = ACTIONS(3090),
    [anon_sym_rescue] = ACTIONS(3090),
    [anon_sym_EQ] = ACTIONS(3180),
    [anon_sym_COMMA] = ACTIONS(3164),
    [anon_sym_PIPE] = ACTIONS(3092),
    [anon_sym_SEMI] = ACTIONS(3090),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3090),
    [anon_sym_AMP] = ACTIONS(3092),
    [anon_sym_LT] = ACTIONS(3092),
    [anon_sym_if] = ACTIONS(3090),
    [anon_sym_unless] = ACTIONS(3090),
    [anon_sym_while] = ACTIONS(3090),
    [anon_sym_until] = ACTIONS(3090),
    [anon_sym_in] = ACTIONS(3090),
    [anon_sym_when] = ACTIONS(3090),
    [anon_sym_EQ_GT] = ACTIONS(3090),
    [anon_sym_DOT_DOT] = ACTIONS(3092),
    [anon_sym_CARET] = ACTIONS(3092),
    [anon_sym_elsif] = ACTIONS(3090),
    [anon_sym_else] = ACTIONS(3090),
    [anon_sym_then] = ACTIONS(3090),
    [anon_sym_ensure] = ACTIONS(3090),
    [anon_sym_COLON_COLON2] = ACTIONS(3090),
    [anon_sym_AMP_DOT] = ACTIONS(3090),
    [anon_sym_PLUS_EQ] = ACTIONS(3182),
    [anon_sym_DASH_EQ] = ACTIONS(3182),
    [anon_sym_STAR_EQ] = ACTIONS(3182),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(3182),
    [anon_sym_SLASH_EQ] = ACTIONS(3182),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(3182),
    [anon_sym_PIPE_EQ] = ACTIONS(3182),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(3182),
    [anon_sym_AMP_EQ] = ACTIONS(3182),
    [anon_sym_PERCENT_EQ] = ACTIONS(3182),
    [anon_sym_GT_GT_EQ] = ACTIONS(3182),
    [anon_sym_LT_LT_EQ] = ACTIONS(3182),
    [anon_sym_CARET_EQ] = ACTIONS(3182),
    [anon_sym_QMARK] = ACTIONS(3090),
    [anon_sym_and] = ACTIONS(3090),
    [anon_sym_or] = ACTIONS(3090),
    [anon_sym_PIPE_PIPE] = ACTIONS(3092),
    [anon_sym_AMP_AMP] = ACTIONS(3092),
    [anon_sym_LT_LT] = ACTIONS(3092),
    [anon_sym_GT_GT] = ACTIONS(3092),
    [anon_sym_LT_EQ] = ACTIONS(3092),
    [anon_sym_GT] = ACTIONS(3092),
    [anon_sym_GT_EQ] = ACTIONS(3090),
    [anon_sym_PLUS] = ACTIONS(3092),
    [anon_sym_SLASH] = ACTIONS(3092),
    [anon_sym_PERCENT] = ACTIONS(3092),
    [anon_sym_EQ_EQ] = ACTIONS(3092),
    [anon_sym_BANG_EQ] = ACTIONS(3090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3090),
    [anon_sym_LT_EQ_GT] = ACTIONS(3090),
    [anon_sym_EQ_TILDE] = ACTIONS(3090),
    [anon_sym_BANG_TILDE] = ACTIONS(3090),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(3090),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(3090),
    [sym__binary_star] = ACTIONS(3090),
    [sym__binary_star_star] = ACTIONS(3090),
    [sym__element_reference_bracket] = ACTIONS(3090),
  },
  [2142] = {
    [sym_heredoc_body] = STATE(2142),
    [ts_builtin_sym_end] = ACTIONS(2956),
    [sym_identifier] = ACTIONS(2046),
    [aux_sym_program_token1] = ACTIONS(2046),
    [anon_sym_BEGIN] = ACTIONS(2046),
    [anon_sym_LBRACE] = ACTIONS(2956),
    [anon_sym_END] = ACTIONS(2046),
    [anon_sym_def] = ACTIONS(2046),
    [anon_sym_LPAREN] = ACTIONS(2956),
    [anon_sym_COLON_COLON] = ACTIONS(2956),
    [anon_sym_SEMI] = ACTIONS(2956),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2956),
    [anon_sym_STAR] = ACTIONS(2956),
    [anon_sym_nil] = ACTIONS(2046),
    [anon_sym_class] = ACTIONS(2046),
    [anon_sym_module] = ACTIONS(2046),
    [anon_sym_return] = ACTIONS(2046),
    [anon_sym_yield] = ACTIONS(2046),
    [anon_sym_break] = ACTIONS(2046),
    [anon_sym_next] = ACTIONS(2046),
    [anon_sym_redo] = ACTIONS(2046),
    [anon_sym_retry] = ACTIONS(2046),
    [anon_sym_if] = ACTIONS(2046),
    [anon_sym_unless] = ACTIONS(2046),
    [anon_sym_while] = ACTIONS(2046),
    [anon_sym_until] = ACTIONS(2046),
    [anon_sym_for] = ACTIONS(2046),
    [anon_sym_case] = ACTIONS(2046),
    [anon_sym_LBRACK] = ACTIONS(2956),
    [anon_sym_DOT_DOT] = ACTIONS(2046),
    [anon_sym_begin] = ACTIONS(2046),
    [anon_sym_PLUS] = ACTIONS(2956),
    [anon_sym_defined_QMARK] = ACTIONS(2956),
    [anon_sym_not] = ACTIONS(2046),
    [anon_sym_BANG] = ACTIONS(2956),
    [anon_sym_TILDE] = ACTIONS(2956),
    [anon_sym_undef] = ACTIONS(2046),
    [anon_sym_alias] = ACTIONS(2046),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2046),
    [sym_float] = ACTIONS(2046),
    [sym_super] = ACTIONS(2046),
    [sym_self] = ACTIONS(2046),
    [sym_true] = ACTIONS(2046),
    [sym_false] = ACTIONS(2046),
    [sym_constant] = ACTIONS(2046),
    [aux_sym_constant_suffix_token1] = ACTIONS(2956),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2046),
    [sym_instance_variable] = ACTIONS(2956),
    [sym_class_variable] = ACTIONS(2956),
    [sym_global_variable] = ACTIONS(2956),
    [sym_character] = ACTIONS(2956),
    [anon_sym_DASH_GT] = ACTIONS(2956),
    [sym_simple_symbol] = ACTIONS(2956),
    [sym__string_start] = ACTIONS(2956),
    [sym__symbol_start] = ACTIONS(2956),
    [sym__subshell_start] = ACTIONS(2956),
    [sym__regex_start] = ACTIONS(2956),
    [sym__string_array_start] = ACTIONS(2956),
    [sym__symbol_array_start] = ACTIONS(2956),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2956),
    [sym__unary_minus] = ACTIONS(2956),
    [sym__unary_minus_num] = ACTIONS(2956),
    [sym__binary_minus] = ACTIONS(2956),
    [sym__identifier_suffix] = ACTIONS(2956),
    [sym__constant_suffix] = ACTIONS(2956),
  },
  [2143] = {
    [sym_heredoc_body] = STATE(2143),
    [ts_builtin_sym_end] = ACTIONS(2968),
    [sym_identifier] = ACTIONS(2966),
    [aux_sym_program_token1] = ACTIONS(2966),
    [anon_sym_BEGIN] = ACTIONS(2966),
    [anon_sym_LBRACE] = ACTIONS(2968),
    [anon_sym_END] = ACTIONS(2966),
    [anon_sym_def] = ACTIONS(2966),
    [anon_sym_LPAREN] = ACTIONS(2968),
    [anon_sym_COLON_COLON] = ACTIONS(2968),
    [anon_sym_SEMI] = ACTIONS(2968),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2968),
    [anon_sym_STAR] = ACTIONS(2968),
    [anon_sym_nil] = ACTIONS(2966),
    [anon_sym_class] = ACTIONS(2966),
    [anon_sym_module] = ACTIONS(2966),
    [anon_sym_return] = ACTIONS(2966),
    [anon_sym_yield] = ACTIONS(2966),
    [anon_sym_break] = ACTIONS(2966),
    [anon_sym_next] = ACTIONS(2966),
    [anon_sym_redo] = ACTIONS(2966),
    [anon_sym_retry] = ACTIONS(2966),
    [anon_sym_if] = ACTIONS(2966),
    [anon_sym_unless] = ACTIONS(2966),
    [anon_sym_while] = ACTIONS(2966),
    [anon_sym_until] = ACTIONS(2966),
    [anon_sym_for] = ACTIONS(2966),
    [anon_sym_case] = ACTIONS(2966),
    [anon_sym_LBRACK] = ACTIONS(2968),
    [anon_sym_DOT_DOT] = ACTIONS(2966),
    [anon_sym_begin] = ACTIONS(2966),
    [anon_sym_PLUS] = ACTIONS(2968),
    [anon_sym_defined_QMARK] = ACTIONS(2968),
    [anon_sym_not] = ACTIONS(2966),
    [anon_sym_BANG] = ACTIONS(2968),
    [anon_sym_TILDE] = ACTIONS(2968),
    [anon_sym_undef] = ACTIONS(2966),
    [anon_sym_alias] = ACTIONS(2966),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2966),
    [sym_float] = ACTIONS(2966),
    [sym_super] = ACTIONS(2966),
    [sym_self] = ACTIONS(2966),
    [sym_true] = ACTIONS(2966),
    [sym_false] = ACTIONS(2966),
    [sym_constant] = ACTIONS(2966),
    [aux_sym_constant_suffix_token1] = ACTIONS(2968),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2966),
    [sym_instance_variable] = ACTIONS(2968),
    [sym_class_variable] = ACTIONS(2968),
    [sym_global_variable] = ACTIONS(2968),
    [sym_character] = ACTIONS(2968),
    [anon_sym_DASH_GT] = ACTIONS(2968),
    [sym_simple_symbol] = ACTIONS(2968),
    [sym__string_start] = ACTIONS(2968),
    [sym__symbol_start] = ACTIONS(2968),
    [sym__subshell_start] = ACTIONS(2968),
    [sym__regex_start] = ACTIONS(2968),
    [sym__string_array_start] = ACTIONS(2968),
    [sym__symbol_array_start] = ACTIONS(2968),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2968),
    [sym__unary_minus] = ACTIONS(2968),
    [sym__unary_minus_num] = ACTIONS(2968),
    [sym__binary_minus] = ACTIONS(2968),
    [sym__identifier_suffix] = ACTIONS(2968),
    [sym__constant_suffix] = ACTIONS(2968),
  },
  [2144] = {
    [sym_heredoc_body] = STATE(2144),
    [ts_builtin_sym_end] = ACTIONS(2972),
    [sym_identifier] = ACTIONS(2970),
    [aux_sym_program_token1] = ACTIONS(2970),
    [anon_sym_BEGIN] = ACTIONS(2970),
    [anon_sym_LBRACE] = ACTIONS(2972),
    [anon_sym_END] = ACTIONS(2970),
    [anon_sym_def] = ACTIONS(2970),
    [anon_sym_LPAREN] = ACTIONS(2972),
    [anon_sym_COLON_COLON] = ACTIONS(2972),
    [anon_sym_SEMI] = ACTIONS(2972),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2972),
    [anon_sym_STAR] = ACTIONS(2972),
    [anon_sym_nil] = ACTIONS(2970),
    [anon_sym_class] = ACTIONS(2970),
    [anon_sym_module] = ACTIONS(2970),
    [anon_sym_return] = ACTIONS(2970),
    [anon_sym_yield] = ACTIONS(2970),
    [anon_sym_break] = ACTIONS(2970),
    [anon_sym_next] = ACTIONS(2970),
    [anon_sym_redo] = ACTIONS(2970),
    [anon_sym_retry] = ACTIONS(2970),
    [anon_sym_if] = ACTIONS(2970),
    [anon_sym_unless] = ACTIONS(2970),
    [anon_sym_while] = ACTIONS(2970),
    [anon_sym_until] = ACTIONS(2970),
    [anon_sym_for] = ACTIONS(2970),
    [anon_sym_case] = ACTIONS(2970),
    [anon_sym_LBRACK] = ACTIONS(2972),
    [anon_sym_DOT_DOT] = ACTIONS(2970),
    [anon_sym_begin] = ACTIONS(2970),
    [anon_sym_PLUS] = ACTIONS(2972),
    [anon_sym_defined_QMARK] = ACTIONS(2972),
    [anon_sym_not] = ACTIONS(2970),
    [anon_sym_BANG] = ACTIONS(2972),
    [anon_sym_TILDE] = ACTIONS(2972),
    [anon_sym_undef] = ACTIONS(2970),
    [anon_sym_alias] = ACTIONS(2970),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2970),
    [sym_float] = ACTIONS(2970),
    [sym_super] = ACTIONS(2970),
    [sym_self] = ACTIONS(2970),
    [sym_true] = ACTIONS(2970),
    [sym_false] = ACTIONS(2970),
    [sym_constant] = ACTIONS(2970),
    [aux_sym_constant_suffix_token1] = ACTIONS(2972),
    [aux_sym_identifier_suffix_token1] = ACTIONS(2970),
    [sym_instance_variable] = ACTIONS(2972),
    [sym_class_variable] = ACTIONS(2972),
    [sym_global_variable] = ACTIONS(2972),
    [sym_character] = ACTIONS(2972),
    [anon_sym_DASH_GT] = ACTIONS(2972),
    [sym_simple_symbol] = ACTIONS(2972),
    [sym__string_start] = ACTIONS(2972),
    [sym__symbol_start] = ACTIONS(2972),
    [sym__subshell_start] = ACTIONS(2972),
    [sym__regex_start] = ACTIONS(2972),
    [sym__string_array_start] = ACTIONS(2972),
    [sym__symbol_array_start] = ACTIONS(2972),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2972),
    [sym__unary_minus] = ACTIONS(2972),
    [sym__unary_minus_num] = ACTIONS(2972),
    [sym__binary_minus] = ACTIONS(2972),
    [sym__identifier_suffix] = ACTIONS(2972),
    [sym__constant_suffix] = ACTIONS(2972),
  },
  [2145] = {
    [sym_heredoc_body] = STATE(2145),
    [ts_builtin_sym_end] = ACTIONS(1110),
    [sym_identifier] = ACTIONS(1124),
    [aux_sym_program_token1] = ACTIONS(1124),
    [anon_sym_BEGIN] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1110),
    [anon_sym_END] = ACTIONS(1124),
    [anon_sym_def] = ACTIONS(1124),
    [anon_sym_LPAREN] = ACTIONS(1110),
    [anon_sym_COLON_COLON] = ACTIONS(1110),
    [anon_sym_SEMI] = ACTIONS(1110),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1110),
    [anon_sym_STAR] = ACTIONS(1110),
    [anon_sym_nil] = ACTIONS(1124),
    [anon_sym_class] = ACTIONS(1124),
    [anon_sym_module] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_yield] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_next] = ACTIONS(1124),
    [anon_sym_redo] = ACTIONS(1124),
    [anon_sym_retry] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_unless] = ACTIONS(1124),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_until] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1110),
    [anon_sym_DOT_DOT] = ACTIONS(1124),
    [anon_sym_begin] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1110),
    [anon_sym_defined_QMARK] = ACTIONS(1110),
    [anon_sym_not] = ACTIONS(1124),
    [anon_sym_BANG] = ACTIONS(1110),
    [anon_sym_TILDE] = ACTIONS(1110),
    [anon_sym_undef] = ACTIONS(1124),
    [anon_sym_alias] = ACTIONS(1124),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(1124),
    [sym_float] = ACTIONS(1124),
    [sym_super] = ACTIONS(1124),
    [sym_self] = ACTIONS(1124),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [sym_constant] = ACTIONS(1124),
    [aux_sym_constant_suffix_token1] = ACTIONS(1110),
    [aux_sym_identifier_suffix_token1] = ACTIONS(1124),
    [sym_instance_variable] = ACTIONS(1110),
    [sym_class_variable] = ACTIONS(1110),
    [sym_global_variable] = ACTIONS(1110),
    [sym_character] = ACTIONS(1110),
    [anon_sym_DASH_GT] = ACTIONS(1110),
    [sym_simple_symbol] = ACTIONS(1110),
    [sym__string_start] = ACTIONS(1110),
    [sym__symbol_start] = ACTIONS(1110),
    [sym__subshell_start] = ACTIONS(1110),
    [sym__regex_start] = ACTIONS(1110),
    [sym__string_array_start] = ACTIONS(1110),
    [sym__symbol_array_start] = ACTIONS(1110),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(1110),
    [sym__unary_minus] = ACTIONS(1110),
    [sym__unary_minus_num] = ACTIONS(1110),
    [sym__binary_minus] = ACTIONS(1110),
    [sym__identifier_suffix] = ACTIONS(1110),
    [sym__constant_suffix] = ACTIONS(1110),
  },
  [2146] = {
    [sym_heredoc_body] = STATE(2146),
    [ts_builtin_sym_end] = ACTIONS(743),
    [aux_sym_program_token1] = ACTIONS(743),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(2990),
    [anon_sym_rescue] = ACTIONS(743),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(3184),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(743),
    [anon_sym_unless] = ACTIONS(743),
    [anon_sym_while] = ACTIONS(743),
    [anon_sym_until] = ACTIONS(743),
    [anon_sym_do] = ACTIONS(743),
    [anon_sym_EQ_GT] = ACTIONS(2740),
    [anon_sym_LBRACK2] = ACTIONS(2740),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(743),
    [anon_sym_and] = ACTIONS(743),
    [anon_sym_or] = ACTIONS(743),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(743),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [2147] = {
    [sym__pattern_expr] = STATE(4894),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4788),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4750),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2147),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2904),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2906),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2908),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2910),
    [sym_float] = ACTIONS(2910),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2908),
  },
  [2148] = {
    [sym_heredoc_body] = STATE(2148),
    [sym_identifier] = ACTIONS(3142),
    [anon_sym_BEGIN] = ACTIONS(3142),
    [anon_sym_LBRACE] = ACTIONS(3144),
    [anon_sym_RBRACE] = ACTIONS(3144),
    [anon_sym_END] = ACTIONS(3142),
    [anon_sym_def] = ACTIONS(3142),
    [anon_sym_LPAREN] = ACTIONS(3144),
    [anon_sym_COLON_COLON] = ACTIONS(3144),
    [anon_sym_SEMI] = ACTIONS(3144),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3144),
    [anon_sym_STAR] = ACTIONS(3144),
    [anon_sym_nil] = ACTIONS(3142),
    [anon_sym_class] = ACTIONS(3142),
    [anon_sym_module] = ACTIONS(3142),
    [anon_sym_return] = ACTIONS(3142),
    [anon_sym_yield] = ACTIONS(3142),
    [anon_sym_break] = ACTIONS(3142),
    [anon_sym_next] = ACTIONS(3142),
    [anon_sym_redo] = ACTIONS(3142),
    [anon_sym_retry] = ACTIONS(3142),
    [anon_sym_if] = ACTIONS(3142),
    [anon_sym_unless] = ACTIONS(3142),
    [anon_sym_while] = ACTIONS(3142),
    [anon_sym_until] = ACTIONS(3142),
    [anon_sym_for] = ACTIONS(3142),
    [anon_sym_case] = ACTIONS(3142),
    [anon_sym_LBRACK] = ACTIONS(3144),
    [anon_sym_DOT_DOT] = ACTIONS(3142),
    [anon_sym_begin] = ACTIONS(3142),
    [anon_sym_PLUS] = ACTIONS(3144),
    [anon_sym_defined_QMARK] = ACTIONS(3144),
    [anon_sym_not] = ACTIONS(3142),
    [anon_sym_BANG] = ACTIONS(3144),
    [anon_sym_TILDE] = ACTIONS(3144),
    [anon_sym_undef] = ACTIONS(3142),
    [anon_sym_alias] = ACTIONS(3142),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3142),
    [sym_float] = ACTIONS(3142),
    [sym_super] = ACTIONS(3142),
    [sym_self] = ACTIONS(3142),
    [sym_true] = ACTIONS(3142),
    [sym_false] = ACTIONS(3142),
    [sym_constant] = ACTIONS(3142),
    [aux_sym_constant_suffix_token1] = ACTIONS(3144),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3142),
    [sym_instance_variable] = ACTIONS(3144),
    [sym_class_variable] = ACTIONS(3144),
    [sym_global_variable] = ACTIONS(3144),
    [sym_character] = ACTIONS(3144),
    [anon_sym_DASH_GT] = ACTIONS(3144),
    [sym_simple_symbol] = ACTIONS(3144),
    [sym__string_start] = ACTIONS(3144),
    [sym__symbol_start] = ACTIONS(3144),
    [sym__subshell_start] = ACTIONS(3144),
    [sym__regex_start] = ACTIONS(3144),
    [sym__string_array_start] = ACTIONS(3144),
    [sym__symbol_array_start] = ACTIONS(3144),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3144),
    [sym__unary_minus] = ACTIONS(3144),
    [sym__unary_minus_num] = ACTIONS(3144),
    [sym__binary_minus] = ACTIONS(3144),
    [sym__identifier_suffix] = ACTIONS(3144),
    [sym__constant_suffix] = ACTIONS(3144),
  },
  [2149] = {
    [sym_heredoc_body] = STATE(2149),
    [sym_identifier] = ACTIONS(3126),
    [anon_sym_BEGIN] = ACTIONS(3126),
    [anon_sym_LBRACE] = ACTIONS(3128),
    [anon_sym_RBRACE] = ACTIONS(3128),
    [anon_sym_END] = ACTIONS(3126),
    [anon_sym_def] = ACTIONS(3126),
    [anon_sym_LPAREN] = ACTIONS(3128),
    [anon_sym_COLON_COLON] = ACTIONS(3128),
    [anon_sym_SEMI] = ACTIONS(3128),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3128),
    [anon_sym_STAR] = ACTIONS(3128),
    [anon_sym_nil] = ACTIONS(3126),
    [anon_sym_class] = ACTIONS(3126),
    [anon_sym_module] = ACTIONS(3126),
    [anon_sym_return] = ACTIONS(3126),
    [anon_sym_yield] = ACTIONS(3126),
    [anon_sym_break] = ACTIONS(3126),
    [anon_sym_next] = ACTIONS(3126),
    [anon_sym_redo] = ACTIONS(3126),
    [anon_sym_retry] = ACTIONS(3126),
    [anon_sym_if] = ACTIONS(3126),
    [anon_sym_unless] = ACTIONS(3126),
    [anon_sym_while] = ACTIONS(3126),
    [anon_sym_until] = ACTIONS(3126),
    [anon_sym_for] = ACTIONS(3126),
    [anon_sym_case] = ACTIONS(3126),
    [anon_sym_LBRACK] = ACTIONS(3128),
    [anon_sym_DOT_DOT] = ACTIONS(3126),
    [anon_sym_begin] = ACTIONS(3126),
    [anon_sym_PLUS] = ACTIONS(3128),
    [anon_sym_defined_QMARK] = ACTIONS(3128),
    [anon_sym_not] = ACTIONS(3126),
    [anon_sym_BANG] = ACTIONS(3128),
    [anon_sym_TILDE] = ACTIONS(3128),
    [anon_sym_undef] = ACTIONS(3126),
    [anon_sym_alias] = ACTIONS(3126),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3126),
    [sym_float] = ACTIONS(3126),
    [sym_super] = ACTIONS(3126),
    [sym_self] = ACTIONS(3126),
    [sym_true] = ACTIONS(3126),
    [sym_false] = ACTIONS(3126),
    [sym_constant] = ACTIONS(3126),
    [aux_sym_constant_suffix_token1] = ACTIONS(3128),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3126),
    [sym_instance_variable] = ACTIONS(3128),
    [sym_class_variable] = ACTIONS(3128),
    [sym_global_variable] = ACTIONS(3128),
    [sym_character] = ACTIONS(3128),
    [anon_sym_DASH_GT] = ACTIONS(3128),
    [sym_simple_symbol] = ACTIONS(3128),
    [sym__string_start] = ACTIONS(3128),
    [sym__symbol_start] = ACTIONS(3128),
    [sym__subshell_start] = ACTIONS(3128),
    [sym__regex_start] = ACTIONS(3128),
    [sym__string_array_start] = ACTIONS(3128),
    [sym__symbol_array_start] = ACTIONS(3128),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3128),
    [sym__unary_minus] = ACTIONS(3128),
    [sym__unary_minus_num] = ACTIONS(3128),
    [sym__binary_minus] = ACTIONS(3128),
    [sym__identifier_suffix] = ACTIONS(3128),
    [sym__constant_suffix] = ACTIONS(3128),
  },
  [2150] = {
    [sym_heredoc_body] = STATE(2150),
    [sym_identifier] = ACTIONS(3146),
    [anon_sym_BEGIN] = ACTIONS(3146),
    [anon_sym_LBRACE] = ACTIONS(3148),
    [anon_sym_RBRACE] = ACTIONS(3148),
    [anon_sym_END] = ACTIONS(3146),
    [anon_sym_def] = ACTIONS(3146),
    [anon_sym_LPAREN] = ACTIONS(3148),
    [anon_sym_COLON_COLON] = ACTIONS(3148),
    [anon_sym_SEMI] = ACTIONS(3148),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3148),
    [anon_sym_STAR] = ACTIONS(3148),
    [anon_sym_nil] = ACTIONS(3146),
    [anon_sym_class] = ACTIONS(3146),
    [anon_sym_module] = ACTIONS(3146),
    [anon_sym_return] = ACTIONS(3146),
    [anon_sym_yield] = ACTIONS(3146),
    [anon_sym_break] = ACTIONS(3146),
    [anon_sym_next] = ACTIONS(3146),
    [anon_sym_redo] = ACTIONS(3146),
    [anon_sym_retry] = ACTIONS(3146),
    [anon_sym_if] = ACTIONS(3146),
    [anon_sym_unless] = ACTIONS(3146),
    [anon_sym_while] = ACTIONS(3146),
    [anon_sym_until] = ACTIONS(3146),
    [anon_sym_for] = ACTIONS(3146),
    [anon_sym_case] = ACTIONS(3146),
    [anon_sym_LBRACK] = ACTIONS(3148),
    [anon_sym_DOT_DOT] = ACTIONS(3146),
    [anon_sym_begin] = ACTIONS(3146),
    [anon_sym_PLUS] = ACTIONS(3148),
    [anon_sym_defined_QMARK] = ACTIONS(3148),
    [anon_sym_not] = ACTIONS(3146),
    [anon_sym_BANG] = ACTIONS(3148),
    [anon_sym_TILDE] = ACTIONS(3148),
    [anon_sym_undef] = ACTIONS(3146),
    [anon_sym_alias] = ACTIONS(3146),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3146),
    [sym_float] = ACTIONS(3146),
    [sym_super] = ACTIONS(3146),
    [sym_self] = ACTIONS(3146),
    [sym_true] = ACTIONS(3146),
    [sym_false] = ACTIONS(3146),
    [sym_constant] = ACTIONS(3146),
    [aux_sym_constant_suffix_token1] = ACTIONS(3148),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3146),
    [sym_instance_variable] = ACTIONS(3148),
    [sym_class_variable] = ACTIONS(3148),
    [sym_global_variable] = ACTIONS(3148),
    [sym_character] = ACTIONS(3148),
    [anon_sym_DASH_GT] = ACTIONS(3148),
    [sym_simple_symbol] = ACTIONS(3148),
    [sym__string_start] = ACTIONS(3148),
    [sym__symbol_start] = ACTIONS(3148),
    [sym__subshell_start] = ACTIONS(3148),
    [sym__regex_start] = ACTIONS(3148),
    [sym__string_array_start] = ACTIONS(3148),
    [sym__symbol_array_start] = ACTIONS(3148),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3148),
    [sym__unary_minus] = ACTIONS(3148),
    [sym__unary_minus_num] = ACTIONS(3148),
    [sym__binary_minus] = ACTIONS(3148),
    [sym__identifier_suffix] = ACTIONS(3148),
    [sym__constant_suffix] = ACTIONS(3148),
  },
  [2151] = {
    [sym_heredoc_body] = STATE(2151),
    [sym_identifier] = ACTIONS(3102),
    [anon_sym_BEGIN] = ACTIONS(3102),
    [anon_sym_LBRACE] = ACTIONS(3104),
    [anon_sym_RBRACE] = ACTIONS(3104),
    [anon_sym_END] = ACTIONS(3102),
    [anon_sym_def] = ACTIONS(3102),
    [anon_sym_LPAREN] = ACTIONS(3104),
    [anon_sym_COLON_COLON] = ACTIONS(3104),
    [anon_sym_SEMI] = ACTIONS(3104),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3104),
    [anon_sym_STAR] = ACTIONS(3104),
    [anon_sym_nil] = ACTIONS(3102),
    [anon_sym_class] = ACTIONS(3102),
    [anon_sym_module] = ACTIONS(3102),
    [anon_sym_return] = ACTIONS(3102),
    [anon_sym_yield] = ACTIONS(3102),
    [anon_sym_break] = ACTIONS(3102),
    [anon_sym_next] = ACTIONS(3102),
    [anon_sym_redo] = ACTIONS(3102),
    [anon_sym_retry] = ACTIONS(3102),
    [anon_sym_if] = ACTIONS(3102),
    [anon_sym_unless] = ACTIONS(3102),
    [anon_sym_while] = ACTIONS(3102),
    [anon_sym_until] = ACTIONS(3102),
    [anon_sym_for] = ACTIONS(3102),
    [anon_sym_case] = ACTIONS(3102),
    [anon_sym_LBRACK] = ACTIONS(3104),
    [anon_sym_DOT_DOT] = ACTIONS(3102),
    [anon_sym_begin] = ACTIONS(3102),
    [anon_sym_PLUS] = ACTIONS(3104),
    [anon_sym_defined_QMARK] = ACTIONS(3104),
    [anon_sym_not] = ACTIONS(3102),
    [anon_sym_BANG] = ACTIONS(3104),
    [anon_sym_TILDE] = ACTIONS(3104),
    [anon_sym_undef] = ACTIONS(3102),
    [anon_sym_alias] = ACTIONS(3102),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3102),
    [sym_float] = ACTIONS(3102),
    [sym_super] = ACTIONS(3102),
    [sym_self] = ACTIONS(3102),
    [sym_true] = ACTIONS(3102),
    [sym_false] = ACTIONS(3102),
    [sym_constant] = ACTIONS(3102),
    [aux_sym_constant_suffix_token1] = ACTIONS(3104),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3102),
    [sym_instance_variable] = ACTIONS(3104),
    [sym_class_variable] = ACTIONS(3104),
    [sym_global_variable] = ACTIONS(3104),
    [sym_character] = ACTIONS(3104),
    [anon_sym_DASH_GT] = ACTIONS(3104),
    [sym_simple_symbol] = ACTIONS(3104),
    [sym__string_start] = ACTIONS(3104),
    [sym__symbol_start] = ACTIONS(3104),
    [sym__subshell_start] = ACTIONS(3104),
    [sym__regex_start] = ACTIONS(3104),
    [sym__string_array_start] = ACTIONS(3104),
    [sym__symbol_array_start] = ACTIONS(3104),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3104),
    [sym__unary_minus] = ACTIONS(3104),
    [sym__unary_minus_num] = ACTIONS(3104),
    [sym__binary_minus] = ACTIONS(3104),
    [sym__identifier_suffix] = ACTIONS(3104),
    [sym__constant_suffix] = ACTIONS(3104),
  },
  [2152] = {
    [sym__pattern_expr] = STATE(4894),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2152),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2153] = {
    [sym__pattern_expr] = STATE(4479),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4732),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4664),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2153),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2916),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2920),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2922),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2924),
    [sym_float] = ACTIONS(2924),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2922),
  },
  [2154] = {
    [sym_heredoc_body] = STATE(2154),
    [sym_identifier] = ACTIONS(3150),
    [anon_sym_BEGIN] = ACTIONS(3150),
    [anon_sym_LBRACE] = ACTIONS(3152),
    [anon_sym_RBRACE] = ACTIONS(3152),
    [anon_sym_END] = ACTIONS(3150),
    [anon_sym_def] = ACTIONS(3150),
    [anon_sym_LPAREN] = ACTIONS(3152),
    [anon_sym_COLON_COLON] = ACTIONS(3152),
    [anon_sym_SEMI] = ACTIONS(3152),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3152),
    [anon_sym_STAR] = ACTIONS(3152),
    [anon_sym_nil] = ACTIONS(3150),
    [anon_sym_class] = ACTIONS(3150),
    [anon_sym_module] = ACTIONS(3150),
    [anon_sym_return] = ACTIONS(3150),
    [anon_sym_yield] = ACTIONS(3150),
    [anon_sym_break] = ACTIONS(3150),
    [anon_sym_next] = ACTIONS(3150),
    [anon_sym_redo] = ACTIONS(3150),
    [anon_sym_retry] = ACTIONS(3150),
    [anon_sym_if] = ACTIONS(3150),
    [anon_sym_unless] = ACTIONS(3150),
    [anon_sym_while] = ACTIONS(3150),
    [anon_sym_until] = ACTIONS(3150),
    [anon_sym_for] = ACTIONS(3150),
    [anon_sym_case] = ACTIONS(3150),
    [anon_sym_LBRACK] = ACTIONS(3152),
    [anon_sym_DOT_DOT] = ACTIONS(3150),
    [anon_sym_begin] = ACTIONS(3150),
    [anon_sym_PLUS] = ACTIONS(3152),
    [anon_sym_defined_QMARK] = ACTIONS(3152),
    [anon_sym_not] = ACTIONS(3150),
    [anon_sym_BANG] = ACTIONS(3152),
    [anon_sym_TILDE] = ACTIONS(3152),
    [anon_sym_undef] = ACTIONS(3150),
    [anon_sym_alias] = ACTIONS(3150),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3150),
    [sym_float] = ACTIONS(3150),
    [sym_super] = ACTIONS(3150),
    [sym_self] = ACTIONS(3150),
    [sym_true] = ACTIONS(3150),
    [sym_false] = ACTIONS(3150),
    [sym_constant] = ACTIONS(3150),
    [aux_sym_constant_suffix_token1] = ACTIONS(3152),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3150),
    [sym_instance_variable] = ACTIONS(3152),
    [sym_class_variable] = ACTIONS(3152),
    [sym_global_variable] = ACTIONS(3152),
    [sym_character] = ACTIONS(3152),
    [anon_sym_DASH_GT] = ACTIONS(3152),
    [sym_simple_symbol] = ACTIONS(3152),
    [sym__string_start] = ACTIONS(3152),
    [sym__symbol_start] = ACTIONS(3152),
    [sym__subshell_start] = ACTIONS(3152),
    [sym__regex_start] = ACTIONS(3152),
    [sym__string_array_start] = ACTIONS(3152),
    [sym__symbol_array_start] = ACTIONS(3152),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3152),
    [sym__unary_minus] = ACTIONS(3152),
    [sym__unary_minus_num] = ACTIONS(3152),
    [sym__binary_minus] = ACTIONS(3152),
    [sym__identifier_suffix] = ACTIONS(3152),
    [sym__constant_suffix] = ACTIONS(3152),
  },
  [2155] = {
    [sym__pattern_expr] = STATE(4479),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4618),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4593),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2155),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2934),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2938),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1709),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2942),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2156] = {
    [sym_heredoc_body] = STATE(2156),
    [sym_identifier] = ACTIONS(3110),
    [anon_sym_BEGIN] = ACTIONS(3110),
    [anon_sym_LBRACE] = ACTIONS(3112),
    [anon_sym_RBRACE] = ACTIONS(3112),
    [anon_sym_END] = ACTIONS(3110),
    [anon_sym_def] = ACTIONS(3110),
    [anon_sym_LPAREN] = ACTIONS(3112),
    [anon_sym_COLON_COLON] = ACTIONS(3112),
    [anon_sym_SEMI] = ACTIONS(3112),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3112),
    [anon_sym_STAR] = ACTIONS(3112),
    [anon_sym_nil] = ACTIONS(3110),
    [anon_sym_class] = ACTIONS(3110),
    [anon_sym_module] = ACTIONS(3110),
    [anon_sym_return] = ACTIONS(3110),
    [anon_sym_yield] = ACTIONS(3110),
    [anon_sym_break] = ACTIONS(3110),
    [anon_sym_next] = ACTIONS(3110),
    [anon_sym_redo] = ACTIONS(3110),
    [anon_sym_retry] = ACTIONS(3110),
    [anon_sym_if] = ACTIONS(3110),
    [anon_sym_unless] = ACTIONS(3110),
    [anon_sym_while] = ACTIONS(3110),
    [anon_sym_until] = ACTIONS(3110),
    [anon_sym_for] = ACTIONS(3110),
    [anon_sym_case] = ACTIONS(3110),
    [anon_sym_LBRACK] = ACTIONS(3112),
    [anon_sym_DOT_DOT] = ACTIONS(3110),
    [anon_sym_begin] = ACTIONS(3110),
    [anon_sym_PLUS] = ACTIONS(3112),
    [anon_sym_defined_QMARK] = ACTIONS(3112),
    [anon_sym_not] = ACTIONS(3110),
    [anon_sym_BANG] = ACTIONS(3112),
    [anon_sym_TILDE] = ACTIONS(3112),
    [anon_sym_undef] = ACTIONS(3110),
    [anon_sym_alias] = ACTIONS(3110),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3110),
    [sym_float] = ACTIONS(3110),
    [sym_super] = ACTIONS(3110),
    [sym_self] = ACTIONS(3110),
    [sym_true] = ACTIONS(3110),
    [sym_false] = ACTIONS(3110),
    [sym_constant] = ACTIONS(3110),
    [aux_sym_constant_suffix_token1] = ACTIONS(3112),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3110),
    [sym_instance_variable] = ACTIONS(3112),
    [sym_class_variable] = ACTIONS(3112),
    [sym_global_variable] = ACTIONS(3112),
    [sym_character] = ACTIONS(3112),
    [anon_sym_DASH_GT] = ACTIONS(3112),
    [sym_simple_symbol] = ACTIONS(3112),
    [sym__string_start] = ACTIONS(3112),
    [sym__symbol_start] = ACTIONS(3112),
    [sym__subshell_start] = ACTIONS(3112),
    [sym__regex_start] = ACTIONS(3112),
    [sym__string_array_start] = ACTIONS(3112),
    [sym__symbol_array_start] = ACTIONS(3112),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3112),
    [sym__unary_minus] = ACTIONS(3112),
    [sym__unary_minus_num] = ACTIONS(3112),
    [sym__binary_minus] = ACTIONS(3112),
    [sym__identifier_suffix] = ACTIONS(3112),
    [sym__constant_suffix] = ACTIONS(3112),
  },
  [2157] = {
    [sym__pattern_expr] = STATE(4479),
    [sym_as_pattern] = STATE(4480),
    [sym__pattern_expr_alt] = STATE(4480),
    [sym_alternative_pattern] = STATE(4476),
    [sym_array_pattern] = STATE(4446),
    [sym_find_pattern] = STATE(4446),
    [sym_hash_pattern] = STATE(4446),
    [sym__pattern_expr_basic] = STATE(4474),
    [sym_parenthesized_pattern] = STATE(4446),
    [sym__pattern_value] = STATE(4446),
    [sym__pattern_range] = STATE(4438),
    [sym__pattern_primitive] = STATE(4448),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_variable_reference_pattern] = STATE(4451),
    [sym_expression_reference_pattern] = STATE(4451),
    [sym__pattern_constant] = STATE(4375),
    [sym__pattern_constant_resolution] = STATE(4376),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2157),
    [sym_lambda] = STATE(4411),
    [sym_identifier] = ACTIONS(2800),
    [anon_sym_LBRACE] = ACTIONS(2802),
    [anon_sym_LPAREN] = ACTIONS(2806),
    [anon_sym_COLON_COLON] = ACTIONS(2808),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2812),
    [anon_sym_nil] = ACTIONS(2816),
    [anon_sym_LBRACK] = ACTIONS(2818),
    [anon_sym_DOT_DOT] = ACTIONS(2820),
    [sym_line] = ACTIONS(1651),
    [sym_file] = ACTIONS(1651),
    [sym_encoding] = ACTIONS(1651),
    [anon_sym_CARET] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(1651),
    [sym_true] = ACTIONS(1651),
    [sym_false] = ACTIONS(1651),
    [sym_constant] = ACTIONS(2826),
    [anon_sym_DASH_GT] = ACTIONS(2828),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
  [2158] = {
    [sym_heredoc_body] = STATE(2158),
    [sym_identifier] = ACTIONS(3122),
    [anon_sym_BEGIN] = ACTIONS(3122),
    [anon_sym_LBRACE] = ACTIONS(3124),
    [anon_sym_RBRACE] = ACTIONS(3124),
    [anon_sym_END] = ACTIONS(3122),
    [anon_sym_def] = ACTIONS(3122),
    [anon_sym_LPAREN] = ACTIONS(3124),
    [anon_sym_COLON_COLON] = ACTIONS(3124),
    [anon_sym_SEMI] = ACTIONS(3124),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3124),
    [anon_sym_STAR] = ACTIONS(3124),
    [anon_sym_nil] = ACTIONS(3122),
    [anon_sym_class] = ACTIONS(3122),
    [anon_sym_module] = ACTIONS(3122),
    [anon_sym_return] = ACTIONS(3122),
    [anon_sym_yield] = ACTIONS(3122),
    [anon_sym_break] = ACTIONS(3122),
    [anon_sym_next] = ACTIONS(3122),
    [anon_sym_redo] = ACTIONS(3122),
    [anon_sym_retry] = ACTIONS(3122),
    [anon_sym_if] = ACTIONS(3122),
    [anon_sym_unless] = ACTIONS(3122),
    [anon_sym_while] = ACTIONS(3122),
    [anon_sym_until] = ACTIONS(3122),
    [anon_sym_for] = ACTIONS(3122),
    [anon_sym_case] = ACTIONS(3122),
    [anon_sym_LBRACK] = ACTIONS(3124),
    [anon_sym_DOT_DOT] = ACTIONS(3122),
    [anon_sym_begin] = ACTIONS(3122),
    [anon_sym_PLUS] = ACTIONS(3124),
    [anon_sym_defined_QMARK] = ACTIONS(3124),
    [anon_sym_not] = ACTIONS(3122),
    [anon_sym_BANG] = ACTIONS(3124),
    [anon_sym_TILDE] = ACTIONS(3124),
    [anon_sym_undef] = ACTIONS(3122),
    [anon_sym_alias] = ACTIONS(3122),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3122),
    [sym_float] = ACTIONS(3122),
    [sym_super] = ACTIONS(3122),
    [sym_self] = ACTIONS(3122),
    [sym_true] = ACTIONS(3122),
    [sym_false] = ACTIONS(3122),
    [sym_constant] = ACTIONS(3122),
    [aux_sym_constant_suffix_token1] = ACTIONS(3124),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3122),
    [sym_instance_variable] = ACTIONS(3124),
    [sym_class_variable] = ACTIONS(3124),
    [sym_global_variable] = ACTIONS(3124),
    [sym_character] = ACTIONS(3124),
    [anon_sym_DASH_GT] = ACTIONS(3124),
    [sym_simple_symbol] = ACTIONS(3124),
    [sym__string_start] = ACTIONS(3124),
    [sym__symbol_start] = ACTIONS(3124),
    [sym__subshell_start] = ACTIONS(3124),
    [sym__regex_start] = ACTIONS(3124),
    [sym__string_array_start] = ACTIONS(3124),
    [sym__symbol_array_start] = ACTIONS(3124),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3124),
    [sym__unary_minus] = ACTIONS(3124),
    [sym__unary_minus_num] = ACTIONS(3124),
    [sym__binary_minus] = ACTIONS(3124),
    [sym__identifier_suffix] = ACTIONS(3124),
    [sym__constant_suffix] = ACTIONS(3124),
  },
  [2159] = {
    [sym_heredoc_body] = STATE(2159),
    [sym_identifier] = ACTIONS(3098),
    [anon_sym_BEGIN] = ACTIONS(3098),
    [anon_sym_LBRACE] = ACTIONS(3100),
    [anon_sym_RBRACE] = ACTIONS(3100),
    [anon_sym_END] = ACTIONS(3098),
    [anon_sym_def] = ACTIONS(3098),
    [anon_sym_LPAREN] = ACTIONS(3100),
    [anon_sym_COLON_COLON] = ACTIONS(3100),
    [anon_sym_SEMI] = ACTIONS(3100),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3100),
    [anon_sym_STAR] = ACTIONS(3100),
    [anon_sym_nil] = ACTIONS(3098),
    [anon_sym_class] = ACTIONS(3098),
    [anon_sym_module] = ACTIONS(3098),
    [anon_sym_return] = ACTIONS(3098),
    [anon_sym_yield] = ACTIONS(3098),
    [anon_sym_break] = ACTIONS(3098),
    [anon_sym_next] = ACTIONS(3098),
    [anon_sym_redo] = ACTIONS(3098),
    [anon_sym_retry] = ACTIONS(3098),
    [anon_sym_if] = ACTIONS(3098),
    [anon_sym_unless] = ACTIONS(3098),
    [anon_sym_while] = ACTIONS(3098),
    [anon_sym_until] = ACTIONS(3098),
    [anon_sym_for] = ACTIONS(3098),
    [anon_sym_case] = ACTIONS(3098),
    [anon_sym_LBRACK] = ACTIONS(3100),
    [anon_sym_DOT_DOT] = ACTIONS(3098),
    [anon_sym_begin] = ACTIONS(3098),
    [anon_sym_PLUS] = ACTIONS(3100),
    [anon_sym_defined_QMARK] = ACTIONS(3100),
    [anon_sym_not] = ACTIONS(3098),
    [anon_sym_BANG] = ACTIONS(3100),
    [anon_sym_TILDE] = ACTIONS(3100),
    [anon_sym_undef] = ACTIONS(3098),
    [anon_sym_alias] = ACTIONS(3098),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3098),
    [sym_float] = ACTIONS(3098),
    [sym_super] = ACTIONS(3098),
    [sym_self] = ACTIONS(3098),
    [sym_true] = ACTIONS(3098),
    [sym_false] = ACTIONS(3098),
    [sym_constant] = ACTIONS(3098),
    [aux_sym_constant_suffix_token1] = ACTIONS(3100),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3098),
    [sym_instance_variable] = ACTIONS(3100),
    [sym_class_variable] = ACTIONS(3100),
    [sym_global_variable] = ACTIONS(3100),
    [sym_character] = ACTIONS(3100),
    [anon_sym_DASH_GT] = ACTIONS(3100),
    [sym_simple_symbol] = ACTIONS(3100),
    [sym__string_start] = ACTIONS(3100),
    [sym__symbol_start] = ACTIONS(3100),
    [sym__subshell_start] = ACTIONS(3100),
    [sym__regex_start] = ACTIONS(3100),
    [sym__string_array_start] = ACTIONS(3100),
    [sym__symbol_array_start] = ACTIONS(3100),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3100),
    [sym__unary_minus] = ACTIONS(3100),
    [sym__unary_minus_num] = ACTIONS(3100),
    [sym__binary_minus] = ACTIONS(3100),
    [sym__identifier_suffix] = ACTIONS(3100),
    [sym__constant_suffix] = ACTIONS(3100),
  },
  [2160] = {
    [sym_heredoc_body] = STATE(2160),
    [sym_identifier] = ACTIONS(3114),
    [anon_sym_BEGIN] = ACTIONS(3114),
    [anon_sym_LBRACE] = ACTIONS(3116),
    [anon_sym_RBRACE] = ACTIONS(3116),
    [anon_sym_END] = ACTIONS(3114),
    [anon_sym_def] = ACTIONS(3114),
    [anon_sym_LPAREN] = ACTIONS(3116),
    [anon_sym_COLON_COLON] = ACTIONS(3116),
    [anon_sym_SEMI] = ACTIONS(3116),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3116),
    [anon_sym_STAR] = ACTIONS(3116),
    [anon_sym_nil] = ACTIONS(3114),
    [anon_sym_class] = ACTIONS(3114),
    [anon_sym_module] = ACTIONS(3114),
    [anon_sym_return] = ACTIONS(3114),
    [anon_sym_yield] = ACTIONS(3114),
    [anon_sym_break] = ACTIONS(3114),
    [anon_sym_next] = ACTIONS(3114),
    [anon_sym_redo] = ACTIONS(3114),
    [anon_sym_retry] = ACTIONS(3114),
    [anon_sym_if] = ACTIONS(3114),
    [anon_sym_unless] = ACTIONS(3114),
    [anon_sym_while] = ACTIONS(3114),
    [anon_sym_until] = ACTIONS(3114),
    [anon_sym_for] = ACTIONS(3114),
    [anon_sym_case] = ACTIONS(3114),
    [anon_sym_LBRACK] = ACTIONS(3116),
    [anon_sym_DOT_DOT] = ACTIONS(3114),
    [anon_sym_begin] = ACTIONS(3114),
    [anon_sym_PLUS] = ACTIONS(3116),
    [anon_sym_defined_QMARK] = ACTIONS(3116),
    [anon_sym_not] = ACTIONS(3114),
    [anon_sym_BANG] = ACTIONS(3116),
    [anon_sym_TILDE] = ACTIONS(3116),
    [anon_sym_undef] = ACTIONS(3114),
    [anon_sym_alias] = ACTIONS(3114),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3114),
    [sym_float] = ACTIONS(3114),
    [sym_super] = ACTIONS(3114),
    [sym_self] = ACTIONS(3114),
    [sym_true] = ACTIONS(3114),
    [sym_false] = ACTIONS(3114),
    [sym_constant] = ACTIONS(3114),
    [aux_sym_constant_suffix_token1] = ACTIONS(3116),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3114),
    [sym_instance_variable] = ACTIONS(3116),
    [sym_class_variable] = ACTIONS(3116),
    [sym_global_variable] = ACTIONS(3116),
    [sym_character] = ACTIONS(3116),
    [anon_sym_DASH_GT] = ACTIONS(3116),
    [sym_simple_symbol] = ACTIONS(3116),
    [sym__string_start] = ACTIONS(3116),
    [sym__symbol_start] = ACTIONS(3116),
    [sym__subshell_start] = ACTIONS(3116),
    [sym__regex_start] = ACTIONS(3116),
    [sym__string_array_start] = ACTIONS(3116),
    [sym__symbol_array_start] = ACTIONS(3116),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3116),
    [sym__unary_minus] = ACTIONS(3116),
    [sym__unary_minus_num] = ACTIONS(3116),
    [sym__binary_minus] = ACTIONS(3116),
    [sym__identifier_suffix] = ACTIONS(3116),
    [sym__constant_suffix] = ACTIONS(3116),
  },
  [2161] = {
    [sym__pattern_expr] = STATE(5625),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2161),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2162] = {
    [sym_heredoc_body] = STATE(2162),
    [sym_identifier] = ACTIONS(3134),
    [anon_sym_BEGIN] = ACTIONS(3134),
    [anon_sym_LBRACE] = ACTIONS(3136),
    [anon_sym_RBRACE] = ACTIONS(3136),
    [anon_sym_END] = ACTIONS(3134),
    [anon_sym_def] = ACTIONS(3134),
    [anon_sym_LPAREN] = ACTIONS(3136),
    [anon_sym_COLON_COLON] = ACTIONS(3136),
    [anon_sym_SEMI] = ACTIONS(3136),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3136),
    [anon_sym_STAR] = ACTIONS(3136),
    [anon_sym_nil] = ACTIONS(3134),
    [anon_sym_class] = ACTIONS(3134),
    [anon_sym_module] = ACTIONS(3134),
    [anon_sym_return] = ACTIONS(3134),
    [anon_sym_yield] = ACTIONS(3134),
    [anon_sym_break] = ACTIONS(3134),
    [anon_sym_next] = ACTIONS(3134),
    [anon_sym_redo] = ACTIONS(3134),
    [anon_sym_retry] = ACTIONS(3134),
    [anon_sym_if] = ACTIONS(3134),
    [anon_sym_unless] = ACTIONS(3134),
    [anon_sym_while] = ACTIONS(3134),
    [anon_sym_until] = ACTIONS(3134),
    [anon_sym_for] = ACTIONS(3134),
    [anon_sym_case] = ACTIONS(3134),
    [anon_sym_LBRACK] = ACTIONS(3136),
    [anon_sym_DOT_DOT] = ACTIONS(3134),
    [anon_sym_begin] = ACTIONS(3134),
    [anon_sym_PLUS] = ACTIONS(3136),
    [anon_sym_defined_QMARK] = ACTIONS(3136),
    [anon_sym_not] = ACTIONS(3134),
    [anon_sym_BANG] = ACTIONS(3136),
    [anon_sym_TILDE] = ACTIONS(3136),
    [anon_sym_undef] = ACTIONS(3134),
    [anon_sym_alias] = ACTIONS(3134),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3134),
    [sym_float] = ACTIONS(3134),
    [sym_super] = ACTIONS(3134),
    [sym_self] = ACTIONS(3134),
    [sym_true] = ACTIONS(3134),
    [sym_false] = ACTIONS(3134),
    [sym_constant] = ACTIONS(3134),
    [aux_sym_constant_suffix_token1] = ACTIONS(3136),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3134),
    [sym_instance_variable] = ACTIONS(3136),
    [sym_class_variable] = ACTIONS(3136),
    [sym_global_variable] = ACTIONS(3136),
    [sym_character] = ACTIONS(3136),
    [anon_sym_DASH_GT] = ACTIONS(3136),
    [sym_simple_symbol] = ACTIONS(3136),
    [sym__string_start] = ACTIONS(3136),
    [sym__symbol_start] = ACTIONS(3136),
    [sym__subshell_start] = ACTIONS(3136),
    [sym__regex_start] = ACTIONS(3136),
    [sym__string_array_start] = ACTIONS(3136),
    [sym__symbol_array_start] = ACTIONS(3136),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3136),
    [sym__unary_minus] = ACTIONS(3136),
    [sym__unary_minus_num] = ACTIONS(3136),
    [sym__binary_minus] = ACTIONS(3136),
    [sym__identifier_suffix] = ACTIONS(3136),
    [sym__constant_suffix] = ACTIONS(3136),
  },
  [2163] = {
    [sym_heredoc_body] = STATE(2163),
    [sym_identifier] = ACTIONS(3118),
    [anon_sym_BEGIN] = ACTIONS(3118),
    [anon_sym_LBRACE] = ACTIONS(3120),
    [anon_sym_RBRACE] = ACTIONS(3120),
    [anon_sym_END] = ACTIONS(3118),
    [anon_sym_def] = ACTIONS(3118),
    [anon_sym_LPAREN] = ACTIONS(3120),
    [anon_sym_COLON_COLON] = ACTIONS(3120),
    [anon_sym_SEMI] = ACTIONS(3120),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3120),
    [anon_sym_STAR] = ACTIONS(3120),
    [anon_sym_nil] = ACTIONS(3118),
    [anon_sym_class] = ACTIONS(3118),
    [anon_sym_module] = ACTIONS(3118),
    [anon_sym_return] = ACTIONS(3118),
    [anon_sym_yield] = ACTIONS(3118),
    [anon_sym_break] = ACTIONS(3118),
    [anon_sym_next] = ACTIONS(3118),
    [anon_sym_redo] = ACTIONS(3118),
    [anon_sym_retry] = ACTIONS(3118),
    [anon_sym_if] = ACTIONS(3118),
    [anon_sym_unless] = ACTIONS(3118),
    [anon_sym_while] = ACTIONS(3118),
    [anon_sym_until] = ACTIONS(3118),
    [anon_sym_for] = ACTIONS(3118),
    [anon_sym_case] = ACTIONS(3118),
    [anon_sym_LBRACK] = ACTIONS(3120),
    [anon_sym_DOT_DOT] = ACTIONS(3118),
    [anon_sym_begin] = ACTIONS(3118),
    [anon_sym_PLUS] = ACTIONS(3120),
    [anon_sym_defined_QMARK] = ACTIONS(3120),
    [anon_sym_not] = ACTIONS(3118),
    [anon_sym_BANG] = ACTIONS(3120),
    [anon_sym_TILDE] = ACTIONS(3120),
    [anon_sym_undef] = ACTIONS(3118),
    [anon_sym_alias] = ACTIONS(3118),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3118),
    [sym_float] = ACTIONS(3118),
    [sym_super] = ACTIONS(3118),
    [sym_self] = ACTIONS(3118),
    [sym_true] = ACTIONS(3118),
    [sym_false] = ACTIONS(3118),
    [sym_constant] = ACTIONS(3118),
    [aux_sym_constant_suffix_token1] = ACTIONS(3120),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3118),
    [sym_instance_variable] = ACTIONS(3120),
    [sym_class_variable] = ACTIONS(3120),
    [sym_global_variable] = ACTIONS(3120),
    [sym_character] = ACTIONS(3120),
    [anon_sym_DASH_GT] = ACTIONS(3120),
    [sym_simple_symbol] = ACTIONS(3120),
    [sym__string_start] = ACTIONS(3120),
    [sym__symbol_start] = ACTIONS(3120),
    [sym__subshell_start] = ACTIONS(3120),
    [sym__regex_start] = ACTIONS(3120),
    [sym__string_array_start] = ACTIONS(3120),
    [sym__symbol_array_start] = ACTIONS(3120),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3120),
    [sym__unary_minus] = ACTIONS(3120),
    [sym__unary_minus_num] = ACTIONS(3120),
    [sym__binary_minus] = ACTIONS(3120),
    [sym__identifier_suffix] = ACTIONS(3120),
    [sym__constant_suffix] = ACTIONS(3120),
  },
  [2164] = {
    [sym__pattern_expr] = STATE(5585),
    [sym_as_pattern] = STATE(4863),
    [sym__pattern_expr_alt] = STATE(4863),
    [sym_alternative_pattern] = STATE(4865),
    [sym_array_pattern] = STATE(4819),
    [sym_find_pattern] = STATE(4819),
    [sym_hash_pattern] = STATE(4819),
    [sym__pattern_expr_basic] = STATE(4756),
    [sym_parenthesized_pattern] = STATE(4819),
    [sym__pattern_value] = STATE(4819),
    [sym__pattern_range] = STATE(4757),
    [sym__pattern_primitive] = STATE(4746),
    [sym__pattern_lambda] = STATE(4692),
    [sym__pattern_literal] = STATE(4692),
    [sym__keyword_variable] = STATE(4702),
    [sym_variable_reference_pattern] = STATE(4760),
    [sym_expression_reference_pattern] = STATE(4760),
    [sym__pattern_constant] = STATE(4656),
    [sym__pattern_constant_resolution] = STATE(4657),
    [sym_unary_literal] = STATE(4714),
    [sym__literal] = STATE(4702),
    [sym__numeric] = STATE(4699),
    [sym__simple_numeric] = STATE(4714),
    [sym__int_or_float] = STATE(5518),
    [sym_complex] = STATE(4718),
    [sym_rational] = STATE(4718),
    [sym_nil] = STATE(4705),
    [sym_string] = STATE(4702),
    [sym_subshell] = STATE(4702),
    [sym_string_array] = STATE(4702),
    [sym_symbol_array] = STATE(4702),
    [sym_delimited_symbol] = STATE(4699),
    [sym_regex] = STATE(4702),
    [sym_heredoc_body] = STATE(2164),
    [sym_lambda] = STATE(4735),
    [sym_identifier] = ACTIONS(2950),
    [anon_sym_LBRACE] = ACTIONS(2854),
    [anon_sym_LPAREN] = ACTIONS(2856),
    [anon_sym_COLON_COLON] = ACTIONS(2860),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(2862),
    [anon_sym_nil] = ACTIONS(2866),
    [anon_sym_LBRACK] = ACTIONS(2868),
    [anon_sym_DOT_DOT] = ACTIONS(2870),
    [sym_line] = ACTIONS(653),
    [sym_file] = ACTIONS(653),
    [sym_encoding] = ACTIONS(653),
    [anon_sym_CARET] = ACTIONS(655),
    [anon_sym_PLUS] = ACTIONS(2872),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2874),
    [sym_float] = ACTIONS(2874),
    [sym_self] = ACTIONS(653),
    [sym_true] = ACTIONS(653),
    [sym_false] = ACTIONS(653),
    [sym_constant] = ACTIONS(2954),
    [anon_sym_DASH_GT] = ACTIONS(2882),
    [sym_simple_symbol] = ACTIONS(2884),
    [sym__string_start] = ACTIONS(2886),
    [sym__symbol_start] = ACTIONS(2888),
    [sym__subshell_start] = ACTIONS(2890),
    [sym__regex_start] = ACTIONS(2892),
    [sym__string_array_start] = ACTIONS(2894),
    [sym__symbol_array_start] = ACTIONS(2896),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2898),
    [sym__unary_minus_num] = ACTIONS(2872),
  },
  [2165] = {
    [sym_heredoc_body] = STATE(2165),
    [sym_identifier] = ACTIONS(3138),
    [anon_sym_BEGIN] = ACTIONS(3138),
    [anon_sym_LBRACE] = ACTIONS(3140),
    [anon_sym_RBRACE] = ACTIONS(3140),
    [anon_sym_END] = ACTIONS(3138),
    [anon_sym_def] = ACTIONS(3138),
    [anon_sym_LPAREN] = ACTIONS(3140),
    [anon_sym_COLON_COLON] = ACTIONS(3140),
    [anon_sym_SEMI] = ACTIONS(3140),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(3140),
    [anon_sym_STAR] = ACTIONS(3140),
    [anon_sym_nil] = ACTIONS(3138),
    [anon_sym_class] = ACTIONS(3138),
    [anon_sym_module] = ACTIONS(3138),
    [anon_sym_return] = ACTIONS(3138),
    [anon_sym_yield] = ACTIONS(3138),
    [anon_sym_break] = ACTIONS(3138),
    [anon_sym_next] = ACTIONS(3138),
    [anon_sym_redo] = ACTIONS(3138),
    [anon_sym_retry] = ACTIONS(3138),
    [anon_sym_if] = ACTIONS(3138),
    [anon_sym_unless] = ACTIONS(3138),
    [anon_sym_while] = ACTIONS(3138),
    [anon_sym_until] = ACTIONS(3138),
    [anon_sym_for] = ACTIONS(3138),
    [anon_sym_case] = ACTIONS(3138),
    [anon_sym_LBRACK] = ACTIONS(3140),
    [anon_sym_DOT_DOT] = ACTIONS(3138),
    [anon_sym_begin] = ACTIONS(3138),
    [anon_sym_PLUS] = ACTIONS(3140),
    [anon_sym_defined_QMARK] = ACTIONS(3140),
    [anon_sym_not] = ACTIONS(3138),
    [anon_sym_BANG] = ACTIONS(3140),
    [anon_sym_TILDE] = ACTIONS(3140),
    [anon_sym_undef] = ACTIONS(3138),
    [anon_sym_alias] = ACTIONS(3138),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(3138),
    [sym_float] = ACTIONS(3138),
    [sym_super] = ACTIONS(3138),
    [sym_self] = ACTIONS(3138),
    [sym_true] = ACTIONS(3138),
    [sym_false] = ACTIONS(3138),
    [sym_constant] = ACTIONS(3138),
    [aux_sym_constant_suffix_token1] = ACTIONS(3140),
    [aux_sym_identifier_suffix_token1] = ACTIONS(3138),
    [sym_instance_variable] = ACTIONS(3140),
    [sym_class_variable] = ACTIONS(3140),
    [sym_global_variable] = ACTIONS(3140),
    [sym_character] = ACTIONS(3140),
    [anon_sym_DASH_GT] = ACTIONS(3140),
    [sym_simple_symbol] = ACTIONS(3140),
    [sym__string_start] = ACTIONS(3140),
    [sym__symbol_start] = ACTIONS(3140),
    [sym__subshell_start] = ACTIONS(3140),
    [sym__regex_start] = ACTIONS(3140),
    [sym__string_array_start] = ACTIONS(3140),
    [sym__symbol_array_start] = ACTIONS(3140),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(3140),
    [sym__unary_minus] = ACTIONS(3140),
    [sym__unary_minus_num] = ACTIONS(3140),
    [sym__binary_minus] = ACTIONS(3140),
    [sym__identifier_suffix] = ACTIONS(3140),
    [sym__constant_suffix] = ACTIONS(3140),
  },
  [2166] = {
    [sym_heredoc_body] = STATE(2166),
    [anon_sym_LBRACE] = ACTIONS(743),
    [anon_sym_RBRACE] = ACTIONS(743),
    [anon_sym_DOT] = ACTIONS(731),
    [anon_sym_COLON_COLON] = ACTIONS(2990),
    [anon_sym_rescue] = ACTIONS(2740),
    [anon_sym_EQ] = ACTIONS(731),
    [anon_sym_COMMA] = ACTIONS(743),
    [anon_sym_PIPE] = ACTIONS(731),
    [anon_sym_SEMI] = ACTIONS(2740),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(743),
    [anon_sym_AMP] = ACTIONS(731),
    [anon_sym_COLON] = ACTIONS(2992),
    [anon_sym_LT] = ACTIONS(731),
    [anon_sym_if] = ACTIONS(2740),
    [anon_sym_unless] = ACTIONS(2740),
    [anon_sym_while] = ACTIONS(2740),
    [anon_sym_until] = ACTIONS(2740),
    [anon_sym_do] = ACTIONS(743),
    [anon_sym_EQ_GT] = ACTIONS(2740),
    [anon_sym_LBRACK2] = ACTIONS(2740),
    [anon_sym_LPAREN2] = ACTIONS(743),
    [anon_sym_DOT_DOT] = ACTIONS(731),
    [anon_sym_CARET] = ACTIONS(731),
    [anon_sym_COLON_COLON2] = ACTIONS(743),
    [anon_sym_AMP_DOT] = ACTIONS(743),
    [anon_sym_PLUS_EQ] = ACTIONS(743),
    [anon_sym_DASH_EQ] = ACTIONS(743),
    [anon_sym_STAR_EQ] = ACTIONS(743),
    [anon_sym_STAR_STAR_EQ] = ACTIONS(743),
    [anon_sym_SLASH_EQ] = ACTIONS(743),
    [anon_sym_PIPE_PIPE_EQ] = ACTIONS(743),
    [anon_sym_PIPE_EQ] = ACTIONS(743),
    [anon_sym_AMP_AMP_EQ] = ACTIONS(743),
    [anon_sym_AMP_EQ] = ACTIONS(743),
    [anon_sym_PERCENT_EQ] = ACTIONS(743),
    [anon_sym_GT_GT_EQ] = ACTIONS(743),
    [anon_sym_LT_LT_EQ] = ACTIONS(743),
    [anon_sym_CARET_EQ] = ACTIONS(743),
    [anon_sym_QMARK] = ACTIONS(743),
    [anon_sym_and] = ACTIONS(743),
    [anon_sym_or] = ACTIONS(743),
    [anon_sym_PIPE_PIPE] = ACTIONS(731),
    [anon_sym_AMP_AMP] = ACTIONS(731),
    [anon_sym_LT_LT] = ACTIONS(731),
    [anon_sym_GT_GT] = ACTIONS(731),
    [anon_sym_LT_EQ] = ACTIONS(731),
    [anon_sym_GT] = ACTIONS(731),
    [anon_sym_GT_EQ] = ACTIONS(743),
    [anon_sym_PLUS] = ACTIONS(731),
    [anon_sym_SLASH] = ACTIONS(731),
    [anon_sym_PERCENT] = ACTIONS(731),
    [anon_sym_EQ_EQ] = ACTIONS(731),
    [anon_sym_BANG_EQ] = ACTIONS(743),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(743),
    [anon_sym_LT_EQ_GT] = ACTIONS(743),
    [anon_sym_EQ_TILDE] = ACTIONS(743),
    [anon_sym_BANG_TILDE] = ACTIONS(743),
    [sym_comment] = ACTIONS(3),
    [sym__line_break] = ACTIONS(2740),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym__binary_minus] = ACTIONS(743),
    [sym__binary_star] = ACTIONS(743),
    [sym__binary_star_star] = ACTIONS(743),
    [sym__element_reference_bracket] = ACTIONS(743),
  },
  [2167] = {
    [sym__pattern_primitive] = STATE(4457),
    [sym__pattern_lambda] = STATE(4408),
    [sym__pattern_literal] = STATE(4408),
    [sym__keyword_variable] = STATE(4396),
    [sym_unary_literal] = STATE(4383),
    [sym__literal] = STATE(4396),
    [sym__numeric] = STATE(4406),
    [sym__simple_numeric] = STATE(4383),
    [sym__int_or_float] = STATE(5557),
    [sym_complex] = STATE(4387),
    [sym_rational] = STATE(4387),
    [sym_nil] = STATE(4381),
    [sym_string] = STATE(4396),
    [sym_subshell] = STATE(4396),
    [sym_string_array] = STATE(4396),
    [sym_symbol_array] = STATE(4396),
    [sym_delimited_symbol] = STATE(4406),
    [sym_regex] = STATE(4396),
    [sym_heredoc_body] = STATE(2167),
    [sym_lambda] = STATE(4411),
    [anon_sym_LBRACE] = ACTIONS(3186),
    [anon_sym_RBRACE] = ACTIONS(3186),
    [anon_sym_RPAREN] = ACTIONS(3186),
    [anon_sym_end] = ACTIONS(3186),
    [anon_sym_rescue] = ACTIONS(3186),
    [anon_sym_COMMA] = ACTIONS(3186),
    [anon_sym_PIPE] = ACTIONS(3186),
    [anon_sym_SEMI] = ACTIONS(3186),
    [anon_sym_nil] = ACTIONS(3188),
    [anon_sym_if] = ACTIONS(3186),
    [anon_sym_unless] = ACTIONS(3186),
    [anon_sym_while] = ACTIONS(3186),
    [anon_sym_until] = ACTIONS(3186),
    [anon_sym_in] = ACTIONS(3186),
    [anon_sym_do] = ACTIONS(3186),
    [anon_sym_when] = ACTIONS(3186),
    [anon_sym_EQ_GT] = ACTIONS(3186),
    [sym_line] = ACTIONS(3190),
    [sym_file] = ACTIONS(3190),
    [sym_encoding] = ACTIONS(3190),
    [anon_sym_elsif] = ACTIONS(3186),
    [anon_sym_else] = ACTIONS(3186),
    [anon_sym_then] = ACTIONS(3186),
    [anon_sym_ensure] = ACTIONS(3186),
    [anon_sym_and] = ACTIONS(3186),
    [anon_sym_or] = ACTIONS(3186),
    [anon_sym_PLUS] = ACTIONS(2822),
    [sym_comment] = ACTIONS(79),
    [sym_integer] = ACTIONS(2824),
    [sym_float] = ACTIONS(2824),
    [sym_self] = ACTIONS(3190),
    [sym_true] = ACTIONS(3190),
    [sym_false] = ACTIONS(3190),
    [anon_sym_DASH_GT] = ACTIONS(3192),
    [sym__line_break] = ACTIONS(3186),
    [sym_simple_symbol] = ACTIONS(2830),
    [sym__string_start] = ACTIONS(2832),
    [sym__symbol_start] = ACTIONS(2834),
    [sym__subshell_start] = ACTIONS(2836),
    [sym__regex_start] = ACTIONS(2838),
    [sym__string_array_start] = ACTIONS(2840),
    [sym__symbol_array_start] = ACTIONS(2842),
    [sym__heredoc_body_start] = ACTIONS(5),
    [sym_heredoc_beginning] = ACTIONS(2844),
    [sym__unary_minus_num] = ACTIONS(2822),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(3196), 1,
      anon_sym_do,
    STATE(2168), 1,
      sym_heredoc_body,
    STATE(2818), 1,
      sym_argument_list,
    STATE(2910), 1,
      sym_do_block,
    STATE(2911), 1,
      sym_block,
    ACTIONS(127), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(121), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [87] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(3196), 1,
      anon_sym_do,
    STATE(2169), 1,
      sym_heredoc_body,
    STATE(2823), 1,
      sym_argument_list,
    STATE(2877), 1,
      sym_block,
    STATE(2937), 1,
      sym_do_block,
    ACTIONS(179), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(177), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [174] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3080), 1,
      anon_sym_EQ_GT,
    ACTIONS(3184), 1,
      anon_sym_COLON,
    STATE(2170), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 42,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [253] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3198), 1,
      anon_sym_EQ,
    STATE(2171), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3200), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [334] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3202), 1,
      anon_sym_EQ,
    STATE(2172), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3204), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [415] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3206), 1,
      anon_sym_EQ,
    STATE(2173), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3208), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [496] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2992), 1,
      anon_sym_COLON,
    STATE(2174), 1,
      sym_heredoc_body,
    ACTIONS(3080), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [574] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2980), 1,
      anon_sym_LBRACE,
    STATE(2175), 1,
      sym_heredoc_body,
    STATE(2112), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2982), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2978), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [652] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2980), 1,
      anon_sym_LBRACE,
    STATE(2176), 1,
      sym_heredoc_body,
    STATE(2106), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2988), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2986), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [730] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2974), 1,
      anon_sym_COLON_COLON,
    STATE(2177), 1,
      sym_heredoc_body,
    ACTIONS(2976), 2,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
    ACTIONS(2958), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2960), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [808] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2178), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [882] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3210), 1,
      anon_sym_EQ,
    STATE(2179), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3212), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [961] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3214), 1,
      anon_sym_EQ,
    STATE(2180), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3216), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [1040] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2181), 1,
      sym_heredoc_body,
    ACTIONS(2624), 2,
      anon_sym_LBRACE,
      anon_sym_LPAREN2,
    ACTIONS(2622), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2620), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [1115] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3218), 1,
      anon_sym_EQ,
    STATE(2182), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3220), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [1196] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3222), 1,
      anon_sym_EQ,
    STATE(2183), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3224), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [1275] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3226), 1,
      anon_sym_EQ,
    STATE(2184), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3228), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [1356] = 40,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1653), 1,
      anon_sym_CARET,
    ACTIONS(2800), 1,
      sym_identifier,
    ACTIONS(2802), 1,
      anon_sym_LBRACE,
    ACTIONS(2806), 1,
      anon_sym_LPAREN,
    ACTIONS(2808), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2816), 1,
      anon_sym_nil,
    ACTIONS(2818), 1,
      anon_sym_LBRACK,
    ACTIONS(2826), 1,
      sym_constant,
    ACTIONS(2828), 1,
      anon_sym_DASH_GT,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    ACTIONS(2916), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(2920), 1,
      anon_sym_DOT_DOT,
    STATE(2185), 1,
      sym_heredoc_body,
    STATE(4375), 1,
      sym__pattern_constant,
    STATE(4376), 1,
      sym__pattern_constant_resolution,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4438), 1,
      sym__pattern_range,
    STATE(4456), 1,
      sym__pattern_expr_basic,
    STATE(4664), 1,
      sym__pattern_primitive,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2922), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2924), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4451), 2,
      sym_variable_reference_pattern,
      sym_expression_reference_pattern,
    STATE(4446), 5,
      sym_array_pattern,
      sym_find_pattern,
      sym_hash_pattern,
      sym_parenthesized_pattern,
      sym__pattern_value,
    ACTIONS(1651), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [1499] = 40,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(655), 1,
      anon_sym_CARET,
    ACTIONS(2854), 1,
      anon_sym_LBRACE,
    ACTIONS(2856), 1,
      anon_sym_LPAREN,
    ACTIONS(2860), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2866), 1,
      anon_sym_nil,
    ACTIONS(2868), 1,
      anon_sym_LBRACK,
    ACTIONS(2882), 1,
      anon_sym_DASH_GT,
    ACTIONS(2884), 1,
      sym_simple_symbol,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(2888), 1,
      sym__symbol_start,
    ACTIONS(2890), 1,
      sym__subshell_start,
    ACTIONS(2892), 1,
      sym__regex_start,
    ACTIONS(2894), 1,
      sym__string_array_start,
    ACTIONS(2896), 1,
      sym__symbol_array_start,
    ACTIONS(2898), 1,
      sym_heredoc_beginning,
    ACTIONS(2904), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(2906), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2950), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      sym_constant,
    STATE(2186), 1,
      sym_heredoc_body,
    STATE(4656), 1,
      sym__pattern_constant,
    STATE(4657), 1,
      sym__pattern_constant_resolution,
    STATE(4705), 1,
      sym_nil,
    STATE(4735), 1,
      sym_lambda,
    STATE(4750), 1,
      sym__pattern_primitive,
    STATE(4757), 1,
      sym__pattern_range,
    STATE(4763), 1,
      sym__pattern_expr_basic,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2908), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2910), 2,
      sym_integer,
      sym_float,
    STATE(4692), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4699), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4714), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
    STATE(4760), 2,
      sym_variable_reference_pattern,
      sym_expression_reference_pattern,
    STATE(4819), 5,
      sym_array_pattern,
      sym_find_pattern,
      sym_hash_pattern,
      sym_parenthesized_pattern,
      sym__pattern_value,
    ACTIONS(653), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4702), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [1642] = 40,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1709), 1,
      anon_sym_CARET,
    ACTIONS(2800), 1,
      sym_identifier,
    ACTIONS(2802), 1,
      anon_sym_LBRACE,
    ACTIONS(2806), 1,
      anon_sym_LPAREN,
    ACTIONS(2808), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2816), 1,
      anon_sym_nil,
    ACTIONS(2818), 1,
      anon_sym_LBRACK,
    ACTIONS(2826), 1,
      sym_constant,
    ACTIONS(2828), 1,
      anon_sym_DASH_GT,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    ACTIONS(2934), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(2938), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2942), 1,
      sym__symbol_start,
    STATE(2187), 1,
      sym_heredoc_body,
    STATE(4375), 1,
      sym__pattern_constant,
    STATE(4376), 1,
      sym__pattern_constant_resolution,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4438), 1,
      sym__pattern_range,
    STATE(4456), 1,
      sym__pattern_expr_basic,
    STATE(4593), 1,
      sym__pattern_primitive,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2822), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2824), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4451), 2,
      sym_variable_reference_pattern,
      sym_expression_reference_pattern,
    STATE(4446), 5,
      sym_array_pattern,
      sym_find_pattern,
      sym_hash_pattern,
      sym_parenthesized_pattern,
      sym__pattern_value,
    ACTIONS(1651), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [1785] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3230), 1,
      anon_sym_LBRACE,
    ACTIONS(3232), 1,
      anon_sym_do,
    STATE(2188), 1,
      sym_heredoc_body,
    STATE(2238), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2982), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2978), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [1864] = 40,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(655), 1,
      anon_sym_CARET,
    ACTIONS(2854), 1,
      anon_sym_LBRACE,
    ACTIONS(2856), 1,
      anon_sym_LPAREN,
    ACTIONS(2860), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2862), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(2866), 1,
      anon_sym_nil,
    ACTIONS(2868), 1,
      anon_sym_LBRACK,
    ACTIONS(2870), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2882), 1,
      anon_sym_DASH_GT,
    ACTIONS(2884), 1,
      sym_simple_symbol,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(2888), 1,
      sym__symbol_start,
    ACTIONS(2890), 1,
      sym__subshell_start,
    ACTIONS(2892), 1,
      sym__regex_start,
    ACTIONS(2894), 1,
      sym__string_array_start,
    ACTIONS(2896), 1,
      sym__symbol_array_start,
    ACTIONS(2898), 1,
      sym_heredoc_beginning,
    ACTIONS(2950), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      sym_constant,
    STATE(2189), 1,
      sym_heredoc_body,
    STATE(4656), 1,
      sym__pattern_constant,
    STATE(4657), 1,
      sym__pattern_constant_resolution,
    STATE(4705), 1,
      sym_nil,
    STATE(4735), 1,
      sym_lambda,
    STATE(4746), 1,
      sym__pattern_primitive,
    STATE(4757), 1,
      sym__pattern_range,
    STATE(4763), 1,
      sym__pattern_expr_basic,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2872), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2874), 2,
      sym_integer,
      sym_float,
    STATE(4692), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4699), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4714), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
    STATE(4760), 2,
      sym_variable_reference_pattern,
      sym_expression_reference_pattern,
    STATE(4819), 5,
      sym_array_pattern,
      sym_find_pattern,
      sym_hash_pattern,
      sym_parenthesized_pattern,
      sym__pattern_value,
    ACTIONS(653), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4702), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [2007] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2190), 1,
      sym_heredoc_body,
    ACTIONS(2998), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2082] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3230), 1,
      anon_sym_LBRACE,
    ACTIONS(3232), 1,
      anon_sym_do,
    STATE(2191), 1,
      sym_heredoc_body,
    STATE(2243), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2988), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2986), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2161] = 40,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1653), 1,
      anon_sym_CARET,
    ACTIONS(2800), 1,
      sym_identifier,
    ACTIONS(2802), 1,
      anon_sym_LBRACE,
    ACTIONS(2806), 1,
      anon_sym_LPAREN,
    ACTIONS(2808), 1,
      anon_sym_COLON_COLON,
    ACTIONS(2812), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(2816), 1,
      anon_sym_nil,
    ACTIONS(2818), 1,
      anon_sym_LBRACK,
    ACTIONS(2820), 1,
      anon_sym_DOT_DOT,
    ACTIONS(2826), 1,
      sym_constant,
    ACTIONS(2828), 1,
      anon_sym_DASH_GT,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    STATE(2192), 1,
      sym_heredoc_body,
    STATE(4375), 1,
      sym__pattern_constant,
    STATE(4376), 1,
      sym__pattern_constant_resolution,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4438), 1,
      sym__pattern_range,
    STATE(4448), 1,
      sym__pattern_primitive,
    STATE(4456), 1,
      sym__pattern_expr_basic,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2822), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2824), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4451), 2,
      sym_variable_reference_pattern,
      sym_expression_reference_pattern,
    STATE(4446), 5,
      sym_array_pattern,
      sym_find_pattern,
      sym_hash_pattern,
      sym_parenthesized_pattern,
      sym__pattern_value,
    ACTIONS(1651), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [2304] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3234), 1,
      anon_sym_EQ,
    STATE(2193), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3236), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2385] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3238), 1,
      anon_sym_EQ,
    STATE(2194), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3240), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2464] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2974), 1,
      anon_sym_COLON_COLON,
    STATE(2195), 1,
      sym_heredoc_body,
    ACTIONS(2976), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
    ACTIONS(2958), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2960), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2541] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3242), 1,
      anon_sym_EQ,
    STATE(2196), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3244), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2621] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2197), 1,
      sym_heredoc_body,
    ACTIONS(2614), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2616), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2693] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2198), 1,
      sym_heredoc_body,
    ACTIONS(2638), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2765] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3246), 1,
      anon_sym_EQ,
    STATE(2199), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3248), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2843] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    STATE(2200), 1,
      sym_heredoc_body,
    STATE(2264), 1,
      sym_argument_list,
    STATE(2316), 1,
      sym_do_block,
    STATE(2318), 1,
      sym_block,
    ACTIONS(411), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(409), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [2925] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3250), 1,
      anon_sym_EQ,
    STATE(2201), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3252), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3005] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3254), 1,
      anon_sym_EQ,
    STATE(2202), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3256), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3085] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    STATE(2203), 1,
      sym_heredoc_body,
    STATE(2877), 1,
      sym_block,
    STATE(2937), 1,
      sym_do_block,
    STATE(3096), 1,
      sym_argument_list,
    ACTIONS(179), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(177), 36,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3167] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    STATE(2204), 1,
      sym_heredoc_body,
    STATE(2910), 1,
      sym_do_block,
    STATE(2911), 1,
      sym_block,
    STATE(3108), 1,
      sym_argument_list,
    ACTIONS(127), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(121), 36,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3249] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3258), 1,
      anon_sym_EQ,
    STATE(2205), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3260), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3327] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2206), 1,
      sym_heredoc_body,
    ACTIONS(731), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3399] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2207), 1,
      sym_heredoc_body,
    ACTIONS(2630), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2632), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3471] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2208), 1,
      sym_heredoc_body,
    ACTIONS(2624), 3,
      anon_sym_LBRACE,
      anon_sym_do,
      anon_sym_LPAREN2,
    ACTIONS(2622), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2620), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3545] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    ACTIONS(2964), 1,
      anon_sym_do,
    STATE(2209), 1,
      sym_heredoc_body,
    STATE(2257), 1,
      sym_argument_list,
    STATE(2316), 1,
      sym_do_block,
    STATE(2318), 1,
      sym_block,
    ACTIONS(411), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(409), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3629] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2210), 1,
      sym_heredoc_body,
    ACTIONS(2626), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3701] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2211), 1,
      sym_heredoc_body,
    ACTIONS(2642), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2644), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3773] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2212), 1,
      sym_heredoc_body,
    ACTIONS(2618), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2624), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3845] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2213), 1,
      sym_heredoc_body,
    ACTIONS(2634), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2636), 40,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3917] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3262), 1,
      anon_sym_EQ,
    STATE(2214), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3264), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [3995] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3266), 1,
      anon_sym_EQ,
    STATE(2215), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3268), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4073] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2216), 1,
      sym_heredoc_body,
    ACTIONS(3022), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3020), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4144] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    STATE(2217), 1,
      sym_heredoc_body,
    STATE(2244), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    ACTIONS(3272), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3270), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4221] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2218), 1,
      sym_heredoc_body,
    ACTIONS(3038), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3036), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4292] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2219), 1,
      sym_heredoc_body,
    ACTIONS(127), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(121), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4363] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2220), 1,
      sym_heredoc_body,
    ACTIONS(3062), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3060), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4434] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2221), 1,
      sym_heredoc_body,
    ACTIONS(127), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(121), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4505] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2222), 1,
      sym_heredoc_body,
    ACTIONS(3042), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3040), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4576] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2223), 1,
      sym_heredoc_body,
    ACTIONS(3046), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3044), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4647] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2224), 1,
      sym_heredoc_body,
    ACTIONS(3050), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3048), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4718] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2225), 1,
      sym_heredoc_body,
    ACTIONS(2958), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2960), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4789] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3280), 1,
      anon_sym_ri,
    STATE(2226), 1,
      sym_heredoc_body,
    ACTIONS(3278), 2,
      anon_sym_i,
      anon_sym_r,
    ACTIONS(3276), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3274), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4864] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2227), 1,
      sym_heredoc_body,
    ACTIONS(3088), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3086), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [4935] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2228), 1,
      sym_heredoc_body,
    ACTIONS(3054), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3052), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5006] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2990), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3282), 1,
      anon_sym_COLON,
    STATE(2229), 1,
      sym_heredoc_body,
    ACTIONS(2740), 2,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 36,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5083] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2230), 1,
      sym_heredoc_body,
    ACTIONS(3014), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3012), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5154] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2231), 1,
      sym_heredoc_body,
    ACTIONS(3066), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3064), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5225] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2232), 1,
      sym_heredoc_body,
    ACTIONS(3002), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3000), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5296] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2990), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3284), 1,
      anon_sym_COLON,
    STATE(2233), 1,
      sym_heredoc_body,
    ACTIONS(2740), 2,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 36,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5373] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2234), 1,
      sym_heredoc_body,
    STATE(2281), 1,
      sym_string,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5450] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2235), 1,
      sym_heredoc_body,
    ACTIONS(3018), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3016), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5521] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2236), 1,
      sym_heredoc_body,
    ACTIONS(3010), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5592] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2237), 1,
      sym_heredoc_body,
    ACTIONS(3026), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3024), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5663] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2238), 1,
      sym_heredoc_body,
    ACTIONS(3084), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3082), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5734] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2239), 1,
      sym_heredoc_body,
    ACTIONS(3030), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3028), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5805] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2240), 1,
      sym_heredoc_body,
    STATE(2281), 1,
      sym_string,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5884] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2974), 1,
      anon_sym_COLON_COLON,
    STATE(2241), 1,
      sym_heredoc_body,
    ACTIONS(2976), 3,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
    ACTIONS(2958), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2960), 36,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [5959] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2242), 1,
      sym_heredoc_body,
    ACTIONS(3034), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3032), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6030] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2243), 1,
      sym_heredoc_body,
    ACTIONS(3074), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3072), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6101] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3292), 1,
      sym__string_start,
    STATE(2281), 1,
      sym_string,
    STATE(2244), 2,
      sym_heredoc_body,
      aux_sym_chained_string_repeat1,
    ACTIONS(3290), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3288), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6176] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2245), 1,
      sym_heredoc_body,
    ACTIONS(3058), 18,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3056), 39,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6247] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2246), 1,
      sym_heredoc_body,
    STATE(2317), 1,
      sym_argument_list,
    ACTIONS(419), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(417), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6321] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2247), 1,
      sym_heredoc_body,
    STATE(2360), 1,
      sym_argument_list,
    ACTIONS(3297), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3295), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6395] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2248), 1,
      sym_heredoc_body,
    STATE(2319), 1,
      sym_argument_list,
    ACTIONS(415), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(413), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6469] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2249), 1,
      sym_heredoc_body,
    STATE(2321), 1,
      sym_argument_list,
    ACTIONS(427), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(425), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6543] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3299), 1,
      anon_sym_EQ,
    STATE(2250), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3301), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 25,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6619] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2251), 1,
      sym_heredoc_body,
    ACTIONS(2998), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(127), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(121), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6691] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2252), 1,
      sym_heredoc_body,
    ACTIONS(3305), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3303), 47,
      sym__line_break,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6761] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2253), 1,
      sym_heredoc_body,
    ACTIONS(2642), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2644), 47,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6831] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(3307), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2254), 1,
      sym_heredoc_body,
    STATE(2281), 1,
      sym_string,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6909] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    ACTIONS(2964), 1,
      anon_sym_do,
    STATE(2255), 1,
      sym_heredoc_body,
    STATE(2448), 1,
      sym_do_block,
    STATE(2449), 1,
      sym_block,
    ACTIONS(3311), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3309), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [6987] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2990), 1,
      anon_sym_COLON_COLON,
    ACTIONS(3313), 1,
      anon_sym_COLON,
    STATE(2256), 1,
      sym_heredoc_body,
    ACTIONS(2740), 2,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 35,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7063] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    ACTIONS(2964), 1,
      anon_sym_do,
    STATE(2257), 1,
      sym_heredoc_body,
    STATE(2451), 1,
      sym_do_block,
    STATE(2452), 1,
      sym_block,
    ACTIONS(3317), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3315), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7141] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2258), 1,
      sym_heredoc_body,
    ACTIONS(2614), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2616), 47,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7211] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2259), 1,
      sym_heredoc_body,
    ACTIONS(3321), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3319), 47,
      sym__line_break,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7281] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2260), 1,
      sym_heredoc_body,
    STATE(2294), 1,
      sym_argument_list,
    ACTIONS(423), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(421), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7355] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    STATE(2261), 1,
      sym_heredoc_body,
    STATE(2448), 1,
      sym_do_block,
    STATE(2449), 1,
      sym_block,
    ACTIONS(3311), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3309), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7431] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2262), 1,
      sym_heredoc_body,
    STATE(2238), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2982), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2978), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7503] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    ACTIONS(2964), 1,
      anon_sym_do,
    STATE(2263), 1,
      sym_heredoc_body,
    STATE(2291), 1,
      sym_do_block,
    STATE(2292), 1,
      sym_block,
    ACTIONS(3325), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3323), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7581] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    STATE(2264), 1,
      sym_heredoc_body,
    STATE(2451), 1,
      sym_do_block,
    STATE(2452), 1,
      sym_block,
    ACTIONS(3317), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3315), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7657] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2265), 1,
      sym_heredoc_body,
    STATE(2243), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2988), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2986), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7729] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    STATE(2266), 1,
      sym_heredoc_body,
    STATE(2291), 1,
      sym_do_block,
    STATE(2292), 1,
      sym_block,
    ACTIONS(3325), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3323), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7805] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2267), 1,
      sym_heredoc_body,
    STATE(2361), 1,
      sym_argument_list,
    ACTIONS(3329), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3327), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7879] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    ACTIONS(3331), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2268), 1,
      sym_heredoc_body,
    STATE(2281), 1,
      sym_string,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [7959] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3333), 1,
      anon_sym_EQ,
    STATE(2269), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3335), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8034] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2624), 1,
      anon_sym_LPAREN2,
    STATE(2270), 1,
      sym_heredoc_body,
    ACTIONS(2622), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2620), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8105] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3337), 1,
      anon_sym_EQ,
    STATE(2271), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3339), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8180] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3341), 1,
      anon_sym_EQ,
    STATE(2272), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3343), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8255] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2273), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 47,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8324] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3345), 1,
      anon_sym_EQ,
    STATE(2274), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3347), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8399] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3349), 1,
      anon_sym_EQ,
    STATE(2275), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3351), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8474] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2276), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 47,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8543] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2998), 1,
      anon_sym_EQ_GT,
    STATE(2277), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 37,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8614] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3353), 1,
      anon_sym_EQ,
    STATE(2278), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3355), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8689] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3080), 1,
      anon_sym_EQ_GT,
    ACTIONS(3282), 1,
      anon_sym_COLON,
    STATE(2279), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 36,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8762] = 26,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    ACTIONS(2942), 1,
      sym__symbol_start,
    ACTIONS(3188), 1,
      anon_sym_nil,
    ACTIONS(3192), 1,
      anon_sym_DASH_GT,
    STATE(2280), 1,
      sym_heredoc_body,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4457), 1,
      sym__pattern_primitive,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2822), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2824), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    ACTIONS(3190), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
    ACTIONS(3186), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [8873] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2281), 1,
      sym_heredoc_body,
    ACTIONS(3359), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3357), 47,
      sym__line_break,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [8942] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3080), 1,
      anon_sym_EQ_GT,
    ACTIONS(3284), 1,
      anon_sym_COLON,
    STATE(2282), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 36,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9015] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3361), 1,
      anon_sym_EQ,
    STATE(2283), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3363), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9090] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3365), 1,
      anon_sym_EQ,
    STATE(2284), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3367), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9165] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2285), 1,
      sym_heredoc_body,
    ACTIONS(3371), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3369), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9233] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2286), 1,
      sym_heredoc_body,
    ACTIONS(3375), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3373), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9301] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2287), 1,
      sym_heredoc_body,
    ACTIONS(3379), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3377), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9369] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2288), 1,
      sym_heredoc_body,
    ACTIONS(3383), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3381), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9437] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2289), 1,
      sym_heredoc_body,
    ACTIONS(3387), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3385), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9505] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2290), 1,
      sym_heredoc_body,
    ACTIONS(3391), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3389), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9573] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2291), 1,
      sym_heredoc_body,
    ACTIONS(3395), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3393), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9641] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2292), 1,
      sym_heredoc_body,
    ACTIONS(3395), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3393), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9709] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2293), 1,
      sym_heredoc_body,
    ACTIONS(3276), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3274), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9777] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2294), 1,
      sym_heredoc_body,
    ACTIONS(3399), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3397), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9845] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2295), 1,
      sym_heredoc_body,
    ACTIONS(3403), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3401), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9913] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2296), 1,
      sym_heredoc_body,
    ACTIONS(3407), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3405), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [9981] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2297), 1,
      sym_heredoc_body,
    ACTIONS(3411), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3409), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10049] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2298), 1,
      sym_heredoc_body,
    ACTIONS(3415), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3413), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10117] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2299), 1,
      sym_heredoc_body,
    ACTIONS(3419), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3417), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10185] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2300), 1,
      sym_heredoc_body,
    ACTIONS(3423), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3421), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10253] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3427), 1,
      anon_sym_LPAREN,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2301), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4540), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5604), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [10353] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2302), 1,
      sym_heredoc_body,
    ACTIONS(3447), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3445), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10421] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2303), 1,
      sym_heredoc_body,
    ACTIONS(3451), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3449), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10489] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2304), 1,
      sym_heredoc_body,
    ACTIONS(3455), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3453), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10557] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2305), 1,
      sym_heredoc_body,
    ACTIONS(3459), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3457), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10625] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2306), 1,
      sym_heredoc_body,
    ACTIONS(3463), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3461), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10693] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2307), 1,
      sym_heredoc_body,
    ACTIONS(3465), 2,
      anon_sym_LBRACE,
      anon_sym_do,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10763] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2308), 1,
      sym_heredoc_body,
    ACTIONS(3463), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3461), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10831] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2309), 1,
      sym_heredoc_body,
    ACTIONS(3469), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3467), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10899] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2998), 1,
      anon_sym_EQ_GT,
    STATE(2310), 1,
      sym_heredoc_body,
    ACTIONS(127), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(121), 36,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [10969] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2311), 1,
      sym_heredoc_body,
    ACTIONS(3473), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3471), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11037] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3080), 1,
      anon_sym_EQ_GT,
    ACTIONS(3313), 1,
      anon_sym_COLON,
    STATE(2312), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 35,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11109] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2313), 1,
      sym_heredoc_body,
    ACTIONS(3477), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3475), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11177] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2314), 1,
      sym_heredoc_body,
    ACTIONS(3481), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3479), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11245] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3485), 1,
      anon_sym_EQ_GT,
    STATE(2315), 1,
      sym_heredoc_body,
    ACTIONS(3483), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3465), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11315] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2316), 1,
      sym_heredoc_body,
    ACTIONS(3489), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3487), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11383] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2317), 1,
      sym_heredoc_body,
    ACTIONS(3493), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3491), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11451] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2318), 1,
      sym_heredoc_body,
    ACTIONS(3489), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3487), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11519] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2319), 1,
      sym_heredoc_body,
    ACTIONS(3497), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3495), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11587] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3503), 1,
      anon_sym_EQ_GT,
    STATE(2320), 1,
      sym_heredoc_body,
    ACTIONS(3501), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3499), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11657] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2321), 1,
      sym_heredoc_body,
    ACTIONS(3507), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3505), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11725] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2322), 1,
      sym_heredoc_body,
    ACTIONS(3511), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3509), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11793] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2323), 1,
      sym_heredoc_body,
    ACTIONS(3515), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3513), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11861] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2990), 1,
      anon_sym_COLON_COLON,
    STATE(2324), 1,
      sym_heredoc_body,
    ACTIONS(2740), 2,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [11933] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2325), 1,
      sym_heredoc_body,
    ACTIONS(3519), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3517), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12001] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2326), 1,
      sym_heredoc_body,
    ACTIONS(3523), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3521), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12069] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2327), 1,
      sym_heredoc_body,
    ACTIONS(3527), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3525), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12137] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3529), 1,
      anon_sym_EQ,
    STATE(2328), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3531), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 22,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12213] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2329), 1,
      sym_heredoc_body,
    ACTIONS(3535), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3533), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12281] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2330), 1,
      sym_heredoc_body,
    ACTIONS(3539), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3537), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12349] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2331), 1,
      sym_heredoc_body,
    ACTIONS(3543), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3541), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12417] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2332), 1,
      sym_heredoc_body,
    ACTIONS(3547), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3545), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12485] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2333), 1,
      sym_heredoc_body,
    ACTIONS(3551), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3549), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12553] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2334), 1,
      sym_heredoc_body,
    ACTIONS(3555), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3553), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12621] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2335), 1,
      sym_heredoc_body,
    ACTIONS(3559), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3557), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12689] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2336), 1,
      sym_heredoc_body,
    ACTIONS(3563), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3561), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12757] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2337), 1,
      sym_heredoc_body,
    ACTIONS(3567), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3565), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12825] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2338), 1,
      sym_heredoc_body,
    ACTIONS(3571), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3569), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12893] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2339), 1,
      sym_heredoc_body,
    ACTIONS(3575), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3573), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [12961] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2340), 1,
      sym_heredoc_body,
    ACTIONS(3579), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3577), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13029] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2341), 1,
      sym_heredoc_body,
    ACTIONS(3583), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3581), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13097] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3585), 1,
      anon_sym_EQ,
    STATE(2342), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3587), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 23,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13171] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2343), 1,
      sym_heredoc_body,
    ACTIONS(3591), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13239] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2344), 1,
      sym_heredoc_body,
    ACTIONS(3595), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3593), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13307] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym_heredoc_body,
    ACTIONS(3599), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3597), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13375] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3601), 1,
      anon_sym_EQ,
    STATE(2346), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3603), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13449] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2347), 1,
      sym_heredoc_body,
    ACTIONS(3607), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3605), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13517] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2348), 1,
      sym_heredoc_body,
    ACTIONS(3611), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3609), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13585] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2349), 1,
      sym_heredoc_body,
    ACTIONS(3615), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3613), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13653] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2350), 1,
      sym_heredoc_body,
    ACTIONS(3619), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3617), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13721] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2351), 1,
      sym_heredoc_body,
    ACTIONS(3623), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3621), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13789] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3625), 1,
      anon_sym_EQ_GT,
    STATE(2352), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13859] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2353), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13927] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2354), 1,
      sym_heredoc_body,
    ACTIONS(3633), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3631), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [13995] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2355), 1,
      sym_heredoc_body,
    ACTIONS(3637), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3635), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14063] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2356), 1,
      sym_heredoc_body,
    ACTIONS(3641), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3639), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14131] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2357), 1,
      sym_heredoc_body,
    ACTIONS(3645), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3643), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14199] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2358), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14267] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2359), 1,
      sym_heredoc_body,
    ACTIONS(3653), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3651), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14335] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2360), 1,
      sym_heredoc_body,
    ACTIONS(3657), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3655), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14403] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2361), 1,
      sym_heredoc_body,
    ACTIONS(3661), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3659), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14471] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2362), 1,
      sym_heredoc_body,
    ACTIONS(3665), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3663), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14539] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2363), 1,
      sym_heredoc_body,
    ACTIONS(3669), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3667), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14607] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2364), 1,
      sym_heredoc_body,
    ACTIONS(3673), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3671), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14675] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2365), 1,
      sym_heredoc_body,
    ACTIONS(3677), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3675), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14743] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2366), 1,
      sym_heredoc_body,
    ACTIONS(3681), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3679), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14811] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2367), 1,
      sym_heredoc_body,
    ACTIONS(3685), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3683), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14879] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2368), 1,
      sym_heredoc_body,
    ACTIONS(3689), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3687), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [14947] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2369), 1,
      sym_heredoc_body,
    ACTIONS(3018), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3016), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15015] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2370), 1,
      sym_heredoc_body,
    ACTIONS(3022), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3020), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15083] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2371), 1,
      sym_heredoc_body,
    ACTIONS(3026), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3024), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15151] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2372), 1,
      sym_heredoc_body,
    ACTIONS(3693), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3691), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15219] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2373), 1,
      sym_heredoc_body,
    ACTIONS(3697), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3695), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15287] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2374), 1,
      sym_heredoc_body,
    ACTIONS(3030), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3028), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15355] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2375), 1,
      sym_heredoc_body,
    ACTIONS(3034), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3032), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15423] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2376), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15491] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2377), 1,
      sym_heredoc_body,
    ACTIONS(3701), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3699), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15559] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2378), 1,
      sym_heredoc_body,
    ACTIONS(3705), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3703), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15627] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2379), 1,
      sym_heredoc_body,
    ACTIONS(3709), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3707), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15695] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2380), 1,
      sym_heredoc_body,
    ACTIONS(3713), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3711), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15763] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2381), 1,
      sym_heredoc_body,
    ACTIONS(3717), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3715), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15831] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2382), 1,
      sym_heredoc_body,
    ACTIONS(3721), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3719), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15899] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2383), 1,
      sym_heredoc_body,
    ACTIONS(3725), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3723), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [15967] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2384), 1,
      sym_heredoc_body,
    ACTIONS(3729), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3727), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16035] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2385), 1,
      sym_heredoc_body,
    ACTIONS(3733), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3731), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16103] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2386), 1,
      sym_heredoc_body,
    ACTIONS(3737), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3735), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16171] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2387), 1,
      sym_heredoc_body,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16239] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2388), 1,
      sym_heredoc_body,
    ACTIONS(3741), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3739), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16307] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2389), 1,
      sym_heredoc_body,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16375] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2390), 1,
      sym_heredoc_body,
    ACTIONS(3014), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3012), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16443] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2391), 1,
      sym_heredoc_body,
    ACTIONS(3483), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3465), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16511] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3747), 1,
      anon_sym_LPAREN,
    STATE(2392), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4541), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5430), 1,
      sym__method_rest,
    STATE(5582), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [16611] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2393), 1,
      sym_heredoc_body,
    ACTIONS(3751), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3749), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16679] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2394), 1,
      sym_heredoc_body,
    ACTIONS(3755), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3753), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16747] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2395), 1,
      sym_heredoc_body,
    ACTIONS(3759), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3757), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16815] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2396), 1,
      sym_heredoc_body,
    ACTIONS(3763), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3761), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16883] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2397), 1,
      sym_heredoc_body,
    ACTIONS(3767), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3765), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [16951] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2398), 1,
      sym_heredoc_body,
    ACTIONS(3771), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3769), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17019] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2399), 1,
      sym_heredoc_body,
    ACTIONS(3775), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3773), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17087] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2400), 1,
      sym_heredoc_body,
    ACTIONS(3779), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3777), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17155] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2401), 1,
      sym_heredoc_body,
    ACTIONS(3783), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3781), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17223] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2402), 1,
      sym_heredoc_body,
    ACTIONS(3787), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3785), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17291] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3789), 1,
      anon_sym_EQ,
    STATE(2403), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3791), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17365] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2404), 1,
      sym_heredoc_body,
    ACTIONS(3795), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3793), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17433] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2405), 1,
      sym_heredoc_body,
    ACTIONS(3799), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3797), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17501] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2406), 1,
      sym_heredoc_body,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17569] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3801), 1,
      anon_sym_LPAREN,
    STATE(2407), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4539), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5603), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [17669] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2408), 1,
      sym_heredoc_body,
    ACTIONS(3038), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3036), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17737] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2409), 1,
      sym_heredoc_body,
    ACTIONS(3042), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3040), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17805] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(3803), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2410), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 42,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17881] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3805), 1,
      anon_sym_EQ,
    STATE(2411), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3807), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [17955] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2412), 1,
      sym_heredoc_body,
    ACTIONS(3046), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3044), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18023] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3809), 1,
      anon_sym_LPAREN,
    STATE(2413), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4548), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5581), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [18123] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2414), 1,
      sym_heredoc_body,
    ACTIONS(3813), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3811), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18191] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2415), 1,
      sym_heredoc_body,
    ACTIONS(3817), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3815), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18259] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2416), 1,
      sym_heredoc_body,
    ACTIONS(3821), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3819), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18327] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2417), 1,
      sym_heredoc_body,
    ACTIONS(3825), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3823), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18395] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2418), 1,
      sym_heredoc_body,
    ACTIONS(3829), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3827), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18463] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3831), 1,
      anon_sym_LPAREN,
    STATE(2419), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4556), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5632), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [18563] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2420), 1,
      sym_heredoc_body,
    ACTIONS(3050), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3048), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18631] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3833), 1,
      anon_sym_LPAREN,
    STATE(2421), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4546), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5591), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [18731] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2422), 1,
      sym_heredoc_body,
    ACTIONS(3837), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3835), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18799] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2423), 1,
      sym_heredoc_body,
    ACTIONS(3841), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3839), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [18867] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3843), 1,
      anon_sym_LPAREN,
    STATE(2424), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4554), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5634), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [18967] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2425), 1,
      sym_heredoc_body,
    ACTIONS(3847), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3845), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19035] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2426), 1,
      sym_heredoc_body,
    ACTIONS(3851), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3849), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19103] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2427), 1,
      sym_heredoc_body,
    ACTIONS(3855), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3853), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19171] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3857), 1,
      anon_sym_LPAREN,
    STATE(2428), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4553), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5587), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [19271] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3859), 1,
      anon_sym_DOT,
    ACTIONS(3861), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(3863), 1,
      anon_sym_AMP_DOT,
    ACTIONS(3865), 1,
      sym__element_reference_bracket,
    STATE(2429), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19347] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2430), 1,
      sym_heredoc_body,
    ACTIONS(3869), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3867), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19415] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2431), 1,
      sym_heredoc_body,
    ACTIONS(3873), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3871), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19483] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3875), 1,
      anon_sym_LPAREN,
    STATE(2432), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4534), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5636), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [19583] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3877), 1,
      anon_sym_EQ,
    STATE(2433), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3879), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19657] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3881), 1,
      anon_sym_LPAREN,
    STATE(2434), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4559), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5601), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [19757] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3883), 1,
      anon_sym_EQ,
    STATE(2435), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3885), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19831] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2436), 1,
      sym_heredoc_body,
    ACTIONS(3889), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3887), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19899] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2437), 1,
      sym_heredoc_body,
    ACTIONS(3501), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3499), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [19967] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2438), 1,
      sym_heredoc_body,
    ACTIONS(3893), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3891), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20035] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3895), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2439), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4544), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5594), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [20135] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2440), 1,
      sym_heredoc_body,
    ACTIONS(3088), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3086), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20203] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3897), 1,
      anon_sym_LPAREN,
    STATE(2441), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4552), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5622), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [20303] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3899), 1,
      anon_sym_EQ,
    STATE(2442), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3901), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 23,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20377] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2443), 1,
      sym_heredoc_body,
    ACTIONS(3054), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3052), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20445] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2444), 1,
      sym_heredoc_body,
    ACTIONS(3058), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3056), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20513] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2445), 1,
      sym_heredoc_body,
    ACTIONS(3066), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3064), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20581] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2446), 1,
      sym_heredoc_body,
    ACTIONS(3002), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3000), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20649] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2447), 1,
      sym_heredoc_body,
    ACTIONS(3465), 3,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_do,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20719] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2448), 1,
      sym_heredoc_body,
    ACTIONS(3905), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3903), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20787] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2449), 1,
      sym_heredoc_body,
    ACTIONS(3905), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3903), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20855] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2450), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20923] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2451), 1,
      sym_heredoc_body,
    ACTIONS(3913), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3911), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [20991] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2452), 1,
      sym_heredoc_body,
    ACTIONS(3913), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3911), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [21059] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2453), 1,
      sym_heredoc_body,
    ACTIONS(3917), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3915), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [21127] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3919), 1,
      anon_sym_LPAREN,
    STATE(2454), 1,
      sym_heredoc_body,
    STATE(2983), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4549), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5590), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21227] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2455), 1,
      sym_heredoc_body,
    ACTIONS(3923), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3921), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [21295] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3925), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2456), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4555), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5589), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21395] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3927), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2457), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4537), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5595), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21495] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3929), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2458), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4542), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5599), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21595] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3931), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2459), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4545), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5602), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21695] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3933), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2460), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4547), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5639), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21795] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3935), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2461), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4558), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5609), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21895] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3937), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2462), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4543), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5612), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [21995] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3939), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2463), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4536), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5615), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [22095] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3941), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2464), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4550), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5617), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [22195] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    STATE(2465), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 45,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22265] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3943), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2466), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4557), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5621), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [22365] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2467), 1,
      sym_heredoc_body,
    ACTIONS(3947), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3945), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22433] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2468), 1,
      sym_heredoc_body,
    ACTIONS(3951), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3949), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22501] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2469), 1,
      sym_heredoc_body,
    ACTIONS(3955), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3953), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22569] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3425), 1,
      sym_identifier,
    ACTIONS(3435), 1,
      sym_constant,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(3957), 1,
      anon_sym_LPAREN,
    STATE(2287), 1,
      sym__method_rest,
    STATE(2470), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4551), 1,
      sym__method_name,
    STATE(4823), 1,
      sym__nonlocal_variable,
    STATE(5629), 1,
      sym__variable,
    ACTIONS(3433), 2,
      sym_super,
      sym_self,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 4,
      sym__function_identifier,
      sym_operator,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [22669] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2471), 1,
      sym_heredoc_body,
    ACTIONS(3961), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3959), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22737] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2472), 1,
      sym_heredoc_body,
    ACTIONS(3965), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3963), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22805] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2473), 1,
      sym_heredoc_body,
    ACTIONS(3969), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3967), 46,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22873] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3971), 1,
      anon_sym_EQ,
    STATE(2474), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3973), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 22,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [22946] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3975), 1,
      anon_sym_EQ,
    STATE(2475), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3977), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 22,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23019] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(3979), 1,
      anon_sym_EQ,
    STATE(2476), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3981), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 21,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23094] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(3983), 1,
      anon_sym_rescue,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2477), 1,
      sym_heredoc_body,
    STATE(2482), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [23197] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2992), 1,
      anon_sym_COLON,
    STATE(2478), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23266] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4017), 1,
      anon_sym_EQ,
    STATE(2479), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4019), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 21,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23341] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2480), 1,
      sym_heredoc_body,
    STATE(2482), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23410] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2481), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 36,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23477] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4025), 1,
      anon_sym_COMMA,
    STATE(2482), 1,
      sym_heredoc_body,
    STATE(2483), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23548] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4027), 1,
      anon_sym_COMMA,
    STATE(2483), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23617] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2974), 1,
      anon_sym_COLON_COLON,
    STATE(2484), 1,
      sym_heredoc_body,
    ACTIONS(2976), 2,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
    ACTIONS(2958), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(2960), 33,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23688] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2992), 1,
      anon_sym_COLON,
    STATE(2485), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 44,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23757] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4030), 1,
      anon_sym_EQ,
    STATE(2486), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4032), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 21,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23832] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4034), 1,
      anon_sym_EQ,
    STATE(2487), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4036), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 22,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23905] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4038), 1,
      anon_sym_EQ,
    STATE(2488), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4040), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 22,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [23978] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2738), 1,
      anon_sym_COLON,
    STATE(2489), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [24046] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3080), 1,
      anon_sym_EQ_GT,
    STATE(2490), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [24114] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2491), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [24212] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2492), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [24310] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    STATE(2493), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [24406] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    STATE(2494), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [24504] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    STATE(2495), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [24602] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(3983), 1,
      anon_sym_rescue,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2496), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [24702] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    STATE(2497), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [24798] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    STATE(2498), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [24896] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4062), 1,
      anon_sym_EQ,
    STATE(2499), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4064), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 21,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [24968] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(3983), 1,
      anon_sym_rescue,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2500), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [25068] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4066), 1,
      anon_sym_DOT,
    ACTIONS(4068), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(4070), 1,
      anon_sym_AMP_DOT,
    ACTIONS(4072), 1,
      sym__element_reference_bracket,
    STATE(2501), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25142] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4074), 1,
      anon_sym_EQ,
    STATE(2502), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4076), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 20,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25216] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4078), 1,
      anon_sym_EQ,
    STATE(2503), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4080), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 20,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25290] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4082), 1,
      anon_sym_EQ,
    STATE(2504), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4084), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 20,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25364] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4086), 1,
      anon_sym_COLON,
    STATE(2505), 1,
      sym_heredoc_body,
    ACTIONS(731), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(743), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25432] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3865), 1,
      sym__element_reference_bracket,
    ACTIONS(4088), 1,
      anon_sym_DOT,
    ACTIONS(4090), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(4092), 1,
      anon_sym_AMP_DOT,
    STATE(2506), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25506] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4094), 1,
      sym__binary_star_star,
    STATE(2507), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25574] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2508), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25642] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2509), 1,
      sym_heredoc_body,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 40,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25712] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2510), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25780] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2511), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25848] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2512), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 36,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25924] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2513), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 36,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [25998] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2514), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 35,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [26078] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2515), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [26168] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2516), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [26256] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2517), 1,
      sym_heredoc_body,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 38,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [26328] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2518), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [26416] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2519), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [26484] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4100), 1,
      sym__binary_star_star,
    STATE(2520), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [26552] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    STATE(2521), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [26650] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    STATE(2522), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [26748] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4106), 1,
      anon_sym_EQ,
    STATE(2523), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4108), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 21,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [26820] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4110), 1,
      anon_sym_EQ,
    STATE(2524), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4112), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 19,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [26893] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4114), 1,
      anon_sym_EQ,
    STATE(2525), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4116), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 19,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [26966] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2526), 1,
      sym_heredoc_body,
    STATE(2535), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 42,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27033] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4118), 1,
      anon_sym_rescue,
    ACTIONS(4120), 1,
      anon_sym_COMMA,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2527), 1,
      sym_heredoc_body,
    STATE(2535), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 21,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [27136] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4120), 1,
      anon_sym_COMMA,
    STATE(2528), 1,
      sym_heredoc_body,
    STATE(2535), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27205] = 26,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    ACTIONS(3188), 1,
      anon_sym_nil,
    ACTIONS(3192), 1,
      anon_sym_DASH_GT,
    STATE(2529), 1,
      sym_heredoc_body,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4457), 1,
      sym__pattern_primitive,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2922), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2924), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    ACTIONS(3190), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
    ACTIONS(3186), 12,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [27312] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4154), 1,
      anon_sym_EQ,
    STATE(2530), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4156), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 19,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27385] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4158), 1,
      anon_sym_COMMA,
    STATE(2531), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27452] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4161), 1,
      anon_sym_EQ,
    STATE(2532), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4163), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 19,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27525] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    ACTIONS(4165), 1,
      anon_sym_rescue,
    STATE(2533), 1,
      sym_heredoc_body,
    STATE(2535), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [27626] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4167), 1,
      anon_sym_EQ,
    STATE(2534), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4169), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 19,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27699] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4120), 1,
      anon_sym_COMMA,
    STATE(2531), 1,
      aux_sym_exceptions_repeat1,
    STATE(2535), 1,
      sym_heredoc_body,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27768] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4171), 1,
      anon_sym_EQ,
    STATE(2536), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4173), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 20,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27839] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4175), 1,
      anon_sym_EQ,
    STATE(2537), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4177), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 16,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 20,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [27910] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2538), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4557), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [27998] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2539), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4559), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [28086] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(2540), 1,
      sym_heredoc_body,
    ACTIONS(127), 17,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(121), 33,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [28150] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2541), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [28244] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2542), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4544), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [28332] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2543), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4544), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [28420] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(4183), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2544), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [28492] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2545), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4549), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [28580] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2546), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [28676] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2547), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [28740] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2548), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4541), 1,
      sym__method_name,
    STATE(5383), 1,
      sym__method_rest,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [28828] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2549), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [28924] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2550), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4552), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [29012] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2551), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4552), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [29100] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4072), 1,
      sym__element_reference_bracket,
    ACTIONS(4185), 1,
      anon_sym_DOT,
    ACTIONS(4187), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(4189), 1,
      anon_sym_AMP_DOT,
    STATE(2552), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 39,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [29172] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2553), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4555), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [29260] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2554), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4555), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [29348] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    ACTIONS(4165), 1,
      anon_sym_rescue,
    STATE(2555), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [29446] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2556), 1,
      sym_heredoc_body,
    ACTIONS(3579), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3577), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [29510] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2557), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4541), 1,
      sym__method_name,
    STATE(5389), 1,
      sym__method_rest,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [29598] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2558), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [29694] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2559), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4537), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [29782] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2560), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4537), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [29870] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2561), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [29966] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2562), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4542), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [30054] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2563), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4542), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [30142] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4191), 1,
      anon_sym_EQ,
    STATE(2564), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4193), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3090), 17,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
  [30214] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2565), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [30308] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4195), 1,
      anon_sym_EQ,
    STATE(2566), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4197), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 18,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [30378] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2567), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [30474] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4199), 1,
      sym__binary_star_star,
    STATE(2568), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [30540] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2569), 1,
      sym_heredoc_body,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [30604] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2570), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4545), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [30692] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2571), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4545), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [30780] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4201), 1,
      anon_sym_EQ,
    STATE(2572), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4203), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3090), 17,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
  [30852] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2573), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [30918] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2574), 1,
      sym_heredoc_body,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 38,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [30986] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2575), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [31052] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2576), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [31118] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2577), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [31192] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2578), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [31264] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2579), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [31342] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2580), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [31430] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2581), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [31516] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2582), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4559), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [31604] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2583), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [31690] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2584), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [31756] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2585), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4548), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [31844] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2586), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4558), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [31932] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2587), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4558), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32020] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2588), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [32116] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2589), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4548), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32204] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2590), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4540), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32292] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2591), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4539), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32380] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    ACTIONS(4165), 1,
      anon_sym_rescue,
    STATE(2592), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [32478] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2593), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4539), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32566] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2594), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [32630] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2595), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4556), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32718] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2596), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4556), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32806] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2597), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4540), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [32894] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym_heredoc_body,
    ACTIONS(3599), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3597), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [32958] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2599), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4546), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33046] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2600), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [33142] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2601), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4543), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33230] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2602), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4543), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33318] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2603), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4536), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33406] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2604), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4536), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33494] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2605), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4549), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33582] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4205), 1,
      anon_sym_EQ,
    STATE(2606), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4207), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 18,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [33652] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4209), 1,
      sym__binary_star_star,
    STATE(2607), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [33718] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2608), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4546), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33806] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2609), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4551), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33894] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2610), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4557), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [33982] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2611), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4554), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34070] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2612), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4554), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34158] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2613), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [34222] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2614), 1,
      sym_heredoc_body,
    ACTIONS(3653), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3651), 43,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [34286] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2615), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4553), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34374] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2616), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4553), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34462] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2617), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4547), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34550] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2618), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4547), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34638] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2619), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4550), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34726] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2388), 1,
      sym__method_rest,
    STATE(2620), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4550), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34814] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2621), 1,
      sym_heredoc_body,
    STATE(2945), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4534), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34902] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2622), 1,
      sym_heredoc_body,
    STATE(2986), 1,
      sym__method_rest,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4534), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [34990] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4179), 1,
      sym_identifier,
    ACTIONS(4181), 1,
      sym_constant,
    STATE(2425), 1,
      sym__method_rest,
    STATE(2623), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4551), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [35078] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    STATE(2624), 1,
      sym_heredoc_body,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 36,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [35148] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    STATE(2625), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [35213] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    STATE(2626), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [35306] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    STATE(2627), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [35401] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4181), 1,
      sym_constant,
    ACTIONS(4243), 1,
      sym_identifier,
    STATE(2628), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4567), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [35486] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    STATE(2629), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [35579] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    ACTIONS(4245), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2630), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 36,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [35652] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2942), 1,
      sym__symbol_start,
    ACTIONS(4247), 1,
      sym_identifier,
    ACTIONS(4253), 1,
      sym_constant,
    ACTIONS(4261), 1,
      sym_simple_symbol,
    STATE(2631), 1,
      sym_heredoc_body,
    STATE(4582), 1,
      sym__method_name,
    STATE(4853), 1,
      sym_constant_suffix,
    STATE(4856), 1,
      sym_identifier_suffix,
    ACTIONS(4255), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4257), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(4259), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4854), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(4251), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4249), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [35737] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4181), 1,
      sym_constant,
    ACTIONS(4243), 1,
      sym_identifier,
    STATE(2632), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4582), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [35822] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2942), 1,
      sym__symbol_start,
    ACTIONS(4247), 1,
      sym_identifier,
    ACTIONS(4253), 1,
      sym_constant,
    ACTIONS(4261), 1,
      sym_simple_symbol,
    STATE(2633), 1,
      sym_heredoc_body,
    STATE(4567), 1,
      sym__method_name,
    STATE(4853), 1,
      sym_constant_suffix,
    STATE(4856), 1,
      sym_identifier_suffix,
    ACTIONS(4255), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4257), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(4259), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4854), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(4251), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4249), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [35907] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(3443), 1,
      sym_simple_symbol,
    ACTIONS(4181), 1,
      sym_constant,
    ACTIONS(4243), 1,
      sym_identifier,
    STATE(2634), 1,
      sym_heredoc_body,
    STATE(4440), 1,
      sym_constant_suffix,
    STATE(4441), 1,
      sym_identifier_suffix,
    STATE(4581), 1,
      sym__method_name,
    ACTIONS(3437), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(3439), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4443), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(3431), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(3429), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [35992] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2942), 1,
      sym__symbol_start,
    ACTIONS(4247), 1,
      sym_identifier,
    ACTIONS(4253), 1,
      sym_constant,
    ACTIONS(4261), 1,
      sym_simple_symbol,
    STATE(2635), 1,
      sym_heredoc_body,
    STATE(4761), 1,
      sym__method_name,
    STATE(4853), 1,
      sym_constant_suffix,
    STATE(4856), 1,
      sym_identifier_suffix,
    ACTIONS(4255), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4257), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(4259), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(4854), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(4251), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4249), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [36077] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4263), 1,
      sym__binary_star_star,
    STATE(2636), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [36142] = 26,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2884), 1,
      sym_simple_symbol,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(2888), 1,
      sym__symbol_start,
    ACTIONS(2890), 1,
      sym__subshell_start,
    ACTIONS(2892), 1,
      sym__regex_start,
    ACTIONS(2894), 1,
      sym__string_array_start,
    ACTIONS(2896), 1,
      sym__symbol_array_start,
    ACTIONS(2898), 1,
      sym_heredoc_beginning,
    ACTIONS(4265), 1,
      anon_sym_nil,
    ACTIONS(4269), 1,
      anon_sym_DASH_GT,
    STATE(2637), 1,
      sym_heredoc_body,
    STATE(4705), 1,
      sym_nil,
    STATE(4735), 1,
      sym_lambda,
    STATE(4767), 1,
      sym__pattern_primitive,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2908), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2910), 2,
      sym_integer,
      sym_float,
    STATE(4692), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4699), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4714), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
    ACTIONS(4267), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4702), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
    ACTIONS(3186), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [36247] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4118), 1,
      anon_sym_rescue,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    STATE(2638), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 21,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [36344] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4271), 1,
      anon_sym_EQ,
    STATE(2639), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4273), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3090), 16,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
  [36415] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4275), 1,
      sym_identifier,
    ACTIONS(4281), 1,
      sym_constant,
    ACTIONS(4289), 1,
      sym_simple_symbol,
    ACTIONS(4291), 1,
      sym__symbol_start,
    STATE(2631), 1,
      sym__method_name,
    STATE(2640), 1,
      sym_heredoc_body,
    STATE(3185), 1,
      sym_constant_suffix,
    STATE(3186), 1,
      sym_identifier_suffix,
    ACTIONS(4283), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4285), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(4287), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(3202), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(4279), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4277), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [36500] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    STATE(2641), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [36565] = 26,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2884), 1,
      sym_simple_symbol,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(2888), 1,
      sym__symbol_start,
    ACTIONS(2890), 1,
      sym__subshell_start,
    ACTIONS(2892), 1,
      sym__regex_start,
    ACTIONS(2894), 1,
      sym__string_array_start,
    ACTIONS(2896), 1,
      sym__symbol_array_start,
    ACTIONS(2898), 1,
      sym_heredoc_beginning,
    ACTIONS(4265), 1,
      anon_sym_nil,
    ACTIONS(4269), 1,
      anon_sym_DASH_GT,
    STATE(2642), 1,
      sym_heredoc_body,
    STATE(4705), 1,
      sym_nil,
    STATE(4735), 1,
      sym_lambda,
    STATE(4767), 1,
      sym__pattern_primitive,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2872), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2874), 2,
      sym_integer,
      sym_float,
    STATE(4692), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4699), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4714), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
    ACTIONS(4267), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4702), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
    ACTIONS(3186), 10,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [36670] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    STATE(2643), 1,
      sym_heredoc_body,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 37,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [36737] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    STATE(2644), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [36832] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    STATE(2645), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [36927] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    STATE(2646), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [36992] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(4293), 1,
      anon_sym_EQ,
    STATE(2647), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4295), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3090), 16,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
  [37063] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4297), 1,
      anon_sym_EQ,
    STATE(2648), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4299), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3090), 17,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
  [37132] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4301), 1,
      sym__binary_star_star,
    STATE(2649), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [37197] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    STATE(2650), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [37270] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    STATE(2651), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [37341] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    STATE(2652), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 32,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [37418] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    STATE(2653), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [37505] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4303), 1,
      anon_sym_EQ,
    STATE(2654), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(4305), 13,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_STAR_EQ,
      anon_sym_STAR_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PIPE_PIPE_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_AMP_AMP_EQ,
      anon_sym_AMP_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_CARET_EQ,
    ACTIONS(3090), 17,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3092), 17,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_EQ_EQ,
  [37574] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    STATE(2655), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [37659] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    STATE(2656), 1,
      sym_heredoc_body,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 35,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [37728] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    STATE(2657), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [37813] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4307), 1,
      anon_sym_rescue,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2658), 1,
      sym_heredoc_body,
    STATE(2662), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 19,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [37914] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4345), 1,
      anon_sym_COMMA,
    STATE(2659), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [37979] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2660), 1,
      sym_heredoc_body,
    STATE(2662), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 41,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [38044] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4275), 1,
      sym_identifier,
    ACTIONS(4281), 1,
      sym_constant,
    ACTIONS(4289), 1,
      sym_simple_symbol,
    ACTIONS(4291), 1,
      sym__symbol_start,
    STATE(2632), 1,
      sym__method_name,
    STATE(2661), 1,
      sym_heredoc_body,
    STATE(3185), 1,
      sym_constant_suffix,
    STATE(3186), 1,
      sym_identifier_suffix,
    ACTIONS(4283), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4285), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(4287), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
    STATE(3202), 5,
      sym__function_identifier,
      sym_operator,
      sym__nonlocal_variable,
      sym_setter,
      sym_delimited_symbol,
    ACTIONS(4279), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4277), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [38129] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4348), 1,
      anon_sym_COMMA,
    STATE(2659), 1,
      aux_sym_exceptions_repeat1,
    STATE(2662), 1,
      sym_heredoc_body,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [38196] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4307), 1,
      anon_sym_rescue,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2662), 1,
      aux_sym_exceptions_repeat1,
    STATE(2663), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [38293] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    STATE(2664), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [38388] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    STATE(2665), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [38483] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4118), 1,
      anon_sym_rescue,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    STATE(2666), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 21,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [38580] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    STATE(2667), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [38645] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2668), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [38737] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2669), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [38801] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2670), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 25,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [38887] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4307), 1,
      anon_sym_rescue,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2671), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [38981] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2672), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 26,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [39065] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2673), 1,
      sym_heredoc_body,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 35,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [39133] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2674), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [39225] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2675), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 26,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [39309] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2676), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [39373] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2677), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [39437] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2678), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 40,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [39501] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2679), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [39573] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2680), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [39643] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2681), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 23,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [39733] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2682), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [39829] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2683), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [39925] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2684), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 32,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [40001] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4307), 1,
      anon_sym_rescue,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2685), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 19,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [40099] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2686), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [40191] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2687), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [40283] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2688), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 23,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [40373] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2689), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [40469] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2690), 1,
      sym_heredoc_body,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 37,
      sym__line_break,
      sym__binary_minus,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [40535] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2691), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [40627] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(4352), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2692), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 36,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [40697] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    STATE(2693), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 39,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [40763] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2694), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [40855] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2695), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [40923] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    ACTIONS(4354), 1,
      anon_sym_EQ_GT,
    STATE(2696), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [41019] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2697), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [41111] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4356), 1,
      anon_sym_rescue,
    ACTIONS(4358), 1,
      anon_sym_COMMA,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2698), 1,
      sym_heredoc_body,
    STATE(2700), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [41208] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4392), 1,
      anon_sym_COMMA,
    STATE(2699), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [41271] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4358), 1,
      anon_sym_COMMA,
    STATE(2700), 1,
      sym_heredoc_body,
    STATE(2716), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [41336] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2701), 1,
      sym_heredoc_body,
    STATE(2708), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [41399] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2702), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [41490] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(3196), 1,
      anon_sym_do,
    STATE(2703), 1,
      sym_heredoc_body,
    STATE(2818), 1,
      sym_argument_list,
    STATE(2910), 1,
      sym_do_block,
    STATE(2911), 1,
      sym_block,
    ACTIONS(411), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(409), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [41563] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2704), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [41654] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(4399), 1,
      anon_sym_rescue,
    STATE(2700), 1,
      aux_sym_exceptions_repeat1,
    STATE(2705), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [41753] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2706), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [41844] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(4403), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2707), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [41913] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4405), 1,
      anon_sym_COMMA,
    STATE(2699), 1,
      aux_sym_exceptions_repeat1,
    STATE(2708), 1,
      sym_heredoc_body,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [41978] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4407), 1,
      anon_sym_rescue,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2708), 1,
      aux_sym_exceptions_repeat1,
    STATE(2709), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 18,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [42075] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2710), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [42166] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(4399), 1,
      anon_sym_rescue,
    STATE(2700), 1,
      aux_sym_exceptions_repeat1,
    STATE(2711), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [42261] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    STATE(2712), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [42352] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2700), 1,
      aux_sym_exceptions_repeat1,
    STATE(2713), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 39,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [42415] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4358), 1,
      anon_sym_COMMA,
    STATE(2700), 1,
      aux_sym_exceptions_repeat1,
    STATE(2714), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [42480] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4356), 1,
      anon_sym_rescue,
    ACTIONS(4358), 1,
      anon_sym_COMMA,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2700), 1,
      aux_sym_exceptions_repeat1,
    STATE(2715), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [42581] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4445), 1,
      anon_sym_COMMA,
    STATE(2716), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [42644] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2717), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [42736] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4407), 1,
      anon_sym_rescue,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2718), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 18,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [42830] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3184), 1,
      anon_sym_COLON,
    STATE(2719), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [42892] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(3331), 1,
      anon_sym_COLON,
    STATE(2720), 1,
      sym_heredoc_body,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3286), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(3090), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [42962] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3865), 1,
      sym__element_reference_bracket,
    ACTIONS(4448), 1,
      anon_sym_DOT,
    ACTIONS(4450), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(4452), 1,
      anon_sym_AMP_DOT,
    STATE(2721), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43030] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4454), 1,
      sym__binary_star_star,
    STATE(2722), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43092] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2723), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [43186] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2724), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43248] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2725), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 21,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [43336] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2726), 1,
      sym_heredoc_body,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43400] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2727), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43462] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2728), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43524] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2729), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43594] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2730), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43662] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2731), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43736] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2732), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 22,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [43820] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2733), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [43902] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2734), 1,
      sym_heredoc_body,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 32,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [43968] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2735), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [44050] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2736), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [44112] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2737), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [44206] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2738), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 36,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [44272] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2739), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [44366] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2740), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [44428] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2741), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [44518] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3184), 1,
      anon_sym_COLON,
    STATE(2742), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [44580] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2743), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [44670] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2744), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [44760] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(4456), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2745), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [44828] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2746), 1,
      sym_heredoc_body,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 35,
      sym__line_break,
      sym__binary_minus,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [44892] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2747), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [44954] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2748), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [45016] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2749), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [45086] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2750), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [45154] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2751), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [45228] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2752), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 23,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [45312] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2753), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 24,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [45394] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2754), 1,
      sym_heredoc_body,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [45460] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2755), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 24,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [45542] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2756), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 38,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [45604] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2757), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [45694] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(4399), 1,
      anon_sym_rescue,
    STATE(2758), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [45786] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2759), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [45876] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2760), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [45966] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4458), 1,
      sym__binary_star_star,
    STATE(2761), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [46028] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2762), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [46118] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2763), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [46210] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3280), 1,
      anon_sym_ri,
    STATE(2764), 1,
      sym_heredoc_body,
    ACTIONS(3278), 2,
      anon_sym_i,
      anon_sym_r,
    ACTIONS(3276), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3274), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [46274] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2765), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [46366] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2766), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [46458] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2767), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 20,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [46548] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2768), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [46640] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2769), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [46732] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4407), 1,
      anon_sym_rescue,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2770), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 18,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [46826] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(4399), 1,
      anon_sym_rescue,
    STATE(2771), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [46922] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2772), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 20,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [47012] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(4460), 1,
      anon_sym_EQ_GT,
    STATE(2773), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [47106] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4409), 1,
      anon_sym_PIPE,
    ACTIONS(4411), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4413), 1,
      anon_sym_AMP,
    ACTIONS(4417), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4419), 1,
      anon_sym_CARET,
    ACTIONS(4421), 1,
      anon_sym_QMARK,
    ACTIONS(4423), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4425), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4429), 1,
      anon_sym_GT_EQ,
    ACTIONS(4435), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4439), 1,
      sym__binary_star_star,
    STATE(2774), 1,
      sym_heredoc_body,
    ACTIONS(4427), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4431), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4415), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4433), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4437), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [47198] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2775), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 21,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [47286] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2776), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 23,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [47367] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2777), 1,
      sym_heredoc_body,
    STATE(3012), 1,
      sym_argument_list,
    ACTIONS(427), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(425), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [47430] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2778), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [47503] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4356), 1,
      anon_sym_rescue,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2779), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [47598] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4494), 1,
      anon_sym_COMMA,
    STATE(2780), 1,
      sym_heredoc_body,
    STATE(2795), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [47661] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4496), 1,
      anon_sym_rescue,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2780), 1,
      aux_sym_exceptions_repeat1,
    STATE(2781), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [47756] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2782), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [47817] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2783), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [47906] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4530), 1,
      sym__string_start,
    STATE(2833), 1,
      sym_string,
    STATE(2784), 2,
      sym_heredoc_body,
      aux_sym_chained_string_repeat1,
    ACTIONS(3290), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3288), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [47969] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2785), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 22,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [48052] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2786), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [48117] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2787), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [48204] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2788), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 23,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [48285] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2789), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [48374] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2790), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [48463] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2791), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [48524] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    STATE(2792), 1,
      sym_heredoc_body,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3286), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(3090), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [48591] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2793), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [48680] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2794), 1,
      sym_heredoc_body,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 32,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [48745] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4533), 1,
      anon_sym_COMMA,
    STATE(2795), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [48806] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2796), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [48895] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    STATE(2784), 1,
      aux_sym_chained_string_repeat1,
    STATE(2797), 1,
      sym_heredoc_body,
    STATE(2833), 1,
      sym_string,
    ACTIONS(3272), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3270), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [48960] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4494), 1,
      anon_sym_COMMA,
    STATE(2780), 1,
      aux_sym_exceptions_repeat1,
    STATE(2798), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49023] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(3196), 1,
      anon_sym_do,
    STATE(2799), 1,
      sym_heredoc_body,
    STATE(2919), 1,
      sym_do_block,
    STATE(2920), 1,
      sym_block,
    ACTIONS(3325), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3323), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49090] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2800), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49157] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2801), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 20,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [49244] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2802), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [49333] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4494), 1,
      anon_sym_COMMA,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    ACTIONS(4536), 1,
      anon_sym_rescue,
    STATE(2780), 1,
      aux_sym_exceptions_repeat1,
    STATE(2803), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 15,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [49430] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2804), 1,
      sym_heredoc_body,
    STATE(3010), 1,
      sym_argument_list,
    ACTIONS(419), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(417), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49493] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2805), 1,
      sym_heredoc_body,
    STATE(3013), 1,
      sym_argument_list,
    ACTIONS(3297), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3295), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49556] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2806), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [49645] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2807), 1,
      sym_heredoc_body,
    STATE(2833), 1,
      sym_string,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49710] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2808), 1,
      sym_heredoc_body,
    STATE(3011), 1,
      sym_argument_list,
    ACTIONS(415), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(413), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49773] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2809), 1,
      sym_heredoc_body,
    STATE(3014), 1,
      sym_argument_list,
    ACTIONS(3329), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3327), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49836] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4356), 1,
      anon_sym_rescue,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2810), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [49927] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2811), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [49992] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2812), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [50085] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3280), 1,
      anon_sym_ri,
    STATE(2813), 1,
      sym_heredoc_body,
    ACTIONS(3278), 2,
      anon_sym_i,
      anon_sym_r,
    ACTIONS(3276), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3274), 33,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [50148] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2814), 1,
      sym_heredoc_body,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [50211] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2815), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [50280] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2816), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [50369] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2817), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [50430] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(3196), 1,
      anon_sym_do,
    STATE(2818), 1,
      sym_heredoc_body,
    STATE(3065), 1,
      sym_do_block,
    STATE(3067), 1,
      sym_block,
    ACTIONS(3317), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3315), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [50497] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2819), 1,
      sym_heredoc_body,
    STATE(3007), 1,
      sym_argument_list,
    ACTIONS(423), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(421), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [50560] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2820), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [50649] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    STATE(2821), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 19,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [50738] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2822), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [50831] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(3196), 1,
      anon_sym_do,
    STATE(2823), 1,
      sym_heredoc_body,
    STATE(3049), 1,
      sym_do_block,
    STATE(3050), 1,
      sym_block,
    ACTIONS(3311), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3309), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [50898] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2824), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [50991] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2780), 1,
      aux_sym_exceptions_repeat1,
    STATE(2825), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 36,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [51052] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    STATE(2826), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 37,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [51113] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2827), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [51203] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    STATE(2828), 1,
      sym_heredoc_body,
    STATE(2910), 1,
      sym_do_block,
    STATE(2911), 1,
      sym_block,
    STATE(3108), 1,
      sym_argument_list,
    ACTIONS(411), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(409), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [51271] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4496), 1,
      anon_sym_rescue,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2829), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [51363] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2830), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 18,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [51451] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2831), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [51541] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2832), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [51631] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2833), 1,
      sym_heredoc_body,
    ACTIONS(3359), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3357), 35,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [51689] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4496), 1,
      anon_sym_rescue,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2834), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [51781] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2835), 1,
      sym_heredoc_body,
    ACTIONS(3321), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3319), 35,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [51839] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4538), 1,
      anon_sym_rescue,
    ACTIONS(4540), 1,
      anon_sym_COMMA,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(2836), 1,
      sym_heredoc_body,
    STATE(2840), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [51935] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(4574), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2837), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52001] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4576), 1,
      sym__binary_star_star,
    STATE(2838), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52061] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2839), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52121] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4540), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      sym_heredoc_body,
    STATE(2853), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52183] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2841), 1,
      sym_heredoc_body,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 32,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52245] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2842), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52305] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2843), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52365] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2844), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52433] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2845), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52499] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2846), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52571] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2847), 1,
      sym_heredoc_body,
    ACTIONS(2626), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 35,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52629] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2848), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 21,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [52709] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2849), 1,
      sym_heredoc_body,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52773] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2850), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 21,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [52853] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4540), 1,
      anon_sym_COMMA,
    STATE(2840), 1,
      aux_sym_exceptions_repeat1,
    STATE(2851), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52915] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2852), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [52975] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4578), 1,
      anon_sym_COMMA,
    STATE(2853), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53035] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2854), 1,
      sym_heredoc_body,
    ACTIONS(2638), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 35,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53093] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(4581), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(2855), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53159] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    STATE(2856), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [53247] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2857), 1,
      sym_heredoc_body,
    ACTIONS(2642), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2644), 35,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53305] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2858), 1,
      sym_heredoc_body,
    ACTIONS(2614), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2616), 35,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53363] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4583), 1,
      sym__binary_star_star,
    STATE(2859), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53423] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4072), 1,
      sym__element_reference_bracket,
    ACTIONS(4585), 1,
      anon_sym_DOT,
    ACTIONS(4587), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(4589), 1,
      anon_sym_AMP_DOT,
    STATE(2860), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53489] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3465), 1,
      anon_sym_LBRACE,
    STATE(2861), 1,
      sym_heredoc_body,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53549] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    ACTIONS(4591), 1,
      anon_sym_rescue,
    STATE(2840), 1,
      aux_sym_exceptions_repeat1,
    STATE(2862), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [53643] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2863), 1,
      sym_heredoc_body,
    ACTIONS(3465), 2,
      anon_sym_LBRACE,
      anon_sym_COMMA,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53703] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2840), 1,
      aux_sym_exceptions_repeat1,
    STATE(2864), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53763] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2865), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [53853] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2866), 1,
      sym_heredoc_body,
    ACTIONS(3305), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_COLON,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3303), 35,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [53911] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2867), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 18,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [53999] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2868), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [54089] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2869), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [54179] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4500), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4506), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4510), 1,
      anon_sym_QMARK,
    ACTIONS(4512), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2870), 1,
      sym_heredoc_body,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [54269] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4498), 1,
      anon_sym_PIPE,
    ACTIONS(4502), 1,
      anon_sym_AMP,
    ACTIONS(4508), 1,
      anon_sym_CARET,
    ACTIONS(4514), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4518), 1,
      anon_sym_GT_EQ,
    ACTIONS(4524), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4528), 1,
      sym__binary_star_star,
    STATE(2871), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4516), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4520), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4504), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4522), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4526), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [54351] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(2872), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54416] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2873), 1,
      sym_heredoc_body,
    ACTIONS(3923), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3921), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54473] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2874), 1,
      sym_heredoc_body,
    ACTIONS(3649), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54530] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    ACTIONS(4593), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(2875), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54597] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2876), 1,
      sym_heredoc_body,
    ACTIONS(3653), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3651), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54654] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2877), 1,
      sym_heredoc_body,
    ACTIONS(3463), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3461), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54711] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2878), 1,
      sym_heredoc_body,
    ACTIONS(3641), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3639), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54768] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2879), 1,
      sym_heredoc_body,
    ACTIONS(3689), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3687), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54825] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2880), 1,
      sym_heredoc_body,
    ACTIONS(3717), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3715), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54882] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2881), 1,
      sym_heredoc_body,
    ACTIONS(3955), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3953), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54939] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2882), 1,
      sym_heredoc_body,
    ACTIONS(3721), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3719), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [54996] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2883), 1,
      sym_heredoc_body,
    ACTIONS(3725), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3723), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55053] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2884), 1,
      sym_heredoc_body,
    ACTIONS(3795), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3793), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55110] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2885), 1,
      sym_heredoc_body,
    ACTIONS(3799), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3797), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55167] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2886), 1,
      sym_heredoc_body,
    ACTIONS(3825), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3823), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55224] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2887), 1,
      sym_heredoc_body,
    ACTIONS(3829), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3827), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55281] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    STATE(2888), 1,
      sym_heredoc_body,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55342] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2889), 1,
      sym_heredoc_body,
    ACTIONS(3837), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3835), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55399] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2890), 1,
      sym_heredoc_body,
    ACTIONS(3387), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3385), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55456] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2891), 1,
      sym_heredoc_body,
    ACTIONS(3961), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3959), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55513] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    STATE(2892), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55572] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    STATE(2893), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55631] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2894), 1,
      sym_heredoc_body,
    ACTIONS(3851), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3849), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55688] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2895), 1,
      sym_heredoc_body,
    ACTIONS(3088), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3086), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55745] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2896), 1,
      sym_heredoc_body,
    ACTIONS(3054), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3052), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55802] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4599), 1,
      anon_sym_COMMA,
    STATE(2897), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55861] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    STATE(2898), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55928] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    STATE(2899), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [55993] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2900), 1,
      sym_heredoc_body,
    ACTIONS(3058), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3056), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56050] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    STATE(2901), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56121] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    STATE(2902), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [56202] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2903), 1,
      sym_heredoc_body,
    ACTIONS(3066), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3064), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56259] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    ACTIONS(4622), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(2904), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56326] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    STATE(2905), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [56405] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    STATE(2906), 1,
      sym_heredoc_body,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56468] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    STATE(2907), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [56547] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2908), 1,
      sym_heredoc_body,
    ACTIONS(3002), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3000), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56604] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2909), 1,
      sym_heredoc_body,
    ACTIONS(3965), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3963), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56661] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2910), 1,
      sym_heredoc_body,
    ACTIONS(3489), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3487), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56718] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2911), 1,
      sym_heredoc_body,
    ACTIONS(3489), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3487), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56775] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4624), 1,
      anon_sym_rescue,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(2912), 1,
      sym_heredoc_body,
    STATE(2969), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [56870] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2913), 1,
      sym_heredoc_body,
    STATE(2969), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 35,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [56929] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4536), 1,
      anon_sym_rescue,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 1,
      anon_sym_QMARK,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(2914), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 15,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [57020] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2915), 1,
      sym_heredoc_body,
    ACTIONS(3615), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3613), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57077] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2916), 1,
      sym_heredoc_body,
    ACTIONS(3511), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3509), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57134] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2917), 1,
      sym_heredoc_body,
    ACTIONS(3515), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3513), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57191] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2918), 1,
      sym_heredoc_body,
    ACTIONS(3527), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3525), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57248] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2919), 1,
      sym_heredoc_body,
    ACTIONS(3395), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3393), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57305] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2920), 1,
      sym_heredoc_body,
    ACTIONS(3395), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3393), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57362] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2921), 1,
      sym_heredoc_body,
    ACTIONS(3403), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3401), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57419] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2922), 1,
      sym_heredoc_body,
    ACTIONS(3407), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3405), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57476] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2923), 1,
      sym_heredoc_body,
    ACTIONS(3411), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3409), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57533] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2924), 1,
      sym_heredoc_body,
    ACTIONS(3415), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3413), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57590] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2925), 1,
      sym_heredoc_body,
    ACTIONS(3419), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3417), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57647] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2926), 1,
      sym_heredoc_body,
    ACTIONS(3423), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3421), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57704] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2927), 1,
      sym_heredoc_body,
    ACTIONS(3447), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3445), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57761] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4624), 1,
      anon_sym_rescue,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(2928), 1,
      sym_heredoc_body,
    STATE(2969), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 15,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [57852] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2929), 1,
      sym_heredoc_body,
    ACTIONS(3455), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3453), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57909] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2930), 1,
      sym_heredoc_body,
    ACTIONS(3459), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3457), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [57966] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2931), 1,
      sym_heredoc_body,
    ACTIONS(3469), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3467), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58023] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2932), 1,
      sym_heredoc_body,
    ACTIONS(3473), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3471), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58080] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2933), 1,
      sym_heredoc_body,
    ACTIONS(3481), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3479), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58137] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(2934), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [58226] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(2935), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [58315] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2936), 1,
      sym_heredoc_body,
    ACTIONS(3567), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3565), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58372] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2937), 1,
      sym_heredoc_body,
    ACTIONS(3463), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3461), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58429] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2938), 1,
      sym_heredoc_body,
    ACTIONS(3767), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3765), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58486] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2939), 1,
      sym_heredoc_body,
    ACTIONS(3551), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3549), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58543] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2940), 1,
      sym_heredoc_body,
    ACTIONS(3555), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3553), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58600] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2941), 1,
      sym_heredoc_body,
    ACTIONS(3519), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3517), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58657] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    STATE(2942), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58716] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4536), 1,
      anon_sym_rescue,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 1,
      anon_sym_QMARK,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(2943), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 15,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [58807] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2944), 1,
      sym_heredoc_body,
    ACTIONS(3523), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3521), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58864] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2945), 1,
      sym_heredoc_body,
    ACTIONS(3847), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3845), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58921] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(4670), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(2946), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [58986] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2947), 1,
      sym_heredoc_body,
    ACTIONS(3535), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3533), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59043] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2948), 1,
      sym_heredoc_body,
    ACTIONS(3547), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3545), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59100] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2949), 1,
      sym_heredoc_body,
    ACTIONS(3571), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3569), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59157] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2950), 1,
      sym_heredoc_body,
    ACTIONS(3575), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3573), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59214] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2951), 1,
      sym_heredoc_body,
    ACTIONS(3559), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3557), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59271] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2952), 1,
      sym_heredoc_body,
    ACTIONS(3637), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3635), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59328] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2953), 1,
      sym_heredoc_body,
    ACTIONS(3665), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3663), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59385] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2954), 1,
      sym_heredoc_body,
    ACTIONS(3669), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3667), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59442] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2955), 1,
      sym_heredoc_body,
    ACTIONS(3677), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3675), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59499] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2956), 1,
      sym_heredoc_body,
    ACTIONS(3681), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3679), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59556] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2957), 1,
      sym_heredoc_body,
    ACTIONS(3563), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3561), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59613] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4672), 1,
      sym__binary_star_star,
    STATE(2958), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59672] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2959), 1,
      sym_heredoc_body,
    ACTIONS(3539), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3537), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59729] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2960), 1,
      sym_heredoc_body,
    ACTIONS(3611), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3609), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59786] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2961), 1,
      sym_heredoc_body,
    ACTIONS(3821), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3819), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59843] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2962), 1,
      sym_heredoc_body,
    ACTIONS(3451), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3449), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59900] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2963), 1,
      sym_heredoc_body,
    ACTIONS(3841), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3839), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [59957] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2964), 1,
      sym_heredoc_body,
    ACTIONS(3371), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3369), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60014] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2965), 1,
      sym_heredoc_body,
    ACTIONS(3583), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3581), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60071] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 1,
      anon_sym_QMARK,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(2966), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [60160] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2967), 1,
      sym_heredoc_body,
    ACTIONS(3595), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3593), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60217] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4088), 1,
      anon_sym_DOT,
    ACTIONS(4092), 1,
      anon_sym_AMP_DOT,
    ACTIONS(4674), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(4676), 1,
      sym__element_reference_bracket,
    STATE(2968), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60282] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4678), 1,
      anon_sym_COMMA,
    STATE(2897), 1,
      aux_sym_exceptions_repeat1,
    STATE(2969), 1,
      sym_heredoc_body,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60343] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2970), 1,
      sym_heredoc_body,
    ACTIONS(3619), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3617), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60400] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2971), 1,
      sym_heredoc_body,
    ACTIONS(3623), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3621), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60457] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2972), 1,
      sym_heredoc_body,
    ACTIONS(3018), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3016), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60514] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2973), 1,
      sym_heredoc_body,
    ACTIONS(3633), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3631), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60571] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2974), 1,
      sym_heredoc_body,
    ACTIONS(3022), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3020), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60628] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2975), 1,
      sym_heredoc_body,
    ACTIONS(3026), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3024), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60685] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2976), 1,
      sym_heredoc_body,
    ACTIONS(3030), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3028), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60742] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2977), 1,
      sym_heredoc_body,
    ACTIONS(3034), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3032), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60799] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2978), 1,
      sym_heredoc_body,
    ACTIONS(3645), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3643), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60856] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2979), 1,
      sym_heredoc_body,
    ACTIONS(3673), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3671), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60913] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2980), 1,
      sym_heredoc_body,
    ACTIONS(3685), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3683), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [60970] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2981), 1,
      sym_heredoc_body,
    ACTIONS(3693), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3691), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61027] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2982), 1,
      sym_heredoc_body,
    ACTIONS(3697), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3695), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61084] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2983), 1,
      sym_heredoc_body,
    ACTIONS(3379), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3377), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61141] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2984), 1,
      sym_heredoc_body,
    ACTIONS(3889), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3887), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61198] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2985), 1,
      sym_heredoc_body,
    ACTIONS(3737), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3735), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61255] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2986), 1,
      sym_heredoc_body,
    ACTIONS(3741), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3739), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61312] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2987), 1,
      sym_heredoc_body,
    ACTIONS(3751), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3749), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61369] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2988), 1,
      sym_heredoc_body,
    ACTIONS(3755), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3753), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61426] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2989), 1,
      sym_heredoc_body,
    ACTIONS(3759), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3757), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61483] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2990), 1,
      sym_heredoc_body,
    ACTIONS(3771), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3769), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61540] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2991), 1,
      sym_heredoc_body,
    ACTIONS(3701), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3699), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61597] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2992), 1,
      sym_heredoc_body,
    ACTIONS(3775), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3773), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61654] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2993), 1,
      sym_heredoc_body,
    ACTIONS(3779), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3777), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61711] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2994), 1,
      sym_heredoc_body,
    ACTIONS(3783), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3781), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61768] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2995), 1,
      sym_heredoc_body,
    ACTIONS(3787), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3785), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61825] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2996), 1,
      sym_heredoc_body,
    ACTIONS(3038), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3036), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61882] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2997), 1,
      sym_heredoc_body,
    ACTIONS(3042), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3040), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61939] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2998), 1,
      sym_heredoc_body,
    ACTIONS(3046), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3044), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [61996] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2999), 1,
      sym_heredoc_body,
    ACTIONS(3813), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3811), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62053] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3859), 1,
      anon_sym_DOT,
    ACTIONS(3863), 1,
      anon_sym_AMP_DOT,
    ACTIONS(4676), 1,
      sym__element_reference_bracket,
    ACTIONS(4680), 1,
      anon_sym_COLON_COLON2,
    STATE(3000), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62118] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3001), 1,
      sym_heredoc_body,
    ACTIONS(3817), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3815), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62175] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3002), 1,
      sym_heredoc_body,
    ACTIONS(3050), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3048), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62232] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3003), 1,
      sym_heredoc_body,
    ACTIONS(3705), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3703), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62289] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3280), 1,
      anon_sym_ri,
    STATE(3004), 1,
      sym_heredoc_body,
    ACTIONS(3278), 2,
      anon_sym_i,
      anon_sym_r,
    ACTIONS(3276), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3274), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62350] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3005), 1,
      sym_heredoc_body,
    ACTIONS(3709), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3707), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62407] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3006), 1,
      sym_heredoc_body,
    ACTIONS(3729), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3727), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62464] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3007), 1,
      sym_heredoc_body,
    ACTIONS(3399), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3397), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62521] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3008), 1,
      sym_heredoc_body,
    ACTIONS(3010), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62578] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3009), 1,
      sym_heredoc_body,
    ACTIONS(3014), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3012), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62635] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3010), 1,
      sym_heredoc_body,
    ACTIONS(3493), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3491), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62692] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3011), 1,
      sym_heredoc_body,
    ACTIONS(3497), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3495), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62749] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3012), 1,
      sym_heredoc_body,
    ACTIONS(3507), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3505), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62806] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3013), 1,
      sym_heredoc_body,
    ACTIONS(3657), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3655), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62863] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3014), 1,
      sym_heredoc_body,
    ACTIONS(3661), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3659), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62920] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    STATE(3015), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [62979] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4682), 1,
      sym__binary_star_star,
    STATE(3016), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63038] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3017), 1,
      sym_heredoc_body,
    ACTIONS(3763), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3761), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63095] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3018), 1,
      sym_heredoc_body,
    ACTIONS(3969), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3967), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63152] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(3019), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [63239] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3020), 1,
      sym_heredoc_body,
    ACTIONS(3501), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3499), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63296] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3021), 1,
      sym_heredoc_body,
    ACTIONS(3579), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3577), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63353] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3022), 1,
      sym_heredoc_body,
    ACTIONS(3893), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3891), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63410] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3023), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63467] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3024), 1,
      sym_heredoc_body,
    ACTIONS(3917), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3915), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63524] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3025), 1,
      sym_heredoc_body,
    ACTIONS(3745), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63581] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3026), 1,
      sym_heredoc_body,
    ACTIONS(3713), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3711), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63638] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3027), 1,
      sym_heredoc_body,
    ACTIONS(3092), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [63695] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 1,
      anon_sym_QMARK,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(3028), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [63784] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 1,
      anon_sym_QMARK,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(3029), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [63873] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(3030), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 17,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [63960] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3031), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64049] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3032), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64138] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3033), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 17,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [64225] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3034), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64314] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3035), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64403] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    ACTIONS(4591), 1,
      anon_sym_rescue,
    STATE(3036), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64494] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3037), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 17,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [64581] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3038), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64670] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3039), 1,
      sym_heredoc_body,
    ACTIONS(3855), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3853), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [64727] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    ACTIONS(4591), 1,
      anon_sym_rescue,
    STATE(3040), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64818] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 1,
      anon_sym_QMARK,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(3041), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [64907] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3042), 1,
      sym_heredoc_body,
    ACTIONS(3383), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3381), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [64964] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3043), 1,
      sym_heredoc_body,
    ACTIONS(3591), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65021] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4597), 1,
      sym__binary_star_star,
    ACTIONS(4602), 1,
      anon_sym_AMP,
    ACTIONS(4608), 1,
      anon_sym_PIPE,
    ACTIONS(4610), 1,
      anon_sym_CARET,
    ACTIONS(4614), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4616), 1,
      anon_sym_GT_EQ,
    ACTIONS(4618), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4662), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4664), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4666), 1,
      anon_sym_QMARK,
    ACTIONS(4668), 1,
      anon_sym_PIPE_PIPE,
    STATE(3044), 1,
      sym_heredoc_body,
    ACTIONS(4604), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4606), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4595), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4612), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4620), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 16,
      sym__line_break,
      sym__element_reference_bracket,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [65110] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3045), 1,
      sym_heredoc_body,
    ACTIONS(3477), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3475), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65167] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3046), 1,
      sym_heredoc_body,
    ACTIONS(3543), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3541), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65224] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3047), 1,
      sym_heredoc_body,
    ACTIONS(3607), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3605), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65281] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3048), 1,
      sym_heredoc_body,
    ACTIONS(3483), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3465), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65338] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3049), 1,
      sym_heredoc_body,
    ACTIONS(3905), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3903), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65395] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3050), 1,
      sym_heredoc_body,
    ACTIONS(3905), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3903), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65452] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4684), 1,
      sym__binary_star_star,
    STATE(3051), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65511] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3052), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65570] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3053), 1,
      sym_heredoc_body,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65631] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3054), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65690] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3055), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65749] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3056), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65816] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3057), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65881] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3058), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [65952] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3059), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [66033] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3060), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [66112] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3061), 1,
      sym_heredoc_body,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66175] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3062), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [66254] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    STATE(3063), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66313] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym_heredoc_body,
    ACTIONS(3599), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3597), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66370] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3065), 1,
      sym_heredoc_body,
    ACTIONS(3913), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3911), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66427] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4686), 1,
      sym__binary_star_star,
    STATE(3066), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66486] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3067), 1,
      sym_heredoc_body,
    ACTIONS(3913), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3911), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66543] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3068), 1,
      sym_heredoc_body,
    ACTIONS(3375), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3373), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66600] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3069), 1,
      sym_heredoc_body,
    ACTIONS(3391), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3389), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66657] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3070), 1,
      sym_heredoc_body,
    ACTIONS(3276), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3274), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66714] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3071), 1,
      sym_heredoc_body,
    ACTIONS(3951), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3949), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66771] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3072), 1,
      sym_heredoc_body,
    ACTIONS(3869), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3867), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66828] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3073), 1,
      sym_heredoc_body,
    ACTIONS(3873), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3871), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66885] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3074), 1,
      sym_heredoc_body,
    ACTIONS(3947), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3945), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66942] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3075), 1,
      sym_heredoc_body,
    ACTIONS(3629), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [66999] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3076), 1,
      sym_heredoc_body,
    ACTIONS(3733), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3731), 34,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [67056] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    STATE(3077), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [67116] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    STATE(2919), 1,
      sym_do_block,
    STATE(2920), 1,
      sym_block,
    STATE(3078), 1,
      sym_heredoc_body,
    ACTIONS(3325), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3323), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [67178] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3079), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [67234] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(1925), 1,
      sym_constant_suffix,
    STATE(1928), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3080), 1,
      sym_heredoc_body,
    ACTIONS(4688), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4694), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4696), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1930), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4692), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4690), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [67306] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(4698), 1,
      sym_identifier,
    ACTIONS(4704), 1,
      sym_constant,
    STATE(1950), 1,
      sym_identifier_suffix,
    STATE(1955), 1,
      sym_constant_suffix,
    STATE(2799), 1,
      sym_argument_list,
    STATE(3081), 1,
      sym_heredoc_body,
    ACTIONS(4706), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4708), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1947), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4702), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4700), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [67380] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4710), 1,
      sym_identifier,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(4718), 1,
      sym_constant,
    STATE(3082), 1,
      sym_heredoc_body,
    STATE(4841), 1,
      sym_constant_suffix,
    STATE(4842), 1,
      sym_identifier_suffix,
    STATE(5029), 1,
      sym_argument_list,
    ACTIONS(4720), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4722), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(4852), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4714), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4712), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [67454] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3083), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [67540] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3084), 1,
      sym_heredoc_body,
    ACTIONS(3483), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3465), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [67596] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    ACTIONS(4724), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(3085), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [67662] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3086), 1,
      sym_heredoc_body,
    ACTIONS(3501), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3499), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [67718] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4726), 1,
      sym_identifier,
    ACTIONS(4732), 1,
      sym_constant,
    STATE(1908), 1,
      sym_constant_suffix,
    STATE(1914), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3087), 1,
      sym_heredoc_body,
    ACTIONS(4734), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4736), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1911), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4730), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4728), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [67792] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4688), 1,
      sym_identifier,
    ACTIONS(4738), 1,
      sym_constant,
    STATE(1925), 1,
      sym_constant_suffix,
    STATE(1928), 1,
      sym_identifier_suffix,
    STATE(2263), 1,
      sym_argument_list,
    STATE(3088), 1,
      sym_heredoc_body,
    ACTIONS(4694), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4696), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1930), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4692), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4690), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [67866] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4740), 1,
      anon_sym_rescue,
    STATE(3089), 1,
      sym_heredoc_body,
    STATE(3196), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [67958] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3090), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [68046] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4774), 1,
      sym__binary_star_star,
    STATE(3091), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [68104] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4776), 1,
      sym__binary_star_star,
    STATE(3092), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [68162] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(4778), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(3093), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [68226] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3094), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [68312] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3095), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [68398] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    STATE(3049), 1,
      sym_do_block,
    STATE(3050), 1,
      sym_block,
    STATE(3096), 1,
      sym_heredoc_body,
    ACTIONS(3311), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3309), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [68460] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(1908), 1,
      sym_constant_suffix,
    STATE(1914), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3097), 1,
      sym_heredoc_body,
    ACTIONS(4726), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4734), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4736), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1911), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4730), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4728), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [68532] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(4780), 1,
      sym_identifier,
    ACTIONS(4786), 1,
      sym_constant,
    STATE(3098), 1,
      sym_heredoc_body,
    STATE(4814), 1,
      sym_constant_suffix,
    STATE(4815), 1,
      sym_identifier_suffix,
    STATE(5029), 1,
      sym_argument_list,
    ACTIONS(2708), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2732), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(4766), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4784), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4782), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [68606] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3099), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [68694] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3100), 1,
      sym_heredoc_body,
    ACTIONS(3483), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3485), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(3465), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [68752] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4788), 1,
      sym__binary_star_star,
    STATE(3101), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [68810] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3102), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [68868] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4538), 1,
      anon_sym_rescue,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3103), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [68958] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2069), 1,
      sym_constant_suffix,
    STATE(2075), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3104), 1,
      sym_heredoc_body,
    ACTIONS(4790), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4796), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4798), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2078), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4794), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4792), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [69030] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3105), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [69118] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3106), 1,
      sym_heredoc_body,
    ACTIONS(3501), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3503), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(3499), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [69176] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4804), 1,
      anon_sym_EQ2,
    STATE(3107), 1,
      sym_heredoc_body,
    ACTIONS(4800), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(4802), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [69234] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    STATE(3065), 1,
      sym_do_block,
    STATE(3067), 1,
      sym_block,
    STATE(3108), 1,
      sym_heredoc_body,
    ACTIONS(3317), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3315), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [69296] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3109), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [69384] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2198), 1,
      sym_identifier_suffix,
    STATE(2210), 1,
      sym_constant_suffix,
    STATE(2799), 1,
      sym_argument_list,
    STATE(3110), 1,
      sym_heredoc_body,
    ACTIONS(4806), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4812), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4814), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2212), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4810), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4808), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [69456] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3111), 1,
      sym_heredoc_body,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [69516] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3112), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [69574] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(4806), 1,
      sym_identifier,
    ACTIONS(4816), 1,
      sym_constant,
    STATE(2198), 1,
      sym_identifier_suffix,
    STATE(2210), 1,
      sym_constant_suffix,
    STATE(2799), 1,
      sym_argument_list,
    STATE(3113), 1,
      sym_heredoc_body,
    ACTIONS(4812), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4814), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2212), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4810), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4808), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [69648] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3114), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [69734] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3115), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [69820] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3116), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [69878] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3117), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [69944] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3118), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70008] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3119), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70078] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3120), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 18,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [70158] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3121), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [70236] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3122), 1,
      sym_heredoc_body,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70298] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4790), 1,
      sym_identifier,
    ACTIONS(4818), 1,
      sym_constant,
    STATE(2069), 1,
      sym_constant_suffix,
    STATE(2075), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3123), 1,
      sym_heredoc_body,
    ACTIONS(4796), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4798), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2078), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4794), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4792), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [70372] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3124), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70430] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3125), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3286), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(3090), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70488] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3126), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70550] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3127), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3625), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_EQ_GT,
    ACTIONS(3090), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70608] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3128), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [70666] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4688), 1,
      sym_identifier,
    ACTIONS(4820), 1,
      sym_constant,
    STATE(1925), 1,
      sym_constant_suffix,
    STATE(1928), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3129), 1,
      sym_heredoc_body,
    ACTIONS(4694), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4696), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1930), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4692), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4690), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [70740] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3130), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [70830] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4790), 1,
      sym_identifier,
    ACTIONS(4822), 1,
      sym_constant,
    STATE(2069), 1,
      sym_constant_suffix,
    STATE(2075), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3131), 1,
      sym_heredoc_body,
    ACTIONS(4796), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4798), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2078), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4794), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4792), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [70904] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(3132), 1,
      sym_heredoc_body,
    STATE(4841), 1,
      sym_constant_suffix,
    STATE(4842), 1,
      sym_identifier_suffix,
    STATE(5029), 1,
      sym_argument_list,
    ACTIONS(4710), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4720), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4722), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(4852), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4714), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4712), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [70976] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(2069), 1,
      sym_constant_suffix,
    STATE(2075), 1,
      sym_identifier_suffix,
    STATE(2263), 1,
      sym_argument_list,
    STATE(3133), 1,
      sym_heredoc_body,
    ACTIONS(4790), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4796), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4798), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2078), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4794), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4792), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71048] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4790), 1,
      sym_identifier,
    ACTIONS(4824), 1,
      sym_constant,
    STATE(2069), 1,
      sym_constant_suffix,
    STATE(2075), 1,
      sym_identifier_suffix,
    STATE(2263), 1,
      sym_argument_list,
    STATE(3134), 1,
      sym_heredoc_body,
    ACTIONS(4796), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4798), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2078), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4794), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4792), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71122] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4726), 1,
      sym_identifier,
    ACTIONS(4826), 1,
      sym_constant,
    STATE(1908), 1,
      sym_constant_suffix,
    STATE(1914), 1,
      sym_identifier_suffix,
    STATE(2263), 1,
      sym_argument_list,
    STATE(3135), 1,
      sym_heredoc_body,
    ACTIONS(4734), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4736), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1911), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4730), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4728), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71196] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(1925), 1,
      sym_constant_suffix,
    STATE(1928), 1,
      sym_identifier_suffix,
    STATE(2263), 1,
      sym_argument_list,
    STATE(3136), 1,
      sym_heredoc_body,
    ACTIONS(4688), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4694), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4696), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1930), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4692), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4690), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71268] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(3137), 1,
      sym_heredoc_body,
    STATE(4814), 1,
      sym_constant_suffix,
    STATE(4815), 1,
      sym_identifier_suffix,
    STATE(5029), 1,
      sym_argument_list,
    ACTIONS(2708), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2732), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(4780), 2,
      sym_constant,
      sym_identifier,
    STATE(4766), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4784), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4782), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71340] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(4780), 1,
      sym_identifier,
    ACTIONS(4828), 1,
      sym_constant,
    STATE(3138), 1,
      sym_heredoc_body,
    STATE(4814), 1,
      sym_constant_suffix,
    STATE(4815), 1,
      sym_identifier_suffix,
    STATE(5029), 1,
      sym_argument_list,
    ACTIONS(2708), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2732), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(4766), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4784), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4782), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71414] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(97), 1,
      sym__string_start,
    ACTIONS(4830), 1,
      anon_sym_COLON,
    STATE(2217), 1,
      aux_sym_chained_string_repeat1,
    STATE(2281), 1,
      sym_string,
    STATE(3139), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [71478] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3140), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [71562] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3141), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [71646] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3142), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [71732] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(1950), 1,
      sym_identifier_suffix,
    STATE(1955), 1,
      sym_constant_suffix,
    STATE(2799), 1,
      sym_argument_list,
    STATE(3143), 1,
      sym_heredoc_body,
    ACTIONS(4698), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4706), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4708), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1947), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4702), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4700), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71804] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4726), 1,
      sym_identifier,
    ACTIONS(4832), 1,
      sym_constant,
    STATE(1908), 1,
      sym_constant_suffix,
    STATE(1914), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3144), 1,
      sym_heredoc_body,
    ACTIONS(4734), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4736), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1911), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4730), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4728), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [71878] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3145), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [71968] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3146), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [72058] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3147), 1,
      sym_heredoc_body,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72118] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3148), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72176] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3149), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72234] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3150), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72300] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3151), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72364] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3152), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72434] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3153), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [72514] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3154), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [72592] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3155), 1,
      sym_heredoc_body,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72654] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3156), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 20,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [72732] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3157), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [72790] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4624), 1,
      anon_sym_rescue,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3158), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [72882] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4624), 1,
      anon_sym_rescue,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3159), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 15,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [72970] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3160), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [73056] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3161), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [73144] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4538), 1,
      anon_sym_rescue,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3162), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [73234] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    STATE(1908), 1,
      sym_constant_suffix,
    STATE(1914), 1,
      sym_identifier_suffix,
    STATE(2263), 1,
      sym_argument_list,
    STATE(3163), 1,
      sym_heredoc_body,
    ACTIONS(4726), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4734), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4736), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1911), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4730), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4728), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [73306] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4726), 1,
      sym_identifier,
    ACTIONS(4834), 1,
      sym_constant,
    STATE(1908), 1,
      sym_constant_suffix,
    STATE(1914), 1,
      sym_identifier_suffix,
    STATE(2263), 1,
      sym_argument_list,
    STATE(3164), 1,
      sym_heredoc_body,
    ACTIONS(4734), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4736), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1911), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4730), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4728), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [73380] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    ACTIONS(4836), 1,
      anon_sym_EQ_GT,
    STATE(3165), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [73470] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(4698), 1,
      sym_identifier,
    ACTIONS(4838), 1,
      sym_constant,
    STATE(1950), 1,
      sym_identifier_suffix,
    STATE(1955), 1,
      sym_constant_suffix,
    STATE(3078), 1,
      sym_argument_list,
    STATE(3166), 1,
      sym_heredoc_body,
    ACTIONS(4706), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4708), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1947), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4702), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4700), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [73544] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(1950), 1,
      sym_identifier_suffix,
    STATE(1955), 1,
      sym_constant_suffix,
    STATE(3078), 1,
      sym_argument_list,
    STATE(3167), 1,
      sym_heredoc_body,
    ACTIONS(4698), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4706), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4708), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1947), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4702), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4700), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [73616] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(141), 1,
      anon_sym_LPAREN2,
    ACTIONS(4726), 1,
      sym_identifier,
    ACTIONS(4840), 1,
      sym_constant,
    STATE(1908), 1,
      sym_constant_suffix,
    STATE(1914), 1,
      sym_identifier_suffix,
    STATE(2266), 1,
      sym_argument_list,
    STATE(3168), 1,
      sym_heredoc_body,
    ACTIONS(4734), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4736), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(1911), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4730), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4728), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [73690] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3169), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [73776] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3170), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [73862] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3171), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 34,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [73918] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    ACTIONS(4806), 1,
      sym_identifier,
    ACTIONS(4842), 1,
      sym_constant,
    STATE(2198), 1,
      sym_identifier_suffix,
    STATE(2210), 1,
      sym_constant_suffix,
    STATE(3078), 1,
      sym_argument_list,
    STATE(3172), 1,
      sym_heredoc_body,
    ACTIONS(4812), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4814), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2212), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4810), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4808), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [73992] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(313), 1,
      anon_sym_LPAREN2,
    STATE(2198), 1,
      sym_identifier_suffix,
    STATE(2210), 1,
      sym_constant_suffix,
    STATE(3078), 1,
      sym_argument_list,
    STATE(3173), 1,
      sym_heredoc_body,
    ACTIONS(4806), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(4812), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(4814), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2212), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(4810), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(4808), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [74064] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    STATE(3174), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [74142] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4844), 1,
      anon_sym_COMMA,
    STATE(3175), 1,
      sym_heredoc_body,
    STATE(3178), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [74201] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3865), 1,
      sym__element_reference_bracket,
    ACTIONS(4088), 1,
      anon_sym_DOT,
    ACTIONS(4092), 1,
      anon_sym_AMP_DOT,
    ACTIONS(4846), 1,
      anon_sym_COLON_COLON2,
    STATE(3176), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [74264] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4740), 1,
      anon_sym_rescue,
    STATE(3177), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [74353] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4848), 1,
      anon_sym_COMMA,
    STATE(3178), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [74410] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4851), 1,
      anon_sym_rescue,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3179), 1,
      sym_heredoc_body,
    STATE(3207), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [74499] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3180), 1,
      sym_heredoc_body,
    ACTIONS(3451), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(3449), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [74554] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3181), 1,
      sym_heredoc_body,
    ACTIONS(2630), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(2632), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [74609] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    ACTIONS(4885), 1,
      anon_sym_rescue,
    ACTIONS(4887), 1,
      anon_sym_COMMA,
    STATE(3182), 1,
      sym_heredoc_body,
    STATE(3207), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [74704] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4887), 1,
      anon_sym_COMMA,
    STATE(3183), 1,
      sym_heredoc_body,
    STATE(3207), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [74763] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(4456), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(3184), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [74826] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3185), 1,
      sym_heredoc_body,
    ACTIONS(2626), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(2628), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [74881] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3186), 1,
      sym_heredoc_body,
    ACTIONS(2638), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(2640), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [74936] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3187), 1,
      sym_heredoc_body,
    ACTIONS(3551), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(3549), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [74991] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(3985), 1,
      anon_sym_PIPE,
    ACTIONS(3987), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3989), 1,
      anon_sym_AMP,
    ACTIONS(3993), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3995), 1,
      anon_sym_CARET,
    ACTIONS(3997), 1,
      anon_sym_QMARK,
    ACTIONS(3999), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4001), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4005), 1,
      anon_sym_GT_EQ,
    ACTIONS(4011), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4015), 1,
      sym__binary_star_star,
    ACTIONS(4740), 1,
      anon_sym_rescue,
    STATE(3188), 1,
      sym_heredoc_body,
    ACTIONS(4003), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4007), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3991), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4009), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4013), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [75080] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3189), 1,
      sym_heredoc_body,
    ACTIONS(2642), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(2644), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [75135] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3190), 1,
      sym_heredoc_body,
    STATE(3196), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75192] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4889), 1,
      anon_sym_rescue,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3175), 1,
      aux_sym_exceptions_repeat1,
    STATE(3191), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [75283] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4923), 1,
      anon_sym_DOT,
    ACTIONS(4925), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(4927), 1,
      anon_sym_AMP_DOT,
    ACTIONS(4929), 1,
      sym__element_reference_bracket,
    STATE(3192), 1,
      sym_heredoc_body,
    ACTIONS(3591), 8,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75346] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3193), 1,
      sym_heredoc_body,
    STATE(3207), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 33,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75403] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4931), 1,
      anon_sym_rescue,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3194), 1,
      sym_heredoc_body,
    STATE(3197), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [75494] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3175), 1,
      aux_sym_exceptions_repeat1,
    STATE(3195), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75551] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4965), 1,
      anon_sym_COMMA,
    STATE(3196), 1,
      sym_heredoc_body,
    STATE(3201), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75610] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4967), 1,
      anon_sym_COMMA,
    STATE(3197), 1,
      sym_heredoc_body,
    STATE(3204), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75669] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(4969), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(3198), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75732] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3199), 1,
      sym_heredoc_body,
    ACTIONS(4971), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(4973), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [75787] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    ACTIONS(4975), 1,
      anon_sym_rescue,
    STATE(3200), 1,
      sym_heredoc_body,
    STATE(3263), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [75878] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4977), 1,
      anon_sym_COMMA,
    STATE(3201), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [75935] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3202), 1,
      sym_heredoc_body,
    ACTIONS(4800), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(4802), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [75990] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3203), 1,
      sym_heredoc_body,
    ACTIONS(2634), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(2636), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [76045] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4980), 1,
      anon_sym_COMMA,
    STATE(3204), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [76102] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3197), 1,
      aux_sym_exceptions_repeat1,
    STATE(3205), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [76159] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    ACTIONS(4983), 1,
      anon_sym_rescue,
    STATE(3206), 1,
      sym_heredoc_body,
    STATE(3288), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [76250] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4887), 1,
      anon_sym_COMMA,
    STATE(3207), 1,
      sym_heredoc_body,
    STATE(3210), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [76309] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3208), 1,
      sym_heredoc_body,
    ACTIONS(2614), 12,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
      sym_constant,
      sym_identifier,
    ACTIONS(2616), 29,
      sym_simple_symbol,
      sym__symbol_start,
      sym__identifier_suffix,
      sym__constant_suffix,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
      aux_sym_constant_suffix_token1,
      aux_sym_identifier_suffix_token1,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [76364] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    ACTIONS(4885), 1,
      anon_sym_rescue,
    ACTIONS(4887), 1,
      anon_sym_COMMA,
    STATE(3207), 1,
      aux_sym_exceptions_repeat1,
    STATE(3209), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [76455] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4985), 1,
      anon_sym_COMMA,
    STATE(3210), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [76512] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4851), 1,
      anon_sym_rescue,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3207), 1,
      aux_sym_exceptions_repeat1,
    STATE(3211), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [76605] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    ACTIONS(4983), 1,
      anon_sym_rescue,
    STATE(3212), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [76693] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3213), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [76781] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3503), 1,
      anon_sym_EQ_GT,
    STATE(3214), 1,
      sym_heredoc_body,
    ACTIONS(3501), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3499), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [76837] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3215), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 15,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [76919] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3216), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [77007] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3217), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [77095] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3218), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [77179] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3219), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [77263] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3220), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [77349] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3282), 1,
      anon_sym_COLON,
    STATE(3221), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [77405] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3282), 1,
      anon_sym_COLON,
    STATE(3222), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [77461] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3223), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [77547] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5000), 1,
      anon_sym_QMARK,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3224), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [77633] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3225), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [77717] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5000), 1,
      anon_sym_QMARK,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3226), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [77803] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5020), 1,
      anon_sym_rescue,
    ACTIONS(5022), 1,
      anon_sym_COMMA,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3227), 1,
      sym_heredoc_body,
    STATE(3277), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 9,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [77897] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3228), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [77981] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3229), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [78065] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3230), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [78149] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3284), 1,
      anon_sym_COLON,
    STATE(3231), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [78205] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3284), 1,
      anon_sym_COLON,
    STATE(3232), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [78261] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5000), 1,
      anon_sym_QMARK,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3233), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [78347] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5000), 1,
      anon_sym_QMARK,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3234), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [78433] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3235), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [78519] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3236), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 14,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [78603] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3237), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [78689] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3238), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [78775] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5056), 1,
      sym__binary_star_star,
    STATE(3239), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [78831] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3240), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 15,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [78913] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3241), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [78999] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    ACTIONS(5058), 1,
      anon_sym_EQ_GT,
    STATE(3242), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [79087] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    ACTIONS(4975), 1,
      anon_sym_rescue,
    STATE(3243), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [79175] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3244), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79231] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3245), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 14,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [79315] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5020), 1,
      anon_sym_rescue,
    ACTIONS(5022), 1,
      anon_sym_COMMA,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3246), 1,
      sym_heredoc_body,
    STATE(3277), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 11,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [79405] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3247), 1,
      sym_heredoc_body,
    STATE(3288), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79461] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3248), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [79547] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    ACTIONS(4983), 1,
      anon_sym_rescue,
    STATE(3249), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [79635] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3250), 1,
      sym_heredoc_body,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79693] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3251), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79749] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3252), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79805] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3253), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79869] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3254), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79931] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3255), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [79999] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3256), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [80077] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3257), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [80153] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3258), 1,
      sym_heredoc_body,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [80213] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3259), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [80289] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3260), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [80345] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4889), 1,
      anon_sym_rescue,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3261), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [80433] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2034), 1,
      sym_identifier_suffix,
    STATE(2035), 1,
      sym_constant_suffix,
    STATE(3262), 1,
      sym_heredoc_body,
    ACTIONS(5060), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(5066), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(5068), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2033), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(5064), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(5062), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [80499] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5070), 1,
      anon_sym_COMMA,
    STATE(3263), 1,
      sym_heredoc_body,
    STATE(3268), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [80557] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4626), 1,
      anon_sym_PIPE,
    ACTIONS(4628), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4630), 1,
      anon_sym_AMP,
    ACTIONS(4638), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4640), 1,
      anon_sym_CARET,
    ACTIONS(4642), 1,
      anon_sym_QMARK,
    ACTIONS(4644), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4646), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4650), 1,
      anon_sym_GT_EQ,
    ACTIONS(4656), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4660), 1,
      sym__binary_star_star,
    STATE(3264), 1,
      sym_heredoc_body,
    ACTIONS(4648), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4652), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4632), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4654), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4658), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [80641] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3265), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [80727] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3266), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [80811] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3267), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [80897] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5072), 1,
      anon_sym_COMMA,
    STATE(3268), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [80953] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3269), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [81039] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5075), 1,
      sym__binary_star_star,
    STATE(3270), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81095] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3263), 1,
      aux_sym_exceptions_repeat1,
    STATE(3271), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81151] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4931), 1,
      anon_sym_rescue,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3272), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [81239] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3273), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [81323] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3274), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81383] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3275), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81439] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5022), 1,
      anon_sym_COMMA,
    STATE(3276), 1,
      sym_heredoc_body,
    STATE(3277), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81497] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5022), 1,
      anon_sym_COMMA,
    STATE(3277), 1,
      sym_heredoc_body,
    STATE(3322), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81555] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4851), 1,
      anon_sym_rescue,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3278), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [81645] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5077), 1,
      anon_sym_DOT,
    ACTIONS(5079), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(5081), 1,
      anon_sym_AMP_DOT,
    ACTIONS(5083), 1,
      sym__element_reference_bracket,
    STATE(3279), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81707] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4851), 1,
      anon_sym_rescue,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3280), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [81793] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    ACTIONS(4975), 1,
      anon_sym_rescue,
    STATE(3281), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [81881] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(4456), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(3282), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81943] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3283), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [81999] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3284), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 14,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [82083] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3285), 1,
      sym_heredoc_body,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82141] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3286), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82197] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3287), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82253] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5085), 1,
      anon_sym_COMMA,
    STATE(3288), 1,
      sym_heredoc_body,
    STATE(3337), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82311] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3289), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82373] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3290), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82441] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3291), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [82519] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3292), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [82595] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3293), 1,
      sym_heredoc_body,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82655] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3294), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [82731] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3295), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [82787] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4931), 1,
      anon_sym_rescue,
    ACTIONS(4933), 1,
      anon_sym_PIPE,
    ACTIONS(4935), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4941), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4943), 1,
      anon_sym_CARET,
    ACTIONS(4945), 1,
      anon_sym_QMARK,
    ACTIONS(4947), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4949), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4953), 1,
      anon_sym_GT_EQ,
    ACTIONS(4959), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3296), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4939), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4961), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [82875] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3297), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [82961] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3298), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [83047] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5083), 1,
      sym__element_reference_bracket,
    ACTIONS(5087), 1,
      anon_sym_DOT,
    ACTIONS(5089), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(5091), 1,
      anon_sym_AMP_DOT,
    STATE(3299), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83109] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3300), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [83195] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2086), 1,
      sym_constant_suffix,
    STATE(2104), 1,
      sym_identifier_suffix,
    STATE(3301), 1,
      sym_heredoc_body,
    ACTIONS(5093), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(5099), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(5101), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2087), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(5097), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(5095), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [83261] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3286), 1,
      anon_sym_EQ_GT,
    STATE(3302), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83317] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2065), 1,
      sym_constant_suffix,
    STATE(2066), 1,
      sym_identifier_suffix,
    STATE(3303), 1,
      sym_heredoc_body,
    ACTIONS(5103), 2,
      sym_constant,
      sym_identifier,
    ACTIONS(5109), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(5111), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    STATE(2054), 2,
      sym__function_identifier,
      sym_operator,
    ACTIONS(5107), 10,
      anon_sym_STAR,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_PLUS,
      anon_sym_EQ_EQ,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_DASH,
      anon_sym_LBRACK_RBRACK,
    ACTIONS(5105), 20,
      anon_sym_PIPE,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
      anon_sym_CARET,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
      anon_sym_PLUS_AT,
      anon_sym_DASH_AT,
      anon_sym_TILDE_AT,
      anon_sym_LBRACK_RBRACK_EQ,
      anon_sym_BQUOTE,
  [83383] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5113), 1,
      sym__binary_star_star,
    STATE(3304), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83439] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3305), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [83523] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3306), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83579] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3307), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [83663] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5000), 1,
      anon_sym_QMARK,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3308), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [83749] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3309), 1,
      sym_heredoc_body,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83807] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3310), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83863] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3311), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83919] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3312), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [83983] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3313), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [84045] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3314), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [84113] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3315), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [84191] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3316), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [84267] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3317), 1,
      sym_heredoc_body,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [84327] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3318), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [84403] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3319), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [84459] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3625), 1,
      anon_sym_EQ_GT,
    STATE(3320), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [84515] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4889), 1,
      anon_sym_rescue,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3321), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [84603] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5115), 1,
      anon_sym_COMMA,
    STATE(3322), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [84659] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3323), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [84745] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3324), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [84829] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3325), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [84913] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3326), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [84997] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3327), 1,
      sym_heredoc_body,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__binary_minus,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85055] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5118), 1,
      sym__binary_star_star,
    STATE(3328), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85111] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3329), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85167] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3330), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85223] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3331), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85287] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    STATE(3332), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 14,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [85371] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3333), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85433] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5120), 1,
      sym__binary_star_star,
    STATE(3334), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85489] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3335), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85557] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5122), 1,
      sym__binary_star_star,
    STATE(3336), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85613] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5124), 1,
      anon_sym_COMMA,
    STATE(3337), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85669] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3338), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [85747] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3339), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 18,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [85823] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3340), 1,
      sym_heredoc_body,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [85883] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3341), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 18,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [85959] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5127), 1,
      anon_sym_rescue,
    STATE(3277), 1,
      aux_sym_exceptions_repeat1,
    STATE(3342), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 10,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [86051] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3343), 1,
      sym_heredoc_body,
    ACTIONS(3465), 2,
      anon_sym_LBRACE,
      anon_sym_do,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [86107] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5127), 1,
      anon_sym_rescue,
    STATE(3277), 1,
      aux_sym_exceptions_repeat1,
    STATE(3344), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 12,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [86195] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3345), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [86251] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5000), 1,
      anon_sym_QMARK,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3346), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [86337] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(4988), 1,
      anon_sym_PIPE,
    ACTIONS(4990), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4992), 1,
      anon_sym_AMP,
    ACTIONS(4996), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4998), 1,
      anon_sym_CARET,
    ACTIONS(5000), 1,
      anon_sym_QMARK,
    ACTIONS(5002), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5004), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5008), 1,
      anon_sym_GT_EQ,
    ACTIONS(5014), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5018), 1,
      sym__binary_star_star,
    STATE(3347), 1,
      sym_heredoc_body,
    ACTIONS(5006), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5010), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4994), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5012), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5016), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [86423] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3277), 1,
      aux_sym_exceptions_repeat1,
    STATE(3348), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 32,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [86479] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    ACTIONS(5129), 1,
      anon_sym_rescue,
    ACTIONS(5131), 1,
      anon_sym_COMMA,
    STATE(3349), 1,
      sym_heredoc_body,
    STATE(3403), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [86571] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3350), 1,
      sym_heredoc_body,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [86625] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3485), 1,
      anon_sym_EQ_GT,
    STATE(3351), 1,
      sym_heredoc_body,
    ACTIONS(3483), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3465), 31,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [86681] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    ACTIONS(5133), 1,
      anon_sym_rescue,
    STATE(3352), 1,
      sym_heredoc_body,
    STATE(3403), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [86771] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4937), 1,
      anon_sym_AMP,
    ACTIONS(4963), 1,
      sym__binary_star_star,
    STATE(3353), 1,
      sym_heredoc_body,
    ACTIONS(4951), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4955), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4957), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [86835] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3354), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [86918] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3355), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [87001] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    ACTIONS(5197), 1,
      anon_sym_QMARK,
    STATE(3356), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [87086] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    ACTIONS(5197), 1,
      anon_sym_QMARK,
    STATE(3357), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [87171] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5211), 1,
      anon_sym_QMARK,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3358), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [87256] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5231), 1,
      sym__binary_star_star,
    STATE(3359), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [87311] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3360), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [87398] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3361), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [87453] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3362), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 13,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [87536] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3363), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [87619] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3364), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [87702] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    ACTIONS(5197), 1,
      anon_sym_QMARK,
    STATE(3365), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [87787] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3366), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [87854] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3367), 1,
      sym_heredoc_body,
    STATE(3403), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [87909] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4776), 1,
      sym__binary_star_star,
    STATE(3368), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [87964] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    ACTIONS(2942), 1,
      sym__symbol_start,
    ACTIONS(3188), 1,
      anon_sym_nil,
    ACTIONS(3192), 1,
      anon_sym_DASH_GT,
    STATE(3369), 1,
      sym_heredoc_body,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4447), 1,
      sym__pattern_primitive,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2822), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2824), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    ACTIONS(3190), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [88057] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3370), 1,
      sym_heredoc_body,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88114] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3371), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88169] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3372), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88224] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3373), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88287] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3374), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88348] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3375), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88415] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3376), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 15,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [88492] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3377), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [88567] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3378), 1,
      sym_heredoc_body,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88626] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3379), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [88701] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    STATE(3380), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88756] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5211), 1,
      anon_sym_QMARK,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3381), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [88841] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3382), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [88894] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    ACTIONS(3188), 1,
      anon_sym_nil,
    ACTIONS(3192), 1,
      anon_sym_DASH_GT,
    STATE(3383), 1,
      sym_heredoc_body,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4447), 1,
      sym__pattern_primitive,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2822), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2824), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    ACTIONS(3190), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [88987] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5233), 1,
      anon_sym_COMMA,
    STATE(3384), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89042] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3385), 1,
      sym_heredoc_body,
    ACTIONS(3599), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3597), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89095] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3386), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [89176] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3387), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [89259] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3388), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89318] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3389), 1,
      sym_heredoc_body,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89375] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3390), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89430] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3391), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89485] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3392), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89548] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3393), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89609] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3394), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89676] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3395), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 15,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [89753] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3396), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [89828] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3397), 1,
      sym_heredoc_body,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [89887] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3398), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [89962] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3399), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [90017] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3400), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [90104] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3313), 1,
      anon_sym_COLON,
    STATE(3401), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [90159] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3313), 1,
      anon_sym_COLON,
    STATE(3402), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [90214] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5131), 1,
      anon_sym_COMMA,
    STATE(3384), 1,
      aux_sym_exceptions_repeat1,
    STATE(3403), 1,
      sym_heredoc_body,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [90271] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3404), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [90332] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3405), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [90419] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    ACTIONS(5133), 1,
      anon_sym_rescue,
    STATE(3406), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [90506] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5211), 1,
      anon_sym_QMARK,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3407), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [90591] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5211), 1,
      anon_sym_QMARK,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3408), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [90676] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5211), 1,
      anon_sym_QMARK,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3409), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [90761] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3410), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [90844] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5211), 1,
      anon_sym_QMARK,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3411), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [90929] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5236), 1,
      sym__binary_star_star,
    STATE(3412), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [90984] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3413), 1,
      sym_heredoc_body,
    ACTIONS(3465), 3,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_do,
    ACTIONS(3010), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3008), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91039] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5211), 1,
      anon_sym_QMARK,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3414), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 12,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [91124] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5238), 1,
      sym__binary_star_star,
    STATE(3415), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91179] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3416), 1,
      sym_heredoc_body,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91232] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4072), 1,
      sym__element_reference_bracket,
    ACTIONS(4185), 1,
      anon_sym_DOT,
    ACTIONS(4189), 1,
      anon_sym_AMP_DOT,
    ACTIONS(5240), 1,
      anon_sym_COLON_COLON2,
    STATE(3417), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91293] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3418), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [91374] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3419), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91427] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3420), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91482] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    ACTIONS(5197), 1,
      anon_sym_QMARK,
    STATE(3421), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [91567] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3422), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [91650] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3423), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [91727] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3424), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [91802] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3425), 1,
      sym_heredoc_body,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91861] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3426), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [91936] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5131), 1,
      anon_sym_COMMA,
    STATE(3403), 1,
      aux_sym_exceptions_repeat1,
    STATE(3427), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [91993] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3428), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [92076] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3429), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [92159] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3430), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 14,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [92240] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3431), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [92323] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5127), 1,
      anon_sym_rescue,
    STATE(3432), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 12,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [92408] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3433), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [92491] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3434), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [92574] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    ACTIONS(5197), 1,
      anon_sym_QMARK,
    STATE(3435), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [92659] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3436), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 14,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [92740] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3437), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [92827] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3438), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [92914] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4122), 1,
      anon_sym_PIPE,
    ACTIONS(4124), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4126), 1,
      anon_sym_AMP,
    ACTIONS(4130), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4132), 1,
      anon_sym_CARET,
    ACTIONS(4134), 1,
      anon_sym_QMARK,
    ACTIONS(4136), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4138), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4142), 1,
      anon_sym_GT_EQ,
    ACTIONS(4148), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4152), 1,
      sym__binary_star_star,
    ACTIONS(5133), 1,
      anon_sym_rescue,
    STATE(3439), 1,
      sym_heredoc_body,
    ACTIONS(4140), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4144), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4146), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4150), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [93001] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3440), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [93088] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3441), 1,
      sym_heredoc_body,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__binary_minus,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93145] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3442), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93200] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3443), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93255] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3444), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93318] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3445), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93379] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3446), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93446] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3447), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [93523] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3448), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [93598] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3449), 1,
      sym_heredoc_body,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93657] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3450), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 17,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [93732] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3451), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93787] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3452), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [93870] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3453), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 30,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [93925] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5199), 1,
      anon_sym_PIPE,
    ACTIONS(5201), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5203), 1,
      anon_sym_AMP,
    ACTIONS(5207), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5209), 1,
      anon_sym_CARET,
    ACTIONS(5213), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5215), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5219), 1,
      anon_sym_GT_EQ,
    ACTIONS(5225), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5229), 1,
      sym__binary_star_star,
    STATE(3454), 1,
      sym_heredoc_body,
    ACTIONS(5217), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5221), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5205), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5223), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5227), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 13,
      sym__element_reference_bracket,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [94008] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2830), 1,
      sym_simple_symbol,
    ACTIONS(2832), 1,
      sym__string_start,
    ACTIONS(2834), 1,
      sym__symbol_start,
    ACTIONS(2836), 1,
      sym__subshell_start,
    ACTIONS(2838), 1,
      sym__regex_start,
    ACTIONS(2840), 1,
      sym__string_array_start,
    ACTIONS(2842), 1,
      sym__symbol_array_start,
    ACTIONS(2844), 1,
      sym_heredoc_beginning,
    ACTIONS(3188), 1,
      anon_sym_nil,
    ACTIONS(3192), 1,
      anon_sym_DASH_GT,
    STATE(3455), 1,
      sym_heredoc_body,
    STATE(4381), 1,
      sym_nil,
    STATE(4411), 1,
      sym_lambda,
    STATE(4447), 1,
      sym__pattern_primitive,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2922), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2924), 2,
      sym_integer,
      sym_float,
    STATE(4383), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
    STATE(4406), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4408), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    ACTIONS(3190), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4396), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [94101] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3456), 1,
      sym_heredoc_body,
    ACTIONS(3653), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3651), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [94154] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5242), 1,
      anon_sym_EQ_GT,
    STATE(3457), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [94241] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5127), 1,
      anon_sym_rescue,
    STATE(3458), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 10,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [94330] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3459), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [94389] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3460), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [94444] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(3461), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [94503] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2884), 1,
      sym_simple_symbol,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(2888), 1,
      sym__symbol_start,
    ACTIONS(2890), 1,
      sym__subshell_start,
    ACTIONS(2892), 1,
      sym__regex_start,
    ACTIONS(2894), 1,
      sym__string_array_start,
    ACTIONS(2896), 1,
      sym__symbol_array_start,
    ACTIONS(2898), 1,
      sym_heredoc_beginning,
    ACTIONS(4265), 1,
      anon_sym_nil,
    ACTIONS(4269), 1,
      anon_sym_DASH_GT,
    STATE(3462), 1,
      sym_heredoc_body,
    STATE(4705), 1,
      sym_nil,
    STATE(4735), 1,
      sym_lambda,
    STATE(4789), 1,
      sym__pattern_primitive,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2908), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2910), 2,
      sym_integer,
      sym_float,
    STATE(4692), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4699), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4714), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
    ACTIONS(4267), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4702), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [94596] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3463), 1,
      sym_heredoc_body,
    ACTIONS(3579), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3577), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [94649] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2884), 1,
      sym_simple_symbol,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(2888), 1,
      sym__symbol_start,
    ACTIONS(2890), 1,
      sym__subshell_start,
    ACTIONS(2892), 1,
      sym__regex_start,
    ACTIONS(2894), 1,
      sym__string_array_start,
    ACTIONS(2896), 1,
      sym__symbol_array_start,
    ACTIONS(2898), 1,
      sym_heredoc_beginning,
    ACTIONS(4265), 1,
      anon_sym_nil,
    ACTIONS(4269), 1,
      anon_sym_DASH_GT,
    STATE(3464), 1,
      sym_heredoc_body,
    STATE(4705), 1,
      sym_nil,
    STATE(4735), 1,
      sym_lambda,
    STATE(4789), 1,
      sym__pattern_primitive,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2872), 2,
      sym__unary_minus_num,
      anon_sym_PLUS,
    ACTIONS(2874), 2,
      sym_integer,
      sym_float,
    STATE(4692), 2,
      sym__pattern_lambda,
      sym__pattern_literal,
    STATE(4699), 2,
      sym__numeric,
      sym_delimited_symbol,
    STATE(4714), 2,
      sym_unary_literal,
      sym__simple_numeric,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
    ACTIONS(4267), 6,
      sym_line,
      sym_file,
      sym_encoding,
      sym_self,
      sym_true,
      sym_false,
    STATE(4702), 7,
      sym__keyword_variable,
      sym__literal,
      sym_string,
      sym_subshell,
      sym_string_array,
      sym_symbol_array,
      sym_regex,
  [94742] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3465), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 32,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [94795] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_in,
    ACTIONS(4636), 1,
      anon_sym_EQ_GT,
    ACTIONS(4885), 1,
      anon_sym_rescue,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3466), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [94884] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    ACTIONS(5197), 1,
      anon_sym_QMARK,
    STATE(3467), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [94969] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5167), 1,
      anon_sym_PIPE,
    ACTIONS(5169), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5171), 1,
      anon_sym_AMP,
    ACTIONS(5175), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5177), 1,
      anon_sym_CARET,
    ACTIONS(5179), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5181), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5185), 1,
      anon_sym_GT_EQ,
    ACTIONS(5191), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5195), 1,
      sym__binary_star_star,
    ACTIONS(5197), 1,
      anon_sym_QMARK,
    STATE(3468), 1,
      sym_heredoc_body,
    ACTIONS(5183), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5187), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5173), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5189), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5193), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [95054] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3469), 1,
      sym_heredoc_body,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__binary_minus,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [95111] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3470), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [95166] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3471), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [95221] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3472), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 31,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [95276] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3473), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [95339] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4885), 1,
      anon_sym_rescue,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3474), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [95424] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3475), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [95507] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3476), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [95587] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3477), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [95653] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3478), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 13,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [95733] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5020), 1,
      anon_sym_rescue,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3479), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 9,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [95821] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5290), 1,
      sym__binary_star_star,
    STATE(3480), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [95875] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3481), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [95957] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5020), 1,
      anon_sym_rescue,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3482), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 11,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [96041] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3483), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [96123] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3484), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [96205] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    STATE(3485), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [96281] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    STATE(3486), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [96355] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3487), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 12,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [96437] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3488), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [96503] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3489), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [96561] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5306), 1,
      anon_sym_QMARK,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3490), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [96645] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3491), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [96699] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5306), 1,
      anon_sym_QMARK,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3492), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [96783] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3493), 1,
      sym_heredoc_body,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [96841] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    STATE(3494), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 15,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [96915] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3495), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [96969] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2992), 1,
      anon_sym_COLON,
    STATE(3496), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [97023] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3497), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 15,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [97099] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3498), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [97173] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3499), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [97255] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    ACTIONS(5310), 1,
      anon_sym_rescue,
    STATE(3500), 1,
      sym_heredoc_body,
    STATE(3558), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [97345] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3501), 1,
      sym_heredoc_body,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [97403] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4844), 1,
      anon_sym_COMMA,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    ACTIONS(5312), 1,
      anon_sym_rescue,
    STATE(3175), 1,
      aux_sym_exceptions_repeat1,
    STATE(3502), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [97493] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1998), 1,
      sym__string_start,
    STATE(2833), 1,
      sym_string,
    STATE(3503), 1,
      sym_heredoc_body,
    STATE(3513), 1,
      aux_sym_chained_string_repeat1,
    ACTIONS(3272), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3270), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [97551] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    ACTIONS(5129), 1,
      anon_sym_rescue,
    STATE(3504), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [97637] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3505), 1,
      sym_heredoc_body,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__binary_minus,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [97693] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3506), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 16,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [97767] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3507), 1,
      sym_heredoc_body,
    ACTIONS(2642), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2644), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [97819] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3508), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [97873] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5306), 1,
      anon_sym_QMARK,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3509), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [97957] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3510), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [98039] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3511), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98093] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3512), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 12,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [98175] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5314), 1,
      sym__string_start,
    STATE(2833), 1,
      sym_string,
    STATE(3513), 2,
      sym_heredoc_body,
      aux_sym_chained_string_repeat1,
    ACTIONS(3290), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3288), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98231] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4844), 1,
      anon_sym_COMMA,
    STATE(3175), 1,
      aux_sym_exceptions_repeat1,
    STATE(3514), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98287] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4929), 1,
      sym__element_reference_bracket,
    ACTIONS(5317), 1,
      anon_sym_DOT,
    ACTIONS(5319), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(5321), 1,
      anon_sym_AMP_DOT,
    STATE(3515), 1,
      sym_heredoc_body,
    ACTIONS(3591), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3589), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98347] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3516), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98401] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5306), 1,
      anon_sym_QMARK,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3517), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [98485] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5323), 1,
      sym__binary_star_star,
    STATE(3518), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98539] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3519), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [98625] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3520), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [98711] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3521), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [98793] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3522), 1,
      sym_heredoc_body,
    ACTIONS(2614), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2616), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98845] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3523), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 12,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [98927] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3524), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 30,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [98981] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3525), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 12,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [99063] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    ACTIONS(5325), 1,
      anon_sym_rescue,
    ACTIONS(5327), 1,
      anon_sym_COMMA,
    STATE(3526), 1,
      sym_heredoc_body,
    STATE(3575), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [99153] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3527), 1,
      sym_heredoc_body,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [99209] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5306), 1,
      anon_sym_QMARK,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3528), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [99293] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    ACTIONS(5310), 1,
      anon_sym_rescue,
    STATE(3529), 1,
      sym_heredoc_body,
    STATE(3558), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [99379] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    ACTIONS(5329), 1,
      anon_sym_rescue,
    STATE(3530), 1,
      sym_heredoc_body,
    STATE(3575), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [99467] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3531), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 12,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [99549] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_DOT,
    ACTIONS(4891), 1,
      anon_sym_PIPE,
    ACTIONS(4893), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4895), 1,
      anon_sym_AMP,
    ACTIONS(4899), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4901), 1,
      anon_sym_CARET,
    ACTIONS(4903), 1,
      anon_sym_QMARK,
    ACTIONS(4905), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4907), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4911), 1,
      anon_sym_GT_EQ,
    ACTIONS(4917), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4921), 1,
      sym__binary_star_star,
    ACTIONS(5085), 1,
      anon_sym_COMMA,
    ACTIONS(5331), 1,
      anon_sym_rescue,
    STATE(3288), 1,
      aux_sym_exceptions_repeat1,
    STATE(3532), 1,
      sym_heredoc_body,
    ACTIONS(4909), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4913), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4897), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4915), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4919), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [99639] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5306), 1,
      anon_sym_QMARK,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3533), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [99723] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3534), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [99777] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3535), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [99831] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3536), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [99893] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3537), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [99979] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4211), 1,
      sym__binary_star_star,
    ACTIONS(4213), 1,
      anon_sym_PIPE,
    ACTIONS(4215), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4217), 1,
      anon_sym_AMP,
    ACTIONS(4221), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4223), 1,
      anon_sym_CARET,
    ACTIONS(4225), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4227), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4231), 1,
      anon_sym_GT_EQ,
    ACTIONS(4237), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4241), 1,
      anon_sym_QMARK,
    ACTIONS(5129), 1,
      anon_sym_rescue,
    STATE(3538), 1,
      sym_heredoc_body,
    ACTIONS(4229), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4233), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4219), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4235), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4239), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [100065] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5333), 1,
      anon_sym_EQ_GT,
    STATE(3539), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [100151] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    STATE(3540), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100211] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2992), 1,
      anon_sym_COLON,
    STATE(3541), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100265] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3542), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100327] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1998), 1,
      sym__string_start,
    STATE(2833), 1,
      sym_string,
    STATE(3503), 1,
      aux_sym_chained_string_repeat1,
    STATE(3543), 1,
      sym_heredoc_body,
    ACTIONS(3092), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100385] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    STATE(3544), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100445] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4853), 1,
      anon_sym_PIPE,
    ACTIONS(4855), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4857), 1,
      anon_sym_AMP,
    ACTIONS(4861), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4863), 1,
      anon_sym_CARET,
    ACTIONS(4865), 1,
      anon_sym_QMARK,
    ACTIONS(4867), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4869), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4873), 1,
      anon_sym_GT_EQ,
    ACTIONS(4879), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4883), 1,
      sym__binary_star_star,
    STATE(3545), 1,
      sym_heredoc_body,
    ACTIONS(4871), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4875), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4859), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4877), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4881), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 12,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [100527] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5274), 1,
      anon_sym_PIPE,
    ACTIONS(5276), 1,
      anon_sym_AMP,
    ACTIONS(5278), 1,
      anon_sym_CARET,
    ACTIONS(5286), 1,
      sym__binary_star_star,
    ACTIONS(5294), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5296), 1,
      anon_sym_GT_EQ,
    ACTIONS(5298), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5302), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5304), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5306), 1,
      anon_sym_QMARK,
    ACTIONS(5308), 1,
      anon_sym_PIPE_PIPE,
    STATE(3546), 1,
      sym_heredoc_body,
    ACTIONS(5280), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5282), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5284), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5292), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5300), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [100611] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5337), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5343), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5347), 1,
      anon_sym_QMARK,
    ACTIONS(5349), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3547), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [100694] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5367), 1,
      anon_sym_COMMA,
    STATE(3548), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100747] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    ACTIONS(5329), 1,
      anon_sym_rescue,
    STATE(3549), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [100832] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2738), 1,
      anon_sym_COLON,
    STATE(3550), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100885] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5370), 1,
      sym__binary_star_star,
    STATE(3551), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [100938] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3552), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [101019] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3553), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [101100] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5337), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5343), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5349), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3554), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [101181] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5337), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5343), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5349), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3555), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [101262] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3556), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [101343] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3557), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [101424] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5372), 1,
      anon_sym_COMMA,
    STATE(3558), 1,
      sym_heredoc_body,
    STATE(3560), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [101479] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5327), 1,
      anon_sym_COMMA,
    STATE(3559), 1,
      sym_heredoc_body,
    STATE(3575), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [101534] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5374), 1,
      anon_sym_COMMA,
    STATE(3560), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [101587] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3561), 1,
      sym_heredoc_body,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [101644] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5337), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5343), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5347), 1,
      anon_sym_QMARK,
    ACTIONS(5349), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3562), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [101727] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5377), 1,
      sym__binary_star_star,
    STATE(3563), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [101780] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3564), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [101855] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(343), 1,
      sym__string_start,
    ACTIONS(5379), 1,
      anon_sym_COLON,
    STATE(2797), 1,
      aux_sym_chained_string_repeat1,
    STATE(2833), 1,
      sym_string,
    STATE(3565), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [101914] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3558), 1,
      aux_sym_exceptions_repeat1,
    STATE(3566), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 29,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [101967] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3567), 1,
      sym_heredoc_body,
    STATE(3582), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102020] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    ACTIONS(5310), 1,
      anon_sym_rescue,
    STATE(3568), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [102107] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3569), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102166] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5135), 1,
      anon_sym_PIPE,
    ACTIONS(5137), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5139), 1,
      anon_sym_AMP,
    ACTIONS(5143), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5145), 1,
      anon_sym_CARET,
    ACTIONS(5147), 1,
      anon_sym_QMARK,
    ACTIONS(5149), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5151), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5155), 1,
      anon_sym_GT_EQ,
    ACTIONS(5161), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5165), 1,
      sym__binary_star_star,
    STATE(3570), 1,
      sym_heredoc_body,
    ACTIONS(5153), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5157), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5141), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5159), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5163), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [102247] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2738), 1,
      anon_sym_COLON,
    STATE(3571), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102300] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4542), 1,
      anon_sym_PIPE,
    ACTIONS(4544), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4546), 1,
      anon_sym_AMP,
    ACTIONS(4550), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4552), 1,
      anon_sym_CARET,
    ACTIONS(4554), 1,
      anon_sym_QMARK,
    ACTIONS(4556), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4558), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4562), 1,
      anon_sym_GT_EQ,
    ACTIONS(4568), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4572), 1,
      sym__binary_star_star,
    ACTIONS(5329), 1,
      anon_sym_rescue,
    STATE(3572), 1,
      sym_heredoc_body,
    ACTIONS(4560), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4564), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4548), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4566), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4570), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [102385] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3573), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [102458] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3574), 1,
      sym_heredoc_body,
    STATE(3579), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 29,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102511] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5327), 1,
      anon_sym_COMMA,
    STATE(3575), 1,
      sym_heredoc_body,
    STATE(3581), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102566] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3576), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102627] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    STATE(3577), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 11,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [102708] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3578), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [102781] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5381), 1,
      anon_sym_COMMA,
    STATE(3548), 1,
      aux_sym_exceptions_repeat1,
    STATE(3579), 1,
      sym_heredoc_body,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102836] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4309), 1,
      anon_sym_PIPE,
    ACTIONS(4311), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4313), 1,
      anon_sym_AMP,
    ACTIONS(4321), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4323), 1,
      anon_sym_CARET,
    ACTIONS(4325), 1,
      anon_sym_QMARK,
    ACTIONS(4327), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4329), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4333), 1,
      anon_sym_GT_EQ,
    ACTIONS(4339), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4343), 1,
      sym__binary_star_star,
    ACTIONS(5310), 1,
      anon_sym_rescue,
    STATE(3580), 1,
      sym_heredoc_body,
    ACTIONS(4331), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4335), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4315), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4337), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4341), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [102919] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5383), 1,
      anon_sym_COMMA,
    STATE(3581), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [102972] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5386), 1,
      anon_sym_COMMA,
    STATE(3582), 1,
      sym_heredoc_body,
    STATE(3596), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [103027] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5337), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5343), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5347), 1,
      anon_sym_QMARK,
    ACTIONS(5349), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3583), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [103110] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4086), 1,
      anon_sym_COLON,
    STATE(3584), 1,
      sym_heredoc_body,
    ACTIONS(2626), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2628), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [103163] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3585), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [103228] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5337), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5343), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5347), 1,
      anon_sym_QMARK,
    ACTIONS(5349), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3586), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [103311] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5388), 1,
      anon_sym_rescue,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3587), 1,
      sym_heredoc_body,
    STATE(3607), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [103396] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5422), 1,
      anon_sym_rescue,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3582), 1,
      aux_sym_exceptions_repeat1,
    STATE(3588), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [103485] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5335), 1,
      anon_sym_PIPE,
    ACTIONS(5337), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5339), 1,
      anon_sym_AMP,
    ACTIONS(5343), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5345), 1,
      anon_sym_CARET,
    ACTIONS(5347), 1,
      anon_sym_QMARK,
    ACTIONS(5349), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5351), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5355), 1,
      anon_sym_GT_EQ,
    ACTIONS(5361), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3589), 1,
      sym_heredoc_body,
    ACTIONS(5353), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5357), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5341), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5363), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [103568] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5422), 1,
      anon_sym_rescue,
    STATE(3582), 1,
      aux_sym_exceptions_repeat1,
    STATE(3590), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [103653] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3591), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [103706] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4086), 1,
      anon_sym_COLON,
    STATE(3592), 1,
      sym_heredoc_body,
    ACTIONS(2638), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(2640), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LPAREN2,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [103759] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3593), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [103812] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5428), 1,
      anon_sym_rescue,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3594), 1,
      sym_heredoc_body,
    STATE(3665), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [103897] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5085), 1,
      anon_sym_COMMA,
    STATE(3288), 1,
      aux_sym_exceptions_repeat1,
    STATE(3595), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [103952] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5462), 1,
      anon_sym_COMMA,
    STATE(3596), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [104005] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3597), 1,
      sym_heredoc_body,
    ACTIONS(5359), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [104060] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3598), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [104113] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5365), 1,
      sym__binary_star_star,
    STATE(3599), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [104166] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5465), 1,
      anon_sym_rescue,
    STATE(3579), 1,
      aux_sym_exceptions_repeat1,
    STATE(3600), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [104251] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5388), 1,
      anon_sym_rescue,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3601), 1,
      sym_heredoc_body,
    STATE(3607), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [104340] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3575), 1,
      aux_sym_exceptions_repeat1,
    STATE(3602), 1,
      sym_heredoc_body,
    ACTIONS(3649), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [104393] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5465), 1,
      anon_sym_rescue,
    STATE(3603), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [104475] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(3604), 1,
      sym_heredoc_body,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4497), 1,
      sym__hash_pattern_any_rest,
    STATE(5764), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(5467), 2,
      sym_constant,
      sym_identifier,
    STATE(4483), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5841), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5469), 6,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5471), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [104545] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3605), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [104619] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3606), 1,
      sym_heredoc_body,
    ACTIONS(3321), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3319), 27,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [104669] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5473), 1,
      anon_sym_COMMA,
    STATE(3607), 1,
      sym_heredoc_body,
    STATE(3624), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [104723] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5422), 1,
      anon_sym_rescue,
    STATE(3608), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [104805] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(3609), 1,
      sym_heredoc_body,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4519), 1,
      sym__hash_pattern_any_rest,
    STATE(5764), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(5467), 2,
      sym_constant,
      sym_identifier,
    STATE(4483), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5841), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5475), 6,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5477), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [104875] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3610), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [104955] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3611), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [105039] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    STATE(3612), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [105119] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3613), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [105203] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3614), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [105287] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3615), 1,
      sym_heredoc_body,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      sym__binary_minus,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105341] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3616), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105393] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3617), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105445] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3618), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105505] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3619), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105557] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3620), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [105637] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    ACTIONS(5325), 1,
      anon_sym_rescue,
    STATE(3621), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [105721] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3622), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105779] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3623), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105843] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5515), 1,
      anon_sym_COMMA,
    STATE(3624), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [105895] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(5518), 1,
      anon_sym_rescue,
    STATE(3625), 1,
      sym_heredoc_body,
    STATE(3836), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [105983] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3626), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [106057] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3627), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [106137] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5520), 1,
      anon_sym_COMMA,
    STATE(3628), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [106189] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3629), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 11,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [106267] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3630), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [106339] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3631), 1,
      sym_heredoc_body,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [106395] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3632), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [106459] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3633), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 11,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [106537] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3634), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [106609] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3635), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [106661] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3636), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [106713] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5535), 1,
      anon_sym_QMARK,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3637), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [106795] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4104), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5535), 1,
      anon_sym_QMARK,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3638), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [106877] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5555), 1,
      anon_sym_rescue,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3639), 1,
      sym_heredoc_body,
    STATE(3817), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [106963] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3640), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [107043] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3641), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107097] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3642), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 11,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [107175] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5388), 1,
      anon_sym_rescue,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3643), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [107261] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3644), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [107341] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    STATE(3645), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107393] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3646), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 28,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107445] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3647), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [107529] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3648), 1,
      sym_heredoc_body,
    STATE(3760), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107581] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(5331), 1,
      anon_sym_rescue,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3649), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [107665] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    STATE(3650), 1,
      sym_heredoc_body,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107719] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    STATE(3651), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107771] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    STATE(3652), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107823] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    STATE(3653), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [107883] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3654), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 10,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [107963] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    STATE(3655), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [108021] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5535), 1,
      anon_sym_QMARK,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3656), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [108103] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5535), 1,
      anon_sym_QMARK,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3657), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [108185] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3658), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [108265] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5535), 1,
      anon_sym_QMARK,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3659), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [108347] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5535), 1,
      anon_sym_QMARK,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3660), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [108429] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5621), 1,
      sym__binary_star_star,
    STATE(3661), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [108481] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3662), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 26,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [108537] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5623), 1,
      sym__binary_star_star,
    STATE(3663), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [108589] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    STATE(3664), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [108653] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5625), 1,
      anon_sym_COMMA,
    STATE(3628), 1,
      aux_sym_exceptions_repeat1,
    STATE(3665), 1,
      sym_heredoc_body,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [108707] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(5627), 1,
      anon_sym_rescue,
    ACTIONS(5629), 1,
      anon_sym_COMMA,
    STATE(3666), 1,
      sym_heredoc_body,
    STATE(3836), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 7,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [108793] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3667), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [108873] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3668), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [108947] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3669), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [109019] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    STATE(3670), 1,
      sym_heredoc_body,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109075] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3671), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [109147] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3672), 1,
      sym_heredoc_body,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__binary_minus,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109201] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(5627), 1,
      anon_sym_rescue,
    ACTIONS(5629), 1,
      anon_sym_COMMA,
    STATE(3673), 1,
      sym_heredoc_body,
    STATE(3836), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109291] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3674), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [109371] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    STATE(3675), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109423] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3676), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109475] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(5312), 1,
      anon_sym_rescue,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3677), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [109559] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(5331), 1,
      anon_sym_rescue,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3678), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [109643] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3679), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109699] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5631), 1,
      sym__binary_star_star,
    STATE(3680), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 27,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109751] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    ACTIONS(5633), 1,
      anon_sym_EQ_GT,
    STATE(3681), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [109835] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5635), 1,
      anon_sym_COMMA,
    STATE(3682), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [109887] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3683), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 9,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [109969] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3684), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [110041] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3685), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [110121] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3686), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110173] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5638), 1,
      sym__binary_star_star,
    STATE(3687), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110225] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3688), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110277] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3689), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110337] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3690), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110395] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3691), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110459] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3692), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [110533] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3693), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [110605] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3694), 1,
      sym_heredoc_body,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110661] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5640), 1,
      anon_sym_COMMA,
    STATE(3695), 1,
      sym_heredoc_body,
    STATE(3760), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [110715] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3696), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [110787] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3697), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [110869] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3698), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [110951] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3699), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [111031] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5642), 1,
      anon_sym_EQ_GT,
    STATE(3700), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [111115] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3701), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [111199] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3702), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [111277] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3703), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [111361] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    STATE(3704), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [111415] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3705), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [111495] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    ACTIONS(5640), 1,
      anon_sym_COMMA,
    ACTIONS(5644), 1,
      anon_sym_rescue,
    STATE(3706), 1,
      sym_heredoc_body,
    STATE(3760), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [111583] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3707), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [111663] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5388), 1,
      anon_sym_rescue,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3708), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [111745] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3709), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [111823] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3665), 1,
      aux_sym_exceptions_repeat1,
    STATE(3710), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [111875] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3607), 1,
      aux_sym_exceptions_repeat1,
    STATE(3711), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [111927] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3712), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [112007] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3713), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112059] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3714), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [112139] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5525), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5531), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5535), 1,
      anon_sym_QMARK,
    ACTIONS(5537), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3715), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [112221] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3716), 1,
      sym_heredoc_body,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112275] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3717), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112327] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3718), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112379] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3719), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112439] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3720), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112497] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3721), 1,
      sym_heredoc_body,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112561] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5539), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3722), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [112635] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3723), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [112707] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3724), 1,
      sym_heredoc_body,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112763] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5523), 1,
      anon_sym_PIPE,
    ACTIONS(5527), 1,
      anon_sym_AMP,
    ACTIONS(5533), 1,
      anon_sym_CARET,
    ACTIONS(5543), 1,
      anon_sym_GT_EQ,
    ACTIONS(5549), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3725), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5541), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5545), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5529), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5547), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5551), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [112835] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5553), 1,
      sym__binary_star_star,
    STATE(3726), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112887] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3727), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [112939] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3728), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [113019] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3729), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [113069] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3730), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [113149] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3731), 1,
      sym_heredoc_body,
    ACTIONS(3092), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3090), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [113199] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3732), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 11,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [113277] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3733), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [113357] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3734), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 14,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [113429] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3735), 1,
      sym_heredoc_body,
    ACTIONS(3483), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3465), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [113479] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3736), 1,
      sym_heredoc_body,
    ACTIONS(3501), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3499), 28,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [113529] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3737), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [113609] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3738), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 10,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [113689] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3739), 1,
      sym_heredoc_body,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__binary_minus,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [113743] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3740), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [113825] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3741), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [113877] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    ACTIONS(5422), 1,
      anon_sym_rescue,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    STATE(3742), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [113963] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3743), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 28,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [114015] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3744), 1,
      sym_heredoc_body,
    ACTIONS(3305), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3303), 27,
      sym__string_start,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [114065] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5428), 1,
      anon_sym_rescue,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3745), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [114147] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3746), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [114227] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3747), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [114307] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(5518), 1,
      anon_sym_rescue,
    STATE(3748), 1,
      sym_heredoc_body,
    STATE(3836), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [114391] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    ACTIONS(5646), 1,
      anon_sym_rescue,
    ACTIONS(5648), 1,
      anon_sym_COMMA,
    STATE(3749), 1,
      sym_heredoc_body,
    STATE(3817), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [114479] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3750), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [114539] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(5312), 1,
      anon_sym_rescue,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3751), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [114623] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3752), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [114681] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3753), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 10,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [114761] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    STATE(3754), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 9,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [114843] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3755), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [114923] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3756), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [115003] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    ACTIONS(5650), 1,
      anon_sym_rescue,
    STATE(3757), 1,
      sym_heredoc_body,
    STATE(3760), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [115089] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_DOT,
    ACTIONS(4742), 1,
      anon_sym_PIPE,
    ACTIONS(4744), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4746), 1,
      anon_sym_AMP,
    ACTIONS(4750), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4752), 1,
      anon_sym_CARET,
    ACTIONS(4754), 1,
      anon_sym_QMARK,
    ACTIONS(4756), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4758), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4762), 1,
      anon_sym_GT_EQ,
    ACTIONS(4768), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4772), 1,
      sym__binary_star_star,
    ACTIONS(5325), 1,
      anon_sym_rescue,
    STATE(3758), 1,
      sym_heredoc_body,
    ACTIONS(4760), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4764), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4748), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4766), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4770), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [115173] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3759), 1,
      sym_heredoc_body,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [115229] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5640), 1,
      anon_sym_COMMA,
    STATE(3682), 1,
      aux_sym_exceptions_repeat1,
    STATE(3760), 1,
      sym_heredoc_body,
    ACTIONS(4023), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [115283] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3761), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [115356] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5688), 1,
      anon_sym_QMARK,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3762), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [115437] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(5518), 1,
      anon_sym_rescue,
    STATE(3763), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [115518] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3764), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [115569] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5555), 1,
      anon_sym_rescue,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3765), 1,
      sym_heredoc_body,
    ACTIONS(3539), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [115652] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3766), 1,
      sym_heredoc_body,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [115707] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3767), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [115778] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5710), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5716), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3768), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [115857] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3769), 1,
      sym_heredoc_body,
    STATE(3817), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [115908] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5710), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5716), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    ACTIONS(5738), 1,
      anon_sym_QMARK,
    STATE(3770), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [115989] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3771), 1,
      sym_heredoc_body,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116042] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3772), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116093] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3773), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116144] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3774), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116203] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3775), 1,
      sym_heredoc_body,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116262] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3776), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116319] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3777), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116382] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3778), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [116455] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3779), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [116526] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3780), 1,
      sym_heredoc_body,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116581] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3781), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116640] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3782), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [116711] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3783), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116762] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3784), 1,
      sym_heredoc_body,
    STATE(3836), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 27,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116813] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5710), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5716), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3785), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 9,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [116892] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    STATE(3786), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [116945] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3787), 1,
      sym_heredoc_body,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [117002] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3788), 1,
      sym_heredoc_body,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [117065] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5629), 1,
      anon_sym_COMMA,
    STATE(3789), 1,
      sym_heredoc_body,
    STATE(3836), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [117118] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3790), 1,
      sym_heredoc_body,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [117191] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3791), 1,
      sym_heredoc_body,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [117262] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3792), 1,
      sym_heredoc_body,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [117317] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5754), 1,
      anon_sym_COMMA,
    STATE(3793), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [117368] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3794), 1,
      sym_heredoc_body,
    ACTIONS(3629), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [117419] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    STATE(3795), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [117496] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3796), 1,
      sym_heredoc_body,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [117567] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3797), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [117618] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(3798), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [117697] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5688), 1,
      anon_sym_QMARK,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3799), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [117778] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    STATE(3800), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [117855] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5785), 1,
      anon_sym_EQ_GT,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3801), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [117938] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5428), 1,
      anon_sym_rescue,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3665), 1,
      aux_sym_exceptions_repeat1,
    STATE(3802), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118025] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3803), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118084] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3804), 1,
      sym_heredoc_body,
    ACTIONS(4098), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118135] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4052), 1,
      anon_sym_DOT,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5710), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5716), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    ACTIONS(5738), 1,
      anon_sym_QMARK,
    STATE(3805), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [118216] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3806), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118267] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    STATE(3807), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [118344] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3808), 1,
      sym_heredoc_body,
    ACTIONS(3745), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [118425] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3809), 1,
      sym_heredoc_body,
    ACTIONS(4098), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118476] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(3810), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [118555] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3811), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118612] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3812), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118675] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(3813), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [118754] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    ACTIONS(5650), 1,
      anon_sym_rescue,
    STATE(3814), 1,
      sym_heredoc_body,
    ACTIONS(3653), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [118837] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3815), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [118918] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5789), 1,
      sym__binary_star_star,
    STATE(3816), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [118969] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5648), 1,
      anon_sym_COMMA,
    STATE(3793), 1,
      aux_sym_exceptions_repeat1,
    STATE(3817), 1,
      sym_heredoc_body,
    ACTIONS(4023), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119022] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    STATE(3818), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 25,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119077] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5688), 1,
      anon_sym_QMARK,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3819), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [119158] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5688), 1,
      anon_sym_QMARK,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3820), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [119239] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    STATE(3821), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 10,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [119316] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5710), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5716), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    ACTIONS(5738), 1,
      anon_sym_QMARK,
    STATE(3822), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [119397] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3823), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [119476] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3824), 1,
      sym_heredoc_body,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119527] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5791), 1,
      sym__binary_star_star,
    STATE(3825), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119578] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3826), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119637] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5793), 1,
      sym__binary_star_star,
    STATE(3827), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119688] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3828), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [119769] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3829), 1,
      sym_heredoc_body,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119822] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3830), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [119901] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3831), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [119952] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(3832), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [120031] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3833), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120082] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5795), 1,
      sym__binary_star_star,
    STATE(3834), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120133] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3835), 1,
      sym_heredoc_body,
    ACTIONS(4052), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [120214] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5629), 1,
      anon_sym_COMMA,
    STATE(3836), 1,
      sym_heredoc_body,
    STATE(3846), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120267] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3837), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120324] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    STATE(3838), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120387] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5555), 1,
      anon_sym_rescue,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3839), 1,
      sym_heredoc_body,
    ACTIONS(3653), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [120470] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3840), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120533] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3841), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [120604] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3842), 1,
      sym_heredoc_body,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120659] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3843), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [120742] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3844), 1,
      sym_heredoc_body,
    ACTIONS(4056), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 8,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [120821] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(3845), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [120900] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5797), 1,
      anon_sym_COMMA,
    STATE(3846), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [120951] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3847), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [121022] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3848), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [121073] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3849), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [121124] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    STATE(3850), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [121197] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3851), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [121270] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(3852), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [121349] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3853), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [121432] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3854), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [121513] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    STATE(3855), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [121584] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3856), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [121663] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3857), 1,
      sym_heredoc_body,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [121718] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3858), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [121801] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    STATE(3859), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 13,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [121872] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3860), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [121951] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(3861), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 9,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [122030] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5688), 1,
      anon_sym_QMARK,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3862), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [122111] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3863), 1,
      sym_heredoc_body,
    ACTIONS(4060), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [122192] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3864), 1,
      sym_heredoc_body,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122245] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3865), 1,
      sym_heredoc_body,
    ACTIONS(4104), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [122326] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3866), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [122397] = 24,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5465), 1,
      anon_sym_rescue,
    STATE(3579), 1,
      aux_sym_exceptions_repeat1,
    STATE(3867), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122484] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3868), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122535] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3869), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122586] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5800), 1,
      sym__binary_star_star,
    STATE(3870), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122637] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3871), 1,
      sym_heredoc_body,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122690] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3872), 1,
      sym_heredoc_body,
    ACTIONS(4044), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 8,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [122769] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5710), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5716), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    ACTIONS(5738), 1,
      anon_sym_QMARK,
    STATE(3873), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [122850] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3874), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122901] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3875), 1,
      sym_heredoc_body,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [122954] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    STATE(3876), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [123005] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4056), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3877), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 9,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [123084] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3878), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [123163] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5648), 1,
      anon_sym_COMMA,
    STATE(3817), 1,
      aux_sym_exceptions_repeat1,
    STATE(3879), 1,
      sym_heredoc_body,
    ACTIONS(3649), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [123216] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4044), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(3880), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 9,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [123295] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    STATE(3881), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 27,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [123346] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    STATE(3882), 1,
      sym_heredoc_body,
    ACTIONS(3629), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [123397] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3883), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [123476] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5557), 1,
      anon_sym_PIPE,
    ACTIONS(5559), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5561), 1,
      anon_sym_AMP,
    ACTIONS(5565), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5567), 1,
      anon_sym_CARET,
    ACTIONS(5569), 1,
      anon_sym_QMARK,
    ACTIONS(5571), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5573), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5577), 1,
      anon_sym_GT_EQ,
    ACTIONS(5583), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5587), 1,
      sym__binary_star_star,
    ACTIONS(5650), 1,
      anon_sym_rescue,
    STATE(3884), 1,
      sym_heredoc_body,
    ACTIONS(3539), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5575), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5579), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5563), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5581), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5585), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3537), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [123559] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(3885), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [123638] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4360), 1,
      anon_sym_PIPE,
    ACTIONS(4362), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4364), 1,
      anon_sym_AMP,
    ACTIONS(4368), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4370), 1,
      anon_sym_CARET,
    ACTIONS(4372), 1,
      anon_sym_QMARK,
    ACTIONS(4374), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4376), 1,
      anon_sym_AMP_AMP,
    ACTIONS(4380), 1,
      anon_sym_GT_EQ,
    ACTIONS(4386), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4390), 1,
      sym__binary_star_star,
    ACTIONS(5518), 1,
      anon_sym_rescue,
    STATE(3886), 1,
      sym_heredoc_body,
    ACTIONS(4378), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4382), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4366), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4384), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4388), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [123723] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5802), 1,
      sym__binary_star_star,
    STATE(3887), 1,
      sym_heredoc_body,
    ACTIONS(3909), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 26,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [123774] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_DOT,
    ACTIONS(5708), 1,
      anon_sym_PIPE,
    ACTIONS(5710), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5712), 1,
      anon_sym_AMP,
    ACTIONS(5716), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5718), 1,
      anon_sym_CARET,
    ACTIONS(5720), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5722), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5726), 1,
      anon_sym_GT_EQ,
    ACTIONS(5732), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5736), 1,
      sym__binary_star_star,
    ACTIONS(5738), 1,
      anon_sym_QMARK,
    STATE(3888), 1,
      sym_heredoc_body,
    ACTIONS(5724), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5728), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5714), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5730), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5734), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [123855] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    STATE(3889), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [123912] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3890), 1,
      sym_heredoc_body,
    ACTIONS(4060), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 6,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [123992] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3891), 1,
      sym_heredoc_body,
    ACTIONS(4056), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [124070] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5381), 1,
      anon_sym_COMMA,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5836), 1,
      anon_sym_rescue,
    STATE(3579), 1,
      aux_sym_exceptions_repeat1,
    STATE(3892), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3647), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [124158] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3893), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [124236] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3894), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [124314] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3895), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 9,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [124390] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3896), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [124468] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3897), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [124546] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5646), 1,
      anon_sym_rescue,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3898), 1,
      sym_heredoc_body,
    ACTIONS(3653), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [124628] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5870), 1,
      anon_sym_rescue,
    ACTIONS(5872), 1,
      anon_sym_COMMA,
    STATE(3899), 1,
      sym_heredoc_body,
    STATE(4018), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [124712] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3900), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [124790] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3901), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [124868] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3902), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [124946] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3903), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [125024] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5390), 1,
      anon_sym_PIPE,
    ACTIONS(5392), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5394), 1,
      anon_sym_AMP,
    ACTIONS(5398), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5400), 1,
      anon_sym_CARET,
    ACTIONS(5402), 1,
      anon_sym_QMARK,
    ACTIONS(5404), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5406), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5410), 1,
      anon_sym_GT_EQ,
    ACTIONS(5416), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5420), 1,
      sym__binary_star_star,
    STATE(3904), 1,
      sym_heredoc_body,
    ACTIONS(5408), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5412), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5396), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5414), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5418), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [125102] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3905), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [125184] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3906), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 9,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [125260] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3907), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [125342] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3908), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [125424] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3909), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [125502] = 25,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5625), 1,
      anon_sym_COMMA,
    ACTIONS(5874), 1,
      anon_sym_rescue,
    STATE(3665), 1,
      aux_sym_exceptions_repeat1,
    STATE(3910), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3647), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [125590] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5428), 1,
      anon_sym_rescue,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3911), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [125674] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5644), 1,
      anon_sym_rescue,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3912), 1,
      sym_heredoc_body,
    ACTIONS(3539), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3537), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [125756] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3913), 1,
      sym_heredoc_body,
    ACTIONS(4052), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 6,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [125836] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3914), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [125886] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5876), 1,
      sym__binary_star_star,
    STATE(3915), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [125936] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3916), 1,
      sym_heredoc_body,
    ACTIONS(4098), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [125986] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5890), 1,
      anon_sym_QMARK,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3917), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [126066] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5890), 1,
      anon_sym_QMARK,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3918), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [126146] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3919), 1,
      sym_heredoc_body,
    ACTIONS(4044), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 7,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [126224] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5890), 1,
      anon_sym_QMARK,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3920), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [126304] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5890), 1,
      anon_sym_QMARK,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3921), 1,
      sym_heredoc_body,
    ACTIONS(4052), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [126384] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    ACTIONS(5910), 1,
      anon_sym_COMMA,
    STATE(3922), 1,
      sym_heredoc_body,
    STATE(5249), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5914), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5912), 4,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_EQ_GT,
      anon_sym_then,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [126468] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5916), 1,
      sym__binary_star_star,
    STATE(3923), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [126518] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3924), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [126572] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3925), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [126622] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_in,
    ACTIONS(4319), 1,
      anon_sym_EQ_GT,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5627), 1,
      anon_sym_rescue,
    STATE(3926), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [126706] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4462), 1,
      anon_sym_PIPE,
    ACTIONS(4464), 1,
      anon_sym_AMP,
    ACTIONS(4468), 1,
      anon_sym_CARET,
    ACTIONS(4472), 1,
      anon_sym_GT_EQ,
    ACTIONS(4478), 1,
      anon_sym_EQ_EQ,
    ACTIONS(4482), 1,
      sym__binary_star_star,
    ACTIONS(4484), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4486), 1,
      anon_sym_DOT_DOT,
    ACTIONS(4488), 1,
      anon_sym_QMARK,
    ACTIONS(4490), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(4492), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5627), 1,
      anon_sym_rescue,
    STATE(3927), 1,
      sym_heredoc_body,
    ACTIONS(4470), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(4474), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4466), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4476), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4480), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 7,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [126786] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3928), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [126836] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3929), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [126914] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3930), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [126992] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3931), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [127070] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3932), 1,
      sym_heredoc_body,
    ACTIONS(3629), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127120] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5465), 1,
      anon_sym_rescue,
    STATE(3933), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127204] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5381), 1,
      anon_sym_COMMA,
    STATE(3579), 1,
      aux_sym_exceptions_repeat1,
    STATE(3934), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 25,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127256] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5381), 1,
      anon_sym_COMMA,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5836), 1,
      anon_sym_rescue,
    STATE(3579), 1,
      aux_sym_exceptions_repeat1,
    STATE(3935), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127340] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3936), 1,
      sym_heredoc_body,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127398] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3937), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [127476] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3938), 1,
      sym_heredoc_body,
    ACTIONS(4056), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 7,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [127554] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5890), 1,
      anon_sym_QMARK,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3939), 1,
      sym_heredoc_body,
    ACTIONS(4060), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [127634] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5918), 1,
      sym__binary_star_star,
    STATE(3940), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127684] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3941), 1,
      sym_heredoc_body,
    ACTIONS(3629), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127734] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3942), 1,
      sym_heredoc_body,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127790] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3943), 1,
      sym_heredoc_body,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127842] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3944), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127892] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3945), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [127942] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3946), 1,
      sym_heredoc_body,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [128000] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3947), 1,
      sym_heredoc_body,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [128056] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3948), 1,
      sym_heredoc_body,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [128118] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3949), 1,
      sym_heredoc_body,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [128190] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3950), 1,
      sym_heredoc_body,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [128260] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3951), 1,
      sym_heredoc_body,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [128314] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3952), 1,
      sym_heredoc_body,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [128384] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3953), 1,
      sym_heredoc_body,
    ACTIONS(4098), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [128434] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3954), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [128516] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3955), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 24,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [128570] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    ACTIONS(5920), 1,
      anon_sym_EQ_GT,
    STATE(3956), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [128652] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5922), 1,
      anon_sym_EQ_GT,
    STATE(3957), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4350), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [128734] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3958), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [128816] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(3959), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [128898] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5924), 1,
      sym__binary_star_star,
    STATE(3960), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [128948] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3961), 1,
      sym_heredoc_body,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [129010] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3962), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [129088] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    ACTIONS(5625), 1,
      anon_sym_COMMA,
    ACTIONS(5874), 1,
      anon_sym_rescue,
    STATE(3665), 1,
      aux_sym_exceptions_repeat1,
    STATE(3963), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [129172] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3964), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 8,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [129250] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3965), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [129330] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3966), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [129410] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5890), 1,
      anon_sym_QMARK,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3967), 1,
      sym_heredoc_body,
    ACTIONS(3745), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [129490] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5646), 1,
      anon_sym_rescue,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3968), 1,
      sym_heredoc_body,
    ACTIONS(3539), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3537), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [129572] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5878), 1,
      anon_sym_PIPE,
    ACTIONS(5880), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5882), 1,
      anon_sym_AMP,
    ACTIONS(5886), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5888), 1,
      anon_sym_CARET,
    ACTIONS(5890), 1,
      anon_sym_QMARK,
    ACTIONS(5892), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5894), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5898), 1,
      anon_sym_GT_EQ,
    ACTIONS(5904), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5908), 1,
      sym__binary_star_star,
    STATE(3969), 1,
      sym_heredoc_body,
    ACTIONS(4104), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5896), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5900), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5884), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5902), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5906), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 6,
      sym__element_reference_bracket,
      anon_sym_COMMA,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [129652] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5926), 1,
      anon_sym_rescue,
    STATE(3970), 1,
      sym_heredoc_body,
    STATE(4018), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3647), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [129734] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3971), 1,
      sym_heredoc_body,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [129806] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3972), 1,
      sym_heredoc_body,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [129876] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3973), 1,
      sym_heredoc_body,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [129930] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3974), 1,
      sym_heredoc_body,
    ACTIONS(4044), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 7,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [130008] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3975), 1,
      sym_heredoc_body,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [130078] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3976), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 6,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [130158] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3977), 1,
      sym_heredoc_body,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__binary_minus,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130210] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3978), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130260] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3979), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130310] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3980), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130368] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3981), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130424] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5644), 1,
      anon_sym_rescue,
    ACTIONS(5804), 1,
      anon_sym_PIPE,
    ACTIONS(5806), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5808), 1,
      anon_sym_AMP,
    ACTIONS(5812), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5814), 1,
      anon_sym_CARET,
    ACTIONS(5816), 1,
      anon_sym_QMARK,
    ACTIONS(5818), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5820), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5824), 1,
      anon_sym_GT_EQ,
    ACTIONS(5830), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3982), 1,
      sym_heredoc_body,
    ACTIONS(3653), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5822), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5826), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5810), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5832), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130506] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3983), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130568] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5834), 1,
      sym__binary_star_star,
    STATE(3984), 1,
      sym_heredoc_body,
    ACTIONS(5828), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130620] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3985), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [130692] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3986), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [130762] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3987), 1,
      sym_heredoc_body,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130816] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3988), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 12,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [130886] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(3989), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 26,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [130936] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5479), 1,
      anon_sym_PIPE,
    ACTIONS(5481), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5483), 1,
      anon_sym_AMP,
    ACTIONS(5491), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5493), 1,
      anon_sym_CARET,
    ACTIONS(5495), 1,
      anon_sym_QMARK,
    ACTIONS(5497), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5499), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5503), 1,
      anon_sym_GT_EQ,
    ACTIONS(5509), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5513), 1,
      sym__binary_star_star,
    STATE(3990), 1,
      sym_heredoc_body,
    ACTIONS(5501), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5505), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5485), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5507), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5511), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [131014] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5930), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5936), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5940), 1,
      anon_sym_QMARK,
    ACTIONS(5942), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(3991), 1,
      sym_heredoc_body,
    ACTIONS(4052), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4050), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131093] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(3992), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131142] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    STATE(3993), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131199] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(3994), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131254] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    ACTIONS(5970), 1,
      anon_sym_EQ_GT,
    STATE(3995), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4350), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131335] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    STATE(3996), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131396] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    STATE(3997), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [131467] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(3998), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 7,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [131544] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(3999), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 7,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [131621] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4000), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5914), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4102), 5,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_EQ_GT,
      anon_sym_then,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131700] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4001), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 8,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [131775] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4002), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 7,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [131852] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    STATE(4003), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [131921] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(4004), 1,
      sym_heredoc_body,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [131974] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    STATE(4005), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 11,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [132043] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    STATE(4006), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132096] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4007), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 7,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [132173] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4008), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4350), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132254] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4009), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5914), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3743), 5,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_EQ_GT,
      anon_sym_then,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132333] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5930), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5936), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5942), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4010), 1,
      sym_heredoc_body,
    ACTIONS(4056), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 6,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [132410] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5930), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5936), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5940), 1,
      anon_sym_QMARK,
    ACTIONS(5942), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4011), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132489] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5625), 1,
      anon_sym_COMMA,
    STATE(3665), 1,
      aux_sym_exceptions_repeat1,
    STATE(4012), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132540] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4013), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(3627), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132621] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4014), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(3627), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132702] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    ACTIONS(5994), 1,
      anon_sym_EQ_GT,
    STATE(4015), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4350), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132783] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5996), 1,
      sym__binary_star_star,
    STATE(4016), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132832] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5930), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5936), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5940), 1,
      anon_sym_QMARK,
    ACTIONS(5942), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4017), 1,
      sym_heredoc_body,
    ACTIONS(3629), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132911] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5872), 1,
      anon_sym_COMMA,
    STATE(4018), 1,
      sym_heredoc_body,
    STATE(4028), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [132962] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(4019), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 25,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133011] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5930), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5936), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5942), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4020), 1,
      sym_heredoc_body,
    ACTIONS(4044), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 6,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [133088] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5930), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5936), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5940), 1,
      anon_sym_QMARK,
    ACTIONS(5942), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4021), 1,
      sym_heredoc_body,
    ACTIONS(4048), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4046), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133167] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(4022), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [133244] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(4023), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [133321] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(4024), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [133398] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5024), 1,
      anon_sym_PIPE,
    ACTIONS(5026), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5028), 1,
      anon_sym_AMP,
    ACTIONS(5032), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5034), 1,
      anon_sym_CARET,
    ACTIONS(5036), 1,
      anon_sym_QMARK,
    ACTIONS(5038), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5040), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5044), 1,
      anon_sym_GT_EQ,
    ACTIONS(5050), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5054), 1,
      sym__binary_star_star,
    ACTIONS(5926), 1,
      anon_sym_rescue,
    STATE(4025), 1,
      sym_heredoc_body,
    ACTIONS(5042), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5046), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5030), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5048), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5052), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3651), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [133477] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4026), 1,
      sym_heredoc_body,
    ACTIONS(4098), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133526] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(4027), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [133603] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5998), 1,
      anon_sym_COMMA,
    STATE(4028), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133652] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4029), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 8,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [133727] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4030), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(3627), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133808] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(4031), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 25,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133857] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3539), 1,
      anon_sym_DOT,
    ACTIONS(5589), 1,
      sym__binary_star_star,
    ACTIONS(5591), 1,
      anon_sym_PIPE,
    ACTIONS(5593), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5595), 1,
      anon_sym_AMP,
    ACTIONS(5599), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5601), 1,
      anon_sym_CARET,
    ACTIONS(5603), 1,
      anon_sym_QMARK,
    ACTIONS(5605), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5607), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5611), 1,
      anon_sym_GT_EQ,
    ACTIONS(5617), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6001), 1,
      anon_sym_rescue,
    STATE(4032), 1,
      sym_heredoc_body,
    ACTIONS(5609), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5613), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6003), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(3537), 3,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(5597), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5615), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5619), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133940] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4018), 1,
      aux_sym_exceptions_repeat1,
    STATE(4033), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 25,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [133989] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4034), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(3627), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134070] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5872), 1,
      anon_sym_COMMA,
    STATE(4018), 1,
      aux_sym_exceptions_repeat1,
    STATE(4035), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134121] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4036), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4350), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134202] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5930), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5936), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5940), 1,
      anon_sym_QMARK,
    ACTIONS(5942), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4037), 1,
      sym_heredoc_body,
    ACTIONS(4060), 2,
      anon_sym_DOT,
      anon_sym_COLON2,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4058), 5,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134281] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5487), 1,
      anon_sym_in,
    ACTIONS(5489), 1,
      anon_sym_EQ_GT,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(4038), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 23,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134334] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(4039), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 25,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134383] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6005), 1,
      sym__binary_star_star,
    STATE(4040), 1,
      sym_heredoc_body,
    ACTIONS(3909), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3907), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134432] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    STATE(4041), 1,
      sym_heredoc_body,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__line_break,
      sym__binary_minus,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_then,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134483] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4042), 1,
      sym_heredoc_body,
    ACTIONS(3629), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134532] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4043), 1,
      sym_heredoc_body,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__binary_minus,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134583] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4044), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134632] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4045), 1,
      sym_heredoc_body,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134681] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4046), 1,
      sym_heredoc_body,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 8,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134738] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4047), 1,
      sym_heredoc_body,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134793] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4048), 1,
      sym_heredoc_body,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_DOT,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [134854] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5944), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4049), 1,
      sym_heredoc_body,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [134925] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4050), 1,
      sym_heredoc_body,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [134994] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4051), 1,
      sym_heredoc_body,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 9,
      anon_sym_DOT,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [135047] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5928), 1,
      anon_sym_PIPE,
    ACTIONS(5932), 1,
      anon_sym_AMP,
    ACTIONS(5938), 1,
      anon_sym_CARET,
    ACTIONS(5948), 1,
      anon_sym_GT_EQ,
    ACTIONS(5954), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5958), 1,
      sym__binary_star_star,
    STATE(4052), 1,
      sym_heredoc_body,
    ACTIONS(5946), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5950), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4048), 3,
      anon_sym_DOT,
      anon_sym_DOT_DOT,
      anon_sym_COLON2,
    ACTIONS(5934), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5952), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5956), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__element_reference_bracket,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [135116] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4053), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 7,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [135193] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(4054), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [135270] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4055), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4401), 4,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [135348] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6021), 1,
      anon_sym_QMARK,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4056), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3743), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [135424] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4057), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 24,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [135472] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    STATE(4058), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 7,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [135546] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4059), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [135622] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4060), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [135698] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4061), 1,
      sym_heredoc_body,
    STATE(4065), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 24,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [135746] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5244), 1,
      anon_sym_PIPE,
    ACTIONS(5246), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5248), 1,
      anon_sym_AMP,
    ACTIONS(5252), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5254), 1,
      anon_sym_CARET,
    ACTIONS(5256), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5258), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5262), 1,
      anon_sym_GT_EQ,
    ACTIONS(5268), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5272), 1,
      sym__binary_star_star,
    ACTIONS(5288), 1,
      anon_sym_QMARK,
    ACTIONS(5870), 1,
      anon_sym_rescue,
    STATE(4062), 1,
      sym_heredoc_body,
    ACTIONS(5260), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5264), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5250), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5266), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5270), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [135824] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5836), 1,
      anon_sym_rescue,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4063), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3651), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [135906] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4064), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [135954] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6073), 1,
      anon_sym_COMMA,
    STATE(4065), 1,
      sym_heredoc_body,
    STATE(4123), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [136004] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4066), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [136080] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    STATE(4067), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [136136] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4068), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [136212] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4069), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [136288] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    STATE(4070), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 7,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [136362] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4071), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [136438] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4072), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [136492] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    STATE(4073), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [136552] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4074), 1,
      sym_heredoc_body,
    ACTIONS(2632), 13,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_EQ_GT,
    ACTIONS(2630), 19,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
      sym_identifier,
  [136598] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4075), 1,
      sym_heredoc_body,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__binary_minus,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [136648] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4076), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5914), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6075), 4,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [136726] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4077), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [136802] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    STATE(4078), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [136872] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    STATE(4079), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [136940] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4080), 1,
      sym_heredoc_body,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [136992] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    STATE(4081), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [137060] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4082), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 24,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137108] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(4083), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [137184] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5874), 1,
      anon_sym_rescue,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4084), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137262] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(4085), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [137338] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(4086), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [137414] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4087), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137462] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4088), 1,
      sym_heredoc_body,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__line_break,
      sym__binary_minus,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137512] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4089), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137560] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4090), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137608] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4091), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137664] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4092), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137718] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4093), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137778] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4094), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [137848] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4095), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4395), 4,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137926] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4096), 1,
      sym_heredoc_body,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [137978] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4097), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [138046] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4098), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [138094] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6021), 1,
      anon_sym_QMARK,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4099), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [138170] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6021), 1,
      anon_sym_QMARK,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4100), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(3627), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [138246] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4101), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 7,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [138320] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6021), 1,
      anon_sym_QMARK,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4102), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [138396] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6021), 1,
      anon_sym_QMARK,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4103), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4050), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [138472] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6021), 1,
      anon_sym_QMARK,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4104), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4058), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [138548] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4105), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 24,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [138596] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4106), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4350), 4,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [138676] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    STATE(4107), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [138728] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(4108), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4395), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [138804] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5430), 1,
      anon_sym_PIPE,
    ACTIONS(5432), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5434), 1,
      anon_sym_AMP,
    ACTIONS(5438), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5440), 1,
      anon_sym_CARET,
    ACTIONS(5442), 1,
      anon_sym_QMARK,
    ACTIONS(5444), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5446), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5450), 1,
      anon_sym_GT_EQ,
    ACTIONS(5456), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5460), 1,
      sym__binary_star_star,
    STATE(4109), 1,
      sym_heredoc_body,
    ACTIONS(5448), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5452), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5436), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5454), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5458), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4401), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [138880] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4110), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 4,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [138960] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4111), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 4,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139040] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4112), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4443), 4,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139118] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6077), 1,
      anon_sym_rescue,
    ACTIONS(6079), 1,
      anon_sym_COMMA,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4113), 1,
      sym_heredoc_body,
    STATE(4194), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3647), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139200] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4060), 1,
      anon_sym_DOT,
    ACTIONS(5676), 1,
      anon_sym_PIPE,
    ACTIONS(5678), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5680), 1,
      anon_sym_AMP,
    ACTIONS(5684), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5686), 1,
      anon_sym_CARET,
    ACTIONS(5688), 1,
      anon_sym_QMARK,
    ACTIONS(5690), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5692), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5696), 1,
      anon_sym_GT_EQ,
    ACTIONS(5702), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5706), 1,
      sym__binary_star_star,
    STATE(4114), 1,
      sym_heredoc_body,
    ACTIONS(5694), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5698), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6113), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(4058), 3,
      sym__element_reference_bracket,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(5682), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5700), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5704), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139280] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4115), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 7,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [139354] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6073), 1,
      anon_sym_COMMA,
    STATE(4065), 1,
      aux_sym_exceptions_repeat1,
    STATE(4116), 1,
      sym_heredoc_body,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139404] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6073), 1,
      anon_sym_COMMA,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    ACTIONS(6115), 1,
      anon_sym_rescue,
    STATE(4065), 1,
      aux_sym_exceptions_repeat1,
    STATE(4117), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3647), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139486] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5836), 1,
      anon_sym_rescue,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4118), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139564] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6011), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6017), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6021), 1,
      anon_sym_QMARK,
    ACTIONS(6023), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6025), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4119), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4102), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [139640] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    ACTIONS(6117), 1,
      anon_sym_rescue,
    STATE(4065), 1,
      aux_sym_exceptions_repeat1,
    STATE(4120), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139720] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4121), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4443), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [139796] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4122), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4441), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [139872] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6119), 1,
      anon_sym_COMMA,
    STATE(4123), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [139920] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4124), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4397), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [139996] = 23,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5424), 1,
      anon_sym_in,
    ACTIONS(5426), 1,
      anon_sym_EQ_GT,
    ACTIONS(5874), 1,
      anon_sym_rescue,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4125), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3651), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140078] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    ACTIONS(6122), 1,
      anon_sym_rescue,
    STATE(4126), 1,
      sym_heredoc_body,
    STATE(4194), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3647), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140158] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6009), 1,
      anon_sym_PIPE,
    ACTIONS(6013), 1,
      anon_sym_AMP,
    ACTIONS(6019), 1,
      anon_sym_CARET,
    ACTIONS(6029), 1,
      anon_sym_GT_EQ,
    ACTIONS(6035), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6039), 1,
      sym__binary_star_star,
    STATE(4127), 1,
      sym_heredoc_body,
    ACTIONS(6027), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6031), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6015), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6033), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6037), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 10,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [140226] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4128), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [140293] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4129), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [140360] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4130), 1,
      sym_heredoc_body,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140411] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6158), 1,
      anon_sym_QMARK,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4131), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4058), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140486] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4132), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [140553] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4133), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 6,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [140626] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4134), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140673] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4135), 1,
      sym_heredoc_body,
    ACTIONS(3449), 12,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_EQ_GT,
    ACTIONS(3451), 19,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_DOT_DOT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
      sym_identifier,
  [140718] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4136), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4058), 5,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140793] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    ACTIONS(6122), 1,
      anon_sym_rescue,
    STATE(4137), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140870] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4138), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [140945] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4139), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4397), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141020] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4140), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141095] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4141), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 23,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141142] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4142), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(4441), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141219] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4143), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4058), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141294] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(212), 1,
      sym__terminator,
    STATE(4144), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6188), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141373] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4145), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(4102), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141450] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4146), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 6,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [141523] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4147), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4350), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141602] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4148), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 21,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141653] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4149), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3627), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141732] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5750), 1,
      anon_sym_in,
    ACTIONS(5752), 1,
      anon_sym_EQ_GT,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4150), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3627), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141811] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4151), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4443), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141886] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4152), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4441), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [141961] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4153), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142008] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6190), 1,
      anon_sym_COMMA,
    STATE(4154), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3745), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3743), 22,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142055] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4155), 1,
      sym_heredoc_body,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__line_break,
      sym__binary_minus,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142104] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4156), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142151] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4157), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142198] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4158), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142253] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4159), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142306] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4160), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142365] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4161), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [142434] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4162), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [142501] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4163), 1,
      sym_heredoc_body,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142552] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4164), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [142619] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4165), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 23,
      sym__line_break,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142666] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4166), 1,
      sym_heredoc_body,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__binary_minus,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142715] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4167), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142762] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4168), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142809] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4169), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142864] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4170), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 6,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [142937] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4171), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [142990] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4172), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [143059] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6158), 1,
      anon_sym_QMARK,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4173), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143134] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6158), 1,
      anon_sym_QMARK,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4174), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143209] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4175), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143256] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4176), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 6,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
  [143329] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6158), 1,
      anon_sym_QMARK,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4177), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4046), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143404] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4178), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4054), 6,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
  [143477] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4179), 1,
      sym_heredoc_body,
    STATE(4194), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 23,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143524] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6158), 1,
      anon_sym_QMARK,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4180), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4050), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143599] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4181), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3743), 5,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143674] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6079), 1,
      anon_sym_COMMA,
    STATE(4182), 1,
      sym_heredoc_body,
    STATE(4194), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(3649), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3647), 22,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143723] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4183), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4395), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143798] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4184), 1,
      sym_heredoc_body,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 20,
      sym__binary_minus,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143847] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    ACTIONS(6117), 1,
      anon_sym_rescue,
    STATE(4185), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3651), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143924] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4186), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4401), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [143999] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4187), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144074] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4188), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144149] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4189), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5914), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6193), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144226] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5838), 1,
      anon_sym_PIPE,
    ACTIONS(5840), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5842), 1,
      anon_sym_AMP,
    ACTIONS(5846), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5848), 1,
      anon_sym_CARET,
    ACTIONS(5850), 1,
      anon_sym_QMARK,
    ACTIONS(5852), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5854), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5858), 1,
      anon_sym_GT_EQ,
    ACTIONS(5864), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5868), 1,
      sym__binary_star_star,
    STATE(4190), 1,
      sym_heredoc_body,
    ACTIONS(5856), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5860), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5914), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5844), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5862), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6195), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5866), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144303] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(4191), 1,
      sym_heredoc_body,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4397), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144378] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4192), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144425] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4193), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144472] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6079), 1,
      anon_sym_COMMA,
    STATE(4154), 1,
      aux_sym_exceptions_repeat1,
    STATE(4194), 1,
      sym_heredoc_body,
    ACTIONS(4023), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4021), 22,
      sym__binary_minus,
      sym__binary_star,
      sym__binary_star_star,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144521] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6146), 1,
      anon_sym_PIPE,
    ACTIONS(6148), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6150), 1,
      anon_sym_AMP,
    ACTIONS(6154), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6156), 1,
      anon_sym_CARET,
    ACTIONS(6158), 1,
      anon_sym_QMARK,
    ACTIONS(6160), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6162), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6166), 1,
      anon_sym_GT_EQ,
    ACTIONS(6172), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6176), 1,
      sym__binary_star_star,
    STATE(4195), 1,
      sym_heredoc_body,
    ACTIONS(6164), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6168), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6199), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6152), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6170), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6197), 3,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_do,
    ACTIONS(6174), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144598] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4196), 1,
      sym_heredoc_body,
    ACTIONS(3549), 12,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      anon_sym_EQ_GT,
    ACTIONS(3551), 19,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_EQ,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_DOT_DOT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
      sym_identifier,
  [144643] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4197), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4042), 6,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
  [144716] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(4198), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6201), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6193), 3,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_do,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144793] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5652), 1,
      anon_sym_PIPE,
    ACTIONS(5654), 1,
      anon_sym_AMP,
    ACTIONS(5658), 1,
      anon_sym_CARET,
    ACTIONS(5660), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5664), 1,
      anon_sym_GT_EQ,
    ACTIONS(5670), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5674), 1,
      sym__binary_star_star,
    ACTIONS(5779), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5781), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5783), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5787), 1,
      anon_sym_QMARK,
    STATE(4199), 1,
      sym_heredoc_body,
    ACTIONS(5662), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5666), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6201), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5656), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5668), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6195), 3,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_do,
    ACTIONS(5672), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144870] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4200), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144925] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4201), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [144978] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4202), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145037] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4203), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [145106] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4204), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 9,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [145173] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4205), 1,
      sym_heredoc_body,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145224] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4206), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4046), 5,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145299] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(243), 1,
      sym__terminator,
    STATE(4207), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6188), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145378] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4208), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 23,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145425] = 22,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5960), 1,
      sym__binary_star_star,
    ACTIONS(5962), 1,
      anon_sym_AMP,
    ACTIONS(5972), 1,
      anon_sym_PIPE,
    ACTIONS(5974), 1,
      anon_sym_CARET,
    ACTIONS(5978), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5980), 1,
      anon_sym_GT_EQ,
    ACTIONS(5982), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5986), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5988), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5990), 1,
      anon_sym_QMARK,
    ACTIONS(5992), 1,
      anon_sym_PIPE_PIPE,
    STATE(239), 1,
      sym__terminator,
    STATE(4209), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    ACTIONS(5964), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5966), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6188), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5968), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5976), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5984), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145504] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4210), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4102), 5,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145579] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6081), 1,
      anon_sym_PIPE,
    ACTIONS(6083), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6085), 1,
      anon_sym_AMP,
    ACTIONS(6089), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6091), 1,
      anon_sym_CARET,
    ACTIONS(6093), 1,
      anon_sym_QMARK,
    ACTIONS(6095), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6097), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6101), 1,
      anon_sym_GT_EQ,
    ACTIONS(6107), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6111), 1,
      sym__binary_star_star,
    STATE(4211), 1,
      sym_heredoc_body,
    ACTIONS(6099), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6103), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6087), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6105), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4050), 5,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6109), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145654] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4212), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4050), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145729] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4213), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4046), 5,
      anon_sym_RPAREN,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145804] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    STATE(4214), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      anon_sym_RPAREN,
      anon_sym_DOT_DOT_DOT,
      anon_sym_in,
      anon_sym_EQ_GT,
      anon_sym_QMARK,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145863] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    STATE(4215), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145909] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4216), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4054), 5,
      anon_sym_COMMA,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [145981] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4217), 1,
      sym_heredoc_body,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146031] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    STATE(4218), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146083] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6115), 1,
      anon_sym_rescue,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6253), 1,
      anon_sym_QMARK,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4219), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3651), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146159] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    ACTIONS(6265), 1,
      anon_sym_QMARK,
    STATE(4220), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146233] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    ACTIONS(6265), 1,
      anon_sym_QMARK,
    STATE(4221), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4102), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146307] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    ACTIONS(6267), 1,
      anon_sym_PIPE,
    STATE(4222), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6193), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146383] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    ACTIONS(6269), 1,
      anon_sym_PIPE,
    STATE(4223), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6195), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146459] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    STATE(4224), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 14,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146517] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6253), 1,
      anon_sym_QMARK,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4225), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(3627), 4,
      anon_sym_rescue,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146591] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4226), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 14,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146649] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3276), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3280), 1,
      anon_sym_ri,
    STATE(4227), 1,
      sym_heredoc_body,
    ACTIONS(3278), 2,
      anon_sym_i,
      anon_sym_r,
    ACTIONS(3274), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [146697] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4228), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 7,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [146765] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4229), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146811] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    STATE(4230), 1,
      sym_heredoc_body,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__binary_minus,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146859] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6253), 1,
      anon_sym_QMARK,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4231), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(3627), 4,
      anon_sym_rescue,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146933] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    STATE(4232), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [146987] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4233), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4042), 5,
      anon_sym_rescue,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147059] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4234), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [147125] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    STATE(4235), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147171] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4236), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147217] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6253), 1,
      anon_sym_QMARK,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4237), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4058), 4,
      anon_sym_rescue,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147291] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4238), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 7,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [147359] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    STATE(4239), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147405] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    ACTIONS(6265), 1,
      anon_sym_QMARK,
    STATE(4240), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4050), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147479] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4241), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4054), 5,
      anon_sym_rescue,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147551] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4242), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4042), 5,
      anon_sym_COMMA,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147623] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6253), 1,
      anon_sym_QMARK,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4243), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4046), 4,
      anon_sym_rescue,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147697] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    ACTIONS(6265), 1,
      anon_sym_QMARK,
    STATE(4244), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4058), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147771] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    ACTIONS(6265), 1,
      anon_sym_QMARK,
    STATE(4245), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4046), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147845] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4246), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6193), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147921] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5740), 1,
      anon_sym_AMP,
    ACTIONS(5748), 1,
      sym__binary_star_star,
    ACTIONS(5757), 1,
      anon_sym_PIPE,
    ACTIONS(5759), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(5763), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5765), 1,
      anon_sym_CARET,
    ACTIONS(5767), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(5769), 1,
      anon_sym_AMP_AMP,
    ACTIONS(5771), 1,
      anon_sym_GT_EQ,
    ACTIONS(5773), 1,
      anon_sym_EQ_EQ,
    ACTIONS(5777), 1,
      anon_sym_QMARK,
    STATE(4247), 1,
      sym_heredoc_body,
    ACTIONS(5742), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(5744), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6007), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6195), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
    ACTIONS(5746), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(5761), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(5775), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [147997] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6077), 1,
      anon_sym_rescue,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6253), 1,
      anon_sym_QMARK,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4248), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3651), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148073] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    ACTIONS(6265), 1,
      anon_sym_QMARK,
    STATE(4249), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3743), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148147] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4250), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [148213] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4251), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [148279] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    STATE(4252), 1,
      sym_heredoc_body,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 17,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148329] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6205), 1,
      anon_sym_PIPE,
    ACTIONS(6207), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6213), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6215), 1,
      anon_sym_CARET,
    ACTIONS(6217), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6219), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6223), 1,
      anon_sym_GT_EQ,
    ACTIONS(6229), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    ACTIONS(6265), 1,
      anon_sym_QMARK,
    STATE(4253), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6211), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3627), 4,
      anon_sym_COMMA,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6231), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148403] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4254), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148455] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4255), 1,
      sym_heredoc_body,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 19,
      sym__binary_minus,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148503] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4256), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148549] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6041), 1,
      sym__binary_star_star,
    ACTIONS(6043), 1,
      anon_sym_PIPE,
    ACTIONS(6045), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6047), 1,
      anon_sym_AMP,
    ACTIONS(6051), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6053), 1,
      anon_sym_CARET,
    ACTIONS(6055), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6057), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6061), 1,
      anon_sym_GT_EQ,
    ACTIONS(6067), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6071), 1,
      anon_sym_QMARK,
    STATE(4257), 1,
      sym_heredoc_body,
    ACTIONS(6059), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6063), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6049), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6065), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4397), 4,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6069), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148623] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4258), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148669] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6209), 1,
      anon_sym_AMP,
    ACTIONS(6233), 1,
      sym__binary_star_star,
    STATE(4259), 1,
      sym_heredoc_body,
    ACTIONS(6221), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6225), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6227), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 15,
      anon_sym_COMMA,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148723] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6243), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6249), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6253), 1,
      anon_sym_QMARK,
    ACTIONS(6255), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6257), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4260), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4050), 4,
      anon_sym_rescue,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148797] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    ACTIONS(6241), 1,
      anon_sym_PIPE,
    ACTIONS(6245), 1,
      anon_sym_AMP,
    ACTIONS(6251), 1,
      anon_sym_CARET,
    ACTIONS(6259), 1,
      anon_sym_GT_EQ,
    ACTIONS(6261), 1,
      anon_sym_EQ_EQ,
    STATE(4261), 1,
      sym_heredoc_body,
    ACTIONS(6235), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6237), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6239), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6247), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6263), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 8,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [148863] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6203), 1,
      sym__binary_star_star,
    STATE(4262), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 22,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_rescue,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148909] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4263), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 5,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 13,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [148966] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6293), 1,
      anon_sym_COLON2,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4264), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149041] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6307), 1,
      anon_sym_COLON2,
    STATE(4265), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149116] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6309), 1,
      anon_sym_COLON2,
    STATE(4266), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149191] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6311), 1,
      anon_sym_COLON2,
    STATE(4267), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149266] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6313), 1,
      anon_sym_COLON2,
    STATE(4268), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149341] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6315), 1,
      anon_sym_RPAREN,
    STATE(4269), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149416] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6319), 1,
      anon_sym_COLON2,
    STATE(4270), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149491] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6321), 1,
      anon_sym_COLON2,
    STATE(4271), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149566] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6323), 1,
      anon_sym_COLON2,
    STATE(4272), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149641] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6325), 1,
      anon_sym_COLON2,
    STATE(4273), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149716] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6327), 1,
      anon_sym_COLON2,
    STATE(4274), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149791] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6329), 1,
      anon_sym_COLON2,
    STATE(4275), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149866] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6331), 1,
      anon_sym_COLON2,
    STATE(4276), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [149941] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6333), 1,
      anon_sym_COLON2,
    STATE(4277), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150016] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4278), 1,
      sym_heredoc_body,
    ACTIONS(743), 9,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(6335), 20,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [150059] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6337), 1,
      anon_sym_COLON2,
    STATE(4279), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150134] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    STATE(4280), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4397), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150207] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6339), 1,
      anon_sym_COLON2,
    STATE(4281), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150282] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6341), 1,
      anon_sym_COLON2,
    STATE(4282), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150357] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4283), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3627), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150430] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4284), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(3627), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150503] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6343), 1,
      anon_sym_COLON2,
    STATE(4285), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150578] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6345), 1,
      anon_sym_COLON2,
    STATE(4286), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150653] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6347), 1,
      anon_sym_COLON2,
    STATE(4287), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150728] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6349), 1,
      anon_sym_COLON2,
    STATE(4288), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150803] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4289), 1,
      sym_heredoc_body,
    ACTIONS(3629), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(3627), 21,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150848] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4290), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4054), 4,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150919] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6351), 1,
      anon_sym_COLON2,
    STATE(4291), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [150994] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6353), 1,
      anon_sym_COLON2,
    STATE(4292), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151069] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6355), 1,
      anon_sym_COLON2,
    STATE(4293), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151144] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6357), 1,
      anon_sym_COLON2,
    STATE(4294), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151219] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6359), 1,
      anon_sym_COLON2,
    STATE(4295), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151294] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6361), 1,
      anon_sym_COLON2,
    STATE(4296), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151369] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6363), 1,
      anon_sym_COLON2,
    STATE(4297), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151444] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6365), 1,
      anon_sym_COLON2,
    STATE(4298), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151519] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6367), 1,
      anon_sym_COLON2,
    STATE(4299), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151594] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6369), 1,
      anon_sym_COLON2,
    STATE(4300), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151669] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6371), 1,
      anon_sym_COLON2,
    STATE(4301), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151744] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6373), 1,
      anon_sym_COLON2,
    STATE(4302), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151819] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6375), 1,
      anon_sym_COLON2,
    STATE(4303), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151894] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6377), 1,
      anon_sym_RPAREN,
    STATE(4304), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [151969] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6379), 1,
      anon_sym_COLON2,
    STATE(4305), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152044] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6381), 1,
      anon_sym_COLON2,
    STATE(4306), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152119] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6383), 1,
      anon_sym_EQ_GT,
    STATE(4307), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152194] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6385), 1,
      anon_sym_RPAREN,
    STATE(4308), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152269] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6387), 1,
      anon_sym_COLON2,
    STATE(4309), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152344] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1705), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4310), 1,
      sym_heredoc_body,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4497), 1,
      sym__hash_pattern_any_rest,
    STATE(5723), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6389), 2,
      sym_constant,
      sym_identifier,
    STATE(4483), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5862), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5469), 5,
      sym__line_break,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5471), 9,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [152407] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6391), 1,
      anon_sym_COLON2,
    STATE(4311), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152482] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4312), 1,
      sym_heredoc_body,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 18,
      sym__binary_minus,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152529] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6393), 1,
      anon_sym_COLON2,
    STATE(4313), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152604] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4314), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152649] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4315), 1,
      sym_heredoc_body,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 21,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152694] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4316), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 6,
      anon_sym_PIPE,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 14,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152747] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4317), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(4042), 4,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152818] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4318), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 14,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152869] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4319), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4046), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [152942] = 17,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4320), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 6,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
  [153009] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4321), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 7,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [153074] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4322), 1,
      sym_heredoc_body,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(4048), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4046), 16,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153123] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4048), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4323), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
    ACTIONS(4046), 7,
      anon_sym_DOT_DOT_DOT,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [153188] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6395), 1,
      anon_sym_COLON2,
    STATE(4324), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153263] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    STATE(4325), 1,
      sym_heredoc_body,
    ACTIONS(4098), 7,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_LT,
      anon_sym_DOT_DOT,
      anon_sym_LT_EQ,
      anon_sym_GT,
      anon_sym_EQ_EQ,
    ACTIONS(4096), 21,
      sym__binary_minus,
      sym__binary_star,
      anon_sym_DOT_DOT_DOT,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_GT_EQ,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153308] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4326), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4050), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153381] = 20,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    STATE(4327), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(4058), 3,
      anon_sym_COLON2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153454] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6397), 1,
      anon_sym_COLON2,
    STATE(4328), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153529] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6399), 1,
      anon_sym_COLON2,
    STATE(4329), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153604] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6401), 1,
      anon_sym_COLON2,
    STATE(4330), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153679] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6403), 1,
      anon_sym_RPAREN,
    STATE(4331), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153754] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1705), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4332), 1,
      sym_heredoc_body,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4519), 1,
      sym__hash_pattern_any_rest,
    STATE(5723), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6389), 2,
      sym_constant,
      sym_identifier,
    STATE(4483), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5862), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5475), 5,
      sym__line_break,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5477), 9,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [153817] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6405), 1,
      anon_sym_RPAREN,
    STATE(4333), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153892] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6407), 1,
      anon_sym_RPAREN,
    STATE(4334), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [153967] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6409), 1,
      anon_sym_RPAREN,
    STATE(4335), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154042] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6411), 1,
      anon_sym_RPAREN,
    STATE(4336), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154117] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6413), 1,
      anon_sym_RPAREN,
    STATE(4337), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154192] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6415), 1,
      anon_sym_COLON2,
    STATE(4338), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154267] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6417), 1,
      anon_sym_RPAREN,
    STATE(4339), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154342] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6419), 1,
      anon_sym_RPAREN,
    STATE(4340), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154417] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6421), 1,
      anon_sym_RPAREN,
    STATE(4341), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154492] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6423), 1,
      anon_sym_RPAREN,
    STATE(4342), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154567] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6425), 1,
      anon_sym_RPAREN,
    STATE(4343), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154642] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6427), 1,
      anon_sym_COLON2,
    STATE(4344), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154717] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6429), 1,
      anon_sym_COLON2,
    STATE(4345), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154792] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6431), 1,
      anon_sym_RPAREN,
    STATE(4346), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154867] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6433), 1,
      anon_sym_RPAREN,
    STATE(4347), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [154942] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6435), 1,
      anon_sym_RPAREN,
    STATE(4348), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155017] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6437), 1,
      anon_sym_RPAREN,
    STATE(4349), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155092] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6439), 1,
      anon_sym_RPAREN,
    STATE(4350), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155167] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6441), 1,
      anon_sym_RPAREN,
    STATE(4351), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155242] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6443), 1,
      anon_sym_RPAREN,
    STATE(4352), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155317] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6445), 1,
      anon_sym_RPAREN,
    STATE(4353), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155392] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6447), 1,
      anon_sym_RPAREN,
    STATE(4354), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155467] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6124), 1,
      anon_sym_PIPE,
    ACTIONS(6126), 1,
      anon_sym_AMP,
    ACTIONS(6130), 1,
      anon_sym_CARET,
    ACTIONS(6134), 1,
      anon_sym_GT_EQ,
    ACTIONS(6140), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6144), 1,
      sym__binary_star_star,
    ACTIONS(6178), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6180), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6182), 1,
      anon_sym_QMARK,
    ACTIONS(6184), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6186), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6449), 1,
      anon_sym_RPAREN,
    STATE(4355), 1,
      sym_heredoc_body,
    ACTIONS(6132), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6136), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6317), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6128), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6138), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6142), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155542] = 21,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6271), 1,
      anon_sym_PIPE,
    ACTIONS(6273), 1,
      anon_sym_AMP,
    ACTIONS(6275), 1,
      anon_sym_CARET,
    ACTIONS(6283), 1,
      sym__binary_star_star,
    ACTIONS(6285), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6289), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6291), 1,
      anon_sym_QMARK,
    ACTIONS(6297), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(6299), 1,
      anon_sym_AMP_AMP,
    ACTIONS(6301), 1,
      anon_sym_GT_EQ,
    ACTIONS(6303), 1,
      anon_sym_EQ_EQ,
    ACTIONS(6451), 1,
      anon_sym_COLON2,
    STATE(4356), 1,
      sym_heredoc_body,
    ACTIONS(6277), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(6279), 2,
      sym__binary_minus,
      anon_sym_PLUS,
    ACTIONS(6295), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6281), 3,
      sym__binary_star,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(6287), 3,
      anon_sym_LT,
      anon_sym_LT_EQ,
      anon_sym_GT,
    ACTIONS(6305), 5,
      anon_sym_BANG_EQ,
      anon_sym_EQ_EQ_EQ,
      anon_sym_LT_EQ_GT,
      anon_sym_EQ_TILDE,
      anon_sym_BANG_TILDE,
  [155617] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4357), 1,
      sym_heredoc_body,
    ACTIONS(3086), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [155657] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4358), 1,
      sym_heredoc_body,
    ACTIONS(3052), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [155697] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4359), 1,
      sym_heredoc_body,
    ACTIONS(3056), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [155737] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4360), 1,
      sym_heredoc_body,
    ACTIONS(6453), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [155777] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4361), 1,
      sym_heredoc_body,
    ACTIONS(3064), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [155817] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOT_DOT,
    STATE(4362), 1,
      sym_heredoc_body,
    ACTIONS(3319), 27,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [155859] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4363), 1,
      sym_heredoc_body,
    ACTIONS(3000), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [155899] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3305), 1,
      anon_sym_DOT_DOT,
    STATE(4364), 1,
      sym_heredoc_body,
    ACTIONS(3303), 27,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [155941] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4365), 1,
      sym_heredoc_body,
    ACTIONS(3016), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [155981] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4366), 1,
      sym_heredoc_body,
    ACTIONS(3020), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156021] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4367), 1,
      sym_heredoc_body,
    ACTIONS(3024), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156061] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4368), 1,
      sym_heredoc_body,
    ACTIONS(2740), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156101] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4369), 1,
      sym_heredoc_body,
    ACTIONS(3028), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156141] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4370), 1,
      sym_heredoc_body,
    ACTIONS(3032), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156181] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4371), 1,
      sym_heredoc_body,
    ACTIONS(3036), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156221] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4372), 1,
      sym_heredoc_body,
    ACTIONS(3040), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156261] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4373), 1,
      sym_heredoc_body,
    ACTIONS(3044), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156301] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4374), 1,
      sym_heredoc_body,
    ACTIONS(3048), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156341] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6457), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6459), 1,
      anon_sym_LBRACK2,
    ACTIONS(6461), 1,
      anon_sym_LPAREN2,
    STATE(4375), 1,
      sym_heredoc_body,
    ACTIONS(6455), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156387] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4376), 1,
      sym_heredoc_body,
    ACTIONS(6463), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156427] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4377), 1,
      sym_heredoc_body,
    ACTIONS(3008), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156467] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4378), 1,
      sym_heredoc_body,
    ACTIONS(3012), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [156507] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4379), 1,
      sym_heredoc_body,
    ACTIONS(2976), 28,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156547] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3583), 1,
      anon_sym_DOT_DOT,
    STATE(4380), 1,
      sym_heredoc_body,
    ACTIONS(3581), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156588] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6465), 1,
      anon_sym_DOT_DOT,
    STATE(4381), 1,
      sym_heredoc_body,
    ACTIONS(2998), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156629] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3026), 1,
      anon_sym_DOT_DOT,
    STATE(4382), 1,
      sym_heredoc_body,
    ACTIONS(3024), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156670] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3375), 1,
      anon_sym_DOT_DOT,
    STATE(4383), 1,
      sym_heredoc_body,
    ACTIONS(3373), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156711] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3555), 1,
      anon_sym_DOT_DOT,
    STATE(4384), 1,
      sym_heredoc_body,
    ACTIONS(3553), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156752] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3559), 1,
      anon_sym_DOT_DOT,
    STATE(4385), 1,
      sym_heredoc_body,
    ACTIONS(3557), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156793] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3030), 1,
      anon_sym_DOT_DOT,
    STATE(4386), 1,
      sym_heredoc_body,
    ACTIONS(3028), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156834] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3276), 1,
      anon_sym_DOT_DOT,
    STATE(4387), 1,
      sym_heredoc_body,
    ACTIONS(3274), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156875] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3563), 1,
      anon_sym_DOT_DOT,
    STATE(4388), 1,
      sym_heredoc_body,
    ACTIONS(3561), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156916] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3403), 1,
      anon_sym_DOT_DOT,
    STATE(4389), 1,
      sym_heredoc_body,
    ACTIONS(3401), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156957] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3407), 1,
      anon_sym_DOT_DOT,
    STATE(4390), 1,
      sym_heredoc_body,
    ACTIONS(3405), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [156998] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3411), 1,
      anon_sym_DOT_DOT,
    STATE(4391), 1,
      sym_heredoc_body,
    ACTIONS(3409), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157039] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3034), 1,
      anon_sym_DOT_DOT,
    STATE(4392), 1,
      sym_heredoc_body,
    ACTIONS(3032), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157080] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3415), 1,
      anon_sym_DOT_DOT,
    STATE(4393), 1,
      sym_heredoc_body,
    ACTIONS(3413), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157121] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3371), 1,
      anon_sym_DOT_DOT,
    STATE(4394), 1,
      sym_heredoc_body,
    ACTIONS(3369), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157162] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3018), 1,
      anon_sym_DOT_DOT,
    STATE(4395), 1,
      sym_heredoc_body,
    ACTIONS(3016), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157203] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6467), 1,
      anon_sym_DOT_DOT,
    STATE(4396), 1,
      sym_heredoc_body,
    ACTIONS(3286), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157244] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3038), 1,
      anon_sym_DOT_DOT,
    STATE(4397), 1,
      sym_heredoc_body,
    ACTIONS(3036), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157285] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3042), 1,
      anon_sym_DOT_DOT,
    STATE(4398), 1,
      sym_heredoc_body,
    ACTIONS(3040), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157326] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3046), 1,
      anon_sym_DOT_DOT,
    STATE(4399), 1,
      sym_heredoc_body,
    ACTIONS(3044), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157367] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3050), 1,
      anon_sym_DOT_DOT,
    STATE(4400), 1,
      sym_heredoc_body,
    ACTIONS(3048), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157408] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3002), 1,
      anon_sym_DOT_DOT,
    STATE(4401), 1,
      sym_heredoc_body,
    ACTIONS(3000), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157449] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3733), 1,
      anon_sym_DOT_DOT,
    STATE(4402), 1,
      sym_heredoc_body,
    ACTIONS(3731), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157490] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3737), 1,
      anon_sym_DOT_DOT,
    STATE(4403), 1,
      sym_heredoc_body,
    ACTIONS(3735), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157531] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3595), 1,
      anon_sym_DOT_DOT,
    STATE(4404), 1,
      sym_heredoc_body,
    ACTIONS(3593), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157572] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3022), 1,
      anon_sym_DOT_DOT,
    STATE(4405), 1,
      sym_heredoc_body,
    ACTIONS(3020), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157613] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3713), 1,
      anon_sym_DOT_DOT,
    STATE(4406), 1,
      sym_heredoc_body,
    ACTIONS(3711), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157654] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3477), 1,
      anon_sym_DOT_DOT,
    STATE(4407), 1,
      sym_heredoc_body,
    ACTIONS(3475), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157695] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6471), 1,
      anon_sym_DOT_DOT,
    STATE(4408), 1,
      sym_heredoc_body,
    ACTIONS(6469), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157736] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_DOT_DOT,
    STATE(4409), 1,
      sym_heredoc_body,
    ACTIONS(3060), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157777] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2992), 1,
      anon_sym_COLON,
    STATE(4410), 1,
      sym_heredoc_body,
    ACTIONS(2740), 26,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157818] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6473), 1,
      anon_sym_DOT_DOT,
    STATE(4411), 1,
      sym_heredoc_body,
    ACTIONS(3625), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157859] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3543), 1,
      anon_sym_DOT_DOT,
    STATE(4412), 1,
      sym_heredoc_body,
    ACTIONS(3541), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157900] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3607), 1,
      anon_sym_DOT_DOT,
    STATE(4413), 1,
      sym_heredoc_body,
    ACTIONS(3605), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157941] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3615), 1,
      anon_sym_DOT_DOT,
    STATE(4414), 1,
      sym_heredoc_body,
    ACTIONS(3613), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [157982] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3010), 1,
      anon_sym_DOT_DOT,
    STATE(4415), 1,
      sym_heredoc_body,
    ACTIONS(3008), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158023] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3014), 1,
      anon_sym_DOT_DOT,
    STATE(4416), 1,
      sym_heredoc_body,
    ACTIONS(3012), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158064] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3511), 1,
      anon_sym_DOT_DOT,
    STATE(4417), 1,
      sym_heredoc_body,
    ACTIONS(3509), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158105] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3515), 1,
      anon_sym_DOT_DOT,
    STATE(4418), 1,
      sym_heredoc_body,
    ACTIONS(3513), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158146] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3969), 1,
      anon_sym_DOT_DOT,
    STATE(4419), 1,
      sym_heredoc_body,
    ACTIONS(3967), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158187] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3909), 1,
      anon_sym_DOT_DOT,
    STATE(4420), 1,
      sym_heredoc_body,
    ACTIONS(3907), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158228] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3947), 1,
      anon_sym_DOT_DOT,
    STATE(4421), 1,
      sym_heredoc_body,
    ACTIONS(3945), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158269] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3951), 1,
      anon_sym_DOT_DOT,
    STATE(4422), 1,
      sym_heredoc_body,
    ACTIONS(3949), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158310] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3955), 1,
      anon_sym_DOT_DOT,
    STATE(4423), 1,
      sym_heredoc_body,
    ACTIONS(3953), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158351] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3383), 1,
      anon_sym_DOT_DOT,
    STATE(4424), 1,
      sym_heredoc_body,
    ACTIONS(3381), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158392] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3961), 1,
      anon_sym_DOT_DOT,
    STATE(4425), 1,
      sym_heredoc_body,
    ACTIONS(3959), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158433] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3965), 1,
      anon_sym_DOT_DOT,
    STATE(4426), 1,
      sym_heredoc_body,
    ACTIONS(3963), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158474] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3567), 1,
      anon_sym_DOT_DOT,
    STATE(4427), 1,
      sym_heredoc_body,
    ACTIONS(3565), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158515] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3088), 1,
      anon_sym_DOT_DOT,
    STATE(4428), 1,
      sym_heredoc_body,
    ACTIONS(3086), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158556] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3054), 1,
      anon_sym_DOT_DOT,
    STATE(4429), 1,
      sym_heredoc_body,
    ACTIONS(3052), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158597] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3058), 1,
      anon_sym_DOT_DOT,
    STATE(4430), 1,
      sym_heredoc_body,
    ACTIONS(3056), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158638] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3066), 1,
      anon_sym_DOT_DOT,
    STATE(4431), 1,
      sym_heredoc_body,
    ACTIONS(3064), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158679] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3527), 1,
      anon_sym_DOT_DOT,
    STATE(4432), 1,
      sym_heredoc_body,
    ACTIONS(3525), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158720] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4433), 1,
      sym_heredoc_body,
    ACTIONS(6475), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [158758] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6467), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6477), 1,
      anon_sym_COLON,
    STATE(4434), 1,
      sym_heredoc_body,
    ACTIONS(3286), 24,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158800] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4435), 1,
      sym_heredoc_body,
    ACTIONS(6475), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [158838] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4436), 1,
      sym_heredoc_body,
    ACTIONS(6479), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [158876] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4437), 1,
      sym_heredoc_body,
    ACTIONS(6479), 26,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [158914] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4438), 1,
      sym_heredoc_body,
    ACTIONS(6481), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158951] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4439), 1,
      sym_heredoc_body,
    ACTIONS(6483), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [158988] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4440), 1,
      sym_heredoc_body,
    ACTIONS(2628), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
    ACTIONS(2626), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      sym_identifier,
  [159027] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4441), 1,
      sym_heredoc_body,
    ACTIONS(2640), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
    ACTIONS(2638), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      sym_identifier,
  [159066] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6487), 1,
      anon_sym_LBRACE,
    ACTIONS(6489), 1,
      anon_sym_do,
    STATE(4436), 1,
      sym_do_block,
    STATE(4437), 1,
      sym_block,
    STATE(4442), 1,
      sym_heredoc_body,
    ACTIONS(6485), 21,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159111] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4443), 1,
      sym_heredoc_body,
    ACTIONS(4802), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
    ACTIONS(4800), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      sym_identifier,
  [159150] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4444), 1,
      sym_heredoc_body,
    ACTIONS(2644), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
    ACTIONS(2642), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      sym_identifier,
  [159189] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4445), 1,
      sym_heredoc_body,
    ACTIONS(2616), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
    ACTIONS(2614), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      sym_identifier,
  [159228] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4446), 1,
      sym_heredoc_body,
    ACTIONS(3080), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159265] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4447), 1,
      sym_heredoc_body,
    ACTIONS(6491), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159302] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6493), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6495), 1,
      anon_sym_DOT_DOT,
    STATE(4448), 1,
      sym_heredoc_body,
    ACTIONS(6455), 23,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159343] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4449), 1,
      sym_heredoc_body,
    ACTIONS(6497), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159380] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4450), 1,
      sym_heredoc_body,
    ACTIONS(6499), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159417] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4451), 1,
      sym_heredoc_body,
    ACTIONS(6455), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159454] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4452), 1,
      sym_heredoc_body,
    ACTIONS(6501), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159491] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4453), 1,
      sym_heredoc_body,
    ACTIONS(6503), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159528] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4454), 1,
      sym_heredoc_body,
    ACTIONS(6505), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159565] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4455), 1,
      sym_heredoc_body,
    ACTIONS(4973), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
    ACTIONS(4971), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      sym_identifier,
  [159604] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4456), 1,
      sym_heredoc_body,
    ACTIONS(6507), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159641] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4457), 1,
      sym_heredoc_body,
    ACTIONS(6509), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159678] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4458), 1,
      sym_heredoc_body,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4519), 1,
      sym__hash_pattern_any_rest,
    STATE(5657), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6511), 2,
      sym_constant,
      sym_identifier,
    STATE(4483), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5773), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5475), 4,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5477), 6,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [159737] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4459), 1,
      sym_heredoc_body,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4497), 1,
      sym__hash_pattern_any_rest,
    STATE(5657), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6511), 2,
      sym_constant,
      sym_identifier,
    STATE(4483), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5773), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5469), 4,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(5471), 6,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [159796] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4460), 1,
      sym_heredoc_body,
    ACTIONS(2636), 10,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_LPAREN,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
    ACTIONS(2634), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_STAR,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      sym_identifier,
  [159835] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6487), 1,
      anon_sym_LBRACE,
    STATE(4436), 1,
      sym_do_block,
    STATE(4437), 1,
      sym_block,
    STATE(4461), 1,
      sym_heredoc_body,
    ACTIONS(6485), 22,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159878] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4462), 1,
      sym_heredoc_body,
    ACTIONS(3503), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159915] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4463), 1,
      sym_heredoc_body,
    ACTIONS(6513), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159952] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6487), 1,
      anon_sym_LBRACE,
    STATE(4433), 1,
      sym_block,
    STATE(4435), 1,
      sym_do_block,
    STATE(4464), 1,
      sym_heredoc_body,
    ACTIONS(6515), 22,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [159995] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4465), 1,
      sym_heredoc_body,
    ACTIONS(6517), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160032] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6487), 1,
      anon_sym_LBRACE,
    ACTIONS(6489), 1,
      anon_sym_do,
    STATE(4433), 1,
      sym_block,
    STATE(4435), 1,
      sym_do_block,
    STATE(4466), 1,
      sym_heredoc_body,
    ACTIONS(6515), 21,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160077] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4467), 1,
      sym_heredoc_body,
    ACTIONS(6519), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160114] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4468), 1,
      sym_heredoc_body,
    ACTIONS(3485), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160151] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4469), 1,
      sym_heredoc_body,
    ACTIONS(6521), 25,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160188] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6525), 1,
      anon_sym_PIPE,
    STATE(4470), 1,
      sym_heredoc_body,
    STATE(4475), 1,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6523), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160228] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4471), 1,
      sym_heredoc_body,
    ACTIONS(6529), 3,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(6527), 21,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160266] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4472), 1,
      sym_heredoc_body,
    ACTIONS(6531), 24,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160302] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6533), 1,
      anon_sym_COLON,
    STATE(4473), 1,
      sym_heredoc_body,
    ACTIONS(3080), 23,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160340] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6525), 1,
      anon_sym_PIPE,
    STATE(4470), 1,
      aux_sym_alternative_pattern_repeat1,
    STATE(4474), 1,
      sym_heredoc_body,
    ACTIONS(6535), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160380] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6539), 1,
      anon_sym_PIPE,
    STATE(4475), 2,
      sym_heredoc_body,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6537), 22,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160418] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4476), 1,
      sym_heredoc_body,
    ACTIONS(6535), 24,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160454] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    STATE(4477), 1,
      sym_heredoc_body,
    ACTIONS(6542), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160492] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    STATE(4478), 1,
      sym_heredoc_body,
    ACTIONS(6546), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160530] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    STATE(4479), 1,
      sym_heredoc_body,
    ACTIONS(6548), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160568] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4480), 1,
      sym_heredoc_body,
    ACTIONS(6550), 24,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_EQ_GT,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160604] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4481), 1,
      sym_heredoc_body,
    ACTIONS(6552), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160639] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4482), 1,
      sym_heredoc_body,
    ACTIONS(6554), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160674] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4483), 1,
      sym_heredoc_body,
    ACTIONS(6556), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160709] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4484), 1,
      sym_heredoc_body,
    ACTIONS(6558), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160744] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4485), 1,
      sym_heredoc_body,
    ACTIONS(6560), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160779] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4486), 1,
      sym_heredoc_body,
    ACTIONS(6562), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160814] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4487), 1,
      sym_heredoc_body,
    ACTIONS(6564), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160849] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4488), 1,
      sym_heredoc_body,
    ACTIONS(6566), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160884] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4489), 1,
      sym_heredoc_body,
    ACTIONS(6568), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160919] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4490), 1,
      sym_heredoc_body,
    ACTIONS(6570), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160954] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4491), 1,
      sym_heredoc_body,
    ACTIONS(6572), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [160989] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4492), 1,
      sym_heredoc_body,
    ACTIONS(6574), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161024] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4493), 1,
      sym_heredoc_body,
    ACTIONS(6576), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161059] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4494), 1,
      sym_heredoc_body,
    ACTIONS(6578), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161094] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4495), 1,
      sym_heredoc_body,
    STATE(4905), 1,
      sym__hash_pattern_any_rest,
    STATE(4926), 1,
      sym_keyword_pattern,
    STATE(5813), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6580), 2,
      sym_constant,
      sym_identifier,
    STATE(4899), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5892), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5477), 3,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5475), 5,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [161151] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4496), 1,
      sym_heredoc_body,
    ACTIONS(6582), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161186] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4497), 1,
      sym_heredoc_body,
    ACTIONS(6584), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161221] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4498), 1,
      sym_heredoc_body,
    ACTIONS(6558), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161256] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4499), 1,
      sym_heredoc_body,
    ACTIONS(6586), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161291] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4500), 1,
      sym_heredoc_body,
    ACTIONS(6485), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161326] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4501), 1,
      sym_heredoc_body,
    ACTIONS(6558), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161361] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4502), 1,
      sym_heredoc_body,
    ACTIONS(6588), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161396] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4503), 1,
      sym_heredoc_body,
    ACTIONS(6335), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161431] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4504), 1,
      sym_heredoc_body,
    STATE(4905), 1,
      sym__hash_pattern_any_rest,
    STATE(4926), 1,
      sym_keyword_pattern,
    STATE(5942), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6590), 2,
      sym_constant,
      sym_identifier,
    STATE(4899), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5839), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5475), 4,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(5477), 4,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [161488] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4505), 1,
      sym_heredoc_body,
    STATE(4902), 1,
      sym__hash_pattern_any_rest,
    STATE(4926), 1,
      sym_keyword_pattern,
    STATE(5942), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6590), 2,
      sym_constant,
      sym_identifier,
    STATE(4899), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5839), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5469), 4,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
    ACTIONS(5471), 4,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [161545] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4506), 1,
      sym_heredoc_body,
    ACTIONS(6592), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161580] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4507), 1,
      sym_heredoc_body,
    ACTIONS(6558), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161615] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4508), 1,
      sym_heredoc_body,
    ACTIONS(6594), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161650] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4509), 1,
      sym_heredoc_body,
    ACTIONS(4350), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161685] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4510), 1,
      sym_heredoc_body,
    ACTIONS(6596), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161720] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4511), 1,
      sym_heredoc_body,
    ACTIONS(6598), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161755] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4512), 1,
      sym_heredoc_body,
    ACTIONS(6554), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161790] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4513), 1,
      sym_heredoc_body,
    ACTIONS(6600), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161825] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4514), 1,
      sym_heredoc_body,
    STATE(4902), 1,
      sym__hash_pattern_any_rest,
    STATE(4926), 1,
      sym_keyword_pattern,
    STATE(5813), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6580), 2,
      sym_constant,
      sym_identifier,
    STATE(4899), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5892), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
    ACTIONS(5471), 3,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(5469), 5,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [161882] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4515), 1,
      sym_heredoc_body,
    ACTIONS(6527), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161917] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4516), 1,
      sym_heredoc_body,
    ACTIONS(6602), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161952] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6604), 1,
      sym_identifier,
    ACTIONS(6610), 1,
      anon_sym_nil,
    STATE(4517), 1,
      sym_heredoc_body,
    ACTIONS(6606), 6,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(6608), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [161993] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4518), 1,
      sym_heredoc_body,
    ACTIONS(6612), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162028] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4519), 1,
      sym_heredoc_body,
    ACTIONS(5469), 23,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162063] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6614), 1,
      anon_sym_COMMA,
    STATE(4520), 2,
      sym_heredoc_body,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6548), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162099] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6617), 1,
      anon_sym_COMMA,
    STATE(4521), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162135] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6622), 1,
      anon_sym_COMMA,
    STATE(4522), 1,
      sym_heredoc_body,
    STATE(4533), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6620), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162173] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6626), 1,
      anon_sym_COMMA,
    STATE(4523), 1,
      sym_heredoc_body,
    STATE(4525), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162211] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6628), 1,
      anon_sym_COMMA,
    STATE(4524), 1,
      sym_heredoc_body,
    STATE(4526), 1,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(5475), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162249] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6632), 1,
      anon_sym_COMMA,
    STATE(4520), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(4525), 1,
      sym_heredoc_body,
    ACTIONS(6630), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162287] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6634), 1,
      anon_sym_COMMA,
    STATE(4526), 2,
      sym_heredoc_body,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6582), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162323] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6626), 1,
      anon_sym_COMMA,
    STATE(4527), 1,
      sym_heredoc_body,
    STATE(4529), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162361] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6637), 1,
      sym_identifier,
    STATE(4528), 1,
      sym_heredoc_body,
    ACTIONS(2650), 6,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(2658), 15,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162399] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6626), 1,
      anon_sym_COMMA,
    STATE(4520), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(4529), 1,
      sym_heredoc_body,
    ACTIONS(6630), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162437] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    ACTIONS(6641), 1,
      anon_sym_COMMA,
    STATE(4530), 1,
      sym_heredoc_body,
    ACTIONS(6639), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162475] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6643), 1,
      anon_sym_COMMA,
    STATE(4524), 1,
      aux_sym__hash_pattern_body_repeat1,
    STATE(4531), 1,
      sym_heredoc_body,
    ACTIONS(6568), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162513] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    ACTIONS(6641), 1,
      anon_sym_COMMA,
    STATE(4532), 1,
      sym_heredoc_body,
    ACTIONS(6576), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162551] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6622), 1,
      anon_sym_COMMA,
    STATE(4521), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4533), 1,
      sym_heredoc_body,
    ACTIONS(6645), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162589] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(785), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(143), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4534), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [162648] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6655), 1,
      anon_sym_rescue,
    STATE(4535), 1,
      sym_heredoc_body,
    ACTIONS(6594), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162683] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(777), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(139), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4536), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [162742] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(831), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(165), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4537), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [162801] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6655), 1,
      anon_sym_rescue,
    STATE(4538), 1,
      sym_heredoc_body,
    ACTIONS(6596), 20,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [162836] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(827), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(163), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4539), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [162895] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(799), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(150), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4540), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [162954] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(807), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(153), 1,
      sym_parameters,
    STATE(208), 1,
      sym__terminator,
    STATE(4541), 1,
      sym_heredoc_body,
    STATE(5344), 1,
      sym__body_expr,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5527), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163013] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(803), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(152), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4542), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163072] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(765), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(135), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4543), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163131] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(791), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(146), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4544), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163190] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(813), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(156), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4545), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163249] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(815), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(157), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4546), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163308] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(833), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(166), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4547), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163367] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(801), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(151), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4548), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163426] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(821), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(160), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4549), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163485] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(835), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(167), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4550), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163544] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(825), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(162), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4551), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163603] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(837), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(168), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4552), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163662] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(779), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(140), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4553), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163721] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(771), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(137), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4554), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163780] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(819), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(159), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4555), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163839] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(809), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(154), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4556), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163898] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(797), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(149), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4557), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [163957] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(767), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(136), 1,
      sym_parameters,
    STATE(216), 1,
      sym__terminator,
    STATE(2351), 1,
      sym__body_expr,
    STATE(4558), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5514), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164016] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(789), 1,
      anon_sym_EQ,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6649), 1,
      anon_sym_LPAREN,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    STATE(145), 1,
      sym_parameters,
    STATE(229), 1,
      sym__terminator,
    STATE(2971), 1,
      sym__body_expr,
    STATE(4559), 1,
      sym_heredoc_body,
    STATE(5433), 1,
      sym__simple_formal_parameter,
    STATE(5564), 1,
      sym_bare_parameters,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164075] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6487), 1,
      anon_sym_LBRACE,
    ACTIONS(6657), 1,
      sym_identifier,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6665), 1,
      anon_sym_do,
    STATE(4560), 1,
      sym_heredoc_body,
    STATE(5469), 1,
      sym__simple_formal_parameter,
    STATE(4424), 2,
      sym_do_block,
      sym_block,
    STATE(5395), 2,
      sym_parameters,
      sym_bare_parameters,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164129] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6667), 1,
      anon_sym_COMMA,
    STATE(4561), 1,
      sym_heredoc_body,
    STATE(4571), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6645), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [164165] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6657), 1,
      sym_identifier,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6669), 1,
      anon_sym_LBRACE,
    ACTIONS(6671), 1,
      anon_sym_do,
    STATE(4562), 1,
      sym_heredoc_body,
    STATE(5469), 1,
      sym__simple_formal_parameter,
    STATE(4424), 2,
      sym_do_block,
      sym_block,
    STATE(5399), 2,
      sym_parameters,
      sym_bare_parameters,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164219] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3184), 1,
      anon_sym_COLON,
    STATE(4563), 1,
      sym_heredoc_body,
    ACTIONS(2740), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_and,
      anon_sym_or,
  [164253] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6657), 1,
      sym_identifier,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6673), 1,
      anon_sym_LBRACE,
    ACTIONS(6675), 1,
      anon_sym_do,
    STATE(4564), 1,
      sym_heredoc_body,
    STATE(5469), 1,
      sym__simple_formal_parameter,
    STATE(4688), 2,
      sym_do_block,
      sym_block,
    STATE(5405), 2,
      sym_parameters,
      sym_bare_parameters,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164307] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6667), 1,
      anon_sym_COMMA,
    STATE(4561), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4565), 1,
      sym_heredoc_body,
    ACTIONS(6620), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [164343] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6657), 1,
      sym_identifier,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6677), 1,
      anon_sym_LBRACE,
    ACTIONS(6679), 1,
      anon_sym_do,
    STATE(4566), 1,
      sym_heredoc_body,
    STATE(5469), 1,
      sym__simple_formal_parameter,
    STATE(4688), 2,
      sym_do_block,
      sym_block,
    STATE(5454), 2,
      sym_parameters,
      sym_bare_parameters,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164397] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4567), 1,
      sym_heredoc_body,
    ACTIONS(6681), 20,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164429] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(175), 1,
      anon_sym_do,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    ACTIONS(6657), 1,
      sym_identifier,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    STATE(4568), 1,
      sym_heredoc_body,
    STATE(5469), 1,
      sym__simple_formal_parameter,
    STATE(2288), 2,
      sym_do_block,
      sym_block,
    STATE(5468), 2,
      sym_parameters,
      sym_bare_parameters,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164483] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6657), 1,
      sym_identifier,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6685), 1,
      anon_sym_do,
    STATE(4569), 1,
      sym_heredoc_body,
    STATE(5469), 1,
      sym__simple_formal_parameter,
    STATE(5341), 2,
      sym_parameters,
      sym_bare_parameters,
    STATE(5462), 2,
      sym_do_block,
      sym_block,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164537] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(307), 1,
      anon_sym_do,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(6657), 1,
      sym_identifier,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    STATE(4570), 1,
      sym_heredoc_body,
    STATE(5469), 1,
      sym__simple_formal_parameter,
    STATE(3042), 2,
      sym_do_block,
      sym_block,
    STATE(5484), 2,
      sym_parameters,
      sym_bare_parameters,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164591] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6687), 1,
      anon_sym_COMMA,
    STATE(4571), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [164625] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6690), 1,
      sym_identifier,
    ACTIONS(6692), 1,
      anon_sym_COMMA,
    ACTIONS(6694), 1,
      anon_sym_PIPE,
    ACTIONS(6696), 1,
      anon_sym_SEMI,
    STATE(4572), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5374), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164680] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4573), 1,
      sym_heredoc_body,
    ACTIONS(6698), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164711] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6700), 1,
      anon_sym_rescue,
    STATE(4574), 1,
      sym_heredoc_body,
    ACTIONS(6596), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [164744] = 16,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6690), 1,
      sym_identifier,
    ACTIONS(6702), 1,
      anon_sym_COMMA,
    ACTIONS(6704), 1,
      anon_sym_PIPE,
    ACTIONS(6706), 1,
      anon_sym_SEMI,
    STATE(4575), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5489), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [164799] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6708), 1,
      anon_sym_COMMA,
    STATE(4576), 2,
      sym_heredoc_body,
      aux_sym_undef_repeat1,
    ACTIONS(6681), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164832] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4577), 1,
      sym_heredoc_body,
    ACTIONS(6711), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164863] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6715), 1,
      anon_sym_COMMA,
    STATE(4576), 1,
      aux_sym_undef_repeat1,
    STATE(4578), 1,
      sym_heredoc_body,
    ACTIONS(6713), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164898] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4579), 1,
      sym_heredoc_body,
    ACTIONS(6717), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164929] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4580), 1,
      sym_heredoc_body,
    ACTIONS(6719), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164960] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6715), 1,
      anon_sym_COMMA,
    STATE(4578), 1,
      aux_sym_undef_repeat1,
    STATE(4581), 1,
      sym_heredoc_body,
    ACTIONS(6721), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [164995] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4582), 1,
      sym_heredoc_body,
    ACTIONS(6723), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165026] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4583), 1,
      sym_heredoc_body,
    ACTIONS(6725), 19,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165057] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6467), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6727), 1,
      anon_sym_COLON,
    STATE(4584), 1,
      sym_heredoc_body,
    ACTIONS(3286), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [165092] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6729), 1,
      anon_sym_EQ2,
    STATE(4585), 1,
      sym_heredoc_body,
    ACTIONS(4802), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_do,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165125] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6700), 1,
      anon_sym_rescue,
    STATE(4586), 1,
      sym_heredoc_body,
    ACTIONS(6594), 18,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [165158] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3551), 1,
      anon_sym_DOT_DOT,
    STATE(4587), 1,
      sym_heredoc_body,
    ACTIONS(3549), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [165190] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6690), 1,
      sym_identifier,
    ACTIONS(6731), 1,
      anon_sym_PIPE,
    ACTIONS(6733), 1,
      anon_sym_SEMI,
    STATE(4588), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5268), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [165242] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4589), 1,
      sym_heredoc_body,
    ACTIONS(6735), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6717), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165274] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6690), 1,
      sym_identifier,
    ACTIONS(6737), 1,
      anon_sym_PIPE,
    ACTIONS(6739), 1,
      anon_sym_SEMI,
    STATE(4590), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5268), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [165326] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6690), 1,
      sym_identifier,
    ACTIONS(6741), 1,
      anon_sym_PIPE,
    ACTIONS(6743), 1,
      anon_sym_SEMI,
    STATE(4591), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5268), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [165378] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6745), 1,
      anon_sym_rescue,
    STATE(4592), 1,
      sym_heredoc_body,
    ACTIONS(6594), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [165410] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6747), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6749), 1,
      anon_sym_DOT_DOT,
    STATE(4593), 1,
      sym_heredoc_body,
    ACTIONS(6455), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [165444] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4594), 1,
      sym_heredoc_body,
    ACTIONS(6735), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6751), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165476] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4595), 1,
      sym_heredoc_body,
    ACTIONS(6735), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6753), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165508] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6745), 1,
      anon_sym_rescue,
    STATE(4596), 1,
      sym_heredoc_body,
    ACTIONS(6596), 17,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
      anon_sym_and,
      anon_sym_or,
  [165540] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4597), 1,
      sym_heredoc_body,
    ACTIONS(6735), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6755), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165572] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4598), 1,
      sym_heredoc_body,
    ACTIONS(6735), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6757), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165604] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3451), 1,
      anon_sym_DOT_DOT,
    STATE(4599), 1,
      sym_heredoc_body,
    ACTIONS(3449), 17,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [165636] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6690), 1,
      sym_identifier,
    ACTIONS(6759), 1,
      anon_sym_PIPE,
    ACTIONS(6761), 1,
      anon_sym_SEMI,
    STATE(4600), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5268), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [165688] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4601), 1,
      sym_heredoc_body,
    ACTIONS(6735), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6763), 16,
      sym__line_break,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_then,
      anon_sym_ensure,
  [165720] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6765), 1,
      anon_sym_end,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    STATE(4602), 1,
      sym_heredoc_body,
    STATE(4890), 1,
      sym__terminator,
    STATE(5265), 1,
      aux_sym_case_repeat1,
    STATE(5266), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5753), 1,
      sym_else,
    ACTIONS(1341), 2,
      sym__line_break,
      anon_sym_SEMI,
  [165779] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(6783), 1,
      anon_sym_end,
    STATE(4603), 1,
      sym_heredoc_body,
    STATE(4921), 1,
      sym__terminator,
    STATE(5284), 1,
      aux_sym_case_match_repeat1,
    STATE(5286), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5728), 1,
      sym_else,
    ACTIONS(1341), 2,
      sym__line_break,
      anon_sym_SEMI,
  [165838] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3276), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3280), 1,
      anon_sym_ri,
    STATE(4604), 1,
      sym_heredoc_body,
    ACTIONS(3278), 2,
      anon_sym_i,
      anon_sym_r,
    ACTIONS(3274), 13,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [165873] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6785), 1,
      anon_sym_COLON,
    STATE(4605), 1,
      sym_heredoc_body,
    ACTIONS(3080), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [165904] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6787), 1,
      anon_sym_PIPE,
    STATE(4606), 1,
      sym_heredoc_body,
    STATE(4612), 1,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6523), 15,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [165937] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4607), 1,
      sym_heredoc_body,
    ACTIONS(6789), 3,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(6527), 14,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [165968] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(6791), 1,
      anon_sym_end,
    STATE(4608), 1,
      sym_heredoc_body,
    STATE(4885), 1,
      sym__terminator,
    STATE(5271), 1,
      aux_sym_case_repeat1,
    STATE(5272), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5830), 1,
      sym_else,
    ACTIONS(1341), 2,
      sym__line_break,
      anon_sym_SEMI,
  [166027] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(6793), 1,
      anon_sym_end,
    STATE(4609), 1,
      sym_heredoc_body,
    STATE(4932), 1,
      sym__terminator,
    STATE(5287), 1,
      aux_sym_case_repeat1,
    STATE(5288), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5964), 1,
      sym_else,
    ACTIONS(1341), 2,
      sym__line_break,
      anon_sym_SEMI,
  [166086] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(6795), 1,
      anon_sym_end,
    STATE(4610), 1,
      sym_heredoc_body,
    STATE(4908), 1,
      sym__terminator,
    STATE(5274), 1,
      aux_sym_case_repeat1,
    STATE(5275), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5732), 1,
      sym_else,
    ACTIONS(1341), 2,
      sym__line_break,
      anon_sym_SEMI,
  [166145] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6797), 1,
      sym_identifier,
    ACTIONS(6799), 1,
      anon_sym_RPAREN,
    STATE(4611), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5499), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [166194] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6801), 1,
      anon_sym_PIPE,
    STATE(4612), 2,
      sym_heredoc_body,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6537), 15,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [166225] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(6804), 1,
      anon_sym_RBRACE,
    STATE(4613), 1,
      sym_heredoc_body,
    STATE(4927), 1,
      sym__hash_pattern_any_rest,
    STATE(4930), 1,
      sym_keyword_pattern,
    STATE(5813), 1,
      sym_string,
    STATE(5829), 1,
      sym__hash_pattern_body,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6580), 2,
      sym_constant,
      sym_identifier,
    STATE(4899), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5892), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
  [166276] = 15,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2886), 1,
      sym__string_start,
    ACTIONS(6806), 1,
      anon_sym_RBRACE,
    STATE(4614), 1,
      sym_heredoc_body,
    STATE(4927), 1,
      sym__hash_pattern_any_rest,
    STATE(4930), 1,
      sym_keyword_pattern,
    STATE(5701), 1,
      sym__hash_pattern_body,
    STATE(5813), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6580), 2,
      sym_constant,
      sym_identifier,
    STATE(4899), 2,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
    STATE(5892), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
  [166327] = 19,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(6808), 1,
      anon_sym_end,
    STATE(4615), 1,
      sym_heredoc_body,
    STATE(4914), 1,
      sym__terminator,
    STATE(5225), 1,
      aux_sym_case_repeat1,
    STATE(5228), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5733), 1,
      sym_else,
    ACTIONS(1341), 2,
      sym__line_break,
      anon_sym_SEMI,
  [166386] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6797), 1,
      sym_identifier,
    ACTIONS(6810), 1,
      anon_sym_RPAREN,
    STATE(4616), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5522), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [166435] = 14,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6797), 1,
      sym_identifier,
    ACTIONS(6812), 1,
      anon_sym_RPAREN,
    STATE(4617), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5575), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [166484] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6787), 1,
      anon_sym_PIPE,
    STATE(4606), 1,
      aux_sym_alternative_pattern_repeat1,
    STATE(4618), 1,
      sym_heredoc_body,
    ACTIONS(6535), 15,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [166517] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1645), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2918), 1,
      anon_sym_STAR,
    ACTIONS(6647), 1,
      sym_identifier,
    ACTIONS(6651), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6653), 1,
      anon_sym_AMP,
    ACTIONS(6814), 1,
      anon_sym_LPAREN,
    STATE(4619), 1,
      sym_heredoc_body,
    STATE(5545), 1,
      sym__formal_parameter,
    STATE(5550), 1,
      sym_parameters,
    STATE(5552), 1,
      sym__simple_formal_parameter,
    STATE(5558), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [166563] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6816), 1,
      sym_identifier,
    STATE(4620), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5268), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [166609] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3282), 1,
      anon_sym_COLON,
    STATE(4621), 1,
      sym_heredoc_body,
    ACTIONS(2740), 15,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [166639] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6797), 1,
      sym_identifier,
    STATE(4622), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5268), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [166685] = 13,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(1675), 1,
      anon_sym_STAR_STAR,
    ACTIONS(2864), 1,
      anon_sym_STAR,
    ACTIONS(6659), 1,
      anon_sym_LPAREN,
    ACTIONS(6661), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6663), 1,
      anon_sym_AMP,
    ACTIONS(6690), 1,
      sym_identifier,
    STATE(4623), 1,
      sym_heredoc_body,
    STATE(5243), 1,
      sym_parameters,
    STATE(5245), 1,
      sym__simple_formal_parameter,
    STATE(5268), 1,
      sym__formal_parameter,
    STATE(5283), 7,
      sym_forward_parameter,
      sym_splat_parameter,
      sym_hash_splat_parameter,
      sym_hash_splat_nil,
      sym_block_parameter,
      sym_keyword_parameter,
      sym_optional_parameter,
  [166731] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6604), 1,
      sym_identifier,
    ACTIONS(6610), 1,
      anon_sym_nil,
    STATE(4624), 1,
      sym_heredoc_body,
    ACTIONS(6606), 5,
      sym__line_break,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(6608), 9,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [166765] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    STATE(2053), 1,
      sym__terminator,
    STATE(4625), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    ACTIONS(1089), 8,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_ensure,
  [166807] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    STATE(2053), 1,
      sym__terminator,
    STATE(4626), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    ACTIONS(6818), 8,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_end,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_ensure,
  [166849] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4627), 1,
      sym_heredoc_body,
    ACTIONS(2632), 16,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_and,
      anon_sym_or,
  [166877] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    ACTIONS(6820), 1,
      anon_sym_COMMA,
    STATE(4628), 1,
      sym_heredoc_body,
    ACTIONS(6576), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [166908] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3276), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3278), 1,
      anon_sym_r,
    STATE(4629), 1,
      sym_heredoc_body,
    ACTIONS(3280), 2,
      anon_sym_i,
      anon_sym_ri,
    ACTIONS(3274), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [166941] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6637), 1,
      sym_identifier,
    STATE(4630), 1,
      sym_heredoc_body,
    ACTIONS(2650), 5,
      sym__line_break,
      ts_builtin_sym_end,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(2658), 9,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [166972] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6822), 1,
      anon_sym_COMMA,
    STATE(4631), 2,
      sym_heredoc_body,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6582), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167001] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6825), 1,
      anon_sym_COMMA,
    STATE(4632), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167030] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6828), 1,
      anon_sym_COMMA,
    STATE(4632), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4633), 1,
      sym_heredoc_body,
    ACTIONS(6645), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167061] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6830), 1,
      anon_sym_COMMA,
    STATE(4634), 1,
      sym_heredoc_body,
    STATE(4635), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167092] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6832), 1,
      anon_sym_COMMA,
    STATE(4635), 1,
      sym_heredoc_body,
    STATE(4639), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6630), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167123] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6834), 1,
      anon_sym_COMMA,
    STATE(4636), 1,
      sym_heredoc_body,
    STATE(4638), 1,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6568), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167154] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3276), 1,
      anon_sym_DOT_DOT,
    ACTIONS(3278), 1,
      anon_sym_r,
    STATE(4637), 1,
      sym_heredoc_body,
    ACTIONS(3280), 2,
      anon_sym_i,
      anon_sym_ri,
    ACTIONS(3274), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [167187] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6836), 1,
      anon_sym_COMMA,
    STATE(4631), 1,
      aux_sym__hash_pattern_body_repeat1,
    STATE(4638), 1,
      sym_heredoc_body,
    ACTIONS(5475), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167218] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6838), 1,
      anon_sym_COMMA,
    STATE(4639), 2,
      sym_heredoc_body,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6548), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167247] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6830), 1,
      anon_sym_COMMA,
    STATE(4640), 1,
      sym_heredoc_body,
    STATE(4643), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167278] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6828), 1,
      anon_sym_COMMA,
    STATE(4633), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4641), 1,
      sym_heredoc_body,
    ACTIONS(6620), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167309] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    ACTIONS(6820), 1,
      anon_sym_COMMA,
    STATE(4642), 1,
      sym_heredoc_body,
    ACTIONS(6639), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167340] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6830), 1,
      anon_sym_COMMA,
    STATE(4639), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(4643), 1,
      sym_heredoc_body,
    ACTIONS(6630), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167371] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6467), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6841), 1,
      anon_sym_COLON,
    STATE(4644), 1,
      sym_heredoc_body,
    ACTIONS(3286), 13,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [167402] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6843), 1,
      anon_sym_rescue,
    STATE(4645), 1,
      sym_heredoc_body,
    ACTIONS(6594), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167430] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4646), 1,
      sym_heredoc_body,
    ACTIONS(3012), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167456] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4647), 1,
      sym_heredoc_body,
    ACTIONS(3064), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167482] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4648), 1,
      sym_heredoc_body,
    ACTIONS(3000), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167508] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4649), 1,
      sym_heredoc_body,
    ACTIONS(3016), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167534] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4650), 1,
      sym_heredoc_body,
    ACTIONS(3020), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167560] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(4651), 1,
      sym_heredoc_body,
    STATE(5012), 1,
      sym_argument_list,
    STATE(5425), 1,
      sym_do_block,
    STATE(5429), 1,
      sym_block,
    ACTIONS(177), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [167598] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4652), 1,
      sym_heredoc_body,
    ACTIONS(3024), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167624] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4653), 1,
      sym_heredoc_body,
    ACTIONS(2740), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_and,
      anon_sym_or,
  [167650] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4654), 1,
      sym_heredoc_body,
    ACTIONS(2976), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_and,
      anon_sym_or,
  [167676] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4655), 1,
      sym_heredoc_body,
    ACTIONS(3028), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167702] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6847), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6849), 1,
      anon_sym_LBRACK2,
    ACTIONS(6851), 1,
      anon_sym_LPAREN2,
    STATE(4656), 1,
      sym_heredoc_body,
    ACTIONS(6455), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [167734] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4657), 1,
      sym_heredoc_body,
    ACTIONS(6463), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_and,
      anon_sym_or,
  [167760] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4658), 1,
      sym_heredoc_body,
    ACTIONS(6453), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_and,
      anon_sym_or,
  [167786] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3284), 1,
      anon_sym_COLON,
    STATE(4659), 1,
      sym_heredoc_body,
    ACTIONS(2740), 13,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_and,
      anon_sym_or,
  [167814] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3305), 1,
      anon_sym_DOT_DOT,
    STATE(4660), 1,
      sym_heredoc_body,
    ACTIONS(3303), 13,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [167842] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4661), 1,
      sym_heredoc_body,
    ACTIONS(3032), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167868] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4662), 1,
      sym_heredoc_body,
    ACTIONS(3008), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167894] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6843), 1,
      anon_sym_rescue,
    STATE(4663), 1,
      sym_heredoc_body,
    ACTIONS(6596), 13,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [167922] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6853), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6855), 1,
      anon_sym_DOT_DOT,
    STATE(4664), 1,
      sym_heredoc_body,
    ACTIONS(6455), 12,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [167952] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4665), 1,
      sym_heredoc_body,
    ACTIONS(3036), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [167978] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_DOT_DOT,
    STATE(4666), 1,
      sym_heredoc_body,
    ACTIONS(3319), 13,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COLON,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168006] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4667), 1,
      sym_heredoc_body,
    ACTIONS(6335), 5,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(743), 9,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [168034] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(4668), 1,
      sym_heredoc_body,
    STATE(5013), 1,
      sym_argument_list,
    STATE(5373), 1,
      sym_do_block,
    STATE(5379), 1,
      sym_block,
    ACTIONS(121), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [168072] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4669), 1,
      sym_heredoc_body,
    ACTIONS(3040), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [168098] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4670), 1,
      sym_heredoc_body,
    ACTIONS(3044), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [168124] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4671), 1,
      sym_heredoc_body,
    ACTIONS(3056), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [168150] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4672), 1,
      sym_heredoc_body,
    ACTIONS(3086), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [168176] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4673), 1,
      sym_heredoc_body,
    ACTIONS(3052), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [168202] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4674), 1,
      sym_heredoc_body,
    ACTIONS(3048), 14,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [168228] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3050), 1,
      anon_sym_DOT_DOT,
    STATE(4675), 1,
      sym_heredoc_body,
    ACTIONS(3048), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168255] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3451), 1,
      anon_sym_DOT_DOT,
    STATE(4676), 1,
      sym_heredoc_body,
    ACTIONS(3449), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168282] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3002), 1,
      anon_sym_DOT_DOT,
    STATE(4677), 1,
      sym_heredoc_body,
    ACTIONS(3000), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168309] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3551), 1,
      anon_sym_DOT_DOT,
    STATE(4678), 1,
      sym_heredoc_body,
    ACTIONS(3549), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168336] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3563), 1,
      anon_sym_DOT_DOT,
    STATE(4679), 1,
      sym_heredoc_body,
    ACTIONS(3561), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168363] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6857), 1,
      anon_sym_PIPE,
    STATE(4680), 1,
      sym_heredoc_body,
    STATE(4686), 1,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6523), 11,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [168392] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3014), 1,
      anon_sym_DOT_DOT,
    STATE(4681), 1,
      sym_heredoc_body,
    ACTIONS(3012), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168419] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3411), 1,
      anon_sym_DOT_DOT,
    STATE(4682), 1,
      sym_heredoc_body,
    ACTIONS(3409), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168446] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3046), 1,
      anon_sym_DOT_DOT,
    STATE(4683), 1,
      sym_heredoc_body,
    ACTIONS(3044), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168473] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3947), 1,
      anon_sym_DOT_DOT,
    STATE(4684), 1,
      sym_heredoc_body,
    ACTIONS(3945), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168500] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3951), 1,
      anon_sym_DOT_DOT,
    STATE(4685), 1,
      sym_heredoc_body,
    ACTIONS(3949), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168527] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6859), 1,
      anon_sym_PIPE,
    STATE(4686), 2,
      sym_heredoc_body,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6537), 11,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [168554] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3030), 1,
      anon_sym_DOT_DOT,
    STATE(4687), 1,
      sym_heredoc_body,
    ACTIONS(3028), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168581] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3383), 1,
      anon_sym_DOT_DOT,
    STATE(4688), 1,
      sym_heredoc_body,
    ACTIONS(3381), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168608] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3034), 1,
      anon_sym_DOT_DOT,
    STATE(4689), 1,
      sym_heredoc_body,
    ACTIONS(3032), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168635] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3969), 1,
      anon_sym_DOT_DOT,
    STATE(4690), 1,
      sym_heredoc_body,
    ACTIONS(3967), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168662] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6862), 1,
      anon_sym_COLON,
    STATE(4691), 1,
      sym_heredoc_body,
    ACTIONS(3080), 12,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [168689] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6471), 1,
      anon_sym_DOT_DOT,
    STATE(4692), 1,
      sym_heredoc_body,
    ACTIONS(6469), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168716] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6864), 1,
      sym_identifier,
    ACTIONS(6866), 1,
      anon_sym_nil,
    STATE(4693), 1,
      sym_heredoc_body,
    ACTIONS(6608), 4,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6606), 7,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_RBRACK,
  [168747] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3088), 1,
      anon_sym_DOT_DOT,
    STATE(4694), 1,
      sym_heredoc_body,
    ACTIONS(3086), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168774] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3038), 1,
      anon_sym_DOT_DOT,
    STATE(4695), 1,
      sym_heredoc_body,
    ACTIONS(3036), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168801] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3042), 1,
      anon_sym_DOT_DOT,
    STATE(4696), 1,
      sym_heredoc_body,
    ACTIONS(3040), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168828] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6868), 1,
      anon_sym_COMMA,
    STATE(4697), 1,
      sym_heredoc_body,
    STATE(4726), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6645), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [168857] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3371), 1,
      anon_sym_DOT_DOT,
    STATE(4698), 1,
      sym_heredoc_body,
    ACTIONS(3369), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168884] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3713), 1,
      anon_sym_DOT_DOT,
    STATE(4699), 1,
      sym_heredoc_body,
    ACTIONS(3711), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168911] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3955), 1,
      anon_sym_DOT_DOT,
    STATE(4700), 1,
      sym_heredoc_body,
    ACTIONS(3953), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168938] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3062), 1,
      anon_sym_DOT_DOT,
    STATE(4701), 1,
      sym_heredoc_body,
    ACTIONS(3060), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168965] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6467), 1,
      anon_sym_DOT_DOT,
    STATE(4702), 1,
      sym_heredoc_body,
    ACTIONS(3286), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [168992] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3415), 1,
      anon_sym_DOT_DOT,
    STATE(4703), 1,
      sym_heredoc_body,
    ACTIONS(3413), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169019] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3054), 1,
      anon_sym_DOT_DOT,
    STATE(4704), 1,
      sym_heredoc_body,
    ACTIONS(3052), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169046] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6465), 1,
      anon_sym_DOT_DOT,
    STATE(4705), 1,
      sym_heredoc_body,
    ACTIONS(2998), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169073] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3407), 1,
      anon_sym_DOT_DOT,
    STATE(4706), 1,
      sym_heredoc_body,
    ACTIONS(3405), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169100] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3965), 1,
      anon_sym_DOT_DOT,
    STATE(4707), 1,
      sym_heredoc_body,
    ACTIONS(3963), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169127] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(4708), 1,
      sym_heredoc_body,
    STATE(5012), 1,
      sym_argument_list,
    STATE(5425), 1,
      sym_do_block,
    STATE(5429), 1,
      sym_block,
    ACTIONS(177), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [169164] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3313), 1,
      anon_sym_COLON,
    STATE(4709), 1,
      sym_heredoc_body,
    ACTIONS(2740), 12,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COLON_COLON,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_LBRACK2,
      anon_sym_LPAREN2,
      anon_sym_and,
      anon_sym_or,
  [169191] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(4710), 1,
      sym_heredoc_body,
    STATE(5013), 1,
      sym_argument_list,
    STATE(5373), 1,
      sym_do_block,
    STATE(5379), 1,
      sym_block,
    ACTIONS(121), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [169228] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3403), 1,
      anon_sym_DOT_DOT,
    STATE(4711), 1,
      sym_heredoc_body,
    ACTIONS(3401), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169255] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3058), 1,
      anon_sym_DOT_DOT,
    STATE(4712), 1,
      sym_heredoc_body,
    ACTIONS(3056), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169282] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3567), 1,
      anon_sym_DOT_DOT,
    STATE(4713), 1,
      sym_heredoc_body,
    ACTIONS(3565), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169309] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3375), 1,
      anon_sym_DOT_DOT,
    STATE(4714), 1,
      sym_heredoc_body,
    ACTIONS(3373), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169336] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3026), 1,
      anon_sym_DOT_DOT,
    STATE(4715), 1,
      sym_heredoc_body,
    ACTIONS(3024), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169363] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3066), 1,
      anon_sym_DOT_DOT,
    STATE(4716), 1,
      sym_heredoc_body,
    ACTIONS(3064), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169390] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3527), 1,
      anon_sym_DOT_DOT,
    STATE(4717), 1,
      sym_heredoc_body,
    ACTIONS(3525), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169417] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3276), 1,
      anon_sym_DOT_DOT,
    STATE(4718), 1,
      sym_heredoc_body,
    ACTIONS(3274), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169444] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3733), 1,
      anon_sym_DOT_DOT,
    STATE(4719), 1,
      sym_heredoc_body,
    ACTIONS(3731), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169471] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3737), 1,
      anon_sym_DOT_DOT,
    STATE(4720), 1,
      sym_heredoc_body,
    ACTIONS(3735), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169498] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3010), 1,
      anon_sym_DOT_DOT,
    STATE(4721), 1,
      sym_heredoc_body,
    ACTIONS(3008), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169525] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3022), 1,
      anon_sym_DOT_DOT,
    STATE(4722), 1,
      sym_heredoc_body,
    ACTIONS(3020), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169552] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3477), 1,
      anon_sym_DOT_DOT,
    STATE(4723), 1,
      sym_heredoc_body,
    ACTIONS(3475), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169579] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3543), 1,
      anon_sym_DOT_DOT,
    STATE(4724), 1,
      sym_heredoc_body,
    ACTIONS(3541), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169606] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3607), 1,
      anon_sym_DOT_DOT,
    STATE(4725), 1,
      sym_heredoc_body,
    ACTIONS(3605), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169633] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6870), 1,
      anon_sym_COMMA,
    STATE(4726), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [169660] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3615), 1,
      anon_sym_DOT_DOT,
    STATE(4727), 1,
      sym_heredoc_body,
    ACTIONS(3613), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169687] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6467), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6873), 1,
      anon_sym_COLON,
    STATE(4728), 1,
      sym_heredoc_body,
    ACTIONS(3286), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169716] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3583), 1,
      anon_sym_DOT_DOT,
    STATE(4729), 1,
      sym_heredoc_body,
    ACTIONS(3581), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169743] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3018), 1,
      anon_sym_DOT_DOT,
    STATE(4730), 1,
      sym_heredoc_body,
    ACTIONS(3016), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169770] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6868), 1,
      anon_sym_COMMA,
    STATE(4697), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4731), 1,
      sym_heredoc_body,
    ACTIONS(6620), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [169799] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6857), 1,
      anon_sym_PIPE,
    STATE(4680), 1,
      aux_sym_alternative_pattern_repeat1,
    STATE(4732), 1,
      sym_heredoc_body,
    ACTIONS(6535), 11,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [169828] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3515), 1,
      anon_sym_DOT_DOT,
    STATE(4733), 1,
      sym_heredoc_body,
    ACTIONS(3513), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169855] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3511), 1,
      anon_sym_DOT_DOT,
    STATE(4734), 1,
      sym_heredoc_body,
    ACTIONS(3509), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169882] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6473), 1,
      anon_sym_DOT_DOT,
    STATE(4735), 1,
      sym_heredoc_body,
    ACTIONS(3625), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169909] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3595), 1,
      anon_sym_DOT_DOT,
    STATE(4736), 1,
      sym_heredoc_body,
    ACTIONS(3593), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169936] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3555), 1,
      anon_sym_DOT_DOT,
    STATE(4737), 1,
      sym_heredoc_body,
    ACTIONS(3553), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169963] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3559), 1,
      anon_sym_DOT_DOT,
    STATE(4738), 1,
      sym_heredoc_body,
    ACTIONS(3557), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [169990] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3909), 1,
      anon_sym_DOT_DOT,
    STATE(4739), 1,
      sym_heredoc_body,
    ACTIONS(3907), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170017] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3961), 1,
      anon_sym_DOT_DOT,
    STATE(4740), 1,
      sym_heredoc_body,
    ACTIONS(3959), 12,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170044] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6467), 1,
      anon_sym_DOT_DOT,
    ACTIONS(6875), 1,
      anon_sym_COLON,
    STATE(4741), 1,
      sym_heredoc_body,
    ACTIONS(3286), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_DOT_DOT_DOT,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170072] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(4742), 1,
      sym_heredoc_body,
    STATE(4965), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2986), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [170102] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6877), 1,
      anon_sym_COMMA,
    STATE(4743), 1,
      sym_heredoc_body,
    STATE(4751), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6645), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [170130] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(4744), 1,
      sym_heredoc_body,
    STATE(4955), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2978), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [170160] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(731), 1,
      anon_sym_DOT,
    ACTIONS(743), 1,
      anon_sym_COLON_COLON,
    ACTIONS(6729), 1,
      anon_sym_EQ2,
    STATE(4745), 1,
      sym_heredoc_body,
    ACTIONS(4800), 2,
      anon_sym_EQ,
      anon_sym_STAR,
    ACTIONS(4802), 7,
      sym__line_break,
      anon_sym_LPAREN,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      sym_identifier,
  [170192] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6879), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6881), 1,
      anon_sym_DOT_DOT,
    STATE(4746), 1,
      sym_heredoc_body,
    ACTIONS(6455), 10,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170220] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6883), 1,
      anon_sym_rescue,
    STATE(4747), 1,
      sym_heredoc_body,
    ACTIONS(6594), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [170246] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6883), 1,
      anon_sym_rescue,
    STATE(4748), 1,
      sym_heredoc_body,
    ACTIONS(6596), 11,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [170272] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6877), 1,
      anon_sym_COMMA,
    STATE(4743), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4749), 1,
      sym_heredoc_body,
    ACTIONS(6620), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [170300] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6885), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(6887), 1,
      anon_sym_DOT_DOT,
    STATE(4750), 1,
      sym_heredoc_body,
    ACTIONS(6455), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170328] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6889), 1,
      anon_sym_COMMA,
    STATE(4751), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 10,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [170354] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4752), 1,
      sym_heredoc_body,
    ACTIONS(6892), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6757), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [170379] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4753), 1,
      sym_heredoc_body,
    STATE(5764), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(5467), 2,
      sym_constant,
      sym_identifier,
    STATE(5841), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
  [170414] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4754), 1,
      sym_heredoc_body,
    ACTIONS(6517), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170437] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4755), 1,
      sym_heredoc_body,
    ACTIONS(6519), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170460] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6894), 1,
      anon_sym_PIPE,
    STATE(4756), 1,
      sym_heredoc_body,
    STATE(4799), 1,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6535), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170487] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4757), 1,
      sym_heredoc_body,
    ACTIONS(6481), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170510] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6896), 1,
      anon_sym_COMMA,
    STATE(4758), 2,
      sym_heredoc_body,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6582), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [170535] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6899), 1,
      sym_identifier,
    STATE(4759), 1,
      sym_heredoc_body,
    ACTIONS(2658), 4,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(2650), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_RBRACK,
  [170562] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4760), 1,
      sym_heredoc_body,
    ACTIONS(6455), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170585] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6901), 1,
      anon_sym_COMMA,
    STATE(4761), 1,
      sym_heredoc_body,
    STATE(4809), 1,
      aux_sym_undef_repeat1,
    ACTIONS(6721), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [170612] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4762), 1,
      sym_heredoc_body,
    ACTIONS(2644), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [170635] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4763), 1,
      sym_heredoc_body,
    ACTIONS(6507), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170658] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4764), 1,
      sym_heredoc_body,
    ACTIONS(2616), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [170681] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6903), 1,
      anon_sym_PIPE,
    STATE(4765), 2,
      sym_heredoc_body,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6537), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170706] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4766), 1,
      sym_heredoc_body,
    ACTIONS(2624), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [170729] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4767), 1,
      sym_heredoc_body,
    ACTIONS(6509), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170752] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6906), 1,
      anon_sym_COMMA,
    STATE(4768), 1,
      sym_heredoc_body,
    STATE(4818), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [170779] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4769), 1,
      sym_heredoc_body,
    ACTIONS(6513), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170802] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4770), 1,
      sym_heredoc_body,
    ACTIONS(6892), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6717), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [170827] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6908), 1,
      anon_sym_rescue,
    STATE(4771), 1,
      sym_heredoc_body,
    ACTIONS(6594), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [170852] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6910), 1,
      anon_sym_EQ2,
    STATE(4772), 1,
      sym_heredoc_body,
    ACTIONS(4802), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [170877] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4773), 1,
      sym_heredoc_body,
    ACTIONS(743), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [170900] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4774), 1,
      sym_heredoc_body,
    STATE(5723), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6389), 2,
      sym_constant,
      sym_identifier,
    STATE(5862), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
  [170935] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4775), 1,
      sym_heredoc_body,
    ACTIONS(2632), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [170958] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6912), 1,
      anon_sym_COLON,
    STATE(4776), 1,
      sym_heredoc_body,
    ACTIONS(3080), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [170983] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6908), 1,
      anon_sym_rescue,
    STATE(4777), 1,
      sym_heredoc_body,
    ACTIONS(6596), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_and,
      anon_sym_or,
  [171008] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4778), 1,
      sym_heredoc_body,
    ACTIONS(2624), 3,
      anon_sym_LBRACE,
      anon_sym_do,
      anon_sym_LPAREN2,
    ACTIONS(2620), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [171033] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4779), 1,
      sym_heredoc_body,
    ACTIONS(6501), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171056] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6914), 1,
      anon_sym_PIPE,
    STATE(4780), 1,
      sym_heredoc_body,
    STATE(4794), 1,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6523), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171083] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6916), 1,
      anon_sym_COMMA,
    STATE(4781), 2,
      sym_heredoc_body,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6548), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [171108] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6906), 1,
      anon_sym_COMMA,
    STATE(4782), 1,
      sym_heredoc_body,
    STATE(4784), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [171135] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4783), 1,
      sym_heredoc_body,
    ACTIONS(6497), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171158] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6906), 1,
      anon_sym_COMMA,
    STATE(4781), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(4784), 1,
      sym_heredoc_body,
    ACTIONS(6630), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [171185] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4496), 1,
      sym_keyword_pattern,
    STATE(4785), 1,
      sym_heredoc_body,
    STATE(5657), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6511), 2,
      sym_constant,
      sym_identifier,
    STATE(5773), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
  [171220] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4786), 1,
      sym_heredoc_body,
    ACTIONS(3485), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171243] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4787), 1,
      sym_heredoc_body,
    ACTIONS(6479), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [171266] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6914), 1,
      anon_sym_PIPE,
    STATE(4780), 1,
      aux_sym_alternative_pattern_repeat1,
    STATE(4788), 1,
      sym_heredoc_body,
    ACTIONS(6535), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171293] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4789), 1,
      sym_heredoc_body,
    ACTIONS(6491), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171316] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4790), 1,
      sym_heredoc_body,
    ACTIONS(3503), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171339] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4791), 1,
      sym_heredoc_body,
    ACTIONS(6483), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171362] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4792), 1,
      sym_heredoc_body,
    ACTIONS(6892), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6755), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [171387] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    ACTIONS(6919), 1,
      anon_sym_COMMA,
    STATE(4793), 1,
      sym_heredoc_body,
    ACTIONS(6576), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [171414] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6921), 1,
      anon_sym_PIPE,
    STATE(4794), 2,
      sym_heredoc_body,
      aux_sym_alternative_pattern_repeat1,
    ACTIONS(6537), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171439] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4795), 1,
      sym_heredoc_body,
    ACTIONS(6505), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171462] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4796), 1,
      sym_heredoc_body,
    ACTIONS(6924), 3,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(6527), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171487] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6926), 1,
      anon_sym_rescue,
    STATE(4797), 1,
      sym_heredoc_body,
    ACTIONS(6594), 10,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [171512] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6926), 1,
      anon_sym_rescue,
    STATE(4798), 1,
      sym_heredoc_body,
    ACTIONS(6596), 10,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [171537] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6894), 1,
      anon_sym_PIPE,
    STATE(4765), 1,
      aux_sym_alternative_pattern_repeat1,
    STATE(4799), 1,
      sym_heredoc_body,
    ACTIONS(6523), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171564] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4800), 1,
      sym_heredoc_body,
    ACTIONS(6503), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171587] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4801), 1,
      sym_heredoc_body,
    ACTIONS(6892), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6763), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [171612] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4802), 1,
      sym_heredoc_body,
    ACTIONS(6552), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171635] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4803), 1,
      sym_heredoc_body,
    ACTIONS(6586), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171658] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4804), 1,
      sym_heredoc_body,
    ACTIONS(6475), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [171681] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4805), 1,
      sym_heredoc_body,
    ACTIONS(6475), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [171704] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6928), 1,
      anon_sym_COMMA,
    STATE(4806), 1,
      sym_heredoc_body,
    STATE(4808), 1,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6568), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [171731] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6930), 1,
      anon_sym_LBRACE,
    ACTIONS(6932), 1,
      anon_sym_do,
    STATE(4807), 1,
      sym_heredoc_body,
    STATE(5114), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2978), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [171760] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6934), 1,
      anon_sym_COMMA,
    STATE(4758), 1,
      aux_sym__hash_pattern_body_repeat1,
    STATE(4808), 1,
      sym_heredoc_body,
    ACTIONS(5475), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [171787] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6901), 1,
      anon_sym_COMMA,
    STATE(4809), 1,
      sym_heredoc_body,
    STATE(4825), 1,
      aux_sym_undef_repeat1,
    ACTIONS(6713), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [171814] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4810), 1,
      sym_heredoc_body,
    ACTIONS(6521), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171837] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4811), 1,
      sym_heredoc_body,
    STATE(4926), 1,
      sym_keyword_pattern,
    STATE(5942), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6590), 2,
      sym_constant,
      sym_identifier,
    STATE(5839), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
  [171872] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6544), 1,
      anon_sym_EQ_GT,
    ACTIONS(6919), 1,
      anon_sym_COMMA,
    STATE(4812), 1,
      sym_heredoc_body,
    ACTIONS(6639), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [171899] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4813), 1,
      sym_heredoc_body,
    ACTIONS(2636), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [171922] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4814), 1,
      sym_heredoc_body,
    ACTIONS(2628), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [171945] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4815), 1,
      sym_heredoc_body,
    ACTIONS(2640), 11,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [171968] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4816), 1,
      sym_heredoc_body,
    ACTIONS(6499), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [171991] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6930), 1,
      anon_sym_LBRACE,
    ACTIONS(6932), 1,
      anon_sym_do,
    STATE(4817), 1,
      sym_heredoc_body,
    STATE(5124), 2,
      sym_do_block,
      sym_block,
    ACTIONS(2986), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172020] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6936), 1,
      anon_sym_COMMA,
    STATE(4781), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(4818), 1,
      sym_heredoc_body,
    ACTIONS(6630), 9,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [172047] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4819), 1,
      sym_heredoc_body,
    ACTIONS(3080), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172070] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4820), 1,
      sym_heredoc_body,
    ACTIONS(6892), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6753), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [172095] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4821), 1,
      sym_heredoc_body,
    ACTIONS(6892), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6751), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [172120] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2886), 1,
      sym__string_start,
    STATE(4822), 1,
      sym_heredoc_body,
    STATE(4926), 1,
      sym_keyword_pattern,
    STATE(5813), 1,
      sym_string,
    ACTIONS(2878), 2,
      sym__constant_suffix,
      aux_sym_constant_suffix_token1,
    ACTIONS(2880), 2,
      sym__identifier_suffix,
      aux_sym_identifier_suffix_token1,
    ACTIONS(6580), 2,
      sym_constant,
      sym_identifier,
    STATE(5892), 2,
      sym_constant_suffix,
      sym_identifier_suffix,
  [172155] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(731), 1,
      anon_sym_DOT,
    ACTIONS(743), 1,
      anon_sym_COLON_COLON,
    ACTIONS(4800), 1,
      anon_sym_STAR,
    STATE(4823), 1,
      sym_heredoc_body,
    ACTIONS(4802), 8,
      sym__line_break,
      anon_sym_LPAREN,
      anon_sym_EQ,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      sym_identifier,
  [172184] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4824), 1,
      sym_heredoc_body,
    ACTIONS(2632), 11,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172207] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6938), 1,
      anon_sym_COMMA,
    STATE(4825), 2,
      sym_heredoc_body,
      aux_sym_undef_repeat1,
    ACTIONS(6681), 9,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [172232] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4826), 1,
      sym_heredoc_body,
    ACTIONS(6479), 11,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
      anon_sym_and,
      anon_sym_or,
  [172255] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6941), 1,
      anon_sym_rescue,
    ACTIONS(6943), 1,
      anon_sym_if,
    ACTIONS(6945), 1,
      anon_sym_unless,
    ACTIONS(6947), 1,
      anon_sym_while,
    ACTIONS(6949), 1,
      anon_sym_until,
    ACTIONS(6951), 1,
      anon_sym_do,
    STATE(360), 1,
      sym__terminator,
    STATE(2373), 1,
      sym_do,
    STATE(4827), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [172293] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6953), 1,
      anon_sym_if,
    ACTIONS(6955), 1,
      anon_sym_unless,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(297), 1,
      sym__terminator,
    STATE(4828), 1,
      sym_heredoc_body,
    STATE(5322), 1,
      sym__guard,
    STATE(5544), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5543), 2,
      sym_if_guard,
      sym_unless_guard,
  [172329] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    STATE(4829), 1,
      sym_heredoc_body,
    ACTIONS(6542), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172353] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6961), 1,
      anon_sym_COLON,
    STATE(4830), 1,
      sym_heredoc_body,
    ACTIONS(3080), 9,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172377] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4831), 1,
      sym_heredoc_body,
    ACTIONS(2632), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172399] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6673), 1,
      anon_sym_LBRACE,
    ACTIONS(6963), 1,
      anon_sym_do,
    STATE(4804), 1,
      sym_do_block,
    STATE(4805), 1,
      sym_block,
    STATE(4832), 1,
      sym_heredoc_body,
    ACTIONS(6515), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172429] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4833), 1,
      sym_heredoc_body,
    ACTIONS(2636), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172451] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(269), 1,
      sym__terminator,
    STATE(4834), 1,
      sym_heredoc_body,
    STATE(5297), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [172489] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4835), 1,
      sym_heredoc_body,
    ACTIONS(2624), 3,
      anon_sym_LBRACE,
      anon_sym_do,
      anon_sym_LPAREN2,
    ACTIONS(2620), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172513] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(4836), 1,
      sym_heredoc_body,
    STATE(5013), 1,
      sym_argument_list,
    STATE(5373), 1,
      sym_do_block,
    STATE(5379), 1,
      sym_block,
    ACTIONS(409), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172547] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4837), 1,
      sym_heredoc_body,
    ACTIONS(6965), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6717), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
  [172571] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(273), 1,
      sym__terminator,
    STATE(4838), 1,
      sym_heredoc_body,
    STATE(5310), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [172609] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(264), 1,
      sym__terminator,
    STATE(4839), 1,
      sym_heredoc_body,
    STATE(5300), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [172647] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4840), 1,
      sym_heredoc_body,
    ACTIONS(2636), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [172669] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4841), 1,
      sym_heredoc_body,
    ACTIONS(2628), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172691] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4842), 1,
      sym_heredoc_body,
    ACTIONS(2640), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172713] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6967), 1,
      anon_sym_COMMA,
    STATE(4843), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [172737] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4844), 1,
      sym_heredoc_body,
    ACTIONS(4973), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [172759] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6673), 1,
      anon_sym_LBRACE,
    STATE(4787), 1,
      sym_do_block,
    STATE(4826), 1,
      sym_block,
    STATE(4845), 1,
      sym_heredoc_body,
    ACTIONS(6485), 7,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172787] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4846), 1,
      sym_heredoc_body,
    ACTIONS(6335), 10,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172809] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6970), 1,
      anon_sym_rescue,
    STATE(4847), 1,
      sym_heredoc_body,
    ACTIONS(6594), 9,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [172833] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4848), 1,
      sym_heredoc_body,
    ACTIONS(2644), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172855] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6972), 1,
      anon_sym_COMMA,
    STATE(4843), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4849), 1,
      sym_heredoc_body,
    ACTIONS(6645), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [172881] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6673), 1,
      anon_sym_LBRACE,
    STATE(4804), 1,
      sym_do_block,
    STATE(4805), 1,
      sym_block,
    STATE(4850), 1,
      sym_heredoc_body,
    ACTIONS(6515), 7,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [172909] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4851), 1,
      sym_heredoc_body,
    ACTIONS(743), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172931] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4852), 1,
      sym_heredoc_body,
    ACTIONS(2624), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [172953] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4853), 1,
      sym_heredoc_body,
    ACTIONS(2628), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [172975] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4854), 1,
      sym_heredoc_body,
    ACTIONS(4802), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [172997] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6970), 1,
      anon_sym_rescue,
    STATE(4855), 1,
      sym_heredoc_body,
    ACTIONS(6596), 9,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
      anon_sym_and,
      anon_sym_or,
  [173021] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4856), 1,
      sym_heredoc_body,
    ACTIONS(2640), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [173043] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6941), 1,
      anon_sym_rescue,
    ACTIONS(6943), 1,
      anon_sym_if,
    ACTIONS(6945), 1,
      anon_sym_unless,
    ACTIONS(6947), 1,
      anon_sym_while,
    ACTIONS(6949), 1,
      anon_sym_until,
    ACTIONS(6974), 1,
      anon_sym_do,
    STATE(331), 1,
      sym__terminator,
    STATE(4857), 1,
      sym_heredoc_body,
    STATE(5364), 1,
      sym_do,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173081] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    STATE(4858), 1,
      sym_heredoc_body,
    ACTIONS(6546), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [173105] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6941), 1,
      anon_sym_rescue,
    ACTIONS(6943), 1,
      anon_sym_if,
    ACTIONS(6945), 1,
      anon_sym_unless,
    ACTIONS(6947), 1,
      anon_sym_while,
    ACTIONS(6949), 1,
      anon_sym_until,
    ACTIONS(6974), 1,
      anon_sym_do,
    STATE(331), 1,
      sym__terminator,
    STATE(4859), 1,
      sym_heredoc_body,
    STATE(5367), 1,
      sym_do,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173143] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(272), 1,
      sym__terminator,
    STATE(4860), 1,
      sym_heredoc_body,
    STATE(5305), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173181] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6976), 1,
      anon_sym_rescue,
    ACTIONS(6980), 1,
      anon_sym_if,
    ACTIONS(6982), 1,
      anon_sym_unless,
    ACTIONS(6984), 1,
      anon_sym_while,
    ACTIONS(6986), 1,
      anon_sym_until,
    STATE(2145), 1,
      sym__terminator,
    STATE(4861), 1,
      sym_heredoc_body,
    ACTIONS(1089), 2,
      ts_builtin_sym_end,
      aux_sym_program_token1,
    ACTIONS(6978), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173217] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6972), 1,
      anon_sym_COMMA,
    STATE(4849), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4862), 1,
      sym_heredoc_body,
    ACTIONS(6620), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [173243] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4863), 1,
      sym_heredoc_body,
    ACTIONS(6550), 10,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [173265] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6729), 1,
      anon_sym_EQ2,
    STATE(4864), 1,
      sym_heredoc_body,
    ACTIONS(4800), 2,
      anon_sym_EQ,
      anon_sym_STAR,
    ACTIONS(4802), 7,
      sym__line_break,
      anon_sym_LPAREN,
      anon_sym_SEMI,
      anon_sym_DOT_DOT_DOT,
      anon_sym_STAR_STAR,
      anon_sym_AMP,
      sym_identifier,
  [173291] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4865), 1,
      sym_heredoc_body,
    ACTIONS(6535), 10,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [173313] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4866), 1,
      sym_heredoc_body,
    ACTIONS(6531), 10,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_EQ_GT,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [173335] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(265), 1,
      sym__terminator,
    STATE(4867), 1,
      sym_heredoc_body,
    STATE(5294), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173373] = 11,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6976), 1,
      anon_sym_rescue,
    ACTIONS(6980), 1,
      anon_sym_if,
    ACTIONS(6982), 1,
      anon_sym_unless,
    ACTIONS(6984), 1,
      anon_sym_while,
    ACTIONS(6986), 1,
      anon_sym_until,
    STATE(2145), 1,
      sym__terminator,
    STATE(4868), 1,
      sym_heredoc_body,
    ACTIONS(6818), 2,
      ts_builtin_sym_end,
      aux_sym_program_token1,
    ACTIONS(6978), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173409] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6941), 1,
      anon_sym_rescue,
    ACTIONS(6943), 1,
      anon_sym_if,
    ACTIONS(6945), 1,
      anon_sym_unless,
    ACTIONS(6947), 1,
      anon_sym_while,
    ACTIONS(6949), 1,
      anon_sym_until,
    ACTIONS(6988), 1,
      anon_sym_do,
    STATE(334), 1,
      sym__terminator,
    STATE(2981), 1,
      sym_do,
    STATE(4869), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173447] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6941), 1,
      anon_sym_rescue,
    ACTIONS(6943), 1,
      anon_sym_if,
    ACTIONS(6945), 1,
      anon_sym_unless,
    ACTIONS(6947), 1,
      anon_sym_while,
    ACTIONS(6949), 1,
      anon_sym_until,
    ACTIONS(6951), 1,
      anon_sym_do,
    STATE(360), 1,
      sym__terminator,
    STATE(2372), 1,
      sym_do,
    STATE(4870), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173485] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4871), 1,
      sym_heredoc_body,
    ACTIONS(6965), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6753), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
  [173509] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4872), 1,
      sym_heredoc_body,
    ACTIONS(6965), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6751), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
  [173533] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4873), 1,
      sym_heredoc_body,
    ACTIONS(2644), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [173555] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(267), 1,
      sym__terminator,
    STATE(4874), 1,
      sym_heredoc_body,
    STATE(5290), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173593] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4875), 1,
      sym_heredoc_body,
    ACTIONS(2616), 10,
      sym__line_break,
      ts_builtin_sym_end,
      aux_sym_program_token1,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
  [173615] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(266), 1,
      sym__terminator,
    STATE(4876), 1,
      sym_heredoc_body,
    STATE(5321), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173653] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4877), 1,
      sym_heredoc_body,
    ACTIONS(6965), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6755), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
  [173677] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4878), 1,
      sym_heredoc_body,
    ACTIONS(6965), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6757), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
  [173701] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6941), 1,
      anon_sym_rescue,
    ACTIONS(6943), 1,
      anon_sym_if,
    ACTIONS(6945), 1,
      anon_sym_unless,
    ACTIONS(6947), 1,
      anon_sym_while,
    ACTIONS(6949), 1,
      anon_sym_until,
    ACTIONS(6988), 1,
      anon_sym_do,
    STATE(334), 1,
      sym__terminator,
    STATE(2982), 1,
      sym_do,
    STATE(4879), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [173739] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4880), 1,
      sym_heredoc_body,
    ACTIONS(6965), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6763), 8,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_if,
      anon_sym_unless,
      anon_sym_while,
      anon_sym_until,
      anon_sym_do,
  [173763] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6673), 1,
      anon_sym_LBRACE,
    ACTIONS(6963), 1,
      anon_sym_do,
    STATE(4787), 1,
      sym_do_block,
    STATE(4826), 1,
      sym_block,
    STATE(4881), 1,
      sym_heredoc_body,
    ACTIONS(6485), 6,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [173793] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4882), 1,
      sym_heredoc_body,
    ACTIONS(2616), 10,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_LPAREN2,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [173815] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6992), 1,
      aux_sym_string_array_token1,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(6998), 1,
      sym__string_end,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    STATE(4883), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5512), 1,
      aux_sym_symbol_array_repeat1,
  [173852] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7002), 1,
      aux_sym_string_array_token1,
    ACTIONS(7004), 1,
      sym__string_end,
    STATE(4884), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5563), 1,
      aux_sym_string_array_repeat1,
  [173889] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7006), 1,
      anon_sym_end,
    STATE(4885), 1,
      sym_heredoc_body,
    STATE(5246), 1,
      aux_sym_case_repeat1,
    STATE(5252), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5967), 1,
      sym_else,
  [173926] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7008), 1,
      anon_sym_COMMA,
    STATE(4886), 1,
      sym_heredoc_body,
    STATE(4895), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [173951] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7010), 1,
      aux_sym_string_array_token1,
    ACTIONS(7012), 1,
      sym__string_end,
    STATE(4887), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5574), 1,
      aux_sym_string_array_repeat1,
  [173988] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7014), 1,
      aux_sym_string_array_token1,
    ACTIONS(7016), 1,
      sym__string_end,
    STATE(4888), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5568), 1,
      aux_sym_string_array_repeat1,
  [174025] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    ACTIONS(7018), 1,
      anon_sym_COMMA,
    STATE(4889), 1,
      sym_heredoc_body,
    ACTIONS(6576), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174050] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7020), 1,
      anon_sym_end,
    STATE(4890), 1,
      sym_heredoc_body,
    STATE(5273), 1,
      aux_sym_case_repeat1,
    STATE(5282), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5869), 1,
      sym_else,
  [174087] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7022), 1,
      anon_sym_COMMA,
    STATE(4891), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 7,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [174110] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7025), 1,
      aux_sym_string_array_token1,
    ACTIONS(7027), 1,
      sym__string_end,
    STATE(4892), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5546), 1,
      aux_sym_string_array_repeat1,
  [174147] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7029), 1,
      anon_sym_COMMA,
    STATE(4893), 1,
      sym_heredoc_body,
    STATE(4897), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6645), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174172] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    STATE(4894), 1,
      sym_heredoc_body,
    ACTIONS(6548), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174195] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7008), 1,
      anon_sym_COMMA,
    STATE(4895), 1,
      sym_heredoc_body,
    STATE(4923), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6630), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174220] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7031), 1,
      anon_sym_COMMA,
    STATE(4891), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4896), 1,
      sym_heredoc_body,
    ACTIONS(6645), 7,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [174245] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7033), 1,
      anon_sym_COMMA,
    STATE(4897), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174268] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    ACTIONS(7018), 1,
      anon_sym_COMMA,
    STATE(4898), 1,
      sym_heredoc_body,
    ACTIONS(6639), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174293] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4899), 1,
      sym_heredoc_body,
    ACTIONS(6556), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174314] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7036), 1,
      aux_sym_string_array_token1,
    ACTIONS(7038), 1,
      sym__string_end,
    STATE(4900), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5513), 1,
      aux_sym_string_array_repeat1,
  [174351] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7040), 1,
      aux_sym_string_array_token1,
    ACTIONS(7042), 1,
      sym__string_end,
    STATE(4901), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5523), 1,
      aux_sym_string_array_repeat1,
  [174388] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4902), 1,
      sym_heredoc_body,
    ACTIONS(6584), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174409] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7044), 1,
      aux_sym_string_array_token1,
    ACTIONS(7046), 1,
      sym__string_end,
    STATE(4903), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5517), 1,
      aux_sym_symbol_array_repeat1,
  [174446] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7029), 1,
      anon_sym_COMMA,
    STATE(4893), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4904), 1,
      sym_heredoc_body,
    ACTIONS(6620), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174471] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4905), 1,
      sym_heredoc_body,
    ACTIONS(5469), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174492] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7048), 1,
      aux_sym_string_array_token1,
    ACTIONS(7050), 1,
      sym__string_end,
    STATE(4906), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5540), 1,
      aux_sym_symbol_array_repeat1,
  [174529] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7052), 1,
      aux_sym_string_array_token1,
    ACTIONS(7054), 1,
      sym__string_end,
    STATE(4907), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5521), 1,
      aux_sym_symbol_array_repeat1,
  [174566] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7056), 1,
      anon_sym_end,
    STATE(4908), 1,
      sym_heredoc_body,
    STATE(5261), 1,
      aux_sym_case_repeat1,
    STATE(5262), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5735), 1,
      sym_else,
  [174603] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7058), 1,
      aux_sym_string_array_token1,
    ACTIONS(7060), 1,
      sym__string_end,
    STATE(4909), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5504), 1,
      aux_sym_string_array_repeat1,
  [174640] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7062), 1,
      aux_sym_string_array_token1,
    ACTIONS(7064), 1,
      sym__string_end,
    STATE(4910), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5498), 1,
      aux_sym_symbol_array_repeat1,
  [174677] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7066), 1,
      aux_sym_string_array_token1,
    ACTIONS(7068), 1,
      sym__string_end,
    STATE(4911), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5524), 1,
      aux_sym_symbol_array_repeat1,
  [174714] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7070), 1,
      anon_sym_COMMA,
    STATE(4912), 2,
      sym_heredoc_body,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6582), 7,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174737] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7073), 1,
      aux_sym_string_array_token1,
    ACTIONS(7075), 1,
      sym__string_end,
    STATE(4913), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5520), 1,
      aux_sym_string_array_repeat1,
  [174774] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7077), 1,
      anon_sym_end,
    STATE(4914), 1,
      sym_heredoc_body,
    STATE(5231), 1,
      aux_sym_case_repeat1,
    STATE(5232), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5824), 1,
      sym_else,
  [174811] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7079), 1,
      aux_sym_string_array_token1,
    ACTIONS(7081), 1,
      sym__string_end,
    STATE(4915), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5500), 1,
      aux_sym_symbol_array_repeat1,
  [174848] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7083), 1,
      anon_sym_COMMA,
    STATE(4912), 1,
      aux_sym__hash_pattern_body_repeat1,
    STATE(4916), 1,
      sym_heredoc_body,
    ACTIONS(5475), 7,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [174873] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7085), 1,
      aux_sym_string_array_token1,
    ACTIONS(7087), 1,
      sym__string_end,
    STATE(4917), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5541), 1,
      aux_sym_string_array_repeat1,
  [174910] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7089), 1,
      anon_sym_rescue,
    STATE(4918), 1,
      sym_heredoc_body,
    ACTIONS(6594), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [174933] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7091), 1,
      aux_sym_string_array_token1,
    ACTIONS(7093), 1,
      sym__string_end,
    STATE(4919), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5551), 1,
      aux_sym_symbol_array_repeat1,
  [174970] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7095), 1,
      aux_sym_string_array_token1,
    ACTIONS(7097), 1,
      sym__string_end,
    STATE(4920), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5497), 1,
      aux_sym_string_array_repeat1,
  [175007] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7099), 1,
      anon_sym_end,
    STATE(4921), 1,
      sym_heredoc_body,
    STATE(5230), 1,
      aux_sym_case_repeat1,
    STATE(5233), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5879), 1,
      sym_else,
  [175044] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7101), 1,
      aux_sym_string_array_token1,
    ACTIONS(7103), 1,
      sym__string_end,
    STATE(4922), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5572), 1,
      aux_sym_symbol_array_repeat1,
  [175081] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7105), 1,
      anon_sym_COMMA,
    STATE(4923), 2,
      sym_heredoc_body,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6548), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175104] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7089), 1,
      anon_sym_rescue,
    STATE(4924), 1,
      sym_heredoc_body,
    ACTIONS(6596), 8,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [175127] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7031), 1,
      anon_sym_COMMA,
    STATE(4896), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(4925), 1,
      sym_heredoc_body,
    ACTIONS(6620), 7,
      sym__line_break,
      anon_sym_LBRACE,
      anon_sym_SEMI,
      anon_sym_do,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [175152] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4926), 1,
      sym_heredoc_body,
    ACTIONS(6582), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175173] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4927), 1,
      sym_heredoc_body,
    ACTIONS(6568), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175194] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7108), 1,
      aux_sym_string_array_token1,
    ACTIONS(7110), 1,
      sym__string_end,
    STATE(4928), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5562), 1,
      aux_sym_symbol_array_repeat1,
  [175231] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7112), 1,
      anon_sym_COMMA,
    STATE(4929), 1,
      sym_heredoc_body,
    STATE(4934), 1,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6568), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175256] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7114), 1,
      anon_sym_COMMA,
    STATE(4916), 1,
      aux_sym__hash_pattern_body_repeat1,
    STATE(4930), 1,
      sym_heredoc_body,
    ACTIONS(6568), 7,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175281] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7116), 1,
      anon_sym_COMMA,
    STATE(4931), 2,
      sym_heredoc_body,
      aux_sym__hash_pattern_body_repeat1,
    ACTIONS(6582), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175304] = 12,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7119), 1,
      anon_sym_end,
    STATE(4932), 1,
      sym_heredoc_body,
    STATE(5215), 1,
      aux_sym_case_repeat1,
    STATE(5216), 1,
      aux_sym_case_match_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5977), 1,
      sym_else,
  [175341] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4933), 1,
      sym_heredoc_body,
    ACTIONS(6564), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175362] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7121), 1,
      anon_sym_COMMA,
    STATE(4931), 1,
      aux_sym__hash_pattern_body_repeat1,
    STATE(4934), 1,
      sym_heredoc_body,
    ACTIONS(5475), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175387] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4935), 1,
      sym_heredoc_body,
    ACTIONS(6578), 9,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175408] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7008), 1,
      anon_sym_COMMA,
    STATE(4936), 1,
      sym_heredoc_body,
    STATE(4937), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175433] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7123), 1,
      anon_sym_COMMA,
    STATE(4923), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(4937), 1,
      sym_heredoc_body,
    ACTIONS(6630), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175458] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7125), 1,
      anon_sym_COMMA,
    STATE(4938), 1,
      sym_heredoc_body,
    STATE(5006), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6630), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175482] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7127), 1,
      aux_sym_string_array_token1,
    ACTIONS(7129), 1,
      sym__string_end,
    STATE(4939), 1,
      sym_heredoc_body,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
  [175516] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7131), 1,
      anon_sym_rescue,
    STATE(4940), 1,
      sym_heredoc_body,
    ACTIONS(6594), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175538] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7131), 1,
      anon_sym_rescue,
    STATE(4941), 1,
      sym_heredoc_body,
    ACTIONS(6596), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175560] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4942), 1,
      sym_heredoc_body,
    ACTIONS(121), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175580] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4943), 1,
      sym_heredoc_body,
    ACTIONS(4350), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175600] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7133), 1,
      anon_sym_end,
    ACTIONS(7135), 1,
      anon_sym_rescue,
    ACTIONS(7137), 1,
      anon_sym_ensure,
    STATE(4944), 1,
      sym_heredoc_body,
    STATE(4995), 1,
      aux_sym__body_statement_repeat1,
    STATE(5404), 3,
      sym_else,
      sym_ensure,
      sym_rescue,
  [175630] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4945), 1,
      sym_heredoc_body,
    ACTIONS(6485), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175650] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4946), 1,
      sym_heredoc_body,
    ACTIONS(3086), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175670] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7139), 1,
      anon_sym_COMMA,
    STATE(4947), 1,
      sym_heredoc_body,
    STATE(5034), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6620), 6,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [175694] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4948), 1,
      sym_heredoc_body,
    ACTIONS(6527), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175714] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4949), 1,
      sym_heredoc_body,
    ACTIONS(3052), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175734] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    STATE(2053), 1,
      sym__terminator,
    STATE(4950), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [175766] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4951), 1,
      sym_heredoc_body,
    ACTIONS(3056), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175786] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4952), 1,
      sym_heredoc_body,
    ACTIONS(3064), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175806] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4953), 1,
      sym_heredoc_body,
    ACTIONS(3000), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175826] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7141), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7144), 1,
      aux_sym_string_array_token1,
    ACTIONS(7146), 1,
      sym_escape_sequence,
    ACTIONS(7149), 1,
      sym_string_content,
    ACTIONS(7152), 1,
      sym__string_end,
    ACTIONS(7154), 1,
      sym__short_interpolation,
    STATE(5234), 1,
      sym_interpolation,
    STATE(4954), 2,
      sym_heredoc_body,
      aux_sym__literal_contents,
  [175858] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4955), 1,
      sym_heredoc_body,
    ACTIONS(3082), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175878] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7157), 1,
      anon_sym_rescue,
    STATE(4956), 1,
      sym_heredoc_body,
    ACTIONS(6594), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175900] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7157), 1,
      anon_sym_rescue,
    STATE(4957), 1,
      sym_heredoc_body,
    ACTIONS(6596), 7,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [175922] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(7159), 1,
      anon_sym_in,
    STATE(4958), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    STATE(5324), 1,
      sym_in,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175950] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4959), 1,
      sym_heredoc_body,
    ACTIONS(3016), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175970] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4960), 1,
      sym_heredoc_body,
    ACTIONS(3020), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [175990] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4961), 1,
      sym_heredoc_body,
    ACTIONS(3024), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176010] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4962), 1,
      sym_heredoc_body,
    ACTIONS(3028), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176030] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4963), 1,
      sym_heredoc_body,
    ACTIONS(3032), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176050] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    ACTIONS(7161), 1,
      anon_sym_COMMA,
    STATE(4964), 1,
      sym_heredoc_body,
    ACTIONS(6576), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176074] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4965), 1,
      sym_heredoc_body,
    ACTIONS(3072), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176094] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4966), 1,
      sym_heredoc_body,
    ACTIONS(6598), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176114] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4967), 1,
      sym_heredoc_body,
    ACTIONS(3036), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176134] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4968), 1,
      sym_heredoc_body,
    ACTIONS(3040), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176154] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4969), 1,
      sym_heredoc_body,
    ACTIONS(3044), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176174] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4970), 1,
      sym_heredoc_body,
    ACTIONS(3048), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176194] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7163), 1,
      aux_sym_string_array_token1,
    ACTIONS(7165), 1,
      sym__string_end,
    STATE(4913), 1,
      aux_sym__literal_contents,
    STATE(4971), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176228] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7125), 1,
      anon_sym_COMMA,
    STATE(4972), 1,
      sym_heredoc_body,
    STATE(5028), 1,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6624), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176252] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4973), 1,
      sym_heredoc_body,
    ACTIONS(6592), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176272] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4974), 1,
      sym_heredoc_body,
    ACTIONS(6602), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176292] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    ACTIONS(7161), 1,
      anon_sym_COMMA,
    STATE(4975), 1,
      sym_heredoc_body,
    ACTIONS(6639), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176316] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4976), 1,
      sym_heredoc_body,
    ACTIONS(6560), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176336] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4977), 1,
      sym_heredoc_body,
    ACTIONS(6566), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176356] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7167), 1,
      anon_sym_COMMA,
    STATE(4978), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 6,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [176378] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7170), 1,
      aux_sym_string_array_token1,
    ACTIONS(7172), 1,
      sym__string_end,
    STATE(4887), 1,
      aux_sym__literal_contents,
    STATE(4979), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176412] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7174), 1,
      aux_sym_string_array_token1,
    ACTIONS(7176), 1,
      sym__string_end,
    STATE(4915), 1,
      aux_sym__literal_contents,
    STATE(4980), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176446] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7178), 1,
      aux_sym_string_array_token1,
    ACTIONS(7180), 1,
      sym__string_end,
    STATE(4911), 1,
      aux_sym__literal_contents,
    STATE(4981), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176480] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7182), 1,
      aux_sym_string_array_token1,
    ACTIONS(7184), 1,
      sym__string_end,
    STATE(4888), 1,
      aux_sym__literal_contents,
    STATE(4982), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176514] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7186), 1,
      aux_sym_string_array_token1,
    ACTIONS(7188), 1,
      sym__string_end,
    STATE(4917), 1,
      aux_sym__literal_contents,
    STATE(4983), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176548] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7190), 1,
      aux_sym_string_array_token1,
    ACTIONS(7192), 1,
      sym__string_end,
    STATE(4901), 1,
      aux_sym__literal_contents,
    STATE(4984), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176582] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4985), 1,
      sym_heredoc_body,
    ACTIONS(6570), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176602] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7194), 1,
      aux_sym_string_array_token1,
    ACTIONS(7196), 1,
      sym__string_end,
    STATE(4919), 1,
      aux_sym__literal_contents,
    STATE(4986), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176636] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7198), 1,
      aux_sym_string_array_token1,
    ACTIONS(7200), 1,
      sym__string_end,
    STATE(4906), 1,
      aux_sym__literal_contents,
    STATE(4987), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176670] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(4988), 1,
      sym_heredoc_body,
    ACTIONS(2960), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [176690] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4989), 1,
      sym_heredoc_body,
    ACTIONS(6572), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176710] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4990), 1,
      sym_heredoc_body,
    ACTIONS(6554), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176730] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7202), 1,
      aux_sym_string_array_token1,
    ACTIONS(7204), 1,
      sym__string_end,
    STATE(4920), 1,
      aux_sym__literal_contents,
    STATE(4991), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176764] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7206), 1,
      aux_sym_string_array_token1,
    ACTIONS(7208), 1,
      sym__string_end,
    STATE(4910), 1,
      aux_sym__literal_contents,
    STATE(4992), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176798] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7210), 1,
      aux_sym_string_array_token1,
    ACTIONS(7212), 1,
      sym__string_end,
    STATE(4903), 1,
      aux_sym__literal_contents,
    STATE(4993), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176832] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4994), 1,
      sym_heredoc_body,
    ACTIONS(6594), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176852] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7135), 1,
      anon_sym_rescue,
    ACTIONS(7137), 1,
      anon_sym_ensure,
    ACTIONS(7214), 1,
      anon_sym_end,
    STATE(4995), 1,
      sym_heredoc_body,
    STATE(4998), 1,
      aux_sym__body_statement_repeat1,
    STATE(5404), 3,
      sym_else,
      sym_ensure,
      sym_rescue,
  [176882] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7216), 1,
      anon_sym_COMMA,
    STATE(4996), 1,
      sym_heredoc_body,
    STATE(5014), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6620), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176906] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4997), 1,
      sym_heredoc_body,
    ACTIONS(6596), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [176926] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7218), 1,
      anon_sym_end,
    ACTIONS(7220), 1,
      anon_sym_rescue,
    ACTIONS(7223), 1,
      anon_sym_else,
    ACTIONS(7226), 1,
      anon_sym_ensure,
    STATE(4998), 2,
      sym_heredoc_body,
      aux_sym__body_statement_repeat1,
    STATE(5404), 3,
      sym_else,
      sym_ensure,
      sym_rescue,
  [176954] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7229), 1,
      aux_sym_string_array_token1,
    ACTIONS(7231), 1,
      sym__string_end,
    STATE(4892), 1,
      aux_sym__literal_contents,
    STATE(4999), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [176988] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7233), 1,
      aux_sym_string_array_token1,
    ACTIONS(7235), 1,
      sym__string_end,
    STATE(4928), 1,
      aux_sym__literal_contents,
    STATE(5000), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177022] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7133), 1,
      anon_sym_end,
    ACTIONS(7135), 1,
      anon_sym_rescue,
    ACTIONS(7137), 1,
      anon_sym_ensure,
    STATE(4998), 1,
      aux_sym__body_statement_repeat1,
    STATE(5001), 1,
      sym_heredoc_body,
    STATE(5404), 3,
      sym_else,
      sym_ensure,
      sym_rescue,
  [177052] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5002), 1,
      sym_heredoc_body,
    ACTIONS(6554), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177072] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5003), 1,
      sym_heredoc_body,
    ACTIONS(6562), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177092] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7237), 1,
      aux_sym_string_array_token1,
    ACTIONS(7239), 1,
      sym__string_end,
    STATE(4907), 1,
      aux_sym__literal_contents,
    STATE(5004), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177126] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5005), 1,
      sym_heredoc_body,
    ACTIONS(6574), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177146] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7241), 1,
      anon_sym_COMMA,
    STATE(5006), 2,
      sym_heredoc_body,
      aux_sym__array_pattern_n_repeat1,
    ACTIONS(6548), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177168] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7125), 1,
      anon_sym_COMMA,
    STATE(4938), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(5007), 1,
      sym_heredoc_body,
    ACTIONS(6624), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177192] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7244), 1,
      aux_sym_string_array_token1,
    ACTIONS(7246), 1,
      sym__string_end,
    STATE(4922), 1,
      aux_sym__literal_contents,
    STATE(5008), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177226] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5009), 1,
      sym_heredoc_body,
    ACTIONS(6576), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177246] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5010), 1,
      sym_heredoc_body,
    ACTIONS(6600), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177266] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5011), 1,
      sym_heredoc_body,
    ACTIONS(6612), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177286] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(5012), 1,
      sym_heredoc_body,
    STATE(5428), 1,
      sym_do_block,
    STATE(5431), 1,
      sym_block,
    ACTIONS(3309), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177314] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(5013), 1,
      sym_heredoc_body,
    STATE(5486), 1,
      sym_do_block,
    STATE(5493), 1,
      sym_block,
    ACTIONS(3315), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177342] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7216), 1,
      anon_sym_COMMA,
    STATE(5014), 1,
      sym_heredoc_body,
    STATE(5021), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6645), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177366] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5015), 1,
      sym_heredoc_body,
    ACTIONS(6558), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177386] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5016), 1,
      sym_heredoc_body,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(7248), 4,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
  [177408] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5017), 1,
      sym_heredoc_body,
    ACTIONS(6558), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177428] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5018), 1,
      sym_heredoc_body,
    ACTIONS(6588), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177448] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5019), 1,
      sym_heredoc_body,
    ACTIONS(6558), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177468] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5020), 1,
      sym_heredoc_body,
    ACTIONS(6558), 8,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_COMMA,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177488] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7250), 1,
      anon_sym_COMMA,
    STATE(5021), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177510] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5022), 1,
      sym_heredoc_body,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
    ACTIONS(7253), 4,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
  [177532] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5023), 1,
      sym_heredoc_body,
    ACTIONS(3008), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177552] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5024), 1,
      sym_heredoc_body,
    ACTIONS(3012), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177572] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7255), 1,
      aux_sym_string_array_token1,
    ACTIONS(7257), 1,
      sym__string_end,
    STATE(4884), 1,
      aux_sym__literal_contents,
    STATE(5025), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177606] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7259), 1,
      aux_sym_string_array_token1,
    ACTIONS(7261), 1,
      sym__string_end,
    STATE(4883), 1,
      aux_sym__literal_contents,
    STATE(5026), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177640] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7263), 1,
      aux_sym_string_array_token1,
    ACTIONS(7265), 1,
      sym__string_end,
    STATE(4909), 1,
      aux_sym__literal_contents,
    STATE(5027), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177674] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7267), 1,
      anon_sym_COMMA,
    STATE(5006), 1,
      aux_sym__array_pattern_n_repeat1,
    STATE(5028), 1,
      sym_heredoc_body,
    ACTIONS(6630), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_do,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [177698] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(5029), 1,
      sym_heredoc_body,
    STATE(5422), 1,
      sym_do_block,
    STATE(5424), 1,
      sym_block,
    ACTIONS(3323), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177726] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(7159), 1,
      anon_sym_in,
    STATE(5030), 1,
      sym_heredoc_body,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    STATE(5308), 1,
      sym_in,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177754] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5031), 1,
      sym_heredoc_body,
    ACTIONS(3060), 8,
      sym__element_reference_bracket,
      anon_sym_RPAREN,
      anon_sym_DOT,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177774] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    ACTIONS(7159), 1,
      anon_sym_in,
    STATE(5032), 1,
      sym_heredoc_body,
    STATE(5299), 1,
      sym_in,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [177802] = 10,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6767), 1,
      anon_sym_rescue,
    ACTIONS(6769), 1,
      anon_sym_if,
    ACTIONS(6771), 1,
      anon_sym_unless,
    ACTIONS(6773), 1,
      anon_sym_while,
    ACTIONS(6775), 1,
      anon_sym_until,
    STATE(2145), 1,
      sym__terminator,
    STATE(5033), 1,
      sym_heredoc_body,
    ACTIONS(6978), 2,
      sym__line_break,
      anon_sym_SEMI,
  [177834] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7139), 1,
      anon_sym_COMMA,
    STATE(4978), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(5034), 1,
      sym_heredoc_body,
    ACTIONS(6645), 6,
      sym__line_break,
      anon_sym_rescue,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [177858] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7269), 1,
      aux_sym_string_array_token1,
    ACTIONS(7271), 1,
      sym__string_end,
    STATE(4954), 1,
      aux_sym__literal_contents,
    STATE(5035), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177892] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(6994), 1,
      sym_escape_sequence,
    ACTIONS(6996), 1,
      sym_string_content,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7273), 1,
      aux_sym_string_array_token1,
    ACTIONS(7275), 1,
      sym__string_end,
    STATE(4900), 1,
      aux_sym__literal_contents,
    STATE(5036), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
  [177926] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(2958), 1,
      sym__simple_numeric,
    STATE(5037), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [177953] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7281), 1,
      sym__string_end,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    STATE(5038), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [177982] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7285), 1,
      sym__string_end,
    STATE(5039), 1,
      sym_heredoc_body,
    STATE(5182), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178011] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5040), 1,
      sym_heredoc_body,
    ACTIONS(3036), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178030] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7287), 1,
      anon_sym_COMMA,
    STATE(5041), 1,
      sym_heredoc_body,
    STATE(5126), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6620), 5,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [178053] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7289), 1,
      sym__string_end,
    STATE(5042), 1,
      sym_heredoc_body,
    STATE(5089), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178082] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7291), 1,
      sym__string_end,
    STATE(5043), 1,
      sym_heredoc_body,
    STATE(5091), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178111] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7293), 1,
      sym__string_end,
    STATE(5044), 1,
      sym_heredoc_body,
    STATE(5092), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178140] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7295), 1,
      sym__string_end,
    STATE(5045), 1,
      sym_heredoc_body,
    STATE(5094), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178169] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5046), 1,
      sym_heredoc_body,
    ACTIONS(7297), 7,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_ensure,
  [178188] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7299), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5047), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [178217] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7301), 1,
      sym__string_end,
    STATE(5048), 1,
      sym_heredoc_body,
    STATE(5167), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178246] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7303), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7307), 1,
      sym_heredoc_end,
    ACTIONS(7309), 1,
      sym__short_interpolation,
    STATE(5049), 1,
      sym_heredoc_body,
    STATE(5147), 1,
      aux_sym_heredoc_body_repeat1,
    STATE(5303), 1,
      sym_interpolation,
    ACTIONS(7305), 2,
      sym_heredoc_content,
      sym_escape_sequence,
  [178275] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7311), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5050), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [178304] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6957), 1,
      anon_sym_then,
    ACTIONS(7313), 1,
      anon_sym_COMMA,
    STATE(301), 1,
      sym__terminator,
    STATE(5051), 1,
      sym_heredoc_body,
    STATE(5133), 1,
      aux_sym_when_repeat1,
    STATE(5576), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [178333] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5052), 1,
      sym_heredoc_body,
    ACTIONS(3024), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178352] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3385), 1,
      sym__pow,
    STATE(4040), 1,
      sym__simple_numeric,
    STATE(5053), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [178379] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(3551), 1,
      sym__simple_numeric,
    STATE(5054), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [178406] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(3239), 1,
      sym__simple_numeric,
    STATE(5055), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [178433] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3368), 1,
      sym__simple_numeric,
    STATE(3385), 1,
      sym__pow,
    STATE(5056), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [178460] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5057), 1,
      sym_heredoc_body,
    ACTIONS(3040), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178479] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5058), 1,
      sym_heredoc_body,
    ACTIONS(3086), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178498] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5059), 1,
      sym_heredoc_body,
    ACTIONS(3044), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178517] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2540), 1,
      anon_sym_EQ_GT,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(281), 1,
      sym__terminator,
    STATE(5060), 1,
      sym_heredoc_body,
    STATE(5325), 1,
      sym_exception_variable,
    STATE(5482), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [178546] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3385), 1,
      sym__pow,
    STATE(3940), 1,
      sym__simple_numeric,
    STATE(5061), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [178573] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7315), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5062), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [178602] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7317), 1,
      sym__string_end,
    STATE(5063), 1,
      sym_heredoc_body,
    STATE(5070), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178631] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7319), 1,
      sym__string_end,
    STATE(5064), 1,
      sym_heredoc_body,
    STATE(5071), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178660] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7321), 1,
      sym__string_end,
    STATE(5065), 1,
      sym_heredoc_body,
    STATE(5072), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178689] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7323), 1,
      sym__string_end,
    STATE(5066), 1,
      sym_heredoc_body,
    STATE(5073), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178718] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5067), 1,
      sym_heredoc_body,
    ACTIONS(3048), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178737] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5068), 1,
      sym_heredoc_body,
    ACTIONS(3028), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178756] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(3518), 1,
      sym__simple_numeric,
    STATE(5069), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [178783] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7325), 1,
      sym__string_end,
    STATE(5070), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178812] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7327), 1,
      sym__string_end,
    STATE(5071), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178841] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7329), 1,
      sym__string_end,
    STATE(5072), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178870] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7331), 1,
      sym__string_end,
    STATE(5073), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178899] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5074), 1,
      sym_heredoc_body,
    ACTIONS(3020), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178918] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7303), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7309), 1,
      sym__short_interpolation,
    ACTIONS(7333), 1,
      sym_heredoc_end,
    STATE(5049), 1,
      aux_sym_heredoc_body_repeat1,
    STATE(5075), 1,
      sym_heredoc_body,
    STATE(5303), 1,
      sym_interpolation,
    ACTIONS(7305), 2,
      sym_heredoc_content,
      sym_escape_sequence,
  [178947] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5076), 1,
      sym_heredoc_body,
    ACTIONS(3052), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [178966] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7335), 1,
      sym__string_end,
    STATE(5077), 1,
      sym_heredoc_body,
    STATE(5151), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [178995] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7337), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5078), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179024] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7339), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5079), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179053] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7341), 1,
      sym__string_end,
    STATE(5080), 1,
      sym_heredoc_body,
    STATE(5102), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179082] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7343), 1,
      anon_sym_COMMA,
    STATE(5081), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 5,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [179103] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2720), 1,
      sym__string_start,
    STATE(5082), 1,
      sym_heredoc_body,
    STATE(5122), 1,
      aux_sym_chained_string_repeat1,
    STATE(5291), 1,
      sym_string,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [179128] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(3870), 1,
      sym__simple_numeric,
    STATE(5083), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [179155] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7346), 1,
      sym__string_end,
    STATE(5084), 1,
      sym_heredoc_body,
    STATE(5096), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179184] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7348), 1,
      sym__string_end,
    STATE(5085), 1,
      sym_heredoc_body,
    STATE(5112), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179213] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7350), 1,
      sym__string_end,
    STATE(5086), 1,
      sym_heredoc_body,
    STATE(5135), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179242] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3270), 1,
      sym__simple_numeric,
    STATE(5087), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [179269] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7352), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5088), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179298] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7354), 1,
      sym__string_end,
    STATE(5089), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179327] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7356), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5090), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179356] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7358), 1,
      sym__string_end,
    STATE(5091), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179385] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7360), 1,
      sym__string_end,
    STATE(5092), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179414] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3328), 1,
      sym__simple_numeric,
    STATE(3385), 1,
      sym__pow,
    STATE(5093), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [179441] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7362), 1,
      sym__string_end,
    STATE(5094), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179470] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5095), 1,
      sym_heredoc_body,
    ACTIONS(7364), 7,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_in,
      anon_sym_when,
      anon_sym_elsif,
      anon_sym_else,
      anon_sym_ensure,
  [179489] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7366), 1,
      sym__string_end,
    STATE(5096), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179518] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3385), 1,
      sym__pow,
    STATE(3663), 1,
      sym__simple_numeric,
    STATE(5097), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [179545] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3278), 1,
      anon_sym_r,
    STATE(5098), 1,
      sym_heredoc_body,
    ACTIONS(3280), 2,
      anon_sym_i,
      anon_sym_ri,
    ACTIONS(3274), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [179568] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7368), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5099), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179597] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5100), 1,
      sym_heredoc_body,
    ACTIONS(3032), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [179616] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7370), 1,
      sym__string_end,
    STATE(5101), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179645] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7372), 1,
      sym__string_end,
    STATE(5102), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179674] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(2636), 1,
      sym__simple_numeric,
    STATE(5103), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [179701] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5104), 1,
      sym_heredoc_body,
    ACTIONS(3060), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [179720] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7374), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5105), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179749] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3304), 1,
      sym__simple_numeric,
    STATE(3385), 1,
      sym__pow,
    STATE(5106), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [179776] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7376), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5107), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179805] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7378), 1,
      sym__string_end,
    STATE(5108), 1,
      sym_heredoc_body,
    STATE(5161), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179834] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7380), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5109), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179863] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7382), 1,
      sym__string_end,
    STATE(5110), 1,
      sym_heredoc_body,
    STATE(5162), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179892] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7384), 1,
      sym__string_end,
    STATE(5111), 1,
      sym_heredoc_body,
    STATE(5145), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179921] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7386), 1,
      sym__string_end,
    STATE(5112), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [179950] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7388), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5113), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [179979] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5114), 1,
      sym_heredoc_body,
    ACTIONS(3082), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [179998] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3385), 1,
      sym__pow,
    STATE(3834), 1,
      sym__simple_numeric,
    STATE(5115), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [180025] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5116), 1,
      sym_heredoc_body,
    ACTIONS(3056), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [180044] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7390), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5117), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180073] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5118), 1,
      sym_heredoc_body,
    ACTIONS(7392), 3,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [180094] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7394), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5119), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180123] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7396), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5120), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180152] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7398), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5121), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180181] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2720), 1,
      sym__string_start,
    STATE(5122), 1,
      sym_heredoc_body,
    STATE(5132), 1,
      aux_sym_chained_string_repeat1,
    STATE(5291), 1,
      sym_string,
    ACTIONS(3270), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [180206] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7400), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5123), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180235] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5124), 1,
      sym_heredoc_body,
    ACTIONS(3072), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [180254] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3385), 1,
      sym__pow,
    STATE(3915), 1,
      sym__simple_numeric,
    STATE(5125), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [180281] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7287), 1,
      anon_sym_COMMA,
    STATE(5081), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(5126), 1,
      sym_heredoc_body,
    ACTIONS(6645), 5,
      anon_sym_RPAREN,
      anon_sym_rescue,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [180304] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7402), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5127), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180333] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7404), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5128), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180362] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7406), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5129), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180391] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5130), 1,
      sym_heredoc_body,
    ACTIONS(3064), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [180410] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3680), 1,
      sym__simple_numeric,
    STATE(5131), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [180437] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7408), 1,
      sym__string_start,
    STATE(5291), 1,
      sym_string,
    STATE(5132), 2,
      sym_heredoc_body,
      aux_sym_chained_string_repeat1,
    ACTIONS(3288), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [180460] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6957), 1,
      anon_sym_then,
    ACTIONS(7313), 1,
      anon_sym_COMMA,
    STATE(302), 1,
      sym__terminator,
    STATE(5133), 1,
      sym_heredoc_body,
    STATE(5314), 1,
      aux_sym_when_repeat1,
    STATE(5539), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [180489] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(2649), 1,
      sym__simple_numeric,
    STATE(5134), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [180516] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7411), 1,
      sym__string_end,
    STATE(5135), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [180545] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7413), 1,
      sym__string_end,
    STATE(5136), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [180574] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3092), 1,
      sym__simple_numeric,
    STATE(3385), 1,
      sym__pow,
    STATE(5137), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [180601] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7415), 1,
      anon_sym_rescue,
    STATE(5138), 1,
      sym_heredoc_body,
    ACTIONS(6594), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [180622] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7415), 1,
      anon_sym_rescue,
    STATE(5139), 1,
      sym_heredoc_body,
    ACTIONS(6596), 6,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [180643] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7417), 1,
      anon_sym_COMMA,
    STATE(5140), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [180664] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3334), 1,
      sym__simple_numeric,
    STATE(5141), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [180691] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7420), 1,
      sym__string_end,
    STATE(5142), 1,
      sym_heredoc_body,
    STATE(5150), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [180720] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3016), 1,
      sym__simple_numeric,
    STATE(5143), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [180747] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7422), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5144), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180776] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7424), 1,
      sym__string_end,
    STATE(5145), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [180805] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3923), 1,
      sym__simple_numeric,
    STATE(5146), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [180832] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7426), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7432), 1,
      sym_heredoc_end,
    ACTIONS(7434), 1,
      sym__short_interpolation,
    STATE(5303), 1,
      sym_interpolation,
    ACTIONS(7429), 2,
      sym_heredoc_content,
      sym_escape_sequence,
    STATE(5147), 2,
      sym_heredoc_body,
      aux_sym_heredoc_body_repeat1,
  [180859] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(3661), 1,
      sym__simple_numeric,
    STATE(5148), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [180886] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7437), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5149), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [180915] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7439), 1,
      sym__string_end,
    STATE(5150), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [180944] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7441), 1,
      sym__string_end,
    STATE(5151), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [180973] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5152), 1,
      sym_heredoc_body,
    ACTIONS(2960), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [180992] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3887), 1,
      sym__simple_numeric,
    STATE(5153), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [181019] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7443), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5154), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181048] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7445), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5155), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181077] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7447), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5156), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181106] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3563), 1,
      sym__simple_numeric,
    STATE(5157), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181133] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3412), 1,
      sym__simple_numeric,
    STATE(5158), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [181160] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7449), 1,
      anon_sym_COMMA,
    STATE(5140), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(5159), 1,
      sym_heredoc_body,
    ACTIONS(6645), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [181183] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3687), 1,
      sym__simple_numeric,
    STATE(5160), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181210] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7451), 1,
      sym__string_end,
    STATE(5161), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [181239] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7453), 1,
      sym__string_end,
    STATE(5162), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [181268] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7455), 1,
      sym__string_end,
    STATE(5163), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [181297] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7457), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5164), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181326] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5165), 1,
      sym_heredoc_body,
    ACTIONS(3008), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [181345] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3480), 1,
      sym__simple_numeric,
    STATE(5166), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181372] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7459), 1,
      sym__string_end,
    STATE(5167), 1,
      sym_heredoc_body,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [181401] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7461), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5168), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181430] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3827), 1,
      sym__simple_numeric,
    STATE(5169), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(325), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [181457] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7463), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5170), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181486] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3359), 1,
      sym__simple_numeric,
    STATE(3385), 1,
      sym__pow,
    STATE(5171), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [181513] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3385), 1,
      sym__pow,
    STATE(3825), 1,
      sym__simple_numeric,
    STATE(5172), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [181540] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3816), 1,
      sym__simple_numeric,
    STATE(5173), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181567] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7465), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5174), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181596] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5175), 1,
      sym_heredoc_body,
    ACTIONS(3012), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [181615] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3336), 1,
      sym__simple_numeric,
    STATE(5176), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(253), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181642] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7152), 1,
      sym__string_end,
    ACTIONS(7467), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7473), 1,
      sym__short_interpolation,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7470), 2,
      sym_string_content,
      sym_escape_sequence,
    STATE(5177), 2,
      sym_heredoc_body,
      aux_sym__literal_contents,
  [181669] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7476), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5178), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181698] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7478), 1,
      sym__string_end,
    STATE(5136), 1,
      aux_sym__literal_contents,
    STATE(5179), 1,
      sym_heredoc_body,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [181727] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7480), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5180), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181756] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7482), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5181), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [181785] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7484), 1,
      sym__string_end,
    STATE(5177), 1,
      aux_sym__literal_contents,
    STATE(5182), 1,
      sym_heredoc_body,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [181814] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(2838), 1,
      sym__simple_numeric,
    STATE(5183), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181841] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7449), 1,
      anon_sym_COMMA,
    STATE(5159), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(5184), 1,
      sym_heredoc_body,
    ACTIONS(6620), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [181864] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7486), 1,
      sym__string_end,
    STATE(5101), 1,
      aux_sym__literal_contents,
    STATE(5185), 1,
      sym_heredoc_body,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [181893] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(2859), 1,
      sym__simple_numeric,
    STATE(5186), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181920] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3415), 1,
      sym__simple_numeric,
    STATE(5187), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [181947] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(3091), 1,
      sym__simple_numeric,
    STATE(5188), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [181974] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3101), 1,
      sym__simple_numeric,
    STATE(5189), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182001] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(4016), 1,
      sym__simple_numeric,
    STATE(5190), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [182028] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(2722), 1,
      sym__simple_numeric,
    STATE(5191), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182055] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7488), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5192), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182084] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(2761), 1,
      sym__simple_numeric,
    STATE(5193), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182111] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(3064), 1,
      sym__pow,
    STATE(3960), 1,
      sym__simple_numeric,
    STATE(5194), 1,
      sym_heredoc_body,
    STATE(5560), 1,
      sym__int_or_float,
    ACTIONS(393), 2,
      sym_integer,
      sym_float,
    STATE(3070), 2,
      sym_complex,
      sym_rational,
  [182138] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2568), 1,
      sym__simple_numeric,
    STATE(2598), 1,
      sym__pow,
    STATE(5195), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182165] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7490), 1,
      sym_identifier,
    ACTIONS(7494), 1,
      anon_sym_do,
    STATE(5196), 1,
      sym_heredoc_body,
    ACTIONS(7492), 5,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
  [182188] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(2607), 1,
      sym__simple_numeric,
    STATE(5197), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182215] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2507), 1,
      sym__simple_numeric,
    STATE(2598), 1,
      sym__pow,
    STATE(5198), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182242] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(2520), 1,
      sym__simple_numeric,
    STATE(5199), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182269] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2598), 1,
      sym__pow,
    STATE(3051), 1,
      sym__simple_numeric,
    STATE(5200), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182296] = 8,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(2345), 1,
      sym__pow,
    STATE(3066), 1,
      sym__simple_numeric,
    STATE(5201), 1,
      sym_heredoc_body,
    STATE(5554), 1,
      sym__int_or_float,
    ACTIONS(81), 2,
      sym_integer,
      sym_float,
    STATE(2293), 2,
      sym_complex,
      sym_rational,
  [182323] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7496), 1,
      sym__string_end,
    STATE(5163), 1,
      aux_sym__literal_contents,
    STATE(5202), 1,
      sym_heredoc_body,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [182352] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7498), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5203), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182381] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7500), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5204), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182410] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7502), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5205), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182439] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7504), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5206), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182468] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7277), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7283), 1,
      sym__short_interpolation,
    ACTIONS(7506), 1,
      sym__string_end,
    STATE(5038), 1,
      aux_sym__literal_contents,
    STATE(5207), 1,
      sym_heredoc_body,
    STATE(5307), 1,
      sym_interpolation,
    ACTIONS(7279), 2,
      sym_string_content,
      sym_escape_sequence,
  [182497] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7508), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5208), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182526] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7510), 1,
      sym__string_end,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5209), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182555] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5210), 1,
      sym_heredoc_body,
    ACTIONS(3016), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [182574] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5211), 1,
      sym_heredoc_body,
    ACTIONS(121), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [182593] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    ACTIONS(7512), 1,
      sym__string_end,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5212), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182622] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5213), 1,
      sym_heredoc_body,
    ACTIONS(3000), 7,
      sym__line_break,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [182641] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7514), 1,
      anon_sym_COMMA,
    STATE(5214), 1,
      sym_heredoc_body,
    STATE(5267), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6620), 4,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [182663] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7516), 1,
      anon_sym_end,
    STATE(5215), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5946), 1,
      sym_else,
  [182691] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7518), 1,
      anon_sym_end,
    STATE(5216), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5958), 1,
      sym_else,
  [182719] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5910), 1,
      anon_sym_COMMA,
    STATE(5217), 1,
      sym_heredoc_body,
    STATE(5249), 1,
      aux_sym_exceptions_repeat1,
    ACTIONS(5912), 4,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_EQ_GT,
      anon_sym_then,
  [182741] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(5218), 1,
      sym_heredoc_body,
    STATE(5459), 1,
      sym_argument_list,
    ACTIONS(417), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [182763] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7520), 1,
      anon_sym_COMMA,
    STATE(5219), 2,
      sym_heredoc_body,
      aux_sym_exceptions_repeat1,
    ACTIONS(3743), 4,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_EQ_GT,
      anon_sym_then,
  [182783] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4739), 1,
      sym__simple_numeric,
    STATE(5220), 1,
      sym_heredoc_body,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2874), 2,
      sym_integer,
      sym_float,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
  [182807] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7523), 1,
      anon_sym_rescue,
    STATE(5221), 1,
      sym_heredoc_body,
    ACTIONS(6594), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [182827] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(5222), 1,
      sym_heredoc_body,
    STATE(5331), 1,
      sym_argument_list,
    ACTIONS(413), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [182849] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    STATE(4939), 1,
      aux_sym__literal_contents,
    STATE(5223), 1,
      sym_heredoc_body,
    STATE(5234), 1,
      sym_interpolation,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [182875] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7523), 1,
      anon_sym_rescue,
    STATE(5224), 1,
      sym_heredoc_body,
    ACTIONS(6596), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [182895] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7077), 1,
      anon_sym_end,
    STATE(5225), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5824), 1,
      sym_else,
  [182923] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5226), 1,
      sym_heredoc_body,
    ACTIONS(3078), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [182941] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(5227), 1,
      sym_heredoc_body,
    STATE(5347), 1,
      sym_argument_list,
    ACTIONS(425), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [182963] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7525), 1,
      anon_sym_end,
    STATE(5228), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5886), 1,
      sym_else,
  [182991] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4420), 1,
      sym__simple_numeric,
    STATE(5229), 1,
      sym_heredoc_body,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2924), 2,
      sym_integer,
      sym_float,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
  [183015] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7527), 1,
      anon_sym_end,
    STATE(5230), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5889), 1,
      sym_else,
  [183043] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7529), 1,
      anon_sym_end,
    STATE(5231), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5838), 1,
      sym_else,
  [183071] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7531), 1,
      anon_sym_end,
    STATE(5232), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5840), 1,
      sym_else,
  [183099] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7533), 1,
      anon_sym_end,
    STATE(5233), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5904), 1,
      sym_else,
  [183127] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5234), 1,
      sym_heredoc_body,
    ACTIONS(7535), 3,
      anon_sym_POUND_LBRACE,
      aux_sym_string_array_token1,
      sym_escape_sequence,
    ACTIONS(7537), 3,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
  [183147] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7539), 1,
      sym_identifier,
    ACTIONS(7541), 1,
      anon_sym_LPAREN,
    STATE(4791), 1,
      sym__nonlocal_variable,
    STATE(5235), 1,
      sym_heredoc_body,
    ACTIONS(7543), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [183171] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7545), 1,
      anon_sym_rescue,
    STATE(5236), 1,
      sym_heredoc_body,
    ACTIONS(6594), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [183191] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5237), 1,
      sym_heredoc_body,
    ACTIONS(7547), 3,
      anon_sym_POUND_LBRACE,
      aux_sym_string_array_token1,
      sym_escape_sequence,
    ACTIONS(7549), 3,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
  [183211] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4420), 1,
      sym__simple_numeric,
    STATE(5238), 1,
      sym_heredoc_body,
    STATE(5557), 1,
      sym__int_or_float,
    ACTIONS(2824), 2,
      sym_integer,
      sym_float,
    STATE(4387), 2,
      sym_complex,
      sym_rational,
  [183235] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7551), 1,
      anon_sym_end,
    STATE(5239), 1,
      sym_heredoc_body,
    STATE(5254), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5970), 1,
      sym_else,
  [183263] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7551), 1,
      anon_sym_end,
    STATE(5240), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5970), 1,
      sym_else,
  [183291] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5241), 1,
      sym_heredoc_body,
    ACTIONS(7553), 3,
      anon_sym_POUND_LBRACE,
      aux_sym_string_array_token1,
      sym_escape_sequence,
    ACTIONS(7555), 3,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
  [183311] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5242), 1,
      sym_heredoc_body,
    ACTIONS(3070), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [183329] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5243), 1,
      sym_heredoc_body,
    ACTIONS(7557), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [183347] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7545), 1,
      anon_sym_rescue,
    STATE(5244), 1,
      sym_heredoc_body,
    ACTIONS(6596), 5,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
      anon_sym_and,
      anon_sym_or,
  [183367] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5245), 1,
      sym_heredoc_body,
    ACTIONS(7559), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [183385] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7561), 1,
      anon_sym_end,
    STATE(5246), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5710), 1,
      sym_else,
  [183413] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5247), 1,
      sym_heredoc_body,
    ACTIONS(7563), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [183431] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5248), 1,
      sym_heredoc_body,
    STATE(5340), 1,
      sym__simple_numeric,
    STATE(5571), 1,
      sym__int_or_float,
    ACTIONS(2702), 2,
      sym_integer,
      sym_float,
    STATE(5479), 2,
      sym_complex,
      sym_rational,
  [183455] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(5910), 1,
      anon_sym_COMMA,
    STATE(5219), 1,
      aux_sym_exceptions_repeat1,
    STATE(5249), 1,
      sym_heredoc_body,
    ACTIONS(7565), 4,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_EQ_GT,
      anon_sym_then,
  [183477] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7567), 1,
      anon_sym_end,
    STATE(5250), 1,
      sym_heredoc_body,
    STATE(5277), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5761), 1,
      sym_else,
  [183505] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5251), 1,
      sym_heredoc_body,
    ACTIONS(3921), 6,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [183523] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7569), 1,
      anon_sym_end,
    STATE(5252), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5713), 1,
      sym_else,
  [183551] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7571), 1,
      anon_sym_rescue,
    STATE(5253), 1,
      sym_heredoc_body,
    ACTIONS(6594), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [183571] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7573), 1,
      anon_sym_end,
    STATE(5254), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5689), 1,
      sym_else,
  [183599] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7571), 1,
      anon_sym_rescue,
    STATE(5255), 1,
      sym_heredoc_body,
    ACTIONS(6596), 5,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [183619] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5256), 1,
      sym_heredoc_body,
    ACTIONS(3683), 6,
      sym__element_reference_bracket,
      anon_sym_LBRACE,
      anon_sym_DOT,
      anon_sym_do,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [183637] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7567), 1,
      anon_sym_end,
    STATE(5257), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5761), 1,
      sym_else,
  [183665] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7575), 1,
      sym_identifier,
    ACTIONS(7577), 1,
      anon_sym_LPAREN,
    STATE(4439), 1,
      sym__nonlocal_variable,
    STATE(5258), 1,
      sym_heredoc_body,
    ACTIONS(3441), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [183689] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5259), 1,
      sym_heredoc_body,
    ACTIONS(2630), 3,
      anon_sym_POUND_LBRACE,
      aux_sym_string_array_token1,
      sym_escape_sequence,
    ACTIONS(2632), 3,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
  [183709] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(4739), 1,
      sym__simple_numeric,
    STATE(5260), 1,
      sym_heredoc_body,
    STATE(5518), 1,
      sym__int_or_float,
    ACTIONS(2910), 2,
      sym_integer,
      sym_float,
    STATE(4718), 2,
      sym_complex,
      sym_rational,
  [183733] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7579), 1,
      anon_sym_end,
    STATE(5261), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5797), 1,
      sym_else,
  [183761] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7581), 1,
      anon_sym_end,
    STATE(5262), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5819), 1,
      sym_else,
  [183789] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7575), 1,
      sym_identifier,
    ACTIONS(7577), 1,
      anon_sym_LPAREN,
    STATE(4439), 1,
      sym__nonlocal_variable,
    STATE(5263), 1,
      sym_heredoc_body,
    ACTIONS(4259), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [183813] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5264), 1,
      sym_heredoc_body,
    ACTIONS(3006), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [183831] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7020), 1,
      anon_sym_end,
    STATE(5265), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5869), 1,
      sym_else,
  [183859] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7583), 1,
      anon_sym_end,
    STATE(5266), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5809), 1,
      sym_else,
  [183887] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7514), 1,
      anon_sym_COMMA,
    STATE(5267), 1,
      sym_heredoc_body,
    STATE(5289), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6645), 4,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [183909] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5268), 1,
      sym_heredoc_body,
    ACTIONS(7585), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [183927] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(6990), 1,
      anon_sym_POUND_LBRACE,
    ACTIONS(7000), 1,
      sym__short_interpolation,
    STATE(5035), 1,
      aux_sym__literal_contents,
    STATE(5234), 1,
      sym_interpolation,
    STATE(5269), 1,
      sym_heredoc_body,
    ACTIONS(6996), 2,
      sym_string_content,
      sym_escape_sequence,
  [183953] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5270), 1,
      sym_heredoc_body,
    ACTIONS(7587), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [183971] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7006), 1,
      anon_sym_end,
    STATE(5271), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5967), 1,
      sym_else,
  [183999] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7589), 1,
      anon_sym_end,
    STATE(5272), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5690), 1,
      sym_else,
  [184027] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7591), 1,
      anon_sym_end,
    STATE(5273), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5704), 1,
      sym_else,
  [184055] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7056), 1,
      anon_sym_end,
    STATE(5274), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5735), 1,
      sym_else,
  [184083] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7593), 1,
      anon_sym_end,
    STATE(5275), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5802), 1,
      sym_else,
  [184111] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7595), 1,
      anon_sym_end,
    STATE(5276), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5913), 1,
      sym_else,
  [184139] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7597), 1,
      anon_sym_end,
    STATE(5277), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5654), 1,
      sym_else,
  [184167] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(5278), 1,
      sym_heredoc_body,
    STATE(5349), 1,
      sym_argument_list,
    ACTIONS(3295), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [184189] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(5279), 1,
      sym_heredoc_body,
    STATE(5355), 1,
      sym_argument_list,
    ACTIONS(3327), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [184211] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7599), 1,
      anon_sym_end,
    STATE(5276), 1,
      aux_sym_case_repeat1,
    STATE(5280), 1,
      sym_heredoc_body,
    STATE(5505), 1,
      sym_when,
    STATE(5857), 1,
      sym_else,
  [184239] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7599), 1,
      anon_sym_end,
    STATE(5281), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5857), 1,
      sym_else,
  [184267] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7601), 1,
      anon_sym_end,
    STATE(5282), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5718), 1,
      sym_else,
  [184295] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5283), 1,
      sym_heredoc_body,
    ACTIONS(7603), 6,
      anon_sym_LBRACE,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_do,
  [184313] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7605), 1,
      anon_sym_end,
    STATE(5284), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5931), 1,
      sym_else,
  [184341] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(4716), 1,
      anon_sym_LPAREN2,
    STATE(5285), 1,
      sym_heredoc_body,
    STATE(5488), 1,
      sym_argument_list,
    ACTIONS(421), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [184363] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7099), 1,
      anon_sym_end,
    STATE(5286), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5879), 1,
      sym_else,
  [184391] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6779), 1,
      anon_sym_when,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7119), 1,
      anon_sym_end,
    STATE(5287), 1,
      sym_heredoc_body,
    STATE(5316), 1,
      aux_sym_case_repeat1,
    STATE(5505), 1,
      sym_when,
    STATE(5977), 1,
      sym_else,
  [184419] = 9,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6777), 1,
      anon_sym_in,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7607), 1,
      anon_sym_end,
    STATE(5288), 1,
      sym_heredoc_body,
    STATE(5326), 1,
      aux_sym_case_match_repeat1,
    STATE(5532), 1,
      sym_in_clause,
    STATE(5674), 1,
      sym_else,
  [184447] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7609), 1,
      anon_sym_COMMA,
    STATE(5289), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(6588), 4,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
      anon_sym_and,
      anon_sym_or,
  [184467] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7612), 1,
      anon_sym_end,
    ACTIONS(7614), 1,
      anon_sym_elsif,
    STATE(5290), 1,
      sym_heredoc_body,
    STATE(5882), 2,
      sym_elsif,
      sym_else,
  [184490] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5291), 1,
      sym_heredoc_body,
    ACTIONS(3357), 5,
      sym__string_start,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [184507] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5292), 1,
      sym_heredoc_body,
    ACTIONS(3303), 5,
      sym__string_start,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [184524] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5293), 1,
      sym_heredoc_body,
    ACTIONS(3319), 5,
      sym__string_start,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [184541] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7614), 1,
      anon_sym_elsif,
    ACTIONS(7616), 1,
      anon_sym_end,
    STATE(5294), 1,
      sym_heredoc_body,
    STATE(5887), 2,
      sym_elsif,
      sym_else,
  [184564] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7553), 1,
      anon_sym_POUND_LBRACE,
    STATE(5295), 1,
      sym_heredoc_body,
    ACTIONS(7555), 4,
      sym_heredoc_content,
      sym_heredoc_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [184583] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2630), 1,
      anon_sym_POUND_LBRACE,
    STATE(5296), 1,
      sym_heredoc_body,
    ACTIONS(2632), 4,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [184602] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7614), 1,
      anon_sym_elsif,
    ACTIONS(7618), 1,
      anon_sym_end,
    STATE(5297), 1,
      sym_heredoc_body,
    STATE(5919), 2,
      sym_elsif,
      sym_else,
  [184625] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7553), 1,
      anon_sym_POUND_LBRACE,
    STATE(5298), 1,
      sym_heredoc_body,
    ACTIONS(7555), 4,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [184644] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6988), 1,
      anon_sym_do,
    STATE(334), 1,
      sym__terminator,
    STATE(2884), 1,
      sym_do,
    STATE(5299), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [184667] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7614), 1,
      anon_sym_elsif,
    ACTIONS(7620), 1,
      anon_sym_end,
    STATE(5300), 1,
      sym_heredoc_body,
    STATE(5702), 2,
      sym_elsif,
      sym_else,
  [184690] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7622), 1,
      anon_sym_COMMA,
    STATE(5301), 2,
      sym_heredoc_body,
      aux_sym__mlhs_repeat1,
    ACTIONS(7253), 3,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_in,
  [184709] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7547), 1,
      anon_sym_POUND_LBRACE,
    STATE(5302), 1,
      sym_heredoc_body,
    ACTIONS(7549), 4,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [184728] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7625), 1,
      anon_sym_POUND_LBRACE,
    STATE(5303), 1,
      sym_heredoc_body,
    ACTIONS(7627), 4,
      sym_heredoc_content,
      sym_heredoc_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [184747] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7629), 1,
      anon_sym_COMMA,
    STATE(5301), 1,
      aux_sym__mlhs_repeat1,
    STATE(5304), 1,
      sym_heredoc_body,
    ACTIONS(2764), 3,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_in,
  [184768] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7614), 1,
      anon_sym_elsif,
    ACTIONS(7631), 1,
      anon_sym_end,
    STATE(5305), 1,
      sym_heredoc_body,
    STATE(5749), 2,
      sym_elsif,
      sym_else,
  [184791] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(3164), 1,
      anon_sym_COMMA,
    STATE(5304), 1,
      aux_sym__mlhs_repeat1,
    STATE(5306), 1,
      sym_heredoc_body,
    ACTIONS(7633), 3,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_in,
  [184812] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7535), 1,
      anon_sym_POUND_LBRACE,
    STATE(5307), 1,
      sym_heredoc_body,
    ACTIONS(7537), 4,
      sym_string_content,
      sym__string_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [184831] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6951), 1,
      anon_sym_do,
    STATE(360), 1,
      sym__terminator,
    STATE(2404), 1,
      sym_do,
    STATE(5308), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [184854] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7635), 1,
      anon_sym_EQ,
    ACTIONS(7637), 1,
      anon_sym_COLON,
    STATE(5309), 1,
      sym_heredoc_body,
    ACTIONS(7603), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [184875] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7614), 1,
      anon_sym_elsif,
    ACTIONS(7639), 1,
      anon_sym_end,
    STATE(5310), 1,
      sym_heredoc_body,
    STATE(5795), 2,
      sym_elsif,
      sym_else,
  [184898] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5311), 1,
      sym_heredoc_body,
    ACTIONS(7643), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(7641), 3,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
  [184917] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5312), 1,
      sym_heredoc_body,
    ACTIONS(7643), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(7645), 3,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
  [184936] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(2630), 1,
      anon_sym_POUND_LBRACE,
    STATE(5313), 1,
      sym_heredoc_body,
    ACTIONS(2632), 4,
      sym_heredoc_content,
      sym_heredoc_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [184955] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7647), 1,
      anon_sym_COMMA,
    STATE(5314), 2,
      sym_heredoc_body,
      aux_sym_when_repeat1,
    ACTIONS(7650), 3,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
  [184974] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7652), 1,
      anon_sym_EQ,
    ACTIONS(7654), 1,
      anon_sym_COLON,
    STATE(5315), 1,
      sym_heredoc_body,
    ACTIONS(7603), 3,
      anon_sym_COMMA,
      anon_sym_PIPE,
      anon_sym_SEMI,
  [184995] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7658), 1,
      anon_sym_when,
    STATE(5505), 1,
      sym_when,
    ACTIONS(7656), 2,
      anon_sym_end,
      anon_sym_else,
    STATE(5316), 2,
      sym_heredoc_body,
      aux_sym_case_repeat1,
  [185016] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5317), 1,
      sym_heredoc_body,
    ACTIONS(7661), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(6554), 3,
      anon_sym_RPAREN,
      anon_sym_COMMA,
      anon_sym_RBRACK,
  [185035] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7663), 1,
      anon_sym_EQ,
    ACTIONS(7665), 1,
      anon_sym_COLON,
    STATE(5318), 1,
      sym_heredoc_body,
    ACTIONS(7603), 3,
      anon_sym_LBRACE,
      anon_sym_COMMA,
      anon_sym_do,
  [185056] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7547), 1,
      anon_sym_POUND_LBRACE,
    STATE(5319), 1,
      sym_heredoc_body,
    ACTIONS(7549), 4,
      sym_heredoc_content,
      sym_heredoc_end,
      sym__short_interpolation,
      sym_escape_sequence,
  [185075] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(280), 1,
      sym__terminator,
    STATE(5320), 1,
      sym_heredoc_body,
    STATE(5343), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [185098] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6781), 1,
      anon_sym_else,
    ACTIONS(7614), 1,
      anon_sym_elsif,
    ACTIONS(7667), 1,
      anon_sym_end,
    STATE(5321), 1,
      sym_heredoc_body,
    STATE(5932), 2,
      sym_elsif,
      sym_else,
  [185121] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(298), 1,
      sym__terminator,
    STATE(5322), 1,
      sym_heredoc_body,
    STATE(5556), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [185144] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5323), 1,
      sym_heredoc_body,
    ACTIONS(3743), 5,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_EQ_GT,
      anon_sym_then,
  [185161] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6974), 1,
      anon_sym_do,
    STATE(331), 1,
      sym__terminator,
    STATE(5324), 1,
      sym_heredoc_body,
    STATE(5417), 1,
      sym_do,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [185184] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6957), 1,
      anon_sym_then,
    STATE(275), 1,
      sym__terminator,
    STATE(5325), 1,
      sym_heredoc_body,
    STATE(5435), 1,
      sym_then,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [185207] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7671), 1,
      anon_sym_in,
    STATE(5532), 1,
      sym_in_clause,
    ACTIONS(7669), 2,
      anon_sym_end,
      anon_sym_else,
    STATE(5326), 2,
      sym_heredoc_body,
      aux_sym_case_match_repeat1,
  [185228] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5327), 1,
      sym_heredoc_body,
    ACTIONS(3465), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185244] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5328), 1,
      sym_heredoc_body,
    ACTIONS(3777), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185260] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5241), 1,
      sym__nonlocal_variable,
    STATE(5329), 1,
      sym_heredoc_body,
    ACTIONS(7674), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [185278] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7678), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(7680), 1,
      sym__element_reference_bracket,
    STATE(5330), 1,
      sym_heredoc_body,
    ACTIONS(7676), 2,
      anon_sym_DOT,
      anon_sym_AMP_DOT,
  [185298] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5331), 1,
      sym_heredoc_body,
    ACTIONS(3495), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185314] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5332), 1,
      sym_heredoc_body,
    ACTIONS(3417), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185330] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5333), 1,
      sym_heredoc_body,
    ACTIONS(3945), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185346] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5334), 1,
      sym_heredoc_body,
    ACTIONS(3781), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185362] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5335), 1,
      sym_heredoc_body,
    ACTIONS(3785), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185378] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5336), 1,
      sym_heredoc_body,
    ACTIONS(7682), 4,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
  [185394] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5337), 1,
      sym_heredoc_body,
    ACTIONS(3867), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185410] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5338), 1,
      sym_heredoc_body,
    ACTIONS(3871), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185426] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5339), 1,
      sym_heredoc_body,
    ACTIONS(3835), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185442] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5340), 1,
      sym_heredoc_body,
    ACTIONS(3907), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185458] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6683), 1,
      anon_sym_LBRACE,
    ACTIONS(6845), 1,
      anon_sym_do,
    STATE(5341), 1,
      sym_heredoc_body,
    STATE(5406), 2,
      sym_do_block,
      sym_block,
  [185478] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5342), 1,
      sym_heredoc_body,
    ACTIONS(3617), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185494] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5343), 1,
      sym_heredoc_body,
    ACTIONS(7684), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [185510] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5344), 1,
      sym_heredoc_body,
    ACTIONS(3621), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185526] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5345), 1,
      sym_heredoc_body,
    ACTIONS(3631), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185542] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5346), 1,
      sym_heredoc_body,
    ACTIONS(3887), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185558] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5347), 1,
      sym_heredoc_body,
    ACTIONS(3505), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185574] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5348), 1,
      sym_heredoc_body,
    ACTIONS(3643), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185590] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5349), 1,
      sym_heredoc_body,
    ACTIONS(3655), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185606] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6737), 1,
      anon_sym_PIPE,
    ACTIONS(6739), 1,
      anon_sym_SEMI,
    ACTIONS(7686), 1,
      anon_sym_COMMA,
    STATE(5350), 1,
      sym_heredoc_body,
    STATE(5356), 1,
      aux_sym_parameters_repeat1,
  [185628] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7688), 1,
      anon_sym_COMMA,
    ACTIONS(6588), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
    STATE(5351), 2,
      sym_heredoc_body,
      aux_sym_command_argument_list_repeat1,
  [185646] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5352), 1,
      sym_heredoc_body,
    ACTIONS(3671), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185662] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5353), 1,
      sym_heredoc_body,
    ACTIONS(3517), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185678] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5354), 1,
      sym_heredoc_body,
    ACTIONS(3467), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185694] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5355), 1,
      sym_heredoc_body,
    ACTIONS(3659), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185710] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7691), 1,
      anon_sym_COMMA,
    ACTIONS(7585), 2,
      anon_sym_PIPE,
      anon_sym_SEMI,
    STATE(5356), 2,
      sym_heredoc_body,
      aux_sym_parameters_repeat1,
  [185728] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5357), 1,
      sym_heredoc_body,
    ACTIONS(3471), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185744] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5358), 1,
      sym_heredoc_body,
    ACTIONS(3915), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185760] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5359), 1,
      sym_heredoc_body,
    ACTIONS(3521), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185776] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5360), 1,
      sym_heredoc_body,
    ACTIONS(3811), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185792] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7696), 1,
      anon_sym_COMMA,
    STATE(5361), 1,
      sym_heredoc_body,
    STATE(5443), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(7694), 2,
      anon_sym_LBRACE,
      anon_sym_do,
  [185812] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5362), 1,
      sym_heredoc_body,
    ACTIONS(3731), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185828] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5363), 1,
      sym_heredoc_body,
    ACTIONS(3815), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185844] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5364), 1,
      sym_heredoc_body,
    ACTIONS(3691), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185860] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5365), 1,
      sym_heredoc_body,
    ACTIONS(3373), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185876] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5366), 1,
      sym_heredoc_body,
    ACTIONS(3827), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185892] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5367), 1,
      sym_heredoc_body,
    ACTIONS(3695), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185908] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5368), 1,
      sym_heredoc_body,
    ACTIONS(7698), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [185924] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5369), 1,
      sym_heredoc_body,
    ACTIONS(7643), 2,
      anon_sym_and,
      anon_sym_or,
    ACTIONS(7700), 2,
      sym__line_break,
      anon_sym_SEMI,
  [185942] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5370), 1,
      sym_heredoc_body,
    ACTIONS(3765), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185958] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5371), 1,
      sym_heredoc_body,
    ACTIONS(3699), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [185974] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6731), 1,
      anon_sym_PIPE,
    ACTIONS(6733), 1,
      anon_sym_SEMI,
    ACTIONS(7702), 1,
      anon_sym_COMMA,
    STATE(5356), 1,
      aux_sym_parameters_repeat1,
    STATE(5372), 1,
      sym_heredoc_body,
  [185996] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5373), 1,
      sym_heredoc_body,
    ACTIONS(3487), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186012] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7704), 1,
      anon_sym_COMMA,
    ACTIONS(7706), 1,
      anon_sym_PIPE,
    ACTIONS(7708), 1,
      anon_sym_SEMI,
    STATE(5350), 1,
      aux_sym_parameters_repeat1,
    STATE(5374), 1,
      sym_heredoc_body,
  [186034] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5375), 1,
      sym_heredoc_body,
    ACTIONS(7710), 4,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
  [186050] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5376), 1,
      sym_heredoc_body,
    ACTIONS(3389), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186066] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5377), 1,
      sym_heredoc_body,
    ACTIONS(3475), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186082] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5378), 1,
      sym_heredoc_body,
    ACTIONS(3541), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186098] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5379), 1,
      sym_heredoc_body,
    ACTIONS(3487), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186114] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5380), 1,
      sym_heredoc_body,
    ACTIONS(3703), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186130] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5381), 1,
      sym_heredoc_body,
    ACTIONS(3735), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186146] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5382), 1,
      sym_heredoc_body,
    ACTIONS(7253), 4,
      anon_sym_RPAREN,
      anon_sym_EQ,
      anon_sym_COMMA,
      anon_sym_in,
  [186162] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5383), 1,
      sym_heredoc_body,
    ACTIONS(3845), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186178] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5384), 1,
      sym_heredoc_body,
    ACTIONS(3707), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186194] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5385), 1,
      sym_heredoc_body,
    ACTIONS(7712), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [186210] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5386), 1,
      sym_heredoc_body,
    ACTIONS(3727), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186226] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5387), 1,
      sym_heredoc_body,
    ACTIONS(3533), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186242] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7714), 1,
      sym_identifier,
    STATE(5388), 1,
      sym_heredoc_body,
    ACTIONS(7492), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [186260] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5389), 1,
      sym_heredoc_body,
    ACTIONS(3739), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186276] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5390), 1,
      sym_heredoc_body,
    ACTIONS(3537), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186292] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5391), 1,
      sym_heredoc_body,
    ACTIONS(3605), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186308] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7716), 1,
      anon_sym_rescue,
    STATE(5392), 1,
      sym_heredoc_body,
    ACTIONS(6594), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
  [186326] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5393), 1,
      sym_heredoc_body,
    ACTIONS(3445), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186342] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5394), 1,
      sym_heredoc_body,
    ACTIONS(3953), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186358] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6487), 1,
      anon_sym_LBRACE,
    ACTIONS(6489), 1,
      anon_sym_do,
    STATE(5395), 1,
      sym_heredoc_body,
    STATE(4419), 2,
      sym_do_block,
      sym_block,
  [186378] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7716), 1,
      anon_sym_rescue,
    STATE(5396), 1,
      sym_heredoc_body,
    ACTIONS(6596), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
  [186396] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5397), 1,
      sym_heredoc_body,
    ACTIONS(3545), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186412] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5398), 1,
      sym_heredoc_body,
    ACTIONS(3639), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186428] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6669), 1,
      anon_sym_LBRACE,
    ACTIONS(7718), 1,
      anon_sym_do,
    STATE(5399), 1,
      sym_heredoc_body,
    STATE(4419), 2,
      sym_do_block,
      sym_block,
  [186448] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5400), 1,
      sym_heredoc_body,
    ACTIONS(3761), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186464] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5401), 1,
      sym_heredoc_body,
    ACTIONS(3749), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186480] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5402), 1,
      sym_heredoc_body,
    ACTIONS(3509), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186496] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5295), 1,
      sym__nonlocal_variable,
    STATE(5403), 1,
      sym_heredoc_body,
    ACTIONS(7720), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [186514] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5404), 1,
      sym_heredoc_body,
    ACTIONS(7722), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [186530] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6673), 1,
      anon_sym_LBRACE,
    ACTIONS(6963), 1,
      anon_sym_do,
    STATE(5405), 1,
      sym_heredoc_body,
    STATE(4690), 2,
      sym_do_block,
      sym_block,
  [186550] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5406), 1,
      sym_heredoc_body,
    ACTIONS(3967), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186566] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5407), 1,
      sym_heredoc_body,
    ACTIONS(3753), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186582] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7724), 1,
      anon_sym_COMMA,
    ACTIONS(7585), 2,
      sym__line_break,
      anon_sym_SEMI,
    STATE(5408), 2,
      sym_heredoc_body,
      aux_sym_parameters_repeat1,
  [186600] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5409), 1,
      sym_heredoc_body,
    ACTIONS(3687), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186616] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5410), 1,
      sym_heredoc_body,
    ACTIONS(3715), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186632] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5411), 1,
      sym_heredoc_body,
    ACTIONS(3413), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186648] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5412), 1,
      sym_heredoc_body,
    ACTIONS(3613), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186664] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5413), 1,
      sym_heredoc_body,
    ACTIONS(3719), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186680] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5414), 1,
      sym_heredoc_body,
    ACTIONS(3449), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186696] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5415), 1,
      sym_heredoc_body,
    ACTIONS(3853), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186712] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5416), 1,
      sym_heredoc_body,
    ACTIONS(3513), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186728] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5417), 1,
      sym_heredoc_body,
    ACTIONS(3793), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186744] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5418), 1,
      sym_heredoc_body,
    ACTIONS(3569), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186760] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5419), 1,
      sym_heredoc_body,
    ACTIONS(3711), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186776] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5420), 1,
      sym_heredoc_body,
    ACTIONS(3090), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186792] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5421), 1,
      sym_heredoc_body,
    ACTIONS(3525), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186808] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5422), 1,
      sym_heredoc_body,
    ACTIONS(3393), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186824] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    ACTIONS(7161), 1,
      anon_sym_COMMA,
    STATE(5423), 1,
      sym_heredoc_body,
    ACTIONS(7727), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [186844] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5424), 1,
      sym_heredoc_body,
    ACTIONS(3393), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186860] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5425), 1,
      sym_heredoc_body,
    ACTIONS(3461), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186876] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5426), 1,
      sym_heredoc_body,
    ACTIONS(3573), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186892] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5427), 1,
      sym_heredoc_body,
    ACTIONS(3797), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186908] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5428), 1,
      sym_heredoc_body,
    ACTIONS(3903), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186924] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5429), 1,
      sym_heredoc_body,
    ACTIONS(3461), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186940] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5430), 1,
      sym_heredoc_body,
    ACTIONS(3377), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186956] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5431), 1,
      sym_heredoc_body,
    ACTIONS(3903), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186972] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5432), 1,
      sym_heredoc_body,
    ACTIONS(3635), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [186988] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7729), 1,
      anon_sym_COMMA,
    STATE(5433), 1,
      sym_heredoc_body,
    STATE(5480), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(7731), 2,
      sym__line_break,
      anon_sym_SEMI,
  [187008] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5434), 1,
      sym_heredoc_body,
    ACTIONS(3949), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187024] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5435), 1,
      sym_heredoc_body,
    ACTIONS(7733), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [187040] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5436), 1,
      sym_heredoc_body,
    ACTIONS(3663), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187056] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5437), 1,
      sym_heredoc_body,
    ACTIONS(2956), 4,
      anon_sym_end,
      anon_sym_in,
      anon_sym_when,
      anon_sym_else,
  [187072] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5438), 1,
      sym_heredoc_body,
    ACTIONS(3667), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187088] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5439), 1,
      sym_heredoc_body,
    ACTIONS(3675), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187104] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7735), 1,
      anon_sym_rescue,
    STATE(5440), 1,
      sym_heredoc_body,
    ACTIONS(6594), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
  [187122] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5441), 1,
      sym_heredoc_body,
    ACTIONS(6075), 4,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_then,
  [187138] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7739), 1,
      anon_sym_COMMA,
    STATE(5442), 1,
      sym_heredoc_body,
    STATE(5467), 1,
      aux_sym_command_argument_list_repeat1,
    ACTIONS(7737), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [187158] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7741), 1,
      anon_sym_COMMA,
    ACTIONS(7585), 2,
      anon_sym_LBRACE,
      anon_sym_do,
    STATE(5443), 2,
      sym_heredoc_body,
      aux_sym_parameters_repeat1,
  [187176] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5444), 1,
      sym_heredoc_body,
    ACTIONS(3609), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187192] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5445), 1,
      sym_heredoc_body,
    ACTIONS(3823), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187208] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5446), 1,
      sym_heredoc_body,
    ACTIONS(3757), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187224] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7746), 1,
      anon_sym_COLON_COLON2,
    ACTIONS(7748), 1,
      sym__element_reference_bracket,
    STATE(5447), 1,
      sym_heredoc_body,
    ACTIONS(7744), 2,
      anon_sym_DOT,
      anon_sym_AMP_DOT,
  [187244] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5448), 1,
      sym_heredoc_body,
    ACTIONS(7750), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [187260] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5298), 1,
      sym__nonlocal_variable,
    STATE(5449), 1,
      sym_heredoc_body,
    ACTIONS(7752), 3,
      sym_instance_variable,
      sym_class_variable,
      sym_global_variable,
  [187278] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7735), 1,
      anon_sym_rescue,
    STATE(5450), 1,
      sym_heredoc_body,
    ACTIONS(6596), 3,
      anon_sym_RPAREN,
      anon_sym_and,
      anon_sym_or,
  [187296] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5451), 1,
      sym_heredoc_body,
    ACTIONS(3499), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187312] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5452), 1,
      sym_heredoc_body,
    ACTIONS(3385), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187328] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5453), 1,
      sym_heredoc_body,
    ACTIONS(3891), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187344] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6677), 1,
      anon_sym_LBRACE,
    ACTIONS(7754), 1,
      anon_sym_do,
    STATE(5454), 1,
      sym_heredoc_body,
    STATE(4690), 2,
      sym_do_block,
      sym_block,
  [187364] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5455), 1,
      sym_heredoc_body,
    ACTIONS(3679), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187380] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5456), 1,
      sym_heredoc_body,
    ACTIONS(3849), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187396] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5457), 1,
      sym_heredoc_body,
    ACTIONS(3819), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187412] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5458), 1,
      sym_heredoc_body,
    ACTIONS(3401), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187428] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5459), 1,
      sym_heredoc_body,
    ACTIONS(3491), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187444] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5460), 1,
      sym_heredoc_body,
    ACTIONS(3553), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187460] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7756), 1,
      anon_sym_EQ,
    ACTIONS(7758), 1,
      anon_sym_COLON,
    STATE(5461), 1,
      sym_heredoc_body,
    ACTIONS(7603), 2,
      anon_sym_RPAREN,
      anon_sym_COMMA,
  [187480] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5462), 1,
      sym_heredoc_body,
    ACTIONS(3381), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187496] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5463), 1,
      sym_heredoc_body,
    ACTIONS(3557), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187512] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5464), 1,
      sym_heredoc_body,
    ACTIONS(3561), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187528] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5465), 1,
      sym_heredoc_body,
    ACTIONS(3959), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187544] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5466), 1,
      sym_heredoc_body,
    ACTIONS(3769), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187560] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7760), 1,
      anon_sym_COMMA,
    STATE(5351), 1,
      aux_sym_command_argument_list_repeat1,
    STATE(5467), 1,
      sym_heredoc_body,
    ACTIONS(2336), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [187580] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2962), 1,
      anon_sym_LBRACE,
    ACTIONS(2964), 1,
      anon_sym_do,
    STATE(5468), 1,
      sym_heredoc_body,
    STATE(2473), 2,
      sym_do_block,
      sym_block,
  [187600] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7696), 1,
      anon_sym_COMMA,
    STATE(5361), 1,
      aux_sym_parameters_repeat1,
    STATE(5469), 1,
      sym_heredoc_body,
    ACTIONS(7731), 2,
      anon_sym_LBRACE,
      anon_sym_do,
  [187620] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5470), 1,
      sym_heredoc_body,
    ACTIONS(3839), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187636] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7748), 1,
      sym__element_reference_bracket,
    ACTIONS(7762), 1,
      anon_sym_COLON_COLON2,
    STATE(5471), 1,
      sym_heredoc_body,
    ACTIONS(7744), 2,
      anon_sym_DOT,
      anon_sym_AMP_DOT,
  [187656] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5472), 1,
      sym_heredoc_body,
    ACTIONS(3963), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187672] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5473), 1,
      sym_heredoc_body,
    ACTIONS(3453), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187688] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5474), 1,
      sym_heredoc_body,
    ACTIONS(3369), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187704] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5475), 1,
      sym_heredoc_body,
    ACTIONS(3457), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187720] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5476), 1,
      sym_heredoc_body,
    ACTIONS(7764), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [187736] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5477), 1,
      sym_heredoc_body,
    ACTIONS(3405), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187752] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5478), 1,
      sym_heredoc_body,
    ACTIONS(3581), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187768] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5479), 1,
      sym_heredoc_body,
    ACTIONS(3274), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187784] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7729), 1,
      anon_sym_COMMA,
    STATE(5408), 1,
      aux_sym_parameters_repeat1,
    STATE(5480), 1,
      sym_heredoc_body,
    ACTIONS(7694), 2,
      sym__line_break,
      anon_sym_SEMI,
  [187804] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5481), 1,
      sym_heredoc_body,
    ACTIONS(3593), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187820] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5482), 1,
      sym_heredoc_body,
    ACTIONS(7766), 4,
      anon_sym_end,
      anon_sym_rescue,
      anon_sym_else,
      anon_sym_ensure,
  [187836] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5483), 1,
      sym_heredoc_body,
    ACTIONS(3479), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187852] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(3194), 1,
      anon_sym_LBRACE,
    ACTIONS(3196), 1,
      anon_sym_do,
    STATE(5484), 1,
      sym_heredoc_body,
    STATE(3018), 2,
      sym_do_block,
      sym_block,
  [187872] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5485), 1,
      sym_heredoc_body,
    ACTIONS(3421), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187888] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5486), 1,
      sym_heredoc_body,
    ACTIONS(3911), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187904] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5487), 1,
      sym_heredoc_body,
    ACTIONS(3549), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187920] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5488), 1,
      sym_heredoc_body,
    ACTIONS(3397), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187936] = 7,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7768), 1,
      anon_sym_COMMA,
    ACTIONS(7770), 1,
      anon_sym_PIPE,
    ACTIONS(7772), 1,
      anon_sym_SEMI,
    STATE(5372), 1,
      aux_sym_parameters_repeat1,
    STATE(5489), 1,
      sym_heredoc_body,
  [187958] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5490), 1,
      sym_heredoc_body,
    ACTIONS(3409), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187974] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5491), 1,
      sym_heredoc_body,
    ACTIONS(3565), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [187990] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5492), 1,
      sym_heredoc_body,
    ACTIONS(3773), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [188006] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5493), 1,
      sym_heredoc_body,
    ACTIONS(3911), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [188022] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5494), 1,
      sym_heredoc_body,
    ACTIONS(3723), 4,
      sym__element_reference_bracket,
      anon_sym_DOT,
      anon_sym_COLON_COLON2,
      anon_sym_AMP_DOT,
  [188038] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7776), 1,
      anon_sym_PIPE,
    STATE(5495), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188057] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(240), 1,
      sym__terminator,
    STATE(5496), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [188074] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7352), 1,
      sym__string_end,
    ACTIONS(7778), 1,
      aux_sym_string_array_token1,
    STATE(5497), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [188093] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7356), 1,
      sym__string_end,
    ACTIONS(7780), 1,
      aux_sym_string_array_token1,
    STATE(5498), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [188112] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7782), 1,
      anon_sym_RPAREN,
    ACTIONS(7784), 1,
      anon_sym_COMMA,
    STATE(5499), 1,
      sym_heredoc_body,
    STATE(5515), 1,
      aux_sym_parameters_repeat1,
  [188131] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7368), 1,
      sym__string_end,
    ACTIONS(7786), 1,
      aux_sym_string_array_token1,
    STATE(5500), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [188150] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2556), 1,
      anon_sym_RBRACE,
    ACTIONS(7788), 1,
      anon_sym_COMMA,
    STATE(5501), 1,
      sym_heredoc_body,
    STATE(5566), 1,
      aux_sym_hash_repeat1,
  [188169] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7790), 1,
      anon_sym_PIPE,
    STATE(5502), 1,
      sym_heredoc_body,
    STATE(5531), 1,
      aux_sym_block_parameters_repeat1,
  [188188] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7792), 1,
      anon_sym_RBRACE,
    ACTIONS(7794), 1,
      anon_sym_COMMA,
    STATE(5503), 1,
      sym_heredoc_body,
    STATE(5561), 1,
      aux_sym_hash_repeat1,
  [188207] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7437), 1,
      sym__string_end,
    ACTIONS(7796), 1,
      aux_sym_string_array_token1,
    STATE(5504), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [188226] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5505), 1,
      sym_heredoc_body,
    ACTIONS(7798), 3,
      anon_sym_end,
      anon_sym_when,
      anon_sym_else,
  [188241] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7800), 1,
      anon_sym_RBRACE,
    ACTIONS(7802), 1,
      anon_sym_COMMA,
    STATE(5501), 1,
      aux_sym_hash_repeat1,
    STATE(5506), 1,
      sym_heredoc_body,
  [188260] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7804), 1,
      anon_sym_RPAREN,
    STATE(5507), 1,
      sym_heredoc_body,
    ACTIONS(7806), 2,
      anon_sym_and,
      anon_sym_or,
  [188277] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2566), 1,
      anon_sym_RBRACE,
    ACTIONS(7808), 1,
      anon_sym_COMMA,
    STATE(5508), 1,
      sym_heredoc_body,
    STATE(5566), 1,
      aux_sym_hash_repeat1,
  [188296] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7810), 1,
      anon_sym_PIPE,
    STATE(5509), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188315] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7812), 1,
      anon_sym_PIPE,
    STATE(5510), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188334] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7814), 1,
      anon_sym_RPAREN,
    STATE(5511), 1,
      sym_heredoc_body,
    ACTIONS(7806), 2,
      anon_sym_and,
      anon_sym_or,
  [188351] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7461), 1,
      sym__string_end,
    ACTIONS(7816), 1,
      aux_sym_string_array_token1,
    STATE(5512), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [188370] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7394), 1,
      sym__string_end,
    ACTIONS(7818), 1,
      aux_sym_string_array_token1,
    STATE(5513), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [188389] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(238), 1,
      sym__terminator,
    STATE(5514), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [188406] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7784), 1,
      anon_sym_COMMA,
    ACTIONS(7820), 1,
      anon_sym_RPAREN,
    STATE(5515), 1,
      sym_heredoc_body,
    STATE(5567), 1,
      aux_sym_parameters_repeat1,
  [188425] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5516), 1,
      sym_heredoc_body,
    ACTIONS(3006), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [188440] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7398), 1,
      sym__string_end,
    ACTIONS(7822), 1,
      aux_sym_string_array_token1,
    STATE(5517), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [188459] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7824), 1,
      anon_sym_i,
    ACTIONS(7826), 1,
      anon_sym_ri,
    ACTIONS(7828), 1,
      anon_sym_r,
    STATE(5518), 1,
      sym_heredoc_body,
  [188478] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7830), 1,
      anon_sym_PIPE,
    STATE(5519), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188497] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7463), 1,
      sym__string_end,
    ACTIONS(7832), 1,
      aux_sym_string_array_token1,
    STATE(5520), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [188516] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7465), 1,
      sym__string_end,
    ACTIONS(7834), 1,
      aux_sym_string_array_token1,
    STATE(5521), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [188535] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7784), 1,
      anon_sym_COMMA,
    ACTIONS(7836), 1,
      anon_sym_RPAREN,
    STATE(5522), 1,
      sym_heredoc_body,
    STATE(5549), 1,
      aux_sym_parameters_repeat1,
  [188554] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7402), 1,
      sym__string_end,
    ACTIONS(7838), 1,
      aux_sym_string_array_token1,
    STATE(5523), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [188573] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7488), 1,
      sym__string_end,
    ACTIONS(7840), 1,
      aux_sym_string_array_token1,
    STATE(5524), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [188592] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7842), 1,
      anon_sym_PIPE,
    STATE(5510), 1,
      aux_sym_block_parameters_repeat1,
    STATE(5525), 1,
      sym_heredoc_body,
  [188611] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7844), 1,
      anon_sym_RBRACE,
    ACTIONS(7846), 1,
      anon_sym_COMMA,
    STATE(5508), 1,
      aux_sym_hash_repeat1,
    STATE(5526), 1,
      sym_heredoc_body,
  [188630] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(211), 1,
      sym__terminator,
    STATE(5527), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [188647] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(213), 1,
      sym__terminator,
    STATE(5528), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [188664] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7784), 1,
      anon_sym_COMMA,
    ACTIONS(7848), 1,
      anon_sym_RPAREN,
    STATE(5529), 1,
      sym_heredoc_body,
    STATE(5567), 1,
      aux_sym_parameters_repeat1,
  [188683] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7850), 1,
      anon_sym_PIPE,
    STATE(5530), 1,
      sym_heredoc_body,
    STATE(5534), 1,
      aux_sym_block_parameters_repeat1,
  [188702] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7852), 1,
      anon_sym_PIPE,
    STATE(5531), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188721] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5532), 1,
      sym_heredoc_body,
    ACTIONS(7854), 3,
      anon_sym_end,
      anon_sym_in,
      anon_sym_else,
  [188736] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5533), 1,
      sym_heredoc_body,
    ACTIONS(7587), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [188751] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7856), 1,
      anon_sym_PIPE,
    STATE(5534), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188770] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7858), 1,
      anon_sym_PIPE,
    STATE(5535), 1,
      sym_heredoc_body,
    STATE(5536), 1,
      aux_sym_block_parameters_repeat1,
  [188789] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7860), 1,
      anon_sym_PIPE,
    STATE(5536), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188808] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7862), 1,
      anon_sym_PIPE,
    STATE(5537), 1,
      sym_heredoc_body,
    STATE(5538), 1,
      aux_sym_block_parameters_repeat1,
  [188827] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7864), 1,
      anon_sym_PIPE,
    STATE(5538), 1,
      sym_heredoc_body,
    STATE(5547), 1,
      aux_sym_block_parameters_repeat1,
  [188846] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5539), 1,
      sym_heredoc_body,
    ACTIONS(7866), 3,
      anon_sym_end,
      anon_sym_when,
      anon_sym_else,
  [188861] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7447), 1,
      sym__string_end,
    ACTIONS(7868), 1,
      aux_sym_string_array_token1,
    STATE(5540), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [188880] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7457), 1,
      sym__string_end,
    ACTIONS(7870), 1,
      aux_sym_string_array_token1,
    STATE(5541), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [188899] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5542), 1,
      sym_heredoc_body,
    ACTIONS(3070), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [188914] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5543), 1,
      sym_heredoc_body,
    ACTIONS(7872), 3,
      sym__line_break,
      anon_sym_SEMI,
      anon_sym_then,
  [188929] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5544), 1,
      sym_heredoc_body,
    ACTIONS(7874), 3,
      anon_sym_end,
      anon_sym_in,
      anon_sym_else,
  [188944] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5545), 1,
      sym_heredoc_body,
    ACTIONS(7585), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [188959] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7422), 1,
      sym__string_end,
    ACTIONS(7876), 1,
      aux_sym_string_array_token1,
    STATE(5546), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [188978] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7878), 1,
      anon_sym_COMMA,
    ACTIONS(7881), 1,
      anon_sym_PIPE,
    STATE(5547), 2,
      sym_heredoc_body,
      aux_sym_block_parameters_repeat1,
  [188995] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7883), 1,
      anon_sym_PIPE,
    STATE(5519), 1,
      aux_sym_block_parameters_repeat1,
    STATE(5548), 1,
      sym_heredoc_body,
  [189014] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7784), 1,
      anon_sym_COMMA,
    ACTIONS(7885), 1,
      anon_sym_RPAREN,
    STATE(5549), 1,
      sym_heredoc_body,
    STATE(5567), 1,
      aux_sym_parameters_repeat1,
  [189033] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5550), 1,
      sym_heredoc_body,
    ACTIONS(7557), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [189048] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7480), 1,
      sym__string_end,
    ACTIONS(7887), 1,
      aux_sym_string_array_token1,
    STATE(5551), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [189067] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5552), 1,
      sym_heredoc_body,
    ACTIONS(7559), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [189082] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5553), 1,
      sym_heredoc_body,
    ACTIONS(3078), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [189097] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7889), 1,
      anon_sym_i,
    ACTIONS(7891), 1,
      anon_sym_ri,
    ACTIONS(7893), 1,
      anon_sym_r,
    STATE(5554), 1,
      sym_heredoc_body,
  [189116] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5555), 1,
      sym_heredoc_body,
    ACTIONS(7563), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [189131] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5556), 1,
      sym_heredoc_body,
    ACTIONS(7895), 3,
      anon_sym_end,
      anon_sym_in,
      anon_sym_else,
  [189146] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7897), 1,
      anon_sym_i,
    ACTIONS(7899), 1,
      anon_sym_ri,
    ACTIONS(7901), 1,
      anon_sym_r,
    STATE(5557), 1,
      sym_heredoc_body,
  [189165] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5558), 1,
      sym_heredoc_body,
    ACTIONS(7603), 3,
      sym__line_break,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [189180] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7903), 1,
      anon_sym_PIPE,
    STATE(5509), 1,
      aux_sym_block_parameters_repeat1,
    STATE(5559), 1,
      sym_heredoc_body,
  [189199] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7905), 1,
      anon_sym_i,
    ACTIONS(7907), 1,
      anon_sym_ri,
    ACTIONS(7909), 1,
      anon_sym_r,
    STATE(5560), 1,
      sym_heredoc_body,
  [189218] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2564), 1,
      anon_sym_RBRACE,
    ACTIONS(7911), 1,
      anon_sym_COMMA,
    STATE(5561), 1,
      sym_heredoc_body,
    STATE(5566), 1,
      aux_sym_hash_repeat1,
  [189237] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7445), 1,
      sym__string_end,
    ACTIONS(7913), 1,
      aux_sym_string_array_token1,
    STATE(5562), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [189256] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7311), 1,
      sym__string_end,
    ACTIONS(7915), 1,
      aux_sym_string_array_token1,
    STATE(5563), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [189275] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(241), 1,
      sym__terminator,
    STATE(5564), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [189292] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(205), 1,
      sym__terminator,
    STATE(5565), 1,
      sym_heredoc_body,
    ACTIONS(751), 2,
      sym__line_break,
      anon_sym_SEMI,
  [189309] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7917), 1,
      anon_sym_RBRACE,
    ACTIONS(7919), 1,
      anon_sym_COMMA,
    STATE(5566), 2,
      sym_heredoc_body,
      aux_sym_hash_repeat1,
  [189326] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7585), 1,
      anon_sym_RPAREN,
    ACTIONS(7922), 1,
      anon_sym_COMMA,
    STATE(5567), 2,
      sym_heredoc_body,
      aux_sym_parameters_repeat1,
  [189343] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7498), 1,
      sym__string_end,
    ACTIONS(7925), 1,
      aux_sym_string_array_token1,
    STATE(5568), 1,
      sym_heredoc_body,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
  [189362] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7927), 1,
      aux_sym_string_array_token1,
    ACTIONS(7930), 1,
      sym__string_end,
    STATE(5569), 2,
      sym_heredoc_body,
      aux_sym_string_array_repeat1,
  [189379] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7774), 1,
      anon_sym_COMMA,
    ACTIONS(7932), 1,
      anon_sym_PIPE,
    STATE(5495), 1,
      aux_sym_block_parameters_repeat1,
    STATE(5570), 1,
      sym_heredoc_body,
  [189398] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7934), 1,
      anon_sym_i,
    ACTIONS(7936), 1,
      anon_sym_ri,
    ACTIONS(7938), 1,
      anon_sym_r,
    STATE(5571), 1,
      sym_heredoc_body,
  [189417] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7508), 1,
      sym__string_end,
    ACTIONS(7940), 1,
      aux_sym_string_array_token1,
    STATE(5572), 1,
      sym_heredoc_body,
    STATE(5573), 1,
      aux_sym_symbol_array_repeat1,
  [189436] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7942), 1,
      aux_sym_string_array_token1,
    ACTIONS(7945), 1,
      sym__string_end,
    STATE(5573), 2,
      sym_heredoc_body,
      aux_sym_symbol_array_repeat1,
  [189453] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(7510), 1,
      sym__string_end,
    ACTIONS(7947), 1,
      aux_sym_string_array_token1,
    STATE(5569), 1,
      aux_sym_string_array_repeat1,
    STATE(5574), 1,
      sym_heredoc_body,
  [189472] = 6,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7784), 1,
      anon_sym_COMMA,
    ACTIONS(7949), 1,
      anon_sym_RPAREN,
    STATE(5529), 1,
      aux_sym_parameters_repeat1,
    STATE(5575), 1,
      sym_heredoc_body,
  [189491] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5576), 1,
      sym_heredoc_body,
    ACTIONS(7951), 3,
      anon_sym_end,
      anon_sym_when,
      anon_sym_else,
  [189506] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5577), 1,
      sym_heredoc_body,
    ACTIONS(7953), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189520] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5578), 1,
      sym_heredoc_body,
    ACTIONS(7955), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189534] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5579), 1,
      sym_heredoc_body,
    ACTIONS(7957), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189548] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5580), 1,
      sym_heredoc_body,
    ACTIONS(7959), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189562] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5581), 1,
      sym_heredoc_body,
    ACTIONS(7961), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189576] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5582), 1,
      sym_heredoc_body,
    ACTIONS(7963), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189590] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5583), 1,
      sym_heredoc_body,
    ACTIONS(7965), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189604] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5584), 1,
      sym_heredoc_body,
    ACTIONS(7967), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189618] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    ACTIONS(7969), 1,
      anon_sym_RPAREN,
    STATE(5585), 1,
      sym_heredoc_body,
  [189634] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5586), 1,
      sym_heredoc_body,
    ACTIONS(7971), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189648] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5587), 1,
      sym_heredoc_body,
    ACTIONS(7973), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189662] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7159), 1,
      anon_sym_in,
    STATE(5324), 1,
      sym_in,
    STATE(5588), 1,
      sym_heredoc_body,
  [189678] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5589), 1,
      sym_heredoc_body,
    ACTIONS(7975), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189692] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5590), 1,
      sym_heredoc_body,
    ACTIONS(7977), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189706] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5591), 1,
      sym_heredoc_body,
    ACTIONS(7979), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189720] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5592), 1,
      sym_heredoc_body,
    ACTIONS(7981), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189734] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7983), 1,
      ts_builtin_sym_end,
    ACTIONS(7985), 1,
      aux_sym_program_token1,
    STATE(5593), 1,
      sym_heredoc_body,
  [189750] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5594), 1,
      sym_heredoc_body,
    ACTIONS(7987), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189764] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5595), 1,
      sym_heredoc_body,
    ACTIONS(7989), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189778] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5596), 1,
      sym_heredoc_body,
    ACTIONS(7991), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189792] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5597), 1,
      sym_heredoc_body,
    ACTIONS(7993), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189806] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7159), 1,
      anon_sym_in,
    STATE(5299), 1,
      sym_in,
    STATE(5598), 1,
      sym_heredoc_body,
  [189822] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5599), 1,
      sym_heredoc_body,
    ACTIONS(7995), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189836] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5600), 1,
      sym_heredoc_body,
    ACTIONS(7997), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189850] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5601), 1,
      sym_heredoc_body,
    ACTIONS(7999), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189864] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5602), 1,
      sym_heredoc_body,
    ACTIONS(8001), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189878] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5603), 1,
      sym_heredoc_body,
    ACTIONS(8003), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189892] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5604), 1,
      sym_heredoc_body,
    ACTIONS(8005), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189906] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7159), 1,
      anon_sym_in,
    STATE(5308), 1,
      sym_in,
    STATE(5605), 1,
      sym_heredoc_body,
  [189922] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8007), 1,
      anon_sym_RPAREN,
    ACTIONS(8009), 1,
      anon_sym_EQ,
    STATE(5606), 1,
      sym_heredoc_body,
  [189938] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5607), 1,
      sym_heredoc_body,
    ACTIONS(8011), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189952] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2654), 1,
      anon_sym_LPAREN,
    STATE(5358), 1,
      sym_parenthesized_statements,
    STATE(5608), 1,
      sym_heredoc_body,
  [189968] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5609), 1,
      sym_heredoc_body,
    ACTIONS(8013), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189982] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5610), 1,
      sym_heredoc_body,
    ACTIONS(8015), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [189996] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5611), 1,
      sym_heredoc_body,
    ACTIONS(8017), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190010] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5612), 1,
      sym_heredoc_body,
    ACTIONS(8019), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190024] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5613), 1,
      sym_heredoc_body,
    ACTIONS(8021), 2,
      anon_sym_COMMA,
      anon_sym_PIPE,
  [190038] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5614), 1,
      sym_heredoc_body,
    ACTIONS(8023), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190052] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5615), 1,
      sym_heredoc_body,
    ACTIONS(8025), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190066] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5616), 1,
      sym_heredoc_body,
    ACTIONS(8027), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190080] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5617), 1,
      sym_heredoc_body,
    ACTIONS(8029), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190094] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5618), 1,
      sym_heredoc_body,
    ACTIONS(7917), 2,
      anon_sym_RBRACE,
      anon_sym_COMMA,
  [190108] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5619), 1,
      sym_heredoc_body,
    ACTIONS(8031), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190122] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5620), 1,
      sym_heredoc_body,
    ACTIONS(8033), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190136] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5621), 1,
      sym_heredoc_body,
    ACTIONS(8035), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190150] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5622), 1,
      sym_heredoc_body,
    ACTIONS(8037), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190164] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5623), 1,
      sym_heredoc_body,
    ACTIONS(8039), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190178] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5624), 1,
      sym_heredoc_body,
    ACTIONS(8041), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190192] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6959), 1,
      anon_sym_EQ_GT,
    ACTIONS(8043), 1,
      anon_sym_RPAREN,
    STATE(5625), 1,
      sym_heredoc_body,
  [190208] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5626), 1,
      sym_heredoc_body,
    ACTIONS(8045), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190222] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7770), 1,
      anon_sym_PIPE,
    ACTIONS(7772), 1,
      anon_sym_SEMI,
    STATE(5627), 1,
      sym_heredoc_body,
  [190238] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5628), 1,
      sym_heredoc_body,
    ACTIONS(743), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190252] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5629), 1,
      sym_heredoc_body,
    ACTIONS(8047), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190266] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    STATE(5630), 1,
      sym_heredoc_body,
    ACTIONS(8009), 2,
      anon_sym_EQ,
      anon_sym_in,
  [190280] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5631), 1,
      sym_heredoc_body,
    ACTIONS(8049), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190294] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5632), 1,
      sym_heredoc_body,
    ACTIONS(8051), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190308] = 5,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7706), 1,
      anon_sym_PIPE,
    ACTIONS(7708), 1,
      anon_sym_SEMI,
    STATE(5633), 1,
      sym_heredoc_body,
  [190324] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5634), 1,
      sym_heredoc_body,
    ACTIONS(8053), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190338] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5635), 1,
      sym_heredoc_body,
    ACTIONS(8055), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190352] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5636), 1,
      sym_heredoc_body,
    ACTIONS(8057), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190366] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5637), 1,
      sym_heredoc_body,
    ACTIONS(8059), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190380] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5638), 1,
      sym_heredoc_body,
    ACTIONS(7727), 2,
      anon_sym_RPAREN,
      anon_sym_RBRACK,
  [190394] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    STATE(5639), 1,
      sym_heredoc_body,
    ACTIONS(8061), 2,
      anon_sym_DOT,
      anon_sym_COLON_COLON,
  [190408] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8063), 1,
      anon_sym_end,
    STATE(5640), 1,
      sym_heredoc_body,
  [190421] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8065), 1,
      anon_sym_RPAREN,
    STATE(5641), 1,
      sym_heredoc_body,
  [190434] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8067), 1,
      anon_sym_end,
    STATE(5642), 1,
      sym_heredoc_body,
  [190447] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8069), 1,
      anon_sym_end,
    STATE(5643), 1,
      sym_heredoc_body,
  [190460] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8071), 1,
      anon_sym_EQ,
    STATE(5644), 1,
      sym_heredoc_body,
  [190473] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8073), 1,
      anon_sym_end,
    STATE(5645), 1,
      sym_heredoc_body,
  [190486] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8075), 1,
      anon_sym_end,
    STATE(5646), 1,
      sym_heredoc_body,
  [190499] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8077), 1,
      sym_constant,
    STATE(5647), 1,
      sym_heredoc_body,
  [190512] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8079), 1,
      anon_sym_end,
    STATE(5648), 1,
      sym_heredoc_body,
  [190525] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8081), 1,
      anon_sym_EQ,
    STATE(5649), 1,
      sym_heredoc_body,
  [190538] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8083), 1,
      anon_sym_end,
    STATE(5650), 1,
      sym_heredoc_body,
  [190551] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8085), 1,
      anon_sym_EQ,
    STATE(5651), 1,
      sym_heredoc_body,
  [190564] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8087), 1,
      anon_sym_RBRACE,
    STATE(5652), 1,
      sym_heredoc_body,
  [190577] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8089), 1,
      anon_sym_end,
    STATE(5653), 1,
      sym_heredoc_body,
  [190590] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8091), 1,
      anon_sym_end,
    STATE(5654), 1,
      sym_heredoc_body,
  [190603] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8093), 1,
      anon_sym_end,
    STATE(5655), 1,
      sym_heredoc_body,
  [190616] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8095), 1,
      anon_sym_end,
    STATE(5656), 1,
      sym_heredoc_body,
  [190629] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6841), 1,
      anon_sym_COLON,
    STATE(5657), 1,
      sym_heredoc_body,
  [190642] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8097), 1,
      anon_sym_EQ,
    STATE(5658), 1,
      sym_heredoc_body,
  [190655] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8099), 1,
      anon_sym_EQ,
    STATE(5659), 1,
      sym_heredoc_body,
  [190668] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8101), 1,
      anon_sym_RBRACK,
    STATE(5660), 1,
      sym_heredoc_body,
  [190681] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8103), 1,
      anon_sym_RBRACK,
    STATE(5661), 1,
      sym_heredoc_body,
  [190694] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8105), 1,
      anon_sym_RBRACK,
    STATE(5662), 1,
      sym_heredoc_body,
  [190707] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8107), 1,
      anon_sym_end,
    STATE(5663), 1,
      sym_heredoc_body,
  [190720] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8109), 1,
      anon_sym_COLON,
    STATE(5664), 1,
      sym_heredoc_body,
  [190733] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8111), 1,
      anon_sym_end,
    STATE(5665), 1,
      sym_heredoc_body,
  [190746] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8113), 1,
      anon_sym_end,
    STATE(5666), 1,
      sym_heredoc_body,
  [190759] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8115), 1,
      anon_sym_end,
    STATE(5667), 1,
      sym_heredoc_body,
  [190772] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8117), 1,
      anon_sym_EQ,
    STATE(5668), 1,
      sym_heredoc_body,
  [190785] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8119), 1,
      anon_sym_end,
    STATE(5669), 1,
      sym_heredoc_body,
  [190798] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8121), 1,
      anon_sym_EQ,
    STATE(5670), 1,
      sym_heredoc_body,
  [190811] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8123), 1,
      sym__string_end,
    STATE(5671), 1,
      sym_heredoc_body,
  [190824] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8125), 1,
      anon_sym_end,
    STATE(5672), 1,
      sym_heredoc_body,
  [190837] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8127), 1,
      anon_sym_end,
    STATE(5673), 1,
      sym_heredoc_body,
  [190850] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8129), 1,
      anon_sym_end,
    STATE(5674), 1,
      sym_heredoc_body,
  [190863] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8131), 1,
      anon_sym_end,
    STATE(5675), 1,
      sym_heredoc_body,
  [190876] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8133), 1,
      anon_sym_EQ,
    STATE(5676), 1,
      sym_heredoc_body,
  [190889] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8135), 1,
      sym_constant,
    STATE(5677), 1,
      sym_heredoc_body,
  [190902] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8137), 1,
      anon_sym_end,
    STATE(5678), 1,
      sym_heredoc_body,
  [190915] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8139), 1,
      anon_sym_COLON,
    STATE(5679), 1,
      sym_heredoc_body,
  [190928] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8141), 1,
      anon_sym_EQ,
    STATE(5680), 1,
      sym_heredoc_body,
  [190941] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8143), 1,
      sym_identifier,
    STATE(5681), 1,
      sym_heredoc_body,
  [190954] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8145), 1,
      anon_sym_LBRACE,
    STATE(5682), 1,
      sym_heredoc_body,
  [190967] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8147), 1,
      anon_sym_RBRACK,
    STATE(5683), 1,
      sym_heredoc_body,
  [190980] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8149), 1,
      anon_sym_end,
    STATE(5684), 1,
      sym_heredoc_body,
  [190993] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8151), 1,
      anon_sym_EQ,
    STATE(5685), 1,
      sym_heredoc_body,
  [191006] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8153), 1,
      anon_sym_RBRACE,
    STATE(5686), 1,
      sym_heredoc_body,
  [191019] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8155), 1,
      anon_sym_end,
    STATE(5687), 1,
      sym_heredoc_body,
  [191032] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8101), 1,
      anon_sym_RPAREN,
    STATE(5688), 1,
      sym_heredoc_body,
  [191045] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8157), 1,
      anon_sym_end,
    STATE(5689), 1,
      sym_heredoc_body,
  [191058] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8159), 1,
      anon_sym_end,
    STATE(5690), 1,
      sym_heredoc_body,
  [191071] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7567), 1,
      anon_sym_end,
    STATE(5691), 1,
      sym_heredoc_body,
  [191084] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8161), 1,
      anon_sym_EQ,
    STATE(5692), 1,
      sym_heredoc_body,
  [191097] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8163), 1,
      anon_sym_RBRACK,
    STATE(5693), 1,
      sym_heredoc_body,
  [191110] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8103), 1,
      anon_sym_RPAREN,
    STATE(5694), 1,
      sym_heredoc_body,
  [191123] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8165), 1,
      sym_constant,
    STATE(5695), 1,
      sym_heredoc_body,
  [191136] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8167), 1,
      anon_sym_end,
    STATE(5696), 1,
      sym_heredoc_body,
  [191149] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8169), 1,
      anon_sym_EQ,
    STATE(5697), 1,
      sym_heredoc_body,
  [191162] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8171), 1,
      anon_sym_EQ,
    STATE(5698), 1,
      sym_heredoc_body,
  [191175] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8173), 1,
      anon_sym_EQ,
    STATE(5699), 1,
      sym_heredoc_body,
  [191188] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8175), 1,
      anon_sym_RBRACE,
    STATE(5700), 1,
      sym_heredoc_body,
  [191201] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8177), 1,
      anon_sym_RBRACE,
    STATE(5701), 1,
      sym_heredoc_body,
  [191214] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8179), 1,
      anon_sym_end,
    STATE(5702), 1,
      sym_heredoc_body,
  [191227] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8105), 1,
      anon_sym_RPAREN,
    STATE(5703), 1,
      sym_heredoc_body,
  [191240] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8181), 1,
      anon_sym_end,
    STATE(5704), 1,
      sym_heredoc_body,
  [191253] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8183), 1,
      anon_sym_EQ,
    STATE(5705), 1,
      sym_heredoc_body,
  [191266] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8185), 1,
      anon_sym_end,
    STATE(5706), 1,
      sym_heredoc_body,
  [191279] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8187), 1,
      anon_sym_EQ,
    STATE(5707), 1,
      sym_heredoc_body,
  [191292] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8189), 1,
      anon_sym_end,
    STATE(5708), 1,
      sym_heredoc_body,
  [191305] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8191), 1,
      anon_sym_RBRACE,
    STATE(5709), 1,
      sym_heredoc_body,
  [191318] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8193), 1,
      anon_sym_end,
    STATE(5710), 1,
      sym_heredoc_body,
  [191331] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8195), 1,
      anon_sym_COLON,
    STATE(5711), 1,
      sym_heredoc_body,
  [191344] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8197), 1,
      anon_sym_EQ,
    STATE(5712), 1,
      sym_heredoc_body,
  [191357] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8199), 1,
      anon_sym_end,
    STATE(5713), 1,
      sym_heredoc_body,
  [191370] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8201), 1,
      anon_sym_COLON,
    STATE(5714), 1,
      sym_heredoc_body,
  [191383] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8203), 1,
      ts_builtin_sym_end,
    STATE(5715), 1,
      sym_heredoc_body,
  [191396] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8205), 1,
      anon_sym_EQ,
    STATE(5716), 1,
      sym_heredoc_body,
  [191409] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8207), 1,
      anon_sym_EQ,
    STATE(5717), 1,
      sym_heredoc_body,
  [191422] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8209), 1,
      anon_sym_end,
    STATE(5718), 1,
      sym_heredoc_body,
  [191435] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8211), 1,
      anon_sym_RBRACE,
    STATE(5719), 1,
      sym_heredoc_body,
  [191448] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8213), 1,
      anon_sym_end,
    STATE(5720), 1,
      sym_heredoc_body,
  [191461] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8215), 1,
      anon_sym_end,
    STATE(5721), 1,
      sym_heredoc_body,
  [191474] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8217), 1,
      anon_sym_EQ,
    STATE(5722), 1,
      sym_heredoc_body,
  [191487] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6727), 1,
      anon_sym_COLON,
    STATE(5723), 1,
      sym_heredoc_body,
  [191500] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8219), 1,
      anon_sym_EQ,
    STATE(5724), 1,
      sym_heredoc_body,
  [191513] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8221), 1,
      anon_sym_LBRACE,
    STATE(5725), 1,
      sym_heredoc_body,
  [191526] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8223), 1,
      anon_sym_EQ,
    STATE(5726), 1,
      sym_heredoc_body,
  [191539] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8225), 1,
      anon_sym_EQ,
    STATE(5727), 1,
      sym_heredoc_body,
  [191552] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7099), 1,
      anon_sym_end,
    STATE(5728), 1,
      sym_heredoc_body,
  [191565] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8227), 1,
      anon_sym_EQ,
    STATE(5729), 1,
      sym_heredoc_body,
  [191578] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8229), 1,
      sym_identifier,
    STATE(5730), 1,
      sym_heredoc_body,
  [191591] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8231), 1,
      anon_sym_EQ,
    STATE(5731), 1,
      sym_heredoc_body,
  [191604] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7056), 1,
      anon_sym_end,
    STATE(5732), 1,
      sym_heredoc_body,
  [191617] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7077), 1,
      anon_sym_end,
    STATE(5733), 1,
      sym_heredoc_body,
  [191630] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8233), 1,
      anon_sym_end,
    STATE(5734), 1,
      sym_heredoc_body,
  [191643] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7579), 1,
      anon_sym_end,
    STATE(5735), 1,
      sym_heredoc_body,
  [191656] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8235), 1,
      anon_sym_RBRACK,
    STATE(5736), 1,
      sym_heredoc_body,
  [191669] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8237), 1,
      anon_sym_EQ,
    STATE(5737), 1,
      sym_heredoc_body,
  [191682] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8239), 1,
      anon_sym_EQ,
    STATE(5738), 1,
      sym_heredoc_body,
  [191695] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8241), 1,
      anon_sym_end,
    STATE(5739), 1,
      sym_heredoc_body,
  [191708] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8243), 1,
      anon_sym_end,
    STATE(5740), 1,
      sym_heredoc_body,
  [191721] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8245), 1,
      anon_sym_COLON,
    STATE(5741), 1,
      sym_heredoc_body,
  [191734] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8247), 1,
      anon_sym_EQ,
    STATE(5742), 1,
      sym_heredoc_body,
  [191747] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8249), 1,
      anon_sym_end,
    STATE(5743), 1,
      sym_heredoc_body,
  [191760] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8251), 1,
      sym_constant,
    STATE(5744), 1,
      sym_heredoc_body,
  [191773] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8253), 1,
      sym__string_end,
    STATE(5745), 1,
      sym_heredoc_body,
  [191786] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8255), 1,
      anon_sym_EQ,
    STATE(5746), 1,
      sym_heredoc_body,
  [191799] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8257), 1,
      sym_identifier,
    STATE(5747), 1,
      sym_heredoc_body,
  [191812] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8259), 1,
      anon_sym_RPAREN,
    STATE(5748), 1,
      sym_heredoc_body,
  [191825] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8261), 1,
      anon_sym_end,
    STATE(5749), 1,
      sym_heredoc_body,
  [191838] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8263), 1,
      anon_sym_EQ,
    STATE(5750), 1,
      sym_heredoc_body,
  [191851] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8265), 1,
      anon_sym_EQ,
    STATE(5751), 1,
      sym_heredoc_body,
  [191864] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8267), 1,
      sym__string_end,
    STATE(5752), 1,
      sym_heredoc_body,
  [191877] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7020), 1,
      anon_sym_end,
    STATE(5753), 1,
      sym_heredoc_body,
  [191890] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8269), 1,
      anon_sym_EQ,
    STATE(5754), 1,
      sym_heredoc_body,
  [191903] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8271), 1,
      anon_sym_end,
    STATE(5755), 1,
      sym_heredoc_body,
  [191916] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8273), 1,
      anon_sym_COLON,
    STATE(5756), 1,
      sym_heredoc_body,
  [191929] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8275), 1,
      anon_sym_EQ,
    STATE(5757), 1,
      sym_heredoc_body,
  [191942] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8277), 1,
      anon_sym_RBRACK,
    STATE(5758), 1,
      sym_heredoc_body,
  [191955] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8279), 1,
      sym_constant,
    STATE(5759), 1,
      sym_heredoc_body,
  [191968] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8281), 1,
      anon_sym_RPAREN,
    STATE(5760), 1,
      sym_heredoc_body,
  [191981] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7597), 1,
      anon_sym_end,
    STATE(5761), 1,
      sym_heredoc_body,
  [191994] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8283), 1,
      anon_sym_RBRACE,
    STATE(5762), 1,
      sym_heredoc_body,
  [192007] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8285), 1,
      anon_sym_RBRACK,
    STATE(5763), 1,
      sym_heredoc_body,
  [192020] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6477), 1,
      anon_sym_COLON,
    STATE(5764), 1,
      sym_heredoc_body,
  [192033] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8287), 1,
      sym_constant,
    STATE(5765), 1,
      sym_heredoc_body,
  [192046] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8289), 1,
      anon_sym_EQ,
    STATE(5766), 1,
      sym_heredoc_body,
  [192059] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8291), 1,
      anon_sym_EQ,
    STATE(5767), 1,
      sym_heredoc_body,
  [192072] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8293), 1,
      anon_sym_end,
    STATE(5768), 1,
      sym_heredoc_body,
  [192085] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8295), 1,
      sym__string_end,
    STATE(5769), 1,
      sym_heredoc_body,
  [192098] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8297), 1,
      sym_identifier,
    STATE(5770), 1,
      sym_heredoc_body,
  [192111] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8299), 1,
      sym_constant,
    STATE(5771), 1,
      sym_heredoc_body,
  [192124] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8301), 1,
      anon_sym_RBRACE,
    STATE(5772), 1,
      sym_heredoc_body,
  [192137] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6862), 1,
      anon_sym_COLON,
    STATE(5773), 1,
      sym_heredoc_body,
  [192150] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8303), 1,
      anon_sym_RBRACK,
    STATE(5774), 1,
      sym_heredoc_body,
  [192163] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8305), 1,
      anon_sym_RBRACK,
    STATE(5775), 1,
      sym_heredoc_body,
  [192176] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8307), 1,
      anon_sym_end,
    STATE(5776), 1,
      sym_heredoc_body,
  [192189] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8309), 1,
      anon_sym_end,
    STATE(5777), 1,
      sym_heredoc_body,
  [192202] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8311), 1,
      anon_sym_end,
    STATE(5778), 1,
      sym_heredoc_body,
  [192215] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8313), 1,
      anon_sym_RBRACK,
    STATE(5779), 1,
      sym_heredoc_body,
  [192228] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2644), 1,
      anon_sym_COLON,
    STATE(5780), 1,
      sym_heredoc_body,
  [192241] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8315), 1,
      anon_sym_end,
    STATE(5781), 1,
      sym_heredoc_body,
  [192254] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8317), 1,
      sym_constant,
    STATE(5782), 1,
      sym_heredoc_body,
  [192267] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8319), 1,
      anon_sym_EQ,
    STATE(5783), 1,
      sym_heredoc_body,
  [192280] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8321), 1,
      anon_sym_EQ,
    STATE(5784), 1,
      sym_heredoc_body,
  [192293] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8323), 1,
      anon_sym_end,
    STATE(5785), 1,
      sym_heredoc_body,
  [192306] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8325), 1,
      anon_sym_end,
    STATE(5786), 1,
      sym_heredoc_body,
  [192319] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8327), 1,
      anon_sym_end,
    STATE(5787), 1,
      sym_heredoc_body,
  [192332] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8235), 1,
      anon_sym_RPAREN,
    STATE(5788), 1,
      sym_heredoc_body,
  [192345] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8305), 1,
      anon_sym_RPAREN,
    STATE(5789), 1,
      sym_heredoc_body,
  [192358] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8329), 1,
      anon_sym_RBRACE,
    STATE(5790), 1,
      sym_heredoc_body,
  [192371] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8331), 1,
      sym__string_end,
    STATE(5791), 1,
      sym_heredoc_body,
  [192384] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8333), 1,
      sym_constant,
    STATE(5792), 1,
      sym_heredoc_body,
  [192397] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8313), 1,
      anon_sym_RPAREN,
    STATE(5793), 1,
      sym_heredoc_body,
  [192410] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8335), 1,
      anon_sym_EQ,
    STATE(5794), 1,
      sym_heredoc_body,
  [192423] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8337), 1,
      anon_sym_end,
    STATE(5795), 1,
      sym_heredoc_body,
  [192436] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8339), 1,
      anon_sym_EQ,
    STATE(5796), 1,
      sym_heredoc_body,
  [192449] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8341), 1,
      anon_sym_end,
    STATE(5797), 1,
      sym_heredoc_body,
  [192462] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8343), 1,
      anon_sym_EQ,
    STATE(5798), 1,
      sym_heredoc_body,
  [192475] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8345), 1,
      anon_sym_end,
    STATE(5799), 1,
      sym_heredoc_body,
  [192488] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8347), 1,
      anon_sym_end,
    STATE(5800), 1,
      sym_heredoc_body,
  [192501] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8349), 1,
      anon_sym_RBRACE,
    STATE(5801), 1,
      sym_heredoc_body,
  [192514] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8351), 1,
      anon_sym_end,
    STATE(5802), 1,
      sym_heredoc_body,
  [192527] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8353), 1,
      anon_sym_RBRACE,
    STATE(5803), 1,
      sym_heredoc_body,
  [192540] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8355), 1,
      anon_sym_end,
    STATE(5804), 1,
      sym_heredoc_body,
  [192553] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8357), 1,
      anon_sym_end,
    STATE(5805), 1,
      sym_heredoc_body,
  [192566] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8359), 1,
      anon_sym_RBRACK,
    STATE(5806), 1,
      sym_heredoc_body,
  [192579] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8361), 1,
      anon_sym_RBRACE,
    STATE(5807), 1,
      sym_heredoc_body,
  [192592] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8363), 1,
      anon_sym_end,
    STATE(5808), 1,
      sym_heredoc_body,
  [192605] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8365), 1,
      anon_sym_end,
    STATE(5809), 1,
      sym_heredoc_body,
  [192618] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8367), 1,
      anon_sym_end,
    STATE(5810), 1,
      sym_heredoc_body,
  [192631] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8369), 1,
      anon_sym_EQ,
    STATE(5811), 1,
      sym_heredoc_body,
  [192644] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8371), 1,
      anon_sym_end,
    STATE(5812), 1,
      sym_heredoc_body,
  [192657] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6875), 1,
      anon_sym_COLON,
    STATE(5813), 1,
      sym_heredoc_body,
  [192670] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8373), 1,
      anon_sym_RBRACE,
    STATE(5814), 1,
      sym_heredoc_body,
  [192683] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8375), 1,
      anon_sym_RPAREN,
    STATE(5815), 1,
      sym_heredoc_body,
  [192696] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8377), 1,
      anon_sym_EQ,
    STATE(5816), 1,
      sym_heredoc_body,
  [192709] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8379), 1,
      anon_sym_EQ,
    STATE(5817), 1,
      sym_heredoc_body,
  [192722] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8381), 1,
      sym__string_end,
    STATE(5818), 1,
      sym_heredoc_body,
  [192735] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8383), 1,
      anon_sym_end,
    STATE(5819), 1,
      sym_heredoc_body,
  [192748] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8385), 1,
      anon_sym_end,
    STATE(5820), 1,
      sym_heredoc_body,
  [192761] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8387), 1,
      sym_constant,
    STATE(5821), 1,
      sym_heredoc_body,
  [192774] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8389), 1,
      sym__string_end,
    STATE(5822), 1,
      sym_heredoc_body,
  [192787] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8391), 1,
      anon_sym_end,
    STATE(5823), 1,
      sym_heredoc_body,
  [192800] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7529), 1,
      anon_sym_end,
    STATE(5824), 1,
      sym_heredoc_body,
  [192813] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8393), 1,
      anon_sym_EQ,
    STATE(5825), 1,
      sym_heredoc_body,
  [192826] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8395), 1,
      anon_sym_RBRACE,
    STATE(5826), 1,
      sym_heredoc_body,
  [192839] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8397), 1,
      anon_sym_RBRACK,
    STATE(5827), 1,
      sym_heredoc_body,
  [192852] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8399), 1,
      anon_sym_end,
    STATE(5828), 1,
      sym_heredoc_body,
  [192865] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8401), 1,
      anon_sym_RBRACE,
    STATE(5829), 1,
      sym_heredoc_body,
  [192878] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7006), 1,
      anon_sym_end,
    STATE(5830), 1,
      sym_heredoc_body,
  [192891] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8403), 1,
      anon_sym_RBRACE,
    STATE(5831), 1,
      sym_heredoc_body,
  [192904] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8405), 1,
      anon_sym_COLON,
    STATE(5832), 1,
      sym_heredoc_body,
  [192917] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8407), 1,
      anon_sym_end,
    STATE(5833), 1,
      sym_heredoc_body,
  [192930] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8409), 1,
      anon_sym_end,
    STATE(5834), 1,
      sym_heredoc_body,
  [192943] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8411), 1,
      anon_sym_EQ,
    STATE(5835), 1,
      sym_heredoc_body,
  [192956] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8413), 1,
      sym_constant,
    STATE(5836), 1,
      sym_heredoc_body,
  [192969] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8415), 1,
      anon_sym_end,
    STATE(5837), 1,
      sym_heredoc_body,
  [192982] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8417), 1,
      anon_sym_end,
    STATE(5838), 1,
      sym_heredoc_body,
  [192995] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6912), 1,
      anon_sym_COLON,
    STATE(5839), 1,
      sym_heredoc_body,
  [193008] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8419), 1,
      anon_sym_end,
    STATE(5840), 1,
      sym_heredoc_body,
  [193021] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6533), 1,
      anon_sym_COLON,
    STATE(5841), 1,
      sym_heredoc_body,
  [193034] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8421), 1,
      anon_sym_end,
    STATE(5842), 1,
      sym_heredoc_body,
  [193047] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8423), 1,
      anon_sym_EQ,
    STATE(5843), 1,
      sym_heredoc_body,
  [193060] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8425), 1,
      anon_sym_EQ,
    STATE(5844), 1,
      sym_heredoc_body,
  [193073] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8427), 1,
      anon_sym_EQ,
    STATE(5845), 1,
      sym_heredoc_body,
  [193086] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8429), 1,
      anon_sym_end,
    STATE(5846), 1,
      sym_heredoc_body,
  [193099] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8431), 1,
      anon_sym_end,
    STATE(5847), 1,
      sym_heredoc_body,
  [193112] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8433), 1,
      anon_sym_RBRACK,
    STATE(5848), 1,
      sym_heredoc_body,
  [193125] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8435), 1,
      anon_sym_end,
    STATE(5849), 1,
      sym_heredoc_body,
  [193138] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8437), 1,
      anon_sym_EQ,
    STATE(5850), 1,
      sym_heredoc_body,
  [193151] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8439), 1,
      anon_sym_EQ,
    STATE(5851), 1,
      sym_heredoc_body,
  [193164] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8441), 1,
      anon_sym_EQ,
    STATE(5852), 1,
      sym_heredoc_body,
  [193177] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8443), 1,
      anon_sym_EQ,
    STATE(5853), 1,
      sym_heredoc_body,
  [193190] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8445), 1,
      anon_sym_end,
    STATE(5854), 1,
      sym_heredoc_body,
  [193203] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8447), 1,
      sym_constant,
    STATE(5855), 1,
      sym_heredoc_body,
  [193216] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8449), 1,
      anon_sym_RBRACE,
    STATE(5856), 1,
      sym_heredoc_body,
  [193229] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7595), 1,
      anon_sym_end,
    STATE(5857), 1,
      sym_heredoc_body,
  [193242] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8451), 1,
      anon_sym_COLON,
    STATE(5858), 1,
      sym_heredoc_body,
  [193255] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8453), 1,
      sym_identifier,
    STATE(5859), 1,
      sym_heredoc_body,
  [193268] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8455), 1,
      anon_sym_end,
    STATE(5860), 1,
      sym_heredoc_body,
  [193281] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8457), 1,
      anon_sym_end,
    STATE(5861), 1,
      sym_heredoc_body,
  [193294] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6785), 1,
      anon_sym_COLON,
    STATE(5862), 1,
      sym_heredoc_body,
  [193307] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8459), 1,
      anon_sym_EQ,
    STATE(5863), 1,
      sym_heredoc_body,
  [193320] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(2616), 1,
      anon_sym_COLON,
    STATE(5864), 1,
      sym_heredoc_body,
  [193333] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8461), 1,
      anon_sym_EQ,
    STATE(5865), 1,
      sym_heredoc_body,
  [193346] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8463), 1,
      anon_sym_RPAREN,
    STATE(5866), 1,
      sym_heredoc_body,
  [193359] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8465), 1,
      anon_sym_EQ,
    STATE(5867), 1,
      sym_heredoc_body,
  [193372] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8467), 1,
      anon_sym_end,
    STATE(5868), 1,
      sym_heredoc_body,
  [193385] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7591), 1,
      anon_sym_end,
    STATE(5869), 1,
      sym_heredoc_body,
  [193398] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8469), 1,
      anon_sym_end,
    STATE(5870), 1,
      sym_heredoc_body,
  [193411] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8471), 1,
      sym_identifier,
    STATE(5871), 1,
      sym_heredoc_body,
  [193424] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8473), 1,
      anon_sym_end,
    STATE(5872), 1,
      sym_heredoc_body,
  [193437] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8475), 1,
      anon_sym_end,
    STATE(5873), 1,
      sym_heredoc_body,
  [193450] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8477), 1,
      anon_sym_COLON,
    STATE(5874), 1,
      sym_heredoc_body,
  [193463] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8479), 1,
      sym_uninterpreted,
    STATE(5875), 1,
      sym_heredoc_body,
  [193476] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8481), 1,
      anon_sym_EQ,
    STATE(5876), 1,
      sym_heredoc_body,
  [193489] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8483), 1,
      anon_sym_EQ,
    STATE(5877), 1,
      sym_heredoc_body,
  [193502] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8485), 1,
      anon_sym_EQ,
    STATE(5878), 1,
      sym_heredoc_body,
  [193515] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7527), 1,
      anon_sym_end,
    STATE(5879), 1,
      sym_heredoc_body,
  [193528] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8487), 1,
      anon_sym_end,
    STATE(5880), 1,
      sym_heredoc_body,
  [193541] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8489), 1,
      anon_sym_end,
    STATE(5881), 1,
      sym_heredoc_body,
  [193554] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8491), 1,
      anon_sym_end,
    STATE(5882), 1,
      sym_heredoc_body,
  [193567] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8493), 1,
      ts_builtin_sym_end,
    STATE(5883), 1,
      sym_heredoc_body,
  [193580] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8495), 1,
      sym__string_end,
    STATE(5884), 1,
      sym_heredoc_body,
  [193593] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8497), 1,
      anon_sym_RBRACE,
    STATE(5885), 1,
      sym_heredoc_body,
  [193606] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8499), 1,
      anon_sym_end,
    STATE(5886), 1,
      sym_heredoc_body,
  [193619] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8501), 1,
      anon_sym_end,
    STATE(5887), 1,
      sym_heredoc_body,
  [193632] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8503), 1,
      anon_sym_end,
    STATE(5888), 1,
      sym_heredoc_body,
  [193645] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8505), 1,
      anon_sym_end,
    STATE(5889), 1,
      sym_heredoc_body,
  [193658] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8507), 1,
      anon_sym_EQ,
    STATE(5890), 1,
      sym_heredoc_body,
  [193671] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8509), 1,
      sym__string_end,
    STATE(5891), 1,
      sym_heredoc_body,
  [193684] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6961), 1,
      anon_sym_COLON,
    STATE(5892), 1,
      sym_heredoc_body,
  [193697] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8511), 1,
      anon_sym_RBRACE,
    STATE(5893), 1,
      sym_heredoc_body,
  [193710] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8513), 1,
      anon_sym_end,
    STATE(5894), 1,
      sym_heredoc_body,
  [193723] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8515), 1,
      anon_sym_EQ,
    STATE(5895), 1,
      sym_heredoc_body,
  [193736] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8517), 1,
      anon_sym_COLON,
    STATE(5896), 1,
      sym_heredoc_body,
  [193749] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8519), 1,
      ts_builtin_sym_end,
    STATE(5897), 1,
      sym_heredoc_body,
  [193762] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8521), 1,
      anon_sym_end,
    STATE(5898), 1,
      sym_heredoc_body,
  [193775] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8523), 1,
      anon_sym_EQ,
    STATE(5899), 1,
      sym_heredoc_body,
  [193788] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8525), 1,
      sym_constant,
    STATE(5900), 1,
      sym_heredoc_body,
  [193801] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8527), 1,
      anon_sym_end,
    STATE(5901), 1,
      sym_heredoc_body,
  [193814] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8529), 1,
      anon_sym_RBRACE,
    STATE(5902), 1,
      sym_heredoc_body,
  [193827] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8531), 1,
      anon_sym_end,
    STATE(5903), 1,
      sym_heredoc_body,
  [193840] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8533), 1,
      anon_sym_end,
    STATE(5904), 1,
      sym_heredoc_body,
  [193853] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8535), 1,
      anon_sym_EQ,
    STATE(5905), 1,
      sym_heredoc_body,
  [193866] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8537), 1,
      anon_sym_RBRACK,
    STATE(5906), 1,
      sym_heredoc_body,
  [193879] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8539), 1,
      anon_sym_EQ,
    STATE(5907), 1,
      sym_heredoc_body,
  [193892] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8541), 1,
      anon_sym_COLON,
    STATE(5908), 1,
      sym_heredoc_body,
  [193905] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8543), 1,
      anon_sym_RBRACK,
    STATE(5909), 1,
      sym_heredoc_body,
  [193918] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8545), 1,
      sym__string_end,
    STATE(5910), 1,
      sym_heredoc_body,
  [193931] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8547), 1,
      anon_sym_RBRACE,
    STATE(5911), 1,
      sym_heredoc_body,
  [193944] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8549), 1,
      anon_sym_end,
    STATE(5912), 1,
      sym_heredoc_body,
  [193957] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8551), 1,
      anon_sym_end,
    STATE(5913), 1,
      sym_heredoc_body,
  [193970] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8553), 1,
      anon_sym_end,
    STATE(5914), 1,
      sym_heredoc_body,
  [193983] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8555), 1,
      anon_sym_EQ,
    STATE(5915), 1,
      sym_heredoc_body,
  [193996] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7599), 1,
      anon_sym_end,
    STATE(5916), 1,
      sym_heredoc_body,
  [194009] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8557), 1,
      anon_sym_end,
    STATE(5917), 1,
      sym_heredoc_body,
  [194022] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8559), 1,
      anon_sym_EQ,
    STATE(5918), 1,
      sym_heredoc_body,
  [194035] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8561), 1,
      anon_sym_end,
    STATE(5919), 1,
      sym_heredoc_body,
  [194048] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8563), 1,
      anon_sym_end,
    STATE(5920), 1,
      sym_heredoc_body,
  [194061] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7551), 1,
      anon_sym_end,
    STATE(5921), 1,
      sym_heredoc_body,
  [194074] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8565), 1,
      sym_identifier,
    STATE(5922), 1,
      sym_heredoc_body,
  [194087] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8567), 1,
      anon_sym_end,
    STATE(5923), 1,
      sym_heredoc_body,
  [194100] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8569), 1,
      anon_sym_RBRACK,
    STATE(5924), 1,
      sym_heredoc_body,
  [194113] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8571), 1,
      anon_sym_EQ,
    STATE(5925), 1,
      sym_heredoc_body,
  [194126] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8573), 1,
      anon_sym_RBRACK,
    STATE(5926), 1,
      sym_heredoc_body,
  [194139] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8575), 1,
      anon_sym_RBRACE,
    STATE(5927), 1,
      sym_heredoc_body,
  [194152] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8577), 1,
      anon_sym_RPAREN,
    STATE(5928), 1,
      sym_heredoc_body,
  [194165] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8579), 1,
      anon_sym_end,
    STATE(5929), 1,
      sym_heredoc_body,
  [194178] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8581), 1,
      anon_sym_end,
    STATE(5930), 1,
      sym_heredoc_body,
  [194191] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8583), 1,
      anon_sym_end,
    STATE(5931), 1,
      sym_heredoc_body,
  [194204] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8585), 1,
      anon_sym_end,
    STATE(5932), 1,
      sym_heredoc_body,
  [194217] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8587), 1,
      anon_sym_EQ,
    STATE(5933), 1,
      sym_heredoc_body,
  [194230] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8589), 1,
      anon_sym_RBRACK,
    STATE(5934), 1,
      sym_heredoc_body,
  [194243] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8591), 1,
      anon_sym_RBRACK,
    STATE(5935), 1,
      sym_heredoc_body,
  [194256] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8593), 1,
      anon_sym_EQ,
    STATE(5936), 1,
      sym_heredoc_body,
  [194269] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8595), 1,
      sym_identifier,
    STATE(5937), 1,
      sym_heredoc_body,
  [194282] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8597), 1,
      anon_sym_end,
    STATE(5938), 1,
      sym_heredoc_body,
  [194295] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8599), 1,
      anon_sym_end,
    STATE(5939), 1,
      sym_heredoc_body,
  [194308] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8601), 1,
      anon_sym_end,
    STATE(5940), 1,
      sym_heredoc_body,
  [194321] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8603), 1,
      anon_sym_end,
    STATE(5941), 1,
      sym_heredoc_body,
  [194334] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(6873), 1,
      anon_sym_COLON,
    STATE(5942), 1,
      sym_heredoc_body,
  [194347] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8605), 1,
      anon_sym_end,
    STATE(5943), 1,
      sym_heredoc_body,
  [194360] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8607), 1,
      anon_sym_end,
    STATE(5944), 1,
      sym_heredoc_body,
  [194373] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8609), 1,
      anon_sym_COLON,
    STATE(5945), 1,
      sym_heredoc_body,
  [194386] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8611), 1,
      anon_sym_end,
    STATE(5946), 1,
      sym_heredoc_body,
  [194399] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8613), 1,
      anon_sym_end,
    STATE(5947), 1,
      sym_heredoc_body,
  [194412] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8615), 1,
      anon_sym_end,
    STATE(5948), 1,
      sym_heredoc_body,
  [194425] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8617), 1,
      anon_sym_end,
    STATE(5949), 1,
      sym_heredoc_body,
  [194438] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8619), 1,
      sym_uninterpreted,
    STATE(5950), 1,
      sym_heredoc_body,
  [194451] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8621), 1,
      anon_sym_end,
    STATE(5951), 1,
      sym_heredoc_body,
  [194464] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8623), 1,
      sym_constant,
    STATE(5952), 1,
      sym_heredoc_body,
  [194477] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8625), 1,
      anon_sym_RBRACE,
    STATE(5953), 1,
      sym_heredoc_body,
  [194490] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8627), 1,
      anon_sym_end,
    STATE(5954), 1,
      sym_heredoc_body,
  [194503] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8629), 1,
      anon_sym_end,
    STATE(5955), 1,
      sym_heredoc_body,
  [194516] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8631), 1,
      sym_identifier,
    STATE(5956), 1,
      sym_heredoc_body,
  [194529] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8633), 1,
      anon_sym_RBRACE,
    STATE(5957), 1,
      sym_heredoc_body,
  [194542] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8635), 1,
      anon_sym_end,
    STATE(5958), 1,
      sym_heredoc_body,
  [194555] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8637), 1,
      anon_sym_end,
    STATE(5959), 1,
      sym_heredoc_body,
  [194568] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8639), 1,
      sym_identifier,
    STATE(5960), 1,
      sym_heredoc_body,
  [194581] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8641), 1,
      sym_identifier,
    STATE(5961), 1,
      sym_heredoc_body,
  [194594] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8643), 1,
      anon_sym_RBRACE,
    STATE(5962), 1,
      sym_heredoc_body,
  [194607] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8645), 1,
      anon_sym_end,
    STATE(5963), 1,
      sym_heredoc_body,
  [194620] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7119), 1,
      anon_sym_end,
    STATE(5964), 1,
      sym_heredoc_body,
  [194633] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8647), 1,
      anon_sym_end,
    STATE(5965), 1,
      sym_heredoc_body,
  [194646] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8649), 1,
      anon_sym_EQ,
    STATE(5966), 1,
      sym_heredoc_body,
  [194659] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7561), 1,
      anon_sym_end,
    STATE(5967), 1,
      sym_heredoc_body,
  [194672] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8651), 1,
      anon_sym_EQ,
    STATE(5968), 1,
      sym_heredoc_body,
  [194685] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8653), 1,
      anon_sym_end,
    STATE(5969), 1,
      sym_heredoc_body,
  [194698] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7573), 1,
      anon_sym_end,
    STATE(5970), 1,
      sym_heredoc_body,
  [194711] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8655), 1,
      anon_sym_end,
    STATE(5971), 1,
      sym_heredoc_body,
  [194724] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8657), 1,
      anon_sym_RBRACE,
    STATE(5972), 1,
      sym_heredoc_body,
  [194737] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8659), 1,
      anon_sym_end,
    STATE(5973), 1,
      sym_heredoc_body,
  [194750] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8661), 1,
      anon_sym_RBRACE,
    STATE(5974), 1,
      sym_heredoc_body,
  [194763] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8663), 1,
      anon_sym_end,
    STATE(5975), 1,
      sym_heredoc_body,
  [194776] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8665), 1,
      anon_sym_end,
    STATE(5976), 1,
      sym_heredoc_body,
  [194789] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(7516), 1,
      anon_sym_end,
    STATE(5977), 1,
      sym_heredoc_body,
  [194802] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8667), 1,
      anon_sym_end,
    STATE(5978), 1,
      sym_heredoc_body,
  [194815] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8669), 1,
      anon_sym_end,
    STATE(5979), 1,
      sym_heredoc_body,
  [194828] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8671), 1,
      anon_sym_RBRACE,
    STATE(5980), 1,
      sym_heredoc_body,
  [194841] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8673), 1,
      anon_sym_end,
    STATE(5981), 1,
      sym_heredoc_body,
  [194854] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8675), 1,
      anon_sym_end,
    STATE(5982), 1,
      sym_heredoc_body,
  [194867] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8677), 1,
      anon_sym_end,
    STATE(5983), 1,
      sym_heredoc_body,
  [194880] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8679), 1,
      anon_sym_end,
    STATE(5984), 1,
      sym_heredoc_body,
  [194893] = 4,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(79), 1,
      sym_comment,
    ACTIONS(8681), 1,
      anon_sym_end,
    STATE(5985), 1,
      sym_heredoc_body,
  [194906] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(5), 1,
      sym__heredoc_body_start,
    ACTIONS(8683), 1,
      anon_sym_EQ,
    STATE(5986), 1,
      sym_heredoc_body,
  [194919] = 1,
    ACTIONS(8685), 1,
      ts_builtin_sym_end,
  [194923] = 1,
    ACTIONS(8687), 1,
      ts_builtin_sym_end,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(2168)] = 0,
  [SMALL_STATE(2169)] = 87,
  [SMALL_STATE(2170)] = 174,
  [SMALL_STATE(2171)] = 253,
  [SMALL_STATE(2172)] = 334,
  [SMALL_STATE(2173)] = 415,
  [SMALL_STATE(2174)] = 496,
  [SMALL_STATE(2175)] = 574,
  [SMALL_STATE(2176)] = 652,
  [SMALL_STATE(2177)] = 730,
  [SMALL_STATE(2178)] = 808,
  [SMALL_STATE(2179)] = 882,
  [SMALL_STATE(2180)] = 961,
  [SMALL_STATE(2181)] = 1040,
  [SMALL_STATE(2182)] = 1115,
  [SMALL_STATE(2183)] = 1196,
  [SMALL_STATE(2184)] = 1275,
  [SMALL_STATE(2185)] = 1356,
  [SMALL_STATE(2186)] = 1499,
  [SMALL_STATE(2187)] = 1642,
  [SMALL_STATE(2188)] = 1785,
  [SMALL_STATE(2189)] = 1864,
  [SMALL_STATE(2190)] = 2007,
  [SMALL_STATE(2191)] = 2082,
  [SMALL_STATE(2192)] = 2161,
  [SMALL_STATE(2193)] = 2304,
  [SMALL_STATE(2194)] = 2385,
  [SMALL_STATE(2195)] = 2464,
  [SMALL_STATE(2196)] = 2541,
  [SMALL_STATE(2197)] = 2621,
  [SMALL_STATE(2198)] = 2693,
  [SMALL_STATE(2199)] = 2765,
  [SMALL_STATE(2200)] = 2843,
  [SMALL_STATE(2201)] = 2925,
  [SMALL_STATE(2202)] = 3005,
  [SMALL_STATE(2203)] = 3085,
  [SMALL_STATE(2204)] = 3167,
  [SMALL_STATE(2205)] = 3249,
  [SMALL_STATE(2206)] = 3327,
  [SMALL_STATE(2207)] = 3399,
  [SMALL_STATE(2208)] = 3471,
  [SMALL_STATE(2209)] = 3545,
  [SMALL_STATE(2210)] = 3629,
  [SMALL_STATE(2211)] = 3701,
  [SMALL_STATE(2212)] = 3773,
  [SMALL_STATE(2213)] = 3845,
  [SMALL_STATE(2214)] = 3917,
  [SMALL_STATE(2215)] = 3995,
  [SMALL_STATE(2216)] = 4073,
  [SMALL_STATE(2217)] = 4144,
  [SMALL_STATE(2218)] = 4221,
  [SMALL_STATE(2219)] = 4292,
  [SMALL_STATE(2220)] = 4363,
  [SMALL_STATE(2221)] = 4434,
  [SMALL_STATE(2222)] = 4505,
  [SMALL_STATE(2223)] = 4576,
  [SMALL_STATE(2224)] = 4647,
  [SMALL_STATE(2225)] = 4718,
  [SMALL_STATE(2226)] = 4789,
  [SMALL_STATE(2227)] = 4864,
  [SMALL_STATE(2228)] = 4935,
  [SMALL_STATE(2229)] = 5006,
  [SMALL_STATE(2230)] = 5083,
  [SMALL_STATE(2231)] = 5154,
  [SMALL_STATE(2232)] = 5225,
  [SMALL_STATE(2233)] = 5296,
  [SMALL_STATE(2234)] = 5373,
  [SMALL_STATE(2235)] = 5450,
  [SMALL_STATE(2236)] = 5521,
  [SMALL_STATE(2237)] = 5592,
  [SMALL_STATE(2238)] = 5663,
  [SMALL_STATE(2239)] = 5734,
  [SMALL_STATE(2240)] = 5805,
  [SMALL_STATE(2241)] = 5884,
  [SMALL_STATE(2242)] = 5959,
  [SMALL_STATE(2243)] = 6030,
  [SMALL_STATE(2244)] = 6101,
  [SMALL_STATE(2245)] = 6176,
  [SMALL_STATE(2246)] = 6247,
  [SMALL_STATE(2247)] = 6321,
  [SMALL_STATE(2248)] = 6395,
  [SMALL_STATE(2249)] = 6469,
  [SMALL_STATE(2250)] = 6543,
  [SMALL_STATE(2251)] = 6619,
  [SMALL_STATE(2252)] = 6691,
  [SMALL_STATE(2253)] = 6761,
  [SMALL_STATE(2254)] = 6831,
  [SMALL_STATE(2255)] = 6909,
  [SMALL_STATE(2256)] = 6987,
  [SMALL_STATE(2257)] = 7063,
  [SMALL_STATE(2258)] = 7141,
  [SMALL_STATE(2259)] = 7211,
  [SMALL_STATE(2260)] = 7281,
  [SMALL_STATE(2261)] = 7355,
  [SMALL_STATE(2262)] = 7431,
  [SMALL_STATE(2263)] = 7503,
  [SMALL_STATE(2264)] = 7581,
  [SMALL_STATE(2265)] = 7657,
  [SMALL_STATE(2266)] = 7729,
  [SMALL_STATE(2267)] = 7805,
  [SMALL_STATE(2268)] = 7879,
  [SMALL_STATE(2269)] = 7959,
  [SMALL_STATE(2270)] = 8034,
  [SMALL_STATE(2271)] = 8105,
  [SMALL_STATE(2272)] = 8180,
  [SMALL_STATE(2273)] = 8255,
  [SMALL_STATE(2274)] = 8324,
  [SMALL_STATE(2275)] = 8399,
  [SMALL_STATE(2276)] = 8474,
  [SMALL_STATE(2277)] = 8543,
  [SMALL_STATE(2278)] = 8614,
  [SMALL_STATE(2279)] = 8689,
  [SMALL_STATE(2280)] = 8762,
  [SMALL_STATE(2281)] = 8873,
  [SMALL_STATE(2282)] = 8942,
  [SMALL_STATE(2283)] = 9015,
  [SMALL_STATE(2284)] = 9090,
  [SMALL_STATE(2285)] = 9165,
  [SMALL_STATE(2286)] = 9233,
  [SMALL_STATE(2287)] = 9301,
  [SMALL_STATE(2288)] = 9369,
  [SMALL_STATE(2289)] = 9437,
  [SMALL_STATE(2290)] = 9505,
  [SMALL_STATE(2291)] = 9573,
  [SMALL_STATE(2292)] = 9641,
  [SMALL_STATE(2293)] = 9709,
  [SMALL_STATE(2294)] = 9777,
  [SMALL_STATE(2295)] = 9845,
  [SMALL_STATE(2296)] = 9913,
  [SMALL_STATE(2297)] = 9981,
  [SMALL_STATE(2298)] = 10049,
  [SMALL_STATE(2299)] = 10117,
  [SMALL_STATE(2300)] = 10185,
  [SMALL_STATE(2301)] = 10253,
  [SMALL_STATE(2302)] = 10353,
  [SMALL_STATE(2303)] = 10421,
  [SMALL_STATE(2304)] = 10489,
  [SMALL_STATE(2305)] = 10557,
  [SMALL_STATE(2306)] = 10625,
  [SMALL_STATE(2307)] = 10693,
  [SMALL_STATE(2308)] = 10763,
  [SMALL_STATE(2309)] = 10831,
  [SMALL_STATE(2310)] = 10899,
  [SMALL_STATE(2311)] = 10969,
  [SMALL_STATE(2312)] = 11037,
  [SMALL_STATE(2313)] = 11109,
  [SMALL_STATE(2314)] = 11177,
  [SMALL_STATE(2315)] = 11245,
  [SMALL_STATE(2316)] = 11315,
  [SMALL_STATE(2317)] = 11383,
  [SMALL_STATE(2318)] = 11451,
  [SMALL_STATE(2319)] = 11519,
  [SMALL_STATE(2320)] = 11587,
  [SMALL_STATE(2321)] = 11657,
  [SMALL_STATE(2322)] = 11725,
  [SMALL_STATE(2323)] = 11793,
  [SMALL_STATE(2324)] = 11861,
  [SMALL_STATE(2325)] = 11933,
  [SMALL_STATE(2326)] = 12001,
  [SMALL_STATE(2327)] = 12069,
  [SMALL_STATE(2328)] = 12137,
  [SMALL_STATE(2329)] = 12213,
  [SMALL_STATE(2330)] = 12281,
  [SMALL_STATE(2331)] = 12349,
  [SMALL_STATE(2332)] = 12417,
  [SMALL_STATE(2333)] = 12485,
  [SMALL_STATE(2334)] = 12553,
  [SMALL_STATE(2335)] = 12621,
  [SMALL_STATE(2336)] = 12689,
  [SMALL_STATE(2337)] = 12757,
  [SMALL_STATE(2338)] = 12825,
  [SMALL_STATE(2339)] = 12893,
  [SMALL_STATE(2340)] = 12961,
  [SMALL_STATE(2341)] = 13029,
  [SMALL_STATE(2342)] = 13097,
  [SMALL_STATE(2343)] = 13171,
  [SMALL_STATE(2344)] = 13239,
  [SMALL_STATE(2345)] = 13307,
  [SMALL_STATE(2346)] = 13375,
  [SMALL_STATE(2347)] = 13449,
  [SMALL_STATE(2348)] = 13517,
  [SMALL_STATE(2349)] = 13585,
  [SMALL_STATE(2350)] = 13653,
  [SMALL_STATE(2351)] = 13721,
  [SMALL_STATE(2352)] = 13789,
  [SMALL_STATE(2353)] = 13859,
  [SMALL_STATE(2354)] = 13927,
  [SMALL_STATE(2355)] = 13995,
  [SMALL_STATE(2356)] = 14063,
  [SMALL_STATE(2357)] = 14131,
  [SMALL_STATE(2358)] = 14199,
  [SMALL_STATE(2359)] = 14267,
  [SMALL_STATE(2360)] = 14335,
  [SMALL_STATE(2361)] = 14403,
  [SMALL_STATE(2362)] = 14471,
  [SMALL_STATE(2363)] = 14539,
  [SMALL_STATE(2364)] = 14607,
  [SMALL_STATE(2365)] = 14675,
  [SMALL_STATE(2366)] = 14743,
  [SMALL_STATE(2367)] = 14811,
  [SMALL_STATE(2368)] = 14879,
  [SMALL_STATE(2369)] = 14947,
  [SMALL_STATE(2370)] = 15015,
  [SMALL_STATE(2371)] = 15083,
  [SMALL_STATE(2372)] = 15151,
  [SMALL_STATE(2373)] = 15219,
  [SMALL_STATE(2374)] = 15287,
  [SMALL_STATE(2375)] = 15355,
  [SMALL_STATE(2376)] = 15423,
  [SMALL_STATE(2377)] = 15491,
  [SMALL_STATE(2378)] = 15559,
  [SMALL_STATE(2379)] = 15627,
  [SMALL_STATE(2380)] = 15695,
  [SMALL_STATE(2381)] = 15763,
  [SMALL_STATE(2382)] = 15831,
  [SMALL_STATE(2383)] = 15899,
  [SMALL_STATE(2384)] = 15967,
  [SMALL_STATE(2385)] = 16035,
  [SMALL_STATE(2386)] = 16103,
  [SMALL_STATE(2387)] = 16171,
  [SMALL_STATE(2388)] = 16239,
  [SMALL_STATE(2389)] = 16307,
  [SMALL_STATE(2390)] = 16375,
  [SMALL_STATE(2391)] = 16443,
  [SMALL_STATE(2392)] = 16511,
  [SMALL_STATE(2393)] = 16611,
  [SMALL_STATE(2394)] = 16679,
  [SMALL_STATE(2395)] = 16747,
  [SMALL_STATE(2396)] = 16815,
  [SMALL_STATE(2397)] = 16883,
  [SMALL_STATE(2398)] = 16951,
  [SMALL_STATE(2399)] = 17019,
  [SMALL_STATE(2400)] = 17087,
  [SMALL_STATE(2401)] = 17155,
  [SMALL_STATE(2402)] = 17223,
  [SMALL_STATE(2403)] = 17291,
  [SMALL_STATE(2404)] = 17365,
  [SMALL_STATE(2405)] = 17433,
  [SMALL_STATE(2406)] = 17501,
  [SMALL_STATE(2407)] = 17569,
  [SMALL_STATE(2408)] = 17669,
  [SMALL_STATE(2409)] = 17737,
  [SMALL_STATE(2410)] = 17805,
  [SMALL_STATE(2411)] = 17881,
  [SMALL_STATE(2412)] = 17955,
  [SMALL_STATE(2413)] = 18023,
  [SMALL_STATE(2414)] = 18123,
  [SMALL_STATE(2415)] = 18191,
  [SMALL_STATE(2416)] = 18259,
  [SMALL_STATE(2417)] = 18327,
  [SMALL_STATE(2418)] = 18395,
  [SMALL_STATE(2419)] = 18463,
  [SMALL_STATE(2420)] = 18563,
  [SMALL_STATE(2421)] = 18631,
  [SMALL_STATE(2422)] = 18731,
  [SMALL_STATE(2423)] = 18799,
  [SMALL_STATE(2424)] = 18867,
  [SMALL_STATE(2425)] = 18967,
  [SMALL_STATE(2426)] = 19035,
  [SMALL_STATE(2427)] = 19103,
  [SMALL_STATE(2428)] = 19171,
  [SMALL_STATE(2429)] = 19271,
  [SMALL_STATE(2430)] = 19347,
  [SMALL_STATE(2431)] = 19415,
  [SMALL_STATE(2432)] = 19483,
  [SMALL_STATE(2433)] = 19583,
  [SMALL_STATE(2434)] = 19657,
  [SMALL_STATE(2435)] = 19757,
  [SMALL_STATE(2436)] = 19831,
  [SMALL_STATE(2437)] = 19899,
  [SMALL_STATE(2438)] = 19967,
  [SMALL_STATE(2439)] = 20035,
  [SMALL_STATE(2440)] = 20135,
  [SMALL_STATE(2441)] = 20203,
  [SMALL_STATE(2442)] = 20303,
  [SMALL_STATE(2443)] = 20377,
  [SMALL_STATE(2444)] = 20445,
  [SMALL_STATE(2445)] = 20513,
  [SMALL_STATE(2446)] = 20581,
  [SMALL_STATE(2447)] = 20649,
  [SMALL_STATE(2448)] = 20719,
  [SMALL_STATE(2449)] = 20787,
  [SMALL_STATE(2450)] = 20855,
  [SMALL_STATE(2451)] = 20923,
  [SMALL_STATE(2452)] = 20991,
  [SMALL_STATE(2453)] = 21059,
  [SMALL_STATE(2454)] = 21127,
  [SMALL_STATE(2455)] = 21227,
  [SMALL_STATE(2456)] = 21295,
  [SMALL_STATE(2457)] = 21395,
  [SMALL_STATE(2458)] = 21495,
  [SMALL_STATE(2459)] = 21595,
  [SMALL_STATE(2460)] = 21695,
  [SMALL_STATE(2461)] = 21795,
  [SMALL_STATE(2462)] = 21895,
  [SMALL_STATE(2463)] = 21995,
  [SMALL_STATE(2464)] = 22095,
  [SMALL_STATE(2465)] = 22195,
  [SMALL_STATE(2466)] = 22265,
  [SMALL_STATE(2467)] = 22365,
  [SMALL_STATE(2468)] = 22433,
  [SMALL_STATE(2469)] = 22501,
  [SMALL_STATE(2470)] = 22569,
  [SMALL_STATE(2471)] = 22669,
  [SMALL_STATE(2472)] = 22737,
  [SMALL_STATE(2473)] = 22805,
  [SMALL_STATE(2474)] = 22873,
  [SMALL_STATE(2475)] = 22946,
  [SMALL_STATE(2476)] = 23019,
  [SMALL_STATE(2477)] = 23094,
  [SMALL_STATE(2478)] = 23197,
  [SMALL_STATE(2479)] = 23266,
  [SMALL_STATE(2480)] = 23341,
  [SMALL_STATE(2481)] = 23410,
  [SMALL_STATE(2482)] = 23477,
  [SMALL_STATE(2483)] = 23548,
  [SMALL_STATE(2484)] = 23617,
  [SMALL_STATE(2485)] = 23688,
  [SMALL_STATE(2486)] = 23757,
  [SMALL_STATE(2487)] = 23832,
  [SMALL_STATE(2488)] = 23905,
  [SMALL_STATE(2489)] = 23978,
  [SMALL_STATE(2490)] = 24046,
  [SMALL_STATE(2491)] = 24114,
  [SMALL_STATE(2492)] = 24212,
  [SMALL_STATE(2493)] = 24310,
  [SMALL_STATE(2494)] = 24406,
  [SMALL_STATE(2495)] = 24504,
  [SMALL_STATE(2496)] = 24602,
  [SMALL_STATE(2497)] = 24702,
  [SMALL_STATE(2498)] = 24798,
  [SMALL_STATE(2499)] = 24896,
  [SMALL_STATE(2500)] = 24968,
  [SMALL_STATE(2501)] = 25068,
  [SMALL_STATE(2502)] = 25142,
  [SMALL_STATE(2503)] = 25216,
  [SMALL_STATE(2504)] = 25290,
  [SMALL_STATE(2505)] = 25364,
  [SMALL_STATE(2506)] = 25432,
  [SMALL_STATE(2507)] = 25506,
  [SMALL_STATE(2508)] = 25574,
  [SMALL_STATE(2509)] = 25642,
  [SMALL_STATE(2510)] = 25712,
  [SMALL_STATE(2511)] = 25780,
  [SMALL_STATE(2512)] = 25848,
  [SMALL_STATE(2513)] = 25924,
  [SMALL_STATE(2514)] = 25998,
  [SMALL_STATE(2515)] = 26078,
  [SMALL_STATE(2516)] = 26168,
  [SMALL_STATE(2517)] = 26256,
  [SMALL_STATE(2518)] = 26328,
  [SMALL_STATE(2519)] = 26416,
  [SMALL_STATE(2520)] = 26484,
  [SMALL_STATE(2521)] = 26552,
  [SMALL_STATE(2522)] = 26650,
  [SMALL_STATE(2523)] = 26748,
  [SMALL_STATE(2524)] = 26820,
  [SMALL_STATE(2525)] = 26893,
  [SMALL_STATE(2526)] = 26966,
  [SMALL_STATE(2527)] = 27033,
  [SMALL_STATE(2528)] = 27136,
  [SMALL_STATE(2529)] = 27205,
  [SMALL_STATE(2530)] = 27312,
  [SMALL_STATE(2531)] = 27385,
  [SMALL_STATE(2532)] = 27452,
  [SMALL_STATE(2533)] = 27525,
  [SMALL_STATE(2534)] = 27626,
  [SMALL_STATE(2535)] = 27699,
  [SMALL_STATE(2536)] = 27768,
  [SMALL_STATE(2537)] = 27839,
  [SMALL_STATE(2538)] = 27910,
  [SMALL_STATE(2539)] = 27998,
  [SMALL_STATE(2540)] = 28086,
  [SMALL_STATE(2541)] = 28150,
  [SMALL_STATE(2542)] = 28244,
  [SMALL_STATE(2543)] = 28332,
  [SMALL_STATE(2544)] = 28420,
  [SMALL_STATE(2545)] = 28492,
  [SMALL_STATE(2546)] = 28580,
  [SMALL_STATE(2547)] = 28676,
  [SMALL_STATE(2548)] = 28740,
  [SMALL_STATE(2549)] = 28828,
  [SMALL_STATE(2550)] = 28924,
  [SMALL_STATE(2551)] = 29012,
  [SMALL_STATE(2552)] = 29100,
  [SMALL_STATE(2553)] = 29172,
  [SMALL_STATE(2554)] = 29260,
  [SMALL_STATE(2555)] = 29348,
  [SMALL_STATE(2556)] = 29446,
  [SMALL_STATE(2557)] = 29510,
  [SMALL_STATE(2558)] = 29598,
  [SMALL_STATE(2559)] = 29694,
  [SMALL_STATE(2560)] = 29782,
  [SMALL_STATE(2561)] = 29870,
  [SMALL_STATE(2562)] = 29966,
  [SMALL_STATE(2563)] = 30054,
  [SMALL_STATE(2564)] = 30142,
  [SMALL_STATE(2565)] = 30214,
  [SMALL_STATE(2566)] = 30308,
  [SMALL_STATE(2567)] = 30378,
  [SMALL_STATE(2568)] = 30474,
  [SMALL_STATE(2569)] = 30540,
  [SMALL_STATE(2570)] = 30604,
  [SMALL_STATE(2571)] = 30692,
  [SMALL_STATE(2572)] = 30780,
  [SMALL_STATE(2573)] = 30852,
  [SMALL_STATE(2574)] = 30918,
  [SMALL_STATE(2575)] = 30986,
  [SMALL_STATE(2576)] = 31052,
  [SMALL_STATE(2577)] = 31118,
  [SMALL_STATE(2578)] = 31192,
  [SMALL_STATE(2579)] = 31264,
  [SMALL_STATE(2580)] = 31342,
  [SMALL_STATE(2581)] = 31430,
  [SMALL_STATE(2582)] = 31516,
  [SMALL_STATE(2583)] = 31604,
  [SMALL_STATE(2584)] = 31690,
  [SMALL_STATE(2585)] = 31756,
  [SMALL_STATE(2586)] = 31844,
  [SMALL_STATE(2587)] = 31932,
  [SMALL_STATE(2588)] = 32020,
  [SMALL_STATE(2589)] = 32116,
  [SMALL_STATE(2590)] = 32204,
  [SMALL_STATE(2591)] = 32292,
  [SMALL_STATE(2592)] = 32380,
  [SMALL_STATE(2593)] = 32478,
  [SMALL_STATE(2594)] = 32566,
  [SMALL_STATE(2595)] = 32630,
  [SMALL_STATE(2596)] = 32718,
  [SMALL_STATE(2597)] = 32806,
  [SMALL_STATE(2598)] = 32894,
  [SMALL_STATE(2599)] = 32958,
  [SMALL_STATE(2600)] = 33046,
  [SMALL_STATE(2601)] = 33142,
  [SMALL_STATE(2602)] = 33230,
  [SMALL_STATE(2603)] = 33318,
  [SMALL_STATE(2604)] = 33406,
  [SMALL_STATE(2605)] = 33494,
  [SMALL_STATE(2606)] = 33582,
  [SMALL_STATE(2607)] = 33652,
  [SMALL_STATE(2608)] = 33718,
  [SMALL_STATE(2609)] = 33806,
  [SMALL_STATE(2610)] = 33894,
  [SMALL_STATE(2611)] = 33982,
  [SMALL_STATE(2612)] = 34070,
  [SMALL_STATE(2613)] = 34158,
  [SMALL_STATE(2614)] = 34222,
  [SMALL_STATE(2615)] = 34286,
  [SMALL_STATE(2616)] = 34374,
  [SMALL_STATE(2617)] = 34462,
  [SMALL_STATE(2618)] = 34550,
  [SMALL_STATE(2619)] = 34638,
  [SMALL_STATE(2620)] = 34726,
  [SMALL_STATE(2621)] = 34814,
  [SMALL_STATE(2622)] = 34902,
  [SMALL_STATE(2623)] = 34990,
  [SMALL_STATE(2624)] = 35078,
  [SMALL_STATE(2625)] = 35148,
  [SMALL_STATE(2626)] = 35213,
  [SMALL_STATE(2627)] = 35306,
  [SMALL_STATE(2628)] = 35401,
  [SMALL_STATE(2629)] = 35486,
  [SMALL_STATE(2630)] = 35579,
  [SMALL_STATE(2631)] = 35652,
  [SMALL_STATE(2632)] = 35737,
  [SMALL_STATE(2633)] = 35822,
  [SMALL_STATE(2634)] = 35907,
  [SMALL_STATE(2635)] = 35992,
  [SMALL_STATE(2636)] = 36077,
  [SMALL_STATE(2637)] = 36142,
  [SMALL_STATE(2638)] = 36247,
  [SMALL_STATE(2639)] = 36344,
  [SMALL_STATE(2640)] = 36415,
  [SMALL_STATE(2641)] = 36500,
  [SMALL_STATE(2642)] = 36565,
  [SMALL_STATE(2643)] = 36670,
  [SMALL_STATE(2644)] = 36737,
  [SMALL_STATE(2645)] = 36832,
  [SMALL_STATE(2646)] = 36927,
  [SMALL_STATE(2647)] = 36992,
  [SMALL_STATE(2648)] = 37063,
  [SMALL_STATE(2649)] = 37132,
  [SMALL_STATE(2650)] = 37197,
  [SMALL_STATE(2651)] = 37270,
  [SMALL_STATE(2652)] = 37341,
  [SMALL_STATE(2653)] = 37418,
  [SMALL_STATE(2654)] = 37505,
  [SMALL_STATE(2655)] = 37574,
  [SMALL_STATE(2656)] = 37659,
  [SMALL_STATE(2657)] = 37728,
  [SMALL_STATE(2658)] = 37813,
  [SMALL_STATE(2659)] = 37914,
  [SMALL_STATE(2660)] = 37979,
  [SMALL_STATE(2661)] = 38044,
  [SMALL_STATE(2662)] = 38129,
  [SMALL_STATE(2663)] = 38196,
  [SMALL_STATE(2664)] = 38293,
  [SMALL_STATE(2665)] = 38388,
  [SMALL_STATE(2666)] = 38483,
  [SMALL_STATE(2667)] = 38580,
  [SMALL_STATE(2668)] = 38645,
  [SMALL_STATE(2669)] = 38737,
  [SMALL_STATE(2670)] = 38801,
  [SMALL_STATE(2671)] = 38887,
  [SMALL_STATE(2672)] = 38981,
  [SMALL_STATE(2673)] = 39065,
  [SMALL_STATE(2674)] = 39133,
  [SMALL_STATE(2675)] = 39225,
  [SMALL_STATE(2676)] = 39309,
  [SMALL_STATE(2677)] = 39373,
  [SMALL_STATE(2678)] = 39437,
  [SMALL_STATE(2679)] = 39501,
  [SMALL_STATE(2680)] = 39573,
  [SMALL_STATE(2681)] = 39643,
  [SMALL_STATE(2682)] = 39733,
  [SMALL_STATE(2683)] = 39829,
  [SMALL_STATE(2684)] = 39925,
  [SMALL_STATE(2685)] = 40001,
  [SMALL_STATE(2686)] = 40099,
  [SMALL_STATE(2687)] = 40191,
  [SMALL_STATE(2688)] = 40283,
  [SMALL_STATE(2689)] = 40373,
  [SMALL_STATE(2690)] = 40469,
  [SMALL_STATE(2691)] = 40535,
  [SMALL_STATE(2692)] = 40627,
  [SMALL_STATE(2693)] = 40697,
  [SMALL_STATE(2694)] = 40763,
  [SMALL_STATE(2695)] = 40855,
  [SMALL_STATE(2696)] = 40923,
  [SMALL_STATE(2697)] = 41019,
  [SMALL_STATE(2698)] = 41111,
  [SMALL_STATE(2699)] = 41208,
  [SMALL_STATE(2700)] = 41271,
  [SMALL_STATE(2701)] = 41336,
  [SMALL_STATE(2702)] = 41399,
  [SMALL_STATE(2703)] = 41490,
  [SMALL_STATE(2704)] = 41563,
  [SMALL_STATE(2705)] = 41654,
  [SMALL_STATE(2706)] = 41753,
  [SMALL_STATE(2707)] = 41844,
  [SMALL_STATE(2708)] = 41913,
  [SMALL_STATE(2709)] = 41978,
  [SMALL_STATE(2710)] = 42075,
  [SMALL_STATE(2711)] = 42166,
  [SMALL_STATE(2712)] = 42261,
  [SMALL_STATE(2713)] = 42352,
  [SMALL_STATE(2714)] = 42415,
  [SMALL_STATE(2715)] = 42480,
  [SMALL_STATE(2716)] = 42581,
  [SMALL_STATE(2717)] = 42644,
  [SMALL_STATE(2718)] = 42736,
  [SMALL_STATE(2719)] = 42830,
  [SMALL_STATE(2720)] = 42892,
  [SMALL_STATE(2721)] = 42962,
  [SMALL_STATE(2722)] = 43030,
  [SMALL_STATE(2723)] = 43092,
  [SMALL_STATE(2724)] = 43186,
  [SMALL_STATE(2725)] = 43248,
  [SMALL_STATE(2726)] = 43336,
  [SMALL_STATE(2727)] = 43400,
  [SMALL_STATE(2728)] = 43462,
  [SMALL_STATE(2729)] = 43524,
  [SMALL_STATE(2730)] = 43594,
  [SMALL_STATE(2731)] = 43662,
  [SMALL_STATE(2732)] = 43736,
  [SMALL_STATE(2733)] = 43820,
  [SMALL_STATE(2734)] = 43902,
  [SMALL_STATE(2735)] = 43968,
  [SMALL_STATE(2736)] = 44050,
  [SMALL_STATE(2737)] = 44112,
  [SMALL_STATE(2738)] = 44206,
  [SMALL_STATE(2739)] = 44272,
  [SMALL_STATE(2740)] = 44366,
  [SMALL_STATE(2741)] = 44428,
  [SMALL_STATE(2742)] = 44518,
  [SMALL_STATE(2743)] = 44580,
  [SMALL_STATE(2744)] = 44670,
  [SMALL_STATE(2745)] = 44760,
  [SMALL_STATE(2746)] = 44828,
  [SMALL_STATE(2747)] = 44892,
  [SMALL_STATE(2748)] = 44954,
  [SMALL_STATE(2749)] = 45016,
  [SMALL_STATE(2750)] = 45086,
  [SMALL_STATE(2751)] = 45154,
  [SMALL_STATE(2752)] = 45228,
  [SMALL_STATE(2753)] = 45312,
  [SMALL_STATE(2754)] = 45394,
  [SMALL_STATE(2755)] = 45460,
  [SMALL_STATE(2756)] = 45542,
  [SMALL_STATE(2757)] = 45604,
  [SMALL_STATE(2758)] = 45694,
  [SMALL_STATE(2759)] = 45786,
  [SMALL_STATE(2760)] = 45876,
  [SMALL_STATE(2761)] = 45966,
  [SMALL_STATE(2762)] = 46028,
  [SMALL_STATE(2763)] = 46118,
  [SMALL_STATE(2764)] = 46210,
  [SMALL_STATE(2765)] = 46274,
  [SMALL_STATE(2766)] = 46366,
  [SMALL_STATE(2767)] = 46458,
  [SMALL_STATE(2768)] = 46548,
  [SMALL_STATE(2769)] = 46640,
  [SMALL_STATE(2770)] = 46732,
  [SMALL_STATE(2771)] = 46826,
  [SMALL_STATE(2772)] = 46922,
  [SMALL_STATE(2773)] = 47012,
  [SMALL_STATE(2774)] = 47106,
  [SMALL_STATE(2775)] = 47198,
  [SMALL_STATE(2776)] = 47286,
  [SMALL_STATE(2777)] = 47367,
  [SMALL_STATE(2778)] = 47430,
  [SMALL_STATE(2779)] = 47503,
  [SMALL_STATE(2780)] = 47598,
  [SMALL_STATE(2781)] = 47661,
  [SMALL_STATE(2782)] = 47756,
  [SMALL_STATE(2783)] = 47817,
  [SMALL_STATE(2784)] = 47906,
  [SMALL_STATE(2785)] = 47969,
  [SMALL_STATE(2786)] = 48052,
  [SMALL_STATE(2787)] = 48117,
  [SMALL_STATE(2788)] = 48204,
  [SMALL_STATE(2789)] = 48285,
  [SMALL_STATE(2790)] = 48374,
  [SMALL_STATE(2791)] = 48463,
  [SMALL_STATE(2792)] = 48524,
  [SMALL_STATE(2793)] = 48591,
  [SMALL_STATE(2794)] = 48680,
  [SMALL_STATE(2795)] = 48745,
  [SMALL_STATE(2796)] = 48806,
  [SMALL_STATE(2797)] = 48895,
  [SMALL_STATE(2798)] = 48960,
  [SMALL_STATE(2799)] = 49023,
  [SMALL_STATE(2800)] = 49090,
  [SMALL_STATE(2801)] = 49157,
  [SMALL_STATE(2802)] = 49244,
  [SMALL_STATE(2803)] = 49333,
  [SMALL_STATE(2804)] = 49430,
  [SMALL_STATE(2805)] = 49493,
  [SMALL_STATE(2806)] = 49556,
  [SMALL_STATE(2807)] = 49645,
  [SMALL_STATE(2808)] = 49710,
  [SMALL_STATE(2809)] = 49773,
  [SMALL_STATE(2810)] = 49836,
  [SMALL_STATE(2811)] = 49927,
  [SMALL_STATE(2812)] = 49992,
  [SMALL_STATE(2813)] = 50085,
  [SMALL_STATE(2814)] = 50148,
  [SMALL_STATE(2815)] = 50211,
  [SMALL_STATE(2816)] = 50280,
  [SMALL_STATE(2817)] = 50369,
  [SMALL_STATE(2818)] = 50430,
  [SMALL_STATE(2819)] = 50497,
  [SMALL_STATE(2820)] = 50560,
  [SMALL_STATE(2821)] = 50649,
  [SMALL_STATE(2822)] = 50738,
  [SMALL_STATE(2823)] = 50831,
  [SMALL_STATE(2824)] = 50898,
  [SMALL_STATE(2825)] = 50991,
  [SMALL_STATE(2826)] = 51052,
  [SMALL_STATE(2827)] = 51113,
  [SMALL_STATE(2828)] = 51203,
  [SMALL_STATE(2829)] = 51271,
  [SMALL_STATE(2830)] = 51363,
  [SMALL_STATE(2831)] = 51451,
  [SMALL_STATE(2832)] = 51541,
  [SMALL_STATE(2833)] = 51631,
  [SMALL_STATE(2834)] = 51689,
  [SMALL_STATE(2835)] = 51781,
  [SMALL_STATE(2836)] = 51839,
  [SMALL_STATE(2837)] = 51935,
  [SMALL_STATE(2838)] = 52001,
  [SMALL_STATE(2839)] = 52061,
  [SMALL_STATE(2840)] = 52121,
  [SMALL_STATE(2841)] = 52183,
  [SMALL_STATE(2842)] = 52245,
  [SMALL_STATE(2843)] = 52305,
  [SMALL_STATE(2844)] = 52365,
  [SMALL_STATE(2845)] = 52433,
  [SMALL_STATE(2846)] = 52499,
  [SMALL_STATE(2847)] = 52571,
  [SMALL_STATE(2848)] = 52629,
  [SMALL_STATE(2849)] = 52709,
  [SMALL_STATE(2850)] = 52773,
  [SMALL_STATE(2851)] = 52853,
  [SMALL_STATE(2852)] = 52915,
  [SMALL_STATE(2853)] = 52975,
  [SMALL_STATE(2854)] = 53035,
  [SMALL_STATE(2855)] = 53093,
  [SMALL_STATE(2856)] = 53159,
  [SMALL_STATE(2857)] = 53247,
  [SMALL_STATE(2858)] = 53305,
  [SMALL_STATE(2859)] = 53363,
  [SMALL_STATE(2860)] = 53423,
  [SMALL_STATE(2861)] = 53489,
  [SMALL_STATE(2862)] = 53549,
  [SMALL_STATE(2863)] = 53643,
  [SMALL_STATE(2864)] = 53703,
  [SMALL_STATE(2865)] = 53763,
  [SMALL_STATE(2866)] = 53853,
  [SMALL_STATE(2867)] = 53911,
  [SMALL_STATE(2868)] = 53999,
  [SMALL_STATE(2869)] = 54089,
  [SMALL_STATE(2870)] = 54179,
  [SMALL_STATE(2871)] = 54269,
  [SMALL_STATE(2872)] = 54351,
  [SMALL_STATE(2873)] = 54416,
  [SMALL_STATE(2874)] = 54473,
  [SMALL_STATE(2875)] = 54530,
  [SMALL_STATE(2876)] = 54597,
  [SMALL_STATE(2877)] = 54654,
  [SMALL_STATE(2878)] = 54711,
  [SMALL_STATE(2879)] = 54768,
  [SMALL_STATE(2880)] = 54825,
  [SMALL_STATE(2881)] = 54882,
  [SMALL_STATE(2882)] = 54939,
  [SMALL_STATE(2883)] = 54996,
  [SMALL_STATE(2884)] = 55053,
  [SMALL_STATE(2885)] = 55110,
  [SMALL_STATE(2886)] = 55167,
  [SMALL_STATE(2887)] = 55224,
  [SMALL_STATE(2888)] = 55281,
  [SMALL_STATE(2889)] = 55342,
  [SMALL_STATE(2890)] = 55399,
  [SMALL_STATE(2891)] = 55456,
  [SMALL_STATE(2892)] = 55513,
  [SMALL_STATE(2893)] = 55572,
  [SMALL_STATE(2894)] = 55631,
  [SMALL_STATE(2895)] = 55688,
  [SMALL_STATE(2896)] = 55745,
  [SMALL_STATE(2897)] = 55802,
  [SMALL_STATE(2898)] = 55861,
  [SMALL_STATE(2899)] = 55928,
  [SMALL_STATE(2900)] = 55993,
  [SMALL_STATE(2901)] = 56050,
  [SMALL_STATE(2902)] = 56121,
  [SMALL_STATE(2903)] = 56202,
  [SMALL_STATE(2904)] = 56259,
  [SMALL_STATE(2905)] = 56326,
  [SMALL_STATE(2906)] = 56405,
  [SMALL_STATE(2907)] = 56468,
  [SMALL_STATE(2908)] = 56547,
  [SMALL_STATE(2909)] = 56604,
  [SMALL_STATE(2910)] = 56661,
  [SMALL_STATE(2911)] = 56718,
  [SMALL_STATE(2912)] = 56775,
  [SMALL_STATE(2913)] = 56870,
  [SMALL_STATE(2914)] = 56929,
  [SMALL_STATE(2915)] = 57020,
  [SMALL_STATE(2916)] = 57077,
  [SMALL_STATE(2917)] = 57134,
  [SMALL_STATE(2918)] = 57191,
  [SMALL_STATE(2919)] = 57248,
  [SMALL_STATE(2920)] = 57305,
  [SMALL_STATE(2921)] = 57362,
  [SMALL_STATE(2922)] = 57419,
  [SMALL_STATE(2923)] = 57476,
  [SMALL_STATE(2924)] = 57533,
  [SMALL_STATE(2925)] = 57590,
  [SMALL_STATE(2926)] = 57647,
  [SMALL_STATE(2927)] = 57704,
  [SMALL_STATE(2928)] = 57761,
  [SMALL_STATE(2929)] = 57852,
  [SMALL_STATE(2930)] = 57909,
  [SMALL_STATE(2931)] = 57966,
  [SMALL_STATE(2932)] = 58023,
  [SMALL_STATE(2933)] = 58080,
  [SMALL_STATE(2934)] = 58137,
  [SMALL_STATE(2935)] = 58226,
  [SMALL_STATE(2936)] = 58315,
  [SMALL_STATE(2937)] = 58372,
  [SMALL_STATE(2938)] = 58429,
  [SMALL_STATE(2939)] = 58486,
  [SMALL_STATE(2940)] = 58543,
  [SMALL_STATE(2941)] = 58600,
  [SMALL_STATE(2942)] = 58657,
  [SMALL_STATE(2943)] = 58716,
  [SMALL_STATE(2944)] = 58807,
  [SMALL_STATE(2945)] = 58864,
  [SMALL_STATE(2946)] = 58921,
  [SMALL_STATE(2947)] = 58986,
  [SMALL_STATE(2948)] = 59043,
  [SMALL_STATE(2949)] = 59100,
  [SMALL_STATE(2950)] = 59157,
  [SMALL_STATE(2951)] = 59214,
  [SMALL_STATE(2952)] = 59271,
  [SMALL_STATE(2953)] = 59328,
  [SMALL_STATE(2954)] = 59385,
  [SMALL_STATE(2955)] = 59442,
  [SMALL_STATE(2956)] = 59499,
  [SMALL_STATE(2957)] = 59556,
  [SMALL_STATE(2958)] = 59613,
  [SMALL_STATE(2959)] = 59672,
  [SMALL_STATE(2960)] = 59729,
  [SMALL_STATE(2961)] = 59786,
  [SMALL_STATE(2962)] = 59843,
  [SMALL_STATE(2963)] = 59900,
  [SMALL_STATE(2964)] = 59957,
  [SMALL_STATE(2965)] = 60014,
  [SMALL_STATE(2966)] = 60071,
  [SMALL_STATE(2967)] = 60160,
  [SMALL_STATE(2968)] = 60217,
  [SMALL_STATE(2969)] = 60282,
  [SMALL_STATE(2970)] = 60343,
  [SMALL_STATE(2971)] = 60400,
  [SMALL_STATE(2972)] = 60457,
  [SMALL_STATE(2973)] = 60514,
  [SMALL_STATE(2974)] = 60571,
  [SMALL_STATE(2975)] = 60628,
  [SMALL_STATE(2976)] = 60685,
  [SMALL_STATE(2977)] = 60742,
  [SMALL_STATE(2978)] = 60799,
  [SMALL_STATE(2979)] = 60856,
  [SMALL_STATE(2980)] = 60913,
  [SMALL_STATE(2981)] = 60970,
  [SMALL_STATE(2982)] = 61027,
  [SMALL_STATE(2983)] = 61084,
  [SMALL_STATE(2984)] = 61141,
  [SMALL_STATE(2985)] = 61198,
  [SMALL_STATE(2986)] = 61255,
  [SMALL_STATE(2987)] = 61312,
  [SMALL_STATE(2988)] = 61369,
  [SMALL_STATE(2989)] = 61426,
  [SMALL_STATE(2990)] = 61483,
  [SMALL_STATE(2991)] = 61540,
  [SMALL_STATE(2992)] = 61597,
  [SMALL_STATE(2993)] = 61654,
  [SMALL_STATE(2994)] = 61711,
  [SMALL_STATE(2995)] = 61768,
  [SMALL_STATE(2996)] = 61825,
  [SMALL_STATE(2997)] = 61882,
  [SMALL_STATE(2998)] = 61939,
  [SMALL_STATE(2999)] = 61996,
  [SMALL_STATE(3000)] = 62053,
  [SMALL_STATE(3001)] = 62118,
  [SMALL_STATE(3002)] = 62175,
  [SMALL_STATE(3003)] = 62232,
  [SMALL_STATE(3004)] = 62289,
  [SMALL_STATE(3005)] = 62350,
  [SMALL_STATE(3006)] = 62407,
  [SMALL_STATE(3007)] = 62464,
  [SMALL_STATE(3008)] = 62521,
  [SMALL_STATE(3009)] = 62578,
  [SMALL_STATE(3010)] = 62635,
  [SMALL_STATE(3011)] = 62692,
  [SMALL_STATE(3012)] = 62749,
  [SMALL_STATE(3013)] = 62806,
  [SMALL_STATE(3014)] = 62863,
  [SMALL_STATE(3015)] = 62920,
  [SMALL_STATE(3016)] = 62979,
  [SMALL_STATE(3017)] = 63038,
  [SMALL_STATE(3018)] = 63095,
  [SMALL_STATE(3019)] = 63152,
  [SMALL_STATE(3020)] = 63239,
  [SMALL_STATE(3021)] = 63296,
  [SMALL_STATE(3022)] = 63353,
  [SMALL_STATE(3023)] = 63410,
  [SMALL_STATE(3024)] = 63467,
  [SMALL_STATE(3025)] = 63524,
  [SMALL_STATE(3026)] = 63581,
  [SMALL_STATE(3027)] = 63638,
  [SMALL_STATE(3028)] = 63695,
  [SMALL_STATE(3029)] = 63784,
  [SMALL_STATE(3030)] = 63873,
  [SMALL_STATE(3031)] = 63960,
  [SMALL_STATE(3032)] = 64049,
  [SMALL_STATE(3033)] = 64138,
  [SMALL_STATE(3034)] = 64225,
  [SMALL_STATE(3035)] = 64314,
  [SMALL_STATE(3036)] = 64403,
  [SMALL_STATE(3037)] = 64494,
  [SMALL_STATE(3038)] = 64581,
  [SMALL_STATE(3039)] = 64670,
  [SMALL_STATE(3040)] = 64727,
  [SMALL_STATE(3041)] = 64818,
  [SMALL_STATE(3042)] = 64907,
  [SMALL_STATE(3043)] = 64964,
  [SMALL_STATE(3044)] = 65021,
  [SMALL_STATE(3045)] = 65110,
  [SMALL_STATE(3046)] = 65167,
  [SMALL_STATE(3047)] = 65224,
  [SMALL_STATE(3048)] = 65281,
  [SMALL_STATE(3049)] = 65338,
  [SMALL_STATE(3050)] = 65395,
  [SMALL_STATE(3051)] = 65452,
  [SMALL_STATE(3052)] = 65511,
  [SMALL_STATE(3053)] = 65570,
  [SMALL_STATE(3054)] = 65631,
  [SMALL_STATE(3055)] = 65690,
  [SMALL_STATE(3056)] = 65749,
  [SMALL_STATE(3057)] = 65816,
  [SMALL_STATE(3058)] = 65881,
  [SMALL_STATE(3059)] = 65952,
  [SMALL_STATE(3060)] = 66033,
  [SMALL_STATE(3061)] = 66112,
  [SMALL_STATE(3062)] = 66175,
  [SMALL_STATE(3063)] = 66254,
  [SMALL_STATE(3064)] = 66313,
  [SMALL_STATE(3065)] = 66370,
  [SMALL_STATE(3066)] = 66427,
  [SMALL_STATE(3067)] = 66486,
  [SMALL_STATE(3068)] = 66543,
  [SMALL_STATE(3069)] = 66600,
  [SMALL_STATE(3070)] = 66657,
  [SMALL_STATE(3071)] = 66714,
  [SMALL_STATE(3072)] = 66771,
  [SMALL_STATE(3073)] = 66828,
  [SMALL_STATE(3074)] = 66885,
  [SMALL_STATE(3075)] = 66942,
  [SMALL_STATE(3076)] = 66999,
  [SMALL_STATE(3077)] = 67056,
  [SMALL_STATE(3078)] = 67116,
  [SMALL_STATE(3079)] = 67178,
  [SMALL_STATE(3080)] = 67234,
  [SMALL_STATE(3081)] = 67306,
  [SMALL_STATE(3082)] = 67380,
  [SMALL_STATE(3083)] = 67454,
  [SMALL_STATE(3084)] = 67540,
  [SMALL_STATE(3085)] = 67596,
  [SMALL_STATE(3086)] = 67662,
  [SMALL_STATE(3087)] = 67718,
  [SMALL_STATE(3088)] = 67792,
  [SMALL_STATE(3089)] = 67866,
  [SMALL_STATE(3090)] = 67958,
  [SMALL_STATE(3091)] = 68046,
  [SMALL_STATE(3092)] = 68104,
  [SMALL_STATE(3093)] = 68162,
  [SMALL_STATE(3094)] = 68226,
  [SMALL_STATE(3095)] = 68312,
  [SMALL_STATE(3096)] = 68398,
  [SMALL_STATE(3097)] = 68460,
  [SMALL_STATE(3098)] = 68532,
  [SMALL_STATE(3099)] = 68606,
  [SMALL_STATE(3100)] = 68694,
  [SMALL_STATE(3101)] = 68752,
  [SMALL_STATE(3102)] = 68810,
  [SMALL_STATE(3103)] = 68868,
  [SMALL_STATE(3104)] = 68958,
  [SMALL_STATE(3105)] = 69030,
  [SMALL_STATE(3106)] = 69118,
  [SMALL_STATE(3107)] = 69176,
  [SMALL_STATE(3108)] = 69234,
  [SMALL_STATE(3109)] = 69296,
  [SMALL_STATE(3110)] = 69384,
  [SMALL_STATE(3111)] = 69456,
  [SMALL_STATE(3112)] = 69516,
  [SMALL_STATE(3113)] = 69574,
  [SMALL_STATE(3114)] = 69648,
  [SMALL_STATE(3115)] = 69734,
  [SMALL_STATE(3116)] = 69820,
  [SMALL_STATE(3117)] = 69878,
  [SMALL_STATE(3118)] = 69944,
  [SMALL_STATE(3119)] = 70008,
  [SMALL_STATE(3120)] = 70078,
  [SMALL_STATE(3121)] = 70158,
  [SMALL_STATE(3122)] = 70236,
  [SMALL_STATE(3123)] = 70298,
  [SMALL_STATE(3124)] = 70372,
  [SMALL_STATE(3125)] = 70430,
  [SMALL_STATE(3126)] = 70488,
  [SMALL_STATE(3127)] = 70550,
  [SMALL_STATE(3128)] = 70608,
  [SMALL_STATE(3129)] = 70666,
  [SMALL_STATE(3130)] = 70740,
  [SMALL_STATE(3131)] = 70830,
  [SMALL_STATE(3132)] = 70904,
  [SMALL_STATE(3133)] = 70976,
  [SMALL_STATE(3134)] = 71048,
  [SMALL_STATE(3135)] = 71122,
  [SMALL_STATE(3136)] = 71196,
  [SMALL_STATE(3137)] = 71268,
  [SMALL_STATE(3138)] = 71340,
  [SMALL_STATE(3139)] = 71414,
  [SMALL_STATE(3140)] = 71478,
  [SMALL_STATE(3141)] = 71562,
  [SMALL_STATE(3142)] = 71646,
  [SMALL_STATE(3143)] = 71732,
  [SMALL_STATE(3144)] = 71804,
  [SMALL_STATE(3145)] = 71878,
  [SMALL_STATE(3146)] = 71968,
  [SMALL_STATE(3147)] = 72058,
  [SMALL_STATE(3148)] = 72118,
  [SMALL_STATE(3149)] = 72176,
  [SMALL_STATE(3150)] = 72234,
  [SMALL_STATE(3151)] = 72300,
  [SMALL_STATE(3152)] = 72364,
  [SMALL_STATE(3153)] = 72434,
  [SMALL_STATE(3154)] = 72514,
  [SMALL_STATE(3155)] = 72592,
  [SMALL_STATE(3156)] = 72654,
  [SMALL_STATE(3157)] = 72732,
  [SMALL_STATE(3158)] = 72790,
  [SMALL_STATE(3159)] = 72882,
  [SMALL_STATE(3160)] = 72970,
  [SMALL_STATE(3161)] = 73056,
  [SMALL_STATE(3162)] = 73144,
  [SMALL_STATE(3163)] = 73234,
  [SMALL_STATE(3164)] = 73306,
  [SMALL_STATE(3165)] = 73380,
  [SMALL_STATE(3166)] = 73470,
  [SMALL_STATE(3167)] = 73544,
  [SMALL_STATE(3168)] = 73616,
  [SMALL_STATE(3169)] = 73690,
  [SMALL_STATE(3170)] = 73776,
  [SMALL_STATE(3171)] = 73862,
  [SMALL_STATE(3172)] = 73918,
  [SMALL_STATE(3173)] = 73992,
  [SMALL_STATE(3174)] = 74064,
  [SMALL_STATE(3175)] = 74142,
  [SMALL_STATE(3176)] = 74201,
  [SMALL_STATE(3177)] = 74264,
  [SMALL_STATE(3178)] = 74353,
  [SMALL_STATE(3179)] = 74410,
  [SMALL_STATE(3180)] = 74499,
  [SMALL_STATE(3181)] = 74554,
  [SMALL_STATE(3182)] = 74609,
  [SMALL_STATE(3183)] = 74704,
  [SMALL_STATE(3184)] = 74763,
  [SMALL_STATE(3185)] = 74826,
  [SMALL_STATE(3186)] = 74881,
  [SMALL_STATE(3187)] = 74936,
  [SMALL_STATE(3188)] = 74991,
  [SMALL_STATE(3189)] = 75080,
  [SMALL_STATE(3190)] = 75135,
  [SMALL_STATE(3191)] = 75192,
  [SMALL_STATE(3192)] = 75283,
  [SMALL_STATE(3193)] = 75346,
  [SMALL_STATE(3194)] = 75403,
  [SMALL_STATE(3195)] = 75494,
  [SMALL_STATE(3196)] = 75551,
  [SMALL_STATE(3197)] = 75610,
  [SMALL_STATE(3198)] = 75669,
  [SMALL_STATE(3199)] = 75732,
  [SMALL_STATE(3200)] = 75787,
  [SMALL_STATE(3201)] = 75878,
  [SMALL_STATE(3202)] = 75935,
  [SMALL_STATE(3203)] = 75990,
  [SMALL_STATE(3204)] = 76045,
  [SMALL_STATE(3205)] = 76102,
  [SMALL_STATE(3206)] = 76159,
  [SMALL_STATE(3207)] = 76250,
  [SMALL_STATE(3208)] = 76309,
  [SMALL_STATE(3209)] = 76364,
  [SMALL_STATE(3210)] = 76455,
  [SMALL_STATE(3211)] = 76512,
  [SMALL_STATE(3212)] = 76605,
  [SMALL_STATE(3213)] = 76693,
  [SMALL_STATE(3214)] = 76781,
  [SMALL_STATE(3215)] = 76837,
  [SMALL_STATE(3216)] = 76919,
  [SMALL_STATE(3217)] = 77007,
  [SMALL_STATE(3218)] = 77095,
  [SMALL_STATE(3219)] = 77179,
  [SMALL_STATE(3220)] = 77263,
  [SMALL_STATE(3221)] = 77349,
  [SMALL_STATE(3222)] = 77405,
  [SMALL_STATE(3223)] = 77461,
  [SMALL_STATE(3224)] = 77547,
  [SMALL_STATE(3225)] = 77633,
  [SMALL_STATE(3226)] = 77717,
  [SMALL_STATE(3227)] = 77803,
  [SMALL_STATE(3228)] = 77897,
  [SMALL_STATE(3229)] = 77981,
  [SMALL_STATE(3230)] = 78065,
  [SMALL_STATE(3231)] = 78149,
  [SMALL_STATE(3232)] = 78205,
  [SMALL_STATE(3233)] = 78261,
  [SMALL_STATE(3234)] = 78347,
  [SMALL_STATE(3235)] = 78433,
  [SMALL_STATE(3236)] = 78519,
  [SMALL_STATE(3237)] = 78603,
  [SMALL_STATE(3238)] = 78689,
  [SMALL_STATE(3239)] = 78775,
  [SMALL_STATE(3240)] = 78831,
  [SMALL_STATE(3241)] = 78913,
  [SMALL_STATE(3242)] = 78999,
  [SMALL_STATE(3243)] = 79087,
  [SMALL_STATE(3244)] = 79175,
  [SMALL_STATE(3245)] = 79231,
  [SMALL_STATE(3246)] = 79315,
  [SMALL_STATE(3247)] = 79405,
  [SMALL_STATE(3248)] = 79461,
  [SMALL_STATE(3249)] = 79547,
  [SMALL_STATE(3250)] = 79635,
  [SMALL_STATE(3251)] = 79693,
  [SMALL_STATE(3252)] = 79749,
  [SMALL_STATE(3253)] = 79805,
  [SMALL_STATE(3254)] = 79869,
  [SMALL_STATE(3255)] = 79931,
  [SMALL_STATE(3256)] = 79999,
  [SMALL_STATE(3257)] = 80077,
  [SMALL_STATE(3258)] = 80153,
  [SMALL_STATE(3259)] = 80213,
  [SMALL_STATE(3260)] = 80289,
  [SMALL_STATE(3261)] = 80345,
  [SMALL_STATE(3262)] = 80433,
  [SMALL_STATE(3263)] = 80499,
  [SMALL_STATE(3264)] = 80557,
  [SMALL_STATE(3265)] = 80641,
  [SMALL_STATE(3266)] = 80727,
  [SMALL_STATE(3267)] = 80811,
  [SMALL_STATE(3268)] = 80897,
  [SMALL_STATE(3269)] = 80953,
  [SMALL_STATE(3270)] = 81039,
  [SMALL_STATE(3271)] = 81095,
  [SMALL_STATE(3272)] = 81151,
  [SMALL_STATE(3273)] = 81239,
  [SMALL_STATE(3274)] = 81323,
  [SMALL_STATE(3275)] = 81383,
  [SMALL_STATE(3276)] = 81439,
  [SMALL_STATE(3277)] = 81497,
  [SMALL_STATE(3278)] = 81555,
  [SMALL_STATE(3279)] = 81645,
  [SMALL_STATE(3280)] = 81707,
  [SMALL_STATE(3281)] = 81793,
  [SMALL_STATE(3282)] = 81881,
  [SMALL_STATE(3283)] = 81943,
  [SMALL_STATE(3284)] = 81999,
  [SMALL_STATE(3285)] = 82083,
  [SMALL_STATE(3286)] = 82141,
  [SMALL_STATE(3287)] = 82197,
  [SMALL_STATE(3288)] = 82253,
  [SMALL_STATE(3289)] = 82311,
  [SMALL_STATE(3290)] = 82373,
  [SMALL_STATE(3291)] = 82441,
  [SMALL_STATE(3292)] = 82519,
  [SMALL_STATE(3293)] = 82595,
  [SMALL_STATE(3294)] = 82655,
  [SMALL_STATE(3295)] = 82731,
  [SMALL_STATE(3296)] = 82787,
  [SMALL_STATE(3297)] = 82875,
  [SMALL_STATE(3298)] = 82961,
  [SMALL_STATE(3299)] = 83047,
  [SMALL_STATE(3300)] = 83109,
  [SMALL_STATE(3301)] = 83195,
  [SMALL_STATE(3302)] = 83261,
  [SMALL_STATE(3303)] = 83317,
  [SMALL_STATE(3304)] = 83383,
  [SMALL_STATE(3305)] = 83439,
  [SMALL_STATE(3306)] = 83523,
  [SMALL_STATE(3307)] = 83579,
  [SMALL_STATE(3308)] = 83663,
  [SMALL_STATE(3309)] = 83749,
  [SMALL_STATE(3310)] = 83807,
  [SMALL_STATE(3311)] = 83863,
  [SMALL_STATE(3312)] = 83919,
  [SMALL_STATE(3313)] = 83983,
  [SMALL_STATE(3314)] = 84045,
  [SMALL_STATE(3315)] = 84113,
  [SMALL_STATE(3316)] = 84191,
  [SMALL_STATE(3317)] = 84267,
  [SMALL_STATE(3318)] = 84327,
  [SMALL_STATE(3319)] = 84403,
  [SMALL_STATE(3320)] = 84459,
  [SMALL_STATE(3321)] = 84515,
  [SMALL_STATE(3322)] = 84603,
  [SMALL_STATE(3323)] = 84659,
  [SMALL_STATE(3324)] = 84745,
  [SMALL_STATE(3325)] = 84829,
  [SMALL_STATE(3326)] = 84913,
  [SMALL_STATE(3327)] = 84997,
  [SMALL_STATE(3328)] = 85055,
  [SMALL_STATE(3329)] = 85111,
  [SMALL_STATE(3330)] = 85167,
  [SMALL_STATE(3331)] = 85223,
  [SMALL_STATE(3332)] = 85287,
  [SMALL_STATE(3333)] = 85371,
  [SMALL_STATE(3334)] = 85433,
  [SMALL_STATE(3335)] = 85489,
  [SMALL_STATE(3336)] = 85557,
  [SMALL_STATE(3337)] = 85613,
  [SMALL_STATE(3338)] = 85669,
  [SMALL_STATE(3339)] = 85747,
  [SMALL_STATE(3340)] = 85823,
  [SMALL_STATE(3341)] = 85883,
  [SMALL_STATE(3342)] = 85959,
  [SMALL_STATE(3343)] = 86051,
  [SMALL_STATE(3344)] = 86107,
  [SMALL_STATE(3345)] = 86195,
  [SMALL_STATE(3346)] = 86251,
  [SMALL_STATE(3347)] = 86337,
  [SMALL_STATE(3348)] = 86423,
  [SMALL_STATE(3349)] = 86479,
  [SMALL_STATE(3350)] = 86571,
  [SMALL_STATE(3351)] = 86625,
  [SMALL_STATE(3352)] = 86681,
  [SMALL_STATE(3353)] = 86771,
  [SMALL_STATE(3354)] = 86835,
  [SMALL_STATE(3355)] = 86918,
  [SMALL_STATE(3356)] = 87001,
  [SMALL_STATE(3357)] = 87086,
  [SMALL_STATE(3358)] = 87171,
  [SMALL_STATE(3359)] = 87256,
  [SMALL_STATE(3360)] = 87311,
  [SMALL_STATE(3361)] = 87398,
  [SMALL_STATE(3362)] = 87453,
  [SMALL_STATE(3363)] = 87536,
  [SMALL_STATE(3364)] = 87619,
  [SMALL_STATE(3365)] = 87702,
  [SMALL_STATE(3366)] = 87787,
  [SMALL_STATE(3367)] = 87854,
  [SMALL_STATE(3368)] = 87909,
  [SMALL_STATE(3369)] = 87964,
  [SMALL_STATE(3370)] = 88057,
  [SMALL_STATE(3371)] = 88114,
  [SMALL_STATE(3372)] = 88169,
  [SMALL_STATE(3373)] = 88224,
  [SMALL_STATE(3374)] = 88287,
  [SMALL_STATE(3375)] = 88348,
  [SMALL_STATE(3376)] = 88415,
  [SMALL_STATE(3377)] = 88492,
  [SMALL_STATE(3378)] = 88567,
  [SMALL_STATE(3379)] = 88626,
  [SMALL_STATE(3380)] = 88701,
  [SMALL_STATE(3381)] = 88756,
  [SMALL_STATE(3382)] = 88841,
  [SMALL_STATE(3383)] = 88894,
  [SMALL_STATE(3384)] = 88987,
  [SMALL_STATE(3385)] = 89042,
  [SMALL_STATE(3386)] = 89095,
  [SMALL_STATE(3387)] = 89176,
  [SMALL_STATE(3388)] = 89259,
  [SMALL_STATE(3389)] = 89318,
  [SMALL_STATE(3390)] = 89375,
  [SMALL_STATE(3391)] = 89430,
  [SMALL_STATE(3392)] = 89485,
  [SMALL_STATE(3393)] = 89548,
  [SMALL_STATE(3394)] = 89609,
  [SMALL_STATE(3395)] = 89676,
  [SMALL_STATE(3396)] = 89753,
  [SMALL_STATE(3397)] = 89828,
  [SMALL_STATE(3398)] = 89887,
  [SMALL_STATE(3399)] = 89962,
  [SMALL_STATE(3400)] = 90017,
  [SMALL_STATE(3401)] = 90104,
  [SMALL_STATE(3402)] = 90159,
  [SMALL_STATE(3403)] = 90214,
  [SMALL_STATE(3404)] = 90271,
  [SMALL_STATE(3405)] = 90332,
  [SMALL_STATE(3406)] = 90419,
  [SMALL_STATE(3407)] = 90506,
  [SMALL_STATE(3408)] = 90591,
  [SMALL_STATE(3409)] = 90676,
  [SMALL_STATE(3410)] = 90761,
  [SMALL_STATE(3411)] = 90844,
  [SMALL_STATE(3412)] = 90929,
  [SMALL_STATE(3413)] = 90984,
  [SMALL_STATE(3414)] = 91039,
  [SMALL_STATE(3415)] = 91124,
  [SMALL_STATE(3416)] = 91179,
  [SMALL_STATE(3417)] = 91232,
  [SMALL_STATE(3418)] = 91293,
  [SMALL_STATE(3419)] = 91374,
  [SMALL_STATE(3420)] = 91427,
  [SMALL_STATE(3421)] = 91482,
  [SMALL_STATE(3422)] = 91567,
  [SMALL_STATE(3423)] = 91650,
  [SMALL_STATE(3424)] = 91727,
  [SMALL_STATE(3425)] = 91802,
  [SMALL_STATE(3426)] = 91861,
  [SMALL_STATE(3427)] = 91936,
  [SMALL_STATE(3428)] = 91993,
  [SMALL_STATE(3429)] = 92076,
  [SMALL_STATE(3430)] = 92159,
  [SMALL_STATE(3431)] = 92240,
  [SMALL_STATE(3432)] = 92323,
  [SMALL_STATE(3433)] = 92408,
  [SMALL_STATE(3434)] = 92491,
  [SMALL_STATE(3435)] = 92574,
  [SMALL_STATE(3436)] = 92659,
  [SMALL_STATE(3437)] = 92740,
  [SMALL_STATE(3438)] = 92827,
  [SMALL_STATE(3439)] = 92914,
  [SMALL_STATE(3440)] = 93001,
  [SMALL_STATE(3441)] = 93088,
  [SMALL_STATE(3442)] = 93145,
  [SMALL_STATE(3443)] = 93200,
  [SMALL_STATE(3444)] = 93255,
  [SMALL_STATE(3445)] = 93318,
  [SMALL_STATE(3446)] = 93379,
  [SMALL_STATE(3447)] = 93446,
  [SMALL_STATE(3448)] = 93523,
  [SMALL_STATE(3449)] = 93598,
  [SMALL_STATE(3450)] = 93657,
  [SMALL_STATE(3451)] = 93732,
  [SMALL_STATE(3452)] = 93787,
  [SMALL_STATE(3453)] = 93870,
  [SMALL_STATE(3454)] = 93925,
  [SMALL_STATE(3455)] = 94008,
  [SMALL_STATE(3456)] = 94101,
  [SMALL_STATE(3457)] = 94154,
  [SMALL_STATE(3458)] = 94241,
  [SMALL_STATE(3459)] = 94330,
  [SMALL_STATE(3460)] = 94389,
  [SMALL_STATE(3461)] = 94444,
  [SMALL_STATE(3462)] = 94503,
  [SMALL_STATE(3463)] = 94596,
  [SMALL_STATE(3464)] = 94649,
  [SMALL_STATE(3465)] = 94742,
  [SMALL_STATE(3466)] = 94795,
  [SMALL_STATE(3467)] = 94884,
  [SMALL_STATE(3468)] = 94969,
  [SMALL_STATE(3469)] = 95054,
  [SMALL_STATE(3470)] = 95111,
  [SMALL_STATE(3471)] = 95166,
  [SMALL_STATE(3472)] = 95221,
  [SMALL_STATE(3473)] = 95276,
  [SMALL_STATE(3474)] = 95339,
  [SMALL_STATE(3475)] = 95424,
  [SMALL_STATE(3476)] = 95507,
  [SMALL_STATE(3477)] = 95587,
  [SMALL_STATE(3478)] = 95653,
  [SMALL_STATE(3479)] = 95733,
  [SMALL_STATE(3480)] = 95821,
  [SMALL_STATE(3481)] = 95875,
  [SMALL_STATE(3482)] = 95957,
  [SMALL_STATE(3483)] = 96041,
  [SMALL_STATE(3484)] = 96123,
  [SMALL_STATE(3485)] = 96205,
  [SMALL_STATE(3486)] = 96281,
  [SMALL_STATE(3487)] = 96355,
  [SMALL_STATE(3488)] = 96437,
  [SMALL_STATE(3489)] = 96503,
  [SMALL_STATE(3490)] = 96561,
  [SMALL_STATE(3491)] = 96645,
  [SMALL_STATE(3492)] = 96699,
  [SMALL_STATE(3493)] = 96783,
  [SMALL_STATE(3494)] = 96841,
  [SMALL_STATE(3495)] = 96915,
  [SMALL_STATE(3496)] = 96969,
  [SMALL_STATE(3497)] = 97023,
  [SMALL_STATE(3498)] = 97099,
  [SMALL_STATE(3499)] = 97173,
  [SMALL_STATE(3500)] = 97255,
  [SMALL_STATE(3501)] = 97345,
  [SMALL_STATE(3502)] = 97403,
  [SMALL_STATE(3503)] = 97493,
  [SMALL_STATE(3504)] = 97551,
  [SMALL_STATE(3505)] = 97637,
  [SMALL_STATE(3506)] = 97693,
  [SMALL_STATE(3507)] = 97767,
  [SMALL_STATE(3508)] = 97819,
  [SMALL_STATE(3509)] = 97873,
  [SMALL_STATE(3510)] = 97957,
  [SMALL_STATE(3511)] = 98039,
  [SMALL_STATE(3512)] = 98093,
  [SMALL_STATE(3513)] = 98175,
  [SMALL_STATE(3514)] = 98231,
  [SMALL_STATE(3515)] = 98287,
  [SMALL_STATE(3516)] = 98347,
  [SMALL_STATE(3517)] = 98401,
  [SMALL_STATE(3518)] = 98485,
  [SMALL_STATE(3519)] = 98539,
  [SMALL_STATE(3520)] = 98625,
  [SMALL_STATE(3521)] = 98711,
  [SMALL_STATE(3522)] = 98793,
  [SMALL_STATE(3523)] = 98845,
  [SMALL_STATE(3524)] = 98927,
  [SMALL_STATE(3525)] = 98981,
  [SMALL_STATE(3526)] = 99063,
  [SMALL_STATE(3527)] = 99153,
  [SMALL_STATE(3528)] = 99209,
  [SMALL_STATE(3529)] = 99293,
  [SMALL_STATE(3530)] = 99379,
  [SMALL_STATE(3531)] = 99467,
  [SMALL_STATE(3532)] = 99549,
  [SMALL_STATE(3533)] = 99639,
  [SMALL_STATE(3534)] = 99723,
  [SMALL_STATE(3535)] = 99777,
  [SMALL_STATE(3536)] = 99831,
  [SMALL_STATE(3537)] = 99893,
  [SMALL_STATE(3538)] = 99979,
  [SMALL_STATE(3539)] = 100065,
  [SMALL_STATE(3540)] = 100151,
  [SMALL_STATE(3541)] = 100211,
  [SMALL_STATE(3542)] = 100265,
  [SMALL_STATE(3543)] = 100327,
  [SMALL_STATE(3544)] = 100385,
  [SMALL_STATE(3545)] = 100445,
  [SMALL_STATE(3546)] = 100527,
  [SMALL_STATE(3547)] = 100611,
  [SMALL_STATE(3548)] = 100694,
  [SMALL_STATE(3549)] = 100747,
  [SMALL_STATE(3550)] = 100832,
  [SMALL_STATE(3551)] = 100885,
  [SMALL_STATE(3552)] = 100938,
  [SMALL_STATE(3553)] = 101019,
  [SMALL_STATE(3554)] = 101100,
  [SMALL_STATE(3555)] = 101181,
  [SMALL_STATE(3556)] = 101262,
  [SMALL_STATE(3557)] = 101343,
  [SMALL_STATE(3558)] = 101424,
  [SMALL_STATE(3559)] = 101479,
  [SMALL_STATE(3560)] = 101534,
  [SMALL_STATE(3561)] = 101587,
  [SMALL_STATE(3562)] = 101644,
  [SMALL_STATE(3563)] = 101727,
  [SMALL_STATE(3564)] = 101780,
  [SMALL_STATE(3565)] = 101855,
  [SMALL_STATE(3566)] = 101914,
  [SMALL_STATE(3567)] = 101967,
  [SMALL_STATE(3568)] = 102020,
  [SMALL_STATE(3569)] = 102107,
  [SMALL_STATE(3570)] = 102166,
  [SMALL_STATE(3571)] = 102247,
  [SMALL_STATE(3572)] = 102300,
  [SMALL_STATE(3573)] = 102385,
  [SMALL_STATE(3574)] = 102458,
  [SMALL_STATE(3575)] = 102511,
  [SMALL_STATE(3576)] = 102566,
  [SMALL_STATE(3577)] = 102627,
  [SMALL_STATE(3578)] = 102708,
  [SMALL_STATE(3579)] = 102781,
  [SMALL_STATE(3580)] = 102836,
  [SMALL_STATE(3581)] = 102919,
  [SMALL_STATE(3582)] = 102972,
  [SMALL_STATE(3583)] = 103027,
  [SMALL_STATE(3584)] = 103110,
  [SMALL_STATE(3585)] = 103163,
  [SMALL_STATE(3586)] = 103228,
  [SMALL_STATE(3587)] = 103311,
  [SMALL_STATE(3588)] = 103396,
  [SMALL_STATE(3589)] = 103485,
  [SMALL_STATE(3590)] = 103568,
  [SMALL_STATE(3591)] = 103653,
  [SMALL_STATE(3592)] = 103706,
  [SMALL_STATE(3593)] = 103759,
  [SMALL_STATE(3594)] = 103812,
  [SMALL_STATE(3595)] = 103897,
  [SMALL_STATE(3596)] = 103952,
  [SMALL_STATE(3597)] = 104005,
  [SMALL_STATE(3598)] = 104060,
  [SMALL_STATE(3599)] = 104113,
  [SMALL_STATE(3600)] = 104166,
  [SMALL_STATE(3601)] = 104251,
  [SMALL_STATE(3602)] = 104340,
  [SMALL_STATE(3603)] = 104393,
  [SMALL_STATE(3604)] = 104475,
  [SMALL_STATE(3605)] = 104545,
  [SMALL_STATE(3606)] = 104619,
  [SMALL_STATE(3607)] = 104669,
  [SMALL_STATE(3608)] = 104723,
  [SMALL_STATE(3609)] = 104805,
  [SMALL_STATE(3610)] = 104875,
  [SMALL_STATE(3611)] = 104955,
  [SMALL_STATE(3612)] = 105039,
  [SMALL_STATE(3613)] = 105119,
  [SMALL_STATE(3614)] = 105203,
  [SMALL_STATE(3615)] = 105287,
  [SMALL_STATE(3616)] = 105341,
  [SMALL_STATE(3617)] = 105393,
  [SMALL_STATE(3618)] = 105445,
  [SMALL_STATE(3619)] = 105505,
  [SMALL_STATE(3620)] = 105557,
  [SMALL_STATE(3621)] = 105637,
  [SMALL_STATE(3622)] = 105721,
  [SMALL_STATE(3623)] = 105779,
  [SMALL_STATE(3624)] = 105843,
  [SMALL_STATE(3625)] = 105895,
  [SMALL_STATE(3626)] = 105983,
  [SMALL_STATE(3627)] = 106057,
  [SMALL_STATE(3628)] = 106137,
  [SMALL_STATE(3629)] = 106189,
  [SMALL_STATE(3630)] = 106267,
  [SMALL_STATE(3631)] = 106339,
  [SMALL_STATE(3632)] = 106395,
  [SMALL_STATE(3633)] = 106459,
  [SMALL_STATE(3634)] = 106537,
  [SMALL_STATE(3635)] = 106609,
  [SMALL_STATE(3636)] = 106661,
  [SMALL_STATE(3637)] = 106713,
  [SMALL_STATE(3638)] = 106795,
  [SMALL_STATE(3639)] = 106877,
  [SMALL_STATE(3640)] = 106963,
  [SMALL_STATE(3641)] = 107043,
  [SMALL_STATE(3642)] = 107097,
  [SMALL_STATE(3643)] = 107175,
  [SMALL_STATE(3644)] = 107261,
  [SMALL_STATE(3645)] = 107341,
  [SMALL_STATE(3646)] = 107393,
  [SMALL_STATE(3647)] = 107445,
  [SMALL_STATE(3648)] = 107529,
  [SMALL_STATE(3649)] = 107581,
  [SMALL_STATE(3650)] = 107665,
  [SMALL_STATE(3651)] = 107719,
  [SMALL_STATE(3652)] = 107771,
  [SMALL_STATE(3653)] = 107823,
  [SMALL_STATE(3654)] = 107883,
  [SMALL_STATE(3655)] = 107963,
  [SMALL_STATE(3656)] = 108021,
  [SMALL_STATE(3657)] = 108103,
  [SMALL_STATE(3658)] = 108185,
  [SMALL_STATE(3659)] = 108265,
  [SMALL_STATE(3660)] = 108347,
  [SMALL_STATE(3661)] = 108429,
  [SMALL_STATE(3662)] = 108481,
  [SMALL_STATE(3663)] = 108537,
  [SMALL_STATE(3664)] = 108589,
  [SMALL_STATE(3665)] = 108653,
  [SMALL_STATE(3666)] = 108707,
  [SMALL_STATE(3667)] = 108793,
  [SMALL_STATE(3668)] = 108873,
  [SMALL_STATE(3669)] = 108947,
  [SMALL_STATE(3670)] = 109019,
  [SMALL_STATE(3671)] = 109075,
  [SMALL_STATE(3672)] = 109147,
  [SMALL_STATE(3673)] = 109201,
  [SMALL_STATE(3674)] = 109291,
  [SMALL_STATE(3675)] = 109371,
  [SMALL_STATE(3676)] = 109423,
  [SMALL_STATE(3677)] = 109475,
  [SMALL_STATE(3678)] = 109559,
  [SMALL_STATE(3679)] = 109643,
  [SMALL_STATE(3680)] = 109699,
  [SMALL_STATE(3681)] = 109751,
  [SMALL_STATE(3682)] = 109835,
  [SMALL_STATE(3683)] = 109887,
  [SMALL_STATE(3684)] = 109969,
  [SMALL_STATE(3685)] = 110041,
  [SMALL_STATE(3686)] = 110121,
  [SMALL_STATE(3687)] = 110173,
  [SMALL_STATE(3688)] = 110225,
  [SMALL_STATE(3689)] = 110277,
  [SMALL_STATE(3690)] = 110337,
  [SMALL_STATE(3691)] = 110395,
  [SMALL_STATE(3692)] = 110459,
  [SMALL_STATE(3693)] = 110533,
  [SMALL_STATE(3694)] = 110605,
  [SMALL_STATE(3695)] = 110661,
  [SMALL_STATE(3696)] = 110715,
  [SMALL_STATE(3697)] = 110787,
  [SMALL_STATE(3698)] = 110869,
  [SMALL_STATE(3699)] = 110951,
  [SMALL_STATE(3700)] = 111031,
  [SMALL_STATE(3701)] = 111115,
  [SMALL_STATE(3702)] = 111199,
  [SMALL_STATE(3703)] = 111277,
  [SMALL_STATE(3704)] = 111361,
  [SMALL_STATE(3705)] = 111415,
  [SMALL_STATE(3706)] = 111495,
  [SMALL_STATE(3707)] = 111583,
  [SMALL_STATE(3708)] = 111663,
  [SMALL_STATE(3709)] = 111745,
  [SMALL_STATE(3710)] = 111823,
  [SMALL_STATE(3711)] = 111875,
  [SMALL_STATE(3712)] = 111927,
  [SMALL_STATE(3713)] = 112007,
  [SMALL_STATE(3714)] = 112059,
  [SMALL_STATE(3715)] = 112139,
  [SMALL_STATE(3716)] = 112221,
  [SMALL_STATE(3717)] = 112275,
  [SMALL_STATE(3718)] = 112327,
  [SMALL_STATE(3719)] = 112379,
  [SMALL_STATE(3720)] = 112439,
  [SMALL_STATE(3721)] = 112497,
  [SMALL_STATE(3722)] = 112561,
  [SMALL_STATE(3723)] = 112635,
  [SMALL_STATE(3724)] = 112707,
  [SMALL_STATE(3725)] = 112763,
  [SMALL_STATE(3726)] = 112835,
  [SMALL_STATE(3727)] = 112887,
  [SMALL_STATE(3728)] = 112939,
  [SMALL_STATE(3729)] = 113019,
  [SMALL_STATE(3730)] = 113069,
  [SMALL_STATE(3731)] = 113149,
  [SMALL_STATE(3732)] = 113199,
  [SMALL_STATE(3733)] = 113277,
  [SMALL_STATE(3734)] = 113357,
  [SMALL_STATE(3735)] = 113429,
  [SMALL_STATE(3736)] = 113479,
  [SMALL_STATE(3737)] = 113529,
  [SMALL_STATE(3738)] = 113609,
  [SMALL_STATE(3739)] = 113689,
  [SMALL_STATE(3740)] = 113743,
  [SMALL_STATE(3741)] = 113825,
  [SMALL_STATE(3742)] = 113877,
  [SMALL_STATE(3743)] = 113963,
  [SMALL_STATE(3744)] = 114015,
  [SMALL_STATE(3745)] = 114065,
  [SMALL_STATE(3746)] = 114147,
  [SMALL_STATE(3747)] = 114227,
  [SMALL_STATE(3748)] = 114307,
  [SMALL_STATE(3749)] = 114391,
  [SMALL_STATE(3750)] = 114479,
  [SMALL_STATE(3751)] = 114539,
  [SMALL_STATE(3752)] = 114623,
  [SMALL_STATE(3753)] = 114681,
  [SMALL_STATE(3754)] = 114761,
  [SMALL_STATE(3755)] = 114843,
  [SMALL_STATE(3756)] = 114923,
  [SMALL_STATE(3757)] = 115003,
  [SMALL_STATE(3758)] = 115089,
  [SMALL_STATE(3759)] = 115173,
  [SMALL_STATE(3760)] = 115229,
  [SMALL_STATE(3761)] = 115283,
  [SMALL_STATE(3762)] = 115356,
  [SMALL_STATE(3763)] = 115437,
  [SMALL_STATE(3764)] = 115518,
  [SMALL_STATE(3765)] = 115569,
  [SMALL_STATE(3766)] = 115652,
  [SMALL_STATE(3767)] = 115707,
  [SMALL_STATE(3768)] = 115778,
  [SMALL_STATE(3769)] = 115857,
  [SMALL_STATE(3770)] = 115908,
  [SMALL_STATE(3771)] = 115989,
  [SMALL_STATE(3772)] = 116042,
  [SMALL_STATE(3773)] = 116093,
  [SMALL_STATE(3774)] = 116144,
  [SMALL_STATE(3775)] = 116203,
  [SMALL_STATE(3776)] = 116262,
  [SMALL_STATE(3777)] = 116319,
  [SMALL_STATE(3778)] = 116382,
  [SMALL_STATE(3779)] = 116455,
  [SMALL_STATE(3780)] = 116526,
  [SMALL_STATE(3781)] = 116581,
  [SMALL_STATE(3782)] = 116640,
  [SMALL_STATE(3783)] = 116711,
  [SMALL_STATE(3784)] = 116762,
  [SMALL_STATE(3785)] = 116813,
  [SMALL_STATE(3786)] = 116892,
  [SMALL_STATE(3787)] = 116945,
  [SMALL_STATE(3788)] = 117002,
  [SMALL_STATE(3789)] = 117065,
  [SMALL_STATE(3790)] = 117118,
  [SMALL_STATE(3791)] = 117191,
  [SMALL_STATE(3792)] = 117262,
  [SMALL_STATE(3793)] = 117317,
  [SMALL_STATE(3794)] = 117368,
  [SMALL_STATE(3795)] = 117419,
  [SMALL_STATE(3796)] = 117496,
  [SMALL_STATE(3797)] = 117567,
  [SMALL_STATE(3798)] = 117618,
  [SMALL_STATE(3799)] = 117697,
  [SMALL_STATE(3800)] = 117778,
  [SMALL_STATE(3801)] = 117855,
  [SMALL_STATE(3802)] = 117938,
  [SMALL_STATE(3803)] = 118025,
  [SMALL_STATE(3804)] = 118084,
  [SMALL_STATE(3805)] = 118135,
  [SMALL_STATE(3806)] = 118216,
  [SMALL_STATE(3807)] = 118267,
  [SMALL_STATE(3808)] = 118344,
  [SMALL_STATE(3809)] = 118425,
  [SMALL_STATE(3810)] = 118476,
  [SMALL_STATE(3811)] = 118555,
  [SMALL_STATE(3812)] = 118612,
  [SMALL_STATE(3813)] = 118675,
  [SMALL_STATE(3814)] = 118754,
  [SMALL_STATE(3815)] = 118837,
  [SMALL_STATE(3816)] = 118918,
  [SMALL_STATE(3817)] = 118969,
  [SMALL_STATE(3818)] = 119022,
  [SMALL_STATE(3819)] = 119077,
  [SMALL_STATE(3820)] = 119158,
  [SMALL_STATE(3821)] = 119239,
  [SMALL_STATE(3822)] = 119316,
  [SMALL_STATE(3823)] = 119397,
  [SMALL_STATE(3824)] = 119476,
  [SMALL_STATE(3825)] = 119527,
  [SMALL_STATE(3826)] = 119578,
  [SMALL_STATE(3827)] = 119637,
  [SMALL_STATE(3828)] = 119688,
  [SMALL_STATE(3829)] = 119769,
  [SMALL_STATE(3830)] = 119822,
  [SMALL_STATE(3831)] = 119901,
  [SMALL_STATE(3832)] = 119952,
  [SMALL_STATE(3833)] = 120031,
  [SMALL_STATE(3834)] = 120082,
  [SMALL_STATE(3835)] = 120133,
  [SMALL_STATE(3836)] = 120214,
  [SMALL_STATE(3837)] = 120267,
  [SMALL_STATE(3838)] = 120324,
  [SMALL_STATE(3839)] = 120387,
  [SMALL_STATE(3840)] = 120470,
  [SMALL_STATE(3841)] = 120533,
  [SMALL_STATE(3842)] = 120604,
  [SMALL_STATE(3843)] = 120659,
  [SMALL_STATE(3844)] = 120742,
  [SMALL_STATE(3845)] = 120821,
  [SMALL_STATE(3846)] = 120900,
  [SMALL_STATE(3847)] = 120951,
  [SMALL_STATE(3848)] = 121022,
  [SMALL_STATE(3849)] = 121073,
  [SMALL_STATE(3850)] = 121124,
  [SMALL_STATE(3851)] = 121197,
  [SMALL_STATE(3852)] = 121270,
  [SMALL_STATE(3853)] = 121349,
  [SMALL_STATE(3854)] = 121432,
  [SMALL_STATE(3855)] = 121513,
  [SMALL_STATE(3856)] = 121584,
  [SMALL_STATE(3857)] = 121663,
  [SMALL_STATE(3858)] = 121718,
  [SMALL_STATE(3859)] = 121801,
  [SMALL_STATE(3860)] = 121872,
  [SMALL_STATE(3861)] = 121951,
  [SMALL_STATE(3862)] = 122030,
  [SMALL_STATE(3863)] = 122111,
  [SMALL_STATE(3864)] = 122192,
  [SMALL_STATE(3865)] = 122245,
  [SMALL_STATE(3866)] = 122326,
  [SMALL_STATE(3867)] = 122397,
  [SMALL_STATE(3868)] = 122484,
  [SMALL_STATE(3869)] = 122535,
  [SMALL_STATE(3870)] = 122586,
  [SMALL_STATE(3871)] = 122637,
  [SMALL_STATE(3872)] = 122690,
  [SMALL_STATE(3873)] = 122769,
  [SMALL_STATE(3874)] = 122850,
  [SMALL_STATE(3875)] = 122901,
  [SMALL_STATE(3876)] = 122954,
  [SMALL_STATE(3877)] = 123005,
  [SMALL_STATE(3878)] = 123084,
  [SMALL_STATE(3879)] = 123163,
  [SMALL_STATE(3880)] = 123216,
  [SMALL_STATE(3881)] = 123295,
  [SMALL_STATE(3882)] = 123346,
  [SMALL_STATE(3883)] = 123397,
  [SMALL_STATE(3884)] = 123476,
  [SMALL_STATE(3885)] = 123559,
  [SMALL_STATE(3886)] = 123638,
  [SMALL_STATE(3887)] = 123723,
  [SMALL_STATE(3888)] = 123774,
  [SMALL_STATE(3889)] = 123855,
  [SMALL_STATE(3890)] = 123912,
  [SMALL_STATE(3891)] = 123992,
  [SMALL_STATE(3892)] = 124070,
  [SMALL_STATE(3893)] = 124158,
  [SMALL_STATE(3894)] = 124236,
  [SMALL_STATE(3895)] = 124314,
  [SMALL_STATE(3896)] = 124390,
  [SMALL_STATE(3897)] = 124468,
  [SMALL_STATE(3898)] = 124546,
  [SMALL_STATE(3899)] = 124628,
  [SMALL_STATE(3900)] = 124712,
  [SMALL_STATE(3901)] = 124790,
  [SMALL_STATE(3902)] = 124868,
  [SMALL_STATE(3903)] = 124946,
  [SMALL_STATE(3904)] = 125024,
  [SMALL_STATE(3905)] = 125102,
  [SMALL_STATE(3906)] = 125184,
  [SMALL_STATE(3907)] = 125260,
  [SMALL_STATE(3908)] = 125342,
  [SMALL_STATE(3909)] = 125424,
  [SMALL_STATE(3910)] = 125502,
  [SMALL_STATE(3911)] = 125590,
  [SMALL_STATE(3912)] = 125674,
  [SMALL_STATE(3913)] = 125756,
  [SMALL_STATE(3914)] = 125836,
  [SMALL_STATE(3915)] = 125886,
  [SMALL_STATE(3916)] = 125936,
  [SMALL_STATE(3917)] = 125986,
  [SMALL_STATE(3918)] = 126066,
  [SMALL_STATE(3919)] = 126146,
  [SMALL_STATE(3920)] = 126224,
  [SMALL_STATE(3921)] = 126304,
  [SMALL_STATE(3922)] = 126384,
  [SMALL_STATE(3923)] = 126468,
  [SMALL_STATE(3924)] = 126518,
  [SMALL_STATE(3925)] = 126572,
  [SMALL_STATE(3926)] = 126622,
  [SMALL_STATE(3927)] = 126706,
  [SMALL_STATE(3928)] = 126786,
  [SMALL_STATE(3929)] = 126836,
  [SMALL_STATE(3930)] = 126914,
  [SMALL_STATE(3931)] = 126992,
  [SMALL_STATE(3932)] = 127070,
  [SMALL_STATE(3933)] = 127120,
  [SMALL_STATE(3934)] = 127204,
  [SMALL_STATE(3935)] = 127256,
  [SMALL_STATE(3936)] = 127340,
  [SMALL_STATE(3937)] = 127398,
  [SMALL_STATE(3938)] = 127476,
  [SMALL_STATE(3939)] = 127554,
  [SMALL_STATE(3940)] = 127634,
  [SMALL_STATE(3941)] = 127684,
  [SMALL_STATE(3942)] = 127734,
  [SMALL_STATE(3943)] = 127790,
  [SMALL_STATE(3944)] = 127842,
  [SMALL_STATE(3945)] = 127892,
  [SMALL_STATE(3946)] = 127942,
  [SMALL_STATE(3947)] = 128000,
  [SMALL_STATE(3948)] = 128056,
  [SMALL_STATE(3949)] = 128118,
  [SMALL_STATE(3950)] = 128190,
  [SMALL_STATE(3951)] = 128260,
  [SMALL_STATE(3952)] = 128314,
  [SMALL_STATE(3953)] = 128384,
  [SMALL_STATE(3954)] = 128434,
  [SMALL_STATE(3955)] = 128516,
  [SMALL_STATE(3956)] = 128570,
  [SMALL_STATE(3957)] = 128652,
  [SMALL_STATE(3958)] = 128734,
  [SMALL_STATE(3959)] = 128816,
  [SMALL_STATE(3960)] = 128898,
  [SMALL_STATE(3961)] = 128948,
  [SMALL_STATE(3962)] = 129010,
  [SMALL_STATE(3963)] = 129088,
  [SMALL_STATE(3964)] = 129172,
  [SMALL_STATE(3965)] = 129250,
  [SMALL_STATE(3966)] = 129330,
  [SMALL_STATE(3967)] = 129410,
  [SMALL_STATE(3968)] = 129490,
  [SMALL_STATE(3969)] = 129572,
  [SMALL_STATE(3970)] = 129652,
  [SMALL_STATE(3971)] = 129734,
  [SMALL_STATE(3972)] = 129806,
  [SMALL_STATE(3973)] = 129876,
  [SMALL_STATE(3974)] = 129930,
  [SMALL_STATE(3975)] = 130008,
  [SMALL_STATE(3976)] = 130078,
  [SMALL_STATE(3977)] = 130158,
  [SMALL_STATE(3978)] = 130210,
  [SMALL_STATE(3979)] = 130260,
  [SMALL_STATE(3980)] = 130310,
  [SMALL_STATE(3981)] = 130368,
  [SMALL_STATE(3982)] = 130424,
  [SMALL_STATE(3983)] = 130506,
  [SMALL_STATE(3984)] = 130568,
  [SMALL_STATE(3985)] = 130620,
  [SMALL_STATE(3986)] = 130692,
  [SMALL_STATE(3987)] = 130762,
  [SMALL_STATE(3988)] = 130816,
  [SMALL_STATE(3989)] = 130886,
  [SMALL_STATE(3990)] = 130936,
  [SMALL_STATE(3991)] = 131014,
  [SMALL_STATE(3992)] = 131093,
  [SMALL_STATE(3993)] = 131142,
  [SMALL_STATE(3994)] = 131199,
  [SMALL_STATE(3995)] = 131254,
  [SMALL_STATE(3996)] = 131335,
  [SMALL_STATE(3997)] = 131396,
  [SMALL_STATE(3998)] = 131467,
  [SMALL_STATE(3999)] = 131544,
  [SMALL_STATE(4000)] = 131621,
  [SMALL_STATE(4001)] = 131700,
  [SMALL_STATE(4002)] = 131775,
  [SMALL_STATE(4003)] = 131852,
  [SMALL_STATE(4004)] = 131921,
  [SMALL_STATE(4005)] = 131974,
  [SMALL_STATE(4006)] = 132043,
  [SMALL_STATE(4007)] = 132096,
  [SMALL_STATE(4008)] = 132173,
  [SMALL_STATE(4009)] = 132254,
  [SMALL_STATE(4010)] = 132333,
  [SMALL_STATE(4011)] = 132410,
  [SMALL_STATE(4012)] = 132489,
  [SMALL_STATE(4013)] = 132540,
  [SMALL_STATE(4014)] = 132621,
  [SMALL_STATE(4015)] = 132702,
  [SMALL_STATE(4016)] = 132783,
  [SMALL_STATE(4017)] = 132832,
  [SMALL_STATE(4018)] = 132911,
  [SMALL_STATE(4019)] = 132962,
  [SMALL_STATE(4020)] = 133011,
  [SMALL_STATE(4021)] = 133088,
  [SMALL_STATE(4022)] = 133167,
  [SMALL_STATE(4023)] = 133244,
  [SMALL_STATE(4024)] = 133321,
  [SMALL_STATE(4025)] = 133398,
  [SMALL_STATE(4026)] = 133477,
  [SMALL_STATE(4027)] = 133526,
  [SMALL_STATE(4028)] = 133603,
  [SMALL_STATE(4029)] = 133652,
  [SMALL_STATE(4030)] = 133727,
  [SMALL_STATE(4031)] = 133808,
  [SMALL_STATE(4032)] = 133857,
  [SMALL_STATE(4033)] = 133940,
  [SMALL_STATE(4034)] = 133989,
  [SMALL_STATE(4035)] = 134070,
  [SMALL_STATE(4036)] = 134121,
  [SMALL_STATE(4037)] = 134202,
  [SMALL_STATE(4038)] = 134281,
  [SMALL_STATE(4039)] = 134334,
  [SMALL_STATE(4040)] = 134383,
  [SMALL_STATE(4041)] = 134432,
  [SMALL_STATE(4042)] = 134483,
  [SMALL_STATE(4043)] = 134532,
  [SMALL_STATE(4044)] = 134583,
  [SMALL_STATE(4045)] = 134632,
  [SMALL_STATE(4046)] = 134681,
  [SMALL_STATE(4047)] = 134738,
  [SMALL_STATE(4048)] = 134793,
  [SMALL_STATE(4049)] = 134854,
  [SMALL_STATE(4050)] = 134925,
  [SMALL_STATE(4051)] = 134994,
  [SMALL_STATE(4052)] = 135047,
  [SMALL_STATE(4053)] = 135116,
  [SMALL_STATE(4054)] = 135193,
  [SMALL_STATE(4055)] = 135270,
  [SMALL_STATE(4056)] = 135348,
  [SMALL_STATE(4057)] = 135424,
  [SMALL_STATE(4058)] = 135472,
  [SMALL_STATE(4059)] = 135546,
  [SMALL_STATE(4060)] = 135622,
  [SMALL_STATE(4061)] = 135698,
  [SMALL_STATE(4062)] = 135746,
  [SMALL_STATE(4063)] = 135824,
  [SMALL_STATE(4064)] = 135906,
  [SMALL_STATE(4065)] = 135954,
  [SMALL_STATE(4066)] = 136004,
  [SMALL_STATE(4067)] = 136080,
  [SMALL_STATE(4068)] = 136136,
  [SMALL_STATE(4069)] = 136212,
  [SMALL_STATE(4070)] = 136288,
  [SMALL_STATE(4071)] = 136362,
  [SMALL_STATE(4072)] = 136438,
  [SMALL_STATE(4073)] = 136492,
  [SMALL_STATE(4074)] = 136552,
  [SMALL_STATE(4075)] = 136598,
  [SMALL_STATE(4076)] = 136648,
  [SMALL_STATE(4077)] = 136726,
  [SMALL_STATE(4078)] = 136802,
  [SMALL_STATE(4079)] = 136872,
  [SMALL_STATE(4080)] = 136940,
  [SMALL_STATE(4081)] = 136992,
  [SMALL_STATE(4082)] = 137060,
  [SMALL_STATE(4083)] = 137108,
  [SMALL_STATE(4084)] = 137184,
  [SMALL_STATE(4085)] = 137262,
  [SMALL_STATE(4086)] = 137338,
  [SMALL_STATE(4087)] = 137414,
  [SMALL_STATE(4088)] = 137462,
  [SMALL_STATE(4089)] = 137512,
  [SMALL_STATE(4090)] = 137560,
  [SMALL_STATE(4091)] = 137608,
  [SMALL_STATE(4092)] = 137664,
  [SMALL_STATE(4093)] = 137718,
  [SMALL_STATE(4094)] = 137778,
  [SMALL_STATE(4095)] = 137848,
  [SMALL_STATE(4096)] = 137926,
  [SMALL_STATE(4097)] = 137978,
  [SMALL_STATE(4098)] = 138046,
  [SMALL_STATE(4099)] = 138094,
  [SMALL_STATE(4100)] = 138170,
  [SMALL_STATE(4101)] = 138246,
  [SMALL_STATE(4102)] = 138320,
  [SMALL_STATE(4103)] = 138396,
  [SMALL_STATE(4104)] = 138472,
  [SMALL_STATE(4105)] = 138548,
  [SMALL_STATE(4106)] = 138596,
  [SMALL_STATE(4107)] = 138676,
  [SMALL_STATE(4108)] = 138728,
  [SMALL_STATE(4109)] = 138804,
  [SMALL_STATE(4110)] = 138880,
  [SMALL_STATE(4111)] = 138960,
  [SMALL_STATE(4112)] = 139040,
  [SMALL_STATE(4113)] = 139118,
  [SMALL_STATE(4114)] = 139200,
  [SMALL_STATE(4115)] = 139280,
  [SMALL_STATE(4116)] = 139354,
  [SMALL_STATE(4117)] = 139404,
  [SMALL_STATE(4118)] = 139486,
  [SMALL_STATE(4119)] = 139564,
  [SMALL_STATE(4120)] = 139640,
  [SMALL_STATE(4121)] = 139720,
  [SMALL_STATE(4122)] = 139796,
  [SMALL_STATE(4123)] = 139872,
  [SMALL_STATE(4124)] = 139920,
  [SMALL_STATE(4125)] = 139996,
  [SMALL_STATE(4126)] = 140078,
  [SMALL_STATE(4127)] = 140158,
  [SMALL_STATE(4128)] = 140226,
  [SMALL_STATE(4129)] = 140293,
  [SMALL_STATE(4130)] = 140360,
  [SMALL_STATE(4131)] = 140411,
  [SMALL_STATE(4132)] = 140486,
  [SMALL_STATE(4133)] = 140553,
  [SMALL_STATE(4134)] = 140626,
  [SMALL_STATE(4135)] = 140673,
  [SMALL_STATE(4136)] = 140718,
  [SMALL_STATE(4137)] = 140793,
  [SMALL_STATE(4138)] = 140870,
  [SMALL_STATE(4139)] = 140945,
  [SMALL_STATE(4140)] = 141020,
  [SMALL_STATE(4141)] = 141095,
  [SMALL_STATE(4142)] = 141142,
  [SMALL_STATE(4143)] = 141219,
  [SMALL_STATE(4144)] = 141294,
  [SMALL_STATE(4145)] = 141373,
  [SMALL_STATE(4146)] = 141450,
  [SMALL_STATE(4147)] = 141523,
  [SMALL_STATE(4148)] = 141602,
  [SMALL_STATE(4149)] = 141653,
  [SMALL_STATE(4150)] = 141732,
  [SMALL_STATE(4151)] = 141811,
  [SMALL_STATE(4152)] = 141886,
  [SMALL_STATE(4153)] = 141961,
  [SMALL_STATE(4154)] = 142008,
  [SMALL_STATE(4155)] = 142055,
  [SMALL_STATE(4156)] = 142104,
  [SMALL_STATE(4157)] = 142151,
  [SMALL_STATE(4158)] = 142198,
  [SMALL_STATE(4159)] = 142253,
  [SMALL_STATE(4160)] = 142306,
  [SMALL_STATE(4161)] = 142365,
  [SMALL_STATE(4162)] = 142434,
  [SMALL_STATE(4163)] = 142501,
  [SMALL_STATE(4164)] = 142552,
  [SMALL_STATE(4165)] = 142619,
  [SMALL_STATE(4166)] = 142666,
  [SMALL_STATE(4167)] = 142715,
  [SMALL_STATE(4168)] = 142762,
  [SMALL_STATE(4169)] = 142809,
  [SMALL_STATE(4170)] = 142864,
  [SMALL_STATE(4171)] = 142937,
  [SMALL_STATE(4172)] = 142990,
  [SMALL_STATE(4173)] = 143059,
  [SMALL_STATE(4174)] = 143134,
  [SMALL_STATE(4175)] = 143209,
  [SMALL_STATE(4176)] = 143256,
  [SMALL_STATE(4177)] = 143329,
  [SMALL_STATE(4178)] = 143404,
  [SMALL_STATE(4179)] = 143477,
  [SMALL_STATE(4180)] = 143524,
  [SMALL_STATE(4181)] = 143599,
  [SMALL_STATE(4182)] = 143674,
  [SMALL_STATE(4183)] = 143723,
  [SMALL_STATE(4184)] = 143798,
  [SMALL_STATE(4185)] = 143847,
  [SMALL_STATE(4186)] = 143924,
  [SMALL_STATE(4187)] = 143999,
  [SMALL_STATE(4188)] = 144074,
  [SMALL_STATE(4189)] = 144149,
  [SMALL_STATE(4190)] = 144226,
  [SMALL_STATE(4191)] = 144303,
  [SMALL_STATE(4192)] = 144378,
  [SMALL_STATE(4193)] = 144425,
  [SMALL_STATE(4194)] = 144472,
  [SMALL_STATE(4195)] = 144521,
  [SMALL_STATE(4196)] = 144598,
  [SMALL_STATE(4197)] = 144643,
  [SMALL_STATE(4198)] = 144716,
  [SMALL_STATE(4199)] = 144793,
  [SMALL_STATE(4200)] = 144870,
  [SMALL_STATE(4201)] = 144925,
  [SMALL_STATE(4202)] = 144978,
  [SMALL_STATE(4203)] = 145037,
  [SMALL_STATE(4204)] = 145106,
  [SMALL_STATE(4205)] = 145173,
  [SMALL_STATE(4206)] = 145224,
  [SMALL_STATE(4207)] = 145299,
  [SMALL_STATE(4208)] = 145378,
  [SMALL_STATE(4209)] = 145425,
  [SMALL_STATE(4210)] = 145504,
  [SMALL_STATE(4211)] = 145579,
  [SMALL_STATE(4212)] = 145654,
  [SMALL_STATE(4213)] = 145729,
  [SMALL_STATE(4214)] = 145804,
  [SMALL_STATE(4215)] = 145863,
  [SMALL_STATE(4216)] = 145909,
  [SMALL_STATE(4217)] = 145981,
  [SMALL_STATE(4218)] = 146031,
  [SMALL_STATE(4219)] = 146083,
  [SMALL_STATE(4220)] = 146159,
  [SMALL_STATE(4221)] = 146233,
  [SMALL_STATE(4222)] = 146307,
  [SMALL_STATE(4223)] = 146383,
  [SMALL_STATE(4224)] = 146459,
  [SMALL_STATE(4225)] = 146517,
  [SMALL_STATE(4226)] = 146591,
  [SMALL_STATE(4227)] = 146649,
  [SMALL_STATE(4228)] = 146697,
  [SMALL_STATE(4229)] = 146765,
  [SMALL_STATE(4230)] = 146811,
  [SMALL_STATE(4231)] = 146859,
  [SMALL_STATE(4232)] = 146933,
  [SMALL_STATE(4233)] = 146987,
  [SMALL_STATE(4234)] = 147059,
  [SMALL_STATE(4235)] = 147125,
  [SMALL_STATE(4236)] = 147171,
  [SMALL_STATE(4237)] = 147217,
  [SMALL_STATE(4238)] = 147291,
  [SMALL_STATE(4239)] = 147359,
  [SMALL_STATE(4240)] = 147405,
  [SMALL_STATE(4241)] = 147479,
  [SMALL_STATE(4242)] = 147551,
  [SMALL_STATE(4243)] = 147623,
  [SMALL_STATE(4244)] = 147697,
  [SMALL_STATE(4245)] = 147771,
  [SMALL_STATE(4246)] = 147845,
  [SMALL_STATE(4247)] = 147921,
  [SMALL_STATE(4248)] = 147997,
  [SMALL_STATE(4249)] = 148073,
  [SMALL_STATE(4250)] = 148147,
  [SMALL_STATE(4251)] = 148213,
  [SMALL_STATE(4252)] = 148279,
  [SMALL_STATE(4253)] = 148329,
  [SMALL_STATE(4254)] = 148403,
  [SMALL_STATE(4255)] = 148455,
  [SMALL_STATE(4256)] = 148503,
  [SMALL_STATE(4257)] = 148549,
  [SMALL_STATE(4258)] = 148623,
  [SMALL_STATE(4259)] = 148669,
  [SMALL_STATE(4260)] = 148723,
  [SMALL_STATE(4261)] = 148797,
  [SMALL_STATE(4262)] = 148863,
  [SMALL_STATE(4263)] = 148909,
  [SMALL_STATE(4264)] = 148966,
  [SMALL_STATE(4265)] = 149041,
  [SMALL_STATE(4266)] = 149116,
  [SMALL_STATE(4267)] = 149191,
  [SMALL_STATE(4268)] = 149266,
  [SMALL_STATE(4269)] = 149341,
  [SMALL_STATE(4270)] = 149416,
  [SMALL_STATE(4271)] = 149491,
  [SMALL_STATE(4272)] = 149566,
  [SMALL_STATE(4273)] = 149641,
  [SMALL_STATE(4274)] = 149716,
  [SMALL_STATE(4275)] = 149791,
  [SMALL_STATE(4276)] = 149866,
  [SMALL_STATE(4277)] = 149941,
  [SMALL_STATE(4278)] = 150016,
  [SMALL_STATE(4279)] = 150059,
  [SMALL_STATE(4280)] = 150134,
  [SMALL_STATE(4281)] = 150207,
  [SMALL_STATE(4282)] = 150282,
  [SMALL_STATE(4283)] = 150357,
  [SMALL_STATE(4284)] = 150430,
  [SMALL_STATE(4285)] = 150503,
  [SMALL_STATE(4286)] = 150578,
  [SMALL_STATE(4287)] = 150653,
  [SMALL_STATE(4288)] = 150728,
  [SMALL_STATE(4289)] = 150803,
  [SMALL_STATE(4290)] = 150848,
  [SMALL_STATE(4291)] = 150919,
  [SMALL_STATE(4292)] = 150994,
  [SMALL_STATE(4293)] = 151069,
  [SMALL_STATE(4294)] = 151144,
  [SMALL_STATE(4295)] = 151219,
  [SMALL_STATE(4296)] = 151294,
  [SMALL_STATE(4297)] = 151369,
  [SMALL_STATE(4298)] = 151444,
  [SMALL_STATE(4299)] = 151519,
  [SMALL_STATE(4300)] = 151594,
  [SMALL_STATE(4301)] = 151669,
  [SMALL_STATE(4302)] = 151744,
  [SMALL_STATE(4303)] = 151819,
  [SMALL_STATE(4304)] = 151894,
  [SMALL_STATE(4305)] = 151969,
  [SMALL_STATE(4306)] = 152044,
  [SMALL_STATE(4307)] = 152119,
  [SMALL_STATE(4308)] = 152194,
  [SMALL_STATE(4309)] = 152269,
  [SMALL_STATE(4310)] = 152344,
  [SMALL_STATE(4311)] = 152407,
  [SMALL_STATE(4312)] = 152482,
  [SMALL_STATE(4313)] = 152529,
  [SMALL_STATE(4314)] = 152604,
  [SMALL_STATE(4315)] = 152649,
  [SMALL_STATE(4316)] = 152694,
  [SMALL_STATE(4317)] = 152747,
  [SMALL_STATE(4318)] = 152818,
  [SMALL_STATE(4319)] = 152869,
  [SMALL_STATE(4320)] = 152942,
  [SMALL_STATE(4321)] = 153009,
  [SMALL_STATE(4322)] = 153074,
  [SMALL_STATE(4323)] = 153123,
  [SMALL_STATE(4324)] = 153188,
  [SMALL_STATE(4325)] = 153263,
  [SMALL_STATE(4326)] = 153308,
  [SMALL_STATE(4327)] = 153381,
  [SMALL_STATE(4328)] = 153454,
  [SMALL_STATE(4329)] = 153529,
  [SMALL_STATE(4330)] = 153604,
  [SMALL_STATE(4331)] = 153679,
  [SMALL_STATE(4332)] = 153754,
  [SMALL_STATE(4333)] = 153817,
  [SMALL_STATE(4334)] = 153892,
  [SMALL_STATE(4335)] = 153967,
  [SMALL_STATE(4336)] = 154042,
  [SMALL_STATE(4337)] = 154117,
  [SMALL_STATE(4338)] = 154192,
  [SMALL_STATE(4339)] = 154267,
  [SMALL_STATE(4340)] = 154342,
  [SMALL_STATE(4341)] = 154417,
  [SMALL_STATE(4342)] = 154492,
  [SMALL_STATE(4343)] = 154567,
  [SMALL_STATE(4344)] = 154642,
  [SMALL_STATE(4345)] = 154717,
  [SMALL_STATE(4346)] = 154792,
  [SMALL_STATE(4347)] = 154867,
  [SMALL_STATE(4348)] = 154942,
  [SMALL_STATE(4349)] = 155017,
  [SMALL_STATE(4350)] = 155092,
  [SMALL_STATE(4351)] = 155167,
  [SMALL_STATE(4352)] = 155242,
  [SMALL_STATE(4353)] = 155317,
  [SMALL_STATE(4354)] = 155392,
  [SMALL_STATE(4355)] = 155467,
  [SMALL_STATE(4356)] = 155542,
  [SMALL_STATE(4357)] = 155617,
  [SMALL_STATE(4358)] = 155657,
  [SMALL_STATE(4359)] = 155697,
  [SMALL_STATE(4360)] = 155737,
  [SMALL_STATE(4361)] = 155777,
  [SMALL_STATE(4362)] = 155817,
  [SMALL_STATE(4363)] = 155859,
  [SMALL_STATE(4364)] = 155899,
  [SMALL_STATE(4365)] = 155941,
  [SMALL_STATE(4366)] = 155981,
  [SMALL_STATE(4367)] = 156021,
  [SMALL_STATE(4368)] = 156061,
  [SMALL_STATE(4369)] = 156101,
  [SMALL_STATE(4370)] = 156141,
  [SMALL_STATE(4371)] = 156181,
  [SMALL_STATE(4372)] = 156221,
  [SMALL_STATE(4373)] = 156261,
  [SMALL_STATE(4374)] = 156301,
  [SMALL_STATE(4375)] = 156341,
  [SMALL_STATE(4376)] = 156387,
  [SMALL_STATE(4377)] = 156427,
  [SMALL_STATE(4378)] = 156467,
  [SMALL_STATE(4379)] = 156507,
  [SMALL_STATE(4380)] = 156547,
  [SMALL_STATE(4381)] = 156588,
  [SMALL_STATE(4382)] = 156629,
  [SMALL_STATE(4383)] = 156670,
  [SMALL_STATE(4384)] = 156711,
  [SMALL_STATE(4385)] = 156752,
  [SMALL_STATE(4386)] = 156793,
  [SMALL_STATE(4387)] = 156834,
  [SMALL_STATE(4388)] = 156875,
  [SMALL_STATE(4389)] = 156916,
  [SMALL_STATE(4390)] = 156957,
  [SMALL_STATE(4391)] = 156998,
  [SMALL_STATE(4392)] = 157039,
  [SMALL_STATE(4393)] = 157080,
  [SMALL_STATE(4394)] = 157121,
  [SMALL_STATE(4395)] = 157162,
  [SMALL_STATE(4396)] = 157203,
  [SMALL_STATE(4397)] = 157244,
  [SMALL_STATE(4398)] = 157285,
  [SMALL_STATE(4399)] = 157326,
  [SMALL_STATE(4400)] = 157367,
  [SMALL_STATE(4401)] = 157408,
  [SMALL_STATE(4402)] = 157449,
  [SMALL_STATE(4403)] = 157490,
  [SMALL_STATE(4404)] = 157531,
  [SMALL_STATE(4405)] = 157572,
  [SMALL_STATE(4406)] = 157613,
  [SMALL_STATE(4407)] = 157654,
  [SMALL_STATE(4408)] = 157695,
  [SMALL_STATE(4409)] = 157736,
  [SMALL_STATE(4410)] = 157777,
  [SMALL_STATE(4411)] = 157818,
  [SMALL_STATE(4412)] = 157859,
  [SMALL_STATE(4413)] = 157900,
  [SMALL_STATE(4414)] = 157941,
  [SMALL_STATE(4415)] = 157982,
  [SMALL_STATE(4416)] = 158023,
  [SMALL_STATE(4417)] = 158064,
  [SMALL_STATE(4418)] = 158105,
  [SMALL_STATE(4419)] = 158146,
  [SMALL_STATE(4420)] = 158187,
  [SMALL_STATE(4421)] = 158228,
  [SMALL_STATE(4422)] = 158269,
  [SMALL_STATE(4423)] = 158310,
  [SMALL_STATE(4424)] = 158351,
  [SMALL_STATE(4425)] = 158392,
  [SMALL_STATE(4426)] = 158433,
  [SMALL_STATE(4427)] = 158474,
  [SMALL_STATE(4428)] = 158515,
  [SMALL_STATE(4429)] = 158556,
  [SMALL_STATE(4430)] = 158597,
  [SMALL_STATE(4431)] = 158638,
  [SMALL_STATE(4432)] = 158679,
  [SMALL_STATE(4433)] = 158720,
  [SMALL_STATE(4434)] = 158758,
  [SMALL_STATE(4435)] = 158800,
  [SMALL_STATE(4436)] = 158838,
  [SMALL_STATE(4437)] = 158876,
  [SMALL_STATE(4438)] = 158914,
  [SMALL_STATE(4439)] = 158951,
  [SMALL_STATE(4440)] = 158988,
  [SMALL_STATE(4441)] = 159027,
  [SMALL_STATE(4442)] = 159066,
  [SMALL_STATE(4443)] = 159111,
  [SMALL_STATE(4444)] = 159150,
  [SMALL_STATE(4445)] = 159189,
  [SMALL_STATE(4446)] = 159228,
  [SMALL_STATE(4447)] = 159265,
  [SMALL_STATE(4448)] = 159302,
  [SMALL_STATE(4449)] = 159343,
  [SMALL_STATE(4450)] = 159380,
  [SMALL_STATE(4451)] = 159417,
  [SMALL_STATE(4452)] = 159454,
  [SMALL_STATE(4453)] = 159491,
  [SMALL_STATE(4454)] = 159528,
  [SMALL_STATE(4455)] = 159565,
  [SMALL_STATE(4456)] = 159604,
  [SMALL_STATE(4457)] = 159641,
  [SMALL_STATE(4458)] = 159678,
  [SMALL_STATE(4459)] = 159737,
  [SMALL_STATE(4460)] = 159796,
  [SMALL_STATE(4461)] = 159835,
  [SMALL_STATE(4462)] = 159878,
  [SMALL_STATE(4463)] = 159915,
  [SMALL_STATE(4464)] = 159952,
  [SMALL_STATE(4465)] = 159995,
  [SMALL_STATE(4466)] = 160032,
  [SMALL_STATE(4467)] = 160077,
  [SMALL_STATE(4468)] = 160114,
  [SMALL_STATE(4469)] = 160151,
  [SMALL_STATE(4470)] = 160188,
  [SMALL_STATE(4471)] = 160228,
  [SMALL_STATE(4472)] = 160266,
  [SMALL_STATE(4473)] = 160302,
  [SMALL_STATE(4474)] = 160340,
  [SMALL_STATE(4475)] = 160380,
  [SMALL_STATE(4476)] = 160418,
  [SMALL_STATE(4477)] = 160454,
  [SMALL_STATE(4478)] = 160492,
  [SMALL_STATE(4479)] = 160530,
  [SMALL_STATE(4480)] = 160568,
  [SMALL_STATE(4481)] = 160604,
  [SMALL_STATE(4482)] = 160639,
  [SMALL_STATE(4483)] = 160674,
  [SMALL_STATE(4484)] = 160709,
  [SMALL_STATE(4485)] = 160744,
  [SMALL_STATE(4486)] = 160779,
  [SMALL_STATE(4487)] = 160814,
  [SMALL_STATE(4488)] = 160849,
  [SMALL_STATE(4489)] = 160884,
  [SMALL_STATE(4490)] = 160919,
  [SMALL_STATE(4491)] = 160954,
  [SMALL_STATE(4492)] = 160989,
  [SMALL_STATE(4493)] = 161024,
  [SMALL_STATE(4494)] = 161059,
  [SMALL_STATE(4495)] = 161094,
  [SMALL_STATE(4496)] = 161151,
  [SMALL_STATE(4497)] = 161186,
  [SMALL_STATE(4498)] = 161221,
  [SMALL_STATE(4499)] = 161256,
  [SMALL_STATE(4500)] = 161291,
  [SMALL_STATE(4501)] = 161326,
  [SMALL_STATE(4502)] = 161361,
  [SMALL_STATE(4503)] = 161396,
  [SMALL_STATE(4504)] = 161431,
  [SMALL_STATE(4505)] = 161488,
  [SMALL_STATE(4506)] = 161545,
  [SMALL_STATE(4507)] = 161580,
  [SMALL_STATE(4508)] = 161615,
  [SMALL_STATE(4509)] = 161650,
  [SMALL_STATE(4510)] = 161685,
  [SMALL_STATE(4511)] = 161720,
  [SMALL_STATE(4512)] = 161755,
  [SMALL_STATE(4513)] = 161790,
  [SMALL_STATE(4514)] = 161825,
  [SMALL_STATE(4515)] = 161882,
  [SMALL_STATE(4516)] = 161917,
  [SMALL_STATE(4517)] = 161952,
  [SMALL_STATE(4518)] = 161993,
  [SMALL_STATE(4519)] = 162028,
  [SMALL_STATE(4520)] = 162063,
  [SMALL_STATE(4521)] = 162099,
  [SMALL_STATE(4522)] = 162135,
  [SMALL_STATE(4523)] = 162173,
  [SMALL_STATE(4524)] = 162211,
  [SMALL_STATE(4525)] = 162249,
  [SMALL_STATE(4526)] = 162287,
  [SMALL_STATE(4527)] = 162323,
  [SMALL_STATE(4528)] = 162361,
  [SMALL_STATE(4529)] = 162399,
  [SMALL_STATE(4530)] = 162437,
  [SMALL_STATE(4531)] = 162475,
  [SMALL_STATE(4532)] = 162513,
  [SMALL_STATE(4533)] = 162551,
  [SMALL_STATE(4534)] = 162589,
  [SMALL_STATE(4535)] = 162648,
  [SMALL_STATE(4536)] = 162683,
  [SMALL_STATE(4537)] = 162742,
  [SMALL_STATE(4538)] = 162801,
  [SMALL_STATE(4539)] = 162836,
  [SMALL_STATE(4540)] = 162895,
  [SMALL_STATE(4541)] = 162954,
  [SMALL_STATE(4542)] = 163013,
  [SMALL_STATE(4543)] = 163072,
  [SMALL_STATE(4544)] = 163131,
  [SMALL_STATE(4545)] = 163190,
  [SMALL_STATE(4546)] = 163249,
  [SMALL_STATE(4547)] = 163308,
  [SMALL_STATE(4548)] = 163367,
  [SMALL_STATE(4549)] = 163426,
  [SMALL_STATE(4550)] = 163485,
  [SMALL_STATE(4551)] = 163544,
  [SMALL_STATE(4552)] = 163603,
  [SMALL_STATE(4553)] = 163662,
  [SMALL_STATE(4554)] = 163721,
  [SMALL_STATE(4555)] = 163780,
  [SMALL_STATE(4556)] = 163839,
  [SMALL_STATE(4557)] = 163898,
  [SMALL_STATE(4558)] = 163957,
  [SMALL_STATE(4559)] = 164016,
  [SMALL_STATE(4560)] = 164075,
  [SMALL_STATE(4561)] = 164129,
  [SMALL_STATE(4562)] = 164165,
  [SMALL_STATE(4563)] = 164219,
  [SMALL_STATE(4564)] = 164253,
  [SMALL_STATE(4565)] = 164307,
  [SMALL_STATE(4566)] = 164343,
  [SMALL_STATE(4567)] = 164397,
  [SMALL_STATE(4568)] = 164429,
  [SMALL_STATE(4569)] = 164483,
  [SMALL_STATE(4570)] = 164537,
  [SMALL_STATE(4571)] = 164591,
  [SMALL_STATE(4572)] = 164625,
  [SMALL_STATE(4573)] = 164680,
  [SMALL_STATE(4574)] = 164711,
  [SMALL_STATE(4575)] = 164744,
  [SMALL_STATE(4576)] = 164799,
  [SMALL_STATE(4577)] = 164832,
  [SMALL_STATE(4578)] = 164863,
  [SMALL_STATE(4579)] = 164898,
  [SMALL_STATE(4580)] = 164929,
  [SMALL_STATE(4581)] = 164960,
  [SMALL_STATE(4582)] = 164995,
  [SMALL_STATE(4583)] = 165026,
  [SMALL_STATE(4584)] = 165057,
  [SMALL_STATE(4585)] = 165092,
  [SMALL_STATE(4586)] = 165125,
  [SMALL_STATE(4587)] = 165158,
  [SMALL_STATE(4588)] = 165190,
  [SMALL_STATE(4589)] = 165242,
  [SMALL_STATE(4590)] = 165274,
  [SMALL_STATE(4591)] = 165326,
  [SMALL_STATE(4592)] = 165378,
  [SMALL_STATE(4593)] = 165410,
  [SMALL_STATE(4594)] = 165444,
  [SMALL_STATE(4595)] = 165476,
  [SMALL_STATE(4596)] = 165508,
  [SMALL_STATE(4597)] = 165540,
  [SMALL_STATE(4598)] = 165572,
  [SMALL_STATE(4599)] = 165604,
  [SMALL_STATE(4600)] = 165636,
  [SMALL_STATE(4601)] = 165688,
  [SMALL_STATE(4602)] = 165720,
  [SMALL_STATE(4603)] = 165779,
  [SMALL_STATE(4604)] = 165838,
  [SMALL_STATE(4605)] = 165873,
  [SMALL_STATE(4606)] = 165904,
  [SMALL_STATE(4607)] = 165937,
  [SMALL_STATE(4608)] = 165968,
  [SMALL_STATE(4609)] = 166027,
  [SMALL_STATE(4610)] = 166086,
  [SMALL_STATE(4611)] = 166145,
  [SMALL_STATE(4612)] = 166194,
  [SMALL_STATE(4613)] = 166225,
  [SMALL_STATE(4614)] = 166276,
  [SMALL_STATE(4615)] = 166327,
  [SMALL_STATE(4616)] = 166386,
  [SMALL_STATE(4617)] = 166435,
  [SMALL_STATE(4618)] = 166484,
  [SMALL_STATE(4619)] = 166517,
  [SMALL_STATE(4620)] = 166563,
  [SMALL_STATE(4621)] = 166609,
  [SMALL_STATE(4622)] = 166639,
  [SMALL_STATE(4623)] = 166685,
  [SMALL_STATE(4624)] = 166731,
  [SMALL_STATE(4625)] = 166765,
  [SMALL_STATE(4626)] = 166807,
  [SMALL_STATE(4627)] = 166849,
  [SMALL_STATE(4628)] = 166877,
  [SMALL_STATE(4629)] = 166908,
  [SMALL_STATE(4630)] = 166941,
  [SMALL_STATE(4631)] = 166972,
  [SMALL_STATE(4632)] = 167001,
  [SMALL_STATE(4633)] = 167030,
  [SMALL_STATE(4634)] = 167061,
  [SMALL_STATE(4635)] = 167092,
  [SMALL_STATE(4636)] = 167123,
  [SMALL_STATE(4637)] = 167154,
  [SMALL_STATE(4638)] = 167187,
  [SMALL_STATE(4639)] = 167218,
  [SMALL_STATE(4640)] = 167247,
  [SMALL_STATE(4641)] = 167278,
  [SMALL_STATE(4642)] = 167309,
  [SMALL_STATE(4643)] = 167340,
  [SMALL_STATE(4644)] = 167371,
  [SMALL_STATE(4645)] = 167402,
  [SMALL_STATE(4646)] = 167430,
  [SMALL_STATE(4647)] = 167456,
  [SMALL_STATE(4648)] = 167482,
  [SMALL_STATE(4649)] = 167508,
  [SMALL_STATE(4650)] = 167534,
  [SMALL_STATE(4651)] = 167560,
  [SMALL_STATE(4652)] = 167598,
  [SMALL_STATE(4653)] = 167624,
  [SMALL_STATE(4654)] = 167650,
  [SMALL_STATE(4655)] = 167676,
  [SMALL_STATE(4656)] = 167702,
  [SMALL_STATE(4657)] = 167734,
  [SMALL_STATE(4658)] = 167760,
  [SMALL_STATE(4659)] = 167786,
  [SMALL_STATE(4660)] = 167814,
  [SMALL_STATE(4661)] = 167842,
  [SMALL_STATE(4662)] = 167868,
  [SMALL_STATE(4663)] = 167894,
  [SMALL_STATE(4664)] = 167922,
  [SMALL_STATE(4665)] = 167952,
  [SMALL_STATE(4666)] = 167978,
  [SMALL_STATE(4667)] = 168006,
  [SMALL_STATE(4668)] = 168034,
  [SMALL_STATE(4669)] = 168072,
  [SMALL_STATE(4670)] = 168098,
  [SMALL_STATE(4671)] = 168124,
  [SMALL_STATE(4672)] = 168150,
  [SMALL_STATE(4673)] = 168176,
  [SMALL_STATE(4674)] = 168202,
  [SMALL_STATE(4675)] = 168228,
  [SMALL_STATE(4676)] = 168255,
  [SMALL_STATE(4677)] = 168282,
  [SMALL_STATE(4678)] = 168309,
  [SMALL_STATE(4679)] = 168336,
  [SMALL_STATE(4680)] = 168363,
  [SMALL_STATE(4681)] = 168392,
  [SMALL_STATE(4682)] = 168419,
  [SMALL_STATE(4683)] = 168446,
  [SMALL_STATE(4684)] = 168473,
  [SMALL_STATE(4685)] = 168500,
  [SMALL_STATE(4686)] = 168527,
  [SMALL_STATE(4687)] = 168554,
  [SMALL_STATE(4688)] = 168581,
  [SMALL_STATE(4689)] = 168608,
  [SMALL_STATE(4690)] = 168635,
  [SMALL_STATE(4691)] = 168662,
  [SMALL_STATE(4692)] = 168689,
  [SMALL_STATE(4693)] = 168716,
  [SMALL_STATE(4694)] = 168747,
  [SMALL_STATE(4695)] = 168774,
  [SMALL_STATE(4696)] = 168801,
  [SMALL_STATE(4697)] = 168828,
  [SMALL_STATE(4698)] = 168857,
  [SMALL_STATE(4699)] = 168884,
  [SMALL_STATE(4700)] = 168911,
  [SMALL_STATE(4701)] = 168938,
  [SMALL_STATE(4702)] = 168965,
  [SMALL_STATE(4703)] = 168992,
  [SMALL_STATE(4704)] = 169019,
  [SMALL_STATE(4705)] = 169046,
  [SMALL_STATE(4706)] = 169073,
  [SMALL_STATE(4707)] = 169100,
  [SMALL_STATE(4708)] = 169127,
  [SMALL_STATE(4709)] = 169164,
  [SMALL_STATE(4710)] = 169191,
  [SMALL_STATE(4711)] = 169228,
  [SMALL_STATE(4712)] = 169255,
  [SMALL_STATE(4713)] = 169282,
  [SMALL_STATE(4714)] = 169309,
  [SMALL_STATE(4715)] = 169336,
  [SMALL_STATE(4716)] = 169363,
  [SMALL_STATE(4717)] = 169390,
  [SMALL_STATE(4718)] = 169417,
  [SMALL_STATE(4719)] = 169444,
  [SMALL_STATE(4720)] = 169471,
  [SMALL_STATE(4721)] = 169498,
  [SMALL_STATE(4722)] = 169525,
  [SMALL_STATE(4723)] = 169552,
  [SMALL_STATE(4724)] = 169579,
  [SMALL_STATE(4725)] = 169606,
  [SMALL_STATE(4726)] = 169633,
  [SMALL_STATE(4727)] = 169660,
  [SMALL_STATE(4728)] = 169687,
  [SMALL_STATE(4729)] = 169716,
  [SMALL_STATE(4730)] = 169743,
  [SMALL_STATE(4731)] = 169770,
  [SMALL_STATE(4732)] = 169799,
  [SMALL_STATE(4733)] = 169828,
  [SMALL_STATE(4734)] = 169855,
  [SMALL_STATE(4735)] = 169882,
  [SMALL_STATE(4736)] = 169909,
  [SMALL_STATE(4737)] = 169936,
  [SMALL_STATE(4738)] = 169963,
  [SMALL_STATE(4739)] = 169990,
  [SMALL_STATE(4740)] = 170017,
  [SMALL_STATE(4741)] = 170044,
  [SMALL_STATE(4742)] = 170072,
  [SMALL_STATE(4743)] = 170102,
  [SMALL_STATE(4744)] = 170130,
  [SMALL_STATE(4745)] = 170160,
  [SMALL_STATE(4746)] = 170192,
  [SMALL_STATE(4747)] = 170220,
  [SMALL_STATE(4748)] = 170246,
  [SMALL_STATE(4749)] = 170272,
  [SMALL_STATE(4750)] = 170300,
  [SMALL_STATE(4751)] = 170328,
  [SMALL_STATE(4752)] = 170354,
  [SMALL_STATE(4753)] = 170379,
  [SMALL_STATE(4754)] = 170414,
  [SMALL_STATE(4755)] = 170437,
  [SMALL_STATE(4756)] = 170460,
  [SMALL_STATE(4757)] = 170487,
  [SMALL_STATE(4758)] = 170510,
  [SMALL_STATE(4759)] = 170535,
  [SMALL_STATE(4760)] = 170562,
  [SMALL_STATE(4761)] = 170585,
  [SMALL_STATE(4762)] = 170612,
  [SMALL_STATE(4763)] = 170635,
  [SMALL_STATE(4764)] = 170658,
  [SMALL_STATE(4765)] = 170681,
  [SMALL_STATE(4766)] = 170706,
  [SMALL_STATE(4767)] = 170729,
  [SMALL_STATE(4768)] = 170752,
  [SMALL_STATE(4769)] = 170779,
  [SMALL_STATE(4770)] = 170802,
  [SMALL_STATE(4771)] = 170827,
  [SMALL_STATE(4772)] = 170852,
  [SMALL_STATE(4773)] = 170877,
  [SMALL_STATE(4774)] = 170900,
  [SMALL_STATE(4775)] = 170935,
  [SMALL_STATE(4776)] = 170958,
  [SMALL_STATE(4777)] = 170983,
  [SMALL_STATE(4778)] = 171008,
  [SMALL_STATE(4779)] = 171033,
  [SMALL_STATE(4780)] = 171056,
  [SMALL_STATE(4781)] = 171083,
  [SMALL_STATE(4782)] = 171108,
  [SMALL_STATE(4783)] = 171135,
  [SMALL_STATE(4784)] = 171158,
  [SMALL_STATE(4785)] = 171185,
  [SMALL_STATE(4786)] = 171220,
  [SMALL_STATE(4787)] = 171243,
  [SMALL_STATE(4788)] = 171266,
  [SMALL_STATE(4789)] = 171293,
  [SMALL_STATE(4790)] = 171316,
  [SMALL_STATE(4791)] = 171339,
  [SMALL_STATE(4792)] = 171362,
  [SMALL_STATE(4793)] = 171387,
  [SMALL_STATE(4794)] = 171414,
  [SMALL_STATE(4795)] = 171439,
  [SMALL_STATE(4796)] = 171462,
  [SMALL_STATE(4797)] = 171487,
  [SMALL_STATE(4798)] = 171512,
  [SMALL_STATE(4799)] = 171537,
  [SMALL_STATE(4800)] = 171564,
  [SMALL_STATE(4801)] = 171587,
  [SMALL_STATE(4802)] = 171612,
  [SMALL_STATE(4803)] = 171635,
  [SMALL_STATE(4804)] = 171658,
  [SMALL_STATE(4805)] = 171681,
  [SMALL_STATE(4806)] = 171704,
  [SMALL_STATE(4807)] = 171731,
  [SMALL_STATE(4808)] = 171760,
  [SMALL_STATE(4809)] = 171787,
  [SMALL_STATE(4810)] = 171814,
  [SMALL_STATE(4811)] = 171837,
  [SMALL_STATE(4812)] = 171872,
  [SMALL_STATE(4813)] = 171899,
  [SMALL_STATE(4814)] = 171922,
  [SMALL_STATE(4815)] = 171945,
  [SMALL_STATE(4816)] = 171968,
  [SMALL_STATE(4817)] = 171991,
  [SMALL_STATE(4818)] = 172020,
  [SMALL_STATE(4819)] = 172047,
  [SMALL_STATE(4820)] = 172070,
  [SMALL_STATE(4821)] = 172095,
  [SMALL_STATE(4822)] = 172120,
  [SMALL_STATE(4823)] = 172155,
  [SMALL_STATE(4824)] = 172184,
  [SMALL_STATE(4825)] = 172207,
  [SMALL_STATE(4826)] = 172232,
  [SMALL_STATE(4827)] = 172255,
  [SMALL_STATE(4828)] = 172293,
  [SMALL_STATE(4829)] = 172329,
  [SMALL_STATE(4830)] = 172353,
  [SMALL_STATE(4831)] = 172377,
  [SMALL_STATE(4832)] = 172399,
  [SMALL_STATE(4833)] = 172429,
  [SMALL_STATE(4834)] = 172451,
  [SMALL_STATE(4835)] = 172489,
  [SMALL_STATE(4836)] = 172513,
  [SMALL_STATE(4837)] = 172547,
  [SMALL_STATE(4838)] = 172571,
  [SMALL_STATE(4839)] = 172609,
  [SMALL_STATE(4840)] = 172647,
  [SMALL_STATE(4841)] = 172669,
  [SMALL_STATE(4842)] = 172691,
  [SMALL_STATE(4843)] = 172713,
  [SMALL_STATE(4844)] = 172737,
  [SMALL_STATE(4845)] = 172759,
  [SMALL_STATE(4846)] = 172787,
  [SMALL_STATE(4847)] = 172809,
  [SMALL_STATE(4848)] = 172833,
  [SMALL_STATE(4849)] = 172855,
  [SMALL_STATE(4850)] = 172881,
  [SMALL_STATE(4851)] = 172909,
  [SMALL_STATE(4852)] = 172931,
  [SMALL_STATE(4853)] = 172953,
  [SMALL_STATE(4854)] = 172975,
  [SMALL_STATE(4855)] = 172997,
  [SMALL_STATE(4856)] = 173021,
  [SMALL_STATE(4857)] = 173043,
  [SMALL_STATE(4858)] = 173081,
  [SMALL_STATE(4859)] = 173105,
  [SMALL_STATE(4860)] = 173143,
  [SMALL_STATE(4861)] = 173181,
  [SMALL_STATE(4862)] = 173217,
  [SMALL_STATE(4863)] = 173243,
  [SMALL_STATE(4864)] = 173265,
  [SMALL_STATE(4865)] = 173291,
  [SMALL_STATE(4866)] = 173313,
  [SMALL_STATE(4867)] = 173335,
  [SMALL_STATE(4868)] = 173373,
  [SMALL_STATE(4869)] = 173409,
  [SMALL_STATE(4870)] = 173447,
  [SMALL_STATE(4871)] = 173485,
  [SMALL_STATE(4872)] = 173509,
  [SMALL_STATE(4873)] = 173533,
  [SMALL_STATE(4874)] = 173555,
  [SMALL_STATE(4875)] = 173593,
  [SMALL_STATE(4876)] = 173615,
  [SMALL_STATE(4877)] = 173653,
  [SMALL_STATE(4878)] = 173677,
  [SMALL_STATE(4879)] = 173701,
  [SMALL_STATE(4880)] = 173739,
  [SMALL_STATE(4881)] = 173763,
  [SMALL_STATE(4882)] = 173793,
  [SMALL_STATE(4883)] = 173815,
  [SMALL_STATE(4884)] = 173852,
  [SMALL_STATE(4885)] = 173889,
  [SMALL_STATE(4886)] = 173926,
  [SMALL_STATE(4887)] = 173951,
  [SMALL_STATE(4888)] = 173988,
  [SMALL_STATE(4889)] = 174025,
  [SMALL_STATE(4890)] = 174050,
  [SMALL_STATE(4891)] = 174087,
  [SMALL_STATE(4892)] = 174110,
  [SMALL_STATE(4893)] = 174147,
  [SMALL_STATE(4894)] = 174172,
  [SMALL_STATE(4895)] = 174195,
  [SMALL_STATE(4896)] = 174220,
  [SMALL_STATE(4897)] = 174245,
  [SMALL_STATE(4898)] = 174268,
  [SMALL_STATE(4899)] = 174293,
  [SMALL_STATE(4900)] = 174314,
  [SMALL_STATE(4901)] = 174351,
  [SMALL_STATE(4902)] = 174388,
  [SMALL_STATE(4903)] = 174409,
  [SMALL_STATE(4904)] = 174446,
  [SMALL_STATE(4905)] = 174471,
  [SMALL_STATE(4906)] = 174492,
  [SMALL_STATE(4907)] = 174529,
  [SMALL_STATE(4908)] = 174566,
  [SMALL_STATE(4909)] = 174603,
  [SMALL_STATE(4910)] = 174640,
  [SMALL_STATE(4911)] = 174677,
  [SMALL_STATE(4912)] = 174714,
  [SMALL_STATE(4913)] = 174737,
  [SMALL_STATE(4914)] = 174774,
  [SMALL_STATE(4915)] = 174811,
  [SMALL_STATE(4916)] = 174848,
  [SMALL_STATE(4917)] = 174873,
  [SMALL_STATE(4918)] = 174910,
  [SMALL_STATE(4919)] = 174933,
  [SMALL_STATE(4920)] = 174970,
  [SMALL_STATE(4921)] = 175007,
  [SMALL_STATE(4922)] = 175044,
  [SMALL_STATE(4923)] = 175081,
  [SMALL_STATE(4924)] = 175104,
  [SMALL_STATE(4925)] = 175127,
  [SMALL_STATE(4926)] = 175152,
  [SMALL_STATE(4927)] = 175173,
  [SMALL_STATE(4928)] = 175194,
  [SMALL_STATE(4929)] = 175231,
  [SMALL_STATE(4930)] = 175256,
  [SMALL_STATE(4931)] = 175281,
  [SMALL_STATE(4932)] = 175304,
  [SMALL_STATE(4933)] = 175341,
  [SMALL_STATE(4934)] = 175362,
  [SMALL_STATE(4935)] = 175387,
  [SMALL_STATE(4936)] = 175408,
  [SMALL_STATE(4937)] = 175433,
  [SMALL_STATE(4938)] = 175458,
  [SMALL_STATE(4939)] = 175482,
  [SMALL_STATE(4940)] = 175516,
  [SMALL_STATE(4941)] = 175538,
  [SMALL_STATE(4942)] = 175560,
  [SMALL_STATE(4943)] = 175580,
  [SMALL_STATE(4944)] = 175600,
  [SMALL_STATE(4945)] = 175630,
  [SMALL_STATE(4946)] = 175650,
  [SMALL_STATE(4947)] = 175670,
  [SMALL_STATE(4948)] = 175694,
  [SMALL_STATE(4949)] = 175714,
  [SMALL_STATE(4950)] = 175734,
  [SMALL_STATE(4951)] = 175766,
  [SMALL_STATE(4952)] = 175786,
  [SMALL_STATE(4953)] = 175806,
  [SMALL_STATE(4954)] = 175826,
  [SMALL_STATE(4955)] = 175858,
  [SMALL_STATE(4956)] = 175878,
  [SMALL_STATE(4957)] = 175900,
  [SMALL_STATE(4958)] = 175922,
  [SMALL_STATE(4959)] = 175950,
  [SMALL_STATE(4960)] = 175970,
  [SMALL_STATE(4961)] = 175990,
  [SMALL_STATE(4962)] = 176010,
  [SMALL_STATE(4963)] = 176030,
  [SMALL_STATE(4964)] = 176050,
  [SMALL_STATE(4965)] = 176074,
  [SMALL_STATE(4966)] = 176094,
  [SMALL_STATE(4967)] = 176114,
  [SMALL_STATE(4968)] = 176134,
  [SMALL_STATE(4969)] = 176154,
  [SMALL_STATE(4970)] = 176174,
  [SMALL_STATE(4971)] = 176194,
  [SMALL_STATE(4972)] = 176228,
  [SMALL_STATE(4973)] = 176252,
  [SMALL_STATE(4974)] = 176272,
  [SMALL_STATE(4975)] = 176292,
  [SMALL_STATE(4976)] = 176316,
  [SMALL_STATE(4977)] = 176336,
  [SMALL_STATE(4978)] = 176356,
  [SMALL_STATE(4979)] = 176378,
  [SMALL_STATE(4980)] = 176412,
  [SMALL_STATE(4981)] = 176446,
  [SMALL_STATE(4982)] = 176480,
  [SMALL_STATE(4983)] = 176514,
  [SMALL_STATE(4984)] = 176548,
  [SMALL_STATE(4985)] = 176582,
  [SMALL_STATE(4986)] = 176602,
  [SMALL_STATE(4987)] = 176636,
  [SMALL_STATE(4988)] = 176670,
  [SMALL_STATE(4989)] = 176690,
  [SMALL_STATE(4990)] = 176710,
  [SMALL_STATE(4991)] = 176730,
  [SMALL_STATE(4992)] = 176764,
  [SMALL_STATE(4993)] = 176798,
  [SMALL_STATE(4994)] = 176832,
  [SMALL_STATE(4995)] = 176852,
  [SMALL_STATE(4996)] = 176882,
  [SMALL_STATE(4997)] = 176906,
  [SMALL_STATE(4998)] = 176926,
  [SMALL_STATE(4999)] = 176954,
  [SMALL_STATE(5000)] = 176988,
  [SMALL_STATE(5001)] = 177022,
  [SMALL_STATE(5002)] = 177052,
  [SMALL_STATE(5003)] = 177072,
  [SMALL_STATE(5004)] = 177092,
  [SMALL_STATE(5005)] = 177126,
  [SMALL_STATE(5006)] = 177146,
  [SMALL_STATE(5007)] = 177168,
  [SMALL_STATE(5008)] = 177192,
  [SMALL_STATE(5009)] = 177226,
  [SMALL_STATE(5010)] = 177246,
  [SMALL_STATE(5011)] = 177266,
  [SMALL_STATE(5012)] = 177286,
  [SMALL_STATE(5013)] = 177314,
  [SMALL_STATE(5014)] = 177342,
  [SMALL_STATE(5015)] = 177366,
  [SMALL_STATE(5016)] = 177386,
  [SMALL_STATE(5017)] = 177408,
  [SMALL_STATE(5018)] = 177428,
  [SMALL_STATE(5019)] = 177448,
  [SMALL_STATE(5020)] = 177468,
  [SMALL_STATE(5021)] = 177488,
  [SMALL_STATE(5022)] = 177510,
  [SMALL_STATE(5023)] = 177532,
  [SMALL_STATE(5024)] = 177552,
  [SMALL_STATE(5025)] = 177572,
  [SMALL_STATE(5026)] = 177606,
  [SMALL_STATE(5027)] = 177640,
  [SMALL_STATE(5028)] = 177674,
  [SMALL_STATE(5029)] = 177698,
  [SMALL_STATE(5030)] = 177726,
  [SMALL_STATE(5031)] = 177754,
  [SMALL_STATE(5032)] = 177774,
  [SMALL_STATE(5033)] = 177802,
  [SMALL_STATE(5034)] = 177834,
  [SMALL_STATE(5035)] = 177858,
  [SMALL_STATE(5036)] = 177892,
  [SMALL_STATE(5037)] = 177926,
  [SMALL_STATE(5038)] = 177953,
  [SMALL_STATE(5039)] = 177982,
  [SMALL_STATE(5040)] = 178011,
  [SMALL_STATE(5041)] = 178030,
  [SMALL_STATE(5042)] = 178053,
  [SMALL_STATE(5043)] = 178082,
  [SMALL_STATE(5044)] = 178111,
  [SMALL_STATE(5045)] = 178140,
  [SMALL_STATE(5046)] = 178169,
  [SMALL_STATE(5047)] = 178188,
  [SMALL_STATE(5048)] = 178217,
  [SMALL_STATE(5049)] = 178246,
  [SMALL_STATE(5050)] = 178275,
  [SMALL_STATE(5051)] = 178304,
  [SMALL_STATE(5052)] = 178333,
  [SMALL_STATE(5053)] = 178352,
  [SMALL_STATE(5054)] = 178379,
  [SMALL_STATE(5055)] = 178406,
  [SMALL_STATE(5056)] = 178433,
  [SMALL_STATE(5057)] = 178460,
  [SMALL_STATE(5058)] = 178479,
  [SMALL_STATE(5059)] = 178498,
  [SMALL_STATE(5060)] = 178517,
  [SMALL_STATE(5061)] = 178546,
  [SMALL_STATE(5062)] = 178573,
  [SMALL_STATE(5063)] = 178602,
  [SMALL_STATE(5064)] = 178631,
  [SMALL_STATE(5065)] = 178660,
  [SMALL_STATE(5066)] = 178689,
  [SMALL_STATE(5067)] = 178718,
  [SMALL_STATE(5068)] = 178737,
  [SMALL_STATE(5069)] = 178756,
  [SMALL_STATE(5070)] = 178783,
  [SMALL_STATE(5071)] = 178812,
  [SMALL_STATE(5072)] = 178841,
  [SMALL_STATE(5073)] = 178870,
  [SMALL_STATE(5074)] = 178899,
  [SMALL_STATE(5075)] = 178918,
  [SMALL_STATE(5076)] = 178947,
  [SMALL_STATE(5077)] = 178966,
  [SMALL_STATE(5078)] = 178995,
  [SMALL_STATE(5079)] = 179024,
  [SMALL_STATE(5080)] = 179053,
  [SMALL_STATE(5081)] = 179082,
  [SMALL_STATE(5082)] = 179103,
  [SMALL_STATE(5083)] = 179128,
  [SMALL_STATE(5084)] = 179155,
  [SMALL_STATE(5085)] = 179184,
  [SMALL_STATE(5086)] = 179213,
  [SMALL_STATE(5087)] = 179242,
  [SMALL_STATE(5088)] = 179269,
  [SMALL_STATE(5089)] = 179298,
  [SMALL_STATE(5090)] = 179327,
  [SMALL_STATE(5091)] = 179356,
  [SMALL_STATE(5092)] = 179385,
  [SMALL_STATE(5093)] = 179414,
  [SMALL_STATE(5094)] = 179441,
  [SMALL_STATE(5095)] = 179470,
  [SMALL_STATE(5096)] = 179489,
  [SMALL_STATE(5097)] = 179518,
  [SMALL_STATE(5098)] = 179545,
  [SMALL_STATE(5099)] = 179568,
  [SMALL_STATE(5100)] = 179597,
  [SMALL_STATE(5101)] = 179616,
  [SMALL_STATE(5102)] = 179645,
  [SMALL_STATE(5103)] = 179674,
  [SMALL_STATE(5104)] = 179701,
  [SMALL_STATE(5105)] = 179720,
  [SMALL_STATE(5106)] = 179749,
  [SMALL_STATE(5107)] = 179776,
  [SMALL_STATE(5108)] = 179805,
  [SMALL_STATE(5109)] = 179834,
  [SMALL_STATE(5110)] = 179863,
  [SMALL_STATE(5111)] = 179892,
  [SMALL_STATE(5112)] = 179921,
  [SMALL_STATE(5113)] = 179950,
  [SMALL_STATE(5114)] = 179979,
  [SMALL_STATE(5115)] = 179998,
  [SMALL_STATE(5116)] = 180025,
  [SMALL_STATE(5117)] = 180044,
  [SMALL_STATE(5118)] = 180073,
  [SMALL_STATE(5119)] = 180094,
  [SMALL_STATE(5120)] = 180123,
  [SMALL_STATE(5121)] = 180152,
  [SMALL_STATE(5122)] = 180181,
  [SMALL_STATE(5123)] = 180206,
  [SMALL_STATE(5124)] = 180235,
  [SMALL_STATE(5125)] = 180254,
  [SMALL_STATE(5126)] = 180281,
  [SMALL_STATE(5127)] = 180304,
  [SMALL_STATE(5128)] = 180333,
  [SMALL_STATE(5129)] = 180362,
  [SMALL_STATE(5130)] = 180391,
  [SMALL_STATE(5131)] = 180410,
  [SMALL_STATE(5132)] = 180437,
  [SMALL_STATE(5133)] = 180460,
  [SMALL_STATE(5134)] = 180489,
  [SMALL_STATE(5135)] = 180516,
  [SMALL_STATE(5136)] = 180545,
  [SMALL_STATE(5137)] = 180574,
  [SMALL_STATE(5138)] = 180601,
  [SMALL_STATE(5139)] = 180622,
  [SMALL_STATE(5140)] = 180643,
  [SMALL_STATE(5141)] = 180664,
  [SMALL_STATE(5142)] = 180691,
  [SMALL_STATE(5143)] = 180720,
  [SMALL_STATE(5144)] = 180747,
  [SMALL_STATE(5145)] = 180776,
  [SMALL_STATE(5146)] = 180805,
  [SMALL_STATE(5147)] = 180832,
  [SMALL_STATE(5148)] = 180859,
  [SMALL_STATE(5149)] = 180886,
  [SMALL_STATE(5150)] = 180915,
  [SMALL_STATE(5151)] = 180944,
  [SMALL_STATE(5152)] = 180973,
  [SMALL_STATE(5153)] = 180992,
  [SMALL_STATE(5154)] = 181019,
  [SMALL_STATE(5155)] = 181048,
  [SMALL_STATE(5156)] = 181077,
  [SMALL_STATE(5157)] = 181106,
  [SMALL_STATE(5158)] = 181133,
  [SMALL_STATE(5159)] = 181160,
  [SMALL_STATE(5160)] = 181183,
  [SMALL_STATE(5161)] = 181210,
  [SMALL_STATE(5162)] = 181239,
  [SMALL_STATE(5163)] = 181268,
  [SMALL_STATE(5164)] = 181297,
  [SMALL_STATE(5165)] = 181326,
  [SMALL_STATE(5166)] = 181345,
  [SMALL_STATE(5167)] = 181372,
  [SMALL_STATE(5168)] = 181401,
  [SMALL_STATE(5169)] = 181430,
  [SMALL_STATE(5170)] = 181457,
  [SMALL_STATE(5171)] = 181486,
  [SMALL_STATE(5172)] = 181513,
  [SMALL_STATE(5173)] = 181540,
  [SMALL_STATE(5174)] = 181567,
  [SMALL_STATE(5175)] = 181596,
  [SMALL_STATE(5176)] = 181615,
  [SMALL_STATE(5177)] = 181642,
  [SMALL_STATE(5178)] = 181669,
  [SMALL_STATE(5179)] = 181698,
  [SMALL_STATE(5180)] = 181727,
  [SMALL_STATE(5181)] = 181756,
  [SMALL_STATE(5182)] = 181785,
  [SMALL_STATE(5183)] = 181814,
  [SMALL_STATE(5184)] = 181841,
  [SMALL_STATE(5185)] = 181864,
  [SMALL_STATE(5186)] = 181893,
  [SMALL_STATE(5187)] = 181920,
  [SMALL_STATE(5188)] = 181947,
  [SMALL_STATE(5189)] = 181974,
  [SMALL_STATE(5190)] = 182001,
  [SMALL_STATE(5191)] = 182028,
  [SMALL_STATE(5192)] = 182055,
  [SMALL_STATE(5193)] = 182084,
  [SMALL_STATE(5194)] = 182111,
  [SMALL_STATE(5195)] = 182138,
  [SMALL_STATE(5196)] = 182165,
  [SMALL_STATE(5197)] = 182188,
  [SMALL_STATE(5198)] = 182215,
  [SMALL_STATE(5199)] = 182242,
  [SMALL_STATE(5200)] = 182269,
  [SMALL_STATE(5201)] = 182296,
  [SMALL_STATE(5202)] = 182323,
  [SMALL_STATE(5203)] = 182352,
  [SMALL_STATE(5204)] = 182381,
  [SMALL_STATE(5205)] = 182410,
  [SMALL_STATE(5206)] = 182439,
  [SMALL_STATE(5207)] = 182468,
  [SMALL_STATE(5208)] = 182497,
  [SMALL_STATE(5209)] = 182526,
  [SMALL_STATE(5210)] = 182555,
  [SMALL_STATE(5211)] = 182574,
  [SMALL_STATE(5212)] = 182593,
  [SMALL_STATE(5213)] = 182622,
  [SMALL_STATE(5214)] = 182641,
  [SMALL_STATE(5215)] = 182663,
  [SMALL_STATE(5216)] = 182691,
  [SMALL_STATE(5217)] = 182719,
  [SMALL_STATE(5218)] = 182741,
  [SMALL_STATE(5219)] = 182763,
  [SMALL_STATE(5220)] = 182783,
  [SMALL_STATE(5221)] = 182807,
  [SMALL_STATE(5222)] = 182827,
  [SMALL_STATE(5223)] = 182849,
  [SMALL_STATE(5224)] = 182875,
  [SMALL_STATE(5225)] = 182895,
  [SMALL_STATE(5226)] = 182923,
  [SMALL_STATE(5227)] = 182941,
  [SMALL_STATE(5228)] = 182963,
  [SMALL_STATE(5229)] = 182991,
  [SMALL_STATE(5230)] = 183015,
  [SMALL_STATE(5231)] = 183043,
  [SMALL_STATE(5232)] = 183071,
  [SMALL_STATE(5233)] = 183099,
  [SMALL_STATE(5234)] = 183127,
  [SMALL_STATE(5235)] = 183147,
  [SMALL_STATE(5236)] = 183171,
  [SMALL_STATE(5237)] = 183191,
  [SMALL_STATE(5238)] = 183211,
  [SMALL_STATE(5239)] = 183235,
  [SMALL_STATE(5240)] = 183263,
  [SMALL_STATE(5241)] = 183291,
  [SMALL_STATE(5242)] = 183311,
  [SMALL_STATE(5243)] = 183329,
  [SMALL_STATE(5244)] = 183347,
  [SMALL_STATE(5245)] = 183367,
  [SMALL_STATE(5246)] = 183385,
  [SMALL_STATE(5247)] = 183413,
  [SMALL_STATE(5248)] = 183431,
  [SMALL_STATE(5249)] = 183455,
  [SMALL_STATE(5250)] = 183477,
  [SMALL_STATE(5251)] = 183505,
  [SMALL_STATE(5252)] = 183523,
  [SMALL_STATE(5253)] = 183551,
  [SMALL_STATE(5254)] = 183571,
  [SMALL_STATE(5255)] = 183599,
  [SMALL_STATE(5256)] = 183619,
  [SMALL_STATE(5257)] = 183637,
  [SMALL_STATE(5258)] = 183665,
  [SMALL_STATE(5259)] = 183689,
  [SMALL_STATE(5260)] = 183709,
  [SMALL_STATE(5261)] = 183733,
  [SMALL_STATE(5262)] = 183761,
  [SMALL_STATE(5263)] = 183789,
  [SMALL_STATE(5264)] = 183813,
  [SMALL_STATE(5265)] = 183831,
  [SMALL_STATE(5266)] = 183859,
  [SMALL_STATE(5267)] = 183887,
  [SMALL_STATE(5268)] = 183909,
  [SMALL_STATE(5269)] = 183927,
  [SMALL_STATE(5270)] = 183953,
  [SMALL_STATE(5271)] = 183971,
  [SMALL_STATE(5272)] = 183999,
  [SMALL_STATE(5273)] = 184027,
  [SMALL_STATE(5274)] = 184055,
  [SMALL_STATE(5275)] = 184083,
  [SMALL_STATE(5276)] = 184111,
  [SMALL_STATE(5277)] = 184139,
  [SMALL_STATE(5278)] = 184167,
  [SMALL_STATE(5279)] = 184189,
  [SMALL_STATE(5280)] = 184211,
  [SMALL_STATE(5281)] = 184239,
  [SMALL_STATE(5282)] = 184267,
  [SMALL_STATE(5283)] = 184295,
  [SMALL_STATE(5284)] = 184313,
  [SMALL_STATE(5285)] = 184341,
  [SMALL_STATE(5286)] = 184363,
  [SMALL_STATE(5287)] = 184391,
  [SMALL_STATE(5288)] = 184419,
  [SMALL_STATE(5289)] = 184447,
  [SMALL_STATE(5290)] = 184467,
  [SMALL_STATE(5291)] = 184490,
  [SMALL_STATE(5292)] = 184507,
  [SMALL_STATE(5293)] = 184524,
  [SMALL_STATE(5294)] = 184541,
  [SMALL_STATE(5295)] = 184564,
  [SMALL_STATE(5296)] = 184583,
  [SMALL_STATE(5297)] = 184602,
  [SMALL_STATE(5298)] = 184625,
  [SMALL_STATE(5299)] = 184644,
  [SMALL_STATE(5300)] = 184667,
  [SMALL_STATE(5301)] = 184690,
  [SMALL_STATE(5302)] = 184709,
  [SMALL_STATE(5303)] = 184728,
  [SMALL_STATE(5304)] = 184747,
  [SMALL_STATE(5305)] = 184768,
  [SMALL_STATE(5306)] = 184791,
  [SMALL_STATE(5307)] = 184812,
  [SMALL_STATE(5308)] = 184831,
  [SMALL_STATE(5309)] = 184854,
  [SMALL_STATE(5310)] = 184875,
  [SMALL_STATE(5311)] = 184898,
  [SMALL_STATE(5312)] = 184917,
  [SMALL_STATE(5313)] = 184936,
  [SMALL_STATE(5314)] = 184955,
  [SMALL_STATE(5315)] = 184974,
  [SMALL_STATE(5316)] = 184995,
  [SMALL_STATE(5317)] = 185016,
  [SMALL_STATE(5318)] = 185035,
  [SMALL_STATE(5319)] = 185056,
  [SMALL_STATE(5320)] = 185075,
  [SMALL_STATE(5321)] = 185098,
  [SMALL_STATE(5322)] = 185121,
  [SMALL_STATE(5323)] = 185144,
  [SMALL_STATE(5324)] = 185161,
  [SMALL_STATE(5325)] = 185184,
  [SMALL_STATE(5326)] = 185207,
  [SMALL_STATE(5327)] = 185228,
  [SMALL_STATE(5328)] = 185244,
  [SMALL_STATE(5329)] = 185260,
  [SMALL_STATE(5330)] = 185278,
  [SMALL_STATE(5331)] = 185298,
  [SMALL_STATE(5332)] = 185314,
  [SMALL_STATE(5333)] = 185330,
  [SMALL_STATE(5334)] = 185346,
  [SMALL_STATE(5335)] = 185362,
  [SMALL_STATE(5336)] = 185378,
  [SMALL_STATE(5337)] = 185394,
  [SMALL_STATE(5338)] = 185410,
  [SMALL_STATE(5339)] = 185426,
  [SMALL_STATE(5340)] = 185442,
  [SMALL_STATE(5341)] = 185458,
  [SMALL_STATE(5342)] = 185478,
  [SMALL_STATE(5343)] = 185494,
  [SMALL_STATE(5344)] = 185510,
  [SMALL_STATE(5345)] = 185526,
  [SMALL_STATE(5346)] = 185542,
  [SMALL_STATE(5347)] = 185558,
  [SMALL_STATE(5348)] = 185574,
  [SMALL_STATE(5349)] = 185590,
  [SMALL_STATE(5350)] = 185606,
  [SMALL_STATE(5351)] = 185628,
  [SMALL_STATE(5352)] = 185646,
  [SMALL_STATE(5353)] = 185662,
  [SMALL_STATE(5354)] = 185678,
  [SMALL_STATE(5355)] = 185694,
  [SMALL_STATE(5356)] = 185710,
  [SMALL_STATE(5357)] = 185728,
  [SMALL_STATE(5358)] = 185744,
  [SMALL_STATE(5359)] = 185760,
  [SMALL_STATE(5360)] = 185776,
  [SMALL_STATE(5361)] = 185792,
  [SMALL_STATE(5362)] = 185812,
  [SMALL_STATE(5363)] = 185828,
  [SMALL_STATE(5364)] = 185844,
  [SMALL_STATE(5365)] = 185860,
  [SMALL_STATE(5366)] = 185876,
  [SMALL_STATE(5367)] = 185892,
  [SMALL_STATE(5368)] = 185908,
  [SMALL_STATE(5369)] = 185924,
  [SMALL_STATE(5370)] = 185942,
  [SMALL_STATE(5371)] = 185958,
  [SMALL_STATE(5372)] = 185974,
  [SMALL_STATE(5373)] = 185996,
  [SMALL_STATE(5374)] = 186012,
  [SMALL_STATE(5375)] = 186034,
  [SMALL_STATE(5376)] = 186050,
  [SMALL_STATE(5377)] = 186066,
  [SMALL_STATE(5378)] = 186082,
  [SMALL_STATE(5379)] = 186098,
  [SMALL_STATE(5380)] = 186114,
  [SMALL_STATE(5381)] = 186130,
  [SMALL_STATE(5382)] = 186146,
  [SMALL_STATE(5383)] = 186162,
  [SMALL_STATE(5384)] = 186178,
  [SMALL_STATE(5385)] = 186194,
  [SMALL_STATE(5386)] = 186210,
  [SMALL_STATE(5387)] = 186226,
  [SMALL_STATE(5388)] = 186242,
  [SMALL_STATE(5389)] = 186260,
  [SMALL_STATE(5390)] = 186276,
  [SMALL_STATE(5391)] = 186292,
  [SMALL_STATE(5392)] = 186308,
  [SMALL_STATE(5393)] = 186326,
  [SMALL_STATE(5394)] = 186342,
  [SMALL_STATE(5395)] = 186358,
  [SMALL_STATE(5396)] = 186378,
  [SMALL_STATE(5397)] = 186396,
  [SMALL_STATE(5398)] = 186412,
  [SMALL_STATE(5399)] = 186428,
  [SMALL_STATE(5400)] = 186448,
  [SMALL_STATE(5401)] = 186464,
  [SMALL_STATE(5402)] = 186480,
  [SMALL_STATE(5403)] = 186496,
  [SMALL_STATE(5404)] = 186514,
  [SMALL_STATE(5405)] = 186530,
  [SMALL_STATE(5406)] = 186550,
  [SMALL_STATE(5407)] = 186566,
  [SMALL_STATE(5408)] = 186582,
  [SMALL_STATE(5409)] = 186600,
  [SMALL_STATE(5410)] = 186616,
  [SMALL_STATE(5411)] = 186632,
  [SMALL_STATE(5412)] = 186648,
  [SMALL_STATE(5413)] = 186664,
  [SMALL_STATE(5414)] = 186680,
  [SMALL_STATE(5415)] = 186696,
  [SMALL_STATE(5416)] = 186712,
  [SMALL_STATE(5417)] = 186728,
  [SMALL_STATE(5418)] = 186744,
  [SMALL_STATE(5419)] = 186760,
  [SMALL_STATE(5420)] = 186776,
  [SMALL_STATE(5421)] = 186792,
  [SMALL_STATE(5422)] = 186808,
  [SMALL_STATE(5423)] = 186824,
  [SMALL_STATE(5424)] = 186844,
  [SMALL_STATE(5425)] = 186860,
  [SMALL_STATE(5426)] = 186876,
  [SMALL_STATE(5427)] = 186892,
  [SMALL_STATE(5428)] = 186908,
  [SMALL_STATE(5429)] = 186924,
  [SMALL_STATE(5430)] = 186940,
  [SMALL_STATE(5431)] = 186956,
  [SMALL_STATE(5432)] = 186972,
  [SMALL_STATE(5433)] = 186988,
  [SMALL_STATE(5434)] = 187008,
  [SMALL_STATE(5435)] = 187024,
  [SMALL_STATE(5436)] = 187040,
  [SMALL_STATE(5437)] = 187056,
  [SMALL_STATE(5438)] = 187072,
  [SMALL_STATE(5439)] = 187088,
  [SMALL_STATE(5440)] = 187104,
  [SMALL_STATE(5441)] = 187122,
  [SMALL_STATE(5442)] = 187138,
  [SMALL_STATE(5443)] = 187158,
  [SMALL_STATE(5444)] = 187176,
  [SMALL_STATE(5445)] = 187192,
  [SMALL_STATE(5446)] = 187208,
  [SMALL_STATE(5447)] = 187224,
  [SMALL_STATE(5448)] = 187244,
  [SMALL_STATE(5449)] = 187260,
  [SMALL_STATE(5450)] = 187278,
  [SMALL_STATE(5451)] = 187296,
  [SMALL_STATE(5452)] = 187312,
  [SMALL_STATE(5453)] = 187328,
  [SMALL_STATE(5454)] = 187344,
  [SMALL_STATE(5455)] = 187364,
  [SMALL_STATE(5456)] = 187380,
  [SMALL_STATE(5457)] = 187396,
  [SMALL_STATE(5458)] = 187412,
  [SMALL_STATE(5459)] = 187428,
  [SMALL_STATE(5460)] = 187444,
  [SMALL_STATE(5461)] = 187460,
  [SMALL_STATE(5462)] = 187480,
  [SMALL_STATE(5463)] = 187496,
  [SMALL_STATE(5464)] = 187512,
  [SMALL_STATE(5465)] = 187528,
  [SMALL_STATE(5466)] = 187544,
  [SMALL_STATE(5467)] = 187560,
  [SMALL_STATE(5468)] = 187580,
  [SMALL_STATE(5469)] = 187600,
  [SMALL_STATE(5470)] = 187620,
  [SMALL_STATE(5471)] = 187636,
  [SMALL_STATE(5472)] = 187656,
  [SMALL_STATE(5473)] = 187672,
  [SMALL_STATE(5474)] = 187688,
  [SMALL_STATE(5475)] = 187704,
  [SMALL_STATE(5476)] = 187720,
  [SMALL_STATE(5477)] = 187736,
  [SMALL_STATE(5478)] = 187752,
  [SMALL_STATE(5479)] = 187768,
  [SMALL_STATE(5480)] = 187784,
  [SMALL_STATE(5481)] = 187804,
  [SMALL_STATE(5482)] = 187820,
  [SMALL_STATE(5483)] = 187836,
  [SMALL_STATE(5484)] = 187852,
  [SMALL_STATE(5485)] = 187872,
  [SMALL_STATE(5486)] = 187888,
  [SMALL_STATE(5487)] = 187904,
  [SMALL_STATE(5488)] = 187920,
  [SMALL_STATE(5489)] = 187936,
  [SMALL_STATE(5490)] = 187958,
  [SMALL_STATE(5491)] = 187974,
  [SMALL_STATE(5492)] = 187990,
  [SMALL_STATE(5493)] = 188006,
  [SMALL_STATE(5494)] = 188022,
  [SMALL_STATE(5495)] = 188038,
  [SMALL_STATE(5496)] = 188057,
  [SMALL_STATE(5497)] = 188074,
  [SMALL_STATE(5498)] = 188093,
  [SMALL_STATE(5499)] = 188112,
  [SMALL_STATE(5500)] = 188131,
  [SMALL_STATE(5501)] = 188150,
  [SMALL_STATE(5502)] = 188169,
  [SMALL_STATE(5503)] = 188188,
  [SMALL_STATE(5504)] = 188207,
  [SMALL_STATE(5505)] = 188226,
  [SMALL_STATE(5506)] = 188241,
  [SMALL_STATE(5507)] = 188260,
  [SMALL_STATE(5508)] = 188277,
  [SMALL_STATE(5509)] = 188296,
  [SMALL_STATE(5510)] = 188315,
  [SMALL_STATE(5511)] = 188334,
  [SMALL_STATE(5512)] = 188351,
  [SMALL_STATE(5513)] = 188370,
  [SMALL_STATE(5514)] = 188389,
  [SMALL_STATE(5515)] = 188406,
  [SMALL_STATE(5516)] = 188425,
  [SMALL_STATE(5517)] = 188440,
  [SMALL_STATE(5518)] = 188459,
  [SMALL_STATE(5519)] = 188478,
  [SMALL_STATE(5520)] = 188497,
  [SMALL_STATE(5521)] = 188516,
  [SMALL_STATE(5522)] = 188535,
  [SMALL_STATE(5523)] = 188554,
  [SMALL_STATE(5524)] = 188573,
  [SMALL_STATE(5525)] = 188592,
  [SMALL_STATE(5526)] = 188611,
  [SMALL_STATE(5527)] = 188630,
  [SMALL_STATE(5528)] = 188647,
  [SMALL_STATE(5529)] = 188664,
  [SMALL_STATE(5530)] = 188683,
  [SMALL_STATE(5531)] = 188702,
  [SMALL_STATE(5532)] = 188721,
  [SMALL_STATE(5533)] = 188736,
  [SMALL_STATE(5534)] = 188751,
  [SMALL_STATE(5535)] = 188770,
  [SMALL_STATE(5536)] = 188789,
  [SMALL_STATE(5537)] = 188808,
  [SMALL_STATE(5538)] = 188827,
  [SMALL_STATE(5539)] = 188846,
  [SMALL_STATE(5540)] = 188861,
  [SMALL_STATE(5541)] = 188880,
  [SMALL_STATE(5542)] = 188899,
  [SMALL_STATE(5543)] = 188914,
  [SMALL_STATE(5544)] = 188929,
  [SMALL_STATE(5545)] = 188944,
  [SMALL_STATE(5546)] = 188959,
  [SMALL_STATE(5547)] = 188978,
  [SMALL_STATE(5548)] = 188995,
  [SMALL_STATE(5549)] = 189014,
  [SMALL_STATE(5550)] = 189033,
  [SMALL_STATE(5551)] = 189048,
  [SMALL_STATE(5552)] = 189067,
  [SMALL_STATE(5553)] = 189082,
  [SMALL_STATE(5554)] = 189097,
  [SMALL_STATE(5555)] = 189116,
  [SMALL_STATE(5556)] = 189131,
  [SMALL_STATE(5557)] = 189146,
  [SMALL_STATE(5558)] = 189165,
  [SMALL_STATE(5559)] = 189180,
  [SMALL_STATE(5560)] = 189199,
  [SMALL_STATE(5561)] = 189218,
  [SMALL_STATE(5562)] = 189237,
  [SMALL_STATE(5563)] = 189256,
  [SMALL_STATE(5564)] = 189275,
  [SMALL_STATE(5565)] = 189292,
  [SMALL_STATE(5566)] = 189309,
  [SMALL_STATE(5567)] = 189326,
  [SMALL_STATE(5568)] = 189343,
  [SMALL_STATE(5569)] = 189362,
  [SMALL_STATE(5570)] = 189379,
  [SMALL_STATE(5571)] = 189398,
  [SMALL_STATE(5572)] = 189417,
  [SMALL_STATE(5573)] = 189436,
  [SMALL_STATE(5574)] = 189453,
  [SMALL_STATE(5575)] = 189472,
  [SMALL_STATE(5576)] = 189491,
  [SMALL_STATE(5577)] = 189506,
  [SMALL_STATE(5578)] = 189520,
  [SMALL_STATE(5579)] = 189534,
  [SMALL_STATE(5580)] = 189548,
  [SMALL_STATE(5581)] = 189562,
  [SMALL_STATE(5582)] = 189576,
  [SMALL_STATE(5583)] = 189590,
  [SMALL_STATE(5584)] = 189604,
  [SMALL_STATE(5585)] = 189618,
  [SMALL_STATE(5586)] = 189634,
  [SMALL_STATE(5587)] = 189648,
  [SMALL_STATE(5588)] = 189662,
  [SMALL_STATE(5589)] = 189678,
  [SMALL_STATE(5590)] = 189692,
  [SMALL_STATE(5591)] = 189706,
  [SMALL_STATE(5592)] = 189720,
  [SMALL_STATE(5593)] = 189734,
  [SMALL_STATE(5594)] = 189750,
  [SMALL_STATE(5595)] = 189764,
  [SMALL_STATE(5596)] = 189778,
  [SMALL_STATE(5597)] = 189792,
  [SMALL_STATE(5598)] = 189806,
  [SMALL_STATE(5599)] = 189822,
  [SMALL_STATE(5600)] = 189836,
  [SMALL_STATE(5601)] = 189850,
  [SMALL_STATE(5602)] = 189864,
  [SMALL_STATE(5603)] = 189878,
  [SMALL_STATE(5604)] = 189892,
  [SMALL_STATE(5605)] = 189906,
  [SMALL_STATE(5606)] = 189922,
  [SMALL_STATE(5607)] = 189938,
  [SMALL_STATE(5608)] = 189952,
  [SMALL_STATE(5609)] = 189968,
  [SMALL_STATE(5610)] = 189982,
  [SMALL_STATE(5611)] = 189996,
  [SMALL_STATE(5612)] = 190010,
  [SMALL_STATE(5613)] = 190024,
  [SMALL_STATE(5614)] = 190038,
  [SMALL_STATE(5615)] = 190052,
  [SMALL_STATE(5616)] = 190066,
  [SMALL_STATE(5617)] = 190080,
  [SMALL_STATE(5618)] = 190094,
  [SMALL_STATE(5619)] = 190108,
  [SMALL_STATE(5620)] = 190122,
  [SMALL_STATE(5621)] = 190136,
  [SMALL_STATE(5622)] = 190150,
  [SMALL_STATE(5623)] = 190164,
  [SMALL_STATE(5624)] = 190178,
  [SMALL_STATE(5625)] = 190192,
  [SMALL_STATE(5626)] = 190208,
  [SMALL_STATE(5627)] = 190222,
  [SMALL_STATE(5628)] = 190238,
  [SMALL_STATE(5629)] = 190252,
  [SMALL_STATE(5630)] = 190266,
  [SMALL_STATE(5631)] = 190280,
  [SMALL_STATE(5632)] = 190294,
  [SMALL_STATE(5633)] = 190308,
  [SMALL_STATE(5634)] = 190324,
  [SMALL_STATE(5635)] = 190338,
  [SMALL_STATE(5636)] = 190352,
  [SMALL_STATE(5637)] = 190366,
  [SMALL_STATE(5638)] = 190380,
  [SMALL_STATE(5639)] = 190394,
  [SMALL_STATE(5640)] = 190408,
  [SMALL_STATE(5641)] = 190421,
  [SMALL_STATE(5642)] = 190434,
  [SMALL_STATE(5643)] = 190447,
  [SMALL_STATE(5644)] = 190460,
  [SMALL_STATE(5645)] = 190473,
  [SMALL_STATE(5646)] = 190486,
  [SMALL_STATE(5647)] = 190499,
  [SMALL_STATE(5648)] = 190512,
  [SMALL_STATE(5649)] = 190525,
  [SMALL_STATE(5650)] = 190538,
  [SMALL_STATE(5651)] = 190551,
  [SMALL_STATE(5652)] = 190564,
  [SMALL_STATE(5653)] = 190577,
  [SMALL_STATE(5654)] = 190590,
  [SMALL_STATE(5655)] = 190603,
  [SMALL_STATE(5656)] = 190616,
  [SMALL_STATE(5657)] = 190629,
  [SMALL_STATE(5658)] = 190642,
  [SMALL_STATE(5659)] = 190655,
  [SMALL_STATE(5660)] = 190668,
  [SMALL_STATE(5661)] = 190681,
  [SMALL_STATE(5662)] = 190694,
  [SMALL_STATE(5663)] = 190707,
  [SMALL_STATE(5664)] = 190720,
  [SMALL_STATE(5665)] = 190733,
  [SMALL_STATE(5666)] = 190746,
  [SMALL_STATE(5667)] = 190759,
  [SMALL_STATE(5668)] = 190772,
  [SMALL_STATE(5669)] = 190785,
  [SMALL_STATE(5670)] = 190798,
  [SMALL_STATE(5671)] = 190811,
  [SMALL_STATE(5672)] = 190824,
  [SMALL_STATE(5673)] = 190837,
  [SMALL_STATE(5674)] = 190850,
  [SMALL_STATE(5675)] = 190863,
  [SMALL_STATE(5676)] = 190876,
  [SMALL_STATE(5677)] = 190889,
  [SMALL_STATE(5678)] = 190902,
  [SMALL_STATE(5679)] = 190915,
  [SMALL_STATE(5680)] = 190928,
  [SMALL_STATE(5681)] = 190941,
  [SMALL_STATE(5682)] = 190954,
  [SMALL_STATE(5683)] = 190967,
  [SMALL_STATE(5684)] = 190980,
  [SMALL_STATE(5685)] = 190993,
  [SMALL_STATE(5686)] = 191006,
  [SMALL_STATE(5687)] = 191019,
  [SMALL_STATE(5688)] = 191032,
  [SMALL_STATE(5689)] = 191045,
  [SMALL_STATE(5690)] = 191058,
  [SMALL_STATE(5691)] = 191071,
  [SMALL_STATE(5692)] = 191084,
  [SMALL_STATE(5693)] = 191097,
  [SMALL_STATE(5694)] = 191110,
  [SMALL_STATE(5695)] = 191123,
  [SMALL_STATE(5696)] = 191136,
  [SMALL_STATE(5697)] = 191149,
  [SMALL_STATE(5698)] = 191162,
  [SMALL_STATE(5699)] = 191175,
  [SMALL_STATE(5700)] = 191188,
  [SMALL_STATE(5701)] = 191201,
  [SMALL_STATE(5702)] = 191214,
  [SMALL_STATE(5703)] = 191227,
  [SMALL_STATE(5704)] = 191240,
  [SMALL_STATE(5705)] = 191253,
  [SMALL_STATE(5706)] = 191266,
  [SMALL_STATE(5707)] = 191279,
  [SMALL_STATE(5708)] = 191292,
  [SMALL_STATE(5709)] = 191305,
  [SMALL_STATE(5710)] = 191318,
  [SMALL_STATE(5711)] = 191331,
  [SMALL_STATE(5712)] = 191344,
  [SMALL_STATE(5713)] = 191357,
  [SMALL_STATE(5714)] = 191370,
  [SMALL_STATE(5715)] = 191383,
  [SMALL_STATE(5716)] = 191396,
  [SMALL_STATE(5717)] = 191409,
  [SMALL_STATE(5718)] = 191422,
  [SMALL_STATE(5719)] = 191435,
  [SMALL_STATE(5720)] = 191448,
  [SMALL_STATE(5721)] = 191461,
  [SMALL_STATE(5722)] = 191474,
  [SMALL_STATE(5723)] = 191487,
  [SMALL_STATE(5724)] = 191500,
  [SMALL_STATE(5725)] = 191513,
  [SMALL_STATE(5726)] = 191526,
  [SMALL_STATE(5727)] = 191539,
  [SMALL_STATE(5728)] = 191552,
  [SMALL_STATE(5729)] = 191565,
  [SMALL_STATE(5730)] = 191578,
  [SMALL_STATE(5731)] = 191591,
  [SMALL_STATE(5732)] = 191604,
  [SMALL_STATE(5733)] = 191617,
  [SMALL_STATE(5734)] = 191630,
  [SMALL_STATE(5735)] = 191643,
  [SMALL_STATE(5736)] = 191656,
  [SMALL_STATE(5737)] = 191669,
  [SMALL_STATE(5738)] = 191682,
  [SMALL_STATE(5739)] = 191695,
  [SMALL_STATE(5740)] = 191708,
  [SMALL_STATE(5741)] = 191721,
  [SMALL_STATE(5742)] = 191734,
  [SMALL_STATE(5743)] = 191747,
  [SMALL_STATE(5744)] = 191760,
  [SMALL_STATE(5745)] = 191773,
  [SMALL_STATE(5746)] = 191786,
  [SMALL_STATE(5747)] = 191799,
  [SMALL_STATE(5748)] = 191812,
  [SMALL_STATE(5749)] = 191825,
  [SMALL_STATE(5750)] = 191838,
  [SMALL_STATE(5751)] = 191851,
  [SMALL_STATE(5752)] = 191864,
  [SMALL_STATE(5753)] = 191877,
  [SMALL_STATE(5754)] = 191890,
  [SMALL_STATE(5755)] = 191903,
  [SMALL_STATE(5756)] = 191916,
  [SMALL_STATE(5757)] = 191929,
  [SMALL_STATE(5758)] = 191942,
  [SMALL_STATE(5759)] = 191955,
  [SMALL_STATE(5760)] = 191968,
  [SMALL_STATE(5761)] = 191981,
  [SMALL_STATE(5762)] = 191994,
  [SMALL_STATE(5763)] = 192007,
  [SMALL_STATE(5764)] = 192020,
  [SMALL_STATE(5765)] = 192033,
  [SMALL_STATE(5766)] = 192046,
  [SMALL_STATE(5767)] = 192059,
  [SMALL_STATE(5768)] = 192072,
  [SMALL_STATE(5769)] = 192085,
  [SMALL_STATE(5770)] = 192098,
  [SMALL_STATE(5771)] = 192111,
  [SMALL_STATE(5772)] = 192124,
  [SMALL_STATE(5773)] = 192137,
  [SMALL_STATE(5774)] = 192150,
  [SMALL_STATE(5775)] = 192163,
  [SMALL_STATE(5776)] = 192176,
  [SMALL_STATE(5777)] = 192189,
  [SMALL_STATE(5778)] = 192202,
  [SMALL_STATE(5779)] = 192215,
  [SMALL_STATE(5780)] = 192228,
  [SMALL_STATE(5781)] = 192241,
  [SMALL_STATE(5782)] = 192254,
  [SMALL_STATE(5783)] = 192267,
  [SMALL_STATE(5784)] = 192280,
  [SMALL_STATE(5785)] = 192293,
  [SMALL_STATE(5786)] = 192306,
  [SMALL_STATE(5787)] = 192319,
  [SMALL_STATE(5788)] = 192332,
  [SMALL_STATE(5789)] = 192345,
  [SMALL_STATE(5790)] = 192358,
  [SMALL_STATE(5791)] = 192371,
  [SMALL_STATE(5792)] = 192384,
  [SMALL_STATE(5793)] = 192397,
  [SMALL_STATE(5794)] = 192410,
  [SMALL_STATE(5795)] = 192423,
  [SMALL_STATE(5796)] = 192436,
  [SMALL_STATE(5797)] = 192449,
  [SMALL_STATE(5798)] = 192462,
  [SMALL_STATE(5799)] = 192475,
  [SMALL_STATE(5800)] = 192488,
  [SMALL_STATE(5801)] = 192501,
  [SMALL_STATE(5802)] = 192514,
  [SMALL_STATE(5803)] = 192527,
  [SMALL_STATE(5804)] = 192540,
  [SMALL_STATE(5805)] = 192553,
  [SMALL_STATE(5806)] = 192566,
  [SMALL_STATE(5807)] = 192579,
  [SMALL_STATE(5808)] = 192592,
  [SMALL_STATE(5809)] = 192605,
  [SMALL_STATE(5810)] = 192618,
  [SMALL_STATE(5811)] = 192631,
  [SMALL_STATE(5812)] = 192644,
  [SMALL_STATE(5813)] = 192657,
  [SMALL_STATE(5814)] = 192670,
  [SMALL_STATE(5815)] = 192683,
  [SMALL_STATE(5816)] = 192696,
  [SMALL_STATE(5817)] = 192709,
  [SMALL_STATE(5818)] = 192722,
  [SMALL_STATE(5819)] = 192735,
  [SMALL_STATE(5820)] = 192748,
  [SMALL_STATE(5821)] = 192761,
  [SMALL_STATE(5822)] = 192774,
  [SMALL_STATE(5823)] = 192787,
  [SMALL_STATE(5824)] = 192800,
  [SMALL_STATE(5825)] = 192813,
  [SMALL_STATE(5826)] = 192826,
  [SMALL_STATE(5827)] = 192839,
  [SMALL_STATE(5828)] = 192852,
  [SMALL_STATE(5829)] = 192865,
  [SMALL_STATE(5830)] = 192878,
  [SMALL_STATE(5831)] = 192891,
  [SMALL_STATE(5832)] = 192904,
  [SMALL_STATE(5833)] = 192917,
  [SMALL_STATE(5834)] = 192930,
  [SMALL_STATE(5835)] = 192943,
  [SMALL_STATE(5836)] = 192956,
  [SMALL_STATE(5837)] = 192969,
  [SMALL_STATE(5838)] = 192982,
  [SMALL_STATE(5839)] = 192995,
  [SMALL_STATE(5840)] = 193008,
  [SMALL_STATE(5841)] = 193021,
  [SMALL_STATE(5842)] = 193034,
  [SMALL_STATE(5843)] = 193047,
  [SMALL_STATE(5844)] = 193060,
  [SMALL_STATE(5845)] = 193073,
  [SMALL_STATE(5846)] = 193086,
  [SMALL_STATE(5847)] = 193099,
  [SMALL_STATE(5848)] = 193112,
  [SMALL_STATE(5849)] = 193125,
  [SMALL_STATE(5850)] = 193138,
  [SMALL_STATE(5851)] = 193151,
  [SMALL_STATE(5852)] = 193164,
  [SMALL_STATE(5853)] = 193177,
  [SMALL_STATE(5854)] = 193190,
  [SMALL_STATE(5855)] = 193203,
  [SMALL_STATE(5856)] = 193216,
  [SMALL_STATE(5857)] = 193229,
  [SMALL_STATE(5858)] = 193242,
  [SMALL_STATE(5859)] = 193255,
  [SMALL_STATE(5860)] = 193268,
  [SMALL_STATE(5861)] = 193281,
  [SMALL_STATE(5862)] = 193294,
  [SMALL_STATE(5863)] = 193307,
  [SMALL_STATE(5864)] = 193320,
  [SMALL_STATE(5865)] = 193333,
  [SMALL_STATE(5866)] = 193346,
  [SMALL_STATE(5867)] = 193359,
  [SMALL_STATE(5868)] = 193372,
  [SMALL_STATE(5869)] = 193385,
  [SMALL_STATE(5870)] = 193398,
  [SMALL_STATE(5871)] = 193411,
  [SMALL_STATE(5872)] = 193424,
  [SMALL_STATE(5873)] = 193437,
  [SMALL_STATE(5874)] = 193450,
  [SMALL_STATE(5875)] = 193463,
  [SMALL_STATE(5876)] = 193476,
  [SMALL_STATE(5877)] = 193489,
  [SMALL_STATE(5878)] = 193502,
  [SMALL_STATE(5879)] = 193515,
  [SMALL_STATE(5880)] = 193528,
  [SMALL_STATE(5881)] = 193541,
  [SMALL_STATE(5882)] = 193554,
  [SMALL_STATE(5883)] = 193567,
  [SMALL_STATE(5884)] = 193580,
  [SMALL_STATE(5885)] = 193593,
  [SMALL_STATE(5886)] = 193606,
  [SMALL_STATE(5887)] = 193619,
  [SMALL_STATE(5888)] = 193632,
  [SMALL_STATE(5889)] = 193645,
  [SMALL_STATE(5890)] = 193658,
  [SMALL_STATE(5891)] = 193671,
  [SMALL_STATE(5892)] = 193684,
  [SMALL_STATE(5893)] = 193697,
  [SMALL_STATE(5894)] = 193710,
  [SMALL_STATE(5895)] = 193723,
  [SMALL_STATE(5896)] = 193736,
  [SMALL_STATE(5897)] = 193749,
  [SMALL_STATE(5898)] = 193762,
  [SMALL_STATE(5899)] = 193775,
  [SMALL_STATE(5900)] = 193788,
  [SMALL_STATE(5901)] = 193801,
  [SMALL_STATE(5902)] = 193814,
  [SMALL_STATE(5903)] = 193827,
  [SMALL_STATE(5904)] = 193840,
  [SMALL_STATE(5905)] = 193853,
  [SMALL_STATE(5906)] = 193866,
  [SMALL_STATE(5907)] = 193879,
  [SMALL_STATE(5908)] = 193892,
  [SMALL_STATE(5909)] = 193905,
  [SMALL_STATE(5910)] = 193918,
  [SMALL_STATE(5911)] = 193931,
  [SMALL_STATE(5912)] = 193944,
  [SMALL_STATE(5913)] = 193957,
  [SMALL_STATE(5914)] = 193970,
  [SMALL_STATE(5915)] = 193983,
  [SMALL_STATE(5916)] = 193996,
  [SMALL_STATE(5917)] = 194009,
  [SMALL_STATE(5918)] = 194022,
  [SMALL_STATE(5919)] = 194035,
  [SMALL_STATE(5920)] = 194048,
  [SMALL_STATE(5921)] = 194061,
  [SMALL_STATE(5922)] = 194074,
  [SMALL_STATE(5923)] = 194087,
  [SMALL_STATE(5924)] = 194100,
  [SMALL_STATE(5925)] = 194113,
  [SMALL_STATE(5926)] = 194126,
  [SMALL_STATE(5927)] = 194139,
  [SMALL_STATE(5928)] = 194152,
  [SMALL_STATE(5929)] = 194165,
  [SMALL_STATE(5930)] = 194178,
  [SMALL_STATE(5931)] = 194191,
  [SMALL_STATE(5932)] = 194204,
  [SMALL_STATE(5933)] = 194217,
  [SMALL_STATE(5934)] = 194230,
  [SMALL_STATE(5935)] = 194243,
  [SMALL_STATE(5936)] = 194256,
  [SMALL_STATE(5937)] = 194269,
  [SMALL_STATE(5938)] = 194282,
  [SMALL_STATE(5939)] = 194295,
  [SMALL_STATE(5940)] = 194308,
  [SMALL_STATE(5941)] = 194321,
  [SMALL_STATE(5942)] = 194334,
  [SMALL_STATE(5943)] = 194347,
  [SMALL_STATE(5944)] = 194360,
  [SMALL_STATE(5945)] = 194373,
  [SMALL_STATE(5946)] = 194386,
  [SMALL_STATE(5947)] = 194399,
  [SMALL_STATE(5948)] = 194412,
  [SMALL_STATE(5949)] = 194425,
  [SMALL_STATE(5950)] = 194438,
  [SMALL_STATE(5951)] = 194451,
  [SMALL_STATE(5952)] = 194464,
  [SMALL_STATE(5953)] = 194477,
  [SMALL_STATE(5954)] = 194490,
  [SMALL_STATE(5955)] = 194503,
  [SMALL_STATE(5956)] = 194516,
  [SMALL_STATE(5957)] = 194529,
  [SMALL_STATE(5958)] = 194542,
  [SMALL_STATE(5959)] = 194555,
  [SMALL_STATE(5960)] = 194568,
  [SMALL_STATE(5961)] = 194581,
  [SMALL_STATE(5962)] = 194594,
  [SMALL_STATE(5963)] = 194607,
  [SMALL_STATE(5964)] = 194620,
  [SMALL_STATE(5965)] = 194633,
  [SMALL_STATE(5966)] = 194646,
  [SMALL_STATE(5967)] = 194659,
  [SMALL_STATE(5968)] = 194672,
  [SMALL_STATE(5969)] = 194685,
  [SMALL_STATE(5970)] = 194698,
  [SMALL_STATE(5971)] = 194711,
  [SMALL_STATE(5972)] = 194724,
  [SMALL_STATE(5973)] = 194737,
  [SMALL_STATE(5974)] = 194750,
  [SMALL_STATE(5975)] = 194763,
  [SMALL_STATE(5976)] = 194776,
  [SMALL_STATE(5977)] = 194789,
  [SMALL_STATE(5978)] = 194802,
  [SMALL_STATE(5979)] = 194815,
  [SMALL_STATE(5980)] = 194828,
  [SMALL_STATE(5981)] = 194841,
  [SMALL_STATE(5982)] = 194854,
  [SMALL_STATE(5983)] = 194867,
  [SMALL_STATE(5984)] = 194880,
  [SMALL_STATE(5985)] = 194893,
  [SMALL_STATE(5986)] = 194906,
  [SMALL_STATE(5987)] = 194919,
  [SMALL_STATE(5988)] = 194923,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5075),
  [7] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 0, 0, 0),
  [9] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1924),
  [11] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5875),
  [13] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5682),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(741),
  [17] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5725),
  [19] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2301),
  [21] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [23] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5900),
  [25] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2143),
  [27] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [29] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1972),
  [31] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2101),
  [33] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1975),
  [35] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1978),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(79),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(77),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(75),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(68),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2247),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2267),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(406),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(409),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(410),
  [55] = {.entry = {.count = 1, .reusable = false}}, SHIFT(404),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1962),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(283),
  [61] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(941),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(199),
  [67] = {.entry = {.count = 1, .reusable = true}}, SHIFT(585),
  [69] = {.entry = {.count = 1, .reusable = true}}, SHIFT(587),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(593),
  [73] = {.entry = {.count = 1, .reusable = true}}, SHIFT(539),
  [75] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2635),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2640),
  [79] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2226),
  [83] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2108),
  [85] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1982),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1985),
  [89] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1929),
  [91] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2376),
  [93] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4568),
  [95] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2380),
  [97] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5084),
  [99] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5179),
  [101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5039),
  [103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5048),
  [105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4979),
  [107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4980),
  [109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(561),
  [111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5037),
  [113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1803),
  [115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1985),
  [117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1714),
  [119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [121] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__lhs, 1, 0, 0),
  [123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2459),
  [125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(329),
  [127] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__lhs, 1, 0, 0),
  [129] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5900),
  [131] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1972),
  [133] = {.entry = {.count = 1, .reusable = false}}, SHIFT(47),
  [135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(46),
  [137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(45),
  [139] = {.entry = {.count = 1, .reusable = false}}, SHIFT(48),
  [141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [145] = {.entry = {.count = 1, .reusable = false}}, SHIFT(649),
  [147] = {.entry = {.count = 1, .reusable = false}}, SHIFT(651),
  [149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1909),
  [153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1969),
  [155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1966),
  [157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1910),
  [159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(495),
  [163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(586),
  [165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5198),
  [167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5711),
  [169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1966),
  [171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(498),
  [173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(147),
  [177] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__lhs, 1, 0, 2),
  [179] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__lhs, 1, 0, 2),
  [181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(254),
  [183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1918),
  [185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2457),
  [187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(59),
  [189] = {.entry = {.count = 1, .reusable = false}}, SHIFT(63),
  [191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(64),
  [193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(58),
  [195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(603),
  [197] = {.entry = {.count = 1, .reusable = false}}, SHIFT(605),
  [199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(606),
  [201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(606),
  [203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5191),
  [211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5741),
  [213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(682),
  [215] = {.entry = {.count = 1, .reusable = false}}, SHIFT(69),
  [217] = {.entry = {.count = 1, .reusable = false}}, SHIFT(72),
  [219] = {.entry = {.count = 1, .reusable = false}}, SHIFT(74),
  [221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(73),
  [223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(653),
  [225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(668),
  [227] = {.entry = {.count = 1, .reusable = false}}, SHIFT(596),
  [229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(596),
  [231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(613),
  [233] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1938),
  [235] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2463),
  [237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(87),
  [239] = {.entry = {.count = 1, .reusable = false}}, SHIFT(88),
  [241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(89),
  [243] = {.entry = {.count = 1, .reusable = false}}, SHIFT(90),
  [245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(597),
  [247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(598),
  [249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(599),
  [251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(599),
  [253] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3004),
  [255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(709),
  [257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(699),
  [259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(610),
  [261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5055),
  [263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5756),
  [265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(708),
  [267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(250),
  [269] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1935),
  [271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [273] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2424),
  [275] = {.entry = {.count = 1, .reusable = false}}, SHIFT(351),
  [277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5782),
  [279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2220),
  [281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1974),
  [283] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1980),
  [285] = {.entry = {.count = 1, .reusable = false}}, SHIFT(91),
  [287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(92),
  [289] = {.entry = {.count = 1, .reusable = false}}, SHIFT(93),
  [291] = {.entry = {.count = 1, .reusable = false}}, SHIFT(95),
  [293] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2805),
  [295] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2809),
  [297] = {.entry = {.count = 1, .reusable = false}}, SHIFT(414),
  [299] = {.entry = {.count = 1, .reusable = false}}, SHIFT(415),
  [301] = {.entry = {.count = 1, .reusable = false}}, SHIFT(402),
  [303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(407),
  [305] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1961),
  [307] = {.entry = {.count = 1, .reusable = false}}, SHIFT(142),
  [309] = {.entry = {.count = 1, .reusable = false}}, SHIFT(277),
  [311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(438),
  [315] = {.entry = {.count = 1, .reusable = false}}, SHIFT(203),
  [317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(619),
  [319] = {.entry = {.count = 1, .reusable = false}}, SHIFT(621),
  [321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(622),
  [323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(622),
  [325] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2764),
  [327] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1954),
  [329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2221),
  [331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1989),
  [333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2000),
  [335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1941),
  [337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3027),
  [339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4570),
  [341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3026),
  [343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5207),
  [345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5185),
  [347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5080),
  [349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5111),
  [351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5027),
  [353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5004),
  [355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(717),
  [357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(714),
  [359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(547),
  [361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5106),
  [363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5714),
  [365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2000),
  [367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(707),
  [369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [373] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1942),
  [375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2413),
  [377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(109),
  [379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(108),
  [381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(106),
  [383] = {.entry = {.count = 1, .reusable = false}}, SHIFT(107),
  [385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(536),
  [387] = {.entry = {.count = 1, .reusable = false}}, SHIFT(537),
  [389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(538),
  [391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(538),
  [393] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2813),
  [395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(730),
  [397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(562),
  [401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5171),
  [403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5945),
  [405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(728),
  [407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_identifier_call, 1, 0, 4),
  [411] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__function_identifier_call, 1, 0, 4),
  [413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break, 1, 0, 0),
  [415] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break, 1, 0, 0),
  [417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield, 1, 0, 0),
  [419] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_yield, 1, 0, 0),
  [421] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return, 1, 0, 0),
  [423] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return, 1, 0, 0),
  [425] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_next, 1, 0, 0),
  [427] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_next, 1, 0, 0),
  [429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1167),
  [431] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2458),
  [433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(49),
  [435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(51),
  [437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(52),
  [439] = {.entry = {.count = 1, .reusable = false}}, SHIFT(50),
  [441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(531),
  [443] = {.entry = {.count = 1, .reusable = false}}, SHIFT(532),
  [445] = {.entry = {.count = 1, .reusable = false}}, SHIFT(533),
  [447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(533),
  [449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5195),
  [457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5896),
  [459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [461] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1917),
  [463] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2439),
  [465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(527),
  [467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(528),
  [469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(534),
  [471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(686),
  [475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(645),
  [479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5183),
  [481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5908),
  [483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(690),
  [485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(96),
  [487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(86),
  [489] = {.entry = {.count = 1, .reusable = false}}, SHIFT(81),
  [491] = {.entry = {.count = 1, .reusable = false}}, SHIFT(80),
  [493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(655),
  [495] = {.entry = {.count = 1, .reusable = false}}, SHIFT(656),
  [497] = {.entry = {.count = 1, .reusable = false}}, SHIFT(657),
  [499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(657),
  [501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [503] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1939),
  [505] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2460),
  [507] = {.entry = {.count = 1, .reusable = false}}, SHIFT(82),
  [509] = {.entry = {.count = 1, .reusable = false}}, SHIFT(83),
  [511] = {.entry = {.count = 1, .reusable = false}}, SHIFT(84),
  [513] = {.entry = {.count = 1, .reusable = false}}, SHIFT(85),
  [515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [517] = {.entry = {.count = 1, .reusable = false}}, SHIFT(550),
  [519] = {.entry = {.count = 1, .reusable = false}}, SHIFT(551),
  [521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(700),
  [525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(701),
  [527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(594),
  [529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5200),
  [531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5664),
  [533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(695),
  [535] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1933),
  [537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [539] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2421),
  [541] = {.entry = {.count = 1, .reusable = false}}, SHIFT(103),
  [543] = {.entry = {.count = 1, .reusable = false}}, SHIFT(99),
  [545] = {.entry = {.count = 1, .reusable = false}}, SHIFT(100),
  [547] = {.entry = {.count = 1, .reusable = false}}, SHIFT(97),
  [549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [551] = {.entry = {.count = 1, .reusable = false}}, SHIFT(574),
  [553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(575),
  [555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(735),
  [559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(727),
  [561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(529),
  [563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5093),
  [565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5874),
  [567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(729),
  [569] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1937),
  [571] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2461),
  [573] = {.entry = {.count = 1, .reusable = false}}, SHIFT(105),
  [575] = {.entry = {.count = 1, .reusable = false}}, SHIFT(101),
  [577] = {.entry = {.count = 1, .reusable = false}}, SHIFT(102),
  [579] = {.entry = {.count = 1, .reusable = false}}, SHIFT(98),
  [581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(616),
  [583] = {.entry = {.count = 1, .reusable = false}}, SHIFT(617),
  [585] = {.entry = {.count = 1, .reusable = false}}, SHIFT(618),
  [587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(736),
  [593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(604),
  [595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5069),
  [597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5832),
  [599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [601] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1940),
  [603] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2407),
  [605] = {.entry = {.count = 1, .reusable = false}}, SHIFT(110),
  [607] = {.entry = {.count = 1, .reusable = false}}, SHIFT(111),
  [609] = {.entry = {.count = 1, .reusable = false}}, SHIFT(112),
  [611] = {.entry = {.count = 1, .reusable = false}}, SHIFT(113),
  [613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(545),
  [615] = {.entry = {.count = 1, .reusable = false}}, SHIFT(555),
  [617] = {.entry = {.count = 1, .reusable = false}}, SHIFT(556),
  [619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(556),
  [621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(743),
  [623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(744),
  [625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(588),
  [627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5056),
  [629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5858),
  [631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(753),
  [633] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1932),
  [635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(673),
  [637] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2466),
  [639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(116),
  [641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2890),
  [643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5695),
  [645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2050),
  [647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(723),
  [649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(120),
  [651] = {.entry = {.count = 1, .reusable = false}}, SHIFT(723),
  [653] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4705),
  [655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5235),
  [657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [661] = {.entry = {.count = 1, .reusable = false}}, SHIFT(608),
  [663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(609),
  [665] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2634),
  [667] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2661),
  [669] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1934),
  [671] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2219),
  [673] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1922),
  [675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3171),
  [677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5103),
  [681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1703),
  [683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2289),
  [685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1956),
  [687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(674),
  [689] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [691] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5647),
  [693] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1958),
  [697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(118),
  [699] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3736),
  [701] = {.entry = {.count = 1, .reusable = false}}, SHIFT(710),
  [703] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [705] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1957),
  [707] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2540),
  [709] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1945),
  [711] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3729),
  [713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(808),
  [715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1376),
  [719] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5679),
  [721] = {.entry = {.count = 1, .reusable = true}}, SHIFT(795),
  [723] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3106),
  [725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3086),
  [727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2320),
  [729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3214),
  [731] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__variable, 1, 0, 0),
  [733] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5348),
  [735] = {.entry = {.count = 1, .reusable = false}}, SHIFT(677),
  [737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2121),
  [739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1704),
  [741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(540),
  [743] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__variable, 1, 0, 0),
  [745] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1704),
  [747] = {.entry = {.count = 1, .reusable = false}}, SHIFT(268),
  [749] = {.entry = {.count = 1, .reusable = false}}, SHIFT(299),
  [751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2041),
  [753] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2978),
  [755] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2357),
  [757] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2364),
  [759] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2979),
  [761] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5352),
  [763] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2348),
  [765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(898),
  [767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(847),
  [769] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2960),
  [771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [773] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2230),
  [775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4575),
  [777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(842),
  [781] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4681),
  [783] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3009),
  [785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(855),
  [787] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5175),
  [789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(874),
  [791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(903),
  [793] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2390),
  [795] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4646),
  [797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(908),
  [801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(886),
  [805] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5444),
  [807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(907),
  [809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(853),
  [811] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4416),
  [813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(864),
  [815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [817] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5024),
  [819] = {.entry = {.count = 1, .reusable = true}}, SHIFT(845),
  [821] = {.entry = {.count = 1, .reusable = true}}, SHIFT(909),
  [823] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2088),
  [825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(910),
  [827] = {.entry = {.count = 1, .reusable = true}}, SHIFT(901),
  [829] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4378),
  [831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(899),
  [833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(879),
  [835] = {.entry = {.count = 1, .reusable = true}}, SHIFT(882),
  [837] = {.entry = {.count = 1, .reusable = true}}, SHIFT(868),
  [839] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4677),
  [841] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2446),
  [843] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2900),
  [845] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2908),
  [847] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2976),
  [849] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4951),
  [851] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4953),
  [853] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4962),
  [855] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2374),
  [857] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4359),
  [859] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4363),
  [861] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4369),
  [863] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4671),
  [865] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4655),
  [867] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4430),
  [869] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4401),
  [871] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4386),
  [873] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2100),
  [875] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2082),
  [877] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2093),
  [879] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2245),
  [881] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2232),
  [883] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2239),
  [885] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4712),
  [887] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4687),
  [889] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2444),
  [891] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5116),
  [893] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5213),
  [895] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5068),
  [897] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4648),
  [899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2438),
  [901] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2079),
  [903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range, 2, 0, 17),
  [905] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range, 2, 0, 17),
  [907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1673),
  [909] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2260),
  [911] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2246),
  [913] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2248),
  [915] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2249),
  [917] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1673),
  [919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1732),
  [921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1639),
  [923] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1640),
  [925] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1641),
  [927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1641),
  [929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2258),
  [931] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2253),
  [933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2080),
  [935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1659),
  [937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5199),
  [939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2253),
  [941] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5453),
  [943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_splat_argument, 1, 0, 0),
  [945] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_splat_argument, 1, 0, 0),
  [947] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3022),
  [949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1665),
  [951] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1665),
  [953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1731),
  [955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1614),
  [957] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1615),
  [959] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1616),
  [961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1616),
  [963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1637),
  [965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5197),
  [967] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2931),
  [969] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2975),
  [971] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2998),
  [973] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5470),
  [975] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5337),
  [977] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5346),
  [979] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5393),
  [981] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5475),
  [983] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5354),
  [985] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4961),
  [987] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4969),
  [989] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2423),
  [991] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2430),
  [993] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2436),
  [995] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4367),
  [997] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4373),
  [999] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4652),
  [1001] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4670),
  [1003] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4382),
  [1005] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4399),
  [1007] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2092),
  [1009] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2097),
  [1011] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2237),
  [1013] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2963),
  [1015] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2223),
  [1017] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3072),
  [1019] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4715),
  [1021] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2984),
  [1023] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4683),
  [1025] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5052),
  [1027] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5059),
  [1029] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2412),
  [1031] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2302),
  [1033] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2305),
  [1035] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2309),
  [1037] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2927),
  [1039] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2371),
  [1041] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2930),
  [1043] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5386),
  [1045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1566),
  [1047] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1566),
  [1049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1730),
  [1051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1467),
  [1053] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1905),
  [1055] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1468),
  [1057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1468),
  [1059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1464),
  [1061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5134),
  [1063] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3006),
  [1065] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2384),
  [1067] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1, 0, 2),
  [1069] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression, 1, 0, 2),
  [1071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(496),
  [1073] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1244),
  [1075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(647),
  [1077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1729),
  [1079] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1921),
  [1081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2863),
  [1083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4572),
  [1085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2861),
  [1087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3350),
  [1089] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statements, 1, 0, 0),
  [1091] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__statements, 1, 0, 0),
  [1093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3413),
  [1095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2307),
  [1097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2447),
  [1099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3343),
  [1101] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1909),
  [1104] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5682),
  [1107] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(741),
  [1110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0),
  [1112] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5725),
  [1115] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2466),
  [1118] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(329),
  [1121] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5900),
  [1124] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0),
  [1126] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2050),
  [1129] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1704),
  [1132] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1972),
  [1135] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2101),
  [1138] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1975),
  [1141] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1978),
  [1144] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(49),
  [1147] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(51),
  [1150] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(52),
  [1153] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(50),
  [1156] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2247),
  [1159] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2267),
  [1162] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(406),
  [1165] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(409),
  [1168] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(410),
  [1171] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(404),
  [1174] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1962),
  [1177] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(283),
  [1180] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(424),
  [1183] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1704),
  [1186] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(199),
  [1189] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(601),
  [1192] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(607),
  [1195] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(608),
  [1198] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(609),
  [1201] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2634),
  [1204] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2661),
  [1207] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2226),
  [1210] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2108),
  [1213] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1969),
  [1216] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1966),
  [1219] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1910),
  [1222] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2376),
  [1225] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(4568),
  [1228] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2380),
  [1231] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5084),
  [1234] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5179),
  [1237] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5039),
  [1240] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5048),
  [1243] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(4979),
  [1246] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(4980),
  [1249] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(525),
  [1252] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(5103),
  [1255] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1703),
  [1258] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(1966),
  [1261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [1263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1146),
  [1265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(530),
  [1267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1726),
  [1269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2406),
  [1271] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_then, 1, 0, 0),
  [1273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1244),
  [1275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1335),
  [1277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1400),
  [1279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1401),
  [1281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1402),
  [1283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1402),
  [1285] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_then, 2, 0, 0),
  [1287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2368),
  [1289] = {.entry = {.count = 1, .reusable = false}}, SHIFT(411),
  [1291] = {.entry = {.count = 1, .reusable = false}}, SHIFT(262),
  [1293] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2879),
  [1295] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5413),
  [1297] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5409),
  [1299] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else, 1, 0, 0),
  [1301] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2882),
  [1303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1146),
  [1305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1245),
  [1307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1336),
  [1309] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1337),
  [1311] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1338),
  [1313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1338),
  [1315] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2382),
  [1317] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_elsif, 3, 0, 64),
  [1319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1660),
  [1321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1660),
  [1323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1516),
  [1325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1590),
  [1327] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1591),
  [1329] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1592),
  [1331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1592),
  [1333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1612),
  [1335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5193),
  [1337] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rescue, 4, 0, 94),
  [1339] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5400),
  [1341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5437),
  [1343] = {.entry = {.count = 1, .reusable = false}}, SHIFT(810),
  [1345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(383),
  [1347] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3017),
  [1349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(380),
  [1351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1708),
  [1353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1152),
  [1355] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1153),
  [1357] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1164),
  [1359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1164),
  [1361] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rescue, 3, 0, 74),
  [1363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rescue, 3, 0, 72),
  [1365] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rescue, 2, 0, 0),
  [1367] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2396),
  [1369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(386),
  [1371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4415),
  [1373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1613),
  [1375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1613),
  [1377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1399),
  [1379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1517),
  [1381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1518),
  [1383] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1519),
  [1385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1519),
  [1387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1565),
  [1389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5186),
  [1391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2236),
  [1393] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2206),
  [1395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(351),
  [1397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1425),
  [1399] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2819),
  [1401] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2804),
  [1403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2808),
  [1405] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2777),
  [1407] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1425),
  [1409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1107),
  [1411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1108),
  [1413] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1109),
  [1415] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1110),
  [1417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1110),
  [1419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2858),
  [1421] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2857),
  [1423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2207),
  [1425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1424),
  [1427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5087),
  [1429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2857),
  [1431] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else, 2, 0, 0),
  [1433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3008),
  [1435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4721),
  [1437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4662),
  [1439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5165),
  [1441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4377),
  [1443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2085),
  [1445] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_in_clause, 3, 0, 68),
  [1447] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_in_clause, 4, 0, 111),
  [1449] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_ensure, 1, 0, 0),
  [1451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5023),
  [1453] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_when, 3, 0, 68),
  [1455] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_when, 4, 0, 89),
  [1457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2387),
  [1459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1701),
  [1461] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1701),
  [1463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1446),
  [1465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1447),
  [1467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1448),
  [1469] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1449),
  [1471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1449),
  [1473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1688),
  [1475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5176),
  [1477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1669),
  [1479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1669),
  [1481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1360),
  [1483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1361),
  [1485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1362),
  [1487] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1363),
  [1489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1363),
  [1491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1668),
  [1493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5158),
  [1495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1609),
  [1497] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1609),
  [1499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1316),
  [1501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1317),
  [1503] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1318),
  [1505] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1319),
  [1507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1319),
  [1509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1608),
  [1511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5141),
  [1513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1859),
  [1515] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1859),
  [1517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1863),
  [1519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1842),
  [1521] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1843),
  [1523] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1844),
  [1525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1844),
  [1527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1858),
  [1529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5201),
  [1531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1465),
  [1533] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1465),
  [1535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1147),
  [1537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1246),
  [1539] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1247),
  [1541] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1248),
  [1543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1248),
  [1545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1333),
  [1547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5143),
  [1549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4358),
  [1551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1689),
  [1553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1689),
  [1555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1297),
  [1557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1298),
  [1559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1299),
  [1561] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1300),
  [1563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1300),
  [1565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1672),
  [1567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5166),
  [1569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4429),
  [1571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(680),
  [1573] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1043),
  [1575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(602),
  [1577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1042),
  [1579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2228),
  [1581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2443),
  [1583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4704),
  [1585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4949),
  [1587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5076),
  [1589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2896),
  [1591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2099),
  [1593] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2456),
  [1595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1638),
  [1597] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1638),
  [1599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1466),
  [1601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1567),
  [1603] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1568),
  [1605] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1569),
  [1607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1569),
  [1609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1588),
  [1611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5189),
  [1613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1677),
  [1615] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1677),
  [1617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1426),
  [1619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1427),
  [1621] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1428),
  [1623] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1429),
  [1625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1429),
  [1627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1676),
  [1629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5187),
  [1631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4673),
  [1633] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2110),
  [1635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(671),
  [1637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [1639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5855),
  [1641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(721),
  [1643] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1916),
  [1645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4517),
  [1647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(121),
  [1649] = {.entry = {.count = 1, .reusable = false}}, SHIFT(721),
  [1651] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4381),
  [1653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5258),
  [1655] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2077),
  [1657] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2107),
  [1659] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2068),
  [1661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2465),
  [1663] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5494),
  [1665] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2312),
  [1667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(670),
  [1669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5771),
  [1671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [1673] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1958),
  [1675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4693),
  [1677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [1679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(705),
  [1681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1067),
  [1683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1068),
  [1685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1069),
  [1687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1070),
  [1689] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2277),
  [1691] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2310),
  [1693] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2256),
  [1695] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3302),
  [1697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(951),
  [1699] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2170),
  [1701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(703),
  [1703] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1944),
  [1705] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4624),
  [1707] = {.entry = {.count = 1, .reusable = false}}, SHIFT(703),
  [1709] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5263),
  [1711] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1044),
  [1713] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1148),
  [1715] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1149),
  [1717] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1150),
  [1719] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2146),
  [1721] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2883),
  [1723] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [1725] = {.entry = {.count = 1, .reusable = false}}, SHIFT(704),
  [1727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1418),
  [1729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1419),
  [1731] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1420),
  [1733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1423),
  [1735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [1737] = {.entry = {.count = 1, .reusable = false}}, SHIFT(718),
  [1739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [1741] = {.entry = {.count = 1, .reusable = false}}, SHIFT(720),
  [1743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1151),
  [1745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(917),
  [1747] = {.entry = {.count = 1, .reusable = false}}, SHIFT(918),
  [1749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(919),
  [1751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1086),
  [1753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [1755] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2174),
  [1757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(672),
  [1759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [1761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5952),
  [1763] = {.entry = {.count = 1, .reusable = true}}, SHIFT(725),
  [1765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(119),
  [1767] = {.entry = {.count = 1, .reusable = false}}, SHIFT(725),
  [1769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1702),
  [1771] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2190),
  [1773] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2251),
  [1775] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2166),
  [1777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3125),
  [1779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5137),
  [1781] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2282),
  [1783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [1785] = {.entry = {.count = 1, .reusable = false}}, SHIFT(697),
  [1787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1139),
  [1789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1142),
  [1791] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1143),
  [1793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1144),
  [1795] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2233),
  [1797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1182),
  [1799] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2279),
  [1801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [1803] = {.entry = {.count = 1, .reusable = false}}, SHIFT(724),
  [1805] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1089),
  [1807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1090),
  [1809] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1091),
  [1811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1092),
  [1813] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2229),
  [1815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1514),
  [1817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1043),
  [1819] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1150),
  [1821] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [1823] = {.entry = {.count = 1, .reusable = false}}, SHIFT(715),
  [1825] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1224),
  [1827] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1225),
  [1829] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1226),
  [1831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1227),
  [1833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1492),
  [1835] = {.entry = {.count = 1, .reusable = true}}, SHIFT(698),
  [1837] = {.entry = {.count = 1, .reusable = false}}, SHIFT(698),
  [1839] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [1841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1635),
  [1843] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1636),
  [1845] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1653),
  [1847] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1243),
  [1849] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2470),
  [1851] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1681),
  [1853] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1681),
  [1855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1205),
  [1857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1206),
  [1859] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1207),
  [1861] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1208),
  [1863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1208),
  [1865] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1664),
  [1867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5157),
  [1869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5295),
  [1871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4580),
  [1873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4577),
  [1875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5452),
  [1877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5241),
  [1879] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 2, 0, 0), SHIFT_REPEAT(2143),
  [1882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(685),
  [1884] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1103),
  [1886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(526),
  [1888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5298),
  [1890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [1892] = {.entry = {.count = 1, .reusable = false}}, SHIFT(711),
  [1894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1805),
  [1896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1819),
  [1898] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1820),
  [1900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1821),
  [1902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1862),
  [1904] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2383),
  [1906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(702),
  [1908] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1802),
  [1910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(548),
  [1912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1103),
  [1914] = {.entry = {.count = 1, .reusable = false}}, SHIFT(919),
  [1916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5782),
  [1918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(719),
  [1920] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1493),
  [1922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [1924] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2464),
  [1926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1861),
  [1928] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1861),
  [1930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1822),
  [1932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1823),
  [1934] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1824),
  [1936] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1825),
  [1938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1825),
  [1940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1860),
  [1942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5160),
  [1944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1183),
  [1946] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1183),
  [1948] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1144),
  [1950] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2441),
  [1952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1359),
  [1954] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1359),
  [1956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1003),
  [1958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1004),
  [1960] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1005),
  [1962] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1006),
  [1964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [1966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1358),
  [1968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5131),
  [1970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [1972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [1974] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1589),
  [1976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(669),
  [1978] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2428),
  [1980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1693),
  [1982] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1693),
  [1984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1124),
  [1986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1125),
  [1988] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1126),
  [1990] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1127),
  [1992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1127),
  [1994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3522),
  [1996] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3507),
  [1998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5077),
  [2000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1692),
  [2002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5169),
  [2004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3507),
  [2006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1493),
  [2008] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1227),
  [2010] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2454),
  [2012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1662),
  [2014] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1662),
  [2016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1276),
  [2018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1277),
  [2020] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1278),
  [2022] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1279),
  [2024] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1279),
  [2026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1661),
  [2028] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5153),
  [2030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1377),
  [2032] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1377),
  [2034] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1423),
  [2036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1802),
  [2038] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1821),
  [2040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(737),
  [2042] = {.entry = {.count = 1, .reusable = false}}, SHIFT(952),
  [2044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(535),
  [2046] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__terminator, 1, 0, 0),
  [2048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [2050] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1398),
  [2052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(615),
  [2054] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2462),
  [2056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1696),
  [2058] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1696),
  [2060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1380),
  [2062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1381),
  [2064] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1382),
  [2066] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1383),
  [2068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1383),
  [2070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1680),
  [2072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5173),
  [2074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1589),
  [2076] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1092),
  [2078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(966),
  [2080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(975),
  [2082] = {.entry = {.count = 1, .reusable = false}}, SHIFT(997),
  [2084] = {.entry = {.count = 1, .reusable = false}}, SHIFT(998),
  [2086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(998),
  [2088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [2090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(572),
  [2092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1398),
  [2094] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1653),
  [2096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(915),
  [2098] = {.entry = {.count = 1, .reusable = false}}, SHIFT(915),
  [2100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(916),
  [2102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1045),
  [2104] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1046),
  [2106] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1047),
  [2108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1047),
  [2110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(914),
  [2112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5188),
  [2114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(952),
  [2116] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1070),
  [2118] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2432),
  [2120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1633),
  [2122] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1633),
  [2124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1022),
  [2126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1023),
  [2128] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1024),
  [2130] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1025),
  [2132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1025),
  [2134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1632),
  [2136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5146),
  [2138] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2434),
  [2140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1699),
  [2142] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1699),
  [2144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1544),
  [2146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1545),
  [2148] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1546),
  [2150] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1547),
  [2152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1547),
  [2154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1698),
  [2156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5194),
  [2158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(755),
  [2160] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2419),
  [2162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1685),
  [2164] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1685),
  [2166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1494),
  [2168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1495),
  [2170] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1496),
  [2172] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1497),
  [2174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1497),
  [2176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1684),
  [2178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5190),
  [2180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1334),
  [2182] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1334),
  [2184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(935),
  [2186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [2188] = {.entry = {.count = 1, .reusable = false}}, SHIFT(937),
  [2190] = {.entry = {.count = 1, .reusable = false}}, SHIFT(938),
  [2192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(938),
  [2194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1145),
  [2196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5054),
  [2198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [2200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(568),
  [2202] = {.entry = {.count = 1, .reusable = false}}, SHIFT(570),
  [2204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [2206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(541),
  [2208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1275),
  [2210] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1275),
  [2212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1709),
  [2214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1711),
  [2216] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1712),
  [2218] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1713),
  [2220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1713),
  [2222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1274),
  [2224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5097),
  [2226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1841),
  [2228] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1841),
  [2230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1784),
  [2232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1785),
  [2234] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1786),
  [2236] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1787),
  [2238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1787),
  [2240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1840),
  [2242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5148),
  [2244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1515),
  [2246] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1515),
  [2248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(982),
  [2250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [2252] = {.entry = {.count = 1, .reusable = false}}, SHIFT(984),
  [2254] = {.entry = {.count = 1, .reusable = false}}, SHIFT(985),
  [2256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(985),
  [2258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1397),
  [2260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5083),
  [2262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1543),
  [2264] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1543),
  [2266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1403),
  [2268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1414),
  [2270] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1416),
  [2272] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1417),
  [2274] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1417),
  [2276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1542),
  [2278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5115),
  [2280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1082),
  [2282] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1082),
  [2284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1292),
  [2286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1339),
  [2288] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1340),
  [2290] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1351),
  [2292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1351),
  [2294] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1061),
  [2296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5172),
  [2298] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1951),
  [2300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2067),
  [2302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2175),
  [2304] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2437),
  [2306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4807),
  [2308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1658),
  [2310] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1658),
  [2312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1184),
  [2314] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1185),
  [2316] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1186),
  [2318] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1187),
  [2320] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1187),
  [2322] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1657),
  [2324] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5061),
  [2326] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2262),
  [2328] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2367),
  [2330] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5451),
  [2332] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3020),
  [2334] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2057),
  [2336] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__argument_list_with_trailing_comma, 2, 0, 0),
  [2338] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5256),
  [2340] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__argument_list_with_trailing_comma, 3, 0, 0),
  [2342] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4744),
  [2344] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1586),
  [2346] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1586),
  [2348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1737),
  [2350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1738),
  [2352] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1739),
  [2354] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1766),
  [2356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1766),
  [2358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1585),
  [2360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5125),
  [2362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2980),
  [2364] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2490),
  [2366] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pair, 2, 0, 29),
  [2368] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2324),
  [2370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4935),
  [2372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2188),
  [2374] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_pattern, 2, 0, 28),
  [2376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1066),
  [2378] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1066),
  [2380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(957),
  [2382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(958),
  [2384] = {.entry = {.count = 1, .reusable = false}}, SHIFT(959),
  [2386] = {.entry = {.count = 1, .reusable = false}}, SHIFT(960),
  [2388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(960),
  [2390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1065),
  [2392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5053),
  [2394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(379),
  [2396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(625),
  [2398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(626),
  [2400] = {.entry = {.count = 1, .reusable = false}}, SHIFT(627),
  [2402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(628),
  [2404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(365),
  [2406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(631),
  [2408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(263),
  [2410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(640),
  [2412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(641),
  [2414] = {.entry = {.count = 1, .reusable = false}}, SHIFT(642),
  [2416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(644),
  [2418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(270),
  [2420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(611),
  [2422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(658),
  [2424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(659),
  [2426] = {.entry = {.count = 1, .reusable = false}}, SHIFT(660),
  [2428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(661),
  [2430] = {.entry = {.count = 1, .reusable = true}}, SHIFT(666),
  [2432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(362),
  [2434] = {.entry = {.count = 1, .reusable = true}}, SHIFT(391),
  [2436] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [2438] = {.entry = {.count = 1, .reusable = true}}, SHIFT(662),
  [2440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(663),
  [2442] = {.entry = {.count = 1, .reusable = false}}, SHIFT(664),
  [2444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(665),
  [2446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(667),
  [2448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(636),
  [2450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(637),
  [2452] = {.entry = {.count = 1, .reusable = false}}, SHIFT(638),
  [2454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(639),
  [2456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(643),
  [2458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(385),
  [2460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [2462] = {.entry = {.count = 1, .reusable = true}}, SHIFT(373),
  [2464] = {.entry = {.count = 1, .reusable = true}}, SHIFT(397),
  [2466] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pair, 2, 0, 29),
  [2468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4494),
  [2470] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pair, 2, 0, 28),
  [2472] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pair, 2, 0, 28),
  [2474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4487),
  [2476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(632),
  [2478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(633),
  [2480] = {.entry = {.count = 1, .reusable = false}}, SHIFT(634),
  [2482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(635),
  [2484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(612),
  [2486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(623),
  [2488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(624),
  [2490] = {.entry = {.count = 1, .reusable = false}}, SHIFT(629),
  [2492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [2494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(584),
  [2496] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_forward_argument, 1, 0, 0),
  [2498] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_forward_argument, 1, 0, 0),
  [2500] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_splat_argument, 1, 0, 0),
  [2502] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hash_splat_argument, 1, 0, 0),
  [2504] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_argument, 1, 0, 0),
  [2506] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_argument, 1, 0, 0),
  [2508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [2510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(552),
  [2512] = {.entry = {.count = 1, .reusable = false}}, SHIFT(553),
  [2514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [2516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(600),
  [2518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(589),
  [2520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(590),
  [2522] = {.entry = {.count = 1, .reusable = false}}, SHIFT(591),
  [2524] = {.entry = {.count = 1, .reusable = true}}, SHIFT(592),
  [2526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(569),
  [2528] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2505),
  [2530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3351),
  [2532] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2315),
  [2534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3100),
  [2536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3084),
  [2538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3735),
  [2540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1979),
  [2542] = {.entry = {.count = 1, .reusable = false}}, SHIFT(260),
  [2544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [2546] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2481),
  [2548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4933),
  [2550] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2178),
  [2552] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2489),
  [2554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2391),
  [2556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2938),
  [2558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3048),
  [2560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5332),
  [2562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2925),
  [2564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5370),
  [2566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2397),
  [2568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5327),
  [2570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2299),
  [2572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(288),
  [2574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(408),
  [2576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(221),
  [2578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(306),
  [2580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [2582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(374),
  [2584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [2586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(426),
  [2588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(274),
  [2590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [2592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [2594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(328),
  [2596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(399),
  [2598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(322),
  [2600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(364),
  [2602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(311),
  [2604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(419),
  [2606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_parameter, 2, 0, 30),
  [2608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_keyword_parameter, 2, 0, 30),
  [2610] = {.entry = {.count = 1, .reusable = false}}, SHIFT(485),
  [2612] = {.entry = {.count = 1, .reusable = false}}, SHIFT(482),
  [2614] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_constant_suffix, 1, 0, 0),
  [2616] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_constant_suffix, 1, 0, 0),
  [2618] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__call, 3, 0, 49),
  [2620] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scope_resolution, 3, 0, 51),
  [2622] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scope_resolution, 3, 0, 51),
  [2624] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__call, 3, 0, 49),
  [2626] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__function_identifier, 1, 0, 6),
  [2628] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_identifier, 1, 0, 6),
  [2630] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__nonlocal_variable, 1, 0, 0),
  [2632] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__nonlocal_variable, 1, 0, 0),
  [2634] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_operator, 1, 0, 0),
  [2636] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_operator, 1, 0, 0),
  [2638] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__function_identifier, 1, 0, 7),
  [2640] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_identifier, 1, 0, 7),
  [2642] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_identifier_suffix, 1, 0, 0),
  [2644] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_identifier_suffix, 1, 0, 0),
  [2646] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4278),
  [2648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [2650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_splat_parameter, 1, 0, 0),
  [2652] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2392),
  [2654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [2656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5744),
  [2658] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_splat_parameter, 1, 0, 0),
  [2660] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rest_assignment, 1, 0, 0),
  [2662] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5031),
  [2664] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1976),
  [2666] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1977),
  [2668] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5285),
  [2670] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5218),
  [2672] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5222),
  [2674] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5227),
  [2676] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5278),
  [2678] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5279),
  [2680] = {.entry = {.count = 1, .reusable = false}}, SHIFT(417),
  [2682] = {.entry = {.count = 1, .reusable = false}}, SHIFT(403),
  [2684] = {.entry = {.count = 1, .reusable = false}}, SHIFT(412),
  [2686] = {.entry = {.count = 1, .reusable = false}}, SHIFT(413),
  [2688] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1963),
  [2690] = {.entry = {.count = 1, .reusable = false}}, SHIFT(276),
  [2692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(429),
  [2694] = {.entry = {.count = 1, .reusable = false}}, SHIFT(201),
  [2696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5248),
  [2698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5608),
  [2700] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5608),
  [2702] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5098),
  [2704] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4773),
  [2706] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4942),
  [2708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4764),
  [2710] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4762),
  [2712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4775),
  [2714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5420),
  [2716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4569),
  [2718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5419),
  [2720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5042),
  [2722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5064),
  [2724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5044),
  [2726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5045),
  [2728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4983),
  [2730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4986),
  [2732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4762),
  [2734] = {.entry = {.count = 1, .reusable = false}}, SHIFT(681),
  [2736] = {.entry = {.count = 1, .reusable = false}}, SHIFT(676),
  [2738] = {.entry = {.count = 1, .reusable = false}}, SHIFT(750),
  [2740] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_constant, 1, 0, 0),
  [2742] = {.entry = {.count = 1, .reusable = false}}, SHIFT(712),
  [2744] = {.entry = {.count = 1, .reusable = false}}, SHIFT(689),
  [2746] = {.entry = {.count = 1, .reusable = false}}, SHIFT(696),
  [2748] = {.entry = {.count = 1, .reusable = false}}, SHIFT(693),
  [2750] = {.entry = {.count = 1, .reusable = false}}, SHIFT(691),
  [2752] = {.entry = {.count = 1, .reusable = false}}, SHIFT(740),
  [2754] = {.entry = {.count = 1, .reusable = false}}, SHIFT(722),
  [2756] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4667),
  [2758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [2760] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__mlhs, 3, 0, 0),
  [2762] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__mlhs, 3, 0, 0),
  [2764] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__mlhs, 2, 0, 0),
  [2766] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__mlhs, 2, 0, 0),
  [2768] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rest_assignment, 1, 0, 0),
  [2770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5836),
  [2772] = {.entry = {.count = 1, .reusable = false}}, SHIFT(124),
  [2774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1767),
  [2776] = {.entry = {.count = 1, .reusable = false}}, SHIFT(125),
  [2778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1268),
  [2780] = {.entry = {.count = 1, .reusable = false}}, SHIFT(123),
  [2782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1866),
  [2784] = {.entry = {.count = 1, .reusable = false}}, SHIFT(131),
  [2786] = {.entry = {.count = 1, .reusable = false}}, SHIFT(127),
  [2788] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4851),
  [2790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5759),
  [2792] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5104),
  [2794] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5211),
  [2796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4831),
  [2798] = {.entry = {.count = 1, .reusable = false}}, SHIFT(129),
  [2800] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4446),
  [2802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4613),
  [2804] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__array_pattern_n, 2, 0, 77),
  [2806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2161),
  [2808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5765),
  [2810] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__array_pattern_n, 2, 0, 77),
  [2812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3383),
  [2814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4528),
  [2816] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4409),
  [2818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2071),
  [2820] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3383),
  [2822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5238),
  [2824] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4227),
  [2826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4368),
  [2828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4562),
  [2830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4406),
  [2832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5063),
  [2834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5043),
  [2836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5065),
  [2838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5066),
  [2840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4991),
  [2842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4992),
  [2844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4396),
  [2846] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_pattern, 2, 0, 29),
  [2848] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_keyword_pattern, 2, 0, 29),
  [2850] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_keyword_pattern, 2, 0, 28),
  [2852] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4830),
  [2854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4614),
  [2856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2164),
  [2858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4463),
  [2860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5792),
  [2862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3464),
  [2864] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4759),
  [2866] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4701),
  [2868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2076),
  [2870] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3464),
  [2872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5220),
  [2874] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4629),
  [2876] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4709),
  [2878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5864),
  [2880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5780),
  [2882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4566),
  [2884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4699),
  [2886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5085),
  [2888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5142),
  [2890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5110),
  [2892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5202),
  [2894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4984),
  [2896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4987),
  [2898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4702),
  [2900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4769),
  [2902] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4776),
  [2904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3462),
  [2906] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3462),
  [2908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5260),
  [2910] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4637),
  [2912] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4659),
  [2914] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4691),
  [2916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3455),
  [2918] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4528),
  [2920] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3455),
  [2922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5229),
  [2924] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4604),
  [2926] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4621),
  [2928] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4473),
  [2930] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4410),
  [2932] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4605),
  [2934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3369),
  [2936] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4630),
  [2938] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3369),
  [2940] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4563),
  [2942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5108),
  [2944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4630),
  [2946] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__chained_command_call, 3, 0, 49),
  [2948] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__chained_command_call, 3, 0, 49),
  [2950] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4819),
  [2952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4759),
  [2954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4653),
  [2956] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__terminator, 1, 0, 0),
  [2958] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_scope_resolution, 2, 0, 14),
  [2960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_scope_resolution, 2, 0, 14),
  [2962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [2964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [2966] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [2968] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_empty_statement, 1, 0, 0),
  [2970] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__statements_repeat1, 1, 0, 0),
  [2972] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__statements_repeat1, 1, 0, 0),
  [2974] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern_constant_resolution, 2, 0, 14),
  [2976] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_constant_resolution, 2, 0, 14),
  [2978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_element_reference, 3, 0, 48),
  [2980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [2982] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_element_reference, 3, 0, 48),
  [2984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(161),
  [2986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_element_reference, 4, 0, 48),
  [2988] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_element_reference, 4, 0, 48),
  [2990] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern_constant, 1, 0, 0),
  [2992] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2012),
  [2994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4462),
  [2996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4790),
  [2998] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__keyword_variable, 1, 0, 0),
  [3000] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 3, 0, 0),
  [3002] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 3, 0, 0),
  [3004] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 3, 0, 0),
  [3006] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 3, 0, 0),
  [3008] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 2, 0, 0),
  [3010] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 2, 0, 0),
  [3012] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 2, 0, 0),
  [3014] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 2, 0, 0),
  [3016] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 4, 0, 95),
  [3018] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 4, 0, 95),
  [3020] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 4, 0, 95),
  [3022] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 4, 0, 95),
  [3024] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 4, 0, 75),
  [3026] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 4, 0, 75),
  [3028] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 4, 0, 96),
  [3030] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 4, 0, 96),
  [3032] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 4, 0, 97),
  [3034] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 4, 0, 97),
  [3036] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 5, 0, 114),
  [3038] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 5, 0, 114),
  [3040] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 5, 0, 115),
  [3042] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 5, 0, 115),
  [3044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 5, 0, 96),
  [3046] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 5, 0, 96),
  [3048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 6, 0, 121),
  [3050] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 6, 0, 121),
  [3052] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 3, 0, 75),
  [3054] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 3, 0, 75),
  [3056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 3, 0, 75),
  [3058] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 3, 0, 75),
  [3060] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_nil, 1, 0, 0),
  [3062] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_nil, 1, 0, 0),
  [3064] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_block, 3, 0, 16),
  [3066] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_block, 3, 0, 16),
  [3068] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 2, 0, 0),
  [3070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 2, 0, 0),
  [3072] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_element_reference, 5, 0, 103),
  [3074] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_element_reference, 5, 0, 103),
  [3076] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameters, 4, 0, 0),
  [3078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 4, 0, 0),
  [3080] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_expr_basic, 1, 0, 0),
  [3082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_element_reference, 4, 0, 79),
  [3084] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_element_reference, 4, 0, 79),
  [3086] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block, 3, 0, 16),
  [3088] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block, 3, 0, 16),
  [3090] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__primary, 1, 0, 0),
  [3092] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__primary, 1, 0, 0),
  [3094] = {.entry = {.count = 1, .reusable = false}}, SHIFT(787),
  [3096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(848),
  [3098] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 4, 0, 113),
  [3100] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 4, 0, 113),
  [3102] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 8, 0, 126),
  [3104] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 8, 0, 126),
  [3106] = {.entry = {.count = 1, .reusable = false}}, SHIFT(773),
  [3108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(869),
  [3110] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 3, 0, 0),
  [3112] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 3, 0, 0),
  [3114] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 5, 0, 117),
  [3116] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 5, 0, 117),
  [3118] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 4, 0, 0),
  [3120] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 4, 0, 0),
  [3122] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 5, 0, 0),
  [3124] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 5, 0, 0),
  [3126] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 7, 0, 124),
  [3128] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 7, 0, 124),
  [3130] = {.entry = {.count = 1, .reusable = false}}, SHIFT(473),
  [3132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(511),
  [3134] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 5, 0, 119),
  [3136] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 5, 0, 119),
  [3138] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 6, 0, 122),
  [3140] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 6, 0, 122),
  [3142] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 6, 0, 123),
  [3144] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 6, 0, 123),
  [3146] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 7, 0, 125),
  [3148] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 7, 0, 125),
  [3150] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameters, 2, 0, 0),
  [3152] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameters, 2, 0, 0),
  [3154] = {.entry = {.count = 1, .reusable = false}}, SHIFT(476),
  [3156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(502),
  [3158] = {.entry = {.count = 1, .reusable = false}}, SHIFT(771),
  [3160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(860),
  [3162] = {.entry = {.count = 1, .reusable = false}}, SHIFT(462),
  [3164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1960),
  [3166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(493),
  [3168] = {.entry = {.count = 1, .reusable = false}}, SHIFT(464),
  [3170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(497),
  [3172] = {.entry = {.count = 1, .reusable = false}}, SHIFT(790),
  [3174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [3176] = {.entry = {.count = 1, .reusable = false}}, SHIFT(803),
  [3178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(905),
  [3180] = {.entry = {.count = 1, .reusable = false}}, SHIFT(762),
  [3182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(870),
  [3184] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2037),
  [3186] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_range, 2, 0, 17),
  [3188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4409),
  [3190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4381),
  [3192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4560),
  [3194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(290),
  [3196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [3198] = {.entry = {.count = 1, .reusable = false}}, SHIFT(474),
  [3200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(500),
  [3202] = {.entry = {.count = 1, .reusable = false}}, SHIFT(788),
  [3204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(846),
  [3206] = {.entry = {.count = 1, .reusable = false}}, SHIFT(769),
  [3208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(844),
  [3210] = {.entry = {.count = 1, .reusable = false}}, SHIFT(764),
  [3212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [3214] = {.entry = {.count = 1, .reusable = false}}, SHIFT(471),
  [3216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [3218] = {.entry = {.count = 1, .reusable = false}}, SHIFT(481),
  [3220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(513),
  [3222] = {.entry = {.count = 1, .reusable = false}}, SHIFT(792),
  [3224] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [3226] = {.entry = {.count = 1, .reusable = false}}, SHIFT(758),
  [3228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(900),
  [3230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(287),
  [3232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [3234] = {.entry = {.count = 1, .reusable = false}}, SHIFT(794),
  [3236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(875),
  [3238] = {.entry = {.count = 1, .reusable = false}}, SHIFT(463),
  [3240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [3242] = {.entry = {.count = 1, .reusable = false}}, SHIFT(767),
  [3244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [3246] = {.entry = {.count = 1, .reusable = false}}, SHIFT(468),
  [3248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [3250] = {.entry = {.count = 1, .reusable = false}}, SHIFT(469),
  [3252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [3254] = {.entry = {.count = 1, .reusable = false}}, SHIFT(793),
  [3256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(877),
  [3258] = {.entry = {.count = 1, .reusable = false}}, SHIFT(798),
  [3260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(888),
  [3262] = {.entry = {.count = 1, .reusable = false}}, SHIFT(478),
  [3264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(484),
  [3266] = {.entry = {.count = 1, .reusable = false}}, SHIFT(797),
  [3268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(885),
  [3270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_chained_string, 2, 0, 0),
  [3272] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_chained_string, 2, 0, 0),
  [3274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_numeric, 1, 0, 0),
  [3276] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__simple_numeric, 1, 0, 0),
  [3278] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__int_or_float, 1, 0, 0),
  [3280] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__int_or_float, 1, 0, 0),
  [3282] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2049),
  [3284] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2058),
  [3286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_literal, 1, 0, 0),
  [3288] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_chained_string_repeat1, 2, 0, 0),
  [3290] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_chained_string_repeat1, 2, 0, 0),
  [3292] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_chained_string_repeat1, 2, 0, 0), SHIFT_REPEAT(5084),
  [3295] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_redo, 1, 0, 0),
  [3297] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_redo, 1, 0, 0),
  [3299] = {.entry = {.count = 1, .reusable = false}}, SHIFT(459),
  [3301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(507),
  [3303] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 2, 0, 8),
  [3305] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 2, 0, 8),
  [3307] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1084),
  [3309] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 2, 0, 18),
  [3311] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 2, 0, 18),
  [3313] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2060),
  [3315] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 2, 0, 20),
  [3317] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 2, 0, 20),
  [3319] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 3, 0, 23),
  [3321] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 3, 0, 23),
  [3323] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 3, 0, 50),
  [3325] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 3, 0, 50),
  [3327] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_retry, 1, 0, 0),
  [3329] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_retry, 1, 0, 0),
  [3331] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2013),
  [3333] = {.entry = {.count = 1, .reusable = false}}, SHIFT(782),
  [3335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [3337] = {.entry = {.count = 1, .reusable = false}}, SHIFT(472),
  [3339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(492),
  [3341] = {.entry = {.count = 1, .reusable = false}}, SHIFT(775),
  [3343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(911),
  [3345] = {.entry = {.count = 1, .reusable = false}}, SHIFT(475),
  [3347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [3349] = {.entry = {.count = 1, .reusable = false}}, SHIFT(757),
  [3351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [3353] = {.entry = {.count = 1, .reusable = false}}, SHIFT(761),
  [3355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(878),
  [3357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_chained_string_repeat1, 1, 0, 0),
  [3359] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_chained_string_repeat1, 1, 0, 0),
  [3361] = {.entry = {.count = 1, .reusable = false}}, SHIFT(763),
  [3363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(904),
  [3365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(784),
  [3367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(871),
  [3369] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 3, 0, 26),
  [3371] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 3, 0, 26),
  [3373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__numeric, 1, 0, 0),
  [3375] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__numeric, 1, 0, 0),
  [3377] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_method, 2, 0, 13),
  [3379] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_method, 2, 0, 13),
  [3381] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda, 2, 0, 16),
  [3383] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda, 2, 0, 16),
  [3385] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_statements, 2, 0, 0),
  [3387] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_statements, 2, 0, 0),
  [3389] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__primary, 1, 0, 5),
  [3391] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__primary, 1, 0, 5),
  [3393] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 4, 0, 80),
  [3395] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 4, 0, 80),
  [3397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return, 2, 0, 0),
  [3399] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return, 2, 0, 0),
  [3401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 5, 0, 55),
  [3403] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 5, 0, 55),
  [3405] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 5, 0, 26),
  [3407] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 5, 0, 26),
  [3409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 5, 0, 56),
  [3411] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 5, 0, 56),
  [3413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 5, 0, 27),
  [3415] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 5, 0, 27),
  [3417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash, 5, 0, 0),
  [3419] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hash, 5, 0, 0),
  [3421] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 4, 0, 81),
  [3423] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 4, 0, 81),
  [3425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4745),
  [3427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1060),
  [3429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4460),
  [3431] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4460),
  [3433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5628),
  [3435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4823),
  [3437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4445),
  [3439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4444),
  [3441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4074),
  [3443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4443),
  [3445] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 4, 0, 60),
  [3447] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 4, 0, 60),
  [3449] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_delimited_symbol, 2, 0, 8),
  [3451] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_delimited_symbol, 2, 0, 8),
  [3453] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 4, 0, 82),
  [3455] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 4, 0, 82),
  [3457] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_singleton_class, 5, 0, 66),
  [3459] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_singleton_class, 5, 0, 66),
  [3461] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 2, 0, 19),
  [3463] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 2, 0, 19),
  [3465] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash, 2, 0, 0),
  [3467] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class, 5, 0, 83),
  [3469] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class, 5, 0, 83),
  [3471] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class, 5, 0, 84),
  [3473] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class, 5, 0, 84),
  [3475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subshell, 2, 0, 9),
  [3477] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subshell, 2, 0, 9),
  [3479] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 5, 0, 84),
  [3481] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module, 5, 0, 84),
  [3483] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hash, 2, 0, 0),
  [3485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_pattern, 2, 0, 0),
  [3487] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 2, 0, 21),
  [3489] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 2, 0, 21),
  [3491] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield, 2, 0, 0),
  [3493] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_yield, 2, 0, 0),
  [3495] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break, 2, 0, 0),
  [3497] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break, 2, 0, 0),
  [3499] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array, 2, 0, 0),
  [3501] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array, 2, 0, 0),
  [3503] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_pattern, 2, 0, 0),
  [3505] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_next, 2, 0, 0),
  [3507] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_next, 2, 0, 0),
  [3509] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_complex, 2, 0, 0),
  [3511] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_complex, 2, 0, 0),
  [3513] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_complex, 2, 0, 22),
  [3515] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_complex, 2, 0, 22),
  [3517] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if, 5, 0, 85),
  [3519] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if, 5, 0, 85),
  [3521] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if, 5, 0, 86),
  [3523] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if, 5, 0, 86),
  [3525] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rational, 2, 0, 0),
  [3527] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rational, 2, 0, 0),
  [3529] = {.entry = {.count = 1, .reusable = false}}, SHIFT(460),
  [3531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(494),
  [3533] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unless, 5, 0, 85),
  [3535] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unless, 5, 0, 85),
  [3537] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__body_expr, 2, 0, 16),
  [3539] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__body_expr, 2, 0, 16),
  [3541] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_regex, 2, 0, 10),
  [3543] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_regex, 2, 0, 10),
  [3545] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unless, 5, 0, 86),
  [3547] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unless, 5, 0, 86),
  [3549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_delimited_symbol, 3, 0, 23),
  [3551] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_delimited_symbol, 3, 0, 23),
  [3553] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subshell, 3, 0, 24),
  [3555] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subshell, 3, 0, 24),
  [3557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_regex, 3, 0, 25),
  [3559] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_regex, 3, 0, 25),
  [3561] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 3, 0, 0),
  [3563] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 3, 0, 0),
  [3565] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 4, 0, 27),
  [3567] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 4, 0, 27),
  [3569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do, 3, 0, 0),
  [3571] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do, 3, 0, 0),
  [3573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 5, 0, 66),
  [3575] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 5, 0, 66),
  [3577] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__arg, 1, 0, 1),
  [3579] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__arg, 1, 0, 1),
  [3581] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 3, 0, 0),
  [3583] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 3, 0, 0),
  [3585] = {.entry = {.count = 1, .reusable = false}}, SHIFT(479),
  [3587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(509),
  [3589] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__arg, 1, 0, 0),
  [3591] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__arg, 1, 0, 0),
  [3593] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 3, 0, 27),
  [3595] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 3, 0, 27),
  [3597] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__unary_minus_pow, 2, 0, 12),
  [3599] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__unary_minus_pow, 2, 0, 12),
  [3601] = {.entry = {.count = 1, .reusable = false}}, SHIFT(804),
  [3603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(894),
  [3605] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 2, 0, 0),
  [3607] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 2, 0, 0),
  [3609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 3, 0, 60),
  [3611] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 3, 0, 60),
  [3613] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 2, 0, 0),
  [3615] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 2, 0, 0),
  [3617] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash, 3, 0, 0),
  [3619] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hash, 3, 0, 0),
  [3621] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 2, 0, 31),
  [3623] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 2, 0, 31),
  [3625] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_lambda, 1, 0, 0),
  [3627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary, 2, 0, 11),
  [3629] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary, 2, 0, 11),
  [3631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_statements, 3, 0, 0),
  [3633] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_statements, 3, 0, 0),
  [3635] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 5, 0, 87),
  [3637] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 5, 0, 87),
  [3639] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if, 4, 0, 63),
  [3641] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if, 4, 0, 63),
  [3643] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class, 3, 0, 14),
  [3645] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class, 3, 0, 14),
  [3647] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment, 3, 0, 53),
  [3649] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment, 3, 0, 53),
  [3651] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_operator_assignment, 3, 0, 40),
  [3653] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_operator_assignment, 3, 0, 40),
  [3655] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_redo, 2, 0, 0),
  [3657] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_redo, 2, 0, 0),
  [3659] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_retry, 2, 0, 0),
  [3661] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_retry, 2, 0, 0),
  [3663] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 5, 0, 33),
  [3665] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 5, 0, 33),
  [3667] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 5, 0, 91),
  [3669] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 5, 0, 91),
  [3671] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 3, 0, 14),
  [3673] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module, 3, 0, 14),
  [3675] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 5, 0, 92),
  [3677] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 5, 0, 92),
  [3679] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 5, 0, 0),
  [3681] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 5, 0, 0),
  [3683] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 2, 0, 0),
  [3685] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 2, 0, 0),
  [3687] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if, 4, 0, 64),
  [3689] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if, 4, 0, 64),
  [3691] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while, 3, 0, 32),
  [3693] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while, 3, 0, 32),
  [3695] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_until, 3, 0, 32),
  [3697] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_until, 3, 0, 32),
  [3699] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 3, 0, 33),
  [3701] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 3, 0, 33),
  [3703] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 3, 0, 0),
  [3705] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 3, 0, 0),
  [3707] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array, 3, 0, 0),
  [3709] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array, 3, 0, 0),
  [3711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__literal, 1, 0, 0),
  [3713] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__literal, 1, 0, 0),
  [3715] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unless, 4, 0, 63),
  [3717] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unless, 4, 0, 63),
  [3719] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unless, 4, 0, 64),
  [3721] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unless, 4, 0, 64),
  [3723] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do, 2, 0, 0),
  [3725] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do, 2, 0, 0),
  [3727] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_begin, 3, 0, 0),
  [3729] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_begin, 3, 0, 0),
  [3731] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 6, 0, 55),
  [3733] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 6, 0, 55),
  [3735] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 6, 0, 56),
  [3737] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 6, 0, 56),
  [3739] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_singleton_method, 6, 0, 104),
  [3741] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_singleton_method, 6, 0, 104),
  [3743] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0),
  [3745] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0),
  [3747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1885),
  [3749] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 5, 0, 105),
  [3751] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 5, 0, 105),
  [3753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_singleton_class, 6, 0, 106),
  [3755] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_singleton_class, 6, 0, 106),
  [3757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class, 6, 0, 107),
  [3759] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class, 6, 0, 107),
  [3761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 2, 0, 0),
  [3763] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 2, 0, 0),
  [3765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash, 4, 0, 0),
  [3767] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hash, 4, 0, 0),
  [3769] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 6, 0, 66),
  [3771] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 6, 0, 66),
  [3773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 6, 0, 108),
  [3775] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 6, 0, 108),
  [3777] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 6, 0, 109),
  [3779] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 6, 0, 109),
  [3781] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 6, 0, 33),
  [3783] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 6, 0, 33),
  [3785] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 6, 0, 112),
  [3787] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 6, 0, 112),
  [3789] = {.entry = {.count = 1, .reusable = false}}, SHIFT(477),
  [3791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(506),
  [3793] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for, 4, 0, 65),
  [3795] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for, 4, 0, 65),
  [3797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 4, 0, 66),
  [3799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 4, 0, 66),
  [3801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1886),
  [3803] = {.entry = {.count = 1, .reusable = false}}, SHIFT(977),
  [3805] = {.entry = {.count = 1, .reusable = false}}, SHIFT(785),
  [3807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(896),
  [3809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1887),
  [3811] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 7, 0, 66),
  [3813] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 7, 0, 66),
  [3815] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 7, 0, 116),
  [3817] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 7, 0, 116),
  [3819] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 3, 0, 61),
  [3821] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 3, 0, 61),
  [3823] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 4, 0, 33),
  [3825] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 4, 0, 33),
  [3827] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_match, 4, 0, 69),
  [3829] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_match, 4, 0, 69),
  [3831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1888),
  [3833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1889),
  [3835] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case, 4, 0, 0),
  [3837] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case, 4, 0, 0),
  [3839] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_rest, 3, 0, 30),
  [3841] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_rest, 3, 0, 30),
  [3843] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1891),
  [3845] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_singleton_method, 4, 0, 59),
  [3847] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_singleton_method, 4, 0, 59),
  [3849] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_begin, 4, 0, 0),
  [3851] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_begin, 4, 0, 0),
  [3853] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class, 4, 0, 62),
  [3855] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class, 4, 0, 62),
  [3857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1892),
  [3859] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3104),
  [3861] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3131),
  [3863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3104),
  [3865] = {.entry = {.count = 1, .reusable = true}}, SHIFT(422),
  [3867] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_class, 4, 0, 14),
  [3869] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_class, 4, 0, 14),
  [3871] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 4, 0, 62),
  [3873] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module, 4, 0, 62),
  [3875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1893),
  [3877] = {.entry = {.count = 1, .reusable = false}}, SHIFT(772),
  [3879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [3881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1894),
  [3883] = {.entry = {.count = 1, .reusable = false}}, SHIFT(774),
  [3885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [3887] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_module, 4, 0, 14),
  [3889] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_module, 4, 0, 14),
  [3891] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_begin, 2, 0, 0),
  [3893] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_begin, 2, 0, 0),
  [3895] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1895),
  [3897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1890),
  [3899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(480),
  [3901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(508),
  [3903] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 3, 0, 52),
  [3905] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 3, 0, 52),
  [3907] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_literal, 2, 0, 11),
  [3909] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_literal, 2, 0, 11),
  [3911] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call, 3, 0, 54),
  [3913] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call, 3, 0, 54),
  [3915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_unary, 2, 0, 11),
  [3917] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_unary, 2, 0, 11),
  [3919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1881),
  [3921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 3, 0, 0),
  [3923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 3, 0, 0),
  [3925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1896),
  [3927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1897),
  [3929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(912),
  [3931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1898),
  [3933] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1899),
  [3935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1900),
  [3937] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1901),
  [3939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1902),
  [3941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1903),
  [3943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1904),
  [3945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 4, 0, 0),
  [3947] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 4, 0, 0),
  [3949] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 4, 0, 55),
  [3951] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 4, 0, 55),
  [3953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_array, 4, 0, 26),
  [3955] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_array, 4, 0, 26),
  [3957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(913),
  [3959] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 4, 0, 0),
  [3961] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 4, 0, 0),
  [3963] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbol_array, 4, 0, 56),
  [3965] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbol_array, 4, 0, 56),
  [3967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lambda, 3, 0, 37),
  [3969] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_lambda, 3, 0, 37),
  [3971] = {.entry = {.count = 1, .reusable = false}}, SHIFT(461),
  [3973] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [3975] = {.entry = {.count = 1, .reusable = false}}, SHIFT(470),
  [3977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [3979] = {.entry = {.count = 1, .reusable = false}}, SHIFT(466),
  [3981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(486),
  [3983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1656),
  [3985] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1645),
  [3987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(200),
  [3989] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1646),
  [3991] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1647),
  [3993] = {.entry = {.count = 1, .reusable = false}}, SHIFT(200),
  [3995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1645),
  [3997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1879),
  [3999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1648),
  [4001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1649),
  [4003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1650),
  [4005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1647),
  [4007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1642),
  [4009] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1643),
  [4011] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1651),
  [4013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1651),
  [4015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1644),
  [4017] = {.entry = {.count = 1, .reusable = false}}, SHIFT(777),
  [4019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(891),
  [4021] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_right_assignment_list, 2, 0, 0),
  [4023] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_right_assignment_list, 2, 0, 0),
  [4025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(827),
  [4027] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(827),
  [4030] = {.entry = {.count = 1, .reusable = false}}, SHIFT(766),
  [4032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [4034] = {.entry = {.count = 1, .reusable = false}}, SHIFT(783),
  [4036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(862),
  [4038] = {.entry = {.count = 1, .reusable = false}}, SHIFT(779),
  [4040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(843),
  [4042] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range, 3, 0, 41),
  [4044] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range, 3, 0, 41),
  [4046] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary, 3, 0, 40),
  [4048] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary, 3, 0, 40),
  [4050] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_conditional, 5, 0, 102),
  [4052] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_conditional, 5, 0, 102),
  [4054] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_range, 2, 0, 15),
  [4056] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_range, 2, 0, 15),
  [4058] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rescue_modifier_arg, 3, 0, 38),
  [4060] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rescue_modifier_arg, 3, 0, 38),
  [4062] = {.entry = {.count = 1, .reusable = false}}, SHIFT(457),
  [4064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(505),
  [4066] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3133),
  [4068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3134),
  [4070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3133),
  [4072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [4074] = {.entry = {.count = 1, .reusable = false}}, SHIFT(467),
  [4076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [4078] = {.entry = {.count = 1, .reusable = false}}, SHIFT(786),
  [4080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(867),
  [4082] = {.entry = {.count = 1, .reusable = false}}, SHIFT(791),
  [4084] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [4086] = {.entry = {.count = 1, .reusable = false}}, SHIFT(439),
  [4088] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3097),
  [4090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3168),
  [4092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3097),
  [4094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1352),
  [4096] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pow, 3, 0, 40),
  [4098] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pow, 3, 0, 40),
  [4100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1652),
  [4102] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_splat_argument, 2, 0, 0),
  [4104] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_splat_argument, 2, 0, 0),
  [4106] = {.entry = {.count = 1, .reusable = false}}, SHIFT(458),
  [4108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [4110] = {.entry = {.count = 1, .reusable = false}}, SHIFT(465),
  [4112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [4114] = {.entry = {.count = 1, .reusable = false}}, SHIFT(770),
  [4116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(893),
  [4118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1356),
  [4120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(825),
  [4122] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1620),
  [4124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [4126] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1621),
  [4128] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1622),
  [4130] = {.entry = {.count = 1, .reusable = false}}, SHIFT(204),
  [4132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1620),
  [4134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1878),
  [4136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1623),
  [4138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1624),
  [4140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1625),
  [4142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1622),
  [4144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1617),
  [4146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1618),
  [4148] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1626),
  [4150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1626),
  [4152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1619),
  [4154] = {.entry = {.count = 1, .reusable = false}}, SHIFT(781),
  [4156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(849),
  [4158] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(825),
  [4161] = {.entry = {.count = 1, .reusable = false}}, SHIFT(759),
  [4163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(906),
  [4165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1631),
  [4167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(760),
  [4169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [4171] = {.entry = {.count = 1, .reusable = false}}, SHIFT(799),
  [4173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(890),
  [4175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(800),
  [4177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(892),
  [4179] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4864),
  [4181] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4443),
  [4183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1559),
  [4185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3163),
  [4187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3164),
  [4189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3163),
  [4191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(802),
  [4193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [4195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(801),
  [4197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [4199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1263),
  [4201] = {.entry = {.count = 1, .reusable = false}}, SHIFT(765),
  [4203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(889),
  [4205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(778),
  [4207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(887),
  [4209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1627),
  [4211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1473),
  [4213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1474),
  [4215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [4217] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1475),
  [4219] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1476),
  [4221] = {.entry = {.count = 1, .reusable = false}}, SHIFT(245),
  [4223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1474),
  [4225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1477),
  [4227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [4229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1479),
  [4231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1476),
  [4233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1471),
  [4235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1472),
  [4237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1480),
  [4239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1480),
  [4241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1874),
  [4243] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4585),
  [4245] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2036),
  [4247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4772),
  [4249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4840),
  [4251] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4840),
  [4253] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4854),
  [4255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4875),
  [4257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4873),
  [4259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4627),
  [4261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4854),
  [4263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1250),
  [4265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4701),
  [4267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4705),
  [4269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4564),
  [4271] = {.entry = {.count = 1, .reusable = false}}, SHIFT(796),
  [4273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(895),
  [4275] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3107),
  [4277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3203),
  [4279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3203),
  [4281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3202),
  [4283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3208),
  [4285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3189),
  [4287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3181),
  [4289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3202),
  [4291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5086),
  [4293] = {.entry = {.count = 1, .reusable = false}}, SHIFT(776),
  [4295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(873),
  [4297] = {.entry = {.count = 1, .reusable = false}}, SHIFT(768),
  [4299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(866),
  [4301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1481),
  [4303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(780),
  [4305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(902),
  [4307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1490),
  [4309] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1344),
  [4311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(261),
  [4313] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1345),
  [4315] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1346),
  [4317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2021),
  [4319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2022),
  [4321] = {.entry = {.count = 1, .reusable = false}}, SHIFT(261),
  [4323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1344),
  [4325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1871),
  [4327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1347),
  [4329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1348),
  [4331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1349),
  [4333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1346),
  [4335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1341),
  [4337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1342),
  [4339] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1350),
  [4341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1350),
  [4343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1343),
  [4345] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(818),
  [4348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [4350] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1, 0, 0),
  [4352] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1769),
  [4354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(336),
  [4356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(948),
  [4358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(812),
  [4360] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1254),
  [4362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(271),
  [4364] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1255),
  [4366] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1256),
  [4368] = {.entry = {.count = 1, .reusable = false}}, SHIFT(271),
  [4370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1254),
  [4372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1870),
  [4374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1257),
  [4376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1258),
  [4378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1259),
  [4380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1256),
  [4382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1251),
  [4384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1252),
  [4386] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1260),
  [4388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1260),
  [4390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1253),
  [4392] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(837),
  [4395] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pair, 3, 0, 57),
  [4397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rescue_modifier_expression, 3, 0, 38),
  [4399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1422),
  [4401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pair, 3, 0, 58),
  [4403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1783),
  [4405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [4407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1607),
  [4409] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1596),
  [4411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(278),
  [4413] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1597),
  [4415] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1598),
  [4417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(278),
  [4419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1596),
  [4421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1877),
  [4423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1599),
  [4425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1600),
  [4427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1601),
  [4429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1598),
  [4431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1593),
  [4433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1594),
  [4435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1602),
  [4437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1602),
  [4439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1595),
  [4441] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_argument, 2, 0, 0),
  [4443] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_splat_argument, 2, 0, 0),
  [4445] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(812),
  [4448] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3080),
  [4450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3129),
  [4452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3080),
  [4454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1166),
  [4456] = {.entry = {.count = 1, .reusable = false}}, SHIFT(939),
  [4458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1603),
  [4460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(330),
  [4462] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1180),
  [4464] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1192),
  [4466] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1200),
  [4468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1180),
  [4470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1219),
  [4472] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1200),
  [4474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1174),
  [4476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [4478] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1220),
  [4480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1220),
  [4482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1176),
  [4484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(279),
  [4486] = {.entry = {.count = 1, .reusable = false}}, SHIFT(279),
  [4488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1768),
  [4490] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1201),
  [4492] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1202),
  [4494] = {.entry = {.count = 1, .reusable = true}}, SHIFT(814),
  [4496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1540),
  [4498] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1525),
  [4500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [4502] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1526),
  [4504] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1527),
  [4506] = {.entry = {.count = 1, .reusable = false}}, SHIFT(286),
  [4508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1525),
  [4510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1875),
  [4512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1528),
  [4514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1529),
  [4516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1530),
  [4518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1527),
  [4520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1522),
  [4522] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1523),
  [4524] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1531),
  [4526] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1531),
  [4528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1524),
  [4530] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_chained_string_repeat1, 2, 0, 0), SHIFT_REPEAT(5207),
  [4533] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(814),
  [4536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1375),
  [4538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1584),
  [4540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(838),
  [4542] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1848),
  [4544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [4546] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1849),
  [4548] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1850),
  [4550] = {.entry = {.count = 1, .reusable = false}}, SHIFT(312),
  [4552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1848),
  [4554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1880),
  [4556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1851),
  [4558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1852),
  [4560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1853),
  [4562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1850),
  [4564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1845),
  [4566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1846),
  [4568] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1854),
  [4570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1854),
  [4572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1847),
  [4574] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1700),
  [4576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(931),
  [4578] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(838),
  [4581] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1168),
  [4583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1532),
  [4585] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3136),
  [4587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3088),
  [4589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3136),
  [4591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1857),
  [4593] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2047),
  [4595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1405),
  [4597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1406),
  [4599] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(816),
  [4602] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1408),
  [4604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1412),
  [4606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1404),
  [4608] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1407),
  [4610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1407),
  [4612] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1409),
  [4614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1411),
  [4616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1409),
  [4618] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1413),
  [4620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1413),
  [4622] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2055),
  [4624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1273),
  [4626] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1157),
  [4628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [4630] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1158),
  [4632] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1159),
  [4634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2027),
  [4636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2025),
  [4638] = {.entry = {.count = 1, .reusable = false}}, SHIFT(359),
  [4640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1157),
  [4642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1869),
  [4644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1160),
  [4646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1161),
  [4648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1162),
  [4650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1159),
  [4652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1154),
  [4654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1155),
  [4656] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1163),
  [4658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1163),
  [4660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1156),
  [4662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [4664] = {.entry = {.count = 1, .reusable = false}}, SHIFT(310),
  [4666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1873),
  [4668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1410),
  [4670] = {.entry = {.count = 1, .reusable = false}}, SHIFT(933),
  [4672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1801),
  [4674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3144),
  [4676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(423),
  [4678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [4680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3123),
  [4682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1415),
  [4684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1816),
  [4686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1855),
  [4688] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1930),
  [4690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1923),
  [4692] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1923),
  [4694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1926),
  [4696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1927),
  [4698] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1947),
  [4700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1952),
  [4702] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1952),
  [4704] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1943),
  [4706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1946),
  [4708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1953),
  [4710] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4852),
  [4712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4833),
  [4714] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4833),
  [4716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(433),
  [4718] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4835),
  [4720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4882),
  [4722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4848),
  [4724] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2064),
  [4726] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1911),
  [4728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1913),
  [4730] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1913),
  [4732] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1931),
  [4734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1906),
  [4736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1915),
  [4738] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1920),
  [4740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [4742] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1573),
  [4744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(325),
  [4746] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1574),
  [4748] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1575),
  [4750] = {.entry = {.count = 1, .reusable = false}}, SHIFT(325),
  [4752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1573),
  [4754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1876),
  [4756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1576),
  [4758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1577),
  [4760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1578),
  [4762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1575),
  [4764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1570),
  [4766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1571),
  [4768] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1579),
  [4770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1579),
  [4772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1572),
  [4774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1058),
  [4776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1536),
  [4778] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1059),
  [4780] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4766),
  [4782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4813),
  [4784] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4813),
  [4786] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2039),
  [4788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1580),
  [4790] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2078),
  [4792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2070),
  [4794] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2070),
  [4796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2073),
  [4798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2074),
  [4800] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__method_name, 1, 0, 0),
  [4802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__method_name, 1, 0, 0),
  [4804] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3199),
  [4806] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2212),
  [4808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2213),
  [4810] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2213),
  [4812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2197),
  [4814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2211),
  [4816] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2208),
  [4818] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2181),
  [4820] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1919),
  [4822] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2072),
  [4824] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2081),
  [4826] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1936),
  [4828] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4778),
  [4830] = {.entry = {.count = 1, .reusable = false}}, SHIFT(978),
  [4832] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1948),
  [4834] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1912),
  [4836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(333),
  [4838] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1949),
  [4840] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1907),
  [4842] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2270),
  [4844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(823),
  [4846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3087),
  [4848] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(823),
  [4851] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1064),
  [4853] = {.entry = {.count = 1, .reusable = false}}, SHIFT(923),
  [4855] = {.entry = {.count = 1, .reusable = true}}, SHIFT(368),
  [4857] = {.entry = {.count = 1, .reusable = false}}, SHIFT(924),
  [4859] = {.entry = {.count = 1, .reusable = false}}, SHIFT(925),
  [4861] = {.entry = {.count = 1, .reusable = false}}, SHIFT(368),
  [4863] = {.entry = {.count = 1, .reusable = true}}, SHIFT(923),
  [4865] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1867),
  [4867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(926),
  [4869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(927),
  [4871] = {.entry = {.count = 1, .reusable = true}}, SHIFT(928),
  [4873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(925),
  [4875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(920),
  [4877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [4879] = {.entry = {.count = 1, .reusable = false}}, SHIFT(929),
  [4881] = {.entry = {.count = 1, .reusable = true}}, SHIFT(929),
  [4883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(922),
  [4885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(934),
  [4887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(811),
  [4889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1123),
  [4891] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1114),
  [4893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(296),
  [4895] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1115),
  [4897] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1116),
  [4899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(296),
  [4901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1114),
  [4903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1749),
  [4905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1117),
  [4907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1118),
  [4909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1119),
  [4911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1116),
  [4913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1111),
  [4915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1112),
  [4917] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1120),
  [4919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1120),
  [4921] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1113),
  [4923] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3110),
  [4925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3113),
  [4927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3110),
  [4929] = {.entry = {.count = 1, .reusable = true}}, SHIFT(440),
  [4931] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1332),
  [4933] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1323),
  [4935] = {.entry = {.count = 1, .reusable = true}}, SHIFT(309),
  [4937] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1324),
  [4939] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1325),
  [4941] = {.entry = {.count = 1, .reusable = false}}, SHIFT(309),
  [4943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1323),
  [4945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1753),
  [4947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1326),
  [4949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1327),
  [4951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1328),
  [4953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1325),
  [4955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1320),
  [4957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1321),
  [4959] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1329),
  [4961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1329),
  [4963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1322),
  [4965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(826),
  [4967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(836),
  [4969] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1265),
  [4971] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_setter, 2, 0, 30),
  [4973] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_setter, 2, 0, 30),
  [4975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1445),
  [4977] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(826),
  [4980] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(836),
  [4983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1204),
  [4985] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(811),
  [4988] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1453),
  [4990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(304),
  [4992] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1454),
  [4994] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1455),
  [4996] = {.entry = {.count = 1, .reusable = false}}, SHIFT(304),
  [4998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1453),
  [5000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1764),
  [5002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1456),
  [5004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1457),
  [5006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [5008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1455),
  [5010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1450),
  [5012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1451),
  [5014] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1459),
  [5016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1459),
  [5018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1452),
  [5020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1179),
  [5022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(829),
  [5024] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1809),
  [5026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(388),
  [5028] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1810),
  [5030] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1811),
  [5032] = {.entry = {.count = 1, .reusable = false}}, SHIFT(388),
  [5034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1809),
  [5036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1872),
  [5038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1812),
  [5040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1813),
  [5042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1814),
  [5044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1811),
  [5046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1806),
  [5048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1807),
  [5050] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1815),
  [5052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1815),
  [5054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1808),
  [5056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1104),
  [5058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(337),
  [5060] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2033),
  [5062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2030),
  [5064] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2030),
  [5066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2031),
  [5068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2032),
  [5070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(831),
  [5072] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(831),
  [5075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1121),
  [5077] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3167),
  [5079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3166),
  [5081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3167),
  [5083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(427),
  [5085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [5087] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3173),
  [5089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3172),
  [5091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3173),
  [5093] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2087),
  [5095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2102),
  [5097] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2102),
  [5099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2089),
  [5101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2083),
  [5103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2054),
  [5105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2062),
  [5107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2062),
  [5109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2063),
  [5111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2056),
  [5113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1238),
  [5115] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(829),
  [5118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1267),
  [5120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1330),
  [5122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1460),
  [5124] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(833),
  [5127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1839),
  [5129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1002),
  [5131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [5133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1223),
  [5135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1266),
  [5137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(389),
  [5139] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1270),
  [5141] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1441),
  [5143] = {.entry = {.count = 1, .reusable = false}}, SHIFT(389),
  [5145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1266),
  [5147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1470),
  [5149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1538),
  [5151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1628),
  [5153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1697),
  [5155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1441),
  [5157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1165),
  [5159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1169),
  [5161] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1715),
  [5163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1715),
  [5165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1170),
  [5167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1433),
  [5169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(326),
  [5171] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1434),
  [5173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1435),
  [5175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(326),
  [5177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1433),
  [5179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1436),
  [5181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1437),
  [5183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1438),
  [5185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1435),
  [5187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1430),
  [5189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1431),
  [5191] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1439),
  [5193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1439),
  [5195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1432),
  [5197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1760),
  [5199] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1367),
  [5201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [5203] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1368),
  [5205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1369),
  [5207] = {.entry = {.count = 1, .reusable = false}}, SHIFT(305),
  [5209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1367),
  [5211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1757),
  [5213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1370),
  [5215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1371),
  [5217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1372),
  [5219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1369),
  [5221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1364),
  [5223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1365),
  [5225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1373),
  [5227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1373),
  [5229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1366),
  [5231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(976),
  [5233] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(824),
  [5236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1374),
  [5238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1440),
  [5240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3135),
  [5242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(358),
  [5244] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1051),
  [5246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(392),
  [5248] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1052),
  [5250] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1053),
  [5252] = {.entry = {.count = 1, .reusable = false}}, SHIFT(392),
  [5254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1051),
  [5256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1054),
  [5258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1055),
  [5260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1056),
  [5262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1053),
  [5264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1048),
  [5266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1049),
  [5268] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1057),
  [5270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1057),
  [5272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1050),
  [5274] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1304),
  [5276] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1305),
  [5278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1304),
  [5280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1309),
  [5282] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1301),
  [5284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1302),
  [5286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1303),
  [5288] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1868),
  [5290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1313),
  [5292] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1306),
  [5294] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1308),
  [5296] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1306),
  [5298] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1310),
  [5300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1310),
  [5302] = {.entry = {.count = 1, .reusable = true}}, SHIFT(314),
  [5304] = {.entry = {.count = 1, .reusable = false}}, SHIFT(314),
  [5306] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1759),
  [5308] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1307),
  [5310] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1315),
  [5312] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1021),
  [5314] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_chained_string_repeat1, 2, 0, 0), SHIFT_REPEAT(5077),
  [5317] = {.entry = {.count = 1, .reusable = false}}, SHIFT(3143),
  [5319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3081),
  [5321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3143),
  [5323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1695),
  [5325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1396),
  [5327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(841),
  [5329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1838),
  [5331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(981),
  [5333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(339),
  [5335] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1212),
  [5337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(347),
  [5339] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1213),
  [5341] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1214),
  [5343] = {.entry = {.count = 1, .reusable = false}}, SHIFT(347),
  [5345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1212),
  [5347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1758),
  [5349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1215),
  [5351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1216),
  [5353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1217),
  [5355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1214),
  [5357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1209),
  [5359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1210),
  [5361] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1218),
  [5363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1218),
  [5365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1211),
  [5367] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(830),
  [5370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(974),
  [5372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(815),
  [5374] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(815),
  [5377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1221),
  [5379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(441),
  [5381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(830),
  [5383] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(841),
  [5386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(817),
  [5388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1296),
  [5390] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1231),
  [5392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [5394] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1232),
  [5396] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1233),
  [5398] = {.entry = {.count = 1, .reusable = false}}, SHIFT(384),
  [5400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1231),
  [5402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1750),
  [5404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1234),
  [5406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1235),
  [5408] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1236),
  [5410] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [5412] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1228),
  [5414] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1229),
  [5416] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1237),
  [5418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1237),
  [5420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1230),
  [5422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1240),
  [5424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2018),
  [5426] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2020),
  [5428] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1610),
  [5430] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1181),
  [5432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [5434] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1241),
  [5436] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1242),
  [5438] = {.entry = {.count = 1, .reusable = false}}, SHIFT(366),
  [5440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1181),
  [5442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1744),
  [5444] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1249),
  [5446] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1261),
  [5448] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1262),
  [5450] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1242),
  [5452] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1171),
  [5454] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1172),
  [5456] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1264),
  [5458] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1264),
  [5460] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1173),
  [5462] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(817),
  [5465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1312),
  [5467] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5841),
  [5469] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__hash_pattern_body, 3, 0, 0),
  [5471] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__hash_pattern_body, 3, 0, 0),
  [5473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [5475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__hash_pattern_body, 2, 0, 0),
  [5477] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__hash_pattern_body, 2, 0, 0),
  [5479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1096),
  [5481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(387),
  [5483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1097),
  [5485] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1098),
  [5487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2023),
  [5489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2024),
  [5491] = {.entry = {.count = 1, .reusable = false}}, SHIFT(387),
  [5493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1096),
  [5495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1748),
  [5497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1099),
  [5499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1100),
  [5501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1101),
  [5503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1098),
  [5505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1093),
  [5507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1094),
  [5509] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1102),
  [5511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1102),
  [5513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1095),
  [5515] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(834),
  [5518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1707),
  [5520] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(813),
  [5523] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1829),
  [5525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(371),
  [5527] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1830),
  [5529] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1831),
  [5531] = {.entry = {.count = 1, .reusable = false}}, SHIFT(371),
  [5533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1829),
  [5535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1865),
  [5537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1832),
  [5539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1833),
  [5541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1834),
  [5543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1831),
  [5545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1826),
  [5547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1827),
  [5549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1835),
  [5551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1835),
  [5553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1828),
  [5555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1587),
  [5557] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1131),
  [5559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(372),
  [5561] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1132),
  [5563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1133),
  [5565] = {.entry = {.count = 1, .reusable = false}}, SHIFT(372),
  [5567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1131),
  [5569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1762),
  [5571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1134),
  [5573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1135),
  [5575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1136),
  [5577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1133),
  [5579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1128),
  [5581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1129),
  [5583] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1137),
  [5585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1137),
  [5587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1130),
  [5589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1009),
  [5591] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1010),
  [5593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(367),
  [5595] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1011),
  [5597] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1012),
  [5599] = {.entry = {.count = 1, .reusable = false}}, SHIFT(367),
  [5601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1010),
  [5603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1747),
  [5605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1014),
  [5607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1015),
  [5609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1016),
  [5611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1012),
  [5613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1007),
  [5615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1008),
  [5617] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1017),
  [5619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1017),
  [5621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1798),
  [5623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1728),
  [5625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(813),
  [5627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1041),
  [5629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(839),
  [5631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1018),
  [5633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [5635] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(821),
  [5638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1836),
  [5640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(821),
  [5642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(344),
  [5644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1038),
  [5646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1512),
  [5648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(835),
  [5650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1141),
  [5652] = {.entry = {.count = 1, .reusable = false}}, SHIFT(965),
  [5654] = {.entry = {.count = 1, .reusable = false}}, SHIFT(967),
  [5656] = {.entry = {.count = 1, .reusable = false}}, SHIFT(968),
  [5658] = {.entry = {.count = 1, .reusable = true}}, SHIFT(965),
  [5660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(971),
  [5662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(972),
  [5664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(968),
  [5666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(962),
  [5668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(963),
  [5670] = {.entry = {.count = 1, .reusable = false}}, SHIFT(973),
  [5672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(973),
  [5674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(964),
  [5676] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1283),
  [5678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(375),
  [5680] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1284),
  [5682] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1285),
  [5684] = {.entry = {.count = 1, .reusable = false}}, SHIFT(375),
  [5686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1283),
  [5688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1756),
  [5690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1287),
  [5692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1288),
  [5694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1289),
  [5696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1285),
  [5698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1280),
  [5700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1281),
  [5702] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1290),
  [5704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1290),
  [5706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1282),
  [5708] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1387),
  [5710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(382),
  [5712] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1388),
  [5714] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1389),
  [5716] = {.entry = {.count = 1, .reusable = false}}, SHIFT(382),
  [5718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1387),
  [5720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1390),
  [5722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1391),
  [5724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1392),
  [5726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1389),
  [5728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1384),
  [5730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1385),
  [5732] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1393),
  [5734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1393),
  [5736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1386),
  [5738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1763),
  [5740] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1491),
  [5742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1521),
  [5744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [5746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1485),
  [5748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1486),
  [5750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2026),
  [5752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2028),
  [5754] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(835),
  [5757] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1487),
  [5759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(376),
  [5761] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1509),
  [5763] = {.entry = {.count = 1, .reusable = false}}, SHIFT(376),
  [5765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1487),
  [5767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1513),
  [5769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1520),
  [5771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1509),
  [5773] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1533),
  [5775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1533),
  [5777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1734),
  [5779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(393),
  [5781] = {.entry = {.count = 1, .reusable = false}}, SHIFT(393),
  [5783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(970),
  [5785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(332),
  [5787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1742),
  [5789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1394),
  [5791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1720),
  [5793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1138),
  [5795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1541),
  [5797] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(839),
  [5800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(999),
  [5802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1291),
  [5804] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1029),
  [5806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(394),
  [5808] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1030),
  [5810] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1031),
  [5812] = {.entry = {.count = 1, .reusable = false}}, SHIFT(394),
  [5814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1029),
  [5816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1754),
  [5818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1032),
  [5820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1033),
  [5822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1034),
  [5824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1031),
  [5826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1026),
  [5828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1027),
  [5830] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1035),
  [5832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1035),
  [5834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1028),
  [5836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1736),
  [5838] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1674),
  [5840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(396),
  [5842] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1678),
  [5844] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1679),
  [5846] = {.entry = {.count = 1, .reusable = false}}, SHIFT(396),
  [5848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1674),
  [5850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1741),
  [5852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1683),
  [5854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1686),
  [5856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1687),
  [5858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1679),
  [5860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1667),
  [5862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1670),
  [5864] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1690),
  [5866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1690),
  [5868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1671),
  [5870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1106),
  [5872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(832),
  [5874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1884),
  [5876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1780),
  [5878] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1551),
  [5880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(395),
  [5882] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1552),
  [5884] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1553),
  [5886] = {.entry = {.count = 1, .reusable = false}}, SHIFT(395),
  [5888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1551),
  [5890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1765),
  [5892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1554),
  [5894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1555),
  [5896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1556),
  [5898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1553),
  [5900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1548),
  [5902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1549),
  [5904] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1557),
  [5906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1557),
  [5908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1550),
  [5910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [5912] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_exceptions, 1, 0, 0),
  [5914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1682),
  [5916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1036),
  [5918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1199),
  [5920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [5922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(341),
  [5924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1558),
  [5926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1804),
  [5928] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1501),
  [5930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(400),
  [5932] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1502),
  [5934] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1503),
  [5936] = {.entry = {.count = 1, .reusable = false}}, SHIFT(400),
  [5938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1501),
  [5940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1761),
  [5942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1504),
  [5944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1505),
  [5946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1506),
  [5948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1503),
  [5950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1498),
  [5952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1499),
  [5954] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1507),
  [5956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1507),
  [5958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1500),
  [5960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(944),
  [5962] = {.entry = {.count = 1, .reusable = false}}, SHIFT(949),
  [5964] = {.entry = {.count = 1, .reusable = true}}, SHIFT(956),
  [5966] = {.entry = {.count = 1, .reusable = true}}, SHIFT(942),
  [5968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(943),
  [5970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(335),
  [5972] = {.entry = {.count = 1, .reusable = false}}, SHIFT(945),
  [5974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(945),
  [5976] = {.entry = {.count = 1, .reusable = false}}, SHIFT(950),
  [5978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(955),
  [5980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(950),
  [5982] = {.entry = {.count = 1, .reusable = false}}, SHIFT(961),
  [5984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(961),
  [5986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(401),
  [5988] = {.entry = {.count = 1, .reusable = false}}, SHIFT(401),
  [5990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1740),
  [5992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(954),
  [5994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(340),
  [5996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1508),
  [5998] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(832),
  [6001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1088),
  [6003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1013),
  [6005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1083),
  [6007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1510),
  [6009] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1791),
  [6011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(416),
  [6013] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1792),
  [6015] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1793),
  [6017] = {.entry = {.count = 1, .reusable = false}}, SHIFT(416),
  [6019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1791),
  [6021] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1864),
  [6023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1794),
  [6025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1795),
  [6027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1796),
  [6029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1793),
  [6031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1788),
  [6033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1789),
  [6035] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1797),
  [6037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1797),
  [6039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1790),
  [6041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1718),
  [6043] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1719),
  [6045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [6047] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1721),
  [6049] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1722),
  [6051] = {.entry = {.count = 1, .reusable = false}}, SHIFT(405),
  [6053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1719),
  [6055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1723),
  [6057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1724),
  [6059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1725),
  [6061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1722),
  [6063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1716),
  [6065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1717),
  [6067] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1727),
  [6069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1727),
  [6071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1745),
  [6073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(840),
  [6075] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pattern, 1, 0, 0),
  [6077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1379),
  [6079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(819),
  [6081] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1469),
  [6083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(420),
  [6085] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1482),
  [6087] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1483),
  [6089] = {.entry = {.count = 1, .reusable = false}}, SHIFT(420),
  [6091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1469),
  [6093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1751),
  [6095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1488),
  [6097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1534),
  [6099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1535),
  [6101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1483),
  [6103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1442),
  [6105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1443),
  [6107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1537),
  [6109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1537),
  [6111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1463),
  [6113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1286),
  [6115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1782),
  [6117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1611),
  [6119] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(840),
  [6122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1563),
  [6124] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1581),
  [6126] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1629),
  [6128] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1654),
  [6130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1581),
  [6132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1691),
  [6134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1654),
  [6136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1560),
  [6138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1561),
  [6140] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1694),
  [6142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1694),
  [6144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1564),
  [6146] = {.entry = {.count = 1, .reusable = false}}, SHIFT(989),
  [6148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(418),
  [6150] = {.entry = {.count = 1, .reusable = false}}, SHIFT(990),
  [6152] = {.entry = {.count = 1, .reusable = false}}, SHIFT(991),
  [6154] = {.entry = {.count = 1, .reusable = false}}, SHIFT(418),
  [6156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(989),
  [6158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1746),
  [6160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(993),
  [6162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(994),
  [6164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(995),
  [6166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(991),
  [6168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(986),
  [6170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(987),
  [6172] = {.entry = {.count = 1, .reusable = false}}, SHIFT(996),
  [6174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(996),
  [6176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(988),
  [6178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(421),
  [6180] = {.entry = {.count = 1, .reusable = false}}, SHIFT(421),
  [6182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1710),
  [6184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1666),
  [6186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1675),
  [6188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(953),
  [6190] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(819),
  [6193] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optional_parameter, 3, 0, 76),
  [6195] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_parameter, 3, 0, 76),
  [6197] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_in, 2, 0, 0),
  [6199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(992),
  [6201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(969),
  [6203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1772),
  [6205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1191),
  [6207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(434),
  [6209] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1193),
  [6211] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1194),
  [6213] = {.entry = {.count = 1, .reusable = false}}, SHIFT(434),
  [6215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1191),
  [6217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1195),
  [6219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1196),
  [6221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1197),
  [6223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1194),
  [6225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1188),
  [6227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1189),
  [6229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1198),
  [6231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1198),
  [6233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1190),
  [6235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1778),
  [6237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1770),
  [6239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1771),
  [6241] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1773),
  [6243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(437),
  [6245] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1774),
  [6247] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1775),
  [6249] = {.entry = {.count = 1, .reusable = false}}, SHIFT(437),
  [6251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1773),
  [6253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1752),
  [6255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1776),
  [6257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1777),
  [6259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1775),
  [6261] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1779),
  [6263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1779),
  [6265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1755),
  [6267] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_optional_parameter, 3, 0, 76),
  [6269] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_keyword_parameter, 3, 0, 76),
  [6271] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1074),
  [6273] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1075),
  [6275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1074),
  [6277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1080),
  [6279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1071),
  [6281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1072),
  [6283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1073),
  [6285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [6287] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1076),
  [6289] = {.entry = {.count = 1, .reusable = false}}, SHIFT(442),
  [6291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1743),
  [6293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1331),
  [6295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1077),
  [6297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1078),
  [6299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1079),
  [6301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1076),
  [6303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1081),
  [6305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1081),
  [6307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1037),
  [6309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1203),
  [6311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1295),
  [6313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1378),
  [6315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5631),
  [6317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1663),
  [6319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1314),
  [6321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1444),
  [6323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1511),
  [6325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1140),
  [6327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1395),
  [6329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1461),
  [6331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1562),
  [6333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1353),
  [6335] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_splat_parameter, 2, 0, 14),
  [6337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1604),
  [6339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1882),
  [6341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1705),
  [6343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1087),
  [6345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(946),
  [6347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1062),
  [6349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1293),
  [6351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1177),
  [6353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1733),
  [6355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(979),
  [6357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1271),
  [6359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1354),
  [6361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1421),
  [6363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1000),
  [6365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [6367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1539),
  [6369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1583),
  [6371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1606),
  [6373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1630),
  [6375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1655),
  [6377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5620),
  [6379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(930),
  [6381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1039),
  [6383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(939),
  [6385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5578),
  [6387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1019),
  [6389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5862),
  [6391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1105),
  [6393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1799),
  [6395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1837),
  [6397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1122),
  [6399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1817),
  [6401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1856),
  [6403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5579),
  [6405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5624),
  [6407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5583),
  [6409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5637),
  [6411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5597),
  [6413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5626),
  [6415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1239),
  [6417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5635),
  [6419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5586),
  [6421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5584),
  [6423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5580),
  [6425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5610),
  [6427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1582),
  [6429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1781),
  [6431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5592),
  [6433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5596),
  [6435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5600),
  [6437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5577),
  [6439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5607),
  [6441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5611),
  [6443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5614),
  [6445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5616),
  [6447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5619),
  [6449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5623),
  [6451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1222),
  [6453] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_constant_resolution, 3, 0, 51),
  [6455] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_value, 1, 0, 0),
  [6457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5677),
  [6459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2015),
  [6461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2014),
  [6463] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_constant, 1, 0, 47),
  [6465] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__keyword_variable, 1, 0, 0),
  [6467] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern_literal, 1, 0, 0),
  [6469] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_primitive, 1, 0, 0),
  [6471] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern_primitive, 1, 0, 0),
  [6473] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__pattern_lambda, 1, 0, 0),
  [6475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_call_with_block, 3, 0, 54),
  [6477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2013),
  [6479] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_call_with_block, 3, 0, 52),
  [6481] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_value, 1, 0, 46),
  [6483] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variable_reference_pattern, 2, 0, 14),
  [6485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_call, 2, 0, 18),
  [6487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(294),
  [6489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(164),
  [6491] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_range, 2, 0, 15),
  [6493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2167),
  [6495] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2167),
  [6497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_pattern, 3, 0, 0),
  [6499] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_pattern, 3, 0, 0),
  [6501] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_pattern, 3, 0, 0),
  [6503] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_reference_pattern, 4, 0, 66),
  [6505] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_pattern, 4, 0, 101),
  [6507] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_alternative_pattern_repeat1, 2, 0, 99),
  [6509] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_range, 3, 0, 41),
  [6511] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5773),
  [6513] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_pattern, 3, 0, 101),
  [6515] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_call, 2, 0, 20),
  [6517] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_find_pattern, 4, 0, 101),
  [6519] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_pattern, 4, 0, 101),
  [6521] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_find_pattern, 3, 0, 0),
  [6523] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_alternative_pattern, 2, 0, 78),
  [6525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2192),
  [6527] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1, 0, 3),
  [6529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3262),
  [6531] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_as_pattern, 3, 0, 98),
  [6533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2012),
  [6535] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_expr_alt, 1, 0, 0),
  [6537] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_alternative_pattern_repeat1, 2, 0, 100),
  [6539] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_alternative_pattern_repeat1, 2, 0, 100), SHIFT_REPEAT(2192),
  [6542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_pattern, 3, 0, 57),
  [6544] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5747),
  [6546] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_keyword_pattern, 3, 0, 58),
  [6548] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__array_pattern_n_repeat1, 2, 0, 0),
  [6550] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_expr, 1, 0, 0),
  [6552] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_splat_nil, 2, 0, 0),
  [6554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__argument, 1, 0, 0),
  [6556] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__hash_pattern_any_rest, 1, 0, 0),
  [6558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_unary, 2, 0, 11),
  [6560] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_top_expr_body, 1, 0, 44),
  [6562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_yield_command, 2, 0, 0),
  [6564] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pair, 3, 0, 28),
  [6566] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_top_expr_body, 1, 0, 45),
  [6568] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__hash_pattern_body, 1, 0, 0),
  [6570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_match_pattern, 3, 0, 42),
  [6572] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_command, 2, 0, 0),
  [6574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_command, 2, 0, 0),
  [6576] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__array_pattern_n, 3, 0, 0),
  [6578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pair, 3, 0, 29),
  [6580] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5892),
  [6582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__hash_pattern_body_repeat1, 2, 0, 0),
  [6584] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__hash_pattern_body, 4, 0, 0),
  [6586] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_splat_parameter, 2, 0, 14),
  [6588] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0),
  [6590] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5839),
  [6592] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_test_pattern, 3, 0, 42),
  [6594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_assignment, 3, 0, 53),
  [6596] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_operator_assignment, 3, 0, 40),
  [6598] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_binary, 3, 0, 40),
  [6600] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_next_command, 2, 0, 0),
  [6602] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_top_expr_body, 1, 0, 43),
  [6604] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4499),
  [6606] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_hash_splat_parameter, 1, 0, 0),
  [6608] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_hash_splat_parameter, 1, 0, 0),
  [6610] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4481),
  [6612] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__find_pattern_body, 4, 0, 0),
  [6614] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__array_pattern_n_repeat1, 2, 0, 0), SHIFT_REPEAT(2157),
  [6617] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(451),
  [6620] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_argument_list, 1, 0, 0),
  [6622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(451),
  [6624] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__array_pattern_n, 1, 0, 0),
  [6626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2157),
  [6628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3604),
  [6630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__array_pattern_n, 2, 0, 0),
  [6632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2133),
  [6634] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__hash_pattern_body_repeat1, 2, 0, 0), SHIFT_REPEAT(4753),
  [6637] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4503),
  [6639] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__pattern_top_expr_body, 1, 0, 0),
  [6641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2011),
  [6643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3609),
  [6645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_command_argument_list, 2, 0, 0),
  [6647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5309),
  [6649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4611),
  [6651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5533),
  [6653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5388),
  [6655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1355),
  [6657] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5318),
  [6659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4616),
  [6661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5270),
  [6663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5196),
  [6665] = {.entry = {.count = 1, .reusable = false}}, SHIFT(164),
  [6667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(450),
  [6669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [6671] = {.entry = {.count = 1, .reusable = false}}, SHIFT(155),
  [6673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(292),
  [6675] = {.entry = {.count = 1, .reusable = false}}, SHIFT(148),
  [6677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(291),
  [6679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(141),
  [6681] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_undef_repeat1, 2, 0, 0),
  [6683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [6685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(158),
  [6687] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(450),
  [6690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5315),
  [6692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5633),
  [6694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2154),
  [6696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5730),
  [6698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_begin_block, 4, 0, 0),
  [6700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1272),
  [6702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5627),
  [6704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2129),
  [6706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5937),
  [6708] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_undef_repeat1, 2, 0, 0), SHIFT_REPEAT(2628),
  [6711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_block, 3, 0, 0),
  [6713] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_undef, 3, 0, 0),
  [6715] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2628),
  [6717] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statement, 1, 0, 0),
  [6719] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_begin_block, 3, 0, 0),
  [6721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_undef, 2, 0, 0),
  [6723] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_alias, 3, 0, 35),
  [6725] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_end_block, 4, 0, 0),
  [6727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2036),
  [6729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4455),
  [6731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2120),
  [6733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5960),
  [6735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [6737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2163),
  [6739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5871),
  [6741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2122),
  [6743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5961),
  [6745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1357),
  [6747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2280),
  [6749] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2280),
  [6751] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_modifier, 3, 0, 39),
  [6753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rescue_modifier, 3, 0, 38),
  [6755] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unless_modifier, 3, 0, 39),
  [6757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_modifier, 3, 0, 39),
  [6759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2158),
  [6761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5681),
  [6763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_until_modifier, 3, 0, 39),
  [6765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2991),
  [6767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [6769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [6771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [6773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [6775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [6777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2019),
  [6779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(810),
  [6781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(268),
  [6783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5427),
  [6785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2037),
  [6787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2187),
  [6789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3303),
  [6791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2885),
  [6793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5371),
  [6795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2377),
  [6797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5461),
  [6799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2105),
  [6801] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_alternative_pattern_repeat1, 2, 0, 100), SHIFT_REPEAT(2187),
  [6804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4468),
  [6806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4786),
  [6808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2405),
  [6810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5242),
  [6812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5542),
  [6814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4617),
  [6816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5318),
  [6818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__statements, 2, 0, 0),
  [6820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2029),
  [6822] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__hash_pattern_body_repeat1, 2, 0, 0), SHIFT_REPEAT(4774),
  [6825] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(444),
  [6828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(444),
  [6830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2155),
  [6832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2134),
  [6834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4332),
  [6836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4310),
  [6838] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__array_pattern_n_repeat1, 2, 0, 0), SHIFT_REPEAT(2155),
  [6841] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2047),
  [6843] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1178),
  [6845] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [6847] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5821),
  [6849] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2017),
  [6851] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2016),
  [6853] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2529),
  [6855] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2529),
  [6857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2185),
  [6859] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_alternative_pattern_repeat1, 2, 0, 100), SHIFT_REPEAT(2185),
  [6862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2049),
  [6864] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4803),
  [6866] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4802),
  [6868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(445),
  [6870] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(445),
  [6873] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2055),
  [6875] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2064),
  [6877] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [6879] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2642),
  [6881] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2642),
  [6883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(947),
  [6885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2637),
  [6887] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2637),
  [6889] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(449),
  [6892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(583),
  [6894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2189),
  [6896] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__hash_pattern_body_repeat1, 2, 0, 0), SHIFT_REPEAT(4785),
  [6899] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4846),
  [6901] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2633),
  [6903] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_alternative_pattern_repeat1, 2, 0, 100), SHIFT_REPEAT(2189),
  [6906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2153),
  [6908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(932),
  [6910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4844),
  [6912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2058),
  [6914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2186),
  [6916] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__array_pattern_n_repeat1, 2, 0, 0), SHIFT_REPEAT(2153),
  [6919] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2038),
  [6921] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_alternative_pattern_repeat1, 2, 0, 100), SHIFT_REPEAT(2186),
  [6924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3301),
  [6926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1818),
  [6928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4458),
  [6930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [6932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [6934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4459),
  [6936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2137),
  [6938] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_undef_repeat1, 2, 0, 0), SHIFT_REPEAT(2633),
  [6941] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [6943] = {.entry = {.count = 1, .reusable = true}}, SHIFT(578),
  [6945] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [6947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(580),
  [6949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(581),
  [6951] = {.entry = {.count = 1, .reusable = true}}, SHIFT(360),
  [6953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [6955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [6957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [6959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5770),
  [6961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2060),
  [6963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [6965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [6967] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(443),
  [6970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1063),
  [6972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [6974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(331),
  [6976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [6978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2142),
  [6980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [6982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [6984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(564),
  [6986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [6988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(334),
  [6990] = {.entry = {.count = 1, .reusable = false}}, SHIFT(354),
  [6992] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5168),
  [6994] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5234),
  [6996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5234),
  [6998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4707),
  [7000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5329),
  [7002] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5050),
  [7004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4685),
  [7006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2950),
  [7008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2147),
  [7010] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5209),
  [7012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2285),
  [7014] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5203),
  [7016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2468),
  [7018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2040),
  [7020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2886),
  [7022] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(448),
  [7025] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5144),
  [7027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5434),
  [7029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(452),
  [7031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [7033] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(452),
  [7036] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5119),
  [7038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4422),
  [7040] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5127),
  [7042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4698),
  [7044] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5121),
  [7046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4426),
  [7048] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5156),
  [7050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4736),
  [7052] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5174),
  [7054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2967),
  [7056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2417),
  [7058] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5149),
  [7060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2964),
  [7062] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5090),
  [7064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4404),
  [7066] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5192),
  [7068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2909),
  [7070] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__hash_pattern_body_repeat1, 2, 0, 0), SHIFT_REPEAT(4822),
  [7073] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5170),
  [7075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3071),
  [7077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2339),
  [7079] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5099),
  [7081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2344),
  [7083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4514),
  [7085] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5164),
  [7087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5474),
  [7089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1001),
  [7091] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5180),
  [7093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5481),
  [7095] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5088),
  [7097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4394),
  [7099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5426),
  [7101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5208),
  [7103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2472),
  [7105] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__array_pattern_n_repeat1, 2, 0, 0), SHIFT_REPEAT(2147),
  [7108] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5155),
  [7110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5472),
  [7112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4504),
  [7114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4495),
  [7116] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__hash_pattern_body_repeat1, 2, 0, 0), SHIFT_REPEAT(4811),
  [7119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5445),
  [7121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4505),
  [7123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2135),
  [7125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2152),
  [7127] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_symbol_array_repeat1, 2, 0, 27),
  [7129] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_symbol_array_repeat1, 2, 0, 27),
  [7131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1605),
  [7133] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__body_statement, 1, 0, 0),
  [7135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [7137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [7139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(456),
  [7141] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__literal_contents, 2, 0, 0), SHIFT_REPEAT(354),
  [7144] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__literal_contents, 2, 0, 0),
  [7146] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__literal_contents, 2, 0, 0), SHIFT_REPEAT(5234),
  [7149] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__literal_contents, 2, 0, 0), SHIFT_REPEAT(5234),
  [7152] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__literal_contents, 2, 0, 0),
  [7154] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__literal_contents, 2, 0, 0), SHIFT_REPEAT(5329),
  [7157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1735),
  [7159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1085),
  [7161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2045),
  [7163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5891),
  [7165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2957),
  [7167] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(456),
  [7170] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4982),
  [7172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2347),
  [7174] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5008),
  [7176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2349),
  [7178] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5822),
  [7180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2965),
  [7182] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5884),
  [7184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2336),
  [7186] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4999),
  [7188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5391),
  [7190] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5025),
  [7192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4725),
  [7194] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5000),
  [7196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5412),
  [7198] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5026),
  [7200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4727),
  [7202] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5036),
  [7204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4413),
  [7206] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4993),
  [7208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4414),
  [7210] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5752),
  [7212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4380),
  [7214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__body_statement, 2, 0, 0),
  [7216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(453),
  [7218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__body_statement_repeat1, 2, 0, 0),
  [7220] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__body_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(677),
  [7223] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__body_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(268),
  [7226] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__body_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(299),
  [7229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5769),
  [7231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5464),
  [7233] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5791),
  [7235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5478),
  [7237] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4981),
  [7239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2915),
  [7241] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__array_pattern_n_repeat1, 2, 0, 0), SHIFT_REPEAT(2152),
  [7244] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5745),
  [7246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2341),
  [7248] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rest_assignment, 2, 0, 0),
  [7250] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(453),
  [7253] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__mlhs_repeat1, 2, 0, 0),
  [7255] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5910),
  [7257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4679),
  [7259] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5671),
  [7261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4729),
  [7263] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4971),
  [7265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3047),
  [7267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2138),
  [7269] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_string_array_repeat1, 2, 0, 26),
  [7271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_array_repeat1, 2, 0, 26),
  [7273] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5818),
  [7275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4388),
  [7277] = {.entry = {.count = 1, .reusable = false}}, SHIFT(357),
  [7279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5307),
  [7281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2835),
  [7283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5449),
  [7285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2313),
  [7287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [7289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5292),
  [7291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4135),
  [7293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5377),
  [7295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5378),
  [7297] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_then, 3, 0, 0),
  [7299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2385),
  [7301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2331),
  [7303] = {.entry = {.count = 1, .reusable = false}}, SHIFT(348),
  [7305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5303),
  [7307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5987),
  [7309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5403),
  [7311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4711),
  [7313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [7315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2386),
  [7317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4364),
  [7319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5414),
  [7321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4407),
  [7323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4412),
  [7325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4362),
  [7327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5487),
  [7329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4384),
  [7331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4385),
  [7333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5988),
  [7335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3744),
  [7337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4719),
  [7339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4720),
  [7341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3045),
  [7343] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(447),
  [7346] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2252),
  [7348] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4660),
  [7350] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3180),
  [7352] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4423),
  [7354] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5293),
  [7356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4427),
  [7358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4196),
  [7360] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5460),
  [7362] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5463),
  [7364] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_then, 2, 0, 0),
  [7366] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2259),
  [7368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2337),
  [7370] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2939),
  [7372] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2940),
  [7374] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5362),
  [7376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5381),
  [7378] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4599),
  [7380] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4706),
  [7382] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4723),
  [7384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3046),
  [7386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4666),
  [7388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3076),
  [7390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2985),
  [7392] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_exception_variable, 2, 0, 0),
  [7394] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4389),
  [7396] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4390),
  [7398] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4391),
  [7400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4393),
  [7402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4700),
  [7404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4402),
  [7406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4403),
  [7408] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_chained_string_repeat1, 2, 0, 0), SHIFT_REPEAT(5042),
  [7411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3187),
  [7413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2333),
  [7415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1706),
  [7417] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(454),
  [7420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4676),
  [7422] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5458),
  [7424] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2951),
  [7426] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_heredoc_body_repeat1, 2, 0, 0), SHIFT_REPEAT(348),
  [7429] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_heredoc_body_repeat1, 2, 0, 0), SHIFT_REPEAT(5303),
  [7432] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_heredoc_body_repeat1, 2, 0, 0),
  [7434] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_heredoc_body_repeat1, 2, 0, 0), SHIFT_REPEAT(5403),
  [7437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2881),
  [7439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4678),
  [7441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3606),
  [7443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5477),
  [7445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5490),
  [7447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4713),
  [7449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(454),
  [7451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4587),
  [7453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4737),
  [7455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4738),
  [7457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5394),
  [7459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2335),
  [7461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4682),
  [7463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2921),
  [7465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2936),
  [7467] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__literal_contents, 2, 0, 0), SHIFT_REPEAT(357),
  [7470] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__literal_contents, 2, 0, 0), SHIFT_REPEAT(5307),
  [7473] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__literal_contents, 2, 0, 0), SHIFT_REPEAT(5449),
  [7476] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2922),
  [7478] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2303),
  [7480] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5491),
  [7482] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4703),
  [7484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2334),
  [7486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2962),
  [7488] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2923),
  [7490] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5247),
  [7492] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameter, 1, 0, 0),
  [7494] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_block_parameter, 1, 0, 0),
  [7496] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4724),
  [7498] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2295),
  [7500] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2924),
  [7502] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5411),
  [7504] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2296),
  [7506] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2866),
  [7508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2297),
  [7510] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2469),
  [7512] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2298),
  [7514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [7516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5436),
  [7518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5438),
  [7520] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_exceptions_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [7523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1883),
  [7525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2355),
  [7527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5466),
  [7529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2398),
  [7531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2399),
  [7533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5492),
  [7535] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__literal_contents, 1, 0, 0),
  [7537] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__literal_contents, 1, 0, 0),
  [7539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4791),
  [7541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [7543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4824),
  [7545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1040),
  [7547] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interpolation, 3, 0, 0),
  [7549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpolation, 3, 0, 0),
  [7551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5380),
  [7553] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_interpolation, 2, 0, 0),
  [7555] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_interpolation, 2, 0, 0),
  [7557] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__formal_parameter, 1, 0, 36),
  [7559] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__formal_parameter, 1, 0, 0),
  [7561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2990),
  [7563] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_parameter, 2, 0, 14),
  [7565] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_exceptions, 2, 0, 0),
  [7567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2378),
  [7569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2992),
  [7571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1294),
  [7573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5339),
  [7575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4439),
  [7577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [7579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2362),
  [7581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2363),
  [7583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2887),
  [7585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0),
  [7587] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_forward_parameter, 1, 0, 0),
  [7589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2952),
  [7591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2953),
  [7593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2418),
  [7595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2889),
  [7597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2422),
  [7599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3003),
  [7601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2954),
  [7603] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__simple_formal_parameter, 1, 0, 0),
  [7605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5432),
  [7607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5366),
  [7609] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(446),
  [7612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5398),
  [7614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(411),
  [7616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2878),
  [7618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2880),
  [7620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2356),
  [7622] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__mlhs_repeat1, 2, 0, 0), SHIFT_REPEAT(1973),
  [7625] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_heredoc_body_repeat1, 1, 0, 0),
  [7627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_heredoc_body_repeat1, 1, 0, 0),
  [7629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1959),
  [7631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2381),
  [7633] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__mlhs, 1, 0, 0),
  [7635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1800),
  [7637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [7639] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_elsif, 3, 0, 63),
  [7641] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_guard, 2, 0, 64),
  [7643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [7645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unless_guard, 2, 0, 64),
  [7647] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_when_repeat1, 2, 0, 90), SHIFT_REPEAT(807),
  [7650] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_when_repeat1, 2, 0, 90),
  [7652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [7654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [7656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_repeat1, 2, 0, 0),
  [7658] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_repeat1, 2, 0, 0), SHIFT_REPEAT(810),
  [7661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(566),
  [7663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1269),
  [7665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(820),
  [7667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5410),
  [7669] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_match_repeat1, 2, 0, 70),
  [7671] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_match_repeat1, 2, 0, 70), SHIFT_REPEAT(2019),
  [7674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5259),
  [7676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3132),
  [7678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3082),
  [7680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(425),
  [7682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_destructured_left_assignment, 3, 0, 0),
  [7684] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rescue, 3, 0, 73),
  [7686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4600),
  [7688] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_command_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(455),
  [7691] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(4623),
  [7694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bare_parameters, 2, 0, 0),
  [7696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4620),
  [7698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else, 3, 0, 0),
  [7700] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_superclass, 2, 0, 0),
  [7702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4591),
  [7704] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4590),
  [7706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2156),
  [7708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5922),
  [7710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_when_repeat1, 2, 0, 68),
  [7712] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ensure, 2, 0, 0),
  [7714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5555),
  [7716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1020),
  [7718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(155),
  [7720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5313),
  [7722] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__body_statement_repeat1, 1, 0, 0),
  [7724] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(4619),
  [7727] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__array_pattern_body, 1, 0, 0),
  [7729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4619),
  [7731] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bare_parameters, 1, 0, 0),
  [7733] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rescue, 4, 0, 93),
  [7735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(980),
  [7737] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__argument_list_with_trailing_comma, 1, 0, 0),
  [7739] = {.entry = {.count = 1, .reusable = true}}, SHIFT(432),
  [7741] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(4620),
  [7744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3137),
  [7746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3098),
  [7748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [7750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else, 2, 0, 0),
  [7752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5296),
  [7754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(141),
  [7756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1311),
  [7758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [7760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(435),
  [7762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3138),
  [7764] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rescue, 2, 0, 16),
  [7766] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rescue, 3, 0, 71),
  [7768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4588),
  [7770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2118),
  [7772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5956),
  [7774] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5859),
  [7776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2162),
  [7778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5120),
  [7780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5123),
  [7782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2084),
  [7784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4622),
  [7786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5212),
  [7788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(748),
  [7790] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2115),
  [7792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5342),
  [7794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(751),
  [7796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5178),
  [7798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_repeat1, 1, 0, 0),
  [7800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2970),
  [7802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(742),
  [7804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4800),
  [7806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(595),
  [7808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [7810] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2165),
  [7812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2151),
  [7814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4453),
  [7816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5079),
  [7818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5128),
  [7820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2109),
  [7822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5129),
  [7824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4734),
  [7826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4733),
  [7828] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4717),
  [7830] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2149),
  [7832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5113),
  [7834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5204),
  [7836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5264),
  [7838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5109),
  [7840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5117),
  [7842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2150),
  [7844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2350),
  [7846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [7848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5553),
  [7850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2119),
  [7852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2125),
  [7854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_match_repeat1, 1, 0, 34),
  [7856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2126),
  [7858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2127),
  [7860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2123),
  [7862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2128),
  [7864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2116),
  [7866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_when, 4, 0, 88),
  [7868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5181),
  [7870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5154),
  [7872] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__guard, 1, 0, 0),
  [7874] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_in_clause, 3, 0, 67),
  [7876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5105),
  [7878] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_block_parameters_repeat1, 2, 0, 120), SHIFT_REPEAT(5859),
  [7881] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_block_parameters_repeat1, 2, 0, 120),
  [7883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2148),
  [7885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5226),
  [7887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5205),
  [7889] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2322),
  [7891] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2323),
  [7893] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2327),
  [7895] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_in_clause, 4, 0, 110),
  [7897] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4417),
  [7899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4418),
  [7901] = {.entry = {.count = 1, .reusable = false}}, SHIFT(4432),
  [7903] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2160),
  [7905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2916),
  [7907] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2917),
  [7909] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2918),
  [7911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [7913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5107),
  [7915] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5078),
  [7917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_hash_repeat1, 2, 0, 0),
  [7919] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_hash_repeat1, 2, 0, 0), SHIFT_REPEAT(789),
  [7922] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(4622),
  [7925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5047),
  [7927] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_array_repeat1, 2, 0, 0), SHIFT_REPEAT(5269),
  [7930] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_string_array_repeat1, 2, 0, 0),
  [7932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2159),
  [7934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5402),
  [7936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5416),
  [7938] = {.entry = {.count = 1, .reusable = false}}, SHIFT(5421),
  [7940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5062),
  [7942] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_symbol_array_repeat1, 2, 0, 0), SHIFT_REPEAT(5223),
  [7945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_symbol_array_repeat1, 2, 0, 0),
  [7947] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5206),
  [7949] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5516),
  [7951] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_when, 3, 0, 67),
  [7953] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2571),
  [7955] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2590),
  [7957] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2557),
  [7959] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2539),
  [7961] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2585),
  [7963] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2548),
  [7965] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2589),
  [7967] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2622),
  [7969] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4816),
  [7971] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2616),
  [7973] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2615),
  [7975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2553),
  [7977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2545),
  [7979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2599),
  [7981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2554),
  [7983] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 1, 0, 0),
  [7985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5950),
  [7987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2542),
  [7989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2559),
  [7991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2560),
  [7993] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2608),
  [7995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2562),
  [7997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2563),
  [7999] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2582),
  [8001] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2570),
  [8003] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2591),
  [8005] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2597),
  [8007] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5336),
  [8009] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_left_assignment_list, 1, 0, 0),
  [8011] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2618),
  [8013] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2586),
  [8015] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2543),
  [8017] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2587),
  [8019] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2601),
  [8021] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_block_parameters_repeat1, 2, 0, 118),
  [8023] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2602),
  [8025] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2603),
  [8027] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2604),
  [8029] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2619),
  [8031] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2620),
  [8033] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2605),
  [8035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2538),
  [8037] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2550),
  [8039] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2610),
  [8041] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2593),
  [8043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4450),
  [8045] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2551),
  [8047] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2623),
  [8049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2609),
  [8051] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2595),
  [8053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2611),
  [8055] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2612),
  [8057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2621),
  [8059] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2596),
  [8061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2617),
  [8063] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4405),
  [8065] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2873),
  [8067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4398),
  [8069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2218),
  [8071] = {.entry = {.count = 1, .reusable = true}}, SHIFT(769),
  [8073] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2216),
  [8075] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4695),
  [8077] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2484),
  [8079] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4696),
  [8081] = {.entry = {.count = 1, .reusable = true}}, SHIFT(771),
  [8083] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4675),
  [8085] = {.entry = {.count = 1, .reusable = true}}, SHIFT(475),
  [8087] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4365),
  [8089] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4366),
  [8091] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2366),
  [8093] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2222),
  [8095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4370),
  [8097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [8099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [8101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4795),
  [8103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4754),
  [8105] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4755),
  [8107] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4400),
  [8109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [8111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5483),
  [8113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4371),
  [8115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4372),
  [8117] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [8119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4374),
  [8121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(784),
  [8123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4740),
  [8125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2224),
  [8127] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2394),
  [8129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5439),
  [8131] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2977),
  [8133] = {.entry = {.count = 1, .reusable = true}}, SHIFT(476),
  [8135] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4360),
  [8137] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2445),
  [8139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(749),
  [8141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(777),
  [8143] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5525),
  [8145] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [8147] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2061),
  [8149] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5418),
  [8151] = {.entry = {.count = 1, .reusable = true}}, SHIFT(779),
  [8153] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4649),
  [8155] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4650),
  [8157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5455),
  [8159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2993),
  [8161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(468),
  [8163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2059),
  [8165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2177),
  [8167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2926),
  [8169] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [8171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(465),
  [8173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(768),
  [8175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4959),
  [8177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4783),
  [8179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2325),
  [8181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2994),
  [8183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(782),
  [8185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4967),
  [8187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(478),
  [8189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4960),
  [8191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5319),
  [8193] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2999),
  [8195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [8197] = {.entry = {.count = 1, .reusable = true}}, SHIFT(793),
  [8199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3001),
  [8201] = {.entry = {.count = 1, .reusable = true}}, SHIFT(688),
  [8203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 3, 0, 0),
  [8205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(479),
  [8207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [8209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2995),
  [8211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4357),
  [8213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2929),
  [8215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2384),
  [8217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(461),
  [8219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [8221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(350),
  [8223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(804),
  [8225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(758),
  [8227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(460),
  [8229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5570),
  [8231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(772),
  [8233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3002),
  [8235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4454),
  [8237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(473),
  [8239] = {.entry = {.count = 1, .reusable = true}}, SHIFT(801),
  [8241] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4661),
  [8243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4963),
  [8245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(675),
  [8247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(786),
  [8249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2932),
  [8251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4988),
  [8253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2471),
  [8255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(477),
  [8257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4472),
  [8259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5345),
  [8261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2329),
  [8263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(459),
  [8265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [8267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4425),
  [8269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(785),
  [8271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2933),
  [8273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [8275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(480),
  [8277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4779),
  [8279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5152),
  [8281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2354),
  [8283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5237),
  [8285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4810),
  [8287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4379),
  [8289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(458),
  [8291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(776),
  [8293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2326),
  [8295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5333),
  [8297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4866),
  [8299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2241),
  [8301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4573),
  [8303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4469),
  [8305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4465),
  [8307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5485),
  [8309] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_statement, 1, 0, 0),
  [8311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5359),
  [8313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4467),
  [8315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2431),
  [8317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2225),
  [8319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(467),
  [8321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(778),
  [8323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2304),
  [8325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2996),
  [8327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2997),
  [8329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4428),
  [8331] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5465),
  [8333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4654),
  [8335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(760),
  [8337] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_elsif, 4, 0, 85),
  [8339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(783),
  [8341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2401),
  [8343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(780),
  [8345] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_elsif, 4, 0, 86),
  [8347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4361),
  [8349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5058),
  [8351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2365),
  [8353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4672),
  [8355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2427),
  [8357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2314),
  [8359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4742),
  [8361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5302),
  [8363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5130),
  [8365] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2955),
  [8367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2393),
  [8369] = {.entry = {.count = 1, .reusable = true}}, SHIFT(462),
  [8371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4647),
  [8373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2369),
  [8375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2455),
  [8377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(470),
  [8379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [8381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4421),
  [8383] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2402),
  [8385] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4431),
  [8387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4658),
  [8389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2891),
  [8391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2370),
  [8393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(761),
  [8395] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2113),
  [8397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2379),
  [8399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2311),
  [8401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4449),
  [8403] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5210),
  [8405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(726),
  [8407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2375),
  [8409] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5074),
  [8411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(463),
  [8413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2042),
  [8415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2103),
  [8417] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2414),
  [8419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2415),
  [8421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5100),
  [8423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [8425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(792),
  [8427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [8429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2395),
  [8431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2332),
  [8433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2191),
  [8435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3039),
  [8437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(790),
  [8439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(798),
  [8441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [8443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(800),
  [8445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5401),
  [8447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2052),
  [8449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4583),
  [8451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(733),
  [8453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5613),
  [8455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2231),
  [8457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5407),
  [8459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [8461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [8463] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2973),
  [8465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(469),
  [8467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5446),
  [8469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5040),
  [8471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5548),
  [8473] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5057),
  [8475] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2242),
  [8477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [8479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5897),
  [8481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(464),
  [8483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(762),
  [8485] = {.entry = {.count = 1, .reusable = true}}, SHIFT(471),
  [8487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5067),
  [8489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4392),
  [8491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5353),
  [8493] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [8495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2467),
  [8497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2440),
  [8499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2400),
  [8501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2941),
  [8503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2300),
  [8505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5360),
  [8507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [8509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3074),
  [8511] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_block_body, 1, 0, 0),
  [8513] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4665),
  [8515] = {.entry = {.count = 1, .reusable = true}}, SHIFT(796),
  [8517] = {.entry = {.count = 1, .reusable = true}}, SHIFT(491),
  [8519] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 2, 0, 0),
  [8521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4968),
  [8523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(481),
  [8525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2111),
  [8527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4669),
  [8529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2972),
  [8531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2944),
  [8533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5363),
  [8535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(759),
  [8537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2176),
  [8539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [8541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(683),
  [8543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4452),
  [8545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4684),
  [8547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4694),
  [8549] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5415),
  [8551] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2956),
  [8553] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5473),
  [8555] = {.entry = {.count = 1, .reusable = true}}, SHIFT(763),
  [8557] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5338),
  [8559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [8561] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2947),
  [8563] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4970),
  [8565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5559),
  [8567] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2408),
  [8569] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4817),
  [8571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(794),
  [8573] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3005),
  [8575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2090),
  [8577] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5251),
  [8579] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2091),
  [8581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4716),
  [8583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5328),
  [8585] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5387),
  [8587] = {.entry = {.count = 1, .reusable = true}}, SHIFT(764),
  [8589] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5384),
  [8591] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2265),
  [8593] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [8595] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5502),
  [8597] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2948),
  [8599] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3006),
  [8601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4674),
  [8603] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5386),
  [8605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(3073),
  [8607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2894),
  [8609] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [8611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5334),
  [8613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2409),
  [8615] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2094),
  [8617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5397),
  [8619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5715),
  [8621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2987),
  [8623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2195),
  [8625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2235),
  [8627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2338),
  [8629] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2420),
  [8631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5530),
  [8633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4730),
  [8635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5335),
  [8637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4722),
  [8639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5535),
  [8641] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5537),
  [8643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4395),
  [8645] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2988),
  [8647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2989),
  [8649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(803),
  [8651] = {.entry = {.count = 1, .reusable = true}}, SHIFT(767),
  [8653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4397),
  [8655] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2974),
  [8657] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2227),
  [8659] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5456),
  [8661] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4946),
  [8663] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2949),
  [8665] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5357),
  [8667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2095),
  [8669] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4952),
  [8671] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2895),
  [8673] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2096),
  [8675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4689),
  [8677] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2426),
  [8679] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2903),
  [8681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2098),
  [8683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(787),
  [8685] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_heredoc_body, 3, 0, 0),
  [8687] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_heredoc_body, 2, 0, 0),
};

enum ts_external_scanner_symbol_identifiers {
  ts_external_token__line_break = 0,
  ts_external_token__no_line_break = 1,
  ts_external_token_simple_symbol = 2,
  ts_external_token__string_start = 3,
  ts_external_token__symbol_start = 4,
  ts_external_token__subshell_start = 5,
  ts_external_token__regex_start = 6,
  ts_external_token__string_array_start = 7,
  ts_external_token__symbol_array_start = 8,
  ts_external_token__heredoc_body_start = 9,
  ts_external_token_string_content = 10,
  ts_external_token_heredoc_content = 11,
  ts_external_token__string_end = 12,
  ts_external_token_heredoc_end = 13,
  ts_external_token_heredoc_beginning = 14,
  ts_external_token_SLASH = 15,
  ts_external_token__block_ampersand = 16,
  ts_external_token__splat_star = 17,
  ts_external_token__unary_minus = 18,
  ts_external_token__unary_minus_num = 19,
  ts_external_token__binary_minus = 20,
  ts_external_token__binary_star = 21,
  ts_external_token__singleton_class_left_angle_left_langle = 22,
  ts_external_token_hash_key_symbol = 23,
  ts_external_token__identifier_suffix = 24,
  ts_external_token__constant_suffix = 25,
  ts_external_token__hash_splat_star_star = 26,
  ts_external_token__binary_star_star = 27,
  ts_external_token__element_reference_bracket = 28,
  ts_external_token__short_interpolation = 29,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__line_break] = sym__line_break,
  [ts_external_token__no_line_break] = sym__no_line_break,
  [ts_external_token_simple_symbol] = sym_simple_symbol,
  [ts_external_token__string_start] = sym__string_start,
  [ts_external_token__symbol_start] = sym__symbol_start,
  [ts_external_token__subshell_start] = sym__subshell_start,
  [ts_external_token__regex_start] = sym__regex_start,
  [ts_external_token__string_array_start] = sym__string_array_start,
  [ts_external_token__symbol_array_start] = sym__symbol_array_start,
  [ts_external_token__heredoc_body_start] = sym__heredoc_body_start,
  [ts_external_token_string_content] = sym_string_content,
  [ts_external_token_heredoc_content] = sym_heredoc_content,
  [ts_external_token__string_end] = sym__string_end,
  [ts_external_token_heredoc_end] = sym_heredoc_end,
  [ts_external_token_heredoc_beginning] = sym_heredoc_beginning,
  [ts_external_token_SLASH] = anon_sym_SLASH,
  [ts_external_token__block_ampersand] = sym__block_ampersand,
  [ts_external_token__splat_star] = sym__splat_star,
  [ts_external_token__unary_minus] = sym__unary_minus,
  [ts_external_token__unary_minus_num] = sym__unary_minus_num,
  [ts_external_token__binary_minus] = sym__binary_minus,
  [ts_external_token__binary_star] = sym__binary_star,
  [ts_external_token__singleton_class_left_angle_left_langle] = sym__singleton_class_left_angle_left_langle,
  [ts_external_token_hash_key_symbol] = sym_hash_key_symbol,
  [ts_external_token__identifier_suffix] = sym__identifier_suffix,
  [ts_external_token__constant_suffix] = sym__constant_suffix,
  [ts_external_token__hash_splat_star_star] = sym__hash_splat_star_star,
  [ts_external_token__binary_star_star] = sym__binary_star_star,
  [ts_external_token__element_reference_bracket] = sym__element_reference_bracket,
  [ts_external_token__short_interpolation] = sym__short_interpolation,
};

static const bool ts_external_scanner_states[42][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__no_line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_string_content] = true,
    [ts_external_token_heredoc_content] = true,
    [ts_external_token__string_end] = true,
    [ts_external_token_heredoc_end] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__block_ampersand] = true,
    [ts_external_token__splat_star] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__singleton_class_left_angle_left_langle] = true,
    [ts_external_token_hash_key_symbol] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__hash_splat_star_star] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
    [ts_external_token__short_interpolation] = true,
  },
  [2] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [3] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__block_ampersand] = true,
    [ts_external_token__splat_star] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token_hash_key_symbol] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__hash_splat_star_star] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [4] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__block_ampersand] = true,
    [ts_external_token__splat_star] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token_hash_key_symbol] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__hash_splat_star_star] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [5] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__block_ampersand] = true,
    [ts_external_token__splat_star] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token_hash_key_symbol] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__hash_splat_star_star] = true,
  },
  [6] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [7] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [8] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [9] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__block_ampersand] = true,
    [ts_external_token__splat_star] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token_hash_key_symbol] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__hash_splat_star_star] = true,
  },
  [10] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token_hash_key_symbol] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__hash_splat_star_star] = true,
  },
  [11] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__binary_star_star] = true,
  },
  [12] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [13] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
    [ts_external_token__binary_star_star] = true,
  },
  [14] = {
    [ts_external_token__no_line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [15] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__splat_star] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [16] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__no_line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [17] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__splat_star] = true,
    [ts_external_token__unary_minus] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [18] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [19] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [20] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus_num] = true,
    [ts_external_token__singleton_class_left_angle_left_langle] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [21] = {
    [ts_external_token__line_break] = true,
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus_num] = true,
  },
  [22] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__subshell_start] = true,
    [ts_external_token__regex_start] = true,
    [ts_external_token__string_array_start] = true,
    [ts_external_token__symbol_array_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_beginning] = true,
    [ts_external_token__unary_minus_num] = true,
  },
  [23] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [24] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [25] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [26] = {
    [ts_external_token_simple_symbol] = true,
    [ts_external_token__symbol_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [27] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__binary_star_star] = true,
  },
  [28] = {
    [ts_external_token__string_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__binary_star_star] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [29] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [30] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_SLASH] = true,
    [ts_external_token__binary_minus] = true,
    [ts_external_token__binary_star] = true,
    [ts_external_token__binary_star_star] = true,
  },
  [31] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__string_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [32] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__heredoc_body_start] = true,
  },
  [33] = {
    [ts_external_token__line_break] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [34] = {
    [ts_external_token__string_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token__identifier_suffix] = true,
    [ts_external_token__constant_suffix] = true,
  },
  [35] = {
    [ts_external_token__heredoc_body_start] = true,
  },
  [36] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [37] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_string_content] = true,
    [ts_external_token__string_end] = true,
    [ts_external_token__short_interpolation] = true,
  },
  [38] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_heredoc_content] = true,
    [ts_external_token_heredoc_end] = true,
    [ts_external_token__short_interpolation] = true,
  },
  [39] = {
    [ts_external_token__string_start] = true,
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token__element_reference_bracket] = true,
  },
  [40] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token_string_content] = true,
    [ts_external_token__short_interpolation] = true,
  },
  [41] = {
    [ts_external_token__heredoc_body_start] = true,
    [ts_external_token__string_end] = true,
  },
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_ruby_external_scanner_create(void);
void tree_sitter_ruby_external_scanner_destroy(void *);
bool tree_sitter_ruby_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_ruby_external_scanner_serialize(void *, char *);
void tree_sitter_ruby_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#else
#define TS_PUBLIC __attribute__((visibility("default")))
#endif

TS_PUBLIC const TSLanguage *tree_sitter_ruby(void) {
  static const TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_ruby_external_scanner_create,
      tree_sitter_ruby_external_scanner_destroy,
      tree_sitter_ruby_external_scanner_scan,
      tree_sitter_ruby_external_scanner_serialize,
      tree_sitter_ruby_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
